#!/usr/bin/env python3
import random
from PIL import Image

def convert_to_gif(infile, outfile):
    """Convert an image into a GIF using LZW compression"""
    # Open input and output files
    inp = open(infile, 'rb')
    out = open(outfile, 'wb')
    
    # Read the image
    img = Image.open(inp).convert('RGB')
    
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = img.width, img.height
    packed_fields = (0 << 7) | (0 << 6) | (0 << 5) | (0 << 4) | (0 << 3) | (0 << 2) | (0 << 1) | 0  # No global colour table, no sort flag, no interlace
    lsd = struct.pack("!HHBB", width, height, packed_fields, 0)  # Background colour index is always zero
    
    # 3. Global Color Table
    color_table = b''
    for i in range(2 ** (1 + ((packed_fields >> 4) & 0b111)), 2 ** (1 + ((packed_fields >> 4) & 0b111)) + 2 ** ((packed_fields >> 4) & 0b111)):
        color_table += struct.pack("!BBB", *img.getpixel((i % width, i // width)))
    
    # 4. Graphic Control Extension
    gce = b'\x21\xF9\x04\x00\x00\x00\x00'  # Disposal method: None, User Input Flag: False, Transparent Colour Index: 0
    
    # 5. Image Descriptor
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00'  # Left position is 0, top position is 0, width is the same as the logical screen descriptor, height is the same as the logical screen descriptor
    
    # 6. Image Data
    rng = random.Random()
    rng.seed(12345)
    lzw_minimum_code_size = 8
    blocks = []
    while True:
        block_size = min(rng.randint(1, 255), len(img.tobytes()) - len(blocks) * 255)
        if not block_size:
            break
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
