#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a' + (b'\x00' * 7)
    out.write(header)

    # 2. Logical Screen Descriptor
    lsd = bytearray([
        0x40, 0x00, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    ])
    out.write(lsd)

    # 3. Global Color Table
    gct = [
        (0x00, 0x00, 0x00),
        (0xff, 0xff, 0xff),
    ]
    for i in range(6):
        r, g, b = rng.read(3)
        gct.append((r[0], g[0], b[0]))
    gct = bytes(sum(gct, []))
    out.write(gct)

    # 4. Image Descriptor
    id = bytearray([
        0x2c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    ])
    out.write(id)

    # 5. Local Color Table
    lct = gct
    out.write(lct)

    # 6. Image Data
    data = bytearray()
    while True:
        code = int.from_bytes(rng.read(1), 'big')
        if code == 0x00:
            break
        data += struct.pack('B', code)
    out.write(data)

    # 7. Trailer
    trailer = b';'
    out.write(trailer)
