#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Packed fields
    packed_fields = struct.pack('B', 0b00000000)
    # Background color index
    background_color_index = struct.pack('B', rng.read(1)[0])
    # Pixel aspect ratio
    pixel_aspect_ratio = struct.pack('B', rng.read(1)[0])
    # Write all of this to the output
    out.write(header + width + height + packed_fields + background_color_index + pixel_aspect_ratio)
    
    # 3. Global Color Table
    # We don't need one, so we write nothing here
    
    # 4. Image Descriptor
    image_descriptor = struct.pack('BBHBB', 0x2c, 0, 0, width, height)
    # Packed fields
    packed_fields = struct.pack('B', 0b00000000)
    # Local color table flag
    local_color_table_flag = struct.pack('B', 0b00000000)
    # Interlace flag
    interlace_flag = struct.pack('B', 0b00000000)
    # Sort flag
    sort_flag = struct.pack('B', 0b00000000)
    # Size of local color table
    size_of_local_color_table = struct.pack('B', 0b00000000)
    # Write everything to the output
    out.write(image_descriptor + packed_fields + local_color_table_flag + interlace_flag + sort_flag + size_of_local_color_table)
    
    # 5. LZW Minimum Code Size
    lzw_minimum_code_size = struct.pack('B', 0b00000000)
    out.write(lzw_minimum_code_size)
    
    # 6. Data Sub-Blocks
    # We don't need any data sub-blocks, so we write nothing here
    
    # 7. Trailer
    trailer = struct.pack('B', 0x3b)
    out.write(trailer)
