#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Packed fields
    packed = struct.pack('B', 0b10000000 | ((width & 0xff) >> 7))
    packed += struct.pack('B', (width & 0x7f) << 1)
    packed += struct.pack('B', 0b10000000 | ((height & 0xff) >> 7))
    packed += struct.pack('B', (height & 0x7f) << 1)
    # Background color index
    packed += struct.pack('B', rng.read(1)[0])
    # Pixel aspect ratio
    packed += struct.pack('B', rng.read(1)[0])
    
    # 3. Global Color Table
    # Number of colors in table
    ncolors = 1 << ((packed[4] & 0b00000111) + 1)
    # Read the RGB values from /dev/urandom
    rgbs = [struct.unpack('BBB', rng.read(3)) for _ in range(ncolors)]
    # Write them to the output
    for rgb in rgbs:
        out.write(bytes(rgb))
    
    # 4. Image Data
    # First byte is the LZW code size
    out.write(struct.pack('B', rng.read(1)[0]))
    # Then we write a bunch of random data
    while True:
        try:
            out.write(rng.read(1))
        except KeyboardInterrupt:
            break
