#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Packed field:
    #   global color table flag (0x80)
    #   color resolution (0x70)
    #   sort flag (0x08)
    #   size of global color table (0x07)
    packed_field = 0x87
    background_color_index = 0
    pixel_aspect_ratio = 0
    
    # 3. Global Color Table
    # 256 x 3-byte entries
    global_color_table = rng.read(256 * 3)
    
    # 4. Image Data
    # 1 byte to indicate the start of an image block
    image_separator = b'\x2c'
    # 9 bytes for the image descriptor
    image_descriptor = struct.pack('>BBHBBB',
        0x00, 0x00, width, height,
        0x00, background_color_index, pixel_aspect_ratio
    )
    # 1 byte to indicate that there is no local color table
    local_color_table_flag = b'\x00'
    # 1 byte to indicate the interlacing mode
    interlace_flag = b'\x00'
    # 1 byte to indicate the end of an image block
    image_terminator = b'\x3b'
    
    # Write everything together
    out.write(header +
              struct.pack('>HHB', width, height, packed_field) +
              global_color_table +
              image_separator +
              image_descriptor +
              local_color_table_flag +
              interlace_flag +
              image_terminator
             )
