#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    # We use a fixed color table of 256 colors
    color_table = rng.read(768)
    
    # 4. Graphic Control Extension
    # We use a random delay time
    gce = struct.pack('>BBBHB', 0xF9, 0x04, 0x08, struct.unpack('>I', rng.read(4))[0], rng.read(1)[0])
    
    # 5. Image Descriptor
    image_descriptor = struct.pack('>BBHHBB', 0x2C, 0, 0, width, height, 0)
    
    # 6. Image Data
    # We use a random amount of data
    image_data = rng.read(struct.unpack('>I', rng.read(4))[0])
    
    # 7. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
