#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Packed fields:
    #   - Global Color Table Flag (1 bit)
    #   - Color Resolution (3 bits)
    #   - Sort Flag (1 bit)
    #   - Size of Global Color Table (3 bits)
    packed_fields = [
        0b10000000 |  # Global Color Table Flag
        ((rng.read(1)[0] & 0b11100000) >> 5),  # Color Resolution
        0b00000000 |  # Sort Flag
        ((rng.read(1)[0] & 0b00011100) >> 2)  # Size of Global Color Table
    ]
    lsd = struct.pack('>HBB', width, height, *packed_fields)
    
    # 3. Global Color Table
    # Random colors (each is three bytes)
    global_color_table_length = 2 ** (packed_fields[2] + 1)
    global_color_table = rng.read(global_color_table_length * 3)
    
    # 4. Graphic Control Extension
    # Random delay time (two bytes)
    graphic_control_extension = b'\x21\xF9\x04\x00' + rng.read(2) + b'\x00'
    
    # 5. Image Descriptor
    # Random left/top position (two bytes each)
    image_descriptor = b'\x2C\x00\x00\x00\x00' + rng.read(4) + b'\x00'
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    for _ in range(total_blocks_size):
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
