#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, duration=1.0):
    # Header
    out.write(b'GIF89a')  # Magic signature
    out.write(struct.pack('<HH', width, height))  # Screen size
    out.write(b'\xF0\x00')  # Global color table flag (yes), color resolution (8 bits)
    out.write(b'\x00\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00')  # Netscape application block

    # Graphics control extension
    out.write(b'\x21\xF9\x04\x05\x00\x00\xFF\xFF\xFF')

    # Image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00')

    # Local color table
    for i in range(256):
        out.write(bytes([i]))

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([int(rng.read(1)[0]) % 256]))

    # Trailer
    out.write(b';')
