
import sys, os
from struct import pack

# Check arguments
if len(sys.argv) != 2:
    print("Usage: python3 gifencoder.py <inputfile>")
    exit()

# Open input file and read it into a byte array
with open(sys.argv[1], "rb") as f:
    data = f.read()

# Create an empty bytearray for the GIF header and color table
header = bytearray([])
color_table = bytearray([])

# Set up some variables that we will need later on
width = 0
height = 0
lsd = None
gce = None
image_descriptor = None
image_data = None
trailer = b'\x3b'

# Parse the PNG file
for i in range(len(data)):
    if data[i] == ord('I'):
        width = int.from_bytes(data[(i+4):(i+8)], 'big')
        height = int.from_bytes(data[(i+8):(i+12)], 'big')
        lsd = data[(i-6):(i+12)]
        break

# Parse the PNG file again to find the color table
for i in range(len(data)):
    if data[i] == ord('\xff'):
        if data[i+1] == ord('\xdb'):
            color_table = data[(i+4):(i+79)]
            break

# Parse the PNG file again to find the first IDAT chunk
for i in range(len(data)):
    if data[i] == ord('I'):
        if data[i+1] == ord('D'):
            image_data = data[(i+4):(len(data)-1)]
            break

# Parse the PNG file again to find the first GCE chunk
for i in range(len(data)):
    if data[i] == ord('I'):
        if data[i+1] == ord('E'):
            gce = data[(i+4):(i+15)]
            break

# Parse the PNG file again to find the first IHDR chunk
for i in range(len(data)):
    if data[i] == ord('I'):
        if data[i+1] == ord('H'):
            image_descriptor = data[(i+4):(i+12)]
            break

# Create the GIF header
header += b"GIF89a"
header += width.to_bytes(2, 'big')
header += height.to_bytes(2, 'big')
header += b"\x87\x00\x00"

# Create the output file
outname = os.path.splitext(os.path.basename(sys.argv[1]))[0] + ".gif"
with open(outname, "wb") as out:
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
