#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), byteorder='big')
    height = int.from_bytes(rng.read(2), byteorder='big')
    packed = int.from_bytes(rng.read(1), byteorder='big')
    global_color_table_flag = (packed & 0b10000000) >> 7
    color_resolution = (packed & 0b01110000) >> 4
    sort_flag = (packed & 0b00001000) >> 3
    global_color_table_size = (packed & 0b00000111) + 1
    background_color_index = int.from_bytes(rng.read(1), byteorder='big')
    pixel_aspect_ratio = int.from_bytes(rng.read(1), byteorder='big')
    
    # 3. Global Color Table
    if global_color_table_flag == 1:
        for i in range(global_color_table_size * 3):
            out.write(rng.read(1))
        
    # 4. Image Descriptors
    while True:
        # 4.1 Image Separator
        image_separator = rng.read(1)
        assert image_separator == b'\x2c'
        
        # 4.2 Image Left Position
        left_position = int.from_bytes(rng.read(2), byteorder='big')
        
        # 4.3 Image Top Position
        top_position = int.from_bytes(rng.read(2), byteorder='big')
        
        # 4.4 Image Width
        width = int.from_bytes(rng.read(2), byteorder='big')
        
        # 4.5 Image Height
        height = int.from_bytes(rng.read(2), byteorder='big')
        
        # 4.6 Local Color Table Flag
        local_color_table_flag = (int.from_bytes(rng.read(1), byteorder='big') & 0b10000000) >> 7
        
        # 4.7 Interlace Flag
        interlace_flag = (int.from_bytes(rng.read(1), byteorder='big') & 0b01000000) >> 6
        
        # 4.8 Sort Flag
        sort_flag = (int.from_bytes(rng.read(1), byteorder='big') & 0b00100000) >> 5
        
        # 4.9 Reserved
        reserved = (int.from_bytes(rng.read(1), byteorder='big') & 0b00011111) >> 0
        
        # 4.10 Local Color Table Size
        local_color_table_size = (int.from_bytes(rng.read(1), byteorder='big') & 0b00000111) + 1
        
        # 4.11 LZW Minimum Code Size
        lzw_minimum_code_size = int.from_bytes(rng.read(1), byteorder='big')
        
        # 4.12 Local Color Table
        if local_color_table_flag == 1:
            for i in range(local_color_table_size * 3):
                out.write(rng.read(1))
            
        # 4.13 Image Data
        while True:
            block_size = int.from_bytes(rng.read(1), byteorder='big')
            if block_size == 0:
                break
            else:
                for i in range(block_size):
                    out.write(rng.read(1))
        
        # 4.14 Block Terminator
        block_terminator = rng.read(1)
        assert block_terminator == b'\x00'
        
        # 4.15 Trailer
        trailer = rng.read(1)
        if trailer == b';':
            break
