#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    width = 160 + (rng.read(2)[0] % 40)
    height = 120 + (rng.read(2)[0] % 25)
    color_table_size = 7 + (rng.read(1)[0] % 7)
    background_color = rng.read(1)[0] & 0xF
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBB', width, height, color_table_size, pixel_aspect_ratio))

    # Global color table
    for _ in range(3 * (2 ** color_table_size)):
        out.write(rng.read(1))

    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', 0))

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')
    out.write(struct.pack('<HHB', width, height, 0))

    # Local color table
    for _ in range(3 * (2 ** color_table_size)):
        out.write(rng.read(1))

    # Pixel data
    for y in range(height):
        for x in range(width):
            if rng.read(1)[0] & 0x80 == 0:
                out.write(b'\x00')
            else:
                out.write(rng.read(1))

    # Trailer
    out.write(b';')
