#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jun 14 20:59:06 2018

@author: kazuki.onodera
"""

import numpy as np
import pandas as pd
import os, gc
from glob import glob
from tqdm import tqdm

import sys
sys.path.append(f'/home/{os.environ.get("USER")}/PythonLibrary')
import lgbextension as ex
import lightgbm as lgb
from multiprocessing import cpu_count, Pool
from sklearn.model_selection import StratifiedKFold

import utils, utils_cat
utils.start(__file__)
#==============================================================================

SEED = 71

LOOP = 5
NFOLD = 5

param = {
         'objective': 'binary',
         'metric': 'auc',
         
         'learning_rate': 0.01,
         'max_depth': 6,
         'num_leaves': 63,
         'max_bin': 255,
         
         'min_child_weight': 10,
         'min_data_in_leaf': 150,
         'reg_lambda': 0.5,  # L2 regularization term on weights.
         'reg_alpha': 0.5,  # L1 regularization term on weights.
         
         'colsample_bytree': 0.9,
         'subsample': 0.9,
#         'nthread': 32,
         'nthread': cpu_count(),
         'bagging_freq': 1,
         'verbose':-1,
         }


usecols_train = ['f001_hostgal_specz', 'f001_distmod', 'f001_petroR50_r', 'f001_petroR50_i', 'f001_petroR50_z',
                'f001_petroR90_r', 'f001_petroR90_i', 'f001_petroR90_z', 'f001_petroRad_r', 'f001_petroRad_i',
                'f001_petroRad_z', 'f001_petroTh50_r', 'f001_petroTh50_i', 'f001_petroTh50_z', 'f001_petroFlux_u',
                'f001_petroFlux_g', 'f001_petroFlux_r', 'f001_petroFlux_i', 'f001_petroFlux_z', 'f001_petroFlux_Y',
                'f001_petroFlux_J', 'f001_petroFlux_H', 'f001_petroFlux_Ks', 'f001_deV', 'f001_ngal', 'f001_SB_index',
                'f001_nsa_elpetro_ba', 'f001_nsa_elpetro_pa']

usecols_test = ['object_id', 'f001_hostgal_specz', 'f001_distmod', 'f001_petroR50_r', 'f001_petroR50_i', 'f001_petroR50_z',
               'f001_petroR90_r', 'f001_petroR90_i', 'f001_petroR90_z', 'f001_petroRad_r', 'f001_petroRad_i',
               'f001_petroRad_z', 'f001_petroTh50_r', 'f001_petroTh50_i', 'f001_petroTh50_z', 'f001_petroFlux_u',
               'f001_petroFlux_g', 'f001_petroFlux_r', 'f001_petroFlux_i', 'f001_petroFlux_z', 'f001_petroFlux_Y',
               'f001_petroFlux_J', 'f001_petroFlux_H', 'f001_petroFlux_Ks', 'f001_deV', 'f001_ngal', 'f001_SB_index',
               'f001_nsa_elpetro_ba', 'f001_nsa_elpetro_pa']



X = pd.read_pickle('../feature/train_f001_petroR50_r_f001_petroR50_i_f001_petroR50_z.pkl').drop('object_id', axis=1)
y = utils.load_target().target


def multi_fold(param):
    folds = StratifiedKFold(n_splits=NFOLD, shuffle=True, random