#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_gif(out: BinaryIO, width: int, height: int, rng: BinaryIO):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    lsd = struct.pack('<HHBB', width, height, 0b11111000, 0)  # No global colour table, no transparency
    
    # 3. Global Color Table
    color_table = rng.read(768)
    
    # 4. Graphic Control Extension
    gce = struct.pack('<BBHB', 0xF9, 4, 0, 0)  # Disposal method 0, transparent colour index 0
    
    # 5. Image Descriptor
    image_descriptor = struct.pack('<HHHH', 0, 0, width, height)  # Left, top, width, height
    
    # 6. Image Data
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    for _ in range(total_blocks_size):
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
