#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    global_color_table_size = rng.read(1)[0] & 7
    if global_color_table_size == 0:
        global_color_table_size = 1
    global_color_table = rng.read(global_color_table_size * 3)
    
    # 4. Application Extension Block
    app_ext_block = b'\x21\xff\x0bNETSCAPE2.0\x03\x01' + struct.pack('>I', 0)
    
    # 5. Image Descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00'
    
    # 6. Local Color Table (same as global color table)
    local_color_table = global_color_table
    
    # 7. Raster Data
    raster_data = rng.read((width * height) // 8)
    
    # Write everything to the output stream
    out.write(header)
    out.write(lsd)
    out.write(global_color_table)
