#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write("GIF89a".encode())

    # Logical screen descriptor
    out.write(struct.pack("<HHBB", 1024, 768, 0xF0 | (rng.read(1)[0] & 0x0F), 0))

    # Global color table
    for i in range(1 << 4):
        out.write(struct.pack("<BBB", rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]))

    # Application extension
    out.write((b"\x21\xFF\x0BNETSCAPE2.0\x03\x01" + b"\x00" * 12).ljust(25, b"\x00"))

    while True:
        # Image block header
        out.write(b"\x2C")

        # Image descriptor
        out.write(struct.pack("<HHHH", 0, 0, 1024, 768))

        # Local color table
        for i in range(1 << 4):
            out.write(struct.pack("<BBB", rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]))

        # Image data
        for y in range(768):
            out.write(rng.read(1024))

        # Trailer
        out.write(b";")
