#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a' + (b'\x00' * 7)
    out.write(header)

    # 2. Logical screen descriptor
    lsd = bytearray([
        0xff, 0xff,  # width
        0xff, 0xff,  # height
        0x00,        # background color index
        0x00,        # pixel aspect ratio
    ])
    out.write(lsd)

    # 3. Global color table
    global_color_table = [
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]),
        (rng.read