#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Packed fields
    #   Global Color Table Flag: 1
    #   Color Resolution:        7
    #   Sort Flag:              0
    #   Size of Global Color Table: 7
    packed = 0b10000000 | 0b01111111
    
    # Background color index
    background_color_index = rng.read(1)[0] & 0xFF
    
    # Pixel aspect ratio
    pixel_aspect_ratio = rng.read(1)[0] & 0xFF
    
    # Write everything to the output stream
    out.write(header + struct.pack('<HHB', width, height, packed) + bytes([background_color_index]) + bytes([pixel_aspect_ratio]))
    
    # 3. Global Color Table
    # We use 256 colors, so we need 256 * 3 bytes
    global_color_table = rng.read(768)
    out.write(global_color_table)
    
    # 4. Image Descriptor
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00'
    out.write(image_descriptor)
    
    # 5. Local Color Table
    # We don't have one, so write an empty string
    local_color_table = ''
    out.write(local_color_table)
    
    # 6. Image Data
    # We just write random data here
    image_data = rng.read(width*height)
    out.write(image_data)
    
    # 7. Trailer
    trailer = b';'
    out.write(trailer)
