#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd_fields = [
        0xFF & rng.read(1)[0], # Packed fields
        0xFF & rng.read(1)[0], # Background color index
        0xFF & rng.read(1)[0]  # Pixel aspect ratio
    ]
    
    # Write to output
    out.write(header + struct.pack('>HHBB', width, height, *lsd_fields))
    
    # 3. Global Color Table
    # We don't need one, so we write an empty block
    out.write(struct.pack('B', 0))
    
    # 4. Image Data
    # Loop over all pixels in the image
    for y in range(height):
        for x in range(width):
            # Get a random color index
            color_index = rng.read(1)[0] % 256
            
            # Write it as a single-byte pixel
            out.write(bytes([color_index]))
