#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    gct_size = rng.read(1)[0] % 257 + 1
    gct = []
    for i in range(gct_size):
        r, g, b = [rng.read(1)[0] for _ in range(3)]
        gct += [r, g, b]

    # 4. Image Descriptor
    image_descriptor = struct.pack('>HHBBB', 0, 0, 0b10000000 | rng.read(1)[0], 0, 0)
    
    # 5. Image Data
    # Let's just use all black pixels for now
    idat = b'\x00' * ((width*height)*3)
    
    # 6. Trailer
    trailer = b';'
    
    # Write everything to our output buffer
    out.write(header)
    out.write(lsd)
    out.write(bytes([gct_size]))
    out.write(bytes(gct))
    out.write(image_descriptor)
    out.write(idat)
    out.write(trailer)

