#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# returns its size in bytes.
def generate_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng.randint(0x7f, 0xff)
    height = 1 + rng.randint(0x7f, 0xff)
    flags = b'\xf7' if (width > 256 or height > 256) else b'\x00'
    bgColorIndex = rng.randint(0, 255)
    pixelAspectRatio = b'\x00'
    out.write(struct.pack('<HHBBB', width, height, flags[0], bgColorIndex, pixelAspectRatio[0]))

    # Global color table
    for i in range(0, 256):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    leftPos = 0
    topPos = 0
    width = 1 + rng.randint(0x7f, 0xff)
    height = 1 + rng.randint(0x7f, 0xff)
    flags = b'\x9c'
    localColorTableFlag = b'\x00'
    interlaceFlag = b'\x00'
    sortFlag = b'\x00'
    reserved = b'\x00\x00'
    lctSize = b'\x00'
    out.write(struct.pack('<HHHHBBBBB', leftPos, topPos, width, height, flags[0], localColorTableFlag[0], interlaceFlag[0], sortFlag[0], reserved[0], reserved[1], lctSize[0]))

    # Local color table
    for i in range(0, 256):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng.randint(0, 255)]))

    return 13 + 4 + 256 * 3 + 13 + width * height

