#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    global_color_table_size = rng.read(1)[0] & 7 + 1
    global_color_table_flag = bool((rng.read(1)[0] >> 7) & 1)
    if global_color_table_flag:
        global_color_table = [rng.read(3) for _ in range(global_color_table_size)]
    else:
        global_color_table = []

    # 4. Image Descriptor
    image_left_position = rng.read(2)
    image_top_position = rng.read(2)
    image_width = rng.read(2)
    image_height = rng.read(2)
    packed_fields = rng.read(1)[0]
    local_color_table_flag = bool((packed_fields >> 7) & 1)
    interlace_flag = bool((packed_fields >> 6) & 1)
    sort_flag = bool((packed_fields >> 5) & 1)
    reserved_bits = ((packed_fields << 2) & 0b111000) | ((packed_fields >> 2) & 0b11)
    local_color_table_size = (packed_fields & 0b11) + 1
    local_color_table = [rng.read(3) for _ in range(local_color_table_size)]

    # 5. Image Data
    # TODO: Implement this!
    pass

