#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Image Descriptor
    image_descriptor = struct.pack('>HHBBB', 0, 0, 0b10000000 | rng.read(1)[0], 0, 0)  # Left to right, top to bottom, local color table flag set, no interlacing
    
    # 5. Local Color Table (same as global one)
    out.write(color_table + image_descriptor)
    
    # 6. Image data
    # We'll just use the same color table again but this time we'll write some pixels
    while True:
        # Write a single pixel
        out.write(struct.pack('>BB', rng.read(1)[0], rng.read(1)[0]))
        
        # Check if we're done with this frame
        if rng.read(1)[0] & 0x80 == 0:
            break

