#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
     # 2. Logical screen descriptor
    width = int.from_bytes(rng.read(2), byteorder='big') % (2**10 - 1) + 1
    height = int.from_bytes(rng.read(2), byteorder='big') % (2**10 - 1) + 1

    flags = bytes([
        0b10000000 | # Global color table flag
        ((int.from_bytes(rng.read(1), byteorder='big') & 7) << 4) | # Color resolution
        (((int.from_bytes(rng.read(1), byteorder='big') >> 5) & 1) << 3) | # Sort flag
        (((int.from_bytes(rng.read(1), byteorder='big') >> 6) & 1) << 2) | # Size of global color table
        (((int.from_bytes(rng.read(1), byteorder='big') >> 7) & 1) << 1) # Background color index flag
    ])

    bg_color_index = bytes([int.from_bytes(rng.read(1), byteorder='big')]) if flags[0] & 1 else b'\x00'
    pixel_aspect_ratio = bytes([int.from_bytes(rng.read(1), byteorder='big')])

    logical_screen_descriptor = \
        width.to_bytes(2, byteorder='big', signed=False) +\
        height.to_bytes(2, byteorder='big', signed=False) +\
        flags +\
        bg_color_index +\
        pixel_aspect_ratio

    # 3. Global color table
    global_color_table_size = 2**(flags[0] & 7) * 3
    global_color_table = [
        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],

        rng.read(1)[0],
        rng.read(1)[0],
        rng.read(1)[0],