#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors!
    
    # 4. Image Descriptor
    image_descriptor = struct.pack('>HHBBB', 0, 0, 0, 0b10000000 | ((num_colors - 1) << 4), 0)

    # 5. Image Data
    data = [struct.pack('>H', rng.read(1)[0]) for _ in range((width * height))]

    # 6. Trailer
    trailer = b';'

    # Write everything to our output stream
    out.write(header + lsd + color_table + image_descriptor + b''.join(data) + trailer)

