#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        width: int = 1024, height: int = 768,
        num_colors: int = 256,
        out: BinaryIO = open("test.gif", "wb")) -> None:
    # Header
    header = b"GIF89a"

    # Logical screen descriptor
    lsd = struct.pack("<HHBBB", width, height, (num_colors >> 8), (num_colors & 0xff), 0)

    # Color table
    color_table = bytes()
    for i in range(num_colors):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        color_table += struct.pack("<BBB", r, g, b)

    # Graphics control extension
    gce = struct.pack("<BBHBBBB", 0x21, 0xf9, 4, 0b10000000 | 0b00000000, 0, 0, 0)

    # Image descriptor
    image_descriptor = struct.pack("<BHHHHB", 0x2c, 0, 0, width, height, 0)

    # Image data
    image_data = bytearray()
    for y in range(height):
        for x in range(width):
            image_data += struct.pack("<B", rng.read(1)[0])

    # Trailer
    trailer = b"\x3b"

    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
