#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    header = b'GIF89a' + bytes([
        0x12, 0x34, 0x56, 0x78,   # Width and height of image in pixels
        0xf0,                     # Color table size (0 means default)
        0xff, 0xff,               # Background color index
        0xff, 0xff                # Pixel aspect ratio
    ])
    out.write(header)

    # Global color table
    for i in range(0, 2**((header[7] & 0b111) + 1)):
        out.write(bytes([(i >> 8), i]))

    # Image descriptor
    out.write(struct.pack('<HHB', 0, 0, 0))    # Left position, top position, flags

    # LZW minimum code size
    mincodesize = 2
    out.write(mincodesize.to_bytes(1, byteorder='little'))

    # Data sub-blocks
    while True:

        # Block size
        blocksize = rng.read(1)[0] % 256
        if blocksize == 0:
            break;
        out.write(blocksize.to_bytes(1, byteorder='little'))

        # Data
        data = rng.read(blocksize)
        out.write(data)

