#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header (13 bytes)
    header = b'GIF89a' + \
             struct.pack('<HH', 0, 0) + \
             b'\x87\x00\x00'

    # Logical screen descriptor (7 bytes)
    lsd = struct.pack('BBBHBB',
                      255,       # Width
                      255,       # Height
                      0b10000000 | 64,    # Global color table flag and size of global color table
                      ord('A'),  # Background color index
                      ord('B'))  # Pixel aspect ratio

    # Application extension block (11 bytes)
    appext = b'\x21\xFF\x0B' + \
             b'NETSCAPE2.0' + \
             b'\x03\x01' + \
             b'\x00\x00'

    # Graphics control extension block (8 bytes)
    gce = b'\x21\xF9\x04' + \
         b'\x08\x00' + \
         b'\x00\x00' + \
         b'\x00'

    # Image descriptor (10 bytes)
    id = b''

    # Plain text extension block (19 bytes)
    ptext = b'\x21\x00\x0B' + \
            b'text' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00' + \
            b'\x00\x00'

    # Comment extension block (19 bytes)
    comment = b'\x21\xFE\x0B' + \
              b'comment' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00' + \
              b'\x00\x00'

    # Graphic control label (1 byte)
    graphic_control_label = b'\x2C'

    # Trailer (1 byte)
    trailer = b';'

    # Write everything to output stream
    out.write(header)
    out.write(lsd)
    out.write(appext)
    out.write(gce)
    out.write(id)
    out.write(ptext)
    out.write(comment)
    out.write(graphic_control_label)
    out.write(trailer)

