#!/usr/bin/env python3

import sys
import random
import struct
from io import BytesIO


if __name__ == '__main__':
    if len(sys.argv)!= 2:
        print("Usage: {} <output>".format(sys.argv[0]))
        exit()

    # Open our output file and seed the RNG
    out = open(sys.argv[1], 'wb')
    rng = random.Random(b'Hello World!')
    
    # 1. Header
    header = b'\x89\x50\x4E\x47\x0D\x0A\x1A\x0A'
    
    # 2. Logical Screen Descriptor
    width = rng.randint(1, 1000)
    height = rng.randint(1, 1000)
    bitdepth = rng.choice((1, 2, 4, 8, 16))
    colortype = rng.choice((0, 2, 3, 4, 6))
    compressiontype = rng.choice((0,))
    filtermethod = rng.choice((0,))
    interlace = rng.choice((0, 1))
    lsd = struct.pack('>BBHHBBBB', width, height, bitdepth, colortype,
                      compressiontype, filtermethod, interlace)
    
    # 3. Color Table (RGB triplets)
    num_colors = 2 ** bitdepth
    color_table = rng.read(3 * num_colors)  # Random colors
    
    # 4. Graphics Control Extension (optional)
    gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    
    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    for _ in range(total_blocks_size):
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
