#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # A global color table is used with 256 colors
    gct = rng.read(768)

    # 4. Image Data
    # An image descriptor is written to the output stream
    idd = struct.pack('>BBHHBB', 0x2C, 0, 0, width, height, 0)

    # The local color table is empty
    lct = b''

    # The image data starts with an extension block
    edt = struct.pack('>BB', 0x21, 0xF9)

    # The graphic control extension contains a single field: the delay time
    gce = struct.pack('>BBHB', 0x24, 0xFF, rng.read(2)[0], 0)

    # The plain text extension contains a single field: the grid size
    pte = struct.pack('>BBBB', 0x21, 0x01, 0x00, 0x00)

    # The application extension contains a single field: the loop count
    ae = struct.pack('>BBBBB', 0x21, 0xFF, 0x0B, 'NETSCAPE2.0'.encode(), 3, 1, 0)

    # Write everything together
    out.write(header + lsd + gct + idd + lct + edt + gce + pte + ae)
