#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, duration=1.0):
    # Write GIF header
    out.write('GIF89a'.encode())
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\xF0\x00')  # Global color table with 256 colors
    out.write(b'\xFF\xFF\xFF')  # Background color
    out.write(b'\x00')  # No aspect ratio

    # Write image descriptor
    out.write(b'!\xF9\x04\x00\x00\x00\x00,')
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\x00')  # No local color table
    out.write(b'\x00')  # No interlacing

    # Write graphics control extension
    out.write(b'\x21\xF9\x04\x01\x00\x00\x00')
    out.write(struct.pack('B', int(duration * 100)))
    out.write(b'\x00')  # Transparent color index
    out.write(b'\x00')  # Block terminator

    # Write pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([int(rng.read(1)[0])]))

    # Write trailer
    out.write(b';')

