#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Packed fields
    #   Global Color Table Flag (1 bit): 0
    #   Color Resolution (3 bits): 7
    #   Sort Flag (1 bit): 0
    #   Size of Global Color Table (3 bits): 7
    packed = struct.pack('B', 0b00000111)
    # Background color index (1 byte)
    background_color_index = rng.read(1)[0] % 256
    # Pixel aspect ratio (1 byte)
    pixel_aspect_ratio = rng.read(1)[0] % 256

    # 3. Global Color Table
    global_color_table = [
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', rng.read(3)),
        struct.unpack('BBB', r