#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Graphics Control Extension (optional)
    gce = None
    if rng.read(1)[0]:
        # We want to add an extension!
        # First, we need to figure out how long it will be
        ext_size = rng.read(1)[0]
        # Then, we can read in the extension data
        gce = rng.read(ext_size)

    # 5. Image Descriptor
    idd = b'\x2c\x00\x00\x00\x00'

    # 6. Local Color Table (optional)
    lct = None
    if rng.read(1)[0]:
        # We want to add a local color table!
        # First, we need to figure out how long it will be
        lct_size = rng.read(1)[0]
        # Then, we can read in the color table data
        lct = rng.read(lct_size)

    # 7. Image Data
    image_data = rng.read(width * height)

    # 8. Trailer
    trailer = b';'

    # Write everything together
    out.write(header + lsd + color_table + gce + idd + lct + image_data + trailer)

