#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    width = 16 + (rng.read(1)[0] % 256)
    height = 16 + (rng.read(1)[0] % 256)
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] % 256
    out.write(struct.pack('<HHBB', width, height, background_color_index, pixel_aspect_ratio))
    
    # Global color table
    for i in range(7):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))
        
    # Application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    loop_count = rng.read(2)
    out.write(loop_count)
    out.write(b'\x00')
    
    # Image descriptor
    left = 0
    top = 0
    width = 16 + (rng.read(1)[0] % 256)
    height = 16 + (rng.read(1)[0] % 256)
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    reserved_bits = 0
    out.write(struct.pack('<HHHBB', left, top, width, height, local_color_table_flag | interlace_flag << 1 | sort_flag << 2 | reserved_bits << 3))
    
    # Local color table
    for i in range(7):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))
        
    # LZW minimum code size
    lzw_minimum_code_size = rng.read(1)[0] % 256
    out.write(lzw_minimum_code_size)
    
    # Pixel data
    while True:
        if rng.read(1)[0] % 2 == 0:
            out.write(b'\x00')
        else:
            out.write(b'\x01')
            
        if rng.read(1)[0] % 2 == 0:
            break
