#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 15 14:26:09 2020

@author: jonas
"""
import numpy as np
from scipy.stats import norm, multivariate_normal

def get_marginal(x, y):
    """Get the marginal distribution of x given y."""
    
    # Get the mean and covariance matrix of the joint distribution
    mu = np.mean([x,y], axis=0)
    sigma = np.cov([x,y])
    
    # Create a multivariate normal distribution object
    mvndist = multivariate_normal(mu, sigma)
    
    # Return the marginal distribution of x
    return mvndist.pdf(x)

def get_conditional(x, y, z):
    """Get the conditional distribution of x given y and z."""
    
    # Get the mean and covariance matrix of the joint distribution
    mu = np.mean([x,y,z], axis=0)
    sigma = np.cov([x,y,z])
    
    # Create a multivariate normal distribution object
    mvndist = multivariate_normal(mu, sigma)
    
    # Return the conditional distribution of x given y and z
    return mvndist.pdf(x)

def get_joint(x, y, z):
    """Get the joint distribution of x, y and z."""
    
    # Get the mean and covariance matrix of the joint distribution
    mu = np.mean([x,y,z], axis=0)
    sigma = np.cov([x,y,z])
    
    # Create a multivariate normal distribution object
    mvndist = multivariate_normal(mu, sigma)
    
    # Return the joint distribution of x, y and z
    return mvndist.pdf([x,y,z])

def get_posterior(x, y, z):
    """Get the posterior distribution of x given y and z."""
    
    # Get the mean and covariance matrix of the joint distribution
    mu = np.mean([x,y,z], axis=0)
    sigma = np.cov([x,y,z])
    
    # Create a multivariate normal distribution object
    mvndist = multivariate_normal(mu, sigma)
    
    # Return the posterior distribution of x given y and z
    return mvndist.pdf(x)

def get_likelihood(x, y, z):
    """Get the likelihood of x given y and z."""
    
    # Get the mean and covariance matrix of the joint distribution
    mu = np.mean([x,y,z], axis=0)
    sigma = np.cov([x,y,z])
    
    # Create a multivariate normal distribution object
    mvndist = multivariate_normal(mu, sigma)
    
    # Return the likelihood of x given y and z
    return mvndist.logpdf([x,y,z])

def get_prior(x, y, z):
    """Get the prior distribution of x given y and z."""
    
    # Get the mean and covariance matrix of the joint distribution
    mu = np.mean([x,y,z], axis=0)
    sigma = np.cov([x,y,z])
    
    # Create a multivariate normal distribution object
    mvndist = multivariate_normal(mu, sigma)
    
    # Return the prior distribution of x given y and z
    return mvndist.pdf(x)

def get_bayes_factor(x, y, z):
    """Get the Bayes factor of x given y and z."""
    
    # Get the mean and covariance matrix of the joint distribution
    mu = np.mean([x,y,z], axis=0)
    sigma = np.cov([x,y,z])
    
    # Create a multivariate normal distribution object
    mvndist = multivariate_normal(mu, sigma)
    
    # Return the Bayes factor of x given y and z
    return mvndist.pdf([x,y,z]) / mvndist.pdf(x)

def get_evidence(x, y, z):
    """Get the evidence of x, y and z."""
    
    # Get the mean and covariance matrix of the joint distribution
    mu = np.mean([x,y,z], axis=0)
    sigma = np.cov([x,y,z])
    
    # Create a multivariate normal distribution object
    mvndist = multivariate_normal(mu, sigma)
    
    # Return the evidence of x, y and z
    return mvndist.pdf([x,y,z])

def get_integral(x, y, z):
    """Get the integral of x, y and z."""
    
    #