#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    out.write('GIF89a'.encode())

    # 2. Logical Screen Descriptor
    width = rng.read(1)[0] + 1
    height = rng.read(1)[0] + 1
    color_table_size = 7
    background_color_index = rng.read(1)[0] % (1 << color_table_size)
    pixel_aspect_ratio = 0
    out.write(struct.pack("!BBB", width, height, (color_table_size << 4) | pixel_aspect_ratio))

    # 3. Global Color Table
    for _ in range(1 << color_table_size):
        out.write(rng.read(3))

    # 4. Image Data
    out.write(b'\x2c')
    out.write(struct.pack("!BB", width, height))
    out.write(b'\x00\x00\xff\xff')
    for _ in range((width * height) // 2):
        out.write(rng.read(1))
