#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # We use a global color table with 256 colors
    gct = rng.read(768)

    # 4. Image Data
    # We have one image in this GIF, so we need to write an Image Descriptor
    idd = struct.pack('>HHHHBB', 0, 0, width, height, 0xF, 0)

    # The image data is just a bunch of random bytes
    imgdata = rng.read((width * height) // 2)

    # 5. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + gct + idd + imgdata + trailer)

