#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    global_color_table = rng.read(768)
    
    # 4. Image Descriptors
    # Number of images (using two bytes for variety)
    num_images = struct.unpack('>H', rng.read(2))[0]
    
    # Loop through all images
    for i in range(num_images):
        
        # 5. Image Descriptor
        # Random left position, top position, width, height
        image_descriptor = struct.pack('>HHHH', *struct.unpack('>HHHH', rng.read(8)))
        
        # 6. Local Color Table Flag
        local_color_table_flag = rng.read(1)[0] & 0b10000000
        
        # 7. Interlace Flag
        interlace_flag = rng.read(1)[0] & 0b01000000
        
        # 8. Sort Flag
        sort_flag = rng.read(1)[0] & 0b00100000
        
        # 9. Reserved
        reserved = rng.read(1)[0] & 0b00011111
        
        # 10. Size of Local Color Table
        size_of_local_color_table = rng.read(1)[0] & 0b00000111
        
        # 11. Image Data
        # Number of bits per pixel (using one byte for variety)
        num_bits_per_pixel = rng.read(1)[0]
        
        # Compression method (always 0)
        compression_method = 0
        
        # Image data length (using four bytes for variety)
        image_data_length = struct.unpack('>I', rng.read(4))[0]
        
        # Image data
        image_data = rng.read(image_data_length)
        
        # Write everything to the output file
        out.write(header + lsd + global_color_table + image_descriptor + chr(local_color_table_flag | interlace_flag | sort_flag | reserved | size_of_local_color_table).encode() + chr(num_bits_per_pixel).encode() + struct.pack('>I', compression_method) + struct.pack('>I', image_data_length) + image_data)
