#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')
    
    # Logical Screen Descriptor
    width = 16 + (rng.read(1)[0] & 0xF) * 16
    height = 16 + (rng.read(1)[0] & 0xF) * 16
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBB', width, height, background_color_index, pixel_aspect_ratio))
    
    # Global Color Table
    for i in range(7):
        color = rng.read(3)
        out.write(bytes([min(c, 255) for c in color]))
        
    # Image Descriptor
    left = 0
    top = 0
    width = 16 + (rng.read(1)[0] & 0xF) * 16
    height = 16 + (rng.read(1)[0] & 0xF) * 16
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    reserved = 0
    size_of_local_color_table = 0
    out.write(struct.pack('<HHBBB', left, top, width, height, local_color_table_flag | (interlace_flag << 1) | (sort_flag << 2) | (reserved << 3) | (size_of_local_color_table << 4)))
    
    # Local Color Table
    for i in range(7):
        color = rng.read(3)
        out.write(bytes([min(c, 255) for c in color]))
    
    # LZW Minimum Code Size
    minimum_code_size = rng.read(1)[0] % 8
    out.write(bytes([minimum_code_size]))
    
    # Pixel Data
    while True:
        code = rng.read(1)[0] % 256
        if code == 0xFF:
            break
        else:
            out.write(bytes([code]))
            
    # Trailer
    out.write(b'\x00')
