#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical Screen Descriptor
    width = rng.read(2)
    height = rng.read(2)
    packed = rng.read(1)
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(width + height + packed + bytes([background]) + bytes([aspect_ratio]))

    # 3. Global Color Table
    color_table_size = (packed[0] & 0x7) + 1
    for _ in range(color_table_size * 3):
        out.write(bytes([rng.read(1)[0]]))

    # 4. Image Descriptors
    while True:
        # 4.1 Image Separator
        out.write(b'\x2c')

        # 4.2 Image Descriptor
        left = rng.read(2)
        top = rng.read(2)
        width = rng.read(2)
        height = rng.read(2)
        packed = rng.read(1)
        lct_flag = bool(packed[0] & 0x80)
        interlace_flag = bool(packed[0] & 0x40)
        sort_flag = bool(packed[0] & 0x20)
        reserved = bool(packed[0] & 0x10)
        size_of_lct = ((packed[0] & 0x7) + 1) * 3
        out.write(left + top + width + height + packed)

        if not lct_flag:
            # 4.3 Local Color Table
            for _ in range(size_of_lct):
                out.write(bytes([rng.read(1)[0]]))

        # 4.4 LZW Minimum Code Size
        out.write(bytes([rng.read(1)[0]]))

        # 4.5 Data Sub-Blocks
        while True:
            length = rng.read(1)
            if length == b'\x00':
                break
            data = rng.read(struct.unpack('B', length)[0])
            out.write(length + data)

        # 4.6 Trailer
        if not interlace_flag:
            break

