#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('<BBB', *struct.unpack('>BBB', rng.read(3))) + rng.read(1)
    
    # 3. Global Color Table
    # We use a global color table with 256 colors
    gct = rng.read(768)
    
    # 4. Image Descriptors
    # We have one image descriptor
    idd = struct.pack('<BBHBB', *struct.unpack('>BBHHB', rng.read(9)))
    
    # 5. Local Color Table
    # We use a local color table with 256 colors
    lct = rng.read(768)
    
    # 6. Image Data
    # We have 1000 blocks of data
    imgdata = rng.read(1000)
    
    # 7. Trailer
    trailer = b';'
    
    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(gct)
    out.write(idd)
    out.write(lct)
    out.write(imgdata)
    out.write(trailer)
