#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    global_color_table = rng.read(num_colors * 3)

    # 4. Image Data
    # We will use a single image with no local color table
    # First we need to write the image descriptor
    idd = struct.pack('>BBHHB', 0x2C, 0, 0, width, height, 0)

    # Then we can write the image data itself
    # We will just repeat the same color over and over again
    # This is not a valid GIF, but it's good enough for our purposes
    image_data = rng.read(width * height // 8)

    # Write everything together
    out.write(header + lsd + global_color_table + idd + image_data)
