#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')
    
    # Logical Screen Descriptor
    width = 1 + (int.from_bytes(rng.read(2), 'little', signed=False)) % 640
    height = 1 + (int.from_bytes(rng.read(2), 'little', signed=False)) % 480
    background_color_index = int.from_bytes(rng.read(1), 'big', signed=False)
    pixel_aspect_ratio = int.from_bytes(rng.read(1), 'big', signed=False)
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write(background_color_index.to_bytes(1, 'big'))
    out.write(pixel_aspect_ratio.to_bytes(1, 'big'))
    
    # Global Color Table
    for _ in range(7):
        color = int.from_bytes(rng.read(3), 'big', signed=False)
        out.write(struct.pack('BBB', *[c >> 2 for c in struct.unpack('BBB', struct.pack('>I', color))] ))
        
    # Application Extension Block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')
    
    # Image Descriptor
    left = 0
    top = 0
    width = width
    height = height
    local_color_table_flag = False
    interlace_flag = True
    sort_flag = False
    reserved_local_color_table_size = 7
    out.write(struct.pack('!BHHBBBB', 0x2C, left, top, width, height, local_color_table_flag, interlace_flag, sort_flag, reserved_local_color_table_size))
    
    # Local Color Table
    for _ in range(7):
        color = int.from_bytes(rng.read(3), 'big', signed=False)
        out.write(struct.pack('BBB', *[c >> 2 for c in struct.unpack('BBB', struct.pack('>I', color))] ))
    
    # Graphic Control Extension
    delay_time = 5
    transparent_color_flag = True
    transparent_color_index = 0
    disposal_method = 2
    out.write(struct.pack('!BBHBB', 0x21, 0xf9, 4, delay_time, transparent_color_flag, transparent_color_index, disposal_method))
    
    # Image Data
    for y in range(height):
        for x in range(width):
            color_index = int.from_bytes(rng.read(1), 'big', signed=False)
            out.write(color_index.to_bytes(1, 'big'))
            
    # Trailer
    out.write(b';')
