#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 10 bytes of header
    out.write(b'GIF89a')

    # 7 bytes of screen descriptor
    out.write(struct.pack('<HBB', 256, 256, 0))

    # 4 bytes of color table (black and white)
    out.write(b'\x00\x00\x00\xff\xff\xff\x00\x00\x00\x00')

    # 1 byte of image descriptor
    out.write(b'\x2c')

    # 4 bytes of image position
    out.write(struct.pack('<HH', 0, 0))

    # 4 bytes of image size
    out.write(struct.pack('<HH', 256, 256))

    # 1 byte of flags
    out.write(b'\x00')

    # 1 byte of background color index
    out.write(b'\x00')

    # 1 byte of pixel aspect ratio
    out.write(b'\x00')

    # 1 byte of color table length
    out.write(b'\x00')

    # 1 byte of transparent color index
    out.write(b'\x00')

    # 1 byte of block size
    out.write(b'\x03')

    # 1 byte of terminator
    out.write(b'\x00')

    # 1 byte of extension introducer
    out.write(b'\x21')

    # 1 byte of graphic control label
    out.write(b'\xf9')

    # 4 bytes of graphic control extension
    out.write(struct.pack('<BHH', 0x4, 0, 0))

    # 1 byte of application extension label
    out.write(b'\xff')

    # 11 bytes of application extension
    out.write(b'NETSCAPE2.0\x03\x01')

    # 1 byte of data sub-block size
    out.write(b'\x0b')

    # 1 byte of loop count (forever)
    out.write(b'\x00')

    # 1 byte of terminator
    out.write(b'\x00')

    # 1 byte of trailer
    out.write(b';')

    # 1 byte of image data
    out.write(b'\x00')

    # 1 byte of trailer
    out.write(b'\x3b')

    # 1 byte of padding
    out.write(b'\x00')

    # 256 bytes of image data
    for i in range(256):
        out.write(struct.pack('<BBBB', *[int.from_bytes(rng.read(1), 'little') & 0xFF for _ in range(4)]))
