#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    # Random number of colors in global color table (up to 256)
    ncolors = rng.read(1)[0] % 256 + 1
    
    # Each color is three bytes
    gct = rng.read(ncolors * 3)
    
    # 4. Image Data
    # Random number of images (up to 256)
    nimages = rng.read(1)[0] % 256 + 1
    
    # For each image...
    for i in range(nimages):
        
        # Random width and height (using two bytes for each)
        width, height = struct.unpack('>HH', rng.read(4))
        
        # Other ID fields: Left position, Top position, Packed fields (using a random byte for variety)
        id = struct.pack('>HHBB', width, height, rng.read(1)[0], rng.read(1)[0])
        
        # Local Color Table Flag
        lctf = rng.read(1)[0] & 1
        
        # Interlace Flag
        interlaced = rng.read(1)[0] & 1
        
        # Sort Flag
        sorted = rng.read(1)[0] & 1
        
        # Size of Local Color Table
        if lctf == 1:
            ncolors = rng.read(1)[0] % 256 + 1
            
            # Each color is three bytes
            lct = rng.read(ncolors * 3)
        else:
            lct = None
        
        # Zero-length data block
        datablock = b'\x00'
        
        # Write everything together
        out.write(header + lsd + gct + id + chr(lctf | interlaced << 1 | sorted << 2) + chr(ncolors) + (lct or '') + datablock)
