#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Image Descriptors
    # We will use only one image descriptor, so we can skip the loop
    # We also don't need to worry about interlacing
    idd = struct.pack('>BBHHHBB', 0x2c, 0, 0, width, height, 0, rng.read(1)[0])

    # 5. Local Color Table (same as above)
    local_color_table = rng.read(3 * num_colors)

    # 6. Image Data
    # We will just write a single pixel with a random color index
    image_data = struct.pack('>B', rng.read(1)[0])

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + color_table + idd + local_color_table + image_data + trailer)

