#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors
    
    # 4. Graphics Control Extension (optional)
    gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    
    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    local_color_table = rng.read(1)  # Local color table flag
    interlace = rng.read(1)          # Interlaced image flag
    sort = rng.read(1)               # Sort flag
    
    idd = struct.pack('>HHHHBB', left, top, width, height, local_color_table[0], interlace[0])
    
    # 6. Local Color Table (optional)
    if local_color_table == b'\x01':
        num_colors = (rng.read(1)[0] % 256) or 1
        color_table = rng.read(3 * num_colors)  # Random colors
        
    # 7. Image Data
    data = rng.read()                   # Random data
    
    # 8. Trailer
    trailer = b';'
    
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + idd + color_table + data + trailer)
