#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Packed fields
    packed = 0x87 | ((width & 0b1111100000000000) >> 12) | ((height & 0b1111100000000000) >> 11)
    # Background color index
    background_color_index = rng.read(1)[0] % 256
    # Pixel aspect ratio
    pixel_aspect_ratio = 0
    # Global Color Table Flag
    global_color_table_flag = 0
    # Color Resolution
    color_resolution = 7
    # Sort Flag
    sort_flag = 0
    # Size of Global Color Table
    size_of_global_color_table = 7
    # End of fields
    end_fields = 0
    # Write to output
    out.write(struct.pack('<6sBBB', header, width, height, packed))
    out.write(struct.pack('BB', background_color_index, pixel_aspect_ratio))
    out.write(struct.pack('BB', global_color_table_flag, color_resolution))
    out.write(struct.pack('BB', sort_flag, size_of_global_color_table))
    out.write(struct.pack('B', end_fields))
    
    # 3. Global Color Table
    # Write 256 colors
    for i in range(256):
        # Each color is represented by three bytes
        out.write(rng.read(3))
        
    # 4. Image Descriptor
    # Separator
    separator = 0x2C
    # Left position
    left_position = rng.read(2)
    # Top position
    top_position = rng.read(2)
    # Width
    width = rng.read(2)
    # Height
    height = rng.read(2)
    # Packed fields
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    size_of_local_color_table = 0
    end_fields = 0
    out.write(struct.pack('BBHBBBB', separator, *left_position, *top_position, *width, *height, local_color_table_flag, interlace_flag, sort_flag, size_of_local_color_table))
    
    # 5. Local Color Table
    # Write 0 colors
    
    # 6. Image Data
    # LZW Minimum Code Size
    lzw_minimum_code_size = rng.read(1)[0] % 256
    out.write(struct.pack('B', lzw_minimum_code_size))
    
    # 7. Trailer
    trailer = 0x3B
    out.write(struct.pack('B', trailer))
