#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, duration=10):
    # Magic bytes
    out.write(b"GIF89a")

    # Logical screen descriptor
    out.write(struct.pack("<H", width))
    out.write(struct.pack("<H", height))
    out.write(b"\xF7\x00")  # Global color table flag (yes), color resolution (8 bits per pixel), sort flag (no)
    out.write(b"\xFF")      # Size of global color table (256 entries)
    out.write(b"\x00")      # Background color index
    out.write(b"\x00")      # Pixel aspect ratio (unused)

    # Application extension block
    out.write(b"!\xff\x0bNETSCAPE2.0\x03\x01")
    out.write(struct.pack(">I", duration * 100))
    out.write(b"\x00")

    # Image descriptor
    out.write(b",")
    out.write(struct.pack("<H", width))
    out.write(struct.pack("<H", height))
    out.write(b"\x00")                   # Local color table flag (no)
    out.write(b"\x00")                   # Interlace flag (no)
    out.write(b"\x00")                   # Sort flag (no)
    out.write(b"\xFF")                   # Size of local color table (256 entries)

    # Raster data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack("B", rng.read(1)[0]))

    # Trailer
    out.write(b";")
