#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Packed field
    #   - Global Color Table Flag: 0b10000000
    #   - Color Resolution: 0b00000111
    #   - Sort Flag: 0b00000000
    #   - Size of Global Color Table: 0b00000000
    packed = 0b10000000 | (0b00000111 & rng.read(1)[0]) << 7
    
    # Background color index
    background_color_index = rng.read(1)[0]
    
    # Pixel aspect ratio
    pixel_aspect_ratio = rng.read(1)[0]
    
    # Write everything to the output stream
    out.write(header + struct.pack('<HHBBB', width, height, packed, background_color_index, pixel_aspect_ratio))
    
    # 3. Global Color Table
    # We don't need this because we are not using it
    
    # 4. Image Descriptors
    # We will write one image descriptor per frame
    # Each image descriptor is 10 bytes long
    # The first byte is always 0x2C
    # The next three bytes are the image left position
    # The next three bytes are the image top position
    # The next three bytes are the image width
    # The next three bytes are the image height
    # The next byte is the packed field
    #   - Local Color Table Flag: 0b10000000
    #   - Interlace Flag: 0b00000000
    #   - Sort Flag: 0b00000000
    #   - Reserved: 0b00000000
    #   - Size of Local Color Table: 0b00000000
    # The next byte is the LZW minimum code size
    # The rest of the bytes are the compressed data
    # We will use the same image dimensions as the logical screen descriptor
    # We will use a local color table with 256 colors
    # We will not interlace the frames
    # We will use a minimum code size of 8 bits
    # We will use the LZW compression algorithm
    # We will use the default clear code of 0x11
    # We will use the default end-of-information code of 0x11
    # We will use the default code length of 9 bits
    # We will use the default initial code size of 8 bits
    # We will use the default maximum code size of 12 bits
    # We will use the default code size increment of 1 bit
    # We will use the default code size limit of 4096
    # We will use the default stack size of 4096
    # We will use the default maximum stack size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum prefix code of 4096
    # We will use the default maximum suffix code of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
    # We will use the default maximum code size of 4096
