#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    global_color_table_size = rng.read(1)[0] & 7 + 1
    global_color_table = rng.read(global_color_table_size * 3)
    
    # 4. Image Descriptors
    # We will use one image descriptor with no local color table
    idd = struct.pack('>BBHHBB', 0x2c, 0, 0, width, height, 0)
    
    # 5. Local Color Table (same as global color table)
    local_color_table = global_color_table
    
    # 6. Raster Data
    # We will just write random data
    raster_data = rng.read((width * height) // 8)
    
    # Write everything to the output file
    out.write(header + lsd + global_color_table + idd + local_color_table + raster_data)
