#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)


def generate(out: BinaryIO, rng):
    # Header
    out.write("GIF89a".encode())

    # Logical screen descriptor
    width = 256 + (rng.randint(0, 127)) * 256
    height = 256 + (rng.randint(0, 127)) * 256
    flags = 0b1000_0000 | ((rng.randint(0, 1) << 7) & 0b0100_0000) | \
        ((rng.randint(0, 1) << 6) & 0b0010_0000) | \
        ((rng.randint(0, 3) << 4) & 0b0001_1000) | \
        ((rng.randint(0, 7) << 0) & 0b0000_0111)
    bgColorIndex = rng.randint(0, 255)
    pixelAspectRatio = 0
    out.write(struct.pack("<HHBBB",
                          width, height, flags, bgColorIndex, pixelAspectRatio))

    # Global color table
    for i in range(0, 256):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(struct.pack("<BBBB", r, g, b, 0x00))

    # Image descriptor
    left = 0
    top = 0
    width = 256 + (rng.randint(0, 127)) * 256
    height = 256 + (rng.randint(0, 127)) * 256
    flags = 0b1000_0000 | ((rng.randint(0, 1) << 7) & 0b0100_0000) | \
        ((rng.randint(0, 1) << 6) & 0b0010_0000) | \
        ((rng.randint(0, 3) << 4) & 0b0001_1000) | \
        ((rng.randint(0, 7) << 0) & 0b0000_0111)
    delayTime = rng.randint(0, 65535)
    transparentColorIndex = rng.randint(0, 255)
    out.write(struct.pack("<HHHHBB",
                          left, top, width, height, flags, delayTime, transparentColorIndex))

    # Image data
    for y in range(height):
        for x in range(width):
            if rng.randint(0, 1) == 0:
                out.write(bytes([rng.randint(0, 255)]))
            else:
                out.write(bytes([transparentColorIndex]))
