#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 256 + (rng.randint(0, 100)) % 256
    height = 256 + (rng.randint(0, 100)) % 256
    flags = b'\x87\x00' if width > height else b'\x00\x00'
    bg_color = bytes([rng.randint(0, 255)]) * 3
    pixel_aspect_ratio = b'\x00'
    out.write(struct.pack('<HHBBB', width, height, flags[0], flags[1],
                          bg_color[0], bg_color[1]))
    out.write(pixel_aspect_ratio)

    # Global color table
    global_color_table_size = min((width + height), 256) // 4
    for i in range(global_color_table_size):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)

        out.write(bytes([r, g, b]) * 4)

    # Image descriptor
    left = 0
    top = 0
    flags = b'\x80\x00'
    local_color_table_size = 0
    interlace = False
    sort = True
    local_color_table = None
    delay = 10
    transparent_index = -1
    out.write(struct.pack('<HHBB', left, top, flags[0], flags[1]))
    out.write(struct.pack('B', local_color_table_size))
    out.write(struct.pack('?', interlace))
    out.write(struct.pack('?', sort))
    out.write(local_color_table or b'')
    out.write(struct.pack('<H', delay))
    out.write(struct.pack('B', transparent_index))

    # LZW minimum code size
    lzw_min_code_size = 2
    out.write(struct.pack('B', lzw_min_code_size))

    # Frame data
    frame_data_length = rng.randint(100, 1000)
    for _ in range(frame_data_length):
        byte = rng.randint(0, 255) & 0xFF
        out.write(bytes([byte]))

    # Trailer
    trailer = b';'
    out.write(trailer)


if