#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    header = b'GIF89a' + bytes([
        0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0xF0])
    
    out.write(header)

    # Logical screen descriptor
    logical_screen_descriptor = struct.pack('<HHBBB', 
        0x1234, 0x5678, 0xABCD, 0xEF, 0x01)

    out.write(logical_screen_descriptor)

    # Global color table (palette)
    global_color_table = [
        0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF] * 128

    for i in range(len(global_color_table)):
        out.write(bytes([
            global_color_table[i]]))

    # Image descriptor
    image_descriptor = struct.pack('<BBHHHBB', 
        0b10000000 | 0b00000000, 0x00, 0x00, 0x1234, 0x5678, 0x00, 0x00)

    out.write(image_descriptor)

    # Local color table (palette)
    local_color_table = []

    for _ in range(3):
        local_color_table += [
            0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF] * 128

    for i in range(len(local_color_table)):
        out.write(bytes([
            local_color_table[i]]))

    # Image data
    image_data = bytearray()

    while True:

        # Block size
        block_size = rng.read(1)[0] % 256
        
        if not block_size: break

        # LZW minimum code size
        lzw_minimum_code_size = rng.read(1)[0] % 256

        # Data sub-block
        data_sub_block = rng.read(block_size - 1)

        image_data += bytes([lzw_minimum_code_size]) + data_sub_block

    out.write(bytes([len(image_data)]))
    out.write(image_data)