#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_random(out: BinaryIO, width: int = 256, height: int = 256):
    # Header
    out.write("GIF89a".encode())

    # Logical screen descriptor
    out.write(struct.pack("<HHBBB",
        width, height,
        0x87,       # Global color table flag set
        0xFF,       # Background color index
        0))         # Pixel aspect ratio (1:1)

    # Color table
    for i in range(0, 256 * 3, 3):
        r = ord(os.urandom(1)) % 256
        g = ord(os.urandom(1)) % 256
        b = ord(os.urandom(1)) % 256

        out.write(struct.pack("<BBB", r, g, b))

    # Image descriptor
    out.write(struct.pack("<BHHHHBB",
        0x2C,        # Image separator
        width, height,
        0,           # Left position
        0,           # Top position
        0,           # Width of image block
        0,           # Height of image block
        0))          # LZW minimum code size

    # Image data
    lzw_min_code_size = 2
    while True:
        byte = ord(os.urandom(1)) & 0b11111111

        if byte == 0x00 or len(lzw_data) >= 2 ** lzw_min_code_size + 1:
            lzw_data += bytes([byte])

            if byte!= 0x00:
                lzw_min_code_size = min(len(lzw_data), 12)

            if len(lzw_data) > 2 ** lzw_min_code_size and len(lzw_data) < 2 ** lzw_min_code_size + 1:
                lzw_data += bytes([0x00])

            break

    out.write(bytes([lzw_min_code_size]))
    out.write(lzw_data)

