#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 768, color_depth: int = 8
):

    # Header
    out.write("GIF89a".encode())
    out.write(struct.pack("<HH", width, height))
    out.write((color_depth - 1).to_bytes(1, "little"))
    out.write("\x00\x00".encode())

    # Logical screen descriptor
    out.write(",".encode())
    out.write(struct.pack("<BBBHB", 0xFF, 0x0B, 0x00, 0x00, 0x00))
    out.write((width * height // (2 ** color_depth)).to_bytes(1, "little"))
    out.write("\x00".encode())

    # Global color table
    for i in range(2 ** color_depth):
        out.write(i.to_bytes(1, "little"))

    # Image descriptor
    out.write(";".encode())
    out.write(struct.pack("<BBHBBBB", 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))
    out.write((width % 256).to_bytes(1, "little"))
    out.write((height % 256).to_bytes(1, "little"))
    out.write("\x00".encode())
    out.write((color_depth - 1).to_bytes(1, "little"))
    out.write("\x00".encode())

    # Local color table
    for i in range(2 ** color_depth):
        out.write(i.to_bytes(1, "little"))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(y.to_bytes(1, "little") + x.to_bytes(1, "little"))

    # Trailer
    out.write(";".encode())
