#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 768, color_depth: int = 8
):

    # Header
    out.write("GIF89a".encode())
    out.write(struct.pack("<HH", width, height))
    out.write((color_depth - 1).to_bytes(1, byteorder="little"))
    out.write("\x00".encode())

    # Logical screen descriptor
    out.write("\xFF\x0B".encode())
    out.write(struct.pack("<HHBBBH", 0, 0, 0, 0, 0, 0))

    # Global color table (black and white)
    out.write("\xFF\x03".encode())
    out.write("\x00\x00\x00\xff\xff\xff".encode())

    # Image descriptor
    out.write("\x2c\x00\x00\x00".encode())
    out.write(struct.pack("<HHHHBB", 0, 0, width, height, 0, 0))

    # Local color table (black and white)
    out.write("\xFF\x03".encode())
    out.write("\x00\x00\x00\xff\xff\xff".encode())

    # Image data
    for y in range(height):
        for x in range(width):
            if ((y // 16 + x // 16) % 2 == 0):
                out.write("\xff".encode())
            else:
                out.write("\x00".encode())

