#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')  # Magic header
    width = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1
    flags = b'\x87\x00' if (width > 240 or height > 240) else b'\x00\x00'
    out.write(struct.pack('<HHB', width, height, ord(flags[0])))

    # Global color table
    for i in range(0, 2 ** ((ord(flags[1]) & 0x07) + 1)):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(bytes([r, g, b]))

    # Image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00')
    lzwmincodesize = rng.read(1)[0] % 256 + 1
    out.write(bytes([lzwmincodesize]))

    # LZW data
    # TODO: Implement this!
    pass

