#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write header
    out.write("GIF89a".encode())

    # Generate width and height
    w = 2 ** (1 + rng.read(1)[0] % 7)
    h = 2 ** (1 + rng.read(1)[0] % 7)

    # Write logical screen descriptor
    out.write(struct.pack("<HHBBB", w, h, 0x87, 0, 0))

    # Write global color table
    for _ in range(4):
        out.write(bytes([rng.read(1)[0] & 0xf]))

    # Write loop control block
    out.write(b"\x00\x00")

    # Write image descriptor
    out.write(struct.pack("<HHHHB", 0, 0, w - 1, h - 1, 0x87))

    # Write local color table
    for _ in range(3):
        out.write(bytes([rng.read(1)[0] & 0xf]))

    # Write graphics control extension
    out.write(b"\x21\xF9\x04\x05\x00\x00" + bytes([rng.read(1)[0]]))

    # Write plain text extension
    out.write(b"\x21\x01\x0F\x00")

    # Write application extension
    out.write(b"\x21\xFF\x0BNETSCAPE2.0")
    out.write(b"\x03\x01")
    out.write(b"\x00\x00")

    # Write comment extension
    out.write(b"\x21\xFE\x0CComment\x00")

    # Write graphic control label
    out.write(b"\x21\xF9\x04")

    # Write graphic control data
    out.write(bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]]))

    # Write plain text label
    out.write(b"\x21\x01\x0F")

    # Write plain text data
    out.write(b"\x00\x06")
    out.write(b"Plain Text")

    # Write end of file
    out.write(b";")
