#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng.randint(256 - 2)
    height = 1 + rng.randint(256 - 2)
    packed = (0x80 | ((width & 0x07) << 4) | (height & 0x07))
    out.write(struct.pack('<BHH', packed, width, height))

    # Global color table
    for i in range(0, 2 ** int((packed & 0x07) + 1)):
        r = rng.randint(256)
        g = rng.randint(256)
        b = rng.randint(256)
        out.write(struct.pack('<BBB', r, g, b))

    # Image descriptor
    out.write(struct.pack('<BBHBBHHH', 0x2C, 0, 0, 0, 0, 0, width, height))

    # Local color table
    for i in range(0, 2 ** int((packed & 0x07) + 1)):
        r = rng.randint(256)
        g = rng.randint(256)
        b = rng.randint(256)
        out.write(struct.pack('<BBB', r, g, b))

    # Image data
    for y in range(0, height):
        for x in range(0, width):
            out.write(bytes([rng.randint(256)]))

