#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Use 0 as color resolution, 0 as background color index,
    # 1 as pixel aspect ratio
    lsd = struct.pack('<6HBBBBB',
                      0, 0,
                      width, height,
                      0b00000000, 0b00000000,
                      0b00000001)

    # 3. Global Color Table
    global_color_table = bytearray()
    if rng.readinto(global_color_table):
        raise RuntimeError("Failed to read from /dev/urandom")

    # 4. Image Descriptor
    image_descriptor = struct.pack('<BHHHHB',
                                   0b10000000 | len(global_color_table),
                                   x=0, y=0, w=width, h=height,
                                   local_color_table_size=0)

    # 5. Image Data
    image_data = bytearray()
    while True:
        # Read one byte of data
        d = rng.read(1)

        # If we got EOF, break
        if not d:
            break
        
        # Otherwise append it to our buffer
        image_data += d

    # 6. Trailer
    trailer = b'\x3B'

    # Write everything in order
    out.write(header + lsd + global_color_table +
              image_descriptor + image_data + trailer)

