#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 640
    height = 480
    color_table_size = 256
    global_color_table = True
    background_index = 0
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBBB',
                          width, height,
                          color_table_size,
                          global_color_table << 7 | background_index,
                          pixel_aspect_ratio))

    # Global color table (all black)
    for i in range(0, 256 * 3):
        out.write(b'\x00')

    # Image descriptor
    left = 0
    top = 0
    width = 160
    height = 120
    local_color_table = False
    interlace = False
    sort = False
    reserved = b'\x00\x00'
    lct_size = 256
    out.write(struct.pack('<HHHHBB',
                          left, top,
                          width, height,
                          local_color_table << 7 | interlace << 6 | sort,
                          reserved[0], reserved[1]))

    # Local color table (all white)
    for i in range(0, 256 * 3):
        out.write(b'\xFF')

    # Graphics control extension
    block_size = 4
    packed = 0
    delay = 100
    transparent_index = None
    terminator = b'\x00'
    out.write(struct.pack('BBBB',
                          block_size,
                          packed,
                          delay & 0xff,
                          delay >> 8))
    if transparent_index is not None:
        out.write(transparent_index.to_bytes(1, byteorder='little'))
    else:
        out.write(terminator)

    # Plain text extension
    block_size = 12
    label_length = len("Comment") + 1
    out.write(block_size.to_bytes(1, byteorder='little'))
    out.write((label_length).to_bytes(1, byteorder='little'))
    out.write(b"Comment")
    out.write(b"\x00")
    out.write(b"Hello World!")
    out.write(terminator)

    # Application extension
    application_identifier = "NETSCAPE2.0".encode()
    application_auth_code = "<PASSWORD>".encode()
    sub_block_size = 3
    netscape_extension_block_id = b'\x01'
    block_size = 11
    loop_count = 0
    out.write(sub_block_size.to_bytes(1, byteorder='little'))
    out.write(netscape_extension_block_id)
    out.write(block_size.to_bytes(1, byteorder='little'))
    out.write(application_identifier)
    out.write(application_auth_code)
    out.write(loop_count.to_bytes(2, byteorder='little'))
    out.write(terminator)

    # Graphic Control Extension
    block_size = 4
    packed = 0
    delay = 100
    transparent_index = None
    terminator = b'\x00'
    out.write(struct.pack('BBBB',
                          block_size,
                          packed,
                          delay & 0xff,
                          delay >> 8))
    if transparent_index is not None:
        out.write(transparent_index.to_bytes(1, byteorder='little'))
    else:
        out.write(terminator)

    # Image data
    image_data = []
    for y in range(height):
        for x in range(width):
            image_data += [rng.read(1)]
    out.write(b"".join(image_data))

