#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    w = rng.read(1)[0] % 40 + 10
    h = rng.read(1)[0] % 40 + 10

    # Color table size is always 7
    color_table_size = 7

    # Background color index is always 0
    background_color_index = 0

    # Pixel aspect ratio is always 0
    pixel_aspect_ratio = 0

    logical_screen_descriptor = \
            struct.pack('<6sHHBBB',
                        header,
                        0,
                        w,
                        h,
                        color_table_size,
                        background_color_index,
                        pixel_aspect_ratio
                    )

    # Write to output stream
    out.write(logical_screen_descriptor)


    # 3. Global Color Table
    global_color_table = [b'\x00\x00\x00'] * 256

    # Generate colors in RGB format
    for i in range(len(global_color_table)):
        r = rng.read(1)[0] & 0xff
        g = rng.read(1)[0] & 0xff
        b = rng.read(1)[0] & 0xff

        global_color_table[i] = struct.pack('BBB', r, g, b)

    # Write to output stream
    out.write(b''.join(global_color_table))


    # 4. Image Descriptor
    image_separator = b','

    left = rng.read(1)[0] % w
    top = rng.read(1)[0] % h
    width = rng.read(1)[0] % w - left + 1
    height = rng.read(1)[0] % h - top + 1

    packed = ((left & 0xf8) << 8) | ((top & 0xfe) << 9) | (width >> 3)
    packed |= (((height & 0xf8) << 3) | (8 - (width & 7))) << 12

    flags = 0xc0 | (rng.read(1)[0] & 0x07)

    image_descriptor = struct.pack('<BHHBB',
                                   image_separator,
                                   packed,
                                   height,
                                   width,
                                   flags
                                )

    # Write to output stream
    out.write(image_descriptor)


    # 5. Local Color Table
    local_color_table = []

    if len(local_color_table):
        pass


    # 6. Graphics Control Extension
    graphics_control_extension = b''


    # 7. Plain Text Extension
    plain_text_extension = b''


    # 8. Application Extension
    application_extension = b''


    # 9. Comment Extension
    comment_extension = b''


    # 10. Trailer
    trailer = b';'

    # Write to output stream
    out.write(trailer)


