#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write("GIF89a".encode())

    width = 1024
    height = 768
    bitdepth = 8
    color_table_size = 256
    global_color_table = True
    background_index = 0
    aspect_ratio = b"\x00\x00"

    out.write(struct.pack("<HHBBB",
                          width,
                          height,
                          bitdepth,
                          color_table_size,
                          0))
    if not global_color_table:
        raise NotImplementedError()
    for i in range(color_table_size + 1):
        r = int.from_bytes(rng.read(1), byteorder="little") % 256
        g = int.from_bytes(rng.read(1), byteorder="little") % 256
        b = int.from_bytes(rng.read(1), byteorder="little") % 256

        out.write(struct.pack("<BBBB",
                              r,
                              g,
                              b,
                              0))

    out.write(b"," * (width // 8 * height))
    out.write(aspect_ratio)


if