#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    print("width:", width)
    print("height:", height)
    logicalScreenDescriptor = bytearray()
    logicalScreenDescriptor += struct.pack('B', width & 0xFF)
    logicalScreenDescriptor += struct.pack('B', (width >> 8) & 0xFF)
    logicalScreenDescriptor += struct.pack('B', height & 0xFF)
    logicalScreenDescriptor += struct.pack('B', (height >> 8) & 0xFF)
    logicalScreenDescriptor += struct.pack('B', 0xF7)

    # 3. Global Color Table
    globalColorTable = bytearray()
    globalColorTable += struct.pack('BBB', *struct.unpack('BBB', rng.read(3)))
    globalColorTable += struct.pack('BBB', *struct.unpack('BBB', rng.read(3)))
    globalColorTable += struct.pack('BBB', *struct.unpack('BBB', rng.read(3)))

    # 4. Image Descriptor
    imageDescriptor = bytearray()
    imageDescriptor += struct.pack('B', 0x2C)    # Image Separator
    imageDescriptor += struct.pack('B', 0x00)    # Left Position
    imageDescriptor += struct.pack('B', 0x00)    # Top Position
    imageDescriptor += struct.pack('B', width & 0xFF)   # Width
    imageDescriptor += struct.pack('B', (width >> 8) & 0xFF)
    imageDescriptor += struct.pack('B', height & 0xFF)  # Height
    imageDescriptor += struct.pack('B', (height >> 8) & 0xFF)
    imageDescriptor += struct.pack('B', 0x00)    # Flags
    imageDescriptor += struct.pack('B', 0x00)    # Background Color Index
    imageDescriptor += struct.pack('B', 0x00)    # Pixel Aspect Ratio

    # 5. Local Color Table
    localColorTable = bytearray()
    localColorTable += struct.pack('BBB', *struct.unpack('BBB', rng.read(3)))
    localColorTable += struct.pack('BBB', *struct.unpack('BBB', rng.read(3)))
    localColorTable += struct.pack('BBB', *struct.unpack('BBB', rng.read(3)))

    # 6. Image Data
    imageData = bytearray()
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b11100000 | ((rng.read(1)[0] % 8) + 1))
    imageData += struct.pack('B', 0b