#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    out.write("GIF89a".encode())
    width = 1 + int(rng.read(1)[0] % 256)
    height = 1 + int(rng.read(1)[0] % 256)
    out.write(struct.pack("<HH", width, height))

    # Global Color Table
    global_color_table_size = (
        2 ** ((int(rng.read(1)[0]) & 7) + 1)
    )
    out.write((global_color_table_size * 3).to_bytes(1, "little"))
    for _ in range(global_color_table_size):
        r = int(rng.read(1)[0] % 256)
        g = int(rng.read(1)[0] % 256)
        b = int(rng.read(1)[0] % 256)
        out.write(r.to_bytes(1, "little") +
                  g.to_bytes(1, "little") +
                  b.to_bytes(1, "little"))

    # Loop Control Block
    loop_count = int(rng.read(1)[0] % 256)
    out.write(b"\x00\x00" +
              loop_count.to_bytes(2, "little"))

    # Image Descriptor
    left = 0
    top = 0
    width -= 1
    height -= 1
    packed = (left | (top << 4)).to_bytes(1, "little")
    packed += (width | (height << 4)).to_bytes(1, "little")
    packed += (int(rng.read(1)[0] % 256) |
               (((int(rng.read(1)[0]) % 256) & 0x07) << 4)).to_bytes(1, "little")
    packed += (int(rng.read(1)[0] % 256) |
               (((int(rng.read(1)[0]) % 256) & 0x03) << 6)).to_bytes(1, "little")
    out.write(packed)

    # Local Color Table
    local_color_table_size = (
        2 ** ((int(rng.read(1)[0]) & 7) + 1)
    )
    out.write((local_color_table_size * 3).to_bytes(1, "little"))
    for _ in range(local_color_table_size):
        r = int(rng.read(1)[0] % 256)
        g = int(rng.read(1)[0] % 256)
        b = int(rng.read(1)[0] % 256)
        out.write(r.to_bytes(1, "little") +
                  g.to_bytes(1, "little") +
                  b.to_bytes(1, "little"))

    # Image Data
    image_data_length = width * height
    out.write(((image_data_length // 255) + 1).to_bytes(1, "little"))
    for i in range(image_data_length):
        out.write(
            (
                int(rng.read(1)[0] % 256),
                int(rng.read(1)[0] % 256)
            )
        )

    # Trailer
    out.write(";".encode())


if