#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Write the header
    out.write(b'GIF89a')
    
    # Read the width and height from the RNG
    width = int.from_bytes(rng.read(2), 'big', signed=False) % 100 + 1
    height = int.from_bytes(rng.read(2), 'big', signed=False) % 100 + 1
    print('width={}, height={}'.format(width, height))
    out.write(struct.pack('<HHB', width, height, 0xF7))
    
    # Write the global color table
    for i in range(0, 768, 3):
        out.write(rng.read(1))
        
    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x05\xff\xff\xff\x00')
    
    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # Write the local color table
    for i in range(0, 768, 3):
        out.write(rng.read(1))
        
    # Write the pixel data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))
            
    # Write the trailer
    out.write(b'\x3b')
