#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    out.write(b"GIF89a")
    
    # 2. Logical screen descriptor
    width = int.from_bytes(rng.read(2), "big", signed=False)
    height = int.from_bytes(rng.read(2), "big", signed=False)
    packed = int.from_bytes(rng.read(1), "big", signed=False)
    global_color_table_flag = (packed & 0x80) != 0
    color_resolution = (packed >> 4) & 0x7
    sort_flag = (packed >> 3) & 0x1
    size_of_global_color_table = 2 ** ((packed & 0x7) + 1)
    background_color_index = int.from_bytes(rng.read(1), "big", signed=False)
    pixel_aspect_ratio = int.from_bytes(rng.read(1), "big", signed=False)
    
    if global_color_table_flag:
        for _ in range(size_of_global_color_table):
            out.write(rng.read(3))
            
    # 3. Global color table
        
    # 4. Image block
    while True:
        # 4.1 Image separator
        out.write(b"\x2c\x00\x00\x00")
        
        # 4.2 Image descriptor
        left = int.from_bytes(rng.read(2), "big", signed=False)
        top = int.from_bytes(rng.read(2), "big", signed=False)
        width = int.from_bytes(rng.read(2), "big", signed=False)
        height = int.from_bytes(rng.read(2), "big", signed=False)
        packed = int.from_bytes(rng.read(1), "big", signed=False)
        local_color_table_flag = (packed & 0x80) != 0
        interlace_flag = (packed & 0x40) != 0
        sort_flag = (packed & 0x20) != 0
        size_of_local_color_table = 2 ** ((packed & 0x7) + 1)
        
        if local_color_table_flag:
            for _ in range(size_of_local_color_table):
                out.write(rng.read(3))
                
        # 4.3 Local color table
        
        # 4.4 Image data
        code_size = int.from_bytes(rng.read(1), "big", signed=False)
        while True:
            code = int.from_bytes(rng.read(1), "big", signed=False)
            if code == 0xFF:
                break
            else:
                pass
                
        # 4.5 Block terminator
        
        # 4.6 Trailer
        if not interlace_flag:
            break
        elif top >= height:
            break
        else:
            pass
            
    # 5. Trailer
    out.write(b"\x3B")
