#!/usr/bin/env python3
import argparse
import os
import random
import struct
from PIL import Image


def generate_gif(out, width=640, height=480, duration=5):
    """Generate a GIF file."""
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    lsd = struct.pack('<HHHBB', width, height, duration * 10, 0, 0)

    # 3. Global Color Table
    color_table = (b'\xFF\x00\x00\x00' +  # Red
                   b'\x00\xFF\x00\x00' +  # Green
                   b'\x00\x00\xFF\x00')   # Blue

    # 4. Graphic Control Extension
    gce = struct.pack('<BBBB', 0xF9, 0, 0, duration * 10)

    # 5. Image Descriptor
    image_descriptor = struct.pack('<HHHHB', 0, 0, width, height, 0)

    # 6. Image Data
    rng = random.Random()
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while len(blocks) < total_blocks_size:
        block_size = min(total_blocks_size - len(blocks), rng.read(1)[0])

        # We want to be able to encode all possible data with the given minimum code size
        # So we need to ensure that our data is always smaller than 2**lzw_minimum_code_size
        # If not, we just add some padding until this condition holds true
        while True:
            if block_size > 2 ** lzw_minimum_code_size:
                break

            block_size += rng.read(1)[0]

        blocks.append(rng.read(block_size))

    image_data = struct.pack('>BB', lzw_minimum_code_size, 0) + b''.join(blocks)

    # 7. Trailer
    trailer = b'\x3B'

    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
