#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Color Table
    # We use an empty color table, since we don't need any colors
    color_table = b''

    # 4. Graphic Control Extension
    # We use a single-byte block with no transparency
    gce = struct.pack('BBB', 0xF9, 0x04, 0x00)

    # 5. Image Descriptor
    # We use a single-byte block with no local color table
    image_descriptor = struct.pack('BBHBB', 0x2C, 0x00, width, height, 0x00)

    # 6. Image Data
    # We use a single-byte block with all pixels set to white
    image_data = struct.pack('B', 0xFF) * (width * height)

    # 7. Trailer
    # We use a single-byte block with no data
    trailer = b';'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)

