#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width, height = 640, 480
    packed = struct.pack('<HHBB', width, height, 0xF7, 0)

    # 3. Global Color Table
    global_color_table = [
        0xFF0000, 0xFFFF00, 0x00FF00, 0x00FFFF, 0x0000FF, 0xFF00FF
    ]
    color_table = []
    for c in global_color_table:
        r, g, b = (c >> 16) & 0xFF, (c >> 8) & 0xFF, c & 0xFF
        color_table += [r, g, b]

    # 4. Image Descriptor
    image_descriptor = struct.pack('<BHHHBB', 0x2C, 0, 0, width, height, 0)

    # 5. Local Color Table
    local_color_table = []
    for i in range(3):
        local_color_table += [
            *struct.unpack('<BBB', rng.read(3))
        ]

    # 6. Image Data
    image_data = []
    for y in range(height):
        for x in range(width):
            image_data += [*struct.unpack('<BBB', rng.read(3))]

    # Write everything to `out`
    out.write(header + packed)
    out.write(bytes(color_table))
    out.write(image_descriptor)
    out.write(bytes(local_color_table))
    out.write(bytes(image_data))
