#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    # We will use a global color table with 256 colors
    gct = rng.read(768)
    
    # 4. Image Data
    # We will use a single image with no local color table
    id = b'\x00\x00\x00\x00\x00,\x00\x00\x00!\xf9\x04\x01\x00\x00\x00,\x00\x00\x00!\x04\x00;\x00\x00\x00!\x01\x00\x01w\x00;\xff\x0b\xfe\x0c\xfe\x0d\xfe\x0e\xfe\x0f\xfe\x10\xfe\x11\xfe\x12\xfe\x13\xfe\x14\xfe\x15\xfe\x16\xfe\x17\xfe\x18\xfe\x19\xfe\x1a\xfe\x1b\xfe\x1c\xfe\x1d\xfe\x1e\xfe\x1f\xfe !\xfe"\xfe#\xfe$\xfe%\xfe&\xfe\'\xfe(\xfe)\xfe*\xfe+\xfe,\xfe-\xfe.\xfe/\xfe0\xfe1\xfe2\xfe3\xfe4\xfe5\xfe6\xfe7\xfe8\xfe9\xfe:\xfe;\xfe<\xfe=\xfe>\xfe?\xfe@\xfeA\xfeB\xfeC\xfeD\xfeE\xfeF\xfeG\xfeH\xfeI\xfeJ\xfeK\xfeL\xfeM\xfeN\xfeO\xfeP\xfeQ\xfeR\xfeS\xfeT\xfeU\xfeV\xfeW\xfeX\xfeY\xfeZ\xfe[\xfe\\\xfe]\xfe^\xfe_\xfe`\xfea\xfe