#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    flags = rng.read(1)[0] & 0xE0 | 0x1F
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBB', width, height, flags, background))

    # Global color table
    global_color_table_size = 2 ** ((flags >> 4) + 1)
    for i in range(global_color_table_size):
        red = rng.read(1)[0]
        green = rng.read(1)[0]
        blue = rng.read(1)[0]
        out.write(bytes([red, green, blue]))

    # Image data
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00\xFF\xFF\xFF\x00'
    out.write(image_descriptor)

    local_color_table_size = 2 ** ((flags >> 4) + 1)
    for i in range(local_color_table_size):
        red = rng.read(1)[0]
        green = rng.read(1)[0]
        blue = rng.read(1)[0]
        out.write(bytes([red, green, blue]))

    # Pixel data
    pixel_data = []
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % local_color_table_size
            pixel_data.append(index)

    out.write(bytes(pixel_data))

    # Trailer
    out.write(b';')

