#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, duration=1.0):
    # Header
    out.write(b'GIF89a')  # Magic bytes
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\xF7\x00\x00')  # 8-bit color table (not sorted), no transparency

    # Logical screen descriptor
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<I', int(duration * 100)))

    # Image data
    out.write(b'\x2C\x00\x00\x00\x00')  # Left, top, width, height
    out.write(b'\x00')                  # No local color table
    out.write(b'\x02')                  # 2 bits per pixel
    out.write(b'\x02')                  # 2 interlacing passes

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng.read(1)[0] & 0x3]))

    # Trailer
    out.write(b';')
