#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    out.write(b'GIF89a')
    
    # 2. Logical screen descriptor
    width = int.from_bytes(rng.read(2), 'little', signed=False) % 640 + 1
    height = int.from_bytes(rng.read(2), 'little', signed=False) % 480 + 1
    color_table_size = 7 + (int.from_bytes(rng.read(1), 'big', signed=False) & 0x07)
    background_color_index = int.from_bytes(rng.read(1), 'big', signed=False)
    pixel_aspect_ratio = int.from_bytes(rng.read(1), 'big', signed=False)
    out.write(struct.pack('<HBBB', width, height, color_table_size, background_color_index))
    out.write(pixel_aspect_ratio.to_bytes(1, 'big'))
    
    # 3. Global color table
    for _ in range(2 ** color_table_size):
        out.write(rng.read(3))
        
    # 4. Graphics control extension
    out.write(b'\x21\xf9\x04\x08\x00\x00\xff\xff\xff\xff')
    
    # 5. Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # 6. Local color table
    for _ in range(2 ** color_table_size):
        out.write(rng.read(3))
        
    # 7. Image data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))
            
    # 8. Trailer
    out.write(b';')
