#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Packed fields:
    # - Global Color Table Flag: 1
    # - Color Resolution: 7
    # - Sort Flag: 0
    # - Size of Global Color Table: 7
    packed = 0b11100000 | ((width >> 8) & 0xFF)
    out.write(struct.pack('B', packed))
    out.write(struct.pack('B', width & 0xFF))
    out.write(struct.pack('B', (height >> 8) & 0xFF))
    out.write(struct.pack('B', height & 0xFF))

    # Background color index
    out.write(struct.pack('B', rng.read(1)[0]))

    # Pixel aspect ratio
    out.write(struct.pack('B', rng.read(1)[0]))

    # 3. Global Color Table
    # 768 bytes
    out.write(rng.read(768))

    # 4. Blocks
    while True:
        block_type = rng.read(1)[0]

        if block_type == 0x3B:
            break

        elif block_type == 0x2C:
            # Image Descriptor
            # Random left and top coordinates (two bytes each)
            left, top = struct.unpack('>HH', rng.read(4))

            # Width and height (two bytes each)
            width, height = struct.unpack('>HH', rng.read(4))

            # Packed fields:
            # - Local Color Table Flag: 1
            # - Interlace Flag: 0
            # - Sort Flag: 0
            # - Reserved: 0
            # - Size of Local Color Table: 7
            packed = 0b11100000 | ((left >> 8) & 0xFF)
            out.write(struct.pack('B', packed))
            out.write(struct.pack('B', left & 0xFF))
            out.write(struct.pack('B', (top >> 8) & 0xFF))
            out.write(struct.pack('B', top & 0xFF))
            out.write(struct.pack('B', (width >> 8) & 0xFF))
            out.write(struct.pack('B', width & 0xFF))
            out.write(struct.pack('B', (height >> 8) & 0xFF))
            out.write(struct.pack('B', height & 0xFF))

            # LZW minimum code size
            out.write(struct.pack('B', rng.read(1)[0]))

            # 5. Local Color Table
            # 768 bytes
            out.write(rng.read(768))

            # 6. Data sub-blocks
            while True:
                data_size = rng.read(1)[0]
