#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors
    
    # 4. Image Data
    # We will use a single image with no local color table, so we can skip that part
    # We will also use a single frame, so we can skip that part too
    # We will use a single graphic control extension block, so we can skip that part as well
    # The only thing left is to write the image descriptor
    idd = struct.pack('>BBHHHB', 0x2c, 0, 0, width, height, 0)
    
    # 5. Image Data
    # We will use a single image with no local color table, so we can skip that part
    # We will also use a single frame, so we can skip that part too
    # We will use a single graphic control extension block, so we can skip that part as well
    # The only thing left is to write the image data
    image_data = rng.read(width * height)  # Random pixels
    
    # 6. Trailer
    trailer = b'\x3b'
    
    # Write everything together
    out.write(header + lsd + color_table + idd + image_data + trailer)
