#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given dimensions.
def generate_gif(out: BinaryIO, width: int, height: int) -> None:
    # Write out the header.
    out.write('GIF89a'.encode())
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\x00\x00')  # Background color index.
    out.write(b'\x00')      # Pixel aspect ratio (unused).

    # Write out the global color table.
    for i in range(256):
        r = g = b = i
        out.write(bytes([r, g, b]))

    # Write out the image descriptor.
    out.write(b'!\xF9\x04\x01\x00\x00\x00\x00,\x00\x00\x00')

    # Write out the local color table.
    for i in range(256):
        r = g = b = i
        out.write(bytes([r, g, b]))

    # Write out the graphic control extension.
    out.write(b'\x21\xf9\x04\x01\x00\x01\x00')

    # Write out the image data.
    for y in range(height):
        for x in range(width):
            pixel = y * width + x
            out.write(bytes([pixel % 256]))

    # Write out the trailer.
    out.write(b';')

