#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Packed fields
    packed = 0x80 | ((width >> 7) & 0x7) << 4 | (height >> 7)
    # Background color index
    background_color_index = rng.read(1)[0] % 256
    # Aspect ratio
    aspect_ratio = 0

    # Write everything to the output stream
    out.write(header + struct.pack('<HBBB', width, height, packed, background_color_index, aspect_ratio))

    # 3. Global Color Table
    # We don't need one since we're not using any colors

    # 4. Image Data
    # We just write a single frame with no data
    out.write(struct.pack('<BBB', 0x2c, 0, 0))
    # The image descriptor
    out.write(b'\x00\x00\x00\x00\x00')
    # No local color table
    out.write(b'\x00')
    # LZW minimum code size
    out.write(b'\x08')
    # End of image
    out.write(b'\x00')
