#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # 1 bit of global color table flag
    gct_flag = rng.read(1)[0] & 1

    # 3 bits of color resolution
    color_resolution = rng.read(1)[0] & 7

    # 3 bits of sort flag
    sort_flag = rng.read(1)[0] & 7

    # 3 bits of size of global color table
    size_of_global_color_table = rng.read(1)[0] & 7

    # Background color index
    background_color_index = rng.read(1)[0]

    # Pixel aspect ratio
    pixel_aspect_ratio = rng.read(1)[0]

    # Write all this stuff to the output
    out.write(header + struct.pack('>HHBBB', width, height, gct_flag, color_resolution, sort_flag, size_of_global_color_table))
    out.write(bytes([background_color_index]))
    out.write(bytes([pixel_aspect_ratio]))

    # 3. Global Color Table
    if gct_flag == 1:
        # Read in the colors from /dev/urandom
        global_color_table = rng.read(3 * (2 ** (size_of_global_color_table + 1)))
        out.write(global_color_table)

    # 4. Image Data
    # Loop through every pixel
    for y in range(height):
        for x in range(width):
            # Get the color index for this pixel
            color_index = rng.read(1)[0]
            out.write(bytes([color_index]))

