#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    flags = rng.read(1)[0] & 0xE0 | 0x1F
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBB', width, height, flags, background))

    # Global color table
    global_color_table_size = 2 ** ((flags >> 4) + 1)
    for i in range(global_color_table_size):
        out.write(rng.read(3))

    # Image descriptor
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)[0]
    local_color_table_size = 2 ** ((flags >> 4) + 1)
    interlaced = bool(flags & 0x40)
    sorted = bool(flags & 0x80)
    out.write(left + top + width + height + bytes([flags]))

    # Local color table
    if not (flags & 0x80):
        for i in range(local_color_table_size):
            out.write(rng.read(3))

    # LZW minimum code size
    lzw_minimum_code_size = rng.read(1)[0]
    out.write(bytes([lzw_minimum_code_size]))

    # LZW data
    while True:
        block_size = rng.read(1)[0]
        if block_size == 0:
            break
        out.write(rng.read(block_size))

    # Trailer
    out.write(b'\x3B')

