#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')
    
    # Logical Screen Descriptor
    width = 1 + (next(rng) % 640)
    height = 1 + (next(rng) % 480)
    color_table_size = 7 + (next(rng) % 256)
    background_color_index = next(rng) % 256
    pixel_aspect_ratio = next(rng) % 256
    out.write(struct.pack('<HHBB', width, height, color_table_size, background_color_index))
    out.write(pixel_aspect_ratio.to_bytes(1, 'little'))
    
    # Global Color Table
    for i in range(2 ** color_table_size):
        red = next(rng) % 256
        green = next(rng) % 256
        blue = next(rng) % 256
        out.write(struct.pack('<BBB', red, green, blue))
        
    # Application Extension
    extension_introducer = b'\x21\xff'
    application_extension_label = b'NETSCAPE2.0'
    application_block_size = 0x03
    loop_count = 0xffff
    terminator = b'\x00'
    out.write(extension_introducer + application_extension_label + struct.pack('<BB', application_block_size, loop_count) + terminator)
    
    # Image Descriptor
    image_descriptor = b'\x2c'
    left = 0
    top = 0
    width = width - 1
    height = height - 1
    local_color_table_flag = False
    interlace_flag = True
    sort_flag = False
    reserved_bits = 0
    out.write(image_descriptor + struct.pack('<HHBB', left, top, width, height) + bytes([local_color_table_flag, interlace_flag, sort_flag, reserved_bits]))
    
    # Local Color Table
    for i in range(2 ** color_table_size):
        red = next(rng) % 256
        green = next(rng) % 256
        blue = next(rng) % 256
        out.write(struct.pack('<BBB', red, green, blue))
    
    # LZW Minimum Code Size
    lzw_minimum_code_size = 0x08
    out.write(lzw_minimum_code_size.to_bytes(1, 'little'))
    
    # Pixel Data
    while True:
        code = next(rng) % 2**lzw_minimum_code_size
        if code == 0x00:
            break
        else:
            out.write(code.to_bytes(1, 'little'))
