#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width = rng.read(2)
    height = rng.read(2)
    packed = rng.read(1)
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]

    # 3. Global Color Table
    color_table_size = (packed[0] & 0b00000111) + 1
    color_resolution = ((packed[0] >> 4) & 0b00000011) + 1
    sort_flag = bool((packed[0] >> 6) & 0b00000001)
    global_color_table = [
            rng.read(3) for _ in range(2 ** color_table_size)]

    # 4. Image Data
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\x00'
    image_data = []
    while True:
        control = rng.read(1)
        if not control:
            break
        else:
            image_data += [control]
            block_size = rng.read(1)[0]
            image_data += [block_size]
            if block_size == 0:
                break
            image_data += [rng.read(block_size)]

    # Write everything to the output stream
    out.write(header)
    out.write(width)
    out.write(height)
    out.write(packed)
    out.write(background.to_bytes(1, 'big'))
    out.write(aspect_ratio.to_bytes(1, 'big'))
    for color in global_color_table:
        out.write(color)
    out.write(image_descriptor)
    out.write(struct.pack('<H', int.from_bytes(width, 'big') * int.from_bytes(height, 'big')))
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x87\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.