#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), 'little') % (65536 - 7) + 7
    height = int.from_bytes(rng.read(2), 'little') % (65536 - 7) + 7
    color_table_size = 0b111
    background_color_index = 0
    pixel_aspect_ratio = 0
    global_color_table_flag = True
    color_resolution = 0b11
    sort_flag = False
    size_of_global_color_table = 0b11
    global_color_table = [
        0xff0000, 0xffff00, 0x00ff00, 0x00ffff, 0x0000ff, 0xff00ff,
        0xffffff, 0x000000
    ][:color_table_size]

    logical_screen_descriptor = struct.pack('<HBBB',
        width, height,
        (background_color_index << 4) | (pixel_aspect_ratio & 0xf),
        ((global_color_table_flag << 7) |
            (color_resolution << 4) |
            (sort_flag << 3) |
            (size_of_global_color_table))
    )

    # 3. Global Color Table
    for color in global_color_table:
        out.write(struct.pack('<H', color))

    # 4. Image Descriptors
    image_descriptor = struct.pack('<HHBBB',
        0, 0, width, height,
        (local_color_table_flag << 7) |
            (interlace_flag << 6) |
            (sort_flag << 5) |
            (reserved << 3) |
            (size_of_local_color_table)
    )

    # 5. Local Color Table
    local_color_table = []
    if local_color_table_flag:
        for i in range(size_of_local_color_table):
            local_color_table.append(int.from_bytes(rng.read(3), 'big'))

    # 6. Image Data
    image_data = []
    for y in range(height):
        for x in range(width):
            image_data.append(rng.read(1)[0])

    # Write everything to the output file
    out.write(header)
    out.write(logical_screen_descriptor)
    out.write(image_descriptor)
    for color in local_color_table:
        out.write(struct.pack('<H', color))
    out.write(bytearray(image_data))
