#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Packed fields
    # Global color table flag (0b10000000)
    # Color resolution (0b00000111)
    # Sort flag (0b00000000)
    # Size of global color table (0b00000000)
    packed_fields = 0b10000000 | (0b00000111 << 4) | 0b00000000

    # Background color index
    background_color_index = rng.read(1)[0] % 256

    # Pixel aspect ratio
    pixel_aspect_ratio = rng.read(1)[0] % 256

    # Write everything to the output file
    out.write(header + struct.pack('>HBBB', width, height, packed_fields, background_color_index, pixel_aspect_ratio))

    # 3. Global Color Table
    # We don't use it, so we just write zeros
    out.write(bytes([0] * (768)))

    # 4. Image Data
    # We don't use it, so we just write zeros
    out.write(bytes([0] * (width * height)))
