#!/usr/bin/env python3

import struct
from typing import BinaryIO
from random import Random

def main() -> None:
    rng = Random(0xdeadbeef)
    with open("test.gif", "wb") as f:
        write_gif(f, rng)

def write_gif(out: BinaryIO, rng: Random) -> None:
    # Header
    out.write(b"GIF89a")

    # Logical screen descriptor
    left = bytes([0])
    top = bytes([0])
    width = bytes([255])
    height = bytes([255])
    flags = 0b11000000 | 7   # Global color table size is 8 bits
    local_color_table_size = (flags >> 4) + 1
    interlaced = bool((flags >> 6) & 1)
    out.write(left + top + width + height + bytes([flags]))

    # Local color table
    for i in range(2 ** local_color_table_size):
        out.write(rng.read(3))

    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

