#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header (6 bytes)
    out.write(b'GIF89a')

    # Logical screen descriptor (7 bytes)
    width = 1 + int.from_bytes(rng.read(1), 'little') % 250
    height = 1 + int.from_bytes(rng.read(1), 'little') % 250
    flags = b'\x87\x00' if bool(int.from_bytes(rng.read(1), 'little')) else b'\x07\x00'
    bg_color = rng.read(1)
    pixel_aspect_ratio = b'\x00'
    out.write(width.to_bytes(2, byteorder='little', signed=False))
    out.write(height.to_bytes(2, byteorder='little', signed=False))
    out.write(flags)
    out.write(bg_color)
    out.write(pixel_aspect_ratio)

    # Global color table (3 bytes)
    global_color_table_size = 2 ** ((int.from_bytes(rng.read(1), 'little') & 0x07) + 1)
    background_index = int.from_bytes(rng.read(1), 'little') % global_color_table_size
    sort_flag = False if not bool(int.from_bytes(rng.read(1), 'little')) else True
    global_color_table = [rng.read(3) for _ in range(global_color_table_size)]
    out.write((global_color_table_size).to_bytes(1, byteorder='little', signed=False))
    out.write(background_index.to_bytes(1, byteorder='little', signed=False))
    out.write(sort_flag.to_bytes(1, byteorder='little', signed=False))
    for entry in global_color_table:
        out.write(entry)

    # Image descriptor (9 bytes)
    left = 0
    top = 0
    width = 1 + int.from_bytes(rng.read(1), 'little') % 250
    height = 1 + int.from_bytes(rng.read(1), 'little') % 250
    flags = b'\x87\x00' if bool(int.from_bytes(rng.read(1), 'little')) else b'\x07\x00'
    delay_time = int.from_bytes(rng.read(2), 'little')
    transparent_color_index = int.from_bytes(rng.read(1), 'little') % global_color_table_size
    out.write(left.to_bytes(2, byteorder='little', signed=False))
    out.write(top.to_bytes(2, byteorder='little', signed=False))
    out.write(width.to_bytes(2, byteorder='little', signed=False))
    out.write(height.to_bytes(2, byteorder='little', signed=False))
    out.write(flags)
    out.write(delay_time.to_bytes(2, byteorder='little', signed=False))
    out.write(transparent_color_index.to_bytes(1, byteorder='little', signed=False))

    # Local color table (3 bytes)
    local_color_table_size = 2 ** ((int.from_bytes(rng.read(1), 'little') & 0x07) + 1)
    sort_flag = False if not bool(int.from_bytes(rng.read(1), 'little')) else True
    local_color_table = [rng.read(3) for _ in range(local_color_table_size)]
    out.write((local_color_table_size).to_bytes(1, byteorder='little', signed=False))
    out.write(sort_flag.to_bytes(1, byteorder='little', signed=False))
    for entry in local_color_table:
        out.write(entry)

    # Image data block (1 byte)
    image_data_block_size = 2 ** (((int.from_bytes(rng.read(1), 'little') >> 4) & 0x07) + 1)
    lzw_min_code_size = int.from_bytes(rng.read(1), 'little')
    out.write(((image_data_block_size