#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header (6 bytes):
    #    - Signature ("GIF89a")
    #    - Version ("\x00\x20")
    #    - Width (2 bytes)
    #    - Height (2 bytes)
    #    - Flags (1 byte)
    header = b"GIF89a\x00\x20" + \
             struct.pack(">HHB", *map(lambda _: int(rng.read(2).hex(), base=16), range(4)))

    # 2. Logical Screen Descriptor (7 bytes):
    #    - Background Color Index (1 byte)
    #    - Pixel Aspect Ratio (1 byte)
    #    - Global Color Table Flag (1 bit)
    #    - Sort Flag (1 bit)
    #    - Size of Global Color Table (3 bits)
    #    - Number of Colors in Global Color Table (3 bits)
    lsd = struct.pack("BBB", *[int(rng.read(1).hex(), base=16)]*3)

    # 3. Image Descriptor (10 bytes):
    #    - Left Position (2 bytes)
    #    - Top Position (2 bytes)
    #    - Width (2 bytes)
    #    - Height (2 bytes)
    #    - Flags (1 byte)
    #    - Local Color Table Flag (1 bit)
    #    - Interlace Flag (1 bit)
    #    - Sort Flag (1 bit)
    #    - Reserved (3 bits)
    idd = struct.pack(">HHHHBB", *map(lambda _: int(rng.read(2).hex(), base=16), range(5)))

    # 4. LZW Minimum Code Size (1 byte)
    mincodesize = int(rng.read(1).hex(), base=16)

    # 5. Data Sub-blocks
    data = []
    while True:
        # Generate a random length for this subblock
        blocklen = int(rng.read(1).hex(), base=16)

        if blocklen == 0:
            break

        data += [int(rng.read(blocklen).hex(), base=16)]

    # Write everything to the output stream
    out.write(header)
    out.write(lsd)
    out.write(idd)
    out.write(struct.pack("B", mincodesize))
    for i in data:
        out.write(bytes([i]))


if