#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Color depth is fixed at 8 bits per pixel
    color_depth = b'\x08\x00'
    # Global palette with 256 colors
    global_palette = b'\xFF' * 768
    # Background color index is always zero
    background_color_index = b'\x00'
    # Pixel aspect ratio is always 1:1
    pixel_aspect_ratio = b'\x00\x00'
    logical_screen_descriptor = \
            width + height + color_depth + global_palette + \
                    background_color_index + pixel_aspect_ratio

    # 3. Image Descriptor
    # Frame starts at byte offset 13
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00'

    # 4. Graphics Control Extension
    graphics_control_extension = b'\x21\xF9\x04\x00\x00\x00\x00\x00\x2C\x00\x00'

    # 5. Application Extension
    application_extension = b'\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00'

    # 6. Comment Extension
    comment_extension = b'\x21\xFE\x0DComment String!\x00'

    # 7. Plain Text Extension
    plain_text_extension = b'\x21\x01\x0FHello World!\x00'

    # 8. Trailer
    trailer = b';'

    # Write all of them to the output stream
    out.write(header)
    out.write(logical_screen_descriptor)
    out.write(image_descriptor)
    out.write(graphics_control_extension)
    out.write(application_extension)
    out.write(comment_extension)
    out.write(plain_text_extension)
    out.write(trailer)

