#!/usr/bin/env python3

import struct
from random import Random, SystemRandom

def write_bmp(out, width, height):
    rng = Random()
    rng.seed(SystemRandom().getrandbits(64))
    
    header = bytearray([
        0x42, 0x4d,     # Signature
        0x58,           # File size (little-endian)
        0x00, 0x00,     # Reserved
        0x00, 0x00,     # Reserved
        0x7c, 0x00, 0x00, 0x00,   # Offset to pixel data
        0x00, 0x00, 0x00, 0x00,   # Header size
        0x00, 0x00, 0x00, 0x00,   # Image width
        0x00, 0x00, 0x00, 0x00,   # Image height
        0x01, 0x00,     # Number of color planes
        0x18, 0x00,     # Bits per pixel
        0x00, 0x00, 0x00, 0x00,   # Compression method
        0x00, 0x00, 0x00, 0x00,   # Raw image size
        0x00, 0x00, 0x00, 0x00,   # Horizontal resolution
        0x00, 0x00, 0x00, 0x00,   # Vertical resolution
        0x00, 0x00, 0x00, 0x00,   # Colors in palette
        0x00, 0x00, 0x00, 0x00    # Important colors in palette
    ])

    # Write the file size field
    header[2] = ((len(header) + len(b'\x00' * width[1]) - 54) % 2**16) & 0xff
    header[3] = (((len(header) + len(b'\x00' * width[1]) - 54) // 2**16) & 0xff) & 0xff

    # Write the image dimensions
    header[18] = width[0] & 0xff
    header[19] = (width[0] >> 8) & 0xff
    header[20] = height[0] & 0xff
    header[21] = (height[0] >> 8) & 0xff

    # Write the offset to the pixel data
    header[10] = ((54 + len(b'\x00' * width[1])) % 2**16) & 0xff
    header[11] = (((54 + len(b'\x00' * width[1])) // 2**16) & 0xff) & 0xff

    # Write the raw image size
    header[22] = len(b'\x00' * width[1]) & 0xff
    header[23] = (len(b'\x00' * width[1]) >> 8) & 0xff
    header[24] = len(b'\x00' * width[1]) & 0xff
    header[25] = (len(b'\x00' * width[1]) >> 8) & 0xff

    # Write the horizontal and vertical resolutions
    header[26] = 0x13
    header[27] = 0x0B
    header[28] = 0x00
    header[29] = 0x00

    # Write the number of important colors in the palette
    header[30] = 0xFF
    header[31] = 0x00

    out.write(header)

    # Color table
    for i in range(256):
        out.write(rng.read(3))

    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

