#!/usr/bin/env python3

import struct
from typing import BinaryIO


def encode(out: BinaryIO, rng: BinaryIO):
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)[0] & 0xC0 | 0x07
    local_color_table_size = (flags >> 4) + 1
    interlaced = bool((flags >> 6) & 1)
    out.write(left + top + width + height + bytes([flags]))

    # Local color table
    for i in range(2 ** local_color_table_size):
        out.write(rng.read(3))

    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

