#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    flags = rng.read(1)[0] & 0xE0 | 0x1F
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBBB',
                          width, height, flags, background, aspect_ratio))

    # Global color table
    global_color_table_size = 2 ** ((flags >> 4) & 0x07)
    if global_color_table_size > 0:
        for _ in range(global_color_table_size):
            out.write(bytes([rng.read(1)[0],
                             rng.read(1)[0],
                             rng.read(1)[0]]))

    # Image block
    left = 0
    top = 0
    width = [left, left + width - 1]
    height = [top, top + height - 1]
    packed_fields = bytes([0xF7])
    local_color_table_flag = bool(rng.read(1)[0] & 0x80)
    interlace_flag = bool(rng.read(1)[0] & 0x40)
    sort_flag = bool(rng.read(1)[0] & 0x20)
    reserved = rng.read(1)[0] & 0x18 | 0x07
    lct_size = 2 ** ((reserved >> 3) & 0x07)
    lct_sort_flag = bool((lct_size << 3) & 0x80)
    lct_interlace_flag = bool((lct_size << 3) & 0x40)
    lct_reserved = (lct_size << 3) & 0x38 | 0x07
    lct_size = 2 ** (lct_size & 0x07)
    out.write(struct.pack('<HHHHBB',
                          *width, *height, packed_fields,
                          local_color_table_flag, interlace_flag, sort_flag))

    # Local color table
    if local_color_table_flag:
        for _ in range(lct_size):
            out.write(bytes([rng.read(1)[0],
                             rng.read(1)[0],
                             rng.read(1)[0]]))

    # Table based image data
    if not interlace_flag and not sort_flag:
        for y in range(height[0], height[1]+1):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

