#!/usr/bin/env python3

import struct
from typing import BinaryIO

def write_lzw(out: BinaryIO, width: int, height: int):
    """Write a LZW-compressed TIFF file."""

    # 2. Header
    header = b'\x49\x49\x2A\x00'

    # 3. IFD (Image File Directory)
    ifd = [
        ('NewSubfileType', 2),
        ('ImageWidth', width),
        ('ImageLength', height),
        ('BitsPerSample', 8),
        ('Compression', 5),
        ('PhotometricInterpretation', 1),
        ('SamplesPerPixel', 1),
        ('RowsPerStrip', height),
        ('StripByteCounts', width * height),
        ('XResolution', 72),
        ('YResolution', 72),
        ('PlanarConfiguration', 1),
        ('ResolutionUnit', 2),
    ]

    ifd_entries = []
    for tag, value in ifd:
        entry = struct.pack('<HHII', len(tag), 1, 1, {
            'NewSubfileType': 254,
            'ImageWidth': 256,
            'ImageLength': 257,
            'BitsPerSample': 258,
            'Compression': 259,
            'PhotometricInterpretation': 262,
            'SamplesPerPixel': 277,
            'RowsPerStrip': 278,
            'StripByteCounts': 279,
            'XResolution': 282,
            'YResolution': 283,
            'PlanarConfiguration': 284,
            'ResolutionUnit': 296,
        }[tag]) + bytes(tag, encoding='ascii') + b'\x00' + struct.pack('<I', value)
        ifd_entries.append(entry)

    ifd = b''.join([struct.pack('<H', len(ifd_entries))] + ifd_entries)

    # 4. Color table
    color_table = b''

    # 5. Image data
    # 5a. LZW minimum code size
    lzw_minimum_code_size = 8

    # 5b. Data stream
    rng = Random()
    rng.seed(width * height)

    blocks = []
    while True:
        block_size = rng.randint(1, 128)
        if block_size == 128:
            block_size += rng.read(1)[0]

        blocks.append(rng.read(block_size))

    image_data = struct.pack('>BB', lzw_minimum_code_size, 0) + b''.join(blocks)

    # 7. Trailer
    trailer = b'\x3B'

    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
