#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    flags = rng.read(1)[0] & 0xE0 | 0x1F
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBB', width, height, flags, background))

    # Global color table
    global_color_table_size = 2 ** ((flags >> 4) + 1)
    for i in range(global_color_table_size):
        out.write(rng.read(3))

    # Image descriptor
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)[0] & 0xC0 | 0x07
    local_color_table_size = (flags >> 4) + 1
    interlaced = bool((flags >> 6) & 1)
    lct = [None] * local_color_table_size
    if local_color_table_size!= 0:
        for i in range(local_color_table_size):
            lct[i] = bytes([rng.read(1)[0],
                            rng.read(1)[0],
                            rng.read(1)[0]])
    out.write(left + top + width + height + bytes([flags]))
    for i in range(local_color_table_size):
        out.write(lct[i])

    # LZW minimum code size
    min_code_size = rng.read(1)[0] + 1
    out.write(bytes([min_code_size]))

    # Pixel data
    if not interlaced:
        for y in range(height[0]):
            for x in range(width[0]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    elif interlaced == 1:
        for y in range(height[0]):
            for x in range(width[0]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[2]):
            for x in range(width[2]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[3]):
            for x in range(width[3]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[4]):
            for x in range(width[4]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[5]):
            for x in range(width[5]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[6]):
            for x in range(width[6]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[7]):
            for x in range(width[7]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[8]):
            for x in range(width[8]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[9]):
            for x in range(width[9]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[10]):
            for x in range(width[10]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[11]):
            for x in range(width[11]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[12]):
            for x in range(width[12]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[13]):
            for x in range(width[13]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[14]):
            for x in range(width[14]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[15]):
            for x in range(width[15]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[16]):
            for x in range(width[16]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[17]):
            for x in range(width[17]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[18]):
            for x in range(width[18]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[19]):
            for x in range(width[19]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[20]):
            for x in range(width[20]):
                out.write(bytes([rng.read(1)[0]]))

        for y in range(height[21]):
            for x in range(width[21        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

