#!/usr/bin/env python3

import os
import struct


def generate_random_gif(rng, out):
    """Generate a random GIF file"""

    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = struct.pack('>HBBBBB', width, height, rng.read(1)[0], 0xF0 | rng.read(1)[0], 0, 0)
    lsd = struct.pack('>L', int.from_bytes(packed, byteorder='big'))

    # 3. Global Color Table
    global_color_table_flag = rng.read(1)[0] >> 7
    global_color_table_size = ((rng.read(1)[0] & 0x7) + 1) * 3
    global_color_table = [rng.read(1)[0] for _ in range(global_color_table_size)]
    color_table = b''
    if global_color_table_flag:
        color_table += struct.pack('>B', global_color_table_size)
        color_table += b''.join([struct.pack('>BBB', r, g, b) for r, g, b in zip(*[iter(global_color_table)]*3)])

    # 4. Graphics Control Extension
    graphics_control_extension_flag = rng.read(1)[0] >> 7
    delay_time = rng.read(2)
    transparent_color_index = rng.read(1)[0]
    graphics_control_extension = struct.pack('>BBBB', 0x21, 0xF9, 4, 0)
    graphics_control_extension += struct.pack('>B', graphics_control_extension_flag << 7)
    graphics_control_extension += delay_time
    graphics_control_extension += struct.pack('>B', transparent_color_index)
    graphics_control_extension += b'\x00'

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    data = []
    while True:
        block_size = rng.read(1)[0] & 0xFF

        if block_size == 0:
            break

        data.append(rng.read(block_size))

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything to output
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)


if __name__ == '__main__':
    from sys import argv, stdout

    with open('/dev/urandom', 'rb') as f:
        generate_random_gif(f, stdout.buffer)