#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Set color table size to zero
    color_table_size = 0b00
    # Set global color table flag to false
    global_color_table_flag = False
    # Set sort flag to true
    sort_flag = True
    # Set interlace flag to false
    interlace_flag = False
    # Set reserved bits to all zeros
    reserved_bits = 0b0000
    # Get background color index from RNG
    bg_color_index = ord(rng.read(1)) & 0b00001111
    # Get pixel aspect ratio
    pixel_aspect_ratio = 0b1000
    # Build logical screen descriptor byte
    lsd = struct.pack(
            '>HHBBB',
            width, height,
            color_table_size,
            global_color_table_flag | \
                (sort_flag << 3) | \
                (interlace_flag << 4) | \
                (reserved_bits << 5),
            bg_color_index,
            pixel_aspect_ratio
        )

    # Write everything to output stream
    out.write(header + lsd)

