#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors
    
        
    # 4. Image Descriptor
    image_descriptor = struct.pack('>HHBBB', 0, 0, 0, 0b10000000 | rng.read(1)[0], 0)

    # 5. Image Data
    data = []
    while True:
        # Each frame has a random length between 1-100 pixels wide
        length = rng.read(1)[0] % 100 + 1
        
        # Each pixel is an RGB triple of random values
        rgb = [rng.read(1)[0] for _ in range(length)]

        # If we've reached the end of this frame, break
        if len(rgb) == 0:
            break
            
        # Add the current line to our data buffer
        data += rgb

    # 6. Trailer
    trailer = b';'
    
    # Write all of these things to the output stream
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    out.write(image_descriptor)
    out.write(bytes(data))
    out.write(trailer)

