#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The generated image is of size `width` by `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int, height: int):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    flags = b'\x87\x00'
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBB', width, height, flags, background))

    # Global color table
    global_color_table_size = 2 ** (((flags[0] >> 4) << 1) + 1)
    for i in range(global_color_table_size):
        out.write(rng.read(3))

    # Image descriptor
    left = b'\x00\x00'
    top = b'\x00\x00'
    width = struct.pack('>H', width)
    height = struct.pack('>H', height)
    flags = b'\x87\x00'
    local_color_table_size = (flags[0] >> 4) + 1
    interlaced = False
    out.write(left + top + width + height + flags)

    # Local color table
    for i in range(2 ** local_color_table_size):
        out.write(rng.read(3))

    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass