#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(2)[0] + 1
    height = rng.read(2)[0] + 1
    background_color = rng.read(1)[0] % 256
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBB', width, height, background_color, pixel_aspect_ratio))

    # Global color table (256 colors)
    for i in range(768):
        out.write(rng.read(1))

    # Application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    loop_count = rng.read(2)[0] + 1
    out.write(struct.pack('<H', loop_count))
    out.write(b'\x00')

    # Image descriptor
    left = rng.read(2)[0]
    top = rng.read(2)[0]
    width = rng.read(2)[0] + 1
    height = rng.read(2)[0] + 1
    local_color_table_flag = False
    interlaced = True
    sort_flag = False
    reserved = 0
    out.write(struct.pack('<HHBBB', left, top, width, height, local_color_table_flag, interlaced, sort_flag, reserved))

    # Local color table (256 colors)
    for i in range(768):
        out.write(rng.read(1))

    # LZW minimum code size
    lzw_minimum_code_size = rng.read(1)[0] % 12
    out.write(lzw_minimum_code_size.to_bytes(1, byteorder='big'))

    # Raster data
    while True:
        if rng.read(1)[0] == 0:
            break
        else:
            out.write(rng.read(1))
