#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar  6 14:25:09 2020

@author: jorge
"""
import struct
from PIL import Image, ImageOps

def writeGIF(filename, img):
    
    # Open the output file for writing (truncate existing file)
    out = open(filename,"wb")
    
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = img.size
    packedFields = 0
    if img.info['transparency'] is not None:
        packedFields |= 1 << 3  # Transparent Color Flag
    lsd = struct.pack("<HHBB",width,height,packedFields,0)
        
    # 3. Global Color Table
    globalPalette = []
    for i in range(2**img.palettesize):
        r,g,b = img.getpalette()[i*3:(i+1)*3]
        globalPalette.append((r>>4<<4|r&0xF,g>>4<<4|g&0xF,b>>4<<4|b&0xF))
    color_table = bytearray()
    for rgb in globalPalette:
        color_table += bytes([rgb[0],rgb[1],rgb[2]])
            
    # 4. Graphic Control Extension
    gce = b'\x21\xF9\x04\x01\x00\x00\xFF\xFF\xFF\x00'
    
    # 5. Image Descriptor
    image_descriptor = b'\x2C\x00\x00\x00\x00'
    
    # 6. Image Data
    image_data = bytearray()
    for y in range(height):
        for x in range(width):
            pixel = img.getpixel((x,y))
            index = globalPalette.index(pixel)
            image_data.extend(bytes([index]))
    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)