#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 17 20:54:56 2019

@author: jorge
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
from scipy.optimize import minimize
from sklearn.metrics import mean_squared_error
from matplotlib import pyplot as plt

def gaussian(x, mu, sigma):
    return (np.exp(-((x - mu) ** 2 / (2 * sigma ** 2))) / np.sqrt(2 * np.pi * sigma))

def gaussian_pdf(x, mu, sigma):
    return (np.exp(-((x - mu) ** 2 / (2 * sigma ** 2))) / np.sqrt(2 * np.pi * sigma))
    
def gaussian_cdf(x, mu, sigma):
    return (norm.cdf(x, loc=mu, scale=sigma))
    
def gaussian_inv(p, mu, sigma):
    return norm.ppf(p, loc=mu, scale=sigma)

def gaussian_loglikelihood(x, mu, sigma):
    return (-np.sum(np.log(gaussian(x, mu, sigma))))

def gaussian_mle(x):
    mu = np.mean(x)
    sigma = np.std(x)
    return mu, sigma

def gaussian_fit(x, y):
    def f(params):
        mu, sigma = params
        return np.sum((y - gaussian(x, mu, sigma))**2)
        
    p0 = [np.median(x), np.std(x)]
    res = minimize(f, p0)
    return res.x[0], res.x[1]

def gaussian_sample(n, mu, sigma):
    return np.random.randn(n)*sigma + mu

def gaussian_sample_multivar(n, mu, cov):
    return np.random.multivariate_normal(mu, cov, n)

def gaussian_sample_multivar_cov(n, mu, cov):
    return np.random.multivariate_normal(mu, cov, n)

def gaussian_sample_multivar_corr(n, mu, corr):
    cov = np.zeros([len(mu), len(mu)])
    for i in range(len(mu)):
        for j in range(i+1):
            if i == j:
                cov[i][j] = 1
            else:
                cov[i][j] = corr[i][j]*np.sqrt(cov[i][i]*cov[j][j])
                
    return np.random.multivariate_normal(mu, cov, n)

def gaussian_sample_multivar_corr_mat(n, mu, corr):
    cov = np.zeros([len(mu), len(mu)])
    for i in range(len(mu)):
        for j in range(i+1):
            if i == j:
                cov[i][j] = 1
            else:
                cov[i][j] = corr[i][j]*np.sqrt(cov[i][i]*cov[j][j])
                
    return np.random.multivariate_normal(mu, cov, n)

def gaussian_sample_multivar_corr_vec(n, mu, corr):
    cov = np.diag(np.ones(len(mu)))
    for i in range(len(mu)-1):
        for j in range(i+1, len(mu)):
            cov[i][j] = corr[i][j]
            
    return np.random.multivariate_normal(mu, cov, n)

def gaussian_sample_multivar_corr_vec_sym(n, mu, corr):
    cov = np.diag(np.ones(len(mu)))
    for i in range(len(mu)-1):
        for j in range(i+1, len(mu)):
            cov[i][j] = corr[i][j]
            
    return np.random.multivariate_normal(mu, cov, n)

def gaussian_sample_multivar_corr_vec_asym(n, mu, corr):
    cov = np.diag(np.ones(len(mu)))
    for i in range(len(mu)-1):
        for j in range(i+1, len(mu)):
            cov[i][j] = corr[i][j]
            
    return np.random.multivariate_normal(mu, cov, n)

def gaussian_sample_multivar_corr_vec_asym_low(n, mu, corr):
    cov = np.diag(np.ones(len(mu)))
    for i in range(len(mu)-1):
        for j in range(i+1, len(mu)):
            cov[i][j] = corr[i][j]/10
            
    return np.random.multivariate_normal(mu, cov, n)

def gaussian_sample_multivar_corr_vec_asym_high(n, mu, corr):
    cov = np.diag(np.ones(len(mu)))
    for