#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jun 19 20:56:47 2020

@author: jonasg
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
import matplotlib.pyplot as plt

def gaussian(x, mu, sigma):
    return (np.exp(-((x - mu) ** 2 / (2 * sigma ** 2))) / (sigma * np.sqrt(2 * np.pi)))
    
def gaussian_pdf(x, mu, sigma):
    return (np.exp(-((x - mu) ** 2 / (2 * sigma ** 2))))

def gaussian_cdf(x, mu, sigma):
    return (norm.cdf(x, loc=mu, scale=sigma))

def gaussian_ppf(q, mu, sigma):
    return (norm.ppf(q, loc=mu, scale=sigma))

def gaussian_moments(n, mu, sigma):
    if n == 0:
        return 1
    elif n == 1:
        return mu
    else:
        return (mu**n)*(np.exp(-(n**2)/(2*sigma**2))) + \
            ((n/(np.sqrt(2*np.pi*sigma**2)))*(np.exp(-(n**2)/(2*sigma**2))))
        
def gaussian_mean(mu, sigma):
    return mu

def gaussian_variance(mu, sigma):
    return sigma**2

def gaussian_skewness(mu, sigma):
    return 0

def gaussian_kurtosis(mu, sigma):
    return 3

def gaussian_mode(mu, sigma):
    return mu

def gaussian_median(mu, sigma):
    return mu

def gaussian_entropy(mu, sigma):
    return (1/2)*(1+np.log(2*np.pi*sigma**2))

def gaussian_random(mu, sigma, size = None):
    return np.random.normal(loc=mu, scale=sigma, size=size)

def gaussian_fit(data, p0 = None, bounds = None):
    if p0 is None:
        p0 = [np.mean(data), np.std(data)]
    if bounds is None:
        bounds = [(None, None), (0, None)]
    popt, pcov = curve_fit(gaussian, data, p0 = p0, bounds = bounds)
    return popt[0], popt[1]

def gaussian_fit_multidimensional(data, p0 = None, bounds = None):
    if p0 is None:
        p0 = [np.mean(data, axis = 0), np.cov(data, rowvar = False)]
    if bounds is None:
        bounds = [(None, None), (None, None)]
    popt, pcov = curve_fit(multivariate_normal.pdf, data, p0 = p0, bounds = bounds)
    return popt[0], popt[1]
