#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width = rng.read(2)
    height = rng.read(2)

    # Packed fields
    packed = bytearray([0xF7])  # Global Color Table Flag set, 16-color table
                               # Background color index is 0
                               # Pixel aspect ratio is 1:1

    # 3. Global Color Table
    # 4. Graphic Control Extension
    # 5. Image Descriptor
    # 6. Local Color Table
    # 7. Image Data

    # Write everything to the output stream
    out.write(header + width + height + packed)

