#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('<BBB', *struct.unpack('>BBB', rng.read(3)))
    
    # 3. Global Color Table
    # 768-byte table of RGB triples
    global_color_table = rng.read(768)
    
    # 4. Image Descriptors
    # One per frame, with one frame being generated here
    image_descriptor = struct.pack('<BBHBB', 0x2C, 0, width, height, 0)
    
    # 5. Local Color Table
    # Same as global color table
    local_color_table = global_color_table
    
    # 6. Image Data
    # Each pixel is an index into the color table
    image_data = rng.read((width * height) // 8)
    
    # 7. Trailer
    trailer = b';'
    
    # Write everything to the output file
    out.write(header + lsd + global_color_table + image_descriptor + local_color_table + image_data + trailer)
