#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack(
            'BBB',
            0b10000000 | int(rng.read(1)[0]),
            int(rng.read(1)[0]),
            int(rng.read(1)[0])
        ) + rng.read(1)

    # 3. Global Color Table
    # We don't need to worry about this since we aren't going to use it

    # 4. Image Descriptors
    # We are only going to write one image descriptor
    idd = struct.pack('BBHBB', 0x2c, 0, 0, width, height)

    # 5. Local Color Table
    # We don't need to worry about this since we aren't going to use it

    # 6. Image Data
    # We are just going to write a bunch of random data
    idata = rng.read(width * height)

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + idd + idata + trailer)

