#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor (LSD)
    width = rng.read(2)
    height = rng.read(2)
    packed = rng.read(1)
    background_color_index = rng.read(1)
    pixel_aspect_ratio = rng.read(1)

    lsd = struct.pack('<HHBBB', *[int.from_bytes(x, 'big') for x in [width, height, packed, background_color_index, pixel_aspect_ratio]])

    # 3. Color Table
    color_table = rng.read(768)

    # 4. Graphics Control Extension (GCE)
    block_size = rng.read(1)
    packed = rng.read(1)
    delay_time = rng.read(2)
    transparent_color_index = rng.read(1)
    terminator = rng.read(1)

    gce = struct.pack('<BBHBB', *[int.from_bytes(x, 'big') for x in [block_size, packed, delay_time, transparent_color_index, terminator]])

    # 5. Image Descriptor
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    packed = rng.read(1)

    image_descriptor = struct.pack('<HHHHB', *[int.from_bytes(x, 'big') for x in [left, top, width, height, packed]])

    # 6. Image Data
    image_data = rng.read(width[0] * height[0])

    # 7. Trailer
    trailer = b'\x3b'

    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)