#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Magic header
    out.write(b"GIF89a")

    # Logical screen descriptor
    width = 1 + (next(rng) % 256)
    height = 1 + (next(rng) % 256)
    packed = next(rng) & 0x7F | 0x80
    out.write(struct.pack("!BBB", width, height, packed))

    # Global color table
    for _ in range(3 * (2 ** ((packed >> 4) + 1)):
        out.write(bytes([next(rng) % 256]))

    # Application extension
    out.write(b"\x21\xFF\x0BNETSCAPE2.0\x03\x01")
    out.write(struct.pack("!H", 0))

    # Image descriptor
    out.write(b"\x2C\x00\x00\x00\x00")

    # Local color table
    for _ in range(3 * (2 ** ((packed >> 4) + 1)):
        out.write(bytes([next(rng) % 256]))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([next(rng) % 256]))
