#!/usr/bin/env python3
import random
import sys
from PIL import Image, ImageDraw

if len(sys.argv) != 2:
    print('Usage: {} <output file>'.format(sys.argv[0]))
    exit()

out = open(sys.argv[1], 'wb')

# Generate the GIF
for i in range(100):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = height = 16
    packed_fields = 0b00000000  # No global colour table, no sort flag, no interlace flag
    background_colour_index = 0
    pixel_aspect_ratio = 0
    lsd = struct.pack('<HHBBB', width, height, packed_fields, background_colour_index, pixel_aspect_ratio)
    
    # 3. Global Color Table
    colors = [0xFF0000, 0x00FF00, 0x0000FF]
    color_table = b''
    for c in colors:
        red = (c >> 16) & 0xFF
        green = (c >> 8) & 0xFF
        blue = c & 0xFF
        color_table += bytes([red, green, blue])
    
    # 4. Graphics Control Extension
    gce_block_size = 4
    gce_packed_fields = 0b00000000  # No transparency, no user input, disposal method 0 (no action), no delay time
    transparent_colour_index = 0
    delay_time = 0
    gce = struct.pack('BBBBH', gce_block_size, gce_packed_fields, transparent_colour_index, delay_time, 0)
    
    # 5. Image Descriptor
    image_descriptor = b','
    
    # 6. Image Data
    rng = random.Random()
    lzw_minimum_code_size = 2  # We only have 3 colours so we need at least 2 bits per code
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    for _ in range(total_blocks_size):
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b';'
    
    # Write everything
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)

