#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_gif(out: BinaryIO, rng: BinaryIO):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    global_color_table_flag = bool(packed & 0x80)
    color_resolution = ((packed >> 4) & 0x07) + 1
    sort_flag = bool((packed >> 3) & 0x01)
    global_color_table_size = 2 ** ((packed & 0x07) + 1)
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HBBBBB', width, height, packed, background_color_index, pixel_aspect_ratio, global_color_table_size)
    
    # 3. Global Color Table
    color_table = rng.read(global_color_table_size * 3)
    
    # 4. Graphic Control Extension
    gce = rng.read(6)
    
    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while total_blocks_size > 0:
        block_size = min(total_blocks_size, rng.read(1)[0] + 1)
        blocks.append(rng.read(block_size))
        
        total_blocks_size -= block_size
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    if len(gce) == 6:
        out.write(gce)
    out.write(image_descriptor)
    out.write(struct.pack('B', lzw_minimum_code_size))
    for block in blocks:
        out.write(struct.pack('B', len(block)))
        out.write(block)
