#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Packed field
    #   - Global Color Table Flag (0b10000000)
    #   - Color Resolution (0b00000111)
    #   - Sort Flag (0b00000000)
    #   - Size of Global Color Table (0b00000000)
    packed = 0x87
    
    # Background color index
    background = rng.read(1)[0] % 256
    
    # Pixel aspect ratio
    pixel_aspect = rng.read(1)[0] % 256
    
    # Write everything to the output stream
    out.write(header + struct.pack('<HHB', width, height, packed) + bytes([background]) + bytes([pixel_aspect]))
    
    # 3. Global Color Table
    # We don't use it so we just write zeros
    out.write(bytes(768))
    
    # 4. Graphic Control Extension
    # We don't use it so we just write zeros
    out.write(bytes(8))
    
    # 5. Image Descriptor
    # Random position
    x, y = struct.unpack('>HH', rng.read(4))
    
    # Width and Height
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Packed field
    #   - Local Color Table Flag (0b10000000)
    #   - Interlaced Flag (0b00000000)
    #   - Sort Flag (0b00000000)
    #   - Reserved (0b00000000)
    #   - Size of Local Color Table (0b00000000)
    packed = 0xF0
    
    # LZW Minimum Code Size
    lzw_min_code_size = rng.read(1)[0] % 256
    
    # Write everything to the output stream
    out.write(struct.pack('<HHHBB', x, y, width, height, packed) + bytes([lzw_min_code_size]))
    
    # 6. Local Color Table
    # We don't use it so we just write zeros
    out.write(bytes(768))
    
    # 7. Image Data
    # We don't use it so we just write zeros
    out.write(bytes(1))
    
    # 8. Trailer
    # We don't use it so we just write zeros
    out.write(bytes(1))
