
import random
import struct
from PIL import Image

def generate_gif(out, seed=None):
    if seed is None:
        rng = random.Random()
    else:
        rng = random.Random(seed)
        
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = rng.randint(1, 320)
    height = rng.randint(1, 240)
    global_color_table_flag = 0
    color_resolution = 8
    sort_flag = 0
    global_color_table_size = 7
    background_color_index = rng.randint(0, 2**global_color_table_size - 1)
    pixel_aspect_ratio = 0
    lsd = struct.pack('>HHBBB', width, height, global_color_table_flag, color_resolution, sort_flag, global_color_table_size)
    
    # 3. Global Color Table
    colors = [Image.new('RGB', (1, 1), (rng.randint(0, 255), rng.randint(0, 255), rng.randint(0, 255))) for _ in range(2**global_color_table_size)]
    color_table = b''.join([c.tobytes()[0] for c in colors])
    
    # 4. Graphic Control Extension
    gce_block_size = 4
    gce_packed_fields = 0b00000000
    delay_time = rng.randint(0, 65535)
    transparent_color_index = rng.randint(0, 2**global_color_table_size - 1)
    terminator = 0
    gce = struct.pack('>BBHBB', gce_block_size, gce_packed_fields, delay_time, transparent_color_index, terminator)
    
    # 5. Image Descriptor
    left = rng.randint(0, width - 1)
    top = rng.randint(0, height - 1)
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    for _ in range(total_blocks_size):
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b';'
    
    # Write everything
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)

