#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Graphic Control Extension
    # We'll use a random delay time, transparent color index, and disposal method
    gce = struct.pack('BBB', 0xF9, rng.read(1)[0], rng.read(1)[0])

    # 5. Image Descriptor
    # We'll use a random left offset, top offset, width, height, and packed field
    image_descriptor = struct.pack('>HHHHB', rng.read(2)[0], rng.read(2)[0], width, height, rng.read(1)[0])

    # 6. Image Data
    # We'll just use random data
    image_data = rng.read(width * height)

    # 7. Trailer
    # Just write the end of stream marker
    trailer = b'\x3b'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)