#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_gif(out: BinaryIO, width: int, height: int, duration: float) -> None:
    # Write the header.
    out.write(b'GIF89a')  # Magic number.
    out.write(struct.pack('<H', width))  # Width.
    out.write(struct.pack('<H', height))  # Height.
    out.write(bytes([0xF7]))  # Global color table flag (yes).
    out.write(bytes([0xFF]))  # Color resolution.
    out.write(bytes([0x0B]))  # Sort flag (no).
    out.write(bytes([0x00]))  # Size of global color table.
    out.write(bytes([0x00, 0x00, 0x00]))  # Background color index.
    out.write(bytes([0x00]))  # Pixel aspect ratio.

    # Write the application extension block.
    out.write(bytes([0x21, 0xFF, 0x0B]))  # Extension introducer and label.
    out.write(b'NETSCAPE2.0')  # Application ID.
    out.write(bytes([0x03, 0x01]))  # Block size.
    out.write(bytes([0x00]))  # Subblock index.
    out.write(bytes([0x00, 0x00]))  # Number of times to loop.
    out.write(bytes([0x00]))  # Block terminator.

    # Write the image descriptor.
    out.write(bytes([0x2C]))  # Image separator.
    out.write(struct.pack('<H', 0))  # Left position.
    out.write(struct.pack('<H', 0))  # Top position.
    out.write(struct.pack('<H', width))  # Width.
    out.write(struct.pack('<H', height))  # Height.
    out.write(bytes([0x00]))  # Local color table flag (no).
    out.write(bytes([0x00]))  # Interlace flag (no).
    out.write(bytes([0x00]))  # Sort flag (no).
    out.write(bytes([0x00]))  # Size of local color table.

    # Write the graphic control extension.
    out.write(bytes([0x21, 0xF9, 0x04]))  # Extension introducer and label.
    out.write(bytes([0x00]))  # Block size.
    out.write(bytes([0x01]))  # Disposal method.
    out.write(bytes([0x00]))  # User input flag (no).
    out.write(bytes([0x00]))  # Transparent color flag (no).
    out.write(bytes([0x00]))  # Delay time (low byte).
    out.write(bytes([0x00]))  # Delay time (high byte).
    out.write(bytes([0x00]))  # Transparent color index.
    out.write(bytes([0x00]))  # Block terminator.

    # Write the pixel data.
    for y in range(height):
        for x in range(width):
            r = min(int((x / width) * 256), 255)
            g = min(int((y / height) * 256), 255)
            b = min(int(((x + y) / (width + height)) * 256), 255)
            out.write(bytes([r, g, b]))

    # Write the trailer.
    out.write(bytes([0x3B]))  # Trailer.
