#!/usr/bin/env python3

import random
import struct
import sys

if len(sys.argv) != 2:
    print("Usage: {} <output file>".format(sys.argv[0]))
    exit(1)

outfile = sys.argv[1]

with open(outfile, 'wb') as out:
    rng = random.Random()
    
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width = rng.randint(1, 65535)
    height = rng.randint(1, 65535)
    global_color_table_flag = rng.choice((False, True))
    color_resolution = rng.choice((7, 8))
    sort_flag = rng.choice((False, True))
    global_color_table_size = rng.choice((2 ** i for i in range(1, 8)))
    background_color_index = rng.randint(0, global_color_table_size - 1)
    pixel_aspect_ratio = rng.choice((0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15))
    lsd = struct.pack('>HHBBBBB', width, height, global_color_table_flag, color_resolution, sort_flag, global_color_table_size, background_color_index, pixel_aspect_ratio)

    # 3. Global Color Table
    color_table = rng.getrandbits(8 * global_color_table_size).to_bytes(global_color_table_size * 3, byteorder='big')

    # 4. Graphic Control Extension
    gce = None
    if rng.randint(0, 1):
        delay_time = rng.randint(0, 65535)
        transparent_color_flag = rng.choice((False, True))
        transparent_color_index = rng.randint(0, 255)
        disposal_method = rng.choice((0, 1, 2, 3))
        user_input_flag = rng.choice((False, True))
        gce = struct.pack('>BBBBBB', 0xF9, 4, delay_time, transparent_color_flag, transparent_color_index, disposal_method, user_input_flag)

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes

        if not block_size:
            break
        
        blocks.append(bytes([block_size]) + rng.read(block_size))
        
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table)
    if gce is not None:
        out.write(gce)
    out.write(image_descriptor + image_data + trailer)