#!/usr/bin/env python3

import struct
from typing import BinaryIO
from random import Random

def generate_gif(out: BinaryIO, width: int, height: int, duration: float):
    """Generate an animated GIF file."""
    
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical screen descriptor
    lsd = struct.pack('<HBB', width, height, 0b10000000 | 0b00000000)  # Global color table flag set to true and color resolution set to 8 bits per pixel
    
    # 3. Color table
    color_table = (
        b'\xff\xff\xff' +   # White
        b'\x00\x00\x00' +   # Black
        b'\xff\x00\x00' +   # Red
        b'\x00\xff\x00' +   # Green
        b'\x00\x00\xff' +   # Blue
        b'\xff\xff\x00' +   # Yellow
        b'\x00\xff\xff' +   # Cyan
        b'\xff\x00\xff'     # Magenta
    )
    
    # 4. Graphic control extension
    gce = struct.pack('B', 0b10000000 | 0b00000000) + struct.pack('<H', int(duration * 100)) + b'\x00'
    
    # 5. Image descriptor
    image_descriptor = struct.pack('<HHBB', 0, 0, width, height) + b'\x00'
    
    # 6. Image data
    rng = Random()
    lzw_minimum_code_size = 2
    blocks = []
    while True:
        block_size = min(rng.randint(1, 255), 255 - len(blocks))
        if not block_size:
            break
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b';'
    
    # Write everything
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)

