#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Reads a GIF from stdin and writes it back with a new palette.

def main():
    inp = open("test.gif", "rb")
    out = open("out.gif", "wb")
    
    header = inp.read(6)
    if not header or header[:6] != b"GIF87a":
        raise Exception("Not a valid GIF!")
        
    # Read the logical screen descriptor
    lsd = inp.read(7)
    width, height = struct.unpack("<HH", lsd[1:5])
    
    # Read the global color table
    color_table_size = 2 ** (lsd[4] & 0x07)
    color_table = inp.read(color_table_size * 3)
    
    # Skip any extension blocks
    while True:
        block_type = ord(inp.read(1))
        if block_type == 0x2C: break
        else: inp.seek(-1, 1)
        
        size = ord(inp.read(1))
        inp.seek(size, 1)
        
    # Read the graphics control extension
    gce = inp.read(8)
    
    # Read the image descriptor
    image_descriptor = inp.read(9)
    
    # Read the image data
    image_data = inp.read()
    
    # Read the trailer
    trailer = inp.read()
    
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)