#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# GIF89a specification: https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(out: BinaryIO, width: int, height: int):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    lsd = struct.pack('<6sHHBB', b'NETSCAPE2.0', width, height, 0b00000000, 0)

    # 3. Global Color Table
    color_table = bytes([0] * (3 * 256))

    # 4. Graphic Control Extension
    gce = struct.pack('<BBHBB', 0b10000000 | 0b00000001, 0, 0, 0)

    # 5. Image Descriptor
    image_descriptor = struct.pack('<4sLLBB', b'', 0, 0, width, height, 0)

    # 6. Image Data
    image_data = bytes([0] * (width * height))
    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)