#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Color Table
    # 256 colors (7 bits per channel)
    color_table = rng.read(3 * 256)

    # 4. Graphic Control Extension
    # 1-byte length, 1-byte packed field, 2-byte delay time, 1-byte transparent color index
    gce = struct.pack('BBHB', 4, rng.read(1)[0], struct.unpack('>H', rng.read(2))[0], rng.read(1)[0])

    # 5. Image Descriptor
    # 1-byte length, 2-byte left offset, 2-byte top offset, 2-byte width, 2-byte height, 1-byte packed field
    image_descriptor = struct.pack('>BBHHHHB', 9, rng.read(2)[0], rng.read(2)[0], width, height, rng.read(1)[0])

    # 6. Image Data
    # Random data of size (width * height) bytes
    image_data = rng.read(width * height)

    # 7. Trailer
    # 1-byte length, 1-byte terminator
    trailer = b'\x00\x3b'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)