#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # We don't need to write this because we're not using it

    # 4. Image Data
    # Write the image descriptor
    idd = struct.pack('>BBHHB', 0x2C, 0, 0, width, height, 0)

    # Write the local color table
    # We don't need to write this because we're not using it

    # Write the image data
    # We'll just use a single pixel of black
    image_data = struct.pack('>BBBB', 0, 0, 0, 0)

    # 5. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + idd + image_data + trailer)
