#!/usr/bin/env python3

import sys
from random import Random

if __name__ == '__main__':
    try:
        seed = int(sys.argv[1], base=16)
    except IndexError:
        print('Usage: {} <seed> [<output>]'.format(sys.argv[0]))
        exit()
        
    rng = Random(seed)
    
    if len(sys.argv) > 2:
        out = open(sys.argv[2], 'wb')
    else:
        out = sys.stdout.buffer
    
    # 1. Header
    header = b'\x47\x49\x46\x38\x39\x61'
    
    # 2. Logical Screen Descriptor
    width = rng.randint(1, 65535)
    height = rng.randint(1, 65535)
    packed = ((rng.randint(0, 1) << 7) |  # Global Color Table Flag
              (rng.randint(0, 7) << 4) |  # Color Resolution
              (rng.randint(0, 1) << 3) |  # Sort Flag
              (rng.randint(0, 1) << 2) |  # Size of Global Color Table
              (rng.randint(0, 3)))         # Background Color Index
    lsd = bytes([width & 0xFF, width >> 8, height & 0xFF, height >> 8, packed])
    
    # 3. Global Color Table
    color_table = rng.read(3 * (2 ** (packed & 0x07)))
    
    # 4. Graphic Control Extension
    gce = None
    if rng.randint(0, 1):
        delay_time = rng.randint(0, 65535)
        transparent_color_index = rng.randint(0, 255)
        block_size = rng.randint(0, 255)
        gce = b'\x21\xf9\x04' + bytes([delay_time & 0xFF, delay_time >> 8, transparent_color_index, block_size])
    
    # 5. Image Descriptor
    left = rng.randint(0, 65535)
    top = rng.randint(0, 65535)
    width = rng.randint(1, 65535)
    height = rng.randint(1, 65535)
    local_color_table_flag = rng.randint(0, 1)
    interlace_flag = rng.randint(0, 1)
    sort_flag = rng.randint(0, 1)
    size_of_local_color_table = rng.randint(0, 7)
    packed = (local_color_table_flag << 7) | (interlace_flag << 6) | (sort_flag << 5) | size_of_local_color_table
    image_descriptor = bytes([left & 0xFF, left >> 8, top & 0xFF, top >> 8, width & 0xFF, width >> 8, height & 0xFF, height >> 8, packed])
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    for _ in range(total_blocks_size):
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output file
    out.write(header + lsd + color_table)
    if gce is not None:
        out.write(gce)
    out.write(image_descriptor + image_data + trailer)