#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    # 256 x 3 bytes of RGB colors
    color_table = rng.read(768)
    
    # 4. Graphic Control Extension
    # Always present
    gce = struct.pack('>BBBHB', 0xF9, 0x04, 0x00, 0x00, 0x00)
    
    # 5. Image Descriptor
    # Random position and size
    left, top, width, height = struct.unpack('>HHHH', rng.read(8))
    image_descriptor = struct.pack('>BBHHHB', 0x2C, 0x00, left, top, width, height, 0x00)
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    data = []
    while True:
        block_size = rng.read(1)[0] & 0xFF
        
        if block_size == 0:
            break
        
        data.append(rng.read(block_size))
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to output
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)
