#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), 'big') % 640 + 1
    height = int.from_bytes(rng.read(2), 'big') % 480 + 1
    background_color_index = rng.read(1)[0] & 0x7F
    pixel_aspect_ratio = rng.read(1)[0] & 0x7F
    global_color_table_flag = bool((background_color_index >> 7) & 1)
    color_resolution = (background_color_index >> 4) & 0b11
    sort_flag = bool((background_color_index >> 3) & 1)
    size_of_global_color_table = 2 ** ((background_color_index & 0b111) + 1)
    
    logical_screen_descriptor = struct.pack('<HHBB', width, height, background_color_index, pixel_aspect_ratio)
    
    # 3. Global Color Table
    if global_color_table_flag:
        global_color_table = rng.read(size_of_global_color_table * 3)
    else:
        global_color_table = None
        
    # 4. Image Data
    image_data = []
    for _ in range(width * height):
        image_data.append(rng.read(1)[0])
        
    # 5. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output stream
    out.write(header)
    out.write(logical_screen_descriptor)
    if global_color_table is not None:
        out.write(global_color_table)
