#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('<BBB', *struct.unpack('>BBB', rng.read(3))) + rng.read(1)
    
    # 3. Global Color Table
    # We use a global color table with 256 colors
    gct = rng.read(768)
    
    # 4. Image Descriptors
    # We write one image descriptor per pixel in the image
    ides = []
    for y in range(height):
        for x in range(width):
            # Image Separator
            ides.append(b';')
            
            # Image Left Position
            ides.append(struct.pack('<H', x))
            
            # Image Top Position
            ides.append(struct.pack('<H', y))
            
            # Image Width
            ides.append(struct.pack('<H', 1))
            
            # Image Height
            ides.append(struct.pack('<H', 1))
            
            # Packed Fields
            ides.append(rng.read(1))
            
            # Local Color Table Flag
            ides.append(b'\x00')
            
            # Interlace Flag
            ides.append(b'\x00')
            
            # Sort Flag
            ides.append(b'\x00')
            
            # Size of Local Color Table
            ides.append(b'\x00')
            
            # Reserved
            ides.append(b'\x00')
            
            # Block Size
            ides.append(b'\x00')
        
    # 5. Image Data
    # We write one data block per pixel in the image
    data = []
    for y in range(height):
        for x in range(width):
            # Minimum code size
            data.append(b'\x00')
            
            # LZW-encoded pixel data
            data.append(rng.read(1))
    
    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(gct)
    out.writ