#!/usr/bin/env python3
import os, sys, struct, argparse
from PIL import Image

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('input', help='Input image')
    parser.add_argument('output', nargs='?', default=None, help='Output GIF')
    args = parser.parse_args()
    
    infile = args.input
    outfile = args.output or (os.path.splitext(infile)[0] + '.gif')
    
    im = Image.open(infile)
    im.load()
    
    # Open output file for writing (binary)
    out = open(outfile, 'wb')
    
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = im.size
    packed_fields = 0
    background_color_index = 0
    pixel_aspect_ratio = 0
    lsd = struct.pack("<HHBB", width, height, packed_fields, background_color_index)
    
    # 3. Global Color Table
    global_color_table = []
    for i in range(256):
        r, g, b = im.getpixel((i % 256, i // 256))
        global_color_table += [r, g, b]
    color_table = bytes(global_color_table)
    
    # 4. Graphic Control Extension
    gce = None
    if im.info['duration'] > 0:
        block_size = 0xF9
        delay = int(im.info['duration']) * 10
        transparent_color_index = 0
        disposal_method = 2
        input_flag = 0
        
        gce = struct.pack("!BBB", block_size, delay, transparent_color_index)
        gce += struct.pack("B", disposal_method << 2 | input_flag)
    
    # 5. Image Descriptor
    left_position = 0
    top_position = 0
    width = im.width
    height = im.height
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    reserved_bits = 0
    image_descriptor = struct.pack("!BBBBB", left_position, top_position, width, height, local_color_table_flag << 7 | interlace_flag << 6 | sort_flag << 5 | reserved_bits)
    
    # 6. Image Data
    image_data = im.tobytes()
    
    # 7. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output file
    out.write(header + lsd + color_table)
    if gce is not None:
        out.write(gce)
    out.write(image_descriptor + image_data + trailer)