#!/usr/bin/env python3

import struct
from random import Random

def main():
    rng = Random()
    
    out = open("test.gif", "wb")

    # 1. Header
    header = b"GIF89a"

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    global_color_table_flag = bool(packed & 0x80)
    color_resolution = ((packed >> 4) & 0x07) + 1
    sort_flag = bool(packed & 0x08)
    global_color_table_size = 2 ** ((packed & 0x07) + 1)
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBBBBB', width, height, packed, background_color_index, pixel_aspect_ratio, 0, 0)

    # 3. Global Color Table
    color_table = rng.read(global_color_table_size * 3)

    # 4. Graphic Control Extension (optional)
    gce = None
    if rng.randint(0, 1):
        delay_time = struct.pack('>H', rng.randint(0, 65535))
        transparent_color_index = rng.read(1)[0]
        disposal_method = rng.read(1)[0]
        gce = struct.pack('>BBB', 0xF9, 4, delay_time[0]) + bytes([transparent_color_index, disposal_method])

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes

        if not block_size:
            break
        
        blocks.append(bytes([block_size]) + rng.read(block_size))
        
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table)
    if gce is not None:
        out.write(gce)
    out.write(image_descriptor + image_data + trailer)