#!/usr/bin/env python3

import struct
from typing import BinaryIO

def write_gce(out: BinaryIO, gce):
    out.write(struct.pack('<4s', b'GCE '))
    out.write(struct.pack('>H', 0)) # Version
    out.write(struct.pack('>H', len(gce))) # Header size
    out.write(b'\x01') # Flags
    out.write(b'\x00\x00\x00\x00') # Reserved
    out.write(struct.pack('>I', 0)) # Color table offset
    out.write(struct.pack('>I', 0)) # Image descriptor offset
    out.write(struct.pack('>I', 0)) # Trailer offset
    out.write(struct.pack('>I', 0)) # Text extension offset

    out.write(gce)

def write_gif89a(out: BinaryIO, width, height, palette, data):
    header = b'GIF89a'
    logical_screen_descriptor = struct.pack('<2sHHBBBHH',
        b'!',
        0,
        width,
        height,
        0xf7, # Global color table flag and size of global color table (1 << 7 | 1 << 5 | 1 << 6)
        0,
        0,
        0,
    )
    gce = bytes([
        0x21, 0xf9, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x2c, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x10, 
        0x00, 0x00, 0x00, 0x02, 0x00, 0x01, 0x00, 0x00, 0x02, 0x02, 0x4c, 0x01, 0x00, 0x3b
    ])
    image_descriptor = struct.pack('<BHHHHBB',
        0x2c,
        width,
        height,
        0,
        0,
        0,
        0,
        0,
    )
    image_data = bytearray()
    for y in range(height - 1, -1, -1):
        line = data[y * width : (y+1) * width]
        for x in range(width // 2):
            a = line[(x*2)] & 0xf
            b = line[(x*2)+1] >> 4
            c = line[(x*2)] >> 4
            d = line[(x*2)+1] & 0xf

            image_data += bytes([(a << 4) | b])
            image_data += bytes([(c << 4) | d])
    
    trailer = b''

    out.write(header)
    out.write(logical_screen_descriptor)
    if palette is not None:
        out.write(palette)
        out.write(gce)
    out.write(image_descriptor + image_data + trailer)