#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write('GIF89a'.encode())

    width = 256 + int.from_bytes(rng.read(1), byteorder='big') % 40
    height = 256 + int.from_bytes(rng.read(1), byteorder='big') % 40

    flags = b'\x87' if (int.from_bytes(rng.read(1), byteorder='big') & 1) == 1 else b'\x00'

    out.write(struct.pack('<HHBBB', width, height, int.from_bytes(rng.read(1), byteorder='big'),
                         int.from_bytes(rng.read(1), byteorder='big'), flags))

    # Global Color Table
    for i in range(0, 2 ** ((flags[0] >> 7) + 1)):
        out.write(rng.read(3))

    # Image Descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00')

    lzwmincodesize = min(max(int.from_bytes(rng.read(1), byteorder='big'), 2), 12)
    out.write(lzwmincodesize.to_bytes(1, 'little'))

    # LZW Data
    data = bytes()
    while True:
        code = int.from_bytes(rng.read(1), byteorder='big')

        if code < 256:
            data += code.to_bytes(1, 'little')

            if len(data) >= 2**lzwmincodesize:
                break
        elif code == 256:
            data += b'\x00'
        else:
            data += data[-code:]

    out.write(data)

    # Trailer
    out.write('\x3B'.encode())

