#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng.randint(0x7F)
    height = 1 + rng.randint(0x7F)
    flags = (
        0
        | ((width & 0x40) >> 6) << 7
        | ((height & 0x40) >> 6) << 6
        | (rng.randint(2)) << 5
        | (rng.randint(7)) << 2
        | (rng.randint(3)))
    bg_color = rng.randint(0xFF)
    pixel_aspect_ratio = rng.randint(3)
    out.write(struct.pack('<HBBBBB', width, height, flags, bg_color,
                          pixel_aspect_ratio))

    # Global color table
    for i in range(0, 2 ** int((flags & 7) + 1)):
        r = rng.randint(0xFF)
        g = rng.randint(0xFF)
        b = rng.randint(0xFF)
        out.write(bytes([r, g, b]))

    # Image descriptor
    left = 0
    top = 0
    width = 1 + rng.randint(0x7F)
    height = 1 + rng.randint(0x7F)
    flags = (
        0
        | ((left & 0x40) >> 6) << 7
        | ((top & 0x40) >> 6) << 6
        | (rng.randint(2)) << 5
        | (rng.randint(7)) << 2
        | (rng.randint(3)))
    local_color_table_flag = bool(rng.randint(2))
    interlace_flag = bool(rng.randint(2))
    sort_flag = bool(rng.randint(2))
    reserved = rng.randint(2)
    lct_size = 2 ** int((flags & 7) + 1) if local_color_table_flag else 0
    out.write(struct.pack('<HHBBBB', left, top, width, height, flags,
                          local_color_table_flag))

    # Local color table
    for i in range(lct_size):
        r = rng.randint(0xFF)
        g = rng.randint(0xFF)
        b = rng.randint(0xFF)
        out.write(bytes([r, g, b]))

    # LZW minimum code size
    min_code_size = rng.randint(2**8)
    out.write(bytes([min_code_size]))

    # Data blocks
    data_block_count = rng.randint(0x100)
    for _ in range(data_block_count):
        block_size = rng.randint(0x100)
        out.write(bytes([block_size]))

        # Data
        for j in range(block_size):
            byte = rng.randint(0xFF)
            out.write(bytes([byte]))

            # Escape sequence
            if byte == 0x00 or byte == 0xFF:
                out.write(bytes([rng.randint(0xFF)]))

    # Trailer
    out.write(b';')


if