#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
     # 2. Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), 'big') % (2 ** 10 - 1) + 1
    height = int.from_bytes(rng.read(2), 'big') % (2 ** 10 - 1) + 1

    flags = bytes([int.from_bytes(rng.read(1), 'big')])
    if len(flags)!= 1: raise Exception('Invalid flag length!')
    flags += bytes([0]) * 7

    bgColorIndex = int.from_bytes(rng.read(1), 'big') & 0b00001111
    pixelAspectRatio = int.from_bytes(rng.read(1), 'big')

    logicalScreenDescriptor = \
            header + \
            width.to_bytes(2, 'big') + \
            height.to_bytes(2, 'big') + \
            flags + \
            bgColorIndex.to_bytes(1, 'big') + \
            pixelAspectRatio.to_bytes(1, 'big')

    out.write(logicalScreenDescriptor)

    # 3. Global Color Table
    globalColorTableSize = int.from_bytes(rng.read(1), 'big')
    globalColorTable = bytearray()
    for _ in range(globalColorTableSize):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')

        colorTableEntry = r | (g << 8) | (b << 16)
        globalColorTable.extend(colorTableEntry.to_bytes(4, 'little'))

    out.write(globalColorTable)

    # 4. Loop Control Block
    loopControlBlock = \
            0x21.to_bytes(1, 'big') + \
            0xFF.to_bytes(1, 'big') + \
            0x0B.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00.to_bytes(1, 'big') + \
            0x00