#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a' + bytes([
        0x2C, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
        0xFF, 0xFF, 0xFF, 0xFF
    ])

    out.write(header)

    # 2. Logical screen descriptor
    logical_screen_descriptor = struct.pack('<HHBBB',
        0x7D0B, 0x0000, 0x0000, 0x00, 0x00
    )

    out.write(logical_screen_descriptor)

    # 3. Global color table (palette)
    global_color_table = [
        0x00, 0x00, 0x00,
        0xFF, 0x00, 0x00,
        0x00, 0xFF, 0x00,
        0xFF, 0xFF, 0x00,
        0x00, 0x00, 0xFF,
        0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF
    ] * 64

    for i in range(len(global_color_table)):
        out.write(bytes([
            global_color_table[i]
        ]))

    # 4. Image descriptor
    image_descriptor = struct.pack('<HHHHBB',
        0x0000, 0x0000, 0x0000, 0x0000, 0x00, 0x00
    )

    out.write(image_descriptor)

    # 5. Local color table (palette)
    local_color_table = [
        0x00, 0x00, 0x00,
        0xFF, 0x00, 0x00,
        0x00, 0xFF, 0x00,
        0xFF, 0xFF, 0x00,
        0x00, 0x00, 0xFF,
        0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0xFF,
        0xFF, 0xFF, 0xFF
    ] * 64

    for i in range(len(local_color_table)):
        out.write(bytes([
            local_color_table[i]
        ]))

    # 6. Image data
    image_data = []

    while True:
        byte = rng.read(1)[0]

        if byte == 0x00:
            break

        image_data += [byte]

    out.write(b'\x00')
    out.write(bytes(image_data))