#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
        
    # 3. Global Color Table
    global_color_table_size = int((width * height / 64 + 1) * 4) - 768
    global_color_table = rng.read(global_color_table_size)
    
    
    # 4. Image Descriptor
    image_descriptor = struct.pack('>HHBBB', 0, 0, 0, 0b10000000 | rng.read(1)[0], 0)
    
    # 5. Local Color Table
    local_color_table_size = rng.read(1)[0]
    if local_color_table_size!= 0:
        local_color_table = rng.read(local_color_table_size)
    else:
        local_color_table = None
    
    # 6. Image Data
    image_data_size = ((width * height) // 8) + (((width * height) % 8) > 0)
    image_data = rng.read(image_data_size)
    
    # 7. Trailer
    trailer = b';'
    
    # Write everything to output
    out.write(header)
    out.write(lsd)
    out.write(global_color_table)
    out.write(image_descriptor)
    if local_color_table is not None:
        out.write(local_color_table)
    out.write(image_data)
    out.write(trailer)

