#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng):
    # Header
    out.write("GIF89a".encode())

    width = 120 + (rng.randint(0, 50)) * 4
    height = 60 + (rng.randint(0, 50)) * 4
    print(f"Width={width}, Height={height}")

    flags = b"\x87\x00" if rng.randint(0, 1) == 0 else b"\x07\x00"
    bgColorIndex = rng.randint(0, 255)
    aspectRatio = rng.randint(0, 255)

    out.write(struct.pack("<HHBBB", width, height, flags[0], flags[1],
                          bgColorIndex, aspectRatio))

    # Global Color Table
    for i in range(0, 256):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)

        out.write(struct.pack("BBB", r, g, b))

    # Image Descriptor
    leftPos = 0
    topPos = 0
    width = 120 + (rng.randint(0, 50)) * 4
    height = 60 + (rng.randint(0, 50)) * 4
    flags = b"\x87\x00" if rng.randint(0, 1) == 0 else b"\x07\x00"
    localColorTableFlag = b"\x00"
    interlaceFlag = b"\x00"
    sortFlag = b"\x00"
    reserved = b"\x00\x00"
    lctSize = b"\x00"

    out.write(struct.pack("<HHHHBBBBB", leftPos, topPos, width, height,
                          flags[0], flags[1], localColorTableFlag[0],
                          interlaceFlag[0], sortFlag[0], reserved[0],
                          reserved[1]))

    out.write(lctSize)

    # Local Color Table
    for i in range(0, 256):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)

        out.write(struct.pack("BBB", r, g, b))

    # Image Data
    for y in range(0, height):
        for x in range(0, width):
            colorIndex = rng.randint(0, 255)

            out.write(bytes([colorIndex]))

    # Trailer
    out.write(";".encode())


if