#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'
	# 2. Logical screen descriptor
    width = rng.read(1)[0] % (640 - 50 + 1) + 50
    height = rng.read(1)[0] % (480 - 50 + 1) + 50
    flags = bytes([
        0b00000000 | # Global color table flag
            ((width & 0x7F) << 7) | # Width
            ((height & 0x7F) << 0), # Height
    ])
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = b'\xFF'
    logical_screen_descriptor = header + \
                                flags + \
                                struct.pack('<BBB',
                                            background_color_index,
                                            pixel_aspect_ratio[0],
                                            pixel_aspect_ratio[1])

    # 3. Global color table
    global_color_table = bytearray()
    for i in range(0, 2**int((logical_screen_descriptor[7] >> 7) & 0x7)):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        global_color_table += bytes([r,g,b])

    # 4. Image block
    image_block = b''
    lzw_min_code_size = rng.read(1)[0] % 256
    image_block += bytes([lzw_min_code_size])
    subblocks = []
    while True:
        length = rng.read(1)[0] % 256
        if length == 0: break

        data = rng.read(length)
        subblocks.append(data)
    
    for subblock in subblocks:
        image_block += bytes([len(subblock)]) + subblock

    # 5. Trailer
    trailer = b';'

    # Write everything to output stream
    out.write(logical_screen_descriptor)
    out.write(global_color_table)
    out.write(image_block)
    out.write(trailer)