#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Write header
    out.write(b'GIF89a')

    # Generate image data
    width = 256 + int.from_bytes(rng.read(1), 'big', signed=False) % (2**8 - 256)
    height = 256 + int.from_bytes(rng.read(1), 'big', signed=False) % (2**8 - 256)
    print('Generating {}x{} pixels'.format(width, height))
    out.write(struct.pack('<HHB', width, height, 0xf7))

    # Generate palette
    for i in range(0, 256):
        r = int.from_bytes(rng.read(1), 'big', signed=False) % 256
        g = int.from_bytes(rng.read(1), 'big', signed=False) % 256
        b = int.from_bytes(rng.read(1), 'big', signed=False) % 256
        out.write(struct.pack('<BBB', r, g, b))

    # Generate pixel data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('<B', int.from_bytes(rng.read(1), 'big', signed=False) % 256))

