#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width, height and color depth.
def generate_random(out: BinaryIO, width: int = 1024, height: int = 768, color_depth: int = 8):
    # Header
    out.write("GIF89a".encode())

    # Logical screen descriptor
    out.write(struct.pack("<HHBBB",
        width,
        height,
        0xF7 & (color_depth - 1),
        0x00 | ((color_depth > 8) << 5),
        0x00))

    # Global palette
    for i in range(0, pow(2, color_depth)):
        out.write(struct.pack("<BBBB",
            *[(i >> j*8) % 256 for j in reversed(range(0, color_depth // 8))] + [0]))

    # Image descriptor
    out.write(struct.pack("<BBHBBHHH",
        0x2C,
        0x00,
        0x00,
        0x00,
        0x00,
        width,
        height,
        0x08))

    # Local palette
    if color_depth <= 8:
        out.write(b"\xFF\xFF\xFF")

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([
                *(reversed([(y * x) % 256])),
                0x00]))

