#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given width and height.
def generate_random(width: int, height: int, out: BinaryIO):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB',
        width,  # Width
        height,  # Height
        0xf7,    # Global color table flag (1)
        0x00,    # Background color index
        0x00     # Pixel aspect ratio
    ))

    # Color table
    for i in range(256):
        out.write(struct.pack('BBB',
            *tuple((i >> j & 1) * 255 for j in [2, 1, 0])
        ))

    # Image descriptor
    out.write(struct.pack('<BHHHHBB',
        0x2c,        # Image separator
        width,       # Left position
        height,      # Top position
        width,       # Width
        height,      # Height
        0x00,        # Local color table flag (0)
        0x00         # LZW minimum code size
    ))

    # Image data
    while True:
        # Block size
        block = bytearray()

        # Generate random bytes until we get one that doesn't start with 0x00 or 0xff
        while len(block) == 0 or block[0] >= 0x00 or block[0] <= 0xff:
            block += struct.pack('B',
                ord('A') + ((ord('Z') - ord('A')) * (
                    (
                        sum([
                            ord(chr(i)) ^ (
                                (((i % 4) << 2) | ((i // 4) % 4)) ^
                                0x0f
                            )
                            for i in range(16)
                        ]) % 256
                    ) / 256
                ))
            )

        # Write block to output stream
        out.write(bytes([len(block)]))
        out.write(block)

        if len(block) < 255:
            break


if