#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng.randint(50, 200) // 2
    height = 1 + rng.randint(50, 200) // 2
    packed = (width - 1).to_bytes(2, 'little', signed=False) \
        + (height - 1).to_bytes(2, 'little', signed=False) \
        + b'\x87\x00\x00'

    out.write(packed)

    # Global color table
    for i in range(4 * 6):
        out.write((rng.randint(0, 255)).to_bytes(1, 'little'))

    # Image descriptor
    packed = (0, 0, width, height).to_bytes(4, 'little') \
        + b'\x00' \
        + (0).to_bytes(1, 'little') \
        + (0).to_bytes(1, 'little')

    out.write(packed)

    # Local color table
    for i in range(4 * 6):
        out.write((rng.randint(0, 255)).to_bytes(1, 'little'))

    # LZW minimum code size
    out.write(b'\x02')

    # Data sub-blocks
    data = bytearray()
    while len(data) < 10000:
        if rng.randint(0, 1) == 0:
            # RLE block
            length = min(rng.randint(1, 12), 127)

            if rng.randint(0, 1) == 0:
                # Raw bytes
                data += rng.getrandbits(length * 8).to_bytes(length, 'big')
            else:
                # Repeated byte
                data += ((rng.randint(0, 255))).to_bytes(1, 'big') * length

        else:
            # Table-based compression block
            length = min(rng.randint(1, 12), 127)

            for _ in range(length):
                index = rng.randint(0, 15)

                if index > 0:
                    # Copy from previous frame
                    data += data[-index:]
                elif index == 0:
                    # Repeat last byte
                    data += data[-1].to_bytes(1, 'big')
                else:
                    # Literal byte
                    data += ((rng.randint(0, 255))).to_bytes(1, 'big')

    # Write all data at once to avoid having to seek back and rewrite it later
    out.write(data)


if