
import sys
import random
import struct
from datetime import datetime

def main():
    if len(sys.argv) < 2:
        print("Usage: python gen_gif.py <outfile>")
        return
    
    out = open(sys.argv[1], 'wb')
    rng = random.Random()

    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width = rng.randint(1, 100) * 10
    height = rng.randint(1, 100) * 10
    packed = struct.pack('>BBHBB', width, height, 0x80, 0, 0)
    lsd = packed[:3] + b'\xFF' + packed[3:]

    # 3. Color Table
    color_table = bytes([0]*768)

    # 4. Graphics Control Extension
    if rng.randint(0, 1):
        delay_time = rng.randint(1, 10)*10
        transparent_color_index = rng.randint(0, 255)
        gce = struct.pack('>BBBB', 0xF9, 0x04, delay_time, transparent_color_index)
    else:
        gce = None

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes

        if not block_size:
            break
        
        blocks.append(bytes([block_size]) + rng.read(block_size))
        
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + color_table + (gce or b'') + image_descriptor + image_data + trailer)

