#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt
    out.write(b"GIF89a")  # Magic header
    width = int.from_bytes(rng.read(2), "big", signed=False) % 640 + 1
    height = int.from_bytes(rng.read(2), "big", signed=False) % 480 + 1
    print("width=", width, ", height=", height)
    out.write(struct.pack("<HHB", width, height, 0xF))  # Logical screen descriptor
    out.write(b"\x00\x00\xFF\xFF")  # Global color table (black and white only)
    out.write(b"\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01;")  # Image block
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))

