#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    # We use a global color table with 256 colors
    gct = rng.read(768)
    
    # 4. Application Extension
    # We use an application extension to store the name of our program
    appext_header = struct.pack('BB', 0x21, 0xFF)
    appext_data = struct.pack('>BBB', 0x0B, 0x4E, 0x45) + 'Nice Gif'.encode() + b'\x00'
    
    # 5. Image Descriptor
    idesc = struct.pack('>BBHHHBB', 0x2C, 0x00, 0x00, width, height, 0x80 | rng.read(1)[0], 0x00)
    
    # 6. Local Color Table
    # We use a local color table with 256 colors
    lct = rng.read(768)
    
    # 7. Image Data
    # We use a single data sub-block
    imgdata = struct.pack('BB', 0x21, 0xF9) + rng.read(4)
    
    # 8. Trailer
    trailer = b'\x3B'
    
    # Write everything together
    out.write(header + lsd + gct + appext_header + appext_data + idesc + lct + imgdata + trailer)


with open('/dev/urandom') as rng:
    with open