#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Image Data
    image_data = []
    # Loop over all pixels in the image
    for y in range(height):
        row = []
        for x in range(width):
            # Each pixel is represented by one byte
            # We use this byte as an index to the global color table
            row.append(struct.pack('B', rng.read(1)[0]))
        image_data.append(b''.join(row))

    # Write everything together
    out.write(header + lsd + color_table + b'\x00\x2C' + b''.join(image_data) + b'\x3B')

