#!/usr/bin/env python3

import sys
import os
import random
import struct

if len(sys.argv) != 2:
    print("Usage: {} <output filename>".format(os.path.basename(__file__)))
    exit()

with open(sys.argv[1], 'wb') as out:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width = 100
    height = 100
    packed_fields = 0x80 | ((width - 1) << 4) | (height - 1)
    background_color_index = 0
    aspect_ratio = 0
    lsd = struct.pack('<BBHBB', packed_fields, background_color_index, aspect_ratio, 0, 0)

    # 3. Global Color Table
    color_table = bytes([0]*3*2**((lsd[0] & 0x07)+1))

    # 4. Graphic Control Extension
    gce_block_size = 0xF9
    gce_packed_fields = 0xF9
    delay_time = 0
    transparent_color_index = 0
    terminator = 0
    gce = struct.pack('<BBBBB', gce_block_size, gce_packed_fields, delay_time, transparent_color_index, terminator)

    # 5. Image Descriptor
    rng = random.Random()
    left = rng.randint(0, width - 1)
    top = rng.randint(0, height - 1)
    width = rng.randint(0, width - left)
    height = rng.randint(0, height - top)
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    for _ in range(total_blocks_size):
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)