#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), 'little') % 640 + 1
    height = int.from_bytes(rng.read(2), 'little') % 480 + 1
    background_color_index = rng.read(1)[0] & 0xFF
    pixel_aspect_ratio = rng.read(1)[0] & 0xFF
    global_color_table_flag = (rng.read(1)[0] & 0x80) >> 7
    color_resolution = ((rng.read(1)[0] & 0x70) >> 4) + 1
    sort_flag = (rng.read(1)[0] & 0x08) >> 3
    size_of_global_color_table = 2 ** ((rng.read(1)[0] & 0x07) + 1)
    background_color_index &= size_of_global_color_table - 1
    global_color_table = [None] * size_of_global_color_table
    for i in range(size_of_global_color_table):
        global_color_table[i] = rng.read(3)
        
    # 3. Global Color Table
    if global_color_table_flag == 1:
        for color in global_color_table:
            out.write(color)
            
    # 4. Image Descriptors
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\x00'
    out.write(image_descriptor)
    
    # 5. Local Color Table Flag
    local_color_table_flag = (rng.read(1)[0] & 0x80) >> 7
    
    # 6. Interlace Flag
    interlace_flag = (rng.read(1)[0] & 0x40) >> 6
    
    # 7. Sort Flag
    sort_flag = (rng.read(1)[0] & 0x20) >> 5
    
    # 8. Size of Local Color Table
    size_of_local_color_table = 2 ** ((rng.read(1)[0] & 0x18) >> 3)
    
    # 9. Reserved
    reserved = rng.read(1)[0] & 0x07
    
    # 10. LZW Minimum Code Size
    lzw_minimum_code_size = rng.read(1)[0] & 0xF
    
    # 11. Image Data
    image_data = []
    while True:
        code_size = rng.read(1)[0] & 0xFF
        if code_size == 0x00:
            break
        else:
            image_data.append(code_size)
    out.write(struct.pack('B', len(image_data)))
    out.write(b''.join(image_data))
    
    # 12. Trailer
    trailer = b';'
    out.write(trailer)
    