#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical screen descriptor
    width = rng.read(2)[0] + (rng.read(1)[0] << 8)
    height = rng.read(2)[0] + (rng.read(1)[0] << 8)
    flags = rng.read(1)[0]
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HBBB', width, height, flags, background_color_index))
    out.write(pixel_aspect_ratio)

    # 3. Global color table
    if flags & 0x80 != 0:
        for _ in range(2 ** ((flags >> 4) + 1)):
            out.write(rng.read(3))

    # 4. Image descriptor
    left = rng.read(2)[0] + (rng.read(1)[0] << 8)
    top = rng.read(2)[0] + (rng.read(1)[0] << 8)
    width = rng.read(2)[0] + (rng.read(1)[0] << 8)
    height = rng.read(2)[0] + (rng.read(1)[0] << 8)
    flags = rng.read(1)[0]
    local_color_table_size = (flags >> 4) + 1
    interlaced = flags & 0x40 != 0
    sorted = flags & 0x80 != 0
    out.write(struct.pack('<HHBB', left, top, width, height))
    out.write(flags)

    # 5. Local color table
    if flags & 0x80 == 0:
        for _ in range(2 ** local_color_table_size):
            out.write(rng.read(3))

    # 6. LZW-compressed image data
    lzw_minimum_code_size = rng.read(1)[0]
    out.write(lzw_minimum_code_size)
    while True:
        block_size = rng.read(1)[0]
        if block_size == 0:
            break
        out.write(rng.read(block_size))
    # 7. Trailer
    out.write(b'\x3b')