#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Graphic Control Extension (one per frame)
    # We'll use the same delay time for all frames
    gce = struct.pack('BBB', 0xF9, 0x04, 0x04) + rng.read(1)

    # 5. Image Descriptor
    # Random left and top position, using two bytes for each
    image_left, image_top = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>HHBB', image_left, image_top, 0, 0)

    # 6. Image Data
    # Random length of data, using two bytes for each
    image_length = struct.unpack('>HH', rng.read(4))[0]
    image_data = rng.read(image_length)

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)