#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Write header
    out.write(b'GIF89a')
    
    # Write screen descriptor
    out.write(struct.pack('<H', 16)) # Width
    out.write(struct.pack('<H', 16)) # Height
    out.write(b'\x00\x00')           # Background color index
    out.write(b'\x00')               # Aspect ratio (unused)
    
    # Write global color table
    for i in range(256):
        out.write(struct.pack('BBB', *[i] * 3))
        
    # Write application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<I', 0))
    
    # Write image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # Write local color table
    for i in range(256):
        out.write(struct.pack('BBB', *[i] * 3))
    
    # Write image data
    while True:
        out.write(rng.read(1))
