#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> int:
    # 1. Header
    # Signature
    out.write("GIF".encode())
    
    # Version
    out.write("\x38\x9b".encode())
    
    # Logical screen descriptor
    # Width
    out.write(struct.pack("<H", rng.read(2)))
    
    # Height
    out.write(struct.pack("<H", rng.read(2)))
    
    # Packed fields
    packed = rng.read(1)[0]
    if (packed & 0b10000000):
        raise Exception("Global color table flag is set")
        
    # Background color index
    out.write(struct.pack("<B", rng.read(1)[0]))
    
    # Pixel aspect ratio
    out.write(struct.pack("<B", rng.read(1)[0]))
    
    # Global color table
    for _ in range(0, 768):
        out.write(struct.pack("<B", rng.read(1)[0]))
    
    # 2. Image data
    # Image separator
    out.write(struct.pack("<B", 0x2c))
    
    # Image descriptor
    # Left position
    out.write(struct.pack("<H", rng.read(2)))
    
    # Top position
    out.write(struct.pack("<H", rng.read(2)))
    
    # Width
    out.write(struct.pack("<H", rng.read(2)))
    
    # Height
    out.write(struct.pack("<H", rng.read(2)))
    
    # Packed fields
    packed = rng.read(1)[0]
    if (packed & 0b10000000):
        raise Exception("Local color table flag is set")
        
    # Reserved
    if (packed & 0b01000000):
        raise Exception("Interlace flag is set")
        
    # Sort flag
    if (packed & 0b00100000):
        raise Exception("Sort flag is set")
        
    # Size of local color table
    if (packed & 0b00011100):
        raise Exception("Size of local color table is non-zero")
        
    # Background color index
    out.write(struct.pack("<B", rng.read(1)[0]))
    
    # Pixel aspect ratio
    out.write(struct.pack("<B", rng.read(1)[0]))
    
    # Local color table
    for _ in range(0, 768):
        out.write(struct.pack("<B", rng.read(1)[0]))
    
    # LZW minimum code size
    out.write(struct.pack("<B", rng.read(1)[0]))
    
    # LZW compressed image data
    while True:
        byte = rng.read(1)[0]
        out.write(struct.pack("<B", byte))
        
        if (byte == 0x00):
            break
            
    # Trailer
    out.write(struct.pack("<B", 0x3b))
    
    return 0
