#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Graphics Control Extension (optional)
    if rng.read(1)[0]:
        gce = struct.pack('>BB', rng.read(1)[0], rng.read(1)[0])
    else:
        gce = b''

    # 5. Image Descriptor
    idd = b'\x2c\x00\x00\x00\x00' + struct.pack('>HH', width, height)

    # 6. Local Color Table (optional)
    if rng.read(1)[0]:
        local_color_table = rng.read(3 * num_colors)
    else:
        local_color_table = b''

    # 7. Image Data
    image_data = rng.read(width * height)

    # 8. Trailer
    trailer = b';'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + idd + local_color_table + image_data + trailer)

