#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    width = 1024
    height = 768
    color_table_size = 7
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBB', width, height, color_table_size, background_color_index))
    out.write(pixel_aspect_ratio.to_bytes(1, 'little'))
    
    # Global color table
    for i in range(2 ** (color_table_size + 1)):
        out.write(rng.read(3))
        
    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(b'\x00\x00\x00')
    
    # Image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table_flag = False
    interlace_flag = True
    sort_flag = False
    reserved = 0
    out.write(struct.pack('<HHHBBB', left, top, width, height, local_color_table_flag, interlace_flag, sort_flag, reserved))
    
    # Local color table
    for i in range(2 ** (color_table_size + 1)):
        out.write(rng.read(3))
    
    # Raster data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))
            
        # End of line
        out.write(b'\x00')
    
    # Trailer
    out.write(b';')
