#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Header
    out.write('GIF89a'.encode())
    out.write(struct.pack("<HH", width, height))
    out.write(b'\x00\x00')  # Background color index
    out.write(b'\x00')      # Pixel aspect ratio
    
    # Graphic control extension
    out.write(b'\x21\xf9\x04\x05\xff\xff\xff\x00')
    
    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00' + struct.pack("<HH", width, height) + b'\x00')
    
    # Local color table
    for i in range(2 ** 8):
        r = (i >> 5) & 7
        g = (i >> 2) & 7
        b = i & 3
        
        out.write(bytes([r * 36, g * 36, b * 85]))
    
    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([y % 256, x % 256]))
