#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), 'big') % 65536 + 1
    height = int.from_bytes(rng.read(2), 'big') % 65536 + 1
    background_color_index = int.from_bytes(rng.read(1), 'big') % 256
    pixel_aspect_ratio = int.from_bytes(rng.read(1), 'big') % 256
    global_color_table_flag = bool(int.from_bytes(rng.read(1), 'big'))
    color_resolution = int.from_bytes(rng.read(1), 'big') % 8
    sort_flag = bool(int.from_bytes(rng.read(1), 'big'))
    size_of_global_color_table = 2 ** (color_resolution + 1)
    background_color_index = background_color_index % size_of_global_color_table
    pixel_aspect_ratio = pixel_aspect_ratio % 4
    global_color_table_flag = bool(int.from_bytes(rng.read(1), 'big'))
    global_color_table = []
    if global_color_table_flag:
        for i in range(size_of_global_color_table):
            red = int.from_bytes(rng.read(1), 'big') % 256
            green = int.from_bytes(rng.read(1), 'big') % 256
            blue = int.from_bytes(rng.read(1), 'big') % 256
            global_color_table.append((red, green, blue))
            
    # 3. Graphic Control Extension
    graphic_control_extension = b'\x21\xf9\x04\x01\xff\x00\x00\x00\x00'
        
    # 4. Image Descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00'
    
    # 5. Local Color Table Flag
    local_color_table_flag = bool(int.from_bytes(rng.read(1), 'big'))
    
    # 6. Interlace Flag
    interlace_flag = bool(int.from_bytes(rng.read(1), 'big'))
    
    # 7. Sort Flag
    sort_flag = bool(int.from_bytes(rng.read(1), 'big'))
    
    # 8. Size of Local Color Table
    size_of_local_color_table = 2 ** (int.from_bytes(rng.read(1), 'big') % 8)
    
    # 9. Reserved
    reserved = int.from_bytes(rng.read(1), 'big') % 256
    
    # 10. Local Color Table
    local_color_table = []
    if local_color_table_flag:
        for i in range(size_of_local_color_table):
            red = int.from_bytes(rng.read(1), 'big') % 256
            green = int.from_bytes(rng.read(1), 'big') % 256
            blue = int.from_bytes(rng.read(1), 'big') % 256
            local_color_table.append((red, green, blue))
            
    # 11. LZW Minimum Code Size
    lzw_minimum_code_size = int.from_bytes(rng.read(1), 'big') % 256
    
    # 12. Pixel Data Sub-blocks
    pixel_data_sub_blocks = []
    while True:
        sub_block_size = int.from_bytes(rng.read(1), 'big') % 256
        if sub_block_size == 0:
            break
        pixel_data_sub_blocks.append(rng.read(sub_block_size))
    
    # 13. Trailer
    trailer = b'\x3b'
    
    # Write everything to the output stream
    out.write(header)
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(struct.pack('B', background_color_index))
    out.write(struct.pack('B', pixel_aspect_ratio))
    out.write(graphic_control_extension)
    out.write(image_descriptor)
    out.write(struct.pack('B', local_color_table_flag))
    out.write(struct.pack('B', interlace_flag))