#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width = rng.read(2)
    height = rng.read(2)
    # Packed field:
    #   - Global Color Table Flag = 0
    #   - Color Resolution = 7
    #   - Sort Flag = 0
    #   - Size of Global Color Table = 64 colors
    packed_field = b'\x80\xFF'
    # Background color index
    background_color_index = rng.read(1)
    # Pixel aspect ratio
    pixel_aspect_ratio = b'\x00'
    out.write(width + height + packed_field + background_color_index + pixel_aspect_ratio)

    # 3. Global Color Table
    # 64 x 3-byte RGB values
    out.write(rng.read(64 * 3))

    # 4. Image Data
    # 1. Graphic Control Extension
    #    1. Block size
    block_size = b'\x04'
    #    2. Packed field
    #       - Reserved = 0
    #       - Disposal method = 0 (no disposal specified)
    #       - User input flag = 0
    #       - Transparent color flag = 0
    packed_field = b'\x00'
    #    3. Delay time
    delay_time = rng.read(2)
    #    4. Transparent color index
    transparent_color_index = b'\x00'
    #    5. Block terminator
    block_terminator = b';'
    out.write(block_size + packed_field + delay_time + transparent_color_index + block_terminator)

    # 2. Image Descriptor
    #    1. Separator
    separator = b','
    #    2. Left position
    left_position = rng.read(2)
    #    3. Top position
    top_position = rng.read(2)
    #    4. Width
    width = rng.read(2)
    #    5. Height
    height = rng.read(2)
    #    6. Packed field
    #       - Local Color Table Flag = 0
    #       - Interlace Flag = 0
    #       - Sort Flag = 0
    #       - Size of Local Color Table = 0
    packed_field = b'\x00'
    out.write(separator + left_position + top_position + width + height + packed_field)

    # 3. LZW Minimum Code Size
    lzw_minimum_code_size = rng.read(1)
    out.write(lzw_minimum_code_size)

    # 4. Image Data
    # 1. Clear code
    clear_code = b'\x00'
    # 2. End of Information code
    end_of_information_code = b'\x00'
    # 3. Random codes
    while True:
        # Read one byte from /dev/urandom
        code = rng.read(1)
        if not code:
            break
        # Write it to the output
        out.write(code)

    # 5. Trailer
    trailer = b'\x3B'
    out.write(trailer)

