#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    global_color_table = rng.read(num_colors * 3)

    # 4. Image Descriptors
    # We'll just use one image descriptor, with a random width and height
    image_width, image_height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>HHBB', image_width, image_height, rng.read(1)[0], rng.read(1)[0])

    # 5. Local Color Table (same as above)
    local_color_table = rng.read(num_colors * 3)

    # 6. Image Data
    # We'll just use one data sub-block, with a random length
    data_length = rng.read(1)[0]
    image_data = rng.read(data_length)

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything to the output stream
    out.write(header + lsd + global_color_table + image_descriptor + local_color_table + image_data + trailer)

