#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
    out: BinaryIO,
    width: int, height: int,
    rng: BinaryIO,
):
    
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    lsd = struct.pack('<HHBB', width, height, 0x80 | 0x40, 0)  # Global Color Table Flag and Color Resolution set to 8 bits per primary color

    # 3. Global Color Table
    color_table = rng.read(3 * 2 ** (lsd[3] + 1))  # Read 2^(Color Resolution + 1) colors from /dev/urandom

    # 4. Graphic Control Extension
    gce = None

    # 5. Image Descriptor
    image_descriptor = struct.pack('<BHHHBB', 0x2c, 0, 0, width, height, 0)  # No local color table

    # 6. Image Data
    lzw_minimum_code_size = rng.read(1)[0]  # LZW minimum code size

    blocks = []
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes

        if not block_size:
            break
        
        blocks.append(bytes([block_size]) + rng.read(block_size))
        
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + color_table + (gce or b'') + image_descriptor + image_data + trailer)

