#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = 640
    height = 480
    background_color_index = 0
    pixel_aspect_ratio = 0
    color_table_size = 7
    color_resolution = 7
    sort_flag = 0
    global_color_table_flag = 1
    color_table_bytes = 3 * (2 ** (color_table_size + 1))
    logical_screen_descriptor = struct.pack('<HHBBB',
                                            width, height,
                                            background_color_index,
                                            pixel_aspect_ratio,
                                            color_table_size << 4 | color_resolution << 2 | sort_flag << 1 | global_color_table_flag)
    
    # 3. Global Color Table
    global_color_table = bytearray()
    for i in range(2 ** (color_table_size + 1)):
        red = int.from_bytes(rng.read(1), 'big') % 256
        green = int.from_bytes(rng.read(1), 'big') % 256
        blue = int.from_bytes(rng.read(1), 'big') % 256
        global_color_table += bytes([red, green, blue])
        
    # 4. Image Data
    image_data = bytearray()
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff!'
    image_data += b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\x