#!/usr/bin/env python3

import sys, struct
from PIL import Image

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: %s <filename>" % sys.argv[0])
        exit()
        
    filename = sys.argv[1]
    im = Image.open(filename).convert('RGB')
    width, height = im.size
    data = im.tobytes()
    
    # 1. Header
    header = b'\x47\x49\x46\x38\x39\x61'
    
    # 2. Logical Screen Descriptor (LSD)
    lsd = struct.pack('<HHBB', width, height, 0x80, 0)
    
    # 3. Global Color Table
    color_table = bytes([0]*3*256)
    
    # 4. Graphic Control Extension (GCE)
    gce = b'\x21\xf9\x04\x08\x00\x00\xff\xff\x00'
    
    # 5. Image Descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00' + struct.pack('BB', width, height) + b'\x00'
    
    # 6. Data Blocks
    blocks = []
    while data:
        chunk = data[:255]
        data = data[len(chunk):]
        blocks.append(chunk)
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything together
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    out.write(gce)
    out.write(image_descriptor)
    for block in blocks:
        out.write(struct.pack('B', len(block)))
        out.write(block)
    out.write(trailer)
