#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 4 bytes for the magic header
    out.write(b'GIF89a')

    # 7 bytes for the logical screen descriptor
    out.write(struct.pack('<HHBB', 200, 150, 0xF0 | (rng.read(1)[0] & 0x0F), 0))

    # 3 bytes for the global color table
    out.write(rng.read(3))

    # 1 byte for the background color index
    out.write(rng.read(1))

    # 1 byte for the pixel aspect ratio
    out.write(rng.read(1))

    # 1 byte for the image separator
    out.write(b'\x2C')

    # 9 bytes for the image descriptor
    out.write(struct.pack('<HHHBB', 0, 0, 200, 150, 0))

    # 3 bytes for the local color table
    out.write(rng.read(3))

    # 1 byte for the image data block terminator
    out.write(b'\x00')

    # 1 byte for the trailer
    out.write(b';')

