#!/usr/bin/env python3
import sys
import os
import struct
from Crypto.Random import get_random_bytes


if __name__ == '__main__':
    if len(sys.argv)!= 2:
        print("Usage: {} <output>".format(os.path.basename(__file__)))
        exit()

    # Open our output file
    out = open(sys.argv[1], 'wb')

    # Create a PRNG instance using /dev/urandom as its source
    rng = get_random_bytes

    # 1. Header
    header = b'\x89\x50\x4E\x47\x0D\x0A\x1A\x0A'

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    bit_depth = rng.read(1)[0] & 0b1111111
    color_type = rng.read(1)[0] & 0b111
    compression_method = rng.read(1)[0] & 0b11111111
    filter_method = rng.read(1)[0] & 0b1111111
    interlace_method = rng.read(1)[0] & 0b111
    lsd = struct.pack('>BBHHBBBB', width, height, bit_depth, color_type,
                      compression_method, filter_method, interlace_method)

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Graphics Control Extension (optional)
    if rng.read(1)[0]:
        gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    else:
        gce = b''

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    for _ in range(total_blocks_size):
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)