#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec 10 21:56:49 2017

@author: <NAME>
"""
import sys, os
from PIL import Image


def main():
    if len(sys.argv)!= 3:
        print("Usage: {} input_file output_file".format(os.path.basename(__file__)))
        return
    
    in_filename = sys.argv[1]
    out_filename = sys.argv[2]

    img = Image.open(in_filename).convert('RGB')
    width, height = img.size

    with open(out_filename, 'wb') as out:

        # Header
        header = b'\x47\x49\x46\x38\x37\x61'
        lsd = bytes([width & 0xFF, (width >> 8) & 0xFF,
                     height & 0xFF, (height >> 8) & 0xFF])
        
        # Color table
        color_table = bytearray()
        for i in range(256):
            r, g, b = img.getpixel((i % width, i // width))
            color_table += bytes([b, g, r])

        # Graphics control extension
        gce = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'

        # Image descriptor
        image_descriptor = bytes([
            0x2C, 0x00, 0x00, 0x00,             # Image separator
            0x00, 0x00,                       # Left position
            0x00, 0x00,                       # Top position
            width & 0xFF,                     # Width
            (width >> 8) & 0xFF,
            height & 0xFF,                    # Height
            (height >> 8) & 0xFF,
            0x08,                             # Bit depth
            0x02,                             # Compression method
            0x00,                             # Filter method
            0x00                              # Interlace method
        ])

        # Image data
        pixels = list(img.getdata())
        image_data = bytearray()
        for y in range(height):
            for x in range(width):
                pixel = pixels[(y * width) + x]
                image_data += bytes([
                    pixel[2],
                    pixel[1],
                    pixel[0]])

        trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)