#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Image Descriptor
    image_descriptor = struct.pack('>HHBBB', 0, 0, 0, 0b10000000 | rng.read(1)[0], 0)

    # 5. Image Data
    # Let's make this an infinite loop of frames with random length
    while True:

        # Frame control block
        frame_control_block = struct.pack('>BBH', 0x2C, 0, 0)

        # Frame data
        frame_data = b''
        while len(frame_data) < rng.read(1)[0]:
            frame_data += rng.read(1)

        # Trailer
        trailer = b'\x3B'

        # Write everything to output stream
        out.write(header + lsd + color_table + image_descriptor + frame_control_block + frame_data + trailer)

