#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    out.write("GIF89a".encode())

    width = 1024
    height = 768
    color_table_size = 256
    background_color = 0
    aspect_ratio = b"\x00\x00"

    out.write(struct.pack("<HHBBB",
                          width, height,
                          color_table_size,
                          background_color,
                          *aspect_ratio))

    # Global Color Table
    for i in range(0, (color_table_size + 1)):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        out.write(struct.pack("<BBB",
                              r, g, b))

    # Image Descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table_flag = False
    interlace_flag = True
    sort_flag = False
    reserved_flags = b"\x00\x00"
    lct_size = 256
    lct = []

    if local_color_table_flag:
        for i in range(0, (lct_size + 1)):
            r = rng.read(1)[0] % 256
            g = rng.read(1)[0] % 256
            b = rng.read(1)[0] % 256

            lct.append((r, g, b))

        out.write(struct.pack("<BBBBBHB",
                              left, top,
                              width, height,
                              local_color_table_flag,
                              interlace_flag,
                              sort_flag,
                              *reserved_flags,
                              lct_size))

        for c in lct:
            out.write(struct.pack("<BBB",
                                  c[0], c[1], c[2]))

    else:
        out.write(struct.pack("<BBBBBH",
                              left, top,
                              width, height,
                              local_color_table_flag,
                              interlace_flag,
                              sort_flag,
                              *reserved_flags))

    # LZW Minimum Code Size
    min_code_size = 2 ** 8 - 1
    out.write(min_code_size.to_bytes(1, "big"))

    # Frame Data
    data = bytearray()
    while len(data) < ((width * height) // 2):
        data += rng.read(1)

    out.write(data)

    # Trailer
    out.write("\x3B".encode())

