#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 768, color_depth: int = 8
):

    # Header
    out.write("GIF89a".encode())
    out.write(struct.pack("<HH", width, height))
    out.write((color_depth - 1).to_bytes(1, byteorder="little"))
    out.write("\x00\x00".encode())

    # Logical screen descriptor
    out.write("\xFF\x0B".encode())
    out.write(struct.pack("<HHBBBHB", width, height, (color_depth - 1), 0, 0, 0, 0))

    # Global color table
    for i in range(pow(2, color_depth)):
        out.write(i.to_bytes(1, byteorder="little"))

    # Image descriptor
    out.write("\x2C\x00\x00\x00".encode())
    out.write(struct.pack("<HHBBH", 0, 0, width, height, 0))

    # Local color table
    if color_depth == 8:
        for i in range(256):
            out.write(i.to_bytes(1, byteorder="little"))

    # Image data
    out.write("\x21\xF9\x04\x00\x00\x00\x00\x00\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;".encode())


