#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    out.write(b'GIF89a')

    width = 1024
    height = 768
    bpp = 8
    color_table_size = (bpp == 8 and 256 or 1 << bpp) * 3
    global_color_table = True
    background_index = -1
    aspect_ratio = 0x40
    flags = 0x80 | 0x40 if global_color_table else 0x80
    frame_count = 1

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB',
                          width, height, bpp, color_table_size, flags))
    out.write(bytes([background_index]))
    out.write(bytes([aspect_ratio]))

    # Global color table
    for i in range(color_table_size // 3):
        r = next(rng) & 0xff
        g = next(rng) & 0xff
        b = next(rng) & 0xff

        out.write(bytes([r, g, b]))

    # Image descriptor
    out.write(struct.pack('<BHHBB',
                          0x2c, x, y, bpp, flags))
    out.write(bytes([background_index]))
    out.write(bytes([aspect_ratio]))

    # Pixel data
    for y in range(height):
        for x in range(width):
            index = next(rng) % color_table_size

            out.write(bytes([index]))

