#!/usr/bin/env python3

import struct, sys


def read_uint16(f):
    return int.from_bytes(f.read(2), byteorder='little')


def read_uint32(f):
    return int.from_bytes(f.read(4), byteorder='little')


def read_uint8(f):
    return f.read(1)[0]


def write_uint16(f, value):
    f.write(value.to_bytes(2, 'little'))


def write_uint32(f, value):
    f.write(value.to_bytes(4, 'little'))


def write_uint8(f, value):
    f.write(struct.pack('B', value))


def main():

    if len(sys.argv)!= 5:
        print("Usage: {} <input file> <output file> <offset> <size>".format(
            sys.argv[0]))
        exit()

    input = open(sys.argv[1], "rb")
    output = open(sys.argv[2], "wb")

    offset = int(sys.argv[3])
    size = int(sys.argv[4])

    # Read the header and check that it is a valid DOL
    magic = input.read(4)
    assert (magic == b"DOL\x00"), "Not a valid DOL."

    version = read_uint8(input)
    assert (version in [0, 1]), "Unsupported version of DOL."

    entrypoint = read_uint32(input)
    text_offset = read_uint32(input) + offset
    text_size = read_uint32(input) - text_offset
    data_offset = read_uint32(input) + offset
    data_size = read_uint32(input) - data_offset

    # Write the new header to the output file
    output.write(b"DOL\x00")
    write_uint8(output, version)
    write_uint32(output, entrypoint)
    write_uint32(output, text_offset)
    write_uint32(output, text_size)
    write_uint32(output, data_offset)
    write_uint32(output, data_size)

    # Copy the rest of the file as-is
    while True:

        chunk = input.read(1024 * 1024)
        if not chunk: break

        output.write(chunk)

    input.close()
    output.close()


if