#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given parameters.
def generate(
    out: BinaryIO, width: int = 1024, height: int = 768, delay_ms: int = 50
) -> None:

    # Header
    out.write(b"GIF89a")
    out.write(struct.pack("<HH", width, height))
    out.write(b"\x87\x00\x00")

    # Logical screen descriptor
    out.write(b"\x2c\x00\x00")
    out.write(struct.pack("<HH", width, height))
    out.write(b"\x00\x3b")

    # Global color table (black and white only).
    out.write(b"\x02\x00")
    out.write(b"\x00\x00\x00\xff\xff\xff")

    # Image descriptor
    out.write(b"\x2c\x00\x00")
    out.write(struct.pack("<HH", 0, 0))
    out.write(struct.pack("<HH", width, height))
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")

    # Local color table (black and white only).
    out.write(b"\x02\x00")
    out.write(b"\x00\x00\x00\xff\xff\xff")

    # Image data
    for y in range(height):
        for x in range(width):
            if ((y // 2 * 2 + x // 3 * 3) % 2 == 0) ^ bool((x + y) % 2):
                out.write(b"\x00\x00\x00")
            else:
                out.write(b"\xff\xff\xff")

    # Graphics control extension
    out.write(b"\x21\xf9\x04")
    out.write(struct.pack("B", delay_ms & 0xff))
    out.write(struct.pack("B", (delay_ms >> 8) & 0xff))
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")

    # Application extension
    out.write(b"\x21\xff\x0bNETSCAPE2.0")
    out.write(b"\x03\x01")
    out.write(b"\x00\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")
    out.write(b"\x00")

    # Trailer
    out.write(b";")


if