#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Oct 25 10:47:39 2019

@author: jlee
"""

import os, sys
import numpy as np
from astropy.io import fits
from astropy.table import Table
from scipy.ndimage.filters import gaussian_filter

import matplotlib.pyplot as plt
from matplotlib.ticker import ScalarFormatter
from matplotlib.colors import LogNorm

import warnings; warnings.simplefilter('ignore')

def plot_spectra(wav, flux, ivar, mask, outname):
    """
    Plot the spectra with the masked regions and save it to a file.
    
    Parameters
    ----------
    wav : array
        Wavelength of each pixel in Angstroms.
        
    flux : array
        Flux density of each pixel in erg/cm^2/s/A.
        
    ivar : array
        Inverse variance of each pixel.
        
    mask : array
        Mask for each pixel.
        
    outname : str
        Name of output file.
        
    Returns
    -------
    None
    """
    
    # Create figure
    fig = plt.figure()
    ax = fig.add_subplot(111)
    
    # Plot spectrum
    ax.plot(wav, flux, color='black', lw=0.5)
    
    # Set axes limits
    ax.set_xlim([np.min(wav), np.max(wav)])
    ax.set_ylim([np.median(flux)*0.9, np.median(flux)*1.1])
    
    # Set axes labels
    ax.set_xlabel('Wavelength (Angstroms)')
    ax.set_ylabel('Flux Density (erg cm$^{-2}$ s$^{-1}$ A$^{-1}$)')
    
    # Set ticks
    ax.xaxis.set_major_locator(plt.MultipleLocator(5))
    ax.xaxis.set_minor_locator(plt.MultipleLocator(1))
    ax.yaxis.set_major_formatter(ScalarFormatter())
    ax.yaxis.set_minor_formatter(ScalarFormatter())
    
    # Add horizontal lines at 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 15, 20, 30, 40, 50, 60, 70, 80, 90, 100, 120, 150, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1200, 1500, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 12000, 15000, 20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000, 100000, 120000, 150000, 200000, 300000, 400000, 500000, 600000, 700000, 800000, 900000, 1000000, 1200000, 1500000, 2000000, 3000000, 4000000, 5000000, 6000000, 7000000, 8000000, 9000000, 10000000, 12000000, 15000000, 20000000, 30000000, 40000000, 50000000, 60000000, 70000000, 80000000, 90000000, 100000000, 120000000, 150000000, 200000000, 300000000, 4