#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    packed_fields = rng.read(1)[0] & 0b11111110  # Clear bit 0 (Global Color Table Flag)
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    
    lsd = struct.pack('<HBBB', width, height, packed_fields, background_color_index) + pixel_aspect_ratio.to_bytes(1, 'big')
    
    # 3. Global Color Table
    global_color_table_flag = 1 << 7
    color_resolution = 8
    sort_flag = 0
    global_color_table_size = 7
    
    # 4. Block Size
    block_size = 3 * (2 ** (global_color_table_size + 1))
    
    # 5. Colors
    colors = [struct.pack('BBB', *rng.read(3)) for _ in range(2 ** (global_color_table_size + 1))]
    color_table = bytes([global_color_table_flag | color_resolution | sort_flag | global_color_table_size]) + b''.join(colors)
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        
        if not block_size:
            break
            
        blocks.append(bytes([block_size]) + rng.read(block_size))
        
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0
    
    # 7. Trailer
    trailer = b'\x3B'
    
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)