#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 15 09:26:47 2020

@author: jonasg
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
from scipy.optimize import minimize
from matplotlib import pyplot as plt

def f(x):
    return (np.sin(x[0]) + x[1]**2) * np.exp(-x[0]**2 - x[1]**2)

def g(x):
    return (x[0]**2 + x[1]**2) * np.exp(-x[0]**2 - x[1]**2)

def h(x):
    return np.exp(-x[0]**2 - x[1]**2)

def gradf(x):
    return np.array([np.cos(x[0]) * np.exp(-x[0]**2 - x[1]**2) - 2 * x[0] * x[1] * np.exp(-x[0]**2 - x[1]**2),
                    2 * x[1]**2 * np.exp(-x[0]**2 - x[1]**2) - 2 * x[0] * x[1] * np.exp(-x[0]**2 - x[1]**2)])

def gradg(x):
    return np.array([2 * x[0] * np.exp(-x[0]**2 - x[1]**2) - 2 * x[0] * x[1] * np.exp(-x[0]**2 - x[1]**2),
                     2 * x[1] * np.exp(-x[0]**2 - x[1]**2) - 2 * x[0] * x[1] * np.exp(-x[0]**2 - x[1]**2)])

def gradh(x):
    return np.array([-2 * x[0] * np.exp(-x[0]**2 - x[1]**2),
                     -2 * x[1] * np.exp(-x[0]**2 - x[1]**2)])

def plot_contours():
    x = y = np.linspace(-3, 3, 100)
    X, Y = np.meshgrid(x, y)
    Z = f(np.array([X, Y]).T)
    fig = plt.figure()
    ax = fig.add_subplot(projection='3d')
    surf = ax.plot_surface(X, Y, Z, cmap=plt.cm.coolwarm, linewidth=0, antialiased=False)
    ax.set_zlim(-1, 1)
    ax.view_init(elev=15., azim=-135.)
    plt.show()

def plot_gradients():
    x = y = np.linspace(-3, 3, 100)
    X, Y = np.meshgrid(x, y)
    Z = gradf(np.array([X, Y]).T)
    fig = plt.figure()
    ax = fig.add_subplot(projection='3d')
    surf = ax.plot_surface(X, Y, Z[:, :, 0], cmap=plt.cm.coolwarm, linewidth=0, antialiased=False)
    ax.set_zlim(-1, 1)
    ax.view_init(elev=15., azim=-135.)
    plt.show()
    fig = plt.figure()
    ax = fig.add_subplot(projection='3d')
    surf = ax.plot_surface(X, Y, Z[:, :, 1], cmap=plt.cm.coolwarm, linewidth=0, antialiased=False)
    ax.set_zlim(-1, 1)
    ax.view_init(elev=15., azim=-135.)
    plt.show()

def plot_gradient_descent(x0, gradf, niter=100, lr=0.01):
    xs = []
    ys = []
    for i in range(niter+1):
        grad = gradf(np.array([xs[-1], ys[-1]]))
        xs.append(xs[-1] - lr * grad[0])
        ys.append(ys[-1] - lr * grad[1])
    plt.scatter(xs, ys)
    plt.show()

def plot_newton_raphson(x0, gradf, gradg, gradh, niter=100, lr=0.01):
    xs = [x0[0]]
    ys = [x0[1]]
    for i in range(