#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Magic bytes
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 << (rng.read(1)[0] & 7) + 2
    height = 1 << (rng.read(1)[0] & 7) + 2
    flags = rng.read(1)[0]
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HBBB', width, height, flags, background))

    # Global color table
    if flags & 0x80 != 0:
        colors = 2 ** ((flags >> 4) & 7)
        for _ in range(colors):
            out.write(rng.read(3))

    # Application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<I', 0))

    # Image descriptor
    left = rng.read(1)[0]
    top = rng.read(1)[0]
    width = 1 << (rng.read(1)[0] & 7) + 2
    height = 1 << (rng.read(1)[0] & 7) + 2
    flags = rng.read(1)[0]
    out.write(struct.pack('<HHBB', left, top, width, height))

    # Local color table
    if flags & 0x80 != 0:
        colors = 2 ** ((flags >> 4) & 7)
        for _ in range(colors):
            out.write(rng.read(3))

    # LZW minimum code size
    out.write(b'\x00')

    # Pixel data
    while True:
        code = rng.read(1)[0]
        if code == 0:
            break
        out.write(code.to_bytes(1, byteorder='big'))

    # Trailer
    out.write(b';')
