#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    background_color = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBBBB', width, height, packed, background_color, aspect_ratio, rng.read(1)[0])
    
    # 3. Color Table (Global)
    color_table = rng.read(768)
    
    # 4. Graphic Control Extension
    gce = struct.pack('>BBBHBB', 0xF9, 0x04, 0x08 | rng.read(1)[0], rng.read(2), rng.read(1)[0], rng.read(1)[0])
    
    # 5. Image Descriptor
    image_descriptor = struct.pack('>BBHHHBB', 0x2C, 0, 0, width, height, 0, rng.read(1)[0])
    
    # 6. Data Blocks
    data = []
    
    while True:
        block_size = rng.read(1)[0] & 0xFF
        
        if block_size == 0:
            break
        
        data.append(rng.read(block_size))
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to output
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)
