#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, duration=1.0):
    # Magic header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<H', width))  # Screen width
    out.write(struct.pack('<H', height))  # Screen height
    out.write(b'\x87\x0f')                # Global color table flag (yes), color resolution (8 bits), sort flag (no), size of global color table (15 colors)

    # Background color index
    out.write(b'\xff')

    # Pixel aspect ratio
    out.write(b'\x00')

    # Global color table
    for i in range(2**8):
        out.write(struct.pack('BBB', *[i] * 3))

    # Application extension
    out.write(b'!\xf9\x04netscape\x03\x01')
    out.write(struct.pack('<I', int(duration*100)))
    out.write(b'\x00')

    # Image descriptor
    out.write(b',')                       # Separator
    out.write(struct.pack('<HH', 0, 0))   # Offset X, offset Y
    out.write(struct.pack('<HH', width, height))  # Width, height
    out.write(b'\x80')                    # Local color table flag (yes), interlace flag (no), sorted flag (no), size of local color table (all colors)

    # Local color table
    for i in range(2**8):
        out.write(struct.pack('BBB', *[i] * 3))

    # Raster data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', rng.read(1)[0]))

    # Trailer
    out.write(b';')
