#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480):
    # Write the GIF header
    out.write("GIF89a".encode())
    # Write the screen descriptor
    out.write(struct.pack("<HHBB", width, height, 0xF, 0))
    # Write the global color table (black and white)
    out.write(b"\xFF\xFF\xFF\xFF\xFF\xFF")
    # Write an image descriptor
    out.write(struct.pack("<HHHBB", 0, 0, width, height, 0))
    # Write local color table (black and white)
    out.write(b"\xFF\xFF\xFF\xFF\xFF\xFF")
    # Write the graphic control extension
    out.write(b"\x21\xF9\x04\x0B\x00\x00\x00\x00\x00")
    # Write the image data
    for y in range(height):
        for x in range(width):
            b = int.from_bytes(rng.read(1), "big")
            if b == 0:
                out.write(b"\x00")
            elif b == 1:
                out.write(b"\xFF")
            else:
                raise ValueError("Invalid random byte")
