#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed_fields = int.from_bytes(rng.read(1), 'big')
    global_color_table_flag = (packed_fields >> 7) & 0x1
    color_resolution = ((packed_fields >> 4) & 0x7) + 1
    sort_flag = (packed_fields >> 3) & 0x1
    size_of_global_color_table = 2 ** ((packed_fields & 0x7) + 1)
    background_color_index = int.from_bytes(rng.read(1), 'big')
    pixel_aspect_ratio = int.from_bytes(rng.read(1), 'big')
    
    # 3. Global Color Table
    if global_color_table_flag == 1:
        for _ in range(size_of_global_color_table):
            red, green, blue = struct.unpack('BBB', rng.read(3))
            
            # TODO: Add alpha channel?
        
    # 4. Image Data
    while True:
        block_type = int.from_bytes(rng.read(1), 'big')
        if block_type == 0x3b:
            break
        elif block_type == 0x2c:
            image_descriptor = rng.read(9)
            left, top, width, height = struct.unpack('>HHHH', image_descriptor[:8])
            local_color_table_flag = (image_descriptor[8] >> 7) & 0x1
            interlaced = (image_descriptor[8] >> 6) & 0x1
            sorted = (image_descriptor[8] >> 5) & 0x1
            reserved = (image_descriptor[8] >> 4) & 0x1
            size_of_local_color_table = 2 ** ((image_descriptor[8] & 0xf) + 1)
            lzw_minimum_code_size = int.from_bytes(rng.read(1), 'big')
            
            # 5. Local Color Table
            if local_color_table_flag == 1:
                for _ in range(size_of_local_color_table):
                    red, green, blue = struct.unpack('BBB', rng.read(3))
                    
                    # TODO: Add alpha channel?
            
            # 6. LZW-compressed image data
            lzw_data = []
            while True:
                code_size = int.from_bytes(rng.read(1), 'big')
                if code_size == 0:
                    break
                else:
                    lzw_data.append(code_size)
            
            # 7. Trailer
            trailer = rng.read(1)
