#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # 10 bytes of header
    out.write(b'GIF89a')
    out.write(struct.pack('<H', 24))
    out.write(struct.pack('<H', 50))

    # 7 bytes of global color table
    out.write(b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF')

    # 1 byte of background color index
    out.write(b'\x00')

    # 1 byte of pixel aspect ratio
    out.write(b'\x00')

    # 1 byte of flags
    out.write(b'\x00')

    # 6 bytes of application extension block
    out.write(b'NETSCAPE2.0\x03\x01')

    # 2 bytes of loop count (forever)
    out.write(b'\x00\x00')

    # 1 byte of trailer
    out.write(b';')

    while True:
        # 1 byte of image separator
        out.write(b',')

        # 9 bytes of image descriptor
        out.write(struct.pack('<HHB', 0, 0, 0))

        # 1 byte of local color table flag
        out.write(b'\x00')

        # 1 byte of interlaced flag
        out.write(b'\x00')

        # 2 bytes of sort flag
        out.write(b'\x00\x00')

        # 2 bytes of size
        out.write(struct.pack('<HH', 24, 24))

        for _ in range(24 * 24):
            # 1 byte of LZW code
            out.write(struct.pack('<B', int.from_bytes(rng.read(1), 'big')))

        # 1 byte of block terminator
        out.write(b'\x00')
