#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    background_color = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBBBB', width, height, packed, background_color, aspect_ratio, rng.read(1)[0])
    
    # 3. Global Color Table (if present)
    global_color_table_flag = bool(packed & 0x80)
    if not global_color_table_flag:
        return
    
    num_colors = 2 ** ((packed >> 4) + 1)
    color_table = rng.read(3 * num_colors)  # Random colors
    
    # 4. Graphics Control Extension (optional)
    gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    
    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])
    
    # 6. Local Color Table (if present)
    local_color_table_flag = bool(rng.read(1)[0] & 0x80)
    if not local_color_table_flag:
        return
    
    num_colors = 2 ** ((rng.read(1)[0] >> 4) + 1)
    color_table = rng.read(3 * num_colors)  # Random colors
    
    # 7. LZW-compressed data
    lzw_data = rng.read()  # Random bytes
    
    # 8. Trailer
    trailer = b'\x3b'
    
    # Write everything to the output stream
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    out.write(gce)
    out.write(image_descriptor)
    out.write(color_table)
    out.write(lzw_data)
    out.write(trailer)

# Generate a random GIF file of size `size` and write it to `filename`.
def generate_random_gif_file(filename: str, size: int):
    with open(filename, 'wb') as f:
