#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# given width and height, with each pixel being one byte long
def generate_random_gif(width: int, height: int, out: BinaryIO):
    header = b'GIF89a'  # Magic number
    lsd = struct.pack('<HHBBB',
                      0,  # Screen width
                      height,  # Screen height
                      128,  # Color resolution (bits per pixel)
                      7,  # Background color index
                      0,  # Pixel aspect ratio (ratio between pixels)
                      )
    color_table = bytes([
        0xFF, 0x00, 0x00,   # Red
        0x00, 0xFF, 0x00,   # Green
        0x00, 0x00, 0xFF,   # Blue
        ]) * 256        # Repeat this table for all colors in the palette
    gce = struct.pack('<BBHBBBB',
                      0xF9,       # Extension introducer
                      4,          # Block size
                      width,      # Image width
                      height,     # Image height
                      0,          # Flags (no local color table)
                      0,          # Background color index
                      0           # Pixel aspect ratio (ratio between pixels)
                      )
    image_descriptor = struct.pack('<BBHHHBB',
                                   0x2C,         # Image separator
                                   0,            # Left position
                                   -height,      # Top position
                                   width,        # Width
                                   height,       # Height
                                   0,            # Flags (no local color table)
                                   0             # LZW minimum code size
                                  )
    image_data = bytes([i % 256 for i in range((width+1)*height)])
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)