#!/usr/bin/env python3

import struct
from typing import BinaryIO


def write_png(out: BinaryIO, width: int, height: int):
    """Write an empty PNG file."""
    # https://www.w3.org/TR/PNG/#5PNG-file-signature

    # 1. Header (8 bytes)
    header = b'\x89\x50\x4E\x47\x0D\x0A\x1A\x0A'

    # 2. Logical screen descriptor (13 bytes)
    lsd = struct.pack('>IIBBBBB', width, height, 8, 6, 0, 0, 0)

    # 3. Color table (0 or more bytes)
    # We don't need one for this simple example

    # 4. Graphic control extension (19 bytes)
    gce = struct.pack('>IIccccc', 4, 0b1111111111111111, b'n', b'a', b'm', b'e', b'')

    # 5. Image descriptor (13 bytes)
    image_descriptor = struct.pack('>IIBBBBB', width, height, 8, 2, 0, 0, 0)

    # 6. Image data
    rng = open('/dev/urandom', 'rb')
    lzw_minimum_code_size = 8
    blocks = []
    while True:
        block_size = rng.read(1)[0] % 256
        if not block_size:
            break
        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)