#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
	width &= 0x7FFF
	height &= 0x7FFF

    # Random color table size
    colorTableSize = ord(rng.read(1)) & 0b00001111

    # Background color index
    backgroundColorIndex = ord(rng.read(1))

    # Pixel aspect ratio
    pixelAspectRatio = ord(rng.read(1))

    logicalScreenDescriptor = struct.pack(
        '>HHBBB', width, height, colorTableSize, backgroundColorIndex, pixelAspectRatio
    )

    # 3. Global Color Table
    globalColorTable = bytearray()
    if colorTableSize > 0:
        for i in range(colorTableSize):
            r = ord(rng.read(1))
            g = ord(rng.read(1))
            b = ord(rng.read(1))

            globalColorTable += struct.pack('BBB', r, g, b)

    # 4. Image Descriptor
    imageLeftPosition = 0
    imageTopPosition = 0
    imageWidth = width
    imageHeight = height
    localColorTableFlag = False
    interlaceFlag = False
    sortFlag = False
    reserved = 0
    lctSize = 0
    lctBackgroundColorIndex = 0
    lctPixelAspectRatio = 0

    imageDescriptor = struct.pack(
        '>HHHHBBBBB',
        imageLeftPosition,
        imageTopPosition,
        imageWidth,
        imageHeight,
        localColorTableFlag,
        interlaceFlag,
        sortFlag,
        reserved,
        lctSize,
        lctBackgroundColorIndex,
        lctPixelAspectRatio
    )

    # 5. Local Color Table
    localColorTable = bytearray()
    if localColorTableFlag == True:
        for i in range(lctSize):
            r = ord(rng.read(1))
            g = ord(rng.read(1))
            b = ord(rng.read(1))

            localColorTable += struct.pack('BBB', r, g, b)

    # 6. Block Size
    blockSize = ord(rng.read(1))

    # 7. Extension Introducer
    extensionIntroducer = b'\x21'

    # 8. Application Extension Label
    applicationExtensionLabel = b'\xFF'

    # 9. Application Extension Block Size
    applicationExtensionBlockSize = b'\x0B'

    # 10. Application Identifier
    applicationIdentifier = b'NETSCAPE2.0'

    # 11. Application Authentication Code
    applicationAuthenticationCode = b'\x03\x01\x00\x00' + b'\x00' * 11

    # 12. Sub-block Size
    subBlockSize = ord(rng.read(1))

    # 13. Netscape Looping Extension Label
    netscapeLoopingExtensionLabel = b'\x01'

    # 14. Netscape Looping Extension Block Size
    netscapeLoopingExtensionBlockSize = b'\x03'

    # 15. Loop Count
    loopCount = 0

    netscapeLoopingExtensionBlock = struct.pack(
        'BBB', netscapeLoopingExtensionLabel, netscapeLoopingExtensionBlockSize, loopCount
    )

    # 16. Graphics Control Extension Label
    graphicsControlExtensionLabel = b'\xF9'

    # 17. Graphics Control Extension Block Size
    graphicsControlExtensionBlockSize = b'\x04'

    # 18. Reserved Flags
    reservedFlags = 0

    # 19. Disposal Method
    disposalMethod = 0

    # 20. User Input Flag
    userInputFlag = 0

    # 21. Transparent Color Index
    transparentColorIndex = 0

    # 22. Delay Time
    delayTime = 0

    # 23. Transparent Color Index
    transparentColorIndex = 0

    graphicsControlExtensionBlock = struct.pack(
        'BBBBBBBHBHBBB',
        graphicsControlExtensionLabel,
        graphicsControlExtensionBlockSize,
        reservedFlags,
        disposalMethod,
        userInputFlag,
        transparentColorIndex,
        delayTime,
        transparentColorIndex
    )

    # 24. Plain Text Extension Label
    plainTextExtensionLabel = b'\x01'

    # 25. Plain Text Extension Block Size
    plainTextExtensionBlockSize = b'\x0C'

    # 26. Text Grid Left Position
    textGridLeftPosition = 0

    # 27. Text Grid Top Position
    textGridTopPosition = 0

    # 28. Text Grid Width
    textGridWidth = width

    # 29. Text Grid Height
    textGridHeight = height

    # 30. Cell Width
    cellWidth = 0

    # 31. Cell Height
    cellHeight = 0

    # 32. Text Foreground Color Index
    textForegroundColorIndex = 0

    # 33. Text Background Color Index
    textBackgroundColorIndex = 0

    plainTextExtensionBlock = struct.pack(
        'BBBBHHHHHHBBB',
        plainTextExtensionLabel,
        plainTextExtensionBlockSize,
        textGridLeftPosition,
        textGridTopPosition,
        textGridWidth,
        textGridHeight,
        cellWidth,
        cellHeight,
        textForegroundColorIndex,
        textBackgroundColorIndex
    )

    # 34. Comment Extension Label
    commentExtensionLabel = b'\xFE'

    # 35. Comment Extension Block Size
    commentExtensionBlockSize = b'\x02'

    # 36. Comment
    comment = b''

    commentExtensionBlock = struct.pack(
        'BB', commentExtensionLabel, commentExtension