#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 768, delay: int = 50, loops: int = 0
):

    # Header
    out.write("GIF89a".encode())
    out.write(struct.pack("<HH", width, height))
    out.write("\x87\x00\x00".encode())

    # Logical screen descriptor
    out.write(",".encode())
    out.write(struct.pack("<BBBHB", 0xF7, 0x00, 0x00, 0x00, 0x00))

    # Global color table (black and white)
    out.write(";".encode())
    out.write(struct.pack("<BBBBBB", 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF))

    # Graphic control extension
    out.write("!".encode())
    out.write(struct.pack("<BHHBB", 0xF9, 0x04, delay, 0x00, loops))

    # Image descriptor
    out.write(",".encode())
    out.write(struct.pack("<BHHHBB", 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00))

    # Local color table (black and white)
    out.write(";".encode())
    out.write(struct.pack("<BBBBBB", 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF))

    # Image data
    for y in range(height):
        for x in range(width):
            if ((y * width + x) % 2) == 0:
                out.write(b"\x00")
            else:
                out.write(b"\xFF")

    # Trailer
    out.write(";".encode())
