#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'
	# 2. Logical screen descriptor
    width = int.from_bytes(rng.read(2), 'big') % (2**10-1) + 1
    height = int.from_bytes(rng.read(2), 'big') % (2**10-1) + 1
    flags = bytes([int.from_bytes(rng.read(1), 'big')])
    bgColorIndex = bytes([int.from_bytes(rng.read(1), 'big')])
    pixelAspectRatio = bytes([int.from_bytes(rng.read(1), 'big')])

    lsd = header \
        + width.to_bytes(2,'big') \
        + height.to_bytes(2,'big') \
        + flags \
        + bgColorIndex \
        + pixelAspectRatio

	# 3. Global color table
    globalColorTableFlag = bytes([int.from_bytes(rng.read(1), 'big')])
    if globalColorTableFlag == b'\x01':
        globalColorTableSize = bytes([int.from_bytes(rng.read(1), 'big')])
        globalColorTable = []

        for i in range(globalColorTableSize):
            r = bytes([int.from_bytes(rng.read(1), 'big')])
            g = bytes([int.from_bytes(rng.read(1), 'big')])
            b = bytes([int.from_bytes(rng.read(1), 'big')])

            globalColorTable += [r+g+b]
        
        lsd += globalColorTableFlag + globalColorTableSize + bytes().join(globalColorTable)
    
    else:
    	lsd += globalColorTableFlag + b'\x00'


    # 4. Image block
    imageDescriptor = b''
    imageDescriptor += bytes([int.from_bytes(rng.read(1), 'big')]) # left position of image
    imageDescriptor += bytes([int.from_bytes(rng.read(1), 'big')]) # top position of image
    imageDescriptor += bytes([int.from_bytes(rng.read(1), 'big')]) # width of image
    imageDescriptor += bytes([int.from_bytes(rng.read(1), 'big')]) # height of image
    imageDescriptor += bytes([int.from_bytes(rng.read(1), 'big')]) # local color table flag
    imageDescriptor += bytes([int.from_bytes(rng.read(1), 'big')]) # interlace flag
    imageDescriptor += bytes([int.from_bytes(rng.read(1), 'big')]) # sort flag
    imageDescriptor += bytes([int.from_bytes(rng.read(1), 'big')]) # reserved field

    lsd += imageDescriptor

    # 5. Local color table
    localColorTableFlag = bytes([int.from_bytes(rng.read(1), 'big')])
    if localColorTableFlag == b'\x01':
        localColorTableSize = bytes([int.from_bytes(rng.read(1), 'big')])
        localColorTable = []

        for i in range(localColorTableSize):
            r = bytes([int.from_bytes(rng.read(1), 'big')])
            g = bytes([int.from_bytes(rng.read(1), 'big')])
            b = bytes([int.from_bytes(rng.read(1), 'big')])

            localColorTable += [r+g+b]
        
        lsd += localColorTableFlag + localColorTableSize + bytes().join(localColorTable)
    
    else:
    	lsd += localColorTableFlag + b'\x00'

    # 6. LZW min code size
    lzwMinCodeSize = bytes([int.from_bytes(rng.read(1), 'big')])
    lsd += lzwMinCodeSize

    # 7. Data subblock
    dataSubBlock = b''
    while True:
        dataSubBlockSize = bytes([int.from_bytes(rng.read(1), 'big')])
        if not dataSubBlockSize: break
        dataSubBlock += dataSubBlockSize + rng.read(dataSubBlockSize)

    lsd += dataSubBlock

    # 8. Trailer
    trailer = b'\x3B'

    out.write(lsd + trailer)