#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors
    while len(color_table) < 768:  # Pad with black if needed
        color_table += b'\x00\x00\x00'
    
    # 4. Graphics Control Extension (to set delay between frames)
    gce = b'\x21\xF9\x04' + rng.read(3) + b'\x00'

    # 5. Image Descriptor (to specify frame dimensions)
    image_descriptor = struct.pack('>HHBBB', 0, 0, width, height, 0b10000000 | rng.read(1)[0])

    # 6. Image Data (frame data)
    image_data = b''
    for _ in range((width * height)):
        image_data += struct.pack('>B', rng.read(1)[0])

    # 7. Trailer
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)