#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# returns its size in bytes.
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header (13 bytes).
    out.write(b"GIF89a")

    # Logical screen descriptor (7 bytes).
    width = 256 + int.from_bytes(rng.read(1), "little", signed=True) % 128
    height = 256 + int.from_bytes(rng.read(1), "little", signed=True) % 128
    flags = b"\x00\x00" if bool(int.from_bytes(rng.read(1), "little")) else b"\xff\xff"
    bgColorIndex = int.from_bytes(rng.read(1), "little") & 0xf
    pixelAspectRatio = int.from_bytes(rng.read(1), "little") & 0xf
    out.write(struct.pack("<HHBBB", width, height, flags[0], flags[1], bgColorIndex, pixelAspectRatio))

    # Global color table (3 * 256 = 768 bytes).
    for i in range(256):
        r = int.from_bytes(rng.read(1), "little") & 0xff
        g = int.from_bytes(rng.read(1), "little") & 0xff
        b = int.from_bytes(rng.read(1), "little") & 0xff
        out.write(struct.pack("BBB", r, g, b))

    # Image descriptor (10 bytes).
    leftPos = 0
    topPos = 0
    width = 256 + int.from_bytes(rng.read(1), "little", signed=True) % 128
    height = 256 + int.from_bytes(rng.read(1), "little", signed=True) % 128
    flags = b"\x00\x00" if bool(int.from_bytes(rng.read(1), "little")) else b"\xff\xff"
    localColorTableFlag = b"\x00" if bool(int.from_bytes(rng.read(1), "little")) else b"\xff"
    interlaceFlag = b"\x00" if bool(int.from_bytes(rng.read(1), "little")) else b"\xff"
    sortFlag = b"\x00" if bool(int.from_bytes(rng.read(1), "little")) else b"\xff"
    reserved = b"\x00\x00\x00"
    out.write(struct.pack("<HHHHBBBBB", leftPos, topPos, width, height, flags[0], flags[1], localColorTableFlag[0], localColorTableFlag[1], interlaceFlag[0], interlaceFlag[1]))

    # Local color table (3 * 256 = 768 bytes).
    for i in range(256):
        r = int.from_bytes(rng.read(1), "little") & 0xff
        g = int.from_bytes(rng.read(1), "little") & 0xff
        b = int.from_bytes(rng.read(1), "little") & 0xff
        out.write(struct.pack("BBB", r, g, b))

    # Image data block (width * height bytes).
    for y in range(height):
        for x in range(width):
            out.write(struct.pack("B", int.from_bytes(rng.read(1), "little")))

    return 13 + 7 + 768 + 10 + width * height

