#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
	width &= 0x7FFF
	height &= 0x7FFF

	# Random color resolution
	color_resol = rng.read(1)[0] & 0b111
	color_resol |= ((rng.read(1)[0] & 0b11) << 5)

	# Global Color Table Flag
	gct_flag = bool((rng.read(1)[0] >> 7) & 1)

	# Sorting flag
	sort_flag = bool((rng.read(1)[0] >> 6) & 1)

	# Size of global color table
	size_of_gct = 2 ** ((rng.read(1)[0] & 0b111) + 1)

	# Background color index
	bg_color_index = rng.read(1)[0]

	# Pixel aspect ratio
	pixel_aspect_ratio = rng.read(1)[0] & 0b1111

    # Write to output
    out.write(header)
    out.write(struct.pack('>HHBBB', width, height, color_resol, gct_flag, sort_flag, size_of_gct))
    out.write(bytes([bg_color_index]))
    out.write(bytes([pixel_aspect_ratio]))

    if not gct_flag:
        return
    
    # 3. Global Color Table
    for i in range(size_of_gct):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]

        out.write(bytes([r, g, b]))

    # 4. Image Descriptor
    left = 0
    top = 0
    width = 0xFFFF
    height = 0xFFFF

    packed_left = left | ((top & 0xF) << 12)
    packed_right = (((top & 0xF0) >> 4) << 8) | width

    out.write(packed_left.to_bytes(2, 'big'))
    out.write(packed_right.to_bytes(2, 'big'))

    # LZW minimum code size
    lzw_min_code_size = rng.read(1)[0] + 1

    out.write(bytes([lzw_min_code_size]))

    # 5. Image Data
    # TODO: Implement this part!
    pass