#!/usr/bin/env python3

import struct
from typing import BinaryIO

def write_gif(out: BinaryIO, width: int, height: int):
    """Writes an animated GIF to `out`."""

    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    lsd = struct.pack('>HHBBB', width, height, 0xF7, 0, 0)

    # 3. Global Color Table (empty)
    color_table = b''

    # 4. Graphics Control Extension
    gce = None

    # 5. Image Descriptor
    image_descriptor = struct.pack('>BBHHHBB', 0x2C, 0, 0, width, height, 0, 0)

    # 6. Image Data
    rng = open('/dev/urandom', 'rb')
    lzw_minimum_code_size = 2
    blocks = []
    while True:
        block_size = rng.read(1)[0] % 128

        if block_size == 0:
            continue
        elif block_size == 127:
            block_size += rng.read(1)[0] % 128
        else:
            break
        
        blocks.append(bytes([block_size]) + rng.read(block_size))
    
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table)
    if gce is not None:
        out.write(gce)
    out.write(image_descriptor + image_data + trailer)