#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    out.write(b'GIF89a')

    width = 1 + (rng.read(1)[0] % 256)
    height = 1 + (rng.read(1)[0] % 256)
    out.write(struct.pack('<HH', width, height))

    # Global Color Table
    color_table_size = min((width * height), 256)
    for i in range(color_table_size):
        r = rng.read(1)[0] & 0x7F
        g = rng.read(1)[0] & 0x7F
        b = rng.read(1)[0] & 0x7F

        out.write(bytes([r,g,b]))

    # Image Descriptor
    out.write(b'\x2C\x00\x00\x00')
    out.write(struct.pack('<HHB', 0, 0, 0))

    # Local Color Table
    if color_table_size > 256:
        raise Exception('Too many colors!')

    for i in range(color_table_size):
        r = rng.read(1)[0] & 0x7F
        g = rng.read(1)[0] & 0x7F
        b = rng.read(1)[0] & 0x7F

        out.write(bytes([r,g,b]))

    # Image Data
    data_size = ((width // 8) * height)
    for i in range(data_size):
        out.write(rng.read(1))

