#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate(out: BinaryIO, width: int = 1024, height: int = 768):
    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    out.write(struct.pack('>HHBBB', width, height, 0x80, 0, 0))

    # Global Color Table (black and white)
    out.write(bytes([0xFF, 0xFF, 0xFF]))
    out.write(bytes([0x00, 0x00, 0x00]))

    # Image Descriptor
    out.write(struct.pack('>HHHHB', 0, 0, width - 1, height - 1, 0x00 | 0x80))

    # Local Color Table (black and white)
    out.write(bytes([0xFF, 0xFF, 0xFF]))
    out.write(bytes([0x00, 0x00, 0x00]))

    # Graphic Control Extension
    out.write(bytes([0x21, 0xF9, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x2C]))

    # Image Data
    for y in range(height):
        for x in range(width // 8):
            out.write(bytes([(y >> 7) & 1] * 8))

