#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=1024, height=768):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xF0, 0x0F, 0))

    # Global color table (black and white only)
    for i in range(256):
        if i < 128:
            out.write(bytes([i * 2, i * 2, i * 2]))
        else:
            out.write(bytes([(255 - i) * 2,
                             (255 - i) * 2, (255 - i) * 2]))

    # Image descriptor
    out.write(struct.pack('<BHHBB',
                         0x2C,
                         width,
                         height,
                         0x00,
                         0x00,
                         0))

    # LZW minimum code size
    out.write(b'\x02\x00')

    # Frame data
    while True:
        byte = ord(rng.read(1))

        if byte == 0xFF:
            break

        out.write(bytes([byte]))

    # Trailer
    out.write(b';')


if