#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)
    background_color = rng.read(1)
    aspect_ratio = rng.read(1)
    # 2. Logical Screen Descriptor (LSD)
    lsd = width + height + flags + background_color + aspect_ratio
    color_table_size = rng.read(1)
    color_resolution = rng.read(1)
    sort_flag = rng.read(1)
    global_color_table_flag = rng.read(1)
    color_table_size |= global_color_table_flag
    color_table = rng.read(int(pow(2, color_table_size[0])))
    # 3. Graphic Control Extension (GCE)
    gce = b''
    if rng.read(1)[0]:
        gce += rng.read(4)
        gce += rng.read(1)
        gce += rng.read(1)
    # 4. Image Descriptor
    image_descriptor = rng.read(10)
    # 5. Blocks
    blocks = []
    while True:
        block_length = rng.read(1)
        if not block_length:
            break
        block = rng.read(int(block_length))
        blocks.append(block)
    # 6. Trailer
    trailer = rng.read(1)
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    if len(gce) == 6:
        out.write(gce)
    out.write(image_descriptor)
    for block in blocks:
        out.write(struct.pack('B', len(block)))
        out.write(block)
    out.write(trailer)