#!/usr/bin/env python3
import sys, os, random
from PIL import Image

def main():
    # Check arguments
    if len(sys.argv) < 4 or len(sys.argv) > 6:
        print("Usage: {} <input> <output> <width> <height>".format(sys.argv[0]))
        return 1
    
    # Open input file
    try:
        rng = open(sys.argv[1], 'rb')
    except IOError as e:
        print("Couldn't open {}: {}".format(sys.argv[1], str(e)))
        return 1
    
    # Create output file
    try:
        out = open(sys.argv[2], 'wb')
    except IOError as e:
        print("Couldn't create {}: {}".format(sys.argv[2], str(e)))
        return 1
    
    # Parse width and height from command line
    width = int(sys.argv[3])
    height = int(sys.argv[4])
    
    # Setup RNG
    rng.seek(random.randint(0,os.fstat(rng.fileno()).st_size - 1))
    rng = iter(lambda: rng.read(1), b'')
    
    # 1. Header
    header = b'\x47\x49\x46\x38\x39\x61'
    
    # 2. Logical Screen Descriptor
    lsd = bytearray([0xFF & (width >> 8), 0xFF & width, 0xFF & (height >> 8), 0xFF & height, 0x80 | 0x0F, 0, 0])
    
    # 3. Color Table
    color_table = [b'\xFF\xFF\xFF', b'\x00\x00\x00']
    
    # 4. Graphic Control Extension
    gce = b'\x21\xF9\x04\x08\x00\x00\x00'
    
    # 5. Image Descriptor
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00'
    
    # 6. Image Data
    image_data = []
    while True:
        block_size = next(rng)[0]
        if not block_size:
            break
        
        image_data += [rng.read((block_size % 255) + 1)]
        
    # 7. Trailer
    trailer = b';'

    # Write everything to our output stream in order
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    out.write(gce)
    out.write(image_descriptor)
    out.writelines(image_data)
    out.write(trailer)

