#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Random background color index
    bg_color_index = rng.read(1)[0]

    # Random pixel aspect ratio (ignored by most browsers)
    pixel_aspect_ratio = rng.read(1)[0]

    # Global Color Table Flag (always true)
    global_color_table_flag = 0x80

    # Color Resolution (8 bits per primary color channel)
    color_resolution = 0x07

    # Sort Flag (not sorted)
    sort_flag = 0x00

    # Size of Global Color Table (256 colors)
    size_of_global_color_table = 0x00

    # Background Color Index (same as above)
    bg_color_index = bg_color_index

    # Pixel Aspect Ratio (same as above)
    pixel_aspect_ratio = pixel_aspect_ratio

    # Pack all these fields together
    lsd = struct.pack('>HBBBBBBB', width, height, global_color_table_flag, color_resolution, sort_flag, size_of_global_color_table, bg_color_index, pixel_aspect_ratio)

    # 3. Global Color Table
    # We don't use this table, so we just write zeros here
    color_table = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    image_data = []
    while True:
        block_size = rng.read(1)[0]

        if not block_size:
            break

        image_data.append(rng.read(block_size))

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything to the output stream
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(image_data) + trailer)


if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rng:
        with open('./test.gif', 'wb') as f:
            generate_random_gif(rng, f)