#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int = 1024, height: int = 768) -> None:
    # Header
    out.write('GIF'.encode())
    out.write('\x38\x9b\x00\x00'.encode())
    
    # Logical screen descriptor
    out.write(struct.pack('<HBB', width, height, 0))
    out.write(bytes([0] * 5 + [1]))
    out.write(bytes([0] * 7))
    
    # Graphic control extension
    out.write(bytes([0x21, 0xf9, 0x04, 0x00, 0x00, 0xff, 0x00]))
    
    # Image descriptor
    out.write(struct.pack('<HBBHHB', 0, 0, width, height, 0, 0))
    
    # Local color table
    for i in range(0, 256):
        r = (i >> 5) & 0x1f
        g = (i >> 2) & 0x1f
        b = (i << 3) & 0x1f
        
        out.write(struct.pack('<BBB', r, g, b))
    
    # Image data
    for y in range(0, height):
        for x in range(0, width):
            out.write(bytes([(y ^ x) % 256]))
