#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 16 + (rng.read(1)[0] % 256)
    height = 16 + (rng.read(1)[0] % 256)
    packed = 0x87 | ((width - 1) << 4) | (height - 1)
    out.write(struct.pack('<HBB', packed, 0, 0))

    # Global color table
    for i in range(3 * 2 ** (packed & 7)):
        out.write(bytes([rng.read(1)[0]] * 3))

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\xff\x00')

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x01\x00\x21\xf9\x04\x01\xff\x00')

    # LZW-compressed image data
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x01\x00\x21\xf9\x04\x01\xff\x00')

    # Trailer
    out.write(b';')

