#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> int:
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    out.write(struct.pack('<BBB', *[rng.read(1)[0] for _ in range(3)]))

    # 3. Global Color Table
    # We don't need to write anything here because we aren't using it

    # 4. Graphic Control Extension
    # We don't need to write anything here because we aren't using it

    # 5. Image Descriptor
    # Random left and top position (two bytes each)
    out.write(struct.pack('<HH', *[rng.read(2)[0] for _ in range(2)]))

    # Width and height of image (two bytes each)
    out.write(struct.pack('<HH', width, height))

    # Packed field: Local color table flag, Interlace flag, Sort flag, Reserved
    out.write(struct.pack('<BBB', *[rng.read(1)[0] for _ in range(3)]))

    # 6. Local Color Table
    # We don't need to write anything here because we aren't using it

    # 7. Image Data
    # Write random data until we reach the end of the frame
    while True:
        if rng.read(1)[0] == 0xFF:
            break

    # 8. Trailer
    out.write(b'\x3B')

    return width * height
