
import struct
from random import randint


def generate_random_gif(rng, out):
    """Generate a GIF file of random data"""

    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor (LSD)
    width = randint(0, 65535)
    height = randint(0, 65535)
    packed_fields = 0b00000000
    background_color_index = randint(0, 255)
    pixel_aspect_ratio = 0
    lsd = struct.pack('<HHBBB', width, height, packed_fields, background_color_index, pixel_aspect_ratio)

    # 3. Global Color Table
    color_table = rng.read(3 * 2 ** ((packed_fields >> 4) & 0b111))

    # 4. Graphic Control Extension (GCE)
    gce = b'\x21\xf9\x04\x01\x00\x00\xff\x00'

    # 5. Image Descriptor
    left = randint(0, 65535)
    top = randint(0, 65535)
    width = randint(0, 65535)
    height = randint(0, 65535)
    packed_fields = 0b00000000
    local_color_table_flag = False
    interlace_flag = False
    sort_flag = False
    reserved = 0
    size_of_local_color_table = 0
    image_descriptor = struct.pack('<HHBBBBB', left, top, width, height, packed_fields, local_color_table_flag,
                                   interlace_flag, sort_flag, reserved, size_of_local_color_table)

    # 6. Image Data
    image_data = []
    while True:
        block_size = rng.read(1)[0]
        if not block_size:
            break

        image_data.append(rng.read(block_size))

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything to the output stream
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(image_data) + trailer)


if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rng:
        with open('./test.gif', 'wb') as f:
            generate_random_gif(rng, f)