#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    #   - Width
    width = int.from_bytes(rng.read(2), 'big') % 65536
    #   - Height
    height = int.from_bytes(rng.read(2), 'big') % 65536
    #   - Background color index
    background_color_index = int.from_bytes(rng.read(1), 'big') % 256
    #   - Pixel aspect ratio (unused)
    pixel_aspect_ratio = int.from_bytes(rng.read(1), 'big') % 256
    #   - Global Color Table Flag
    global_color_table_flag = bool(int.from_bytes(rng.read(1), 'big'))
    #   - Color Resolution
    color_resolution = int.from_bytes(rng.read(1), 'big') & 0b111
    #   - Sort Flag
    sort_flag = bool(int.from_bytes(rng.read(1), 'big'))
    #   - Size of Global Color Table
    size_of_global_color_table = int.from_bytes(rng.read(1), 'big') & 0b111

    # 3. Global Color Table
    if global_color_table_flag:
        for _ in range(size_of_global_color_table * 3):
            out.write(rng.read(1))

    # 4. Image Data
    while True:
        # 4.1 Graphic Control Extension
        #   - Block Size
        block_size = int.from_bytes(rng.read(1), 'big')
        
        #   - Packed Fields
        #     - Reserved
        reserved = int.from_bytes(rng.read(1), 'big')
        #     - Disposal Method
        disposal_method = int.from_bytes(rng.read(1), 'big') & 0b111
        #     - User Input Flag
        user_input_flag = bool(int.from_bytes(rng.read(1), 'big'))
        #     - Transparent Color Flag
        transparent_color_flag = bool(int.from_bytes(rng.read(1), 'big'))
        
        #   - Delay Time
        delay_time = int.from_bytes(rng.read(2), 'big')
        
        #   - Transparent Color Index
        transparent_color_index = int.from_bytes(rng.read(1), 'big')
        
        #   - Block Terminator
        block_terminator = int.from_bytes(rng.read(1), 'big')

        # 4.2 Image Descriptor
        #   - Separator
        separator = int.from_bytes(rng.read(1), 'big')
        #   - Left Position
        left_position = int.from_bytes(rng.read(2), 'big')
        #   - Top Position
        top_position = int.from_bytes(rng.read(2), 'big')
        #   - Width
        image_width = int.from_bytes(rng.read(2), 'big')
        #   - Height
        image_height = int.from_bytes(rng.read(2), 'big')
        #   - Local Color Table Flag
        local_color_table_flag = bool(int.from_bytes(rng.read(1), 'big'))
        #   - Interlace Flag
        interlace_flag = bool(int.from_bytes(rng.read(1), 'big'))
        #   - Sort Flag
        sort_flag = bool(int.from_bytes(rng.read(1), 'big'))
        #   - Size of Local Color Table
        size_of_local_color_table = int.from_bytes(rng.read(1), 'big') & 0b111

        # 4.3 Local Color Table
        if local_color_table_flag:
            for _ in range(size_of_local_color_table * 3):
                out.write(rng.read(1))
            
        # 4.4 LZW Minimum Code Size
        lzw_minimum_code_size = int.from_bytes(rng.read(1), 'big')

        # 4.5 Data Sub-Blocks
        while True:
            # 4.5.1 Data Sub-Block Size
            data_sub_block_size = int.from_bytes(rng.read(1), 'big')

            # 4.5.2 Data Sub-Block
            for _ in range(data_sub_block_size):
                out.write(rng.read(1))
                
            # 4.5.3 Zero-length Data Sub-Block
            zero_length_data_sub_block = int.from_bytes(rng.read(1), 'big')
            
