#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given dimensions and color.
def generate_gif(out: BinaryIO, width: int, height: int, color: bytes) -> None:
    # Write the header.
    out.write(b'GIF89a')  # Magic number.
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(bytes([0xF7, 0]))  # Global color table flag + color resolution.
    out.write(bytes([0] * 3))  # Background color index.
    out.write(bytes([0] * 3))  # Pixel aspect ratio.

    # Write the global color table.
    for i in range(256):
        out.write(color)

    # Write the image descriptor.
    out.write(bytes([0x2C, 0, 0, 0, 0, 0, 0, 0, 0, 0]))

    # Write the local color table.
    for i in range(256):
        out.write(color)

    # Write the LZW-compressed pixel data.
    out.write(bytes([0x00, 0x01]))
    out.write(bytes([0xFF]))
    out.write(bytes([0x00]))

