#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # Random number of colors in the global color table
    n_colors = int.from_bytes(rng.read(1), 'big') + 1

    # Each color is three bytes: Red, Green, Blue
    gct = rng.read(n_colors * 3)

    # 4. Image Data
    # Random number of images
    n_images = int.from_bytes(rng.read(1), 'big') + 1

    # Loop through all the images
    for i in range(n_images):
        # 5. Image Descriptor
        idd = struct.pack('>BBHHHBB', 0x2C, 0, 0, width, height, 0, 0)

        # 6. Local Color Table
        # Random number of colors in the local color table
        n_lct_colors = int.from_bytes(rng.read(1), 'big') + 1

        # Each color is three bytes: Red, Green, Blue
        lct = rng.read(n_lct_colors * 3)

        # 7. Image Data
        # Random number of image data blocks
        n_image_data_blocks = int.from_bytes(rng.read(1), 'big') + 1

        # Each block is a single byte with a value between 0 and 255 inclusive
        image_data = rng.read(n_image_data_blocks)

        # Write everything to the output file
        out.write(header)
        out.write(lsd)
        out.write(gct)
        out.write(idd)
        out.write(lct)
        out.write(image_data)

