#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    # We use a global color table with 256 colors
    gct = rng.read(768)
    
    # 4. Graphic Control Extension
    # We use a single graphic control extension to set the transparent color index
    gce = struct.pack('BBBxBB', 0xF9, 0x04, rng.read(1)[0], 0, 0)
    
    # 5. Image Descriptor
    idd = struct.pack('>BBHBBB', 0x2C, 0, 0, width, height, 0)
    
    # 6. Local Color Table
    # We don't use a local color table
    lct = b''
    
    # 7. Image Data
    # We use a single image data block
    imgdata = rng.read(width * height)
    
    # Write everything together
    out.write(header + lsd + gct + gce + idd + lct + imgdata)
