#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    screen_width = rng.read(2)
    screen_height = rng.read(2)

    # Packed fields
    packed_fields = bytearray([0xF7])

    # Background color index
    background_color_index = rng.read(1)

    # Pixel aspect ratio
    pixel_aspect_ratio = rng.read(1)

    # Write everything to the output stream
    out.write(header + screen_width + screen_height + packed_fields + background_color_index + pixel_aspect_ratio)

    # 3. Global Color Table
    # We don't use it so we just write zeros
    global_color_table = bytearray()
    for i in range(768):
        global_color_table += b'\x00\x00\x00'
    out.write(global_color_table)

    # 4. Image Data
    # We don't use it so we just write zeros
    image_data = bytearray()
    for i in range(5):
        image_data += b'\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'
    out.write(image_data)

