#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), 'big') % 640
    height = int.from_bytes(rng.read(2), 'big') % 480
    flags = rng.read(1)[0] & 0b11111000
    background_color_index = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('<HBBBB', width, height, flags, background_color_index, aspect_ratio)

    # 3. Global Color Table
    color_table = rng.read(768)

    # 4. Graphic Control Extension
    gce = b'\x21\xF9\x04\x05\x00\xFF\x00'

    # 5. Image Descriptor
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00'

    # 6. Image Data
    image_data = rng.read(width * height // 8)

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)