#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO, List


def read_uint8(f: BinaryIO) -> int:
    return struct.unpack("<B", f.read(1))[0]


def read_int8(f: BinaryIO) -> int:
    return struct.unpack("<b", f.read(1))[0]


def read_uint16(f: BinaryIO) -> int:
    return struct.unpack("<H", f.read(2))[0]


def read_int16(f: BinaryIO) -> int:
    return struct.unpack("<h", f.read(2))[0]


def read_uint32(f: BinaryIO) -> int:
    return struct.unpack("<I", f.read(4))[0]


def read_int32(f: BinaryIO) -> int:
    return struct.unpack("<i", f.read(4))[0]


def read_float(f: BinaryIO) -> float:
    return struct.unpack("<f", f.read(4))[0]


def write_uint8(f: BinaryIO, value: int):
    f.write(struct.pack("<B", value))


def write_int8(f: BinaryIO, value: int):
    f.write(struct.pack("<b", value))


def write_uint16(f: BinaryIO, value: int):
    f.write(struct.pack("<H", value))


def write_int16(f: BinaryIO, value: int):
    f.write(struct.pack("<h", value))


def write_uint32(f: BinaryIO, value: int):
    f.write(struct.pack("<I", value))


def write_int32(f: BinaryIO, value: int):
    f.write(struct.pack("<i", value))


def write_float(f: BinaryIO, value: float):
    f.write(struct.pack("<f", value))


class