#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main() -> int:
    if len(sys.argv)!= 2 or not os.path.isfile(sys.argv[1]):
        print('Usage:', sys.argv[0], '<input file>')
        return -1

    with open(sys.argv[1], 'rb') as f:
        data = bytearray(f.read())

    # Read the header and extract the number of images from it
    num_images, = struct.unpack('<I', data[:4])
    offset = 4

    for i in range(num_images):
        # Extract the descriptor size (in bytes) and the image size (in pixels)
        desc_size, img_size = struct.unpack('<II', data[offset : offset+8])

        # Skip over the descriptor to get to the raw image data
        offset += desc_size

        # Get the start address of this image's data
        addr, = struct.unpack('<I', data[offset : offset+4])
        
        # Skip over the image data to get to the next one
        offset += img_size * 2

        # Calculate the end address of this image's data
        end_addr = addr + img_size * 2

        # Replace the image data with a JMP instruction that jumps back to itself
        data[addr : end_addr] = b'\xE9' + struct.pack('<i', -(end_addr-addr))
    
    # Write the modified data to disk
    with open(sys.argv[1]+'.patched', 'wb') as f:
        f.write(bytes(data))
    
    return 0

if __name__ == '__main__':
    exit(main())

# The following code is used by the test suite

def create_test_file():
    """Create an input file for use by the test suite."""
    with open('test.bin', 'wb') as out:
        # Header
        out.write(b'BM\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00    out.write(image_descriptor + image_data + b'\x3B')