#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')  # Magic Number (6 bytes)
    width = rng.read(1)[0] & 0x7F | ((rng.read(1)[0]) << 8)
    height = rng.read(1)[0] & 0x7F | ((rng.read(1)[0]) << 8)
    flags = b'\x80\x00' if rng.read(1)[0] % 2 == 0 else b'\x00\x00'
    out.write(width.to_bytes(2, byteorder='little'))
    out.write(height.to_bytes(2, byteorder='little'))
    out.write(flags)    # Flags (2 bytes)

    # Color Table
    color_table_size = min((rng.read(1)[0] + 1), 256)
    for i in range(color_table_size):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(r.to_bytes(1, byteorder='little'))
        out.write(g.to_bytes(1, byteorder='little'))
        out.write(b.to_bytes(1, byteorder='little'))

    # Image Descriptor
    left = rng.read(1)[0] & 0x7F | ((rng.read(1)[0]) << 8)
    top = rng.read(1)[0] & 0x7F | ((rng.read(1)[0]) << 8)
    width = rng.read(1)[0] & 0x7F | ((rng.read(1)[0]) << 8)
    height = rng.read(1)[0] & 0x7F | ((rng.read(1)[0]) << 8)
    flags = b'\x80\x00' if rng.read(1)[0] % 2 == 0 else b'\x00\x00'
    out.write(left.to_bytes(2, byteorder='little'))
    out.write(top.to_bytes(2, byteorder='little'))
    out.write(width.to_bytes(2, byteorder='little'))
    out.write(height.to_bytes(2, byteorder='little'))
    out.write(flags)    # Flags (2 bytes)

    # LZW Minimum Code Size
    lzw_min_code_size = rng.read(1)[0] + 1
    out.write(lzw_min_code_size.to_bytes(1, byteorder='little'))

    # Image Data
    image_data_length = rng.read(1)[0] & 0x7F | ((rng.read(1)[0]) << 8)
    while True:
        data_block_size = rng.read(1)[0] & 0x7F | ((rng.read(1)[0]) << 8)

        if data_block_size > 0:
            data = rng.read(data_block_size)

            if len(data) < data_block_size:
                break
            
            out.write(data)
        elif data_block_size == 0:
            break
    
    # Trailer
    out.write(b';')


if