#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header (13 bytes)
    header = b'GIF89a' + \
             struct.pack('<HH', 2048, 2048)

    # Logical screen descriptor (7 bytes)
    lsd = struct.pack('BBBHBB',
                      64,    # Width
                      64,    # Height
                      0x80,  # Global color table flag
                      7,     # Color resolution
                      0,     # Sorting method
                      0      # GCT size
                  )

    # Application extension block (11 bytes)
    appext = b'\x21\xFF\x0BNETSCAPE2.0' + \
             struct.pack('<BBBB',
                         3,   # Block size
                         1,   # Extension label
                         ord('N'),
                         ord('E')
                     ) + \
             struct.pack('<H', 256) + \
             b'\x00\x00'

    # Graphics control extension block (8 bytes)
    gce = b'\x21\xF9\x04' + \
         b'\x08\x00' + \
         b'\x00'

    # Image descriptor (10 bytes)
    idd = struct.pack('>HHBBBHB',
                      0,       # Left position
                      0,       # Top position
                      64,      # Width
                      64,      # Height
                      0b11100000,  # Local color table flag
                      7,        # LZW minimum code size
                      0         # Image data sub-block offset
                 )

    # Image data sub-blocks (256 bytes each)
    imgdata = bytearray()
    for i in range(256):
        imgdata += rng.read(i % 128 if i > 0 else 1)

    # Trailer (1 byte)
    trailer = b';'

    # Write everything to output stream
    out.write(header)
    out.write(lsd)
    out.write(appext)
    out.write(gce)
    out.write(idd)
    out.write(imgdata)
    out.write(trailer)


if