#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> int:
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    packed_fields = rng.read(1)[0]
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBHB', width, height, packed_fields, background_color_index, pixel_aspect_ratio)

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors in RGB format

    # 4. Image descriptor
    image_descriptor = struct.pack('>BBHBBBBB', 0x2C, 0, 0, 0, 0, 0, 0)  # Left to right top-to-bottom image

    # 5. Extension block
    extension_block_label = b'\x21\xFF'
    sub_block_size = b'\x0B'
    application_identifier = b'NETSCAPE2.0'
    app_auth_code = b'\x03\x01\x00\x00' + b'\x00' * 11

    ext_block = extension_block_label + sub_block_size + application_identifier + app_auth_code

    # 6. Graphics Control Block
    gcb = struct.pack('>BBBBBBBHB', 0xF9, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)

    # 7. Image data
    image_data = b''
    while len(image_data) < ((width * height) // 8):
        image_data += rng.read((width * height) // 8 - len(image_data))

    return len(lsd) + len(color_table) + len(image_descriptor) + len(ext_block) + len(gcb) + len(image_data)


if