#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    header = b'GIF89a'
    
    # Logical Screen Descriptor
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)
    background_color = rng.read(1)
    pixel_aspect_ratio = rng.read(1)
    lsd = width + height + flags + background_color + pixel_aspect_ratio
    
    # Global Color Table
    global_color_table_flag = rng.read(1)
    global_color_table_size = rng.read(1)
    global_color_table = [rng.read(3) for _ in range((global_color_table_size+1)*3)]
    gct = global_color_table_flag + global_color_table_size + b''.join(global_color_table)
    
    # Image Descriptor
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)
    image_descriptor = left + top + width + height + flags
    
    # Local Color Table
    local_color_table_flag = rng.read(1)
    local_color_table_size = rng.read(1)
    local_color_table = [rng.read(3) for _ in range((local_color_table_size+1)*3)]
    lct = local_color_table_flag + local_color_table_size + b''.join(local_color_table)
    
    # Application Extension
    application_extension_label = b'\x21\xff\x0bNETSCAPE2.0'
    application_extension_authentication_code = b'\x03\x01\x00\x00'
    application_extension_application_data = b'\x08\x00'
    ae = application_extension_label + application_extension_authentication_code + application_extension_application_data
    
    # Graphics Control Extension
    graphics_control_extension_label = b'\xf9\x04'
    graphics_control_extension_block_size = b'\x04'
    delay_time = rng.read(2)
    transparent_color_index = rng.read(1)
    terminator = b'\x00'
    gce = graphics_control_extension_label + graphics_control_extension_block_size + delay_time + transparent_color_index + terminator
    
    # Plain Text Extension
    plain_text_extension_label = b'\x01\x0f'
    plain_text_extension_block_size = b'\x0c'
    text_grid_left = rng.read(2)
    text_grid_top = rng.read(2)
    text_grid_width = rng.read(2)
    text_grid_height = rng.read(2)
    cell_width = rng.read(1)
    cell_height = rng.read(1)
    foreground_color_index = rng.read(1)
    background_color_index = rng.read(1)
    terminator = b'\x00'
    pte = plain_text_extension_label + plain_text_extension_block_size + text_grid_left + text_grid_top + text_grid_width + text_grid_height + cell_width + cell_height + foreground_color_index + background_color_index + terminator
    
    # Comment Extension
    comment_extension_label = b'\xfe\x01'
    comment_extension_comment_length = rng.read(1)
    comment_extension_comment = rng.read(comment_extension_comment_length)
    ce = comment_extension_label + comment_extension_comment_length + comment_extension_comment
    
    # Graphic Control Label
    graphic_control_label = b'\x21\xf9\x04'
    
    # Block Size
    block_size = b'\x04'
    
    # Packed Field
    packed_field = b'\x00'
    
    # Delay Time
    delay_time = rng.read(2)
    
    # Transparent Color Index
    transparent_color_index = rng.read(1)
    
    # Terminator
    terminator = b'\x00'
    
    # Image Data
    # 1. LZW Minimum Code Size
    lzw_minimum_code_size = rng.read(1)[0]
    
    # 2. Sub-Blocks
    sub_blocks = []
    while True:
        block_size = rng.read(1)[0]
        if not block_size:
            break

        sub_blocks.append(bytes([block_size]) + rng.read(block_size))

    # 3. Image Data
    image_data = b''.join(sub_blocks) + b'\x00'  # Ending with a block size of 0
    
    
    # Trailer
    trailer = b'\x3b'
    
    # Write everything together
    out.write(header + lsd + gct + image_descriptor + lct + ae + gce + pte + ce + graphic_control_label + block_size + packed_field + delay_time + transparent_color_index + terminator + image_data + trailer)


# Generates a random PNG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_        block_size = rng.read(1)[0]
        if not block_size:
            break
        
        blocks.append(bytes([block_size]) + rng.read(block_size))
        
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + color_table + (gce or b'') + image_descriptor + image_data + trailer)

