#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor (LSD)
    width = rng.read(1)[0] % 64 + 1
    height = rng.read(1)[0] % 64 + 1
    packed = bytes([
        0b00000000 | ((width - 1) & 0b00011111),
        (height - 1) & 0xff,
        rng.read(1)[0],
        rng.read(1)[0],
        0b00000000 | (((rng.read(1)[0] >> 5) & 0b00000011) << 5),
        0b00000000 | (rng.read(1)[0] & 0b00000011)
    ])

    lsd = packed

    # 3. Color Table
    color_table = [bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]]) for _ in range(2 ** int((packed[0] & 0b00000011) + 1))]

    # 4. Graphics Control Extension (GCE)
    gce = bytes([
        0x21,
        0xf9,
        0x04,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,        block_size = rng.read(1)[0]

        if not block_size:
            break

        image_data.append(rng.read(block_size))

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything to the output stream
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(image_data) + trailer)


if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rng:
        with open('./test.gif', 'wb') as f:
            generate_random_gif(rng, f)