#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 768, delay: int = 50, loops: int = 0
):

    # Header
    out.write("GIF89a".encode())
    out.write(struct.pack("<HH", width, height))
    out.write("\x87\x00\x00".encode())

    # Logical screen descriptor
    out.write(",".encode())
    out.write(struct.pack("<BBBHBB", 0xF7, 0x00, 0x00, width, height, 0x00))
    out.write(";".encode())

    # Global color table (black and white)
    out.write(",".encode())
    for i in range(2 ** 8 - 1):
        if i < 128:
            out.write(struct.pack("<BBBB", i, i, i, 0xFF))
        else:
            out.write(struct.pack("<BBBB", i, i, i, 0x00))
    out.write(";".encode())

    # Graphic control extension
    out.write(",".encode())
    out.write(struct.pack("<BHBHHB", 0x21, 0xF9, 4, 0b10000000 | delay << 8, loops, 0))
    out.write(";".encode())

    # Image descriptor
    out.write(",".encode())
    out.write(struct.pack("<BHHHHB", 0x2C, 0, 0, width, height, 0))
    out.write(";".encode())

    # Local color table (black and white)
    out.write(",".encode())
    for i in range(2 ** 8 - 1):
        if i < 128:
            out.write(struct.pack("<BBBB", i, i, i, 0xFF))
        else:
            out.write(struct.pack("<BBBB", i, i, i, 0x00))
    out.write(";".encode())

    # Image data
    out.write(",".encode())
    for y in range(height):
        line = bytearray()

        for x in range(width // 8):

            # Each pixel is represented by one bit per byte
            # The first bit of each byte represents the leftmost pixel
            b = 0
            for p in range(8):
                if ((y * width + x * 8 + p) % 2 == 0) ^ (((x + y) & 1) == 0):
                    b |= 1 << p

            line += bytes([b])

        out.write(line)

    # Trailer
    out.write(";".encode())
