#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a' + \
             bytes([
                 rng.read(1)[0] & ~0x40 | 0x20,   # Version (768)
                 rng.read(1)[0],                  # Width
                 rng.read(1)[0],                  # Height
                 0xF0 | rng.read(1)[0]           # Flags (has color table)
             ])

    # 2. Color Table
    for _ in range(0, 256):
        out.write(bytes([
            rng.read(1)[0],      # Red
            rng.read(1)[0],      # Green
            rng.read(1)[0]       # Blue
        ]))

    # 3. Image Descriptor
    out.write(header[6:])

    # 4. LZW Min Code Size
    out.write(b'\x08')

    # 5. Data Sub-blocks
    while True:
        codesize = rng.read(1)[0]

        if codesize == 0xFF:
            break

        data = [codesize]

        while len(data) < 1 << codesize:
            data += list((rng.read(1)[0], rng.read(1)[0]))

        out.write(struct.pack('B', len(data)))
        out.write(bytes(data))

    # 6. Trailer
    out.write(b';')
