#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = struct.pack('<BBBHBB', 0xf7, 0, 0, width, height, 0)
    lsd = packed[:6] + b'\xff\x0b' + packed[7:]

    # 3. Global Color Table (1 byte per color)
    color_table = rng.read(3 * 2 ** (lsd[4] & 0x07))

    # 4. Graphic Control Extension
    block_size = 4
    gce = struct.pack('BBB', 0x21, 0xf9, 4) + rng.read(block_size - 3)

    # 5. Image Descriptor
    image_descriptor = struct.pack('BBHBB', 0x2c, 0, 0, width, height)

    # 6. Image Data
    data = []
    for _ in range(height):
        data.append(rng.read(block_size))

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything to output
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)