#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int = 1024, height: int = 768):
    # Header
    out.write('GIF89a'.encode())                   # Magic bytes
    out.write(struct.pack("<H", width))            # Width
    out.write(struct.pack("<H", height))           # Height
    out.write(b'\xF0\x00')                         # Background color (black)
    out.write(b'\x00')                             # Pixel aspect ratio

    # Global color table
    for i in range(256):
        out.write(bytes([i]))
        out.write(bytes([i]))
        out.write(bytes([i]))

    # Graphic control extension
    out.write(b'\x21\xF9\x04\x01\x00\x00\xFF\x00') # Block size and flags
    out.write(rng.read(1))                         # Transparent color index
    out.write(b'\x00')                             # Block terminator

    # Image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00') # Block size and flags
    out.write(struct.pack("<H", width))            # Width
    out.write(struct.pack("<H", height))           # Height
    out.write(b'\x00')                             # Local color table flag
    out.write(b'\x00')                             # Interlace flag
    out.write(b'\x00')                             # Sort flag
    out.write(b'\x00')                             # Size of local color table

    # LZW minimum code size
    out.write(b'\x08')

    # Raster data
    while True:
        block_size = rng.read(1)[0] + 1
        if not block_size:
            break
        out.write(block_size.to_bytes(1, "little"))
        out.write(rng.read(block_size))
