#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    out.write(b"GIF89a")
    
    # 2. Logical Screen Descriptor
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)
    background_color = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(width + height + flags + bytes([background_color]) + bytes([aspect_ratio]))
    
    # 3. Global Color Table
    global_color_table_size = (flags[0] & 0x7) + 1
    global_color_table_sort_flag = bool((flags[0] >> 3) & 0x1)
    global_color_table_colors_are_global_flag = bool((flags[0] >> 4) & 0x1)
    global_color_table_colors = [None for _ in range(global_color_table_size)]
    for i in range(global_color_table_size):
        red = rng.read(1)[0]
        green = rng.read(1)[0]
        blue = rng.read(1)[0]
        global_color_table_colors[i] = (red, green, blue)
        
        out.write(bytes([red]) + bytes([green]) + bytes([blue]))
    
    # 4. Image Data
    image_descriptor = b"\x2C\x00\x00\x00\x00\x00\xFF\xFF\xFF\x00"
    out.write(image_descriptor)
    
    # 5. Local Color Table
    local_color_table_size = 0
    local_color_table_sort_flag = False
    local_color_table_colors_are_global_flag = True
    local_color_table_colors = []
    
    # 6. Image Data
    lzw_minimum_code_size = rng.read(1)[0]
    out.write(bytes([lzw_minimum_code_size]))
    
    # 7. Image Data
    while True:
        block_type = rng.read(1)[0]
        if block_type == 0x00:
            break
        elif block_type == 0x01:
            pass
        else:
            raise Exception("Unknown block type: " + str(block_type))
        
        data_length = rng.read(1)[0]
        data = rng.read(data_length)
        out.write(bytes([data_length]) + data)
    
    # 8. Trailer
    out.write(b";")
