#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    background_color = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBBBB', width, height, packed, background_color, aspect_ratio, rng.read(1)[0])
    
    # 3. Global Color Table (if present)
    global_color_table_flag = bool(packed & 0x80)
    if not global_color_table_flag:
        return
    
    num_colors = 2 ** ((packed >> 4) + 1)
    color_table = rng.read(num_colors * 3)
    
    # 4. Graphic Control Extension
    gce = b'\x21\xf9\x04\x05\xff\x00\x00\x00\x00'
    
    # 5. Image Descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00'
    
    # 6. Local Color Table
    local_color_table_flag = bool(rng.read(1)[0] & 0x80)
    if local_color_table_flag:
        pass
        
    # 7. Image Data
    data = []
    while True:
        code = rng.read(1)[0]
        if code == 0x2c:
            break
            
        length = rng.read(1)[0]
        data += [code, length] + list(rng.read(length))
        
    # 8. Trailer
    trailer = b';'
    
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)


if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as f:
        with open('test.gif', 'wb') as o:
            generate_random_gif(f, o)