#!/usr/bin/env python3

import sys
import random
import struct


def main():
    inp = open(sys.argv[1], 'rb')
    out = open(sys.argv[2], 'wb')

    # Read the input file and create a stream of bytes from it
    rng = random.Random()
    rng.seed(inp.read())

    # 1. Header
    header = b'\x47\x49\x46\x38\x39\x61'

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed_fields = struct.pack('>B', rng.randint(0, 2 ** 8 - 1))
    lsd = struct.pack('>HHHBB', width, height, rng.read(1)[0], *struct.unpack('>BB', packed_fields))

    # 3. Global Color Table
    num_colors = 2 ** ((packed_fields & 0b00000111) + 1)
    color_table = rng.read(3 * num_colors)

    # 4. Graphic Control Extension (optional)
    if rng.random() < .5:
        gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    else:
        gce = b''

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Local Color Table (optional)
    if rng.read(1)[0]:  # If we want one, then add an extension
        local_color_table = struct.pack('>BB', 0x21, 0xFF) + rng.read(3 * num_colors)
    else:
        local_color_table = b''

    # 7. Raster data
    raster_data = rng.read(width * height)

    # 8. Trailer
    trailer = b'\x3b'

    # Write everything to the output
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    out.write(gce)
    out.write(image_descriptor)
    out.write(local_color_table)
    out.write(raster_data)
    out.write(trailer)

