#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # A random palette of 256 colors
    color_table = rng.read(3 * 256)

    # 4. Graphic Control Extension
    # Random delay time in ms, transparent color index, disposal method
    gce = struct.pack('>BBB', rng.read(1)[0] % 256, rng.read(1)[0], rng.read(1)[0])

    # 5. Image Descriptor
    # Random left and top position, width and height
    image_descriptor = struct.pack('>HHHH', rng.read(2)[0], rng.read(2)[0], width, height)

    # 6. Image Data
    # A random array of pixels
    image_data = rng.read(width * height)

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)