#!/usr/bin/env python3

import struct
from collections import namedtuple

# https://github.com/python-pillow/Pillow/blob/master/src/PIL/TiffImagePlugin.py

TIFF_HEADER = b"II\x2A\x00"  # little endian, magic number
BIG_ENDIAN = b"\x4D\x4D\x00\x2A"  # big endian, magic number

# TIFFTAG_ values from tif_dirinfo.h
TIFF_IMAGEWIDTH = 256
TIFF_IMAGELENGTH = 257
TIFF_BITSPERSAMPLE = 258
TIFF_COMPRESSION = 259
TIFF_PHOTOMETRIC = 262
TIFF_FILLORDER = 266
TIFF_STRIPOFFSETS = 273
TIFF_ORIENTATION = 274
TIFF_SAMPLESPERPIXEL = 277
TIFF_ROWSPERSTRIP = 278
TIFF_STRIPBYTECOUNTS = 279
TIFF_XRESOLUTION = 282
TIFF_YRESOLUTION = 283
TIFF_PLANARCONFIG = 284
TIFF_RESOLUTIONUNIT = 296
TIFF_SOFTWARE = 305
TIFF_DATETIME = 306
TIFF_ARTIST = 315
TIFF_HOSTCOMPUTER = 316
TIFF_PREDICTOR = 317
TIFF_COLORMAP = 320
TIFF_TILEWIDTH = 322
TIFF_TILELENGTH = 323
TIFF_TILEOFFSETS = 324
TIFF_TILEBYTECOUNTS = 325
TIFF_SUBFILETYPE = 325
TIFF_INKSET = 332
TIFF_INKNAMES = 333
TIFF_NUMBEROFINKS = 334
TIFF_DOTRANGE = 336
TIFF_TARGETPRINTER = 337
TIFF_EXTRASAMPLES = 338
TIFF_SAMPLEFORMAT = 339
TIFF_SMINSAMPLEVALUE = 340
TIFF_SMAXSAMPLEVALUE = 341
TIFF_CLIPPATH = 343
TIFF_XCLIPPATHUNITS = 344
TIFF_YCLIPPATHUNITS = 345
TIFF_INDEXED = 346
TIFF_JPEGTABLES = 347
TIFF_OPIPROXY = 351
TIFF_GLOBALPARAMETERSIFD = 348
TIFF_PROFILETYPE = 349
TIFF_FAXPROFILE = 350
TIFF_CODERESOLUTION = 352
TIFF_DECODE = 353
TIFF_IMAGEDEPTH = 354
TIFF_TILEDEPTH = 355
TIFF_IMAGEDEPTHTYPE = 339
TIFF_TILEDEPTHTYPE = 338
TIFF_FILLEXTRASAMPLES = 360
TIFF_DOTRANGEPRODUCER = 361
TIFF_DOTRANGEREADER = 362
TIFF_STRIPROWCOUNTS = 559
TIFF_SUBIFD = 330
TIFF_TRANSFERFUNCTION = 301
TIFF_REFERENCEBLACKWHITE = 532
TIFF_XMLPACKET = 700
TIFF_PRINTIMAGEREFS = 331
TIFF_PAGENUMBER = 297
TIFF_HALFTONEHINTS = 321
TIFF_COLORMAP2 = 539
TIFF_SESSIONENDINGSTATUS = 285
TIFF_SUBIFD2 = 338
TIFF_DEFINESBITSPERSAMPLE = 550
TIFF_