
import struct
from PIL import Image, ImageOps

# GIF constants
SIGNATURE = b'\x47\x49\x46\x38\x39\x61'
EXTENSION = b'\x21\xFF'
GRAPHICS_CONTROL_LABEL = b'\xF9\x04'
DISPOSAL_METHOD = 2
USER_INPUT_FLAG = 0
TRANSPARENT_COLOR_FLAG = 1
DELAY_TIME = 50
BLOCK_TERMINATOR = b'\x00'
TRAILER = b';'

def create_gif(frames):
    """Create a GIF from a list of images"""
    
    # Create the empty GIF with the first frame
    im = frames[0]
    width, height = im.size
    palette = im.getpalette()
    data = [im.tobytes("raw", "RGB")]
    
    # Build the LSD (Logical Screen Descriptor)
    header = SIGNATURE + struct.pack("<HHHH", width, height, 0x80, 0)
    lsd = header + struct.pack("BB", len(palette) // 3 - 1, 0)
    
    # Build the Global Color Table
    color_table = palette[:3*len(palette)//3]
    
    # Build each frame
    for im in frames[1:]:
        
        # Add the graphics control extension
        gce = EXTENSION + GRAPHICS_CONTROL_LABEL + \
            struct.pack('<BBB', DISPOSAL_METHOD, USER_INPUT_FLAG, TRANSPARENT_COLOR_FLAG) + \
            struct.pack('<H', DELAY_TIME * 10) + BLOCK_TERMINATOR
            
        # Add the image descriptor
        image_descriptor = struct.pack('<BBHHBB', 0x2C, 0, 0, width, height, 0)
        
        # Add the image data block
        data.append(gce + image_descriptor + im.tobytes("raw", "RGB"))
    
    # Build the trailer
    trailer = TRAILER
    
    # Write everything to output
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)