#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    global_color_table_size = rng.read(1)[0] & 7
    if global_color_table_size == 0:
        global_color_table_size = 1
    global_color_table = rng.read(global_color_table_size * 3)

    # 4. Application Extension Block
    app_extension_block = b'\x21\xff\x0bNETSCAPE2.0\x03\x01' + \
                         struct.pack('>H', rng.read(2)[0])

    # 5. Image Descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00' + \
                       struct.pack('>HHBB', width, height, rng.read(1)[0], rng.read(1)[0])

    # 6. Local Color Table
    local_color_table_size = rng.read(1)[0] & 7
    if local_color_table_size == 0:
        local_color_table_size = 1
    local_color_table = rng.read(local_color_table_size * 3)

    # 7. Raster Data
    raster_data = rng.read(width * height)

    # Write everything to the output file
    out.write(header + lsd + global_color_table + app_extension_block +
              image_descriptor + local_color_table + raster_data)

