
    header += b'\x00\x00\x00\x00' * 9  # Reserved for future use

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    bitdepth = rng.read(1)[0] & 0b00001111
    color_planes = rng.read(1)[0] & 0b00001111
    compression_method = rng.read(1)[0] & 0b00001111
    raw_image_size = rng.read(2)
    horizontal_resolution = rng.read(2)
    vertical_resolution = rng.read(2)
    colors_in_palette = rng.read(1)[0] & 0b00001111
    important_colors_used = rng.read(1)[0] & 0b00001111
    lsd = struct.pack('>BHHBBBBBBH',
                      bitdepth, width, height, color_planes,
                      compression_method, raw_image_size[0], raw_image_size[1],
                      horizontal_resolution, vertical_resolution,
                      colors_in_palette, important_colors_used)

    # 3. Color Table
    color_table = b''
    if colors_in_palette > 0:
        for i in range(colors_in_palette):
            r, g, b = [rng.read(1)[0] for _ in range(3)]
            color_table += struct.pack('BBB', r, g, b)

    # 4. Global Color Table
    gct_flag = bool(rng.read(1)[0] & 0b10000000)
    gct_sort_flag = bool(rng.read(1)[0] & 0b01000000)
    gct_color_resolution = ((rng.read(1)[0] & 0b00110000) >> 4)
    gct_global_table_size = (rng.read(1)[0] & 0b00001111)
    gct_global_table = b''
    if gct_flag:
        for i in range(gct_global_table_size ** 3):
            r, g, b = [rng.read(1)[0] for _ in range(3)]
            gct_global_table += struct.pack('BBB', r, g, b)

    gce_flag = bool(rng.read(1)[0] & 0b10000000)
    gce_user_input_flag = bool(rng.read(1)[0] & 0b01000000)
    gce_transparent_index_flag = bool(rng.read(1)[0] & 0b00100000)
    gce_interlace_flag = bool(rng.read(1)[0] & 0b00010000)
    gce_sorted_color_table_flag = bool(rng.read(1)[0] & 0b00001000)
    gce_global_color_table_size = (rng.read(1)[0] & 0b00000111)
    gce = b''
    if gce_flag:
        gce = struct.pack('>BBBBBB',
                          gce_user_input_flag | gce_transparent_index_flag << 1 | gce_interlace_flag << 2 |
                          gce_sorted_color_table_flag << 3 | gce_global_color_table_size << 4,
                          0, 0, 0, 0, 0)
        if gce_transparent_index_flag:
            gce = b''

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes

        if not block_size:
            break

        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)