#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header (6 bytes)
    out.write(b'GIF89a')

    # 2. Logical screen descriptor (7 bytes)
    width = rng.read(1)[0] % 50 + 10
    height = rng.read(1)[0] % 50 + 10
    flags = b'\x87\x00' if rng.read(1)[0] & 1 else b'\x00\x00'
    bg_color = rng.read(1)[0] % 256
    aspect_ratio = b'\x00'
    out.write(struct.pack('<HHBBB', width, height, flags[0], flags[1],
                          bg_color, aspect_ratio))

    # 3. Global color table (3 * 256 = 768 bytes)
    for _ in range(256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', r, g, b))

    # 4. Image descriptor (10 bytes)
    left = rng.read(1)[0] % width
    top = rng.read(1)[0] % height
    flags = b'\x87\x00' if rng.read(1)[0] & 1 else b'\x00\x00'
    delay = rng.read(1)[0] % 256
    transparent_index = rng.read(1)[0] % 256
    out.write(struct.pack('<HHBBB', left, top, flags[0], flags[1],
                          delay, transparent_index))

    # 5. Local color table (3 * 256 = 768 bytes)
    for _ in range(256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', r, g, b))

    # 6. Image data (width * height bytes)
    for i in range(height):
        for j in range(width):
            out.write(bytes([rng.read(1)[0]]))

