#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Color table size is always zero in this case
    color_table_size = 0
    # Background color index is always zero in this case
    background_color_index = 0
    # Pixel aspect ratio is always zero in this case
    pixel_aspect_ratio = 0
    logical_screen_descriptor = struct.pack(
        '>HHBBBH', width, height,
        color_table_size, background_color_index,
        pixel_aspect_ratio
    )

    # 3. Global Color Table
    global_color_table = b''

    # 4. Image Descriptor
    image_left = 0
    image_top = 0
    image_width, image_height = struct.unpack('>HH', rng.read(4))
    # No local color table
    local_color_table_flag = False
    interlace_flag = False
    sort_flag = False
    reserved_bits = 0b0000
    local_color_table_size = 0
    image_descriptor = struct.pack(
        '>HHBBBBB', image_left, image_top,
        image_width, image_height,
        local_color_table_flag, interlace_flag,
        sort_flag, reserved_bits,
        local_color_table_size
    )

    # 5. Local Color Table
    if not local_color_table_flag:
        # Generate a new palette of 256 colors
        global_color_table += bytes([
            0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,

            0x00, 0x00, 0xFF, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0x00, 0xFF, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0x00, 0xFF, 0x00, 0x00, 0xFF, 0x00, 0x00,

            0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00,
            0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00,
            0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00,
            0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00,

            0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
            0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,

            