#!/usr/bin/env python3

import struct


def read_uint8(f):
    return int.from_bytes(f.read(1), byteorder='little')


def write_uint8(f, value):
    f.write(value.to_bytes(1, byteorder='little'))


def read_uint16(f):
    return int.from_bytes(f.read(2), byteorder='little')


def write_uint16(f, value):
    f.write(value.to_bytes(2, byteorder='little'))


def read_uint32(f):
    return int.from_bytes(f.read(4), byteorder='little')


def write_uint32(f, value):
    f.write(value.to_bytes(4, byteorder='little'))


def read_uint64(f):
    return int.from_bytes(f.read(8), byteorder='little')


def write_uint64(f, value):
    f.write(value.to_bytes(8, byteorder='little'))


def read_int8(f):
    return int.from_bytes(f.read(1), byteorder='little', signed=True)


def write_int8(f, value):
    f.write(value.to_bytes(1, byteorder='little', signed=True))


def read_int16(f):
    return int.from_bytes(f.read(2), byteorder='little', signed=True)


def write_int16(f, value):
    f.write(value.to_bytes(2, byteorder='little', signed=True))


def read_int32(f):
    return int.from_bytes(f.read(4), byteorder='little', signed=True)


def write_int32(f, value):
    f.write(value.to_bytes(4, byteorder='little', signed=True))


def read_int64(f):
    return int.from_bytes(f.read(8), byteorder='little', signed=True)


def write_int64(f, value):
    f.write(value.to_bytes(8, byteorder='little', signed=True))


def read_float32(f):
    return struct.unpack('f', f.read(4))[0]


def write_float32(f, value):
    f.write(struct.pack('f', value))


def read_float64(f):
    return struct.unpack('d', f.read(8))[0]


def write_float64(f, value):
    f.write(struct.pack('d', value))


def read_string(f):
    length = read_uint32(f)

    if length == 0:
        return ''

    data = f.read(length - 1)
    assert len(data) == (length - 1)
    assert f.read(1) == b'\x00'

    return str(data, 'utf-8')


def write_string(f, string):
    encoded = bytes(string, 'utf-8') + b'\x00'
    length = len(encoded)

    write_uint32(f, length)
    f.write(encoded)


class