#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a' + bytes([0x42]) * 7

    # 2. Logical Screen Descriptor
    width = rng.read(1)[0] % (2**6+1)
    height = rng.read(1)[0] % (2**6+1)
    flags = [0b00000000, 0b00000000][rng.read(1)[0] % 2]
    bg_color = rng.read(1)[0] % (2**5+1)
    aspect_ratio = rng.read(1)[0] % (2**3+1)
    lsd = bytearray()
    lsd += struct.pack('B', width)
    lsd += struct.pack('B', height)
    lsd += struct.pack('B', flags)
    lsd += struct.pack('B', bg_color)
    lsd += struct.pack('B', aspect_ratio)

    # 3. Global Color Table
    gct = bytearray()
    for i in range(0, 2**(flags & 0b00000111)+1):
        r = rng.read(1)[0] % (2**3+1)
        g = rng.read(1)[0] % (2**3+1)
        b = rng.read(1)[0] % (2**2+1)
        gct += struct.pack('BBB', r, g, b)

    # 4. Image Descriptor
    image_descriptor = bytearray()
    left = rng.read(1)[0] % (2**6+1)
    top = rng.read(1)[0] % (2**6+1)
    width = rng.read(1)[0] % (2**6+1)
    height = rng.read(1)[0] % (2**6+1)
    flags = [0b00000000, 0b00000000][rng.read(1)[0] % 2]
    image_descriptor += struct.pack('B', left)
    image_descriptor += struct.pack('B', top)
    image_descriptor += struct.pack('B', width)
    image_descriptor += struct.pack('B', height)
    image_descriptor += struct.pack('B', flags)

    # 5. Local Color Table
    lct = bytearray()
    if flags & 0b00001000 == 0b00001000:
        for i in range(0, 2**(flags & 0b00000111)+1):
            r = rng.read(1)[0] % (2**3+1)
            g = rng.read(1)[0] % (2**3+1)
            b = rng.read(1)[0] % (2**2+1)
            lct += struct.pack('BBB', r, g, b)

    # 6. Image Data
    data = bytearray()
    for y in range(height):
        for x in range(width):
            color_index = rng.read(1)[0] % ((flags & 0b00000111) + 1)
            data += struct.pack('B', color_index)

    # Write everything to output stream
    out.write(header)
    out.write(lsd)
    out.write(gct)
    out.write(image_descriptor)
    if len(lct) > 0:
        out.write(lct)
    out.write(data)

