#!/usr/bin/env python3

import struct
from random import randint


def generate_random_gif(in_, out):
    """Generate a GIF file containing the data read from in_."""
    
    # Header
    header = b'GIF89a'
    
    # Logical Screen Descriptor (LSD)
    width = randint(200, 500)
    height = randint(200, 500)
    packed = ((width & 0xFF) << 8) | (height & 0xFF)
    lsd = struct.pack('<6sHHBBB', b'Logical Screen Descriptor',
                       width, height,
                       0xF0 | 0, 0, 0, 0, packed)
    
    # Color Table
    colors = [randint(0, 255), randint(0, 255), randint(0, 255)]
    color_table = bytes([len(colors)]) + bytes(colors * 256)
    
    # Graphics Control Extension (GCE)
    delay = randint(10, 40)
    transparent = randint(0, len(color_table))
    gce = struct.pack('<6sBBHB', b'Graphics Control Extension',
                     0xF9, 0, 0,
                     delay, transparent)
    
    # Image Descriptor
    left = randint(0, width - 1)
    top = randint(0, height - 1)
    width = randint(1, min(left, width - left))
    height = randint(1, min(top, height - top))
    packed = (((left & 0xFFF8) >> 3) << 12) | \
             (((top & 0xFFF8) >> 3) << 8) | \
             (((width & 0x07) << 5) |
              ((height & 0x07) << 2) |
              0x00)
    image_descriptor = struct.pack('<6sHHBBBB',
                                   b'Image Descriptor',
                                   0, 0,
                                   width, height,
                                   0, packed)
    
    # Data
    data = []
    for _ in range((height // 8) * width):
        byte = 0
        for i in range(8):
            if randint(0, 1):
                byte |= (1 << i)
        data += [bytes([byte])]
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to output
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)


if __name__ == '__main__':
    from sys import argv

    with open('/dev/urandom', 'rb') as f:
        with open(argv[1], 'wb') as o:
            generate_random_gif(f, o)