#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
	# 2. Logical screen descriptor (LSD)
    lsd = bytes([
        0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 0x01, 0x00, 0x01, 0x00, 0x80, 0x00,
		0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x2c, 0x00, 0x00, 0x00, 0x00, 0x01,
		0x00, 0x01, 0x00, 0x00, 0x02, 0x02, 0x44, 0x01, 0x00, 0x3b])

    # 3. Global color table (GCT)
    gct = [0] * 256 * 3
    for i in range(len(gct)):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        gct[i*3 + 0] = r
        gct[i*3 + 1] = g
        gct[i*3 + 2] = b
    
    # 4. Image descriptor (ID)
    id = bytes([
        0x2c, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0x00, 0x02, 0x02,
		0x44, 0x01, 0x00, 0x3b])

    # 5. Image data
    image_data = []
    for _ in range(0, 10):
        image_data += [
            rng.read(1),
            rng.read(1),
            rng.read(1)]

    # Write everything to output
    out.write(header)
    out.write(lsd)
    out.write(bytes(gct))
    out.write(id)
    out.write(bytes(image_data))

