#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # Number of colors in global color table
    ncolors = rng.read(1)[0] % 256 + 1

    # Actual color table
    gct = []
    for i in range(ncolors):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        gct += [r,g,b]

    # 4. Image Descriptor
    id = struct.pack('>HHBBB', 0, 0, 0, 0, 0)

    # 5. Extension Block
    ext = struct.pack('>BB', 0x21, 0xff)

    # 6. Graphics Control Extension
    gce = struct.pack('>BBBBBBH', 0x21, 0xf9, 0x04, 0, 0, 0, 0)

    # 7. Application Extension
    app = struct.pack('>BBBBB', 0x21, 0xff, 0x0b, 0x4e, 0x45, 0x54, 0x53, 0x43, 0x41, 0x50, 0x45, 0x32, 0x2e, 0x30, 0x03, 0x01, 0x00, 0x00, 0x00)

    # 8. Comment Extension
    comment = struct.pack('>BBBBB', 0x21, 0xfe, 0x01, ord('A'), ord('B'))

    # 9. Plain Text Extension
    text = struct.pack('>BBBBB', 0x21, 0xff, 0x01, ord('C'), ord('D'))

    # 10. Trailer
    trailer = b';'

    # Write everything to the output stream
    out.write(header)
    out.write(lsd)
    out.write(bytes([ncolors]))
    out.write(bytes(gct))
    out.write(id)
    out.write(ext)
    out.write(gce)
    out.write(app)
    out.write(comment)
    out.write(text)
    out.write(trailer)


