#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 768, frames: int = 50
) -> None:

    # Header
    out.write("GIF89a".encode())
    out.write(struct.pack("<HH", width, height))
    out.write("\x87\x00\x00".encode())

    # Logical Screen Descriptor
    out.write(",".encode())
    out.write(struct.pack("<BBBHBHHB", 0xF7, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF))

    # Global Color Table
    for i in range(0, 256):
        out.write(struct.pack("<BBBB", (i * 3) % 255, (i * 5) % 255, (i * 7) % 255, 0x00))

    # Loop Control Block
    out.write(";".encode())
    out.write(struct.pack("<BBH", 0x01, 0x00, 0xFE))

    # Image Descriptor
    out.write(",".encode())
    out.write(struct.pack("<BBHHHHHH", 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write(",".encode())
    out.write(struct.pack("<BBB", 0x00, 0x00, 0x00))
    out.write