#!/usr/bin/env python3
import random
import struct


def generate_random_gif(rng, out):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = [int(rng.read(1)[0] / 2) for _ in range(2)]
    flags = rng.read(1)[0] | ((rng.read(1)[0] & 0x01) << 7)
    bg_color = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBB', width, height, flags, bg_color, aspect_ratio)
    
    # 3. Color Table
    num_colors = min((flags >> 4) + 1, 256)
    
    color_table = rng.read(3 * num_colors)  # Random colors
    
    # 4. Graphics Control Extension (optional)
    gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    
    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    data = []
    while True:
        block_size = rng.read(1)[0] & 0xFF
        
        if block_size == 0:
            break
        
        data.append(rng.read(block_size))
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to output
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)


if __name__ == '__main__':
    from sys import argv

    with open('/dev/urandom', 'rb') as f:
        with open(argv[1], 'wb') as o:
            generate_random_gif(f, o)