#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    background_color = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBBBB', width, height, packed, background_color, aspect_ratio, rng.read(1)[0])
    
    # 3. Global Color Table (if present)
    global_color_table_flag = bool(packed & 0x80)
    if not global_color_table_flag:
        return
    
    color_resolution = ((packed >> 7) & 0b111) + 1
    sort_flag = bool((packed >> 6) & 0b1)
    size_of_global_color_table = 2 ** ((packed >> 3) & 0b111)
    global_color_table_sorted_flag = bool((packed >> 2) & 0b1)
    global_color_table_size = int(((packed & 0b11) << 8) | rng.read(1)[0])

    global_color_table = []
    for _ in range(global_color_table_size):
        r, g, b = [rng.read(1)[0] for i in range(3)]
        global_color_table += [r, g, b]
    
    # 4. Image Descriptor
    image_left_position = struct.unpack('>H', rng.read(2))[0]
    image_top_position = struct.unpack('>H', rng.read(2))[0]
    image_width = struct.unpack('>H', rng.read(2))[0]
    image_height = struct.unpack('>H', rng.read(2))[0]
    packed = rng.read(1)[0]
    local_color_table_flag = bool(packed & 0b10000000)
    interlace_flag = bool(packed & 0b1000000)
    sorted_color_table_flag = bool(packed & 0b100000)
    reserved_bits = (packed & 0b111000) >> 3
    size_of_local_color_table = 2 ** (packed & 0b111)
    
    # 5. Local Color Table
    local_color_table = []
    for _ in range(size_of_local_color_table):
        r, g, b = [rng.read(1)[0] for i in range(3)]
        local_color_table += [r, g, b]
    
    # 6. LZW Minimum Code Size
    minimum_code_size = rng.read(1)[0]
    
    # 7. Data Sub-blocks
    data_sub_block_size = 2 ** minimum_code_size
    end_of_data_sub_block = False
    while not end_of_data_sub_block:
        sub_block_type = rng.read(1)[0]
        
        if sub_block_type == 0x00:
            num_bytes_in_sub_block = rng.read(1)[0]
            
            # TODO: Implement this
            pass
            
        elif sub_block_type == 0x01:
            num_bytes_in_sub_block = rng.read(1)[0]
            
            # TODO: Implement this
            pass
            
        else:
            raise Exception("Invalid sub block type")
            
        
    