#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_gif(out: BinaryIO):
    """Generate a random GIF file"""

    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width = rng.randint(1, 2**11 - 1)
    height = rng.randint(1, 2**11 - 1)
    packed = struct.pack('>HBBBHB', width, height, rng.randint(0, 7), 0, 0, 0)
    lsd = packed[0:7]

    # 3. Color Table
    num_colors = min((width * height) // 256, 256)
    colors = [struct.pack('>BBBB', rng.randint(0, 255), rng.randint(0, 255), rng.randint(0, 255), 0) for _ in range(num_colors)]
    color_table = b''.join(colors)

    # 4. Global Color Extension
    if rng.randint(0, 1):
        gce = struct.pack('>BBB', 0x21, 0xF9, 4) + struct.pack('>B', rng.randint(0, 3)) + b'\x00'
    else:
        gce = b''

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes
        if not block_size:
            break
        blocks.append(rng.read(block_size))
        total_blocks_size -= block_size
        if total_blocks_size <= 0:
            break

    image_data = struct.pack('>BB', lzw_minimum_code_size, len(blocks)) + b''.join(blocks)

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)

