#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    out.write(header)

    # 2. Logical Screen Descriptor
    width = rng.read(1)[0] & 0xFF
    height = rng.read(1)[0] & 0xFF
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0, 0)
    out.write(packed)

    # 3. Color Table
    color_table = [struct.pack('<BBBB', *rng.read(4)) for _ in range(0, 2**((width+height)&7))]
    out.write(b''.join(color_table))

    # 4. Graphics Control Extension
    gce = b''
    packed = struct.pack('<BBHBBBBBHB', 0x21, 0xf9, 4, 0b10000000 | ((rng.read(1)[0] & 0x0F) << 2), 0, 0, 0, 0, 0, 0, 0, 0)
    gce += packed
    gce += struct.pack('<B', 0x00)

    # 5. Image Descriptor
    image_descriptor = struct.pack('<BBHHHHB', 0x2c, 0, 0, 0, 0, 0, 0)
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    data = []
    while True:
        block_size = rng.read(1)[0] & 0xFF
        
        if block_size == 0:
            break
        
        data.append(rng.read(block_size))
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to output
    out.write(header + lsd + color_table + gce + image_descriptor + b''.join(data) + trailer)


if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as f:
        with open('test.gif', 'wb') as o:
            generate_random_gif(f, o)