#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header (6 bytes)
    header = b'GIF89a' + rng.read(5)

    # 2. Logical screen descriptor (7 bytes)
    width = int.from_bytes(rng.read(2), 'big') % 0x10000
    height = int.from_bytes(rng.read(2), 'big') % 0x10000
    flags = bytearray([
        0b00000001 if rng.read(1)[0] & 0b00000001 else 0,
        0b00000010 if rng.read(1)[0] & 0b00000010 else 0,
        0b00000100 if rng.read(1)[0] & 0b00000100 else 0,
        0b00001000 if rng.read(1)[0] & 0b00001000 else 0,
        0b00010000 if rng.read(1)[0] & 0b00010000 else 0,
        0b00100000 if rng.read(1)[0] & 0b00100000 else 0,
        0b01000000 if rng.read(1)[0] & 0b01000000 else 0,
    ])
    bg_color = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    logical_screen_descriptor = struct.pack('>HHBBB', width, height, *flags) \
                                + bytes([bg_color]) \
                                + bytes([pixel_aspect_ratio])

    # 3. Global color table (3 bytes)
    global_color_table_size = rng.read(1)[0] & 0b00001111
    sorted_flag = bool((rng.read(1)[0] & 0b00010000) >> 4)
    global_color_table_bpp = ((rng.read(1)[0] & 0b11100000) >> 5) + 1
    global_color_table_entry_size = 2 ** global_color_table_bpp
    global_color_table_length = 2 ** global_color_table_size
    global_color_table = []
    for i in range(global_color_table_length):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        global_color_table += [r, g, b]

    # 4. Image descriptor (9 bytes)
    left = int.from_bytes(rng.read(2), 'big') % width
    top = int.from_bytes(rng.read(2), 'big') % height
    image_width = int.from_bytes(rng.read(2), 'big') % width
    image_height = int.from_bytes(rng.read(2), 'big') % height
    flags = bytearray([
        0b00000001 if rng.read(1)[0] & 0b00000001 else 0,
        0b00000010 if rng.read(1)[0] & 0b00000010 else 0,
        0b00000100 if rng.read(1)[0] & 0b00000100 else 0,
        0b00001000 if rng.read(1)[0] & 0b00001000 else 0,
        0b00010000 if rng.read(1)[0] & 0b00010000 else 0,
        0b00100000 if rng.read(1)[0] & 0b00100000 else 0,
        0b01000000 if rng.read(1)[0] & 0b01000000 else 0,
    ])
    local_color_table_flag = bool((rng.read(1)[0] & 0b00000001