#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))

    # 3. Global Color Table (palette)
    palette = [0] * 768
    for i in range(len(palette)):
        palette[i] = ord(rng.read(1))

    # 4. Image Descriptor
    image_descriptor = bytearray()
    image_descriptor += struct.pack('<BBH',
        0x2C, # Image Separator
        0,    # Left Position
        0,    # Top Position
        width,# Width
        )
    image_descriptor += struct.pack('<BB',
        0,        # Height
        palette[0],# LZW Minimum Code Size
        )

    # 5. Image Data
    image_data = bytearray()
    for y in range(height):
        for x in range(width):
            image_data.append(
                palette[(ord(rng.read(1)) % len(palette))]
            )

    # Write everything to output stream
    out.write(header + struct.pack('>HH', width, height))
    out.write(bytes(image_descriptor))
    out.write(b'\x00')
    out.write(b','.join([chr((c // 256) & 0xFF).encode(), chr(c & 0xFF).encode()]) for c in palette)
    out.write(b';')
    out.write(bytes(image_data))

