#!/usr/bin/env python3
import random
import struct


def generate(out):
    """Generate an LZW-compressed GIF file"""
    rng = random.Random()
    rng.seed(a=None)

    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = struct.pack('>BBH', rng.read(1)[0], rng.read(1)[0], 0b11111)  # Global Color Table Flag set and Size = 7 bits
    background_index = rng.randint(0, 255)
    pixel_aspect_ratio = rng.randint(0, 255)
    lsd = struct.pack('>HHBBB', width, height, packed[0], packed[1], packed[2]) \
          + bytes([background_index]) + bytes([pixel_aspect_ratio])

    # 3. Global Color Table
    num_colors = (rng.read(1)[0] % 256) or 1
    global_color_table = [struct.pack('>BBBB', *rng.read(4)) for _ in range(num_colors)]
    color_table = b''.join(global_color_table)

    # 4. Graphics Control Extension
    gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes

        if not block_size:
            break

        blocks.append(bytes([block_size]) + rng.read(block_size))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)