#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical screen descriptor
    width = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    height = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    lsd = struct.pack('<HHBBB', width, height, 0xF0, 0, 0)

    # 3. Global color table
    global_color_table_size = rng.read(1)[0] & 0x07
    global_color_table = rng.read(3 * pow(2, global_color_table_size))

    # 4. Graphics control extension
    gce = struct.pack('<BHBHB', 0x21, 0xf9, 4, 0, 0)

    # 5. Image descriptor
    image_descriptor = struct.pack('<BHHHBB', 0x2c, 0, 0, width, height, 0)

    # 6. Local color table
    num_colors = min((global_color_table_size + 1), 256)
    if rng.read(1)[0]:  # If we want one, then add an extension
        local_color_table = struct.pack('>BB', 0x21, 0xFF) + rng.read(3 * num_colors)
    else:
        local_color_table = b''

    # 7. Raster data
    raster_data = rng.read(width * height)

    # 8. Trailer
    trailer = b'\x3b'

    # Write everything to the output
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    out.write(gce)
    out.write(image_descriptor)
    out.write(local_color_table)
    out.write(raster_data)
    out.write(trailer)

