#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    # 2. Logical Screen Descriptor
    # 3. Global Color Table
    # 4. Application Extension
    # 5. Image Descriptor
    # 6. Local Color Table
    # 7. Image Data
    # 8. Trailer

    # 1. Header
    out.write(b'GIF')
    out.write(struct.pack('<BB', 0x39, 0x3a))

    # 2. Logical Screen Descriptor
    out.write(struct.pack('<BHHHBB', 0xff, 0xff, 0xff, 0xff, 0x00, 0x00))

    # 3. Global Color Table
    for i in range(0, 256):
        out.write(struct.pack('<BBB', i, i, i))

    # 4. Application Extension
    out.write(struct.pack('<BB', 0x21, 0xf9))
    out.write(b'\x04\x04\x00\x00\x00')
    out.write(b'\x00')

    # 5. Image Descriptor
    out.write(struct.pack('<BBHHHBB', 0x2c, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00))

    # 6. Local Color Table
    out.write(struct.pack('<BB', 0x00, 0x00))

    # 7. Image Data
    for _ in range(0, 1 << 16):
        out.write(struct.pack('<B', rng.read(1)[0]))

    # 8. Trailer
    out.write(b';')
