#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # We use a global color table with 256 colors, so we need to read 768 bytes from /dev/urandom
    gct = rng.read(768)

    # 4. Image Data
    # We will write one image block per pixel in the image
    idd = []
    for i in range(width * height):
        # The first byte of the image data is the code size, which must be between 2 and 9
        idd.append(struct.pack('BB', rng.read(1)[0] % 8 + 2, rng.read(1)[0]))

    # Write everything together
    out.write(header)
    out.write(lsd)
    out.write(gct)
    for d in idd:
        out.write(d)

