#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480):
    # Magic header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBB', width, height, 0xF7, 0))

    # Global color table (256 colors)
    for i in range(256):
        out.write(struct.pack('B', i))

    # Graphic control extension
    out.write(struct.pack('BBHBB', 0x21, 0xF9, 4, 0, 0))

    # Image descriptor
    out.write(struct.pack('BBHHBB', 0x2C, 0, 0, width, height, 0))

    # Local color table (no colors)
    out.write(struct.pack('B', 0))

    # LZW minimum code size
    out.write(struct.pack('B', 8))

    # Pixel data
    while True:
        # Read 4 bytes of random data from /dev/urandom
        b = rng.read(4)

        if len(b) == 0:
            break

        # Write them as-is to the output stream
        out.write(b)
