#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    # 256 colors in RGB format
    global_color_table = rng.read(768)
    
    # 4. Image Descriptors
    # We only need one image descriptor
    idd = struct.pack('>BBHHHBB', 0x2C, 0, 0, width, height, 0, 0)
    
    # 5. Local Color Table
    # No local color table
    lct = b''
    
    # 6. Image Data
    # Just fill it with random data
    image_data = rng.read(width * height)
    
    # 7. Trailer
    trailer = b';'
    
    # Write everything to the output file
    out.write(header + lsd + global_color_table + idd + lct + image_data + trailer)
