#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Packed fields
    packed = struct.pack('B', 0xF7)
    # Background color index
    background_color_index = struct.pack('B', rng.read(1)[0])
    # Pixel aspect ratio
    pixel_aspect_ratio = struct.pack('B', rng.read(1)[0])
    # Combine all of them together
    logical_screen_descriptor = header + \
                                width + \
                                height + \
                                packed + \
                                background_color_index + \
                                pixel_aspect_ratio

    # 3. Global Color Table
    global_color_table = rng.read(768)

    # 4. Image Descriptor
    image_descriptor = struct.pack('BBHBB',
        0x2C, 0, 0, width[0], height[0]
    )

    # 5. Local Color Table
    local_color_table = rng.read(768)

    # 6. Image Data
    image_data = rng.read(width * height)

    # 7. Trailer
    trailer = struct.pack('B', 0x3B)

    # Write everything to the output file
    out.write(logical_screen_descriptor)
    out.write(global_color_table)
    out.write(image_descriptor)
    out.write(local_color_table)
    out.write(image_data)
    out.write(trailer)
