#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    global_color_table = rng.read(num_colors * 3)
    
    # 4. Image Descriptors
    # Let's use one image descriptor with a random left-top corner, random width and height, no local color table,
    # and a random transparency index
    idd = struct.pack('>HHBBB', rng.read(2)[0], rng.read(2)[0], rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 5. Image Data
    # Let's just use a random amount of data
    image_data = rng.read(width * height // 2)
    
    # 6. Trailer
    trailer = b';'
    
    # Write everything to the output stream
    out.write(header + lsd + global_color_table + idd + image_data + trailer)
