#!/usr/bin/env python3
import random
import sys
import struct

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: {} <output file>".format(sys.argv[0]))
        exit(1)
    
    out = open(sys.argv[1], "wb")
    rng = random.SystemRandom()
    
    # 1. Header
    header = b'\x47\x49\x46\x38\x39\x61'
    
    # 2. Logical Screen Descriptor
    ls_width = rng.randint(1, 65535)
    ls_height = rng.randint(1, 65535)
    global_color_table_flag = rng.choice([False, True])
    color_resolution = rng.choice([False, True])
    sort_flag = rng.choice([False, True])
    size_of_global_color_table = rng.choice([2 ** i for i in range(1, 9)])
    background_color_index = rng.randint(0, size_of_global_color_table - 1)
    pixel_aspect_ratio = rng.choice([False, True])
    
    ls_packed = struct.pack('BBHBBBB', ls_width, ls_height, global_color_table_flag, color_resolution, sort_flag, size_of_global_color_table, background_color_index, pixel_aspect_ratio)
    
    # 3. Global Color Table
    color_table = rng.bytes(3 * size_of_global_color_table)
    
    # 4. Graphic Control Extension
    gce = rng.read(6)
    
    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while total_blocks_size > 0:
        block_size = min(total_blocks_size, rng.read(1)[0] + 1)
        blocks.append(rng.read(block_size))
        
        total_blocks_size -= block_size
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    if len(gce) == 6:
        out.write(gce)
    out.write(image_descriptor)
    for block in blocks:
        out.write(struct.pack('B', len(block)))
        out.write(block)
    out.write(trailer)