
import random
from PIL import Image, ImageOps

def create_gif(out, width=640, height=480):
    """Create a GIF file with a single frame and write it to `out`."""
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    lsd = struct.pack('<HHBB', width, height, 0, 0)

    # 3. Global Color Table
    color_table = (b'\xFF\xFF\xFF'  # White background
                   b'\x00\x00\x00')  # Black foreground

    # 4. Graphic Control Extension
    gce = struct.pack('<BBBHB', 0xF9, 4, 0, 0, 0)

    # 5. Image Descriptor
    image_descriptor = struct.pack('<BBHHBB', 0x2C, 0, 0, width, height, 0)

    # 6. Image Data
    rng = random.SystemRandom()
    lzw_minimum_code_size = 8
    clear_code = 1 << lzw_minimum_code_size
    end_of_information = clear_code + 1
    available_codes = clear_code + 2
    old_code = None
    code_size = lzw_minimum_code_size + 1
    dict = {}
    data = []
    while True:
        try:
            code = int(rng.random() * (1 << code_size))
        except ValueError:
            break
        if code == clear_code:
            code_size = lzw_minimum_code_size + 1
            available_codes = clear_code + 2
            old_code = None
        elif code > available_codes:
            pass
        elif old_code is not None:
            if code == available_codes and old_code != clear_code:
                dict[available_codes] = dict[old_code] + dict[old_code][0]
            else:
                dict[available_codes] = dict[old_code] + [dict[old_code][0]]
            available_codes += 1
            if available_codes >= (1 << code_size):
                code_size += 1
        old_code = code
        if code == end_of_information:
            break
        data.append(dict[code])
    data = b''.join(data)
    blocks = []
    while data:
        data = bytes([block_size]) + rng.read(block_size)
        if len(data) < 255:
            data += b'\x00'  # Ending with a block size of 0
        else:
            blocks[-1] = blocks[-1][:-1] + bytes([len(data)]) + data[1:]
        blocks.append(data)

    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks)

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)