#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('B', 0b00000000 | (rng.read(1)[0] & 0b00001111)) + \
          struct.pack('BB', rng.read(1)[0], rng.read(1)[0]) + \
          struct.pack('B', rng.read(1)[0])
        
    # Write everything to the output stream
    out.write(header)
    out.write(lsd)
    
    # 3. Global Color Table
    # We don't need one because we're not using any colors
    
    # 4. Image Descriptor
    idesc = struct.pack('B', 0x2c) + \
            struct.pack('BB', 0, 0) + \
            struct.pack('BB', width - 1, height - 1) + \
            struct.pack('B', 0)
            
    out.write(idesc)
    
    # 5. Local Color Table
    # We don't need one because we're not using any colors
    
    # 6. Image Data
    # We just write a bunch of zeroes
    out.write(struct.pack('B', 0) * ((width * height) // 8))
    
    # 7. Trailer
    out.write(b';')
