#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    # Use fixed color table size of 7 bits per pixel
    lsd = bytearray([
        0b10000000 | ((width >> 8) & 0x0f),
        width & 0xff,
        0b10000000 | ((height >> 8) & 0x0f),
        height & 0xff,
        0b10000000 | 7,
        0,
        0,
        0
    ])

    # 3. Global Color Table
    gct = bytearray()
    for i in range(pow(2, lsd[4]) * 3):
        # Each entry is a 3-byte RGB triple
        gct += rng.read(1)

    # 4. Image Descriptor
    idd = bytearray([
        0b10000000 | ((width >> 8) & 0x0f),
        width & 0xff,
        0b10000000 | ((height >> 8) & 0x0f),
        height & 0xff,
        0b10000000 | 0,
        0,
        0,
        0
    ])

    # 5. Local Color Table
    lct = bytearray()
    for i in range(pow(2, lsd[4]) * 3):
        # Each entry is a 3-byte RGB triple
        lct += rng.read(1)

    # 6. Graphics Control Extension
    gce = bytearray([
        0b10000000 | 0xf9,
        4,
        0,
        0,
        0,
        0b10000000 | 0,
        0,
        0
    ])

    # 7. Plain Text Extension
    pte = bytearray([
        0b10000000 | 0xfe,
        12,
        0,
        0,
        0,
        0b10000000 | 0,
        0,
        0,
        0,
        0,
        0,
        0,
        0
    ])

    # 8. Application Extension
    ae = bytearray([
        0b10000000 | 0xff,
        11,
        0,
        0,
        0,
        0b10000000 | 0,
        0,
        0,
        0,
        0,
        0,
        0
    ])

    # 9. Comment Extension
    ce = bytearray([
        0b10000000 | 0xfe,
        3,
        0,
        0,
        0,
        0b10000000 | 0
    ])

    # 10. Trailer
    trailer = b';'

    # Write everything to output stream
    out.write(header + lsd + gct + idd + lct + gce + pte + ae + ce + trailer)


