#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> int:
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    packed_fields = rng.read(1)[0]
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBHB', width, height, packed_fields, background_color_index, pixel_aspect_ratio)

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Application Extension Block
    app_ext_label = b'\x21\xFF\x0BNETSCAPE2.0'
    app_ext_block = app_ext_label + b'\x03\x01\x00\x00' + b'\x00' * 11

    # 5. Graphics Control Extension Block
    gce = None
    if bool(rng.read(1)):
        gce = b'\x21\xF9\x04' + rng.read(3) + b'\x00'

        # 6. Image Data
        image_descriptor = struct.pack('>BBH', 0x2C, 0, 0)
        image_data = b''
    else:
        # 7. Image Data
        image_descriptor = struct.pack('>BBHH', 0x2C, 0, 0, 0)
        image_data = b'\x00' * ((width * height) // 2)

    return len(lsd) + len(color_table) + len(app_ext_block) + len(gce or b'') + len(image_descriptor) + len(image_data)

    if gce is not None:
        out.write(gce)
    out.write(image_descriptor + image_data + b'\x3B')