#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    numColors = min(max(1, rng.read(1)[0] % 257), 256)
    gct = []
    for _ in range(numColors):
        r, g, b = [c % 256 for c in rng.read(3)]
        gct += [b, g, r]

    # 4. Image Descriptor
    imgDesc = struct.pack('>HHBBBBB', 0, 0, 0, 0, 0, 0, 0)
    
    # 5. Image Data
    imageData = b''
    while len(imageData) < width * height:
        imageData += struct.pack('>B', rng.read(1)[0])
    
    # 6. Trailer
    trailer = b';'
    
    # Write everything to output stream
    out.write(header + lsd + bytes([len(gct)]) + bytes(gct) + imgDesc + imageData + trailer)

