#!/usr/bin/env python3
import sys
import random
import struct


if __name__ == '__main__':
    try:
        seed = int(sys.argv[1])
    except IndexError:
        print("Usage: {} SEED".format(sys.argv[0]))
        exit()
    
    rng = random.Random(seed)
    
    # Create a new file with a random name and open it
    filename = 'out{}.png'.format(rng.randint(0, 1e10))
    out = open(filename, mode='wb')
    
    # Generate the PNG header
    magic = b'\x89PNG\r\n\x1a\n'
    header = magic + struct.pack('>IIBBBBB',
        13,  # Length of IHDR chunk
        0,   # Chunk type - IHDR
        0,   # Compression method - none
        0,   # Filter method - adaptive
        0,   # Interlace method - none
        0,   # Bit depth - 8 bits
        3     # Color type - true color with alpha channel
    )
    
    # 1. Logical Screen Descriptor
    width = rng.randint(1, 1024)
    height = rng.randint(1, 1024)
    bit_depth = rng.randint(1, 8)
    color_type = rng.randint(0, 6)
    compression_method = rng.randint(0, 3)
    filter_method = rng.randint(0, 4)
    interlace_method = rng.randint(0, 1)
    lsd = struct.pack('>IIIBBBBBBB',
        13,      # Length of LSD
        0x78,    # Chunk type - LSD
        width,   # Width
        height,  # Height
        bit_depth,
        color_type,
        compression_method,
        filter_method,
        interlace_method
    )
    
    # 2. Color Table (only needed when color type is indexed)
    color_table = bytearray()
    if color_type == 3:
        for i in range((2 ** bit_depth)):
            r = rng.randint(0, 255)
            g = rng.randint(0, 255)
            b = rng.randint(0, 255)
            
            color_table += struct.pack('BBB', r, g, b)
    
    # 3. Palette (only needed when color type is paletted)
    
    # 4. Graphics Control Extension (optional)
    gce = None
    if rng.read(1)[0]:
        gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    
    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    image_descriptor = struct.pack('>BHHHHB', 0x2C, left, top, width, height, rng.read(1)[0])
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while total_blocks_size > 0:
        block_size = min(total_blocks_size, rng.read(1)[0] + 1)
        blocks.append(rng.read(block_size))
        
        total_blocks_size -= block_size
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    if gce is not None:
        out.write(gce)
    out.write(image_descriptor)
    for block in blocks:
        out.write(struct.pack('B', len(block)))
        out.write(block)
    out.write(trailer)