#!/usr/bin/env python3

import sys
import os
import struct
from io import BytesIO
import random

if __name__ == '__main__':
    if len(sys.argv)!= 2:
        print("Usage: {} <output file>".format(os.path.basename(__file__)))
        exit(-1)
    
    output_filename = sys.argv[1]

    # Create an empty file to write our PNG data into
    out = open(output_filename, 'wb')

    # Initialize a random number generator using the current time as seed
    rng = random.Random()
    rng.seed()

    # 1. Header
    header = b'\x89PNG\r\n\x1a\n'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Graphics Control Extension (optional)
    if rng.read(1)[0]:
        gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    else:
        gce = None

    # 5. Image Descriptor
    image_descriptor = struct.pack('>BBHHHBB', 0x2C, 0, 0, width, height, rng.read(1)[0], rng.read(1)[0])

    # 6. Image Data
    # First byte is the minimum code size
    lzw_minimum_code_size = rng.read(1)[0]

    # Read all blocks until we get one with size 0
    blocks = []
    while True:
        block_size = rng.read(1)[0]
        if not block_size:
            break
        
        blocks.append(bytes([block_size]) + rng.read(block_size))
        
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything together
    out.write(header + lsd + color_table + (gce or b'') + image_descriptor + image_data + trailer)

