#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Aug 17 20:46:59 2021

@author: zhantao
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
from scipy.optimize import minimize
import matplotlib.pyplot as plt

def f(x):
    return (x[0]**2 + x[1]**2) / 2

def g(x):
    return np.array([np.sin(x[0]) * np.cos(x[1]),
                     np.cos(x[0]) * np.sin(x[1])])

def h(x):
    return np.array([np.exp(-(x[0]**2 + x[1]**2)),
                     np.tanh(x[0] * x[1])])

def dg(x):
    return np.array([[-np.cos(x[0]) * np.sin(x[1]),
                      -np.sin(x[0]) * np.cos(x[1])],
                     [-np.sin(x[0]) * np.sin(x[1]),
                      np.cos(x[0]) * np.cos(x[1])]])

def dh(x):
    return np.array([[-x[0] * np.cos(x[0]) * np.cos(x[1]),
                      -x[1] * np.sin(x[0]) * np.sin(x[1])],
                     [x[0] * np.sin(x[0]) * np.cos(x[1]),
                      x[1] * np.cos(x[0]) * np.sin(x[1])]])

def grad_f(x):
    return g(x).T @ dg(x)

def grad_h(x):
    return h(x).T @ dh(x)

def grad_g(x):
    return g(x).T @ dg(x)

def grad_fg(x):
    return grad_f(x) + grad_g(x)

def grad_fh(x):
    return grad_f(x) + grad_h(x)

def grad_gh(x):
    return grad_g(x) + grad_h(x)

def grad_fgh(x):
    return grad_f(x) + grad_g(x) + grad_h(x)

def grad_fgh_vec(x):
    return np.concatenate((grad_f(x), grad_g(x), grad_h(x)))

def grad_fgh_vec_diff(x):
    return np.concatenate((grad_f(x)-grad_g(x), grad_g(x)-grad_h(x), grad_h(x)-grad_f(x)))

def grad_fgh_vec_sum(x):
    return np.concatenate((grad_f(x)+grad_g(x), grad_g(x)+grad_h(x), grad_h(x)+grad_f(x)))

def grad_fgh_vec_prod(x):
    return np.concatenate((grad_f(x)*grad_g(x), grad_g(x)*grad_h(x), grad_h(x)*grad_f(x)))

def grad_fgh_vec_div(x):
    return np.concatenate((grad_f(x)/grad_g(x), grad_g(x)/grad_h(x), grad_h(x)/grad_f(x)))

def grad_fgh_vec_pow(x):
    return np.concatenate((grad_f(x)**grad_g(x), grad_g(x)**grad_h(x), grad_h(x)**grad_f(x)))

def grad_fgh_vec_log(x):
    return np.concatenate((np.log(grad_f(x))+grad_g(x), np.log(grad_g(x))+grad_h(x), np.log(grad_h(x))+grad