#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=1024, height=768, num_frames=5):
    # GIF header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHB', width, height, 0xF))

    # Global color table (256 colors)
    for i in range(256):
        out.write(bytes([i]))

    # Application extension block
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Graphics control extension block
    gce = b'\x21\xF9\x04\x08\x00\x00\x00\x00\x00'

    # Graphic control extension block
    for _ in range(num_frames):
        # Image descriptor
        out.write(b'\x2C\x00\x00\x00\x00' + struct.pack('<HH', width, height))

        # Local color table (256 colors)
        for i in range(256):
            out.write(bytes([i]))

        # Pixel data
        for y in range(height):
            for x in range(width):
                out.write(bytes([int(rng.read(1)[0]) % 256]))

        # Trailer
        out.write(gce)

    # Trailer
    out.write(b';')
