#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    global_color_table_size = min(2 ** (rng.read(1)[0] + 1), 256)
    global_color_table = rng.read(global_color_table_size * 3)

    # 4. Image Descriptors
    # We will use one image descriptor with a single frame
    idd = struct.pack('>BBHHHBB', 0x2c, 0, 0, width, height, 0, rng.read(1)[0])

    # 5. Local Color Table (same as above)
    local_color_table_size = min(2 ** (rng.read(1)[0] + 1), 256)
    local_color_table = rng.read(local_color_table_size * 3)

    # 6. Image Data
    # The data is just a bunch of random bytes
    image_data = rng.read(width * height)

    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(global_color_table)
    out.write(idd)
    out.write(local_color_table)
    out.write(image_data)
