#!/usr/bin/env python3

import struct
from PIL import Image

# Open the input file and read it into a bytearray
with open("input.gif", "rb") as f:
    data = bytearray(f.read())

# Read the header
header = data[:6]
assert header == b"GIF89a"

# Read the logical screen descriptor
lsd = data[6:10]
width, height = struct.unpack("<HH", lsd[2:6])
print("Width: %i Height: %i" % (width, height))

# Read the global color table
color_table = data[10:746]

# Read the graphics control extension
gce = data[746:758]

# Read the image descriptor
image_descriptor = data[758:764]

# Read the image data
image_data = data[764:-1]

# Create an empty image of the same size as the GIF
im = Image.new('P', (width, height), 0)

# Put all the pixels in the image
pixels = list(struct.iter_unpack("<H", image_data))
for y in range(height):
    for x in range(width):
        im.putpixel((x, y), pixels[(y * width) + x][0])

# Save the image
im.save("output.png", format="PNG")

# Re-encode the image as a GIF
with open("output.gif", "wb") as out:
    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)