#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = struct.unpack('<H', rng.read(2))
    height = struct.unpack('<H', rng.read(2))
    packed = rng.read(1)[0]
    global_color_table_size = (packed >> 7) & 1
    color_resolution = (packed >> 4) & 7
    sort_flag = (packed >> 3) & 1
    global_color_table_sort_index = (packed >> 0) & 7
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]

    # Global color table
    for i in range(2 ** global_color_table_size):
        out.write(rng.read(3))

    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<I', 0))

    # Image descriptor
    left = struct.unpack('<H', rng.read(2))[0]
    top = struct.unpack('<H', rng.read(2))[0]
    width = struct.unpack('<H', rng.read(2))[0]
    height = struct.unpack('<H', rng.read(2))[0]
    packed = rng.read(1)[0]
    local_color_table_size = (packed >> 7) & 1
    interlaced = (packed >> 6) & 1
    sorted = (packed >> 5) & 1
    reserved = (packed >> 4) & 1
    lct_sort_index = (packed >> 0) & 7

    # Local color table
    for i in range(2 ** local_color_table_size):
        out.write(rng.read(3))

    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

