#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    background_color = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBBBB', width, height, packed, background_color, aspect_ratio, rng.read(1)[0])

    # 3. Global Color Table (if present)
    global_color_table_flag = bool(packed & 0x80)
    if not global_color_table_flag:
        return

    num_colors = 2 ** ((packed >> 4) + 1)
    color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Graphics Control Extension (optional)
    gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    image_width, image_height = struct.unpack('>HH', rng.read(4))
    local_color_table_flag = bool(rng.read(1)[0] & 0x80)
    interlaced = bool(rng.read(1)[0] & 0x40)
    sorted = bool(rng.read(1)[0] & 0x20)
    reserved = bool(rng.read(1)[0] & 0x10)
    size_of_local_color_table = (rng.read(1)[0] >> 7) & 0x01
    packed = rng.read(1)[0]
    lct = rng.read(3 * 2 ** ((packed >> 4) + 1))  # Random colors

    # 6. Image Data
    data = rng.read()

    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    out.write(gce)
    out.write(struct.pack('>BBBB', 0x2C, left, top, 0x00))
    out.write(struct.pack('>HHBB', image_width, image_height, packed, 0x00))
    out.write(lct)
    out.write(data)

