#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    out.write(b'GIF89a')
    
    # 2. Logical screen descriptor
    width = int.from_bytes(rng.read(2), 'big', signed=False) % 65536 + 1
    height = int.from_bytes(rng.read(2), 'big', signed=False) % 65536 + 1
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] % 256
    out.write(struct.pack('<HHBB', width, height, background_color_index, pixel_aspect_ratio))
    
    # 3. Global color table
    global_color_table_flag = rng.read(1)[0] & 0x80 != 0
    if global_color_table_flag:
        global_color_table_size = (rng.read(1)[0] & 0x7) + 1
        global_color_table_sort_flag = rng.read(1)[0] & 0x80 != 0
        
        for i in range(global_color_table_size * global_color_table_size):
            red   = rng.read(1)[0] % 256
            green = rng.read(1)[0] % 256
            blue  = rng.read(1)[0] % 256
            
            out.write(struct.pack('<BBB', red, green, blue))
    
    # 4. Graphics control extension
    graphics_control_extension_block = b'\x21\xf9\x04\x01\x00\x00\xff\x00'
    graphics_control_extension_block[3] |= rng.read(1)[0] & 0x01
    graphics_control_extension_block[4] |= rng.read(1)[0] & 0x02
    graphics_control_extension_block[5] |= rng.read(1)[0] & 0x04
    graphics_control_extension_block[6] |= rng.read(1)[0] & 0x08
    graphics_control_extension_block[7] |= rng.read(1)[0] & 0x10
    graphics_control_extension_block[8] |= rng.read(1)[0] & 0x20
    graphics_control_extension_block[9] |= rng.read(1)[0] & 0x40
    graphics_control_extension_block[10] |= rng.read(1)[0] & 0x80
    
    out.write(graphics_control_extension_block)
    
    # 5. Image descriptor
    image_descriptor_block = b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00'
    image_descriptor_block[1] = rng.read(1)[0] % 256
    image_descriptor_block[2] = rng.read(1)[0] % 256
    image_descriptor_block[3] = rng.read(1)[0] % 256
    image_descriptor_block[4] = rng.read(1)[0] % 256
    
    local_color_table_flag = rng.read(1)[0] & 0x80 != 0
    if local_color_table_flag:
        local_color_table_size = (rng.read(1)[0] & 0x7) + 1
        local_color_table_sort_flag = rng.read(1)[0] & 0x80 != 0
        
        for i in range(local_color_table_size * local_color_table_size):
            red   = rng.read(1)[0] % 256
            green = rng.read(1)[0] % 256
            blue  = rng.read(1)[0