#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table
    # We don't use a global color table, so we write an empty one
    gct = b'\x00\x00\x00'
    
    # 4. Image Descriptors
    # We only have one image, so we write one Image Descriptor
    idd = struct.pack('>BBHHHBB', 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
    
    # 5. Local Color Table
    # We don't use a local color table, so we write an empty one
    lct = b'\x00\x00\x00'
    
    # 6. Image Data
    # We just write a bunch of zeros to represent the image data
    imgdata = rng.read(width * height)
    
    # 7. Trailer
    trailer = b';'
    
    # Write everything together
    out.write(header + lsd + gct + idd + lct + imgdata + trailer)
