#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    global_color_table_size = min(rng.read(1)[0] + 1, 256) * 3
    global_color_table = rng.read(global_color_table_size)

    # 4. Image Data
    image_data = []
    while True:
        # Read a byte to see if we should stop
        if not rng.read(1):
            break

        # Get the length of this block
        block_length = rng.read(1)[0] + 1

        # If this is an extension block, skip over it entirely
        if block_length == 0xFF:
            continue

        # Otherwise, read the data
        image_data += [rng.read(block_length)]

    # Write everything together
    out.write(header + lsd + global_color_table + b''.join(image_data))
