#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    flags = rng.read(1)[0] & 0xE0 | 0x1F
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBB', width, height, flags, background))

    # Global color table
    for i in range(2 ** ((flags >> 4) + 1)):
        out.write(rng.read(3))

    # Image descriptor
    left = rng.read(2)[0]
    top = rng.read(2)[0]
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    flags = rng.read(1)[0] & 0xC0 | 0x01
    local_color_table_size = rng.read(1)[0] & 0x07
    interlace = rng.read(1)[0] & 0x40
    sort = rng.read(1)[0] & 0x80
    out.write(struct.pack('<HHB', left, top, width * height))
    out.write(bytes([flags]))
    if local_color_table_size > 0:
        for i in range(2 ** (local_color_table_size + 1)):
            out.write(rng.read(3))

    # Local color table
    for i in range(width * height):
        out.write(bytes([i % 256]))

    # LZW-compressed image data
    out.write(b'\x00\x01')
    while True:
        code = rng.read(1)[0]
        if code == 0:
            break
        out.write(bytes([code]))

    # Trailer
    out.write(b';')
