#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given `width`, `height`, and `pixel_count`.
def generate_random_gif(infile: BinaryIO, out: BinaryIO, width=1024, height=768, pixel_count=100):
    # Write the header.
    out.write('GIF'.encode())
    out.write(struct.pack('<H', 9))
    out.write('\x00\x00\xff\xff\xff\xff'.encode())
    out.write((str(width)+'x'+str(height)).encode() + b'\x00')
    
    # Write the image data.
    for _ in range(pixel_count):
        x = infile.read(1)[0] % width
        y = infile.read(1)[0] % height
        
        r = infile.read(1)[0]
        g = infile.read(1)[0]
        b = infile.read(1)[0]
        
        out.write(struct.pack('BBB', r, g, b))
        out.write(struct.pack('<HH', x, y))

if __name__ == "__main__":
    from sys import stdin, stdout
    with open('/dev/urandom', 'rb') as f:
        generate_random_gif(f, stdout.buffer)