#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Header
    out.write(b'GIF89a')
    
    # Logical Screen Descriptor
    out.write(struct.pack('<HHBB', width, height, 0xF7, 0))
    
    # Global Color Table (16 colors)
    for i in range(256 // 4):
        out.write(bytes([i * 4]))
        
    # Graphic Control Extension
    out.write(b'\x21\xF9\x04\x00\x01\x00\xFF\xFF\xFF\x00')
    
    # Image Descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # Local Color Table (16 colors)
    for i in range(256 // 4):
        out.write(bytes([i * 4]))
    
    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([y % 256]))
            
    # Trailer
    out.write(b'\x3B')
