#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width = rng.read(2)
    height = rng.read(2)
    # Packed fields
    #   - Global Color Table Flag: 0b0
    #   - Color Resolution: 0b00
    #   - Sort Flag: 0b0
    #   - Size of Global Color Table: 0b000
    packed_fields = b'\x00\x00'
    # Background color index
    background_color_index = rng.read(1)
    # Pixel aspect ratio
    pixel_aspect_ratio = rng.read(1)

    # Write everything to the output
    out.write(header + width + height + packed_fields + background_color_index + pixel_aspect_ratio)

    # 3. Global Color Table
    # We don't use one, so we write an empty block
    out.write(struct.pack('<H', 0))

    # 4. Image Data
    # We only support one image in the file
    #   - Left position of image
    left_position = rng.read(2)
    #   - Top position of image
    top_position = rng.read(2)
    #   - Width of image
    width = rng.read(2)
    #   - Height of image
    height = rng.read(2)
    #   - Packed fields
    #     - Local Color Table Flag: 0b0
    #     - Interlace Flag: 0b0
    #     - Sort Flag: 0b0
    #     - Reserved: 0b000
    #     - Size of Local Color Table: 0b000
    packed_fields = b'\x00'
    #   - LZW Minimum Code Size
    lzw_minimum_code_size = rng.read(1)

    # Write everything to the output
    out.write(left_position + top_position + width + height + packed_fields + lzw_minimum_code_size)

    # 5. Local Color Table
    # We don't use one, so we write an empty block
    out.write(struct.pack('<H', 0))

    # 6. Image Data
    # We only support one image in the file
    #   - Length of data
    length = rng.read(2)
    #   - Data
    data = rng.read(length[0] * length[1])

    # Write everything to the output
    out.write(length + data)

    # 7. Trailer
    trailer = b';'

    # Write everything to the output
    out.write(trailer)
