#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width = rng.read(2)
    height = rng.read(2)
    packed = rng.read(1)
    background_color_index = rng.read(1)
    pixel_aspect_ratio = rng.read(1)

    # 3. Global Color Table
    global_color_table_flag = (packed[0] & 0b10000000) >> 7
    color_resolution = (packed[0] & 0b01110000) >> 4
    sort_flag = (packed[0] & 0b00001000) >> 3
    size_of_global_color_table = 2 ** ((packed[0] & 0b00000111) + 1)
    global_color_table = []
    for i in range(size_of_global_color_table):
        red = rng.read(1)[0]
        green = rng.read(1)[0]
        blue = rng.read(1)[0]
        global_color_table.append((red, green, blue))

    # 4. Image Data
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\xff\x00'
    lzw_minimum_code_size = rng.read(1)[0]
    block_size = rng.read(1)[0]
    data = []
    while True:
        byte = rng.read(1)
        if len(byte) == 0:
            break
        data.append(byte[0])

    # Write everything to the output
    out.write(header)
    out.write(width)
    out.write(height)
    out.write(packed)
    out.write(background_color_index)
    out.write(pixel_aspect_ratio)
    out.write(struct.pack('B', global_color_table_flag << 7 | color_resolution << 4 | sort_flag << 3 | size_of_global_color_table))
    for color in global_color_table:
        out.write(bytes([color[0], color[1], color[2]]))
    out.write(image_descriptor)
    out.write(lzw_minimum_code_size)
    out.write(block_size)
    out.write(bytes(data))
