#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # We use a global color table with 256 colors
    gct = rng.read(768)

    # 4. Image Descriptor
    id = struct.pack('>BBHHBB', 0x2C, 0, 0, width, height, 0)

    # 5. Local Color Table
    # We don't use a local color table
    lct = b''

    # 6. Data Sub-Blocks
    # We use a single data sub-block containing a single pixel
    data = struct.pack('>B', 1) + rng.read(1)

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(gct)
    out.write(id)
    out.write(lct)
    out.write(data)
    out.write(trailer)
