#!/usr/bin/env python3
import sys
from random import Random

if len(sys.argv) < 3:
    print("Usage: {} <input> <output>".format(sys.argv[0]), file=sys.stderr)
    exit(1)

with open(sys.argv[1], "rb") as f, open(sys.argv[2], "wb") as out:
    rng = Random()
    rng.seed(f.read())

    # Header
    out.write(b"GIF89a")

    # Global color table
    for i in range(2 ** ((flags >> 4) + 1)):
        out.write(rng.read(3))

    # Image descriptor
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)[0] & 0xC0 | 0x07
    local_color_table_size = (flags >> 4) + 1
    interlaced = bool((flags >> 6) & 1)
    out.write(left + top + width + height + bytes([flags]))

    # Local color table
    for i in range(2 ** local_color_table_size):
        out.write(rng.read(3))

    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

