#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), byteorder='little') % 640 + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % 480 + 1
    flags = rng.read(1)[0] & 7
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]

    logical_screen_descriptor = struct.pack('<HBBB', width, height, flags, background)

    # 3. Global Color Table
    global_color_table = rng.read(768)

    # 4. Image Data
    image_data = rng.read(width * height)

    # Write everything to output
    out.write(header)
    out.write(logical_screen_descriptor)
    out.write(global_color_table)
    out.write(image_data)

if __name__ == "__main__":
    from sys import argv
    if len(argv)!= 2:
        print("Usage: {} <output>".format(argv[0]))
        exit()

    with open('/dev/urandom', 'rb') as f:
        with open(argv[1], 'wb') as o:
            generate_random_gif(f, o)