#!/usr/bin/env python3

import random
import struct
from sys import argv, exit

if __name__ == '__main__':
    if len(argv) != 2:
        print("Usage: {} <output-file>".format(argv[0]))
        exit()

    with open(argv[1], "wb") as out:
        generate_gif(out)

def generate_gif(out):
    rng = random.Random()

    # 1. Header
    header = b"GIF89a"

    # 2. Logical Screen Descriptor
    width = rng.randint(1, 65535)
    height = rng.randint(1, 65535)
    packed_fields = rng.randint(0, 2**4 - 1) << 4 | rng.randint(0, 2**4 - 1)
    background_color_index = rng.randint(0, 2**8 - 1)
    pixel_aspect_ratio = rng.randint(0, 2**8 - 1)
    lsd = struct.pack('<HHBBBB', width, height, packed_fields, background_color_index, pixel_aspect_ratio, 0)

    # 3. Global Color Table
    global_color_table_flag = rng.choice([True, False])
    if global_color_table_flag:
        global_color_table_size = rng.choice([2 ** i for i in range(1, 9)])
        global_color_table = [struct.pack('BBB', *rng.sample(range(256), 3)) for _ in range(global_color_table_size)]
        global_color_table = b''.join(global_color_table)
    else:
        global_color_table = b''

    # 4. Graphic Control Extension
    gce_present = rng.choice([True, False])
    if gce_present:
        delay_time = rng.randint(0, 2**16 - 1)
        transparent_color_index = rng.randint(0, 2**8 - 1)
        disposal_method = rng.randint(0, 2**3 - 1)
        user_input_flag = rng.choice([True, False])
        transparency_flag = rng.choice([True, False])
        gce = struct.pack('BBBBBB', 0xF9, 4, delay_time, transparent_color_index, disposal_method << 2 | user_input_flag << 1 | transparency_flag, 0)
    else:
        gce = b''

    # 5. Image Descriptor
    image_left_position = rng.randint(0, 2**16 - 1)
    image_top_position = rng.randint(0, 2**16 - 1)
    image_width = rng.randint(1, 2**16 - 1)
    image_height = rng.randint(1, 2**16 - 1)
    local_color_table_flag = rng.choice([True, False])
    interlace_flag = rng.choice([True, False])
    sort_flag = rng.choice([True, False])
    image_descriptor = struct.pack('<HHBB', image_left_position, image_top_position, image_width, image_height) \
                       + struct.pack('BB', local_color_table_flag << 7 | interlace_flag << 6 | sort_flag << 5 | 0, 0)

    # 6. Image Data
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while len(blocks) < total_blocks_size:
        block_size = min(total_blocks_size - len(blocks), rng.read(1)[0])
        blocks.append(rng.read(block_size))

    image_data = struct.pack('>BB', lzw_minimum_code_size, 0) + b''.join(blocks)

    # 7. Trailer
    trailer = b'\x3B'

    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
