#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical Screen Descriptor
    width = rng.read(2)[0] + (rng.read(1)[0] << 8)
    height = rng.read(2)[0] + (rng.read(1)[0] << 8)
    packed = rng.read(1)[0]
    background_color_index = packed & 0xff
    aspect_ratio = ((packed >> 7) & 0x7) + 1
    global_color_table_flag = bool((packed >> 6) & 0x1)
    color_resolution = ((packed >> 4) & 0x3) + 1
    sort_flag = bool((packed >> 3) & 0x1)
    size_of_global_color_table = 2 ** ((packed >> 1) & 0x7)

    print('width', width)
    print('height', height)
    print('background_color_index', background_color_index)
    print('aspect_ratio', aspect_ratio)
    print('global_color_table_flag', global_color_table_flag)
    print('color_resolution', color_resolution)
    print('sort_flag', sort_flag)
    print('size_of_global_color_table', size_of_global_color_table)

    out.write(struct.pack('<HBBB', width, height, packed, background_color_index))

    # 3. Global Color Table
    if global_color_table_flag:
        for _ in range(size_of_global_color_table):
            out.write(rng.read(3))

    # 4. Image Data
    while True:
        block_type = rng.read(1)[0]
        if block_type == 0x3b:
            break
        elif block_type == 0x2c:
            # Image Descriptor
            left = rng.read(2)[0] + (rng.read(1)[0] << 8)
            top = rng.read(2)[0] + (rng.read(1)[0] << 8)
            width = rng.read(2)[0] + (rng.read(1)[0] << 8)
            height = rng.read(2)[0] + (rng.read(1)[0] << 8)
            packed = rng.read(1)[0]
            local_color_table_flag = bool((packed >> 7) & 0x1)
            interlace_flag = bool((packed >> 6) & 0x1)
            sort_flag = bool((packed >> 5) & 0x1)
            reserved = bool((packed >> 4) & 0x1)
            size_of_local_color_table = 2 ** ((packed >> 1) & 0x7)

            print('left', left)
            print('top', top)
            print('width', width)
            print('height', height)
            print('local_color_table_flag', local_color_table_flag)
            print('interlace_flag', interlace_flag)
            print('sort_flag', sort_flag)
            print('reserved', reserved)
            print('size_of_local_color_table', size_of_local_color_table)

            out.write(struct.pack('<HHBBB', left, top, width, height, packed))

            # Local Color Table
            if local_color_table_flag:
                for _ in range(size_of_local_color_table):
                    out.write(rng.read(3))

            # LZW Minimum Code Size
            lzw_minimum_code_size = rng.read(1)[0]
            out.write(lzw_minimum_code_size.to_bytes(1, 'big'))

            # Image Data
            while True:
                code_size = rng.read(1)[0]
                if code_size == 0:
                    break
                else:
                    out.write(