#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical screen descriptor
    width = int.from_bytes(rng.read(2), 'big') % 65536
    height = int.from_bytes(rng.read(2), 'big') % 65536
    packed = (0b111 << 4) | (width >> 7)
    lsd = bytes([packed]) + bytes([width & 0xFF]) + bytes([height >> 7]) + bytes([height & 0xFF]) + b'\xF0\x00\xFF'

    # 3. Color table
    color_table = rng.read(3 * 2**((packed & 0b111) + 1))

    # 4. Graphic control extension
    gce = b'\x21\xF9\x04\x08\x00\x00\x00\x00\x00'

    # 5. Image descriptor
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00'

    # 6. Image data
    image_data = rng.read(width*height)
    # 7. Trailer
    trailer = b'\x3B'

    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)
