#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = [256, 1 + (rng.read(1)[0] % 7)]
    height = [256, 1 + (rng.read(1)[0] % 7)]
    flags = b'\x80\x00'  # Global color table flag and size of global color table
    bg_color = bytes([rng.read(1)[0]]) * 3
    aspect_ratio = b'\x00'
    out.write(struct.pack('<HHBBB',
                         width[0],
                         height[0],
                         flags[0],
                         flags[1],
                         bg_color[0],
                         bg_color[1],
                         bg_color[2]))
    out.write(aspect_ratio)

    # Global color table
    colors = []
    for i in range(pow(2, int(flags[1]))):
        colors += [bytes([rng.read(1)[0]]) * 3]
    for c in colors:
        out.write(c)

    # Image descriptor
    left = bytes([rng.read(1)[0]])
    top = bytes([rng.read(1)[0]])
    flags = b'\x00\x00'  # Local color table flag and size of local color table
    lct = None
    if flags[1]:
        lct = []
        for i in range(pow(2, int(flags[1]))):
            lct += [bytes([rng.read(1)[0]]) * 3]
        for c in lct:
            out.write(c)
    width = [width[1], 1 + (rng.read(1)[0] % 7)]
    height = [height[1], 1 + (rng.read(1)[0] % 7)]
    flags = b'\x80\x00'  # Interlace flag and sort flag
    bg_color = bytes([rng.read(1)[0]]) * 3
    pixel_aspect_ratio = b'\x00'
    out.write(struct.pack('<HHBBB',
                         left[0],
                         top[0],
                         width[0],
                         height[0],
                         flags[0],
                         flags[1],
                         bg_color[0],
                         bg_color[1],
                         bg_color[2]))
    out.write(pixel_aspect_ratio)

    # Local color table
    if lct is not None:
        for c in lct:
            out.write(c)

    # Table-based image data
    interlaced = bool(flags[0])
    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

