#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    width = [rng.read(1)[0] % 256 + 1,
             rng.read(1)[0] % 256 + 1]

    height = [rng.read(1)[0] % 256 + 1,
              rng.read(1)[0] % 256 + 1]

    flags = (rng.read(1)[0] & 0x7) | ((rng.read(1)[0] & 0x4) << 1)

    if flags == 0 or flags == 1:
        # Plain color block
        palette = []

        for i in range(256):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]

            palette += bytes((r, g, b))

        out.write(struct.pack('<HHBBB', *width, *height,
                              0xf7, len(palette), *palette))
    elif flags == 2 or flags == 3:
        # LZW compression
        lzw_mincodesize = rng.read(1)[0] % 256 + 1

        out.write(struct.pack('<HHBBB', *width, *height,
                              lzw_mincodesize, 0, 0))

        # TODO: Implement LZW compression
        raise NotImplementedError()
    elif flags == 4 or flags == 5:
        # Interlaced image
        interlace = bool(rng.read(1)[0])

        out.write(struct.pack('<HHBBB?', *width, *height,
                              0, 0, False, interlace))

        for y in range(height[0], height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

