#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + int.from_bytes(rng.read(1), 'little') % 256
    height = 1 + int.from_bytes(rng.read(1), 'little') % 256
    flags = b'\x07\x00' if (width == 1 and height == 1) else b'\x00\x00'
    bg_color = bytes([int.from_bytes(rng.read(1), 'little')]) * 3
    pixel_aspect_ratio = b'\x00'
    out.write(struct.pack('<HHBBB',
                         width, height,
                         *flags,
                         *bg_color,
                         *pixel_aspect_ratio))

    # Global color table
    global_color_table_size = min(
        2 ** ((int.from_bytes(rng.read(1), 'little')) & 0x07) + 1,
        2 ** (((int.from_bytes(rng.read(1), 'little')) >> 4) & 0x07) + 1)
    global_color_table = [bytes([int.from_bytes(rng.read(1), 'little')] * 3)
                          for _ in range(global_color_table_size)]
    out.write(struct.pack('B', global_color_table_size))
    out.write(b''.join(global_color_table))

    # Image descriptor
    left = 0
    top = 0
    width = 1 + int.from_bytes(rng.read(1), 'little') % 256
    height = 1 + int.from_bytes(rng.read(1), 'little') % 256
    flags = b'\x08\x00'
    local_color_table_flag = b'\x00'
    interlace_flag = b'\x00'
    sort_flag = b'\x00'
    reserved = b'\x00\x00'
    local_color_table_size = 0
    local_color_table = []
    image_data = bytearray()
    while len(image_data) < width * height:
        image_data += rng.read((width - len(image_data)) // 8)
    out.write(struct.pack('<HHHHBBBBB',
                         left, top,
                         width, height,
                         *flags,
                         *local_color_table_flag,
                         *interlace_flag,
                         *sort_flag,
                         *reserved,
                         local_color_table_size))
    out.write(b''.join(local_color_table))
    out.write(image_data)

    # Trailer
    out.write(b';')


if