#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    width, height = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    background_color = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBBBB', width, height, packed, background_color, aspect_ratio, rng.read(1)[0])

    # 3. Global Color Table (if present)
    global_color_table_flag = bool(packed & 0x80)
    if not global_color_table_flag:
        return

#    print("Global color table flag is set")
    global_color_table_size = ((packed >> 7) + 1) * 3
    global_color_table = bytearray()
    for i in range(global_color_table_size):
        global_color_table += rng.read(1)

    # 4. Image Descriptor
    image_separator = b'\x2C'
    left, top = struct.unpack('>HH', rng.read(4))
    packed = rng.read(1)[0]
    local_color_table_flag = bool(packed & 0x80)
    interlace_flag = bool(packed & 0x40)
    sort_flag = bool(packed & 0x20)
    reserved = rng.read(1)
    size_of_local_color_table = ((packed >> 7) + 1) * 3
    local_color_table = bytearray()
    for i in range(size_of_local_color_table):
        local_color_table += rng.read(1)
    lzw_minimum_code_size = rng.read(1)[0]

    # 5. LZW Data
    data_subblock_size = rng.read(1)[0]
    data_subblocks = []
    while True:
        subblock = rng.read(data_subblock_size)

        # If this is an empty block, we're done!
        if len(subblock) == 0 or subblock[0] == 0:
            break

        data_subblocks.append(subblock)

    # 6. Trailer
    trailer = b';'

    # Write everything to the output stream
    out.write(header)
    out.write(lsd)
    out.write(global_color_table)
    out.write(image_separator)
    out.write(struct.pack('>HHBBBBB', left, top, packed, lzw_minimum_code_size, data_subblock_size, 0))
    for subblock in data_subblocks:
        out.write(subblock)
    out.write(trailer)


if