#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header (6 bytes)
    # 2. Logical Screen Descriptor (7 bytes)
    # 3. Global Color Table (3 * 256 = 768 bytes)

    # Write header
    out.write(b'GIF')
    out.write(struct.pack('<H', 0))
    out.write(b'\x87\x00\x00')

    # Write logical screen descriptor
    width = rng.read(1)[0] % 49 + 1
    height = rng.read(1)[0] % 49 + 1
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Write global color table
    for i in range(0, 256):
        r = rng.read(1)[0] & 0xFF
        g = rng.read(1)[0] & 0xFF
        b = rng.read(1)[0] & 0xFF

        out.write(struct.pack('BBB', r, g, b))


if