#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(image_data, out):
    """Generate a random GIF using the given data and write it to `out`."""

    # Header
    header = b'GIF89a'

    # Logical Screen Descriptor
    width = len(image_data) // 256
    height = (len(image_data) - (width * 256)) // 2
    lsd = bytes([
        0b10000000 | ((height & 0b11111111) << 4),   # Height
        width & 0xFF,                               # Width
        0b10000000 | (((width >> 8) & 0b11111111)), # Width extension bits
        0b10000000 | ((height >> 8) & 0b11111111),  # Height extension bits
        0b10000000                                # Global Color Table Flag
    ])

    # Global Color Table
    color_table = bytearray()
    for i in range(256):
        color_table += struct.pack('BBB', *(i % 256,) * 3)

    # Graphics Control Extension
    gce = b''

    # Image Descriptor
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00,\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0
    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)


if __name__ == '__main__':
    from os import urandom as get_random_bytes
    with open('/tmp/test.gif', 'wb') as f:
        generate_random_gif(get_random_bytes(), f)