#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')
    width = [256, 4]
    height = [256, 4]
    flags = b'\x87\x00'
    background = bytes([rng.read(1)[0]]) * 3
    aspect = bytes([rng.read(1)[0]]) * 3
    out.write(struct.pack('<HHB', width[0], height[0], ord(' ')))
    out.write(flags + background + aspect)

    # Global color table
    global_color_table_size = (ord(' ') & 7) + 1
    out.write(global_color_table_size.to_bytes(1, byteorder='little'))
    for i in range(pow(2, global_color_table_size)):
        out.write((i).to_bytes(1, byteorder='little'))
        out.write(rng.read(3))

    # Image descriptor
    left = 0
    top = 0
    width[1] += left
    height[1] += top
    packed = ((left & 0xF8) << 8) | \
             ((top & 0xFE) << 9) | \
             (((width[1] - 1) & 0xFF) << 1) | \
             (((height[1] - 1) & 0xFF) >> 7)
    out.write(packed.to_bytes(2, byteorder='big'))
    local_color_table_flag = False
    interlaced = True
    sorted_color_table_flag = False
    local_color_table_size = 0
    out.write(((local_color_table_flag << 7) |
               (interlaced << 6) |
               (sorted_color_table_flag << 5) |
               (local_color_table_size)).to_bytes(1, byteorder='little'))
    if local_color_table_flag:
        for i in range(pow(2, local_color_table_size)):
            out.write((i).to_bytes(1, byteorder='little'))
            out.write(rng.read(3))

    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

