#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    flags = rng.read(1)[0] & 0xE0 | 0x1F
    background = rng.read(1)[0]
    aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBB', width, height, flags, background))
    out.write(aspect_ratio)

    # Global color table
    global_color_table_size = 2 ** ((flags >> 4) & 7)
    for i in range(global_color_table_size):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(r)
        out.write(g)
        out.write(b)

    # Image descriptor
    left = rng.read(2)[0]
    top = rng.read(2)[0]
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    flags = rng.read(1)[0] & 0xE0 | 0x0C
    local_color_table_flag = flags & 0x80!= 0
    interlace_flag = flags & 0x40!= 0
    sort_flag = flags & 0x20!= 0
    reserved_bits = flags & 0x10!= 0
    lct_size = 2 ** ((flags >> 4) & 7)
    out.write(struct.pack('<HHHHB',
                         left, top, width, height, flags))
    if local_color_table_flag:
        for i in range(lct_size):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(r)
            out.write(g)
            out.write(b)
    else:
        pass
    if interlace_flag or sort_flag or reserved_bits:
        raise ValueError('Unsupported image flags')
    else:
        pass

