#!/usr/bin/env python3
import sys, random
from PIL import Image

# Read input image and convert to indexed PNG
img = Image.open(sys.argv[1]).convert('RGB')
out = open("input.png", "wb")
img.save(out, format="PNG", optimize=True)
out.close()

# Open file as binary stream
with open("input.png", 'rb+') as f:
    rng = random.Random(f.read())

    # Header
    signature = rng.read(8)
    chunk_length = rng.read(4)
    chunk_type = rng.read(4)
    out.write(signature + chunk_length + chunk_type)

    # Logical screen descriptor
    width = rng.read(2)
    height = rng.read(2)
    bitdepth = rng.read(1)[0] & 0x0F
    color_type = rng.read(1)[0] & 0xF0 | 0x03
    compression = rng.read(1)[0]
    filtermethod = rng.read(1)[0]
    interlace = rng.read(1)[0]
    out.write(width + height + bytes([bitdepth]) + bytes([color_type]) +
              bytes([compression]) + bytes([filtermethod]) + bytes([interlace]))

    # Global color table
    for i in range(2 ** ((flags >> 4) + 1)):
        out.write(rng.read(3))

    # Image descriptor
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    flags = rng.read(1)[0] & 0xC0 | 0x07
    local_color_table_size = (flags >> 4) + 1
    interlaced = bool((flags >> 6) & 1)
    out.write(left + top + width + height + bytes([flags]))

    # Local color table
    for i in range(2 ** local_color_table_size):
        out.write(rng.read(3))

    # Pixel data
    if not interlaced:
        for y in range(height[1]):
            for x in range(width[1]):
                out.write(bytes([rng.read(1)[0]]))
    else:
        pass

