#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# the given width and height.
def generate_gif(out: BinaryIO, rng: BinaryIO, width: int, height: int):
    # Write header
    out.write(b'GIF89a')  # Magic bytes
    out.write(struct.pack('<H', width))  # Width
    out.write(struct.pack('<H', height))  # Height
    out.write(b'\x00\x00')  # Background color index
    out.write(b'\x00')  # Pixel aspect ratio

    # Write global color table (16 colors)
    for i in range(7):
        out.write(struct.pack('B', 255 * (i % 2)))
        out.write(struct.pack('B', 0))
        out.write(struct.pack('B', 0))
    for i in range(9):
        out.write(struct.pack('B', 255 * ((i + 1) % 2)))
        out.write(struct.pack('B', 255 * ((i + 1) % 2)))
        out.write(struct.pack('B', 255 * ((i + 1) % 2)))

    # Write image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff\xff')

    # Write local color table (16 colors)
    for i in range(7):
        out.write(struct.pack('B', 255 * (i % 2)))
        out.write(struct.pack('B', 0))
        out.write(struct.pack('B', 0))
    for i in range(9):
        out.write(struct.pack('B', 255 * ((i + 1) % 2)))
        out.write(struct.pack('B', 255 * ((i + 1) % 2)))
        out.write(struct.pack('B', 255 * ((i + 1) % 2)))

    # Write LZW-compressed data
    out.write(b'\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0