#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # 3. Global Color Table (let's make it have random size, capped at 256 colors, and fully random)
    num_colors = (rng.read(1)[0] % 256) or 1
    color_table = rng.read(3 * num_colors)  # Random colors
    
    # 4. Image Data
    # We'll use a single image with no local color table
    # First we need to write the Image Separator
    out.write(b'\x2c')
    
    # Then we need to write the Image Descriptor
    idd = struct.pack('>HHBBB', 0, 0, width, height, 0)
    out.write(idd)
    
    # Now we can start writing the image data itself
    # Let's just do one big block of raw RGB data
    out.write(color_table + rng.read(width * height * 3))
    
    # 5. Trailer
    out.write(b';')
