#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # Color table (256 colors, 3 bytes per color)
    color_table = rng.read(3 * 256)

    # 3. Graphic Control Extension
    gce = struct.pack('BBBHBB', 0xF9, 0x04, 0x08, 0, 0, 0)

    # 4. Image Descriptor
    id = struct.pack('BBHHHBB', 0x2C, 0, 0, width, height, 0)

    # 5. Local Color Table
    local_color_table = color_table[:3*2]

    # 6. Image Data
    image_data = rng.read(width * height // 2)

    # 7. Trailer
    trailer = b'\x3b'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + id + local_color_table + image_data + trailer)

