#!/usr/bin/env python3

import struct
from typing import BinaryIO, List, Tuple

from .common import *

# https://github.com/matthew-matvei/pycryptodome/blob/master/lib/Crypto/Util/number.py
def bytes_to_long(data: bytes) -> int:
    return int.from_bytes(data, byteorder='big', signed=False)

# https://github.com/matthew-matvei/pycryptodome/blob/master/lib/Crypto/Util/number.py
def long_to_bytes(n: int, length: int = 0) -> bytes:
    if n < 0:
        raise ValueError("Negative numbers can't be converted to unsigned")
    if not isinstance(length, int):
        raise TypeError('An integer is required')

    # Pad front with zeroes if needed
    if length > 0 and len(hex(n)[2:]) & 1:
        hexstr = '0{}'.format(hex(n))
    else:
        hexstr = hex(n)

    # Strip the "L" from -0x...
    hexstr = hexstr.rstrip('L')

    # Add padding back if needed
    hexstr = hexstr.zfill(length * 2)

    return bytes.fromhex(hexstr)
