#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    packed_fields = rng.read(1)[0] & 0x7F
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] % 256

    # Write everything to the output
    out.write(header + struct.pack('<HBBB', width, height, packed_fields, background_color_index) + pixel_aspect_ratio.to_bytes(1, 'big'))

    # 3. Global Color Table
    # We don't need it, so we just write an empty block
    out.write(b'\x00\x00')

    # 4. Image Data
    # We will write one frame with no local color table
    out.write(b'\x2C\x00\x00\x00\x00\x00\xFF\xFF\xFF\x00\x00\x00\x21\xF9\x04\x01\x00\x00\x00\x00\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;')

    # 5. Trailer
    out.write(b';')
