#!/usr/bin/env python3

import struct
from typing import List, Optional, Tuple

from . import (
    _pack_uint8,
    _unpack_uint16,
    _unpack_uint24,
    _unpack_uint32,
    _unpack_uint8,
)
from .constants import (
    BLE_GAP_ADV_TYPE_COMPLETE_LOCAL_NAME,
    BLE_GAP_ADV_TYPE_INCOMPLETE_LOCAL_NAME,
    BLE_GAP_ADV_TYPE_SHORT_LOCAL_NAME,
    BLE_GAP_ADV_TYPE_SERVICE_DATA,
    BLE_GAP_ADV_TYPE_SERVICE_UUID_128BIT_MORE_AVAILABLE,
    BLE_GAP_ADV_TYPE_SERVICE_UUID_128BIT_COMPLETE,
    BLE_GAP_ADV_TYPE_SERVICE_UUID_COMPLETE,
    BLE_GAP_ADV_TYPE_SERVICE_UUID_INCOMPLETE,
    BLE_GAP_ADV_TYPE_SLAVE_CONNECTION_INTERVAL_RANGE,
    BLE_GAP_ADV_TYPE_TARGET_DIRECTED,
    BLE_GAP_ADV_TYPE_URI,
)
from .types import AdvertisementDataType, ServiceUuid

# https://www.bluetooth.com/specifications/assigned-numbers/generic-access-profile/
BASIC_TYPES = {
    0x01: "Flags",
    0x02: "Incomplete list of 16-bit service UUIDs",
    0x03: "Complete list of 16-bit service UUIDs",
    0x04: "Incomplete list of 32-bit service UUIDs",
    0x05: "Complete list of 32-bit service UUIDs",
    0x06: "Incomplete list of 128-bit service UUIDs",
    0x07: "Complete list of 128-bit service UUIDs",
    0x08: "Shortened local name",
    0x09: "Complete local name",
    0x0A: "TX power level",
    0x0D: "Class of device",
    0x0E: "Simple pairing hash C-192",
    0x0F: "Simple pairing randomizer R-192",
    0x10: "Device ID",
    0x11: "Security manager TK value",
    0x12: "Security manager Out of Band flags",
    0x14: "Slave connection interval range",
    0x15: "List of 16-bit service solicitation UUIDs",
    0x16: "List of 128-bit service solicitation UUIDs",
    0x17: "Service data - 16-bit UUID",
    0x18: "Public target address",
    0x19: "Random target address",
    0x1A: "Appearance",
    0x1B: "Advertising interval",
    0x1C: "LE bluetooth device address",
    0x1D: "LE role",
    0x1E: "Simple pairing hash C-256",
    0x1F: "Simple pairing randomizer R-256",
    0x20: "List of 32-bit service solicitation UUIDs",
    0x21: "Service data - 32-bit UUID",
    0x22: "Service data - 128-bit UUID",
    0x23: "LE secure connections confirmation value",
    0x24: "LE secure connections random value",
    0x25: "URI",
    0x26: "Indoor positioning",
    0x27: "Transport Discovery Data",
    0x28: "Le Supported Features",
    0x29: "Channel Map Update Indication",
    0x2A: "PB-ADV",
    0x2B: "Mesh Message",
    0x2C: "Mesh Beacon",
    