#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng.read(2)[0] & 0xff
    height = 1 + rng.read(2)[0] & 0xff
    background_color_index = rng.read(1)[0] & 0xff
    pixel_aspect_ratio = rng.read(1)[0] & 0xff
    out.write(struct.pack('<HHBB', width, height, background_color_index, pixel_aspect_ratio))

    # Global color table (16 colors)
    for _ in range(3 * 16):
        out.write(rng.read(1))

    # Graphic control extension
    block_size = 4
    extension_introducer = 0x21
    graphic_control_label = 0xf9
    disposal_method = 0x02
    user_input_flag = 0x00
    transparent_color_flag = 0x01
    delay_time = rng.read(2)[0] & 0xffff
    transparent_color_index = rng.read(1)[0] & 0xff
    block_terminator = 0x00
    out.write(struct.pack('<BBBHBBB', block_size, extension_introducer, graphic_control_label, disposal_method, delay_time, transparent_color_flag, transparent_color_index, block_terminator))

    # Image descriptor
    left_position = rng.read(2)[0] & 0xffff
    top_position = rng.read(2)[0] & 0xffff
    width = rng.read(2)[0] & 0xffff
    height = rng.read(2)[0] & 0xffff
    local_color_table_flag = 0x00
    interlace_flag = 0x00
    sort_flag = 0x00
    reserved = 0x00
    out.write(struct.pack('<HHHHBBBB', left_position, top_position, width, height, local_color_table_flag, interlace_flag, sort_flag, reserved))

    # Local color table (16 colors)
    for _ in range(3 * 16):
        out.write(rng.read(1))

    # LZW minimum code size
    lzw_minimum_code_size = rng.read(1)[0] & 0xff
    out.write(lzw_minimum_code_size.to_bytes(1, byteorder='big'))

    # Pixel data
    while True:
        # Read a single byte from the RNG and write it to the output
        b = rng.read(1)[0] & 0xff
        out.write(b.to_bytes(1, byteorder='big'))

        if b == 0x00:
            break
