#!/usr/bin/env python3

import struct
from typing import BinaryIO


def encode(infile: str, outfile: str):
    """Encode an image into GIF format"""

    # Open input and output files
    inp = open(infile, 'rb')
    out = open(outfile, 'wb')

    # Read the header
    magic = inp.read(4)
    width, height = struct.unpack('<LL', inp.read(8))

    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor (LSD)
    lsd = struct.pack('<HHBB', width, height, 0xF, 0)

    # 3. Global Color Table
    color_table = b'\xFF\xFF\xFF' * 2**(1+lzw_minimum_code_size)

    # 4. Graphic Control Extension (GCE)
    gce = b'\x21\xF9\x04\x08\x00\x00\x00\x00\x00'

    # 5. Image Descriptor
    image_descriptor = b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00'

    # 6. Image Data
    rng = inp.read()
    blocks = []
    while True:
        data = bytes([block_size]) + rng.read(block_size)
        if len(data) < 255:
            data += b'\x00'  # Ending with a block size of 0
        elif len(data) > 255:
            data = bytes([len(data)]) + data[1:]
        blocks.append(data)

    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks)

    # 7. Trailer
    trailer = b'\x3B'

    # Write everything to the output file
    out.write(header + lsd + color_table + gce + image_descriptor + image_data + trailer)