#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('>HHBHB', width, height, rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

    # 3. Global Color Table
    # We don't use it so we write an empty one
    gct = b'\x00\x00\x00'

    # 4. Image Descriptors
    # We only have one image, so we just write it once
    idd = struct.pack('>BBHHBB', 0x2c, 0, 0, width, height, 0)

    # 5. Local Color Table
    # We don't use it so we write an empty one
    lct = b'\x00\x00\x00'

    # 6. Image Data
    # We don't use it so we write an empty one
    data = b''

    # Write everything to the output file
    out.write(header + lsd + gct + idd + lct + data)

