#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=1024, height=768, num_frames=5):
    # Write the header
    out.write(b'GIF89a')  # Magic bytes
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\xF0\x00\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF')

    # Write each frame
    for i in range(num_frames):
        # Write the graphic control extension
        out.write(b'\x21\xF9\x04\x00\x00\x00\x00\x00')

        # Write the image descriptor
        out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00')

        # Write the pixel data
        for y in range(height):
            for x in range(width):
                out.write(struct.pack('B', rng.read(1)[0]))

        # Write the trailer
        out.write(b';')

