#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Packed fields
    packed = 0x80 | ((width >> 7) & 0b1111) << 4 | (height >> 7)

    # Background color index
    background_color_index = rng.read(1)[0] % 256

    # Pixel aspect ratio
    pixel_aspect_ratio = 0

    # Write all this to the output
    out.write(header + struct.pack('<HBBB', width, height, packed, background_color_index, pixel_aspect_ratio))

    # 3. Global Color Table
    # We don't need it because we use local color tables

    # 4. Image Data
    while True:
        # Image Separator
        out.write(b'\x2c')

        # Local Image Descriptor
        # Random width and height (using two bytes for each)
        width, height = struct.unpack('>HH', rng.read(4))

        # Packed fields
        packed = 0x80 | ((width >> 7) & 0b1111) << 4 | (height >> 7)

        # LZW Minimum Code Size
        lzw_minimum_code_size = rng.read(1)[0] % 12

        # Write all this to the output
        out.write(struct.pack('<HBB', width, height, packed, lzw_minimum_code_size))

        # 5. Local Color Table
        # We don't need it because we use global color table

        # 6. Image Data
        # We just write zeros here
