#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    packed_fields = rng.read(1)[0]
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    lsd = struct.pack('>HHBHB', width, height, packed_fields, background_color_index, pixel_aspect_ratio)

    # 3. Global Color Table
    global_color_table_size = 7 + rng.read(1)[0] * 3
    global_color_table = rng.read(global_color_table_size)

    # 4. Image Data
    image_data = []
    while True:
        # Read a single-byte length field
        length = rng.read(1)[0]
        if length == 0xFF:
            break
        # Read that many bytes of data
        image_data += [rng.read(length)]

    # 5. Trailer
    trailer = b'\x3b'

    # Write everything to the output stream
    out.write(header)
    out.write(lsd)
    out.write(global_color_table)
    for chunk in image_data:
        out.write(struct.pack('B', len(chunk)))
        out.write(chunk)
    out.write(trailer)
