#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    width = rng.read(2)[0] + 1
    height = rng.read(2)[0] + 1
    color_table_size = 7
    flags = 0x80 | (color_table_size << 4)
    background_color = rng.read(1)[0] % 256
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBBB', width, height, flags, background_color, pixel_aspect_ratio))

    # Global color table
    for _ in range(2 ** color_table_size):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))

    # Image descriptor
    left = rng.read(2)[0] % width
    top = rng.read(2)[0] % height
    width = rng.read(2)[0] % (width - left) + 1
    height = rng.read(2)[0] % (height - top) + 1
    flags = 0x80
    local_color_table_size = 0
    interlaced = False
    out.write(struct.pack('<HHBBB', left, top, width, height, flags))

    # Local color table
    for _ in range(2 ** local_color_table_size):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))

    # LZW minimum code size
    lzw_minimum_code_size = rng.read(1)[0] % 9 + 1
    out.write(lzw_minimum_code_size.to_bytes(1, byteorder='big'))

    # Pixel data
    while True:
        code = rng.read(1)[0] % 256
        if code == 0xFF:
            break
        else:
            out.write(code.to_bytes(1, byteorder='big'))

if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rng:
        with open('./test.gif', 'wb') as f:
            generate_random_gif(rng, f)