#!/usr/bin/env python3

import struct
from typing import BinaryIO

def read_int(f: BinaryIO) -> int:
    return struct.unpack('>i', f.read(4))[0]

def write_int(f: BinaryIO, i: int):
    f.write(struct.pack('>i', i))

def read_float(f: BinaryIO) -> float:
    return struct.unpack('>f', f.read(4))[0]

def write_float(f: BinaryIO, fv: float):
    f.write(struct.pack('>f', fv))

def read_string(f: BinaryIO) -> str:
    s = b''
    while True:
        c = f.read(1)
        if not c or c == b'\x00':
            break
        s += c
    return s.decode()

def write_string(f: BinaryIO, s: str):
    f.write(s.encode())
    f.write(b'\x00')
