#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('<BBB', *struct.unpack('>BBB', rng.read(3))) + rng.read(1)

    # 3. Global Color Table
    # We use a global color table with 256 colors
    gct = rng.read(768)

    # 4. Image Descriptors
    # We use one image descriptor per frame
    idd = struct.pack('<HHHBB', width, height, 0, 0, 0)

    # 5. Local Color Table
    # We use no local color table
    lct = b'\x00'

    # 6. Image Data
    # We use a single data sub-block that is as long as the image
    data = rng.read(width*height)

    # 7. Trailer
    trailer = b';'

    # Write everything to the output stream
    out.write(header+lsd+gct+idd+lct+data+trailer)

