
import struct
from random import Random

def generate_gif(out):
    """Generate a GIF image with random contents."""
    
    rng = Random()
    
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    width = rng.randint(1, 400)
    height = rng.randint(1, 400)
    global_color_table_flag = rng.choice([True, False])
    color_resolution = rng.choice([1, 2, 4, 8])
    sort_flag = rng.choice([True, False])
    global_color_table_size = rng.choice([2 ** i for i in range(1, 9)])
    background_color_index = rng.randint(0, global_color_table_size - 1)
    pixel_aspect_ratio = rng.choice([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15])
    
    lsd = struct.pack('<HHBBBBBB', width, height, global_color_table_flag, color_resolution, sort_flag, global_color_table_size, background_color_index, pixel_aspect_ratio)
    
    # 3. Global Color Table
    color_table = rng.bytes(global_color_table_size * 3)
    
    # 4. Graphics Control Extension
    gce = None
    if rng.choice([True, False]):
        delay_time = rng.randint(0, 65535)
        transparent_color_flag = rng.choice([True, False])
        transparent_color_index = rng.randint(0, global_color_table_size - 1)
        disposal_method = rng.choice([0, 1, 2, 3])
        user_input_flag = rng.choice([True, False])
        gce = struct.pack('!BBBBB', 0xF9, 4, delay_time, transparent_color_flag << 1 | disposal_method, transparent_color_index, user_input_flag)
    
    # 5. Image Descriptor
    local_color_table_flag = rng.choice([True, False])
    interlace_flag = rng.choice([True, False])
    sort_flag = rng.choice([True, False])
    reserved_bit = rng.choice([True, False])
    local_color_table_size = rng.choice([2 ** i for i in range(1, 9)])
    x = rng.randint(0, width - 1)
    y = rng.randint(0, height - 1)
    width = rng.randint(1, width - x)
    height = rng.randint(1, height - y)
    
    image_descriptor = struct.pack('!BBBBBBHBB', 0x2C, local_color_table_flag, interlace_flag, sort_flag, reserved_bit, local_color_table_size, x, y, width, height)
    
    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1
    
    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while total_blocks_size > 0:
        block_size = min(total_blocks_size, rng.read(1)[0] + 1)
        blocks.append(rng.read(block_size))
        
        total_blocks_size -= block_size
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    if gce is not None:
        out.write(gce)
    out.write(image_descriptor)
    for block in blocks:
        out.write(struct.pack('B', len(block)))
        out.write(block)
    out.write(trailer)