#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1. Header
    header = b'GIF89a'
    
    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))
    
    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    lsd = struct.pack('<BBB', *struct.unpack('>BBB', rng.read(3))) + rng.read(1)
    
    # 3. Global Color Table
    # We use a global color table with 256 colors
    gct = rng.read(768)
    
    # 4. Application Extension Block
    # We use an application extension block to store some metadata about the image
    appext = struct.pack('<BBH', 0xFF, 0xFE, 0x000C) \
                + 'NetHack'.encode() \
                + struct.pack('<H', 0x0100) \
                + rng.read(1)
    
    # 5. Image Descriptor
    # We use a single image with no local color table
    idesc = struct.pack('<BBHHBB', 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00)
    
    # 6. Image Data
    # We just write a bunch of random data
    imgdata = rng.read(width*height)
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything together
    out.write(header + lsd + gct + appext + idesc + imgdata + trailer)
