#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> int:
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    w = rng.read(1)[0] % 65 + 10
    h = rng.read(1)[0] % 65 + 10
    out.write(struct.pack('<HH', w, h))
    # Global Color Table Flag set to True
    out.write(b'\x87\x00')
    # Background color index is always zero
    out.write(b'\x00')
    # Pixel aspect ratio is fixed at 4:3
    out.write(b'\x00')

    # 3. Image Descriptor
    # Left position of image is always zero
    out.write(b'\x00\x00')
    # Top position of image is always zero
    out.write(b'\x00\x00')
    # Width of image is randomly chosen from [w-1..w+1]
    out.write(struct.pack('B', rng.read(1)[0] % (w*2 - 1) + w - 1))
    # Height of image is randomly chosen from [h-1..h+1]
    out.write(struct.pack('B', rng.read(1)[0] % (h*2 - 1) + h - 1))
    # Local Color Table Flag set to False
    out.write(b'\x00')
    # LZW minimum code size is fixed at 2
    out.write(b'\x02')

    # 4. Image Data
    # Write all pixels in the image data section
    # Each pixel is written as an unsigned byte
    for _ in range(w * h):
        out.write(bytes([rng.read(1)[0]]))

    return len(b'GIF89a') + \
           len(struct.pack('<HH', w, h)) + \
           len(b'\x87\x00') + \
           len(b'\x00') + \
           len(b'\x00') + \
           len(struct.pack('<HHBBBBBB',
                           0, 0,
                           w, h,
                           0,
                           0,
                           0,
                           2)) + \
           len(b'\x00') + \
           len(b'\x02') + \
           1 + w * h

