#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a'

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    packed_fields = rng.read(1)[0]
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]

    lsd = struct.pack('>HHBBB', width, height, packed_fields, background_color_index, pixel_aspect_ratio)

    # 3. Global Color Table
    global_color_table_size = rng.read(1)[0] & 7
    global_color_table_flag = bool(rng.read(1)[0] & 128)

    if not global_color_table_flag:
        raise Exception("Global color table flag is false")

    global_color_table = [rng.read(3) for _ in range(global_color_table_size)]

    gct = b''
    for entry in global_color_table:
        gct += entry

    # 4. Image Descriptor
    image_left_position = rng.read(2)
    image_top_position = rng.read(2)
    image_width = rng.read(2)
    image_height = rng.read(2)

    local_color_table_flag = bool(rng.read(1)[0] & 128)
    interlace_flag = bool(rng.read(1)[0] & 64)
    sort_flag = bool(rng.read(1)[0] & 32)
    reserved = rng.read(3)
    local_color_table_size = rng.read(1)[0] & 7

    lct = b''
    if local_color_table_flag:
        lct = [rng.read(3) for _ in range(local_color_table_size)]

        for entry in lct:
            lct += entry

    # 5. Image Data
    # TODO: Actually make this data
    image_data = b'\x00\x00\xff\x00\x00\xff\xff\xff\xff\xff\x00\x00\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff