#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # 1. Header
    header = b'GIF89a' + bytes([0x2A] * 7)

    # 2. Logical screen descriptor
    logical_screen_descriptor = (
            struct.pack('>HHBBB',
                        0xFFFF, 0xFFFF,
                        ord('!'), 0b11111110
                    ) +
            bytes([0x00]) * 3
        )

    # 3. Global color table
    global_color_table = bytes()

    for i in range(0, 256):
        red   = int((i / 255) ** 2 * 255) & 0xFF
        green = int(((i % 255) / 255) ** 2 * 255) & 0xFF
        blue  = int((((i * 255) % 255) / 255) ** 2 * 255) & 0xFF

        global_color_table += bytes([red, green, blue])

    # 4. Image descriptor
    image_descriptor = (
            struct.pack('>HHHHBB',
                        0, 0,
                        0xFFFF, 0xFFFF,
                        ord(','), 0b11111110
                    ) +
            bytes([0x00]) * 3
        )

    # 5. Local color table
    local_color_table = bytes()

    for i in range(0, 256):
        red   = int((i / 255) ** 2 * 255) & 0xFF
        green = int(((i % 255) / 255) ** 2 * 255) & 0xFF
        blue  = int((((i * 255) % 255) / 255) ** 2 * 255) & 0xFF

        local_color_table += bytes([red, green, blue])


    # 6. Pixel data
    pixel_data = bytes()

    for _ in range(0, 0xFFFF):
        pixel_data += bytes([int(rng.read(1)[0])])

    # Write everything to the output stream
    out.write(header)
    out.write(logical_screen_descriptor)
    out.write(global_color_table)
    out.write(image_descriptor)
    out.write(local_color_table)
    out.write(pixel_data)

