
        return self._rng.randint(*self._range)


def generate_gif(out):
    """Generate a random GIF file."""
    rng = RNG()

    # 1. Logical Screen Descriptor
    width, height = rng.read(2), rng.read(2)
    packed = struct.pack('>HBBBBB',
                         width,
                         height,
                         0xf0 | ((rng.read(1)[0] & 0x07) << 2),  # Global Color Table Flag and Size
                         0,  # Background Color Index
                         0,  # Pixel Aspect Ratio
                         False
                        )
    lsd = struct.pack('>BBBB',
                      0x47,  # Signature "GIF"
                      packed[0],
                      packed[1],
                      packed[2]
                     )

    # 2. Global Color Table
    num_colors = min((packed[4] & 0x07) + 1, 256)
    color_table = rng.read(num_colors * 3)  # Random colors

    # 3. Local Color Table (optional)
    has_lct = bool(rng.read(1)[0])
    if has_lct:
        num_colors = min((packed[4] & 0x07) + 1, 256)
        color_table = rng.read(3 * num_colors)  # Random colors

    # 4. Graphics Control Extension (optional)
    has_gce = bool(rng.read(1)[0])
    if has_gce:
        gce = struct.pack('>BBBHB', 0x21, 0xF9, 0x04, rng.read(1)[0], rng.read(1)[0])
    else:
        gce = None

    # 5. Image Descriptor
    left, top = struct.unpack('>HH', rng.read(4))
    width, height = struct.unpack('>HH', rng.read(4))
    local_color_table_flag = bool(rng.read(1)[0])
    interlace_flag = bool(rng.read(1)[0])
    sort_flag = bool(rng.read(1)[0])
    reserved_flags = rng.read(1)[0]
    local_color_table_size = rng.read(1)[0]
    local_color_table = rng.read(local_color_table_size * 3)
    image_descriptor = struct.pack('>BHHHHBBBBB',
                                  0x2C,
                                  left,
                                  top,
                                  width,
                                  height,
                                  local_color_table_flag,
                                  interlace_flag,
                                  sort_flag,
                                  reserved_flags,
                                  local_color_table_size
                                 )

    # 6. Image Data (randomly sized, capped at 255 bytes per sub-block)
    lzw_minimum_code_size = (rng.read(1)[0] % 8) or 1

    blocks = []
    total_blocks_size = rng.read(1)[0]  # Make it reasonably small
    while True:
        block_size = rng.read(1)[0]  # Each block can be up to 255 bytes

        if block_size == 0:
            break

        blocks.append(bytes([block_size]) + rng.read(block_size - 1))
    image_data = bytes([lzw_minimum_code_size]) + b''.join(blocks) + b'\x00'  # Ending with a block size of 0

    # 7. Trailer
    out.write(lsd + color_table)
    if gce is not None:
        out.write(gce)
    out.write(image_descriptor + image_data + b'\x3B')