#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    header = b'GIF89a'
    
    # Logical Screen Descriptor (LSD)
    lsd = bytearray([
        0x2c, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x00, 
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    ])
    
    # Color Table
    color_table = bytes()
    
    # Global Color Extension (GCE)
    gce = None
    
    # Image Descriptor
    image_descriptor = bytearray([
        0x2c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    ])
    
    # Blocks
    blocks = []
    
    # Total size of all blocks
    total_blocks_size = sum((len(block) for block in blocks))
    
    # 5. Data Sub-Blocks
    while total_blocks_size > 0:
        # Block Size
        block_size = min(total_blocks_size, rng.read(1)[0] + 1)
        blocks.append(rng.read(block_size))
        
        total_blocks_size -= block_size
    
    # 7. Trailer
    trailer = b'\x3b'
    
    # Write everything to the output file
    out.write(header)
    out.write(lsd)
    out.write(color_table)
    if len(gce) == 6:
        out.write(gce)
    out.write(image_descriptor)
    for block in blocks:
        out.write(struct.pack('B', len(block)))
        out.write(block)
    out.write(trailer)