#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# returns its size in bytes.
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')

    width = 1 + int.from_bytes(rng.read(2), byteorder='big', signed=False) % 500
    height = 1 + int.from_bytes(rng.read(2), byteorder='big', signed=False) % 500

    # Logical screen descriptor
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))
    out.write(b'\x87\x00')

    # Global color table (black & white only)
    for i in range(2 ** 4 - 1):
        out.write(struct.pack('B', i))

    # Image descriptor
    out.write(b'\x2c')
    out.write(((width).to_bytes(2, 'big')[0] | ((height).to_bytes(2, 'big')[0]) << 8).to_bytes(2, 'big'))
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00')

    # Local color table (black & white only)
    for i in range(2 ** 4 - 1):
        out.write(struct.pack('B', i))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', int.from_bytes(rng.read(1), byteorder='big', signed=True)))

    return out.tell()


if