#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> int:
    # 1. Header
    out.write(b'GIF89a')

    # 2. Logical Screen Descriptor
    # Random width and height (using two bytes for each)
    width, height = struct.unpack('>HH', rng.read(4))

    # Other LSD fields: Packed fields (using a random byte for variety), Background color index, Pixel aspect ratio
    packed_fields = rng.read(1)[0]
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] % 256

    lsd = struct.pack(
        '>HHBBB', width, height, packed_fields, background_color_index, pixel_aspect_ratio
    )
    out.write(lsd)

    # 3. Global Color Table
    global_color_table_size = 2 ** ((packed_fields & 7) + 1)
    if global_color_table_size > 0:
        global_color_table = [
            rng.read(3) for _ in range(global_color_table_size)
        ]

        gct = b''
        for rgb in global_color_table:
            gct += rgb[0] + rgb[1] + rgb[2]

        out.write(gct)

    # 4. Image Descriptor
    image_left = rng.read(2)[0] % width
    image_top = rng.read(2)[0] % height
    image_width = rng.read(2)[0] % width
    image_height = rng.read(2)[0] % height

    packed_fields = rng.read(1)[0]
    local_color_table_flag = bool((packed_fields >> 7) & 1)
    interlace_flag = bool((packed_fields >> 6) & 1)
    sort_flag = bool((packed_fields >> 5) & 1)
    reserved_bits = (packed_fields & 31) << 3

    ihd = struct.pack(
        '>HHHHB', image_left, image_top, image_width, image_height, packed_fields
    )
    out.write(ihd)

    # 5. Local Color Table
    if local_color_table_flag:
        local_color_table_size = 2 ** (((packed_fields >> 3) & 7) + 1)
        local_color_table = [
            rng.read(3) for _ in range(local_color_table_size)
        ]

        lct = b''
        for rgb in local_color_table:
            lct += rgb[0] + rgb[1] + rgb[2]

        out.write(lct)

    # 6. Block of Extension Data
    block_size = rng.read(1)[0] % 256
    while block_size!= 0:
        out.write(block_size.to_bytes(1, 'big'))
        out.write(rng.read(block_size))
        block_size = rng.read(1)[0] % 256

    # 7. Image Data
    # We'll just use the same data as the previous frame to keep things simple
    out.write(image_data)

    return len(lsd) + len(gct) + len(ihd) + len(lct) + block_size + len(image_data)


if