#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor (7 bytes)
    width = rng.randint(100, 256)
    height = rng.randint(100, 256)
    flags = b'\x87\x00\x00' if rng.randint(0, 1) == 0 else b'\x00\x00\x00'
    bgColorIndex = rng.randint(0, 256)
    pixelAspectRatio = b'\xff'
    out.write(struct.pack('<HHBBB', width, height, flags[0], flags[1], flags[2], bgColorIndex))
    out.write(pixelAspectRatio)

    # Global color table (3 * 256 = 768 bytes)
    for i in range(256):
        r = rng.randint(0, 256)
        g = rng.randint(0, 256)
        b = rng.randint(0, 256)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor (10 bytes)
    leftPos = rng.randint(0, width - 1)
    topPos = rng.randint(0, height - 1)
    imageWidth = rng.randint(1, width - leftPos)
    imageHeight = rng.randint(1, height - topPos)
    flags = b'\x87\x00\x00' if rng.randint(0, 1) == 0 else b'\x00\x00\x00'
    localColorTableFlag = b'\x01'
    interlaceFlag = b'\x00'
    sortFlag = b'\x00'
    reserved = b'\x00'
    lctSize = b'\x00'
    out.write(struct.pack('<HHHHBBBBB', leftPos, topPos, imageWidth, imageHeight, flags[0], flags[1], flags[2], localColorTableFlag[0], interlaceFlag[0], sortFlag[0], reserved[0]))
    out.write(lctSize)

    # Local color table (3 * 256 = 768 bytes)
    for i in range(256):
        r = rng.randint(0, 256)
        g = rng.randint(0, 256)
        b = rng.randint(0, 256)
        out.write(struct.pack('BBB', r, g, b))

    # Pixel data
    for y in range(imageHeight):
        for x in range(imageWidth):
            out.write(bytes([rng.randint(0, 256)]))

