#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header (13 bytes)
    header = b'GIF89a' + \
             struct.pack('<HH', 0, 0) + \
             b'\x87\x00\x00'

    # Logical screen descriptor (7 bytes)
    lsd = struct.pack('<HHBBBH',
                      256, 256, 0b10000000 | 0b00000000,
                      0b00000000, 0b00000000, 0)

    # Global color table (3 bytes)
    gct = b'\x00\x00\x00'

    # Application extension block (11 bytes)
    appext = b'\x21\xFF\x0B' + \
             b'NETSCAPE2.0' + \
             b'\x03\x01' + \
             b'\x00\x00'

    # Graphics control extension block (8 bytes)
    gce = b'\x21\xF9\x04' + \
          b'\x08\x00\x00\x00\x00'

    # Image descriptor (10 bytes)
    id = b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00\x00'

    # Plain text data sub-block (1 byte)
    ptds = b'\x01'

    # Trailer (1 byte)
    trailer = b';'

    # Write everything to output stream
    out.write(header)
    out.write(lsd)
    out.write(gct)
    out.write(appext)
    out.write(gce)
    out.write(id)
    out.write(ptds)
    out.write(trailer)


if