#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    gc_flags = rng.read(1)[0] & 0b01111000
    gce = struct.pack('B', (0x21 << 4 | 0xf | gc_flags)) + \
          struct.pack('BB', 0xff,   # delay time
                      0         )   # transparent color
    identifier = rng.read(3)
    out.write(header)
    out.write(struct.pack('!HH', *map(int.from_bytes, (
                rng.read(3), rng.read(3)), repeat=None)))
    out.write(gce)
    nframes = int.from_bytes(rng.read(1), byteorder