#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, size=(128, 64), fps=10, nframes=3):
    # first frame
    table = bytes([
        0xff, 0x00, 0x00, 0x00, 0xff, 0x00, 0xff, 0x00, 0xff, 0x00,
        0x00, 0xff, 0x00, 0xff, 0xff, 0x00, 0xff, 0xff, 0x00, 0x00])
    img = [bytes(table) for _ in range(size[1] // 2)] * size[0]
    img = b''.join((b'\x47\x49\x46\x38\x39\x61',
                   struct.pack('<H', size[0]),
                   struct.pack('<H', size[1]),
                   b'\x80\x01\x00',
                   '\xff\xff\xff\x2c\x00\x00\x00\x00',
                   '\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b')) + \
          b''.join(img)

    out.write(img[:35+len(img)-3])

    # rest of the frames
    for i in range(nframes - 1):
        # The IDAT chunk is not compressed: only 1 byte per pixel
        (width, height) = size
        w2 = width // 2
        h2 = height // 2
        data = []
        while len(data) < height*width:
            x = i % w2
            y = i // w2
            if x > w2-y or y >= h2:
                # make background black
                data += [0] * (w2-x+h2-y)*2
            else:
                # make it white, we don't care about the actual colour
                data += [255]*(w2-x+h2-y)*2
        assert len(data) == height*width