#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # 10-byte header
    out.write(b"GIF89a")
    width = rng.read(2)
    height = rng.read(2)
    fields = rng.read(1)

    # Global colormap flag (unused)
    global_color_map_flag = fields[0] >> 7
    global_color_map_size = ((fields[0] & 0x70) >> 4) + 1
    global_color_resolution = (((fields[0] & 0x0c) >> 2) + 1) << 3
    sort_flag = fields[0] & 3

    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]

    colors = []
    for _ in range(global_color_map_size * 3):
        color = bytes([rng.read(1)[0]])
        colors.append(color)

    # Color table
    out.write(bytes(colors))

    # Header terminator
    out.write(b"\x00\x3B")

    while True:
        label = rng.read(1)[0]
        if label == 0xFF:
            break
        size = rng.read(1)[0]

        # Image descriptor
        local_color_table_flag = size >> 7
        interlaced = size >> 6 & 1
        sort_flag = size >> 5 & 1
        reserved = size >> 4 & 1

        left_pos = int.from_bytes(rng.read(2), "big", signed=True)
        top_pos = int.from_bytes(rng.read(2), "big", signed=True)
        width = int.from_bytes(rng.read(2), "big", signed=True)
        height = int.from_bytes(rng.read(2), "big", signed=True)
        
        lct_size = ((size & 0x70) >> 4) + 1

        # Local color map
        colors = []
        for _ in range(lct_size * 3):
            color = bytes([rng.read(1)[0]])
            colors.append(color)
            