#!/usr/bin/env python3

import struct
from typing import BinaryIO, List, Tuple

#: The order in which the bytes should be read from a file to obtain an Int16 value.
int16_endianness = 'little'

def decode(filename) -> List[Tuple]:
    """Decodes the given binary file."""
    
    with open(filename, mode='rb') as f:
        return _decode(f)

def encode(instructions: List[Tuple], filename: str):  # TODO: Takes instructions as list of tuples.
    with open(filename, mode='wb') as f:
        for instruction in instructions:
            _encode(instruction, f)

def _determine_parameter_mode(value, parameter_count=2):
    if isinstance(value, int):
        if value < 0: raise Exception('Illegal parameter value: {}'.format(value))
        
        decimal_code = abs(value) % 10**parameter_count
        
        string_code = ''
        while decimal_code > 0:
            digit = (decimal_code - 1) % 10 + 1
            
            string_code += '%i' % digit
            decimal_code //= 10
        string_code += '0' * parameter_count
        string_code = string_code[-parameter_count:]
        
        result = []
        for i in range(parameter_count):
            result.append(string_code[-(i+1)] == '0')
        
        return tuple(result)
    else:
        assert len(value) <= parameter_count
        result = [False] * parameter_count
        for index, item in enumerate(value):
            result[index] = True
        return tuple(result)

def _get_parameter_values(inputs, parameter_modes):
    input_values = []
    
    for parameters, input in zip(parameter_modes, inputs):
        for mode in reversed(parameters):
            if not mode:
                input_values.insert(0, input())
            elif mode:
                input_values.insert(0, input)
            
    return tuple(input_values)

def _decode(stream: BinaryIO) -> List[Tuple]:
    parts = []
    while True:
        code = stream.read(4).hex()
        if not code: break
        
        opcode = int(code[-2:])
        parameter_modes = _determine_parameter_mode(int(code[:-2]), 3)
        parts.append((opcode, parameter_modes))
    
    return parts

def _encode(part: Tuple, stream: BinaryIO):
    opcode, parameter_modes = part
    assert len(parameter_modes) == 3
    
