#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'  # GIF magic bytes

    # Get the width and height from the RNG
    w = struct.unpack('<H', rng.read(2))[0] + 1
    h = struct.unpack('<H', rng.read(2))[0] + 1

    # Create a simple 2 color image
    img_data = [
        0xFF, 0xFF, *([0xAA]*(w*h)),
        0xFF, 0xFF, *([0x55]*(w*h)),
    ]

    # Write the size of the image and some other data
    out.write(struct.pack('<HHBxBB', w, h, 0x80, 0, 0))

    # Write the global palette - two colors of black and white
    for i in range(16):
        # Set red to 0x00 (black), green to 0xFF (white), and blue to 0xFF
        out.write(b'\x00\xFF\xFF')

    # Loop over frames of the animation
    for frame in range(8):
        # Write the local palette - one instance of each color
        for i in range(16):
            out.write(img_data[i:i+4])

        # Write the graphic control extension
        out.write(struct.pack('<BHBBB', 0xF9, 0x04, 0, 0, 0, 0x0A))

        # Write the image descriptor
        out.write(struct.pack('<BHHBBB', 0