#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 28 17:56:41 2020

@author: pmchozas
"""
import numpy as np
from sklearn import preprocessing, cluster
from scipy.stats import multivariate_normal, norm
import matplotlib.pyplot as plt

def generar(n=50):
    '''
    Genera datos de una normal con media en (0,0) y varianza diagonal [0.1,0.03,0.01]
    
    Input: número de puntos a generar 
    
    Output: matriz de dimension dos con las coordenadas de los puntos generados
    
    '''
    cov = [[0.1, 0],[0, 0.03], [0, 0]]
    mu = [0, 0, 0]
    X = np.array([multivariate_normal.rvs(mean=mu,cov=cov, size=n)]).T   #vector con x y luego otro con z
    return X

def desviacion(X):
    '''
    Calcula la media y la desviación típica de cada columna del vector ingresado
    
    Input: Matriz con columnas que reprensentan atributos
    
    Output: vector (tamaño igual al numero de columnas de la matriz) con la media y la desviación típica de
            cada columna
        
    '''
    mu = np.mean(X, axis = 0)
    std = np.std(X, axis = 0)
    return np.concatenate((mu, std))

def print_estimates(mu, cov, X):
    '''
    Imprime el resultado obtenido después de haber aplicado el algoritmo de k medias
    
    Input: mu (posición estimada de centroide), cov (matriz de covarianza estimada para cada centroide),
           X (matriz de datos)
    
    Output: mu (media de centroide estimada), cov (matriz de covarianza estimada), w (pesos de cada centroide),
            y (puntos estimados)
            
    '''
    w = []                       # Calculo de la frecuencia de cada centroide
    for i in range(len(mu)):
        w.append(np.sum(kmeans.predict(X) == i)/float(len(X)))
    w = np.array(w)

    y = []                       # Calculo de los centroides estimados
    for i in range(len(mu)):
        y.append(np.mean(X[np.where(kmeans.labels_ == i)], axis = 0))
    y = np.array(y).T
    
    print('La media estimada es:\n', mu)
    print('El error estimado es:\n', mu-mu_true)
    print('La matriz de covarianza estimada es:\n', cov)
    print('Los pesos estimados son:\n', w)
    print("Y los centros de masas estimados son:\n", y)
    
def main():
    n = int(input('\nIngrese el número de puntos que se van a generar: '))
    if n < 1:
        raise ValueError('\nDebe ingresar un valor válido :(')
    else:
        X = generar(n)
        print('\nLos puntos originales son: \n\n', X)
        mu_true, sigma_true = desviacion(X)
        print("La media true es: ", mu_true)
        print("La matriz de covarianza true es: \n", np.diag(sigma_true**2))
        kmeans = cluster.KMeans()
        kmeans.fit(X)
        mu, cov = kmeans.cluster_centers_, kmeans.covariance_
