#!/usr/bin/env python3

import struct
from typing import BinaryIO
import random

def gif(data: bytes, offset: int=0) -> tuple[bytes, int]:
    if len(data) < 10:
        raise ValueError('not enough data for a GIF at offset %d' % offset)
    if data[:6] != b'GIF87a' and data[:6] != b'GIF89a':
        raise ValueError('bad GIF magic header at offset %d' % offset)
    datalen = int.from_bytes(data[6:9], byteorder='big')
    return data[10+datalen:]

def encode(rng: BinaryIO, out: BinaryIO    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
