#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    screen = 0x0257 # 100 x 100
    color_table = [
        (103,  96,  42), # RGB
        (202,  94,  42), # RGB
        (203, 100,  29), # RGB
        (212, 104,  29), # RGB
        (213, 110,  29), # RGB
    ]

    # GIF Header
    out.write(header)

    # Logical Screen Descriptor
    out.write(struct.pack('<HHBB', screen, screen, 0b00000000))

    # Global Color Table
    for c in color_table:
        out.write(struct.pack('<BBB', *c))

    # Graphic Control Extension
    gce = bytes([0x21, 0xf9, 0x04, 0x00, 0x01]) # default values
    out.write(gce)

    # Image Descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\xff\x00'
    out.write(image_descriptor)

    # Local Color Table
    for c in color_table:
        out.write(struct.pack('<BBB', *c))

    # Image Data
    image_data = []
    while True:
        # end of current packet
        if len(image_data) == 1024 - 1:
            break

        image_data += struct.unpack(f'>{len(color_table)}H', rng.read(2*len(color_table)))

    # write in blocks of 1 byte per pixel
