#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# assuming that the width of the GIF data is 16 pixels.
def main(out: BinaryIO, rng: bytes):
    # Generate a header
    out.write(b'GIF89a')          # "GIF" + version number ("89a")
    out.write(b'\0\0\xff\0\0\0\0') # Logical screen size (width & height) = 256 pixels
    out.write(bytes([0 if i == 7 else 1 for i in range(8)]))   # Global color table (black & white only)
    out.write(struct.pack('<B', 0))                            # Background color index
    out.write(struct.pack('<B', 0))                            # Pixel aspect ratio
    
    # Generate a bunch of frames at random
    # Last one has no delay so it will loop forever
    for _ in range(1000):
        # Define an image descriptor block
        out.write(bytes([
            0x2c,              # Image separator (',')
            0x00, 0x00,        # X offset
            0x00, 0x00,        # Y offset
            0x00, 0x0f,        # Width
            0x00, 0x0f,        # Height
            0x80                # Local color table flag
        ]))
        
        # Write a palette of four colors
        out.write(struct.pack('<BBBBBBBB', 0, 0, 0, 0xff, 0xff, 0xff, 0, 0))
        
        # Write random data as image data...
        out.write(rng[:14])
