#!/usr/bin/env python3

import struct
from typing import BinaryIO

from Crypto.Cipher import AES

def aes_cbc_decrypt(key: bytes, iv: bytes, data: bytes) -> bytes:
    cipher = AES.new(key, mode=AES.MODE_CBC, IV=iv)
    return cipher.decrypt(data)

# recreated from https://github.com/DanTheMan827/Minecraft-Deobfuscator/blob/master/src/main/java/io/darkhax/minecraft/deobfuscator/util/Utils.java
def build_iv() -> bytes:
    # endianness doesn't matter here because they are all zeroed out
    return struct.pack("!Q", 0x36a6d925b41e95f0 | (0<<12))

def extract_keys(file: BinaryIO):
    assert file.read(4).decode('ascii') == 'BNCR'
    
    length = int.from_bytes(file.read(8), byteorder='big', signed=False)
    keys = []
    for _ in range(length // 4):
        key_id = int.from_bytes(file.read(4), byteorder='big', signed=True)
        if key_id != -1:
            raise ValueError(f"