#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    version = rng.read(7)  # unused part of the header
    width = rng.read(2)
    height = rng.read(2)
    fieldsize = struct.unpack("<H", rng.read(2))[0]
    fields = {}
    for _ in range(fieldsize):
        type = struct.unpack("B", rng.read(1))[0].to_bytes(1, "big")
        length = struct.unpack("B", rng.read(1))[0].to_bytes(1, "big")
        value = rng.read(length)
        fields[type] = (length, value)
    datalen = struct.unpack("<L", rng.read(4))[0]
    data = rng.read(datalen)
    out.write(header + data)
