#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1 # 1-based
    paldata = []
    palsize = (datalen - 4) // 3
    for _ in range(palsize):
        paldata += [
                rng.read(3),
                0xFF,
        ]
            
    head2 = rng.read(3) # everything else is ignored

    # Write output data to `out`.
    out.write(header)
    out.write(struct.pack('<H', datalen))
    out.write(b'\xFF\xFE\x03\x00')
    out.write(bytes(paldata))
   