#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the specified colors.
def write_gif(out:BinaryIO, width:int, height:int, colors) -> None:
    if len(colors) == 0 or len(colors[0]) != 3:
        raise Exception("Colors must be a list of (r,g,b) tuples")

    # No-op application extension
    out.write(struct.pack('<12s', b'GIF87a'))
    out.write(struct.pack('<B', 4))
    out.write(struct.pack('<HB', 5, 8))   # 5 bytes of global color table, 8 bits per pixel
    out.write(b'\x00')                    # background color index
    out.write(b'\x00')                    # default aspect ratio

    # Logical screen descriptor
    out.write(struct.pack('<HBB', width, height, 0))     # logical screen size
    out.write('\xff'.encode())              # color table flag = yes
    out.write('\x00'.encode())              # background color index
    out.write('\xff'.encode())              # global color table size
    out.write(b'\xa9\xad')                  # magic signature for NETSCAPE2.0
    out.write(struct.pack('<H', 0))         # repeat count = forever

    # Global color table
    for r, g, b in colors:
        rgb = (r >> 3 << 5) | (g >> 3 << 2) | (b >> 3)
        out.write(struct.pack('<B', rgb))

    # Image descriptor
    out.write(struct.pack('<6B', 0x2c, 0, 0, width