#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#
# 10 bytes header (screen descriptor, color map, ...)
# 3 bytes for length of "data" field
# followed by random bytes for "data" field
def generate_gif(rng: BinaryIO, out: BinaryIO):
    # "GIF89a" is the endian-agnostic GIF magic number.
    header = b'GIF89a\x02\x00\x24\x00\x00\x00!\xf9\x04\x00\x1c\x00\x00\x00\x00,\x00' \
             b'\x00\x00\x00\x00\xff\xcc\xcc\xff\xcc\x27\x57\x3f\x6e\xff\xb2\x26' \
             b'\x6e\xff\xa9\x26\xc6\xee\xff\xae\x26\xce\xe6\xff\xb5\x26\x7b\xca' \
             b'\xff\xbc\x26\xbb\x9a\xff\xbf\x26\x9a\xff\xbe\x26\x9a\xff\xc1\x26' \
             b'\x9a\xff\xc5\x26\x9a\xff\xba\x26\x9a\xff\xc0\x26\x9a\xff\xc4\x26' \
             b'\x9a\xff\xbc\x26\x9a\xff\xbd\x26\x9a\xff\xc3\x26\x9a\xff\xc7\x26' \
             b'\x9a\xff\xb7\xfe\x6e\x00\x3b'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
