#!/usr/bin/env python3

import struct
from typing import BinaryIO

from pwnlib.context import context
from pwnlib.log import getLogger
from pwnlib.util.fiddling import byteline

log = getLogger(__name__)


def dump(stream: BinaryIO, stat_count: int=None):
    """Dump the contents of a file descriptor to the screen.

    This is useful for debugging on remote systems where ``/dev/fd`` does not
    exist.

    Args:
        stream (BinaryIO): The stream you would like to see the contents of.
            Usually ``sys.stdin``, or an open file which should be closed after
            calling this function.  The remainder of the file will also be read.

    Examples:
        >>> fd = open('file') # doctest: +SKIP
        >>> dump(fd)           # doctest: +SKIP

    Returns:
        bool: Always returns True.
    """
    if hasattr(stream, 'read'):
        data = stream.read()
    else:
        data = bytearray(os.read(stream.fileno(), stat_count))

    log.info("%s" % byteline(data))
    return True
