#!/usr/bin/env python3

import struct
from typing import BinaryIO


def _read_uint16(file: BinaryIO, endianness: str) -> int:
    raw = file.read(2)
    if len(raw) != 2:
        raise EOFError()
    return struct.unpack('<' if endianness == 'little' else '>', raw)[0]


def read_exr_chunk(file: BinaryIO):
    chunk_size = _read_uint16(file, 'big')
    # Skip chunk type (4 bytes) and payload (size bytes).
    file.seek(8 + chunk_size, 1)
    # Read next chunk size.
    return _read_uint16(file, 'big')

