#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng):
    # Header
    out.write('GIF89a'.encode('ascii'))
    
    # Logical Screen Descriptor
    w = struct.unpack("I", rng.read(4))[0] % (2 ** 16 - 1) + 1
    h = struct.unpack("I", rng.read(4))[0] % (2 ** 16 - 1) + 1
    bpp = 8
    gct = 7 # Local Color Table Flag
    bgc = 0
    pixs = 0
    aspect = 0
    print(w, h, bpp, gct, bgc, pixs, aspect)
    lsb = (
        ((w & 0xff00) >> 8) |
        ((h & 0xf000) >> 12) |
        (bpp << 4) |
        (gct << 3) |
        (bgc << 2) |
        (pixs << 1) |
        (aspect << 0)
    )
    msb = (
        0x01 |
        ((w  & 0x00ff) << 0) |
        ((h  & 0x0f00) >> 8)
    )
    out.write(struct.pack("<BBB", lsb, msb, bgc))
  
    # Graphics Control Extension
    gce_flag = 0x80 | 0x05
    delay = struct.unpack(">H", rng.read(2))[0]
    trans = struct.unpack("B", rng.read(1))[0]
    blocksize = 0x04
    print(delay, trans)
    out.write(struct.pack("<BHHBB", gce_flag, delay, trans, blocksize, 0)) # Empty Block Terminator

    # Image Descriptor
    id_flag = 0x2C
    x = struct.unpack(">H", rng.read(2))[0] % w
    y = struct.unpack(">H", rng.read(2))[0] % h
    wi = struct.unpack(">H", rng.read(2))[0] % (w - x) if w > x else w - x
    hi = struct.unpack(">H", rng.read(2))[0] % (h - y) if h > y else h - y
    ct = 0
    ci = 0
    pi = 0
    print(x, y, wi, hi)
    out.write(struct.pack("<BBHHHHBB", id_flag, x, y, wi, hi, ct, ci, pi))
  
    # Global Color Table
    for i in range(2**(3+bpp)):
      r = struct.unpack("B", rng.read(1))[0]
      g = struct.unpack("B", rng.read(1))[0]
      b = struct.unpack("B", rng.read(1))[0]
      out.write(struct.