#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + (b'\0' * 12)
    out.write(header)
    # Write LSD
    local_screen_descriptor = bytearray([0xFF])
    local_screen_descriptor += [0xF0]
    img_width, img_height = struct.unpack('<HH', rng.read(4))[:2]
    local_screen_descriptor += struct.pack("<H", img_width)[-2:]
    local_screen_descriptor += struct.pack("<H", img_height)[-2:]
    local_screen_descriptor += b"\x07" # background color index
    local_screen_descriptor += b"\x1B" # aspect ratio
    out.write(local_screen_descriptor)
    # Write global palette
    # The index of each entry is its RGB channel values bitwise-or'd together
    global_palette = []
    for _ in range(768):
        global_palette += list(struct.unpack("BBB", rng.read(3)))
    # Indexed color mode
    image_descriptor = bytearray([0x2c])
    image_descriptor += [0x00] * 5
    image_descriptor += struct.pack("<H", img_width)[-2:]
    image_descriptor += struct.pack("<H", img_height)[-2:]
    image_descriptor += b"\x00"
    out.write(image_descriptor)
    # Write LCT
    lct = bytearray()
    lct.append((len(global_palette) - 1) & 0xFF)
    lct += bytes(global_palette)
    out.write(lct)
    # Write IDAT
    idat = bytearray(b'\x00')
