#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int=20, height: int=20):
    # Magic bytes at the beginning of the file
    magic = b'GIF89a'
    # Background color (black)
    bg = 0
    # 10 frames per second
    fps = 60
    # Hard code first frame to be black
    data = [bg] * (width * height)
           
    # Write the header
    out.write(magic)
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    
    # Default value for delay between each picture
    delay = 100 // fps if 100 % fps else 0
        
    # Header for the first frame:
    #   7 is always used as the initial value of the control code
    #   the image itself follows it (without a leading separator)
    out.write(b'\x07\x00')
                            
    # Now write the actual content of the frames in order:
    # one byte per pixel, with `delay` between them
    for i in range(len(data)):
        rng.readinto(data[i].to_bytes(1, 'big'))
        out.write(struct.pack("B", data[i]))
        # Delay between pictures
        if i % width == 0 and i != 0:
            out.write(struct.pack("<H", delay))
   
    # EOF
    out.write(b';')
