#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) for its
# palette.
def generate_gif(out: BinaryIO):
    out.write("GIF87a".encode())                 # Magic numbers.
    
    width, height = 160, 144               # Standard size of NES screen.
    
    out.write(struct.pack("<HHB", width, height, 8))   # Width, Height, Depth.
    
    # Background color index.
    out.write(b"\x00\x00")
    
    # Pixel aspect ratio.
    out.write(b"\x21\xf9" + b"\x04\x00" + b"\x05\x00" + b"\xff\xff")
    
    # Graphic Control Extension.
    out.write(b"\x21\xf9\x04\x04\x00\x00\xff\xff")
    
    # Image Descriptor.
    out.write(b"\x2c\x00\x00\x00\x00\x01\x00")
    
    out.write(b"\x00" * (width * height // 4))       # Initial image data.
    
    out