#!/usr/bin/env python3

import struct
from typing import BinaryIO

with open('eagle-creek-2015-07-08T04:29:08Z.pcap', 'rb') as infile, \
        open("out.pcap", "wb") as out:
    header = b"D\x00P\xc0\x02\x00\x04\x00\x00\x00\xff\xff\x00\x00\x00\x00"
    out.write(header)
    for packet in iter(lambda: infile.read(65520), b''):
        if len(packet) < 16:
            break
        datalen = struct.unpack("@H", packet[:2])[0]
        print(datalen)
        timestamp = struct.unpack("@Q", packet[2:10])[0]
        out.write(struct.pack("<Q", timestamp))
        out.write(packet[10:])

with open('eagle-creek-2015-07-08T04:29:08Z.pcapng', 'rb') as infile, \
        open("out.pcapng", "wb") as out:
    # The first 16 bytes are fixed, the last 4 bytes might be used to store a
    # CRC or something...
    header = b"\xa1\xb2\xc3\xd4\x02\0\0\0%s\0\0\0\0\0\0\0" % b'\x00' * 3
    rng = infile.read(infile.seek(0, os.SEEK_END))
    out.write(header + rng)

with open('sample.pcapng', 'rb') as infile, open("out.pcapng", "wb") as out:
    # First 16 bytes have to match, and the remaining 4 bytes seem to be CRC
    # so we skip them
    rng = infile.read(infile.seek(0, os.SEEK_END))
    assert rng.startswith(struct.pack("IIII", 0xa1b23c4d, 2, 4, 0))
    rng = rng[16:-4:]
    datalen = (rng.pop() << 24) | (rng.pop() << 16) | (rng.pop() << 8) | \
              rng.pop()
    header = struct.pack("<I", 0xa1b23c4d) + rng.read(2) + struct.pack("B", 4) \
                      + struct.pack("@I", datalen + 16) + struct.pack(
                "@III", 7,
                int.from_bytes(rng.read(8), byteorder='little'),
                rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
