#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), byteorder='big') # length of initial data block in bytes
    color_table = rng.read(datalen - 1)                    # colour table (256*3B) minus transparent index
    color_map = []
    for col in [color_table[i * 3:(i + 1) * 3] for i in range(256)]:
        if col == b'\x00\x00\x00':
            continue
        color_map.append((int(col[0]), int(col[1]), int(col[2]))) # casting to integers is needed because otherwise it would be interepreted as bytes instead of integers
    background_index = color_table[-1]                         # index of the background colour
    num_loops = int.from_bytes(rng.read(2), byteorder='big')   # amount of loops through the image
    netsize = int.from_bytes(rng.read(2), byteorder='big')      # size of the network dictionary

    while True:
        cmd = rng.read(1)[0]
        if not cmd & 0b10000000:
            subblock_size = rng.read(1)
            pixels = rng.read(subblock_size) # read size pixels

            for pixel in [pixels[i * 4:(i + 1) * 4] for i in range(len(pixels) // 4)]:
                if pixel != b'\xff\xff\xff\x00':
                    try:
                        idx = color_map.index(pixel)       # get index of current pixel from the colour map
                    except ValueError:
                        continue
                    if idx == background_index:           # check if it's the same as the backgroud colour
                        continue                          # don't write this to the output buffer if so
                    out.write(struct.pack('<B',idx))      # write it to the output buffer
        elif cmd == 0b10000000 or \
             cmd == 0b10000011:
            break # end of image and loop thru frames, respectively
        else:
            raise RuntimeError("Unknown GIF command: %0X" % cmd)
