#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def gen(out: BinaryIO, rng):
    header = 'GIF8'.encode('latin-1')
    sig = int.from_bytes(rng.read(6), 'little')
    if (sig >> 24 == ord('a')) and ((sig & 0xffffff) < 0x100000):
        # offset reached end of 4 GB space
        return
    if sig > 0:
        # start with empty screen
        out.write(b'\x47\x49\x46\x38\x39\x61\x01\x00\x1f\x00' \
                  b'\x01\x00\x01\x00\x00\x02\x02\x4c\x01\x00\x3b')
        return

    header += rng.read(5)
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
