#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width, height = 1024, 768
    background_color = rng.read(3).decode('ascii')
    color_table = [rng.read(3).decode('ascii') for _ in range(256)]
    left, top, right, bottom = 0, 0, width - 1, height - 1
    table_size = len(background_color + ''.join(color_table))
    loop_count = 1 # This is not valid GIF
    block_size = 8 + table_size
    global_color_table = False
    out.write(header)
    write_u16le(out, width)
    write_u16le(out, height)
    write_byte(out, 8)
    write_byte(out, int(global_color_table))
    write_byte(out, background_color[1])
    write_byte(out, background_color[2])
    write_byte(out, loop_count)
    write_byte(out, (block_size + 12) & 0xff)
    write_byte(out, ((block_size + 12) >> 8) & 0xff)
    out.write(b'\x00\x00') # Plain text extension
    write_byte(out, table_size)
    write_byte(out, background_color[0])
    out.write(background_color[1].encode('ascii'))
    out.write(background_color[2].encode('ascii'))
    out.write(b'\x00')
    for i in range(len(color_table)):
        c = color_table[i]
        out.write((c[0]).encode('ascii'))
        out.write(c[1].encode('ascii'))
        out.write(c[2].encode('ascii'))
        out.write(b'\x00')
    out.write(b'\x2c\x00\x00\x00\x00')
    write_byte(out, 0x80 | (left & 0xff))
    write_byte(out, top & 0xff)
    write_byte(out, (right & 0xff) | 0x80)
    write_byte(out, bottom & 0xff)
    write_byte(out, 0)
    out.write(b'\x00;')
    out.flush()


def write_u16le(out: BinaryIO, n: int):
    out.write(struct.pack("<H", n & 0xffff))

def write_byte(