#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    width = int.from_bytes(
            rng.read(2), byteorder='big')
    height = int.from_bytes(
            rng.read(2), byteorder='big')
    colormap_len = int.from_bytes(
            rng.read(1), byteorder='big')
    flags = rng.read(1)
    global_colormap = [
        0xff000000 | (int.from_bytes(rng.read(3), byteorder='little', signed=False)),
        0xff000000 | (int.from_bytes(rng.read(3), byteorder='little', signed=False)),
        0xff000000 | (int.from_bytes(rng.read(3), byteorder='little', signed=False))
    ]
    print('Width', width)
    print('Height', height)
    print('Color Map Length', colormap_len)
    print('Global Colormap', global_colormap)
    assert len(header) == 6
    out.write(header)
    # Now we need to write the logical screen descriptor
    sld = struct.pack("!HHBB", width, height, 0xe0, 0)
    out.write(sld)
    # Next comes the global color map
    for i in range(colormap_len + 1):
        c = global_colormap[i] & 0xffffff
        out.write((c >> 16).to_bytes(1, 'little'))
        out.write((c >> 8).to_bytes(1, 'little'))
        out.write((c >> 0).to_bytes