#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given palette.
def generate_gif(out: BinaryIO, width: int, height: int) -> None:
    # Header
    version = b'GIF89a'
    out.write(version)
    
    # Logical screen descriptor
    background_color_index = 0
    aspect_ratio = 0
    global_color_table_flag = 1 if False else 0
    color_resolution = 7
    sort_flag = 0
    size_of_global_color_table = 2**(color_resolution + 1) * 3
    color_table = [b'\xff\xff\xff'] * (256 - len(palette)) + palette
    logical_screen_descriptor = struct.pack('<HB', (width << 4 | width), (height << 4 | height))
    out.write(logical_screen_descriptor)
    
    # Global color table
    for color in color_table:
        out.write(color)
        
    # Application extension
    block_size = b'\x0b'
    application_extension_label = b'NETSCAPE2.0'
    application_authentication_code = b'NA'
    data_subblock_id = 1
    loop_count = 0
    loop_count_bytes = struct.pack('<I', loop_count)[1:]
    out.write(b'\x21')
    out.write(block_size)
    out.write(application_extension_label)
    out.write(application_authentication_code)
    out.write(struct.pack('B', len(loop_count_bytes)))
    out.write(data_subblock_id)
    out.write(loop_count_bytes)
    
    # Graphic control extension
    dispose_option = 2
    user_input_flag = 0
    transparent_color_flag = 1 if True else 0
    delay_time = 20
    transparent_color_index = 0
    graphic_control_extension = struct.pack('BBBBB', (dispose_option << 2 | user_input_flag), ((delay_time & 0xFF00) >> 8), delay_time & 0xFF, transparent_color_index if transparent_color_flag else 0)
    out.write(graphic_control_extension)
    
    # Image descriptor
    left_position = 0
    top_position = 0
    image_width = width
    image_height = height
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    size_of_local_color_table = 0
    image_descriptor = struct.pack('<HHBB', left_position, top_position, image_width, image_height)
    out.write(image_descriptor)
    
    # Local color table
    
    # Raster data
    out.write(b'\x00\x00\x00')
    out.seek(0, 2)
    

# The RGB palette of the output images. Each component is 8 bits and in range [0, 255].
palette = [
    b'\x00\x00\x