#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # i will be our index counter and data buffer
    i = 0
    buf = bytearray()
    while True:
        # read 1 byte from RNG (0x00 or 0xFF)
        c = int.from_bytes(rng.read(1), "little")
        if c == 0: break
        # set the current bit to the current value of I mod 8
        # if it's non-zero, we want to write an FF; otherwise, zeroes
        mask = 1 << (i % 8)
        buf[int(i / 8)] ^= -((c & 1) * mask) & mask
        # increment index
        i += 1
        
    # repeat the buffer for as many times as needed to reach at least the minimum GIF size
    while len(buf) < 64:
        buf += buf
        
    # write header
    out.write('GIF'.encode("ascii"))
    out.write('\x38\x9b'.encode("ascii")) # format version
    out.write(struct.pack("<HH", 50, 50)) # logical screen resolution
    out.write(b'\x00') # background color
    out.write(b'\x00') # pixel aspect ratio
    out.write(b'\x21\xf9\x04\xff\x0b') # Graphic Control Extension
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x00;\x00