#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
  version = b"GIF89a"
  width = rng.read(2)
  height = rng.read(2)
  fields = bytearray([0xF7]) # Graphics Control Extension Block
  fields += b"\x1f\xff"      # color table size / delay time (seconds)
  fields[1] |= int(bool(rng.read(1))) << 1   # transparent bit
  header = version + width + height + fields
  datalen = len(header) % 0xFF
  if datalen == 0:
    datalen = 0xFF
    data = rng.read(datalen)
    out.write(header + data)
