#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    checksum = 0x1a2b3c4d
    for i in range(len(data)):
        checksum ^= (i & 0xff) ^ ord(data[i])

    trailer = b'\x3B' # picture terminator

    out.write(header + \
              len(data).to_bytes(3,'big')+ \
              data + \
              checksum.to_bytes(4, 'big') + \
              trailer)
    