#!/usr/bin/env python3

import struct
from typing import BinaryIO, TypeVar


def read_int(f: BinaryIO) -> int:
    return int.from_bytes(f.read(8), byteorder="big", signed=False)


def write_int(i: int, f: BinaryIO):
    f.write(i.to_bytes(8, "big"))


T = TypeVar("T")


def read_struct(fmt: str, f: BinaryIO) -> T:
    size = struct.calcsize(fmt)
    data = f.read(size)

    if len(data) < size:
        raise ValueError()

    return struct.unpack(fmt, data)[0]  # type: ignore[no-any-return]


def write_struct(fmt: str, value: any, f: BinaryIO):
    value_bytes: bytes = struct.pack(fmt, value)
    f.write(value_bytes)
