#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        seed: int,
        length: int,
        out: BinaryIO):

    rng = open('/dev/urandom', 'rb')

    # Use an xorshift PRNG to make sure we have a good distribution of values in [0..255]
    # (We could do this differently by generating 16 bits at once, but that's less efficient)
    state = seed
    def next():
        nonlocal state
        x = state ^ (state << 13) & 0xFFFFFFFF
        y = x ^ (x >> 17) & 0xFFFFFFFF
        state = y ^ (y << 5) & 0xFFFFFFFF
        return state % 0xFF
    
    for i in range(length // 4):
        val = bytes([next() for _ in range(4)])
        out.write(struct.pack('<L', int.from_bytes(val, byteorder='little')))

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
