#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), 'big') & 0x7F # max length to avoid size explosions
    
    out.write(header)
    out.write(b'\xFF\x0BNETSCAPE2.0')
    out.write((datalen).to_bytes(1, byteorder='big'))

    data = rng.read(5 + datalen)

    out.write(data[:5]) 

    # keep the first three bytes as zero
    out.write(struct.pack('>I', 0)) 
