#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(4), byteorder='big', signed=False);

    print("Generating GIF with data len %d" % (datalen))

    # Write the header
    out.write(header);
    
    # Skip over the rest of the header for now -- we'll come back to it later
    out.seek(7, 1)

    # Generate the palette
    palette = [];
    while len(palette) < 256*3:
        palette += [
            next(rng) & 0xFF, 
            next(rng) & 0xFF, 
            next(rng) & 0xFF 
        ]

    # Write the palette and image descriptor
    for idx in range(len(palette)):
        if idx!= 0 and idx % 4 == 0:
            out.write(b'\x00')

        col = palette[idx]
        col_bytes = struct.pack('BBB', col & 0xFF, (col >> 8) & 0xFF, (col >> 16) & 0xFF)
        
        out.write(col_bytes)

    # Now that all color info is written, add the image descriptor block back
    out.seek(-7, 1)
    
    # Image width, height - both 1 pixel
    out.write(struct.pack('<HHBHBH', 
        1,
        1,
        False,
        1,
        0,
        0
    ))

    # LZW minimum code size
    out.write(b'\x08');

    # Finally write the bytes from /dev/urandom as-is up until the end of our data length
    sofar = 7 + 3 * 256 + 4;
    while sofar < datalen:
        chunksize = min(datalen - sofar, 2**12)
        buf = rng.read(chunksize)

        out.write(buf)
        sofar += chunksize