#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header is 6 bytes
    header = b"GIF89a"

    # Get width and height from urandom
    w = int.from_bytes(rng.read(1), "little") ^ ord("Z") % 0x7f + 1
    h = int.from_bytes(rng.read(1), "little") ^ ord("Y") % 0x7f + 1
    
    out.write(header)

    # Write global color table (always black)
    for i in range(256):
        out.write(b"\x00\x00\x00")

    # Write initial frame start code
    out.write(bytearray([0x21]))
    out.write(b"NETSCAPE2.0")
    out.write(b"\x03")
    out.write(struct.pack(">H", 1))
    out.write(b"\x00")

    # Write image descriptor block with offset to LZW compressed image data
    out.write(bytearray([
        0x2c,                # Image separator
        w & 0xff,            # Left position of image
        h & 0xff,            # Top position of image
        ((w >> 8) | 0x80) & 0xff,     # Width of image
        ((h >> 8) | 0x80) & 0xff,     # Height of image
        0x00,                # Flags (none set so far)
        0x00                 # Index of local color table
    ]))

    lzwmincodesize = 2
    datalen = 4 * w * h - 2;

    # Output the size of the decompressed data (big endian)
    out.write((datalen).to_bytes(2, byteorder="big"))

    # Generate data string (the compressed image)
    data = rng.read(datalen)
    out.write(header + data)
