#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Bytes 0x00-0x1F are reserved for the header.
    version = b'GIF89a'

    width = int(rng.read(2)) % (640 + 1)
    height = int(rng.read(2)) % (320 + 1)

    # Write fixed values to these bytes
    palette_color_resolution = 7
    pixel_aspect_ratio = 0

    screen_descriptor = struct.pack('<BHHBBB',
        0x2C,         # image separator block
        width, height,
        palette_color_resolution,
        pixel_aspect_ratio,
        0            # no color table follows
        )
    
    global_palette = [0] * ((width+height) // 3)+[255]*((6*256)-len((width+height)//3))
    out.write(version)
    out.write(screen_descriptor)
    out.write(bytes(global_palette))

