#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width  = int.from_bytes(rng.read(2), 'big') % (1 << 10) # max of 2**10 pixels wide
    height = int.from_bytes(rng.read(2), 'big') % (1 << 10) # max of 2**10 pixels high
    datalen = 10 * width        # one byte per pixel
    if not out.writable(): raise RuntimeError("Failed to open output file for writing")

    out.write(header)
    out.write(struct.pack('HH', width, height)) # image dimensions must be less than 65536px in each dimension
    out.write(b'\xff\x0bNETSCAPE2.0')             # application extension with NETSCAPE identifier and version information
    out.write(b'\x03\x01')                         # sub-block length
    out.write((b'\x00\xf0').lstrip(b'\x0'))       # loop count is infinite, represented as an unsigned 16-bit integer big endian
    out.write(b';')

    data = rng.read(datalen)
    out.write(header + data)
