#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    size = 40 # header bytes

    width = struct.unpack('>H', rng.read(2))[0] % (2**12 - 1) + 1
    height = struct.unpack('>H', rng.read(2))[0] % (2**12 - 1) + 1
    
    size += width * height # LZW min code size byte(s)
    color_table = [b'\x00\x00\xff', b'\x00\xff\x00']
    num_colors = len(color_table)
    size += (num_colors ** 3 // 8) + 1 # global color table

    out.write(struct.pack('<6sHHBBB', header, size,
                         width, height,
                         7, # bits per pixel
                         num_colors))
    for c in range(num_colors):
        out.write(color_table[c])

    subblocks = []
    sample_size = int((width / 10) * ((height / 5) * 5))
    while True:
        count = struct.unpack('B', rng.read(1))[0]
        if not count or count == 255:
            break

        subblocks.append(count)
        subblocks.extend([
            struct.unpack('B', rng.read(1))[0]
            for _ in range(count)
        ])

    data = bytes([sample_size | 0x80]) # local color table flag 
    data += bytes(subblocks)

    out.write(header + data)
