#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Write magic header
    out.write("GIF89a".encode())
    
    # Colors - maximum 256 colors
    colors = bytearray([0] * (7 + 4 * 256))
    for i in range(1 << 8):
        colors[i*4 : (i+1)*4] = bytearray([
            rng.read(1)[0], 
            rng.read(1)[0], 
            rng.read(1)[0], 
            0x00])
        
    out.write(colors)
    
    while True:
        
        # Block size
        blocksz = rng.read(1)[0]

        if not blocksz or blocksz == 0xff: break
        
        # LZW min code length
        lzwlen = rng.read(1)[0] & 0xf
        
        # Global color map - we don't support indexed images atm
        out.write(bytes([
            0x2c,
            blocksz,
            lzwlen]))

        out.flush()

