#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_random(out: BinaryIO, w: int = 1280, h: int = 720) -> None:
    header = (b'GIF89a', b'\x00\x20\x00\x00', b'\xff\xe0',
              bytes([w % 256, w // 256]), bytes([h % 256, h // 256]))

    palette = [(i, i, i) for i in range(256)] * 4 + [
        # Background colour.
        (255, 255, 255),

        # Foreground colour.
        (*struct.unpack('<H', out.read(2))[0].to_bytes(length=2, byteorder='big'),
         255),
    ]

    hdr = [len(header[i]) for i in range(len(header))]
    assert len(hdr) == 5
    total_size = sum(hdr) + 16 + 256 * 3 + 1

    write_color_table(out, palette)
    write_logical_screen_descriptor(out, total_size - 11)

    # Write remaining headers.
    for data in header:
        out.write(data)


def write_color_table(out: BinaryIO, palettes):
    for rgb in palettes:
        r, g, b = map(lambda c: min(max(c, 0), 255), rgb)
        out.write(bytes((r, g, b)))


def write_logical_screen_descriptor(out: BinaryIO, size: int) -> None:
    logical_screen_desc = struct.pack('BBBBBBBH',
                                      0xf7,   # Size of this block: 7.
                                      0x00,   # Has global color table?
                                       0x00,   # Colour resolution.
                                       0x00,   # Sorting of colours.
                                        0x00,   # GCT size.
                                        0x00,   # Background colour index.
                                        0x00,   # Pixel aspect ratio.
                                        size    # Image size; set later.
                                    )
    out.write(logical_screen_desc)


if