#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # 256 unique colors (first color is supposed to be transparent)
    colors = [bytes([int(c * 0xFF)]) for c in [(i + 1.) / 0xFF for i in range(256)]]

    palette_data = b''.join(colors)
    out.write((b"GIF89a"
               + bytes([256,7,0])
               + len(palette_data).to_bytes(2,"big")
               + palette_data))

    # Number of frames
    numFrames = int(rng() % 128)
    
    print("numFrames", numFrames)

    loopCount = None if rng() > 0.5 else 0
    print("loopCount:", loopCount)

    for currentFrameNumber in range(numFrames):
        frame_x = rng() * 128
        frame_y = rng() * 128

        print("%f %f" % (frame_x, frame_y), end=' ')

        lctFlag = False
        interlaceFlag = bool(rng())
        sortFlag = False
        
        localColorTableSize = 0
        bgIndex = 0
        pixelAspectRatio = 0

        while True:
            blockTypeByte = rng()

            if blockTypeByte < 0.125:
                # The first code specifies the number of bits per pixel

                sizeOfLZWMinimumCode = int(rng() * 3); 
                break;

            elif blockTypeByte < 0.25:
                # LZW Minimum Code Size
                lzwMinimumCodeSize = int(rng()*4) + 1
                print("lzwMinCodeSize: ", lzwMinimumCodeSize)

            elif blockTypeByte < 0.375:
                # Background Color Index
                bgIndex = rng() % 256   

            elif blockTypeByte < 0.5:
                # Pixel Aspect Ratio
                pixelAspectRatio = int(rng() * 16) + 1

            elif blockTypeByte < 0.625:
                # Global Color Table flag set
                lctFlag = True
            
            elif blockTypeByte < 0.75:
                # Local Color Table flag set
                lctFlag = False    
                
            elif blockTypeByte < 0.875:
                pass
    
            elif blockTypeByte <= 1.:
                # Disposal method: clear or not
                pass


        data = bytearray('Hello world', 'ascii')

        lzwClearCode = b'\0'

        lzwTable = {}
        lzwNext = 0
        lzwLast = ''
        lzwBuffer = []

        def flush():
            nonlocal lzwLast
            if lzwLast!= '':
                yield from lzwLast
                yield from b''
                lzwLast = ''

        def nextLzwValue(lzwLookAhead):
            nonlocal lzwNext

            if lzwLookAhead!= '':
                if lzwLookAhead in lzwTable:
                    yield from lzwTable[lzwLookAhead]
                    
                else:
                    lzwTable[lzwLookAhead] = bytes([lzwNext])

                    yield from b''

                    lzwNext += 1
                    assert lzwNext < lzwMinimumCodeSize
                    
                    lzwLast = lzwLookAhead
                
                return 

            value = sorted([(len(v), v) for v in lzwTable])[0][1]
        
            del lzwTable[value]
            yield from value

        with open("../examples/files/barbara.gif","rb") as f:
            lzwBuffer = list(f.read()[6:-1])
            
        
        
        with open("../examples/files/car.gif","wb") as f:
            blockType = {
                0 : "Extension Block", 
                1 : "Image Descriptor", 
            }[int(rng() * len(blockType))]
            print("blockType", blockType)
        
            f.write(struct.pack("<B", 1 << 7 | 1 << 6 | 1 << 5 | 1 << 4 | 1 << 3 | 1 << 2 | 1 << 1 | 1))
            
            f.write(struct.pack("<H", 0)) # Left position
            f.write(struct.pack("<H", 0)) # Top position
            f.write(struct.pack("<H", 32)) # Width
            f.write(struct.pack("<H", 32)) # Height
            f.write(struct.pack("<B", lzwMinimumCodeSize - 1))
            f.write(struct.pack("<B", 0 & 1 << 7 | 1 & 1 << 6 | 1 & 1 << 5 | 1 & 1 << 4 | 0 & 1 << 3 | 0 & 1 << 2 | 0 & 1 << 1 | 0 & 1 << 0 )) # Packed field: no transparency nor user input

            f.write(b'\xFF\x0F') # Application extension
            f.write(b'NETSCAPE2.0')
            f.write(b'\x03\x01')
            f.write(b'\x00\x00')

            f.write(b'\xFF\x0B') # Graphics Control Extension
            f.write(b'\x04')
            f.write(b'\x08\x00\x00\x00')
            f.write(bytes([0xff]))    
            f.write(b'\x00')    
            f.write(struct.pack("<H", 0)) # Delay time: 0 milliseconds

            f.write(b'\xFF\x2C') # Image Data Block
            f.write(b'\x05' + lzwClearCode + bytes([lzwMinimumCodeSize]) + b'\0\0\0')
            f.write(nextLzwValue(lzwBuffer.pop(0)))



        with open("/tmp/car.gif","w+b")as f:
            f.write(b'GIF89a')
            f.write(bytes([256,7,0]));
            f.write(bytes([0, 0, 255, 128, 0, 0, 0, 0, 0, 0]))
            f.write(b"\x00\x21\xF9\x04\x01\n\x00,\x00\x00\x00\x00\x03\x00\x01\x00\x00\x02\x01\x0D\x00\x00;\x00\x3B")
            f.write(bytes([int(rng() * 256)]))
            f.write(bytes([0x00]*16));
            f.write(bytes([0x04]));
            f.write(b'NETSCAPE2.0');
            f.write(bytes([3<<5|1]))
            f.write(bytes([0,0]))
            f.write(b'\x21\xFF\x0B\x00\x00\x03\x00\x01\xFF\x00\x00')
            f.write("\x02\x01\x0D\x00\x00".encode());
            delayTime = int(rng() * 100)+1;
            f.write(bytes([delayTime&0xFF]));
            f.write(bytes([(delayTime>>8)&0xFF]));
            f.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\xFE\x00\x01\x00')
            f.write(bytes([rng()*256]))
            f.write(b'!\xf9\x04\x00\x0d\x00\xe2\x00\x10\xfe\x11\x11\xfe\x22\x22\xfe\x33\x33\xfe\x44\x44\xfe\x55\x55\xfe\x66\x66\xfe\x77\x77\xfe\x88\x88\xfe\x99\x99\xfe\xaa\xaa\xfe\xbb\xbb\xfe\xcc\xcc\xfe\xdd\xdd\xfe\xee\xee\xfe\xef\xef\xfe\xff\xff\x00\x00\x01\x01\x02\x02\x03\x03\x04\x04\x05\x05\x06\x06\x07\x07\x08\x08\t\t\n\n\x0b\x0b\x0c\x0c\r\r\x0e\x0e\x0f\x0f\x