#!/usr/bin/env python3

import struct
from binascii import hexlify, unhexlify


def pack(fmt, *args):
    return struct.pack('<' + fmt, *args)


def unpack(fmt, data):
    return struct.unpack_from('<' + fmt, data)[0]


def check_field(data, fieldname, expected):
    actual = unpack('{}s'.format(len(fieldname)), data[4:])
    if (actual!= bytes(fieldname, 'utf-8')):
        raise Exception("Expected {} but got {}".format(fieldname, repr(actual)))

    next_posn = 4 + len(fieldname)

    assert(next_posn % 2 == 0) # optimistically assume it's aligned
    if next_posn < len(data):
        value = data[next_posn:]
    else:
        value = b""

    #print("{}:{}; {}".format(fieldname, repr(value), repr(expected)))
    print("{}; {}".format(repr(value), repr(expected)))
    assert(value == expected)


def test_tag(t, n, v):
    tag = pack('>HH', t, n)
    if isinstance(v, int):
        length = pack('!H', v)
    elif isinstance(v, str):
        length = pack('!H', len(v)) + bytes(v, "utf-8")
    elif isinstance(v, list):
        length = bytes([len(v)]) + bytes(v)
    else:
        raise Exception("Bad type for field value", type(v))
    
    print("{}".format(hexlify(tag+length).decode()))
    


