#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Write header to out.
    out.write('GIF89a'.encode('ascii'))  # Header
    width = rng.randint(40,60); height = rng.randint(20,50)

    # Write screen descriptor block to out.
    packedWidthHeight = struct.pack('<HH', width, height)


    background = 0x7f; showPackedField = False
    if (width >= 100 and height >= 100):
        showPackedField = True
        packedField = "abcd"   # I couldn't figure this out yet!

    packedField = str()
    for i in range(0, 4):
        packedField += chr((background >> (i * 8)) & 0xFF)
    
    packedBackgroundColor=struct.pack('<BBBB' + 'B' * len(packedField),
                           0xF7,
                           *(ord(c) for c in packedField))
    packedFlags = 0b10000000 | int(showPackedField) << 7
    flags = struct.pack("<B", packedFlags)
    print("packedBackgroundColor=%s"%repr(packedBackgroundColor))
    #print("flags=%s"%str(hex(int.from_bytes(flags, byteorder='little'))))
    with open("/tmp/test","wb") as f:
        #f.write(packedWidthHeight)
        f.write(flags)
        f.write(packedBackgroundColor)

    # Write color table of size 256 bytes.
    #for i in range(0, 256):
    #    hexColor = '%02x%02x%02x' % (rng.randint(0,255), rng.randint(0,255), rng.randint(0,255))
    #    rgbColorBytes = [int(hexColor[i:i+2], 16) for i in (0, 2,4)]
    #    out.write(bytes(rgbColorBytes))

    # Write LZW code length data
    maxCodeLen = 8
    packedMaxCodeLen = ord(maxCodeLen)
    lzwMinCodeLen = 2
    lzwMaxCodeLen = min(lzwMinCodeLen + maxCodeLen - 1, 10)
    gifLzwData = b'\xff\xe0\x02\xf9\x04\xb1\x01\x00'

    # Write image descriptor block to out and write pixel data
    xPos = rng.randint(0, width-1)
    yPos = rng.randint(0, height-1)
    packedXPosYPos = struct.pack('<HH', xPos, yPos)
    packedWidthHeight = struct.pack('<HH', width, height)
    packedFlags = 1 << 7
    flags = struct.pack("<B", packedFlags)
    localPixels = []
    nextPixel = None
    for i in range(0,height*width):
        if nextPixel is not None:
            localPixels.append(nextPixel)
            nextPixel = None
        else: 
            # Get next pixel from gifLzwData or randomly assign one.
            lzwDataIndex = rng.randint(0,len(gifLzwData)-1)
            colorIndexByte = gifLzwData[lzwDataIndex]
            hasLocalTable = colorIndexByte == 0xFF
            if hasLocalTable:
                colorIndexByte = rng.randint(0, 255)

            # If we have seen this index before then get its corresponding byte value otherwise use it directly.
            colorIndexByteValue = colorIndexByte if colorIndexByte < lzwMinCodeLen \
                    else 1 << (colorIndexByte - lzwMinCodeLen)

            nextPixel = colorIndexByteValue if hasLocalTable else 0
    tmp = len(localPixels) and localPixels[-1] or 0
    while tmp > 255:
        localPixels.append(tmp // 255)
        tmp &= 255
    localPixels.append(tmp)
    pixels = list(map(lambda p: struct.pack('<B',p), localPixels))
    pixelsString = b''.join(pixels)
    out.write(packedWidthHeight)
    out.write(packedXPosYPos)
    out.write(flags)
    out.write(pixelsString)


