#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''
Created on Thu Jan  2 15:49:07 2019

@author: arthurd


A random walk is a path chosen by a random walker in the plane. Each step he takes is a random direction, independently of all previous steps taken. The resulting paths might not be symmetric (for example, they might take the same number of steps but go in different directions). As with other random walks this process has some interesting properties that can be studied. For instance we can calculate the probability to end up at any point for a given starting point. 

For the sake of simplicity let's start from one dimension and consider only two possible moves : left or right. We'll represent these movements as +1/-1.


Few examples :
    Walk(0) = [0]
    Walk(1) = [-1,0,1]
    Walk(-1) = [-1,0,1]
    
We want to compute the probability of ending up at a given value x after n steps. To do so we need to count how many times it appears in a given list. That's exactly what np.count_nonzero does!
    
        >>> np.count_nonzero([1,-1,0])
        2
        >>> np.count_nonzero([-1,-1,-1])
        0
'''
import numpy as np
from matplotlib import pyplot as plt

def plotWalk(walk):
    '''
    Plots a walk
    
    Parameters
    ----------
    walk : array like 
        a sequence containing the absolute values of each step of the walk.
    '''
    fig=plt.figure()
    ax=fig.add_subplot(111)
    x=[i*np.sign(v) for i, v in enumerate(walk)]
    y=[abs(v) for v in walk]
    ax.bar(x,y,align='center',color='blue')
    plt.show();
    
    
def computeProbDistributionFromWalk(walk):
    '''
    Computes the probability distribution from a sequence of walks

    Parameters
    ----------
    walk : array like 
    A sequence containing the walks.
    
    Returns
    -------
    probaDistribution : array like
        a sequence containing the probabilities associated to each step in seen walks 

    '''   
    # create an empty dictionary where keys are steps observed in all walks and values are frequency
    freqDistribs={}
    for w in walk:
        for s in w:
            if s == -1:
                continue;
            elif s==1:
                try:
                    freqDistribs[str(s)]+=1
                except KeyError: 
                    freqDistribs[str(s)]=1
            else: 
                raise ValueError('Steps must be either 1 o -1');
    
    
    # fill missing frequencies with zeros
    maxStepFound = str((max(list(freqDistribs))))
    minStepFound = str((min(list(freqDistribs))))
    nbOfMissingValues = int(maxStepFound)-int(minStepFound)+1
    for i in range(-nbOfMissingValues+1):
        key = str(i)
        if key not in freqDistribs.keys():
            freqDistribs[key]=0
            
    return np.array(list(freqDistribs.values()))/len(walk);
            

    


