#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # The screen descriptor (10 bytes):
    #  - 2-byte screen width;
    #  - 2-byte screen height;
    #  - packed byte encoding the color table size and other flags.
    sdesc = struct.pack("!HHB",
                        rng.read(2)[::-1],   # Width of screen, high byte first
                        rng.read(2)[::-1],   # Height of screen, high byte first
                        7                     # Color table is included in frame
                    )

    out.write(b"GIF89a")    # Magic string at beginning of file
    out.write(sdesc)       # Screen descriptor
    out.write(b"\x80\xf7")  # Global color table + "no local map" flag
    for _ in range(64*5):
        out.write(b"\xff\xff\xff")      # Write an entirely opaque white block to the frame
    out.write(struct.pack("BB", 0x3b, 0))     # Terminating image separator

