#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0x7fff

    while datalen > 0:
        chunklen = min([datalen, len(header)])

        if chunklen == len(header):
            # We have the whole header already
            pass
        elif chunklen < len(header):
            out.write(header[:chunklen])

            # We finished writing this chunk of data; get another one
            header += rng.read(1 + (len(header) - chunklen))
        else:
            raise Exception("Impossible case.")

        chunktype = header[6]
        out.write(chunktype)

        sublenstr = rng.read(2)
        sublen = int.from_bytes(
                sublenstr, byteorder="big") & 0xffff

        if chunktype in [b'!', b',', b';']:
            # Logical Screen Descriptor block is 7 bytes long (not including LSD itself)
            sublen -= 7
        elif chunktype not in [0x2c,]:
            print(f"Unknown type {hex(chunktype)}. Continuing...")

        subdata = rng.read(sublen)
        out.write(sublenstr + subdata)

        datalen -= 4 + sublen

