#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    blocksize = 256 * 4 + datalen

    # Data (block size in bytes and then the image data itself)
    out.write(struct.pack('!B', blocksize % 256))
    out.write(b'\0\0')
    out.write(datalen.to_bytes(3, 'big'))
    out.write(rng.read(datalen))
    
    # Trailer (0x3B)
    out.write(b'\x3B')
