#!/usr/bin/env python3
import argparse, sys, os.path
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord


def parse_args():
    parser = argparse.ArgumentParser(description="")
    subparser = parser.add_subparsers()

    split_p = subparser.add_parser("split", help="Split fasta file into 100MB+ chunks")
    split_p.set_defaults(func=split)
    split_p.add_argument("-i", "--input", required=True, type=str, metavar="<file>",
                         help="Input fasta file.")
    split_p.add_argument("-o", "--output", required=False, default="-", type=str, metavar="<dir>",
                         help="Output directory for splitting files [default <stdout>].")
    split_p.add_argument("--prefix", required=False, type=str, metavar="<string>", default="",
                         help='Prefix for output split files (e.g., "chunk_" to produce "chunk_1.fasta").')
    
    args = parser.parse_args()
    return args


def split(args):
    chunk_size = 1024 * 1024 * 100 # 100 MB
    chunk = []
    f_out = None
    i = 0
    f_in = open(args.input, 'r') if isinstance(args.input, str) else args.input
    for record in SeqIO.parse(f_in, format="fasta"):
        seq_len = len(record.seq)
        n_chunks = int((seq_len + chunk_size - 1) / chunk_size)
        if not n_chunks:
            n_chunks = 1
        elif seq_len % chunk_size!= 0 or n_chunks == 1 and seq_len >= chunk_size:
            n_chunks += 1

        if not f_out or i > n_chunks - 1:
            i = 0
            if f_out is not None:
                f_out.close()
            outname = "{}{}.fasta".format(os.path.join(args.output, args.prefix), i)
            while True:
                try:
                    with open(outname, 'x'):
                        break
                except FileExistsError as e:
                    print("{} exists!".format(outname))
                    i += 1
                    outname = "{}{}.fasta".format(os.path.join(args.output, args.prefix), i)

            f_out = open(outname, 'w')
        
        chunk.append(record)
        i += 1
        if i == n_chunks:
            SeqIO.write(chunk, f_out, format="fasta")
            chunk[:] = ()
            i = 0

    if len(chunk) > 0:
        SeqIO.write(chunk, f_out, format="fasta")
    
    f_in.close()
    if f_out is not None:
        f_out.close()
    

if