#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    width  = rng.read(2) # uint_16
    height = rng.read(2) # uint_16

    screen_descriptor    = b''.join([width, height]) + bytes([0xF7])
    global_color_table   = (b'\x4E\xC5\xEA\x4D' +
                            ((bytes([0xFF]) * 58) +
                             bytes([0x00] * 19)))
    
    loop_control         = b''

    extra_flags          = 0x1C | 0x80 # 0b11001100 | 0b10000000

    application_extension= b''.join((bytes([0x21]),
                                    bytes([extra_flags]),
                                    bytes([0xF9]),
                                    bytes([0x04]),
                                    bytes([0x05]))) + \
                           b'NETSCAPE2.0' + \
                           bytes([0x03,
                                  ord('2'),
                                  ord('.'),
                                  ord('0')]) + \
                           bytes([0x01, 0x00])

    block_size           = len(application_extension).to_bytes(1, 'big')

    subblock             = (bytes([0x0B])      +
                           bytes([0x00])      +
                           bytes([0x05])      +
                           bytes([0x00]))


    data                 = (b''                +
                           bytes([0x00])      +
                           bytes([0x00])      +
                           bytes([0x00])      +
                           bytes([0x00],       # 0x00
                                  0x00, 0x00))   # 0x00

    graphic_ctl_ext      = bytes([(len(data)+1)]) + \
                           bytes([0x00])     + \
                           data              + \
                           bytes([0x00])     # Trailing NULL byte

    extension            = bytes([0x21])     + \
                           bytes([0xff])     + \
                           block_size        + \
                           bytes([0xf9])     + \
                           subblock          + \
                           bytes([0])        + \
                           bytes([0])        + \
                           bytes([0])        + \
                           bytes([0])        + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x00])     + \
                           bytes([0x