#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )
    width = 2 << ((datalen >> 4) & 0x07)      # [16..512] in steps of 16
    height = (datalen & 0x0F) * 2            # [1..16] in steps of 2

    chunks = []

    # Header
    chunks.append((header + bytes([width // 256, width % 256])
                   + bytes([(height + 1) // 256, (height + 1) % 256])))

    # Logical screen descriptor
    chunks.append(struct.pack('>HBBBBB',
                               width, height,
                               0xF0,          # Global Color Table Flag
                               7,             # Sorting GCP Flag
                               -1, 8))        # Background color index and Size of global color table

    # Trailer
    chunks.append(b'\x3B')

    out.write(b''.join(chunks))              # Write all data at once

