#!/usr/bin/env python3

import struct
with open('flag', 'rb') as f: flag = f.read()


seed = 0x4d9f782b0e5c71a8fd48ebc6c68897cc # fixed seed to get the exact same output every time
rng = struct._create_struct_random(
        stype=struct.Struct('<I'), seed=seed, state=None
        )
for i in range(len(flag)):
    assert len(flag[i].to_bytes(1, byteorder='little')) == 1
    print(flag[i])
    randbyte = rng.read(1)[0] & (ord('Z')-1) | ord('A')
    newbyte = ((randbyte - ord('A')) ^ ord(flag[i])) + ord('A')
    assert chr(newbyte).isalpha(), \
           "Randomness changed by too much; try again."
    with open('flagout', 'ab+') as fout:
        fout.seek(-1, 2)
        fout.write(chr(newbyte))


# cat flagout; echo
# PE1{seCrEt_gUessing_W3nt_y0u_tO_come_upwItH_alOt_moRe__r3S~~_hTTps://tinyurl.com/EDA5EHCp}


# interactive way of generating this file
# >>> from hashlib import sha256
# >>> import string
# >>> flagstr = ''.join([ch if ch in string.printable else '.' for ch in flag]).encode()
# >>> with open('/dev/urandom', 'rb') as fin: h = sha256((fin.read()+flagstr+b'\n').strip()).hexdigest(); print(','.join(['\\x'+h[8*i:8*(i+1)] for i in range(len(h)//2)]))
# 0x89,0xd8,...,...


# create a GIF out of all those random bytes
with open('flag.gif', 'wb') as out, \
     open('/dev/urandom', 'br') as rng, \
     open('flagout', 'rb') as fout:
         ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
