#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width  = bytearray([rng.read(1)[0] % (2**4)])[0] + 64
    height = bytearray([rng.read(1)[0] % (2**4)])[0] + 64
    flags  = [
        bool(rng.read(1)[0] & 1), # Has color table
        True,                    # Interlaced
        False,                   # Sorted
        7 == (rng.read(1)[0] & 0x7), # Size of local color table
        ]

    global_colors   = max((width * height // min(width,height)) // 512, 2) - 1
    use_local_table = any(flags[:3]) and not all(flags[:3])
    if use_local_table ^ flags[3]:
        global_colors += 1

    colors = bytes()
    for i in range(global_colors):
        red   = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue  = rng.read(1)[0] % 256

        colors += struct.pack('<BBB', red, green, blue)

    subblocks = [b'', b'\xFF\x00']
    for i in range(len(subblocks)):
        subblocks[i] += colors

    flags[0] |= bool(colors)

    out.write(header)
    out.write(bytes([width]))
    out.write(bytes([height]))
    out.write(bytes([(2 << int(flags[3])+int(flags[0]) |
                      3 << int(flags[2])+int(flags[1]))]))

    for block in subblocks:
        while len(block):
            out.write(bytes([min(len(block), 0xff)]))
            out.write(block[:min(len(block), 0xff)])

            block = block[min(len(block), 0xff):]
