#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, datalen=1024):
    # Magic:     'GIF8' (Version 87a, 256 color indexed)
    header = b'GIF89a'

    # Screen Descriptor
    #              |   Width           |   Height          |
    # Bits Per Pixel: 01101100      Color Resolution: 00000          |
    # Sort Flag:      0                            Global Color Table: 1
    # Size of GCT:   01000000                       Background Color Index: 00
    #                      |         Pixel Aspect Ratio        |
    #                      ---------------------------------
    #                       00100000
    screendescriptor = bytes([
        0b01101100,
        datalen & 0xFF,
        (datalen >> 8) & 0xFF,
        (datalen >> 16) & 0xFF,
        0x80 | ((datalen >> 24) & 0x0F),
        0b01000000,
        0,
        0,
        0,
        0,
        0x40])
    
    # Graphics Control Extension
    gce = bytes([
        0x21,
        0xF9,
        0x04,
        0x01,
        ord(' '),
        0x00,
        0x00
        ])

    # Application Extention
    apext = bytes([
        0x21,
        0xFF,
        0x0B,
        0x4E,
        0x45,
        0x54,
        0x53,
        0x43,
        0x41,
        0x50,
        0x45,
        0x32,
        0x2E,
        0x30,
        0x03,
        0x01,
        0x00,
        0x00,
        0x00
        ])

    out.write(header + screendescriptor
              + gce + apext
              rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
