#!/usr/bin/env python3

import time
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The argument `seed` is only used for testing purposes.
def generate_random_gif(
        seed=None, out: BinaryIO
    ) -> bytes:

    if seed is None:
        seed = time.time()

    # Seed the RNG with a 64-bit integer in big endian format
    rng = struct.Struct('>Q').pack(int(seed))

    return b''.join([
        b'GIF89a', # Header
        struct.Struct('>L').pack(len(b'data')), # Data size field
        0xF7, # Application extension introducer

        struct.Struct('>L').pack(12), # Size of application ID subblock

        0x52, 0x49, 0x50, 0x4C, # "RIP" as an application identifier - ASCII
        0x00, # Null separator
        
        bytes(), # Application data (none here!)

        0x00, # Block terminator

        0x21, 0xFF, 0x0B, # Graphics control label & size

        0x4D, 0x00, # Disposal method = leave as is
        0x00, # User input flag = ignore
        0x00, 0x00, # Delay time = 0ms

        0x00, 0x00, # Transparent color index
        0x00, 0x00, # Block terminator

        0x00, 0xFF, 0x00, # Image descriptor label and size

        0x00, 0x00, # Left offset = 0px
        0x00, 0x00, # Top offset = 0px
        0x10, 0x00, # Width = 16px
        0x0A, 0x00, # Height = 10px
        0x00, # Bit depth - set to zero since we have no palette
        0x00, # LZW minimum code size
        0x00, # Block terminator

        # Finally our actual image's pixel data!
        0x2c, 0x00, 0x00, 0x00, 
        0x00, 0x00, 0xff, 0xc0, 
        0x0e, 0x00, 0xff, 0xa0, 
        0x0f, 0x00, 0xf3, 0x80, 
        0x0f, 0x80, 0x01, 0x00, 
        0x0d, 0xe0, 0x00, 0x00, 
        0x0d, 0xd0, 0x00, 0x00, 
        0x0e, 0xb8, 0x00, 0x00, 
        0x0d, 0x9c, 0x00, 0x00, 
        0x1f, 0xbf, 0x80, 0x00, 
        0x2d, 0xef, 0xff, 0xc0, 
    ])