#!/usr/bin/env python3

import struct
from typing import BinaryIO

def write_int16le(out: BinaryIO, val: int):
    """Write an signed 16-bit integer to the provided binary stream in little endian format."""

    # Ensure that we are working with a positive value since Python's struct module does not have
    # support for this by default.
    assert val >= 0
    if val > (2**15 - 1):
        raise ValueError("Cannot encode {} as a 2 bytes integer".format(val))
    
    return out.write(struct.pack('<h', val & 0xffff))
    

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF87a'
    width = ord(rng.read(1)[0]) % 128 + 64; # Randomly select a width between [64..191]
    height = ord(rng.read(1)[0]) % 128 + 64; # and also randomly choose a height similarly.
    write_int16le(out, width * height); # Write the pixel count of our image.
    colordepth = rng.read(1)[0]; # Draw a random (color depth) palette size from the first byte of /dev/urandom.
    out.write(b'\x87\x00') # Write the two fixed values required for truecolor images according to the specification.
    out.write(rng.read((width+height)*colordepth)); # Then draw a random color palette from the rest of /dev/urandom (after skipping one byte because it is always zero).