#!/usr/bin/env python3

import os
import sys
from typing import BinaryIO


# Generates a random GIF file
def generate_random_gif(
        width: int = 640, height: int = 480,
        colorbits: int = 24, numframes: int = 15,
        fps: float = 1.0, outfile: str = 'random.gif',
        seed: int = None):

    # Open /dev/urandom if needed and read some entropy in it
    r = os.fdopen(os.dup(sys.stdin.fileno()), "rb") \
          if not seed else open("/dev/urandom", "rb")
    s = r.read(width * height * (colorbits // 8))
    r.close()

    # Open output file
    w = open(outfile, "wb")

    # Write header
    w.write("GIF89a".encode())

    # Compute LZW compression of image data
    size, lzwsize = write_graphic_control_extension(w,
                                                      width=width,
                                                      height=height,
                                                      numframes=numframes,
                                                      fps=fps)

    # Compute palette colors for depths <= 8-bit
    palette = compute_palette(s, colorbits)

    # Write graphics control extension
    write_image_descriptor_and_imgdata(w,
                                        width=width,
                                        height=height,
                                        uselzwcompression=(len(s) > lzwsize),
                                        rawcolors=True,
                                        palette=[[p] for p in palette],
                                        bitsperpixel=colorbits,
                                        imgdata=s[:lzwsize])

    while len(s) - lzwsize >= (width * height * (colorbits // 8)):

        # Add frame to GIF animation
        if len(s) < size or len(s) == size:
            break
        s = s[(width * height * (colorbits // 8)):]

        # Write graphic control extension
        write_graphic_control_extension(w,
                                         isfirstframe=(not w.tell()),
                                         islastframe=(len(s)==0),
                                         disposalmethod=2,
                                         numframes=numframes+1,
                                         fps=fps)


        lzwsize = min(int(len(s)), int((len(s)-lzwsize)/2)+lzwsize)
        write_image_descriptor_and_imgdata(w,
                                            width=width,
                                            height=height,
                                            uselzwcompression=False,
                                            bitsperpixel=colorbits,
                                            imgdata=s[:lzwsize],
                                            rawcolors=True,
                                            palette=[[p] for p in
                                                    [next(iter(palette))] * ((1<<colorbits)//3)])


    # Close gif file and return
    w.close()
    return outfile
