#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write signature (6 bytes)
    out.write('GIF89a'.encode())

    # Determine how many frames to write
    numframes = 1 + int.from_bytes(
            rng.read(1), 'little', signed=False)

    for i in range(numframes):

        # Create the control block of frame size and delay time
        ctrlblk = struct.pack('<2H',
                0x2c, 0x0b | ((i+1)<<4))

        # Determine the frame size from the random number generator
        framelen = 784 + \
                int.from_bytes(
                    rng.read(1), 'little', signed=True)*4
        
        # Write the control block
        out.write(ctrlblk)

        # Write the LZW stream of image data
        lzwstream = {
            0: [0],
            1: [0, 1],
            2: [0, 1, 2],
            3: [0, 1, 3]
        }[int.from_bytes(rng.read(1),'little')]
        for _ in range((framelen - 11)//2):
            lzwstream += [lzwstream[-1]+1]
            if len(lzwstream)<128:
                lzwstream += list(range(
                            len(lzwstream)-1)+range(
                                len(lzwstream)))
        out.write(struct.pack('<{}H'.format(len(lzwstream)),*lzwstream))