#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# If `size` is None then the size will be randomly selected.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, *,
        size: int=None):

    # The Gif Header
    # We're going to use version 87a but it doesn't seem to matter which one since we won't try parsing this
    # TODO: Better way of handling versions?
    header = b'GIF89a'

    if not size:
        # 0-255, inclusive
        r_size = rng.read(1)[0]

        # Set to very small image sizes

        # 16 x 16
        if r_size <= 48:
            size = 53
        elif r_size <= 64:
            size = 55
        else:
            size = (r_size % 32) + 29

    print('Using size', size)

    # Write the header and size
    out.write(header + bytes([size]) + b'\x00\x00\x00')

    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    
    for _ in range(datalen // 256):
        out.write((b'\x00'*256))

    remaining = datalen - ((datalen//256)*256)
    out.write(b'\x00' * remaining)

