# Copyright (c) 2014-2015 <NAME> <<EMAIL>>
# This file is licensed under the MIT license.

"""Generate a random GIF image."""


__all__ = ['generate']


import io
import os

from.gif import *
from..util import *


def write_random_palette(
        rng: Random, stream: BinaryIO
    ) -> None:
    """Write a new palette to the given stream in GIF format."""
    
    # The first color will be black
    colors = [Color(0x00, 0x00, 0x00)]

    for i in range(7):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)

        colors += [Color(r, g, b)]

    # Discard this palette by reading it again from the stream and discarding
    # what was read. We can't just set the global palette since that's shared
    # between all frames generated by this function call.
    if len(colors) <= 2**n_bits_per_index():
        raise Error('too many unique colors!')
        
    palstream = io.BytesIO()
    GifWriter._write_logical_screen_descriptor(
        size=(len(colors)-1).to_bytes(length=2, byteorder='little'),
        bgcol=b'\xff',
        usemap=False,
        localpal=True,
        stream=palstream)

    GifWriter._write_palette(colors[1:], stream=palstream)
    palstream.seek(0, os.SEEK_SET)


    # Write a frame with our color palette
    descriptor = FrameDescriptor()
    descriptor.hascolortable = True
    descriptor.width = b'\xff\xff'
    descriptor.height = b'\xff\xff'
    descriptor.left = 0
    descriptor.top = 0
    descriptor.interlace = False
    descriptor.sortbit = False
    descriptor.localcolor = True
    descriptor.reserved = False

    writer = GifWriter(outstream=stream)
    writer.add_frame(data=[b'\x00'], descriptor=descriptor)
    writer.set_global_palette(palstream)
    writer.close()