#!/usr/bin/env python3

import struct
import sys
from os import urandom
from typing import BinaryIO


# Generates a random GIF file into `out`. Uses the RNG given in `rng`, or stdin if None is used.
def generate_random_gif(
        rng: BinaryIO=None, out: BinaryIO=sys.stdout
    ):

    # Get the desired datalength and read it from the RNG
    if rng == None:
        rng = sys.stdin

    datalen = 0x1024 * (struct.unpack('B', rng.read(1))[0] & 7)

    # Generate an encoded datalength field with maximum length of 0xFFF
    encdata = []
    while datalen!= 0:
        encdata += [datalen % 0xff]
        datalen //= 0xff
    
    encdata += [0x3B]


    # Write the output to stdout
    frame = b'\x2c\x00\x00\x00\x00\x00\x00\x00!#\xf9\x00\x00\x00,\x00\x00\x02\x00\x01\x00;\n\x00\x04' * 65 + \
             b'\xff\xff\xff\xff' + \
             b'\xff' * ((len(frame) - len('\x2C\x00\x00\x00\x00\x00\x00\x00')) // 3)
    out.buffer.write(b''.join([
        b'GIF89a',
        bytes((encdata[::-1])),
        b'\x00\x00\x00\x00\x03\x00\x08\x00',
        frame
    ]))



