#!/usr/bin/env python3

import os
from tempfile import TemporaryFile

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom). The size of |data| is given in
# bytes and should be 4096 or greater. An invalid value will terminate the
# program with EXIT_FAILURE.
if __name__ == '__main__':
    try:
        if os.environ['USER']!= 'root':
            raise OSError('This script must run as root!')

        # Open /dev/urandom for reading
        urandom = open('/dev/urandom', mode='rb')
        
        # Read the first few bytes from the stream to ensure we have enough data available to read
        urandom.seek(1025)
        buff = urandom.read()
        
        # Determine the length (in bits) of the data
        datalen = len(buff)*8

        # Compute the adjusted size (in bytes) of the data based on the length computed above
        datalen -= 1025 - 7
        datalen += 7 
        datalen //= 8

        # Prepend the required GIFF block header to the raw data stream
        buff = b'\x21\xF9\x04' + struct.pack('>H', datalen) + b'\xFF' + buff[:datalen]

        # Write the resulting GIF image to stdout so that it can be piped directly to another program
        print(b'GIF89a' + buff[8:])

    except Exception as e:
        exit(f"Failed to generate random GIF!\n{e}")
    finally:
        urandom.close()