#!/usr/bin/env python3

import struct

# Generates a random GIF file into `f` using the
# random number generator `rng` (by default /dev/urandom)
def generate_random_gif(
        f: BinaryIO, rng=None):

    # Header
    f.write(b'GIF89a')
    
    # Logical screen descriptor block size and fields
    # (width, height in pixels, packed field; color table included)
    f.write(struct.pack('<H', 47))
    f.write(struct.pack('<HHB',
        1024, 576, 0x87))
    for i in range(0, 48):
        f.write(b'\x00')

    # Global color table (no colors defined yet)
    f.write(b'\x00'*1024*3)

    # Extension blocks until we get to image data block
    while True:

        # Block control character:
        #   extension: 247 or 255
        #   1st frame: 254 or 255
        #   2nd+ frames: 255 only!
        ctrl_char = ord(rng.read(1)[0]) % 3 + 246

        if ctrl_char == 247:

            # Plain text extension: not supported by image viewers anyway
            blksize = rng.read(1)
            label_len = rng.read(1)
            label = rng.read(label_len)
            text_block = rng.read(int.from_bytes(blksize)-1-label_len)

            # Skip this whole block...
            continue
        
        elif ctrl_char == 254 or ctrl_char == 255:

            # Graphic control extension: no actions required
            f.write(struct.pack("<BBBBB",
                3,
                ctrl_char,
                0xFF & ~ctrl_char,
                0,
                0))

            # Skip this whole block...
            continue
            
        else:
            
            # Image descriptor (left off where it should be...)
            f.write(struct.pack("<BBBBHH",
                4, 
                ctrl_char, 
                1024, 576, 
                0, 0))
            break

    # Image data block
    imgdata = rng.read(1024 * 576 // 8)
    f.write(imgdata)