#!/usr/bin/env python3

import argparse
import os
import sys
import base64

from PIL import Image  # type: ignore


# Generates a random image in PNG to stdout
class RandomImage:

    def __init__(self, width, height):
        self._width = width
        self._height = height

        if (width <= 0 or
                height <= 0):
            raise ValueError('Invalid dimensions for image.')


    @property
    def width(self):
        return self._width


    @property
    def height(self):
        return self._height


    # Generate a PNG with some random data
    def png(self, f=sys.stdout.buffer):
        img = Image.new('RGB', (self.width, self.height))

        for x in range(img.size[0]):
            for y in range(img.size[1]):

                r = ord(os.urandom(1)) & 255
                g = ord(os.urandom(1)) & 255
                b = ord(os.urandom(1)) & 255

                img.putpixel((x,y),(r,g,b))

        img.save(f, 'PNG')
    

    # Base64-encodes a PNG
    def base64(self):
        bio = io.BytesIO()
        self.png(bio)

        encoded = base64.b64encode(bio.getvalue()) 
        return str(encoded)[2:-1]



if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Generate a random.gif file.')
    parser.add_argument('--width', action="store", dest='width', type=int, default=1024)
    parser.add_argument('--height', action="store", dest='height', type=int, default=768)
    
    args = parser.parse_args()

    rimg = RandomImage(args.width, args.height)
    print('data:image/gif;base64,'+rimg.base64(), end='')