#!/usr/bin/env python3

import hashlib
import struct
from typing import BinaryIO

# A sequence of bytes which is deterministically generated from a given seed
class RNG:

    def __init__(self, seed):

        # The hash function we'll use to compute our output
        self._hasher = hashlib.md5()

        # Add in some salt to avoid producing identical results for different seeds
        self._hasher.update('Bacon is not a vegetable'.encode())

        # Generate some entropy by feeding it all of the characters in the seed string; this
        # may not be necessary but I'm too lazy to think about whether it's important or not
        for char in seed:
            self._hasher.update(char.encode())

        # Turn our hashed message digest into an initial state for our PRNG
        self._state = [int.from_bytes(
                self._hasher.digest(), 'big')] * 4

    # Returns a pseudorandom integer in the range (0, num)
    def next_int(self, num):

        # Update the internal MD5 hash with the previous state and extract the new state as raw bytes
        self._hasher.update(struct.pack('<I', self._state[0]))
        self._state = list(map(ord, self._hasher.digest()))

        # Convert our new state into a 16-bit unsigned integer
        return struct.unpack('<H', struct.pack('<4B', *self._state))[0] % num


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: RNG, out: BinaryIO
    ):
    header = b'GIF89a'; width = height = 256
    pixeldata = []
    for _ in range(width*height):
        r = g = b = min(max(rng.next_int(257), 1), 255)
        pixeldata += ((b >> 5 & 0x07) << 5 |
                      (g >> 5 & 0x07) << 2 |
                       (r >> 6))
    framedata = struct.pack(f'<{len(pixeldata)}B', *pixeldata)
    datalen = len(framedata).to_bytes(3, byteorder='big')
    out.write(header + datalen + framedata)
