#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (any object with a method called read())
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    def _write_block():
        blocklen = len(data).to_bytes(1, 'big', signed=False)
        out.write(b'\x21') # Extension introducer
        out.write(b'NETSCAPE')  # Application extension label
        out.write(b'\x0B\x00\x00') # Block length
        out.write(b'APNG')   # Netscape application name
        out.write(b'\xFF') # Application block terminator
        out.write(blocklen)
        out.write(data)

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen - 4)
    _write_block()
    out.write(header + data)

