#!/usr/bin/env python3
"""Generates a random GIF file with 1024 bytes of data"""

import sys
import os

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header
    out.write('GIF89a'.encode())

    # Logical Screen Descriptor : 7 bytes (16 bits each)
    colors = [b'\x00', b'\x00'] # No color table used
    background_colorindex = 0x0F # Transparent color index
    width = 64
    height = 64
    gctsize = 0x00 # No global color table
    flags = b'\x00\xFF' # Global color table exists and sorted
    packed = b''.join([colors[0], colors[1],
                struct.pack(">B", background_colorindex),
                struct.pack(">H", width),
                struct.pack(">H", height),
                struct.pack(">B", gctsize),
                flags])
    _logicalscreendescriptor = packed
    out.write(_logicalscreendescriptor)

    # Global Color Table
    for i in range(0, pow(2,gctsize)):
        p = b'\xFF'*3
        out.write(p)

    # Image Descriptor : 9 bytes (16 bits each)
    xcoord = 0
    ycoord = 0
    imagewidth = 64
    imageheight = 64
    localgctsize = 0x00 # No local color table
    interlaceflag = False # Non-interlaced
    sortedgct = True    # Local color table is sorted
    lctleft = 0
    packed = b''.join([struct.pack(">H", xcoord),
                    struct.pack(">H", ycoord),
                    struct.pack(">H", imagewidth),
                    struct.pack(">H", imageheight),
                    struct.pack(">B", localgctsize),
                    b'\x00',      # Reserved
                    b'\x00',      # Background color index
                    b'\x00',      # Pixel aspect ratio
                    ])
    if interlaceflag:
        packed += b'\x40' 
    else:
        packed += b'\x00'
    if sortedgct:
        packed += b'\x08'        
    else:
        packed += b'\x00'
    if lctleft == 0:
        packed += b'\x00'            
    elif lctleft < 225:
        packed += bytes([lctleft//256+1, lctleft%256])
    else:
        raise ValueError("LCT left too large.")
    out.write(packed)
    
    # Local Color Table
    for i in range(0, pow(2,localgctsize)):
        p = b'\xFF'*3
        out.write(p)

    # LZW Minimum Code Size
    mincodesize = 0x02 # Arbitrary value. Must be between 2 and 12 inclusive.
    out.write(mincodesize.to_bytes(1,'little'))

    # Block Terminator
    out.write(b'\x00')

    # Image Data
    from..compress.lzw import compress    
    compressed = compress(rng.read(), mincodesize)
    out.write(compressed)

    # Block Terminator
    out.write(b'\x00')
