#!/usr/bin/env python3

# This script generates a random GIF file in the current directory.
# Inspired by http://www.fileformat.info/format/gif/egff.htm

import os
import sys
import time

import struct


def main():
    if len(sys.argv) > 2 or '-h' in sys.argv[1:] or '--help' in sys.argv[1:]:
        print('Usage:')
        print('\t{} [size]'.format(os.path.basename(__file__)))

    elif len(sys.argv) == 2 and '-' not in sys.argv[1]:
        try:
            size = int(sys.argv[1])

        except ValueError as e:
            raise Exception('{} is not a valid size.'.format(sys.argv[1])) from None


    else:
        rng = open('/dev/urandom', 'rb')

        # Write magic string (b'GIF87a'; b'GIF89a') to buffer
        header = bytes([0x47, 0x49, 0x46, ord(rng.read(1)[0])-105+ord('a')])

        # Use big endian for all fields because this format is old
        width = struct.unpack('>H', rng.read(2))[0]
        height = struct.unpack('>H', rng.read(2))[0]
        packedFields = struct.unpack('>B', rng.read(1))[0]

        # Build palette; we only support 256 colors
        colorResolution = 7
        globalColorTableFlag = bool(packedFields & 0b10000000)
        sortedGlobalColorTableFlag = False
        numGlobalColors = pow(2, colorResolution)
        globalColorTableSize = numGlobalColors * 3
        globalColorTable = [rgbToGif(color) for color in
                [(i, i, i) for i in range(numGlobalColors)]]

        backgroundIndex = 0    # black
        pixelAspectRatio = 0   # square pixels

        frameBlockSize = \
                1 + \     # graphics control extension flag
                1 + \     # delay
                3         # extension block label
        imageDescriptorBlockSize = \
                9 + \      # left offset
                9 + \      # top offset
                8          # LZW min code size
        localColorTableFlag = True
        interlaceFlag = False
        lzwMinimumCodeSize = 2 ** ((packedFields & 0b00000111)+1)
        localColorTableSize = 2 ** ((packedFields & 0b00111000)>>3)
        localColorTable = []
        for _ in range((localColorTableSize - 1)):
            localColorTable += rgbToGif((
                    int.from_bytes(rng.read(1), byteorder='little'),
                    int.from_bytes(rng.read(1), byteorder='little'),
                    int.from_bytes(rng.read(1), byteorder='little')))


        # Write block sizes at offsets specified above
        buffer =HEADER + \
                width.to_bytes(2, byteorder='big') +\
                height.to_bytes(2, byteorder='big') +\
                packedFields.to_bytes(1, byteorder='big') +\

                # Graphics Control Extension Block
                0xf9.to_bytes(1, byteorder='big') +\
                4.to_bytes(1, byteorder='big') +\
                (0).to_bytes(1, byteorder='big') +\       # no transparancy
                0.to_bytes(1, byteorder='big') +\        # delay set to zero
                0xff.to_bytes(1, byteorder='big') +\
                
                # Image Descriptor
                0x2c.to_bytes(1, byteorder='big') +\
                leftOffset.to_bytes(1, byteorder='big') +\
                topOffset.to_bytes(1, byteorder='big') +\

                0.to_bytes(1, byteorder='big') +\
        #print(len(buffer))
        



if __name__ == '__main__':
    main()