#!/usr/bin/env python3

import os
import sys
import argparse

from PIL import Image, ImageSequence


class RandomGifGenerator:

    def __init__(self):

        self._rng = os.open("/dev/urandom", os.O_RDONLY | os.O_NONBLOCK)


    def generate_frame(self):
        arr = os.read(
                self._rng,
                4056 * 12 * 2).decode('utf-8').split("\n")[:-1]
        return [int(x[:7], base=2) for x in arr if len(x) >= 8]


    def save_to_file(self, filename="out.gif"):
        with open(filename, 'wb') as f:

            globalheader = b'\x47\x49\x46\x38\x39\x61\x50\x00' \
                           b'\x04\x00\x00\x00\xff\xfe\x00\x0b' \
                   b'\x00\x6e\x00\x00\x00\x00\x00\xe5\x01' \
                   b'\x00\x00\x00\x00\x00\xff\xdb\x00\x43' \
                   b'\x00\x01\x01\x01\x01\x01\x01\x01\x01' \
                   b'\x01\x01\x01\x01\x01\x01\x01\x01\x01' \
                   b'\x01\x01\x01\x01\x01\x01\x01\x01\x01' \
                   b'\x01\x01\x01\x01\x01\x01\x01\x01\x01' \
                   b'\x01\x01\x01\x01\x01\x01\x01\x01\x01' \
                   b'\x01\x01\x01\x01\x01\x01\x01\x01\x01' \
                   b'\x01\x01\x01\x01\x01\x01\x01\xff\xc0' \
                   b'\x00\x11\x08\x00\x97\x00\xa0\x03\x01' \
                   b'\x22\x00\x02\x11\x01\x03\x11\x01\xff' \
                   b'\xc4\x00\x1f\x00\x00\x01\x05\x01\x01' \
                   b'\x01\x01\x01\x01\x00\x00\x00\x00\x00' \
                   b'\x00\x00\x01\x02\x03\x04\x05\x06\x07' \
                   b'\x08\x09\x0a\x0b\xff\xc4\x00\xb5\x10' \
                   b'\x00\x02\x01\x03\x03\x02\x04\x03\x05' \
                   b'\x05\x04\x04\x00\x00\x01\x7d\x01\x02' \
                   b'\x03\x00\x04\x11\x05\x12\x21\x31\x41' \
                   b'\x06\x13\x51\x61\x07\x22\x71\x14\x32' \
                   b'\x81\x91\xa1\x08\x23\x42\xb1\xc1\x15' \
                   b'\x52\xd1\xf0\x24\x33\x62\x72\x82\x09' \
                   b'\x0a\x16\x17\x18\x19\x1a\x25\x26\x27' \
                   b'\x28\x29\x2a\x34\x35\x36\x37\x38\x39' \
                   b'\x3a\x43\x44\x45\x46\x47\x48\x49\x4a' \
                   b'\x53\x54\x55\x56\x57\x58\x59\x5a\x63' \
                   b'\x64\x65\x66\x67\x68\x69\x6a\x73\x74' \
                   b'\x75\x76\x77\x78\x79\x7a\x83\x84\x85' \
                   b'\x86\x87\x88\x89\x8a\x92\x93\x94\x95' \
                   b'\x96\x97\x98\x99\x9a\xa2\xa3\xa4\xa5' \
                   b'\xa6\xa7\xa8\xa9\xaa\xb2\xb3\xb4\xb5' \
                   b'\xb6\xb7\xb8\xb9\xba\xc2\xc3\xc4\xc5' \
                   b'\xc6\xc7\xc8\xc9\xca\xd2\xd3\xd4\xd5' \
                   b'\xd6\xd7\xd8\xd9\xda\xe1\xe2\xe3\xe4' \
                   b'\xe5\xe6\xe7\xe8\xe9\xea\xf1\xf2\xf3' \
                   b'\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xff\xc4' \
                   b'\x00\x1f\x01\x00\x03\x01\x01\x01\x01' \
                   b'\x01\x01\x01\x01\x01\x00\x00\x00\x00' \
                   b'\x00\x00\x01\x02\x03\x04\x05\x06\x07' \
                   b'\x08\x09\x0a\x0b\xff\xc4\x00\xb5\x11' \
                   b'\x00\x02\x01\x02\x04\x04\x03\x04\x07' \
                   b'\x05\x04\x04\x00\x01\x02\x77\x00\x01' \
                   b'\x02\x03\x11\x04\x05\x21\x31\x06\x12' \
                   b'\x41\x51\x07\x61\x71\x13\x22\x32\x81' \
                   b'\x08\x14\x42\x91\xa1\xb1\xc1\x09\x23' \
                   b'\x33\x52\xf0\x15\x62\x72\xd1\x0a\x16' \
                   b'\x24\x34\xe1\x25\xf1\x17\x18\x19\x1a' \
                   b'\x26\x27\x28\x29\x2a\x35\x36\x37\x38' \
                   b'\x39\x3a\x43\x44\x45\x4