#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom). The width and height of the image is set to 1<<width and 1<<height
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=4, height=5
    ):

    # Generate the header of the gif with the given parameters
    header = (b'GIF89a',
            0xFF, 0x00,                # Logical screen descriptor block, no global color table so first byte is always 0xFF
            0x0B,                      # Size of logical screen reference, in bytes, so it's always 11 right now
            *struct.pack(">H", 1 << width),    # Width of the logical screen reference
            *struct.pack(">H", 1 << height),   # Height of the logical screen reference
            0x87,                      # Global color resolution, this should also be 0x87 for a 256-color palette
            0x00                       # Background color index
            )
    
    # Pack all arguments together as an array of bytes ready to be written
    packed = b"".join(*header)

    # Write the header directly to our output stream
    out.write(packed)

    # Write the main content of the file that we will use as our "image" section.
    # This should just be a bunch of ranom numbers to show how different parts are compressed.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(rng.read(datalen))