#!/usr/bin/env python3

import os
import sys


verbose = False

# Generates a random GIF file as `outfile` using the
# provided `rng`. By default, uses /dev/urandom
def main():

    if len(sys.argv) >= 2 and sys.argv[1] == '-v':

        global verbose
        verbose = True
        rng = open('/dev/urandom', 'rb')

    elif len(sys.argv) > 2 or (len(sys.argv) == 2 and not sys.argv[1].startswith('-')):

        with open(sys.argv[1], 'wb+') as outfile:

            # Use more entropy; if we're given an input file, read up to
            # the first null byte (to avoid eating the entirety of stdin)
            max_length = min((os.stat(sys.argv[1]).st_size - 50), 64*1024)
            
            rng = open('/dev/urandom', 'rb')
            rndfile = randgen_lines(rng, max_length=max_length)
            for line in rndfile:
                outfile.write(line)
            
    else:

        print('Usage:\n\tgen_rand_gif [outfile]\n\t-v')
        
            
# Randomly generates lines of size `max_length` from source `source`, suitable for writing to disk
def randgen_lines(source, max_length):
    
    while True:
        
        length = int.from_bytes(source.read(3), byteorder='big')
        if length < 59 or length > max_length:
            continue
            
        yield source.read(length)[0x4:]
    

if __name__ == '__main__': main()