import sys
import struct
from random import randint
from typing import BinaryIO


# Reads a 16-bit unsigned integer from `f`.
def read_uint16(f):
    return struct.unpack('>H', f.read(2))[0]



# Generates a new Gif image with width `w`, height `h`, and background color `bg`
# The output is written to stdout.
# Returns an array of palette colors as (r, g, b) tuples
def create_new_gif(w: int, h: int, bg=(0x04, 0x05, 0x07)) -> list[tuple]:

    global frame_size, frame_data

    # Palette color constants
    global red, green, blue


    # Write header bytes
    print("GIF89a", file=sys.stdout.buffer)

    # Write logical screen descriptor header
    print(struct.pack('<HHBBB', w, h, 0xF7, 0, 0b10100000 | 0b100 | 0b1), file=sys.stdout.buffer)

    # Write the color table
    for i in range(2**(5)):
        red   = (red * 3) % 0xFF
        green = (green * 3) % 0xFF
        blue  = ((blue >> 1) | (blue << 7)) & 0xFF

        # print("%d: %s" % (i+1, tuple([int((val / 255)**0.35 * 255).to_bytes(length = 1, byteorder='little')[0] for val in [red, green, blue]])), file=sys.stderr)

        # print(struct.pack('<BBBB',*tuple([(val >> 4) + 1 for val in reversed(color)]))), file=sys.stdout.buffer)

    # Calculate size of the first frame
    frame_size = round(((w * 7 + 1) // 8) * h)

    # write application extension block
    print(b'\x21\xFF\x0BNETSCAPE2.0', file=sys.stdout.buffer);
    print(b'\x03\x01\x02', file=sys.stdout.buffer);
    print(struct.pack('<H', 0), file=sys.stdout.buffer);

    # Initialize frame buffer
    pixels = []
    for y in range(h):
        for x in range(w):

            r = (((x ^ y) * 3 + x // 7) * 63) % 0xFF;
            g = (((y ^ x) * 3 + y // 7) * 63) % 0xFF;

            if r < 0x18 or g < 0x18:
                rgb = (0x00, 0x00, 0x00)
            elif r > 0xF8 or g > 0xF8:
                rgb = (0xFF, 0xFF, 0xFF)
            else:
                rgb = tuple([int((val / 255)**0.35 * 255) for val in [r,g,b]])


            pixels += [(rgb[0], rgb[1])] if rgb!= bg else [(0x04, 0x05, 0x07)]



    # Write initial frame
    # lzw minimum code length is 9 bits (max value 511)
    # so we need 512 values <= 255
    mincode = b'\x03\xFE' + b''.join(map(lambda i: i[0].to_bytes(length=1, byteorder="little"), pixels)).ljust(frame_size - len(mincode), b'\xFF')

    # frame control block
    print(b'\x2c\x00\x00', file=sys.stdout.buffer)
    print(struct.pack('<hhBBBBBH', w, h, 0b100 | 0b11, 0b11000000, 0, 0, 3, len(mincode)), file=sys.stdout.buffer)
    print(mincode, file=sys.stdout.buffer)
    print(b';', file=sys.stdout.buffer)

    return pixels
