#!/usr/bin/env python3

import io
import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (default to urandom)
def generate_random_gif(
        rng: BinaryIO = None, out: BinaryIO = sys.stdout
    ):

    if not rng:
        print('No rng specified, using /dev/urandom',file=sys.stderr)
        rng = open('/dev/urandom','rb')
    
    header = b'GIF89a'
    datalen = ord(rng.read(1)) | ord(rng.read(1)<<8) \
             | ord(rng.read(1)<<16) 
    data = rng.read(datalen)

    # Check that we have enough bytes before writing anything else
    assert len(data)+len(header)==datalen+7, "Bad Gif header"

    out.write(header + data)