#!/usr/bin/env python3

import sys
import os
import struct
from typing import BinaryIO

# Generates a random GIF file in `outfile`, using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is determined by its width and height parameters.
def _generate_random_gif(
        rng: BinaryIO, outfile: str, w: int, h: int
    ) -> None:

    # NOTE: width is not actually used in this function, as we write the full bytes to disk rather than use a library to assemble it ourselves :)

    with open(outfile, 'wb') as f:

        print("w: %d" % (w))
        print("h: %d\n" % (h))


        # Write header
        hdr = b'\x47\x49\x46\x38\x39\x61'
        f.write(hdr)

        # Write logical screen descriptor
        lsd = struct.pack('!HBBBBB', w, h, 0xF, 0xFF, 0)
        print("lsd: ", end="")
        for ch in lsd:
            print("%X " % ord(ch)),
        print()
        f.write(lsd)

        # Write local color table
        # TODO: fix me
        # ct = [b'\x00'*7] * 256
        # for i in range(len(ct)):
        #     ct[i] = chr((i >> 5) & 0x7c | (i << 1) & 0xe0 | (i >> 1) & 0x1f).encode('ascii')
        # ct = b"".join(ct)
        ct = b""

        # Write image separator
        sep = b','

        # Write space for frame duration
        dur = b''

        # Write graphic control block
        gcb = struct.pack('!HHBBBBHB', 0, 0, 0xF, 0, 0, 0, 0, 0)

        # Write image data sub-blocks
        num_subblocks = 10
        allocsize = (((num_subblocks+1)*w)+14)/8*8
        for i in range(allocsize):
            if ((i % w == 0 or i >= len(ct)-14) and i > 0):
                f.write(sep)
                f.write(dur)
                f.write(gcb)

                if (i < len(ct)-14):
                    f.write(ct[i:])

            elif (i//w == num_subblocks-1):
                f.write(sep)
                f.write(dur)

            else:
                if (i >= len(ct)-14):
                    f.write(ct[i])
                else:
                    f.write(struct.pack('B', int(rng.read(1))))


# Generates multiple random GIfs into `outdir`.
# The filename format will be `hex(int(time()))-WIDTHxHEIGHT.gif`.
def generate_multiple_gifs(
        rng: BinaryIO, dirname: str, width: int, height: int, count: int
    ) -> None:

    try:
        os.mkdir(dirname)
    except OSError:
        pass


    for i in range(count):
        fname = "%08X-%dx%d.gif" % \
               (int(os.path.getmtime(__file__))+i, width, height)
        print("Generating '%s'" % fname)
        _generate_random_gif(rng, os.path.join(dirname, fname),
                            width, height)


if __name__ == '__main__':
    argc = len(sys.argv)

    RNG = "/dev/urandom"
    WIDTH=128
    HEIGHT=128
    COUNT=10

    if (argc > 1):
        RNG = sys.argv[1]
    if (argc > 2):
        WIDTH = int(sys.argv[2])
    if (argc > 3):
        HEIGHT = int(sys.argv[3])
    if (argc > 4):
        COUNT = int(sys.argv[4])

    with open(RNG, 'rb') as r:
        # generate_multiple_gifs(r, ".", WIDTH, HEIGHT, COUNT)
        _generate_random_gif(r, "./test.gif", WIDTH, HEIGHT)