#!/usr/bin/env python3

import sys
import os
import array
import struct
from typing import BinaryIO


# Generates a random GIF file into `out`. We use /dev/urandom as our
# source of randomness.
def generate_random_gif():
    # Get the size from user input
    try:
        width = int(input("Enter image width (in pixels): "))
        height = int(input("Enter image height (in pixels): "))

    except ValueError:
        print("Invalid dimensions, giving up.")
        return -1
    
    if not 0 < width <= 4294967295 or \
       not 0 < height <= 4294967295:

        print("Dimensions are too large")
        return -1


    # Add the file-size field to the start of the output stream
    out = array.array('B', [0x00] * 10)

    # Generate the header and write it at offset '0'. This is an array
    # instead of bytes because we can't concat them until Python3.5
    out[0:6] = [
            0x47, 0x49, 0x46, 
            0x38, 0x39, 0x61]

    # Write the dimensions after the header
    out[6], out[7] = struct.pack(">HH", width, height)

    # Next comes the screen descriptor, which contains color info, etc... 
    # Again this is stored in an array but concatenated later on
    out[8:10] = [
            0xF7, 0xDC]

    # The remaining fields in the header include flags for things like
    # animated gifs, colour resolution, bit depth, etc...
    # Here's where you could add things if you wanted!

    # Now that everything has been written, let's concatenate all the arrays
    out = bytes(out)

    # Open /dev/urandom for read access with buffered I/O
    with open("/dev/urandom", "rb", buffering=0) as urand:
        # Read the remainder of the data from urand, then write it 
        # to our final Gif
        out += urand.read()

    # Open a new file called random.gif in the current directory
    # and write binary data
    with open("random.gif", "wb") as f:
        # Write the contents of out to random.gif
        f.write(out)


