#!/usr/bin/env python3

import os
import sys
import time
import struct
import argparse

from steganography import encode

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1024 * (1 << ((int.from_bytes(
            rng.read(3), byteorder='big')) & 7))
    data = bytes([((i >> i%8)&1)^((j+1)%65) for j in range(datalen)])

    # TODO: Holy hell this is terribly inefficient...
    encode(data, open('flag', 'rb'), out)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-o', '--output', type=argparse.FileType('wb'), default='/dev/stdout')
    args = parser.parse_args()

    generate_random_gif(os.devnull if '-d' in sys.argv else open('/dev/urandom', 'rb'), args.output)