#!/usr/bin/env python3

import binascii
import os
import sys


def rand16() -> bytes:
    return os.urandom(2)


def rand8() -> bytes:
    return os.urandom(1)


def gen_buf(n: int):
    buf = []
    for _ in range(n - 1):
        buf += [rand8()]

    if n > 0:
       buf += [b'\x00']
    
    return b''.join(buf)



def main():
    with open('output', 'wb') as f:
        print(f'Generating {sys.argv[1]}...')

        # Header
        f.write(b'GIF89a')

        # Logical Screen Descriptor
        width = rand16()
        height = rand16()
        pixel_aspect_ratio = b'\x00\x40'  # (1 : 256)
        background_color_index = rand8() 
        pixel_size = b'\xff'             # True color
        flags = b'\x87'                  # Global Color Table
        gct = b''                        # Empty global palette table
        f.write(width + height +
                 pixel_aspect_ratio + 
                 background_color_index + 
                 pixel_size + 
                 flags + 
                 gct)

        # Image Descriptor (default LZW compression method and
        # no local colour table used)
        left = rand16()                  
        top = rand16()  
        width = rand16()                 
        height = rand16()               
        flags = b'\x08\x00'              
        f.write(left + top +
                 width +
                 height +
                 flags)
        
        # Local Color Table
        colors = [[0xf0,0xf0,0xf0],
                  [0x00,0x00,0x00]]
        rgb = sum([c for c in colors], [])
        ct_len = len(rgb).to_bytes(1, byteorder='big')
        flags = b'\xfe\x00'              
        f.write(ct_len + flags + bytes(rgb))

        # Loop count
        loop_count = rand16()          
        f.write(loop_count)
        
        # Graphics Control Extension (transparent black background)
        ext_block = b'\x21\xf9\x04\x04'  
                             
        # Transparent index
        t_index = b'\x00'                
        f.write(ext_block + t_index)
        f.write(rand8())

        # Plain text extension
        pt_block = b'\x21\x00'          
        f.write(pt_block)
        
        # Text value (int between 0-9)
        txt_val = str(int.from_bytes(
                os.urandom(1), byteorder='big')).encode('utf-8')
         
        # Block size
        block_sz = str(len(txt_val)).encode('utf-8')
        f.write((block_sz + txt_val))
        
        # Application Extension
        app_block = b'\xFF\xFF'         
        f.write(app_block)
        
        # Sub-block ID
        sub_id = b'\x0B'                   
        f.write(sub_id)
        
        # Sub-block length
        sub_length = b'\x0A'             
        f.write(sub_length)
        
        # Data (some sort of signature?)
        sign = b'GIFSIGNEXTENDED'        
        f.write(sign)
        
        # Plain text extension end
        ptend = b'\x00'                    
        f.write(ptend)

        # Graphic control label
        gc_label = b'\x21\xf9\x04\x08'    
        f.write(gc_label)

        # Block size
        gs_len = b'\x04'                     
        f.write(gs_len)

        # Delay time
        delay_time = b'\x00\x00\x00\x00'     
        f.write(delay_time)

        # Transparent color index
        tc_index = b'\x00'                   
        f.write(tc_index)
        

    print("Done")

