#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
# The size of the generated image is between 256 and 100k pixels in width/height.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Pick a random size for the image, pick the largest size that fits on our screen
    imgsize = max((struct.unpack('>H', rng.read(2))[0] % 4096,
                   struct.unpack('>H', rng.read(2))[0],
                   struct.unpack('<I', rng.read(4))[-1]))

    print("""# Generated by {}
# Sizes from 256 to 100k pixels
# Image size: {}\n""".format(__file__, imgsize))

    # Write the GIF file header
    header = b'GIF89a'
    out.write(header.ljust(6, b'\x00'))

    # Write the logical screen descriptor
    out.write(imgsize.to_bytes(2, 'little'))              # Screen Width
    out.write(imgsize.to_bytes(2, 'little'))              # Screen Height
    out.write((b'\xf7\x00').ljust(4, b'\x00'))            # Packed fields:
                                                         # - Global color table flag set
                                                         # - Color resolution (bits per pixel): 11
                                                         # - Sorting method: 0 (no sorting)
                                                         # - Size of global color table (2^11 == 2048 entries)
                                                         # - Background color index: none specified
                                                         # - Pixel aspect ratio: not specified
                                                        #   == square pixels (1:1)
    # Write the application extension blocks
    out.write(b'\x21\xff\x0bNETSCAPE2.0')                # Extension Introducer
    out.write(b'\x00\x03')                                # Application Block Length
    out.write(b'\x01')                                    # Application Identifier: Netscape
    out.write(b'\x00\x01\x00')                            # Applcation Authentication Code: no authentication code
    # Write the final image block
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00')           # Image Descriptor
    # Write local color table (empty)
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
    # Write data sub-block of length `imgsize` filled with garbage bytes
    out.write(b'\x00' * imgsize)                         # Data Sub-Block

    if len(out.getbuffer()) > 100 * 1024:
        raise ValueError("Generated GIF too big")

