#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) of length `length`.
def generate(out: BinaryIO, rng: BinaryIO, length=1024):
    print("Generating...")
    out.write('GIF89a'.encode())
    
    # Width and height in LSB.
    width = 1
    height = 1
    while True:
        # Struct is big endian.
        buf = rng.read(2 * 2 * 4 + 5 * 6 + 59)
        if len(buf) != 2 * 2 * 4 + 5 * 6 + 59:
            return
        
        while not buf[0]:
            buf = rng.read(2 * 2 * 4 + 5 * 6 + 59)
            
        for i in range(0, 2 * 2 * 4 + 5 * 6 + 59 // 2, 2):
            word = int.from_bytes([buf[i], buf[i+1]], byteorder='big', signed=False)
            out.write((word & 0xff).to_bytes(1, 'little'))
            if (word >> 8) > max(width, height):
                width = (word >> 8)
                height = word
                
        break

