#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    width = int.from_bytes(rng.read(1), 'big', signed=False) + 1 # [1..256]
    height = int.from_bytes(rng.read(1), 'big', signed=False) + 1 # [1..256]

    packed = rng.read(datalen)
    rfgc = int.from_bytes(packed[0], 'big', signed=False) & 0x7F
    bgc = packed[1] if (rgfc & 1) else None
    aspectratio = packed[2] if (rgfc & 2) else None
    packed = packed[3:]

    globalct = [(int.from_bytes(packed[0:3], "little"), packed[3]),
                (int.from_bytes(packed[4:7], "little"), packed[7])]
    imgblocks = []
    for i in range(width * height):
        imgblocks.append({"delay": None, "colorindex": None})

    out.write(header)
    out.write(struct.pack("!H", datalen))
    out.write(struct.pack("!HHB",
                          width, height,
                          bool(bgc)))
    out.write(b"\x00")
    if bgc is not None:
        out.write(bgc)
    out.write(rfgc)
    if len(globalct) > 0:
        out.write(struct.pack('!HHBB',
                              globalct[0][0], globalct[1][0],
                              0, 0))
    for i in range(len(imgblocks)):
        imgblocks[i]['delay'] = rng.read(1)[0