#!/usr/bin/env python3

import struct
import sys


with open('xmas', 'wb') as xm, \
     open('/dev/urandom', 'rb') as rng:

    header = b'GIF87a'
    width, height = 256, 100


    def write_uint(out, val):
        """Writes a uint to the output file"""

        nbytes, rem = divmod(val.bit_length(), 8)
        if rem!= 0:
            raise ValueError("Value must be multiple of 8 bits")
        out.write(
                (nbytes - 1).to_bytes(byteorder='big')) # length in bytes from MSB
        for i in range(nbytes-1, -1, -1):
            out.write(((val >> 8*i) & 0xff))



    extension_block = b'\x21\xFF\x0BNETSCAPE2.0' + \
                      struct.pack('>H', len(b'\x03\x01')) + \
                      b'\x03\x01'
    netscape_loop_count = b'\x21\xFF\x0E' + struct.pack('>H', 4) + b'\x00\x00\x00' \
                           b'\xFF\x00'
    color_table = [
                    (0x00, 0x00, 0x00),
                    (0xff, 0x00, 0xff)] * 256
    color_table += [(0xff, 0xff, 0xff)]*(256**2 - len(color_table))
    global_color_map = b'\x21\xF9\x04'
    global_color_map += struct.pack('>H', 8+len(color_table))
    global_color_map += struct.pack('>B', 2**(len(color_table).bit_length())-1 )
    global_color_map += b'\x00'
    global_color_map += b''.join((struct.pack('BBB', r, g, b)
                                 for r,g,b in color_table)) + b'\x00'
    
    frame_data = []
    for ypos in reversed(range(height)):
        for xpos in range(width):
            pixelvalue = ord(rng.read(1)) % 2
            frame_data.append(pixelvalue << 7 | pixelvalue << 6)
            del pixelvalue

    del rng

    write_uint(xm, len(frame_data))
    xm.write(header)
    xm.write(global_color_map)
    xm.write(extension_block)
    xm.write(netscape_loop_count)
    xm.write(b'\x2C')
    write_uint(xm, width)
    write_uint(xm, height)
    write_uint(xm, 0)
    xm.write(b','.join([chr((v>>7)&3|((v>>6)&3)<<2)
                         for v in frame_data]))
    xm.write(b';')



# The following code is inspired by this short thread on /r/python:
# https://www.reddit.com/r/Python/comments/akh5oq/creating_animated_gifs_using_pillow/ecqrmli?utm_source=share&utm_medium=web2x
#
# I basically copied it and turned it into a script to generate random GIFs.
if __name__ == '__main__':

    with open('xmas.gif', 'wb') as out, open('xmas', 'rb') as inp:
        for _ in range(10):
            add_frame(out, inp)



def add_frame(out : IO[bytes],
              rng : BinaryIO
              ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
