#!/usr/bin/env python3

import struct
from Cryptodome.Cipher import AES, PKCS1_OAEP


def get_encrypted_data(key):
    with open("encrypted.dat", "rb") as f:
        encrypted = bytearray(f.read())

    decrypted = decrypt_all(encrypted, key)

    with open("decrypted.dat", "wb") as of:
        of.write(decrypted)


def decrypt_all(data, key):
    aes = AES.new(key, AES.MODE_CBC, IV=b"")

    # Determine the amount of blocks to decrypt by reading the first block's length field and dividing it by 16 bytes (block size).
    # The len() function works on lists too. Subtracting one is because there are always empty bytes at the end.
    numblocks = int((len(data)-1) / 16 - 1)

    return aes.decrypt(bytes(data))[0x20:] + data[numblocks*16+16:-1]


def encrypt(key, data):
    aes = AES.new(key, AES.MODE_CBC, IV=b"")
    
    padded_data = pad(data)
    encrypted = aes.encrypt(padded_data)

    with open('encrypted.dat', 'wb') as f:
        f.write(encrypted)


# Pad the given input data using PKCS7 padding.
def pad(input_data):
    blocksize = 16
    remainder = len(input_data) % blocksize
    if remainder == 0:
        return input_data
    padding_length = blocksize - remainder
    padding = bytes([padding_length]) * padding_length
    return input_data + padding
    

def main():
    print("AES CBC encryption oracle.")

    key = b'\<KEY>'
    pkcs1_cipher = PKCS1_OAEP.new(RSA.generate(2048))
    ciphertext = encrypt(pkcs1_cipher.exportKey(format='DER'), key)

    encrypted = list(ciphertext)

    for i in range(len(encrypted)):
        if encrypted[i]!= 0x00:
            break
        encrypted[i] = 0xff

    while True:
        try: 
            encrypted[i] += 1

            plaintext = decrypt(encrypted[:], key)    
            
            print(plaintext)
            exit()

        except ValueError as e:
            pass


if