#!/usr/bin/env python3

import os
import sys

from randomizer import get_rand_bytes


# Generates a random GIF file into `out`.
# If `verbosity > 0`, print some statistics about how much data was generated).
def generate_random_gif(
        nbytes: int, verbosity: int=1, out: str='/dev/stdout', seedfile: str=None
    ) -> None:
    if not isinstance(nbytes, int):
        raise TypeError("`nbytes` must be an integer")

    with open('/dev/urandom', 'rb') as urand, \
         open(out, 'wb') as outf:

        # Set seed from args or environment variable (if applicable)
        seed = bytes()
        if seedfile is not None and os.path.isfile(seedfile):
            with open(seedfile, 'rb') as file:
                seed = file.read(3)
        elif "SEED" in os.environ.keys():
            seed = os.environ["SEED"].encode('ascii')[:3]
        else:
            seed = get_rand_bytes(3)
            if verbosity > 0:
                print(
                    "[+] No seed found in arguments or environment variables; generating one",
                    file=sys.stderr
                )

        # Write gif header to output
        outf.write(b'GIF89a')

        # Write size of image data to output
        outf.write(struct.pack('!I', nbytes))

        # Write the contents of the image data to output
        outf.write(get_rand_bytes(nbytes, seed=seed))

    if verbosity > 0:
        print("[+] Generated {} bytes".format(nbytes), file=sys.stderr)