#!/usr/bin/env python3

import struct
import sys
from typing import IO


# Writes bytes from the given iterable to the output stream in chunks of size `chunksize`.
# The default chunksize is 1024. This function returns once all items have been written and
# does not flush the underlying buffer - it is up to the caller to do so.
def write_chunks(stream: IO[bytes], items: list(bytes),
                 *, chunksize=1024):

    for item in items:
        
        if isinstance(item, str):
            # That's a bit ugly but I can't figure how else I'd convert any string 
            # (which could be binary or utf-8 encoded or something completely different)
            # to an array of integers that are then converted back by Python to a single char
            # on the other side of this pipe...
            item = [ord(x) for x in item]

        while len(item) > 0:

            # Write at most chunksize bytes from the item to the stream
            n = min(len(item), chunksize)
            
            stream.write(struct.pack('B'*n, *item[:n]))

            del item[:n]
            