#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\xff\xe0\x00' # SOF 0
    datalen = int.from_bytes(
            rng.read(2), byteorder='little')
    data = rng.read(datalen - 2)
    out.write(header + datalen.to_bytes(2, 'little'))
    out.write(data)

    numblocks = int((len(data)+2)/blocksize)
    for i in range(numblocks):
        if ((i+1)*blocksize > len(data)):
            restofdata = data[(i*blocksize):]
        else:
            restofdata = data[
                    (i*blocksize):(i+1)*blocksize
                ]

        out.write(struct.pack('B', 0xff))
        out.write(restofdata)

# Generates a random PNG file into `out` using the
# random number generator `rng` (/dev/urandom)
#
# Currently only generates a simple PNG with
# one IDAT chunk and no IEND chunk since that's
# all we need now
def generate_random_png(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\x89PNG\r\n\x1a\n'
    out.write(header)

