#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'big')
    if not 1 <= datalen <= 65537:
        raise ValueError("invalid string length")
    data = rng.read(datalen)
    commentlen = int.from_bytes(rng.read(2), 'big')
    if not 0 <= commentlen <= 65537:
        raise ValueError("invalid string length")
    comment = rng.read(commentlen)
    
    # Header
    out.write(header+b'\x01\x00\xff\xff'+data+b'\x00'.ljust(4)+struct.pack('<H', len(comment))+comment)
