#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  9 14:23:56 2022

@author: hendrawahyu
"""

import numpy as np
from numpy.linalg import inv, det

def weighted_jacobian(x, u):
    """ Calculate the weighted Jacobian """
    
    J = np.dot(2*np.pi**(-2),u[:,None]*np.exp(-u**2/2)*np.sinh(u).T)
    return J

def covariance_matrix():
    """ Calculate the approximated covariance matrix """
    
    gamma = (2*np.pi)**(-2) * u[:,None] * np.exp(-u**2/2) * np.cosh(u).T
    M = np.dot((gamma+1), gamma.T)/(u**2 + 1)
    return M

def approximate_inverse_covariance():
    """ Calculate the inverse of the approximated covariance matrix """
    
    gamma = (2*np.pi)**(-2) * u[:,None] * np.exp(-u**2/2) * np.cosh(u).T
    c = (2*np.pi)**(-2) * (u/(u**2 + 1))
    M = np.linalg.inv(np.dot(c,(gamma+1)*c.T))
    return M

def log_determinant():
    """ Calculate the log determinant of the true covariance matrix in the given domain """
    
    A = weighted_jacobian(x,u)
    L = np.log(det(A))
    return L

def mode_of_integration(n=100):
    """ Return the mode of integration for a given n """
