#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out:BinaryIO, rng=None):
  n = 0x100 if (rng is None) else ord(rng.read(1)) % 256
  sig = b'GIF89a\x01\x00\x01\x00\x80\x00\xff\xff\xe4'
  out.write(sig)

  # Header
  for i in range(1, 7 + 1):
    c = chr((n >> ((6 - i) * 2))) & 0xFF
    out.write(struct.pack('B', ord(c)))

  # Image descriptor
  out.write(b'\x2C\x04\x00\x00\x00\x00')

  # Local color table
  out.write(b'\x00\x00\x00')

  # Graphic control extension
  out.write(b'\x21\xf9\x04\x01\x00\x00\x00')
  
  # Plain text extension
  out