#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4
    num_colors = 256
    color_table = [
            rng.read(num_colors*3)]
    
    print("writing gif")
    out.write(header)
    write_int_16(out, width)
    write_int_16(out, height)
    out.write(b'\x87\x00') # flags
    out.write(color_table[0]) # global color table

    # background color index
    out.write(struct.pack("<B", (rng.read(1)[0] % num_colors)))

    # pixel aspect ratio; currently hardcoded
    out.write(b"\x00")

    # control block
    out.write(b"\x21\xf9\x04\x00\x00\x00")

    # application block
    out.write(b"\x21\xff\x0bNETSCAPE2.0\x03\x01")

    # repeat count (forever=0xffff)
    out.write(b"\x00\x00\xff\x00")

    # image block
    out.write(b"\x2c")
    write_int_16(out, 0) # left
    write_int_16(out, 0) # top
    write_int_16(out, width)
    write_int_16(out, height)
   