#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out : BinaryIO, rng : BinaryIO):
    gifhead = b'GIF89a'
    out.write(gifhead)

    # Ignore rest of header, we don't care about it
    ignored = 7 + int(struct.unpack('=H', rng.read(2))[0])
    rng.read(ignored)

    while True:
        fieldbyte = ord(rng.read(1))
        
        # If 0x3b (trailer), terminate stream
        if fieldbyte == 0x3b:
            return
    
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # Handle case where there's no image data
        if not hasimgdata:
            continue

        imgdata = rng.read(size)
        out.write(imgdata)