#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        out : BinaryIO,
        rng : BinaryIO
        ):
    
    # Write the magic string
    out.write(b'GIF89a')

    # Write the screen dimensions in little endian
    out.write(struct.pack('<HH', *([int.from_bytes(rng.read(2), 'big') for i in range(2)])))

    # Write the background color index in little endian
    out.write(struct.pack('<H', int.from_bytes(rng.read(2), 'big')))

    # Write zero for the aspect ratio
    out.write(b'\x00\x00')

    # Generate some data for the body
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    # Pad the data to 16 bytes of length and write it to the output stream
    paddinglen = 16 - len(data) % 16
    if paddinglen == 0:
        padding = b''
    else:
        padding = rng.read(paddinglen)
    out.write(data + padding)

