#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:
        # Read block type byte from rng
        blk = rng.read(1)

        # Check for end of image
        if chr(blk[0]) == ';':
            break;
        
        # Write 0x00 terminator and read the size
        out.write(b'\x00')
        size = int.from_bytes(rng.read(1), 'big', signed=False) + 1

        # Read block data from rng
        data = rng