#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
import array
from binascii import hexlify, unhexlify

def main():
    print("""Usage: python3 %s [<input file>] [<output file>]
    
Generates a C header file containing the contents of an ELF binary file.
If no input or output file is given, they default to "elf" and "elf-header.inc".

""" % sys.argv[0])

    ifile = "elf"
    ofile = "elf-header.h"

    if len(sys.argv) > 1:
        ifile = sys.argv[1].strip()
    if len(sys.argv)>2:
        ofile = sys.argv[2].strip()

    with open(ifile, 'rb') as fp:
        data = bytearray(fp.read())

    code_bytes = []
    for i in range(0,len(data),4):
        code_bytes += [(data[i],data[i+1],data[i+2],data[i+3])]

    outstr = """/* Generated by genelfheader.py */
#ifndef __ELFHEADER_H__
#define __ELFHEADER_H__

#include <stdint.h>

static const uint8_t BINARY[] PROGMEM = {
"""
    for i in range(0, len(code_bytes), 16):
        row = []
        for j in range(16):
            if (i + j) >= len(code_bytes):
                break

            b = code_bytes[i+j]
            outstr += "\t\\x%02X," % b[0]

        outstr += "\n"
