#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD field
    gct_len = int((lsd[4] & 0b111100) >> 2) + 1

    # Read global color table size from LSD field
    gct_size = 3 * pow(2, gct_len+1)

    # Read global color table from stream
    gct = rng.read(gct_size)

    # Add global color table to output
    out.write(gct)

    # Generate graphic control extension (GCE) field
    gce = bytearray([0x21, 0xf9, 0x06, 0x00])

    # Write GCE disposition data
    gce += struct.pack('>H', int(rng.read(2).hex(), base=16))

    # Write GCE delay time
    gce += struct.pack('>H', int(rng.read(2).hex(), base=16))

    # Write GCE transparent color index
    gce += rng.read(1)

    # Write GCE terminator
    gce += b'\xff'

    # Add graphic control extension (GCE) fiels to output
    out.write(gce)

    # Generate image block descriptor (IBD) field
    ibd = rng.read(10)

    # Add image block descriptor (IBD) field to output
    out.write(ib