#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'big') + 1
    data = rng.read(datalen) if datalen > 0 else []
    trailer = b';'
    color_table_len = int.from_bytes(rng.read(1), 'big') + 1
    color_table = rng.read(color_table_len * 3)
    background_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] // 16

    # Header
    out.write(header)
    # Logical screen descriptor (LSR)
    width = int.from_bytes(rng.read(2), 'big') + 1
    height = int.from_bytes(rng.read(2), 'big') + 1
    global_palette_flag = bool((rng.read(1)[0]) & 1)
    color_resolution = (rng.read(1)[0] >> 4) // 10 # 0-7 -> 0-8
    sort_flag = bool((rng.read(1)[0]) & 2)
    size_of_global_color_table = 2 ** ((rng.read(1)[0] >> 4) // 10)
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] // 16
    out.write(struct.pack('>HHBBB', width, height, global_palette_flag << 7 | color_resolution << 4 | sort_flag << 3 | size_of_global_color_table << 1 | background_color_index, pixel_aspect_ratio))
    # Global color table
    out.write(color_table)
    # Application Extension
    application_ext_length = 11
    application_id = "FUZZ"
    authentication = rng.read(3)
    data = data + [authentication]
    out.write(struct.pack(">H", application_ext_length))
    out.write(b'\x21\xFF') # extension type / block size
    out.write(application_id.encode())
    out.write(struct.pack(">BBB", authentication[0], authentication[1], authentication[2]))
    for i in range(0, len(data)):
        out.write(struct.pack(">BB",