#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    third_byte = (datalen >> 11) & 0xFF
    fourth_byte = (datalen >> 3) & 0xFF
    fifth_byte = ((datalen & 0x07) << 5) + 0b11111

    palette = [
        rng.read(1)[0],
        rng.read(1)[0]
    ]

    print('palette', palette,
          'datalen', datalen)

    out.write(header)
    out.write(struct.pack('<HHB',
                          width, height, 0))
    
    out.write(struct.pack('BBBB',