#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # GIF magic
    out.write("GIF89a")
    
    # The GIF header
    width, height = 50 + rng.read(2), 40 + rng.read(1)[0]
    # Logical screen descriptor
    out.write(struct.pack('<HHBB', width, height, 0b00000000, 0))
    # Global colour table (black and white)
    out.write(b'\xFF\xFF\xFF'*4)
    
    # There will be a single image in this GIF
    images = [0] * 1
    (images[0],) = struct.unpack("<I", rng.read(4))
    
    # Graphic control extension
    out.write(bytes([0x21, 0xf9, 4, 0, images[0]]))
    # Image descriptor
    out.write(struct.pack("<BBHHB", 0x2c, 0, 0, width, height, 0))
