#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Get extension block length (1-256 bytes)
    blocklen = rng.read(1)[0]

    # Add extension block length to output
    out.write(struct.pack('<B', blocklen))

    # Generate the application extension header
    appext_header = b'\x21\xff' + rng.read(11)

    # Add the application extension header to the output
    out.write(appext_header)

    # Generate the data for the application extension
    appext_data = rng.read(blocklen - 14)

    # Add the application extension data to the output
    out.write(appext_data)

    # Generate the image separator
    imsep = b'\x2c'

    # Add the image separator to the output
    out.write(imsep)

    # Get the graphics control extension (GCE) field
    gce = rng.read(6)

    # Add the graphics control extension to the output
    out.write(gce)

    # Read in the image descriptor (ID) field
    id = rng.read(9)

    # Add ID to the output
    out.write(id)

    # Get the color table (LZW compression) 
    colortable = rng.read(256 * 3)

    # Add colortable to the output
    out.write(colortable)

    # Now we have to read in a bunch of blocks of data which are composed of a size byte
    # followed by that many bytes of actual data

    while True:
        # This will break if there is nothing left to process from rng
        # The try block exists in case of EOF
        try:
            # Find the next data block
            size = rng.read(1)[0]
        except:
            return
        
        imgdata = rng.read(size)
        out.write(imgdata)