#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
        out.write(rng.read(gctlen))

    # Generate image descriptors (IDs) for all images in this animation
    while True:
        # Generate ID
        id = rng.read(10)
        if len(id) == 0:
            break

        # Write the ID to output
        out.write(id)

        # Extract the color table length from the Image Descriptor
        ctlen = ((id[8] & 0b00001111) + 1) * 3

        # If local color table exists, copy it to output
        if int(id[9]) & 0b10000000:
            out.write(rng.read(ctlen))
