#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    width = 2048
    height = 4096
    pixels = width * height

    # Logical Screen Descriptor (LSD)
    out.write(b"GIF87a")
    out.write((width).to_bytes(2, "little"))
    out.write((height).to_bytes(2, "little"))
    out.write((1 << 5 | 1 << 1).to_bytes(1, "big"))
    out.write(b"\xFF\xFF")

    colors = 1 << ((int.from_bytes(
            rng.read(1), "big", signed=False)) & 7)
    colorbits = [("background", 0)] + [(f"color {i}", i) for i in range(colors)]
    global_palette = b"".join([
            x[1].to_bytes(3, "big") for _, x in colorbits])
    out.write(global_palette)
    
    # Graphic Control Extension (GCE)
    gce = b"!\xF9\x04\x00\x00" + \
          int(2**13 * float(int(500*struct.unpack("<I", rng.read(3))[0]) / 2 ** 31)).to_bytes(2, "big") + \
          b"\x00"
    # Application Extension (APE)
    ape = b"!\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00"
    # Image Descriptor (ID)
    id = (width).to_bytes(2,"little") + \
         (height).to_bytes(2,"little") + \
         b"\x00"

    while True:
        blocktype = int.from_bytes(
                rng.read(1), byteorder="big")
