#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header)
    out.write(header)

    while True:

        n = int.from_bytes(
                rng.read(1), byteorder='little')

        if n == 0x3b:
            break

        elif n in [0x2c, 0x21]:
            pass

        elif n == 0x3b:
            break

        elif n > 0xff or n < 0:
            raise RuntimeError("invalid code")

        elif n == 0:

            val = int.from_bytes(
                    rng.read(2), 
                    byteorder='little')

            print('delay',n,val)
            
            out.write(val.to_bytes(
                    2, 'little'))

        else:

            # write block header as short

            val = int.from_bytes(
                    rng.read(2), 
                    byteorder='little')

            print('blockheader',n,val)
            
            out.write(val.to_bytes(
                    2, 'little'))

            # determine how many longs to read and then write them as such
            # Assumes maximum of one byte per long.
            
            numlongs = (val >> 8) - 1
            
            for m in range(numlongs):

                val = int.from_bytes(
                        rng.read(4), 
                        byteorder='little')

                print('pixeldata',m,val)
                
                out.write(val.to_bytes(
                        4, 'little'))

