#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    third_byte = (datalen >> 11) & 0xFF

    fourth_byte = (datalen >> 3) & 0b11111111

    fractal_color = [
            0,
            0,
            2**((rng.read(1)[0]&0b11)//3*3)
                    + 2**((rng.read(1)[0]&0b11)//3*3+1)]

    global_cmap = [fractal_color[0],
                   fractal_color[1],
                   fractal_color[2]]

    print('#', end='')
    for c in global_cmap:
        print(hex(c).replace('0x', '#'), end='')
    print()

    # The color map is constant throughout the entire file
    out.write(struct.pack('<3B', *global_cmap))

    out.write(header + \
              struct.pack('<H', width) + \
              struct.pack('<H', height) + \
              b'\x87\xff\xff')

    background_index = rng.read(1)[0] % 3
    background_color = global_cmap[background_index]

    animation_control = rng.read(1)[0] & 0b11111111

    out.write(struct.pack('<B', first_byte) + \
              struct.pack('<B', second_byte) +\
              struct.pack('<B', third_byte) + \
              struct.pack('<B', fourth_byte))

    # The animation frame is variable, but must be at least 1 pixel wide and tall
    start_offset = width - (width * height) // 1000

    for frame in range(1000):
        print(frame, end=' ')
        data = []
        for y in range(height):
            for x in range(start_offset):
                if y == 0:
                    data.append(background_index)
                else:
                    data.append(data[-width])

            for x in range(start_offset, width-1):
                data.append((data[-1] + data[1])//2)
            data.append(data[-1])

        def get_pixel(x,y):
            return ((data[(y-1)*width+(x-1)] +
                     data[(y-1)*width+(x)] +
                     data[(y-1)*width+(x)+1] +
                     data[(y)*width+(x-1)] +
                     data[(y)*width+(x)] +
                     data[(y)*width+(x)+1] +
                     data[(y+1)*width+(x-1)] +
                     data[(y+1)*width+(x)] +
                     data[(y+1)*width+(x)+1])//8)

        # Array of bits divided by 8
        pixels = [[get_pixel(x,y) for x in