#!/usr/bin/env python3

import struct
from typing import BinaryIO

from .common import Chunk, Header
from ..structures import StructList
from ..utils.logger import getLogger
from ..utils.binary_stream import BinaryStream

log = getLogger(__name__)

# [ 0] XBB Block
# [ 1] XBA Block
# [ 2] XBG Block
# [ 3] XBI Block
# [ 4] XBJ Block
# [ 5] XBK Block
# [ 6] XBL Block
# [ 7] MT4X Block
# [ 8] XBS Block
# [ 9] XBD Block
# [10] XBE Block
# [11] XBF Block
# [12] GEO Block
# [13] XBP Block
# [14] MT5X Block
# [15] MT6X Block
# [16] PACK Block
# [17] XBM Block
# [18] XBN Block
# [19] XBO Block
# [20] XBR Block
# [21] XBS Block
# [22] XBC Block
# [23] XBQ Block
# [24] XBH Block
# [25] XBY Block
# [26] XBZ Block
# [27] XBW Block
# [28] XBA Block
# [29] XBU Block
# [30] XBV Block
# [31] XBT Block
# [32] XBV Block
# [33] XBZ Block
# [34] XBT Block
# [35] XBZ Block
# [36] XBA Block
# [37] XBW Block
# [38] XBY Block
# [39] XBA Block
# [40] XBW Block
# [41] XBZ Block
# [42] XBD Block
# [43] XBA Block
# [44] XBW Block
# [45] XBZ Block
# [46] XBT Block
# [47] XBZ Block
# [48] XBT Block
# [49] XBA Block
# [50] XBW Block
# [51] XBZ Block
# [52] XBT Block
# [53] XBH Block
# [54] XBU Block
# [55] XBA Block
# [56] XBW Block
# [57] XBZ Block
# [58] XBT Block
# [59] XBZ Block
# [60] XBT Block
# [61] XBA Block
# [62] XBW Block
# [63] XBZ Block
# [64] XBT Block
# [65] XBZ Block
# [66] XBT Block
# [67] XBA Block
# [68] XBW Block
# [69] XBZ Block
# [70] XBT Block
# [71] XBZ Block
# [72] XBT Block
# [73] XBA Block
# [74] XBW Block
# [75] XBZ Block
# [76] XBT Block
# [77] XBZ Block
# [78] XBT Block
# [79] XBA Block
# [80] XBW Block
# [81] XBZ Block
# [82] XBT Block
# [83] XBZ Block
# [84] XBT Block
# [85] XBA Block
# [86] XBW Block
# [87] XBZ Block
# [88] XBT Block
# [89] XBZ Block
# [90] XBT Block
# [91] XBA Block
# [92] XBW Block
# [93] XBZ Block
# [94] XBT Block
# [95] XBZ Block
# [96] XBT Block
# [97] XBA Block
# [98] XBW Block
# [99] XBZ Block
# [100] XBT Block
# [101] XBZ Block
# [102] XBD Block
# [103] XBA Block
# [104] XBW Block
# [105] XBZ Block
# [106] XBT Block
# [107] XBZ Block
# [108] XBA Block
# [109] XBW Block
# [110] XBZ Block
# [111] XBT Block
# [112] XBZ Block
# [113] XBA Block
# [114] XBW Block
# [115] XBZ Block
# [116] XBT Block
# [117] XBZ Block
# [118] XBA Block
# [119] XBW Block
# [120] XBZ Block
# [121] XBT Block
# [122] XBZ Block
# [123] XBA Block
# [124] XBW Block
# [125] XBZ Block
# [126] XBT Block
# [127] XBZ Block
# [128] XBA Block
# [129] XBW Block
# [130] XBZ Block
# [131] XBT Block
# [132] XBZ Block
# [133] XBT Block
# [134] XBA Block
# [135] XBW Block
# [136] XBZ Block
# [137] XBT Block
# [138] XBZ Block
# [139] XBA Block
# [140] XBW Block
# [141] XBZ Block
# [142] XBT Block
# [143] XBZ Block
# [144] XBA Block
# [145] XBW Block
# [146] XBZ Block
# [147] XBT Block
# [148] XBZ Block
# [149] XBA Block
# [150] XBW Block
# [151] XBZ Block
# [152] XBT Block
# [153] XBZ Block
# [154] XBA Block
# [155] XBW Block
# [156] XBZ Block
# [157] XBT Block
# [158] XBZ Block
# [159] XBT Block
# [160] XBA Block
# [161] XBW Block
# [162] XBZ Block
# [163] XBT Block
# [164] XBZ Block
# [165] XBT Block
# [166] XBA Block
# [167] XBW Block
# [168] XBZ Block
# [169] XBT Block
# [170] XBZ Block
# [171] XBA Block
# [172] XBW Block
# [173] XBZ Block
# [174] XBA Block
# [175] XBW Block
# [176] XBZ Block
# [177] XBT Block
# [178] XBZ Block
# [179] XBA Block
# [180] XBW Block
# [181] XBZ Block
# [182] XBT Block
# [183] XBZ Block
# [184] XBA Block
# [185] XBW Block
# [186] XBZ Block
# [187] XBT Block
# [188] XBZ Block
# [189] XBA Block
# [190] XBW Block
# [191] XBZ Block
# [192] XBT Block
# [193] XBZ Block
# [194] XBT Block
#