#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Generate some data for the body
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)
    assert len(data) == datalen

    # Now write this data as a block of sub-blocks
    while True:
        if len(data) < 1: break
        
        # Get the length of this sub-block and then just
        # copy it from the data buffer to the main output
        sublen = data[0] + 1
        if len(data) <= sublen: break
        assert len(data) >= sublen
        out.write(data[:sublen+1])
        data = data[sublen+1:]

    # Finally, write an empty EOF marker
    out.write(b'\x00')
