#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Extract and add the image size
    img_width, img_height = struct.unpack("HH", lsd[1:])
    out.write(lsd)

    # TODO: Populate global color table if present
    # TODO: Figure out what those flags mean
    # Add flags
    out.write(b'\x00\x00')

    # Add background color
    out.write(rng.read(1))

    # Loop through every frame in the animation
    for _frame in range(5):
        # Generate part of the Graphics Control Extension field
        gce = rng.read(4)
        
        # Set uninteresting bits
        gce = bytes([
            x | 0b10000000 for x in gce[:2]
        ]) + gce[2:]

        # Write the whole thing to disk
        out.write(gce)

        imgdata = rng.read(img_width * img_height)
        out.write(imgdata)