#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Generate a random color map
    color_map = bytearray()
    for _ in range(0x100):
        color_map.append(rng.read(1)[0])

    # Add the color map to the output
    for i in range(0, len(color_map), 0xFF):
        out.write(struct.pack('<B', 0xFE))
        out.write(color_map[i : i + 0xFF])

    # Generate the dimensions of the image
    width = rng.read(2)[0]
    height = rng.read(2)[0]
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))

    # Generate the graphics control extension packet
    gce = struct.pack('<BBBBH', 0xF9, 0x04, 0x05, 0x00, 0x00)
    out.write(gce)

    # Generate the graphic control label
    gcl = struct.pack('<BBBBHH', 0x2C, 0x00, 0x00, 0x00, width, height)
    out.write(gcl)

    # Generate the image data
    image_data = bytearray([0x47]*((width * height) // 0x0A))
   