#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Get global color table, if present
    gct = None
    if lsd[4] & 0b111 == 0b101:
        gct = rng.read(3*(2**((lsd[6]>>4)+1)) - 1)

    # Add global table to output
    if gct is not None:
        out.write(gct)

    # Get initial colour index for image data
    colorindex = rng.read(1)[0] >> 5

    # Add the Netscape application extension
    appext = b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00'
    out.write(appext)

    while True:
        # Write graphic control extension
        gce = rng.read(1)[0] << 8 | rng.read(1)[0]
        out.write(b'\x21\xf9\x04')
        out.write(struct.pack('HB', gce, rng.read(1)[0]))

        # Get frame dimensions
        width = (rng.read(1)[0]) + ((lsd[1] << 8) & 0xFF00)
        height = (rng.read(1)[0]) + ((lsd[2] << 8) & 0xFF00)

        # Set pixel aspect ratio to square pixels
        pixelratio = 0
        
        # Get buffer length and calculate graphic control delay
        buflen = rng.read(1)[0]
        gcdelay = buflen * 10
