#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    colormap_size = 0b11 << 5 | 0b11111

    colors = [
        0xff0000, # red
        0x00ff00, # green
        0x0000ff] # blue

    color_indexes = []
    for i in range(datalen):
        v = int.from_bytes(
                rng.read(1), byteorder='big')
        color_index = colors[v % len(colors)]

        color_indexes.append(color_index)

    bpp = 8

    repeats = 0
    bg_color = 0xffffff
    aspect_ratio = 1

    global_colour_table_flag = 0x80 # on
    color_resolution = 0x70 # 24 bits
    sort_flag = 0x08 # no sort
    size_of_global_colour_table = 0x07 # 2^bits entries

    flags = global_colour_table_flag \
      | color_resolution \
      | sort_flag \
      | size_of_global_colour_table

    background_colour_index = bg_color >> (3 - bpp)

    pixel_aspect_ratio = 0

    mode = 0x87
    mode_extension = 0
    block_end = 0x00

    out.write(header)
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(first_byte.to_bytes(1, byteorder='big'))
    out.write(second_byte.to_bytes(1, byteorder='big'))
    out.write(colormap_size.to_bytes(1, byteorder='big'))

    for i in range(len(colors)):
        c = (colors[i] & 0xff0000) >> 16
        out.write(c.to_bytes(1, byteorder='big'))

        c = (colors[i] & 0x00ff00) >> 8
        out.write(c.to_bytes(1, byteorder='big'))

        c = (colors[i] & 0x0000ff)
        out.write(c.to_bytes(1, byteorder='big'))

    out.write(block_end.to_bytes(1, byteorder='big'))
    out.write(flags.to_bytes(1, byteorder='big'))
    out.write(background_colour_index.to_bytes(1, byteorder='big'))
    out.write(pixel_aspect_ratio.to_bytes(1, byteorder='big'))
    out.write(mode.to_bytes(1, byteorder='big'))
    out.write(repeats.to_bytes(1, byteorder='big'))
    out.write(0.to_bytes(1, byteorder='big'))
    out.write((width - 1).to_bytes(1, byteorder='big'))
    out.write((height - 1).to_bytes(1, byteorder='big'))