#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG channel

def gen_gif(rng:BinaryIO,out: BinaryIO)->None:

    while True:
    
        # generate header bytes

        hdr = rng.read(6)
        
        if len(hdr) != 6: break

        code = int.from_bytes(hdr[0].encode(),byteorder='little')
        
        print(code)
        
        if code == 0xff:
            continue
        elif code == 0xf9:
            break
        
        # write header with extended code
        
        code += 0b11000000
        
        val = int.from_bytes(
                hdr.replace(hdr[0],code).encode(), 
                byteorder='little')
        
        print('header',val)
        
        out.write(val.to_bytes(
                6, 'little'))

        n = int.from_bytes(
                rng.read(1), 
                byteorder='little')

        print('databytes',n)

        if n == 0x00:

            # block is an extension

            extid = int.from_bytes(
                    rng.read(1), 
                    byteorder='little')

            print('extensionid',extid)

            if extid == 0xfe:

                # loop forever

                print('loopforever')

                blocklen = int.from_bytes(
                        b'\x00\x00\x00',
                        byteorder='little')

                out.write(blocklen.to_bytes(
                        3, 'little'))

            else:

                # unknown extension

                print(f'unknownextension {extid}')
                
                blocklen = int.from_bytes(
                        rng.read(3), 
                        byteorder='little') + 1

                print('blocklength',blocklen)

                out.write(blocklen.to_bytes(
                        3, 'little'))
                
                # write the rest of this extension block
                
                for i in range(blocklen-1):

                    val = int.from_bytes(
                            rng.read(1), 
                            byteorder='little')

                    print('extensiondata',i,val)
                    
                    out.write(val.to_bytes(
                            1, 'little'))


        elif n <= 7:

            # write block data as char

            val = int.from_bytes(
                    rng.read(1), 
                    byteorder='little') << 8 | n

            print('blockheader',n,val)

            out.write(val.to_bytes(
                    2, 'little'))

        else:

            # write block header as short

            val = int.from_bytes(
                    rng.read(2), 
                    byteorder='little')

            print('blockheader',n,val)
            
            out.write(val.to_bytes(
                    2, 'little'))

            # determine how many longs to read and then write them as such
            # Assumes maximum of one byte per long.
            
            numlongs = (val >> 8) - 1
            
            for m in range(numlongs):

                val = int.from_bytes(
                        rng.read(4), 
                        byteorder='little')

                print('pixeldata',m,val)
                
                out.write(val.to_bytes(
                        4, 'little'))

