#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

from typing import Optional, List, Set
from collections import defaultdict

def parse_rule(line: str) -> (str, tuple):
    words = line.split()
    return words[0], tuple(words[1].split(', '))

def get_next_words(rules: dict[str, tuple], word: str) -> list[tuple]:
    next_words = []
    
    for a in rules[word]:
        if len(a) == 1:
            next_words.append((a,))
        else:
            for b in get_next_words(rules, a):
                next_words.append((b[0], a) + b[1:])
                
    return next_words

# this could be more elegant with state machines
def generate_valid_words(rules: dict[str, tuple]) -> set[str]:
    valid_words = {''}
        
    while True:
        new_valid_words = set([])
        for word in valid_words:
            
            # caution! this will also match any of the first letters
            # of longer words; f for example matches foo and foobar
            for words in get_next_words(rules, word[-1]):
                if all(c in valid_words for c in words[:-1]):
                    new_valid_words.add(''.join(words))
        
        if not new_valid_words:
            break
        else:
            valid_words |= new_valid_words
        
    return valid_words
