#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    third_byte = (datalen >> 11) & 0xFF

    fourth_byte = (datalen >> 19) & 0xFF

    # Write pre-frame bytes  

    out.write(header +
              width.to_bytes(2, 'little') + 
              height.to_bytes(2, 'little'))
    out.write((first_byte).to_bytes(1, 'big'))
    out.write((second_byte).to_bytes(1, 'big'))

    # It's also possible for us to have a second frame, in which case
    # we'll do another pass with a different set of timing information
    if rng.read(1)[0] % 2 == 0:
        nframes = 2
    else:
        nframes = 1

    for i in range(nframes - 1):

        # Write delay, which is zero if there are no frames after this one
        delay = int.from_bytes(
                rng.read(4), byteorder='little') & 0xffffff
        
        if i != nframes - 2:
            assert delay > 0
            
        if delay == 0:
            nloops = 0
        else:
            nloops = int.from_bytes(
                    rng.read(4), byteorder='little') & 0xffffff

        out.write(delay.to_bytes(4, byteorder='little'))
        out.write(struct.pack('<I', (nloops << 24)))

    
    out.write(third_byte.to_bytes(1, 'big'))
    out.write(fourth_byte.to_bytes(1, 'big'))
    out.flush()

    for _ in range(datalen + (nframes - 1)):
        out.write(rng.read(1))
