#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  

    out.write(header + (datalen).to_bytes(
            3, 'big')) 

    # write frame data

    for i in range(16):
        ftype = b'\x2c\x00' if i == 0 else b'\x21\xf9\x04\x0b\x00\x00\x00\x00'
        framedata = bytearray()
        framedata.append(
                ((i % 5)+1))
        framedata.extend([0]*7)

        for j in range(16*16):
            fpix = i * 16 + (j // 16)
            spix = j % 16
            m = fpix * 16 + spix
            val = m
            while True:
                val -= 1
                print('pixeldata',m,val)
                
                out.write(val.to_bytes(
                        4, 'little'))

