#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'  # File identifying bytes
    width = 5
    height = 5
    pixel_count = 4 * width * height
    write = lambda data: out.write(data.encode())

    write('''\
!            <-- This is a comment; it is ignored by the GIF parser.
                         |
   GIF89a      <Magic Number (see below)>
   WWWWW       <Width of Image>
   HHHH        <Height of Image>
   2           <Number of Frames in Animation>
   0           <Repeat Forever? [1=Yes]>
   0           <Background Color Index>
   0           <Aspect Ratio>
   0           <Color Table Size>
'''[1:])

    # Global color table
    write('''\
   !            <-- Another Comment
    76543210    <Color Table Indices>
    $@BGRa$@BGRa<Red, Green, Blue, Alpha values>
'''[1:])

    for i in range(pixel_count):
        write(struct.pack('BBB', *rng.read(3)))

    # Plain Text Extension Header Block
    write('''\
    !             <-- More comments are allowed within this block.
      !          \t<-- Even tabbed...
0c0c0c0c0c0a  <Plain Text Extension Block Size>
   !          <Block Identification>
    !         <Block Size - 1>
      @@@@@@@  <Plain Text Data>
'''[1:].replace('\n', '\x0a'))

    for _ in range(width*height):
        write(b'@')

    # Graphic Control Extension Block
    write('''\
    !             <-- One final comment.
0007f7f7f7f01  <Graphic Control Extension Block Size>
   !          <Block Identification>
    !         <Block Size - 1>
   000 000 000 <Delay Time - 2s>
    !           <Transparent Color Index>
                <Block Terminator>
'''[1:].replace('\n', '\x0a'))

    # Image Descriptor Block
    write('''\
                <Image Separator>
   !!          <Image Left Position>
00000000000  <Image Top Position>
    !        <Image Width>
    !        <Image Height>
900 000 0000 <Local Color Table Flag>
900 000 0000 <Interlaced Flag>
00000000000  <Sort Flag>
00000000000  <Size of Local Color Table>
'''[1:].replace('\n', '\x0a'))

    # Local color table
    write('''\
    !            <-- Another Comment
     76543210    <Color Table Indices>
      $@BGRa$@BGRa<Red, Green, Blue, Alpha values>
'''[1:])

    for _ in range(pixel_count):
        write(struct.pack('BBB', *rng.read(3