#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    # Size (x, y), flags 0x80 means color table follows at end of image data
    size = struct.pack('<H', 512 * 512 | 0x80)
    img_desc = struct.pack(
            '<BBBHH',
            0b01000000,   # 0x80 - transparency flag
            0b00000000,   # :0x0 - no local color table
            0x1,          # background color index
            0x0, 0x0,     # x, y offsets
    )
    colormap = bytearray([
        0xff, 0xff, 0xff,      # white
        0x7f, 0xbf, 0xf0,      # halfway between
        0x2f, 0x4f, 0x8f,      # darkest
    ])
    # Pixel data is a stream of bytes alternating between zero and non-zero values,
    # so that 8 pixels per byte can be packed
    data = bytearray()
    first = True
    while True:
        val = int.from_bytes(rng.read(1), 'big')
        for i in range(8):
            if first:
                val ^= 0b10000000
            data.append(val & 0b1111)
            val >>= 4
        first = not first
    assert len(data