#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

from crypto_commons.generic import list_files, load_pem, pkcs7e_unpad
from Crypto.PublicKey import RSA
from Crypto.Random import get_random_bytes as rng
import math

# Get all the files in the directory
files = [x for x in sorted(list_files('.')) if x != __file__]
if len(files) < 2:
    print("not enough input files")
    sys.exit(-1)

key = load_pem(files[-1])
rsa_pubkey = RSA.importKey(key)
out = open(f'{files[-1].replace(".pem", "")}.dec', 'wb')
for f in files[:-1]:
    # Generate some data for the body
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    # Pad the data to 16 bytes of length and write it to the output stream
    paddinglen = 16 - len(data) % 16
    if paddinglen == 0:
        padding = b''
    else:
        padding = rng.read(paddinglen)
    out.write(data + padding)

