#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    global_color_table_size = (rng.read(1)[0] % 4 + 1) * 2

    if width > 1 and height > 1:
        local_color_table_size=None
    else:
        local_color_table_size=(rng.read(1)[0] % 4 + 1) * 2


    # Output the GIF header
    out.write(header)

    # Output logical dimensions of the canvas
    out.write((width & 0xFF).to_bytes(
            1, "big"))
    out.write(((width >> 8) & 0xFF).to_bytes(
            1, "big"))
    out.write((height & 0xFF).to_bytes(
            1, "big"))
    out.write(((height >> 8) & 0xFF).to_bytes(
            1, "big"))
    
    # Output the global color table
    for i in range(0, 2**global_color_table_size):
        
        red = int.from_bytes(
                rng.read(1),
                byteorder="big")&0xff
        green = int.from_bytes(
                rng.read(1),
                byteorder="big")&0xff
        blue = int.from_bytes(
                rng.read(1),
                byteorder="big")&0xff

        out.write(struct.pack("BBB", red,green,blue))

    # Output a single frame
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00')

    # Output the frame offset
    out.write((width & 0xFF).to_bytes(
            1, "big"))
    out.write(((width >> 8) & 0xFF).to_bytes(
            1, "big"))
    out.write((height & 0xFF).to_bytes(
            1, "big"))
    out.write(((height >> 8) & 0xFF).to_bytes(
            1, "big"))

    # Write the local color table size
    out.write((local_color_table_size | 0x80).to_bytes(1,"big"))

    # Write the local color table
    for i in range(0, 2**local_color_table_size):
        
        red = int.from_bytes(
                rng.read(1),
                byteorder="big")&0xff
        green = int.from_bytes(
                rng.read(1),
                byteorder="big")&0xff
        blue = int.from_bytes(
                rng.read(1),
                byteorder="big")&0xff

        out.write(struct.pack("BBB", red,green,blue))

    # Write the row offsets
    offsets = []
    for y in range(0, height):
        offsets.append((y * width) // 8)

    out.write(struct.pack("<{}H".format(height), *offsets))
    
    # Write the pixel data
    for y in range(0, height):
        for x in range(0, width):
            start = 8*int((x+y)*local_color_table_size/(height * width))
            end = 16

        val = int.from_bytes(
            rng.read(end - start), 
            byteorder="little")%256

        out.write(((val).to_bytes(
            min(end - start, 1), "little")))
