#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(1), byteorder='big', signed=False) % 256

    # Header
    out.write(header)

    # Logical screen descriptor
    out.write(b'\0\0')                   # Width
    out.write(b'\0\0')                   # Height
    out.write(b'\x87')                   # GCT color table flag and size (2^8 + 7)
    out.write(int.to_bytes(4, length=1, byteorder='little'))   # Background color index
    out.write(int.to_bytes(1, length=1, byteorder='little'))   # Pixel aspect ratio numerator
    out.write(int.to_bytes(0, length=1, byteorder='little'))   # Pixel aspect ratio denominator
    out.write(int.to_bytes(0, length=1, byteorder='little'))   # Global color table flag and size
    out.write(b'\0'*3)                    # Background color red green blue

    # Image descriptor
    out.write(b';')                      # Separator
    out.write(int.to_bytes((datalen >> 8)&0xff, length=1, byteorder='little'))    # Left offset
    out.write(int.to_bytes(datalen&0xff, length=1, byteorder='little'))              # Top offset
    out.write(int.to_bytes((datalen >> 8)&0xff, length=1, byteorder='little'))    # Width
    out.write(int.to_bytes(datalen&0xff, length=1, byteorder='little'))              # Height
    out.write(int.to_bytes(0, length=1, byteorder='little'))                        # Flags
    out.write(b'\0')                                                       # Color index
    out.write(int.to_bytes(0, length=1, byteorder='little'))                        # Sorting code

    # Block to be duplicated randomly in each frame
    dupblocklen = int.from_bytes(rng.read(1), byteorder='big', signed=False) % 10
    block = rng.read(dupblocklen+1)

    for i in range(datalen):
        if rng.read(1)[-1] == 0:
            blockdata = rng.read(i+1)
            out.write(blockdata[:dupblocklen])
            out.write(blockdata[dupblocklen+1:])
        else:
            out.write(rng.read(i))


