#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  

    out.write(header + (datalen).to_bytes(
            3, 'big')) 

    # write frame data

    for n in range(12):

        if n == 4 or n == 5:

            # write delay time as shorts

            val = int.from_bytes(
                    rng.read(2), 
                    byteorder='little')

            print('delay',n,val)
            
            out.write(val.to_bytes(
                    2, 'little'))

        else:

            # write block header as short

            val = int.from_bytes(
                    rng.read(2), 
                    byteorder='little')

            
            print('block',n,val)

            out.write(val.to_bytes(
                    2, 'little'))

            out.write((val).to_bytes(
                    2, 'little'))

    # write LZW mincode size

    val = int.from_bytes(
            rng.read(1), 
            byteorder='little')

    print('mincodesize',val)

    out.write(val.to_bytes(1, 'little'))

    # LZW code size table

    codes = []

    for i in range(1 << val):
        
        val = int.from_bytes(
                rng.read(1), 
                byteorder='little')

        codes += [val]

    # Write LZW tables and image

    # LZW code size table

    for t in range(len(codes)):

        val = codes[t]
        print('codetable',t,val)
        out.write(val.to_bytes(
                1, 'little'))

    # write image descriptor

    out.write((((0 << 7) | 
               (0 << 6) |
               (0 << 5) |
               ((1 << 4)) |
               (0b0110 << 2)))
             .to_bytes(1, 'big'))

    width = int.from_bytes(
            rng.read(1), 
            byteorder='little')

    height = int.from_bytes(
            rng.read(1), 
            byteorder='little')

    posx = int.from_bytes(
            rng.read(1), 
            byteorder='little')

    posy = int.from_bytes(
            rng.read(1), 
            byteorder='little')

    print('imagedesc',width,height,posx,posy)

    out.write(
            width.to_bytes(1, 'little'))

    out.write(
            height.to_bytes(1, 'little'))

    out.write(
            posx.to_bytes(1, 'little'))

    out.write(
            posy.to_bytes(1, 'little'))


    lzw = rng.read()
    out.write(lzw)

    # Pad with extra zeroes to make it look like a valid image

    for x in range(1024 - len(lzw)):

        out.write(bytearray([0]))

