#!/usr/bin/env python3
import serial
import random, time



def test_simple(s):
    
    # write some data to the board
    s.write('hello world'.encode())

    res = b''
    while True:

        x = s.read()
        print(x)
        res += x
        
        if len(res)>6 and res[-5:]==b'\r\n> ':
            break


    print(res[:-5])





if __name__=="__main__":

    ser = serial.Serial("/dev/ttyACM0")
    time.sleep(2)
    test_simple(ser)

    # lets test writing a few bits back and forth

    for i in range(8):
        testbits = (0xff << i*8) & ((1<<32)-1)
        print("testbits",i,"=",hex(testbits))
        ser.write((testbits>>24).to_bytes(1,'big'))
        ser.write((testbits>>16).to_bytes(1,'big'))
        ser.write((testbits>>8).to_bytes(1,'big'))
        ser.write((testbits>>0).to_bytes(1,'big'))
        res=0
        for j in range(4):
            res |= int.from_bytes(ser.read(),'little')<<(j*8)
        print("result ",i,"=",hex(res))



    # this should be an error condition
    try:
        ser.write((-1).to_bytes(4,'little'))
    except Exception as e:
        pass
    
    # here we expect it to pass because we are not reading back
    try:
        ser.write(random.getrandbits(32).to_bytes(4,'little'))
    except Exception as e:
        raise RuntimeError("Wrote invalid value but failed silently!")


        
    # now let's test out our bit-bang mode
    ser.write(((0xf0 << 24)|9).to_bytes(4,'little'))
    res = b''
    while True:

        x = ser.read()
        res+=x
        print(x.hex().upper())

        
        if len(res)==7:
            break

    
    print("out",res[2],res[3],res[4],res[5].hex().upper())
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    print("loopvar",loopvar)


    # now we'll try to do something dumb that has no reason to work
    ser.write(((badval << 24)|9).to_bytes(4,'little'))
    res = b''
    counter = 0 
    while True:

        x = ser.read()
        res+=x
        print(counter, x.hex().upper())
        counter += 1
        if counter > 10 or (len(res)==7 and res[:4]==b"dead"):
            break

    
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    badval ^= 0b10000000;
    badval &= ~int.from_bytes(b"\xff"*4,'little')
    badval |= (loopvar&0b11111111)+(loopvar>>8)&((1<<24)-1);
    ser.write(((badval << 24)|9).to_bytes(4,'little'));
    print("out",res[2],res[3],res[4],res[5].hex().upper(),bin(loopvar),bin(badval>>24))
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    loopvar |= (loopvar&0b11111111)+(loopvar>>8)&((1<<24)-1);
    print("loopvar",loopvar)


    ser.flushInput();
    ser.write((0<<24).to_bytes(4,'little'))
    res = b''
    counter = 0 
    while True:

        x = ser.read()
        res+=x
        print(counter, x.hex().upper())
        counter += 1
        if counter > 10 or (len(res)==7 and res[:4]==b"dead"):
            break

    
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    badval ^= 0b10000000;
    badval &= ~(int.from_bytes(b"\xff"*4,'little'))
    badval |= (~loopvar&0b11111111)+(~loopvar>>8)&((1<<24)-1)
    ser.write(((badval << 24)|9).to_bytes(4,'little'));
    print("out",res[2],res[3],res[4],res[5].hex().upper(),bin(loopvar),bin(badval>>24))
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    loopvar |= (~loopvar&0b11111111)+(~loopvar>>8)&((1<<24)-1)
    #loopvar |= (0b10000000)
    print("loopvar",loopvar)


    ser.flushInput();
    ser.write((((badval^0b10000000)<<24)|9).to_bytes(4,'little'))
    res = b''
    counter = 0 
    while True:

        x = ser.read()
        res+=x
        print(counter, x.hex().upper())
        counter += 1
        if counter > 10 or (len(res)==7 and res[:4]==b"dead"):
            break

    
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    #loopvar |= 0b10000000
    print("out",res[2],res[3],res[4],res[5].hex().upper(),bin(loopvar),bin(badval>>24))
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    #loopvar |= 0b10000000
    print("loopvar",loopvar)


    ser.flushInput();
    ser.write((((badval^0b10000000)<<24)|9).to_bytes(4,'little'))
    res = b''
    counter = 0 
    while True:

        x = ser.read()
        res+=x
        print(counter, x.hex().upper())
        counter += 1
        if counter > 10 or (len(res)==7 and res[:4]==b"dead"):
            break

    
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    #loopvar |= 0b10000000
    print("out",res[2],res[3],res[4],res[5].hex().upper(),bin(loopvar),bin(badval>>24))
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    #loopvar |= 0b10000000
    print("loopvar",loopvar)




    ser.write(0xdeadbeef.to_bytes(4,'little'))
    res = b''
    counter = 0 
    while True:

        x = ser.read()
        res+=x
        print(counter, x.hex().upper())
        counter += 1
        if counter > 10 or (len(res)==7 and res[:4]==b"dead"):
            break

    
    loopvar = (((res[2]<<8)<<4)|(res[3]<<4)|(res[4]>>4)).to_bytes(4,'big') ^ ((res[5]&0b1111)<<24)
    print("out",res[2],            | 0b10000000)
    print("loopvar",loopvar)
    if loopvar == 0xff:
        badval = 0
    else:
        badval = loopvar + 1

    print