#!/usr/bin/env python3

import struct
from typing import BinaryIO

# https://github.com/python-pillow/Pillow/issues/290
def _save(im, fp, filename):
  try:
    im._dump(_getencoder(im.mode, fp, filename))
    return 1
  except Exception as e:
    print("Error saving", fp)
    raise(e)


def extract_image(rng : BinaryIO, width : int, height : int):

    frames = []
    
    # A single frame can contain multiple images so loop until we get one
    while len(frames)!= 1:

        skip = (width * height - 768) // 4
        
        # Skip to next frame
        rng.seek(skip + 1, 1)
        
        num_patterns = int.from_bytes(rng.read(1), byteorder="big")
        assert((num_patterns >= 1 and num_patterns <= 2)), \
            f"Unexpected number of patterns {num_patterns}"
            
        for i in range(num_patterns):
            
            # Read pattern header
            rng.seek(-1, 1)

            # Number of lines
            num_lines = int.from_bytes(rng.read(1), byteorder="big")
            assert((num_lines == 1 or num_lines == 2)), \
                f"Expected 1 or two lines but got {num_lines}"
                
            if num_lines == 1:
                
                # Read line data
                _save(im=Image.new('L', (width, height)),
                        fp=frames[-1],
                        filename="")
                

                break
            else:
                for j in range(height):
                    ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
     
    # Read past this frame
    while True:
        blocktype = int.from_bytes(
                rng.read(1), byteorder="big")
