#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)
    # Avoid excessive looping & memory usage by making the palette length
    # bounded to 4 bits (< 16 colors). We can fix this by adding some more
    # bytes to the end of the data stream (this is what browsers do too).
    footer = b''+ b','.join([str(i % 16).encode() for i in range(int(datalen / 7))]) + b'\x00\x3B'

    while len(footer) > 255 - datalen:
        footer += [b',%d' % x for x in
                   0 <= int.from_bytes(
                rng.read(1), 'big') < 6]
    out.write(header + len(data).to_bytes(2, 'big') + \
                data + footer)
