#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)

    # Frame header
    out.write(header)
    # Image dimensions
    width = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    height = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))
    # Flags
    flags = (int.from_bytes(
            rng.read(1), byteorder='big')) & 7
    out.write((flags).to_bytes(1, 'big'))
    # Color table size
    colortablesize = (int.from_bytes(
            rng.read(1), byteorder='big')) & 7
    out.write((colortablesize).to_bytes(1, 'big'))
    # Background color index
    bgcoloridx= (int.from_bytes(
            rng.read(1), byteorder='big')) & 255
    out.write((bgcoloridx).to_bytes(1, 'big'))
    # Pixel aspect ratio
    pixelaspectratio= (int.from_bytes(
            rng.read(1), byteorder='big')) & 255
    out.write((pixelaspectratio).to_bytes(1, 'big'))


    for i in range(0, 2):
        lzwmincodesize = (int.from_bytes(
                rng.read(1), byteorder='big')) & 255
        out.write((lzwmincodesize).to_bytes(1, 'big'))

    data = []

    while len(data)!= datalen - 1:
        a = (int.from_bytes(
                rng.read(1),byteorder='big'))
        if a > 247:
            out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
