#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  
    out.write(header)
    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))

    # Write frames until we reach the end of animation
    while True:

        # Write frame block size and flags
        blocksize = int.from_bytes(
                rng.read(1), byteorder="big"
            ) | (1 << 7)
        #print("block size", blocksize)
        out.write((blocksize).to_bytes(length=1, byteorder="big"))


        # Write width and height
        w = int.from_bytes(rng.read(2), byteorder="big")
        h = int.from_bytes(rng.read(2), byteorder="big")
        
        out.write((w).to_bytes(length=2, byteorder="big"))
        out.write((h).to_bytes(length=2, byteorder="big"))

        bgcol    = int.from_bytes(rng.read(1), byteorder="big")
        aspect   = int.from_bytes(rng.read(1), byteorder="big")
        flags    = int.from_bytes(rng.read(1), byteorder="big")
        duration = int.from_bytes(rng.read(2), byteorder="big")

        print("bgcol:", bgcol,
              "aspect:", aspect,
              "flags:", hex(flags))

        # Write palette
        if (flags >> 7) & 1 == 1:
            print("has global color table")

            for c in range(2 ** (aspect + 1)):
                r = int.from_bytes(
                        rng.read(1), byteorder="big")

                g = int.from_bytes(
                        rng.read(1), byteorder="big")

                b = int.from_bytes(
                        rng.read(1), byteorder="big")
                
                out.write(struct.pack(">BBB",r,g,b))
            
        else:
            print("no global color table")

            for p in range(1 << (aspect+1)):
                idx = int.from_bytes(
                        rng.read(1), byteorder="big")
                
                out.write(((idx).to_bytes(1,"big")))

        # Write lzw compressed data
        bytes_left = blocksize - \
                     len(header)-len(struct.pack(">HHB", w, h, flags))+2

        print("bytes left",bytes_left)

        while bytes_left > 0:
            
            controlbyte = int.from_bytes(
                    rng.read(1), byteorder="big")

            n = min(controlbyte, bytes_left)

            out.write((((1 << n) - 1) ^
                       (controlbyte & ~(~0 << n))).to_bytes(n,"big"))


            bytes_left -= n

        if bytes_left < 0:
            print("Warning: negative bytes left!")

