#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        out: BinaryIO,
        rng: BinaryIO,
        width: int, height: int) -> None:

    # Write the GIF magic value
    out.write("\x47\x49\x46\x38".encode())
    # Version of the GIF format to use
    out.write("\x39\x39\xeb\x0e".encode())

    # Flags field contains:
    #   1: Color table size (gives 8 bits per pixel)
    #   1: Has global color table flag
    #   1: Color resolution
    #       (should be full color mode with 7 colors)
    #   1: Sort flag
    #      (sorts the output according to image order)
    #   1: Size of global color table (must be zero because we don't have one)
    #   0: Background color index
    flags = "\x04"

    # Global color table length
    # Doesn't matter for our purposes
    gctlen = b"\x00"

    # Background color index
    bgindex = bytes([badval])

    # Pixel aspect ratio
    pxratio = b"\x00"

    # Put all these together and write them at the start
    header = flags + gctlen + bgindex + pxratio
    print("header:",header)
    out.write(header)

    # Image descriptor block
    imgdesc = ("\x2c\x00"     # Separator ","
               + str((width+1)//8-1).zfill(4).encode()         # Data length field
               + str(height).zfill(4).encode()                  # Height
               + str((width*height//8)+1).zfill(4).encode()    # Width
               + b"\x00"                                          # No local ctable
               + b"\x00")                                         # Interlace off

    # Put it all together
    out.write(imgdesc.encode())

    # LZW minimum code size
    lzwsize = b"\x08"

    # Write this
    out.write(lzwsize)

    # Prefix end character
    pfxend = "!"[0].to_bytes(1,"big")
    
    # Write that
    out.write(pfxend)

    datalen = width * height // 8
    print("datalen",datalen)

    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") % 256) + 1).to_bytes(1,"big"))

    # Now write the loop variable
    # -1 is 0xff
    #  0 is 0x00
    #  1 is 0x01
    # etc
    loopvar = (
            (int.from_bytes(
                rng.read(1), byteorder="big") % 5)
            | 0b10000000)
    print("loopvar",loopvar)
    if loopvar == 0xff:
        badval = 0
    else:
        badval = loopvar + 1

    print