#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    # length of LZW min code size (always 0x82 in this case
    out.write(header + b'\0\0\0\x82')
    chunkdata = bytes()
    for _ in range((datalen+1)//15):
        chunkdata += \
            rng.read(15)
    if len(chunkdata) % 8!= 0:
        chunkdata = chunkdata[:-(len(chunkdata)%8)]
    chunklen = len(chunkdata).to_bytes(
        4, 'big', signed=False
    )
    out.write(b''.join([
            chunklen,
            chunkdata,
            struct.pack('B', 0x00)
          ]))

