#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')
    header = [
        0x100 * (rng.randrange(256) + 1),
        0x100 * (rng.randrange(256) + 1),
        b'\xf7\xff\xff',  # No global palette
    ]

    for _ in range(4):
        packed = []

        for i in range(0, 8):
            byte = 0

            for j in range(0, 8):
                bit = bool(rng.randrange(2))

                if i == 0 and j < 3:
                    header[j] |= int(bit) << i
                    continue

                byte |= int(bit) << (7 - j)

            packed.append(byte)

        out.write(bytes(packed))

    size = out.tell()
    nframes = min((size // 256) // len(header), 255)
    colors = list({int.from_bytes(header[i : i + 3], 'big') for i in range(0, 256, 3)})[:256]
    frames = [[colors[(rng.randrange(len(colors)))]] for _ in range(nframes)]

    out.seek(0)
    out.write(struct.pack('>BBBBBBB', *(list(header) + [0] * 3)[-7:]) + bytes([nframes]))

    # Frames
    for frame in frames:
        out.write(b'\x2c')  # Image separator
        out.write(bytes(frame))
        out.write(bytes([(0xe4 | rng.randrange(3)) & 0xff]) + bytes([0xff, 0]))

