#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  

    out.write(header + (datalen).to_bytes(
            3, 'big')) 

    # write frame data

    for n in range(12):

        if n == 4 or n == 5:

            # write delay time as shorts

            val = int.from_bytes(
                    rng.read(2), 
                    byteorder='little')

            print('delay',n,val)
            
            out.write(val.to_bytes(
                    2, 'little'))

        else:

            # write block header as short

            
            hlen = int.from_bytes(
                    rng.read(1), 
                    byteorder='big') & 0xff
            
            print('block header',n,hlen)

            out.write((hlen).to_bytes(
                    1, 'big'))
            
            # write image block data
            
            while (hlen > 0):

                hlen -= 1
                
                if hlen == 0 and n!= 7:

                    # write terminator
                    
                    print('term',n,hlen)

                    out.write(b'\x00')
                    
                elif hlen == 0 and n == 7:

                    # write end of graphics control extension

                    print('egc term',n,hlen)

                    out.write(b'\x00\x00')

                else:

                    # write block data
                    
                    byt = rng.read(1)
                     
                    print('data',n,hlen,byt)
                    
                    out.write(byt)


    # write trailer

    print('trailer')

    out.write(b';)')


