#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    num_blocks = int.from_bytes(
            rng.read(1), 'big') % (2 ** 6 - 4) # maximum blocks is 63
    if num_blocks == 0:
        return

    data = bytearray()
    for _ in range(num_blocks):
        block_size = int.from_bytes(\
                rng.read(1), 'big') & 0xFF
        data += rng.read(block_size)

    footer = [b'\x00', b'\x3B'][
            0 <= int.from_bytes(
                rng.read(1), 'big') < 6]
    out.write(header + len(data).to_bytes(2, 'big') + \
                data + footer)
