#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = int(2**(int((lsd[4] >> 5)))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    # Make sure we're starting fresh with each frame
    imgdatalen = 0
    codelen = None
    dispose = None
    prevframe = False
    
    while True:
        # Create new frame        
        out.write(b'\x2c')
        
        # Create image descriptor
        id = rng.read(9)
        out.write(id)
