#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# bytes in `in`.
def generate(in: BinaryIO, out: BinaryIO):
    header = b'GIF87a'

    datalen = 0x110
    data = (
        b'\xf7\xff\xff\xff' + # Logical screen descriptor with all defaults set except pixel aspect ratio
        b'\x00\x0b' +         # Width of logical screen is 11 pixels (450 dpi * 11 / 72)
        b'\x00\x0c' +         # Height of logical screen is 12 pixels (450 dpi * 12 / 72)
        b'\x82\x01' +         # Global color table with 2 entries containing black and white
        b'\x00'               # Background index is zero
        )

    for i in range(datalen - len(data)):
        data += struct.pack('B', in.read())

    footer = b';'
    out.write(header +
           datalen.to_bytes(2, 'big') +
           data +
           footer)
