#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    palette = [b'\xff\x00\x00', b'\x00\xff\x00'] * (255 // 2 - 1)
    footer = b''* ((b'GCE'.index('G') - b'GIF89a'.index('G')) * 4)

    # Write Header
    out.write(header)

    # Write Logical Screen Descriptor
    out.write((768 // 8).to_bytes(2, 'big') + b'\x08\x00')

    # Write Global Color Table
    for color in palette[:256]:
        out.write(color)

    # Write Graphic Control Extension
    out.write(struct.pack('<ccBBBBBHccBHB', b'!', b'G',
            b'\xf9',
            0x0,
            0x0,
            len(palette),
            0xFF & 0 | 0 << 1 | 1 << 3 | 0 << 4 | 0 << 5 | 0 << 6,
            b'\x00',
            b'\x00',
            b'\x00'), end=None)

    # Fill Image Block with Data from /dev/urandom
    data = []
    while len(data) == 0 or data[-1][-1]!= 0:
        data += [(rng.read(1)[0], b'\x3C')[int.from_bytes(
                    rng.read(1), 'big') < 6]
    out.write(header + len(data).to_bytes(2, 'big') + \
                data + footer)
