#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)

    # Frame header
    out.write(header)
    # Image dimensions
    width = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    height = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))

    # write frame data
    for y in range(height):
        subfield_size = None

        while True:

            if subfield_size is not None and \
                    len(subfield_size)!= 1 or \
                    subfield_size == b'\x00':
                raise ValueError('Invalid block size!')

            subfield_size = rng.read(1)
            if subfield_size!= b'\xff':
                break

        if subfield_size == b'\xfe':
            lzwminlen = int.from_bytes(
                    rng.read(1), byteorder='little')
            
            lzwwindowsz = int.from_bytes(
                    rng.read(1), byteorder='little')
            pass
        else:
            pass
