#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    third_byte = (datalen >> 11) & 0xFF

    fourth_byte = (datalen >> 19) & 0xFF

    # Write pre-frame bytes  

    out.write(header +
              width.to_bytes(2, 'little') + 
              height.to_bytes(2, 'little') + 
              bytes([first_byte]) +
              bytes([second_byte]))


    # write frame data

    for i in range(1, (datalen//6)+1):
        bytes_written = out.write((third_byte, fourth_byte))
        
        assert bytes_written == 2

        # Use the rest of our RNG output as pixel data
        if (i*6 % datalen)!= 0:
            third_byte, fourth_byte = fourth_byte, ((int.from_bytes(
                rng.read(1), 
                byteorder="little"))).to_bytes(1,"little")

            bytes_written=out.write(fourth_byte)
            
            assert bytes_written==1

    
    # Finish off with some randomness as well? Not entirely convinced this is
    # really necessary but it can't hurt either... 

    end = len(rng.read())

    for i in range(len(rng.read())):
        val = int.from_bytes(
            rng.read(end - start), 
            byteorder="little")%256

        out.write(((val).to_bytes(
            min(end - start, 1), "little")))
