#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    out.write(header+b'\0\x2A')
    while len(data):
        if len(data) >= 768:
            chunklen = bytes([1]) + \
                    (struct.pack('>H', 768))
        else:
            chunklen = bytes([1]) + \
                    (struct.pack('>H', len(data)))
        out.write(chunklen+data[:768])

        data = data[768:]

