#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  
    for c in header:
        out.write(c.to_bytes(1, 'little'))
        
    # Write screen size
    for i in range(2):
        out.write((0x20).to_bytes(1, 'little'))

    for i in range(4):
        out.write(((datalen>>i*8)&0xff).to_bytes(1, 'little'))

    # Write frame
    color_table_size = (int.from_bytes(
            rng.read(1),byteorder='little')&0xf)+1
    out.write(color_table_size.to_bytes(1,'little')) # TODO - default to 7?
    out.write((b'\xf7')[0])                         # Global Color Table flag set bit
    for i in range(color_table_size * 3):
        out.write((0).to_bytes(1, 'little'))        

    for i in range(datalen+1):
        out.write((rng.read(1)[0] | 0x80))
    print