#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  
    out.write(header)
    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") % 256) + 1).to_bytes(1,"big"))

    # Now write the loop variable
    # -1 is 0xff
    #  0 is 0x00
    #  1 is 0x01
    # etc
    loopvar = (
            (int.from_bytes(
                rng.read(1), byteorder="big") % 5)
            | 0b10000000)
    print("loopvar",loopvar)
    if loopvar == 0xff:
        badval = 0
    else:
        badval = 3*loopvar
    out.write((badval).to_bytes(1,"big"))
    
    # Next comes the frame descriptor bytes
    # 
    # Each of these fields has two components:
    # * The actual value to use, which varies depending on whether
    #   there are some sort of color table present or not.
    # * Whether it's user modifiable or not. It seems like "user modifiable"
    #   means that the application could be expected to set this field.
    # 
    # In this case I'm going to assume we're always going to have an empty
    # color table and just hardcode all of them as zeroes. The only value here
    # seems to be the delay time, which we'll also make very small.
    imgdesc = [
            0,       # left
            0,       # top
            0,       # width
            0,       # height
            0,       # local color table flag
            0,       # interlace flag
            0,       # sort flag
            0,       # reserved
            0,       # size of global color table
            0,       # background color index
            0,       # pixel aspect ratio
            0,       # transparent color index
            0,       # delay time
            0]       # transparent color flag
    # FIXME: actually calculate the above values based on what
    #        was sampled from /dev/urandom

    outbuf = []
    # first append each of our 9 integers
    for v in imgdesc:
        outbuf += [(v).to_bytes(1,"little")]
    # next add the terminator
    outbuf[-1] = 33.to_bytes(1,"little")    

    # Finally, put everything together
    for b in outbuf:
        out.write(b)
    
