#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Generate some data for the body
    width = rng.read(1)[0] % 256 + 1;
    height = rng.read(1)[0] % 256 + 1;
    print("Width:", width, "Height:", height)
    
    # Write this metadata as LZW compressed blocks
    lzw_num_bits = rng.read(1)[0] % 9 + 1;
    lzw_min_code_size = rng.read(1)[0];
    print("LZW Num Bits:", lzw_num_bits);
    print("LZW Min Code Size:", lzw_min_code_size);
    out.write(bytes([lzw_num_bits]))
    out.write(bytes([lzw_min_code_size]))

    byte_buf = bytes()
    while len(byte_buf)!= width * height:
        # Add in another byte if we don't have enough yet
        if len(byte_buf) < width * height and rng.read(1):
            byte_buf += struct.pack('B', rng.read(1)[0])

        # Compress with LZW algorithm
        cmd = b'\x00'
        while True:
            try:
                cmd += bytearray([(byte_buf[-1] >> i & 1) << i for i in range(7,-1,-1)])[:-1]
                del byte_buf[-1]
            except IndexError:
                break

            if rng.read(1):
                continue
            
            pfx = [cmd[i : i+8] for i in range(0,len(cmd),8)]
            print(pfx)
            out.write(struct.pack('<H', pfx.index(bytearray(list(reversed((0xff).to_bytes(8,'big')))))) + bytes(cmd))
            break

