#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        bytes=rng.read(4), byteorder='little')

    # Generate a pixel buffer of RGBA values
    pixels = [struct.pack('BBBB', *rng.read(4)) for i in range(datalen)]

    # Insert the length as an unsigned short before writing to file
    data = len(pixels).to_bytes(2, 'little') + b"".join(pixels)
    footer = b'\x3B\xFF'


    with open('../data/output.gif', 'wb') as f:
        f.write(header +
                data + footer)
