#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
            break

        # Check for transparency flag
        has_transparency = False
        if (((fieldbyte >> 3) & 1)):
            has_transparency = True

        # Get size of the following data blocks
        size = int(((ord(rng.read(1)) << 8) | (ord(rng.read(1)))))

        # Skip extra bytes in case there are more than one frame
        if size!= 0:
            rng.read(size)

            continue

        imgdata = rng.read(size)
        out.write(imgdata)