#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Sep  7 18:19:26 2020
@author: <NAME>
"""
import argparse
import sys
import random


parser = argparse.ArgumentParser()

group = parser.add_mutually_exclusive_group(required=True)

group.add_argument('-f','--file', nargs='?', const='',
                   help='destination file name')

group.add_argument('-s','--stdout', action='store_true', default=False,
                   help='output to stdout')

args = parser.parse_args()



rng = random.SystemRandom()

if args.file is not None and len(args.file)>0 :

    try:

        with open(args.file,'wb') as out:

            generate_random_data(out)

    except OSError:
        
        print("File error", file=sys.stderr)
    
    finally:

        if out!= sys.stdout:

            out.close()
else:

    generate_random_data(sys.stdout)









    
      
def generate_random_data(out):

    header = b'\xff\xd8'

    # get image size

    while True:
        

        width = rng.randint(0, 65536//16)*16

        if width % 32 == 0:
            
            break

    while True:
        
        height = rng.randint(0, (65536*2)//16)*16
        
        if height % 16 == 0:
            
            break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    third_byte = (datalen >> 11) & 0xFF

    fourth_byte = (datalen >> 19) & 0xFF

    # Write pre-frame bytes  

    out.write(header +
              width.to_bytes(2, 'little') + 
              height.to_bytes(2, 'little') + 
              bytes([first_byte]) +
              bytes([second_byte]))


    # write frame data

    for i in range((datalen)):

        start = 0
        end = 16

        val = int.from_bytes(
            rng.read(end - start), 
            byteorder="little")%256

        out.write(((val).to_bytes(
            min(end - start, 1), "little")))
