#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write header
    out.write((b"GIF89a\x00"))
    
    width = int.from_bytes(rng.read(2),"big")%256
    height = int.from_bytes(rng.read(2),"big")%256

    print("width",width)
    print("height",height)

    gctflag = b'\x07'

    ctsize = b'\xf7'


    palette = []
    for i in range(0,256):
        r=int.from_bytes(rng.read(1),'big') & 0xff
        g=int.from_bytes(rng.read(1),'big') & 0xff
        b=int.from_bytes(rng.read(1),'big') & 0xff
        print(f"\t{i}: {r},{g},{b}")

        palette += [r]
        palette += [g]
        palette += [b]

    out.write(
        width
      .to_bytes(2,'big'))
    out.write(
        height
      .to_bytes(2,'big'))

    out.write(gctflag)

    out.write(ctsize)

    for p in palette:
        out.write(p.to_bytes(1,'big'))

    bgindex = int.from_bytes(rng.read(1),'big')&0xff

    out.write(bgindex.to_bytes(1,'big'))

    pixelaspectratio = b'\x00'

    out.write(pixelaspectratio)

    lctflag = b'\x01'

    out.write(lctflag)

    out.write(ctsize)

    for p in palette:
        out.write(p.to_bytes(1,'big'))

    out.write(bgindex.to_bytes(1,'big'))

    pixelaspectratio = b'\x00'

    out.write(pixelaspectratio)

    out.write(int
          .from_bytes(rng.read(1),'big')%256
          .to_bytes(1,'big'))

    out.write(b'\x00')

    out.write(int
          .from_bytes(rng.read(1),'big')%256
          .to_bytes(1,'big'))

    out.write(b'\x00')

    out.write(int
          .from_bytes(rng.read(1),'big')%256
          .to_bytes(1,'big'))

    out.write(b'\x00')

    out.write(int
          .from_bytes(rng.read(1),'big') % 256
          .to_bytes(1,'big'))

    out.write(b'\x00')

    # Write background color index
    out.write(
        bgindex
      .to_bytes(1,'big'))

    # Write pixel aspect ratio
    out.write(
        pixelaspectratio
        )

    # Write graphics control extension
    out.write(b'\x21')
    out.write(b'\xf9')

    # Anim delay 0-64k seconds * 10 ms per second, 0 means forever
    animdelay = int.from_bytes(rng.read(1),"big")%64
    if   animdelay==64 : 
        animdelay=(65535*10)+1
    elif animdelay >= 32:
        animdelay=(animdelay*10)-1 

    print("animdelay",animdelay)

    out.write(((animdelay // 256) + 1).to_bytes(1,"big"))
    out.write((((animdelay %% 256) + 1)).to_bytes(1,"big"))

    # Transparent color flag
    out.write(b'\x01')

    # Transparent color index
    transcolidx = (int.from_bytes(rng.read(1),"big")%256) | 0b10000000
    while transcolidx == bgindex or transcolidx == 0 or transcolidx > len(palette):
        transcolidx = (int.from_bytes(rng.read(1),"big")%256) | 0b10000000
    
    print("transcolidx",transcolidx)

    out.write(transcolidx.to_bytes(1,"big"))

    # Block terminator
    out.write(b'\x00')

    # LZW minimum code size
    out.write(b'\x3c')

    # Write image data
    for x in range(0,width*(height//2)):
        out.write(
                    ((int.from_bytes(
                    rng.read(1), byteorder="big") % 256) + 1).to_bytes(1,"big"))

    # Now write the loop variable
    # -1 is 0xff
    #  0 is 0x00
    #  1 is 0x01
    # etc
    loopvar = (
            (int.from_bytes(
                rng.read(1), byteorder="big") % 5)
            | 0b10000000)
    print("loopvar",loopvar)
    if loopvar == 0xff:
        badval = 0
    else:
        badval = loopvar + 1

    print