#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    resolution = 704 # px
    width = int((resolution ** 0.5) / 8 + 0.99999)

    palette = [
        0x00, 0x00, 0x00, # Black
        0xFF, 0x00, 0xFF, # Pink
        0x00, 0xFF, 0xFF, # Cyan
        0xFF, 0xFF, 0x00, # Yellow
        0xFFFF, 0x00, 0x00, # Red
        0xFF, 0xF0, 0xF0, # Fuschia
        0x80, 0x80, 0x80, # Gray
        0x00, 0xFF, 0x00, # Lime
    ] * 16
    palette += [(color >> i & 1) << ((i+1)%3) for color in
                 random_palette(len(palette))]

    global_color_table_size = len(palette) - 1
    code_size = min(global_color_table_size.bit_length() // 2 + 1, 8)
    height = (width >> 1) + 2

    # Write header
    out.write(header +
               width.to_bytes(2, 'little') +
               height.to_bytes(2, 'little'))
    out.write(b'\xff\x0b' + b'NETSCAPE'+ b'2.' +
              bytes([code_size]) + struct.pack('>H', 1))
    out.write(struct.pack('<I', 300))

    # Write logical screen descriptor and global color table
    out.write(global_color_table_size.to_bytes(1,'big') +
             (b'\0'*3).ljust(3*global_color_table_size))
    out.write(((1<<7)+0).to_bytes(1,'big'))

    # Write image data
    out.write(b'\x2c' +
               (width//8).to_bytes(2, 'little') +
               (height//8).to_bytes(2, 'little') +
               b'\0'*4)
    out.write(b'\x87')
    for _ in range(height):
        out.write(bytes([(rng.read(1)[0] >> bit) & 1
                         for bit in reversed(range(8))]))

        if len(out.getbuffer()) > 32*1024**2:
            # write to disk after reaching RAM limit
            out.flush()

    # Write trailer
    out.write(b'\x3B')


# Generate an array of RGB colors from a RNG stream
def random_palette(count):
  return list(zip(*(next(rng)
                    for _ in range(3))
                   for _ in range(int(count))))


if __name__ == '__main__':
     with open("/dev/urandom", mode='rb') as rng,\
          open("test.gif", mode='wb') as out:
        generate_random_gif(rng, out)
else:
   import sys

   def main():
       with open(sys.argv[1], "rb") as rng,\
            open(sys.argv[2], "wb") as out:
           generate_random_gif(rng, out)

   try:
      main()
   except BaseException as e:
      print(e.__class__.__name__)
      raise



def generate_random_gif(rng, out):
    out.write(b"GIF89a")

    # Logical Screen Descriptor
    out.write(struct.pack("<HHBBB",
                         704, # Width
                         704 >> 1, # Height
                         0x87, # Global Color Table Present,
                          7, # Bits per pixel
                         0)) # Background color index
    out.write(b"\x00"*3) # No color table

    # Image Descriptor
    out.write(b";;Image Descriptor\n!\xf9\x04\x01\x00," +
             struct.pack(">H", 704 >> 3) +
             b"\x00\x3b")
    out.write(b"\x00" + b"\x00"*3) # Local color table

    # Image Data
    out.write(b";;Image Data\n!\xfe\x00\x20")

    for line in range(704>>3-1,-1,-1):
        out.write(bytes(
            map(lambda x :
                (((x&1)<<1)|(~x&2)>>1)&0xff,
                (rng.read(1)[0] >> n) & 1
                for n in range(0, 8, 1))),
            )

    out.write(b";;EOF\r\n")
    out.close()


def randbyte():
    while True:
        yield ord(os.urandom(1))
randbyte=randbyte()
for _ in range(27):
    print("%02X"%(ord(os.urandom(1))+2*(random.randint(0,(704*704)>>1)-3)))
exit()

def generate_random_gif(rng, out):
    out.write(b"GIF89a")
    out.write(struct.pack("<HHBBB",
                         704, # Width
                         704 >> 1, # Height
                         0x87, # Global Color Table Present,
                          7, # Bits per pixel
                         0)) # Background color index
    out.write(b"\x00"*3) # No color table
    out.write(b";;Image Descriptor\n!\xf9\x04\x01\x00," +
             struct.pack(">H", 704 >> 3) +
             b"\x00\x3b")
    out.write(b"\x00" + b"\x00"*3) # Local color table
    out.write(b";;Image Data\n!\xfe\x00\x20")
    for _ in range(704>>3-1):
        out.write(bytes(((((randbyte()&1)<<1)|(~randbyte()&2)>>1)&0xff)
                for _ in range(8)))
    out.write(b";;EOF\r\n")
    out.close()

def generate_random_gif(rng, out):
    out.write(b"GIF89a")
    out.write(struct.pack("<HHBBB",
                         704, # Width
                         704 >> 1, # Height
                         0x87, # Global Color Table Present,
                          7, # Bits per pixel
                         0)) # Background color index
    out.write(b"\x00"*3) # No color table
    out.write(b";;Image Descriptor\n!\xf9\x04\x01\x00," +
             struct.pack(">H", 704 >> 3) +
             b"\x00\x3b")
    out.write(b"\x00" + b"\x00"*3) # Local color table
    out.write(b";;Image Data\n!\xfe\x00\x20")
    for _ in range(704 >> 3 - 1):
        out.write(bytes(((((randbyte()&1)<<1)|(~randbyte()&2)>>1)&0xff)\
                     for _ in range(8)))
    out.write(b";;EOF\r\n")
    out.close()


def generate_random_gif(rng, out):
    global_colors=[[0,0,0],[255, 0, 255]]
    out.write(b"GIF89a")
    out.write(struct.pack("<HHBBB",
                         704, # Width
                         704 >> 1, # Height
                         0x87, # Global Color Table Present,
                          7, # Bits per pixel
                         0)) # Background color index
    pallete=[]
    for pnum in range(256):
        if pnum < len(global_colors):
            rgb=global_colors[pnum]
        else:
            rgb=(list(map(lambda x:(randbyte()%256),[0]*3)))
        pallete+=rgb
    pallete=tuple(pallete)

    out.write(b"".join([struct.pack("<B", c) for c in pallete]))
    out.write(b"\x00"*3) # No color table
    out.write(b";;Image Descriptor\n!\xf9\x04\x01\x00," +
             struct.pack(">H", 704 >> 3) +
             b"\x00\x3b")
    out.write(b"\x00" + b"\x00"*3) # Local color table
    out.write(b";;Image Data\n!\xfe\x00\x20")
    for _ in range                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
