#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header: bytes = b'GIF89a\x00\x00\x00\x00\x00\xff\xff\xff\xff'
    out.write(header)

    while True:
        # Get next field byte from RNG
        fieldbyte: int = ord(rng.read(1))

        # Read next two bytes as little endian short for length of block
        sizebytes: bytes = rng.read(2)
        size: int = (ord(sizebytes[0])<<8)+ord(sizebytes[1])
        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # Handle case where there's no image data
        if not hasimgdata:
            continue

        imgdata = rng.read(size)
        out.write(imgdata)