#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Generate some data for the body
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    # Pad the data to 16 bytes of length and write it to the output stream
    paddinglen = 16 - len(data) % 16
    if paddinglen == 0:
        padding = b''
    else:
        padding = b'\x00' * paddinglen
    out.write(struct.pack('!H', datalen + len(padding)))
    out.write(data + padding)

if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rng,\
         open('C02E764B-6F5A-4C82-AED8-107EF38422D3.gif', 'wb') as out:

        generate_random_gif(rng, out)