#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int=512, height: int=512, colordepth: int=4
    ) -> None:
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Write the logical screen descriptor block to the output stream
    out.write(struct.pack('<HHBB', width, height, colordepth * 7, 0))

    # Define how many frames we'll need in total before writing the termination code
    numframes = 8

    while True:

        # Generate some data for the frame
        datalen = int.from_bytes(
                rng.read(2), 'big')
        data = rng.read(datalen)

        # Pad the data to 16 bytes of length and write it to the output stream
        paddinglen = 16 - len(data) % 16
        if paddinglen == 0:
            padding = b''
        else:
            padding = rng.read(paddinglen)
        out.write(data + padding)

        numframes -= 1
        if numframes <= 0: break

    # Append one more RGB image that is filled with black pixels
    frame = bytearray()
    
    for _ in range(width // 2):
        frame += b'\x00\x00\xff'

    framesize = len(frame)

    for i in reversed(range(height)):
        out.write(frame[:framesize][i*width:(i+1)*width])


    # Terminate the GIF by appending the GIF trailer to the output stream
    out.write(b';')