#!/usr/bin/env python3
import os
import sys
import struct
from typing import BinaryIO
from pathlib import Path

def chunks(b: bytes, size=8*1024):
    '''Yield successive n-sized chunks from l.'''
    for i in range(0, len(b), size):
        yield b[i:i+size]
    
# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    #print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  

    out.write(header + (datalen).to_bytes(
            3, 'big')) 

    # write frame data

    while True: 
        blocktype = int.from_bytes(
                rng.read(1), 
                byteorder='little')
        
        #print("block type",blocktype)        
        
        if blocktype == 0x2C: 
            subblocklen = int.from_bytes(
                    rng.read(1), 
                    byteorder='little')
    
            #print('\tsubblock length',subblocklen)            
            
            out.write((blocktype).to_bytes(
                    1, 'little')) 
            
            out.write((subblocklen).to_bytes(
                    1, 'little')) 
        
            for _ in range(5):

                pixelbyte = int.from_bytes(
                        rng.read(1), 
                        byteorder='little')
        
                #print("\tpixel byte",pixelbyte)
            
                out.write(pixelbyte.to_bytes(
                        1, 'little')) 
            
        elif blocktype!= 0x21: # end image marker

            break

        else: 

            subblocklen = int.from_bytes(
                    rng.read(1), 
                    byteorder='little')
        
            #print('\tsubblock length',subblocklen)                        

            out.write((blocktype).to_bytes(
                    1, 'little')) 
                        
            out.write((subblocklen).to_bytes(
                    1, 'little')) 
