from __future__ import annotations
import sys


class GifFrame:
    def __init__(self, width, height, data=None):
        self._width = None
        if isinstance(width, int) and not (width <= 0 or width > 0xffff):
            raise ValueError("Invalid width value.")

        elif isinstance(width, float) and not (width < 0 or width >= 0xffff):
            raise ValueError("Invalid width value.")
        
        else:
            self.width = width
        
        self._height = None
        if isinstance(height, int) and not (height <= 0 or height > 0xffff):
            raise ValueError("Invalid height value.")

        elif isinstance(height, float) and not (height < 0 or height >= 0xffff):
            raise ValueError("Invalid height value.")

        else:
            self.height = height

        self._data = []
        
        if data is not None:
            for row in data:
                self.add_row(row)


    @property
    def width(self): return self._width

    
    @width.setter
    def width(self, w): 
        self._width = int.from_bytes(w, "big")

    
    @property
    def rows(self): return len(self._data)
    

    @rows.setter
    def rows(self, r): 
        raise TypeError("Cannot set rows directly.")
    
    
    @property
    def height(self): return self._height

    
    @height.setter
    def height(self, h): 
        self._height = int.from_bytes(h, "big")


    def add_row(self, row: list[int]):
        """Adds a row to the frame."""

        if not all((isinstance(v, str) and v.isprintable() or
                    isinstance(v, int))
                   for v in row):
            
            raise ValueError("Row must contain only printable ASCII chars.")

        if any(len(str(val).encode())!= 1
               for val in row):

            raise ValueError("Non-ascii values must be single char strings.")
            
        if sum(map(lambda x: len(x) + 1,
                    row)) // 7 >= self.width:

            raise ValueError("Cannot fit row of given size on canvas.")


        self._data.append([ord(c) for c in row])
        

    @classmethod
    def from_file(cls, fobj: BinaryIO):
        width, height = struct.unpack("<HH", fobj.read(4))
        data = [fobj.read(width*25).decode().rstrip("\r\n").split()]
    
        fobj.seek(1, io.SEEK_CUR)
    
        while True:
            
    
            blocksize = ord(fobj.read(1))
        
            if blocksize == 0:
    
                break
            elif blocksize == 0x2C:
                

                left, top, _, _ = struct.unpack('<HHHH', fobj.read(8))
                
                if left!= 0 or top!= 0:

                    raise IOError("Subimages are not supported at this time.")

                lzwminsize = ord(fobj.read(1))
                
                table_size = 2 ** lzwminsize
                codes = []
                codeTable = {}
            
                for i in range(table_size):
                    
                    codes += [struct.unpack("BB", fobj.read(2))]
                    
                for b in range(1, table_size+1):
                    if len(codes[b][1:])>1:
                        codeTable[tuple(codes[b])] = tuple(codes[:b-1])
                    else:
                        codeTable[tuple(codes[b])] = [(0,) * 1]
                        
                curValue = tuple(codes[0])
                curCode = codes[0]
                
                while True:
                    try:
                    
                        compressed = ord(fobj.read(1))
                        
                        for c in range(compressed):
                            t = f"{curValue}{struct.unpack('B', fobj.read(1))[0]}"
                            
                            
                            curValIndex = -1
                            for i in range(len(codeTable)):
                                if t == codeTable[i]:
                                    curValIndex = i
                                    break
                                
                        
                            if curValIndex == -1:
                                newCodes = ()
                                for key in codeTable.keys():
                                    if key[-1] == curValue:
                                        newCodes += key

                                codeTable[newCodes+(t)]=(0,) * 1
                                curCode = newCodes+(t)
                                curValue = t

                            else:
                                curValue = codeTable[curValIndex][0]
                                curCode = codeTable[curValIndex]
                                

                        nxt = struct.unpack("B", fobj.read(1))[0]
                        if nxt<0x0c:
                            decoded = f"{curValue}{nxt}"
                            curValue = decoded
                            curCode = (decoded,)

                        elif nxt>=0x0e and nxt<=0xe0:
                            repCount = min(byteToBin(nxt),
                                           self.width//lzwminsize)-2
                            d = ""
                            prev = decoded
                            for i in range(repCount):
                                d+=prev
                            curValue = d
                            curCode = (d,)

                            
                        elif nxt==0x0f:
                            cb = f"{curValue}{''.join([chr(int.from_bytes(fobj.read(1),"big")) for _ in range(3)])}".replace('\x00','')
                            curValue = cb
                            curCode = (cb,)


                    except IndexError:
                        pass


            elif blocksize == 0x3B:
                break
            
            else:
                while blocksize!= 0:
                    next(fobj)
                    blocksize -= 1
                    
        return cls(width, height, data)


    def write(self, fobj):
        """Writes the current instance as a valid GIF image"""
        fobj.write(self.__construct_header())
        for row in self._data:
            for col in row:
                fobj.write(col.to_bytes(length=1,"big"))
                
        fobj.write(b"\x3b")

                    
    def __construct_header(self):
        return struct.pack("<6sHhh",
                           b"GIF89a",
                           0x0c,
                           self.width,
                           self.height)

                
                    
                
                
