#!/usr/bin/env python3

import os
import sys


# Generates a random GIF file into `out` using the
# random number generator `/dev/urandom`.
def generate_random_gif():
    try:
        with open('/dev/urandom', 'rb') as rng,\
             open('out.gif', 'wb+') as out:
            generate_random_gif(rng=rng, out=out)
    except PermissionError:
        print("""\
ERROR: could not access /dev/urandom for reading;
you must be running as root to run this program.\n""",
              file=sys.stderr)
        raise SystemExit from None


if __name__ == '__main__':
    # If ran directly, and we are root, then create our
    # output file in the current directory...
    if (os.geteuid()!= 0):
        print("""\
ERROR: you must be root to run this program!\n""",
              file=sys.stderr)
        raise SystemExit

    generate_random_gif()


    # Or dump it to stdout?
    """
    elif '-d' in sys.argv:
        with open('/dev/urandom', 'rb') as rng:
            generate_random_gif(rng=rng, out=sys.stdout.buffer)
    else:
        print("usage: {} [-d]\n".format(*sys.argv))
        raise SystemExit
    """