#!/usr/bin/env python3

import struct
from typing import BinaryIO


def generate_color_map():
    color_map = []
    for i in range(0,768):
        color_map += [(i%256)]*3
    return bytes(color_map)

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    global color_map
    header = b'GIF89a'
    image_header = b'\x2c\x00\x00'
    palette_header = b'\x00'
    data_sublen = b'\xf9' + \
                 (struct.pack(">H", 40)) + \
                 b'\x00'
    table_size = struct.pack("<B", len(color_map)//3)
    

    # Header and Global Color Map Table
    out.write(header+image_header+palette_header+data_sublen+table_size)
    out.write(generate_color_map())


    # Frame header
    out.write(b'\x21\xF9\x04\x00\x0f\x00\x00')
    # Image dimensions
    width = [int.from_bytes(rng.read(2),'big')]
    height = [int.from_bytes(rng.read(2),'big')]
    out.write(b'\x00'+memoryview(width)[-1:])
    out.write(b'\x00'+memoryview(height)[-1:])

    # Write image local color map info
    out.write(b'\x2C\x00\x00\x00')
    out.write(generate_color_map())

    #
    out.write(b'\x87\x00\x00')

    # Read the length of the pixel data
    datalen = memoryview(
            int(((int.from_bytes(
                rng.read(3), byteorder='big')) >> 8)
               & 0xfff))[1:]
    
    # Write pixel data as 1 byte per line
    for i in range(0,*datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                  256)).to_bytes(1, "big"))