#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom or a user-specified path).
def generate_random_gif(
        rng=None, out=None, size=(500, 500), colordepth=8
    ):
    # Get an open read handle on /dev/urandom if none was specified
    if not rng:
        try:
            rng = open('/dev/urandom', mode='rb')
        except FileNotFoundError as e:
            print("You need to specify a valid RNG path")
            raise e

    # Get an open write handle on stdout if no handle was specified
    if not out:
        out = sys.stdout.buffer

    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Generate input image data and pad it so that its length is a multiple of blocksize
    imgdatalen = sum(x**2 for x in size)**0.5 * (colordepth // 8)
    imgdata = bytearray()
    while len(imgdata) < imgdatalen:
        n = min((blocksize := 4096), imgdatalen - len(imgdata))

        imgdata += rng.read(n)

    while True:
        hassizeext = False
        while len(imgdata)!= 0:

            if len(imgdata) >= 8:
                # Use this frame
                width, height = [int.from_bytes(imgdata[:2], 'little')] * 2

                # Parse the rest of the header fields
                flags = struct.unpack('B', imgdata[2])[0]
                lctsize = struct.unpack('B', imgdata[3])[0] & 0xf
                gctsize = pow(2, struct.unpack('B', imgdata[3])[0] >> 4)

                # Move forward past the header block
                imgdata = imgdata[4:]
                break
            elif len(imgdata) > blocksize // 4:
                # Use this frame
                width = 1 << (struct.unpack('B', imgdata[-1:])[0] & 7)
                height = 1 << ((struct.unpack('B', imgdata[-1:])[0] >> 3) |
                                ((struct.unpack('B', imgdata[-2:])[0] & 1) << 1))
                
                # Parse the rest of the header fields
                flags = struct.unpack('B', imgdata[0])[0]
                lctsize = struct.unpack('B', imgdata[1])[0] & 0xf
                gctsize = pow(2, struct.unpack('B', imgdata[1])[0] >> 4)

                # Move forward past the header block
                imgdata = imgdata[2:]
                break

        # Determine the remainder of the data based on the logical screen descriptor
        if flags & 8:
            imgdatalen -= 1
        if flags & 4:
            imgdatalen -= 1<<lctsize
        if flags & 2:
            imgdatalen -= 3*(gctsize+1)//4
        if flags & 1:
            pass
            
            
        if hassizeext:
            exttype = struct.unpack('<H', imgdata)[0]
            extlen = struct.unpack('<H', imgdata[2:4])[0]
        
        lzwmincodesize = None
        while imgdatalen!= 0:
            


    return