#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Generate some data for the body
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = []
    while len(data) < datalen:
        # The binary format has one byte indicating how many more bytes there are
        # in this block. It is followed by those bytes, except last. If no more
        # than 254 bytes are remaining at the end of a block we need another
        # block. Otherwise we use all remaining bytes as our own payload.

        rem = datalen - len(data)
        if rem > 254:
            # This is not the final block, so read two bytes which indicate how
            # much additional data is present in the following blocks
            rem = int.from_bytes(
                    rng.read(2), 'big')
        elif rem >= 254:
            # Since we're here, that means we have exactly only one byte left
            # So we use its value directly as our repetition
            rem = int.from_bytes(
                    rng.read(1), 'big')

        # Use the above information to add the required amount of random data to
        # our payload. We don't care about duplicates since they will be filtered
        # later anyway.
        new_payload = rng.read(rem)
        data += [x for x in new_payload if x not in data]

    # Write everything to an output stream with the correct length
    out.write(struct.pack('!H', len(data)))
    out.write(bytearray([x for x in set(data)]))