#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='big') & 0xfff 
    height = int.from_bytes(
            rng.read(2), byteorder='big') & 0xfff 

    # For simplicity's sake, require that images be at least 3 pixels wide and tall
    while width <= 3:
      width += 1

    while height <= 3:
      height += 1

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width > 4095 or height > 4095:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    third_byte = ((datalen % 2**8) >> 11) & 0xFF

    fourth_byte = ((datalen % 2**16) >> 19) & 0xFF

    # Write pre-frame bytes  

    out.write(header +
              width.to_bytes(2, 'little') + 
              height.to_bytes(2, 'little') + 
              bytes([first_byte]) +
              bytes([second_byte]) +
              bytes([third_byte]) +
              bytes([fourth_byte]))


    # write frame data

    for i in range((datalen//2)):

        start = 0
        end = 16

        val = int.from_bytes(
            rng.read(end - start), 
            byteorder="little", signed=False)%256

        out.write((((val+val)&(255))+(val^0xff)).to_bytes(
            1, "little"))