#!/usr/bin/env python3

import argparse
import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, size=None
    ):
    header = b'GIF89a' + \
             [b'\x00'] * 7 + \
             [(size >>  24) & 0xff].to_bytes(1, byteorder='big') + \
             [(size >>  16) & 0xff].to_bytes(1, byteorder='big') + \
             [(size >>   8) & 0xff].to_bytes(1, byteorder='big') + \
             [(size      ) & 0xff].to_bytes(1, byteorder='big')
   
    frame_cnt = int(os.getenv("FRAME_COUNT", "1"))
    
    datalen = int.from_bytes([frame_cnt], byteorder='big') & 0xf
    
    
    # Frame header
    out.write(header)
    frame_data=[b'\x00'*datalen]
    
    
    
    for i in range(0,frame_cnt-1):
        fdata=[]
        for d in frame_data[-datalen:]:
            fdata+=d+[b'\xff'][::((i+1)%2)]
        frame_data.append(fdata)
            
    frame_data += [b'\xff']*(size - len(frame_data))
    assert sum(map(len, frame_data)) == size
    print(frame_data)
    
    for fd in frame_data:
        for d in fd:
            out.write(d)
        
    
        
        
    
