#!/usr/bin/env python3

import struct
from typing import BinaryIO


def _get_disposal_method():
    return (i.to_bytes(length=1,byteorder='big'))


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).  The first argument is the
# width of each image and height of the images are determined by the ratio
# between these values.
def generate_random_gif(in_width: int,
                        out: BinaryIO,
                        rng: BinaryIO
                        ) -> None:
    header = b'GIF89a'
    w = int.from_bytes(rng.read(2), byteorder='big')
    h = int.from_bytes(rng.read(2), byteorder='big')
    wh = int((w << 8) + h)
    print("wh", wh)
    databytes = [b'\x00'] * wh
    disposarray = []
    for j in range(wh):
        databytes[j] = (int.from_bytes(rng.read(1), byteorder='big') % 256).to_bytes(
            1, 'big')
        if len(disposarray) < wh - 1:
            disposarray.append(_get_disposal_method())
    bpp = ord(databytes[0]) >> 4
    background = (ord(databytes[0]) & 0xf) | ((ord(databytes[-1])) << 4)
    nimageblocks = (len(databytes)-1)//bpp
    print("w:", w)
    print("h:", h)
    print("nimageblocks:", nimageblocks)
    print("ncolors:", 2**(bpp+1))
    print("background:", background)
    print('\n'.join([str(['0x%X'%(ord(d)) for d in databytes[start:(start+bpp)]]) + ':'+ str(disposarray[start//bpp]) for start in range(0,len(databytes[:-1]),bpp)]))

    # Write pre-frame bytes
    out.write(header)                    # Magic Number
    out.write(b'\x00'*7)                 # Header data to be filled in later
    out.write(((w >> 8)&0xff).to_bytes(1,"big"))   # Width
    out.write(((w&0xFF)).to_bytes(1,"big"))        # Width
    out.write(((h >> 8)&0xff).to_bytes(1,"big"))   # Height
    out.write(((h&0xFF)).to_bytes(1,"big"))        # Height
    out.write((bpp).to_bytes(1,"big"))            # Bits per pixel
    out.write((0).to_bytes(1,"big"))             # Global Color Table Flag
    out.write((0).to_bytes(1,"big"))             # Background color index
    out.write((0).to_bytes(1,"big"))             # Pixel aspect ratio
