#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file with size `size` into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, size: int, out: BinaryIO
    ):
    # Write the magic string and size header to the output stream
    out.write(b'GIF89a')
    out.write((size & 0xFF).to_bytes(1, "big"))
    out.write((size >> 8 & 0xFF).to_bytes(1, "big"))

    # Generate some data for the body
    data = bytearray()
    while True:
        nxtbyte = rng.read(1)[0]

        # If the first byte is zero, we're done!
        if nxtbyte == 0x00:
            break
        
        data.append(nxtbyte)
    
    # Pad the data to 16 bytes of length and write it to the output stream
    paddinglen = 16 - len(data) % 16
    if paddinglen == 0:
        padding = b''
    else:
        padding = rng.read(paddinglen)

    data += padding
    assert len(data) == size

    out.write(bytes(data))
    
