#!/usr/bin/env python3

import os
import sys
import argparse

from typing import TextIO


FILE_FORMATS = ['svg', 'pdf']


# The length of each block in the output file (in bytes)
BLOCK_LENGTH = {
   'svg':    512 * 1024,   # 512 kB
    'pdf':    int(os.environ['MAX_PDF_SIZE']),
}



def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('outfile',
                        nargs='?', type=argparse.FileType('wb'),
                        default=sys.stdout,
                         help='the desired output filename')
    parser.add_argument('-f', '--format', choices=FILE_FORMATS,
                        required=True,
                        help='the format used for the generated content')

    args = parser.parse_args()

    with args.outfile as outfile:

        while True:

            chunk = os.urandom(BLOCK_LENGTH[args.format])
            
            if not chunk: break
            
            outfile.write(chunk)


if __name__ == '__main__':
    main()