#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = (int.from_bytes(
            rng.read(3), byteorder='big')) & 0xfff;

    print('Writing header...')
    print('header',header) 
    print('datalen',datalen) 
    
    # Write pre-frame bytes  
    out.write(header)
    for _ in range(datalen):
        out.write(((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
     
    # Read past this frame
    while True:
        blocktype = int.from_bytes(
                rng.read(1), byteorder="big")

        if blocktype == 0x2C:
            break
        
        elif blocktype == 0x21:
            blocklen = int.from_bytes(
                    rng.read(1), byteorder="big");

            print("Skipping block type 0x%x of length 0x%x" % 
                  (blocktype, blocklen));
            
            
            for i in range(blocklen - 1):
                rng.read(1);
                
        else:
            raise ValueError("Unknown block type 0x%x." %
                             blocktype)