#!/usr/bin/env python3

import os
from typing import BinaryIO


# Generates a random GIF file with size `size` into `out`
# using the given PRNG and with the seed `seed`.
def generate_random_gif(size: int, seed: bytes,
                        rng: BinaryIO, out: BinaryIO):
    assert len(seed) == 4
    assert isinstance(rng, BinaryIO)
    assert isinstance(out, BinaryIO)
    assert size > 0

    # Seed rng
    rng.seek(os.SEEK_SET)
    rng.write(seed)

    # Header
    out.write(b"GIF89a")
    # Logical screen descriptor
    # Size of logical screen in pixels.
    w = h = size
    out.write(w.to_bytes(2, 'little'))
    out.write(h.to_bytes(2, 'little'))
    out.write(b"\x87\x00")      # Global color map flag
    out.write(b"\x00")         # Background color index
    out.write(b"\x00")         # Pixel aspect ratio numerator
    out.write(b"\x00")         # Pixel aspect ratio denominator

    # Frame control definition
    out.write(b"\x2c")     # extension type
    out.write(b"\x00\x00")     # image left position
    out.write(b"\x00\x00")     # image top position
    out.write(w.to_bytes(2, 'little'))    # image width
    out.write(h.to_bytes(2, 'little'))    # image height
    out.write(b"\x00")             # local flags
    # Application extension block
    out.write(b"\xff\xe1")     # application id
    out.write(b"\x0bNETSCAPE2.0\x03\x01")   # app data subblock
    out.write(b"\x08\x00")                         # netscape looping value
    # Frame data
    for _ in range(0, 2 * size**2 + 2):
        out.write(((int.from_bytes(
                     rng.read(1), byteorder="big") %
                     256)).to_bytes(1, "big"))