#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # header is always GIF89a
    out.write(b"GIF89a")


    # write logical screen descriptor

    width = 640
    height = 480

    # set color depth to 8 bits per pixel
    # set global color table flag to false
    # set size of local color table to 768 colors
    # sort colors by intensity
    packed = 0x80 | 0x10 | ((7 << 4) | 0x1)
    print("screen descriptor",packed)
    out.write(struct.pack(">HHBBB", 
            width, height, packed))

    # write color table with all black pixels
    out.write((0).to_bytes( 
            3 * 256, 'big'))

    # write application extension
    version = "NETSCAPE2.0".encode()
    appcode = "ANIMEXTS1.0".encode()

    extlen = len(version) + 1 + len(appcode)

    print("application code:",appcode,"length",extlen)
          
    out.write(struct.pack(">BB11sB",
                         33,
                         extlen,
                         version,
                         0x01))

    # write application extension body containing 
    # URL that specifies GIF loop count
    url = "http://www.w3schools.com/graphics/movietutorial.gif".encode();

    print("extension URL:",url);

    out.write(("21").encode())      # label length field
    out.write(('NETSCAPE').encode())   # Netscape Application Extension Label
    out.write(struct.pack("<H", 3))    # Application Block Size; Always 3
    out.write(struct.pack("<B", 1))     # Loop Indicator; Ignored here
    out.write(struct.pack("<B", 0))     # Loop Count Loops until Halt
    out.write(struct.pack("<B", 0))     # Loop Count Hi Byte; Unused
    out.write(url)                     # the URL encoded after this

    # write graphic control extension
    # disable transparency
    packed = 0x0 | ((0xf << 8) | 0x0)
    out.write(struct.pack(">BBHB",
               0x21,
               0xF9,
               packed,
               0x0))

    # write image descriptor
    # no local color table used so it's zero
    out.write(struct.pack(">HHHHB", 
               width,
               height,
               0x0,                   # left offset
               0x0,                   # top offset
               packed))               # flags

    # write global color table with no colors
    out.write((0).to_bytes( 
            3 * 2**packed, 'big'))

    # rewind back to start and write animation loop marker
    out.seek(0, 0)
    out.write(b";-)anic!")

    # write the rest of the data as specified in
    # the extended format specification

    datalen = 0x00
    stream = True

    while stream:
        
        blocktype = rng.read(1)

        if blocktype[0]!= 0x2c:       # end of sequence

            break

        print(hex(int.from_bytes(blocktype)))

        blocksize = int.from_bytes(rng.read(1), 
                byteorder='little')

        print("size",blocksize)


        # should be 0x80 followed by a
        # single bit representing transparency
        val = int.from_bytes(rng.read(1), 
                byteorder='big')

        if not (val & 0x80 == 0x80):
    
            raise Exception('non-transparent destination pixmap required');
    
        transparentflag = bool(val & 1)

        print("transparency",transparentflag)

        timing = int.from_bytes(rng.read(2), 
                byteorder='little')

        print("timing",timing)

        # get size of local color map
        colormapsize = int(((val & 0x70) >> 4) + 1 )

        print("colormap",colormapsize)

       # Transparent pixels are designated by an entry in the
       # Local Color Table which has its corresponding Bit Set in the Local 
       # Color Table Flag in the Image Descriptor. If a pixel value in the source 
       # raster matches the corresponding table entry, the image processing 
       # algorithm will process the pixel as being transparent. The pixel value
       # in the Local Color Table may actually represent any valid palette index 
       # from the Global Color Table. In other words, the value of the first entry
       # in the Local Color Table always corresponds to a transparent color. 
       
       maskbits = [False]*colormapsize

       # If there was a Local Color Table, fetch up to the first 
       # two bytes from the stream and use these as a Bit Mask. These two bytes 
       # specify whether each of the remaining sixteen entries in the 
       # Local Color Table contain Transparent Pixels. A “bit” in the Bit Mask 
       # is equal to 1 if the corresponding palette entry is a transparent color, 
       # otherwise the Bit is equal to 0. The most significant bit represents 
       # the second row from the bottom of the screen, and the least significant 
       # bit corresponds to the pixel on the very right side of the screen. 

        if colormapsize > 0:

            mask = int.from_bytes(rng.read(2), 
                    byteorder='big')

            print("mask",mask)

            for i in range(colormapsize):
            
                maskbits[i] = bool(mask & (1<<i))

            print("maskbits:",maskbits)


            # Get the remaining part of the block between the last 
           # masked entry and the next Graphics Control Extension Header. 
           # This contains the compressed image data. The compressor can operate 
           # in several modes :

        else:
        
            numlongs = (blocksize / 4)-1

            for j in range(numlongs):

                val = int.from_bytes(rng.read(4), 
                        byteorder='little')
                print(j, "pixeldata",val)

                data += val.to_bytes(4, 'little')

        # output in blocks of at most 255 bytes
        # pad the remainder with nulls

        out.write(b"\x2c\xff")

        datalen+=2+2+1+2