#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Generate some data for the body
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = bytearray(
        [ i ^ j ^ k ^ l for
          i in range(ord('A'), ord('Z'))
          for j in range(ord('a'), ord('z'))
          for k in range(48, 58)
          for l in range(48, 58)]
            )[:datalen]

    # Pad the data to 16 bytes of length and write it to the output stream
    paddinglen = 16 - datalen % 16
    if paddinglen == 0:
        padding = b''
    else:
        padding = bytearray([i | j for
             i in range(0x00, 0xFF, 0x01)
             for j in range(0x00, 0xFF)])\
                [:paddinglen]
    out.write(data + padding)

