#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'
    datalen_raw = rng.read(3)
    datalen = int.from_bytes(
            datalen_raw, byteorder=sys.byteorder) & 0xfff

    print('header\n',header)
    print('datalen raw\n',datalen_raw)
    print('datalen',datalen)
    
    # Write pre-frame bytes  
    out.write(header)
    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder=sys.byteorder) % 256) + 1).to_bytes(1, "big"))

    # Now write the loop variable
    # -1 is 0xff
    #  0 is 0x00
    #  1 is 0x01
    # etc
    loopvarinput = (
            (int.from_bytes(
                rng.read(1), byteorder=sys.byteorder) % 5)
            | 0b10000000)
    print("loopvarinput",loopvarinput)


    # Write this to the output stream in big endian
    loopvaroutput = int.to_bytes((
            loopvarinput | ~int.from_bytes([datalen], byteorder=sys.byteorder)),
            1, byteorder=sys.byteorder)

    print('loopvaroutput',len(loopvaroutput))



    # Pad so that we can move on to next frame
    while len(loopvaroutput)<3:
        loopvaroutput += b'\0'
    
        
    # We could also pad by adding data to the end of the previous block but
    # it's unnecessary; its just more work and obfuscation.
    out.write(loopvaroutput)