import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    # Set the LZW minimum code size
    lzwminsize = max((struct.pack("<I", datalen)), (2 ** 12))

    # Grab the bits from the byte stream

    first_byte = 0b111 << 5 | ((lzwminsize[0] & 0xf0) >> 4)
    second_byte = (((lzwminsize[0] & 0x0f) << 4) |
                   (datalen & 0xff))
    third_byte = (datalen >> 11) & 0xFF

    fourth_byte = (datalen >> 19) & 0xFF

    # Write pre-frame bytes  

    out.write(header +
              width.to_bytes(2, 'little') + 
              height.to_bytes(2, 'little') + 
              bytes([first_byte]) +
              bytes([second_byte]))


    # write frame data

    for i in range((datalen)):

        start = 0
        end = 16

        val = int.from_bytes(
            rng.read(end - start), 
            byteorder="little")%256

        out.write(((val).to_bytes(
            min(end - start, 1), "little")))


