#!/usr/bin/env python3
"""Generate a random GIF file from /dev/urandom."""

import argparse
from os import urandom as get_randomness
from pathlib import Path


def parseargs() -> dict[str, str]:
    """Parse command line arguments passed to script."""
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("output",
                        type=Path,
                        metavar="<OUTPUT>",
                        help="path of output GIF")
    parser.add_argument("-w", "--width",
                        default=400,
                        type=int,
                        help="width of generated image [default: 400]")
    parser.add_argument("-W", "--max-width",
                        default=None,
                        type=int,
                        help="maximum width when generating an image [default: unlimited]")
    parser.add_argument("-H", "--max-height",
                        default=None,
                        type=int,
                        help="maximum height when generating an image [default: unlimited]")
    return vars(parser.parse_args())


def main():
    args = parseargs()
    with open("/dev/urandom", "rb") as fhandle:

        if not args["max_height"]:
            max_height = float("inf")
        else:
            max_height = args["max_height"] * args["width"] // 100

        if not args["max_width"]:
            max_width = float("inf")
        else:
            max_width = args["max_width"]

        while True:

            width = min(get_randomness(2)[0] >> 3, max_width - 1) + 1
            height = min(get_randomness(2)[0] >> 3, max_height - 1) + 1

            header = b"GIF89a"   \
                     + b"\x00\x00"     \
                     + (width).to_bytes(2, 'little')     \
                     + (height).to_bytes(2, 'little')    \
                     + b'\x87\x00\x00' \
                     + b'\xff'         \
                     + b'\x0b'         \
                     + b'\x4e'         \
                     + b'\x45\x54\x53\x43\x41\x50\x45\x32\x2e\x30\x03\x01'     \
                     + b'\x00\x00'     \
                     + b'\x00\x00'     \
                     + b'\x00'         \
                     + b'\x00\x00\x00' \
                     + b'\x00\x00\x00' \
                     + b'\x00\x00\x00\x00'

            if len(header) <= args["width"]*args["height"]:

                with open(args["output"], "wb+") as ohandle:

                    ohandle.write(header)

                    for y in range(height):

                        x = 0

                        while x < width:

                            templen = 1
                            byt = get_randomness(templen)[0]

                            while byt == ord(b'\x00'):

                                templen += 1
                                byt = get_randomness(templen)[0]

                            ohandle.write(byt.to_bytes(
                                    templen, 'little'))

                            x += templen

            else:

                continue


            break



if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        exit("[ERROR] Interrupted by user.")