#!/usr/bin/env python3

import os
import sys
import argparse
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write pre-frame bytes  
    out.write(header)
    out.write((datalen//256).to_bytes(1,byteorder='big'))
    out.write(((datalen%256)+1).to_bytes(1,byteorder='big'))
    

    # Now write the frame 
    for i in range((datalen+7)//8):
        x = 0
        for j in reversed(range(8)):
            x |= (((int.from_bytes(
                rng.read(1), byteorder="big") % 256) < 128))<<j

        out.write((x).to_bytes(1,byteorder='big'))
        
    # Now write the loop variable
    # -1 is 0xff
    #  0 is 0x00
    #  1 is 0x01
    # etc
    loopvar = (
            (int.from_bytes(
                rng.read(1), byteorder="big") % 5)
            | 0b10000000)
    if loopvar == 0xff:
        badval = 0
    else:
        badval = loopvar + 1

    out.write((badval).to_bytes(1,byteorder='big'))