#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0x7fff

    while True:
        out.write(
                header + ((datalen // 256)
                   ).to_bytes(1, "big")
                )

        # Frame header
        out.write(header)
        # Image dimensions
        width = (int.from_bytes(
                rng.read(2), byteorder='big')) & 0xffff
        height = (int.from_bytes(
                rng.read(2), byteorder='big')) & 0xffff
        out.write((width).to_bytes(2, 'big'))
        out.write((height).to_bytes(2, 'big'))

        # write frame data
        out.write(((datalen%256)//4).to_bytes(1, "big"))
        out.write((((datalen+2)%256)//4).to_bytes(1, "big"))
        out.write((((datalen-1)%256)//4).to_bytes(1, "big"))
        out.write((((datalen*2)%256)//4).to_bytes(1, "big"))
        for i in range(0,(datalen)):
            out.write(
                    ((int.from_bytes(
                        rng.read(1), byteorder="big") %
                     256)).to_bytes(1, "big"))

        out.flush()

        if datalen < 1500: break
        else: datalen -= 1500


    return