#! /usr/bin/env python3

import sys
import os
import binascii
import struct


def main() -> None:
    seed = os.getenv('SEED', None)

    if not seed or len(seed) == 0:
        print("Error: no SEED environment variable set",
              file=sys.stderr)
        exit(1)

    # Use hex-encoded bytes
    try:
        decoded = bytearray.fromhex(seed)
    except ValueError as e:
        print("Error decoding SEED: {} ({})".format(e, seed))
        exit(1)

    print("SEED={}".format(seed))
    print("len(decoded)={}, type(decoded)={}"
         .format(len(decoded), type(decoded)))

    with open('/dev/urandom', mode='wb', buffering=-1) as rng:
        for i in range(16):
            randomint = struct.unpack('<L', rng.read(4))[0]

            randombytes = b'\x00\x00\x00\x00'
            randombytes[i % 4] = (randomint & 0xff).to_bytes(1,
                byteorder='little')[0]

            decoded += randombytes

        filename = "data/output/{}.gif".format(binascii.b2a_hex(decoded[:4]))
        print("Writing output to {}".format(filename))
        with open(filename, mode="wb") as f:
            generate_random_gif(f, decoded)


def generate_random_gif(
        out: BinaryIO, decoded: bytes
    ) -> None:
    print("{}".format(type(decoded)), file=sys.stderr)
    print("{}".format(len(decoded)), file=sys.stderr)
    header = b'GIF89a'
    datalen = len(decoded)
    data = decoded
    footer = b';'
    out.write(header +
           datalen.to_bytes(2, 'big') +
           data +
           footer)