#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='big') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='big') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    third_byte = (datalen >> 11) & 0xFF

    fourth_byte = (datalen >> 19) & 0xFF

    # Write pre-frame bytes  

    out.write(header +
              width.to_bytes(2, 'big') + 
              height.to_bytes(2, 'big') + 
              bytes([first_byte]) +
              bytes([second_byte]))


    # write frame data

    for i in range((datalen)):

        start = 0
        end = 16

        val = int.from_bytes(
            rng.read(end - start), 
            byteorder="little") % 256

        out.write(((val).to_bytes(
                min(end - start, 1), "little")))