#!/usr/bin/env python3

import os
import sys
import argparse
import struct
from typing import BinaryIO


HEADER = 'GIF89a'.encode()
MIN_DATALEN = 4
MAX_DATALEN = 0xfff+1 - MIN_DATALEN

def main():
    parser = argparse.ArgumentParser("Generate a random gif") 
    parser._action_groups.pop()
    required = parser.add_argument_group('required arguments')
    optional = parser.add_argument_group('optional arguments')

    required.add_argument("-o","--output", 
            type=argparse.FileType('wb'), required =True)

    optional.add_argument("--minlen", default=MIN_DATALEN, 
            help="Minimum size to write to output (default="+str(MIN_DATALEN)+")")

    optional.add_argument("--maxlen", default=MAX_DATALEN, 
            help="Maximum size to write to output (default="+str(MAX_DATALEN)+")")
    
    args = parser.parse_args()

    out = args.output 

    if not MIN_DATALEN <= args.minlen < MAX_DATALEN:
        raise ValueError("Invalid minimum length "+str(args.minlen))
    minlen = args.minlen

    if not MIN_DATALEN <= args.maxlen < MAX_DATALEN:
        raise ValueError("Invalid maximum length "+str(args.maxlen))
    maxlen = args.maxlen

    with open("/dev/urandom",'rb') as rng: 
        # Write the header and some data
        datalen = rng.randint(minlen, maxlen)

        out.write(HEADER)
        for _ in range(datalen):
            out.write(((rng.read(1)[0]%256)).to_bytes(1,"little"))

        # Read past this frame
        
        while True:
            blocktype = rng.read(1)[0]

            # Application extension
            if blocktype == 0x21: 
                pass

            # Image descriptor
            elif blocktype == 0x2c: 

                