#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Generate an integer between 50 and 200 inclusive as the
    # size of the screen descriptor block
    width = int.from_bytes(
            rng.read(2), 'big')
    height = int.from_bytes(
            rng.read(2), 'big')
    bgcolor = int.from_bytes(
            rng.read(1), 'big')
    pixel_aspect_ratio = int.from_bytes(
            rng.read(1), 'big')
    
    # Make sure that both the width and height are even numbers
    while ((width & 1)!= 0):
        width += 1
    while ((height & 1)!= 0):
        height += 1

    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))
    out.write(bgcolor.to_bytes(1, 'big'))
    out.write(pixel_aspect_ratio.to_bytes(1, 'big'))

    # Generate some data for the body
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    # Pad the data to 16 bytes of length and write it to the output stream
    paddinglen = 16 - len(data) % 16
    if paddinglen == 0:
        padding = b''
    else:
        padding = rng.read(paddinglen)
    out.write(data + padding)