#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  

    out.write(header + (datalen).to_bytes(
            3, 'big')) 

    # write frame data

    while True:

        # get the next block header
        
        blockheader = int.from_bytes(
                rng.read(2), 
                byteorder='little')
    
        # break from loop at EOF

        if blockheader == 0: 
            return
        
        # TODO: check that it's not an invalid block type?

        # determine how many longs to read and then write them as such
        # Assumes maximum of one byte per long.
            
        numlongs = ((blockheader >> 8) + 1) // 2
        
        for l in range(numlongs):
        
            pixeldata = int.from_bytes(
                    rng.read(4), 
                    byteorder='little')
        
            out.write(pixeldata.to_bytes(
                    4, 'little'))
