#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng`. This should be an
# open stream that supports.read().
#
# The seed parameter can be used to set the state of
# the PRNG, so that multiple calls with the same value
# will produce identical outputs. If None or omitted,
# uses current time as the seed.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, *,
        seed: bytes = None
    ) -> None:
    # Seed the RNG if needed. We use this instead of just calling
    #.seed() on the RNG object because we need reproducible results.
    if seed:
        rng.seek(0)
        rng.write(seed)

    # Header ("GIF89a")
    out.write(b"GIF89a")

    # Logical Screen Descriptor
    out.write(_generate_logical_screen_descriptor())

    out.write(_generate_global_color_table())
    _generate_blocks(rng, out)


def _generate_logical_screen_descriptor():
    # Build SC field
    sc = [
        0x00,           # Screen width
        0x00,           # Screen height
        0xf7            # Packed field
    ]

    # Calculate packed field
    packedField = (sc[2] << 7 |
                   sc[1] << 3 |
                   0x07)

    # Pack up fields
    return struct.pack("!HHB",
                       sc[0],        # Screen width
                       sc[1],        # Screen height
                       packedField   # Packed field
                      )



def _generate_global_color_table():
    # TODO
    pass


def _generate_blocks(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # TODO
    pass