#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file of length `len` into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, len: int, out: BinaryIO
    ):
    header = b'GIF89a'

    if (len < 2):
        raise ValueError("Image too small")

    # Generate the image data
    data = bytearray()
    while (len > 0):

        chunksize = min((len % 16) or 16, len)

        colorcode = rng.randint(33, 254)
        chunk = bytes([colorcode]) * chunksize

        len -= chunksize
        data += chunk

    colorcount = len(data) // 16

    # Write the image to disk as an uncompressed GIF
    out.write(header +
              len(data).to_bytes(2, 'big') +
              b'\x00\xff' +               # Logical screen width and height
              bytes([colorcount]))       # Color table size

    for i in range(0, len(data), 16):
        row = data[i : min(i+16, len(data))]
        out.write(b''.join(row))


    out.write(b';')

