#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    def get_frame_data():
        return [int.from_bytes(
                rng.read(1), 
                byteorder="little")%256
              for _ in range(datalen)]

    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff

    # We want to make sure that we don't accidentally create an image too big!
    while True:
       if width < 1 or height < 1:
           continue

       break

    print('width', width,
          'height', height)

    datalen = width * height // 4

    first_byte = 0b111 << 5 | 0b11111 << 2
    second_byte = 0b111 << 5 | 0b11111 << 2

    third_byte = (datalen >> 11) & 0xFF

    fourth_byte = (datalen >> 19) & 0xFF

    # Write pre-frame bytes  

    out.write(header +
              width.to_bytes(2, 'little') + 
              height.to_bytes(2, 'little') + 
              bytes([first_byte]) +
              bytes([second_byte]) +
              bytes([third_byte]) +
              bytes([fourth_byte]))


    for i in range(20):
        frame_data = get_frame_data()
    
        block = b'\x2C' + \
               chr(0).encode("utf-8") +\
               chr(0).encode("utf-8") +\
               chr(0).encode("utf-8") +\
               chr(0).encode("utf-8") +\
               chr(0).encode("utf-8") +\
               ((0).to_bytes(1,"big")) +\
               ((0).to_bytes(1,"big")) +\
               ((0).to_bytes(1,"big")) +\
               ((0x80).to_bytes(1,"big")) +\
               ((datalen).to_bytes(1,"big")) +\
               ((0).to_bytes(1,"big")) +\
               ((0).to_bytes(1,"big")) 

        out.write(block);


        for j in range(datalen):

            start = 0
            end = 16

            val = frame_data[j]

            out.write(((val).to_bytes(
                min(end - start, 1), "little")))

