#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    global_color_table = True
    
    if global_color_table == False:
        color_table_size = [7] * 4 + [11] * 4 + [15] * 3 + [23] * 1 + [31] * 1 + [7] * 1

        bits = rng.read(1)[0]
        color_table_size[bits // 9 - 1] -= 1
        
        background_index = rng.read(1)[0]
        pixel_aspect_ratio = rng.read(1)[0]
    
    idlen = rng.read(1)[0] & 0x0f;
    screen_descriptor = bytes([idlen])
    if idlen > 0:
        screen_descriptor += bytes([(int.from_bytes(rng.read(1),'little'))])
        screen_descriptor += bytes([(int.from_bytes(rng.read(1),'little'))]) | bytes([(int.from_bytes(rng.read(1),'little'))]) << 8
        screen_descriptor += bytes([(int.from_bytes(rng.read(1),'little'))]) | bytes([(int.from_bytes(rng.read(1),'little'))]) << 8
        flags = bytes([(int.from_bytes(rng.read(1),'little'))])
        flags |= bytes([(int.from_bytes(rng.read(1),'little'))]) << 8
        flags &= bytes([0b11011011])
        
        bg_color_index = rng.read(1)[0]
        while len(screen_descriptor) < 10:
            screen_descriptor += bytes([0x00])

        color_table_size_byte = bytes()
        for i in reversed(range(4)):
            color_table_size_byte += bytes([color_table_size[i]])

        screen_descriptor += bytes(color_table_size_byte)
        screen_descriptor += bytes([background_index])
        screen_descriptor += bytes([pixel_aspect_ratio])

    # Frame info
    imgdata = 9
    imgdata |= (global_color_table and 0x80 or 0x0) << 7
    imgdata |= (idlen and 0x80 or 0x0) << 6
    imgdata |= (imgdata and 0x40 or 0x0) << 5

    print('header',hex(struct.unpack("B", header)), 'global clr table:',
          hex(struct.unpack("B", bytes([imgdata]))))

    # Frame header
    out.write(header)
    # Image descriptor 
    out.write(bytes([imgdata]))
    out.write(screen_descriptor)

    # Write image data
    lzwmincodesize = int.from_bytes(rng.read(1),"little") & 0xff
    out.write(lzwmincodesize.to_bytes(1,"little"))
    codesize = 1 << lzwmincodesize

    qm = dict();
    mq = dict();
    values = list(range(256));
    codesizes = []
    buf = ""
    index = 0
    maxbuf = 0;

    def emitchar():
        nonlocal buf
        val = None
        if len(buf) > 0:
            val = ord(buf[0]);
            del buf[:1];

        return val;

    def readchar():
        nonlocal index
        c = bytes([values[index]])
        index += 1
        if index >= len(values):
            index = 0
        return c;

    def codeclear():
        nonlocal codesize, codesizes, values, qm, mq
        codesize = 1 << lzwmincodesize
        codesizes = []
        values = list(range(256));
        qm = dict()
        mq = dict()

    freenode = "";
    prevnode = ""

    while (True):
        node = freenode!= "" and freenode or (prevnode!= "" and prevnode.decode()) or None
        while not node:
            for i in range(maxbuf):
                buf += bytes([emitchar()])

            for i in range(codesize+1):
                buf += readchar();
            
            buf = str(buf)
            if buf == "":
                break;

            v = None
            try:
                v = qm[buf[-(codesize+1)]]
            except KeyError:    
                pass
                
            if v is not None:
                codesizes += [v+1]
                buf = buf[:-codesizes[-1]];
            else:
                newval = int(buf[-1:])
                values += [newval]
                qm[str(buf)] = newval
                codesizes += [1]
                oldbuf = buf
                buf = buf[:-1];


    print(qm);
    print(mq);
    print("len:", len(qm)+len(mq), "max:", maxbuf);
    print("Code sizes:", codesizes)

    print(values)


