#!/usr/bin/env python3

import os
import sys
import argparse

# Generates a random GIF file that has the given size in bytes. The
# outputted GIF is written to `out`, and `/dev/urandom` is used as the
# source of entropy.
def generate_random_gif(size: int, out: BinaryIO):
    if size > (2**15*7): raise ValueError("Too huge")

    # Generate the binary string representing this image
    img = bytearray()
    img += b"GIF89a" # Header
    for i in range(4): # Logical screen descriptor
        img += os.urandom(1)
    img += size.to_bytes(2, "big") # Image dimension
    img += b"\x00\x00" # Palette
    img += b"\x00"      # Background color index
    img += b"\x00"      # Pixel aspect ratio

    # Write it all to disk
    out.write(img)
    while len(img) % 2**(int(sys.argv[3])+1)!= 0:
        out.write(os.urandom(1))

