#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    halfwidth = [
            [0x06], 
            [0x7A, 0xE5, 0x4F, 0xFC, 
            0xB2, 0x2D, 0xAE, 0x3B, 
             0xF7, 0x9E, 0xCF, 0xCE],
            [0xD9, 0xB3, 0xA7, 0xDF, 
            0xFE, 0xCA, 0xFF, 0xEF, 
             0xFB, 0x63, 0xEF, 0x1C] ]

    fullwidth = [[i] for i in list(range(0xff)) * 8]
    
    background = [int.from_bytes(
                rng.read(4), 
                byteorder='little') % 0xffffff for j in range(8)]

    # create palette

    palette = []

    for entry in background:
        
        r = ((entry >> 16) << 3).to_bytes(
                1, 'big') 

        g = (((entry & 0xff00) >> 8) << 2).to_bytes(
                1, 'big') 

        b = ((entry & 0xff) << 3).to_bytes(
                1, 'big') 

        palette += [b'\x00']

        palette += [r]

        palette += [g]

        palette += [b]
        
    out.write(palette*3)

    out.write(struct.pack('<ccBBBHBHBHHHHHH', 
            0x21, 0xf9, 0x04, 0x00, 0x0c, 
            0xfc, 0x00, 0x00, 0x00, 0x00, 
            0x01, 0x00, 0x01, 0x00))


    # Make each color cube (square)

    for n in range(3):
    
        out.write(struct.pack('<BBBB', 
                0x21, 0xff, 0x0b, 0x4e))

        size = len(fullwidth[n]) + len(halfwidth[n])        
        
        out.write((size//2).to_bytes(
                2, 'little'))
        
        width = fullwidth[n][::1] + \
               [max(1,x//2) for x in reversed(halfwidth[n])]
        
        out.write(b','.join([str(x).encode() 
                   for x in width]))

        out.write(b';')

        size = 8
        out.write(((size<<4) | 4).to_bytes(
                1, byteorder='big'))

        out.write(((size<<4) | 4).to_bytes(
                1, byteorder='big'))



    # End image

    out.write(struct.pack('<BB', 
            0x3b, 0xfe ))
    