#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'

    flags = [
            *[0]*7, 
            0xf0,
            0,
            0,
            0,
            ]

    print(flags)

    width = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff + 1
    height = int.from_bytes(
            rng.read(2), byteorder='little') & 0x7fff + 1

    max_wh = max(width, height)
    bits = len(str(max_wh))
    size = ((bits + 1)*9)//8*256

    # Make sure we can store pixels of this dimension
    while True:
       if width > size or height > size:
           continue

       break

    print('width', width,
          'height', height)

    out.write(header +
              width.to_bytes(2, 'little') + 
              height.to_bytes(2, 'little'))

    to_write = 3

    for flag in flags:
        to_write += 1
        out.write(struct.pack("<B", flag))

    palette=[]

    for i in range(2**bits):
        r = int.from_bytes(rng.read(1), byteorder="") % 256 
        g = int.from_bytes(rng.read(1), byteorder="") % 256 
        b = int.from_bytes(rng.read(1), byteorder="") % 256 

        to_write += 1
        out.write(r.to_bytes(1,"little"))
        to_write += 1
        out.write(g.to_bytes(1,"little"))
        to_write += 1
        out.write(b.to_bytes(1,"little"))

        palette.append((r,g,b,))

    total = sum((len(i) for i in palette))

    num_entries = 2**(bits+1)
    interlace = False

    globalcolortable=False

    sorted_palette = sorted(
            palette, key = lambda x : (
                x[0], 
                x[1],
                x[2]
            ))

    ncolor = []
    
    for p in sorted_palette:
        
        ncolor.extend(p)

    
    print("total", total)
    print("ncolor", ncolor)

    for entry in ncolor:
        to_write+=1
        out.write(entry.to_bytes(1, "little"))
    

