#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted animated
# GIF which also happens to contain a valid image and the 2048-byte
# limit does not apply here, only the one on the size of each frame.
def generate_gif(out:BinaryIO, rng):
    # A single byte
    out.write(b'G')
    
    # A string (here, this is an empty string)
    out.write(struct.pack('B', 0x0))
    
    # The frames
    num_frames = 16 + rng.randint(-7, 0)
    for i in range(num_frames):
        # The screen descriptor bytes
        width = 320 if rng.randint(0, 1) == 1 else 256
        height = 200 if rng.randint(0, 1) == 1 else 192
        
        local_color_table_flag = rng.randint(0, 1)
        interlaced_flag = rng.randint(0, 1)
        sort_flag = rng.randint(0, 1)
        size_of_local_color_table = -1
        if local_color_table_flag == 1:
            size_of_local_color_table = rng.randint(1, 8)
            
        out.write(struct.pack('BBBBBB', width, height,
                              ((size_of_local_color_table << 4) | (sort_flag << 3) \
                               | (interlaced_flag << 2) | local_color_table_flag),
                              0, 0))
        
        # The color table (unused but required)
        for j in range((1 << (3 * size_of_local_color_table + 1)) // 3):
            out.write(b'\xFF\xFF\xFF')
            
        # The graphic control extension block
        out.write(struct.pack('!BHBB', 0xF9, 4, 0x0, 0)) # 0 because we don't use transparency (yet?)
        
        # The IMAGE DESCRIPTOR
        out.write(struct.pack('!BHHBB', 0x2C, 0, 0, width, height))
        
        # The LZW minimum code size byte
        out.write(struct.pack('B', rng.randint(0, 9)))
        
        # The block
        block_length = rng.randint(10, 1<<16)
        assert block_length < 2**18
        out.