#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(2),
        'little',
        signed=True
    ) % 1024 + 1
    data = rng.read(datalen)
    footer = b'\x3b'
    out.write(header)
    out.write(struct.pack('<H', 6)) # width
    out.write(struct.pack('<H', 6)) # height
    out.write(b'\xff\xff')         # transparent color
    out.write(b'\x08')             # sort flag
    out.write(data)