#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write the header
    out.write(header)
    print(f"Datalen: {datalen}")

    # Write the screen descriptor (width, height)
    width = int.from_bytes(rng.read(2), 'big') % 300 + 1
    height = int.from_bytes(rng.read(2), 'big') % 300 + 1
    out.write(struct.pack('<HHHB', width, height, 0, 0))
    print("Width: {}, Height: {}".format(width, height))

    print("Loop count: 0")
    out.write(struct.pack('BB', 0, 0))

    # Fill with color table
    for i in range(768):
        if i == 256:
            break
        out.write(b'\x00\x00')

    return True

