#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 24 10:46:32 2018

@author: tobiaswilde
"""
import sys
sys.path.append("../scripts")
from utils import load_obj, save_obj, make_dirs
import numpy as np
import matplotlib.pyplot as plt
plt.style.use('ggplot')
from scipy.optimize import minimize
from pprint import pprint
import timeit
import pickle
import pandas as pd
from sklearn.svm import SVC
from sklearn.model_selection import GridSearchCV
from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler

start = timeit.default_timer()
## =============================================================================
## Define helper functions
def do_PCA(X_train, X_test):
    # Reduce dimensionlity of data
    scaler = StandardScaler().fit(X_train)
    X_train = scaler.transform(X_train)
    X_test = scaler.transform(X_test)
    
    pca = PCA(n_components=1)
    pca.fit(X_train)
    X_train = pca.transform(X_train)
    X_test = pca.transform(X_test)
    return X_train, X_test

def visualise_boundary(x, y, model, ax=None, title=''):
    h = .02  # step size in the mesh
    xx, yy = np.meshgrid(np.arange(0,1 + h, h), np.arange(0,1 + h, h))
    Z = model.predict(np.c_[xx.ravel(), yy.ravel()])
    Z = Z.reshape(xx.shape)
    
    if not ax:
        fig, ax = plt.subplots()
        
    plt.contourf(xx, yy, Z, alpha=0.3, cmap=plt.cm.coolwarm)
    plt.scatter(x[:, 1], x[:, 0], s=50, c=y, marker='o', edgecolors='k', 
                label=['Class 1', 'Class 2'])
    plt.xlabel('Sepal length')
    plt.ylabel('Sepal width')
    ax.set_title(title)
    ax.legend(loc='upper right')
    return ax


def plot_frontier(X, Y, model, t=.5):
    margin = .1
    h = .02  # step size in the mesh
    x_min, x_max = X[:, 0].min() - margin, X[:, 0].max() + margin
    y_min, y_max = X[:, 1].min() - margin, X[:, 1].max() + margin
    xx, yy = np.meshgrid(np.arange(x_min, x_max, h),
                         np.arange(y_min, y_max, h))
    Z = model.predict(np.c_[xx.ravel(), yy.ravel()])

    # Put the result into a color plot
    Z = Z.reshape(xx.shape)
    plt.figure(figsize=(10,10))
    plt.pcolormesh(xx, yy, Z)
    
    # Plot also the training points
    plt.scatter(X[:, 0], X[:, 1], c=Y, cmap=plt.cm.coolwarm,
            edgecolor='k', s=50)
    plt.xlabel('sepal length')
    plt.ylabel('petal length')
    plt.axis([x_min, x_max, y_min, y_max])
    plt.show()

def plot_decision_regions(X, y, classifier, resolution=0.02):
    colors = ("red", "blue", "lightgreen", "gray", "cyan")
    cmap = ListedColormap(colors[:len(np.unique(y))])

    # Plot the decision boundary. For that, we will assign a color to each
    # point in the mesh [x_min, m_x_max]x[y_min, y_max].
    x_min, x_max = X[:, 0].min() - 1, X[:, 0].max() + 1
    y_min, y_max = X[:, 1].min() - 1, X[:, 1].max() + 1
    xx, yy = np.meshgrid(np.arange(x_min, x_max, resolution),
                         np.arange(y_min, y_max, resolution))
    Z = classifier.predict(np.c_[xx.ravel(), yy.ravel()])

    # Put the result into a color plot
    Z = Z.reshape(xx.shape)
    plt.pcolormesh(xx, yy, Z, cmap=cmap)
    
    # Plot also the training points
    plt.scatter(X[:, 0], X[:, 1], c=y, cmap=cmap,
                edgecolor='k', s=50)
    plt.xlim(xx.min(), xx.max())
    plt.ylim(yy.min(), yy.max())
    
    plt.show()
    
def plot_surface(X, y, model, name):
    from mpl_toolkits import mplot3d
    h = .02  # step size in the mesh
    xx, yy = np.meshgrid(np.arange(-7, 7 + h, h), np.arange(-7, 7 + h, h))
    Z = model.predict(np.c_[xx.ravel(), yy.ravel()])

    # Put the result into a color plot
    Z = Z.reshape(xx.shape)
    fig = plt.figure(figsize=(10, 10))
    ax = fig.add_subplot(111, projection='3d')
    surf = ax.plot_surface(XX, YY, Z, rstride=1, cstride=1, cmap=plt.cm.coolwarm, linewidth=0, antialiased=False)
    ax.view_init(elev=-60., azim=30.)
    ax.set_zlim(-1.01, 1.01)
    ax.set_xlabel('Dimension 1')
    ax.set_ylabel('Dimension 2')
    ax.zaxis.set_major_formatter(plt.NullFormatter())
    plt.savefig(name, format='pdf', dpi=740)
    plt.close()


def get_hyperparamter_for_C(X_train, y_train):
    """
    Given an array of hyperparameters for C and a corresponding dataset
    fit a grid search cv and return the best C
    """
    Cs = range(0, 200, 10) #[0, 1, 10, 50, 100, 1000, 10000, 50000]
    param_grid = {'C': Cs}
    svc = SVC(kernel='linear')
    gs = GridSearchCV(svc, scoring="accuracy", param_grid=param_grid)
    gs.fit(X_train, y_train)
    print("Best params are:")
    print(gs.best_params_)
    return gs.best_params_["C"]
    
# =============================================================================
## Load data
make_dirs("../../models")
path = "./data/"
features = ["sepal length", "sepal width", "petal length", "petal width"]
df = pd.read_csv(path + "iris.data", header=None, names=features)
df.columns = features+["target"]

# Change target values so that they are {0, 1}
df["target"].replace({'Iris-virginica': 1, 'Iris-versicolor': 0}, inplace=True)

# Split data into training and test set
X = df[["sepal length", "sepal width", "pet