#!/usr/bin/env python3

import struct
from typing import BinaryIO

_HEADER = b'PT1F02'
_VERSIONS = [0, 1]
_CHUNKS = ['HEAD', 'BODY']


def check_header(stream: BinaryIO) -> bool:
    """Checks if a stream has the proper header"""
    # read first 8 bytes and compare them to the expected header
    return (struct.unpack('<8s', stream.read(8))[0] == _HEADER)


def get_version(stream: BinaryIO) -> int:
    """Gets the version of an ssp file by reading its first 4 bytes"""
    # seek back to beginning of file
    stream.seek(0)
    # read in the first 4 bytes and unpack them as an unsigned int in little endian order
    version = struct.unpack('<I', stream.read(4))[0]
    assert version <= max(_VERSIONS), f'`{version}` is not a valid version'
    # return value for use
    return version


def find_chunk(name: str, stream: BinaryIO):
