#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the dimensions specified in `data`.
def generate_gif(out: BinaryIO, data: bytes):
    # Header.
    out.write(b'GIF89a')   # Magic header.
    width, height = struct.unpack('<HH', data[:4])  # Image dimensions.
    out.write(data[4:10].lstrip())     # Background color and aspect ratio.

    # Logical Screen Descriptor.
    out.write(struct.pack('<HHB', width, height, 0))  # Width/height/flags.
    out.write(data[10:27])                            # Global palette.
    out.write(b'\x3b\x21\xfe\x04\x05\x00\x2c\x00\x00\x00\x00\x2c\x01\x00\x01\x00\x00\x02\x02D\x01\x00;')

    # Application Extension.
    out.write(data[27:69])                             # Title.
    out.write((b'\x21\xf9\x04\x04\x01\x00\x01' + b'\xFF'*30).rjust(33, b'\x00'))  # Loop count.

    # Graphics Control Extension (needed to separate frames).
    out.write(b'\x21\xf9\x04\x04\x00\x01\x00\x01')      # Block size, flags, delay, transparent index.

    # Pixel data.
    length = int.from_bytes(data[69], 'big')           # Length of pixel data.
    for i, byte in enumerate(data[70:]):               # Write each byte from pixel data as-is.
        if i == length:
            break
        
        out.write(byte.to_bytes(1, 'little'))
