#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Reads a BMP file and produces an uncompressed RGB8 .bmp file
# with no colormap or palette.
def main() -> None:
  inp = open('input.bmp', 'rb')
  out = open('output.bmp', 'wb')

  try:
    if inp.read(2).decode('ascii') != 'BM':
      raise Exception("Error: Not a valid BMP file")
    
    header_size, width, height, planes, bitsperpixel, compress, size, hres, vres, numcolors, importantcolors = struct.unpack("<IIiihhllll", inp.read(40))
  
    if (bitsperpixel != 8):
      raise Exception("Error: Only works for 8-bit images")
      
    # Skip over the rest of the bitmap header
    inp.seek(header_size - 40 + 14)

    # Create a new output file that can be written to by making changes
    # to the memory buffer
    with open('output.bmp', 'r+b') as out:
      # Copy the file signature so we have everything up until this point
      # correct
      out.write(inp.read(header_size - 40 + 14))

      # Set the new bitmap headers
      out.seek(14)
      struct.pack('<IIII', size, width, height, 0)
      out.seek(54)
      struct.pack('<iIHHI', 0, 16, 3, 1, 0)

      # Make sure we skip past the original colortable if it exists
      inp.seek(header_size - 40 + 14 + size)

      # Loop through each scanline of the image
      for y in range(height):
        # The image file is stored upside down, so we write from
        # bottom to top so everything ends up right side up
        out.seek(-(y * ((width*3 + 3)/4)*4), 2)
        
        # Find the start location in the input file for the current line
        linestart = header_size - 40 + 14 + (y*(width+(width & 0x03)))
        inp.seek(linestart)

        # Read the color table if it exists
        if numcolors > 0:
          colortable = rng.read(numcolors*3)
          out.write(colortable)

        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)