#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write the header
    out.write(header)

    # Write width and height
    wh = rng.read(2) * 2 + b'\xff'
    out.write(wh)

    # Write global color table
    for _ in range(0, 768 // 3):
        rgb = rng.read(2)
        out.write(rgb)
    
    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True

