#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb 16 20:57:41 2019

@author: sungkyun
"""
import numpy as np
import cv2
import matplotlib.pyplot as plt
from scipy import optimize
from pylab import *

def color_hist(img, nbins=64):
    # the histogram of the color channels
    rhist = np.histogram(img[:,:,2], bins=nbins)
    ghist = np.histogram(img[:,:,1], bins=nbins)
    bhist = np.histogram(img[:,:,0], bins=nbins)
    
    # manipulating the shape to plot
    rhist = rhist[0].reshape((nbins, 1))
    ghist = ghist[0].reshape((nbins, 1))
    bhist = bhist[0].reshape((nbins, 1))
    
    # concatenate the histograms into one big color histogram
    hist = np.concatenate((rhist, ghist, bhist), axis=1)
    return hist

def linearRegression(x,y):
    x = np.matrix(x)
    y = np.matrix(y).T
    w = (x.T * x).I * x.T * y
    
    return w

def applyLinearFilter():
    img = cv2.imread('../testImage/ChurchofRome.jpg')
    #img = cv2.resize(img, dsize=(128, 64))
    hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
    
    # get the pixel in each channel
    redPixels = []
    greenPixels = []
    bluePixels = []
    for i in range(0,len(hsv)):
        for j in range(0, len(hsv[i])):
            if hsv[i][j][0] < 35 and hsv[i][j][0] > 5:
                if hsv[i][j][1] > 35 and hsv[i][j][2]>30:
                    redPixels.append([i,j])
                elif hsv[i][j][1] > 20 and hsv[i][j][2]>20:
                    greenPixels.append([i,j])
                else:
                    bluePixels.append([i,j])
            
    red = []
    green = []
    blue = []
    
    for pixels in redPixels:
        red.append(pixels[0]*3)
        red.append(pixels[1])
        
    for pixels in greenPixels:
        green.append(pixels[0]*3)
        green.append(pixels[1])
     
    for pixels in bluePixels:
        blue.append(pixels[0]*3)
        blue.append(pixels[1])
    
    print("Red", red)
    print("Green", green)
    print("Blue", blue)
    
    w = linearRegression(np.array(red), np.array(green))
    v = linearRegression(np.array(blue), np.array(green))
    u = linearRegression(np.array(red), np.array(blue))
    
    Y = np.zeros((img.shape[0],img.shape[1]), dtype='uint8') + 255
    for y in range(0, img.shape[0]):
        X = np.ones((img.shape[1], 2))
        X[:, 1] = range(0, img.shape[1])
        for x in range(0, img.shape[1]):
            a = y / 3
            b = x
            result = u @ a
            c = v @ np.array([result])
            d = w @ np.array([c])
            
            X[x, :] = [a, b]
            Y[y, x] = int(d)
            
    kernel = np.ones((5,5),np.uint8)
    blurred = cv2.morphologyEx(Y, cv2.MORPH_OPEN, kernel)
    edge = cv2.Canny(blurred,100,200)
    plt.figure()
    plt.imshow(edge, cmap="gray")
    plt.title("Edge Map")
    plt.xticks([]), plt.yticks([])
    plt.show()

#applyLinearFilter()

def gammaCorrection(img,gamma):
    gamma_table = [np.power(x/255.0,gamma)*255.0 for x in range(256)]
    gamma_table = np.round(np.array(gamma_table)).astype(int)
    return cv2.LUT(img,gamma_table)
    
def adjustGamma(src,dst):
    src = cv2.GaussianBlur(src,(11,11),0)
    
    limg = cv2.cvtColor(src, cv2.COLOR_BGR2RGB)/255.0
    ldst = cv2.