#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to showcase `rng_iter` and `random_excursions`.
def generate(out: BinaryIO, rng):
    out.write(b'GIF89a') # magic number
    
    # header section
    width = 16 * (2 ** next(rng)) + 1
    height = 16 * (2 ** next(rng)) + 1
    background = next(rng) & 0xff
    aspect = 3 * ((next(rng) % 7) - 3)
    color_resolution = 1 << (next(rng) % 4)
    sort_flag = bool(next(rng) & 1)
    global_color_table_size = 1 << (next(rng) % 6)
    global_color_table_sort_flag = next(rng) % 3 == 0
    global_color_table_background = next(rng) & 0xff
    pixel_aspect = 3 * ((next(rng) % 5) - 1)
    palette_type = next(rng) & 1

    c_width = 16 * (2 ** color_resolution) + 1
    c_height = 16 * (2 ** color_resolution) + 1
    colors = [struct.pack('BBB', *[next(rng) & 0xff for _ in range(3)])]
    while len(colors) < c_width * c_height:
        if next(rng) % 5 != 0 or len(colors) == 0:
            i = next(rng) % len(colors)
            new_rgb = list(colors[i][:3])
            for j in range(3):
                new_rgb[j] = min(max(new_rgb[j] + (next(rng) >> 6) - 1, 0), 255)
            colors.append(tuple(new_rgb))
        else:
            new_colors = []
            for i in random_excursions(rng, len(colors)):
                new_rgb = list(colors[i][:3])
                for j in range(3):
                    new_rgb[j] = min(max(new_rgb[j] + (next(rng) >> 6) - 1, 0), 255)
                new_colors.append(tuple(new_rgb))
            colors += new_colors
            
    assert len(colors) <= c_width * c_height
    colors = sorted(colors[:c_width * c_height], key=lambda x: sum([(a << (1 - i)) for i, a in enumerate(reversed(list(x)))]), reverse=global_color_table_sort_flag)[-c_width * c_height:]
    for i in reversed(range(len(colors[-1]))):
        bytes([sum((a >> i) & 1 for a in colors[-1][:i+1]) % 2]).tofile(out)
        
    for b in struct.pack('<HHHBBB', width, height, global_color_table_size, background, aspect, pixel_aspect, sort_flag):
        bytes([b]).tofile(out)
        
    # image data section
    interlaced = next(rng) % 5 == 0
    for y in range(0, height, 8):
        for x in range(0, width, 8):
            bits = 0
            for by in range(min(8, height - y)):
                for bx in range(min(8, width - x)):
                    colors = [] # ACTUALLY NOW I KNOW HOW TO DO THIS BETTER LOL
                    for dy in range(by * 8, by * 8 + 8, 2):
                        for dx in range(bx * 8, bx * 8 + 8, 2):
                            color = next(rng) % len(colors)
                            pixels = tuple(set(colors.pop(-1).tobytes()[::-1].hex().upper()) ^ {pixels})
                            colors.append(bytes.fromhex(pixels)[::-