#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the specified (width, height) dimensions.
def generate(out: BinaryIO, width: int, height: int):
    # https://en.wikipedia.org/wiki/GIF#Example_of_a_GIF_file

    out.write("GIF87a".encode("ascii"))  # Magic number (also has "GIF89a")
    
    # Header
    out.write(struct.pack("<H", width))
    out.write(struct.pack("<H", height))
    out.write("\x0F\x00" + b"\x04" * 5)  # Color table

    # Graphics Control Extension
    out.write(b'\x21\xF9\x04\x04\x00\x00' + b"\x01" * 3)

    # Image Descriptor
    out.write(b"\x2C\x00\x00\x00\x00\x00")

    # LZW minimum code size; always 8 for GIF (1 byte)
    out.write(b"\x08")

    def write_code(value: int) -> None:
        if value > 0xff:
            raise ValueError(f"Code too big to fit in one byte! {value}")
        
        out.write(value.to_bytes(1, "big"))
    
    write_code(0xff)  # End of stream code
    write_code(0xfe)  # Clear Table code

    for y in range(height):
        for x in range(width):
            r = max(min((y / height) ** 0.5, 1), 0)  # Red
