#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=80, height=15,
        num_frames=20):
    # File header
    # ----------------
    # 0-2: signature ("GIF")
    # 3-5: version ("89a")
    out.write("GIF89a".encode('ascii'))

    # Logical screen descriptor (LSD)
    # --------------------------
    # 6-7: logical width in pixels
    # 8-9: logical height in pixels
    out.write(struct.pack("<H", width))
    out.write(struct.pack("<H", height))
    
    # 10: packed field
    #   bit 0-2: aspect ratio (0 = square; 1 = 16:9; 2 = 4:3; 3 = 3:2)
    #   bit 3-7: reserved for future use
    out.write(b"\x00")

    # Global color table
    # -------
    # These are the colors used by all frames of this GIF file. The
    # first one is transparent, the rest are opaque.

    # black -> transparent
    out.write(b"\xff\x00\xff\x00\xff\x00\xff\x00\xff\x00\xff\x00\xff\x00\xff\x00" \
              b"\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff")
    
    # image block separator (always F9h)
    out.write(b"\xf9")

    # GCT
    # -----------
    # frame 1: red->blue gradient w/ solid white background
    out.write(b"\x00" + bytes([c % 256 for c in range(width)]*height))

    # ------
    # frame 2: blue->red gradient w/ solid white background
    out.write(b"\x00" + bytes([c % 256 for c in range(width, 0, -1)]*height))

    # Looping application extension
    # ----------------
    out.write(b"\x21\xff\x0bNETSCAPE2.0\x03\x01")
    out.write(struct.pack("<I", num_frames))
    out.write(b"\x0