#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='little', signed=False
        )
    gce = [0xF9, 0x04] + [0] * 7 # Transparent
    lsd = []
    for i in range(10):
        blocklen = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            )
        lsd += [blocklen]
        if (blocklen == 0):
            break
        lsd += rng.read(blocklen-1).tobytes()

    palette = rng.read(768).tobytes()
    imagedata = rng.read(struct.unpack('<H', rng.read(2))[0]).tobytes()

    out.write(header + len(lsd).to_bytes(2, 'little') \
        + sum([b for b in lsd], []).tobytes() + palette + imagedata)
