#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 12
    version = b'\x00' * (datalen - len(header))
    out.write((header + version).encode('ascii'))
    data = rng.read(int.from_bytes(rng.read(1), byteorder='big'))
    # We just need an image descriptor with a null-length
    # data and a color map index of 0, so it will be ignored
    idescr = b'\xff\xfe\x00\x00\x00\x02\x00'
    out.write(idescr)
