#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = int.from_bytes(rng.read(2), 'big') + 1
    height = int.from_bytes(rng.read(2), 'big') + 1
    color_table_size = (int.from_bytes(rng.read(1), 'little') & 0x7) * 3 + 4
    # Color table
    for i in range(color_table_size // 3):
        out.write(struct.pack('BBB', *[int.from_bytes(rng.read(1), 'big') for _ in range(3)]))
    # Image descriptor
    image_descriptor = b'\x2c\x00\x00\x00' + struct.pack('>HH', width, height) + b'\x00\x00'
    out.write(image_descriptor)
