#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    while True: 
      # Generate the header for each block
      tag = rng.read(1)[0]
      
      if tag == ord('!'): 
        # Comment extension
        
        # Read the size of the comment
        size = rng.read(1)[0]
        assert size > 0
        
        comment = rng.read(size)
        # Add it to the output
        out.write(struct.pack('B', size))
        out.write(b'!')
        out.write(comment)
        continue
            
      elif tag == ord(','):
        # Image Descriptor
        
        # Read out the fields
        leftpos = int.from_bytes(rng.read(2), 'big')
        toppos = int.from_bytes(rng.read(2), 'big')
        width = int.from_bytes(rng.read(2), 'big')
        height = int.from_bytes(rng.read(2), 'big')
        flags = int.from_bytes(rng.read(1), "little")
        
        # Write them to the output
        out.write(struct.pack(">HHBB", leftpos, topopos, width, height)+chr(flags).encode())
                
        # If there are local color tables, write it
        if (flags & 0x80) != 0:
          
          numbits = ((flags >> 4) + 1) * 3
          numcolors = (2 ** numbits) - 1
          colortable = rng.read(numcolors*3)
          out.write(colortable)

        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)