#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    out.write('GIF87a'.encode()) # signature
    
    width = 1 << (rng.read(1)[0] & 0xF)
    height = 1 << ((rng.read(1)[0] >> 4) & 0xF)

    out.write(struct.pack("<HH", width, height)) # size
    out.write(b'\xF\x0') # background color index and aspect ratio
    from_color = rng.read(1)[0] & 0xFF
    to_color = rng.read(1)[0] & 0xFF
        
    flags = rng.read(1)[0]
    has_global_colors = (flags & 0b1000_0000) == 0b1000_0000
    if has_global_colors:
        global_size = 2 << (flags & 0b11)
        for _ in range(global_size*3):
            out.write(chr(rng.read(1)[0]).encode())
        out.write(b'\x0')

    # TODO: frames
   