#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    def output_bytes(s: bytes) -> None:
        out.write(s + b'\0')

    # The first 6 bytes are a magic "GIF89a" string.
    output_bytes(b'GIF89a')

    # The next two bytes are dimensions. We'll use 1x1 for simplicity.
    output_bytes(struct.pack('<HH', 1, 1))

    # The next byte is a global color table flag. We'll set it to true.
    output_bytes(b'\x80')

    # The next byte is the color table size, which we'll also use as an index.
    n = int.from_bytes(rng.read(1), 'little') % 256

    # Next three bytes are bits per pixel, background color, aspect ratio.
    output_bytes(b'\x07\x00\x00\xF4')

    # Now go through each color in the colormap and write its RGB value.
    for i in range(n+1):
        output_bytes((i<<16) | (i<<8) | i)

    # Finally, we need to introduce the graphic control extension block.
    output_bytes(b'\x21\xf9\x04\xff\xff\xff\x00')

    # And now