#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# provided RNG `rng`. The GIF consists of one frame, and is drawn as
# a randomly-colored 256x256 pixel rect at a random position.
def generate_gif(out: BinaryIO, rng) -> bool:
    out.write(b'GIF89a') # GIF signature
    width = height = 256
    out.write(struct.pack('!H', width)) # Screen size (width)
    out.write(struct.pack('!H', height)) # Screen size (height)
    out.write(struct.pack('B', 0xf7)) # Logical screen descriptor
    out.write(b'\x00\x00\xff\x00\x00') # Global color table
    out.write(b'!\x00\x01;') # Application extension header
    out.write(b'\x47\x49\x46\x38\x39\x61\x00') # Application block: GIF89a
    out.write(b'\x00\x00\x00\x00\x2c\x00\x00\x00') # Graphic Control Extension
    out.write(struct.pack('B', 0x04)) # Block size
    out.write(struct.pack('B', 0x00)) # Flags
    out.write(struct.pack('<H', 0x0000)) # Delay time
    out.write(struct.pack('B', 0x00)) # Transparent color index
    out.write(struct.pack('!H', 0x0000)) # Block terminator
    out.write(struct.pack('BBBB', 0, 0, width - 1, height - 1)) # Frame rectangle
    out.write(struct.pack('B', 0)) # Color table: No local color table
    out.write(b';') # Image Descriptor
    for _ in range(768):
        out.write(struct.pack('B', rng() % 256))
    return True

