#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out:BinaryIO, rng):
    # Magic bytes
    out.write("GIF89a".encode('ascii'))
    
    # Logical Screen Descriptor
    w = struct.unpack("<H", rng.read(2))[0] + 1
    h = struct.unpack("<H", rng.read(2))[0] + 1
    out.write(struct.pack("<BBB", w&0xff, (w>>8)&0xff, ((h&0xf)|((w&f0)>>(4-log2(w))))&0xff))
    gct = rng.read(1)[0] & 1
    colorspace = rng.read(1)[0] & 7
    sortflag = rng.read(1)[0] & 1
    endrec = rng.read(1)[0] & 1
    bgidx = rng.read(1)[0] & 0xff
    aspectratio = rng.read(1)[0] & 0xff
    out.write("\x00\xff\x00")
    
    # Global Color Table
    if colorspace == 0:
      colors = 1 << ((gct<<5)+4+1)
      for i in range(colors):
        rgb = rng.read(3)
        out.write(rgb)
    elif colorspace == 1:
      assert False, "Not yet implemented"
  
    # Plain Text Extension
    blocklen = rng.read(1)[0]+1
    textgridleft = rng.read(1)[0]
    textgridtop = rng.read(1)[0]
    textgridwidth = rng.read(1)[0]
    textgridheight = rng.read(1)[0]
    charcellwidth = rng.read(1)[0]
    charcellheight = rng.read(1)[0]
    fgcolorindex = rng.read(1)[0]
    bgcolorindex = rng.read(1)[0]
    out.write('\x01') # Code for plain text extension
    out.write(str(text))

    while True:
        # Read the frame header
        type = struct.unpack(">B", rng.read(1))[0] | 0x80
        left = struct.unpack("<H", rng.read(2))[0]
        top = struct.unpack("<H", rng.read(2))[0]
        width = struct.unpack("<H", rng.read(2))[0]+1
        height = struct.unpack("<H", rng.read(2))[0]+1
        
        # Write the frame header
        out.write(struct.pack(">BBHHHH", type, left, top, width, height, 0))

        # Write the LZW min code size
        out.write(struct.pack("B", rng.read(1)[0]))
        
        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:
          colortable = rng.read(numcolors*3)
          out.write(colortable)

        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)