#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1
    # This must be padded to a multiple of 4
    if datalen % 4 != 0:
        datalen += (4 - datalen % 4)
    # Pixels per row is always 2^bitdepth; we will use bitdepth=8
    pixel_per_row = 2**8
    color_table_size = 1 << 8
    global_color_table_flags = 1<<7
    reserved = 0
    sort_flag = 0
    bitdepth = 8
    background_index = 0
    pixel_aspect_ratio = 0
    canvas_width = 640
    canvas_height = 480
    local_palette_flags = 1<<7
    interlacing_flag = 0
    terminator = 0x3b
    header_data = struct.pack('<BBHBHHBB',
                              datalen,
                              pixel_per_row,
                              color_table_size,
                              global_color_table_flags,
                              reserved,
                              sort_flag,
                              bitdepth,
                              background_index,
                              pixel_aspect_ratio)
    local_palette_header = struct.pack('<BBB',
                                       local_palette_flags,
                                       interlacing_flag,
                                       terminator)
    frame_header = struct.pack(
            '<HHBBHH',
            0, 0,
            pixel_per_row,
            canvas_height,
            local_palette_flags,
            interlacing_flag,
            terminator)
    out.write(header)
    out.write(header_data)
    out.write