#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    width = 1 + (int.from_bytes(rng.read(2), byteorder='little'))
    height = 1 + (int.from_bytes(rng.read(2), byteorder='little'))
    padding = bytes([0] * ((4 - width % 4) % 4))
    global_palette = []

    while True:
        blocktype = rng.read(1)[0]
        if blocktype == 0x2c:
            # Image Descriptor
            left = int.from_bytes(rng.read(2), byteorder='little')
            top = int.from_bytes(rng.read(2), byteorder='little')
            width = int.from_bytes(rng.read(2), byteorder='little')
            height = int.from_bytes(rng.read(2), byteorder='little')
            flags = rng.read(1)[0]
            colorMapSize = (flags >> 7) & 1
            localColorTable = not colorMapSize
            colorReserved = (flags >> 5) & 3
            sortFlag = (flags >> 4) & 1
            interlaceFlag = (flags >> 3) & 1
            sizeOfLocalPalette = (width // 2) ** 2
            lct = [struct.pack('BBB', rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]) for i in range(sizeOfLocalPalette)]
            if localColorTable:
                global_palette.extend(lct)
            assert len(global_palette) < 256
            out.write(header)
            out.write(struct.pack("!H", width))
            out.write(struct.pack("!H", height))
            out.write(b'\x87\x00\x00')
            buf = byter