#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    header = b'GIF89a' # magic number and version
    width, height = (
        struct.unpack('<HH', rng.read(4))[0],
        struct.unpack('<HH', rng.read(4))[0]
    )
    datasize = width * height
    header += (
        # screen descriptor
        struct.pack('<BBHBBBB', # logical screen size
            width & 0xff, width >> 8,
            height & 0xff, height >> 8,
            0, 0, 0
        ), # color table size is 0 to indicate global color table follows
        # loop count (forever)
        0x00, 0x01,
        # global colour table
        # 16 colours
        b'\xff\x00\x00\x00\x00\xff\x00\x00\x00\x00\xff\x00\x00\xff\xff\x00\x00\xff\xff\xff',
        # image descriptor block
        # left offset of image, top offset of image
        struct.pack('<HHB', 0, 0, 0),
        # width height of image
        struct.pack('<HH', width, height),
        # packed fields for local color table
        # no local color table, use global table instead
        b'\x00'
    )
    
    datalen = (
        rng.read(1)[0] | rng.read(1)[0] << 8
    ) % 1024 + 1
    data = rng.read(datalen)
    out.write(header + data)
