#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int, height: int
):
    # The magic numbers in this file.
    MAGIC = b"GIF89a"
    BACKCOLOR = 0xff
    TRANSPARENT = 0x00
    ENDRECORD = [0x3b] * 128 + [0x00]
    IMAGEBLOCKSIZE = width * height
    PALETTELENGTH = 1 << 8

    # Write the header and the global color table.
    out.write(struct.pack("6sHHBB", MAGIC, width, height, 0xf7, 0))
    palette = bytearray(PALETTELENGTH * 4)
    for i in range(PALETTELENGTH):
        palette[i*4] = (TRANSPARENT if i else BACKCOLOR) & 0xff
        palette[i*4+1] = (i >> 5) & 0xff
        palette[i*4+2] = (i >> 2) & 0xff
        palette[i*4+3] = (i >> 0) & 0xff
    out.write(palette)
    
    # Write the image.
    out.write(struct.pack("<H", 0x80 | CAPTION_WIDTH))
    out.write(bytearray([TRANSPARENT]) * CAPTION_HEIGHT)
    imageblocks = []
    while True:
        labelcount = rng.read(1)[0] % ((IMAGEBLOCKSIZE - len(imageblocks)) // 2 + 1)
        assert labelcount > 0
        labels = rng.read(labelcount)
        
        line = bytearray()
        for _ in range(len(labels)):
            index = rng.read(1)[0] % (len(labels) + 1 - len(line))
            if not index:
                line += labels[-1:]
            elif labels[index-1:index] != lines[-1:-1]:
                line += labels[index-1:index]
                
