#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    width, height = 16, 16
    packed = bytearray([0])

    # The first 8 bits are the width in pixels
    packed[0] |= width & 255
    out.write(bytes(packed))

    # The next 8 bits are the height in pixels
    packed[0] = 0
    packed[0] |= height & 255
    out.write(bytes(packed))

    # Byte 3 has some flags set for us
    packed[0] = 0b00100011

    # Finally, we write the LSD to our file
    out.write(bytes(packed))

    # Write global color table
    colormap = [0xff0000, 0x00ff00, 0x0000ff]
    for i in range(len(colormap)):
        rgb = colormap[i].to_bytes(4, "big")
        out.write(rgb[:-1])

    # Create Graphic Control Extension (GCE) block
    gce = bytearray([0x21, 0xf9, 0x04, 0, 0, 0])
    out.write(bytes(gce))

    # Create Application Extension (APPE) block
    appe = bytearray([
        0x21, 0xff, 0x0b, 0x4e, 0x45, 0x54, 0x53, 0x43,
        0x41, 0x50, 0x45, 0x32, 0x2e, 0x30, 0