#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the provided width and height.
def generate_gif(width, height, out: BinaryIO):
    # Write GIF87a header
    out.write('GIF87a'.encode())  # Magic number (file type)
    out.write(struct.pack('<H', width))  # Logical Screen Width
    out.write(struct.pack('<H', height))  # Logical Screen Height
    out.write(b'\12')                     # Background color index (black)
    out.write(b'\0')                      # Pixel aspect ratio
    
    # Write Netscape Application Block for looping animation
    out.write(b'!\xff\x0bNETSCAPE2.0\x03\x01\0\0,\0\0')
    
    # Create LSD (Logical Screen Descriptor). This contains global attributes of the GIF such as resolution
    out.write(struct.pack('<B', 0xf9))   # Packed fields - 1: global color table flag set, 2-4: color resolution = 8 bits per primary color
    out.write(struct.pack('<B', width))  # Logical screen size
    out.write(struct.pack('<B', height))  # Logical screen size
    
    # Add the global color table to the palette, which contains a grayscale gradient. Colors are packed as RGB in this order.
    for i in range(0, 256):
        r = (i & 0xe0) >> 5       # Most significant bits
        g = (i & 0x1c) << 3       # Bits 3-5
        b = (i & 0x03) << 6       # Least significant bits
        
        out.write([r | g | b])
    
    # Write application extension block for animation control
    out.write(b'\x21\xff\x0b')           # XMP extension
    out.write(b'NETSCAPE2.0')            # Application identifier
    out.write(b'\x03\x01')               # Application block length
    out.write(b'\0')                     # Repeat count
    out.write(b'\0')                     # Terminator
    
    # Write first image descriptor block
    out.write(struct.pack('<B', 0x2c))  # Image Separator
    out.write(struct.pack('<B', 0))     # Left position of image on canvas
    out.write(struct.pack('<B', 0))     # Top position of image on canvas
    out.write(struct.pack('<B', width)) # Width of image
    out.write(struct.pack('<B', height))# Height of image
    out.write(b'\0')                     # No local color map
    out.write(b'\x08')                   # Minimum code size 8 bits per pixel
    
    # Dump one row of the image data at a time so that we don't have to keep it all in memory. This also makes writing a progress bar easy.
    for y in range(0,height):
        for x in range(0,width):
