#!/usr/bin/env python3

import sys, struct, random
try:
   import os.path as path
except ImportError:
   import path

CHUNKSIZE = 16
FILE = "file"
RNG = "rng"

def parse_args():
  args = {}
  i = 0
  file = None

  if len(sys.argv) < 3:
      print("Usage: %s <giffile> [<seed>]" % sys.argv[0])
      exit(-1)
      
  file = sys.argv[i+1]
  if not path.exists(file):
      print("File does not exist")
      exit(-1)

  i += 1
  try:
      seed = int(sys.argv[i+1], 16)
  except ValueError:
      seed = random.randrange(0xffffff)

  args[FILE] = open(file, 'rb')
  args[RNG] = random.Random(seed)

  return args

if __name__ == "__main__":
    args = parse_args()
    f = args[FILE]
    rng = args[RNG]

    outf = open("out.gif", 'wb')
    out = outf.raw.write

    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
          return
    
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:
          colortable = rng.read(numcolors*3)
          out.write(colortable)

        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)