#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write the header
    out.write(header)
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        if (i % 128 == 0):
            print(f"{i} bytes written")
        
        # Get two random bytes to make up one color
        c1 = int.from_bytes(rng.read(1), byteorder='big')
        c2 = int.from_bytes(rng.read(1), byteorder='big')
        # Combine them together with the leftmost bits clearing
        # the top three bits, which corresponds to transparency
        color = ((c1 << 8) | c2 ) & 0x7ff

        flags = rng.read(1)[0]

        if flags & 0x80 != 0:
           # We have an extension block!
           # First we read a byte that says how long our block is
           # This can be either 0-254 (inclusive), or -1
           # The latter means we need to use a marker (0x00) instead
           extlen = rng.read(1)[0]
           if extlen == 0xff:
               # Marker found, so we have to read until we find another
               
               # XXX: This might not work because this is also the first possible value
               extlen = 0

               while True:
                   nextbyte = rng.read(1)[0]
                   if nextbyte == 0: break
                   else: extlen += nextbyte
            
           else:
               # No marker, so it must be a normal value >= 1
               pass

           assert 0 <= extlen <= 254
           print(f"Extension length: {extlen}")

           # Now we need to write the extension type, followed by the actual extension
           exttype = rng.read(1)[0]
           extdata = rng.read(extlen)[0]
           print(f"Extension type: {hex(exttype)}")
           print(f"Extension contents: {repr(extdata)}")
           # Finally we add our extension to the end
           out.write(struct.pack('<B', exttype))
           out.write(bytes([extlen]))
           out.write(extdata)
        
        else:
            out.write(color.to_bytes(2, 'big'))

    print("done")
    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True

