#!/usr/bin/env python3
from collections import defaultdict, deque
import sys

def parse_input(s):
    return [tuple([int(x) for x in line.split()]) for line in s.strip().split('\n')]

def print_input(grid):
    for r in grid:
        print(*r)
    print('---')

def build_graph(data):
    graph = defaultdict(set)
    for a, b in data:
        graph[a].add(b)
        graph[b].add(a)
    return graph

def find_paths(graph, start=0):
    queue = deque([start])
    paths = []
    while True:
        if len(queue) == 0:
            break
        v = queue.popleft()
        paths.append(v)
        for w in graph[v]:
            queue.append(w)
    return paths

def find_roots(graph):
    roots = set()
    leaves = set(graph.keys())
    seen = set()
    while True:
        if len(leaves) == 0:
            break
        leaf = leaves.pop()
        seen.add(leaf)
        if any([l in graph and leaf in graph[l] for l in seen]):
            continue
        roots.add(leaf)
    return list(sorted(set(seen).difference(roots)))
