#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1 # length of data section
    width = int.from_bytes(rng.read(2),
                           byteorder='little', signed=False)
    height = int.from_bytes(rng.read(2),
                            byteorder='little', signed=False)
    flags = int.from_bytes(rng.read(1), 'big')
    backc = rng.read(3)
    if (flags & 0b10000000) != 0:
        pixdata = rng.read(width * height // 8)
    else:
        pixdata = bytes([])
    out.write(header)
    out.write(struct.pack('!HHBB', width, height, flags, 0))
    out.write(backc)
    out.write(pixdata)
    for _i in range(datalen - len(header)-10-len(pixdata)):
        frm_ctrl = rng.read(1)
        if frm_ctrl[0] == 0x2C:
            label = b'!'
        elif frm_ctrl[0] >= 0x3B and frm_ctrl[0] <= 0x4F:
            label = b';'
        else:
            label = b','
        out.write(frm_ctrl+label)
        lsd = (int.from_bytes(rng.read(2),
                              byteorder='little', signed=False),
                int.from_bytes(rng.read(2),
                               byteorder='little', signed=False),
                int.from_bytes(rng.read(1),
                               byteorder='big'))
        bgcol = rng.read(3)
        pixdata = rng.read((lsd[0]*lsd[1]+3)//4*4)
        out.write(struct.pack('!HHB