#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)
        
    width = ord(rng.read(1)) + 1
    height = ord(rng.read(1)) + 1
    background = ord(rng.read(1))
    aspectratio = ord(rng.read(1))
    out.write(struct.pack('!HHBB', width, height, background, aspectratio))

    # Read blocks until end block is reached
    while True:
        # Read field byte
        fieldbyte = ord(rng.read(1))
    
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:
          colortable = rng.read(numcolors*3)
          out.write(colortable)

        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)