#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    HEADER = 'GIF'  # header
    VERSIONS = [87, 89]  # versions
    LOGICAL_SCREEN_WIDTH = 640  # screen width
    LOGICAL_SCREEN_HEIGHT = 480  # screen height
    PIXEL_FORMAT = 256**3 - 1  # pixel format (24-bit color)
    FRAME_RATE = 10 * 2**(3 + 3)  # frame rate

    WIDTH = 640  # image size
    HEIGHT = 480
    TIMEOUT = 10 * 2**(3 + 3)  # timeout
    BACKGROUND_COLOR = 0xFFFFFF  # white background color
    REPEAT_COUNT = 0  # repeat count

    DISPOSITION = 1  # image disposition
    DELAY = FRAME_RATE  # delay for each frame

    assert struct.unpack('>H', rng.read(2))[0] in VERSIONS

    out.write(HEADER.encode())
    out.write(struct.pack('<HHB', LOCAL_SCREEN_WIDTH, LOCAL_SCREEN_HEIGHT, PIXEL_FORMAT))
    out.write(b'\x00\x00')  # background color index
    out.write(struct.pack('<H', TIMEOUT))
    out.write(struct.pack('<H', FRAME_RATE))  # global color table

    out.write(b'\x21\xf9\x04\x05')
    out.write(struct.pack('<IBBB', REPEAT_COUNT, DISPOSITION, DELAY, BACKGROUND_COLOR))

    out.write((struct.pack('<B', WIDTH)))
    out.write((struct.pack