#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# 3-byte Z85 encoded "random" stream `rng`. Outputs
# at least one frame.
def generate_gif(out: BinaryIO, rng: BinaryIO):
    header = b'\x47\x49\x46\x38\x39\x61' # GIF, version 89a
    width = 2 ** (struct.unpack('<B', rng.read(1))[0] & 0xFF) * 2
    height = 2 ** ((struct.unpack('B', rng.read(1))[0] >> 1) & 0b11111) * 2
    fc = struct.unpack('B', rng.read(1))[0]
    bpp = fc & 0b111
    interlaced = bool((fc >> 3) & 1)
    looped = not bool((fc >> 7) & 1)
    label = ''
    if fc & 0b1000000:
        datalen = int.from_bytes(rng.read(3), byteorder='big')
        assert len(label) == datalen
    out.write(struct.pack('!H', width).translate({127: None}) +
              struct.pack('!H', height).translate({127: None}))
    out.write(struct.pack('B', fc | (1 << 7)).to_bytes(1, 'little'))
    datalen = struct.unpack('<q',
        rng.read(
            struct.calcsize('q') - 1
        ) + b'\0'
    )[0].to_bytes(
        1,
        'little',
        signed=True
    ) % 1024 + 1
    data = rng.read(datalen)
    out.write(header + data)
