#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='little', signed=False
    )

    # TODO: Make this more efficient
    data = bytes()
    while len(data) < datalen - 6:
        data += rng.read(datalen // 100 + (datalen % 10 == 0))
    
    palette = [
        tuple([
            i & 0xff for i in rng.read(3)
        ]) for _ in range(256)
    ]

    out.write(header)
    out.write(struct.pack('<H', int(datalen)))
    out.write(b'\x87\x00') # Logical screen descriptor
    
    for p in palette:
        out.write(p[0])
        out.write(p[1])
        out.write(p[2])

    out.write(b',' * ((datalen - 6) // 3))
    out.flush()

