#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Image descriptor
    imdesclen = 40
    imdeschdr = \
        [b'\x2c', bytes([imdesclen]),
         # FrameData
         # FrameLength (in bytes)
         bytes([rng.read(1)[0] ^ datalen]),
         # FrameData
         ] + bytes([
             # Left
             rng.read(2)[0], rng.read(2)[0],
             # Top
             rng.read(2)[0], rng.read(2)[0],
             # Width
             rng.read(2)[0], rng.read(2)[0],
             # Height
             rng.read(2)[0], rng.read(2)[0])] * 5

    out.write(header)
    # Logical screen descriptor
    out.write(struct.pack('<6sHHBBBHB',
                         b'0000',
                         32767,
                         32767,
                         7,
                         0,
                         0,
                         0))
    for _ in range(5):
        out.write(imdeschdr[:8])
        # LZW MinCodeSize
        out.write(b'\x00')
        # Packed Fields
        fields = set()
        while len(fields) < 3:
            fields |= {rng.read(1)}

        out.write(bytes(sorted(fields)))
        out.write(b'\x00\x00')

