#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write the header
    out.write(header)
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        if (i % 128 == 0):
            print(f"{i} bytes written")
        
        # Get two random bytes to make up one color
        # pixel value
        rcolor = rng.read(1)[0]
        gcolor = rng.read(1)[0]
        bcolor = rng.read(1)[0]

        # Build an RGB triple from those values
        rgb = [rcolor, gcolor, bcolor]

        # Write the block with that triplet
        block = b'' + \
                struct.pack('BBB', *(rgb*64)) + \
                struct.pack('<B', 0x00)
        while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True

