#!/usr/bin/env python3

import struct
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # bytes to write in the beginning of the file (screen descriptor)
    screen_descriptor = \
            0x2c.to_bytes(1, 'little', signed=False) + \
            datalen.to_bytes(3, 'big', signed=False) + \
            0xf7.to_bytes(1, 'big', signed=True)

    print(f'datalen {datalen}')

    out.write(header)  # File Header
    out.write(screen_descriptor)

    while True:

        # read chunk length and data type
        try:
            current_chunklen = int.from_bytes(
                    rng.read(1), byteorder='big', signed=False
                )

            if current_chunklen == 0x00:
                break
            
            current_datatype = int.from_bytes(
                    rng.read(1), byteorder='big', signed=False
                )
            print(current_datatype)
        except Exception as e:
            print("error occured")
            print(e)
            continue


        following_byte = rng.read(1)[0]

        # graphic control extension block
        if current_datatype == 0xf9:
            pass
        elif following_byte!= 0x00:
            raise ValueError('Invalid chunk format!')

        
        if current_chunklen > 50:
            current_chunklen = 50
        
        if not all([i in range(0, 256) for i in list(struct.pack(">B", current_datatype))]):
            raise ValueError('Invalid datatype!')
        
        if current_chunklen < 255:
            size = [current_chunklen, ]
        else:
            size = [254, *list(struct.pack(">H", current_chunklen))]

        out.write(*size)
        out.write([current_datatype])
        out.write(rng.read(current_chunklen-2))


outstream = open('./output.gif', "wb+")
with open('/dev/urandom', mode="rb+") as instream:
    generate_random_gif(instream, outstream)


