#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big') + 1 # LZW minimum code size is 2

    data_blocks = []
    for _ in range(datalen):
        data_block = bytearray()

        while True:
            byte = rng.read(1)[0]

            if (byte & 0xE0 == 0x20 or
                byte & 0xF0 == 0x70 or
                    byte & 0xF8 == 0xC0):
                break
            else:
                data_block += bytes([byte])
        
        data_block += bytes([0]) # Block terminator
        data_blocks.append(data_block)
    
    header += len(data_blocks).to_bytes(2, 'little')
    header += b',\0' * 4
    out.write(header)


    # Write each data block one by one to ensure that padding works correctly
    for datablock in data_blocks[:-1]:
        out.write(datablock)
        zeroes = 256 - len(datablock)
        if zeroes > 0:
            out.write(b'\x00'*zeroes)
    
    final_block = bytearray()
    final_block[:] = data_blocks[-1][:-1]
    final_block[len(final_block)-1] &= ~0x80 # Set last bit of final block to 0

    out.write(final_block)

