#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(1), 'big') & 0x7F
    
    # read rest of data block
    data = bytearray()

    while len(data) < datalen:
        data += rng.read(datalen - len(data))

    assert len(data) == datalen

    terminator = b'\x21\xFF\x0B' + \
                (b'Image Descriptor' if datalen else b'') + \
                4 * b'\x00'

    print(f"Writing {len(header)} bytes")
    out.write(header)
    out.write(struct.pack('>H', datalen))
    out.write(data)
    out.write(terminator)
