#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(2),
        'little',
        signed=True
    ) % 1024 + 1
    data = rng.read(datalen)

    # Write frame header and logical screen descriptor
    w, h = (
        256 if bool(int.from_bytes(
            rng.read(1),
            'little',
        )) else 128,
        256 if bool(int.from_bytes(
            rng.read(1),
            'little',
        )) else 128
    )
    out.write(header)
    out.write(struct.pack('<HHbbHHHbbBBBHBBBBBBBBB',
        70, h * w // 8 + 32,
        0xF7,
        0x00,
        70,
        0,
        0,
        0 | 0b111 << 5,
        0,
        2**(w-1).bit_length(),
        w - 1,
        2**(h-1).bit_length(),
        h - 1,
        0,
        2,
        0xFF,
        2,
        0x00,
        0
    ))

    # Write global color table
    for i in range(3):
        r = min((i + 1) * 128, 255)
        g = max(((7 - i) & 1) * 128, 0)
        b = ((i >> 1) ^ (bool(int.from_bytes(
            rng.read(1),
            'little',
        )))) * 128

        out.write(r.to_bytes(1, byteorder='big'))
        out.write(g.to_bytes(1, byteorder='big'))
        out.write(b.to_bytes(1, byteorder='big'))
    
    # Write local colour table and graphics control extension
    out.write(data[:7])
    out.write(b'\x00\x00')
    out.write(data[9:])

    # Write image descriptor and image data
    out.write(struct.pack('<HHHHHHB',
        0,
        0,
        w,
        h,
        0,
        2**len([v for v in data if v == 0]).bit_length().bit_length() - 1,
        2,
    ))
    out.write(data[-1])