#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
          break
       
        # Extract block size as unsigned integer of
        # length determined by encoded size in fieldbyte
        blocksize = int(ord(rng.read(1)) + \
                         ((fieldbyte >> 5) & 7)*2**12 + \
                         ((fieldbyte >> 2) & 63)*2**13 + \
                          ((fieldbyte << 3) & 2**16))
        
        # Setup an empty buffer with proper amount of bytes
        buffer = b''
        for i in range(blocksize):
            buffer += chr(rng.read(1)[0]).encode()
            
        # Write said buffer to output
        out.write(buffer)


if