#!/usr/bin/env python3

import struct
from argparse import ArgumentParser, Namespace


def main():
    p = ArgumentParser()

    p.add_argument('input', help='The firmware blob to convert')
    p.add_argument('-o',
                   '--output',
                   default=None,
                   dest='output',
                   metavar='<file>',
                   help='Output file (default: replace <input>)')
    args = p.parse_args()

    if not args.output:
        # We're only going to change the filename extension anyway...
        outfn = args.input.rsplit('.', 1)[0] + ".blob"
    else:
        outfn = args.output

    with open(args.input, 'rb') as f:
        data = bytearray(f.read())

    offset = -48
    while True:

        # This should end at EOF
        try:
            addrlen = int.from_bytes(data[offset+2:][:4], "little")
        except ValueError:
            break

        print("Found potential entry point @", hex(struct.unpack('<I', data[offset:][-4:])[0]))

        sig = b"\x56\x4c\x9d\xc5"

        if data[:len(sig)] == sig and addrlen > len(sig):

            # Skip over signature and address length fields.
            offset += len(sig) + 4

            laddr = struct.unpack("<L", data[offset:][:4])[0]
            uaddr = struct.unpack("<L", data[offset+4:][:4])[0]

            if uaddr <= 0xC0B8A000 or laddr >= 0xC0D4E000 or uaddr!= ((laddr & ~0xFFF) | 0x800):
                continue

            print("Found segment @", hex(uaddr))

            # Copy 4 bytes from start of segment up until next segment begins.
            # This is how we identify a valid file.
            data[offset:] = [i for i in range(offset, min(offset+4, len(data)), 1)]

            break


        offset -= 4


    with open(outfn, 'wb+') as fout:
        fout.write(data)


if