#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')

    # Pallette indexes are 1 byte long, so we can at most have
    256 ** (datalen//3) - 1

    # Assemble the GifPreamble and store it to output
    preamble = [header] + \
                list(struct.pack('>H', datalen)) + \
                [b'\x87\x00']       # Global Color Table
    for i in range(0xF):
        preamble += [b'\x00\x00\x00']    # Background color
    preamble += [b'\xFF\x00',         # Pixel aspect ratio
                 b'\x00',             # Image Left position
                 b'\x00',             # Image Top position
                 b'\x00',             # Image Width
                 b'\x00']             # Image Height
    
    out.write(bytearray(preamble))

    # Read the palette colors from urandom
    pal = []
    for _ in range(0, len(preamble)-4):
        pal += [rng.read(1)]

    # Store the palette in the end of the file
    # This is where we'll put the image data
    out.seek(-len(pal), 2)
    out.write(bytearray(pal))

    # Now add the actual image to the end of the file :)
    for c in map(lambda x : bytes([x]), rng.read()):
        out.seek(-len(c), 2)
        out.write(c)
