#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(2),
        'little',
        signed=True
    ) % 1024 + 1
    data = rng.read(datalen)

    # Write header first...
    for i in range(len(header)):
        out.write(struct.pack('B', ord(header[i])))
    
    # Then write a single frame with unlimited LZW compression
    # and no repeat or disposal information
    palette = [b'\x00\x00\xff'] * (256**3 // 8)
    colorCount = len(palette)//3
    out.write(struct.pack('<HHBBB',
        colorCount & 0xffff,
        colorCount >> 16,
        0xf7, # Color table size field (1 byte) with sorted flag set to false
        0,   # No transparent color index
        0    # No background color index
    ))
    for p in palette:
        out.write(p)
    out.write(data)

