#!/usr/bin/env python3

from typing import BinaryIO
import argparse


def main(input_file: BinaryIO) -> None:
    lengths = [ord(byte) for byte in input_file.read()] + [17, 31, 73, 47, 23]

    numbers = list(range(256))
    pos = skip = 0

    for round in range(64):
        for length in lengths:
            sublist = reversed(numbers[pos : pos + length]) if pos > length else numbers[:length]

            for i, value in enumerate(sublist):
                numbers[(pos + len(sublist) - i - 1) % len(numbers)] ^= value

            pos += (length + skip) % len(numbers)
            skip += 1

    dense = []

    for i in range(0, 256, 16):
        dense.append(reduce(xor, numbers[i : min(len(numbers), i + 16)]))

    print("".join([f"{x:0{2}x}" for x in dense]))


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-f", "--filename", required=True)
    args = parser.parse_args()

    with open(args.filename, "rb") as file:
        main(file)
