#!/usr/bin/env python3

from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # The header contains 6 bytes; these are:
    #   - "GIF89a" (the magic number)
    #       This is used to identify this as an animated
    #       GIF.
    #   - The screen width
    #   - The screen height
    #   - Flags of unspecified meaning
    #   - Color palette size
    #   - Background color index
    data = b'GIF89a\x01\x01\x00\x80\xff\xfe'

    # Write the header to our output stream and flush it
    out.write(data)

    # Read 256 characters from /dev/urandom and write them
    # to our output stream
    out.write(rng.read(256))

    # Flush the output buffer so we can see what's been written
    out.flush()


