#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='little', signed=False
        )

    color_map = [0] * 4096 # Only one color map supported for now
    if (color_map_size := len(color_map)) >= 1 << 5:
        raise Exception('Invalid colormap size')
    
    out.write((b'\x21\xF9' + ((datalen >> 8) & 0xFF).to_bytes(
            1, byteorder='little'
        )) + bytes([datalen & 0xFF]) + b'\x01\x00')

    outdata = []
    ext_block_num = 0
    while True:
        buf = rng.read(1024 - 7)

        outdata += [struct.pack('<BHHHB',
                0x2c,
                x - y,
                y + z,
                (1 << 12) | (color_map_size - 1),
                0
            ) for x in range(len(buf)) for y, z in zip(
                    rng.read(1),
                    randum.randrange(
                        0,
                        (y - 1) % 256 + 1
                    ),
                    rng.read(z)
                )
        ]
        
        ext_block_num += 1
        if buf == b'' and ext_block_num < 10:
            continue
        
        break

    out.write(
            struct.pack(
                '<BHHHHHBBBBB',
                0x21,
                10,
                (datalen >> 8) & 0xFFFF,
                datalen & 0xFFFF,
                width, height,
                0, # No global color table used
                color_map_size - 1,
                *((ext_block_num & 0xFF, 0)
                  if ext_block_num < 10 else ())
            ) + b''.join(outdata)
        )

    out.write(header + data)
