#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and an image of size width*height.


def generate_gif(out: BinaryIO, rng: BinaryIO, width=1024, height=576):
    # Header
    out.write(b"GIF89a")  # Signature
    # Width & Height
    out.write(struct.pack("<HH", width, height))
    # Flags (global color table true, sort flag false, colors sorted by frequency, no global palette)
    flags = b"\x87\x00" + bytes([width * height])
    out.write(flags)

    # Color Table
    for i in range(0, int(2 ** ((flags[0] >> 4) + 1)) - 1):
        r = next(rng) % 256
        g = next(rng) % 256
        b = next(rng) % 256

        out.write((r).to_bytes(1, "little"))
        out.write((g).to_bytes(1, "little"))
        out.write((b).to_bytes(1, "little"))

    # Image Descriptor Block
    # Left Position, Top Position, Width, Height, Flag byte (no local color table used)
    left = flags[1].to_bytes(1, "little")
    top = flags[2].to_bytes(1, "little")
    w = flags[3].to_bytes(1, "little")
    h = flags[4].to_bytes(1, "little")
    descriptor = b"\x00\x00" + left + top + w + h + b"\x08"
    out.write(descriptor)

    # LZW Min Code Size
    minCodeSize = b"\xFF"
    out.write(minCodeSize)

    # Data Stream
    numPixelsWritten = 0
    while numPixelsWritten < width * height:
        code = next(rng) % 256
        if code == 255 or len(codeBytes) >= 255:
            out.write(b"\xFF")

            if code!= 255:
                out.write(len(codeBytes).to_bytes(1, "little"))

                for cb in codeBytes[:-1]:
                    out.write(cb.to_bytes(1, "little"))

                lastByte = codeBytes[-1]
                out.write(((lastByte << 1)).to_bytes(1, "little"))

            numPixelsWritten += len(codeBytes) + 1 if code!= 255 else 1

            codeBytes = []
            continue

        codeBytes.append(code)
