#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'little') + 1 # We add one byte here because each frame must end on even bytes

    nframes = int.from_bytes(rng.read(1), 'little')

    width = int.from_bytes(rng.read(2), 'little')
    
    height = int.from_bytes(rng.read(2), 'little')

    localcolortablesize = int.from_bytes(rng.read(1), 'little')

    globalcolortableflag = int.from_bytes(rng.read(1), 'little')

    colorresolution = int.from_bytes(rng.read(1), 'little')

    sortflag = int.from_bytes(rng.read(1), 'little')

    interlacemode = int.from_bytes(rng.read(1), 'little')

    out.write(header)

    out.write(struct.pack('<H', datalen))

    out.write(struct.pack('<B', nframes))

    out.write(struct.pack('<HHB', width, height, localcolortablesize))

    out.write(struct.pack('<BB', *globalcolortableflag.to_bytes(1, 'little')))

    out.write(struct.pack('<BBB', *colorresolution.to_bytes(1, 'little'), *sortflag.to_bytes(1, 'little')))

    out.write(interlacemode.to_bytes(1, 'little'))
