#!/usr/bin/env python3

from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) of size `size`.
def generate(out: BinaryIO, rng: BinaryIO, size: int):
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(b'\x01\x00\x01\x00')  # width & height (1px x 1px -> 2 bytes each)
    out.write(b'\xff\x00\xff\x00')  # global color table (RGB all max values -> 4 bytes per color in RGBA mode)
    out.write(b'\x00')            # background color index
    out.write(b'\x00')            # pixel aspect ratio

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00')     # image separator
    out.write(str(size%256).encode())   # left position
    out.write(str((size//256)%256).encode()) # top position
    out.write(str(width-1).encode())    # width - 1
    out.write(str(height-1).encode())   # height - 1
    out.write(b'\x00')           # local color table flag
    out.write(b'\x00')           # interlace flag
    out.write(b'\x00')           # sort flag
    out.write(b'\x00')           # reserved flag = no transparency
    out.write(b'\x01')           # frame count = 1

    # The actual image data is written as plain text.
    # Each byte contains two pixels with alternating RGB colors on it.
    for _ in range(size // 2):
        byte = rng.read(1)[0]

        for i in range(8):
            if (byte >> 7-i) % 2 == 1:
                out.write(b'\xf0\xf0\xf0\xf0')
            else:
                out.write(b'\x0f\x0f\x0f\x0f')