#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
          return
    
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('B',fieldbyte))
        out.write(struct.pack('B',size))
  
        # If we're reading an image, write its graphic control extension
        if hasimgdata:
            # Skip two bytes on input (left position + top position)
            out.seek(2,1)
            # Put transparent color index in bytes 6-7
            out.write(rng.read(2))

            # Write four more zeroes after that
            out.write(b'\x00\x00\x00\x00')

         # Now read the image itself
         nbytes = size - 1;
         img = rng.read(nbytes);

         # And finally put the image bytes onto the stream
         out.write(img)

