#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017, <NAME>
# All Rights Reserved.
#
# The file is part of the lbuild project and is released under the
# 2-clause BSD license. See the file `LICENSE.txt` for the full license
# governing this code.

import io


def read(buf):
    """
    Read a single frame from the given buffer object and return it as GIF image

    Args:
        buf: Buffer to read the file from.
    Returns:
        PIL Image.
    Raises:
        InvalidFormatException if the format does not support multiple frames or no more images are in the stream.
    """
    import PIL.Image

    img = None
    while True:
        chunktype = buf.read(4)

        if len(chunktype)!= 4:
            raise EOFError("Unexpected end of buffer")

        length = int.from_bytes(
                buf.read(4), byteorder='little', signed=False
        )
        if chunktype == b"IHDR":
            # This chunk should contain the binary representation of an IHDR
            # block that contains information about the following frame.
            subbuffer = io.BytesIO()
            subbuffer.write(chunktype)
            subbuffer.write(length.to_bytes(4, 'little'))

            with open('temp.gif', mode="wb") as f:
                f.write(subbuffer.getvalue())
                f.write(buf.read(length))

                f.seek(0)
                print(PIL.Image.open(f).info['version'])

            break

        else:
            buf.read(length)

    return img


def write(out, rng, **kwargs
     ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='little', signed=False
    )
    data = rng.read(datalen)
    out.write(header + data)
