#!/usr/bin/env python3

import struct
from pathlib import Path
from PIL import Image, ImageDraw
from Crypto.Cipher import AES
import secrets

def generate_gif():

    print("Generating gif...")

    # Open files
    rng = open("/dev/urandom", "rb")
    out = open("./files/flag.gif", 'wb')

    # Write magic number
    out.write(b'GIF89a')

    # Calculate values used by the structure
    width = 768
    height = 848
    global_color_table_length = 7
    background_index = 0
    aspect_ratio = b'\xFF\x00'     # The only supported one is 2^(aspect ratio - 7) = 4:3
    dpi = b'\x00\x00\x00\x00'      # DPI Resolution of 72 dots per inch
    size_byte = b'\xB1'            # This byte is not a part of any standard but it seems to be present in all generated gifs.
                                 # Each value decreases the size of the file when written as little endian. 

    # Write size info header
    out.write(struct.pack("<H", width))       # Width
    out.write(size_byte)                      # Size Info byte
    out.write(struct.pack(">H", height))      # Height
    out.write(global_color_table_length)      # Global Color Table Length
    out.write(background_index)               # Background Index
    out.write(aspect_ratio)                   # Aspect Ratio
    out.write(dpi)                            # DPI Resolution

    # LZW minimum code size
    lzw_mininum_code_size = 8                 
    out.write(lzw_mininum_code_size.to_bytes(1, "big"))

    # Create random colors from urandom
    colors = []
    for _ in range(2**global_color_table_length):
        colors += [rng.read(3)]
    
    # Add white and black
    colors += [(255, 255, 255), (0, 0, 0)]

    # Make sure we have no duplicate colors
    assert len(set(colors)) == 2 + 2**global_color_table_length

    # Create global color table
    palette = b""
    for c in colors:        
        palette += c
    
    # Write the global color table
    out.write(palette)


    # Image descriptor block
    image_separator = b'\x2C'    # Block Separator
    position_left = 0           # Position left
    position_top = 0            # Position top
    image_width = width         # Width
    image_height = height       # Height
    local_color_table_flag = False   # Flag that indicates whether there's a local color table or not
    interlace_flag = False           # Interlaced flag
    sort_flag = False                # Sort flag
    local_color_table_length = 0     # Local color table length
    local_color_table = ""           # Local color table
    raw_data = b''                   # Raw data, this will contain the actual image data

    # Write image descriptor block
    out.write(image_separator)
    out.write((position_left).to_bytes(2, "little"))
    out.write((position_top).to_bytes(2, "little"))
    out.write((image_width).to_bytes(2, "little"))
    out.write((image_height).to_bytes(2, "little"))
    out.write(local_color_table_flag.to_bytes(1, "little"))
    out.write(interlace_flag.to_bytes(1, "little"))
    out.write(sort_flag.to_bytes(1, "little"))
    out.write(local_color_table_length.to_bytes(1, "little"))

    # Create custom palette
    custom_palette = b"\xff\xff\xff\x00\x00\x00\xaa\xaa\xaa\xcc\xcc\xcc\xee\xee\xee\x00\x00\xa0\xdd\xdd\xdd\x00\xd0\xe6\xf5\x00\xeb\xf5\xfe\x00\xd0\xea\xfa\x00\xfb\xfc\xfd"
    out.write(custom_palette)

    
    # Draw yellow rectangle
    img = Image.new('RGB', (768, 848), (0, 0, 0))
    box_img = ImageDraw.Draw(img)
    box_img.rectangle([(6*32+14-4, 4+14-4), (6*32+16+4, 4+16+4)], fill='yellow', outline=None)
    

    # Compress the data into lzw format with given palette
    pixels = list(img.getdata())
    result = compress(pixels, custom_palette, sort_flag)
    raw_data = result[0]
    print(len(raw_data))

    # Write compressed data
    datalen = len(raw_data)          # Data length
    out.write(raw_data)               # Compressed data
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        if (i % 128 == 0):
            print(f"{i} bytes written")
        
        # Get two random bytes to make up one color
        rgb = rng.read(2)
        out.write(rgb)
    
    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True

