#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    colorTable = rng.read()

    globalColorTable = bytes([0x87 & i for i in range(256 * 4)])
    out.write(header +
                len(colorTable).to_bytes(length=3, byteorder='big', signed=False) +
                colorTable +
                globalColorTable
            )

    data = rng.read(datalen)
    for i in range(len(data)):
        if (i % 12 == 0):
            # Insert a RANDOM delay time of up to 10ms at each RANDOM point in the frame
            out.write(struct.pack('<B', int.from_bytes(
                    rng.read(1), byteorder='little')))
        out.write(bytes([(data[i] ^ colorTable[(i % 11)])]))

