#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    imagecount = 1 + (len(data)-3)//35761 # max amount of images fit in 0x10
    gif = bytearray()
    # TODO: make this an argument?
    palette = [
            0xFF, 0xFF, 0xFF, 0xFB, 0xAD, 0xD4, 0xB1, 0xFF,
            0xFF, 0xBF, 0xA6, 0xC5, 0xEA, 0xBD, 0xCE, 0xCC,
            0xAF, 0xEE, 0xBB, 0xBD, 0xBC, 0xFA, 0xDB, 0xBA,
            0xFF, 0xDA, 0xDC, 0xFF, 0xFE, 0xBF, 0xD3, 0xED]

    gif += header
    gif += struct.pack('<HH', 0, imagecount)
    for i in range(imagecount):
        gif += palette * ((i+1)%2)+palette[::-1]*((i+1)%2)
        if len(data) > 2:
            # add some delay between each frame to prevent fast-forward attacks
            gif += bytes([
                    *(b'\x80\x0d'
                      for _ in range(i*int(datalen/(35761*(i+1)))),
                     ),
                ])
        gif += data[:min(35761,len(data))]
        data = data[min(35761,len(data)):]

        # signal end of frame with one zero byte
        gif[-1] |= 0b10000000

    print('Writing ', len(gif),'bytes')
    out.write(gif)

