#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = gen_lsd()
    out.write(lsd)

    while True:
        # Get the global color table size from LSD
        colors      = lsd[4] & 0x07 + 1
        gctflag     = int((lsd[4] >> 7) & 0x01) == 1
        localcolor  = int((lsd[5]) & 0x0F)
        size        = pow(2, localcolor+1)
        lctpresent  = ((lsd[5]) >> 6) & 0x01 == 1
        interlace   = ((lsd[6]) >> 6) & 0x01 == 1
        sortflag    = ((lsd[6]) >> 5) & 0x01 == 1
        reserved    = int(((lsd[6]) >> 3) & 0x03)
        hasgct      = gctflag or lctpresent
        
        # Get the logical screen descriptor block
        if hasgct:
            lsb = gen_lsb(colors=colors, gct=True)
        else:
            lsb = gen_lsb(colors=colors, gct=False)

        # Write it all to the output buffer
        out.write(lsb)

        # If we don't have a GCT, just write out the next 3 bytes as zeroes
        if not gctflag:
            out.seek(1, 1)

        # Get the palette if that's the case
        if hasgct:
            colortable = gen_colortable(
                rng=rng,
                numcolors=pow(2, localcolor+1),
                sortflag=sortflag,
                reserverd=(reserved<<3)+size
            )
            out.write(colortable)

        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)