#!/usr/bin/env python3

import struct
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    out.write(header + bytes([0x10])) # logical screen width 256px

    out.write(b'\x08\x00')             # color table size 256
    for i in range(0xff):                # write palette rgb values
        out.write(bytes([i] * 3))
    
    out.write(b'\x40\x0e')             # background color index (black)
    out.write(struct.pack('>HH', 0, 0)) # global image descriptor, x and y offsets 0

    block = bytearray()
    while True:
        for i in range(0xf0):
            block.append((i // 0xa) % 7 + ord('A'))

        char = rng.read(1)[0]
        
        if len(block) == 0xe5 or char!= 0x00:
            break
        else:
            block.append(char)
            
    # handle 2nd gif frame
    out.write(header + bytes([0x10]) + block[:0xe5])
    block.extend(rng.read())
    
    # handle 2nd gif frame hexdump
    #print('\n'.join(['{:02X}'.format(i) for i in block][:0xe5]))

    # handle 3rd gif frame
    out.write(header + bytes([0x10]) + block[0xe5:])
    block.extend(rng.read(0x1d))
    
    # handle 3rd gif frame hexdump
    #print('\n'.join(['{:02X}'.format(i) for i in block][0xe5:]))

    block[0xd5] |= 0x20               # set local gce as transparency flag 
    out.write(b'\x3b')                 # terminator
    out.write(block)                   # end of file


if