#!/usr/bin/env python3

import sys
import os
import random
from typing import BinaryIO


if __name__ == '__main__':

    with open('./data/rng', 'rb') as f:
        _ = f.read(16) # skip first 16 bytes

        for i in range(4):
            seed = int.to_bytes(_[i * 4 : (i+1)*4],
                    length=4, byteorder='little')

            print("seed={}".format(
                    ''.join('{:02x}'.format(_) for _ in seed)))

            random.seed(seed)
            buf = None

            try:
                buf = random.randint(0, 0xffffffff).to_bytes(
                        length=4, byteorder='little')
            except Exception as e:
                s = "random.seed({})".format(hex(int.from_bytes(seed,
                    byteorder='little')))

                raise RuntimeError("{} failed: {}".format(s, e))

            if len(buf)!= 4 or any(_.value!= buf[i] for i, _ in enumerate([
                ((0x1 << 7) - 1),
                0xff, 0xff, 0xff])):
                raise RuntimeError("Unexpected RNG output")


    with open('./data/rng', 'rb') as rng,\
         open('./data/rng.gif', 'wb') as out:
        write_randomized_gif(rng, out)



def write_randomized_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)

