#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    ) % 256 + 1 # 0 is not valid

    chunk = (
        0xf7, # application extension block identifier
        struct.pack('B', datalen+3), # length of data in bytes
        b'\x0bNETSCAPE2.0\x03\x01\x00', # identifier and version
        struct.pack('>H', datalen-2), # subblock size should be this - 2?
        b'\x00' * (datalen-2) # padding
     )

    # Generate the image structure
    data = []
    for _ in range(datalen):
        r = int.from_bytes(
            rng.read(1),
            byteorder='big',
            signed=False
        ) & 1
        g = int.from_bytes(
            rng.read(1),
            byteorder='big',
            signed=False
        ) & 1

        if r == 0 or g == 0:
            pixel = 0x0f
        else:
            pixel = 0xf0
        
        data += [pixel]

    framedata = b''
    for i in range(datalen):
        framedata += b'\xff' + data[i].to_bytes(
            1,'little',signed=True
        )
    
    # Write out the file contents
    out.write(header)
    out.write((datalen).to_bytes(1,'big'))
    out.write(chunk)
    out.write(framedata)

