#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show how the random generator works, but could be used for testing
def gen_gif(out: BinaryIO, rng):
  # Header
  out.write(b"GIF89a")

  width = 256
  height = 256
  bpp = 1
  flags = (0x40 | 0x80) & ~(0x80 | 0x40)
  
  # Screen descriptor
  out.write(struct.pack("<HHBBB", width, height, bpp, flags))
  
  # Global color table
  gct_entries = [i * 255 // 256 for i in range(bpp ** 2)]
  out.write(bytes([gct_entries[i] for i in rng.sample(list(range(len(gct_entries))), len(gct_entries))]))  

  # Image descriptor
  out.write(struct.pack("<HBBHHB", 0, 0, bpp, 0, 0, 0))
  image_data = rng.sample(list(range(height)), height)
  out.write(bytes(image_data))
  