#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), "big")

    # Write Magic number and LSD
    out.write(header + struct.pack('>H', 0))

    # Write all frames
    for _ in range(datalen):
        # Get size of current frame (int16 big endian)
        width = int.from_bytes(rng.read(1), 'big') & 0x7F | \
            int.from_bytes(rng.read(1), 'big') << 8

        height = int.from_bytes(rng.read(1), 'big') & 0x7F | \
                int.from_bytes(rng.read(1), 'big') << 8
 
        # Set colour bits per pixel to be 8
        out.write(b'\xF7\x00')

        # Write Width and Height
        out.write(struct.pack('>HH', width, height))

        # Size of local color table (must be at least 256 bytes)
        out.write(b'\xFF\x0B')

        # Flag that indicates the presence of a local color table
        out.write(b'\x00')
        
        # Add an empty code block to indicate there is no global color table
        out.write(b'\x00')

        # Start position of image data (1 byte)
        out.write(b'\x0B')

        # Write palette entries
        for i in range(256):
            out.write(rng.read(3))
    
        # Write actual RGB Data by dividing bitstream by zero-byte and then copy each slice as-is
        while True:
            substream = rng.read(1)[0]

            if not substream > 0: break
            
            length = min((substream, len(rng.getbuffer()) - rng.tell()))

            out.write(rng.read(length))

        # Write trailing 0 for last byte 
        # (this was apparently required by some parsers?)
        out.write(b'\x00')


if