#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add the LSD fields to the header
    for i in range(len(lsd)):
        header += bytes([lsd[i]])
    
    out.write(struct.pack('<H', 640))             # Width
    out.write(struct.pack('<H', 256))             # Height
    out.write(bytes([lsd[3]]))                    # Flags
    out.write(bytes([lsd[4]]))                    # Background color index
    out.write(bytes([lsd[5]]))                    # Pixel aspect ratio

    # Write image data
    for y in range(256):
        block = bytearray()

        # Write sub-blocks containing each row of pixels
        while len(block) < 128:
            byte = rng.read(1)[0]

            # Set high bit if this is not the last frame in the animation
            if byte == 0x2c or byte == 0x21:
                byte |= 0b10000000
            
            block.append(byte)
        
        out.write(block)

    # Image block terminator
    out.write(bytes([0]))