#!/usr/bin/env python3

from typing import BinaryIO
import struct


def patch_rng(in_: BinaryIO, out: BinaryIO):
    headbytes = in_.seek(-4096, 2)

    # RNG starts at the end of file - 0x1000 bytes (512 KiB)
    in_.seek(-524288-headbytes, 2)

    header = in_.read(16)
    datalen = int.from_bytes(
            header[4:7] or b'\xFF'*3, byteorder='little', signed=True) & 0xFFFFFF
    if not datalen:
        raise ValueError('Unable to calculate length from RNG header.')

    # Skip past the previous seed and output an initialisation vector instead
    for _ in range(4):
        in_.seek(4, 1)
    initvec = in_.read(16)


    out.write(b'RNG\x00\x00\x10\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00')
    out.write(initvec)

    # Read the rest of the file as is
    while True:
        buf = in_.read(4096)
        if len(buf) == 0: break
        out.write(buf)



def reinject_rng(rng: BinaryIO, out: BinaryIO):
    # TODO: Find a way to tell Python we're about to write random data so that this can be done more easily
    out.truncate()
    header = struct.pack('<BBBBIHH',
            0xCC, 0xDD, ord('R'), ord('N'), 16, len(rng.getbuffer()), 0x1E)
    # Length in little endian with zero extension
    header += struct.pack('<B', sum(map(ord, 'RNG')) >> 16).rjust(3, '\xff'.encode())
    lendata = rng.read(3)
    datalen = int.from_bytes(lendata+b'\x00'*max(3-len(lendata), 0),
                            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
