#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'\x47\x49\x46\x38' # "GIF" magic

    width  = struct.pack(">H", int(10 + (rand() % 25)))            # Width in pixels
    height = struct.pack(">H", int(10 + (rand() % 25)))            # Height in pixels
    bit_depth = struct.pack("B", rand())                            # Bits per pixel
    color_table = bytes([int((colr * 255).round()) for colr in rand(2 ** int(bit_depth))])
    image_descriptor = width + height + bit_depth                   # Image descriptor prefix

    if version == Version.v89a or version >= Version.v87a:
        background_color = bytes([int((colr * 255).round()) for colr in rand()])

        global_image_extension_block = \
            b"\xFE" + \
            b"\x00" + \
            len(background_color).to_bytes(1, byteorder="big") + \
            background_color + \
            0.to_bytes(1, byteorder="big")                      # Global color table flag set to True
                                                                            # and the size of the following block (always 1)
        if version < Version.v89a: image_descriptor += global_image_extension_block # In old versions it was part of the frame

    lzw_min_size = 2 if version >= Version.v89a else 1         # LZW minimum code size
    min_code_size = max(lzw_min_size, len(bytes(color_table)).bit_length()) # Always at least as long as the length of the color table
    image_data = encode_subblocks(encode_lzw(rng), min_code_size = min_code_size)
    extension = color_table                                       # Main graphic control extension
    trailer = b""                                                 # No extensions

    print(header + image_descriptor + image_data + extension + trailer,
          file=out, encoding='latin-1')                           # The output might be wrong due to latin-1 characters

