#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='little', signed=False
    )
    data = rng.read(datalen)
    colorTable = (b'\x00\xff')*256
    
    # Determine number of bytes for image size
    width = 4
    height = 4

    # Compose output from parts
    result = header + \
             len(data).to_bytes(2, 'little') + \
             768 + \
             width.to_bytes(2, "little") + \
             height.to_bytes(2, "little") + \
             b'\x81' + \
             b'\x00' + \
             b'\x00' + \
             data[:width * height] + \
             colorTable + \
             data[width * height:]

    out.write(result)
