#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write the header
    out.write(header)
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        if (i % 128 == 0):
            print(f"{i} bytes written")
        
        # Get two random bytes to make up one color
        rgb = rng.read(2)
    
        # Convert it from RGB to BGRA
        r = (rgb[0] >> 4) + ((rgb[1] & 0xf0) << 4)
        g = (rgb[1] & 0x0f) + ((rgb[0] & 0xf0) << 4)
        b = (rgb[0] & 0x0f) + ((rgb[1] & 0xf0) << 4)

        # And write it all out!
        out.write((r).to_bytes(1, 'little'))
        out.write((g).to_bytes(1, 'little'))
        out.write((b).to_bytes(1, 'little'))
    
    # Fill out the remaining space with zeros so that the file is valid
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True

