#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):

    # Make sure we don't overwrite anything
    assert len(out.read()) == 0

    # Generate some metadata
    header = bytes([
        0x47, 0x49, 0x46, 0x38,     # GIFF
        0x39,                     # version
        0x02,                     # flags
        0x00, 0x01,               # width
        0x00, 0x01                # height
    ])
    out.write(header)

    # Now write the palette and all frames and data blocks
    num_frames = rng.read(1)[0] % (256 - 8) + 8
    for _ in range(num_frames):

        capacity = rng.read(1)[0] & 0x0F
        length = ((capacity + 1) << 8) | rng.read(1)[0]

        if (length > 128 * 1024):
            raise Exception("Frame too large")

        left = b''
        right = b'\0' * capacity

        while True:
            chunk = bytearray()

            cnt_left = min(rng.read(1)[0], capacity // 2)
            cnt_right = capacity - cnt_left

            chunk += rng.read(cnt_left)
            chunk += b'\xFF' * cnt_right
            chunk[-1] |= rng.read(1)[0] & 0xF0

            chunk[0] &= ~0x80
            lzw = struct.pack('B', len(chunk))

            chunk = lzw + chunk;

            if sum(chunk) < 0xFFFF:
                break

        out.write(length.to_bytes(2, "little"))
        out.write(chunk)


if