#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)

    # Write first part of gif (header + color table)
    out.write(header+rng.read(datalen))  # type: ignore[arg-type]
    
    i = 0
    while True:
        # Get next chunk size from input stream
        data = rng.read(4)

        if not data or len(data)!= 4:
            break
        
        chunksize = int.from_bytes(
                data, byteorder='big', signed=False)
        blocktype = bytes(rng.read(1))[0].to_bytes(
                1, byteorder="big", signed=True)

        pixeldata = (blocktype == b'\x2c') and\
                    (i < 256) and\
                    rng.read((chunksize - 1)) or\
                    None

        if pixeldata!= None:
            assert len(pixeldata) >= 7
            
            out.write(struct.pack("<BHHBBBH", 0x2C,
                                  ord('A'),
                                  0,
                                  0,
                                  0,
                                  ord('!'),
                                  0xFF))

            out.write(pixeldata)
            i += 1
        
        else:
            out.write(bytearray([*chunksize]))
            out.write(blocktype)
            out.write(rng.read(chunksize - 1))
            
    out.flush()

