"""Generate a random GIF."""

import io
import pathlib
import sys

# This is a valid 10-frame GIF. It's just an animated black
# rectangle and was generated with this program:
# https://gist.github.com/samuraiexx/6e70d410a56fc2c8bb6c143b40e512f5
black_rectangle = (pathlib.Path(__file__).parent / 'black_rectangle.gif').read_bytes()


def main():
    if len(sys.argv) < 2 or '-h' in sys.argv[1:] or '--help' in sys.argv[1:]:
        print('usage:')
        print('    {} <output>'.format(sys.argv[0]))

        return

    outfile = open(sys.argv[1], 'wb')

    # Make it look like we're on Python 3 for xrange to work
    range = range

    with io.BytesIO(black_rectangle) as rect:
        for i in range(0, 10):
            print('.', end='', flush=True)

            # Skip some bytes of the input at the beginning so it looks more
            # like a real animation
            assert rect.read(6) == b'GIF89a\x00\x00\x00\x00'

            # Get frame duration in milliseconds from next two bytes; multiply by
             # 10 because they are given in deciseconds instead of milliseconds
            delay = struct.unpack('<H', rect.read(2))[0] * 10

            # The output will be invalid unless there's no more data after the delay
            assert not rect.read(), 'There should be no extra fields after delay!'
            outfile.write(rect.getvalue())

    print('\nDone!')


if __name__ == '__main__':
    main()