#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write the header
    out.write(header)
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        if (i % 64 == 0):
            print(f"{i} bytes written")
        
        x = int.from_bytes(
                rng.read(2), byteorder='big', signed=False) 
        y = int.from_bytes(
                rng.read(2), byteorder='big', signed=False) 

        # Each pixel is encoded as two bytes:
        # - The upper nibble indicates the color index
        # - The lower nibble indicates how many times to repeat it
        c = int.from_bytes(
                rng.read(1), byteorder='big', signed=False) 

        # Encode the colors
        red   = ((c >> 7) << 5) | (((c >> 2) & 0x1F) << 3)
        green = ((c >> 6) & 0x03) | (((c >> 1) & 0x1F) << 3)
        blue  = ((c >> 5) & 0x07) | (((c     ) & 0x1F) << 3)


        # Pack the RGB triplet into a single int
        color = ((red<<16) | (green<<8) | (blue))

        packed_color = struct.pack('<I', color)[-1:]

        # Use a little trickery here so we can get both pairs at once
        out.write(packed_color * min((y+1),(2**12)))

    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True