#!/usr/bin/env python3

import argparse
import os
import sys
import binascii
import struct
from typing import BinaryIO

RAND_FILE = '/dev/urandom'


# Parses the command line arguments and exits with error status if there was an issue
# Returns the output file handle to write to or None on failure
def parse_args() -> (BinaryIO,int):
    parser = argparse.ArgumentParser('A program for generating random GIF files.')

    parser.add_argument('-d', '--debug', action='store_true', help='Enables debug mode which prints more info about what is happening internally.')
    parser.add_argument('-o', '--output-file', type=str, default='', metavar='OUTFILE', required=True, help='The name of the output file that will be created by this script (required).')
    args = vars(parser.parse_args()) # Convert Namespace object to dict

    if not args['debug']:
        global debugOutput
        debugOutput = lambda *a,**k:None


    try:
        outputFileHandle = open(args['output_file'], 'wb+')
    except IOError as e:
        print("Failed to create output file '{}': {}".format(args['output_file'],e))

        return (None,-1)
    
    else:
        return (outputFileHandle,0)



# Generates a random GIF file into `out` using the
# random number generator `rng` (`RAND_FILE`)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)

    return True




if __name__ == '__main__':
    exitCode = 0

    (outputFileHandle,parseArgsExitCode) = parse_args()
    if parseArgsExitCode!= 0:
        exitCode += 1
        sys.exit(-1)
    

    if not generate_random_gif(open(RAND_FILE,'rb+'), outputFileHandle):
        exitCode += 1
        sys.exit(-1)


    sys.exit(exitCode)