#!/usr/bin/env python3

import random
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (for example from /dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    width = random.randint(1, 65535)
    height = random.randint(1, 65535)
    
    flags = bytearray()
    flags[7] |= bool(random.getrandbits(1)) << 7    # Global color table flag
    if not flags[7]:
        flags[7] |= bool(random.getrandbits(1)) << 6    # Color resolution
    if not flags[7]:
        flags[7] |= bool(random.getrandbits(1)) << 5    # Sorting method
    flags[7] |= 0b00011                   # Size of global color table
    flags[7] |= 0b0100 << 3               # Background color index
    flags[7] |= 0b0100 << 4               # Pixel aspect ratio
    bg_color_index = random.randint(
            0, 2 ** (flags[7] & 0b0001 == 1 and
                        2 or bits_in_byte(flags[7] >> 3)) - 1
        )
    pixel_aspect_ratio = random.choice([
                0x00,   # Undefined
                0b0001, # Square pixels only
                0b0010, # 4:3
                0b0011, # 16:9
                ]) & 0b0011
    flags[7] |= pixel_aspect_ratio << 5
    bits_per_pixel = random.randint(1, 8)
    flags[7] |= ((bits_per_pixel - 1) |
                     (bits_per_pixel - 1) << 3 |
                         0b10000 << 6)

    out.write(header)
    write_ushort(width, out)         # Screen descriptor
    write_ushort(height, out)        # "      "
    out.write(struct.pack('B', *flags))     # Flags
    out.write(struct.pack('B', bits_per_pixel))
    out.write(struct.pack('H', *bg_color_index.to_bytes(2, 'little')))
    write_ushort(0xFFFF, out)          # Frame count
    out.write(b'\x00' * 3)            # Reserved
    out.write(b'\x00\xFF')             # Application extension block size
    
    # Application extension block
    out.write(b'NETSCAPE')                 # Extension type string
    out.write(b'\x03F\xFE')                # Application identifier
    out.write(b'\x01')                     # Application auth code
    write_ushort(len(b'\x00\x01'), out)    # Block length
    out.write(b'\x00\x01')                 # Loop subblock ID
    write_ushort((width * height) // 20, out)   # The loop count
    out.write(b'\x00')                     # Null terminator

    for i in range(((width*height)//32)+1):              # Image data
        datlen = min(int(random.weibullvariate(20,.2)*.6+.4*32), 32)
        out.write(b'\x00'*datlen)

    # Image descriptor
    write_ushort(0, out)                  # Left position
    write_ushort(0, out)                  # Top position
    write_ushort(width, out)              # Width
    write_ushort(height, out)             # Height
    out.write(b'\x80')                    # Packed field
    write_ubyte(bits_per_pixel, out)     # LZW minimum code size
