#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom). Returns true on success else false
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:
    MAX_DATALEN = 4096
    MAX_FRAMELEN = 70 * MAX_DATALEN + 5120 + 300
    FRAME_DELIMITER = '----GIF-DATA-END----'
    DATA_START = f'{len(FRAME_DELIMITER)}:'.encode()
    DATALEN = min(MAX_DATALEN, max(0, MAX_FRAMELEN - len(DATA_START)))

    header = b'GIF89a'
    datalen = DATALEN

    # Write the header
    out.write(header)
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    try:
        framecnt = 0
        while (datalen > 0):

            # Make sure we don't overflow the buffer
            datalen -= 2
            
            if (datalen <= 0 or
                    framecnt >= MAX_DATALEN // 128):
                break
        
            # Get two random bytes to make up one color
            rgb = rng.read(2)
            out.write(rgb)

            # Keep track of how many colors have been written
            framecnt += 1
        
    except IOError as e:
        print("Failed!")
        return False
        

    # Add some extra zeros to fill the maximum frame size
    padsize = max(
            0, 
            MAX_FRAMELEN - out.tell(), 
            300
        )

    while len(out.getbuffer()) < 5120 + padsize:
        out.write(b'\x00\x00')

    # Write the delimiter
    out.write(DATA_START+FRAME_DELIMITER.encode('utf-8'))

    return True