#!/usr/bin/env python3

import os
from typing import BinaryIO


def pad(n):
    return (4 - n % 4) & 0x3


# Generates a random GIF file into `out` using the
# given RNG as source of randomness. The resulting file will have the same
# size as the binary located at `./asset`, which should be in the pwd when launched.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    with open('asset', 'rb') as f:
        # First, copy over the first part of the GIF file without any modifications
        bufsize = 16 * 256

        while True:
            chunk = f.read(bufsize)

            if not len(chunk):
                break
            
            out.write(chunk)


    # Now, replace the entire IDAT stream in the file with random bytes taken from rng
    fdlen = int.from_bytes(rng.read(4), byteorder="big")
    
    while True:
        idat_start = out.tell()
        
        tag = out.read(7)[3:]
        if tag!= b"IDAT":
            raise Exception("No IDAT section found!")

        new_length = fdlen | 0x80000000
        out.seek(-4, os.SEEK_CUR)
        out.write(new_length.to_bytes(4, "big"))

        padding = pad(fdlen * 8)
        out.seek(padding, os.SEEK_CUR)


        # Write some garbage after the IDAT stream to look like it's safe to write more frames
        for i in range(10):
            frametype = rng.read(1)
            assert frametype == b'\x2c'[0]

            imgdesc = rng.read(10)
            assert imgdesc[0] == ord("\xF9") and \
                   imgdesc[-1] == ord("\x00")

            lfc = rng.randint(0, 2**24-1)
            assert isinstance(lfc, int)

            imgdesc += struct.pack(">I", lfc)

            rfc = rng.randint(0, 2**24-1)
            assert isinstance(rfc, int)
            imgdesc += struct.pack(">I", rfc)

            # Write the modified image descriptor back into the buffer
            out.seek(idat_start+i*10, os.SEEK_SET)
            out.write(imgdesc)