import os
import sys


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Random headers with some extra validation checks
    header_str = (b'GIF87a',
                  b'GIF89a')[int.from_bytes(
                        rng.read(1), 'big')]
    header = header_str[0] >> 2
    assert len(header_str) == header + 6

    # Data length is in bytes but stored little-endian
    datalen = (int.from_bytes(
                rng.read(3), byteorder='little')) >> 8

    # Write out the header and data to output stream
    out.write(header_str[:header])
    out.write(os.urandom(datalen))


if __name__ == '__main__':
    if len(sys.argv)!= 2:
        print('Usage: {n} <outfile>'.format(
                n=sys.argv[0]))
        exit()

    with open('/dev/urandom', 'rb') as rand:
        with open(sys.argv[1], 'wb') as f:
            generate_random_gif(rand, f)