#!/usr/bin/env python3

import pathlib
import tempfile
import subprocess as sp
import random
import string


def get_example_data() -> bytes:
    with open('example_data') as f:
        return f.read().encode()


def generate_random_gif(length: int) -> str:
    # Create temp files for input and output
    inpath = tempfile.mkstemp()[1]
    outpath = tempfile.mkstemp()[1]

    # Generate example data
    with open(inpath, 'w+b') as infd:
        for _ in range(length):
            char = random.choice(string.printable)
            if len(char) == 0 or ord(char) > 0x7f:
                print(ord(char))
            infd.write(char.encode())

    # Run encoder
    p = sp.run([
        "target/debug/gif",
        "-i",
        "--compress-transparent",
        inpath,
        outpath])
    assert p.returncode == 0

    # Read generated image back from disk
    with open(outpath, 'r+b') as inf:
        result = inf.read()

    # Clean up
    for path in (inpath, outpath):
        try:
            pathlib.Path(path).unlink()
        except FileNotFoundError: pass

    return result