#!/usr / bin / env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    width:   int   = (int.from_bytes(rng.read(1), 'big') & 15) + 16
    height:  int   = (int.from_bytes(rng.read(1), 'big') & 15) + 16

    palette: list[tuple[int]]    = [
        tuple([int.from_bytes(rng.read(1),'big')]) * 3 for i in range(768)]

    imagedata: bytearray        = bytearray()
    imagedata_format: str       = f'<{width // 8}B'
    for i in range(height):
        rowdata: bytes            = rng.read((width + 7) >> 3)
        imagedata += struct.pack(imagedata_format, *rowdata)
    
    header = b'GIF89a'
    datalen = len(palette) * 3 + len(imagedata)
    out.write(header +
             width.to_bytes(2,'big') + 
             height.to_bytes(2,'big'))
    out.write(b'\xf7\x00\x00') 
    out.write(struct.pack('<H', datalen)) 
    out.write(b'\x00')    
    for p in palette: out.write(p)
    out.write(imagedata)