#!/usr/bin/env python3

import os
import struct
import sys
from typing import BinaryIO


MAX_DATA_SIZE = 256 * (1024**2)

def main():
    if len(sys.argv)!= 2:
        print('Usage: {} SAVEFILE'.format(__file__))
        return

    filename = sys.argv[1]
    with open(filename, 'wb') as f:
         generate_random_gif(f)


def generate_random_gif(out: BinaryIO):
    idata = bytes((os.urandom(7))[:7])
    ldata = os.urandom(os.randrange(1, MAX_DATA_SIZE+1))

    # Prepares the Gif Header
    header_fields = {
       'signature': b'GIF89a',
       'version-str': b'89a',
        'width': os.urandom(2),
        'height': os.urandom(2),
    }

    # Formats the header fields and writes them on the output stream
    header = []
    for key, val in sorted(header_fields.items()):
        header += [key, val]

    out.write(b''.join([
        ''.join(header).encode(),
        struct.pack('<B', len(ldata)),  # Framedata length field
        ldata,                              # Payload
        bytes(idata[:-1]),                  # Image descriptor
        idata[-1:]                          # Terminator
    ]))


