#!/usr/bin/env python3

import argparse
import os
import sys


def get_parser() -> argparse.ArgumentParser:

    parser = argparse.ArgumentParser('GIF Random Generator',
        description="Generate a random GIF.")
    
    parser.add_argument('--out-file', '-o', type=str, default='', 
        help="output file (default to stdout)")
    
    parser.add_argument('--length', '-l', metavar='LENGTH', type=int, default=-1, help="length of data")
    
    return parser

def main():

    # parse command line arguments 
    args = get_parser().parse_args();

    if not 0 < args.length <= 2**24 - 69:
        print("Length must be in range [1, ", 2 ** 24 - 69, "]", 
            file=sys.stderr)
        exit(-1);

    with open('/dev/urandom', 'rb') as rand:

        if not args.out_file:

            generate_random_gif(
                rng=rand, 
                out=sys.stdout.buffer
            )
        
        else:

            try:

                with open(args.out_file, 'wb') as f:

                    generate_random_gif(
                        rng=rand, 
                        out=f
                    );
            
            except OSError:
                
                print("Cannot create output file", args.out_file, 
                    file=sys.stderr)
                exit(-1);

if __name__ == "__main__":
    main()