#!/usr/bin/env python3

import sys
import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    width = int(sys.argv[2])
    height = int(sys.argv[3])


    # Get the logical screen descriptor (LSD) field
    lsd = bytes([width&0xff,
                (width>>8)&0xff,
                (height&0xff),
                (height>>8)&0xff,
                0b111 << 5, 
                0x11, 
                0x00])

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = max(1<<(ord(lsd[4])>>5)*3, 2<<8)

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = bytes([(i+256)%256
                   for i in range(gctlen)])
      
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
          break
    
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 
        #print("hasimage=%d" % (hasimgdata,))

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:
          colortable = bytes([(i+256)%256
                              for i in range(numcolors*3)])
          out.write(colortable)

        # Write the actual image data
        imgdata = bytes([(i+256)%256
                         for i in range(size-1)])

        out.write(imgdata)