#!/usr/bin/env python3

import argparse
import os
import sys


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--length', '-l', type=int, default=5000)
    parser.add_argument('output')

    args = parser.parse_args()


    with open('/dev/urandom', 'rb') as raw_rand:

        if not (os.path.exists(args.output) and input('{} already exists; overwrite? [y/N]'.format(args.output)).lower().startswith('y')):
            print('Aborting.')
            return 1


        with open(args.output, 'wb') as f:

            # Header
            f.write(b'GIF89a')


            # Logical screen descriptor
            widthBytes = (args.length).to_bytes(2, 'little')
            heightBytes = (args.length).to_bytes(2, 'little')
            f.write((widthBytes[0]) | ((widthBytes[1]) << 8))
            f.write((heightBytes[0]) | ((heightBytes[1]) << 8))
            f.write(b'\xf7\x00')   # Global color table flag
            f.write(b'\x00')       # Background color index
            f.write(b'\x00')       # Pixel aspect ratio


            # Graphics control extension
            f.write(b';')          # Extension introducer
            f.write(b'<')          # Graphic control label
            f.write(b'\x04')       # Block size
            f.write(b'\x00')       # Flags
            f.write((widthBytes[0]) | ((widthBytes[1]) << 8))        # Width
            f.write((heightBytes[0]) | ((heightBytes[1]) << 8))       # Height
            f.write(b'\x00\xff')    # Packed field: no transparancy, user input off, delay in units of 10ms
            f.write(b'\x00')       # Block terminator


            frameCount = 0


            while True:

                for i in range(6):
                    f.write(raw_rand.read(1))      # LZW minimum code length


                if frameCount == 0 or frameCount % 8 == 7:

                    f.seek(-6, 1);                   # Seek over the block terminator


                    # Plain text sub-block
                    f.write(b',')                    # Sub-block separator
                    f.write(os.linesep.encode())     # Sub-block data
                    f.write(b'!>')                   # Terminating sequence


                else:

                    f.seek(-9, 1);                   # Seek over the block terminators


                    # Image descriptor
                    f.write(b';')                # Extension introducer
                    f.write(b'+')                # Image descriptor label
                    f.write(b'\x00\x00')         # Left position
                    f.write(b'\x00\x00')         # Top position
                    f.write((widthBytes[0]) | ((widthBytes[1]) << 8))            # Width
                    f.write((heightBytes[0]) | ((heightBytes[1]) << 8))           # Height
                    f.write(b'\x00')             # No local color table
                    f.write(b'\x08')             # LZW minimum code length
                    f.write(b'\x02')             # Interlace bit set to "none"
                    f.write(b'\x00\xf7')         # Sort order




                frameCount += 1


                if len(f.getbuffer()) > args.length * 1000000:
                    break


if __name__ == '__main__':
    exit(main())