#!/usr/bin/env python3

import struct
import argparse
from typing import BinaryIO


def parse_args():
    p = argparse.ArgumentParser()

    p.add_argument('-o', '--output')
    p.add_argument('--data-len', type=int)
    return p.parse_args()


def main():
    args = parse_args()

    if not (args.output or args.data_len):
        raise Exception('Either --output or --data-len must be specified.')
    
    rng = open('/dev/urandom', 'rb')
    with open(args.output, 'wb+') as out:
        if args.data_len:
            out.write(b'GIF89a')

            datalen = args.data_len & 0xFFFF # Clip to uint16 max
            data = rng.read(datalen)
            out.write(struct.pack('<H', datalen))
            out.write(data)

        else:
            generate_random_gif(rng, out)
            

if __name__ == '__main__':
    main()