#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:
    header = 'GIF89a'.encode('ascii')

    # Write the header
    assert(out.write(header))

    # Chose a random data length between 0 and 4096
    datalen = rng.randint(0, 4096)

    # Write that data length as three big endian bytes
    assert(out.write((datalen).to_bytes(3, 'big')))

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        # Print progress occasionally
        if ((i + 1) % 128 == 0):
            print(f"{i+1} bytes written", end="\r")
        
        # Get two random bytes to make up one color
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        rgb = [r,g,b]
        out.write(struct.pack('BBB', *rgb))
        
    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True