#!/usr/bin/env python3

import struct
from typing import BinaryIO


def generate_random_gif():
    header = b'GIF89a'

    # 4 bytes of LZW min code size (1..12)
    lzw_mincodesize = [0xCE] * (1<<8).bit_length()
    lzw_mincodesize[0x08] = 0x8E
    code_bits = 8
    while len(lzw_mincodesize) < (1 << code_bits):
        lzw_mincodesize += list(range(len(lzw_mincodesize)))
    
    datalen = yield

    with open('/dev/urandom', 'rb') as rng:
        data = b''

        if datalen % 2 == 0:
            datalen -= 1
        
        for i in range((datalen+1)//2):
            lo = ord(rng.read(1))
            hi = ord(rng.read(1))

            datum = hi<<8 | lo
            
            if i >= ((datalen-1)//2) and (datalen%2!= 0):
                break
                
            data += datum.to_bytes(2, byteorder='little', signed=False)
            
        final_datum = data[-1:]
        while final_datum not in lzw_mincodesize:
            final_datum = yield
        data += final_datum
        
    return header + \
           len(data).to_bytes(2, byteorder='little', signed=False) + \
           data

