import io
import os
import struct
from typing import Tuple


# Generates a random GIF file with size `size`, using the
# random number generator `rng` (/dev/urandom). Returns a tuple
# of `(data length, filename)` where `filename` is the path to
# generated GIF
def gen_random_gif(
        rng: BinaryIO, size: int
    ) -> Tuple[int, str]:

    if not isinstance(size, int):
        raise TypeError('size must be an integer')
    
    if size <= 0 or (size%2!= 0 and size > 1):
        raise ValueError('size must be positive even and less than 65534')

    # Generate data
    leng = size // 2 - 1
    header = bytes([
        0x47,           # 'G'
        0x49,           # 'I'
        0x46,           # 'F'
        0x38,           # '8'
        0x39,           # '9'
        0x61            # 'a'
    ])
    body = bytearray()
    for _ in range(leng * 2):
        body += struct.pack('<B', rng.read(1)[0])
    
    # Write to file
    fd, fn = mkstemp('.gif')
    try:
        f = os.fdopen(fd, "wb")

        # Write headers
        f.write(header)
        f.write(struct.pack(">H", leng))
        
        f.flush()

        return len(body)+10, fn
    except Exception as e:
        print("Exception raised while generating {}".format(e))
    finally:
        close(f)
