#!/usr/bin/env python3

import os
import sys
import io
import struct
from typing import BinaryIO


# Generates a random GIF file using the
# given RNG as source of randomness. The output size is 10kB (less than
# one full JPEG frame).
def generate_random_gif(rng: BinaryIO):

    # Header
    out = io.BytesIO()
    out.write(b"GIF89a")

    # Logical screen descriptor
    width, height = [int.from_bytes(rng.read(2), "big", signed=False) for _i in range(2)]
    flags = bytes([rng.read(1)[0] | 0x7])
    bg_color = rng.read(1)
    pixel_aspect= bytes([rng.read(1)[0]])
    out.write(struct.pack("<HHBBB", width, height, flags[0], *bg_color, pixel_aspect))

    # Color table
    color_table = [[rng.read(1) for i in range(4)] for j in range((2**flags & 7)+1)]
    if len(color_table)>1:
        out.write(b"\x2c\xff\x00".join([b"".join(ct) for ct in color_table]))

    # Image descriptor
    left, top = [int.from_bytes(rng.read(2), "big", signed=False) for _i in range(2)]
    width, height = [int.from_bytes(rng.read(2), "big", signed=False) for _i in range(2)]
    flags = bytes([rng.read(1)[0] | 0xf0])
    print(hex(flags[0]), hex(flags[-1]))
    local_color_table = bool(flags&0x80)
    interlace = bool(flags&0x40)
    sort = False # Interlaced images are always sorted
    reserved = bytes([flags[0]>>3])
    lctsize = ((2**(flags[0] & 7))+1)*3 if local_color_table else None
    if local_color_table:
        print("Local color table:", hex(flags[0]), hex(flags[-1]), local_color_table)
        print("LCT size:", lctsize)
        color_table = [[rng.read(1) for i in range(4)] for j in range(lctsize+1)]
        out.write(b"".join([(b"".join(ct)) for ct in color_table]))
    out.write(struct.pack("<HHBBBBBB", left, top, width, height,
                           flags[0],
                           *(reserved+(interlace<<7)+(sort<<6)+flags[1:])))


    data = rng.read(height*width)
    out.write(b"\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00")
    out.write(bytes([height//256]+[height%256][:1]))
    out.write(bytes([width//256]+[width%256][:1]))
    out.write(b"\x00")

    out.seek(0)
    return out


