#!/usr/bin/env python3

import struct
from typing import BinaryIO


def gen_len(): return (0x40 << 16) | (struct.unpack(">L", bytes.fromhex("3C"))[0])


def len_of_block(length): return length & ((1<<5)-1)


def len_is_sub(length): return bool((length >> 5) & 1)


def len_as_int(length): return length & ~((1<<5)-1)


def is_special_imgtype(block):
    if block == b'\xF9\x04': # Graphic Control Extension
        return True

    elif block == b'\xFF\x01': # Plain text extension
        return True

    else: return False


def get_ext_data(block, headertype):
    """ Returns the data from an extension block """

    if block == b'\xF9\x04':

        flags = ord(block[1])
        delay = struct.unpack('>H', block[3:5])[0] / float(0xFFFF)
        transparent = ord(block[5])

        return {
                "flags": flags,
                "delay": delay,
                "transparent-color-index": transparent }


    elif block == b'\xFF\x01':
        assert ord(headertype) == 0xFF and ord(block[:1]) == 0x01, \
            f"Not a plain text extension! ({ord(headertype)}, {ord(block[:1])})"

        junk, x, y, width, height, cellsize = struct.unpack(">BBHHB", block[1:])

        return {"x": x, "y": y}


    else: raise RuntimeError(f"{block}")



# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    while True:

        if not struct.unpack(">L", rng.read(4))[0]: break

        out.write(b"\x21")


        headertype = rng.read(1)


        if not is_special_imgtype(headertype):

            out.write(headertype)


            lentype = rng.read(1)
            datablock = None


            if len_is_sub(lentype):

                lengroups = 1 << (len_of_block(lentype))


                for _ in range(lengroups - 1):
                    out.write(b"\x80")

                # write last group
                out.write(bytes([len_as_int(lentype)]))



            else:
                
                numblocks = len_of_block(lentype) 
                datablock = rng.read(numblocks)


                for i in range(numblocks):

                    # first block has only one byte
                    if i == 0:
                        out.write(bytes([datablock[i]]))
                    
                    # all other blocks have two consecutive bytes
                    else: 
                        out.write(bytes([datablock[i], datablock[i-1]]))



        else: 
            extbody = rng.read(gen_len())
            
            extdata = get_ext_data(extbody, headertype)
            print(extdata)
                
            

            # extheader = rng.read()
        
        pass


    pass