
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write the header
    out.write(header)
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        if (i % 64 == 0):
            print(f"{i} bytes written.")
        
        # Get three random bytes to make up one pixel
        r, g, b = [int.from_bytes(
                    rng.read(1), 
                    byteorder='big') 
                    >> 5 for _ in range(3)]

        # Construct the byte for this pixel from these RGB values
        px = ((r << 7) | (g << 2) | (b >> 3)) 
        out.write(px.to_bytes(1, 'big'))
    
    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True

