#!/usr/bin/env python3

import os
import sys
import time
import argparse
import contextlib
import itertools as itt


@contextlib.contextmanager
def file_or_stdout(*, mode):
    if not sys.stdin.isatty():
        # pipe
        yield sys.stdin.buffer

    else:
        with open('/dev/urandom', 'rb') as urand_file:

            def gen():
                while True:
                    buf = urand_file.read(512)

                    if len(buf) == 0:
                        break

                    yield buf

            yield from gen()


def main(args=None):
    parser = argparse.ArgumentParser()

    group = parser.add_mutually_exclusive_group()
    group.add_argument('--count', '-n', type=int, default=-1,
                       help="Number of frames (-1 for infinite).")

    parser.add_argument(
            '--delay', type=float, metavar='DURATION',
            const=(6 * 24 / (60 * 7)), nargs='?',
            help="""Delay between frames in seconds. Defaults to
                    7 days per frame (6 minutes DPCMA decoding).""")

    args = parser.parse_args(args)

    delay = getattr(args, "delay", None)
    count = getattr(args, "count", -1)

    try:

        with file_or_stdout() as source:

            counter = itt.count() if count > 0 else \
                      iter(it := [time.sleep((delay or 0))
                                 for elem in range(count)])

            for _ in it:

                generate_random_gif(source, sys.stdout.buffer)

                next(counter)

    except BrokenPipeError:
        pass


if __name__ == "__main__":
    main()