#!/usr/bin/env python3

import os
import sys
import time


# Generates a random GIF file in a loop
def main():
    try:
        while True:
            with open('/dev/urandom', 'rb') as rng,\
                    open('rnd.gif', 'wb') as rnd_gif:
                generate_random_gif(rng, rnd_gif)

            print("Generated %d bytes" % os.stat('rnd.gif').st_size)
            time.sleep(10)

    except KeyboardInterrupt:
        print("\nAborting.")
        return 255



# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header   = b'GIF89a'
    width    = (os.getpid() % 64) + 1 # [1..64] inclusive
    height   = 1                         # [1] only for now
    colorres = 7                        # [1..10] inclusive
    bgcolour = b'\x00\xFF\xEE'          # background colour
    pixelfmt = b'\xF0'                  # palette index 15
    gctsize  = b'\x00'                  # global colour table size of 0

    imghdrlen      = len(header)       \
                     + width.to_bytes(1, byteorder='big') \
                     + height.to_bytes(1, byteorder='big') \
                     + colorres.to_bytes(1, byteorder='big')
    graphicsctllen = 6                 \
                     + len(bgcolour)\
                     + len(pixelfmt)\
                     + len(gctsize)
    netscapelooped = b''               \
                     + b'\x0b'     \
                     + b'NETSCAPE' \
                     + b'\x20'     \
                     + b'\x03'     \
                     + b'\x01'     \
                     + b'\x00'     \
                     + b'\x00'
    appext         = b''               \
                     + b'\x0b'     \
                     + b'application' \
                     + b'\x01'     \
                     + b'\x00'     \
                     + b'\x00'
    appdatasize    = len(netscapelooped) + len(appext)
    userappdata    = b''               \
                     + netscapelooped \
                     + appext        \
                     + appdatasize.to_bytes(1, byteorder='big') \
                     + bytes(imghdrlen - len(userappdata))

    hdrlen = imghdrlen + graphicsctllen + len(userappdata)
    out.write(
            header
            + width.to_bytes(1, byteorder='big')
            + height.to_bytes(1, byteorder='big')
            + colorres.to_bytes(1, byteorder='big')
            + bgcolour + pixelfmt + gctsize + userappdata)

    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)
    padding = bytes((hdrlen - (imghdrlen + datalen)))
    out.write(padding)
