#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')


    # Logical screen descriptor
    width = rng.read(2)
    height = rng.read(2)
    packed = (
                "1"     # Global color table flag
                 + "00"   # Color resolution
                 + str(rng.read(1))  # Sorting
                 + width    # Width
                 + height   # Height
             )
    out.write((packed+'\x00').encode('ascii'))


    # Global color table entry
    for i in range(int(width[::-1].hex(), 16)*int(height[::-1].hex(), 16)):
        r = rng.read(1)
        g = rng.read(1)
        b = rng.read(1)
        alpha = '\xff' if ((i % 255) == 0) else '\x00'
        out.write(r+g+b+alpha)


    # Application extension block
    application_id = 'NETSCAPE2.0'
    appdata_len = 3
    appdata = '?\x0b\x00'
    out.write(application_id.encode('ascii'))
    out.write(appdata_len.to_bytes(1,'little'))
    out.write(appdata.encode('ascii'))


    # Comment extension block
    comment = "Hello, world!"*10
    comment_char_size = len([ord(c) for c in comment])
    out.write("comment".encode('ascii'))
    out.write((str(comment_char_size)+'\x00').encode('ascii'))
    out.write(comment.encode('utf-8'))


    # Image descriptor
    image_left_pos=0
    image_top_pos=0
    image_width = width
    image_height = height
    packed = (
                "\x00" # Local color table flag
                + "\x00" # Interlace flag
                + "\x01" # Sorting
                + "\x00" # Reserved
                + image_left_pos      # Left position
                + image_top_pos       # Top position
                + image_width         # Width
                + image_height        # Height
             )
    out.write((packed+'\x08').encode('ascii'))


    # Pixel data
    pixdat_length = int(image_width[::-1].hex(), 16)*int(image_height[::-1].hex(), 16)
    for i in range(pixdat_length):
        rgb = [rng.read(1) for _ in range(3)]
        alpha = '\xff' if ((i % 255) == 0) else '\x00'
        out.write("".join(rgb)+alpha)


