#!/usr/bin/env python3

import os
import sys
import subprocess
import struct
from typing import BinaryIO


def read_entire_file(f):
    f.seek(0)
    buf = bytearray()
    while True:
        b = f.read(1<<16)
        if not len(b): break
        buf += b
    return bytes(buf)


def to_uint16(x):
    assert x < 2**16
    return struct.pack('>H', x)


def popen(*args):
    print(*args)
    proc = subprocess.Popen(args, stdout=subprocess.PIPE)
    stdout, stderr = proc.communicate()

    return proc.returncode == 0, stdout, stderr



def make_local_files():
    # Make sure we can write files in this directory
    cwd = os.getcwd()
    assert len(cwd) > 0 and cwd[-1]!= '/'

    assert popen('rm -f [0-9]*.[gp]wts [0-9]*.png')

    for i in range(100):
        good = popen('ffmpeg', '-y','-t','00:05','-i', '/dev/zero', '-frames:v', '1', f'{i}.png')

        assert good


    assert popen('convert *[0-9].png *-[0-9].png [0-9]*.png')




def run_encoder(input_filenames, output_filename):
    args = ['./encode']
    for filename in input_filenames:
        args.append(filename)
    args.append('-o')
    args.append(output_filename)
    return popen(*args)






def main():
    if '--create-local-files' in sys.argv:
        make_local_files()
        exit(0)

    with open('/dev/urandom', mode='rb') as rfs:
        with open('./random.gif', mode='wb+') as wfs:
            generate_random_gif(rfs, wfs)

    assert run_encoder(['random.gif'], './random.gif')



    # We expect there to be no differences between./random.gif
    # and./random2.gif.
    assert not popen('cmp', './random.gif', './random2.gif')
    
    assert run_encoder(['./random.gif'], './random2.gif')
    assert popen('cmp', './random.gif', './random2.gif')

    
    assert run_encoder(['./random.gif'], './random.gif')
    assert popen('cmp', './random.gif', './random2.gif')
    





if __name__ == '__main__':
    main()