#!/usr/bin/env python3

import sys
import os
import math
import struct
from typing import BinaryIO


# Generates an animated GIF that shows a moving purple dot bouncing around
def generate_moving_purple_dot():
  with open('/dev/urandom','rb') as rfile:
    with open('./moving_purple_dot.gif','wb') as wfile:
      
      generate_random_gif(rfile,wfile)

      # Generate the header and add to output
      header = bytes([
          0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 
          0x0d, 0x00, 0x00, 0x0b, 0xff, 0x00,
          0xe3, 0x0c, 0xd9, 0xce, 0x00, 0xa4,
          0x0f, 0xed, 0xf3, 0x00, 0x00, 0x3b])
      wfile.write(header)

      # Get the logical screen descriptor (LSD) field
      lsd = rfile.read(7)

      # Add LSD to output
      wfile.write(lsd)

      # Read global color table length from LSD
      gctlen = int(((2**lsd[4] >> 5))*3)

      # Add GCT to output if we have it
      if gctlen > 0:
        gct = rfile.read(gctlen)
        wfile.write(gct)
    
      # Get remaining blocks
      buff = []
      done = False
      i = 0
      while not done:
        done = (ord(rfile.read(1)[0]) & 128)==0
        print(done)
        wfile.write(rfile.read())
        i += 1

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        max_width: int = 256, 
        max_height: int = 256
    ):
    # Generate the header and add to output
    header = [
        0x47, 0x49, 0x46, 0x38, 
        0x39, 0x61, 0x0d, 0x00, 0x00, 
        0x0b, 0xff, 0x00, 0xe3, 0x0c, 
        0xd9, 0xce, 0x00, 0xa4, 0x0f, 
        0xed, 0xf3, 0x00, 0x00, 0x3b
    ]

    header = bytes(header)
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)
    _ = ord(lsd[-1:])
    lsd = lsd[:-1]+bytes([(max(math.ceil(_*max_width/256),4)*3)])+lsd[-1:]
    assert len(lsd)==7, "Invalid LSD"
    
    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = int(((2**lsd[4] >> 5))*3)

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
          return
    
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:
          colortable = rng.read(numcolors*3)
          out.write(colortable)

        # Write the actual image data
        imgdata = rng.read(size-1)
        w = (max(math.floor((i:=ord(imgdata[0]))*max_width/256),4)*3)
        h = (1<<i)-1
        assert all((v<=255 for v in list(imgdata[:h]*2))), f"Invalid width {w} or height {h}"
        out.write(imgdata)