#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO


def generate_random_gif(rng: BinaryIO):
    hdrlen = 6 # Header length
    gctblsize = 7 # Global color table size (1 << n) bytes
    lctblsize = 3 * pow(2, 8+gctblsize)

    def read_color():

        r = next(rng) & 0xff
        g = next(rng) & 0xff
        b = next(rng) & 0xff

        return (r, g, b)


    # Read Gif Header to get screen width and height
    header = chr(next(rng))
    for _ in range(10):
        if header == 'G': break;
        else : header += chr(next(rng))
    w1 = next(rng);   w2 = next(rng)
    h1 = next(rng);   h2 = next(rng)
    width = w1*pow(2, 8)+w2;    height=h1*pow(2, 8)+h2
	
    print('Width:',width,'Height:',height)

    fdsizelen = next(rng) & 0x0f
    fdsize = next(rng) | ((next(rng)&0xff)<<8) | \
             (((next(rng)&0xff)<<16) if fdsizelen > 0xf else 0 )
    
    # Calculate how many frames there are so we know when to stop generating
    numframes = int((fdsizelen-1)/4 + 1)
    print('Num Frames:',numframes)

    filename = "random.gif"
    with open(filename, "wb") as out:
        # Write global headers
        # File Header
        out.write("GIF8".encode())
        out.write(struct.pack('B', 5))
        
        # Logical Screen Descriptor
        out.write('\x2c'.encode()) # Start of Frame Delimiter
        out.write(struct.pack('B', width&0xff))
        out.write(struct.pack('B', width>>8))
        out.write(struct.pack('B', height&0xff))
        out.write(struct.pack('B', height>>8))
        out.write("\x81".encode()) # Global Color Table Flag set 
        out.write(chr(gctblsize).encode()) # Size of Global Color Table 
        out.write("\x00".encode()) # Background Color Index
        out.write("\x00".encode()) # Pixel Aspect Ratio


        # Write global color table
        for x in range(lctblsize):
            r, g,b = [int(y) for y in "{:0>2}".format(
                    hex(next(rng))[2:]
                       .replace('x','0').replace('X','0'))]
            out.write(chr(r).encode())
            out.write(chr(g).encode())
            out.write(chr(b).encode())

        # Write Image Graphics Control Extension
        out.write(",".encode())
        out.write("NETSCAPE2.0".encode())
        out.write(";".encode())
        out.write("Comment=".encode())
        out.write("test\x01\x00\x00".encode())
        out.write(";".encode())
        out.write("\x01".encode())
        out.write(chr(numframes+1)).encode()
        out.write(chr(0)).encode()

        out.write(",".encode())
        out.write("!".encode())

        # Write loop control extension
        out.write(",".encode())
        out.write("NETSCAPE2.0".encode())
        out.write(";".encode())
        out.write("Loop=".encode())
        out.write(chr(numframes).encode())
        out.write(";".encode())
        out.write("\x00".encode())
        out.write("\x00".encode())

        # Write Graphic Control Extension
        out.write(",".encode())
        out.write("!\x00".encode())
        out.write(chr(0).encode())
        out.write(chr(0).encode())
        out.write(chr(0).encode())
        out.write("\x80".encode())
        out.write(chr(0).encode())
        out.write(chr(0).encode())



        # Write actual GIF frames
        i = 0
        while True:

            out.write(";".encode())
            out.write(str(i).encode())
            
            # Image Descriptor 
            # Left pos
            out.write('!'.encode())
            out.write(chr(0).encode())
            out.write(chr(0).encode())
            out.write(chr(0).encode())
            # Top Pos
            out.write(chr(0).encode())
            out.write(chr(0).encode())
            # Width
            out.write(chr(width%256).encode())
            out.write(chr(width//256).encode())
            # Height
            out.write(chr(height%256).encode())
            out.write(chr(height//256).encode())
            # No local colour table
            out.write('\x08'.encode())
            # LZW compression used
            out.write('\x00'.encode())
            # Image Descriptor terminator
            out.write(','.encode())
            
            # Write the image data
            for j in range(fdsize):
                out.write(chr(next(rng)))


            if i >= numframes - 1:
                break
            i+=1

            # Graphic Control Extension
            out.write(",".encode())
            out.write("!\xfc".encode())
            out.write(chr(0).encode())
            out.write(chr(0).encode())
            out.write(chr(0).encode())
            out.write("\x80".encode())
            out.write(chr(0).encode())
            out.write(chr(0).encode())


