#!/usr/bin/env python3

import argparse
import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (using /dev/urandom if no other is given).
# The output filename can be overriden with `--name`.
def generate_random_gif(
        args: argparse.Namespace,
        rng: Optional[BinaryIO] = None
    ):
    # Get a RNG and its length
    urand = open('/dev/urandom') \
        if not rng else rng
    rnglen = os.stat(urand.fileno()).st_size

    # Generate a random name for the image
    fname = "".join([chr((rng.randint(-57, -46))%26+ord('A'))
                    for i in range(rnglen)])+".gif"
    if args.name!= "":
        fname = args.name

    # Write all of that to disk
    with open(fname, "wb") as out:
        header = b'GIF89a'

        datalen = rng.randint(
            -args.max_data_length,
            args.max_data_length
        ) % 1024 + 1
        data = urand.read(datalen)
        out.write(header + data)