#!/usr/bin/env python3

import os
from sys import argv
from typing import BinaryIO

# random number generator `rng` (`<seed> -g`)
def generate_random_gif(
        seed: str, rng: BinaryIO, out: BinaryIO
    ):

    # parse arguments
    args = dict([arg.split('=', maxsplit=1) for arg in argv[2:]])
    args['length'] = 0 if 'length' not in args else \
                     int(args['length'])

    # read input files
    with open(f'{seed}.g', mode='rb') as infile:
        header = infile.read(6)
        datalen = int.from_bytes(
                infile.read(3), byteorder='big')
        length = (datalen // 7) * 4
        data = infile.read()[:length]

    # construct output file
    datalen += len(data)
    data += bytes(b'\x00') * ((datalen // 7) * 4 - len(data))
    if not args['length']:
        args['length'] = datalen
    elif args['length'] < datalen:
        raise ValueError('not enough entropy to produce this many frames')
    out.write(header + args['length'].to_bytes(3, byteorder='big'))
    while len(data):
        framelen = min((datalen // 7) * 4, 256)
        out.write(b'\x2c\x00\x00\xff' +
                  framelen.to_bytes(2, byteorder='little') +
                  data[:framelen])
        data = data[framelen:]

