import struct
import sys


# Generates a 1-frame animation of a single color
def generate_color_animation(
        size, color, duration=0, output=sys.stdout.buffer
    ):

    # Write magic header (size, frame count and loop count)
    output.write(struct.pack('>6L',
        0x47494638,  # "GIF8"
        2*(size[0]+1)*(size[1]),    # Empty screen area
        0x100,   # One frame
        0x00,     # No looping
        0,        # No delay
        0         # Not sorted either
    ))
    
    # Write global palette with transparency index
    output.write(b'\xff\xff\xff\x00' + bytes([color]*128))

    # Write image descriptor
    output.write((0x2c).to_bytes(1, 'big'))       # Image separator start
    output.write(struct.pack('<HHhhBB',
        0, 0,                       # Left offset, top offset
        *size,                      # Frame width, height
        0,                          # No local palette table
        0                           # Transparency index
    ))

    # Write color map with only one entry for this frame
    output.write(b'\xff\xff\xff\x00' + bytes([color]))

    # Write graphics control extension
    output.write(((0x21).to_bytes(1,'big')))    # Extension label
    output.write(b'\xf9')                        # Graphic control
    output.write(duration.to_bytes(2, 'little')) # Duration in msec
    output.write(b'\x00')                        # Transparent color index (none)
    output.write(b'\x00')                        # Terminator

    # Write image separator end
    output.write(b';')