#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Write header
    out.write('GIF89a'.encode())
    
    # Generate width and height. The values are in range [0x000, 0xFFFF]
    w = int.from_bytes(rng.read(2), byteorder='big') & 0xFFFF 
    h = int.from_bytes(rng.read(2), byteorder='big') & 0xFFFF 

    # Write logical screen descriptor
    out.write(struct.pack('<HHBB', w, h, 0b1000_0000, 0))

    # Write global palette (all black)
    for i in range(256):
        out.write((0x00, 0x00, 0x00).to_bytes(length=3, byteorder='little'))

    # Write local colour table block
    out.write(struct.pack('<B', 7)) # Size of LCT (1<<7)
    for i in range(7):
        out.write((i*2**6, i*2**6, i*2**6).to_bytes(length=3, byteorder='little'))

    # Write graphic control extension
    out.write(b'\x21\xF9\x04\x08' + \
              os.urandom(8) + \
              struct.pack('<HBBBBB',
                          0x00FF, 255, 0, 0, 0, 0
                      ))

    # Write image descriptor block
    lzw_mincodesize = min(os.urandom(1)[0], 3+1)
    out.write('\x2C\x00\x00\x00\x00\x00\x00\x00'.encode() + 
              struct.pack('<HH',
                          w, h
                      ))

    # Encode data with LZW
    # TODO: implement LZW compression

    # Write end of input marker
    out.write(b'\x3B')