#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:
    # Write the header
    out.write(b'GIF89a')
    # Write the screen descriptor
    out.write(struct.pack('>HHBBB',
            4096,
            256,
            7,
            0,
            0))
    # Write loop control block
    out.write(struct.pack('<BHBxxxxxxH',
            0b00000001,
            0,
            0))
    # Zero-fill the rest of the global header
    while len(out.getbuffer()) < 10:
        out.write(b'\x00\x00')


    # XXX: This should be done better...
    # But it's okay for now because we don't care about security! :P
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        if (i % 128 == 0):
            print(f"{i} bytes written")
        
        # Get three random bytes to make up one color
        rgb = rng.read(3)
        # Set the alpha channel to full transparency
        rgb += b'\xff'
        out.write(rgb)
    
    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True


