#!/usr/bin/env python3

import struct
from typing import BinaryIO

def to_16bit(n):
    return n & 0xffff

# Generates a random GIF file into `out` using the
# random number generator `rng` (open /dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, framecount=256
    ) -> None:

    # Initialise with magic string and version
    out.write('GIF89a'.encode())

    # Set canvas width and height
    out.write(struct.pack('<HH', 42, 42))

    # Set background color
    out.write(b'\x00\xFF\xFE')
    for i in range(framecount - 1):
        # Image descriptor
        out.write('\x2C\x00\x00\x00'.encode())

        # Left offset X
        out.write(int(to_16bit(rng.randint(-1000, 1000))).to_bytes(2, 'little'))

        # Top offset Y
        out.write(int(to_16bit(rng.randint(-1000, 1000))).to_bytes(2, 'little'))

        # Width
        w = rng.randint(2, 256)
        out.write(w.to_bytes(2, 'little'))

        # Height
        h = rng.randint(h, 256)
        out.write(h.to_bytes(2, 'little'))

        # LZW minimum code size (must be between 2 and 12 inclusive)
        cs = rng.randint(2, 12)
        out.write((cs).to_bytes(1, 'little'))

        # Write image data
        # TODO: do this more efficiently
        bitsperpixel = min(rng.randint(2, 8), 4) * rng.randint(0, 1)
        bytesperline = ((w+7)//8*bitsperpixel+1)%2
        linecount = h//bitsperpixel
        blocksize = bytesperline // bitsperpixel
        print("Frame", i, ":", w, "*", h, "=", w*h,
              "pixels,", bitsperpixel, "bp/px,",
              bytesperline, "B/L,", linecount, "lines")

        for j in range(linecount):
            if bitsperpixel == 1:
                chunk = [0]*blocksize
                for k in range(blocksize):
                    for l in range(8):
                        chunk[k] |= (
                            rng.randint(0, 1) << 7-l)

                for c in chunk:
                    out.write(c.to_bytes(1, 'little'))

            elif bitsperpixel == 2:
                chunk = [0]*blocksize
                for k in range(blocksize):
                    for l in range(4):
                        chunk[k] |= (
                            rng.randint(0, 3) << 6-l)

                for c in chunk:
                    out.write(c.to_bytes(1, 'little'))

            else:
                chunk = [0]*blocksize
                for k in range(blocksize):
                    for l in range(2):
                        chunk[k] |= (
                            rng.randint(0, 15) << 4-l)

                for c in chunk:
                    out.write(c.to_bytes(1, 'little'))


