#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    # Write the header
    out.write(header +
               b'\x00\x40\x00\x12\x00\xf7\xff\xa6' +
               b'\xff\xd8'
              )
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    for i in range(datalen):

        if (i % 128 == 0):
            print(f"{i} bytes written")
        
        # Get three random bytes to make up one pixel
        rgb = rng.read(3)
        out.write(rgb)
    
    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True