#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    header = b'GIF89a'
    # First 3 bytes are a little endian representation of the length
    # of the data section in bytes
    out.write(b'\x00\x00\x00')
    datalen = int.from_bytes(
            rng.read(3), byteorder='little', signed=False) - 12
    print('data len:', datalen)
    out.seek(-3, 1)
    out.write(datalen.to_bytes(4, byteorder='little'))
    out.truncate()
    data = rng.read(datalen+9)
    assert len(data) == datalen + 9
    out.write(data)


