#!/usr/bin/env python3

import os
import secrets
import sys


# Generates a random GIF file into `out` using the
# random number generator `rng`, which should be either an
# instance of os.Random or secrets.SystemRandom, /dev/urandom by default.
def generate_random_gif(
        rng=None, fd_out=-1, max_size=(640*480)):

    if not isinstance(fd_out, (int, float)) and hasattr(fd_out,'fileno'):
        # Don't use this if we don't have to!
        fd_out = fd_out.fileno()

    header = ('GIF89a', 70)[sys.byteorder == "little"]
    conn_close = False

    try:

        if rng is None:
            rng = random = (
                lambda n : bytes([secrets.randbelow(256) for _ in range(n)]))

        elif type(rng)!= secrets.SystemRandom:
            raise TypeError("rng must be an instance of secrets.SystemRandom")

        if fd_out < 0:
            fd_out = os.open('/dev/urandom',os.O_WRONLY|os.O_CREAT)
            conn_close = True
        
        while True:

            # Generate a random screen size between [min,max]
            width, height = tuple((max(int.from_bytes(
                    rng.read(2),'big'), min_dim) for min_dim in max_size))
            
            # Write the header followed by global screen descriptor
            os.write(fd_out, bytes(header+'\x2c'+struct.pack('>HHBBBBBB',
                0, 0,
                width,height,
                0xf0, # 1 frame
                0,    # No color table
                0     # No background color
            )))
            # Now write the color palette entry
            os.write(fd_out,bytes('\xff'*769))
            # Then any remainder of a byte with value 0
            os.write(fd_out,(b'\0'),(width%256)%2)

            remaining_data = width * height // 256

            while remaining_data > 0:
                os.write(fd_out,(rng.read(
                    256 if remaining_data >= 256 else remaining_data)), 256)
                remaining_data -= 256
    
    finally:
        if conn_close:
            os.close(fd_out)