#!/usr/bin/env python3

import struct
from typing import BinaryIO
from itertools import chain

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    image_datasize = [
            *[b'\x00\x02'] * (int.from_bytes(
                    rng.read(1), byteorder='big', signed=False
                ) & 0x7f),

            *[b'\xff\x00'] * ((0x80 - int.from_bytes(
                    rng.read(1), byteorder='big', signed=False
                )) & 0x7f),

            b'\x00\x02\xff\x00'[
                int.from_bytes(
                    rng.read(1), byteorder='big', signed=False
                ),

                int.from_bytes(
                    rng.read(1), byteorder='big', signed=False
                )
            ]
        ]
    
    image_data = list(chain(*[
            rng.read((int.from_bytes(size, byteorder='big') // 2)) 
            for size in image_datasize
        ]))

    width = max([len(data) for data in image_data])
    height = len(image_data)
    while True:
        if len(struct.pack('HHB',width,height,0)) < 1000: break

        width -= 1
        
    blocksize = [
            8,

            59 * width,
            
            59 * width 
        ]

    global_color_table = []
    if bool(int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) & 0x80):
        
        local_color_table = [
                0, 
                2, 

                1 << (
                        int.from_bytes(
                            rng.read(1), byteorder='big', signed=False
                        ) & 0x7
                    ),

                16 + (
                        1 << (
                                int.from_bytes(
                                    rng.read(1), byteorder='big', signed=False
                                ) & 0x7
                            )
                    ),

                256 + (
                        16 + (
                                1 << (
                                        int.from_bytes(
                                            rng.read(1), byteorder='big', signed=False
                                        ) & 0x7
                                    )
                            )
                    )
            ]
        
        global_color_table = [
                None,
                
                [(i >> 16) & 0xff] 
                + [(i >> 8) & 0xff] 
                + [i & 0xff] 
                for i in range(local_color_table[-1]+1)
            ]

    else:
        local_color_table = []
            
    color_table = [*global_color_table, *local_color_table]

    write_list = [
            header,
            
            struct.pack('<HBBBBB',width,height,*local_color_table),

            *(b'\x2C' + 
                len(block).to_bytes(1,'little') + 
                block + 
                b'\x3B'
                for block in [
                        struct.pack(
                            '<HBHHHHB',
                            0xf9, 
                            len(color_table)-1, 
                            
                            len(color_table[0]) // 3-1, 
                            
                            width, 
                            height, 
                            0, 
                            0b11100111, 
                            0
                        ) 
                    ] + [
                        struct.pack('<HHBBB',*colors)+b'\x00' 
                        for colors in color_table
                      ])
        ]+[*image_data, b';'] 

    for block in write_list:
        out.write(block)