#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(b'\xff', 3), byteorder='big') & 0xfff

    # Write the header
    out.write(header)
    print(f"Datalen: {datalen}")

    # Generate and write the rest of the GIF data
    # Use chunks of size 126*4 = 576 bits or 64 bytes because that amounts 
    # to ~1 block on disk
    for chunkidx in range((datalen//64)):
        for idx in range(64):
            
            if ((chunkidx * 64 + idx) % 64 == 0):
                print(f"{(chunkidx + 1)} chunk written ({(chunkidx * 64)}/{datalen} bytes)")

            # Get four random bytes to make up one pixel
            rgb = rng.read(4)
            out.write(rgb)
    
    # Add some extra zeros
    while len(out.getbuffer()) < 5120:
        out.write(b'\x00\x00')

    return True

