#!/usr/bin/env python3
import argparse
from typing import List
from typing import Optional

from PIL import Image


# Get command-line arguments and return them as a list of strings
def get_args() -> List[str]:
    parser = argparse.ArgumentParser(description="Generate random images")

    # Positional arguments
    parser.add_argument('outfile', type=argparse.FileType('wb'),
                        help='The output image filename to write (will be overwritten)')

    cmd_args = parser.parse_args()
    args: List[Optional[str]] = [cmd_args.outfile]  # type: ignore
    return cast(List[str], args)


# Generates a random GIF file into `out`, which should already be opened in "binary"
# mode with its buffer interface (`bytes`)
def generate_random_gif(
        width: int, height: int, frames: int,
        rng: BinaryIO
    ) -> None:
    im = Image.new('RGBA', size=(width, height))
    imgdata = list(im.getdata())
    for _ in range(frames):
        for i in range(len(imgdata)):
            r = rng.read(1)[0] & 255
            g = rng.read(1)[0] & 255
            b = rng.read(1)[0] & 255
            a = rng.read(1)[0] & 255
            if len(imgdata[i]) == 4:
                imgdata[i] = tuple([r, g, b, a])
            else:
                imgdata[i] = tuple([r, g, b])

        im.putdata(imgdata)
        im.save(rng, format='gif')


if __name__ == '__main__':
    outfile: BinaryIO = open(*get_args(), 'wb')
    generate_random_gif(640, 480, 10, outfile)