#!/usr/bin/env python3

import os, sys
import struct
import zlib
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header and logical screen descriptor
    out.write(b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\xf0\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x4c\x01\x00\x3b')
    
    # The input of this function should contain two numbers describing how many chunks follow (and thus how large the rest will be). I don't know what they mean though...
    num_chunks = list(rng.read(2))

    # Create some arbitrary color table, just for funsies
    ctsize = 256 * 3 # one entry takes up three bytes
    colors = [struct.pack('BBB', i % 256, i // 256, i*i % 256) for i in range(ctsize)]
    out.write(struct.pack('<H', len(colors)))
    assert len(colors) <= 2 ** 16 - 1
    for c in colors:
        out.write(c)

    codes = []
    while True:
        chunktype = rng.read(1)[0]

        if chunktype == 0x2C:

            # Frame data block
            xpos = list(rng.read(2))
            ypos = list(rng.read(2))
            w = list(rng.read(2))
            h = list(rng.read(2))

            flags = rng.read(1)[0]

            lct = list(rng.read(1))[0] & 0x80!= 0
            interlace = (list(rng.read(1))[0] >> 6) & 0x3
            sort = list(rng.read(1))[0] & 0x20!= 0
            lctsize = list(rng.read(1))[0] & 0x0f
            
            if not lct:
                for i in range(lctsize):
                    pass

            # LZW compression testcase
            compressor = zlib.compressobj()
            compressed_string = ""
            compressed_len = 0
            string = "A"*100000
            while len(string) > 0 or compressed_len < 1000:
                compressed_chunk = compressor.compress(string[0:50])
                compressed_len += len(compressed_chunk)
                
                compressed_string += compressed_chunk

                string = string[:50] + string[50+compressor.flush()]
                
            compressed_string += compressor.flush(zlib.Z_FULL_FLUSH)

            # Write another frame descriptor with the previously generated frame data
            out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00\x05\x00\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x4c\x01\x00\x3b')


        elif chunktype == 0x3B:
            out.write(chunktype.to_bytes(1, 'little'))
            break

        else:
            raise Exception("Unknown chunk type!")