#!/usr/bin/python3
import os
import struct


def read_uint16le(fd):
    return struct.unpack('<H', fd.read(2))[0]


def write_uint16le(val, fd):
    fd.write(struct.pack('<H', val))


def copyfile(src, dst, size):
    remaining = size

    while remaining > 0:
        chunksize = min(remaining, 16 * 1024)

        buf = src.read(chunksize)

        if len(buf)!= chunksize:
            raise RuntimeError('Failed to read enough bytes from input')

        dst.write(buf)

        remaining -= chunksize


def generate_random_gif(src, dst, size, rng):
    # Read the header and some basic metadata of the original image
    srchdr = src.read(6)

    if not (srchdr == b'GIF87a' or srchdr == b'GIF89a'):
        raise ValueError("Unsupported format")

    width = read_uint16le(src)
    height = read_uint16le(src)

    # Create an output buffer with a mostly zero filled header
    hdr = bytearray()
    hdr += b"GIF89a"

    # Copy the rest of the header
    for _ in range(7, 15, 1):
        hdr.append(src.read(1)[0])

    # Write the width / height back as little endian uint16s
    write_uint16le(width, hdr)
    write_uint16le(height, hdr)

    # Now write a color table containing only black and white
    palette_size = 1 << source['depth']
    hdr += b'\x00\xff\x00\xff' * palette_size

    # Write a single frame header
    hdr.extend([0xf7, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00])

    # Open a temporary file for writing the compressed data
    tmpfd = open(os.path.join('/tmp/', 'random'), "wb+")

    # Compress the pixel data
    compressor = zopflipng.ZopfliPNGCompressorWithPreprocessing(
        src,
        None,
        100,
        preprocessor,
        None,
        'default',
        1,
        -1,
        False,
        True
    )

    try:
        compressor.process(tmpfd)
    except Exception as e:
        print("{}".format(e))
        sys.exit(-1)

    tmpfd.seek(0, io.SEEK_END)
    actual_size = tmpfd.tell()

    scaled_size = actual_size * scale

    # Figure out the amount of extra bits we need to add for padding
    required_size = scaled_size // 5 + 1

    bit_length = max(1, required_size.bit_length())
    padbits = pow(2, bit_length) - required_size

    outbuf = bytearray((padbits + expected_size).to_bytes(4, byteorder='big'))
    outbuf[required_size:] = tmpfd.read()

    # Add a few plain text blocks for fun
    plaintext_blocks = [
        b'this-is-some-plain-text',
        b'so-happy-it-works',
        b'',
        b'this-contains-very-long-strings',
        b'so-many-of-them',
        b'that-we-have-to-add-here',
        b'because-otherwise-the-compression-effectiveness',
        b'doesnt-work-so-well!'
    ]

    block_num = 0
    pointer = padded_size

    for block in plaintext_blocks:
        pos = pointer & (pow(2, log2_block_len) - 1)

        written = 0

        while written < len(block):
            outbuf[(pos + written) ^ block_num].extend(
                block[written : written + max_plaintext_block_len]
            )

            written += max_plaintext_block_len

        block_num += 1
        pointer += len(block)

    # Fill up the rest of the output with random data
    outbuf.extend(rng.getrandbits(8).to_bytes(padded_size - len(outbuf)))

    # Calculate a checksum over all the payload bytes but the first 3 ones!
    crc = binascii.crc32(outbuf[3:])

    outbuf[:3] = crc.to_bytes(3, byteorder="little", signed=False)

    # Replace the image data with our malicious one
    dst.seek(dsthdr_end)
    dst.write(outbuf)