#!/usr/bin/env python3
"""Generates a random GIF file"""

import argparse
import os
import sys

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--output', '-o', default=sys.stdout, type=argparse.FileType('wb'))

    args = parser.parse_args()

    rng = open('/dev/urandom', 'rb')

    # Write 100 GIF headers to verify that it's not just generating invalid files...
    for _ in range(10):
        print("generating {}/{}".format(_, 10))

        generate_random_gif(
                rng, 
                args.output)

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    """Generate a single random Gif image
    
    Keyword arguments:
    rng -- A file-like object returning randomness (e.g /dev/urandom)
    out -- The output file to write the new image onto
    """
    header = b'GIF89a'
    w = struct.unpack('<H', rng.read(2))[0]
    h = struct.unpack('<H', rng.read(2))[0]
    fscreen = bytes([0b1000_0000 |
                    ((w >> 12) & 0b1111) |
                    0b1110_0_0])
    fx = [0b0001_0_00,
          0b0010_0_00,
          0b0100_0_00][rng.read(1)[0]]
    fy = [0b0_000_100,
          0b0_000_010,
          0b0_000_001][rng.read(1)[0]]
    fbkgd = [0b0_000_0000,
             0b0_000_0001][rng.read(1)[0]]
    fcolor = [0b0000_0000,
              0b0000_0010,
              0b0000_1000,
              0b0000_1010,
              0b0010_0000,
              0b0010_0010,
              0b0010_1000,
              0b0010_1010][rng.read(1)[0]]
    fratio = [0b0000_0000,
              0b0000_0100,
              0b0000_0010,
              0b0000_0001,
              0b0001_0000,
              0b0001_0100,
              0b0001_0010,
              0b0001_0001][rng.read(1)[0]]
    fsrct = [0b0000, 
             0b0001,
             0b0010,
             0b0011,
             ][rng.read(1)[0]]
    fgc = list(map(lambda x : 0x7 << (x*5), range(6)))[rng.read(1)[0]]
    fgcr = [0b00_0_0000, 
            *[0b01 << (i*3) for i in range(4)],
            *[0b00 << (i*3) for i in range(4)]][rng.read(1)[0]]
    fbgc = list(map(lambda x : 0x7 << (x*5), range(6)))[rng.read(1)[0]]
    fbgcr = [0b00_0_0000, 
             *[0b01 << (i*3) for i in range(4)],
             *[0b00 << (i*3) for i in range(4)]][rng.read(1)[0]]
    ctl = bytes([fsrct |
                 fbkgd | 
                 fy   | 
                 fx   ])
    color = bytes([(fcolor & 0b1000) >> 3,
                   (fcolor & 0b0100) >> 2,
                   (fcolor & 0b0010) >> 1,
                   (fcolor & 0b0001)])
    ratiopos = bytes([fratio & 0b0011_1111,
                      fratio >> 6])
    gctpos = bytes([fgcr & 0b1111_1000,
                    (fgcr & 0b0000_0111) * 0b1111])
    bgctpos = bytes([fbgcr & 0b1111_1000,
                     (fbgcr & 0b0000_0111) * 0b1111])
    netsize = bytes([h % 0b1_0000 - 1, 
                     w % 0b1_0000 - 1])
    gctsize = bytes([fscreen[0], 
                     h % 0b1000 - 1, 
                     w % 0b1000 - 1])
    lctlen = bytes([0b0111_1111])
    bgdlens = bytes([netsize[0], 
                     netsize[1]])
    colormask = [0 if i == 1 else 1<<i for i in range(7,-1,-1)][rng.read(1)[0]]
    colormask = [(colormask&(1<<(2**i)))!= 0 for i in range(7,-1,-1)]
    lctlen += bytes([sum([[b * 2 ** j for j in range(7, -1, -1)]
                          for b in colormask].count(True))])
    rgb = lambda c : bytes([(c&0b1111_00)>>4,
                            c&0b1111])
    lct = bytes().join([rgb((gc>>5)*36 + gc%0b1_0000)
                        for gc in fgc 
                        if colormask[(gc>>5)]])
    bgcolor = rgb(fbgc)
    ext = bytes([
                 ord(b'%'),
                 0b0_0000_100,
                 0b0000_1000,
                 0b0000_0100,
                 0b00_01_0_0,
                 ord(b'C'),
                 0b0000_0000,
                 0b0_0000_000,
                 0b0000_0000,
                 0b0000_0000, 
                 0b0000_0000,
                 0b0000_0000,
                 ])
    
    out.write(ext +
              bgcolor +
              ext +
              bgdlens +      #[1, 0] +
              ext +           #[2, 0]
              bgdlens +       #[3, 0] +
              ext +           #[4, 0]
              bgctpos +       
              ext +            #[5, 0]
              ctloffset +    
              ext +            #[6, 0]
              lctpos +        
              ext +            #[7, 0]
              lctlen +         
              ext +            #[8, 0]
              gctpos +        
              ext +            #[9, 0]
              gctsize +       #[10, 0]
              ext +            #[11, 0]
              ctl +               #[12, 0]
              ext +            #[13, 0]
              netsize +
              header +
              ctl +
              header+
              width +
              height +
              fscreen +
              palette +
              lct
              )




    print("{} {} {}".format(w, h, sum([fgcr&(1<<(2*i)) > 0 for i in range(12)]))


