#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate a random GIF given an input stream of bytes to use for randomness and output stream to write to.
def generate_gif(input_stream: BinaryIO, output_stream: BinaryIO) -> None:
    # Magic number.
    output_stream.write(b'GIF89a')

    # Width and height.
    width = 1024 + (next(input_stream) % 512) * 4
    height = 768 + (next(input_stream) % 128) * 8
    struct.pack('<HH', width - 1, height - 1).tofile(output_stream)

    # Backgroud color index.
    next(input_stream)
    struct.pack('B', next(input_stream)).tofile(output_stream)

    # Color table.
    colors = [struct.unpack('BBB', ''.join([chr(c) for c in range(i*3, i*3+3)])) for i in range(2, 16)]
    [c.extend(next(input_stream) % 16 for _ in range(3)) for c in colors]
    output_stream.write(bytes((c[0], c[1], c[2]) for c in colors))

    # Image descriptor.
    struct.pack('BB', 0xFF, 0xF1).tofile(output_stream)

    # Local color table.
    local_colors = list(struct.unpack('BBBB', ''.join([chr(n//100) for n in range(0, 100, 25)])))
    local_colors.append(next(input_stream) % 4)
    output_stream.write(bytes(local_colors))

    # Pixel data.
    row_size = (width - 1) // 8 + 1
    pixel_data = []
    for y in range(height):
        struct.pack('BB', 0, next(input_stream) % 2**row_size).tofile(output_stream)
        [pixel_data.append(next(input_stream) % 8) for _ in range(y % 3)]
