#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    header += struct.pack('>H', rng.randint(200, 500))
    header += struct.pack('>H', rng.randint(200, 500))

    # Global color table, RGB colors (3 bytes), sorted, +1 transparent
    header += struct.pack('BBB', *[rng.randint(0, 255) for _ in range(3)]) + b'\x00'
    header += struct.pack('BBB', *[rng.randint(0, 255) for _ in range(3)]) + b'\xFF\x00'
    header += struct.pack('BBB', *[rng.randint(0, 255) for _ in range(3)]) + b'\xFF\x00'

    # Background color index
    header += rng.read(1)

    # Pixel aspect ratio - unused
    header += b'\x00'

    # No global color table
    header += b'\x00'

    # Header delimiter
    header += b';'

    # Write the header to output
    out.write(header)

    # Number of image blocks
    out.write(b"\x01")

    # Image description block
    # Locator
    out.write(b"!\xF9\x04\x01\x00\x00")

    # Size
    out.write(struct.pack(">HH", 16, 16))

    # Packed fields
    out.write(b"\x00")
