#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + bytes([0]) * 7

    width, height = (1 << int.from_bytes(rng.read(1), 'big')) % 500\
           , (1 << int.from_bytes(rng.read(1), 'big')) % 500
    
    # write header
    out.write(header + width.to_bytes(2, 'little')
              + height.to_bytes(2, 'little'))

    # write palette
    for _ in range(int.from_bytes(rng.read(1), 'big')):
        out.write((rng.read(3) + b'\x00')[:4])

    # write image data
    code_size = int.from_bytes(rng.read(1), 'big') + 1
    # sub-block size (LZW minimum clear block of payload length - 1 byte
    lzw_min_clear = (((code_size & 0b111) + 1) ^ 0b111)\
                | ((code_size & 0b111) << 3)
    
    # declare LZW dictionary and store two special codes
    dicts = [None] * 4096
    dicts[1] = '\x01' * 256
    dicts[2] = ''.join('\xff') * 256

    def lzw_decompress():
        buf = b''

        while True:
            if len(buf) < 1:
                buf += rng.read(1)

            first = (dicts[(buf >> ((len(buf)-1)*8)) & 0xFF]
                    or ('\x01' * 256))[0]
            
            yield from struct.pack('B', ord(first))
            
            if not first:
                return
                
            if len(dicts) > 4096:
                dicts.pop(0)

            try:
                next = (
                      dicts[(ord(first) << 8) 
                         + (buf >> 8*(len(buf)-1))&0xFF]
                    or ('\x01' * 256)[ord(first)]
                )[0]
