#!/usr/bin/env python3

import struct
from binascii import hexlify, unhexlify
from datetime import date
from hashlib import sha256
from os import urandom

from bitcoin.core import bip32, script
from coincurve import PrivateKey as ECPrivateKey
from coincurve.ecdsa import Signature as ECSignature
from secp256k1_zkp import Secp256k1PublicKey

from pytest import raises

from hwwclient import (
    CTAP2Status, CTAPDevice, ERR, ParameterException,
    parse_pubkey_response, sign_transaction,
)
from tests.mocks import (
    PublicKeyMock, SignatureMock, SimulatedCTAPDevice,
    create_keypath_reply, create_signature, create_version,
)

# Test keypath
TEST_BIP44_KEYPATH = [0x8000002C, 0x80000A94] + list(range(0, 5))

# Test transaction data
TX_DATA = {
    'version': 2,
    'num_inputs': 1,
    'prev_txid': bytes.fromhex('e7f96c6fbde82162bab8dd13687bc2cb1df187c542f1e0694d28840a2cceffc7'),
    'prev_index': 0,
    'amount': 100000000,
    'sequence': 4294967295,
    'num_outputs': 2,
}

def test_parse_pubkey_response():
    assert parse_pubkey_response([]) == ([], [])
    assert parse_pubkey_response(['ver', 'pub']) == ('ver', 'pub')
    assert parse_pubkey_response(['pub', 'ver', 'extra']) == ('pub', 'ver')

