#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF image in binary form to `out`, using `/dev/urandom`.
# The width and height are randomly generated such that they are multiples of 2
# between minwidth <= w < maxwidth and minheight <= h < maxheight (inclusive).
# The frame count is randomly generated from 1 to nframes inclusive.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, minwidth=64,
        maxwidth=512, minheight=64, maxheight=512, nframes=10):

    # construct headers
    header = b'GIF87a' + struct.pack('<HHHBB',
        minwidth + rng.read(1)[0] % maxwidth - minwidth,
        minheight + rng.read(1)[0] % maxheight - minheight,
        0xFF, 0b10100000 | (rng.read(1)[0] % 0b100), 0)
    out.write(header)

    for i in range(nframes):
        # construct graphic control extension
        gce = (struct.pack('B', 0x21) + b'Ec' + b'\x00\x00' +
            struct.pack('BBB', 0, 0, 0))

        if i == 0:
            # first frame: clear screen before drawing
            gce += b'\x00\x04'
        else:
            # subsequent frames: move existing blocks downwards
            gce += struct.pack('!H', minheight + rng.read(1)[0] % maxheight - minheight)

        # output graphic control extension and image descriptor
        out.write(gce + b',' + (struct.pack('B', 0x2C) +
                struct.pack('<HHHH', 0, 0,
                    minwidth + rng.read(1)[0] % maxwidth - minwidth,
                    minheight + rng.read(1)[0] % max