#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    width = rng.randint(100, 400)
    height = rng.randint(100, 400)

    # logical screen descriptor
    header += struct.pack('<HHBBB',
                width, height,
                0x1f & rng.read(1)[0],
                0b10 << 4 | 0b10 << 2 | 0b10 << 0, # global_color_table_flag=true, background_color_index=0, aspect_ratio=6
                header += struct.pack('>H', rng.randint(200, 500))

    # color table (RGB triplets)
    ctsize = rng.randint(1, 7) * 3
    for _ in range(ctsize):
        header += bytes([rng.randint(0, 255)])

    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
