"""
Generate a random GIF from /dev/urandom.
"""

import struct
import sys
from argparse import ArgumentParser
from typing import BinaryIO


def parse_args() -> str:
    parser = ArgumentParser("generate-random-gif")

    # Generate a default name if none is provided
    if len(sys.argv) == 1:
        return "random.gif"

    # Allow passing an argument to specify output filename.
    parser.add_argument("-o", "--output", type=str, help="set destination for generated image (default: %(default)s)")
    ns = parser.parse_args()

    if not ns.output:
        return "random.gif"

    return ns.output


def generate_gif(out: BinaryIO):
    magic = b'GIF87a\x01\x00\x01\x00\x00\xff\xff\xff,\x00\x00\x00\x21F94\x04\x01\x00\x00\x00!\xf9\x04\x01\x03\x000;\xa0\x00\x00\x00\x00,'
    header = b'\x4e\x45\x54\x53\x43\x41\x50\x45\x32\x2e\x30\x03\x01\x00\x00\x00\x00,\x00,\x00,\x00\x01\x00,\x01\x00\x01\x00\x00\x02\x02D\x88\x00;'

    out.write(magic + header)
    while True:
        data = sys.stdin.buffer.read(1024 * 16)
        out.write(data)


