#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (by default /dev/urandom).
def generate_random_gif(
        rng:        BinaryIO=None,
        width:      int    = 1024,
        height:     int    = 768,
        colordepth: int    = 8,
        bgcolor:    int    = None,
        outfile:    str    = 'output.gif',
    ) -> None:

    # Set up the base colors that will be used to build our palette.
    if not bgcolor or bgcolor < 0 or bgcolor > 255**3:
        bgcolor = 0xFFFFFF ^ ((width*height)%(255**3))

    blackcode = bytes([bgcolor & (255**2)])
    bgcolor &= ~((255**2)-1)
    redcode   = b'\xFF' * (colordepth//8)
    greencode = b'\xFF' * (colordepth//8)
    bluecode  = b'\xFF' * (colordepth//8)

    # Figure out how many bits of color depth we have available.
    shiftwidth = (24 - colordepth) // 3

    with open(outfile, "w+b") as f:
        print("Writing the header")
        f.write(b"GIF89a\x01\x00\x01\x00\x00\x00\x00\x00") # Header
        print("Writing the image")
        f.write(b"\x2C\x00\x00\x00\x00\x00\x00\x00\xFF")   # Image descriptor
        f.write(b"!\xF9\x04\x01\x00\x00\x00,\x00\x00\x00") # Graphics control extension
        f.write(b"\x00\x01\x00\x21\xFE\x09D\x0B\xCF\x00") # Application Extension
        f.write(b"\x0BNETSCAPE2.0\x03\x01")                  # Application Data
        f.write(b"\x00\x00\x00!")                           # Trailer
        print("\nDone.")