"""
Generate a random GIF from /dev/urandom.
"""

import struct
import sys
from argparse import ArgumentParser
from typing import BinaryIO


def parse_args() -> str:
    parser = ArgumentParser("generate-random-gif")

    # Generate a default name if none is provided
    if len(sys.argv) == 1:
        return "random.gif"

    # Allow passing an argument to specify output filename.
    parser.add_argument("-o", "--output", type=str, help="set destination for generated image (default: %(default)s)")
    ns = parser.parse_args()

    if not ns.output:
        raise RuntimeError('No output specified.')
    
    return ns.output


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = bytes([0x47, 0x49, 0x46, 0x38, 0x39, 0x61]) 
    datalen = int.from_bytes(
        rng.read(3), byteorder='big', signed=False)
    print(f"Datalen: {datalen}")

    # We read one less than we need because of the way we're doing
    # packing below. See the note in the GIF spec about this:
    if datalen > 2 ** 20 - 1:
        raise ValueError(f"The length of data must be between 1 and 2 ^ 20 - 1 ({datalen})")
    data = rng.read(datalen)
    print(f"Data: {data}")
    trailer = bytes([0x00])
