#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun 25 10:42:19 2019

@author: peter
"""

import subprocess
from Bio import SeqIO, AlignIO
from os.path import exists

def get_sequence(flanking, fasta):
    """ Get sequence from flanking region of a gene in the given FASTA file.
    
    Parameters
    ----------
        flanking : list of length 2
            The start and end positions for the flanking region to extract.
            
        fasta : str
            Full path to FASTA file containing the original genome assembly.
        
        Returns
        -------
        seq : SeqRecord object
            A record of the flanking region (as SeqRecord).
    """
    # Check if input file exists
    if not exists(fasta):
        print("FATAL ERROR - File {} does not exist.".format(fasta))
        exit()
    else:
        pass
     
    # Extract a portion of the sequence from the original genome using Seqkit.
    cmd = 'seqkit subseq -r {}:{} -o output.fa {}'.format(
                                flanking[0], flanking[1] + 1, fasta)
    try:
        proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        _stdout, _stderr = proc.communicate('y')
        proc.wait()
    except Exception as e:
        print('Error encountered while using SeqKit:\n', e)
        exit()
    
    # Import the extracted subsequence into a SeqRecord object.
    curr_file = "output.fa"
    seq = next(SeqIO.parse(curr_file, "fasta"))
