#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO):
    if (size & 0xffff) != size:
        raise ValueError('Size must be even')
    
    # Write header
    out.write(b'GIF87a')
    out.write(struct.pack('>H', size))
    
    # Start with an initial section
    out.write(b'\x21\xf9\x04\x04\xff\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x4c\x01\x00;')
    
    while size > 0:
        next_chunk = rng.read(min(size, 256))
        
        # If there is no more input
        if len(next_chunk) == 0:
            break
            
