#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def gen_gif(out: BinaryIO, rng):
    # Generate 1024 bytes of random data for LZW dictionary
    dict = bytearray()
    for i in range(1024):
        dict[i] ^= ord(rng.read(1)) & 0xff
    
    # Set up GIF header
    out.write("GIF89a".encode())                   # Magic
    out.write(struct.pack("<HH", 640, 480))        # Canvas size
    out.write(b"\xFF\xFF")                        # No global colour map
    out.write(b"!\xF9\x04\x1\x0\x1\x0\x0\x2c\x0,\x0\x0") # Graphics control block
    out.write((dict + b'\x0').ljust(256, '\x0'))  # Image descriptor
    out.write(b":")                               # Static image separator
    
    out.flush()
    while True:
        src = dict + b'\x0'
        
        # Read next byte from random source, and append it to the end of the list
        v = ord(rng.read(1)) & 0xff
        src += chr(v).encode()
