#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given number of loops and
# frames. Each frame is generated by picking a random color and drawing it on
# a 256x16 background, creating a 1-pixel-wide border. The image uses this palette:
#
#   #0000FF  #00FF00  #FFFF00  #FF0000  #804000
def generate(out: BinaryIO, num_loops: int, num_frames: int):
    out.write(b'GIF89a')

    def write_u16le(n):
        out.write(struct.pack('<H', n))

    def write_color(r, g, b):
        out.write(bytes([r, g, b]))

    write_u16le(256)
    for r in range(256 // 16):
        for g in range(16):
            for b in range(16):
                write_color(r * 16 + 0, g * 16 + 0, b * 16 + 0)
                write_color(r * 16 + 15, g * 16 + 0, b * 16 + 0)
                write_color(r * 16 + 0, g * 16 + 15, b * 16 + 0)
                write_color(r * 16 + 15, g * 16 + 15, b * 16 + 0)
                write_color(r * 16 + 0, g * 16 + 0, b * 16 + 15)
                write_color(r * 16 + 15, g * 16 + 0, b * 16 + 15)
                write_color(r * 16 + 0, g * 16 + 15, b * 16 + 15)
                write_color(r * 16 + 15, g * 16 + 15, b * 16 + 15)

    write_u16le(0)
    write_color(0, 0, 0)

    out.write(b'\xF9\x04\x14\x8E\xCE\x00\x00\x00\x00')

    write_u16le(256)
    write_u16le(16)

    out.write(b'\x00\x00\x00\xFF\xFF\xFF\xFF\xFF\xFF')
    out.write(b'\x21\xF9\x04\x04\x00\x00\x00\x00\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3B')

    for _ in range(num_loops):
        last_frame = -1000000
        while True:
            delay = 4 + (last_frame % 9)
            if last_frame == 0 or delay > 10:
                delay = 10
