"""
Generate a random GIF from /dev/urandom.
"""

import struct
import sys
from argparse import ArgumentParser
from typing import BinaryIO


def parse_args() -> str:
    parser = ArgumentParser("generate-random-gif")

    # Generate a default name if none is provided
    if len(sys.argv) == 1:
        return "random.gif"

    return sys.argv[1]


# Define a binary file type to make sure we correctly read the data
BinaryFileType = BinaryIO


def main():
    outfile: str = parse_args()

    try:
        with open("/dev/urandom", "rb") as fd:
            # Read the first two bytes of the urandom stream and use it to determine
            # the number of iterations for the loop below. This way, we can be sure to not
            # create an infinite loop here.
            num_iterations: int = struct.unpack("<H", fd.read(2))[0] % 5000
            
            # Loop over the remaining bytes (which will be between 1 and 4998 in size).
            # Since there are no guarantees on the distribution of values between these bytes,
            # We'll purposefully ignore the first byte each iteration to avoid creating an
            # infinite loop here.
            for i in range(num_iterations):
                _ = struct.unpack("<B", fd.read(1))[0]
        
                # Write out the rest of the bytes as a single byte
