#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
    out: BinaryIO,
    rng: BinaryIO,
):
  print('Writing header')
  out.write(b'GIF89a') # Magic cookie
  width = int(struct.unpack('<L', rng.read(4))[0]) & 0xffff + 1
  height = int(struct.unpack('<L', rng.read(4))[0]) & 0xffff + 1
  out.write(bytes([width % 256, width // 256])) # Logical screen size
  out.write(bytes([height % 256, height // 256]))
  color_table_size = struct.unpack('B', rng.read(1))[0] & 7
  print('Width: {} px'.format(width))
  print('Height: {} px'.format(height))
  print('Color table size: {} ({} colors)'.format(color_table_size + 1, 1 << color_table_size))
  
  global_colors = [struct.unpack('BBB', rng.read(3)) for i in range(1 << color_table_size)]
  print('Global colors:')
  for col in global_colors:
    print('\t({}, {}, {})'.format(*col))
  out.write(bytes([0xf8, 0, 0, 0, 0, 0])) # Graphic control extension with unused disposal method and transparent color index
  out.write(bytes([0xfe, 0])) # Transparency extension with no transparency
  out.write(b'\x2c\xff\xff\xff\xff\xff\xff\xff') # Image descriptor
  print('Transparent color index: {}'.format((struct.unpack('B', rng.read(1))[0] >> 4) & 7))
  out.write(rng.read(7)) # Local color table
  out.write(b'\xa0\