#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Mar 21 15:07:49 2021

@author: fcosta
"""
import numpy as np
from scipy.spatial import distance_matrix
from scipy.signal import find_peaks

def smooth(y, box_pts):
    box = np.ones(box_pts)/box_pts
    y_smooth = np.convolve(y, box, mode='same')
    return y_smooth

def getSPL(x,fs=22050):
    
    # calculate envelope of signal
    xe = abs(np.fft.rfft(x))**2

    # calculate peak location of envelope
    peaks, _ = find_peaks(xe)

    # Window size is 46ms (as per ISO 3382)
    window_size = int(round(0.046 * fs))
    step = 50
    
    SPL = []

    for p in range(len(peaks)):
        start = peaks[p] - window_size if peaks[p] >= window_size else 0
        
        end = peaks[p] + window_size
        
        b = xe[start : end]
            
        avg_b = sum(b) / len(b)
        
        # This part calculates the average envelope over 50 ms and applies a
        # weighting factor to it. The lower the value, the louder the sound.
        c = smooth(b, window_size // step)
        d = smooth(c, window_size // step)
        
        e = sum(d) / len