#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    N = 10 # Number of frames
    M = 50 # Number of pixels per frame
    w = h = 2 # Other dimensions are not needed
    size = (w * h * 3 + 1) * N + 6 # Size of data section
    magic = "GIF89a"
    header = bytearray()
    header.extend([x for x in magic])
    header.append((size >> 0) & 0xFF)
    header.append((size >> 8) & 0xFF)
    fp = bytearray()
    fp.append(0xF4)                     # Header length
    fp.append(0)                        # Unused fields
    fp.append(0)                        # Background color index
    fp.append(0)                        # Pixel aspect ratio
    trailer = bytearray("tRNS\x00\x00\x03\xf4", "ascii")
    out.write(header)
    out.write(fp)
    for i in range(N):
        px = [rng.read(1)[0] % 255 for _ in range(M)]
        if i == 0:
            px[0] = 0 # First pixel is black!
        else:
            px[0] = 255 - px