#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate a random GIF given an input stream of bytes to use for randomness and output stream to write to.
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Create the header
    out.write("GIF89a".encode())  # Magic number
    out.write("\x01\x00".encode())  # Screen descriptor (width=height=1)
    out.write(b"\xff")  # Background color index
    out.write("\x00".encode())  # Pixel aspect ratio

    # Write frames until we get a terminator byte
    while True:
        image_descriptor = b"\x2c" + struct.pack("<HHBB", 0, 0, 1, 1)
        out.write(image_descriptor)
        out.write(bytearray([rng.read(1)[0] & 7 | 0xe0]))  # Image data is full-color indices
        out.write(b"\x00");  # No local color table
        out.write("\x00".encode())  # Delay between frames in hundredths of seconds
        out.write