#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + bytes([0]) * 7

    width, height = (1 << int.from_bytes(rng.read(1), 'big')) % 500\
                   for _ in range(2)

    global_palette = [tuple(int.from_bytes(rng.read(1), 'big')\
                            for _ in range(3))\
                      for _ in range(8)]

    global_palette = [(rgb[0] * 4 // 63, rgb[1] * 4 // 63, rgb[2] * 4 // 63)\
                      for rgb in global_palette]

    try:
        while True:
            # Write the "Graphic Control Extension" (transparent color table)
            out.write(struct.pack('<BBB', 0xF9, 4, 1))
            out.write(struct.pack('<HHH', 0, 0, 0))

            # Write the "graphic control extension" (end of stream)
            out.write(struct.pack('<BBB', 0xFF, 0, 0x01))

            # Write the "image descriptor" and image data
            if not width or not height:
                break
            iw, ih = width, height
            width, height = ((iw - 1) >> 3, (ih - 1) >> 3) if iw >= 8 else (0, 0)
            out.write(struct.pack('<BBHHBB', 0x2C, 0, 0, iw, ih,\
                                  7 << 7 | 3 if iw > 0 else 3 << 5))
            out.write(struct.pack('<BBBBB', 0, *global_palette[0]))
            if iw == 0:
                continue
            out.write(struct.pack('<BBBBB', 0, *global_palette[0], 1))
            out.write(struct.pack('