#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(struct.unpack('<B', rng.read(1))[0], 'big') % (2**8 - 1) + 4 # image data length
    global_color_table = [int.from_bytes(rng.read(1), 'big') for _ in range(3)] * (2 ** 3 - 1)

    out.write(header)
    out.write(struct.pack('<H', rng.tell() + len(global_color_table)))
    out.write(b'\x00\x00')
    out.write(bytearray([int(c / 5) for c in global_color_table]))
    out.write(b'\xFF\xFF')
    out.write(b'!\xF9\x04') # repeat infinite times
    out.write(