#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Jul 26 17:55:09 2020

@author: thorius
"""
from .. import get_savepath
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

def plot_percentile(df):
    fig, (ax1, ax2) = plt.subplots(ncols=2)
    
    for s in range(len(slope)):
        print('Statistic %s'%s)
        
        x = df[df['slope'] == slope[s]].copy()
        
        x = x[(x['min_p'] > min_p)]
        
        #a = (x>0).mean()
        #b = x[x<0].abs().mean()
        #print('%s - %f%% >=0, %f%% <0'%(slope[s], a*100, b*100))
        
        a = x.groupby(['slope', 'laterality'])['signal'].sum().unstack()
        b = x.groupby(['slope', 'laterality'])['density'].mean().unstack()
        
        def _sel(x):
            if type(x) is str: return True
            else: return False
            
        a = a[_sel]
        b = b[_sel]
        
        a.columns = [int(i) for i in a.columns]
        b.columns = [int(i) for i in b.columns]
        
        a = a.fillna(0)
        b = b.fillna(0)
        
        a.columns.name='laterality'
        b.columns.name='laterality'
        
        a = a / a.sum() * 100.
        b = b.applymap(lambda x: '%d'%round(x)).to_frame().rename({0:'density'}, axis=1)
        
        a = a.reset_index().merge(b, how='left')
        
        a = a.sort_values(['slope', 'laterality'], ascending=[True, False])
        
        a['laterality_code'] = [1]*len(a) + [-1]*len(a)
        
        a = a.set_index([pd.Categorical(list(range(-1,2))), pd.Categorical(a['laterality'])])
        
        a = a.drop(['slope', 'laterality'], axis=1)
        
        a = a.reindex(index=pd.MultiIndex.from_product([[-1, 0, 1], ['Left', 'Right']]), columns=a.columns)
        
        a = a.T
        
        
        ax1.barh(a.index, a.iloc[:, 1::2].values.flatten(), color='blue')
        ax1.barh(a.index, a.iloc[:, ::2].values.flatten(), left=a.iloc[:, 1::2].values.flatten(), color='orange')
        
        a.loc[a['laterality']=='Left']['laterality'] = 'L'
        a.loc[a['laterality']=='Right']['laterality'] = 'R'
        
        handles, labels = ax2.pie(a.loc[a['laterality']=='L']['signal'].values)
        
        ax2.legend(handles, labels)
        
        #ax2.title('%s'%slope[s])
        
    #plt.show()
    
    file_name = 'statistics_for_patients_with_slopes.png' #+str(slope)
    file_name = get_savepath(__file__, file_name)
    plt.savefig(file_name, dpi=300)

def plot_survival(data):
    fig = plt.figure()
    ax = fig.add_subplot(111)
    
    colors = {'S': 'red','D': 'green'}
    
    data2 = data.copy()
    
    for i in range(len(data2)-1):
        if data2['state'].isnull()[i]:
            data2['state'][i]=data2['state'][i+1]
            data2['time'][i]=data2['time'][i+1]
            data2['ID'][i]=data2['ID'][i+1]
    data2 = data2.dropna()
    
    for i in range(len(data2)-1):
        if data2['state'].isnull()[i]:
            data2['state'][i]=data2['state'][i+1]
            data2['time'][i]=data2['time'][i+1]
            data2['ID'][i]=data2['ID'][i+1]
    data2 = data2.dropna()
    
    
    survival = []
    death = []
    for state in data2['state'].unique():
        s = data2[data2['state'] == state]
        survival.append(s.groupby('ID').first())
        death.append(s.groupby('ID').last())
        
    survival = pd.concat(survival)
    death = pd.concat(death)
    
    survival = survival.sort_values('time')
    death = death.sort_values('time')
    
    s = sorted(list(set(np.array(list(survival.year)+list(death.year)))));
    
    for y in s:
        tmp = (survival[survival.year==y]['ID']).astype(str)
        tmp = ','.join(tmp.tolist())
        death.ix[death.year==y, 'ID'] = tmp
    
    death = death[death['ID'] != death['ID'].shift(1)]
    
    survival['state'] = 'S'
    death['state'] = 'D'
    
    combined = death.append(survival)
    
    grouped = combined.groupby(['ID', 'state'])
    grouped_lengths = grouped.size()
    lengths = []
    for key, group in grouped_lengths:
        lengths += [(key,group)]*(group)
    
    lengths = pd.DataFrame(lengths, columns=['ID', 'state'])
    lengths = lengths.drop_duplicates().set_index('ID')
    
    lengths['start'] = None; lengths['end'] = None
    for ID in lengths.index:
        l = lengths[lengths.index==ID]
        lengths.loc[ID, 'start'] = l.groupby('state').first().loc[l.iloc[0][1]]
        lengths.loc[ID, 'end'] = l.groupby('state').last().loc[l.iloc[1][1]+1]
    
    length = []
    number = []
    for ID, row in lengths.iterrows():
        length += [(row['start'], row['end'] - row['start'])]
        number += [ID]
    length = pd.Series(length, index=number)
    
    colors = {k:colors[k] for k in list(sorted(set(combined['state'])))}
    
    for ID, group in lengths.groupby('ID'):
        sd = list(zip(*group['start'], *group['end']))
        ax.broken_barh((sd), (-1,-0.54), facecolors=colors[group['state'][1]], alpha=0.8)
        
    return length, lengths

