#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it so that if we want to corrupt this thing, we can do so in a way that is less obvious.
    
    datalen = struct.unpack('<I', rng.read(3))[0] % 2**16 + 1
    
    # Write header.
    out.write(b'GIF')
    out.write(b'\x38\x9B\x00')
    
    # Write a frame with no transparent color set.
    out.write(b'\xF9\x04\x05\x00\xFF\x00')
    # Write colour table.
    for _ in range(7):
        out.write(struct.pack('BBB', *struct.unpack('>I', rng.read(3))))
    # Write NETSCAPE2.0 app extension.
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00')
    # Write animation control block.
    out.write(b'\x00\x00\x00\x00\x04\x10')
    
    # Generate compressed image data.
    lzw_clear_code = b'\x00'
    lzw_end_of_info = b'\x01'
    lzw_minimum_code_size = b'\x02'
