"""
Generate a random GIF from /dev/urandom.
"""

import struct
import sys
from argparse import ArgumentParser
from typing import BinaryIO


def parse_args() -> str:
    parser = ArgumentParser("generate-random-gif")

    # Generate a default name if none is provided
    if len(sys.argv) == 1:
        return "random.gif"

    # Allow passing an argument to specify output filename.
    parser.add_argument("-o", "--output", type=str, help="set destination for generated image (default: %(default)s)")
    ns = parser.parse_args()

    if not ns.output:
        raise ValueError

    return ns.output


def main(infile: BinaryIO, outfile: BinaryIO):
    # The GIF header according to RFC3986.
    HEADER = b"\x47\x49\x46\x38\x37\x61"

    # Write the GIF header
    outfile.write(HEADER)

    # Read data from infile and write it unchanged to outfile.
    while True:
        chunk = infile.read(1024)
        if not