#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (by default /dev/urandom).
def generate_random_gif(
        rng:        BinaryIO=None,
        width:      int    = 1024,
        height:     int    = 768,
        colordepth: int    = 8,
        bgcolor:    int    = None,
        outfile:    str    = 'output.gif',
    ) -> None:

    # Set up the base colors that will be used to build our palette.
    if not bgcolor or bgcolor < 0 or bgcolor > 255**3:
        bgcolor = 0xFFFFFF ^ ((width*height)%(255**3))

    blackcode = bytes([bgcolor & (255**2)])
    whitecode = bytes([(bgcolor>>8)&255,(bgcolor>>16)&255])
    
    with open(outfile,'wb') as f:

        # The GIF header
        version = b"89a"
        screendescriptor = struct.pack('>HHBBB',
                width,
                height,
                0xF0 | colordepth,
                0,
                0,
        )
        
        # Write it all out!
        f.write(version)           # File type
        f.write(screendescriptor)  # Screen size and colors
        f.write(b"\x2C\x00\x00\x00\x00\x00\x00\x00\xFF")   # Image descriptor
        f.write(b"!\xF9\x04\x01\x00\x00\x00,\x00\x00\x00") # Graphics control extension
        f.write(b"\x00\x01\x00\x21\xFE\x09D\x0B\xCF\x00") # Application Extension
        f.write(b"\x0BNETSCAPE2.0\x03\x01")                  # Application Data
        f.write(b"\x00\x00\x00!")                           # Trailer
        print("\nDone.")