"""
Generate a random GIF from /dev/urandom.
"""

import struct
import sys
from argparse import ArgumentParser
from typing import BinaryIO


def parse_args() -> str:
    parser = ArgumentParser("generate-random-gif")

    # Generate a default name if none is provided
    if len(sys.argv) == 1:
        return "random.gif"

    # Allow passing an argument to specify output filename.
    parser.add_argument("--output", "-o", type=str, required=True)
    args = parser.parse_args()
    return args.output


def generate_header() -> bytes:
    width = height = 256
    color_depth = 8
    global_color_table = False
    loop_count = 0

    header = b""
    header += struct.pack("<ccccBBBBBHB", b"G", b"I", b"F", b"8",
                           version="79a", width=width, height=height,
                           color_depth=color_depth,
                           global_color_table=global_color_table,
                           background_color=0xFFFFFF, pixel_aspect_ratio=0,
                           frame_delay=43, loop_count=loop_count)
    return header


def write_pixel_data(file: BinaryIO):
    for _ in range(256 * 256 // 4):
        r = file.read(1)[0] % (2 ** 8)
        g = file.read(1)[0] % (2 ** 8)
        b = file.read(1)[0] % (2 ** 8)

        # NOTE: The spec says that the Image Descriptor should be used here, but PyCharm's complains about it not being defined...
        file.write(struct.pack("<BBBB", b, g, r, 0))


if