#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it
    # easier to test for false positives.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    
    # Write header and data
    assert datalen >= size 
    out.write(b'\x47\x49\x46\x38\x39\x61') # 'GIF89a' followed by application extension block (0x21ff)
    out.write((datalen - size).to_bytes(3, byteorder='big'))        # Data length
    out.write(b'\x00\x3B')       # Frame descriptor (0x0000 + background colour index)
    for _ in range(datalen):     # Image data
        out.write(rng.read(1))


