#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2),
            byteorder='big', signed=False)

    # 256 Frames for now
    frames = [struct.pack('>HBBBBB', x * 10, 0x04, 0x00, 0x00,
                          0xFF >> (x % 7), 0xFE)
              for x in range(datalen)]

    data = b"".join([b'\x2c\x00\x00\x00' + frame
                     for frame in frames])

    out.write(header + data)
