#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Nov  1 10:49:56 2017
@author: nsde
"""


###############################################################################
### Imports
import sys, os
sys.path.append(os.getcwd()) # Add path to root
from gp import Dataset, plot_corr, GPRegression
import numpy as np
from sklearn.preprocessing import MinMaxScaler
import matplotlib.pyplot as plt
plt.style.use('seaborn')

###############################################################################
### Prepare data for training and testing
X = np.array([[0], [1], [2], [3]])[:, None] * np.linspace(-1, 1) -.5
y = X**3 + np.random.normal(loc=0., scale=.3, size=(len(X),))
data = Dataset(X, y)

###############################################################################
### Train model using gradient descent
model = GPRegression(data, ninducing=10, lr=.1).fit()

###############################################################################
### Predict
preds, variances = model.predict(X)

###############################################################################
### Plot results
fig = plt.figure(figsize=(12, 5))
gs = fig.add_gridspec(nrows=2, ncols=3, width_ratios=[1,.5,1])
axs = gs.subplots()
axs[0].plot(data.X.squeeze(), data.y, 'o', color='k')
axs[0].plot(X, preds, '-', lw=3)
axs[0].fill_between(x=X.flatten(),
                    y1=preds+np.sqrt(variances),
                    y2=preds-np.sqrt(variances),
                    alpha=.2, edgecolor='w')
axs[0].set_title('$\\mathbf{y}$')
axs[1].plot(data.X.squeeze(), (data.y-preds)**2, '.', color='r')
axs[1].set_title('$|\\mathbf{y} - \\hat{\\mathbf{y}}|$')
axs[1].set_yticks([])
axs[2].imshow(model.kern.K(X), cmap='Greys', vmin=-1, vmax=1)
axs[2].set_title('$K(\cdot, \cdot)$')
for