#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    header += struct.pack('>H', rng.randint(200, 500))
    header += struct.pack('>H', rng.randint(200, 500))

    # color table (RGB triplets)
    ctsize = rng.randint(1, 7) * 3
    for _ in range(ctsize):
        header += bytes([rng.randint(0, 255)])

    datalen = int.from_bytes(
            rng.read(3), byteorder='big') % 98 + 1

    data = rng.read(datalen)

    frame_header = b'\x21\xF9\x04\x08'
    
    frame_header += bytes([(0b11 << 2) | (datalen >> 8 & 0b00000011)])
    frame_header += bytes([datalen & 0xFF])

    frame_header += b'\x00'*8

    block = []
    while len(data) > 255:
        block.append(len(data) - 255)
        data = data[:len(data)-255] + b'\x00'

    if len(block) >= 10: raise Exception()
    elif len(block) == 0: pass
    else: 
        block[0] |= 1<<7
        block = [len(block)] + block 
    block = bytes(bytearray(sorted(block)[::-1]))

    frame_header += block

    frame_header += b'\x00\x00'

    header += frame_header[:-2]
    header += data

    crc = hex(crc32(header))[2:]

    assert len(crc) == 8

    header += crc.encode()

    out.write(header)

