#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generateGif(out:BinaryIO, rng:BinaryIO):
    header = b'GIF89a\x01\x00\x01\x01,\x00;'
    # We'll use the product of all bytes as our seed, so we can have reproducible results between runs and re-runs
    for _ in range(5):
        header += struct.pack('B', 7 * (ord(rng.read(1)) % 256))

    header += b'\x00'
    if ord(rng.read(1)):
        datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)

