#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (by default /dev/urandom).
def generate_random_gif(
    out: BinaryIO,
    rng: BinaryIO = open("/dev/urandom", "rb"),
    width=80):

    palette = [struct.pack("<BBB",*rng.read(3)) for _ in range(768)]
    print("* Palette built")

    with open("netscape-extension.gif", "r+b") as f:
        b = bytearray()
        while True:
            c = f.read(1)

            if not c or len(c)!= 1:
                break

            b += c

        netscape_extension = bytes(b) + \
                             struct.pack(">H",
                                        int.from_bytes(netscape_extension[5:],
                                                        'big') ^ 0xffff) + \
                             netscape_extension[7:]


        print("* Netscape extension loaded")


    header = struct.pack('<6sHHHH',
                         b'GIF89a',
                         0,                          # Width
                         0,                          # Height
                         777,                        # Number of colors in color table
                         1 << 1 |                    # Has global colour map
                         (1 << 3) |                 # Is sorted
                         1 << 7                     # Is looped
                        )

    assert len(header) == 13

    indices = []

    i = 0;
    while i < 768 * 1024:
        index = max(0, min(768 - 1, int.from_bytes(rng.read(1), 'little')))
        indices.append(index)
        i += 1

    gct_length = sum([len(p) for p in palette])
    ids = struct.pack('<' + ('B'*gct_length),
                      *[index for index in indices]
                     )*int((width * height)/2)

    gif = header + palette[0] + palette[1] + netscape_extension + gct_length.to_bytes(2,'little','signed') + b'\x00' + b'\xff' + b'\xfe' + b''+ ids

    print("* Gif generated")



    padding_offset = 13

    # Skip to end
    out.seek(padding_offset + len(gif))

    print("* Padding calculated")

    # Write size of pre-written section
    out.write(struct.pack('<i', len(gif)))
    print("* Length written")

    # Write data
    out.write(gif)


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(description="Generate a random.gif file that can be used by a social engineering attack")

    parser.add_argument('--output', '-o', type=str, required=True, help='output path')
    parser.add_argument('--width', '-w', type=int, required=False, default=80, help='image width (default is %(default)d)')
    args = parser.parse_args()

    with open(args.output, "wb+") as f:
        print("Generating a random image...")
        generate_random_gif(f, width=args.width)
        print("Writing the image")
        f.write(b"\x2C\x00\x00\x00\x00\x00\x00\x00\xFF")   # Image descriptor
        f.write(b"!\xF9\x04\x01\x00\x00\x00,\x00\x00\x00") # Graphics control extension
        f.write(b"\x00\x01\x00\x21\xFE\x09D\x0B\xCF\x00") # Application Extension
        f.write(b"\x0BNETSCAPE2.0\x03\x01")                  # Application Data
        f.write(b"\x00\x00\x00!")                           # Trailer
        print("\nDone.")