#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate a random GIF given an input stream of bytes to use for randomness and output stream to write to.
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Create the header
    header = b"GIF89a"

    # Write it all to our new file
    out.write(header)

    # Future work: Add in global color table support?

    # Generate width and height randomly
    width  = int.from_bytes(rng.read(2), "little")
    height = int.from_bytes(rng.read(2), "little")
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))

    # Flags
    gctflag = True if next(rng)!= 0 else False
    imgflag = [next(rng), next(rng)]

    # Get the global color table size
    gctsize = pow(2, max((imgflag[0] & 7) + 1, 2)) - 1

    # GCT exists iff flag is set
    if gctflag:

        # Send the number of colors we want as well as their actual colors.
        out.write(gctsize.to_bytes(1, 'little'))
        for _ in range(pow(2, ((imgflag[0] & 7) + 1))):
            r = int.from_bytes(rng.read(1), 'little')
            g = int.from_bytes(rng.read(1), 'little')
            b = int.from_bytes(rng.read(1), 'little')

            out.write(r.to_bytes(1, 'little'))
            out.write(g.to_bytes(1, 'little'))
            out.write(b.to_bytes(1, 'little'))

    # LZW minimum code size
    mincodesize = next(rng) + 1

    # Image flags
    imgflags = (0x2C     << 4             |
                imgflag[0]                  |
                0x00                        ) \
                 + (reserved << 3)       + mincodesize
    out.write(imgflags.to_bytes(1, 'little'))


    # Skip the rest for now because I'm gonna need to learn more about this format later.

    return