#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as a source of bytes for its content.
def generate_gif(rng: BinaryIO, out: BinaryIO):

    # magic header
    out.write(b'GIF89a')

    # logical screen descriptor
    width = 1024
    height = 768
    color_depth = 8	# bits per pixel (color planes)
    num_colors = pow(2, color_depth)
    bgcoloridx = -1   # no background color specified in GCT
    aspect_ratio = 0
    gctsize = len(bytes()) % 256
    gct = b'' # TODO

    lsd = struct.pack('<BBHHBBB',
                      width & 0xff,       # 1 byte
                      height >> 8,        # 1 byte
                      height & 0xff,      # 1 byte
                      (num_colors << 4) | (gctsize-1),
                      0,                  # 1 bit flag
                      bgcoloridx,         # 1 byte
                      aspect_ratio        # 1 byte
                     )

    out.write(lsd)
    
    # global color table
    if gct:
        out.write(struct.pack('B', gctsize))
        out.write(gct)

    # graphics control extension block (no transparencies used here)
    graphic_control = struct.pack('BBHbbb',
                                  0x21,              # introducer
                                  ord('f'),          # label
                                  int((len(lsd)+len(gct)+8)/8),   # size
                                  0x00,              # flags: NULL
                                  0x00               # delay time
                                 )
    out.write(graphic_control)


    # image descriptor
    image_descriptor = struct.pack('<BBHHHHB',
                                   0x2C,             # separator
                                   xpos&0xFF,        # left position
                                   ypos>>8,          # top position
                                   ypos&0xFF,        # bottom position
                                   width&0xFF,       # width
                                   height&0xFF,      # height
                                   0x80|int(0),      # LZW code size
                                  )
    out.write()

    # plain encoded image data
    datalen = width * height // 2;
    data = rng.read(datalen)
    out.write(header + data)
