"""
Generate a random GIF from /dev/urandom.
"""

import struct
import sys
from argparse import ArgumentParser
from typing import BinaryIO


def parse_args() -> str:
    parser = ArgumentParser("generate-random-gif")

    parser.add_argument("-o", "--output", type=str, default="-",
                        help="Output file name (default is stdout)")

    return parser.parse_args().output


if __name__ == "__main__":
    filename = parse_args()
    if filename == "-":
        out: BinaryIO = sys.stdout.buffer
    else:
        # not using a context manager here because it's annoying to get right for Python 2
        out = open(filename, "wb").buffer

    header = bytearray([0x47])
    header += b"GIF89a"
    width = height = maxval = 1 << 8
    data = struct.pack("<HHB", width, height, maxval)
    runlen = 0xff
    while True:
        runlen -= 1
        if runlen < 0 or len(data) > 256 * 2 ** 10:
            break

        c = ord(open("/dev/urandom", "rb").read(1))
        data += bytes((c,)) * runlen
        data += bytes((runlen,))
        runlen = min(0xff, runlen - 1)

    datalen = len(data)
    assert datalen < 256 ** 2
    out.write(header + struct.pack(">H", datalen))
    out.write(data)


