#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it
    # easier to test for false positives.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    
    # Write header and data
    assert datalen >= size 
    out.write(b'\x47\x49\x46\x38\x39\x61') # 'GIF89a'
    out.write(struct.pack('>H', datalen + 2))        # LZW mincode size
    out.write(struct.pack('BBB', *rng.read(3)))     # Screen descriptor
    out.write(rng.read(size))             # Actual data