#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate a random GIF given an input stream of bytes to use for randomness and output stream to write to.
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Create the header
    header = b"GIF89a"

    # Get the length of the image from the RNG
    width  = (int.from_bytes(rng.read(1), 'big') >> 4) + 1
    height = ((int.from_bytes(rng.read(1), 'big') & 0xf) << 8) | int.from_bytes(rng.read(1), 'big')
    print("Generating {}x{} GIF".format(width,height))

    # Write it all together
    out.write(header)
    out.write(struct.pack('>H', width))
    out.write(struct.pack('>H', height))

    # Get the LZW MinCodeSize and set up this section in the header
    mincodesize = int.from_bytes(rng.read(1), 'big')
    gctsortedflag = True
    lctsortedflag = True
    interlaced = False
    sortflag = 0
    reserved = 0
    imgflags = (gctsortflag << 7) + (lctsortedflag << 6) \
                + (interlaced << 5)     + (sortflag << 4) \
                + (reserved << 3)       + mincodesize
    out.write(imgflags.to_bytes(1, 'little'))


    # Skip the rest for now because I'm gonna need to learn more about this format later.

    return