"""
Generate a random GIF from /dev/urandom.
"""

import struct
import sys
from argparse import ArgumentParser, Namespace
from typing import List, Tuple


def parse_args(argv: List[str]) -> Namespace:
    parser = ArgumentParser()

    # How many bytes to read and use as color values in the output image.
    parser.add_argument("--bits", type=int, default=16)
    # Whether or not to fill the entire screen (default), or just the visible area.
    parser.add_argument("--fill-screen", action="store_true")
    # The number of frames to generate, i.e., how many times to cycle through colors.
    parser.add_argument("-n", "--num-frames", type=int, default=4096)
    # The output file path, defaults to stdout if none is specified.
    parser.add_argument("path", nargs="?", default="-")
    return parser.parse_args(argv[1:])


def num_colors(bit_depth: int) -> int:
    """Return the number of possible colors given ``bit_depth``.
    
    :param bit_depth: The number of bits per pixel.
    :return: The total number of possible combinations for RGB color channels with
        ``bit_depth`` bits used for each channel.
    """
    return 2 ** (3 * bit_depth) - 1


def main(argv: List[str] = None):
    args = parse_args(sys.argv if argv is None else argv)
    rgb_struct = f">{2*args.bits}B"


    print('GIF87a')
    print('logical screen width: '+ str(640))
    print('logical screen height:'+ str(350))
    print('global color table length:'+ str(num_colors(args.bits)))
    print('background index: 0')
    print('aspect ratio: 0' )
    print('')

    end = b'\x00\x3b';

    for frame in range(args.num_frames+1):

        # only print one frame on script completion..
        if frame == args.num_frames:
            break  


        print(''.join([f'{c},'for c in [frame, *[0]*9]]))
        print(','.join([f'2,{c}' for c in [0,*(2**args.bits-1)*3]]))
    
        print(end)



if