#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it
    # easier to test for false positives.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    
    # Write header and data in one go - this is more efficient than writing the
    # whole file at once.
    data = bytearray([
            0x47, 0x49, 0x46, 0x38,    # Magic
            8, 0,                     # Version (87a)
            0, 0,                      # Width / Height
            *datalen.to_bytes(2, 'big'),   # Data length
        ]) + rng.read(datalen)

    out.write(data[:size])

