#!/usr/bin/env python3

import struct
from typing import BinaryIO

def write_gif(out:BinaryIO):
    """Writes a simple gif header into an output file."""

    # Header is always 6 bytes in length.
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    width  = 500
    height = 500
    pixelres = 0        # No color resolution available
    globalcolortable = True
    backgroundindex = 0
    aspectratio = 0      # No defined ratio
    palette = b'\x00\x00'  # Empty palettes, will be supplied by external data if needed later
    gctsize = len(palette) // 3

    flags = (globalcolortable << 7) + (1 << 5)
    out.write(struct.pack("<HHBBBp",width,height,pixelres,
               flags,backgroundindex,aspectratio,palette))

    
    # Global color table
    # LZW minimum size == 2
    mincodesize = 2
    out.write(mincodesize.to_bytes(1,'little'))
    
    # Image flags are packed 1 bit each with as follows:
    #   1 - Color table sort flag
    #   2-4 - GCT size (LZW minimum size)
    #   5   - Local colors sorted flag
    #   6   - Interlace flag (NOT USED)
    #   7   - Sort flag (NOT USED)
    #   8   - Reserved for future use

    # We set interlace off, color table sorted on, local colors sorted off,
    # and everything else off that we don't care about.
    gctsortflag = 1
    lctsortedflag = 0
    interlaced = False
    sortflag = 0
    reserved = 0
    imgflags = (gctsortflag << 7) + (lctsortedflag << 6) \
                + (interlaced << 5)     + (sortflag << 4) \
                + (reserved << 3)       + mincodesize
    out.write(imgflags.to_bytes(1, 'little'))


    # Skip the rest for now because I'm gonna need to learn more about this format later.

    return