#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def gen_gif(out: BinaryIO, rng: BinaryIO):

    # Header (GIF89a) and screen descriptor
    header = b'GIF89a\x00' + \
             bytes([76 & 255,
                   76 >> 8,
                   ((48 << 4) | (0b10)),
                   (((40 * 1000) // 10) & 255),
                   (((40 * 1000) // 10) >> 8)])  # Logical Screen Descriptor

    datalen = int.from_bytes((bytes([rng.read(1)[0]]) |
                              bytes([rng.read(1)[0]]) for _ in range(2)),
                             rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
