#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header block (13 bytes)

    out.write(b"GIF89a")  # Signature
    out.write(struct.pack("!H", 0))   # Screen width (unused) 
    out.write(struct.pack("!H", 0))   # Screen height (unused)
    out.write(struct.pack("B", 7))     # Flags: global color table; 
                                     # header has no lzw min code size field
                                     # background index and pixel aspect ratio not included
    
    # Global Color Table (256 colors * 3 bytes = 768 bytes)
    for _ in range(256):
        out.write(rng.read(3))

    # LZW min code size field
    out.write(bytes([8]))

    # Image Descriptor Block
    # (4 + fixed data length + image data length blocks = 4 blocks)

    # Left corner x position
    out.write(rng.read(2))

    # Left corner y position
    out.write(rng.read(2))

    # Width of image
    out.write(rng.read(2))

    # Height of image
    out.write(rng.read(2))

    # Flags: local color table present; interlace flag is zeroed
             # sort flag is zeroed; local color table is sorted by frequency
    out.write(bytes([0b10010000]))

    # Image Data Block
    for _ in range(100):

        b = rng.randrange(8)
        
        if b == 0 or b == 4:
            val = rng.randrange(2 << b)
            
            while True:
                c = val & 0b11

                if c <= 2:
                    break
                
                val >>= 2
                
            out.write(bytes([c | ((val >> 2) << 4)]))
        else:
            out.write(bytes([b | ((rng.randrange(8 - b) << 4))]))
            
    return
