#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    header += struct.pack('>H', rng.randint(200, 500))
    header += struct.pack('>H', rng.randint(200, 500))

    # color table (RGB triplets)
    ctsize = rng.randint(1, 7) * 3
    for _ in range(ctsize):
        header += bytes([rng.randint(0, 255)])
#    print("header=", header)
    out.write(header + b'\x87\x00')

    # LZW minimum code size
    lzwmin = rng.randint(2, 12)
    out.write(bytes([lzwmin]))

#    print("lzwmin=", lzwmin)

    lzwbuf = [b'\0'] * 4096
    def emitchar():
        if not len(lzwbuf):
            out.write(struct.pack('B', lzwmin))
            return

        char = lzwbuf[0]
        del lzwbuf[0]
        while len(lzwbuf):
            nextchar = lzwbuf[0]

            if char!= '\0':
                # write to output stream
                nbits = 1 + int((nextchar == '\0')) + \
                    int((char < nextchar))

                #print("emitting char:", char, "nbits:", nbits, "nextchar:", nextchar)
                out.write(\
                        bytes([(ord(char) & (~(-1 << nbits))) |
                           ((ord(nextchar) & ((1 << nbits) - 1)) <<
                            (8-nbits))])
                   )

            if char in lzwxtra and nextchar in lzwxtra[char]:
                # reconstructed from dictionary
                lzwxtra[char].remove(nextchar)
                if not lzwxtra[char]:
                    del lzwxtra[char]
            elif chr(ord(char)+1) is not None and lzwmin <= ord(char)+1 <= 12:
                # reconstructed with string of bit shifts
                lzwxtra.setdefault(chr(ord(char)+1), []).append(nextchar)

            if nextchar in lzwxtra:
                break;

            char = nextchar

    lzwxtra = {}
    lzcxtra = ""

    maxlen = 255*lzwmin


    byte_count = 0
    while True:
        byte = bytes([rng.randint(0, 255)])
        byte_count += 1
        lzwbuf.extend(byte)
        lzcxtra += byte.decode()[:maxlen-len(lzcxtra)]
        prefix = lzcxtra[:-maxlen+len(byte)].encode()
        suffix = lzcxtra[-maxlen+len(byte):].encode()
        if prefix in lzwxtra or suffix in lzwxtra:
            continue
        
        lzcxtra = ''
        out.write(byte)
        emitchar()

        if len(lzwbuf) > 2**lzwmin:
            emitchar();
            emitchar();

