#!/usr/bin/env python3


def writeheader(out):

    """
    This function writes the header section of a GIF file. The animation loop count is not yet 
    implemented, so it is zeroed by default. A global palette should be supplied in order to encode
    an indexed image data stream into its decompressed form, but since we're only generating RGB images
    with Pillow, we'll skip that step until we have time to investigate further! 

    TODO: Implement global palette handling here...

    :param out: File object where the PNG should be written to.
    :return: None.
    """

    signature = b'GIF89a'
    version = bytes([0x38, 0x39])
    width = int((out.width / 8) + ((out.width % 8)!= 0)) * 8
    height = out.height
    colorres = 0
    bgindex = 0
    pixelaspectratio = b'\xFF'



    out.write(signature)
    out.write(version)
    
    # Screen descriptor block
    out.write(width.to_bytes(2,"little"))
    out.write(height.to_bytes(2,"little"))
    out.write(bytes([colorres]))
    out.write(bgindex.to_bytes(1,"little"))
    out.write(pixelaspectratio)
    
    # Global colour table specification
    # LZW Minimum Code Size; 1 byte unsigned integer; legal values from 0 to 9, inclusive.
    mincodesize = 0 
    out.write(mincodesize.to_bytes(1,'little'))
    
    # Image flags are packed 1 bit each with as follows:
    #   1 - Color table sort flag
    #   2-4 - GCT size (LZW minimum size)
    #   5   - Local colors sorted flag
    #   6   - Interlace flag (NOT USED)
    #   7   - Sort flag (NOT USED)
    #   8   - Reserved for future use

    # We set interlace off, color table sorted on, local colors sorted off,
    # and everything else off that we don't care about.
    gctsortflag = 1
    lctsortedflag = 0
    interlaced = False
    sortflag = 0
    reserved = 0
    imgflags = (gctsortflag << 7) + (lctsortedflag << 6) \
                + (interlaced << 5)     + (sortflag << 4) \
                + (reserved << 3)       + mincodesize
    out.write(imgflags.to_bytes(1, 'little'))


    # Skip the rest for now because I'm gonna need to learn more about this format later.

    return