#!/usr/bin/env python3

import struct
from typing import BinaryIO

data = [
    0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80,  # byte 1
    0b0000_0001, 0b0000_0011, 0b0000_0111,    # uint8
    0b0000_000100,  0b0000_001100,     # int8
    0b1000_0000,      0b0000_0000
    ]

with open("data.bin", "wb") as f:
    for b in data:
        print(f"{b} -> {struct.pack('B', b)!r}")
        f.write(struct.pack('<B', b))

    for u16 in (0x7FFF, 0xFFFE):
        print(f"u16 {u16:>5}: {struct.pack('<H', u16)}")
        f.write(struct.pack('<H', u16))
    
    for i16 in (-0x8000, -2):
        print(f"i16 {i16:<5}: {struct.pack('<h', i16)}")
        f.write(struct.pack('<h', i16))
    
    for i32 in (2**31-2, -(2**31)):
        print(f"i32 {i32:<5}: {struct.pack('<I', i32)}")
        f.write(struct.pack('<I', i32))

    f.seek(0)
    with open("data.txt", 'w') as tf:
        while True:
            data = f.read(8)
            if not data:
                break

            line =''.join([f'{d:02X}' for d in data])
            tf.write(line + '\n')
