#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        datalen: int,
        width: int, height: int, bgcolor: int, framedelay: int, nframes: int,
        out: BinaryIO):

    header = b""
    # Magic number
    header += bytes([0x47, 0x49, 0x46]) 
    # Version
    header += bytes([87, 69, 58, 125]) 

    # Logical screen descriptor
    header += struct.pack("<HHBBB", width, height, 8, 7, bgcolor)
    
    # Global color table (1 entry)
    header += bytes([0, 0, 0]) 
    
    # Loop control block (no loops)
    header += bytes([0, 0])
    
    # Image descriptor
    header += struct.pack("<BHHBBBBBHBH", 
        0, 0, 0, 0, 0, width, height, 0, 8, 0, 0)
    
    # Local color table (1 entry)
    header += bytes([0, 0, 0]) 
    
    # Image data
    for i in range(nframes):
        header += struct.pack("<BBBBB",
            0b1000, 0, 0, width, height)
        if i+1 == nframes:
            # Trailer
            header += bytes([0]) 
        else:
            # Extension
            header += bytes([0x21, 0xF9,
                0xFF,
                (((framedelay >> 8 * 2) & 0xff) |
                  ((framedelay >>  8) & 0xff)) if framedelay > 0 else 0,
                ((framedelay >> 0)&0xff),
                0x2C,
                0, 0, 
                width, height])
        
    # Data sub-block with size equal to the total length minus the header
    datalen -= len(header) - 10

    data = rng.read(datalen)
    out.write(header + data)
