#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it
    # easier to test for false positives.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    
    # Write header and data length & padding
    gifhead = bytes([
            0x47, 0x49, 0x46,     # Header signature
            0x38, 0x39, 0x61,     # Version string ("89a")

            0b00001111,         # Flags (sorted)
            0b00010000,         # Color table size

            # Frame descriptor block
            0x2c,                # Start of frame marker
            0,                   # Left offset
            0,                   # Top offset
            1,                   # Width in pixels
            1,                   # Height in pixels
            0b00000100,          # Local color table flag
            0                    # LZW min code size
        ]) + \
        0x0A * b'\x00'        # No application extensions or graphics control extensions

        out.write(gifhead)
        out.write(struct.pack('>H', datalen))
        out.write((datalen % 2**3)*b'\x00')