#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate a random GIF given an input stream of bytes to use for randomness and output stream to write to.
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Create the header
    header = b"GIF89a"

    # Get the length of the image from the RNG
    width  = (int.from_bytes(rng.read(1), 'big') >> 4) + 1
    height = ((int.from_bytes(rng.read(1), 'big') & 0xf) << 8) | int.from_bytes(rng.read(1), 'big')
    print("Generating {}x{} GIF".format(width,height))

    # Write it all together
    out.write(header)
    out.write(struct.pack('>H', width))
    out.write(struct.pack('>H', height))

    # Background color index is just zero
    out.write((b'\x00'))
    out.write((b'\x00\xFF\xFF\xFF'))

    # LZW minimum size == 2
    mincodesize = 2
    out.write(mincodesize.to_bytes(1,'little'))
    
    # Image flags are packed 1 bit each with as follows:
    #   1 - Color table sort flag
    #   2-4 - GCT size (LZW minimum size)
    #   5   - Global color table flag 
    #   6   - Interlace Flag
    #   7   - Sort Flag
    #   8   - Reserved
    imageflags = 0b10100000
    out.write(imageflags.to_bytes(1,'little'))

    # Now comes the palette

    # First byte indicates the number of entries in the palette
    # Then that many RGB triplets follow
    numentriesinpalette = mincodesize+1
    numrgbentriesperentry = 3
    palette = [None]*numentriesinpalette
    for i in range(numentriesinpalette):
        r = (int.from_bytes(rng.read(1),'big') % 0xff)+1     # We don't want a bunch of zeros  
        g = (int.from_bytes(rng.read(1),'big') % 0xff)+1     # so we add one first
        b = (int.from_bytes(rng.read(1),'big') % 0xff)+1

        rgb = (r<<16)|(g<<8)|b
        palette[i] = rgb
        out.write(rgb.to_bytes(3,'big'))
    
    # Skip the rest for now because I'm gonna need to learn more about this format later.

    return