#!/usr/bin/env python3

import struct
from typing import BinaryIO

def decode(instream: BinaryIO):
    data = instream.read()
        
    




def encode(instream: BinaryIO, out: BinaryIO):
    """ This is a modified version of the GIF encoding function from 
    https://github.com/python-pillow/Pillow/blob/master/src/libImaging/GifImagePlugin.py"""

    indata = instream.read()
    
    # Header
    out.write(b'GIF89a')

    
    # Logical screen descriptor
    # Width x Height form a 16 bit unsigned integer value
    width, height = struct.unpack("<HH",indata[6:10])
    if len(indata)>10:
        depth = int.from_bytes([indata[10]], byteorder= "big")    
    else: 
        print("Depth not found! Using default.")
        depth = 8
        
    globalcolorcount = pow(2,depth)
    #print(f"Width {width} | Height {height}")
    out.write(struct.pack("<BBHHH", width & 0xff,
                          (width >> 8) & 0xff,
                          height & 0xff,
                          (height >> 8) & 0xff,
                          globalcolorcount-1))
    
    # Global palette
    # The palette consists of one or more colors per pixel. A color is defined by R,G,B components
    # Each color uses 8 bits per component
    if globalcolorcount > 2**depth:
        raise ValueError('Color table overflow')

    # For some reason Pillow saves color tables with an additional 0 byte at the end?
    if globalcolorcount < 2**(depth+1):
        out.write(bytes(pow(globalcolorcount, 3)))
        
    # Actual gif content
    # A graphic control extension contains information specific to the image
    # following it. It allows the application using the graphics to store
    # extra data along with the image. We ignore it here
    while True:        
        b = instream.read(1)
        
        if not b:
            break

        skip = ord(b)!= 0x2C # Comma
        while skip:            
            instream.read(int.from_bytes(instream.read(1),byteorder='big'))
            skip = ord(instream.read(1)[0])!= 0x2C # Comma            
        
        
        
        # Local color table: 2 bytes per entry    
        lctsize = int.from_bytes(instream.read(1),'big') * 3
        if lctsize:
            out.write(instream.read(lctsize))
            
        # Number of pixels
        # Skip LZW code size byte
        pixnum = int.from_bytes(instream.read(1),'big')        
                
        # Read subblocks until there's no more left
        while pixnum>0:                    
            
            blocksize = int.from_bytes(instream.read(1),'big')

            if blocksize == 0:
                break # No more data
                
            # Data subblock
            out.write(instream.read(blocksize))
            
            pixnum -= blocksize # Decrement counter
            
     out.write((b'\x00\xFF\xFF\xFF'))

    # LZW minimum size == 2
    mincodesize = 2
    out.write(mincodesize.to_bytes(1,'little'))
    
    # Image flags are packed 1 bit each with as follows:
    #   1 - Color table sort flag
    #   2-4 - GCT size (LZW minimum size)
    #   5   - Local colors sorted flag
    #   6   - Interlace flag (NOT USED)
    #   7   - Sort flag (NOT USED)
    #   8   - Reserved for future use

    # We set interlace off, color table sorted on, local colors sorted off,
    # and everything else off that we don't care about.
    gctsortflag = 1
    lctsortedflag = 0
    interlaced = False
    sortflag = 0
    reserved = 0
    imgflags = (gctsortflag << 7) + (lctsortedflag << 6) \
                + (interlaced << 5)     + (sortflag << 4) \
                + (reserved << 3)       + mincodesize
    out.write(imgflags.to_bytes(1, 'little'))


    # Skip the rest for now because I'm gonna need to learn more about this format later.

    return