#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        datalen: int,
        headerlen: int,
        seed: bytes,
        out: BinaryIO):

    # Construct and write the GIF header
    gif_version = b'89a'
    width = height = 256
    global_color_table_present = False
    color_resolution = 8
    sort_flag = True
    size = '40x1'
    bg_color_index = 0
    pixel_aspect_ratio = 0
    bg_color = b'\xff\xff\xff'
    gc = (
            gif_version +
            struct.pack('BBB', width, height,
                global_color_table_present << 7 | \
                    color_resolution << 4 | \
                    sort_flag) +
            size.encode() +
            bg_color_index.to_bytes(1, byteorder='little') +
            pixel_aspect_ratio.to_bytes(1, byteorder='little') +
            bg_color * 3
            )
    assert len(gc) == headerlen
    out.write(gc)

    # Generate a PRNG from `seed` to provide
    # deterministic results
    rng = struct.Struct('I').pack(struct.unpack('I', seed)[0])
    data = rng.read(datalen)
    out.write(header + data)
