#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    
    # Write header
    size = 1024 * 8 # 8 KiB

    # Magic number "GIF89a"
    header = b"\x47\x49\x46\x38\x39\x61"
    header += struct.pack("<H", size)
    header += bytes([
        0x82, # Color table size (full)
        0,     # Color resolution 
        0,     # Sort flag (not sorted)
        0      # GCT sort flag (no gct sorting)
        ])
    
    frames = [bytes([i & 0xFF]) for i in range(size)]
    
    out.write(header)
    out.write(b"".join(frames))
    
if __name__ == "__main__":
    from sys import argv
    with open("/dev/urandom", "rb") as rng:
        with open(argv[1], "wb") as out:
            generate_random_gif(rng, out)
        
    print("Generated to '{}'".format(argv[1]))
    
    out.write(header + data)

