#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) of length in bytes `length`.
def generate(rng:BinaryIO, out:BinaryIO, length):
    # Write header.
    out.write('GIF89a'.encode())

    # Write logical screen descriptor.
    out.write(struct.pack('<2H', 1024, 768))

    # Graphic control extension (no disposal method and full-intensity color table).
    out.write((
        0x21
        + 0xf9
        + 0x04
        + int(0b11 << 5)
        + 0x00
        + b'\xff\xff\xff'
    ))

    # Image descriptor.
    out.write(struct.pack('<BHHBBBHB', 
        0x2c,   # image separator
        0x00, 0x00,   # left, top
        0x00, 0x00,   # width, height
        0x00,   # local color table size = no color table
        0b10000000 | 0b10000000,     # LZW codesize = 11 bits
        0x00,   # no LZW min clear code specified
        0x00    # no LZW max end code specified
    ))

    # Color table.
    for i in range(256):
        out.write(bytes([i * 2]))

    # Image data (LZW compression).
    while True:
        buf = [next(rng) % 2]

        while len(buf) < 256 and rng.tell() < length - 1:
            buf += [len(buf), next(rng) % 2]

        if rng.tell() >= length - 1 or all(i == buf[0] for i in buf[:-1]):
            break
        
        out.write(bytes([(len(buf) - 1) // 256, ((len(buf) - 1) & 0xFF)]))
        out.write(bytes(buf[:256]))
    
    out.write(b'\x00')

