#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
        ):

    # Calculate lengths
    hdrlen = 2 + 5               # Header length (6 bytes)
    datalen = size + hdrlen      # Total length of data block
    
    # Write header and data
    assert datalen >= size 
    out.write(b'\x47\x49\x46\x38\x39\x61') # 'GIF89a'
    out.write(struct.pack('>I', size))    # DataLen
    out.write(rng.read(size))             # Actual data