#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    header += struct.pack('>H', rng.randint(200, 500))
    header += struct.pack('>H', rng.randint(200, 500))

    globalcolorcount = 2 << (rng.read(1)[0] & 7)
    colormapsize = int.from_bytes(
            rng.read(1), byteorder='big') * 3
    if globalcolorcount > colormapsize:
        globalcolorcount = colormapsize
    flags = rng.read(1)[0]

    # only use palette
    flags &= ~(1 << 7) | (1 << 4) 

    header += bytes([flags])
    header += bytes([globalcolorcount])

    for i in range(globalcolorcount):
        for j in range(3):
            header += bytes([rng.randint(0, 255)])

    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
