#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    header += struct.pack('>H', rng.randint(200, 500))
    header += struct.pack('>H', rng.randint(200, 500))

    color_table_size = rng.randint(2 ** i for i in range(1, 5))
    global_color_table = bytes([255] * (color_table_size * 4))
    local_color_table = b''

    if rng.randint(0, 1):
        header += global_color_table

        num_of_blocks = rng.randint(1, 11)

        block_sizes = []
        for _ in range(num_of_blocks):
            block_sizes.append(rng.randint(2, 7))
        
        subblocks = []
        for size in block_sizes:
            subblock = bytes()

            for _ in range(size):
                subblock += struct.pack('<B', rng.randint(0, 255))
            
            subblocks.append(subblock)

        lzw_min_code_size = min(len(b).bit_length() - 1 for b in subblocks)
        header += struct.pack('<B', lzw_min_code_size)

        compressed_data = bytes([lzw_min_code_size])

        lzw_max_code_size = len(global_color_table) // 4

        while True:
            max_possible = min(len(b), 2**lzw_max_code_size-1) + 1

            # we want to find one of the longest subblocks that can fit inside this code length
            index = rng.randint(0, max_possible) - 1
            for i, s in enumerate(subblocks[:index]):
                compressed_data += struct.pack('<B', len(s)-1) + s

                del subblocks[i]
            
                break
            
            else: continue

            # no more blocks left
            if not subblocks: 
                break 

        compressed_data += struct.pack('<B', 0x00)
    
        out.write(header + compressed_data)
        
    else: 
        local_color_table += bytes([255] * 4 * rng.randint(1, 256))
        out.write(struct.pack('<B', 0xF7))
        out.write(header + data)
