#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it
    # easier to test for false positives.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big'
        )

    print("Generating %d bytes with datalen=%d" % (size,datalen))


    out.write(b'GIF89a')                   # Signature
    out.write(b'\x01\x02\x03\xf0')         # Version
    out.write(struct.pack('>I', size))    # DataLen
    out.write(rng.read(size))             # Actual data