#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it
    # easier to test for false positives.
    datalens = []
    while len(datalens) < 250 or sum(datalens[::-1]) < size :
        datalens += [int.from_bytes(
                rng.read(3), byteorder='big')]
        
    # Write header and data
    total_len = 0
    for i in range(len(datalens)):
        dl = min(size - total_len, datalens[-i-1])
        if (dl > 0):
            total_len += dl
            out.write(
                    b'\x47\x49\x46\x38\x39\x61' +           # Header
                        struct.pack('>I', dl) +              # Length
                        rng.read(dl))                        # Data