#!/usr/bin/env python3

import io
import struct
from pathlib import Path


def generate_random_gif() -> bytes:
    header = 'GIF89a'.encode('ascii')

    # Random size between 10 and 256k
    # (the biggest possible for an uncompressed image)
    datalen = io.BytesIO().getbuffer()[4] % 0x7fff + 0xa1

    # Zero out the color table
    body = b'\x00' * 3 * pow(2, datalen - 1)

    return header + pack_data(body)


def pack_data(data: bytes) -> bytes:
    datalen = len(data).to_bytes(3, byteorder='big', signed=False)
    return datalen + data


if __name__ == '__main__':
    outdir = Path('./out/')
    outdir.mkdir(exist_ok=True)

    with open('/dev/urandom', mode='rb') as rng:

        # Generate a bunch of images in sequence
        for i in range(4):

            print(f"Generating {i+1} / 4")
            img = generate_random_gif()
            filename = f'{outdir}/img{i}.gif'
            
            with open(filename, mode='wb+') as fd:
                fd.write(img)