#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO


# Generate a random GIF given an input stream of bytes to use for randomness and output stream to write to.
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Create the header
    header = b"GIF89a"

    # Get the length of the image from the RNG
    if len(sys.argv) < 2 :
        width  = (int.from_bytes(rng.read(1), 'big') >> 4) + 1
        height = ((int.from_bytes(rng.read(1), 'big') & 0xf) << 8) | int.from_bytes(rng.read(1), 'big')
    elif len(sys.argv) > 2 :
        width  = int(sys.argv[1])
        height = int(sys.argv[2])
    else :
        raise Exception("Must specify the image dimensions")

    print("Generating {}x{} GIF".format(width,height))

    # Write it all together
    out.write(header)
    out.write(struct.pack('>H', width))
    out.write(struct.pack('>H', height))

    # Background color index is just zero
    out.write((b'\x00'))
    out.write((b'\x00\xFF\xFF\xFF'))

    # LZW minimum size == 2
    mincodesize = 2
    out.write(mincodesize.to_bytes(1,'little'))
    
    # Image flags are packed 1 bit each with as follows:
    #   1 - Color table sort flag
    #   2-4 - GCT size (LZW minimum size)
    #   5   - Local colors sorted flag
    #   6   - Interlace flag (NOT USED)
    #   7   - Sort flag (NOT USED)
    #   8   - Reserved for future use

    # We set interlace off, color table sorted on, local colors sorted off,
    # and everything else off that we don't care about.
    gctsortflag = 1
    lctsortedflag = 0
    interlaced = False
    sortflag = 0
    reserved = 0
    imgflags = (gctsortflag << 7) + (lctsortedflag << 6) \
                + (interlaced << 5)     + (sortflag << 4) \
                + (reserved << 3)       + mincodesize
    out.write(imgflags.to_bytes(1, 'little'))


    # Skip the rest for now because I'm gonna need to learn more about this format later.

    return