#!/usr/bin/env python3

import struct
import random
from typing import BinaryIO


# Generate a valid GIF file with a random screen size and color map
# This is not intended as an actual useful GIF programme for anyone to use!
def main():
    width = 2 ** (16 - 15 * random.randrange(0x2))
    height = 2 ** (16 - 15 * random.randrange(0x4))

    with open('random.gif', 'wb') as outfile:

        # Header (always this format!)
        outfile.write((b'GIF89a'))
        # Screen descriptor block
        outfile.write(struct.pack('<HHBBB',
                                 width, height,
                                 0xf7, 0xff, 0x0f))

        # Global colour table frame
        write_color_table(outfile, 0)
        outfile.write(b'\x2c\x00\x00')
        outfile.write((width*height).to_bytes(2, byteorder='little'))
        outfile.write(b'\x00')

        # Pixel data
        write_pixel_data(outfile, width, height)


def write_pixel_data(out: BinaryIO, width: int, height: int):
    pixel = bytes()
    while len(pixel) < width * height * 3:
        # Using LZW compression each byte can encode up to 12 bits
        # Each bit represents either if it's the first occurence of that
        # 8-bit value or if this is part of an existing sequence
        continue_flag = 1 << 7
        count: int = 1  
        while True:
            code = get_next_code(len(pixel)//3)

            if code > 0xffff:
                raise Exception("Code too long!")
            
            # If we've got more than 10 bits just do raw pixels instead
            assert code <= 0xfff
            # Emit pixel values in groups of three.
            # If there are less than three left just pad them with zeroes.
            chunk = (code >> 8 & 0xf).to_bytes(
                    1, byteorder='big') + \
                    ((code >> 4 & 0xf)).to_bytes(
                    1, byteorder='big') + \
                    ((code & 0xf)).to_bytes(
                    1, byteorder='big') + \
                    (((code << 4) & 0xf)+continue_flag).to_bytes(
                        1, byteorder='big')
            out.write(chunk)

            # Write the pixels which were used from our code back to the stream
            length = min(count+1, width - len(pixel)//3 % width)
            val = randint(0, 255)
            for i in range(length):
                pixel += val.to_bytes(
                        length=1, byteorder='big')

                if len(pixel) >= width*height*3:  
                    break

                # We could have filled in all available space already so
                # reset anyways
                if len(pixel)//3 == width*height:
                    pixel = b''
                    break
            else:
                continue    # Not enough pixels left, start new run
            break
        
        if len(pixel) >= width*height*3:  
            break
    
    print("Oooh, you're full! There isn't enough room for your pixels")
    return

def write_color_table(out: BinaryIO, numcolors: int):
    colors = []
    # Generate some nice colours!
    # This will surely never be perfect though...
    for _ in range(numcolors):
        r = randint(0, 255)
        g = randint(0, 255)
        b = randint(0, 255)
        colors.append([r, g, b])
        
    # Write our palette out
    for c in colors:
        out.write(bytes([c[0], c[1], c[2]]))
    

if __name__ == '__main__':
    main()