#!/usr/bin/env python3

import base64
import os
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it
    # easier to test for false positives.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Generate random image data
    data = b''
    while len(data) < size:
        chunklen = min(size - len(data), 250)

        # Prepend length with a special bit set to prevent filters from being applied
        chunkdata = struct.pack('>B', (chunklen | 0b10000000)).ljust(
                chunklen + 1, base64.b64encode(
                    os.urandom((int)(chunklen *.75))))[1:]
        
        data += chunkdata
    
    # Write header and data
    assert datalen == len(data)
    out.write(b'\x47\x49\x46\x38\x39\x61') # 'GIF89a'
    out.write(struct.pack('>I', datalen))   # DataLen
    out.write(data)                        # Actual data