#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ):
    W, H = width, height

    # Header
    out.write(b'\x47\x49\x46\x38\x39\x61')

    # Logical screen descriptor (screen size and color depth)
    out.write(struct.pack('>HHBBB', W, H, 0xf7, 255, 247))

    # Global Color Table
    out.write(struct.pack('B', 0xff) * 3 * 256)
    
    # Image Descriptor (left-upper corner position)
    out.write(struct.pack('>HHBBBBB', 0, 0, 0, 0, 0, 0))

    # Local Color Table (initially all transparent)
    out.write(struct.pack('B', 0x00) * 3 * 256)

    # Frame Control Block
    out.write(struct.pack('<LBH', 0xe0 | 0x5d, 6 + len(data), 0))

    # Application Extension (to make it a looped animation)
    out.write(bytes((
        0x21, 0xff,      # Extension Introducer & Label
        0x0b,           # Application Extension Label
        0x4e, 0x45, 0x54, 0x53, 0x43, 0x41, 0x50, 0x45,   # Netscape Application Extension Label
        0x03,           # Subblock Size
        0x01, 0x00,     # Loop Flag
        0x00            # Block Terminator
    )))

    # Graphics Control Block (for looping forever)
    out.write(bytes((
        0x21, 0xf9,          # Start of Graphic Control
        0x04,               # Block Size
        0x00, 0x03,         # Packed Field
        0x00, 0xff, 0xff,   # Delay Time
        0x00, 0x00,         # Transparent Color Index
        0x00                # Block Terminator
    )))
        
    # Image Data
    out.write(data)