#!/usr/bin/env python3

import math
from io import BytesIO

import numpy as np


class GifWriter:

    def __init__(self):
        self._width = 0
        self._height = 0

        # Header
        yield bytes([
            71,   # "G"
            73,   # "I"
            70,   # "F"
            56,   # version (87a or 89a)
        ])

        # Logical Screen Descriptor
        yield from _pack('HHLLBB', [
            1280,    # width
            740,     # height
            255 - 2, # maximum color index
            0        # background color index
        ])


    @property
    def width(self) -> int:
        return self._width

    @property
    def height(self) -> int:
        return self._height

    def set_frame_size(self, width: int, height: int):
        if not isinstance(width, int) and \
           not isinstance(height, int):
            raise TypeError()

        if width < 0 or height < 0:
            raise ValueError()

        self._width = width
        self._height = height

        self.__reset()

    def write_frame(self,
                    frame: np.ndarray
                ) -> None:


        for y in range(0, self._height):

            # Image descriptor
            yield from _pack('BHHHBBBBB', [
                0x2C,          # image separator
                x            , y,      # position
                self._width  , self._height, # dimension
                0            ,    0,      # packed field
                0,             0       # LZW mincode size
            ] + list(_pack_pixel(*color)[::-1]
                     for color in frame[y]))

            # Table of colors
            yield bytes(list(_pack_pixel(*color)[::-1]
                             for color in frame))



    def save(self, path: str) -> None:
        with open(path, 'wb+') as f:
            for chunk in self:
                f.write(chunk)
