#!/usr/bin/env python3
"""
Generates a random GIF file into `out`.
"""

import os
import random
import struct

from typing import BinaryIO


# Generates a random GIF file into `out` using `get_random`
# in order to obtain random data.
# 
# The resulting file will have exactly $size bytes of
# data following the GifHeader.
def generate_random_gif(
        size: int, get_random: Callable[[int], bytes], out: BinaryIO
    ) -> None:

    # Use the first three bytes as datalen instead of a fixed value. This makes it
    # easier to test for false positives.
    datalen = struct.unpack('B', get_random(3))[0] & 0x7f
    datalen |= (struct.unpack('B', get_random(2))[0] & 0x7f) << 7
    
    # Write header and data
    assert datalen + 5 <= size 
    print(datalen, file=out)   # Data length
    out.write(b'\x47\x49\x46\x38\x39\x61') # 'GIF89a'
    out.write(b'\xff')              # Application extension block
    out.write(b'\xf9\x04')          # Extension introducer and label
    out.write(b'\0\0')              # Block size
    out.write(b'\0')                # No background color index
    out.write(b'\x2c\0\0\0\0\0\0')  # Image descriptor
    out.write(b'\0\0')              # Left offset == top offset
    out.write(b'\x64\0\0\0')        # Width == height == 100
    out.write(b'\0')                # LZW minimum code size
    out.write(b'\0')                # No transparent pixels specified

    if datalen > 0:
        out.write(get_random(datalen))