#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom). The
# image has no global colour map or local colour maps.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    w = rng.randint(200, 500)
    h = rng.randint(200, 500)
    header += struct.pack('>HHB', w, h, 0x80)

    # pixel aspect ratio is 4:3
    header += b'\x00\x04'

    # background colour index is zero
    header += b'\x00\x00'

    # remaining fields of the screen descriptor
    header += b'\x00\x00'    # x position
    header += b'\x00\x00'    # y position
    header += b'\x00\x00'    # width of graphics area
    header += b'\x00\x00'    # height of graphics area

    # global colour table size (use maximum 256 colours)
    header += struct.pack('B', 2 ** rng.randint(8, 9)))

    # logical screen descriptor block terminator
    header += b'\x3b'

    # global colour table (RGB triplets)
    for i in range(2 ** 8):
        if ((i % 16) == 0): print()
        r = min(max(int((3*i)/2**8), 0), 255)
        g = min(max(int(((2**8-3)*i)/2**8), 0), 255)
        b = min(max(int((i*(2**8-3))/2**8), 0), 255)
        print('%d %d %d'% (r, g, b)),
        header += struct.pack('BBB', r, g, b)
    
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
    