#!/usr/bin/env python3

import os
import sys
from typing import BinaryIO

# Generates a random GIF file in `out`, using the random number generator `rng`.
# Uses the standard C PRNG as defined by /dev/urandom (on Linux).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header for 256 colour Gif files
    header = bytes([
        0x47, 0x49, 0x46, 0x38, 0x39, 0x61,     # "GIF89a"
        # Screen descriptor block
        0x0A,                                   # Screen width LSB
        0x00,                                   # Screen width MSB
        0x00,                                   # Screen height LSB
        0x00,                                   # Screen height MSB
        0xF0,                                   # Colour table size - 256
        0x0B,                                   # Background color index
        0x0A,                                   # Pixel aspect ratio - 10:11
        # Global colours palette
        0xFF, 0x00, 0xFF,   # Blue
        0x00, 0xFF, 0xFF,   # Green
        0xFF, 0xFF, 0x00,   # Red
    ])

    # One block of image data containing a single pixel of each possible colour
    frame_data = bytes([
        # Image descriptor block
        0x2C,                           # Length of image descriptor block
        0x00,                           # Left position LSB
        0x00,                           # Left position MSB
        0x00,                           # Top position LSB
        0x00,                           # Top position MSB
        0x01,                           # Width LSB
        0x00,                           # Width MSB
        0x01,                           # Height LSB
        0x00,                           # Height MSB
        0x00,                           # Flags - lsb = local colour table
        0x00,                           # Colour index of first local colour
        0x00                            # LZW minimum code size
    ] + [
        0x00, 0x00, 0x00,               # Blue
        0x00, 0x00, 0x00,               # Green
        0x00, 0x00, 0x00,               # Red
    ] * 256)

    # The final result is just the header and one frame repeated n times where n
    # is determined by the length of the input data to this function.
    repeats = len(sys.stdin.buffer.read()) // len(frame_data)
    out.write(os.urandom(repeats))