#!/usr/bin/python3

import struct
import secrets as scsl
from typing import BinaryIO


# The magic string for an animated gif
MAGIC = 'GIF87a'.encode()

# The block type used to mark image sections in the main header
IMAGE_BTYPE = 0x2C

# The block types that are allowed inside the local headers
LOCAL_BTYPES = (
    0xFE, # Extension Block
    0x01, # Plain Text Extension Block
    0xFF, # Application Extension Block
)


class LocalHeaderFlags():

    INTERLACED     =  0b10000000
    COLORMAPPED    =  0b01000000
    USED           =  0b00100000
    REVISION       =  0b00010000
    TRANSPARENT    =  0b00001000



def make_local_header(
        width :int, height :int, flags :LocalHeaderFlags=0
    ) -> bytes:

    return struct.pack('<6sHHBBB', MAGIC,
        width, height,
        0xF0 | IMAGE_BTYPE, flags, 0x00, 0x00
    )


def get_colormap() -> bytes:

    return struct.pack('BBB', *scsl.randbits(8) for _ in range(3))

    

if __name__ == '__main__':

    with open("test-random.gif", "wb") as f:

        f.write(make_local_header(450,120, ImageFlags.COLORMAPPED))
        colormap = get_colormap()
        f.write(struct.pack('<BBBBBB',
            0x00, 0x00, ord(colormap[0]), ord(colormap[1]), ord(colormap[2]), 0x00
        ))