#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    header = b'GIF89a'
    screen = b''

    # width and height are 16 bits each
    screen += struct.pack('>H', rng.randint(200, 500))
    screen += struct.pack('>H', rng.randint(200, 500))

    # flags field: packed
    flags = 0x00

    if rng.choice((True, False)):
        # global color table flag set
        flags |= 0x80

        # disposal method
        disp = rng.getrandbits(3)
        if disp == 2:
            disp = 0
        elif disp == 3:
            disp = 1
        flags |= disp << 2

        # sort order
        sort = rng.getrandbits(1)
        flags |= sort << 4

        # reserved fields
        flags |= (0x03 & rng.getrandbits(2)) << 5

    else:
        # local color table flag clear
        flags &= ~0x80

        # reserved fields
        flags |= (rng.getrandbits(2) | 0x01) << 5


    # pack it up
    header += bytes([flags])
    header += screen[0:3]

    # color table (RGB triplets)
    palette = []
    for i in range(rng.randint(1, 7) * 3):
        palette.append(rng.randint(0, 255))
    header += bytes(palette)

    # add gce block
    if flags & 0x80:
        gce = '\xF9\x04'
        gce += chr(flags).encode()
        gce += bytes.fromhex('FFFF')
        gce += b'\x00'
        header += bytes.fromhex('21F904002C0000000000')

    # write out
    header += header[-4:]
    out.write(header)

