#!/usr/bin/env python3

import math
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The bytes that make up the rest of the header
    header_remainder = [
            0x21, 0xF9, 0x04, 0x05,
                0x67, 0x00,

            0xFF, 0x01, 0x0A, 0xF1,
                ((0x20 | (0b1 << 1)) & 0xFF),

            ((0xFFC0 | (32 >> 1)) & 0xFF),
                (((32 >> 1) - 1) & 0xFF),

                0x00, 0x00,

                    ((0xFFC0 | (32 >> 1)) & 0xFF),
                        (((32 >> 1) - 1) & 0xFF),

                        0x00, 0x00]

    frame_count = 5
    palette_size = 1

    for _ in range(frame_count):

        width = max((int)(math.exponweib(1)), 1) * \
                    min((int)(math.exponweib(1)), 1)

        height = max(4, width)

        header_remainder += [
                    0xFF, 0x02,

                        (width % 256) & 0xFF,
                            (width // 256) & 0xFF,

                                (height % 256) & 0xFF,
                                    (height // 256) & 0xFF,

                (palette_size << 4) | 0x00,
                    0x00, 0x00, 0x00,

                    (palette_size << 4) | 0x00,
                        0x00, 0x00, 0x00]


        delay = (int)(math.lognormvariate(1, 1)) * 10

        header_remainder += [
                    0xFF, 0x01,

                        (delay % 256) & 0xFF,
                            (delay // 256) & 0xFF,

                    0xFF, 0x00]

    imagedata = rng.read()

    while len(imagedata) < 5000:
        imagedata += imagedata[:len(imagedata)//2]


    head = [
        0x47, 0x49, 0x46, 0x38, 0x39, 0x61,

        0x00, 0x00, 0x00, 0x00, 0x00, 0x00]

    head[3:] = list(struct.pack('>h', len(head)-6+len(header_remainder)+len(imagedata)))

    out.write(bytes([i for i in head]))
    out.write(bytes([i for i in header_remainder]))
    out.write(imagedata)

