#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# 
# The resulting file will have between 50% and 5 times
# the requested amount of bytes ($size) of data that follows
# the GifHeader.
def generate_random_gif(
        size: int, rng: BinaryIO, out: BinaryIO
    ) -> None:

    if not (2*len(GifMagic) < size):
        raise Exception("Too little space")

    # Pick some random data length
    datalen = len(GifMagic) + \
              int((size - len(GifMagic))/4)*4 + \
              [1, 2][int(rng.read(1)[0] % 2)]

    # Write all data
    out.write(b'\x47\x49\x46\x38\x39\x61')          # 'GIF89a'
    out.write(datalen.to_bytes(3, byteorder="big")) # DataLen
    out.write(GifMagic[:len(GifMagic)-1])           # Magic start
    out.write(rng.read(datalen-len(GifMagic)+1))     # Random content
    out.write(b'\33')                              # EOF