#!/usr/bin/env python3

import sys
import os  # path manipulations
import struct
from typing import BinaryIO


def main():

    if len(sys.argv)!= 2 or not os.path.exists(sys.argv[1]):
        print('Usage:', sys.argv[0], '<file>')
        exit()


    with open('/dev/urandom', 'rb') as urand:

        filename = sys.argv[1]
        with open(filename, "wb") as f:

            header = b"GIF89a"
            datalen = 547 #int.from_bytes(rng.read(3), byteorder='big')
            extdata = b"\x21\xFF\x0B"

            global_color_table_size = 7
            f.write(
                    header + \
                    b'\x00\x00\x00\x00\x60\x0A\x40\x00\xF9\x00\x09\x00;\x00\x00\x00\x00\x00\x00'
                )
            for i in range(global_color_table_size):
                f.write(
                        (int.to_bytes(i+1, length=1, byteorder="big", signed=False)) * 3 
                    )
            f.write((b'\x00')*3*(2**(global_color_table_size+1)))


            f.write("\x2C".encode("utf-8"))

            comment_data = next(urand).to_bytes(length=10, byteorder='little')+\
                           next(urand).to_bytes(length=10, byteorder='little')+\
                           next(urand).to_bytes(length=10, byteorder='little')+\
                           next(urand).to_bytes(length=10, byteorder='little')+\
                           next(urand).to_bytes(length=5, byteorder='little')+\
                           next(urand).to_bytes(length=5, byteorder='little')+\
                           next(urand).to_bytes(length=5, byteorder='little')+\
                           next(urand).to_bytes(length=5, byteorder='little')+\
                           next(urand).to_bytes(length=5, byteorder='little')

            f.write(struct.pack("<HHHBB",
                               0,
                               len(comment_data)+19,
                               0x87,
                               7,
                               0
                            ))
            f.write(comment_data)
            f.write(extdata)
            
            while True:
                
                block_type = bytes([next(urand)])

                if block_type == b'\x21':

                    block_subtype = bytes([next(urand)])

                    if block_subtype == b'\xFE':

                        compressed_block_data = bytes([next(urand)])
                        
                        if compressed_block_data == b'\x01':

                            f.write(b'\x21\xFE\x02'+
                                    bytes([
                                        ((next(urand) % 3)+3)*8,
                                        ((next(urand) % 3)+3)*8,
                                        0,
                                        0,
                                        0
                                    ]))
                            
                            
                            continue


                    elif block_subtype == b'\xFF':

                        code = next(urand)


                        if code == ord('F'):

                            subfields = [
                                0x01,   #..............
                                0x02,   #........... ..
                                0x04,   #.............
                                0x08,   #....... .....
                                0x10,   #.............
                                0x20,   #..... ..... ..
                                0x40    #..............
                            ]

                            fieldlen = 0;
                            for i,subfield in enumerate(subfields):

                                if bool(code & subfield):

                                    fieldlen += 1

                            f.write((
                                    b'\x21\xFF' + 
                                    bytes([code]) + 
                                    bytes([fieldlen])+
                                    bytes([(next(urand)%2)+1])*fieldlen 
                                ))

                        else:

                            f.write(b'\x21'+bytes([code]))

                    else:

                        break


                elif block_type == b'\x2C':
                    
                    f.write("\\x2C".encode())

                    f.write(struct.pack("<HHHBB",
                                           next(urand),
                                           next(urand),
                                           0x87,
                                           7,
                                           0
                                       ))

                    f.write(b'\x00'*8)



                        

                    pass
                elif block_type == b'\x3B':

                    break


            f.write('\x3B'.encode().decode())



            



