#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate a random GIF given an input stream of bytes to use for randomness and output stream to write to.
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Create the header
    out.write(b'GIF89a')

    # Get the length of the image from the user
    while True:

        try:
            width  = int(input("Width? "))
            break
        except ValueError:
            pass

    while True:

        try:
            height = int(input("Height? "))
            break
        except ValueError:
            pass

    print("Generating {}x{} GIF".format(width,height))

    # Print some fake info to the screen so the user can see what's going on without opening it up in another program.
    numcolors      = max([width, height]) * 2
    backgroundcolr = "0x{:02x}{:02x}{:02x}".format(*[int.from_bytes(rng.read(i),'big') for i in range(numcolors)])
    print("background color will be {}".format(backgroundcolr))

    # Write it all together
    out.write(struct.pack('>HHBBB', width, height, numcolors, 0x00, 0xff, 0xff, 0xff))
    out.write(struct.pack(">{}".format("B"*numcolors), *[int.from_bytes(rng.read(i),'big') for i in range(numcolors)]))
    out.write(struct.pack("><BB", *[int.from_bytes(rng.read(i),'big') for i in range(2)]))

    # LZW minimum size == 2
    mincodesize = 2
    out.write(mincodesize.to_bytes(1,'little'))
    
    # Image flags are packed 1 bit each with as follows:
    #   1 - Color table sort flag
    #   2-4 - GCT size (LZW minimum size)
    #   5   - Local colors sorted flag
    #   6   - Interlace flag (NOT USED)
    #   7   - Sort flag (NOT USED)
    #   8   - Reserved for future use

    # We set interlace off, color table sorted on, local colors sorted off,
    # and everything else off that we don't care about.
    gctsortflag = 1
    lctsortedflag = 0
    interlaced = False
    sortflag = 0
    reserved = 0
    imgflags = (gctsortflag << 7) + (lctsortedflag << 6) \
                + (interlaced << 5)     + (sortflag << 4) \
                + (reserved << 3)       + mincodesize
    out.write(imgflags.to_bytes(1, 'little'))