#!/usr/bin/env python3

import struct
from typing import BinaryIO
import zlib

MAX_DATALEN=2**24-1 # 16MiB - 1 byte

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The output is compressed with deflate compression (zlib).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    
    header = b'GIF89a'

    data = bytes()
    while len(data) < MAX_DATALEN:
        datalen = rng.randint(0, 512)
        newdata = rng.read(datalen)

        if len(newdata)!= datalen:
            break
        
        data += newdata
        
    compressed = zlib.compress(data)
    datalen = len(compressed)
    out.write(b''.join([ 
        header,
        struct.pack('>L', datalen),
        compressed
    ]))