#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO


# Generates a random GIF file into `out` by reading from
# /dev/urandom in chunks of size 65536 and writing it into `out`.
def generate_random_gif(out: BinaryIO):
    # First write the magic numbers to say that this is a
    # GIF file version 89a (the first GIF standard).
    out.write('GIF89a'.encode())

    while True:
        length = sys.stdin.buffer.read(1)[0]

        if length == 0x2c:
            # Start of frame descriptor. Read so many bytes
            # as specified into the output buffer.
            chunksize = sys.stdin.buffer.read(4)[::-1]
            chunksize = sum((b << shift for (shift, b)
                in enumerate(chunksize)))

            buf = sys.stdin.buffer.read(chunksize)
            out.write(buf)
        elif length == 0x21:
            # Start of extension block. Write some random junk there.
            length = sys.stdin.buffer.read(1)[0]
            sys.stdout.write(length)
            buf = sys.stdin.buffer.read(length)
            out.write(buf)
        else:
             # End of image. We're done!
            break