#!/usr/bin/env python3

import os
import sys
from io import BytesIO as IOReader
from typing import BinaryIO

# Generates a random GIF file using the
# given seed integer (or automatically fetched from /dev/urandom)
def generate_random_gif(seed=None):
    if not seed:
        with open('/dev/urandom', 'rb') as f:
            seed = int.from_bytes(f.read(4), byteorder='big')

    # Create an 16kb temp buffer to produce a valid GIF
    buf = [0] * (2 ** 14)
    
    for i in range(len(buf)):
        val = ((i << 5) ^ (((i >> 3) & 7) | (~((i >> 1) & 0xff)))) & 0xff
        index = val % i

        tmp = buf[index]
        buf[index] = buf[i]
        buf[i] = tmp
        
    header = bytes([
    0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 
    0x00, 0x3B, 0x00, 0xF5, 0xFF, 0xFF, 0xF3])

    # Encode LZW compression table info
    header += (struct.pack('!H', len(buf)) + 
               struct.pack('!H', len(buf)))

    # Perform the actual compression of the LZW datastream
    compressed = []
    codeword = 0
    prevbyte = None

    def append_codeword():
        nonlocal compressed
        nonlocal codeword
        
        bitflag = 0x80
        while bitflag > 0:
            compressed.append(
                (codeword & bitflag).to_bytes(
                    1, byteorder='big')[0])

            bitflag >>= 1

    for idx in buf:
        if prevbyte is None or prevbyte == idx:
            codeword <<= 1
            codeword |=  1 if idx else 0
        elif prevbyte!= b'\0':
            append_codeword()
            codeword = idx
        prevbyte = idx
        
    if prevbyte!= b'\0':
        append_codeword()

    header += bytes(compressed)
    header += b'\x3b' # EOF
    return header + b''.join(reversed(buf[:idx+1]))

