"""A tool to generates random GIF files."""

import sys
import argparse
from pathlib import Path
from itertools import cycle as cycler

# Header, followed by 0x21FF (extension introducer, label)
# Data length, followed by data
# Frame control block (global color table size set to 16 colors,
# transparent index set to first color, no LZW compression)
# Image descriptor (left position set to 0, top position set to 0,
# width set to data length, height set to 1 pixel, reserved bit field is set
# to all bits off, local color table flag and interlace flags are both
# turned on). Subsequently, the image data follows, followed by an end-of-image code.
# Extension block (subblock ID set to 254, length set to 4, the graphics control
# block follows):
#   - The delay in milliseconds is set to zero. This means that there will be no pauses between frames of this animation. It should be noted that this doesn't mean that there won't be any animated transitions between these frames; they merely won't have any pauses between them.
#   - No transparancy used.
# Graphics Control Block (delay time set to zero, transparent color index set to first color)
# Plain text extension subblock (application id set to "NETSCAPE", application authentication code set to empty bytes, subblock priorities set to all ones, loop count set to two hundred five times, subblock terminator set to 00).
# Comment extension subblock (comment set to the string "I've been meaning to tell you this for so long").
# Trailer
HEADER = b"GIF89a\x21\xFF\x0B\x4E\x45\x54\x53\x43\x41\x50\x45" \
         b"\x32\x2E\x30\x03\x01\x00\x00\x00"
DATALENGTHBYTES = 3
DATASTARTBYTE = HEADER.__len__()
TRANSPARENTINDEX = 1
FRAMECONTROLBLOCKLENGTH = 7
FRAMECONTROLBLOCK = b'\x2C\x00\xAE\x02\x00\x1F\x00\x00'
LEFTPOS = FRAMECONTROLBLOCK[0]
TOPPOS = FRAMECONTROLBLOCK[1]
WIDTH = FRAMECONTROLBLOCK[3]
HEIGHT = FRAMECONTROLBLOCK[4]
LEFTEXTENSIONSUBBLOCKID = 0xF9
LEFTEXTENSIONSUBBLOCKLEN = 0x04
GRAPHICSCONTROLBLOCKBEGIN = LEFTEXTENSIONSUBBLOCKID+4
GRAPHICSCONTROLLENGTH = 0x04
GRAPHICSCONTROLBLOCKEND = GRAPHICSCONTROLBLOCKBEGIN+GRAPGHICSCONTROLLENGTH
COMMENTEXTENSIONSUBBLOCKID = 0xFE
COMMENTEXTENSIONSUBBLOCKLEN = 0x0B
COMMENTEXTENSIONSUBBLOCKTERMINATOR = 0x00
TRAILER = COMMENTEXTENSIONSUBBLOCKID

def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("outputpath")
    args = parser.parse_args()
    
    outputpath = Path(args.outputpath)
    if not outputpath.is_file():
        print('The specified path does not exist or is not a file.')
        return 1

    with open("/dev/urandom", 'rb', buffering=0) as f,\
        outputpath.open('wb') as o:

        o.write(HEADER)
        datalength = struct.pack(">L", f.read(DATALENGTHBYTES))
        o.write(datalength)
        o.write(f.read(datalength[0]))
        o.write(FRAMECONTROLBLOCK)
        o.write(b'\x21\xF9\x04\x00\x00\x00\x00') # left image descriptor
        o.write(FRAMECONTROLBLOCK)
        o.write(b'\x21\xFC\x00\x00\x00\x00\x00') # right image descriptor
        o.write(FRAMECONTROLBLOCK)
        o.write(b'\x21\xFD\x09\x00\x00\x00\x00'+
                b'\x00\x00\x00\x00\x00\x00\n') # graphic control extension
        o.write('\x21\xFE\x0B'.encode())      # comment extension
        o.write('NETSCAPE'.encode()+
                b'\x03\x01\x00\x00\x00\x00\x00\x00\x00\x00\n') 
        o.write('\x21\xFF'.encode())          # application extension
        o.write('\x01\x0F\xFF'.encode())     # plain text extension
        o.write('Comment'.encode()+b'\x00')   # comment
        o.write(b'\x3B')                     # trailer

    return 0

if __name__ == '__main__':
    sys.exit(main())