#!/usr/bin/env python3

import math
import time
from io import BytesIO

# Generates a random GIF file with specified average length `mean_datalen`.
def gen_random_gif(mean_datalen):
    # Create our RNG by seeding from /dev/urandom
    rng = open('/dev/urandom', mode='rb')

    # Generate a random datalen in the range (0; mean_datalen]
    datalen = rndint(rng) + 1
    while datalen <= 0 or datalen > mean_datalen:
        datalen = rndint(rng) + 1

    print("Generated file with {:.2f}KB of actual content.".format((datalen / 1024)))

    # Now we know how big the final file must be, let's write our header...
    buf = BytesIO()
    buf.write(b"GIF89a")   # Signature
    buf.write(b"xxxx")     # Width x Height
    buf.write(b"\xff\x0bNETSCAPE2.0\x03\x01\x00\x00")   # Extension block
    buf.write(b"\x00")      # Transparency Flag=False
    buf.write(b"\xff\x00")  # Loop count=Infinite

    remaining = mean_datalen - len(buf.getvalue())
    if remaining < 0:
        raise Exception("Internal error!")  # Should never happen!

    #... then body until we get enough data to fill up our desired length
    while True:

        # Add more data if this would make us exceed the target datalen
        nextlen = int(math.exp(-remaining * math.log(rndint(rng))))
        if nextlen == 0:
            continue
        
        buf.write(nextlen.to_bytes(1, "big"))
        remaining -= 1

        if remaining <= 0: 
            break

    return buf.getvalue()

def rndint(rng):
    return int.from_bytes(rng.read(1), byteorder="big", signed=True)