#!/usr/bin/env python3

import argparse
import os
import sys
from PIL import Image


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-x', '--width', type=int,
                        default=None, help="Width of output image")
    parser.add_argument('-y', '--height', type=int,
                        default=None, help="Height of output image")
    parser.add_argument('-c', '--color-mode', type=str,
                        choices=['L', 'P'], default='L',
                        dest='colormode', help="Color mode ('L': black "
                                               "and white or 'P': palette)")
    parser.add_argument('--colors', type=int,
                        default=256, help="Number of colors to use when "
                                           "'--color-mode P' is specified.")
    parser.add_argument('--bgcolor', nargs='+', type=int, default=None,
                        metavar=('R G B'),
                        help="Background color if not transparent; specify"
                             "as R, G, B values from 0-255.")
    return parser.parse_args(sys.argv[1:])


def main():
    args = parse_args()

    if None in [args.width, args.height]:
        raise ValueError("Both --width and --height must be provided")

    rng = open("/dev/urandom", "rb")
    img = Image.new(args.colormode, size=(args.width, args.height),
                    color=tuple(args.bgcolor) if args.bgcolor else None)

    imdata = bytearray()
    while len(imdata) < (img.width * img.height):

        if args.colormode == 'L':
            bgr = struct.unpack('BBB', rng.read(3))
            lum = ((bgr[0] + bgr[1] + bgr[2]) // 3)
            imdata.extend([lum])

        elif args.colormode == 'P':
            pnum = min((len(img.getpalette()) // 3) - 1, args.colors - 1)

            plidx = struct.unpack('<B', rng.read(1))[0] % pnum
            rgb = tuple(img.getpalette()[plidx*3+i] for i in range(3))
            imdata.extend(rgb)

    img.putdata([x for x in imdata[:]])
    img.save("-", format="GIF")


if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        print("Error:", str(e), sep='\n')
        exit(-1)