#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. Length (1 byte) for image descriptor
#   3. Width (2 bytes, little endian).
#   4. Height (2 bytes, little endian).
#   5. Zero byte for flags.
#   6. A randomly-chosen color table index (1 byte).
def generate_gif(out: BinaryIO):
  out.write('GIF89a'.encode())
  out.write(b'\x07') # Image descriptor length
  out.write(struct.pack("<H", 1))
  out.write(struct.pack("<H", 1))
  out.write(b"\x00") # Flags
