#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)
#     print('lsd', bytes(lsd).hex())
    width  = int.from_bytes(lsd[0:2], 'little') + 1
    height = int.from_bytes(lsd[2:4], 'little') + 1
#     print('width', width, 'height', height)
    packed_byte = lsd[4]
    flags = packed_byte & 0x7

    # Add LSD field to output
    out.write(lsd[:6])

    if (flags == 0):
        out.write(b'\x00\x3B')
    else:
        ext = rng.read(1)[0]

        while True:
            # End of extensions
            if (ext == 0x00):
                break

            # Plain text extension
            elif (ext == 0xF9 or ext == 0xFE):
                size = rng.read(1)[0]

                if ((size!= 12) and
                    (size!= 12+int((packed_byte >> 3)&1))
                   ):
                    continue

                subblock = bytearray()
                
                for i in range(0, 1+int(((packed_byte>>3)&1)*8), 1):
                    subblock += rng.read(1)

                out.write(struct.pack('<cBBBHB',
                                    ext.to_bytes(1, 'big'),
                                    1+len([i for i in subblock]),
                                    *subblock
                                   ))

            # Graphic control extension
            elif (ext == 0xF9):
                size = rng.read(1)[0]

                if (size!= 4):
                    continue

                block = bytearray([
                    2,
                    rng.read(1)[0] | 0x80,
                    rng.read(1)[0],
                    rng.read(1)[0],
                    rng.read(1)[0]
                ])

                out.write(struct.pack('<cBBBHB',
                                      *block
                                     ))
            
            # Application Extension
            elif (ext == 0xFF):
                flags = rng.read(1)[0]

                if (ext == 0x01):
                    continue   # Skip "NETSCAPE" application extension

                if (flags&0xF != 0):
                    continue

                size = rng.read(1)[0]

                subblock = bytearray()

                for i in range(0, 1+int(((flags>>3)&1)*8), 1):
                    subblock += rng.read(1)

                out.write(struct.pack('<cBBBHB',
                                    ext.to_bytes(1, 'big'),
                                    1+len([i for i in subblock]),
                                    *subblock
                                   ))

