#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # create header with random size
    width = struct.unpack('<L', rng.read(4))[0]
    height = struct.unpack('<L', rng.read(4))[0]

    header = (
            b'GIF89a'           # magic
            b'\x01\x00'         # screen desc width=1, height=0
            b'\x80\xff'         # global color table (red, green, blue)
            b'\x01\x00'         # background and aspect ratio
            )

    assert len(header) == 13

    datalen = width * height
    data = rng.read(datalen)
    out.write(header + data)
    print(f"generated {len(data)}-byte image")

    # seek back to beginning of file
    out.seek(0, 0)

