#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    # The last byte of the GIF file must not be an
    # image separator byte (0x2c or 0x21)
    while True:
        footer = rng.read()
        if len(footer) == 0 or footer[-1] & 0xf7 != 0x2c and \
                    footer[-1] & 0xf7 != 0x21:
            break

    assert(len(data))

    # Remove all but the first six frames
    data = [s for s in struct.iter_unpack('<H', data)
            if int.from_bytes(
                rng.read(1), 'big') < 6]

    out.write(header + len(data).to_bytes(2, 'big') + \
            data + footer)
