#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width, height = 20 * 12 // 16, 5 * 8 // 16
    color_table = [struct.pack('BBB', *rng.read(3)) for _ in range(256)]
    frame_delay = 0.1
    transparent = True
    disposal = 0 if transparent else 2
    loop = 0 # No looping of animation
    comment = f"Random GIF\nGenerated with {__file__}"
    loop_count = len(comment.encode())

    def write_header():
        out.write(b'GIF89a')
        out.write(_int_to_bytes(width))
        out.write(_int_to_bytes(height))
        out.write(b'\xF7\x00\x00')
        out.write(color_table[0])
        out.write(b'\x00;')

    def write_frame(left=0, top=0):
        out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
        out.write(_int_to_bytes(loop_count))
        out.write(b'\x00')
        out.write(_int_to_bytes(disposal << 2 | transparent))
        out.write(_int_to_bytes(frame_delay * 100))
        out.write(b'\x2C')
        out.write(_int_to_bytes(left))
        out.write(_int_to_bytes(top))
        out.write(_int_to_bytes(width))
        out.write(_int_to_bytes(height))
        out.write(b'\x01')
        out.write(b','.join([