#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + bytes([0] * 5) # 24 byte header
    signature = bytes([0xFF, 0xFE])
    eoi = bytes([0x00])
    def get_byte():
        return int.from_bytes(rng.read(1), 'little') % 256
    def get_word():
        low = get_byte()
        high = get_byte()
        return (high << 8 | low) & 0xFFFF
    def get_int():
        return get_word() + (get_word() << 16)
    # sizeof(color table entry) == 3 per color
    palette = [
        0 for _ in range(0x100*3)
    ]
    for i in range(len(palette)):
        palette[i] = get_byte()
    out.write(header + signature)
    while True:
        block_type = get_byte()
        if block_type == 0x2C:
            version = rng.read(1) # unused
            width = get_word()
            height = get_word()
            flags = rng.read(1)[0] # unused
            
            left = get_word() # x pos
            top = get_word() # y pos
            width = get_word() # width
            height = get_word() # height
            color = get_byte()
            interlaced = rng.read(1)[0]
            lzwmincode = get_byte()
            out.write(struct.pack('<BHBBHHB', block_type, width, height, flags, \
                            left, top, color, interlaced))
            while True:
                data = rng.read(1)
                if len(data) < 1: break
                out.write(data)
        elif block_type == 0x3b:
            out.write(eoi)