#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Read global color table if it exists
    if gctlen > 0:
        gct = rng.read(gctlen)
        out.write(gct)

    # Read the application extension record (AER)
    appext = rng.read(11)

    # Add AER to output, without the terminating null byte
    out.write(appext[:-1])

    # Parse the block size from the AER
    blocksize = int.from_bytes(appext[:2], 'big')

    # Read the width/height of image from the app data subblock
    w, h = struct.unpack('>HxHH', appext[6:-1])
