#!/usr/bin/env python3

import os
import sys

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng`
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(5) + bytes([2])

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gct_length = 1 << ((int.from_bytes(lsd[4:], 'big') & 0b00000111) + 1)

    # Generate colors for the global color table
    if gct_length > 0:
        for _ in range(gct_length):
            out.write(rng.read(3))

    # Write data sub-blocks until end of image block is encountered
    start_of_image_found = False
    while True:
        # Set up stream with size information for data block
        data_block_size = int.from_bytes(rng.read(1), 'big')

        # If this is an end of image indicator, break out
        if data_block_size == 0:
            break
        
        # Add the data block verbatim to the output
        out.write(rng.read(data_block_size))
