#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        gct_size=256, loop_count=0xFFFF) -> None:
    # See https://www.w3.org/Graphics/GIF/spec-gif89a.txt for details
    # of the various GIF sections
    out.write([71, 73, 70, 56]) # Magic header

    out.write(struct.pack("<B", 3 << 4 | 3)) # 8-bit color, 3 cols
    out.write(b"!\xF9\x04\x04") # Graphics control extension
    out.write(struct.pack(">H", loop_count)) # Repeat count

    out.write(struct.pack("BB", 44, 0)) # Image descriptor

    # Write out the global color table
    out.write([0] * (gct_size * 3))
    for _ in range(gct_size):
        out.write(bytes([rng.read(1)[0]] * 3))

    # Write out a dummy image that is all black
    width = 1
    height = 1
    out.write(struct.pack("<HH", width, height))
    out.write(([0] * (width * height)).