#!/usr/bin/env python3

import os
import sys

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng`
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(5) + bytes([2])

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have one
    if gctlen > 0:
      gct = rng.read(gctlen)

      out.write(gct)
      
    # Get background color index
    bgidx = int(lsd[4] & 7)

    # Write an extension block on top of the GIF file
    extblock = bytes([0x21, 0xF9, 0x04, 0x04, (bgidx << 4), 0xFF, 0])
    out.write(extblock)

    # For each frame, we need to do the same thing as above but with some more
    # things thrown in for good measure.
    while True:
        
        # Read the next byte and if it's EOF then break out of loop
        fieldbyte = rng.read(1)[0]
        if (fieldbyte == '\x00'):
          break

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream as they are
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:

          # We can only handle up to 256 colors so just fake that many and skip
          # over the rest
          if numcolors > 256:
            numcolors = 256

          colortable = rng.read(numcolors*3)
          out.write(colortable)

          # Skip over the rest of the colors in our palette since we only support
          # 256
          leftovercolorbytes = max((numcolors - 256)*3, 0)
          if leftovercolorbytes > 0:
              rng.seek(leftovercolorbytes, 1)
              
        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)