#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. 2 bytes of length of block (N).
#   3. N bytes of block.
#   4. Repeat from step 2 for each remaining byte in data.
def gen_gif(data, out):
    magic = b"GIF89a"
    nbytes = len(data)
    while True:
        if nbytes == 0:
            break
        ndata = min(nbytes, 256)
        nbytes -= ndata
        # Write length of block
        out.write(struct.pack("B", ndata))
        # Write block
        out.write(data[:ndata])
        data = data[ndata:]
    # Write magic number at end of stream.
    out.write(magic)

def read_gif(inp : BinaryIO)-> str:
    ndata = 0
    s = ""
    while True:
        buf = inp.read(1)
        if not buf or buf == b"G":
            break
        elif buf == ord('!'):
            ndata = 0
            continue
        else:
            ndata += 1
            assert ndata < 256
            s = chr(buf[0]) + s
    return s