#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu May 14 09:25:47 2020

@author: danielmachado
"""
import os
os.chdir('/home/danielmachado/Dropbox/PhD/projects')
from virology_data import VirologyData, TaggedVirus
from virology_utils import VirologyUtils as Utils
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
###
plt.style.use('seaborn-poster')
plt.rcParams['font.family'] = 'serif'
plt.rcParams["figure.figsize"] = (6, 3)
plt.rcParams["axes.linewidth"] = 0.5
plt.rcParams["lines.linewidth"] = 0.5
plt.rcParams["xtick.major.width"] = 0.5
plt.rcParams["ytick.major.width"] = 0.5
plt.rcParams["grid.alpha"] = 0.2
plt.rcParams["legend.frameon"] = True
plt.rcParams["legend.facecolor"] = [1, 1, 1]
plt.rcParams["legend.edgecolor"] = [0.5, 0.5, 0.5]
plt.rcParams["legend.fancybox"] = False
plt.rcParams["legend.fontsize"] = "medium"
plt.rcParams["legend.handlelength"] = 2
plt.rcParams["patch.force_edgecolor"] = True
plt.rcParams["patch.linewidth"] = 0.5
plt.rcParams["savefig.bbox"] = "tight"
plt.rcParams["savefig.transparent"] = True
plt.rcParams["text.usetex"] = False
plt.rcParams["xtick.direction"] = "in"
plt.rcParams["ytick.direction"] = "in"
#%%
virus = TaggedVirus(name = 'SARS-CoV-2',
                    adsorbed=True,
                    tcell_reactive=False,
                    humoral_reactive=False,
                    duration=[np.inf],
                    incubation=None,
                    peak_day=0,
                    vdrift=None,
                    sdrift=None,
                    virus_copies_to_floating_proportion=[0.1])

prevalence = {virus: 0.01}
initial_conditions = {virus: 0.0001}
#%%
infection_model = Utils.get_SIR_based_model(viruses=list(prevalence.keys()),
                                            prevalence=prevalence,
                                            initial_conditions=initial_conditions)[0]

#%%
population = 10**6
time = np.linspace(-2, 25, 100)
solution = infection_model.integrate(time, population)

#%%
fig, ax = plt.subplots()