#!/usr/bin/env python3

import struct
from typing import BinaryIO

def write_varlen(stream: BinaryIO, value):
    """Write a variable-length integer to the stream."""
    while True:
        byte = (value & 0x7F) | 0x80
        value >>= 7
        if not value:
            # The last byte has bit 7 set to zero.
            byte &= 0x7F
        stream.write(struct.pack('B', byte))
        if value == 0: break

def read_varlen(stream: BinaryIO) -> int:
    """Read a variable-length integer from the stream."""
    i = 0
    while True:
        b = ord(next(iter(stream)))
        assert b >= 0x80 and b <= 0xFE
        i <<= 7