#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write(b"GIF89a")

    width = 1 + (rng.randint(0, 256) % 47) * 3
    height = 1 + (rng.randint(0, 256) % 25)

    # Logical screen descriptor
    out.write(struct.pack("<HHBBB", width, height, 0xF0, 2 << 4 | 0, 0))

    # Global color table with black and white
    for i in range(256):
        if 1 == i % 8 or not i:
            r = g = b = ((i & 0x8) >> 1) * 0xFF
        else:
            r = g = b = 0x2A

        out.write(struct.pack("BBBB", r, g, b, 0))

    # Image descriptor
    palette_length = 1 << 4 | 0
    left = top = 0
    data_size = width * height // 8
    local_color_table = False
    interlace = 0
    sort = 0
    reserved = 0
    lct_entries = 0
    out.write(struct.pack("<HHHHB", left, top, width, height,
                           palette_length << 4 | local_color_table << 3
                               | interlace << 2 | sort))
    out.write(b"\0\0")
    out.write(struct.pack(">L", data_size))

    # Encode frame as all zeroes - why? Because, well...
    buffer = bytearray((width * height // 8) + 1)
    buffer[0] = 0x2C
    for _ in range(data_size):
        buffer[_] = rng.randint(0, 2) ^ 1
    buffer[-1] = 0x00

    out.write(buffer)

