#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big') & 0x1FFF # Limit at 8192 pixels wide

    frames = []
    for i in range(datalen):
        frame = [
            7*rng.read(1)[0] % (256 * 7) + 1, # Color values are from 1 to 127 inclusive
            7*rng.read(1)[0] % (256 * 7) + 1,
            rng.read(4)[0],
            randint(0, 1)]

        # Padding of length 4 bytes
        rng.read(4) 
        frames += frame

    out.write(header)
    out.write((datalen).to_bytes(2, byteorder='big'))
    out.write(b'\x87\x00\x00')

    for f in frames:
        if len(f) == 4:
            out.write(struct.pack('<BBBB', *f))
        else:
            raise AssertionError("Frame does not have 4 elements")
