#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. 2 bytes of length of block (N).
#   3. N bytes of block.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='little', signed=False)

    # Each frame consists of:
    #    0x2c -> Image separator ("!").
    #    0x00 -> Left position (X).
    #    0x00 -> Top position (Y).
    #    0x5f -> Width (W).
    #    0xb7 -> Height (H).
    #    0x00 -> Flags (F): 0x00 = no local color table or interlacing.
    #    0x00 -> Size of global color table (LCT size).
    #    0x00 -> Background color index (Bk).
    #    0x00 -> Pixel aspect ratio (Px/Py).
    #    LCT size * 3 bytes for global color table (color array).
    framedata = [
        0x2c, 0x00, 0x00, 0x5f, 0xb7, 0x00, 0x00, 0x00, 
        0x00, 0x00, 0x00] + list(rng.read(datalen)) + \
        ([0x00]*(2 << datalen[0]))
    
    while len(framedata) % 2 == 1:
        framedata += [0x00];
    
    data = [header] + framedata;
    out.write(struct.pack('>{}B'.format(len(data)), *data));
    