#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(4), byteorder='big', signed=False
    )
    data = rng.read(datalen)
    
    if len(data)!= datalen:
        raise ValueError("input stream not as long as expected")
    
    out.write(header +
            bytes([
                *struct.pack('<HHHHH',
                768, 512, 0b100000000000, 0x08, 0xff),
                *struct.pack('B', 0x00)]) + 
            data)
    
    print(f"generated {len(data)}-byte image")

    # seek back to beginning of file
    out.seek(0, 0)

