#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    logicalScreenWidth, logicalScreenHeight = (
        2**16 - 5 * rng.read(2),
        2 ** 16 - 5 * rng.read(2)
    )

    backgroundColourIndex = int.from_bytes(rng.read(1), 'little')
    
    pixelAspectRatio = int.from_bytes(rng.read(1), 'little')

    backgroundColorRGB = [int.from_bytes(rng.read(1), 'little') for _ in range(3)]

    flags = rng.read(1)[0] # Unused; must be set to zero

    globalColorTableFlag = bool(flags & 2**7)
    colorResolution = ((flags >> 4) & 0b111) + 1
    sortFlag = bool((flags >> 3) & 0b1)
    sizeOfGlobalColorTable = 2**(colorResolution + 1)


    globalColorTable = [backgroundColorRGB
                         ] if not globalColorTableFlag else \
                        [[
                            i % 0x100,
                            (i // 0x100) % 0x100,
                            (i // 0x10000) % 0x100
                        ] 
                        for i in range(sizeOfGlobalColorTable)
                        ]

    # Write Header
    out.write(header)
    out.write(struct.pack('<HH', logicalScreenWidth, logicalScreenHeight))
    out.write(backgroundColourIndex.to_bytes(1, "little"))
    out.write(pixelAspectRatio.to_bytes(1, "little"))
    out.write(globalColorTableFlag.to_bytes(1, "little"))
    out.write(((sortFlag << 3) | (colorResolution-1)).to_bytes(1, "little"))
    out.write(sizeOfGlobalColorTable.to_bytes(1, "little"))
    for row in globalColorTable:
        for c in row:
            out.write(c.to_bytes(1, "little"))

    # Write Image
    imageDescriptorBlockPrefix = bytes([
        0b11111100|7,
        logicalScreenWidth % 256,
        logicalScreenHeight % 256,
    ])
    out.write(imageDescriptorBlockPrefix)
    # seek back to beginning of file
    out.seek(0, 0)

