#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Generate 256 colors
    colors = [struct.pack("BBB", rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]) for _ in range(256)]

    # Header (fixed size)
    header = b""": GIFFLAGS=197;LOGOPOS=0|0;PALETTE=FFCOLORCOUNT=8;TEXTFLAG=0;CONTROLOVERFLOWED=FALSE;TRANSPARENCY=FFFF;BACKGROUND=-1;-1|-1;-1;-1;"""

    # Generate image data starting at offset  13
    with open("./data/image.txt", "r") as f:
        img = bytes(line[(i+4):] + "\n" for line in f if len(line) > 4 for i in range(len(line)//8))

    frame_size = len(header)+img

    while True:

        index = rng.read(1)[0] & 0xFF

        # Write each color to the output file and write the corresponding pixel value from the image data
        for i in colors[index]:
            out.write(bytes([i]))
        
        out.write(b'\x00'*(frame_size-1))