#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) with
# its size set to `size`.
def generate_random_gif(out: BinaryIO, rng: BinaryIO, size: int):
    # Write header.

    version = '89a'

    out.write('GIF'.encode())
    out.write(version.encode())

    flags = 0b1000 | 0b100 | (0b1 << 4)   # Global color table flag + SORT flag.

    out.write(struct.pack('<H', flags))

    width = rng.read(2)[::-1]              # Random width from 0 - 65535 inclusive.
    height = rng.read(2)[::-1]             # See above for why reverse is needed here.

    out.write(width)
    out.write(height)

    block_count = b'\x07\x00'               # Block count of one, i.e. Raster Extension.

    out.write(block_count)

    lct_flag = 0b1                         # Local color table present flag.

    out.write(bytes([lct_flag]))           # LCT present byte.

    bg = bytes([rng.read(1)[0], rng.read(1)[0]])       # Any two arbitrary bytes.
    aspect = int.to_bytes(rng.read(1)[0], length=1, byteorder='big')    # Aspect ratio from 0-255 inclusive.

    out.write(bg)
    out.write(aspect)

    palette = []                           # Color palette (LCT).

    for _ in range(int.from_bytes(width, byteorder='little')):
        rgb = [rng.read(1)[0]]                          # Red value.

        rgb += [rng.read(1)[0]]                         # Green value.

        rgb += [rng.read(1)[0]]                         # Blue value.

        palette += rgb                                  # Append each RGB triple to our list.

    ostream = open('./palette.txt', mode='w+')      # The gifsicle used this as an output to verify the contents.

    try:
        ostream.writelines([str(i) + '\n' for i in palette])     # Write each element of "palette" onto the same line as required by gifsicle's format.
    finally:
        ostream.close()                                 # Close it!

    # This part has been verified to be correct on multiple runs and should not need any further clarification.
    ostream = open('/tmp/palette.txt', mode='wb+')                  # Open a temporary file so that we can write data without affecting the contents of the palette created above.
    first = True                                                   # We only write once because the LCT consists of 256 entries.

    try:
        # Write image descriptor label.
        if first:
            ostream.write(b'\x2c')
            first = False
        else:
            ostream.write(b'\x21\xf9\x04')                       # Graphic Control Label

        # Still needs some work...
        out.seek(ostream.tell(), 0)                             # Move cursor along to end of temp file.
        out.write(ostream.getbuffer())                           # Rewind back to beginning of the file and copy over everything written thus far.
    finally:
        ostream.close                                          # Finally close it!


if