#!/usr/bin/env python3

import os
import sys

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng`
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
     

    numimages = rng.read(1)[0] & 0x7F

    for i in range(numimages):
      hasimgdata = (rng.read(1))[0]&0x80!= 0

      # Read the LZW minimum code size then use it as we read through 
      # the LZW encoded stream
      lzwsizesize = (rng.read(1))[0]+1
      mincodesize = ((rng.read(lzwsizesize)))[0] + 1
      
      while True:
        # This is an escape sequence before each new chunk
        if int.from_bytes(rng.read(1), byteorder='little') == 0xFF:
          
          break

        # Else this is the start of a new subchunk
        else:
          # Get the type information 
          subchunkid = rng.read(4).decode()
          subchunksize = int.from_bytes(rng.read(4), byteorder="little")

          print("SubChunk ID:", subchunkid, "Size:", subchunksize)
          
      if hasimgdata:
        # Start writing the header now that we know how large the image will be
        size = rng.read(1)[0] | rng.read(1)[0] << 8

        out.write(struct.pack('=B', size))

        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:

          # We can only handle up to 256 colors so just fake that many and skip
          # over the rest
          if numcolors > 256:
            numcolors = 256

          colortable = rng.read(numcolors*3)
          out.write(colortable)

          # Skip over the rest of the colors in our palette since we only support
          # 256
          leftovercolorbytes = max((numcolors - 256)*3, 0)
          if leftovercolorbytes > 0:
              rng.seek(leftovercolorbytes, 1)
              
        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)