#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # The rest is just copied from an image I found on the internet.
    body = (
            0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 0x2c, 0x00, 
            0x21, 0xFF, 0x0B, 0x4E, 0x45, 0x54, 0x53, 0x43, 
            0x41, 0x50, 0x45, 0x32, 0x2E, 0x30, 0x03, 0x01, 
            0x00, 0x00, 0x00, 0x2C, 0x00, 0x00, 0x00, 0x00, 
            0x02, 0x00, 0x01, 0x00, 0x00, 0x02, 0x02, 0x44, 
            0x01, 0x00, 0x3B, 
        ) + \
        len(b"Mona Lisa").to_bytes(length=1, byteorder="little") + \
        b"Mona Lisa" + \
        0x00.to_bytes(length=1, byteorder="little")
    
    trailer = bytes(
        [
            0x3B, 
            len(data).to_bytes(length=1, byteorder="little"), 
            *struct.unpack('>H', datalen.to_bytes(length=2, byteorder="big")),
            0x00, 
            0x3B
        ]
    )

    gif = header + body + data + trailer

    out.write(gif)

