#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output.
    print('Generating header...', end='', flush=True)

    magic = bytes([0x47, 0x49, 0x46])

    out.write(magic)

    width = int.from_bytes(rng.read(2), byteorder='big')
    height = int.from_bytes(rng.read(2), byteorder='big')

    w = b'\x00' if (width >> 8 == 0) else \
           b'\xff' if (width > 0xFF) else \
             width.to_bytes(1, 'big', signed=False) 

    h = b'\x00' if (height >> 8 == 0) else \
           b'\xff' if (height > 0xFF) else \
             height.to_bytes(1, 'big', signed=False)  

    out.write(w + h)

    flags = bytes([0x80 | rng.read()[0] & ~0x40])
    bg = bytes([rng.read()[0]])
    aspect = bytes([rng.read()[0]])
    
    print('done')

    out.write(flags + bg + aspect)

    # Generate each frame in turn.
    for _ in range(0x5):

        # Print what part of generation is happening.
        if (_ % 0x5 == 0):
            d = '\rWriting frames...' if (_!= 0) else\
                'Writing first frame...'.ljust(25)

            print(d, end='', flush=True)

        # Write the delay time.
        dt = b'\x00\x00'
        while(dt[0] >= 0xF0 or dt[-1] < 0xA):
            dt = rng.read(2)
        
        out.write(dt)

        # Write the lzw min code size.
        mcs = rng.read()[0] & 0x0F 
        mcs |= 0x01  # This must be set.

        out.write(mcs.to_bytes(1, 'big'))

        # For each row of pixels...
        for y in range(0, height):
            
            #...and each column, write the packed value.
            for x in range(0, width):

                pixel = rng.read()
                
                out.write(pixel[:-1])

                c = ord(pixel[-1:])

                if ((c & 0x80)):
                    out.write((c ^ 0xFF).to_bytes(1, 'little'))

    print('\ndone.')
