#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(b'\xff'*4, 'big')
    appext = b'\x21\xFF\x0BNETSCAPE2.0'

    # Write standard GIF header and application extension header
    out.write(header + (datalen).to_bytes(4, byteorder='big'))
    out.write((len(appext)).to_bytes(1, byteorder='big') + appext)

    # Use this to write the actual image data, 7-bit encoded
    def _outputBits(val):
        if val == -1:
            return

        for i in range(7,-1,-1):
            bit = ((val >> i) & 1!= 0)
            c = struct.pack('c', bytes([int(bit)]))[0]
            out.write(struct.pack('c', c))

    # Write the NETSCAPE color table
    ctsize = 1 << 6
    colortable = [(i*ctsize//(ctsize+1), i*(255-ctsize)//(ctsize+1), 0)
                  for i in range(0, ctsize)]
    out.write(colortable[0][0].to_bytes(1,byteorder='big')
              + colortable[0][1].to_bytes(1,byteorder='big')
              + colortable[0][2].to_bytes(1,byteorder='big'))

    # 0
    _outputBits(-1)

    # 1: Loop count; 0 means infinite loop
    out.write(((0).to_bytes(1, byteorder='big')))

    # 256 colors used
    _outputBits(2)

    # Termination
    _outputBits(0)

