#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)
#     print('lsd', bytes(lsd).hex())
    width  = int.from_bytes(lsd[0:2], 'little') + 1
    height = int.from_bytes(lsd[2:4], 'little') + 1
#     print('width', width, 'height', height)
    packed_byte = lsd[4]
    flags = packed_byte & 0x7

    # Add LSD field to output
    out.write(lsd[:6])

    if (flags == 0):
        out.write(b'\x00\x3B')
    else:
        ext = rng.read(1)[0]

        while True:
            # End of extensions
            if (ext == 0x00):
                break

            # Plain text extension
            elif (ext == 0xF9 or ext == 0xFE):
                size = rng.read(1)[0]

                if ((size!= 12) and
                    (size!= 12+int((packed_byte >> 3)&1))
                   ):
                    continue
                
                data = rng.read(size)
                out.write(struct.pack('<BB', size, ext))
                out.write(data)
            
            # Application extension
            elif (ext >= 0xFF):
                size = rng.read(1)[0]
                app_name = rng.read(size)
                subblock_size = rng.read(1)[0]
                data = rng.read(subblock_size)
#                 print('\tApplication Extension',
#                       bytes([ext]),
#                       bytes(app_name),
#                       len(data))            
                out.write(struct.pack('<BBB', size,
                                      app_name,
                                      0))
                out.write(data)
            else:
                continue
    
    # Write color table and image descriptors 
    for i in range(height-1,-1,-1):
        lzw_code_length = rng.read(1)[0]
        out.write(struct.pack('<B', lzw_code_length))

        for j in range(0, width*3, 3):
            b = rng.read(1)[0]
            g = rng.read(1)[0]
            r = rng.read(1)[0]
            out.write(struct.pack('<BBB', b,g,r))
        
        out.write(b'\x00')

    # Finish by adding the trailer and closing the file
    out.write(b';')

