#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(1), 'big', signed=False) % 256 + 1 # Clamp length to [0,257)

    # Write fixed header
    out.write(header)
    out.write(struct.pack('<H', datalen))
    out.write(b'\x80\0') # Define screen descriptor as 0x80 (no color table)
    out.write(struct.pack('B', int.from_bytes(rng.read(1), 'big', signed=False))) # Width byte 1
    out.write(struct.pack('B', int.from_bytes(rng.read(1), 'big', signed=False))) # width byte 2
    out.write(b'\x00') # Height byte 1
    out.write(b'\x00') # Height byte 2
    out.write(b'\x00\0') # Color table position
    out.write(b'\x00') # Background color index
    out.write(b'\0') # Aspect ratio
    out.write(b'\xf9\xff\0\1\x00\1\0\1') # Graphics control extension block

    for _ in range(datalen):
        palette_idx = int.from_bytes(rng.read(1), 'big', signed=False)
        out.write(palette_idx.to_bytes(1,'big'))

    out.write(b'\0') # Block terminator

