#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write GIFF header to output stream
    out.write(b"GIF89a")

    # Generate global logical screen descriptor
    width = rng.read(1)[0] % (2**5 + 1) * 2 + 1
    height = rng.read(1)[0] % (2**5 + 1) * 2 + 1

    packed = bytearray()
    packed += bytes([width])
    packed += bytes([height])
    packed += b"\x87\x00\x00"                       # Global color table size
    packed += bytes([rng.read(1)[0] & 0x0F])      # Background color index
    packed += bytes([(rng.read(1)[0] & 0x60) | 0xF])   # Pixel aspect ratio

    out.write(bytes(packed))
    print ("Generated dimensions {}x{}".format(width, height))

    # Write local colour table with black as first entry
    palette = [0xff, 0x00, 0x00, 0x00]            # Black
    for _ in range(2 ** ((packed[9] >> 4) + 1)):
        palette += [(rng.read(1)[0] & 0xF0) |
                     (rng.read(1)[0] & 0x0F)]        # Add more frames

    out.write(bytes(palette))                      # Palette frame
    out.write(b'\x00' * (((width // 8) * height))) # Image data frame

    return width, height
    out.seek(0, 0)

