#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    width, height = map(lambda x: int.from_bytes(x, "big"), [
            rng.read(2), rng.read(2)])
    palette_size = (width // 16) * ((height // 16) - 1)
    localcolortable = bytes([
            0xFF, 0xFE, 0xFB, 0xFA, 0xF6, 0xF4, 0xEF, 0xED,
            0xEA, 0xE7, 0xE3, 0xE0, 0xDC, 0xD8, 0xD5, 0xD1])

    print(f"generating Gif with dimensions ({width}, {height}) and {palette_size} colors")
    header = b'GIF89a'
    color_table = (b'\0' * 3) + localcolortable + (b'\0' * (762 if palette_size > 256 else 768))
    global_color_table = (b'\0' * 3) + (b'\xFF' * (1 << 10))[0:(2 << 10)]
    screendesc = struct.pack("<HHHHBBB",
            width, height, 0xf7, 0, 0xff, 0, 0)
    logicalscreen = (global_color_table +
                     screendesc +
                     bytes([0x91] + ([palette_size] if palette_size < 256 else [])))
    data = rng.read((width // 16 + 1) *
                    (((height // 16) - 1) * (2 << 10)))
    out.write(header + logicalscreen + color_table + data)
    print(f"generated {len(data)}-byte image")


    # seek back to beginning of file
    out.seek(0, 0)