#!/usr/bin/env python3

import sys
import io
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = 'GIF89a'.encode('ascii')
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    codelen = len(data)

    # Write the header and all bytes preceding the first descriptor
    out.write(header + data[:10] + b'\xFF\x00\x2C\x00\x00\x00\x00\x00\x00\x00\x00\xFB\x00\x00\x00')

    # Now we're going to overwrite the descriptor
    out.seek(-2, 2)
    out.write(struct.pack('<H', codelen))
    out.seek(-2, 1)

    # Then write the rest of the file
    out.write(data[10:])

    # Finally, just add some padding at the end
    numloops = 0xC0
    paddinglen = (numloops*3).to_bytes(1, byteorder='little')
    out.write((b'\x3B' + paddinglen)*int((1340-(out.tell()+1))/3))

    assert out.tell() < 1340