#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
###########################################################################

import sys
import os
import time
import zlib


def generate_random_gif(outdir):
    if not os.path.exists(outdir):
        raise Exception("Non existing directory %r" % outdir)
    
    t = time.time() * 1e+6 # uS precision
    f = open("/dev/urandom", "rb")
    size = int.from_bytes(f.read(2), byteorder='big')
    filename = "%s/%i_%d.gif" % (outdir, os.getpid(), int(t))
    print("[%i] Generating %d bytes to '%s'" % (os.getpid(), size, filename))
    w = open(filename, "wb+")
    
    # Header
    w.write(b"GIF89a")
    w.write((size & 2**16 - 1).to_bytes(2, byteorder="big"))
    w.flush()
    
    # Data
    data = f.read(size)
    crc = zlib.crc32(data)
    w.write(data)
    w.close()
    
    # Footer
    f.close()
    w = open(filename, "ab")
    w.write(zlib.crc32(data).to_bytes(4, byteorder="big"))
    w.write(zlib.adler32(data).to_bytes(4, byteorder="big"))
    w.write(b"\x00\x3B")
