#!/usr/bin/env python3

from io import BytesIO
from PIL import Image
import struct
from os import urandom
from sys import argv as args

if len(args) < 2 or '-h' in args or '--help' in args:
    print('usage:', args[0], '<output filename>')
    exit()

img_size = (140, 70)    # this is arbitrary

# generates an image and returns it as bytes
def get_image():
    image = Image.new("RGB", img_size, "white")
    pixels = [pixel for pixel in image.getdata()]
    
    return image

def main():
    with open(args[1], 'wb+') as f:
        
        while True:
            
            # generate new image
            image = get_image()
    
            # convert to bytes
            stream = BytesIO()
            image.save(stream, format="png")
            png_data = stream.getvalue()
            

            # write preamble
            header = b'\x47\x49\x46\x38\x39\x61'     # magic num
            block_size = struct.pack('<i', len(png_data))   # size of image
            f.write(b''.join([
                header,                           # magic num
                block_size,                       # size of image
                png_data                          # actual image data
            ]))

            if input("\nKeep generating? ('q' to quit): ") == 'q': break
        
    
