#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate a random color in RGB space (TODO: HSV)
def gen_color() -> bytes:
    return bytes([int.from_bytes(
            open('/dev/urandom').read(1), 'big') % 256 for _ in range(3)])


# Generates a random GIF file with the given width and height
# into `out`. The output will have no headers or footers but is otherwise valid.
def generate_random_gif(
        w: int, h: int, out: BinaryIO
    ):
    # TODO: Make this not zero-copy
    # For the case where we don't want to write directly to disk
    buf = bytearray((w * h) // 4 * 7)

    i = 0
    while i < len(buf):

        # Write one row of the image data
        start = i
        end = min(start + ((h - (i // w)) if i!=
                             0 else h), len(buf)-3)

        buf[start:end] += gen_color() + b'\xFF'

        i = end

    out.write(struct.pack('<HHB', w, h, 0xF7))
    out.write(buf[:len(buf)//7*4])