#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO


def get_rng() -> BinaryIO:
    if len(sys.argv) > 1 and sys.argv[1] == '-':
        return sys.stdin.buffer
    else:
        return open('/dev/urandom', mode='rb')


# Generates a random GIF file to standard output or `-`
# using the specified RNG (`/dev/urandom`)
if __name__ == '__main__':
    with get_rng() as rng:

        header = b'GIF89a'
        datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

        print('header',header,'datalen',datalen)
    
        # Write pre-frame bytes  

        sys.stdout.buffer.write(header + (datalen).to_bytes(
                3, 'big')) 

        # write frame data

        while True:
            ch = int.from_bytes(
                rng.read(1), byteorder="big", signed=False)

            if ch < ord('#'):
                print("! Char:",ch,"<",chr(ch),">")
            
            sys.stdout.buffer.write((ch).to_bytes(1, "big"))