#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom). The height
# and width can be given as arguments (default is [15,27]).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width=27, height=15):
    color_map_size = 256
    global_color_table = bytes([x for x in range(0,4*color_map_size)])
    local_color_table = b''
    imgdesc = [
                width&0xff,        #(w&0xff)<<0 | (w>>8)&0xff,
                0,                #(h&0xff)<<0 | (h>>8)&0xff,
                b'\x87',          #(l&0xff)<<0 | (l>>8)&0xff,
                0b00000000      ] #(sorted<<3|reserved<<2|interlace<<1|palette<<0)
    
    data = []
    while len(local_color_table)<(1<<(imgdesc[0]+1)):
        local_color_table += bytes(rng.read(3))

    data += [
            1,                   # block size
            *imgdesc[:3],         # image descriptor
            b'\x00',              # background color index
            0,                   # pixel aspect ratio

            len(global_color_table)+len(local_color_table),   # subblock size
            int(''.join(['%02X'%i for i in list(global_color_table)]),16),
            int(''.join(['%02X'%i for i in list(local_color_table)]),16)
           ]

    subblocks = data[:-1]
    data[0] = sum([(len(v)&0xff)<<k
                  if isinstance(v,(int,float)) else k
                  for k,v in enumerate(subblocks,start=2)])
    data[-1]=sum([(len(v)&0xff)<<k
                   if isinstance(v,(int,float)) else k
                   for k,v in enumerate(reversed(subblocks),start=2)])
    
    out.write(struct.pack('<{}s'.format(len(data)), *data))
    print("generated {}-byte image".format(len(data)))

    # seek back to beginning of file
    out.seek(0, 0)