import struct
from typing import BinaryIO


def gen_rand_color():
  """Generate an RGB value between [0..255]."""

  r = int.from_bytes(urandom(1), byteorder='little') % 256
  g = int.from_bytes(urandom(1), byteorder='little') % 256
  b = int.from_bytes(urandom(1), byteorder='little') % 256
  
  return bytes([r, g, b])
    

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = bytearray()
    lsd += rng.read(7)
    lsd[4] |= 0b10000000  # Set global colour table flag

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(lsd[4] & 7) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = bytearray()

      for _ in range(gctlen // 3):
        gct += gen_rand_color()

      out.write(gct)

    # We're going to start at offset X = Y = 0    
    xpos = ypos = 0

    while True:
        iblock = bytearray()

        # Get the size which depends on whether or not image data was generated so far
        framesize = 2 + urandom(1)[0] % 128 + 1

        # The first byte contains information about how wide our square will be
        iblock += urandom(1) & 0b10000000 | min((xpos // 25), 7) << 5 | max(min(ypos // 25, 7), 2)

        # Go ahead and set the number of colors
        ctablelen = urandom(1)[0] & 7

        iblock += ctablelen

        # Do we need a color table?
        if len(iblock) < framesize:
            for _ in range(2**ctablelen):
                iblock += gen_rand_color()

        # Does our frame have image data anyway?
        if iblock[-1]!= 0:

            # Yes! Then let's generate some more
            imgrgb = bytearray()
            
            imgrgblen = framesize - len(iblock)

            for _ in range(imgrgblen):
              imgrgb += gen_rand_color()

            iblock += imgrgb

        print(framesize, len(iblock))
        assert len(iblock) <= framesize

        out.write(iblock[:framesize])
            
        if xpos >= 1024-25 and ypos >= 1024-25:
          break

        xpos += urandom(1)[0] % 15 + 1
        ypos += urandom(1)[0] % 15 + 1