"""
Random GIF Generator
====================

This script generates a random GIF file to my `/tmp`.

Usage:

  python3 rand-gif.py

The generated files will be named as follows:

  <timestamp>-rand<num>.gif

Timestamp is in ISO format and is used as part of the filename for better sorting
ability. The second component is randomly generated integer from 0 to 1000 (i.e.
`4`-digit random number).

Author:  <NAME> (<EMAIL>)
         https://github.com/shenwei356/util-scripts
License: MIT
"""


import os
import time
import shutil
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'

    # header
    out.write(header)

    datalen = int.from_bytes(
            rng.read(2), 'big')

    # logical screen descriptor
    out.write(b'\x00\x00\x00\x00')    # xoffset
    out.write(b'\x00\x00\x00\x00')    # yoffset
    out.write(b'\x00\x00\x00\x00')    # width
    out.write(b'\x00\x00\x00\x00')    # height
    out.write(b'\x0F')                  # packed field

    # global color table
    out.write(b'\x7C\x00\xFE\xFF')     # RGB color
    out.write(b'\x00\xFC\xFF\xFF')     # RGB color
    out.write(b'\x00\xFD\xBA\xFF')     # RGB color
    out.write(b'\x00\xFB\xAC\xFF')     # RGB color
    out.write(b'\x00\xDA\xA7\xFF')     # RGB color
    out.write(b'\x00\xD8\xC3\xFF')     # RGB color
    out.write(b'\x00\xC5\xEA\xFF')     # RGB color
    out.write(b'\x00\xB5\xF2\xFF')     # RGB color
    out.write(b'\x00\xF7\xFE\xFF')     # RGB color
    out.write(b'\x7D\xFD\xFF\xFF')     # RGB color
    out.write(b'\x00\xF7\xFB\xFF')     # RGB color
    out.write(b'\x00\xEF\xF9\xFF')     # RGB color
    out.write(b'\x00\xDC\xED\xFF')     # RGB color
    out.write(b'\x00\xBC\xE5\xFF')     # RGB color
    out.write(b'\x00\xBB\xD9\xFF')     # RGB color
    out.write(b'\x00\xAA\xCC\xFF')     # RGB color
    out.write(b'\x00\xEE\xFE\xFF')     # RGB color
    out.write(b'\x00\xEE\xEE\xFF')     # RGB color
    out.write(b'\x00\xDD\xE1\xFF')     # RGB color
    out.write(b'\x00\xCC\xD4\xFF')     # RGB color
    out.write(b'\x00\xBB\xC7\xFF')     # RGB color
    out.write(b'\x00\xAA\xBA\xFF')     # RGB color
    out.write(b'\x00\xFE\xFE\xFF')     # RGB color
    out.write(b'\x00\xEC\xE8\xFF')     # RGB color
    out.write(b'\x00\xEA\xE5\xFF')     # RGB color
    out.write(b'\x00\xE7\xE4\xFF')     # RGB color
    out.write(b'\x00\xE5\xE2\xFF')     # RGB color
    out.write(b'\x00\xE3\xE0\xFF')     # RGB color
    out.write(b'\x00\xE1\xDF\xFF')     # RGB color
    out.write(b'\x00\xDE\xDB\xFF')     # RGB color
    out.write(b'\x00\xDC\xD9\xFF')     # RGB color
    out.write(b'\x00\xDA\xD7\xFF')     # RGB color
    out.write(b'\x00\xD8\xD5\xFF')     # RGB color
    out.write(b'\x00\xD6\xD3\xFF')     # RGB color
    out.write(b'\x00\xD4\xD1\xFF')     # RGB color
    out.write(b'\x00\xD2\xCF\xFF')     # RGB color
    out.write(b'\x00\xD0\xCD\xFF')     # RGB color
    out.write(b'\x00\xCE\xCC\xFF')     # RGB color
    out.write(b'\x00\xCB\xCA\xFF')     # RGB color
    out.write(b'\x00\xC9\xC8\xFF')     # RGB color
    out.write(b'\x00\xC7\xC6\xFF')     # RGB color
    out.write(b'\x00\xC5\xC4\xFF')     # RGB color
    out.write(b'\x00\xC2\xC2\xFF')     # RGB color
    out.write(b'\x00\xC0\xC0\xFF')     # RGB color
    out.write(b'\x00\xBE\xBE\xFF')     # RGB color
    out.write(b'\x00\xBC\xBC\xFF')     # RGB color
    out.write(b'\x00\xBA\xBA\xFF')     # RGB color
    out.write(b'\x00\xB8\xB8\xFF')     # RGB color
    out.write(b'\x00\xB6\xB6\xFF')     # RGB color
    out.write(b'\x00\xB4\xB4\xFF')     # RGB color
    out.write(b'\x00\xB2\xB2\xFF')     # RGB color
    out.write(b'\x00\xB0\xB0\xFF')     # RGB color
    out.write(b'\x00\xAE\xAE\xFF')     # RGB color
    out.write(b'\x00\xAC\xAC\xFF')     # RGB color
    out.write(b'\x00\xAA\xAA\xFF')     # RGB color
    out.write(b'\x00\xA8\xA8\xFF')     # RGB color
    out.write(b'\x00\xA6\xA6\xFF')     # RGB color
    out.write(b'\x00\xA4\xA4\xFF')     # RGB color
    out.write(b'\x00\xA2\xA2\xFF')     # RGB color
    out.write(b'\x00\xA0\xA0\xFF')     # RGB color
    out.write(b'\x00\x9E\x9E\xFF')     # RGB color
    out.write(b'\x00\x9C\x9C\xFF')     # RGB color
    out.write(b'\x00\x9A\x9A\xFF')     # RGB color
    out.write(b'\x00\x98\x98\xFF')     # RGB color
    out.write(b'\x00\x96\x96\xFF')     # RGB color
    out.write(b'\x00\x94\x94\xFF')     # RGB color
    out.write(b'\x00\x92\x92\xFF')     # RGB color
    out.write(b'\x00\x90\x90\xFF')     # RGB color
    out.write(b'\x00\x8E\x8E\xFF')     # RGB color
    out.write(b'\x00\x8C\x8C\xFF')     # RGB color
    out.write(b'\x00\x8A\x8A\xFF')     # RGB color
    out.write(b'\x00\x88\x88\xFF')     # RGB color
    out.write(b'\x00\x86\x86\xFF')     # RGB color
    out.write(b'\x00\