#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'
    width = 256
    height = 128
    maxcolor = 2 ** 4 - 1
    colorpalette = []
    for i in range(maxcolor):
        colorpalette += [i] * 3
    datapixels = []
    for _ in range(width*height//2):
        datapixels += [int.from_bytes([
                rng.read(1)[0],
                rng.read(1)[0]],
                byteorder='little')]
    screendescriptor = (b'\x2c'
                        + width.to_bytes(2, byteorder='little')
                        + height.to_bytes(2, byteorder='little')
                        + b'\xf7\x00\x00')
    globalpalette = (struct.pack('>bbb',
                                 *[0xff]*3)
                     * maxcolor)
    data = (screendescriptor
            + bytes(datapixels)
            + globalpalette[::3])
    out.write(header
              + struct.pack('>H', len(data))
              + data)
    print(f"generated {len(data)}-byte image ({width} x {height})")