#!/usr/bin/env python3
"""Generate a random GIF image using /dev/urandom."""


import argparse
import itertools
import glob
import subprocess
import tempfile
import sys
import json
import base64 as _base64
import io
from PIL import Image  # type: ignore
from typing import AnyStr, Union
try:
    from os import scandir
except ImportError:
    from scandir import scandir   # type: ignore


def main():
    psr = argparse.ArgumentParser()

    psr.add_argument('width', nargs='?', default=100,
                     help="output gif width")
    psr.add_argument('height', nargs='?', default=75,
                     help="output gif height")

    args = psr.parse_args()

    with open('/dev/urandom', mode='rb') as fpr:

        print("generating GIF...".format())
        generate_random_png(fpr, int(args.width),
                            int(args.height))

def generate_random_png(
        rng: BinaryIO, width: int, height: int
    ) -> None:
    """Generate a random PNG image of the given width and height."""
    
    img = []
    for x in range(0, width):
        row = [int.from_bytes(
                rng.read(1), byteorder='little', signed=False)
               for y in range(0, height)]
        img.append(row)
    im = Image.new("P", (width, height), color=255)
    im.putdata([item for sublist in img for item in sublist])
    im.save("/tmp/test.png","PNG")
    return "/tmp/test.png"