#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Reserve space for the header and length bytes
    buffer = bytearray([
        0, 0, 0, 0, 
        0, 0])
    
    # Read in 15 more bytes (for example maximum block size of /dev/rand)
    buffer[0:2] = rng.read(2)

    if not buffer[0]:
        # The first byte is zero; this means that we're at the end of the stream!

        # TODO What do we want to do here? Just return? Raise an exception?
        raise Exception('RNG ran out of entropy!')

    # Write to the output
    out.write(buffer[0:4])

    # Endian-convert the two length bytes so they can be used as an index later on
    buffer[4], buffer[5] = buffer[5], buffer[4] 

    # How many chunks are left before we reach our buffer limit?
    max_chunks = int(len(buffer)/os.sysconf("SC_PAGESIZE") - buffer[5])

    while true:
        chunksize = min(max_chunks, 
            int.from_bytes(rng.read(2), 'big'))

        # Break out of the loop if we reached EOF on the RNG
        if not chunksize:
            break

        # Add random bytes to the buffer
        buffer += rng.read(chunksize)

        # Split off the chunk
        chunk = buffer[:chunksize]
        
        # Delete already read from buffer
        del buffer[:chunksize]

        # Write it to stdout 
        out.write(chunk)