#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. 2 bytes of length of block (N).
#   3. N bytes of block.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'
    out.write(header)
    out.flush()
    
    # We'll read from it in chunks of 1024
    chunksize = 1024
    
    # Read one chunk of size 'chunksize'.
    data = rng.read(chunksize)
    
    # Until we're done reading all data, write them to our destination file.
    while len(data):
        
        # Write the length of this chunk first as little-endian.
        out.write(len(data).to_bytes(2, 'little'))

        # Now flush any buffered output into the file so that its length is known.
        out.flush()

        # Write the actual data itself in big endian.
        for i in range(0, len(data)):
            out.write(struct.pack('B', data[i]))
            
        # Clear the data buffer since we've now written everything.
        del data[:]

        # Read another chunk of data...
        data = rng.read(chunksize)
        
    # Flush any remaining data to disk.
    out.flush()