#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  

    out.write(header + (datalen).to_bytes(
            3, 'big')) 

    previous = None

    # write frame data

    for i in range(datalen):

        r = int.from_bytes(
                rng.read(1), byteorder="big") % 4
        
        if not previous is None and \
           len(previous) > 0 and \
           (r == 0 or previous[len(previous)-1]!= r):

            # Generate new color
            # TODO: Fix this magic constant!
            col = [((i % 7) * 31) % 256,
                   (((r+1)%4)*10)*(int)(r%2==0)+
                       (((r+3)%4)*10)*(int)(r%2!=0),
                   ((r+2)%4)*31]

            previous += [col[0],
                         col[1],
                         col[2]]

        elif previous is None:

            previous = []

        else:
            
            previous += [(i % 256)]

        out.write(((previous)[i]).to_bytes(
                1, "big"))