#!/usr/bin/env python3

import struct
from typing import BinaryIO

def write_u16le(x: int, f: BinaryIO):
    f.write(struct.pack('<H', x))

def write_u32le(x: int, f: BinaryIO):
    f.write(struct.pack('<L', x))


def randomize(rng: BinaryIO, ostream: BinaryIO, length: int):

    # Write header
    ostream.write(b'\x47\x49\x46')

    # Write width and height as zeroes to indicate that it isn't known yet
    write_u16le(0, ostream)
    write_u16le(0, ostream)

    # Write global color table info (no colors in use)
    ostream.write('\x00')
    write_u16le(0, ostream)

    # Set background color index to be transparent so PNG can erase
    ostream.write('\xff')

    # Write loop count as zero indicating infinite looping
    for _ in range(8):
        ostream.write('\x00')

    # TODO: Experimental section
    write_u16le(length - 1, ostream)       # No of frames
    write_u16le(length * 25, ostream)      # Frame duration
    write_u16le(length * 25 // 10, ostream) # Pixel delay
    # End experimental section


    # Start image block
    ostream.write('\x2c')

    # Loop through the entire frame to add each pixel
    for i in range(length):

        numcols = rng.read(1)[0] & 0b00011111;

        if not numcols:
            continue

        # Add transparency flag for first color
        ostream.write('\xf9\x01')

        for j in range(numcols):

            # Write palette indices
            col = rng.read(1)[0] & 0b00001111;
            ostream.write(col.to_bytes(1, byteorder='big'))


        # Encode pixels column by column
        while True:
            pxltype = rng.read(1)[0] >> 5;
            
            if pxltype == 0:
                break

            elif pxltype == 1:

                # Copy next N pixels without modification from previous line
                copylen = rng.read(1)[0];
                
                if len(pxls) < copylen:
                    raise Exception("Not enough pixels left in current row")

                ostream.write((copylen | 0xe0).to_bytes(1, byteorder='big'));
                ostream.write(pxls[-copylen:])
                del pxls[copylen:]

            else:
                raise Exception("Invalid pixel type encountered")

        # Pad with empty pixels to end on a multiple of 256 bytes
        while len(pxls) > 255:
            pxls += '\x00';
        
        # Now encode how many pixels are used this time around
        ostream.write(((len(pxls) - 1) | 0x00).to_bytes(1, byteorder='big'));
        ostream.write(pxls);

        # Generate more pixels
        pxls = ''
        for j in range(rnd_width):
            pxls += '%02x' % rng.read(1)[0];
    