#!/usr/bin/env python3

import argparse
import os
import sys

from typing import BinaryIO


# Generates a random GIF file and writes it to the given stream
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        size_max: bytes
    ):

    @classmethod
    def clamp(n, lowerbound, upperbound):
      return max(min(upperbound, n), lowerbound)


    header = b'GIF89a'
    datalen = bytearray()

    # Generate a random length between one and SIZE_MAX
    while len(datalen) == 0 or datalen[0] > size_max:

        datalen.clear()
        datalen += rng.read(1)
        
        if len(datalen)!= 1 or datalen[0] < ord('!'):
            continue
        elif datalen[0] >= ord('A') and datalen[0] <= ord('Z'):
           datalen += (rng.read(1)) & 0x7f 

        else:
            datalen += [
                # Build an integer from two consecutive bytes
                ((rng.read(1)[0]) << 8) | rng.read(1)[0],
                # Clamp the result to be in range [*size_min*, *size_max*]
                clamp((
                    (
                        rng.read(1)[0] 
                        & 0b00001111 
                    ) << 8 
                ) | rng.read(1)[0], 0, size_max)
            ]


    print(
        f'[+] Generated a new {size_max} bytes long GIF ({datalen}):\n{datalen}',
        flush=True, end='\r'
    )
    out.write(header + datalen)



if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-o', '--output-file', type=str, metavar='FILE', default='/tmp/random-gif')
    args = vars(parser.parse_args())

    gensize = os.path.getsize(args['output_file'])
    with open('/dev/urandom', 'rb') as fi:
        with open(args['output_file'], 'wb+') as fo:

            try:

                while True:

                    generate_random_gif(fi, fo, gensize)

            except KeyboardInterrupt:

                pass

            finally:

                print('\n[+] Done!')