import os
import time
import shutil
from typing import BinaryIO


# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # TODO: Make this a proper GIF file instead of just bytes
    # TODO: Do not read all at once (the whole /dev/urandom) into memory
    #       Use https://docs.python.org/3/library/io.html?highlight=stream#binary-i-o
    header = b'GIF89a'
    width = 640
    height = 640
    palette = [b'\x7f\xff\xff',
               b'\x7f\x00\x00'] * 256
    frame = [[palette[int.from_bytes(rng.read(1),'big')]
              for _ in range(width)]
             for __ in range(height)]
    print('The image is %dx%d pixels large.'
          %(len(frame[0]), len(frame)))
    print('Each pixel uses two bytes to store its color.')
    durations = [0]
    while True:

        # add new frame if needed
        if time.time() > durations[-1]:

            # frame duration
            delay = max((durations[-1],
                         float(os.environ['RANDOM_GIF_DELAY']))
                        )
            durations += [durations[-1]+delay]
            frame += [[palette[int.from_bytes(rng.read(1),'big')]
                      for _ in range(width)]
                     for __ in range(height)]


        # write the resulting GIF file
        try:
            out.write(
                header+width.to_bytes(2,'big')+
                height.to_bytes(2,'big'))
            out.write(chr(len(palette)).encode())
            out.write(b"".join(palette))
            out.write(
                chr(int(0.5*durations[0])).encode()+
                chr(0).encode()+chr(0).encode())
            out.write(chr(0).encode())
            codesize = 0  # LZW minimum encoding size
            prefix = []   # empty string
            suffix = frame[0][0]    # first row's leftmost pixel
            out.write(codesize.to_bytes(1,'big'))

            for line in frame:

                lineprefix = line[:0:-1]     # reverse line
                linecode = ([suffix[:0:-1]]
                            +lineprefix
                            +[suffix])

                codesizes = [
                    min([len(pfx)+len(sfx) < 128 and
                          sfx!= pfx[:-1] or len(sfx)-1
                         for sfx in linecode])
                    for pfx in prefix]
                if sum(codesizes) >= 2**codesize:

                    # flush previous code
                    out.write(chr(127).encode())
                    out.write(chr(prefix[0][0]).encode())
                    for pfx in prefix:
                        out.write(chr(len(pfx)).encode())
                        out.write(pfx)
                    prefix = [linecode[0]]
                    codesize += 1

                else:
                    for i,(csz,sc) in enumerate(zip(codesizes,linecode)):

                        if csz == 1 and sc == prefix[0][:0:-1]:

                            # no change from previous code
                            prefix[0] = sc
                        elif csz < 128:
                            # literal with given length
                            out.write(
                                chr(csz).encode()+chr(sc[0]).encode())
                        else:
                            # length determinant + first byte
                            out.write(
                                ((csz-128).to_bytes(1,byteorder='big')
                                 +sc[0].to_bytes(1,byteorder='big')))

                suffix = line[-1]
                prefix += [line[-1:]]
        except BrokenPipeError:
            break
        finally:
            # cleanup
            del frame[:height]