#!/usr/bin/env python3

import struct

# Generates a random GIF file into `out` using the
# random number generator `rng` which should be /dev/urandom
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # First bit of version tag is fixed at 89
    out.write(b"GIF89a")

    # Next three bytes are screen descriptor
    screen_width,screen_height=struct.unpack("<HH",
            rng.read(4))

    packed_bits=(int.from_bytes(rng.read(1),"big")&7)+\
            (((int.from_bytes(rng.read(1),"big")&0x3f)<<8)&0xFF00)+\
            (((int.from_bytes(rng.read(1),"big")&1)<<14))+\
            (((int.from_bytes(rng.read(1),"big")&0x3f)<<20)&0xF00000)

    global color_table_flag
    if(packed_bits&0x8000):
        out.write((0).to_bytes(1,"big"))
        
        color_table_flag=True

        num_color_entries=(packed_bits&0x7F)+1
        
        background_index=(int.from_bytes(
                rng.read(1),"big")&0xf)+1
       
        aspect_ratio=(packed_bits>>8)&0xff
        out.write(((aspect_ratio<<8)|
                 (num_color_entries-1))+
                 background_index)
        
        for i in range(num_color_entries):
            out.write(rng.read(3))

    else: 
        out.write(
                (0x80|(background_index-1)).to_bytes(
                    1,"big"))

        color_table_flag=False


    # Then there's flags and loop counter
    flagbyte=(int.from_bytes(rng.read(1),"big")&(0x7<<5))+\
            (((int.from_bytes(rng.read(1),"big")&3)<<8)&0x700)

    out.write((((flagbyte<<4)|15)+(num_of_frames%15)-1).to_bytes(1,"big"))

    # Now we have a delay time
    out.write(rng.read(2))
    
    packed_bits=int.from_bytes(rng.read(1),"big")

    num_of_frames=((packed_bits&0x3f)+1)*10+ \
            (((packed_bits>>6)&3)+1)

    for j in range(num_of_frames):
        # We'll put the block size first as a full word
        out.write(((7<<8)|0).to_bytes(2,"big"))

        next_byte=int.from_bytes(
                rng.read(1),"big")

        while True:

            if (next_byte == 0x2c or 
                    next_byte == 0x00):
                
                break
            
            elif (next_byte == 0x21):

                commandbyte=int.from_bytes(
                        rng.read(1),"big")

                if commandbyte==0xff:
                    
                    break

                else:

                    lengthcode=commandbyte&0x07
                    lengthextent=lengthcode+1
                    extradata=int.from_bytes(
                            rng.read(lengthextent),"big")

                    if commandbyte==0xf9:

                        out.write(
                                (extradata).to_bytes(
                                    2,"big"))
                        
                    elif commandbyte==0xfe:
                        
                        out.write(
                                (7<<8|
                                 (extradata)).to_bytes(
                                    2,"big"))

                    else:                    
                        continue
            else: 
                raise Exception("Bad GIF encoding")

            next_byte=int.from_bytes(
                    rng.read(1),"big")
        
        local_image_descriptor=(int.from_bytes(
                rng.read(1),"big")&0x7)+\
                (((int.from_bytes(rng.read(1),"big")&0x7f)<<8)&0xFF00)+\
                (((int.from_bytes(rng.read(1),"big")&1)<<15)&0x8000)+\
                (((int.from_bytes(rng.read(1),"big")&0x7f)<<21)&0x780000)
    


        if not color_table_flag: 
            out.write(((local_image_descriptor&0xFF00)>>8,\
                              local_image_descriptor&0xFF)\
                            .to_bytes(2,"big"))
        
        width=(local_image_descriptor&\
              ~((1<<14)-1))//256
    
        height=(local_image_descriptor>>8)&\
               ~((1<<14)-1)
 
        out.write((((width*height)*3)&0xFFFF).\
                      to_bytes(2,"big"))

        out.write(b'\x08')

        lzwmincodesize=\
                int.from_bytes(rng.read(1),"big")&0x07+1

        # LZW min code size plus one
        out.write(\
                (lzwmincodesize).to_bytes(1,"big"))

        # Since we're generating some randomness,
        # this can't be too large!
        image_data=[]

        # First add an empty line as terminator
        image_data.append([])

        # Read all pixels from input stream
        currentpixel=[0] * 3
        
        while len(currentpixel)!=width*height-len(image_data[0]):
            
            currentcode=0

            code_stack=[currentcode]

            # And now read the strings...

            while True:
            
                if currentcode>=lzwmincodesize:
        
                    bitsneeded=(lzwmincodesize-1).bit_length()
        
                    newbits=int.from_bytes(
                            rng.read(bitsneeded),"big")

                    currentcode+=newbits
                else:
                    pass


                try: 
        
                    currentcode=code_stack[-1]\
                            +code_lookup_table[
                                    currentcode][
                                            newbits]

                except IndexError: 

                    print("ERROR AT POSITION")
                    return
                    
                except KeyError:
                    
                    code_lookup_table[currentcode]=[]
                    
                    if len(code_lookup_table)==4096: 
                        print("CODE TABLE FULL")
                        break
                        
                    else:
                        
                        code_lookup_table[currentcode].insert(0,[currentcode])
                        code_lookup_table[currentcode].extend(
                                [currentcode,currentcode])

                        code_stack.append(currentcode)

                        continue

                code_stack.pop(-1)

                if currentcode<256:

                    image_data[len(image_data)-1].append(chr(currentcode))

                    currentpixel.append(
                            code_lookup_table[
                                    currentcode][0][0])
                        
                    code_lookup_table=code_lookup_table[:currentcode]

                    code_lookup_table.append([currentpixel[:]])

                    currentpixel=currentpixel[1:]
                    
                    code_lookup_table.append(code_stack[:])

        if len(image_data)>height: 
            del(image_data[height:])
            
        for row in image_data:
            for pixel in row:
                out.write(pixel.encode())
                
        code_lookup_table=[]



