import os
import time
import shutil
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        input: BinaryIO, output: BinaryIO,
        max_width=640, max_height=360):

    # read and check GIF magic
    header = input.read(6)
    if header!= b"GIF87a\n":
        raise ValueError("not a valid GIF file")
    
    width = 0
    height = 0
    
    
    while True:
        
        blocktype = input.read(1).decode('ascii', errors='ignore')

        # end of file?
        if not blocktype:
            break

        elif blocktype == '!':

            while True:

                subblocktype = input.read(1).decode('ascii', errors='ignore')
                
                if subblocktype == '#':
                    subblocklength = int.from_bytes(input.read(1), byteorder="little", signed=False)
                    
                    if subblocktype == "#":
                        extlabel = input.read(subblocklength).decode('ascii').strip()

                        if extlabel == "NETSCAPE2.0":

                            while True:

                                subsubblocktype = input.read(1).decode('ascii', errors='ignore')
                                
                                if subsubblocktype in ("!", ";"):
                                    pass
                                    
                                
                                else:
                                    break
                            
                        
                        elif extlabel == "XMP Data XMP":
                            xmpdata = input.read(subblocklength - len(extlabel)).decode('utf-8').strip().splitlines()

                            for xmpline in xmpdata:
                                if "<rdf:width>" in xmpline:
                                    _, _, wstr = xmpline.partition("<rdf:width>")
                                    width = int(wstr[:len(wstr)-2])

                                if "<rdf:height>" in xmpline:
                                    _, _, hstr = xmpline.partition("<rdf:height>")
                                    height = int(hstr[:len(hstr)-2])


                        
                else:
                    break


        elif blocktype == ',':
            
            imagedescriptor = input.read(9)
            break
        
        
    print(f"{width}x{height}")



