#!/usr/bin/env python3

import os
import sys

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng`
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Set default values for starting position and flags
    firstpos = 7
    usegct   = False
    
    # Start by getting the logical screen descriptor (LSD) field
    lsd = rng.read(5) + bytes([2])

    # Add LSD to output
    out.write(lsd)

    # Read the background control (BG) field which should always be 0xF7
    bg = rng.read(1)[0]    
    assert(bg==0xF7)
    out.write(bg.to_bytes(1,'little'))
    
    # Read the pixel aspect ratio (PAR) field which should always be 0
    par = rng.read(1)[0]
    assert(par==0)
    out.write(par.to_bytes(1,'little'))
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        firstpos += 1
      
        # Check if this is the last block
        if (((fieldbyte & 128)!= 0)):
          firstpos -= 1
          break
          
        # Get size of block
        size = ord(rng.read(1))
        firstpos += 1

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream as they are
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))
        firstpos += 2

        if fieldbyte & 128 == 0:
          continue

        if size < 7 or size >= pow(2,16)-29:
          print("invalid size")
          exit()
          
            
        if size <= 69:
          if size<69:
            print ("need more bytes than given in frame header")
            exit()
            
          out.write(rng.read(size-1))
          firstpos+=size
          continue
          
        #Write the block terminator
        out.write(ord('\0').to_bytes(1,'little'))
        firstpos += 1

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:
          usegct = True
          # We can only handle up to 256 colors so just fake that many and skip
          # over the rest
          if numcolors > 256:
            numcolors = 256

          colortable = rng.read(numcolors*3)
          out.write(colortable)

          # Skip over the rest of the colors in our palette since we only support
          # 256
          leftovercolorbytes = max((numcolors - 256)*3, 0)
          if leftovercolorbytes > 0:
              rng.seek(leftovercolorbytes, 1)
              
        # Write the actual image data
        imgdata = rng.read(size-1)
        out.write(imgdata)