#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (`/dev/urandom`). The length of the payload is
# determined by the lower bits in our first two bytes from `rng`. This means we
# are limited to at most 512 bytes of payload. This is the full GIF spec limit.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Read the length field (first two bytes) and mask it with 0xFF00 so that it
    # can be treated as unsigned short. We use big-endian here because almost all
    # systems these days are little endian anyway :D
    datalen = int.from_bytes(
        rng.read(2),
        'big'
    ) & 0xFF00 >> 8

    data = rng.read(datalen)
    out.write(header + data)
