#!/usr/bin/env python3

import sys

try:
    from PIL import Image
except ImportError as e:
    print("This script requires the Pillow package", file=sys.stderr)
    exit()
    
import io
import itertools
import math
import os
import struct
from typing import Tuple

MAX_DATA_SIZE = 4 * 1024**3


# Generates a random GIF file into `out` using the
# random number generator `rng` (like /dev/urandom).
# Return value is a tuple containing (width, height).
def generate_random_image(
        rng: BinaryIO, out: BinaryIO
    ) -> Tuple[int, int]:
    src_size = (16, 16)

    src_palette = []
    for i in range(0, 7):
        src_palette += [i*85]*3 # grayscale palette

    im = Image.new('P', size=(src_size))
    im.putpalette(src_palette)

    # build the pixel data manually so we can control each bit
    rows = []
    for rownum, y in enumerate(range(-1, -1 - src_size[1], -1)):
        bytesize = round((src_size[0] * 8) // 8.0)

        bits = list(map(lambda x: bool(int(x)), format(
                int.from_bytes(rng.read(bytesize), 'little'), '#0'+str(bytesize*8)+'b')[2::]))

        pixels = []
        for colnum, x in zip(itertools.count(), range((src_size[0])//8)):
            px = 0
            for idx, bitidx in enumerate(range(rownum % 8, len(bits)-(rownum % 8)+1, 8)):
                px |= (bits[bitidx] << ((7-colnum)%8))

            pixels.append(px)
        
        rows.append(bytes([pixels[i] for i in reversed(range(src_size[0]))]))
    
    im.putdata(rows)
    im.save(out, format="gif")

    return src_size
