#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# Copyright (c) 2019-2020 The Pybricks Authors

"""
Generates a GIF file using system randomness.
"""


def _get_seed():
    """Returns an integer that can be used as seed for generating random numbers."""

    # get current microseconds since Unix epoch
    now = int(time.monotonic() * 1e6) & ((1 << 48) - 1)

    # add something based on the clock to make sure we don't use the same seed every boot
    now += int((int(time.clock() * 100)) ** 2) ^ (1 << 15)

    return now


def generate_random_gif(out: BinaryIO):
    """Generate a "psuedo" random GIF image in memory and write it to the specified output stream."""

    from PIL import Image

    width = height = 100

    img = Image.new("P", [width, height])

    pixels = []
    palette = []

    for y in range(height):

        for x in range(width):

            r = g = b = 0x7f
            s = (r,g,b)

            if len(palette) == 256 or not s in palette:
                i = len(palette)

                try:
                    # insert new colors at beginning of list
                    palette.insert(0,s)

                    # only set first color to transparent
                    if i > 1:
                        img.putpixel((x,y),i+1)

                except ValueError:
                    pass

            else:
                p = palette.index(s)+1

                img.putpixel((x,y),p)


    img.putpalette([int(v*255) for v in sum(palette, ())] + [0]*(256-len(palette)*3))


    img.save(out,"GIF")

