#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1<<24 - 1
    blocksize = 1 << 20 # 1MB blocks for now
    colorcount = 2**bits_per_color
    out.write(header)
    write_u16(out, 700)
    write_u16(out, 500)
    out.write(b'\xF0\x00')
    endblock = False
    while not endblock:
        subblocks = []
        while len(subblocks) < blocksize and \
                sum([len(sb) for sb in subblocks]) <= datalen:
            subblocks.append(bytes(rng.read(struct.pack("B", rng.read(1)[0] % (datalen+1))[0])))
        packeddata = bytes(b'\xFF' + b''.join(subblocks))
        if len(packeddata) > datalen or len(packeddata)<=0:
            break;
        out.write(struct.pack("B", len(packeddata)))
        out.write(packeddata)
        if len(packeddata)<datalen:
            endblock = True

    out.write(b';')
    return
