#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a' # Magic header
    width = 160; height = 120
    background_color = 5
    pixel_aspect_ratio = -1
    flags = { 0x80 : True } # This is an animation (for future use)

    def putbyte(b: int):
        out.write(struct.pack('B', b))

    def putword(w: int):
        putbyte((w >>  0) & 0xff);
        putbyte((w >>  8) & 0xff);

    # Write the GIF header...
    out.write(header)
    putword(width)
    putword(height)
    putbyte(pixel_aspect_ratio)
    out.write(b'\x00') # No global color table

    # ...and write the first frame.
    lsd = bytearray()
    lsd += struct.pack("BB",
            (background_color << 4) | ((width >> 7) & 0x07),
            (height          >> 0) & 0xff)
    lsd += struct.pack("B",
            0  | ((width      ) & 0x07) << 1
                | ((height      ) & 0x07) << 4
                | (((flags[0x80]) * 0) + 1) << 0)

    out.write(lsd)

    for _ in range(0, 2):
        putbyte(0) # Control code 0 means "don't draw anything"
        putword(0) # Dummy delay time
        putbyte(0) # Transparent color index

    local_color_table_size = 2 ** (1 + (lsd[4] & 0x07))
    local_color_table = []
    for i in range(0, local_color_table_size):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        local_color_table += [r,g,b]
    out.write(local_color_table)

    image_data = bytearray([0])
    for y in range(0, height):
        for x in range(0, width):
            color = rng.read(1)[0] % 256
            image_data.append(color & 0xf8)
            image_data.append(color