#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header)
    for _ in range(26):
        out.write(int.to_bytes(
                int.from_bytes(rng.read(1), 'big'),
                3, byteorder='big'))
    out.write(rng.read(1))

    blocksize = 0xff
    padding = b'\x00' * (blocksize - 1)
    while datalen > 0:
        byte = rng.read(datalen if datalen < blocksize else blocksize)
        datalen -= len(byte)
        if not byte[-1:] == padding[0]:
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            out.write(byte[:blocksize])

