#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )



# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Signature
    header = b'GIF89a'
    
    # Width and Height
    w = rng.read(2)   # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    header += w + h

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Background color index
    header += b'\x00'

    # Pixel aspect ratio (1:1)
    header += b'\x00\x00'

    # Global color table flag
    header += b'\x87\x00'

    # Flags for Image Descriptor Block
    header += b'\x00\x00\xff\xff'

    # No local color table flag
    header += b'\x00'

    # No interlacing flag
    header += b'\x00'

    # Header Terminator
    header += b';'

    # Write header
    out.write(header)

    # Get and write first frame
    bs = rng.read(1)              # get block size
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
