#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):
        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output

    @staticmethod
    def generate_graphics_control_extension(delay):
        block_size = 0x04
        extension_code = 0xf9
        flags = 0x01   # Transparent color flag
        transparent_color_index = 0xff
        delay_time = delay * 10

        return struct.pack('<BBBBI',
                           block_size, extension_code, flags,
                           transparent_color_index, delay_time)

    @staticmethod
    def generate_image_descriptor(left, top, width, height):
        local_color_table_flag = 0x80
        interlace_flag = 0x40
        sort_flag = 0x20
        reserved_local_color_table_size = 0x07
        
        return struct.pack('<BBHHB',
                           left, top, width, height,
                           local_color_table_flag | reserved_local_color_table_size)
    
    @staticmethod
    def generate_global_color_table(palette):
        table_size = len(palette) - 1
        colors = []

        for i in range(1 << (3 + table_size)):
            r, g, b = palette[i]
            colors.append((b >> 3, g >> 3, r >> 3))
            
        return bytes(colors)

    @staticmethod
    def convert_to_lzw_blocks(data: bytes, min_code_size):
        lzw = LZWCompression()
        code_size = min_code_size

        while True:
            yield lzw.compress(bytes([code_size]), data)
            if lzw.is_done:
                break
            code_size += 1
    
    def run():
        with open(self.output, 'w') as f:
            print("Generating", self.output)
            f.write('\x47\x49\x46\x38\x39\x61')
            f.write(struct.pack('<H', 0xffff))
            f.write(struct.pack('<HH', 0, 0))
            f.write(struct.pack('<HH', 500, 500))
            f.write(struct.pack('<B', 7))

            for j in range(32):
                for i in range(j):
                    f.write(RandomGifGenerator.generate_graphics_control_extension(1))
                    f.write(RandomGifGenerator.generate_image_descriptor(j*16, i*16, 16, 16))
                    f.write(RandomGifGenerator.generate_global_color_table([
                        (0, 0, 0), (16, 0, 0), (0, 16, 0), (16, 16, 0),
                        (0, 0, 16), (16, 0, 16), (0, 16, 16), (16, 16, 16)]))
                    f.writelines(RandomGifGenerator.convert_to_lzw_blocks(os.urandom(2**3*2**3), 5))
                    f.write('\x00')


def _usage():
    print("Usage: {} <image>".format(__file__))
    exit(-1)

