#!/usr/bin/env python3

import struct
from typing import BinaryIO

def extract(infile: str, outfile: str):
    with open(outfile, 'wb+') as out:
        with open(infile, 'rb') as f:
            extract_gif(f, out)

def extract_gif(rng: BinaryIO, out: BinaryIO):
    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)

    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
