#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int) -> None:
    # Write the GIF header.
    out.write('GIF89a'.encode())  # Magic identifier.
    # The logical dimensions of the image.
    out.write(struct.pack('<HH', width, height))
    # Color table with one color, fully transparent.
    # This is the default value, so we don't need to write it explicitly if we want background translucency (which we do).
    out.write('\0\xFF\0\xFF\0')
    # Background color index. We must set this or else the first frame will be black.
    out.write(b'\0')
    # Graphic control extension. Required for each frame.
    out.write(b'\245\010\0\0\0\0\0')
    # Image descriptor.
    out.write(b',\0\1\0\0\0!\0')
    # LZW minimum code size.
    out.write(b'\0')

def main() -> None:
    generate_gif(open('/dev/stdout