#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    width = 256
    height = 192
    
    # Header (GIF87a signature)
    out.write("GIF87a".encode())
    # Logical screen descriptior
    out.write(struct.pack('<HHBB', width, height, 0xF, 0))
    # Global color table
    data = bytearray([i % 256 for i in range(256*3)])
    out.write(data)
    # Graphics control extension
    out.write(b"\x21\xf9\x04\x00\xff\x00")
    # Image descriptor
    out.write(struct.pack('<HHHBB', 0, 0, width, height, 0xF))
    # Local color table
    out.write(data)

    def write_func(v):
        v &= 0xFF
        if v < 0x80:
            return v
