#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # Read a bit from our RNG
    first_byte = ord(rng.read(1)) & 0b01111111

    if first_byte <= 32:
            # This is harder to do right:

            # Make sure the frame size will be at least two bytes wide and tall.
            width = (first_byte << 8) | ord(rng.read(1)) + 2
            height = (first_byte << 8) | ord(rng.read(1)) + 2
            out.write(struct.pack('!3sHHB', header, width, height, 0xF4))

            # Palette has 1 color
            out.write(b'\x00\x00\xFF')

            # Graphics control extension with disposal method 2 (restore to background color)
            out.write(b'\x21\xf9\x04\x01\x00\x00\xff\x00')

            # Image descriptor
            out.write(b'\x2c\x00\x00\x00\x00\x00\xFF\x00')

            # Local palette
            out.write(b'\x00\x00\xFF')

            # Fill the image with that one color.
            # We use the width as stored in the GCE for this, which may include
            # some padding we don't have to write, but that's fine.
            out.write((width * 3 - 1) * b'\x00\x00\xFF')
    else:
        # This is relatively easy to do right:
        out.write(struct.pack('!6sHBBB', header,