#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the provided `palette`.
def generate_gif(out: BinaryIO, palette: list[int]) -> None:
    # Write out the 'GIF87a' header.
    out.write("GIF87a".encode())
    
    # 'screen width' and 'screen height'.
    out.write(struct.pack("<H", 16))
    out.write(struct.pack("<H", 16))
    
    # Global color table flag, color resolution (8-bit), sort flag (0 =
    # not sorted), size of global color table (-1 means 2^n - 1).
    out.write(b"\x80\x00\x00")
    
    for entry in palette:
        out.write(struct.pack("<B", entry))
        
    # Background color index.
    out.write(b"\x00")
    
    # Pixel aspect ratio (unused in this case).
    out.write(b"\x00")
    
    # An application extension, which we use to store data about the
    # colors used. This is very useful!
    out.write(struct.pack("<B", 0x21)) # extension introducer
    out.write(struct.pack("<B", 0xFF)) # application identifier
    out.write(struct.pack("<B", 0x0B)) # block length
    out.write("NETSCAPE2.0".encode()) # 'Netscape 2.0' application name
    out.write(struct.pack("<B", 0x03)) # loop count is 3
    out.write(struct.pack("<B", 0x01)) # terminator
    
    # Remaining bytes are all 0xFF.
