#!/usr/bin/env python3
import os
from flask import Flask, jsonify, request, Response
from werkzeug.exceptions import InternalServerError, ServiceUnavailable, NotFound

# from backend import get_info, get_top_concepts, get_subgraph
from backend import get_info, get_subgraph, get_similarity_matrix, get_embedding, get_entity_embeddings

app = Flask(__name__)

@app.route('/')
def index():
    return jsonify({'status':'ok'})

@app.route('/get', methods=['GET'])
def get():
    # get entity name and concept from url
    entity_name = request.args.get("entity")
    if not entity_name:
        print(request.args)
        raise BadRequest('Parameter "entity" must be provided')    
    concept = request.args.get("concept", None)

    try:
        res = get_info(entity_name, concept)
    except Exception as e:
        print(e)
        raise InternalServerError()

    for k in ('description', 'count'):
        if res[k] == []:
            del res[k] 
        
    return jsonify(res)

@app.route('/subgraph', methods=['GET'])
def subgraph():
    # get entity name and concept from url
    entity_name = request.args.get("entity")
    if not entity_name:
        raise BadRequest('Parameter "entity" must be provided')
    concept = request.args.get("concept", None)

    try:
        res = get_subgraph(entity_name, concept)
    except Exception as e:
        print(e)
        raise InternalServerError()

    return jsonify(res)

@app.route('/similarity_matrix', methods=['GET'])
def similarity_matrix():
    # get entity name and concept from url
    entity_name = request.args.get("entity")
    if not entity_name:
        raise BadRequest('Parameter "entity" must be provided')
    concept = request.args.get("concept", None)

    try:
        res = get_similarity_matrix(entity_name, concept)
    except Exception as e:
        print(e)
        raise InternalServerError()

    return jsonify(res)

@app.route('/embedding', methods=['GET'])
def embedding():
    # get entity name and concept from url
    entity_name = request.args.get("entity")
    if not entity_name:
        raise BadRequest('Parameter "entity" must be provided')
    concept = request.args.get("concept", None)

    try:
        res = get_embedding(entity_name, concept)
    except Exception as e:
        print(e)
        raise InternalServerError()

    return jsonify(res)

@app.route('/entity_embeddings', methods=['GET'])
def entity_embeddings():
   # get entity name and concept from url
    entity_name = request.args.get("entities")
    if not entity_name:
        raise Bad