#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # HEADER

    # Magic bytes (GIF87a or GIF89a)
    magic = [ord('G'), ord('I'), ord('F')]
    if ord(rng.read(1)) & 2:
        magic.append(ord('8'))
    else:
        magic.append(ord('9'))
    magic.extend([ord('a'), 0])

    out.write(bytes(magic))

    # Logical screen size
    width, height = struct.unpack(">HH", rng.read(4))
    out.write(struct.pack(">HH", width, height))

    # Background color
    background = rng.read(1)[0]
    out.write(struct.pack("B", background))

    # Aspect ratio and color map flags
    aspect_flags = rng.read(1)[0] + 1
    out.write(struct.pack("B", aspect_flags))

    if aspect_flags == 5:
        out.write(rng.read(16))
    elif aspect_flags == 6:
        out.write(rng.read(256 * 3))

    # TODO: Application extension (optional)

    # Image descriptor block
    out.write(b'!\xff\x0bNETSCAPE2.0\x03\x01')

    # Loop count and termination code
    out.write(b'\x00\x01\x00')

    # IMAGE DATA

    for _ in range(randrange(width * height // 50)):
        pass
