#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1<<24
    data = rng.read(datalen)
    # GIF files are little-endian; if we want to write integers
    # directly to the output file, we need to reverse byte order
    # for them.
    struct.pack('<H', 700, out=out)
    struct.pack('<H', 500, out=out)
    struct.pack('B', 0xff, out=out)

    assert len(header) == 6
    out.write(header)
    # first frame
    lsd = struct.pack('BBBHH', 7, 9, 1, 0, 0, 0)
    gct = struct.pack('BBBBBB', 0xff, 0x00, 0xff, 0x00, 0xff, 0x00)
    imd = struct.pack('BBBxxBB', 0x21, 3, 4, 3, 2, 0)
    xywh = struct.pack('BBHHB', 0x2c, 0, 0, 7, 5, 0)
    out.write