#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)

    # I want to use this field to specify it as not having one, but
    # my version of Python doesn't have the relevant constant
    global_color_table_flag = 0x7F

    out.write(struct.pack(
            '<BBBBIHBBBBB',
            *header,
            image_width, image_height,
            0, # packed fields
            global_color_table_flag,
            0, # color resolution
            0, # sort flag
            255, # max value for global color table
            # no need for background color index or pixel aspect ratio
        ))

    # end of logical screen descriptor; start of global color table
    out.write(b'\x00')
    out.write(rng.read(768))

    while True:
        # control extension
        # flags: unknown, block terminator, reserved
        # block size: 0x04
        # transparent color index:
        # delay time: from 5ms to 4s in steps of 10ms
        # transparent color index:
        # block terminator
        flags = rng.read(1)[0] & ~0b10000000
        delay_time = ord(rng.read(1)) + 5
        transparent_index = rng.read(1)[0]
        comment = rng.read(rng.read(1)[0])
        out.write(struct.pack('<BBBBI',
                flags, 0x04 | 0x80,
                delay_time, transparent_index, len(comment)))
        out.write(comment)
