#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def gen_gif(out:BinaryIO, rng):
    header = struct.pack('<6s', b'GIF87a')
    width = struct.unpack("<H", rng.read(2))[0] % 1000
    height = struct.unpack("<H", rng.read(2))[0] % 1000
    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt page 70
    image_block = (struct.pack("!HHB", width, height, 0b10000000) +
                   open("/usr/share/dict/words").readline().encode() +
                   b'\x00\x00' * 10)
    out.write(header + image_block)
    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt page 54
    datalen = struct.unpack(
        ">H",
        rng.read(2)
    ) & 0xffffff
    data = rng.read(datalen)
    out.write(header + data)
