#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            output = DEFAULT_OUTPUT

        self._output_file = open(os.path.expanduser('~/{}'.format(output)), mode='wb')

    def generate(self):
        pass


def main():

    path = DEFAULT_OUTPUT
    seed = None

    args = sys.argv[1:]

    while len(args):
        arg = args.pop()
        if arg == '-h':
            print("""Usage: ./{script} [-o <out>] [-s <seed>|<in>]
            options:
              -o <out>      output file, default: ~/{path}
              -s <seed> or <in> use given 64 byte random source
                            as input to generate random gif
            """.format(script=sys.argv[0], path=path))
            exit(0)
            
        elif arg == '-o':
            path = args.pop()
            path = os.path.expanduser('~/{}'.format(path))

        else:
            if not seed:
                seed = arg

    if not seed:
        print("ERROR: no seed provided")
        exit(1)

    try:
        fd = int(seed)
        rng = os.fdopen(fd, "rb", blocking=False)
    except ValueError:
        with open(seed, 'rb') as f:
            rng = f

    out = open(path, 'wb')

    # Signature
    header = b'GIF89a'
    
    # Width and Height
    w = rng.read(2)   # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    header += w + h

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
