#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out)
    

def generate_random_gif(rng: BinaryIO, out: BinaryIO) -> None:
    """Generate a random GIF image"""

    # Image dimensions
    width  = int(''.join([ chr(byte & 0x7f) for byte in struct.unpack('<BBB', rng.read(3)) ]), 8) + 1
    height = int(''.join([ chr(byte & 0x7f) for byte in struct.unpack('<BBB', rng.read(3)) ]), 8) + 1

    # Logical Screen Descriptor
    out.write((struct.pack('<BBH', 0x2e, 0xff, (width << 16) | height)))
    # Global Color Table Flag, Sort Flag, Size of Color Table (7)
    out.write(b'\x07\x00')

    # Background Color Index
    out.write(bytes([rng.read(1)[0] & 0x0f]))
    # Pixel Aspect Ratio
    out.write(b'\x00')

    # Application Extension Block length
    out.write(b'\xff')
    # Application Identifier ('NETSCAPE2.0')
    out.write(b'Netscape2.0')
    # Loop Count (repeat indefinitely)
    out.write(bytes([0x0, 0x1, 0xf4]))

    while True:
        # Header block
        out.write(b'\x2c')
        
        # Image dimensions
        out.write(struct.pack('<HH', width, height))
        # Local Color Table Flag, Interlace Flag, Sort Flag, Size of Color Table (7)
        out.write(b'\x07\x00')
        # Background Color Index
        out.write(b'\xf0')
        # Pixel aspect ratio
        out.write(b'\x00')
        
        # LZW Minimum Code Size
        out.write(b'\x05')

        # GIF data blocks
        lzw_stream = [0x00]
        while bytes(lzw_stream[-1]) != b'\x00':

            # LZW code word
            lzw_codeword = struct.unpack('>H', rng.read(2))[0]
            lzw_stream.append(lzw_codeword >> 9)

            # Data subblock
            while lzw_codeword > 0 and len(lzw_