#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate(out: BinaryIO, rng: BinaryIO) -> None:
    header = b'GIF89a\x1b\x00\x00\x00\x00\xff\xfe\xff\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x00;\x00\x00\x00\x00,\x00\x00\x00\x01\x00\x04\x00\x00\x00\x00,\x00!\xf9\x04\x01\x00\x00\x00\x00,\x00,\x00\x00\x00\x00,\x00\x05\x00\x01\x00\x01\x00\x00;'
    data = bytes([next(rng) for _ in range(0x7D00)])
    padding = (b'\x00',) * 16
    out.write(header + len(data).to_bytes(3, 'big'))
    while byte := rng.read(blocksize):
        if blocksize - len(byte) > 0:
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            out.write(byte[:blocksize])

