#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    header = b"GIF87a"
    # Image width and height
    width, height = 1024, 1024
    # Standard color table with three colors (black, green, white)
    color_table = [
        0x000000, 0x00FF00, 0xFFFFFF,
    ]
    # Length of colormap in bytes
    color_map_length = len(color_table) * 3
    # Size of image in bytes
    image_size = width * height
    # Signal end of metadata block
    meta_end = bytearray([0x00])
    # Repeat forever
    loop_forever = struct.pack("!HHB", 0, 0, 0)
    # Plain text extension "Hello World!"
    text_extension = struct.pack(
            "BBHB", 0x21, 0xFF, 11, 1
        ) + b"Hello world!\n" + meta_end
    # App extension
    app_extension = struct.pack(
            "!BBHB5s",
            0x21, 0xFF, 11, 16,
            b"Python   ",
        ) + meta_end
    # Logical screen descriptor
    lsd = struct.pack(
            "!HHHHBB",
            width, height,
            # Global color map flag
            0b1 << 7 | color_map_length - 1,
            0, # Background color index
            0, # Aspect ratio
        )
    # Graphic control extension (off for now)
    gce = struct.pack(
            "!BBBB",
            # Extension introducer
            0x21,
            # Graphic control label
            0xF9,
            0, # Block size
            # Disposal method
             out.write(header + data)
