#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Signature
    out.write(b"GIF89a")
    while True:
        # Loop over all of the chunks in this image
        for chunk_type in [b"!", b"IEND"] + [None] * 127:
            if chunk_type is None:
                # Choose a non-standard length between 0 and 65536
                length = (struct.unpack("<H", rng.read(2))[0] & 0xffff) % 65536
            else:
                # Choose a standard length of either zero or one byte
                length = 0 if rng.read(1)[0] % 2 == 0 else 1
            
            if chunk_type is not None:
                # Write the type and length of this chunk
                out.write(chunk_type)
                
                # Write the contents of this chunk
                out.write((struct.pack(">H", length)))
                out.writelines([rng.read(length)] * (length > 0))
    