#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open(self.output, 'wb') as fp:
            
            # GIF Header. 87a instead of plain a to prevent confusion between the two
            fp.write('GIF87a'.encode())

            # Logical Screen Descriptor
            # Width and Height are both set to 400 here because that is what the original GIF contained.
            # I don't intend to mess with this part too much for now.
            # Packing bytes in a weird way so that they come out in the right order.
            fp.write(struct.pack('H', 400))
            fp.write(struct.pack('H', 400))
            fp.write(b'\x00\x01')   # Has Global Color Table (to keep things simple)
                                     # TODO: find another GIF with a global color table to direct copy from it
            fp.write(struct.pack('B', 0))    # Background Color Index
            fp.write(struct.pack('B', 0))    # Pixel Aspect Ratio
            
            # Global Colors Table
            # 256 colors are supported
            # Each color is represented by three bytes.
            # RGB values are randomly generated [0-255]
            r = lambda : random.randint(0, 255)
            fp.write(struct.pack('BBB', r(), r(), r()))
            fp.write(struct.pack('BBB', r(), r(), r()))
            fp.write(struct.pack('BBB', r(), r(), r()))
            ...
            
            # Graphic Control Extension
            # No Graphic Control Extension is present (just leave this blank; will fill in later)
            # Why don't we use an extension block? Because there's no need to.
            fp.write(b'\xFF\xF9\x04\x04\x00\x00\x00\x00')
            
            # Image Descriptor
            fp.write(struct.pack('H', 0))       # Left Position
            fp.write(struct.pack('H', 0))       # Top Position
            fp.write(struct.pack('H', 400))     # Width
            fp.write(struct.pack('H', 400))     # Height
            fp.write(b'\x00')                   # No Local Color Table
            fp.write(struct.pack('B', 0))       # Reserved / LCT Sort Flag
            fp.write(b'\x00')                   # Number of bits per pixel
            
            # Image Data Block
            # One pixel only; all 256 colors are used at once.
            # Rearranged into groups of 256 pixels wide
            # The first pixel is at the bottom left.
            image_data = []
            for i in range(256 * 256):
                image_data.append((i // 256) % 256)
                image_data.append(i % 256)
                image_data.append(random.randint(0, 255))
        

# driver function
def main(argv):
    try:
        gif = RandomGif