#!/usr/bin/env python3

import struct
from typing import BinaryIO

PNG_MAGIC = [137, 80, 78, 71]
# Chunk types that we want.
CHUNKS = ['IHDR', 'PLTE']


def read_png(f: BinaryIO) -> dict[str, bytes]:
    """Read a PNG file from the given argument and return the data in a mapping."""
    assert f.read(len(PNG_MAGIC)) == bytearray(PNG_MAGIC), "Not a valid PNG file"
    chunks = {}
    while True:
        chunk_length = int.from_bytes(f.read(4), 'big')
        if not chunk_length:
            break
        chunk_type = f.read(4).decode('ascii').rstrip('\x00