#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int) -> None:
  # Write header information.
  out.write("GIF89a")
  
  # Ensure that the header is properly padded for the initial data block.
  while len(out.getbuffer()) % 256 != 0:
    out.write("\x00")
    
  def set_bit(value: int, bit: int):
    return (1 << bit) | value
  
  # Set up header flags indicating unused fields.
  gif_flags = 0x00
  gif_flags = set_bit(gif_flags, 0) # Global Color Table Flag (No).
  gif_flags = set_bit(gif_flags, 1) # Sort Flag (No).
  gif_flags = set_bit(gif_flags, 4) # Local Color Table Flag (Yes).
  
  # Write global color table and screen descriptor.
  out.write(struct.pack("<BHHBB", 7, width - 1, height - 1, gif_flags))
  
  # Write local color table.
  out.write(b"\x00\x00\x00" * 3 + b"\xFF\xFF\xFF")
  
  # Write application extension to indicate disposal method; see
  # https://www.w3.org/Graphics/GIF/spec-gif87.txt (p. 18) for more details.
  # Note that this extension will be ignored by browsers such as Firefox or Chrome.
  #
  # Take advantage of the fact that this extension has an unofficial status. It's not
  # worth it to implement the full GIF spec here.
  out.write((
      "\x21\xff\x0bNETSCAPE2.0\x03\x01\x00" # Extension Header
      + "\x00"                             # Subblock ID
      +