#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'  # GIF magic value
    width = rng.read(2)
    height = rng.read(2)
    flags = bytearray([0x1B])  # Global Color Table Flag + Bit Depth
    background = rng.read(1)
    pixel_aspect = bytearray([0xff])   # No aspect ratio information available
    global_colors = [0] * 768
    for i in range(768):
        local_color = rng.read(3)
        global_colors[i] = (local_color[0], local_color[1], local_color[2])

    palette = bytearray()
    for color in global_colors:
         palette += bytes(bytearray(color))

    # End the image data section (IDS)
    trailer = b'\x3b'

    out.write(header)
    out.write(width)
    out.write(height)
    out.