#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: "BinaryIO", out: "BinaryIO"):
    # header (13 bytes, randomly generated)
    header = bytearray(b'\x47\x49\x46\x38')
    for _ in range(12 - len(header)):
        header += rng.read(1)

    while True:
        # read input
        data_length = int.from_bytes(rng.read(2), 'little', signed=False)
        print("data_length = ", data_length)
        block_id = int.from_bytes(rng.read(1), 'big', signed=True)
        #print("block_id = ", block_id)

        if data_length == 0 or block_id < 0:
            break

        data = rng.read(data_length)
        out.write(header + data)
