#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write header
    out.write('GIF89a'.encode())

    # Logical Screen Descriptor (LSDescriptor)
    # Width: 640 pixels
    # Height: 480 pixels
    # Color Resolution and Depth: 256 colors (2**8 = 256)
    # Background Color Index: 1 (Black)
    # Pixel Aspect Ratio: 255 (default): no aspect ratio set
    out.write(struct.pack("<HHBB", 640, 480, 0xF7, 0))

    # Global Color Table (ColorTable)
    # Number of Colors: 256
    # Color Resolution and Depth: 8 bits (256 colors)
    out.write(b'\xFF' + bytes([i for i in range(256)]))

    # Graphic Control Extension (GraphicControl)
    # Block Size: 4
    # Packed Fields:
    #   - Reserved: 0
    #   - Disposal Method: 3 (restore to background color)
    #   - User Input Flag: False
    #   - Transparent Color Flag: True
    # Delay Time = 10ms
    # Transparent Color Index: 1 (Black)
    out.write(struct.pack("BHBB", 0xF9, 0x13, 0x01, 1))

    # Image Descriptor (ImageBlock)
    # Left Position: 0px
    # Top Position: 0px
    # Width: 640 pixels (same as LSDescriptor)
    # Height: 480 pixels (same as LSDescriptor)
    # Local Color Table: False
    # Interlaced: False
    # Sort Flag: False
    out.write(struct.