#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )



# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Signature
    header = b'GIF89a'
    
    # Width and Height
    w = rng.read(2)   # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    header += w + h

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        out.write(struct.pack('B', rng.read(1)[0]))

    # Background color index
    bcg = rng.read(1)[0]
    header += struct.pack('B', bcg)

    # Pixel aspect ratio (unused)
    par = rng.read(1)[0]
    header += struct.pack('B', par)

    out.write(header)

    # Write image data
    # 255 is the end of the stream marker
    while True:

        # Image separator
        sep = rng.read(1)[0]

        if sep == 255:
            break
        
        lsda = rng.read(1)[0]

        left_xpos = struct.unpack('B', rng.read(1))[0]
        top_ypos = struct.unpack('B', rng.read(1))[0]
        width = struct.unpack('B', rng.read(1))[0]
        height = struct.unpack('B', rng.read(1))[0]

        # TODO fix this
        # Packed field - local color table exists
        has_lct = bool(lsda & 0b10000000)

        # LCT sort flag
        lct_sort = bool(lsda & 0b01000000)

        # Global Color Table size
        gct_size = ((lsda >> 4) & 7) + 1
        gct_size *= 3

        # Resolution
        resolution = (lsda & 0b00000111) + 1

        # Global color table index
        gct_idx = struct.unpack('B', rng.read(1))[0]

        # Reserved byte - always zero
        reserved = rng.read(1)[0]

        # Local color table
        for _ in range(lsda - 1):
            pass

        out.write((sep, lsda))
        out.write((left_xpos, top_ypos, width, height))
        out.write(struct.pack('B', gct_size))
        out.write((reserved,))

        # Data subblocks
        while True:

            # Block size
            bs = rng.read(1)

            if bs == b'\x00':
