#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
    out: BinaryIO,
    rng: BinaryIO):

    # The header starts with 6 bytes
    # - 3 for "GIF" + \x00 as padding
    # - 3 for version ("87a") + \x20 as padding
    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt
    header = b'GIF\x00\x01\x00\x20\x00'

    width, height = struct.unpack("BB", rng.read(2))

    # Then there are two fields of size 2 each
    # https://web.archive.org/web/20141005013613/http://www.vurdalakov.net/misc/gif/*
    background_color = int.to_bytes(int(rng.read(1).hex(), base=16), length=1, byteorder='big')
    pixel_aspect_ratio = int.to_bytes(int(rng.read(1).hex(), base=16), length=1, byteorder='big')

    header += (width, height, background_color, pixel_aspect_ratio)
    
    out.write(header)
