#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)

    datalen = int.from_bytes(
        rng.read(1), byteorder='big')

    # the extension block is very simple, it doesn't contain any fields that we're interested in
    extlen = 0x1b

    total_size = len(header) + 2 * struct.calcsize('<H') + datalen + extlen;
    print("Random GIF", str(image_width) + " x " + str(image_height), str(total_size) + " bytes")

    data = rng.read(datalen)

    out.write(header + data)
