#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

        
    # First byte is always 'G'

    gif_magic = b"\x47"
    data = bytes([gif_magic])
    i = 1
    
    def randbyte() -> int:
        nonlocal i
        
        if i == len(data):
            data += rng.read(i * 2)

        return data[i] ^ data[i + 1] & (0b1111 << 4)

    
    data += bytes((randbyte(), ))
    i+=1
    
    
    # Second byte is always 'I'
    gif_type = b"\x49"
    data += bytes([gif_type])
    i+=1


    # Next 6 bytes are version string
    gif_version = b"\x56\x38\x37\x39\x61"
    data += gif_version
    i+=6
    

    # Rest of header block

    while True:

        # Read the first byte
        mask = randbyte()

        # End when we see 0
        if not mask: break


        # Figure out what to do with it
        if 0b10000000 & mask!= 0:
            
            # Extention, skip and continue
            extlen = randbyte()

            for _ in range(extlen):
                randbyte()

        else:

            # Only handle graphic control extension
            if mask == 0xF9:

                # Packed field
                packed = [
                    randbyte() for _ in range(3)
                ]
            
                
                # Delay time
                delay = randint(0, 0xFFFF)
                


                # Transparent color index
                transidx = -1 if bool(packed[2] >> 7) else None


                # Handle disposal method
                dispose = {
                    0: "GIF_DISPOSE_NONE",
                    1: "GIF_DISPOSE_BACKGROUND",
                    2: "GIF_DISPOSE_PREVIOUS",
                }[packed[2] & 0b111]
                
                pass