#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1)[0].to_bytes(1, byteorder='big'), 'big')
    data = rng.read(datalen)
    out.write(header +
            len(data).to_bytes(2, 'little', signed=False))
    out.write(data)
    while True:
        blocktype = rng.read(1)[0]
        if blocktype in { 0x2c, 0x21 }:
            # image descriptor or extension.
            # We don't care for that right now... Just skip it.
            assert rng.read(4 * 7 == b'\xff\xff\xff\xff')
            continue
        elif blocktype == 0x3b:
            break
