#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            header  = bytearray()                       # Header bytes
            frames  = list()                            # Frames to write
            
            width   = int.from_bytes(rng.read(2),     "little") + 150
            height  = int.from_bytes(rng.read(2),     "little") + 150
            maxval  = int.from_bytes(rng.read(2),     "little") // 8 * 7
            bgndcol = int.from_bytes(rng.read(4),     "little")
            aspect  = float.fromhex("0x" + rng.read(6).hex()) / 100

            header += b'\x47\x49\x46\x38\x37\x61'       \
                      + (width*height).to_bytes(4, "little", signed=False)         \
                      + b'\x00'                              \
                      + bgndcol.to_bytes(3,                "big")                   \
                      + b'\x00\x3B'                          \
                      + (b'\xFF\xFE'                         \
                         if maxval < (2**16)-1               \
                         else b'\x00\x00')                    \
                      + b'\x00\xF7\x00\x00'                  \
                      + b'\x00\xB0\x00\x00'                  \
                      + aspect.to_bytes(3,                 "big")                   \
                      + b'\x00\x00\x00'                      \
                      + (maxval % 256).to_bytes(1,          "big")                   \
                      + (maxval//256 % 256).to_bytes(1,      "big")                   \
                      + b'\x00\x3B'                         

        
            header += b'\x2C\x00\x00\x00\x00'            \
                       + width.to_bytes(2,                 "little")              \
                       + height.to_bytes(2,                "little")              \
                       + b'\x00\x00'                         \
                       + b'\x00\x00'                         \
                       + b'\x00\x00'                         \
                       + b'\x00\x00'                         \
                       + b'\x00\x00'                         \
                       + b'\x00\x00'                         \
                       + b'\x00\x44\x00\x3B'                 \
                       + b'\xD7\x00\xBB'                     \
                       + b'\xE0\x00\xDD'                     \
                       + b'\xE7\x00\xF3'


            for i in range((width*height)//32):
                frame = bytearray()

                local_bgndcol = rng.read(3)

                dc_changes = [local_bgndcol]
                
                while len(dc_changes)!= 2 and str(dc_changes[-1])!= str(local_bgndcol):
                    next_dc = rng.read(3)
                    if str(next_dc) == str(local_bgndcol):
                        continue

                    dc_change = lambda x, y : abs(int.from_bytes(x,"little")) > abs(int.from_bytes(y,"little"))
                    
                    if dc_change(dc_changes[len(dc_changes)-1], next_dc):
                        del dc_changes[-1]

                    dc_changes.append(next_dc)

                line_length = (width - min([i+1, 7])) * 3

                dc_change_start = rng.randint(0,line_length-1)
                next_dc_idx     = rng.randrange(0,min([len(dc_changes),2]))
                dc_change_end   = (dc_change_start + rnd_byte(rng)) % line_length
                if dc_change_end >= dc_change_start:
                    dc_changes[ next_dc_idx ] = change_color(dc_changes[ next_dc_idx ],
                                                              rng.read(3))


                local_colors = []

                for j in range(2):
                
                    color = rng.read(3)
                    while len(local_colors)!= 2 and str(local_colors[-1])!= str(color):
                        next_color = rng.read(3)

                        if str(next_color) == str(color):
                            continue
                        
                        local_colors.append(next_color)
                    
                local_color_idx = 0 if rnd_bit(rng)==True else 1

                local_data = bytearray()
                for k in range(width):
                    
                    line = bytearray(b'\x00'*line_length)

                    self._update_line(line,
                                     dc_change_start,
                                     dc_change_end,
                                     dc_changes[next_dc_idx],
                                     local_colors[local_color_idx])

                    next_dc_idx = (next_dc_idx + rnd_byte(rng) )%min([len(dc_changes),2])
                    if dc_change_end < width:
                        dc_change_end =(dc_change_end + rnd_byte(rng)) % width

                    local_color_idx = (local_color_idx + rnd_bit(rng))%3

                    local_data += line
            
                size = len(header)+len(frame)
                size = f'{size}'.encode('ascii')

                if len(size)<2:
                    size = size + b'\x00'
                elif len(size)<3:
                    size = b'\x00'+size 
                elif len(size)>3:
                    size = size[:3]

                frame += size             \
                     + b'\x2c'             \
                     + width.to_bytes(2, "little")             \
                     + height.to_bytes(2, "little")            \
                     + b'\x00\x00'         \
                     + ((width*(k+1)))//2.to_bytes(2, "little", signed=False)\
                     + local_bgndcol        \
                     + local_data          

                frames.append(frame)

            block = header

            for frame in frames:
                block += frame
                
            for _ in range(rnd_byte(rng)):
                block += b'\x00'*(1024-len(block)-(len(frames)*10))
            
            out.write(block)
    
    def update_line(self, line, dc_change_start, dc_change_end, new_value, old_value ):

        for i in range(dc_change_start,(dc_change_end+1)%len(line)):
            col_idx = i*3   
            colA = bytes(line[col_idx:col_idx+3])
            colB = bytes(new_value[col_idx:col_idx+3])
            colC = bytes(old_value[col_idx:col_idx+3])

            line[col_idx:col_idx+3] = [
                max([(colA[_]^colB[_]),(colC[_]^colB[_])][_] for _ in range(3)] 
            )



def rnd_bit(rng):
    return bool(ord(rng.read(1)));
        
def rnd_byte(rng):
    return ord(rng.read(1));

def change_color(col, rgb_values):
    """Change color by a small amount."""

    r = int.from_bytes(rgb_values[0:1],"little");
    g = int.from_bytes(rgb_values[1:2],"little");
    b = int.from_bytes(rgb_values[2:3],"little");
    
    return tuple([(col[_]+r)&0xff
                   for _ in range(3)])
















