#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    blocksize, padding = 256 * (data[1] & 0x07) + 1, [b'\x00']

    # Write header and length of image to output
    out.write(header + bytes([
        0, 0, 0, 0,         # width, height
        datalen % blocksize | padding[0],    # LZW minimum code size
        0]))                # no colors in global palette
    
    while len(data):

        # If we have enough data left for another block...
        if len(data) >= blocksize or not len(data):

            # Mark this block as being so
            out.write((len(data) < blocksize).to_bytes(
                1, 'little'))

            # Output the packed data
            for i in range(
                    0, min(blocksize,
                            len(data)), 4):
                out.write(struct.pack('>L',
                    ((data[i+3] << 24)
                        | (data[i+2] << 16)
                        | (data[i+1] << 8 )
                        | (data[i]))))
            
            # Read more data
            data = rng.read(datalen)
        
        else:

            # Pad with zeroes until next full block
            data += b'\x00' * \
                (blocksize - len(data))