#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    print('Generating a random gif...')

    header = b'GIF89a\x01\x00\x01\x00\xf0\x02\x00\x00\x00\x00,\x00\x00' \
             b'\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'

    width, height = 16, 16
    colors = 2 ** (width * height)
    size = int((colors // 4) * 7)
    rgb = [struct.unpack('<BBB', rng.read(3)) for _ in range(size)]

    # Select only unique colors
    colors = list({tuple(color) for color in rgb})[:colors]
    
    data = bytes()
    data += struct.pack(
        '<HHHBB', width, height, len(rgb),
        0b1000 | ((len(rgb) - 1) & 0b1111), 0x00
    )

    for color in colors:
        r, g, b = map(lambda n: n >> 5, color)

        data += bytes([r << 4 | g, b])

    # Add frames to the end of the file
    for i in range(15):
        frame = bytearray()
        
        frame += b'\x2c\x00\x00\x00'
        frame += b'\xff\x0bNETSCAPE2.0\x03\x01'
        frame += struct.pack('<H', i + 1)
        frame += b''* 3

        data += memoryview(frame).tobytes()

    out.write(header + data)
