#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        self._output = DEFAULT_OUTPUT if not output else output

        return super().__init__()


    @property
    def output(self):
        """Output file path."""

        return self._output


    def run(self, rng: BinaryIO):

        with open(self._output, mode='wb') as out:

            while True:

                generate_sub_frame(out, rng)


                try:
                    # read 4 bytes to check if EOF was reached
                    rng.read(4)
                except Exception:  # catch any exception
                    break   # stop reading after last frame has been reached




def generate_sub_frame(out, rng):

    # Header
    identifier = rng.read(6)
    descriptor = rng.read(1)
    header = b'GIF89a'
    
    # Width and Height
    w = rng.read(2)   # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    header += w + h

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
