#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    assert len(data) == datalen, 'Not enough input bytes'

    # Header
    out.write(header + struct.pack('>HHBB', 420, 615, 8, 7))
    
    # Logical screen descriptor
    palette_size = (len(data) % 3) * 8
    global_color_table_flag = bool(palette_size)
    background_color = None if global_color_table_flag else 255
    pixel_aspect_ratio = 0x00
    flags |= 0b00001000   # sort flag
    flags |= 0b00000100   # color table size is specified in bits 5-7 of the descriptor
    flags |= 0b00000010   # global color table flag
    out.write(struct.pack('<HBBB', 40, 444, background_color or 255, flags | (global_color_table_flag << 7)))

    # Global Color Table
    for i in range(palette_size):
        r = next(rng) & 0xFF
        g = next(rng) & 0xFF
        b = next(rng) & 0xFF
        out.write(struct.pack('BBB', r, g, b))

    out.write(data)

    out.write(b'\x3B\x00')


if