#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    blocksize, padding = 256 * (data[1] & 0x07) + 1, [b'\x00']

    # Write header and logical screen descriptor
    out.write(header)
    out.write((
        4, 0, 0, 0, 0, 0, 0, 0,
        1, 0, 0, 0).pack('>BBHHBBBB'))

    # Append all subblocks to initialize LZW code table
    compressed_data = b''
    for i in range(blocksize):
        chunklen = min(
                datalen - len(compressed_data), blocksize - i)

        compressed_chunk = compress(data[:chunklen])
        compressed_data += compressed_chunk

        while len(compressed_chunk)!= blocksize:
            compressed_data += padding
            if datalen == len(compressed_data):
                break

            chunklen = min(
                    datalen - len(compressed_data), blocksize - i)
            compressed_chunk = compress(data[-chunklen:])
            compressed_data = compressed_data[:-len(padding)] \
                             + compressed_chunk

        datablock = compressed_data[i : i+blocksize]
        out.write(struct.pack('<H', len(datablock)) + datablock)
