#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1
    width = int.from_bytes(
            rng.read(2), byteorder='little')
    height = int.from_bytes(
            rng.read(2), byteorder='little')

    # Setting width to zero will cause an error, so we'll just set it to one
    if (width == 0):
        width = 1

    palette = [rng.read(4) for _ in range(256)]
    out.write(header)
    out.write(struct.pack('<HH', width, height))
    out.write(datalen.to_bytes(
            3, byteorder='big'))
    out.write((b'\xF7').repeat(7))
    
    coded_string = []
    while True:
        c = rng.read(1)[0]

        # EOF
        if (c == 255): break
        
        # RLE
        elif ((c & 0x80)!= 0): 
            seg_len = c - 0x7F

            if (seg_len > width - len(coded_string)):
                raise Exception('RLE segment too long.')
            
            coded_string += list(
                    rng.read(seg_len)[:width-len(coded_string)]) * \
                    int.from_bytes(rng.read(1), 'big')
            
        else:    
            coded_string.append(c)
        
    out.write(bytearray([palette[i][l] 
        for i in coded_string for l in range(4)]))