#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False
    )

    for i in range(datalen):
        palette_idx = i % 256 // (datalen / len(header))

        # The magic value is chosen so that it can be interpreted both as an
        # integer and as a float. See https://en.wikipedia.org/wiki/LZW
        #
        # Since we're just outputting raw bytes here, I didn't bother to ensure
        # that this actually yielded something valid. In practice, I know that
        # all of my image files will have some form of compression applied,
        # whether or not they are lossless.
        val = ((palette_idx % 256) + 0x7F).to_bytes(
            1, byteorder='big', signed=True
            )

        if i < len(header):
            out.write(
                header[i].to_bytes(
                    1, byteorder='big', signed=False
                    ))

            continue

        out.write(val)


