#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
    
        with open('random-palette', mode='w') as palettefile:

            for i in range (0, 256):

                r = os.urandom(1)
                g = os.urandom(1)
                b = os.urandom(1)

                # To get a value between 0 and 255 we need to do some shifting
                palettefile.write((str(int.from_bytes(r, byteorder='little')) + ','
                                    + str(int.from_bytes(g, byteorder='little')) + ',' 
                                    + str(int.from_bytes(b, byteorder='little')))
                                  )

                if int.from_bytes(i, byteorder='little') < 255:
                    palettefile.write('\n')

        print("Palette file generated!")


        frame_generator = FrameGenerator()
        frame_generator.generate_frames()
        
        # Create the GIF header
        gif_header_data : bytes = struct.pack('<6sHHBBB', b"GIF89a", 
                                            frame_generator.width, 
                                            frame_generator.height,
                                            7, # LZW
                                            0, 
                                            0, 
                                            0)

        # Write the header out to disk
        with open(self.output, mode='wb') as outfile:
            
            outfile.write(gif_header_data)
            
            # Since this is just a test program I will hard code all of these values 
            # but you should read them from the input frames instead. 
            gif_graphics_control_extention_data : bytes = struct.pack('<BBHB', 
                                                                     4, # size
                                                                     0x0B, # type - graphic control extension
                                                                     0x00, # flags
                                                                     100) # Delay of 100ms


            # We need to add one frame per loop
            for i in range(frame_generator.loopcount):
                
                # Add the graphics control extention data 
                outfile.write(gif_graphics_control_extention_data)

                # Read the image data into memory so that it can be compressed later on. 
                with open ('frame-' + str(i), mode='rb') as f:
                    img_data = f.read()

                    # Add a label specifying how many pixels there are in each line 
                    outfile.write(struct.pack('<BBBB', 0x2C, 
                                               0x00, 
                                               ((frame_generator.width+7)//8)*frame_generator.height, 
                                               0))
                    
                    # Add the length of the image data to the file
                    outfile.write(struct.pack('<sH', len(img_data).to_bytes(2,'big'), 0xffff))

                    # Add the raw pixel data itself
                    outfile.write(img_data)
                    
                # End of Image marker                
                outfile.write(struct.pack('<BB', 0x3B, 0x00))





class