#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    header = bytes([
        0x47, 0x49, 0x46,         # Magic value and version
        0x38,                     # Header size in bytes (always 7)
        0x1B,                     # Color resolution
        0x08,                     # Background color index
        0x01,                     # Pixel aspect ratio
        0x00, 0x00,               # Image width + height (upper left corner)
        0x2C, 0x00, 0x00           # Frame control + delay time
            ])

    palette_size = 256 * 3
    data_size = 1.6e+6 - len(header) - palette_size
    frame_data_size = int((data_size / 0xFFFF) * 2)

    background_color = 0xFFFFFF & rng.read(3)
    frame_data = bytearray()
    for _ in range(frame_data_size // 2):
        # Each pixel is a two-byte RGB value
        next_pixel = [background_color]
        while any(next_pixel == c for c in frame_data):
            next_pixel[0] ^= 0xFF
        frame_data += bytearray(struct.pack(">H", next_pixel[0]))

    with open('palette.bin', 'rb') as pal:
        palette = pal.read()[:len(frame_data)]

    print("Writing " + str(len(header)) + " bytes of GIF header")
    out.write(header)
    print("Writing " + str(len(palette)) +
          " bytes of 256 x 8-bit palette values")
    out.write(palette)
    print("Writing " + str(len(frame_data)) + " bytes of image data")
    out.write(bytes(frame_data))

