#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(2), byteorder='little', signed=False
    )

    # Print Ranges
    print('Data range: %d - 65535 (0x%X - 0xFFFF)' %
          (datalen >> 1, datalen >> 1))

    # Generate data
    block = bytes()
    for i in range(datalen):
        if len(block) == 0 or len(block) >= 0xFF:
            block += b'\x00'

        block += bytes([rng.read(1)[0] ^
                        (i & 0xff) ^ ((i << 4) & 0xff)])

    out.write(header +
              len(block).to_bytes(2, byteorder='little') +
              block)


if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rd:
        generate_random_gif(rd, open('./test.gif', 'wb'))
