#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open('header.gif', mode='wb') as header,\
             open('data.bin', mode='wb') as data:
            
            # Header
            magic = b'GIF89a'
            version = [0x47, 0x49, 0x46]

            width = int.from_bytes(rng.read(2), byteorder='big', signed=False) + 1
            height = int.from_bytes(rng.read(2), byteorder='big', signed=False) + 1

            packed = bytes((
                0xF7,                 # Frame control bits
                *width.to_bytes(2, byteorder='big'),     # Width
                *height.to_bytes(2, byteorder='big'),    # Height
                0x00, 0x00             # Packed field (none set)
                 ))
                
            color_table = bytearray([
                0xFF, 0xFF, 0xFF,      # Background
                0x00, 0x00, 0x00       # Foreground
                ])
            
            image_descriptor = bytearray([
                0x2C,                         # Image descriptor block
                0x00, 0x00,                   # Left offset
                0x00, 0x00,                   # Top offset
                *width.to_bytes(2, byteorder='big'),          # Width
                *height.to_bytes(2, byteorder='big'),         # Height
                0x00,                         # Local palette (not included in the file)
                0x00                          # LZW minimum size for codes (any value is accepted by specs)
                ])


            # Write everything to files
            header.write(magic)
            header.write(bytes(version))
            header.write(packed)
            
            data.write(image_descriptor)

            # Generate a random table of RGB colors and write it to ghetto GIF
            n_colors = len(color_table) // 3

            for i in range(n_colors):
                red   = int.from_bytes(rng.read(1), byteorder='big', signed=True) % 256
                green = int.from_bytes(rng.read(1), byteorder='big', signed=True) % 256
                blue  = int.from_bytes(rng.read(1), byteorder='big', signed=True) % 256

                color_table[i*3] = red
                color_table[i*3+1] = green
                color_table[i*3+2] = blue

            header.write(bytes(color_table))

            # Add the rest of the data from /dev/urandom into the binary file
            while True:
                chunk = rng.read(512)
                
                if not chunk or len(chunk) == 0:
                    break
                    
                data.write(chunk)


        # Append the binary file onto the end of the GIF
        with open(self.output, "ab") as out:
            with open("data.bin", "rb") as datafile:
                buffer = datafile.read()
                out.seek(-1, os.SEEK_END)
                out.truncate()
                out.write(buffer)

                
    def clean(self):
        os.remove('header.gif')
        os.remove('data.bin')
        

if