#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate(out: BinaryIO, rng):
    # Write header

    file_header = b'\x47\x49\x46'  # magic value

    width = rng.randint(10, 255)
    height = rng.randint(10, 255)
    flags = 8 | (rng.randint(0, 7) << 5)
    palette = [
        (((rng.getrandbits(5)) & 0xff), ((rng.getrandbits(5)) & 0xff),
         ((rng.getrandbits(5)) & 0xff)),
        (((rng.getrandbits(5)) & 0xff), ((rng.getrandbits(5)) & 0xff),
         ((rng.getrandbits(5)) & 0xff)),
        (((rng.getrandbits(5)) & 0xff), ((rng.getrandbits(5)) & 0xff),
         ((rng.getrandbits(5)) & 0xff))]
    block = bytes()
    for i in range(width * height):
        block += bytearray([palette[rng.randint(0, len(palette)-1)][0],
                            palette[rng.randint(0, len(palette)-1)][1],
                            palette[rng.randint(0, len(palette)-1)][2]])

    # Magic Value
    out.write(file_header)
    # Version
    out.write(b'\x38')
    out.write(struct.pack('<HHB',
                          width,
                          height,
                          0xf7))
    # Block size
    out.write(b'\x00')
    # Color table size
    out.write(bytearray([len(palette)]))
    for c in palette:
        out.write(bytes([(c[0] >> 3),
                         ((c[0]) & 0x07) << 5 |
                         (c[1] >> 3),
                         ((c[1]) & 0x07) << 5 |
                         (c[2] >> 3)]))
    # Extension area
    out.write(b'\x21\xff\x0bNETSCAPE2.0')
    out.write(b'\x03\x01')
    out.write(b'\x00\x00')
    out.write(b'\x00')
    out.write(b'\x3b')
    # Image descriptor and image data
    out.write(b'\x2c')
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(block)

    out.write(header + data)
