#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        out: BinaryIO,
        width: int, height: int, loop_count: int):

    # Header and Logical Screen Descriptor
    header = b'GIF89a' \
             + (b'\x21\xff\x0b'
                + (width % 0x100).to_bytes(1, 'big')  # Width
                + ((width >> 8) % 0x100).to_bytes(1, 'big')) \
             + (b'\x00'
                + (height % 0x100).to_bytes(1, 'big') # Height
                + ((height >> 8) % 0x100).to_bytes(1, 'big')) \
             + b'\x0f'                              \
             + (loop_count % 0x100).to_bytes(1, 'big')     \
             + ((loop_count >> 8) % 0x100).to_bytes(1, 'big')

    # Image Descriptor
    # TODO: choose image type as well?
    header += b'\x2c'                            \
              + (0 % 0x100).to_bytes(1, 'big')   \
              + ((0 >> 8) % 0x100).to_bytes(1, 'big') \
              + (0 % 0x100).to_bytes(1, 'big')   \
              + ((0 >> 8) % 0x100).to_bytes(1, 'big') \
              + (width % 0x100).to_bytes(1, 'big') \
              + ((width >> 8) % 0x100).to_bytes(1, 'big') \
              + (height % 0x100).to_bytes(1, 'big') \
              + ((height >> 8) % 0x100).to_bytes(1, 'big')

    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False
    ) & 0xffffff
    data = rng.read(datalen)
    out.write(header + data)
