#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # 4 bytes of magic
    out.write(header)
    # Write width and height in pixel bytes, Little-Endian (LSB first)
    width = int.to_bytes((1 << 7 | rng.read(1)[0]), length=2, byteorder='little')
    height = int.to_bytes((1 << 7 | rng.read(1)[0]), length=2, byteorder='little')
    out.write(width + height)
    # Write color table size as one byte that is always zero for this example
    out.write(b'\x00')
    gif_filetype = [
        b'\xF7\xA5', b'\x0F\xDC', b'\x37\x1A', b'\x0E\xE6',
        b'\xB5\x4C', b'\xEE\xD9', b'\x7D\xAD', b'\x6A\xC7',
        b'\xCB\x1B', b'\x8A\x42', b'\x19\xBD', b'\xF7\x77',
        b'\x75\x6A', b'\xAF\x8F', b'\xBC\x94', b'\x0A\x26']
    out.write(gif_filetype[rng.read(1)[0]])

    # Write screen descriptor block with resolution 640 x 480
    out.write(struct.pack('>HHBBBHH', 640, 480, 0xf, 0, 0, 0))

    frame_count = rng.read(1)[0] % 10
    print('# of frames:', frame_count)
    delay = rng.read(2)[0] % 30
    # Write global application extension with subblocks
    out.write(b'\x21\xFF')
    appext_size = struct.pack('<H', 2*frame_count+1+(frame_count > 0)*2)
    out.write(appext_size)
    out.write(b'\x09\x04')
    out.write(b'\x01\x00')
    if frame_count > 0:
        out.write(struct.pack('BB', frame_count, delay))

    for _ in range(frame_count):
        out.write(b'\x2c')

        lzwmincodesize = min(rng.read(1)[0], 12)
        out.write(lzwmincodesize.to_bytes(1, 'big'))
        
        # TODO: add image data here
    # TODO: add trailer

