#!/usr/bin/env python3
import re
from string import ascii_lowercase as letters
from itertools import product


def main():
    # Read in input file and remove trailing new line characters.
    data = open('input.txt', 'r').read().rstrip()

    # Part 1: count all the valid passwords according to their rules.
    print(f"Part 1: {count_valid_passwords(data)}")

    # Part 2: count only those passwords that meet additional criteria apart from the three listed above.
    print(
        f"Part 2: {count_valid_passwords(data, rule=lambda a, b, c, d: (a == b)!= (b == c))}")


# Count how many passwords are valid based on initial password rules.
def count_valid_passwords(data, rule=None):
    return len([p for p in read_password_rules(data) if is_valid(*p, rule)])


# Return True or False depending on whether the given password meets its validation rules.
def is_valid(a, b, c, d, rule):
    # Password matches rules when any character falls within the range of either of two sets.
    return ((a <= c) | (c < b)) & rule((a <= c), (c < b)) & (d not in [a, b, c])


# Parse all lines in the input file into tuples containing each password's set of four rules.
def read_password_rules(lines):
    pattern = r'(\w+)-(\w+) (\w+): (\w+)'              # Matches passwords of format "x-y z: w".
    regex = re.compile(pattern)                        # Compile the regular expression once.
    for line in lines.split('\n'):                      # Split the input by newline characters.
        match = regex.match(line).groups()             # Get an iterator with matched groups.
        yield map(int, match[:4]), match[4]            # Convert first four groups to ints, leave last one alone.


if