#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('>HH', rng.read(4))
    
    # pixel aspect ratio is 8 bits
    pixelaspectratio = ord(rng.read(1))
    
    flags = [*(0 for _ in range(8))]
    for i in range(7):
        if (pixelaspectratio & (1 << i)):
            flags[i] = 1

    # global color table present
    backgroundcolorindex = ord(rng.read(1))
    flag = bool((flags[-2] << 1) | flags[-1])
    print(backgroundcolorindex, flag)

    sortflag = flags[5]
    size = int(sum([(f << s) for f,s in zip(flags[:3],range(3,-1,-1))]))
    datalen = ((image_width * image_height) // size)

    data = rng.read(datalen)
    out.write(header + data)
