#!/usr/bin/env python3

import argparse
import os
import struct
from typing import BinaryIO

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--seed', required=True, help='The seed to use.')
    args = parser.parse_args()

    # Open both stdout and stderr as binary I/O streams
    with open('stdout.txt', 'wb') as out, \
         open('/dev/urandom', 'rb') as urandom:

        # Seed the RNG from the command line argument
        urandom.seek(int(args.seed))

        generate_random_gif(urandom, out)


# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)