#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    PILOT = b"GIF89a\x04\x00\x0d\x00\x00"
    WIDTH_HEIGHT = bytes([
        0xFF & ((int.from_bytes(
            rng.read(1), byteorder="big") % 128) | 128),
        0xFF & ((int.from_bytes(
            rng.read(1), byteorder="big") % 128) | 128)
    ])
    COLOR_BITS = bytes([
        0xFF & (((int.from_bytes(
            rng.read(1), byteorder="big") % 1) << 7)),
        0xFF & ((int.from_bytes(
            rng.read(1), byteorder="big") % 128))
    ])
    TRANSPARENCY = bytes([0xff & (
        (int.from_bytes(rng.read(1), "big") % 256) >> 1)])
    
    BLOCKSIZE = 256*(0xa1 & int.from_bytes(WIDTH_HEIGHT, "big"))+1
    PADDING = bytes([(BLOCKSIZE-1)&0xff]+[0]*((BLOCKSIZE-1)>>8))
    LENGTH = bytes([len(PILOT)+len(COLOR_BITS)+len(TRANSPARENCY)+
                    len(WIDTH_HEIGHT)])

    out.write(PILOT+LENGTH+WIDTH_HEIGHT+COLOR_BITS+TRANSPARENCY)
    while byte:=rng.read(BLOCKSIZE):
        if len(byte)<BLOCKSIZE:
            # Pad if necessary
            out.write(byte+PADDING)
        elif len(byte)==BLOCKSIZE:
            out.write(byte)
    