#!/usr/bin/env python3

import sys
import timeit
from functools import partial
from io import BytesIO
from typing import IO

import numpy as np


# Generate a random data array of size `size` and dtype=`dtype`,
# using `/dev/urandom`. The data is chopped to `chop_size` chunks.
def randgen(size=None, dtype='uint64',
            chop_size=np.int_(sys.maxsize)):

    with open('/dev/urandom', 'rb') as fd:

        # If no size specified, read everything from /dev/urandom
        if not size:

            while True:
                data = fd.read()

                yield np.frombuffer(data, dtype=dtype).astype('i')

        else:

            chunk = next(randgen())

            for i in range((size // chop_size)+1):

                if len(chunk[0]) <= (size - i*chop_size):

                    yield chunk[:][:(size - i*chop_size)]
                    break

                elif len(chunk[0]) > (size - i*chop_size):

                    yield chunk[:][-(size - i*chop_size):]
                    break

                else:
                    continue


def timer(func, *args, **kwargs):
    print("Time elapsed: {}".format(
          str(timeit.timeit(
              partial(func, *args, **kwargs), number=5))))



if __name__ == '__main__':


    def gen(*args, **kwargs):
        return np.concatenate([next(randgen(*args, **kwargs))
                               for _ in range(7)])


    timer(gen, size=(1 << 24), nptype='uint64')