#!/usr/bin/env python3

import os
import sys
import struct


MAGIC = 'GIF89a'.encode('ascii')
DEFAULT_DATALEN = len(MAGIC) + 120

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# with a maximum sized image at most `maxdata`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, maxdata=None):

    if not maxdata:
        maxdata = DEFAULT_DATALEN

    maxdata -= len(MAGIC)

    def readlen():
        return min(struct.unpack('<H', rng.read(2))[0], maxdata)

    out.write(MAGIC)

    for _ in range(7):
        buf = rng.read(readlen())

        out.write(buf[:3])
        out.write(struct.pack("<B", (ord(buf[3]) & ~0x2)))
        out.write(buf[4:])


if __name__ == '__main__':
    try:
        f = open('/dev/urandom')
        generate_random_gif(f, sys.stdout.buffer)
    except IOError as e:
        print("Could not open /dev/urandom:", str(e))
        sys.exit(-1)