#!/usr/bin/env python3

import os
import sys
import subprocess
import contextlib


@contextlib.contextmanager
def open_or_exit(path):
    try:
        yield open(path, 'wb')
    except FileNotFoundError as e:
        exit(f"File not found at {path}: {str(e)}") from None
    finally:
        if "OUTFILE" in locals():
            OUTFILE.close()


with open("/dev/urandom", "rb") as RNG:
    with open_or_exit("random-gifs/source/file_0.txt") as TXT:

        # Generate a random txt file containing sequence of bytes
        def generate_randomly_sized_text_file(
                rng: BinaryIO, out: BinaryIO, maxlen=None
            ) -> int:

            # Write random text to the output file and return its size
            text = ""
            while True:
                byte = hex(rng.read(1)[0])[2:]

                char = chr(int(byte, base=16))
                if char == '\n':
                    text += '\\n\n'
                    break
                elif char!= '\\':
                    text += char
                else:
                    text = ''

            length = len(text)
            for line in text.splitlines():
                out.write(line.encode('utf-8'))
                out.write(b'\n')

            return length


        def generate_random_gif():
            gif = subprocess.Popen(['convert', '-size', '50x30', 'txt:-', 'png:-'],
                                    stdin=subprocess.PIPE, stdout=subprocess.PIPE)


            generate_randomly_sized_text_file(RNG, TXT)
            # Read generated PNG file
            pngbytes = gif.stdout.read()
            # Encode it again without metadata
            gifencode = subprocess.run(['convert', '-', 'gif:-'], input=pngbytes, capture_output=True)
            return gifencode.stdout





    # TODO: Implement actual GIF generation (see https://github.com/aguinet/drawbot/blob/master/examples/issues/issue_575_clippingTestImage.py)


