#!/usr/bin/env python3

import sys
import os
import struct
import time
from typing import BinaryIO
from pathlib import Path

PACKAGE_ROOT = str(Path(__file__) / '..' / '..').replace('\\','/')
sys.path.append(PACKAGE_ROOT)

from src.common.utils import hexdump
from src.common.config import load_cfg



RANDOM_FILE = '/dev/urandom'
CFG_FILE = 'data/random-gifgen.yaml'
CFG = None

def load_cfg_(cfg_file):
    global CFG
    CFG = load_cfg(cfg_file)
    print('Using config:')
    print(CFG)


def main():

    cfg_file = Path('.') / 'data' / 'random-gifgen.yaml'
    if len(sys.argv) == 2 or len(sys.argv) > 3:
        usage()

    
    try:
        load_cfg_(cfg_file)
    except Exception as e:
        print("Failed loading configuration:", e)
        exit(-1)
    

    
    if len(sys.argv) >= 2:
        output = sys.argv[1]
    elif CFG['output']:
        output = CFG['output']
    else:
        output = DEFAULT_OUTPUT
        
    print('Output:', output)
    
    

    try:
    
        with open(output, "bw+") as f_o:
            
            # Signature
            f_o.write(b"GIF89a")
    
        
            # Width and Height
            f_o.write((struct.pack("<H", 400)))      # width is 16 bit
            f_o.write((struct.pack("<H", 450)))      # height is 16 bit
    
            # Colour table size
            f_o.write(bytes([3*(1 << int(CFG['color_table']))]))
    
            # Block Size (Data Sub-Block within Data Stream)
            f_o.write(bytes([int(CFG["sub-block"])]))


            buffer = []
            for i in range(int(CFG['num_frames'])):
                
                print('Frame',i,'...')

                while True:
                    
                    pixel_color = randint(0, 2**int(CFG['color_table'])-1)
                    
                    if pixel_color not in buffer:
                        break
                    
                buffer.append(pixel_color)


                buffer_size = int(len(buffer)/3)*3 # Step by three because we assume rgb pixels are used
                
               # print('Buffer:',buffer)
                num_pixels = int(CFG['width'])*int(CFG['height'])
                assert num_pixels <= buffer_size
            
                pixel_list = [buffer[:3]]
                del buffer[:3]
                for j in range(num_pixels//3 - 1):

                    while True:

                        pixel_color = randint(0, 2**int(CFG['color_table'])-1)
                        
                        if pixel_color not in buffer:
                            break
                        
                    pixel_list.append(pixel_color)

                    del buffer[:3]

                #print('Pixel list:\n',pixel_list)
                
                # Write out the frame info to the file
                f_o.write(bytes([int(CFG["sub-block"])]))
                f_o.write(bytes([ord(CFG["separator"])]))

                offset = 0
                for pxl in pixel_list:
                    color = [(pxl & 255), (pxl >> 8) & 255, (pxl >> 16) & 255]
                    f_o.write(bytes(color))
                    offset += 3

                # Fill up last byte of this row so that the following one has space
                padding = ((offset / 3 + 1) % 8) % 8
                print('Padding:',padding)
                for _ in range(padding):
                   f_o.write(bytes([0,0,0]))



    except FileNotFoundError:
        traceback.format_exc()
        print("Could not create '%s'"%output)
        exit(-1)
    finally:
        pass

    exit(0)






if __name__=="__main__":
    main()
































