#!/usr/bin/env python3

import sys
import os
import io
import timeit
import random
import functools
import argparse
import struct
from typing import BinaryIO

def timer(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        start = timeit.default_timer()

        res = f(*args, **kwargs)

        end = timeit.default_timer()
        print('elapsed:', (end - start))

        return res

    return wrapper


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
@timer
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, size: int=1 << 20
    ):
    fmt = 'B' * size
    #data = bytes(os.urandom((size)))
    data = struct.pack(fmt, *[random.randint(0, 255) for i in range(size)])
    #out.write(b'\x47\x49\x46\x38\x39\x61')
    #out.write(data)
    out.write(struct.pack('<6s', b'GIF89a'))


