#!/usr/bin/env python3
"""Generate a random GIF file."""

import sys
import os
import struct
from typing import BinaryIO, List

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )



# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    )->List[int]:

    # Signature
    header = [b'G', b'I', b'F', b'8', b'9', b'a']
        
    # Header
    header += list(rng.read(6))

    # Logical Screen Descriptor
    sd = []
    for _ in range(4): 
        sd += list(struct.pack('B', rng.read(1)[0])) 
    header += sd

    # Global Color Table Specification
    gct = []
    gctsize = header[-2] // 3
    for i in range(gctsize):
        gct += list(struct.pack('BBB',
                                *[
                                    rng.read(1)[0], 
                                    rng.read(1)[0],
                                    rng.read(1)[0]
                                ]
                            ))
    header += gct

    # Image Descriptor
    idd = []
    for _ in range(5): 
        idd += list(struct.pack('B', rng.read(1)[0])) 

    header += idd

    # Image data
    imdata = []
    while True:
        imdata += list(rng.read(1))[0]
        if len(imdata)%2!= 0:
            imdata += list(rng.read(1))[0]
        if len(imdata)%2 == 0 and imdata[len(imdata)-3] == ord(b';'):
            break

    header += imdata
    out.write(bytes(header))

    # Write extra byte if odd amount of data
    if len(imdata) > 0 and len(imdata)%2!= 0:
        out.write(bytes(1))

