#!/usr/bin/env python3

import os
import sys
import timeit

import imageio

from PIL import Image

from itertools import permutations


# Return `n` (pseudo-)random bytes from `rng`. This is used to seed the RNGs for
# both Python and C implementations. For example, if we have two generators with
# 4-byte seeds, then this returns `[seed0, seed1, seed2, seed3]`, or the first
# four bytes of each input stream.
def get_four_bytes(rng):
    return [next(rng).to_bytes(1, 'little', signed=False)] * 4

# Generate N bytes of randomness from `rng` and write it into `fout` in little
# endian order.
def generate_random_bytes(
        n: int, rng: BinaryIO, fout: BinaryIO
    ) -> None:

    while True:

        # Get 64KiB chunks at a time until we've written enough bytes
        chunk = rng.read(min(n, 65536))
        n -= len(chunk)
        if not chunk: break
        fout.write(chunk)

    assert n == 0

# Write a single frame to the output file. The frames will be reordered so that
# they're generated randomly, but still repeatable.
def generate_frame(rng, frame, palette):
    cur_cell = next(rng)

    row_of_pixels = []
    for _ in range(8):
        pixel = pallete[cur_cell % len(palette)]
        row_of_pixels += pixel
        cur_cell //= len(palette)


    print(row_of_pixels[:3])

    lenx = min(len(row_of_pixels)//8, 60)
    im = Image.frombytes('P', (lenx, 8), bytes(row_of_pixels[:lenx*8]))
    img = np.array(im)
    writer.append_data(img)



if __name__ == '__main__':
    import numpy as np
    from io import BytesIO

    rng = open('/dev/urandom', 'rb')
    outfile = sys.argv[1]

    parts = [''.join([str(c) for c in perm]).ljust(3, '-') 
        for perm in list(permutations((0,1,2)))][::-1]
    
    images=[Image.new("RGB", (10, 10), c) for c in parts]
    images[0].save('test.gif', save_all=True, append_images=images[1:], optimize=False, duration=1000, loop=0)

    exit()

    streams = [BytesIO(), BytesIO()]
    pallete = [(0, 0, 0)]
    # pallete += [(np.uint8(r//8)*8, r%8*8, 0) for r in range(1, 7)]
    pallete += [(i//8*8, i%8*8, 0) for i in range(16)]

    for sidx in range(2):
        generate_random_bytes(
            65536, 
            rng, 
            streams[sidx],
        )
        print(streams[sidx]._buffer)

    im = Image.new('RGB', (10, 10))
    writer = imageio.get_writer('./test.gif', mode='I', duration=.001)

    def impl():
        rndgen = streams[int.from_bytes(os.urandom(1),'little')]
        while True:
            generate_frame(
                iter(rndgen.read(3)), 
                im, 
                pallete,
            )

    try:
        print(timeit.timeit(impl, number=100))
    finally:
        writer.close()