#!/usr/bin/env python3

import struct
from typing import BinaryIO
from PIL import Image


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom). The
# resulting image size is [(2**N - 1), (2**M - 1)]
def generate_random_gif(
        N: int, M: int,
        rng: BinaryIO, out: BinaryIO
    ):

    # Construct Gif Signature
    magic = b'GIF87a' if N == 7 else \
            b'GIF89a' if N <  7 and M > 7 else IndexError()
    
    # Write Header
    screenwidth  = ((1 << N)  - 1).to_bytes(2, "little")
    screenheight = ((1 << M)  - 1).to_bytes(2, "little")
    globalcolours =    b'\x00\x00'
    backgroundcolour = b'\xff\xff'
    pixelaspectratio = b'\x00\x04'
    out.write(magic + screenwidth + screenheight
                + globalcolours + backgroundcolour
                + pixelaspectratio)

    # Generate Image Data
    imgdata = Image.new("P", (screenwidth[0], screenheight[0]))
    imgdata.putpalette([0xff, 0xff, 0xff] * 256)
    for line in range(imgdata.size[1]):
        for col in range(imgdata.size[0]):
            imgdata.putpixel((col, line),
                            rng.read(1)[0] % len(imgdata.getcolors()))
    # Write Image
    out.write(b'')
    imgdata.save(out, format="gif")