#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate all possible nibbles (0-15) for a given bit value
# For example, get_nibbles(6) returns [2, 4] because
# the binary representation is 110 and these are its nibbles
def get_nibbles(bit):
    return ([1 << i for i in range(4)] if ((bit & 8) >> 3) else []) \
           + ([1 << i for i in range(4, 8)] if ((bit & 4) >> 2) else []) \
           + ([1 << i for i in range(8, 12)] if ((bit & 2) >> 1) else []) \
           + ([1 << i for i in range(12, 16)] if (((bit & 1) ^ (~bit)) & 1) == False else [])



# Converts a decimal integer to a 16 bit little endian word
def dec2leword(n):
    hibyte = n >> 8
    lobyte = n & 0xFF
    return bytes([lobyte,hibyte])


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=None, height=None, delay=None
    ):

    # Set default values if not specified as arguments
    if width == None:
        width = random.randrange(1, 2**9 - 1)
    if height == None:
        height = random.randrange(1, 2**9 - 1)
    if delay == None:
        delay = random.randrange(1, 2**3 - 1)# | random.randint(0, 7) << 3

    # Build the first part of the GIF file which is required before any image
    # data can be written to it
    header = bytes('GIF89a', 'ascii') \
             + bytes([width,height]) \
             + bytes([delay]) \
             + b'\x01\x00\xf7\xff'# color table with one entry set to background color

    out.write(header)

    while True:#for col in range(0, width*height):

        # Get the next byte from /dev/urandom
        r = ord(rng.read(1))

        # Write this byte's high and low nibble to the output stream
        out.write(bytes(get_nibbles(r)))

        # If we just wrote a byte that was the last of the row, write the ending zero nibble
        if (col % width) == (width - 1):
            out.write(b'\x0')

        # If this row has finished, exit the loop
        if (col % width) == (width - 1):
            break