#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (`/dev/urandom`) and
# writes it to stdout if `stdio` is true.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, stdio=False
    ) -> None:
    # Use `os.urandom` instead of `rng.read` for performance reasons as we're not
    # reading from this stream infinitely, so there's no need to re-init each time
    data = os.urandom(1<<16)

    # Write to stdout?
    if (stdio): print('Writing', len(data), 'bytes...')

    # Build the GIF header, with dimensions flipped around since they are in order
    # Height <--> Width
    width = height = int((len(data)**0.5) +.5)
    imageDesc = b'\x2C\x00\x00'+struct.pack('<HHBBB', width, height, 0xFF, 0, 0, 0)+b'\x00'
    returnDataSize = 1<<24
    globalColorTable = bytes([
        0xFF, 0xFF, 0xFF,        0x00, 0x00, 0x00, 
        0xF9, 0xE7, 0xD8,        0xC8, 0xA4, 0x8A, 
        0xFA, 0xEE, 0xE1,        0xBF, 0x9B, 0x7D, 
        0xFC, 0xC8, 0x8E,        0xAA, 0x68, 0x52, 
        0xFF, 0xDD, 0xBB,        0x7D, 0x3F, 0x27, 
        0xFE, 0xAC, 0x70,        0x52, 0x25, 0x12, 
        0xFD, 0x96, 0x49,        0x2F, 0x15, 0x0A, 
        0xFD, 0x80, 0x18,        0x12, 0x09, 0x05
    ])
    subHeader = b''+struct.pack('<BBBBBBBBBHB',
            0x21,                # Extension Introducer

            0xF9,                # Graphic Control Label
            0x04,                # Block Size

            int(width*height*.1), # Delay Time
            int(width*height*.1), # Transparent Color Index
            
            ord('!'),             # Block Terminator
            
            0x2C,                # Image Descriptor Label
            0x00,                # Left Offset
            0x00,                # Top Offset
            width & 0xFF,        # Width Low
            height & 0xFF,       # Height Low
            0x08,                # Local Color Table Flag
            0x07,                # LZW Min Code Size
            0x00                 # Block Terminator
        )
    
    # Write to stdout?
    if (stdio): 
        print('Width:', width, '\nHeight:', height, end='\n')
        
        print('Image descriptor:')
        print('\tWidth:', width, '\n\tHeight:', height, 
            '\n\tDelay time:', int(width*height*.1), 
            '\n\tTransparent color index:', int(width*height*.1))

    # Output chunks into binary form
    out.write(header + data)
