#!/usr/bin/python3

"""
Generates a random GIF file given
the desired output path. The resulting
file will be completely filled with
randomly generated data.
"""

__author__ = "<NAME>"


import os
import time
import struct
from typing import BinaryIO


# Defaults
DEFAULT_RANDOM_FILE  = '/dev/urandom'
DEFAULT_OUTPUT       = 'random.gif'

# Header constants
SIGNATURE            = b'GIF89a'
WIDTH                = 4  # bits
HEIGHT               = 4  # bits
COLOR_TABLE_SIZE     = 1  # byte
BLOCK_SIZE           = 1  # byte
SUB_BLOCK            = 1  # byte
DATA                 = 1  # byte


def main(argv):

    start = time.clock()
    print('Starting...')

    # Get command line arguments
    try:
        input_file  = argv[1]
        output_path = argv[2]
    except IndexError:
        raise ValueError("Please provide both paths.") from None

    # Get random data
    with open(input_file, 'rb') as f:
        random_data = f.read()

    # Create output directory
    dirname = os.path.dirname(os.path.abspath(output_path))
    dir_exists = os.path.isdir(dirname)
    if not dir_exists:
        os.makedirs(dir_exists)

    # Generate the header
    header = SIGNATURE
    header += struct.pack('<H', len(random_data)),      WIDTH
    header += struct.pack('<H', len(random_data)),      HEIGHT
    header += struct.pack('<B', len(random_data), DATA)
    header += random_data 
    
    # Write the header and random data to disk 
    with open(output_path, 'wb') as f:
        f.write(header)

    end = time.clock()
    elapsed = round((end - start), 5)
    print('Done! Took {} seconds.'.format(elapsed))

    return 0
        


if __name__ == '__main__':
    
    status = main(sys.argv)
    sys.exit(status)