#!/usr/bin/env python3
'''
Created on 5 Feb 2017

@author: julianporter
'''
from collections import namedtuple
import hashlib
import os
import struct


class RandomGifGenerator(object):
    
    def __init__(self, rng:BinaryIO=None):
        if not rng:
            self._rng=os.open('/dev/urandom',os.O_RDONLY)
        else:
            self._rng=rng
        
    def __enter__(self):
        return self

    def __exit__(self,*args,**kwargs):
        close(self._rng)
        del self._rng
            
    def writeHeader(self,out:BinaryIO):
        
        header = b'GIF89a'
        datalen = int.from_bytes(
                os.readv(self._rng,(3,)),byteorder='big',signed=False)
        digest=hashlib.sha256()
        for i in range(datalen+4096):
            digest.update(os.readv(self._rng,(1,),offset=i))

        checksum=digest.digest()[0:3]
        print('CHECKSUM {}'.format(checksum))
        
            
        # datalen = len(data)
        header +=datalen.to_bytes(3,'big') 
        header+=b'\x87\x00\xff'
        header+=checksum
        header+=struct.pack('<HHHBBBBBHB','','','','',"NETSCAPE","2.0",3," ",2)[-7:]
        header+=b';'
        # print(header)
        
        out.write(header)
        
        
                
    def writeData(self,out:BinaryIO,length:int=None):
        if length is None:
            while True:
                
                chunksize=min((32*1024,max(1024,os.fstatvfs(self._rng).f_bavail)))
                
                toRead=chunksize
                while toRead>0:
                    data=os.readv(self._rng,(1,),count=toRead,offset=-toRead)
                    out.write(data)
                    toRead-=len(data)
                    
                    
                yield chunksize
        else:
            current=0
            while current<length:
                chunksize=min((32*1024,length-current,max(1024,os.fstatvfs(self._rng).f_bavail)))
                
                
                
                size=os.readv(self._rng,(1,),count=chunksize)
                out.write(size)
                current+=len(size)
                yield chunksize
            
            
            raise StopIteration

            
            
            

        