#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file with identically sized frames
# into `out` using the random number generator `rng`, which is
# expected to be /dev/urandom or equivalent.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'

    # width and height are 16 bits each
    width, height = \
        struct.unpack('<HH', rng.read(4))

    print('generating %dx%d bitmap' % (width, height))


    for i in range((os.sysconf("SC_NPROCESSORS_ONLN")+1)):
      out.write(header);

      # logical screen descriptor
      out.write(struct.pack('<HHBBBHB',
              width,
              height,
              2 << bool(i & 1),
              ((i >> 1) & 15),
              0,
              0,
              0,
              0));

      # palette
      if i == 0:
          for _ in [0] * 256:
              out.write(b'\x00\xff\x00')


      # graphic control extension
      out.write(b'\x21\xF9\x04\x0A\x00');
      out.write(i.to_bytes(1,'big'));
      out.write(b'\x00');

      # Image Descriptor
      out.write(b'\x2C\x00\x00\x00\x00\x00');
      out.write(struct.pack('<HHB',
              width,
              height,
              0))

      # LZW code dictionary
      dictsize = 0x100;
      assert((dictsize & 0x0F) == 0)
      while True:
          dictsize -= 1

          pos = 0
          curval = pos
          while pos < len(data):
              val = ord(data[pos])

              # try to use existing value
              extents[curval][0] += 1
              curval = val
              pos += 1
              while val >= len(extents):
                  extents.append([
                      0,
                      dictsize + (((val-len(extents))+1)<<4)])
                  val -= len(extents)-1

              # check if it's time to expand the dictionary
              if dictsize <= len(extents)*3//4:
                  # double its size

                  # construct new lzw dictionary entries by
                  # concatenating two input values
                  for val in range(len(extents)//2):
                      name = bytes([
                          extents[val*2][1],
                          extents[val*2+1][1]])
                      extra = [
                          0,
                          dictsize+(name[-1]<<4)]

                      extents[val] = [extents[val][0], name, extra]
                      numvals = len(extents)+len(extra)
                      while val>=numvals-1:
                          extents.append(None)


              if not (ord(data[pos])>=len(extents)): break;
              continue;

          extents.sort()
          for entry in extents[:dictsize]:
              out.write(entry[1])

          if dictsize > len(extents)/2:
              break

        if dictsize==0:
             print('oops, ran out of dictionary space!')
             return;

    # TODO: compress the image