'''
Created on 5 Feb 2017

@author: julianporter
'''
import random
import struct
from typing import BinaryIO
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    
    # Write header
    header=b''
    for i in range(6):
        header+=struct.pack('c', format(random.randint(0,255),'02x').encode())
        
    size=[random.choice([2**i|j for j in [0]])for i in range(4)]
    out.write(header+size[0].to_bytes(1,'big')+size[1].to_bytes(1,'big'))

    # Write color table
    for i in range(pow(2,int((header[-1]-1)/3))+1):
        r=random.uniform(0,1)
        g=random.uniform(0,1)
        b=random.uniform(0,1)

        col=(int(r*255),int(g*255),int(b*255))
        
        for j in range(3):
            out.write(struct.pack('<B',col[j]))  

    # Write global colormap
    if len(header)>7 and (header[-7:-4] == b'\x90\xCF\x13'): 
        numcolors=pow(2,(header[-4])&0x07)+1
        for i in range(numcolors-1):
                out.write(struct.pack('<BBB',
                    *[random.randrange(0,256,1) for _ in range(3)]))
        
        
        

    # Write loop control
    if len(header)>7 and (header[-7:-4] == b'\x90\xCF\x13'):
        out.write(struct.pack('<BBHBBBBB',
             *(random.randrange(0,256,1) for _ in range(8))))
    
    # Write image descriptor
    x=[random.randint(0,max(size)) for _ in range(2)]
    y=[random.randint(0,max(size)) for _ in range(2)]


    if len(header)>7 and (header[-7:-4]!= b'\x90\xCF\x13'):
        out.write(struct.pack('<HHHHBBBB',
              *size,*x,*y,[random.randrange(0,256,1) for _ in range(8)]))
    else:
        out.write(struct.pack('<BBBHBH',
              *[random.randrange(0,256,1) for _ in range(6)]))

    # Write local colormap
    if len(header)>7 and (header[-7:-4] == b'\x90\xCF\x13'):
        out.write(struct.pack('<BB',*[random.randrange(0,256,1) for _ in range(2)]))

    # Write pixels  
    for row in range(size[1]):
        interlace=False
  
        for irow in range(row % 4,size[1],4):
            lzw=bytearray()
            
            lastmatch=-1
            while True:
                matchlen=random.randint(2,12)
                
                lzw.extend([lastmatch & 0xFF,matchlen & 0xFF]+
                          [random.randrange(0,256,1) for _ in range(matchlen)])
                lastmatch=lzw[-2]
                
                if random.random()>0.7: break

            lzwstr=b''.join(map(lambda x : bytes([x]),lzw))
            length=len(lzwstr).to_bytes(1,'little')
            out.write(length+lzwstr)
            
    pass
    
