#! /usr/bin/python3

import os
import sys
from argparse import ArgumentParser
from typing import BinaryIO


def main():
    parser = ArgumentParser()

    # The length of the generated headers
    parser.add_argument('length', help="the length in bytes to generate", type=int)

    args = parser.parse_args()

    with open('/dev/urandom', 'rb') as devrand:
        generate_random_gif(devrand, sys.stdout.buffer, args.length)


# Generates a random GIF file into `out` using the
# random number generator `rng` (opened for binary reading and writing).
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO,
        length: int):

    # Header magic string, 6 bytes long
    header = b"GIF89a"
    
    # Length of pixel data in bytes, 3 byte field, big endian encoded
    datalenbytes = length.to_bytes(3, "big")
    
    # Pixel data
    data = rng.read(length)
    
    # Write everything into stdout as binary data
    out.write(header + datalenbytes + data)


if __name__ == "__main__":
    main()