#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Creates an infinite stream of bytes from /dev/urandom
def urandom() -> BinaryIO:
    return open('/dev/urandom', 'rb')


# Generates a random GIF file into `out` using the
# random number generator `rng`. If `rng` is None, then
# it defaults to /dev/urandom
def generate_random_gif(
        rng: BinaryIO=None, out: BinaryIO
    ) -> None:
    
    if not rng:
        rng = urandom()

    # Each frame should be preceded by this string
    prefix = b'\x00\x21\xF9\x04\x05\x0A\x00'

    # The first two bytes are the image size in pixels
    # followed by a field separator (0xFF). This is followed
    # by the rest of the frame info
    image_width, image_height = \
        struct.unpack('<HH', os.urandom(4))

    
    preamble = prefix + image_width.to_bytes(
            2, byteorder='little') + image_height.to_bytes(
                1, byteorder='little')


    # Skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)

    # Each frame must have its own local colour palette so
    # we ignore them for now
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Each frame starts with the image ordeder value 0x2C (' ')
    # which is immediately followed by a length indicator for
    # the sub-block that follows
    out.write(preamble + b'\x2C' + len(data).to_bytes(1, byteorder='little'))

    # Finally write the actual encoded data
    out.write(data)