#!/usr/bin/env python3
'''
Created on 5 Feb 2017

@author: julianporter
'''
import struct
from typing import BinaryIO


class RandomGeneratorError(Exception):
    
    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)
        
    
def readStruct(fobj : BinaryIO,format,size=None):
    if size is None:
        return struct.unpack(format, fobj.read(struct.calcsize(format)))
    else:
        assert not isinstance(format,str)
        return [x for x in (struct.unpack(f,{b"B":1,b"H":2}[fmt], fobj.read(s)) for fmt, s in zip(format,size))]


def writeStruct(fobj : BinaryIO,dataTuple,format):
    fobj.write(struct.pack(format,*dataTuple))
    

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:
        
    # Read in a header 
    header = b'GIF89a'

    try:
        version, width, height = readStruct(
                rng,'3sHH',[len(header),(3,2)])
        
        globalColorTable = True
    
        if version == b'GIF8':
            globalColorTable = False
            colorResolution, sortFlag, gctSize = readStruct(
                    rng,'BBB',3)
            
            if gctSize > 0:
                lut = []
                
                for i in range(gctSize):
                    lut += [(i+1)*16] + list(readStruct(
                            rng,'BBB',3))
                    
                print("LUT",lut)
                
        elif version!= b'GIF89a':
            raise RandomGeneratorError('Invalid GIF file')
            
    except Exception as e:
        raise RandomGeneratorError('Failed to read initial parts of GIF file',e)
        return False
    
    # Write out the same header
    writeStruct(out,'3sHH','3sHH'+(b'BBB'*int(globalColorTable))+b'B'*gctSize)
    
    # Write out the rest of the content
    while True:
        blockLen = readStruct(rng,'B')[0]
        blockType = readStruct(rng,'c',blockLen)[0].decode()
        
        print(blockType)
        if blockType == 'I':
            continue
        elif blockType == 'L':
            break
        elif blockType == '!':
            pass
            
        else:
            raise RandomGeneratorError('Unknown code type')
            

    
    print("GOT IT")
    return True
                
    
