#!/usr/bin/env python3

import struct
import math
import sys
from typing import BinaryIO


# Generates a random GIF file into `out`, using the
# given random number generator `rng` (e.g., /dev/urandom).
# The sizes of frames are between minsize and maxsize bytes.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        minsize: int, maxsize: int
    ):

    # Write the magic string
    out.write('GIF89a'.encode())

    # Calculate how many frames there will be
    numframes = round(math.exp(2 ** rng.read(1)))


    for i in range(numframes):

        # Generate the frame size randomly
        framelen = int.from_bytes(
            4 * rng.read(1), byteorder='little', signed=False
        ) & 0xffffff

        # Make sure it's within bounds
        if framelen < minsize or framelen > maxsize:
            continue

        # Generate the pixel data
        framedata = rng.read(framelen)

        # Write the correct chunk header
        out.write((
            '\x2c\x00\x00\x00' + f'{framelen:0>6X}'.encode()
        ))

        # Write the pixel data
        out.write(framedata)

