#!/usr/bin/env python3

import os
import sys
from PIL import Image
from argparse import ArgumentParser


def main():
    ap = ArgumentParser(description="Generate a random gif")

    ap.add_argument("outfile", nargs="?",
                    help="path to output file (defaults to stdout)", default="")

    args = ap.parse_args()

    im = generate_random_imdata()
    im.save(sys.stdout.buffer if not args.outfile else open(args.outfile, "wb"))


# Generates a Pillow image with random data in it that can be used for testing
# purposes; the image will always have width 54 and height 127
def generate_random_imdata() -> Image:
    rng = os.fdopen(os.open("/dev/urandom", os.O_RDONLY)) # pylint: disable=consider-using-with
    return Image.frombytes('L', (width(), height()), bytes(generate_image_stream(rng)))


# Returns an iterator over the pixel values of this file's image data
def generate_image_stream(f):
    while True:
        newbyte = f.read(1)

        if len(newbyte) == 0:
            break
        
        yield from map(lambda x: x & 1, newbyte)

# The width of the generated image is hard-coded as 54 pixels because many
# terminal emulators assume that the the resulting image fits within 80 columns.
def width():
    return 54
    
# The height of the generated image is essentially unlimited so we only use a
# small amount of storage space in this script rather than limit it by memory size:
# each frame uses up approximately 6KB.
def height():
    return 127
