#!/usr/bin/env python3

# Copyright © 2017 <NAME> <<EMAIL>>
# This work is free. You can redistribute it and/or modify it under the terms of the Do What The Fuck You Want To Public License, Version 2, as published by Sam Hocevar. See http://www.wtfpl.net/ for more details.

# Generate random GIF files (and a ton more).
# Example usages:
# $ cat /dev/urandom |./generate_random_gif.py > random.gif
# $ dd if=/dev/urandom count=500 bs=1 | base64 | sed s/'\n'/'\\n'/g | xclip -selection clipboard
# $ echo "foobar" | tr A-Z a-z |./generate_random_gif.py > random.gif

from PIL import Image
import os
import subprocess
import sys
import tempfile
import argparse


__all__ = []


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output

    
    def generate_random_gif(self):
        
        tmpfile = tempfile.NamedTemporaryFile()
        pipe = subprocess.Popen(['convert'], stdin=subprocess.PIPE, stdout=tmpfile)
        
        rng = sys.stdin.buffer if hasattr(sys.stdin, 'buffer') else sys.stdin 
        while True:
            
            byte = rng.read(1)
            if len(byte) == 0:
                break
                
            pipe.stdin.write(byte)
            
        pipe.communicate()

        img = Image.open(tmpfile)
        img.save(self.output, format="GIF")



if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Generate random GIF files.')
    parser.add_argument('output', metavar='OUTPUT', nargs='?', default=None,
                        help='Output filename.')

    args = parser.parse_args()
    r = RandomGifGenerator(args.output)
    r.generate_random_gif()