#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def main():
    if len(sys.argv)!= 2:
        print('usage: {} <output-file>'.format(sys.argv[0]), file=sys.stderr)
        return -1

    try:
        with open('/dev/urandom', 'rb') as urandom,\
             open(sys.argv[1], 'wb') as out:

            generate_random_gif(urandom, out)

        return 0

    except Exception as err:
        # an error happened, but we still want to get rid of the output file in case it was partially written to and might be corrupt now or later
        if os.path.exists(sys.argv[1]):
                os.remove(sys.argv[1])
        raise


# Generates a random GIF file into `out` using the
# random number generator `rng` (opened for read or write)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    
    # 24 bits are used to store the length of image data
    datalen = bytes([int((os.fstat(rng.fileno()).st_size / 16711680.0) * 4096)])
    out.write(header + datalen)

    while True:
        chunklen = struct.unpack('>H', rng.read(2))[0]

        if not chunklen:
            break
        
        out.write(b'\x2c\xff\xf9')
        out.write(chunklen.to_bytes(2, 'little'))
        out.write(rng.read(chunklen))
        out.write(b';')

