#!/usr/bin/env python3

import struct
from typing import BinaryIO
from random import randrange as rr; from time import monotonic as t
import re

# Generates a random GIF file into `out` using the
# random number generator `rng` (with seed `seed`)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, seed=rr(2**64)):

    tm = t()
    print("Random seed:", seed)
    
    # We only want to use alphanumeric characters for an ID
    def genid(): return b"G" +\
        bytes([ord('A') + ord('Z') if c < 0x5d else
               ord('a') + ord('z') - 0xa for i in range(20)
                   for c in [
                        rr(2 ** 16 | 0x7f)]
            ])
        
    while True:
        size = 1 << ((tm + seed ^ tm % 4) & 10); tm += 1
        
        if not any((
            lambda: size > len(genid()),
            lambda: size + 6 >= len(b"\0" * mdlen),
            lambda: size + 6 >= len(b"".join(
                [struct.pack('<H', s + 1)
                 for s in sizes]
            )),
            lambda: size + 25 >= 256 * len(sizes)
        )) or size <= mdlen:
            
            break
            
    mdlen = rr(size // 2) & 0xffff
    md = list(range(mdlen))
    for p in range(1, mdlen):
        q = rr(p + 1)

        md[p], md[q] = md[q], md[p]

    
    sizes = []; curr = 1
    while sum(map(lambda x: x << 2, sizes)) + 256 < size:
        sizes.append(curr)
        curr <<= 1


    out.write(b"GIF")
    out.write(struct.pack("<B", 89))
    out.write(bytes([0xff]))
    out.write(struct.pack("<BH", 0, mdlen))
    for ln in md:
        out.write(struct.pack("<H", ln + 1))

    # LZW minimum code size
    lzwmincodesize = min(rr(7) + 2,
                         max(3, rr(3)))
    out.write(struct.pack("<BBBBB",
                          lzwmincodesize,
                          0, 0, 0, 0))

    rows = []
    while len(rows)!= len(sizes):
        rowlen = 1 << ((tm + seed ^ tm % 8) & 15); tm += 1
        line = []
        while len(line)!= rowlen and \
              any((
                  lambda: len(line) == 2 ** (lzwmincodesize+1) - 1,
                  lambda: len(line) == sizes[-1],
                  lambda: len(line)+1!= sizes[0],
              )):

            line.append(
                chr(rr(2**16)) 
            ); nextrow = rr(rowlen)
            if nextrow == rowlen: continue

            row = "".join(line).encode('ascii')
            if row[:max(1, rr(len(row)//8))]!= b'\x00':
                rows.append(row)
                line = ""
            else:
                print(
                    "Skipping zero-byte row length",
                    len(rows), "/", len(sizes),
                    ", row length", len(row), 
                    end="\r" if len(rows)%10==0 else "\n"
                )


        out.write(struct.pack("<B",
                             ord('!') if  2**8  <= len(line) <= 2**9 else
                             ord('\xFF') if 2**16 <= len(line) <= 2**17 else
                             0))    
        out.write( struct.pack(
             "<%dB"%(1 if ord('!')<=line[0]<0xff else 
                  2 if 0xff<line[0]<0xffff else
                  4 if 0xffff<line[0]<0xffffff else
                    8),
             *(line[:1<<ln])
             for ln in range(lzwmincodesize)[::-1]
        )); 
    print("")