import io
import os
import subprocess
import sys


def make_rng():
    # Use /dev/urandom as our pseudorandom source (if available).
    if sys.platform == 'linux':
        return open('/dev/urandom', 'rb')

    # If /dev/urandom doesn't exist, use "od -vAn -N4" to produce 4 bytes of
    # entropy and decode them with base64. This is not ideal but it will do for
    # now.
    print('warning: no pseudo-random device found!', file=sys.stderr)
    proc = subprocess.Popen(['od', '-vAn', '-N4'],
                            stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
    pipe = io.BytesIO(proc.communicate()[0])
    decoder = io.TextIOWrapper(pipe, errors='ignore')
    return decoder


def main():
    out = sys.stdout.buffer
    rng = make_rng()

    while True:
        # Generate a new random gif at each iteration.
        generate_random_gif(rng, out)

        # Add some jitter so we don't end up generating identical gifs all over
        # the place.
        sleep_time = float(os.environ['JITTER'])
        try:
            time.sleep(sleep_time)
        except KeyboardInterrupt:
            break


if __name__ == '__main__':
    main()