#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is determined by
# the first byte from `rng`, and can't be larger than
# 2^24 bytes (16 MiB) or smaller than 17 bytes.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Generate random width and height for the image.
    imgwidth  = int.from_bytes(b'\x01\x00\x00'+rng.read(2), 'big') % 16383 + 1
    imgheight = int.from_bytes(b'\x01\x00\x00'+rng.read(2), 'big') %  8192 + 1

    # Generate a color table consisting of 256 colors that are all white.
    colortable = []
    for _ in range(256):
        colortable.extend((0xFF,) * 3)

    # Write the magic header to the output file.
    out.write(b"GIF89a")

    # Write the logical screen descriptor.
    out.write(struct.pack('<HHBBB', imgwidth, imgheight, 0xF0, 0, 0))

    # Write the global color table.
    out.write(bytearray(colortable))

    # Write an application extension containing the loop count.
    out.write(b"\x21\xFF\x0BNETSCAPE2.0")
    out.write(b"\x03\x01")
    out.write(struct.pack("<H", 15))
    out.write(b"\x00")

    # Write the plain text section "NETSCAPE2.0".
    out.write(b"\x21\x00")
    out.write(b"Hello, world!")

    # Finish the GIF file with the image data block.
    out.write(b"\x2C\x00\x00\x00\x00")