#!/usr/bin/env python3

import struct
from typing import BinaryIO
from PIL import Image, ImageChops


def get_transparent():
    img = Image.new('RGBA', (160, 220))

    datas = []
    
    for i in range(img.height):
      if i % 5 == 0:
          continue

      new_data = [int((i * j) / 3) for j in range(img.width)]
            
      for x in range(len(new_data)):
        if new_data[x] < 127:
            new_data[x] += 127
        else:
            new_data[x] -= 127

      datas.append(bytearray(new_data))

    img.putdata(datas)

    return img


def generate_same_size(file: BinaryIO):
  buf = file.read()

  width = int.from_bytes(buf[:2], byteorder="little")
  height = int.from_bytes(buf[2:4], byteorder="little")

  image = Image.open("image.png").convert("P", palette=Image.ADAPTIVE)

  buf = image.tobytes()

  readable = "RGB"

  if len(buf)!= abs(width*height*(len(readable)//3)):
    raise ValueError(f"Invalid size {len(buf)}!= {abs(width*height*(len(readable)//3))}!")
  
  data = list(struct.iter_unpack("<B", buf))

  im = Image.new("RGBA", (width, height))
  im.putdata([x for _, x in sorted(zip(range(-1,-len(data)-1,-1), data))])

  return im

