#!/usr/bin/env python3

import struct
from typing import Sequence, Tuple

# Generates an uncompressed GIFT format to be used for testing purposes
def generate_uncompressed_gift(
        width: int, height: int, frames: Sequence[Tuple[int, bytes]]
    ) -> bytes:

    result = []
    
    # Initialize with boilerplate
    result += [struct.pack('4s', b'GIF')]
    result += [b'\x01']  # Version
    result += [width.to_bytes(2, byteorder='little')]
    result += [height.to_bytes(2, byteorder='little')]
    result += [b'\x00']  # Flags (none set!)
    result += [(len(frames)).to_bytes(2, byteorder='little')]

    # Push each frame's content chunk and then its duration chunk before moving on
    for duration, data in frames:
        result += [data]
        result += [(duration).to_bytes(2, byteorder='little')]
    
    return b''.join(result)

