#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out`. The data is generated with the
# provided seed and length. To make this reproducible, pass in an integer as
# seed which will be used to initialize Python's PRNG for reproducibility. For
# instance, if you want to have reproducible output for all outputs of different
# seeds on system A and B, run: `# /path/to/generator.py <seed>` where `/path/`
# exists on both systems and points to the path of this file on system A. On
# system B, do not pass any argument (i.e., just execute `./generator.py`).
# This way, on system A, its stdout will lead to the same bytes being written to
# the input pipe passed to system B.
def generate_random_gif(
        seed: int, length: int, out: BinaryIO):

    # Initialize our RNG based on the requested seed; we use a 4-byte unsigned
    # big endian value here to ensure that the whole stream stays reproducible
    prng = struct.Struct('>L').pack(os.urandom(4)[0])
    prng += os.urandom((length - len(prng)) // 2)
    prng += bytes([1, 0]) * ((length - len(prng)) % 2)
    assert len(prng) == length

    header = b'GIF89a'
    datalen = length
    data = prng
    out.write(header + struct.pack('>H', datalen) + data)