#!/usr/bin/env python3

import sys
import os
import struct
from typing import BinaryIO

from PIL import Image

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )



# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Signature
    header = b'GIF89a'
    
    # Width and Height
    w = rng.read(2)   # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    header += w + h

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    img = [Image.frombytes('L', (h[0], w[0]), bytearray(data))]

    # Image Descriptor
    IDXTBL = {
        0x2c : True,
        0x21 : False
    }

    while IDXTBL[rng.read(1)[0]]:
        pass

    
    # Application Extension
    ext = []

    ext.append(b'\x21\xFF\x0BNETSCAPE2.0')
    ext.append(b'\x03\x01\x04')

    ext.append(struct.pack('<H', int((rng.read(1)[0] << 8)) | int(rng.read(1)[0])))

    ext.append(b'Random Gif Generator!')
    img.extend(ext)

    img[0].save(
        fp=out, 
        format='gif', 
        save_all=True, 
        append_images=img[1:],
        duration=int(rng.read(1)[0]),
        loop=0
    )
    
