#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jan 25 14:57:20 2022

@author: dhulls
"""
import os
from datetime import datetime, timedelta
from pytz import timezone
from .utils import load_json


def init_default(db):
    """Creates defaults for the db"""
    
    ############################
    # Create a default user
    ############################
    
    print("> Creating a default user")
    
    if 'users' not in db.collection_names():
        db.create_collection('users')
        
    user = {
            'name': 'admin',
            'password': '$pbkdf2-sha256$29000$EJMGvgXPQOjDnN8yTaCp8A$iRqxYKibbh0uLGRtPMUJZuQeEHcwWVmS1lLMdV0G2Iw=',
            'email': '',
           }
            
    db['users'].insert_one(user)
            
    
    ############################
    # Create a default host
    ############################
    
    print("> Creating a default host")
    
    if 'hosts' not in db.collection_names():
        db.create_collection('hosts')
        
    hosts = [{'host': 'localhost'}, {'host': '127.0.0.1'}]
    
    for h in hosts:
        h['created'] = datetime.now().isoformat()
        
        ip = h['host']
        try:
            r = os.system("ping -c 1 " + ip + " > /dev/null &")
        except KeyboardInterrupt:
            raise
        except BaseException as e:
            pass
        else:
            if r == 0:
                h['status'] = True
                
    db['hosts'].insert_many(hosts)
    db.command({'collMod': 'hosts', 'validator': {'host': 'yes'}})
    db.command({'collMod': 'hosts', 'validationLevel': 'strict'})
    
    
    ############################
    # Create default groups
    ############################
    
    print("> Creating default groups")
    
    from bson import ObjectId
    
    if 'groups' not in db.collection_names():
        db.create_collection('groups')
    
    group = {
            '_id': ObjectId(),
            'name': 'local',
            'description': 'The local group is used for users that are logged into this machine',
           }
    
    db['groups'].insert_one(group)
    
    if 'default_groups' not in db.collection_names():
        db.create_collection('default_groups')
        
    db['default_groups'].find_one_and_update(
        {},
        {'$push':