#!/usr/bin/env python3

import struct
from typing import BinaryIO

def read_varint(f:BinaryIO):
    out = 0
    for i in range(8):
        b = f.read(1)
        if not b:
            break # EOF
        b = int(b[0])
        out |= (b & 0x7f) << (i*7)
        if(not (b & 0x80)):
            return out

def write_varint(f:BinaryIO, val:int):
    if val < 0: raise ValueError('cannot write negative varints')
            
    while True:
        cval = val % 0x80
        val //= 0x80
        
        if val == 0:
            # last byte
            cval |= 0x80

        f.write(bytes([cval]))

        if val == 0:
            return

def read_varstring(f:BinaryIO):
    size = read_varint(f)
    
    data = f.read(size)
    if len(data) != size:
        raise Exception("unexpected EOF reading varstr")
    
    return bytes(data)#.decode()

def write_varstring(f:BinaryIO, string:bytes):
    write_varint(f, len(string))
    f.write(string)

def read_hash256(f:BinaryIO):
    rv = []
    for _ in range(32):
        b = f.read(1)
        if not b or len(b) != 1:
            raise Exception('missing hash256')
        rv.append(struct.unpack('B', b)[0