#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given `palette`.
def gen_gif(out: BinaryIO, palette: bytes) -> None:
    # NETSCAPE2.0 app extension (for animated image)
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00')

    # GIF header
    out.write(b'GIF89a')

    # Globals via Logical Screen Descriptor
    out.write(struct.pack('<HBB', 400, 400, 0x80 | len(palette)))
    out.write(palette)

    # Local Image Descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # LZW Minimum Code Size
    out.write(b'\x08')

    # EOF
    out.write(b'\x3b')

