#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    for _ in range(257):
        # The label says 10 but there are actually two more
        # bytes before the next descriptor
        # Write the descriptor
        print('.', end='')
        out.write(data[10:(10+12)])

        # Skip the frame data (it's not needed)
        data = data[(10+12):]

    print(' Done!', flush=True)
