#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Go through the remaining bytes one two-byte chunk at a time
    # and write them back out if they are not equal to the bytes
    # we just wrote out
    last_byte = 10
    while last_byte < len(data):
        print("Processing bytes", last_byte, '-', min(len(data), last_byte+2), '...', end='', flush=True)
        # Read two bytes (or less if there aren't enough)
        current_frame = data[last_byte:(last_byte+2)]
        # Compare them with what we just wrote
        if current_frame != data[(last_byte-10):last_byte]:
            # If they differ write them back
            out.write(current_frame)
        # And then append whatever was left from the beginning
        print("Appending", len(data[last_byte:]), 'bytes...', end='', flush=True)
        out.write(data[last_byte:])
        print('done.', flush=True)

    # Finally add the trailer to finish off the output.
    # Normally this would be part of the application logic
    # when writing a gif but since this is intended only
    # for demonstration purposes we do it here directly.
    # See https://github.com/decompme/decompilation-guide/issues/46
    print('Adding the trailer and EOF', end='', flush=True)
    # 2 bytes for signature
    # 1 byte for version
    # 1 byte for flags
    # 2 bytes for background color index
    # 2 bytes for pixel aspect ratio
    out.write(b'\x3B\x00\x00\x00')
    print(' done', flush=True)

