#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate(out:BinaryIO, rng):
    out.write(b'GIF89a') # .gif magic header

    width = rng.randint(1, 400)
    height = rng.randint(1, 400)
    times = rng.randint(1, 50) * 20
    zeros = [0] * 16
    
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))

    out.write(struct.pack('B', 0xF7)) # Global Color Table Flag
    out.write(struct.pack('B', 0))   # Color Resolution
    out.write(struct.pack('B', 7))   # Sort Method
    out.write(struct.pack('B', 0))  # Size of Global Color Table

    for i in range(16):
        out.write(b'\xFF\xDD\xD1\xEA') # RGB color
        
    out.write(struct.pack('B', 0x21)) # Image Descriptor Extension Block
    out.write(struct.pack('B', 0xF9)) # Graphic Control Label
    out.write(struct.pack('B', 0x04)) # Extension block size
    out.write(struct.pack('B', 0x00)) # Transparent Index
    out.write(struct.pack('<H', 0xFFFF)) # Delay Time
    out.write(struct.pack('B', 0x00)) # Transparency Flag
    out.write(b'\x00')                # Block Terminator

    out.write(struct.pack('B', 0x2C)) # Image Descriptor ID
    out.write(struct.pack('B', 0))   # x-position
    out.write(struct.pack('B', 0))   # y-position
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, '