#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out:BinaryIO, rng):
    out.write(b'GIF89a') # File signature
    width = 2 ** struct.unpack('<H', rng.read(2))[0]
    height = 2 ** struct.unpack('<H', rng.read(2))[0]
    print("Width:", width)
    print("Height:", height)
    # Global color table
    out.write(b'\xff\xff\xff\x00') # Background color (white)
    out.write(b'\xf1\xe5\x4e\x00') # Pixel 1 (aquamarine)
    out.write(b'\xd7\x6f\x3c\x00') # Pixel 2 (orange red)
    out.write(b'\x40\xac\xc4\x00') # Pixel 3 (turquoise)
    out.write(b'\x40\xa6\xbe\x00') # Pixel 4 (light slate gray)
    out.write(b'\x00\x00\x00\x00') # Pixel 5 (black)
    out.write(b'\xff\xff\xff\x00') # Pixel 6 (white)
    # Logical Screen Descriptor
    out.write(struct.pack("<HHB", width, height, 0x0f | 0x80)) # Width, Height and global color palette info
    out.write(b'\x00') # Background color index
    out.write(b'\x00') # Aspect ratio
    # Image descriptor
    out.write(struct.pack('<HHBB', 0, 0, width, height)) # Left/Top/Width/Height
    out.write(b'\x87') # Local color table flag / interlaced (yes)
    out.write(b'\x00') # Sort flag / local color table size
    for y in range(height):
        pixels = []
        for x in range(width):
            pixels += [rng.read(1), b'\x00']
        out.write(bytes(pixels)) # Actual pixel data
        # Line terminator
        if y != height - 1:
            out.write(b';')
