#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Read the rest of the bytes
    while True:
        if len(data) < 2 or data[0] != 0x2c:
            print('Error - missing Image Descriptor!?', flush=True)
            exit(-1)

        print('.', end='', flush=True)
        out.write(b'\x2c\xff' + data[1:3])
        data = data[3:]
        
        restype = data[0]
        if restype == 0x2c:
            continue
            
        elif restype == 0x00:
            break
            
        else:
            print('Error - Unknown record type %02X!'%restype)
            print('%r'%data)
            exit(-1)

    print('done.\n', flush=True)
