#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        out: BinaryIO,
        rng: BinaryIO,
        width: int = 100,
        height: int = 100):

    # header
    
    header = b'\x47\x49\x46\x38\x39\x61'

    # read width and height

    if height > 0:
        datalen = width * height
    else:
        datalen = struct.unpack('>H', rng.read(2))[
            0]     # Write pre-frame bytes  

    out.write(header + (datalen).to_bytes(
            3, 'big')) 

    # write frame data

    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
