#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    for _ in range(len(header)):
        out.write(rng.read(1))
    print('.', end='', flush=True)
    for _ in range(126):
        out.write(rng.read(1))
    print('. Done!')

    # Start writing descriptors with random fields
    while True:
        desc = rng.read(1)
        if len(desc) == 0:
            break
        
        out.write(desc)

        if ord(desc[0]) & 0b10000000 == 0b10000000:
            # Image descriptor; read width, height and the rest of the fields
            imgwidth = int.from_bytes(
                    rng.read(2), byteorder='little')
            imgheight = int.from_bytes(
                    rng.read(2), byteorder='little')
            
            # Read all the fields between the width/height pair
            # and the local colour table length, which are always 4 bytes long
            for _ in range(5):
                out.write(rng.read(1))

            out.write(rng.read(imgwidth * imgheight))
            skipcount = int.from_bytes(
                    rng.read(2), byteorder='little')
            out.write((skipcount-1).to_bytes(
                    2, byteorder='little'))

### End of gif generation function ###

# Tests the bug against a set of inputs
def test_bug():
    import sys
    from Crypto.Random import new as randnew
    import io

    try:
        fname = sys.argv[1]
    except IndexError:
        exit()

    # Open the input file
    with open(fname, 'rb') as f:
        img = f.read()
        # Patch the attack vector so we can easily trigger it
        buf = bytearray(img[:72]+b'\xff\xff'+img[74:])

    # Create a new instance of the vulnerable class. It takes a 256-byte buffer
    # to initialize itself with, but the actual size doesn’t matter because we
    # won’t ever read or write anything to it. We just use this object as a seed
    # for our PRNG.
    v1 = VulnClass(b'\x00'*256)
    prng = randnew().read
    # The first thing we want to do before generating the random GIF file
    # is to feed our PRNG with some random values. This ensures the generated
    # GIF will contain random bytes (and not only zeros)
    prng.feed(v1.to_str())

    # Now we can generate a random GIF file using the same PRNG we used above
    generate_random_gif(prng, io.BytesIO())
