#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    for _ in range(257):
        # The label says 10 but there are actually two more
        # bytes after the label indicating the length of
        # the block. We know it's always >10 so just set it to 0x10
        # (https://www.w3.org/Graphics/GIF/spec-gif89a.txt)

        # TODO: Should experiment with variations on these values too
        #       see https://github.com/decompme/decompilation-guide/issues/47
        out.write(b'\x2C\x00\x00\x00\x00\x00\x00\xFB\x00\x00\x00')


    last_byte = out.tell() - 1

    paddinglen = bytes([last_byte % 0x100])
    if len(data) >= 11:
        out.write(data[10:-12])
        numloops = ord(data[-12])
    else:
        paddinglen += bytes(10 + last_byte // 0x100 - len(data))
        numloops = 0
    # Write the leftover bytes before the last descriptor
    print('Writing remaining bytes', end='', flush=True)
    out.seek(-1, 2)
    out.write(paddinglen)
    out.write(struct.pack('<B', numloops))
    out.flush()
