#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = 600
    height = 400
    background = tuple(struct.unpack('BBB', rng.read(3)))
    # MUST be '!' (or else it won't be interpreted as valid GIF files :/)
    # TODO: Check this is not a problem in practice
    color_table = tuple(tuple((i,i,i)) for i in range(256)) + ((0xFF,)*3,)
    colors = len(color_table)
    # Have some flashing colors
    color_indexes = []
    while len(color_indexes) < 100:
        if rng.read(1)[0] == 0x00:
            continue
        color_indexes.append(int(rng.read(1)[0]) % colors)
    delay = 0
    transparent = None

    def to_bytes(g: int):
        return bytes([(g >> (8 * i)) & 0xff for i in reversed(range(8))])
    rng.seek(0, 0)
    data = bytearray()
    while True:
        g = int.from_bytes(rng.read(8), "little", signed=True)
        if g >= 0:
            data += to_bytes((colors - 1) | 1 << 7)
