#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    for _ in range(257):
        # The label says 10 but there are actually two more
        # bytes after the label indicating the length of the
        # Graphics Control Extension block
        assert len(data) >= 14

        # Read the Graphic Control Extension
        gce_block_size = int.from_bytes(
                data[0:1], byteorder='little')
        gce_flags = struct.unpack('<B', data[1:2])[0]
        gce_delay = int.from_bytes(
                data[2:4], byteorder='little')
        gce_transparent_index = int.from_bytes(
                data[4:5], byteorder='little')
        gce_terminator = int.from_bytes(
                data[5:6], byteorder='little')

        # The next two fields are image descriptor size and
        # image descriptor. We only care about the latter, so
        # we skip the former
        id_sizelen = int.from_bytes(
                data[6:8], byteorder='little')
        id_id = int.from_bytes(
                data[8:12], byteorder='little')

        # Write the graphic control extension and image descriptor
        print('Writing the graphic control extension and image descriptor', end='', flush=True)
        out.write(
            struct.pack('<BBBHB',
                        0b10000000 | (gce_block_size & 0xFF),
                        gce_flags,
                        gce_delay,
                        gce_transparent_index,
                        gce_terminator))
        out.write(struct.pack('<HH', id_sizelen, id_id))
        print(' done.', flush=True)

        data = data[12:]

    print('Done writing.',