#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    for _ in range(257):
        # The label says 10 but there are actually two more
        # bytes after the label indicating the length of
        # the block. We know it's always >10 so just set it to 0x10
        # (https://www.w3.org/Graphics/GIF/spec-gif89a.txt)
        out.write(struct.pack('<H', 0x10))

        # Write an image descriptor
        out.write(b'\xFF\x00\x00\x00\x00,\x00\x00\x00,\x00')

        # Write the bytes of the next descriptor
        if len(data) >= 10:
            out.write(data[:10])
            data = data[10:]
            continue
        else:
            break
    
    # Finally, write the rest of the bytes until we reach EOF or 255
    while True:
        chunk = rng.read(255 - 6)
