#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header (6 bytes)
    out.write('GIF89a'.encode())                   # Identification code
    width = int.from_bytes(rng.read(2), 'little')  # Image dimensions
    height = int.from_bytes(rng.read(2), 'little')
    screen_size = ((width + 15) & ~15, (height + 15) & ~15)
    out.write(struct.pack("!H", screen_size[0] * screen_size[1]))  # Screen size in pixels
    background_color = rng.read(1)[0]             # Background color index
    assert 0 <= background_color < 2**8
    out.write(b'\x00')                            # Pixel aspect ratio (always default)

    # Logical screen descriptor (7 bytes)
    out.write(struct.pack("!BBB", background_color, 0, 0))  # Global color table flag (no) and table size (zero)
    out.write(b'\x2C' if background_color else b'\x21')     # Image separator or extension block
    out.write(struct.pack("!HHBB", 0, 0, width, height))    # Origin and image dimensions (top-left corner is at coordinate (0,0))
    packed = rng.read(1)[0]                                # Packed field for local color table (not sorted, uses global colors)
    assert packed == 0b10000000                            # Control byte checking
    out.write(b'\x08\xFF\xF0\x00\x00\x00\x21\xF9\x04\x01\x00\x00\x00\x00\x00\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;') # Graphic control extension (transparent)
    out.write(b'\x21\xFE\