#!/usr/bin/env python3
# © AaronSadler
import struct
from PIL import Image

def google_converted_tiff_to_jpg(infile,outfile,show=False):
    # Initially written by "Senya" from Stack Overflow
    # http://bit.ly/google-maps-image
  
    with open(infile,'rb') as f:
      image = convert_google_imagetype_to_jpeg(f)
      with open(outfile,"wb") as f2:
        f2.write(image)
      
      if show==True:
        im = Image.open(outfile)
        im.show()


def convert_google_imagetype_to_jpeg(infile):
  """Convert Google's proprietary imagetype to JPEG"""

  width, height, data = map(lambda x: int(x), infile.readline().split())
  
  jpegdata = b"\xff\xd8" + infile.read()
  jpegdata += b"\xff\xd9"
  
  rgbdata = []
  pixptr = 0
  rowstride = (width * 3 + 3) // 4 * 4
  for row in range(height):
    rowstart = pixptr
    for col in range(width):
      rgbdata.append(int(jpegdata[rowstart:rowstart+rowstride][col*3:col*3+3], 16))
      pixptr += 4
  
  rgbstring = ''.join(map(chr, rgbdata))
  return b'\xff\xd8' + Image.frombytes("RGB", (width,height), rgbstring).tobytes()
  
  
def convert_netscape_imagetype_to_jpeg(infile):
  """Convert Netscape's proprietary imagetype to JPEG"""

  # http://www.astro-wise.com/cgi-bin/css/finder?t=131129&id=1373
 
  def read_string(rng):
    """Read a string until a null byte"""
    s = ""
    b = rng.read(1)
    while len(b) and b != '\0':
      s += b
      b = rng.read(1)
    return s
  
  # Open the file handle as a binary stream
  rng = None
  with open(infile, 'rb') as f:
    rng = f
    
    # Check that it's a valid NIF file
    magic = rng.read(4)
    assert(magic == b'NIF\0');
    version = rng.read(2)
    assert(version == b' \n');
    
    # Read info chunk
    chunktype = rng.read(4)
    chunklen = struct.unpack('=L', rng.read(4))[0]
    assert(chunktype == b'INFO')

    infostr = ''
    for i in range(chunklen//4):
      infostr += struct.unpack('=4s', rng.read(4))
    infodict = dict([kv.split('=') for kv in infostr.split('\0')[:-1]])
    
    assert(infodict['Format']=='JFIF');
    assert(infodict['MimeType']=='image/jpeg');
    
    # Start reading frames
    framecount = 0
    framelen = 0
    while True:
      framenum = framecount + 1
      print('Processing frame %d...' % framenum)
      framecount += 1
  
      # Header
      headerstr = rng.read(4)
      headerlen = struct.unpack('=L', rng.read(4))[0]
      assert(headerstr == b'HEAD')
  
      # Frame Chunk
      framenum = struct.unpack('=L', rng.read(4))[0]
      framelen = struct.unpack('=L', rng.read(4))[0]
      assert(framenum == framecount);
      assert(framelen == rng.tell());
      
      # Parse keyframes
      chunksize = struct.unpack('=L', rng.read(4))[0]
      assert(chunksize == 0)
      assert(rng.read(4) == b'KEYF');
      keyframecount = struct.unpack('=L', rng.read(4))[0]
      
      for kf in range(keyframecount):
        pass;
  
      # Parse animation chunk
      animtag = rng.read(32)
      animlen = struct.unpack('=L', rng.read(4))[0];
      assert(animtag == b'AnimationChunk\x00\x00\x00\x1a')
      
      aclen = struct.unpack('=H', rng.read(2))[0]
      ac = rng.read(aclen);
  
      # Find first FRAME
      tag = rng.read(4)
      while tag != b'FRAM':
        taglen = struct.unpack('=L', rng.read(4))[0]
        rng.seek(taglen, 1)
        tag = rng.read(4)
  
      # Get BitsPerSample and ColorMapLength
      bitsperpixel = ord(rng.read(1));
      bitplanes = ord(rng.read(1));
      colormaplength = ord(rng.read(1));
      compression = ord(rng.read(1));
      hres = ord(rng.read(1)) + ord(rng.read(1))*256;
      vres = ord(rng.read(1)) + ord(rng.read(1))*256;
      colorsused = ord(rng.read(1)) + ord(rng.read(1))*256;
      colorsimportant = ord(rng.read(1)) + ord(rng.read(1))*256;
      
      assert(bitsperpixel == 8)
      assert(bitplanes == 1)
      assert(hres==640 or hres==720)
      assert(vres==480)
      assert(colorsused==0 or colorsimportant==256)
      assert(compression==0)
      
      # Read palette
      palentry = 0
      pal = [];
      while palentry<256:
        red = ord(rng.read(1))
        green = ord(rng.read(1))
        blue = ord(rng.read(1))
        flag = ord(rng.read(1))
          
        if (flag == 0):
            break
          
        pal.extend([red,green,blue]);
            
        palentry += 1

      # Read Bitmap Data 
      assert(ord(rng.read(1))==0);
      assert(ord(rng.read(1))==0);
      width = ord(rng.read(1));
      height = ord(rng.read(1))*256 + width;
      width += 1;
      print ('Frame Size: ', width, 'x', height);
      scanlinesize = (((width + 3) / 4) * 4) * 3;
      frame = b''
      for scanline in range(height):
          frame += rng.read(scanlinesize)
  
  return frame
  
  
def convert_googleslide_imagetype_to_jpeg(infile):
  """Convert Google Slide's proprietary imagetype to JPEG"""

  # https://github.com/matschaffer/googleslide
  
  def read_string(rng):
    """Read a string until a null byte"""
    s = ""
    b = rng.read(1)
    while len(b) and b != '\0':
      s += b
      b = rng.read(1)
    return s
  
  # Open the file handle as a binary stream
  rng = None
  with open(infile, 'rb') as f:
    rng = f
    
    # check for magic number
    m = rng    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
          return
    
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:
          lct = rng.read(numcolors*3)
          out.write(lct)

        imgdata = bytes([ord(rng.read(1)) for _ in range(size)])

        out.write(imgdata)