#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the length information in `len`.
def generate_gif(out: BinaryIO, len):
    img = 0xFF * (1 << (8 * len)) + 0xFFFF
    
    # PNG header
    out.write(b'\x89PNG\r\n')
    out.write(struct.pack('>I', 13))                   # IHDR chunk length
    out.write(struct.pack('4s', b'IHDR'))              # IHDR chunk identifier
    out.write(struct.pack('>IIBB', 2 ** len, 2 ** len, 8, 6))  # IHDR data
    out.write(b'\x00\x00\x00')                         # CRC for IHDR
    
    # PLTE chunk
    out.write(struct.pack('>I', 5))                    # PLTE chunk length
    out.write(struct.pack('4s', b'PLTE'))              # PLTE chunk identifier
    out.write(struct.pack('>BBB', 0, 0, 0))            # PLTE data
    out.write(b'\x00\x00\x00')                         # CRC for PLTE
    
    # IDAT chunk
    out.write(struct.pack('>I', 2 ** len + 1))         # IDAT chunk length
    out.write(struct.pack('4s', b'IDAT'))              # IDAT chunk identifier
    out.write(struct.pack('B', 0))                     # compression method
    out.write(struct.pack('<I', img & 0xffffffff))
    if len > 2:  # write 4 bytes of image data at a time
        out.write(struct.pack('<I', (img >> 32) & 0xffffffff))
    if len > 1:  # write 2 bytes of image data at a time
        out.write(struct.pack('<H', img & 0xffff))
    if len > 0:  # write 1 byte of image data at a time
        out.write(struct.pack('B', img & 0xff))
    crc = 0x7a7d8