#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), 'big') + 4
    data = bytearray(datalen)
    for i in range(datalen):
        data[i] = rng.read(1)[0] & 0xff
    appext_len = 2 * (rng.read(1)[0] % 64)
    animext = bytearray([0b10000000])
    if appext_len > 0:
        animext.append(appext_len - 1)
        appext = bytearray()
        for _ in range(appext_len // 2):
            b = rng.read(1)
            appext.extend([b[0], ~b[0]])
        animext.extend(appext)
    imgdesc = bytearray()
    subblock_id = 0
    while True:
        subblock_size = rng.read(1)[0] & 0xff
        if subblock_size < 1 or subblock_size > 255:
            break
        imgdesc.append(subblock_size)
        imgdesc.extend(
                rng.read(subblock_size))
        subblock_id += 1
    eimgdesc = bytearray()
    if rng.read(1)[0] & 0xff == 0:
        eimgdesc.append(0b1)
    else:
        eimgdesc.append(0b0)
    color_table = bytearray()
    for _ in range((data[3] >> 4 | data[3] << 4 & 0xf0)):
        color_table.extend(
                list(struct.unpack('BBB', rng.read(3))))
    out.write(header)
    out.write(struct.pack(
