#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Read global color table if present
    if gctlen != 0:
        globalcolor_table = rng.read(gctlen)
        out.write(globalcolor_table)

    while True:
        # Read a graphic control extension (GCE) packet
        # First byte is fixed value of 0x21, next byte is
        # fixed 0xf9 for compatibility with existing parsers
        
        gce = rng.read(6)
        if len(gce) < 6:
            raise Exception('unexpected eof')
            
        # Add GCE to output
        out.write(gce)

        # Read image descriptor (ID) packet
        idp = rng.read(9)
        out.write(idp)

        # Read local color table length from ID
        lctlen = 2**(int((idp[8] >> 5))) * 3

        # Read local color table if present
        if lctlen != 0:
            local_color_table = rng.read(lctlen