#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Magic and version (GIF89a)
    out.write(b"GIF89a")

    # Background color
    out.write(struct.pack("B", 0))

    # Pixel aspect ratio (unspecified)
    out.write(struct.pack("B", 0))

    # Global color table with one color in RGB quadruple format
    out.write(struct.pack(
        "B",
        4 * 1 + 7   # length of global color table, bits-per-pixel
    ))

    # Color data for each pixel (see above)
    out.write(b"\xff\x00\x00")

    # Graphic control extension
    out.write(struct.pack(
        "BBHBB",
        0x21,                            # GCE
        0xf9,                            # GCE tag
        4,                               # byte count
        1 << 1,                          # transparency flag
        0,                               # delay time
        0,                               # transparent color index
        ((next(rng), next(rng), next(rng)) if next(rng) == 0 else (0, 0, 0))[None],  # background color
    ))

    # Image descriptor
    out.write(struct.pack(
        "BBHHBB",
        0x2c,                            # image separtor
        0, 0,                            # x,y position of image
        64, 64,                          # width, height of image
        0,                               # packed fields (really?)
    ))

    # Local color table (same as above)
    out.write(struct.pack(
        "B",
        4 * 1 + 7,
    ))

    # Actual image data
    for y in range(64):
        for x in range(64):
            out.write(struct.pack