#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        out: BinaryIO, rng: 'Random', size: int = 500):
    # Write header
    out.write(b'GIF89a')
    out.write(rng.bytes(6))
    out.write((size & 0xff).to_bytes(2, byteorder='little'))
    out.write((size >> 8).to_bytes(2, byteorder='little'))
    colorcount = rng.randint(2, 257)
    colorbytes = rng.bytes(colorcount * 3)
    if len(colorbytes) < 768:
        paddinglen = (768 - len(colorbytes)).to_bytes(1, byteorder='little')
    else:
        paddinglen = b'\x00'
    out.write(b'\x00\x00\xff\x00\x00,\x00\x00\x00')
    out.write(paddinglen + colorbytes[:(768-len(paddinglen))])
    for _ in range(rng.randrange(1)):
        out.write(struct.pack('<H', rng.randrange(4096)))
        out.write(struct.pack('<H', rng.randrange(4096)))
        out.write(struct.pack('<H', rng.randrange(4096)))
        out.write(b'\x00\x00')
    out.write(struct.pack('<B', 0))
    frameheight = rng.randrange(1, 32768)
    framewidth = rng.randrange(1, 32768)
    out.write(struct.pack('<B', 0x2c))
    out.write(struct.pack('<HHBB', 0, 0, framewidth & 0xff, framewidth >> 8))
    out.write(struct.pack('<HH', frameheight & 0xff, frameheight >> 8))
    out.write(struct.pack('<B', 0xf0 | rng.randint(0, 0xe << 1)))
    out.write(struct.pack('<HHHBBBB', 0, 0, 0, 0, 0, 0, 0))
    out.write(b'\x00')
    # Body
    while True:
        blocktype = rng.randint(0, 0xc3+1)
        if blocktype == 0x3b:
            break
        elif blocktype == 0x21:
            extlength = rng.randint(1, 257)
            if extlength > 0:
                out.write(struct.pack('<BB', 0x21, extlength))
                out.write(rng.bytes(extlength))
        elif blocktype == 0x2c:
            framedatalen = rng.randint(1, 2**24)
            out.write(struct.pack('<BBB', 0x2c, framedatalen & 0xff, framedatalen >> 8))
            out.write(struct.pack('<HHBB', 0, 0, framewidth & 0xff, framewidth >> 8))
            out.write(struct.pack('<HH', frameheight & 0xff, frameheight >> 8))
            out.write(struct.pack('<B', 0xf0 | rng.randint(0, 0xe << 1)))
            out.write(struct.pack('<HHHBBBB', 0, 0, 0, 0, 0, 0, 0))
            out.write(b'\x00')
            databytes = rng.bytes(framedatalen)
            if databytes[-1] != 0xff:
                databytes += bytes([0xff])
            out.write(databytes)
        else:
            out.write(struct.pack('<BB', blocktype, rng.randint(1,257)))
            out.write(rng.bytes(rng.randint(1, 2**24)))
    numloops = rng.randint(1, 2**16)
    out.seek(-1, 2)
    out.write(paddinglen)
    out.write(struct.pack('<B', numloops))
    out.flush()
