#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gct_len = 2 ** ((lsd[4] & 0b00000111) + 1)

    if gct_len > 0:
        # Get the global color table (GCT)
        gct = [struct.unpack('BBB', rng.read(3)) for _ in range(gct_len)]

        # Add GCT to output
        out.write(bytes([int(x) for sublist in gct for x in sublist]))

    for n in range(256):
        # Get LZW fields
        lzw = rng.read(10)

        # Skip image size and additional data
        pos = 4

        # Get local color table length
        lct_len = 2 ** (((lzw[pos] & 0b00000001) << 3) | (lzw[pos+1] >> 5))
        pos += 2

        if lct_len > 0:
            # Get LCT
            lct = [struct.unpack('BBB', rng.read(3)) for _ in range(lct_len)]
            out.write(lct)

        imgdata = bytes([ord(rng.read(1)) for _ in range(size)])

        out.write(imgdata)