#!/usr/bin/env python3

import sys

if __name__ == "__main__":
    with open(sys.argv[2], 'wb+') as outfile:
        process_gif(open(sys.argv[1], 'rb'), outfile)
\end{code}

The concept is to read the input file in one go and write a new gif whose data section consists of the first part of the old gif plus random data plus the last part of the old gif. The last bit is just to append a `;` if the original gif did not have one (the second `[...]`) - that can be done by reading an extra byte from the input file (which will always be either `0` or `1`, but one of them indicates a `;`.

This produces a gif that looks like this:
<IMAGE>

However, this is still not what you are going for because you really want a totally random image.

The best way to get that would be to generate your own palette (using other random methods) and then generate the image data using those colors. This is more complex, since it depends on how you get your images into the first place. If you know how they look, it should be easier to figure out. But I strongly suspect that this is the complexity that you are trying to avoid.

Answer: \begin{blockquote}
How do I put some random bytes at the beginning of GIF? ... The only solution I come up with is to read all bytes into memory, put some random stuff there and save a new file.
\end{blockquote}

You can use [Random Access Files](https://docs.python.org/3/tutorial/inputoutput.html#random-access-files) (also known as "streams") to skip over the existing GIF header and seek back to the end to insert your random chunk, like so:

\begin{code}
def process_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)
    footer = [b'\x00', b'\x3B'][
            0 <= int.from_bytes(
                rng.read(1), 'big') < 6]
    out.write(header + len(data).to_bytes(2, 'big') + \
            data + footer)
