#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    header = b'GIF89a'

    # Read file size
    filesize = int.from_bytes(rng.read(4), 'little')

    for i in range(0, filesize - 16):
        # Block size
        blocksize = int.from_bytes(rng.read(1), 'big')
        
        if blocksize == 0x2c:
            # Extension
            
            data = rng.read(blocksize-1)
            
            # Comment
            out.write(header + len(data).to_bytes(2, 'big') + \
                      data + b'\x00\x3b')
        elif blocksize == 0x21:
            # Extension
            
            extcode = int.from_bytes(rng.read(1), 'big')
            
            if extcode == 0xf9 or extcode == 0xfe:
                # App extenstion
                length = int.from_bytes(rng.read(1), 'big')
                data = rng.read(length)
            
                footer = b'\x00\x3b'
                if extcode == 0xfe and length > 0:
                    footer = b''
            
                # Write to output
                out.write(header + len(data).to_bytes(2, 'big') + \
            data + footer)
