#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) % 0xFF + 2

    # Write header and global palette
    out.write(header)
    for i in range(766):
        value = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            ) % 0x1F * 0x11
        out.write(struct.pack('B', value))

    # Write frames
    while datalen > 0:

        # Get frame descriptor bytes
        descriptor = [int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            ) % 0xF3]

        # Add either block size or terminator to descriptor
        if datalen <= 0xFF:
            descriptor += [(datalen & 0xFF)]
        else:
            descriptor[0] |= 0x04
            descriptor += [(datalen >> 8) & 0xFF, (datalen & 0xFF)]
            print("Warning: Max data len exceeded")

        # Write frame descriptor
        out.write(bytes(descriptor))

        # Decrement data left
        datalen -= len(descriptor)

        # Write data until zero length reached
        while datalen > 0:

            datalen_byte = min([
                    (datalen // 255) + 1,
                    255,
                    datalen
                ])
            datalen -= datalen_byte

            data = []
            for j in range(datalen_byte):
                data += [
                        rng.read(1)[0],
                        rng.read(1)[0] ^ 0xFF
                    ]
            
            out.write(b''.join(data))


if