#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Write frame descriptor
    globalframecontrol = 0xFC | (rng.read(1)[0] & ~0x40) # Set no local color table flag
    frameposition = out.tell()
    framenumber = 1
    print(f"Writing frame {framenumber}", end='', flush=True)
    out.write(struct.pack(">HHB",
                768,
                len(data[10:]),
                globalframecontrol))
    print(" done.", flush=True)

    # Write everything after the frame descriptor as-is
    print("Writing remaining data...", end='', flush=True)
    out.write(data[10:])
    print(" done.")

