#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # The size of this block is 7 bytes
    logicalScreenDescriptor = \
            bytearray([
                0x40, 0x00, 0x0D, 0x00,
                ord('H'), ord('E'), ord('L'), 
                ord('T'),  0x01, 
            ])

    # These fields are all set to zero for our purposes
    globalColorTable = 256 * [b'\x00']
    backgroundIndex = 0
    pixelAspectRatio = 0

    # All the other fields need to equal their 
    # corresponding values in the above example
    loopControlFlag = 0
    colorResolution = 0
    sortFlag = 0
    globalColorTableFlag = 0

    # These are set to zero because they're unused
    truncatedImage = 0
    numberOfLines = 0
    
    # For debugging purposes, we print the contents
    # of these fields just so you know what's going on.
    print("logical screen descriptor:")
    for i in range(len(logicalScreenDescriptor)):
        if (i % 16 == 0):
            print()

        print("%x " % logicalScreenDescriptor[i], end='')
    
    print("\n")

    print("global color table:", "".join(["%x" % x for x in globalColorTable]))
    print("background index: ", "%d" % backgroundIndex)
    print("pixel aspect ratio:", "%d" % pixelAspectRatio)
    print("loop control flag: ", "%d" % loopControlFlag)
    print("color resolution: ", "%d" % colorResolution)
    print("sort flag: ", "%d" % sortFlag)
    print("global color table flag: ", "%d" % globalColorTableFlag)
    print("truncated image: ", "%s" % str(bool(truncatedImage)))
    print("number of lines: ", "%d" % numberOfLines)

    out.write(header + bytes(logicalScreenDescriptor) + 
        bytes(globalColorTable))

    numFrames = struct.unpack("<H", rng.read(2))[0] % 0xFFFF
    print("num frames: ", numFrames)

    for _i in range(numFrames):
        print("new frame!")
        
        length = struct.unpack("<H", rng.read(2))[0] % 0xFFFF

        dataStart = out.tell()
        print("framedata length: ", length)

        out.write(struct.pack("<BHHB",
            0x2C,      # Image Seperator
            0,         # Left Position
            0,         # Top Position
            0,         # Width
            0          # Height
        ))

        print("framedata start at: ", hex(dataStart), "length ", length)

        for j in range(length):
            # Each field has its own implicit preamble
            byte = struct.unpack("<B", rng.read(1))[0]

            # We reset the LSB instead of writing it
            # normally to ensure there aren't any artifacts.
            out.seek(-1, 1)
            
            out.write(bytes([byte & ~1]))