#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes   (13 + color table)
    while len(header+b'\0'+struct.pack('<H',datalen)) > 0x4000000:
        out.write(header+b'\0'+struct.pack('<H',datalen))

        header += rng.read(1)
        datalen -= 256

    out.write(b'\0\0\0\x0cNETSCAPE2.0')
    out.write(b'\x0b\x00\x00\x00\x00')
    out.write(b'\x00\x01\x00\x00\xff\xff\x00\x00\xff\xff\x00\x00\xff\xff\x00\x00')


    out.write((b'\0'*2)+struct.pack('<H',datalen))
    data = rng.read(datalen)

    for i in range(datalen):
        if data[i] == ord('\x00'):
            data[i] = 0x0e

    out.write(data)

