#!/usr/bin/env python3

import struct
from typing import BinaryIO

from.types import *


def write_c2info(file: BinaryIO, bkp_header: BKPHeader,
                 header_len: int = 0x68):
    # TODO: add a check for sane values (e.g. checksums)

    if header_len < len(bkp_header):
        raise ValueError("header length is less than the size of "
                         f"BKPHeader ({hex(header_len)} vs {hex(len(header_len))})")

    original_pos = file.tell()
    file.seek(0x14)
    magic = file.read(4).decode('ascii')
    if magic!= 'BKR ':
        raise ValueError(f"Invalid magic string in c2info header at position 0x14: '{magic}'")

    file.seek(original_pos)

    # wrap with an empty BKPHeader
    data = bytearray(header_len - len(bkp_header))
    data += bytes(bkp_header)

    # dump everything from here onwards into the new file
    footer = struct.pack('<II', *reversed((HEADER_MAGIC << 16 | HEADER_VERSION,
                                          len(data))))
    file.write(bytes(BARE_C2INFO_HEADER) +
              data + footer)
