#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. 5 bytes of logical screen descriptor with width and height set to 0.
#       - As this isn't actually rendered, we can use any value as long as it's non-zero.
#   3. 3 bytes which represent an empty color table entry (RGB all zeros).
#   4. 1 byte representing an image separator character for frame 1.
#       - The order of frames is determined by the order they're added later on.
#   5. Loop count set to infinite (`\x00`).
#   6. Image data block made up of pixels repeated over each axis (i.e. four times x*y in size)
#      where each pixel is one byte representing the index within the color table above used.
def generate_random_gif(out: BinaryIO):
    # Header & Logical Screen Descriptor
    out.write(b"GIF89a")
    out.write(struct.pack("<HHBBB", 0, 0, 0, 7, 0))

    # Empty Color Table Entry
    out.write(b"\x00\x00\x00")

    # Frame Separator
    out.write(b"\x21\xF9\x04\x00")

    # Loop Count
    out.write(b"\x00")

    # Body
    for y in range(0, 50 * 100):
        for _ in range(0, 100):
            if y % 2 == 0:
                out.write(b"\xFF")
            else:
                out.write(b"\x00")

        if y % 100 == 99:
            break

