#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')

    # Set up color table to be 256 colors long
    ctsize = b'\x01\x00'
    sctsize = int.from_bytes(ctsize, 'big') << 4
    colortable = bytes([ sctsize ]) + \
                rng.read((sctsize >> 4) * (7+3))

    gctflag = b'\x80'
    application_ext = b'\x21\xff\x0bNETSCAPE2.0'+bytes([gctflag])+\
                        b'\x03\x01'+b'\x00'*int.from_bytes(\
                            ctsize, 'big')
        
    extslen = len(application_ext).to_bytes(2,'big')
    extblock = b'\x21\xf9' + extslen + application_ext

    blockdata = header + datalen.to_bytes(2, 'big') + \
                    colortable + extblock

    out.write(blockdata)

