#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + bytes([0x21]) # Version, size, no color table

    out.write(header)

    width = 64
    height = 64

    flags = bytearray()
    flags += [
        0b1 << 5 |  # Interlace bit
        0b1 << 7    # Sort flag
    ] * 8

    frame_raw_size = (width * height) // 8

    lsd = [
        # Image block
        0xff & ((frame_raw_size >> 0) & 0xFF),
        0xff & ((frame_raw_size >> 8) & 0xFF),

        # Left offset
        0,
        # Top offset
        0,
        # Width
        0xff & (height % 0x100),
        0xff & (height >> 8),
        # Height
        0xff & (width % 0x100),
        0xff & (width >> 8),

        0x80 | len(flags),
        *flags
    ]

    out.write(bytes(lsd))


    for i in range((width * height) // 8):
        if rng.read(1)[0] > 0x80:
            value = rng.read(1)[0] - 0x100
        else:
            value = 0

        out.write(struct.pack('B', value))

    palette = []
    for _ in range(2**int(rng.read(1).decode("utf-8"))):
        r = int.from_bytes(rng.read(1), "little")
        g = int.from_bytes(rng.read(1), "little")
        b = int.from_bytes(rng.read(1), "little")
        palette.append(r)
        palette.append(g)
        palette.append(b)

    [out.write(bytes([c])) for c in palette]
    
    end = b'\x3b\x00'
    out.write(end)


if