#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Read global color table if present
    if gctlen != 0:
      gct = rng.read(gctlen)
      out.write(gct)

    while True:
        blocktype = rng.read(1)[0]
        if blocktype == 0x3b: # stop marker
            break
        elif blocktype != 0x2c: # unknown block type
            continue
	else:
          # get next byte
          data = rng.read(1)

          # resolve the size of image data in bytes
          lzwsize = int(data[0])
          if lzwsize == 0xff:
              lzwsize += rng.read(1)
          
          # read the compressed GIF data
          lzw = rng.read(lzwsize)

          # write to output
          out.write(struct.pack('<B', lzwsize))
          out.write(lzw)

        # check for repeat or clear code blocks
        if len(lzw): 
            last = ord(lzw[-1:])
            if not ((last >= 0x00) & (last <= 0x0f)):
                continue
        
        # don't forget to close it all off!
        out.write(b'\x00')
    
    return 
