#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if int(gctlen):
        gct = bytes([0]) + rng.read(gctlen - 1)
        crc = bytes([0xFF, 0x0B])[::-1]

        out.write(gct)
        out.write(crc)

    # Loop over how many images there are
    num_images = ord(rng.read(1)) & 0b01111111

    for i in range(num_images):
        # Create image descriptor
        id = rng.read(9)[0:-1]

        out.write(id)

        size = 2**(int(((id[8] << 8 | id[7]) >> 6) & 0b11))

        # If we have local color table, read that as well.
        if id[-1] >> 7 == 1:
            lclen = 2**(id[8] & 0b00000111)

            lct = bytes([0]) + rng.read(lclen * 3)

            crc = bytes([0xFF, 0x0B])[::-1]

            out.write(lct)

        imgdata = bytes([ord(rng.read(1)) for _ in range(size)])

        out.write(imgdata)