#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    for _ in range(257):
        # The label says 10 but there are actually two more
        # bytes after the label indicating the length of
        # the block. We know it's always >10 so just set it to 0x10
        # (https://www.w3.org/Graphics/GIF/spec-gif89a.txt)
        out.write(struct.pack('<H', 0x10))
        out.write(rng.read(0x10))
    # Write the last descriptor!
    out.write(b'\x3B')

    # And pad up until the end of the file, which is easy enough because
    # we've already written exactly datalen bytes before ending.
    print('Padding global color table', end='', flush=True)
    while True:
        if out.tell() == datalen+1: break
        out.write(b'\xFF')
