#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
            break

        # This isn't our final block - what kind of block is it?
        blocktype = fieldbyte & 63
        print("Block Type", blocktype)

        # How long is the sub-block?
        blocksize = int.from_bytes(rng.read(1), 'little') - 1
        print('Sub-block size', blocksize)

        if blocktype in [0x21, 0x2C]:

            out.write(b'\xff')
        
            if blocktype == 0x21:
                # Write label byte
                out.write(struct.pack('<B', blocktype))
            elif blocktype == 0x2C:
                # Write label byte
                out.write(struct.pack('<B', blocktype + 1))

            # Write data bytes
            out.write(rng.read(blocksize))

        else:
          print("Skipping unsupported block")

          # Skip forward by the amount of remaining data in the block
          out.seek(out.tell() + blocksize)


    # Finally insert EOF marker
    out.write(b'\x3b')
    
