#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    out.write(b"GIF89a")

    # Logical screen descriptor
    width = 16 + rng.read(1)[0] % (2**8 - 15)
    height = 16 + rng.read(1)[0] % (2**8 - 15)
    bits_per_pixel = 1 + rng.read(1)[0] % 7    # [1..8]
    packed = bytes([
        0 |              # Global color table present
        0x80 |           # Color resolution bit is set
        0x40 |           # Sort flag
        0 |              # Size of global color table
        bits_per_pixel   # Bits per pixel
    ])
    out.write(struct.pack("<HHB", width, height, packed))
    
    # Image descriptor
    left = rng.read(1)[0] % (width+1)
    top = rng.read(1)[0] % (height+1)
    flags = rng.read(1)[0] % 0x8c # All LZW used
    packed = bytes([flags])
    out.write(struct.pack("<HBBB", left, top, width, height))

    # Trailer
    out.write(bytes([0]))

