#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header (6 bytes)
    buf = bytearray([0x47, 0x49, 0x46, 0x38, 0x39])

    # Logical screen descriptor (7 bytes)
    width = 1 + rng.read(1)[0] % 256
    height = 1 + rng.read(1)[0] % 256
    buf += struct.pack("<HHBBB", width, height,
                       rgb=rng.read(1)[0], pixelaspectratio=0)

    # Global color table (3 bytes if transparent)
    ncolors = min(width * height // 256, 255 - int(bool(buf[7])))
    for i in range(ncolors):
        buf += struct.pack("BBB",
                          rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
    
    # Image descriptor (10 bytes)
    buf += struct.pack("<HHBBBBBH",
                       0, 0,      # 0-based offset to next image
                       width % 256, height % 256,
                       0,          # local color table flag
                       int(bool(buf[7])),         # interlaced flag
                       bitsperpixel=8)             # bits per pixel
    blocksize = ((len(buf) + 3) >> 2) + 1     # block size (in 32-bit words)
    buf += b"\x00" * (((blocksize + 1) << 2) - len(buf))

    # Trailer (1 bytes) -- end of file
    buf += b";"

    # Write generated buffer to output stream
    out.write(bytes(buf))

