#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
# Write header
    print("Writing header")
    out.write(header + (b'\x00'*6))

    # Write screen descriptor and background color index
    print("Writing screen descriptor and background color index")
    out.write(struct.pack('>HBB', 412, 74, 1))
    out.write(rng.read(3))
    
    # Write global color table with only white color
    print("Writing global color table with one entry")
    out.write(b'\xFF\x00\xFF\x00\xFF\x00')
    out.write((b'\x00' * 45))

    # Write frame control block
    print("Writing frame control")
    out.write(b'\x2C\x00\x00' + bytes([rng.read(1)[0] for _ in range(4)]))

    # Write local color table
    print("Writing local color table at offset", out.tell())
    lctsize = [rng.read(1)[0]]*int.from_bytes(rng.read(1), 'little')
    while sum(lctsize) > 2**depth+1:
        lctsize[int.from_bytes(rng.read(1), 'little')] -= 1
    # Write LCT
    out.write(b''.join([bytes([elem]) for elem in lctsize]))
    if depth == 8:
        out.write(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff')
        out.write(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff')
        out.write(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff')
        out.write(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff')
        out.write(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff')
        out.write(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff')
        out.write(b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff')
    else:
        out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
        out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
        out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
        out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
        out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
        out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
        out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write left position, top position, width, height
    out.write(struct.pack(">HHHH",
        1,
        1,
        rng.read(2)+1,
        rng.read(2)+1,
    ))

    # Write flags
    out.write(b'\b')

    # Write lzw minimum code size
    out.write(b'\b')

    # Write image data
    print("Writing image data at offset ", out.tell(), "of length", len(data))
    out.write(data)

