#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    for _ in range(257):
        # The spec says that each block has a fixed size of 10 always.
        # If it doesn't have one, skip over it.
        if len(data) < 10: break

        (left, top, width, height, flags) = struct.unpack('<HHHHB', data[0:10])

        if left == -1 and top == -1: break;

        print(f"Writing descriptor at ({width}, {height})...",
                end='', flush=True);
        out.write(flags.to_bytes(1, 'little'))
        out.write((width).to_bytes(2, 'little'))
        out.write((height).to_bytes(2, 'little'))
        out.write(b'\x00\x00')
        out.write(b'\x00')
        out.write(data[10:10+((Flags(flags) & Flags.LCTF_TABLE) >> 4)])
        data = data[10 + ((Flags(flags) & Flags.LCTF_TABLE) >> 4):]
        print("done.", flush=True)

