#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)
    
    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
          return
    
        # Get size of block
        size = int(((fieldbyte << 8) | ord(rng.read(1)))) - 1;

        # Get the current position in the file so we can skip over it after
        # copying it into the new gif
        pos = out.tell()

        # Copy bytes over to new gif
        data = rng.read(size);
        out.write(data)

        # Go back and write the correct size into the field before the copy
        out.seek(pos + 1)
        out.write(struct.pack("B", len(data) + 1));

        # Skip unknown block bytes
        out.seek(out.tell() + 1)
