#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Read global color table if present
    if gctlen != 0:
      gct = rng.read(gctlen)
      out.write(gct)

    while True:
        blocktype = rng.read(1)[0]
        if blocktype == 0x3b: # stop marker
            break
        elif blocktype != 0x2c: # unknown block type
            continue

        # Read the rest of the extension
        extheader = rng.read(6)
        out.write(extheader)
        
        # Parse extension header
        hasgct   = (extheader[0] & 0x80) != 0
        hasimgdata = (extheader[0] & 0x08) != 0
        size    = int(struct.unpack(">H", extheader[2:])[0]) + 1
        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:
          lct = rng.read(numcolors*3)
          out.write(lct)

        lzwmincodesize = rng.read(1)[0]
        out.write(bytes([lzwmincodesize]))

        compressedstream = bytes()
        for i in range(0, size):
            newbyte = rng.read(1)[0]
            compressedstream += bytes([newbyte])
        out.write(compressedstream)