#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#  * Header is always "GIF89a" (6 bytes), fixed size
#  * Data length is a single short (2 bytes), little endian order
#  * Random data of variable length (determined by data length field).
def generate_gif(rng: BinaryIO, out: BinaryIO):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='little', signed=False
    )
    data = rng.read(datalen)

    out.write(header + datalen.to_bytes(2, 'little'))
    out.write(data)

