#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. 2 bytes of length of block (N).
#   3. N-byte data block containing image descriptor and color table.
#   4. 1 byte separator character (0x2C) to indicate that this is an extension.
#   5. 2 bytes of subblock size for image data, which should be 0x00.
#   6. N-byte image data (likely to contain garbage).
#   7. EOF


def gen_random(out: BinaryIO):
    # Make sure we write enough bytes so we don't get truncation at the end.
    out.write(b'GIF89a')

    buf = b'\x00\x00' + \
          struct.pack('!HH', *([1] + [i % 256 for i in range(768)])) + \
          b'\x00'

    out.write(struct.pack('!H', len(buf)))
    out.write(buf)
    out.write(b'\x2c\x00')
    out.write(b'\x2c\x00')
