#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write Header (ID and size)
    out.write(header +
              datalen.to_bytes(3,byteorder='big'))
    
    # Write Color Table Size
    colorTableSizeFlagAndSortFlag = \
            1 << 7 | 1 << 4 | 1 << 2 | 1 # Global Color Map Flag, Sort Flag, Size flag
    out.write((colorTableSizeFlagAndSortFlag).to_bytes(1,"big"))
    out.write(b'\x07\xFF\xFF\xFF\xFF\xFF\xFF') # Background color
    out.write(b'\x00\x00\x00') # Pixel Aspect Ratio

    # Write Image Descriptor
    leftPos = 0
    topPos = 0
    width = 320
    height = 240
    localColorTableFlag = True
    imageDescriptorFlags = 0b0000_0010 | 0b1000_0000
    if not localColorTableFlag:
        imageDescriptorFlags |= 0b0000_1000 # Interlace Flag
    else:
        imageDescriptorFlags |= 0b0000_0100 # Local Color Table Flag
    out.write((leftPos).to_bytes(2,"big"))
    out.write((topPos).to_bytes(2,"big"))
    out.write(((width << 8) | (height & 0xff)).to_bytes(2, "big"))
    #print("image descriptor",(leftPos,(topPos),(width << 8) | (height & 0xff)),"flags",imageDescriptorFlags)
    out.write(imageDescriptorFlags.to_bytes(1, "big"))


    # Generate Frame Control Block
    marker = bytes([
        0x21, 
        0xf9, 
        0x04, 
        ])
    blocksize = len(marker+b"\x00\x00\x00")
    trailer = b"\x00"
    fcb = marker + blocksize.to_bytes(3, 'little')+trailer
    out.write(fcb)

    # Write Graphic Control Extension with delay of 0
    gce = b""
    gce += bytes([         
        0x21,           # Extension Introducer
        0xf9,           # Graphic Control Label
        4,              # Block Size
        # Packed field
        1<<7,           # reserved
        0,              # disposal method
        0,              # user input - no transparancy involved
        0x2c            # transparent color index is 44
        ])
    out.write(gce)
    out.write(struct.pack("<i", 0)) # Delay Time is zero ms

    # Write Logical Screen Descriptor
    lsd = b""
    lsd += bytes([
        # Screen Width
        320, 

        # Screen Height
        240,

        # Packed Field
        1<<7|      # Global Color Table Flag
        1<<4,       # Color Resolution
        1<<2|      # Sort Flag
        1,         # Size of Global Color Table

        7,     # BackGround Color Index

        0,     # Pixel Aspect Ratio 
    ])
    out.write(lsd)
    
    # Copy pixels from /dev/urandom to our output stream
    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
