#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output buffer
    header = b"GIF89a" + bytes([0] * 7)

    out.write(header)

    # Create an infinite loop of subblocks until EOF reached
    while True:
        # Get our block size from our RNG (we can't do more than 256 blocks)
        blocksize = int.from_bytes(
            rng.read(1), byteorder="big", signed=False
        )

        if not blocksize or blocksize == 256: break

        # Write the block size to our output stream; we will fill in
        # the rest with zeroes later on
        out.write(blocksize.to_bytes(
            1, "big", signed=False))

        # Now write up to 255 data bytes for this block
        datalen = min(blocksize, 255)
        
        out.write(rng.read(datalen))

        # Check whether we have more bytes to read after the first one
        hasMoreData = bool.from_bytes(
            rng.read(1), byteorder="big", signed=False
        )

        # If so, then we are going to need padding
        if hasMoreData:
            numPadBytes = abs(blocksize - datalen - 2)

            out.write((numPadBytes).to_bytes(
                1, "big", signed=False
            ))

            out.write(b"\x00" * numPadBytes)
