#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        (rng.read(1)) * 75291 % 4096 + 1024, 'big')

    # pad to an even size
    if datalen & 0x1 == 1:
        datalen += 1

    data = bytearray()
    for i in range(datalen):
        data.append(int.from_bytes((rng.read(1)), 'big'))
        
    footer = [b'\x00', b'\x3B'][
            0 <= int.from_bytes(
                rng.read(1), 'big') < 6]
    out.write(header + len(data).to_bytes(2, 'big') + \
            data + footer)
