#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3
    size = int(gctlen / 3)

    while True:
        # Get block type byte
        bt = rng.read(1)[0]
        
        # Check for end of file
        if bt == 0x3b:
            break

        # Read offset bytes
        offsbytes = rng.read(3)
        offset = struct.unpack('>H', offsbytes[:2])[0]
        size = ord(offsbytes[-1]) + 1

        # Write first part of the image descriptor
        idbytes = b'\x2c\x00\x00\x00\x00\x00'
        out.write(idbytes)
        
        # Write graphics control extension
        extbyte = b'\xf9\x04'
        out.write(extbyte)

        # Write graphic control label and some params
        ctrlbytes = b'\x04\x00\x00\xff\xff\xff\x00'
        out.write(ctrlbytes)

        # Write local color table if any is necessary
        numcolors = 2**(bt & 0x07)

        if numcolors > 0:
          colortable = rng.read(numcolors*3)
          out.write(colortable)

        # Now write image data 
        imgdata = rng.read(size-1)
        out.write(imgdata)