#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff - 7

    # Write pre-frame bytes  
    out.write(header + (datalen + 7).to_bytes(
        3, 'big')) 
    out.write((b'\x21\xFF\x0B') + \
              ("NETSCAPE2.0".encode("ascii")))
    out.write(((datalen + 3).to_bytes(1,"little")+
              b'\x03\x01\x00') +
              (os.urandom(1)))
    out.write(b"\x00,\x00\x00")

    # write frame data
    for i in range(datalen // 64):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))