#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    def write_byte(b):
        # Avoid writing an EOF byte (byte with value 0)
        if b == 0: return

        out.write(struct.pack('=c', bytes([b])))


    # Putting this here makes it easier to remove later
    # when we add valid times and color tables.
    while True:
        hdrlen = rng.read(7)[6]
        if not bool(hdrlen & 0x40): continue

        break

    out.write(rng.read(hdrlen))
    for _ in range((hdrlen // 8) - 1):
        out.write(rng.read(int.from_bytes(
                rng.read(1), 'big')))

