#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom). The
# generated GIF file has a width of 64 and height of
# 64 pixels (2^7 x 2^7). All colors are randomized
# as well.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'\x47\x49\x46\x38\x39\x61')

    # Logical Screen Descriptor
    out.write((2**7).to_bytes(2, 'big'))   # Width  is 2^7 * 2px per pixel
    out.write((2**7).to_bytes(2, 'big'))   # Height is 2^7 * 2px per pixel
    out.write(b'\xf0')                     # Global Color Table Indicator
    out.write(b'\xff\xff\xff\x00\x00\x00') # Palette
    out.write(b'\x2c')                     # Background Color Index
    out.write(b'\x00')                     # Pixel Aspect Ratio
    
    # Image Descriptor
    out.write(b'\x21\xf9\x04\x05')         # Graphic Control Extension
    out.write(b'\x00')                     # Block Size
    out.write(struct.pack('>B', 0))       # Packed field
    out.write(((2**7)**2)*3)              # Delay Time in ms
    out.write(b'\xff')                     # Transparent Color Index
    out.write(b'\x00')                     # Terminator for graphic control extension
    
    # Image Data
    for i in range((2**7)**2):
        red   = ord(rng.read(1)) % 255
        green = ord(rng.read(1)) % 255
        blue  = ord(rng.read(1)) % 255

        out.write(red.to_bytes(1, 'big'))          # Red
        out.write(green.to_bytes(1, 'big'))        # Green
        out.write(blue.to_bytes(1, 'big'))         # Blue


    print("Generated random GIF")