#!/usr/bin/env python3

import struct
from typing import BinaryIO
from PIL import Image, ImageDraw, ImageFont
from random import getrandbits

# Generate a random GIF image into `out`.
# This is mostly here to show that the vulnerability
# can be triggered by a crafted framedata field in an image descriptor.
def generate_random_gif(
        width: int, height: int, fps: float, duration:float,
        fontname: str, text:str, fgcolor: tuple, bgcolor: tuple,
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Basic GIF header.
    # The magic number is "GIF89a" but we'll skip the version
    # Gif89a = Version 89a Compressed without LZW compression
    # See http://giflib.sourceforge.net/whatsinagif/index.html
    gifhead = b'GIF89a'

    # A single bitmap is used for each frame
    # The palette should contain 2 colors
    # one for the bg and another one for the fg.
    # The images are generated using the Pillow library
    palette = [bgcolor, fgcolor]
    baseimg = Image.new('P', (width, height), 0)
    img = Image.new('P', (width, height))
    draw = None
    def make_frame():
        nonlocal draw
        # Random choice between foreground and background colours
        col = palette[(getrandbits(1))]
        # Draw a rectangle with a random colour
        if draw == None:
            baseimg.putpalette([int(i*255) for i in col+col]+[0]*768)
            draw = ImageDraw.Draw(baseimg)
        draw.rectangle([(0,0),(width,height)], fill=(0,))
        
        # Overlay a random sized text string.
        # The position will be chosen randomly within the bounds
        # of the image, while the colour will stay fixed.
        # These parameters are controlled via the command line arguments.
        fontsize = 10+((width+height)//64)
        fontpath = fontname
        font = ImageFont.truetype(fontpath, fontsize)
        xsize, ysize = draw.textsize(text, font=font)
        rposx = randrange(max(xsize//2,0), min(width-xsize//2, 0-(width-xsize)))
        rposy = randrange(max(ysize//2,0), min(height-ysize//2, 0-(height-ysize)))
        draw.text((rposx, rposy), text, font=font, fill=tuple(map(lambda c:int(c*255), col)))
        return baseimg
    
    imgs = []
    imgdurations = []
    for _ in range(fps * duration // 0.015):
        imgs.append(make_frame())
        # For each frame pick a random delay time between 0ms and 15ms.
        # The actual encoding algorithm will take care of throwing away delays shorter than 11ms.
        imgdurations.append(randrange(0,15)*10)
    # Convert the list of frames into a single animated GIF image
    imgs[0].save(fp=out, format="gif", save_all=True, append_images=imgs[1:], optimize=False, duration=imgdurations, loop=0)


