#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generate the color table from the given seed as described in
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt Section 17 and Appendix A
def gen_color_table(seed : int)->list:


    # Color Table is initially only black and white
    ct = [(0, 0, 0), (255, 255, 255)]

    # Use the output of /dev/random to fill in the rest of the colors
    while len(ct)<256:

        r = int.from_bytes(
            open("/dev/urandom", "rb").read(4), 
            byteorder="big"
        ) & 0xffffff
        if not (r,0,0) in ct and r > 0:
            ct += [tuple([i//256*255 for i in r])]

    return ct

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  

    out.write(header + (datalen+1+3).to_bytes(
            3, 'big')) 
    out.write((b'\xff\x0bNETSCAPE2.0\x03\x01').ljust(
            12," "))
    out.write(((datalen//3)%256).to_bytes(
            1,byteorder="big"))
    out.write((datalen%256).to_bytes(
            1,byteorder="big"))
    out.write(b'\x00')

    # write frame data

    for i in range(datalen):
        out.write(gen_color_table(
                1)[(int.from_bytes(
                            rng.read(1), byteorder="big") %
                               (len(gen_color_table(1))))])


    out.write(b"\x2C")       # Start image descriptor
    out.write((((datalen)//8)+1).to_bytes(
            1, byteorder="big"))   # Width
    out.write((((datalen)//8)+1).to_bytes(
            1, byteorder="big"))     # Height
    out.write(b"\x00")      # No local color table
    out.write(b"\x00")      # Interlace bit
    out.write(b"\x00")      # Sorting bit
    out.write(b"\x00")      # Reserved bits
    out.write(b"\x00")      # Size of LZW minimum code size
    out.write(b"\xFF")      # Block terminator    
