#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Write the header and the screen descriptor to
    # the output stream
    def write_header():

        global terminator
        header = b'GIF89a'
        width = int.from_bytes(
                rng.read(2), 'big') % (2 << 11)
        height = int.from_bytes(
                rng.read(2), 'big') % (2 << 11)
        colortablesize = int.from_bytes(
                rng.read(1), 'big') & 7;
        bgindex = int.from_bytes(
                rng.read(1), 'big')
        pixelaspectratio = int.from_bytes(
                rng.read(1), 'big')
        terminator = [b';\n\x00',
                      b'\x00;\n'][
                        bool((int.from_bytes(
                            rng.read(1), 'big')))]
        buf = bytearray()
        for i in range(colortablesize):
            buf += bytes([int.from_bytes(
                    rng.read(1), 'big')])*3
        return header+width.to_bytes(
                2, 'big')+height.to_bytes(
                    2, 'big')+colortablesize.to_bytes(
                        1, 'big')+buf+\
            bgindex.to_bytes(1, 'big')+pixelaspectratio.to_bytes(
                1, 'big')+terminator
    
    # Writes the LZW image data to the output stream
    def write_image():
        
        global terminator
        
        codesize = int.from_bytes(
                rng.read(1), 'big') + 1
        lzwminlen = 2 ** codesize
    
        # Generate all codes of length up to
        # lzwminlen from the input data read so far
        dataptr = 0
        datasize = len(datapos)-dataptr
        bufsz = min(datasize, 4095-lzwminlen)
        table = dict([(i, [i]) for i in range(codesize)])
        entry = list(range(bufsz))
        nextcode = codesize
        while True:
    
            if (nextcode >= (lzwminlen)): 
                result = datapos[dataptr :
                                dataptr + bufsz]
                bufsz -= len(result)
                dataptr += len(result)
            else:
                
                if not result: 
                    break
    
                if (len(result)<bufsz):
                    result += [entry[-1]] *\
                            (bufsz - len(result))
    
                for c in result:
                    try: 
                        table[nextcode].append(table[c][0])
                    except KeyError as e:
                        pass
                    table[nextcode].append(c)
                    nextcode += 1
                    if (nextcode == lzwminlen): 
    
                        # We reached the maximum size of our
                        # dictionary. Output it on the stream and
                        # start anew with a minimum sized dictionary
                        buf = bytearray()
                        for v in table.values(): 
                            l = len(v)
                            try: 
                                buf.extend([\
                                        (v[0]).to_bytes(
                                            1, 'big'),
                                        ((l-2).bit_length()).to_bytes(
                                            1, 'big')[0]])
                            except IndexError as e: 
                                print("len:" + str(len(v)))
                                exit(-1)

                            buf.extend(v[:l-1])
                            
                            if (l!= 2**buf[1]+2):
                                raise ValueError(
                                    "Dictionary size mismatch!")
                        del table[:]
                        table[lzwminlen] = [(lzwminlen)]
                        nextcode = codesize
                        yield buf
                        continue

            bufsz = min(datasize-(bufsz+dataptr), 4095-lzwminlen)
            nextcode = 2 * codesize
    
        buf = bytearray()
        lastbyte = None
        for value in (result + (entry[-1],)*bufsz): 
            try: 
                buf.extend(\
                        (table[value][0]).to_bytes(
                            1, 'big'))
            except KeyError as e: 
    
                # If we encounter a symbol that is not yet present
                # in our dictionary, add it at its longest possible prefix
                newraw = []
                if lastbyte is not None: 
                    newraw = table[lastbyte]  

                for i in reversed(newraw[:-1]): 
                    try: 
                        table[value].insert(
                                0, (table[i][0]))
                        break
                    except (KeyError, IndexError,) as e: 
                        pass
            
                table[value] = newraw + [value]
                buf.extend(\
                        (table[value][0]).to_bytes(
                            1, 'big'))
    
        del table[:]
        table[lzwminlen] = [(lzwminlen)]
        nextcode = codesize
        
        yield buf
        yield terminator
        
    bufstream = iter([])
    bufstream = write_header()
    for chunk in write_image(): 
        bufstream += chunk
    out.write(''.join(list(bufstream)).encode('ascii'))