#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Seek past the header and then grab the size from the first descriptor
    # That tells us how large our global colour table should be
    out.seek(6)
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') - out.tell()
    assert datalen >= 4

    data = rng.read(datalen)
    out.write(data)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    for i in range(257):
        out.write(struct.pack('<HHB',
            0x10,
            0x10*i+datalen,
            0xFB
        ))
        with open('frame_%d.bin'%i, 'wb') as f:
            f.write(rng.read(0x10))

        out.write(rng.read(0x10))
    # Write the final terminator
    out.write(b'\x3B')

    # And pad up until the end of the file, which is easy enough because
    # we've already written exactly datalen bytes before ending.
    print('Padding global color table', end='')
    while True:
        if out.tell() == datalen+1:
            print('. Done!')
            return
        out.write(rng.read(1))
    print('')