#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    for i in range(257):
        # Each descriptor is 13 bytes plus the size indicator
        # which is 2 bytes for each dimension.

        if i == 256:
            break

        xpos = rng.randint(
                0, 0xFFFF
                ).to_bytes(
                    2,
                    'big'
                    )

        ypos = rng.randint(
                0, 0xFFFF
                ).to_bytes(
                    2,
                    'big'
                    )


        width = rng.randint(
                1, 0xFFFF
                ).to_bytes(
                    2,
                    'big'
                    )


        height = rng.randint(
                1, 0xFFFF
                ).to_bytes(
                    2,
                    'big'
                    )

        # TODO: Should experiment with variations on these values too
        #       see https://github.com/decompme/decompilation-guide/issues/47
        out.write(b'\x2C\x00\x00\x00' +
                  xpos +
                  ypos +
                  width +
                  height +
                  b'\x00\xFB\x00\x00\x00')



    last_byte = out.tell() - 1
    # At the moment this code doesn't support appending arbitrary
    # bytes onto the output so we'll have to manually compute where
    # our data will start relative to the rest of the file
    print(f'The generated gif is {last_byte} bytes long.')

    # For now we'll hardcode the number of loops to use, since we don't
    # yet know how much space remains between the final descriptor and EOF
    numloops = 0xC0

    paddinglen = (numloops * 3).to_bytes(size=2, byteorder='little')

    # Write the leftover bytes before the last descriptor
    print('Writing remaining bytes', end='', flush=True)
    out.seek(-1, 2)
    out.write(paddinglen)
    out.write(struct.pack('<B', numloops))
    out.flush()
