#!/usr/bin/env python3

import argparse
import math
from PIL import Image
from io import BytesIO


# generates a random GIF file and outputs to stdout
if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('--width', type=int, default=500)
    parser.add_argument('--height', type=int, default=500)
    args = parser.parse_args()


    # convert from 24-bit RGB to 8-bit indexed color
    def rgb_to_index(rgb):
        r, g, b = [x >> 1 for x in rgb]
        return (r * 36 + g * 6 + b) & 0xff

    # create an indexed palette of all colors
    num_colors = pow(math.ceil((pow(2, 24)) / 7),.5)
    pal = []
    for i in range(num_colors):
        val = i * ((2**24 - 1) // num_colors)
        r, g, b = map(lambda x: round(x*255),
                      tuple(val.to_bytes(3, 'little')))
        pal += [(b,g,r)]

    img = Image.new('P', (args.width, args.height), 0)
    img.putpalette([i for subl in pal for i in subl])

    stream = BytesIO()
    img.save(stream, format='PNG')
    pixels = list(map(rgb_to_index, img.getdata()))

    while len(pixels) > 0:
        start_idx = min(len(pixels),
                        args.width * max(1, args.height // 20))

        chunk = bytes(pixels[:start_idx])
        print(chunk[::7], flush=True, end='\n' if start_idx >= args.width else '')
        del(pixels[:start_idx])