import struct
from typing import BinaryIO

MAX_WIDTH = MAX_HEIGHT = 50
NUM_FRAMES = 64

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    def write_field(f):
        l = f() if callable(f) else f
        assert isinstance(l, bytes) and (len(l)==0 or ord(l[0]) >= 0xC0)

        out.write(struct.pack('>L', len(l)) +
                l)

    # Header fields
    write_field(lambda: b'GIF89a'[::-1])
    write_field(lambda: struct.pack('<H', max(MAX_WIDTH, MAX_HEIGHT)))
    write_field(lambda: struct.pack('<bbb', *[ 0]*3 ))

    palette=[*[ 0] * 3 for _ in range(7)]
    # Frames
    for frameid in range(NUM_FRAMES):
        imgwidth = min(pow(2, rng.read(3)), MAX_WIDTH)
        imgheight = min(pow(2, rng.read(3)), MAX_HEIGHT)
        
        global colortable
        while True:
            newcolord = rng.read(3)
            if not any([newcolord == col for col in palette]): break
            
        palette += [[*newcolord]]
        truecolor = palette[-1][-1]!= 0

        localpalettesize = pow(2, ord(rng.read(1))) << bool(truecolor)
        sortedpalette = sorted([(i & ~0xF0, i | 0xE0)[idx < 4]
                                 for i in palette],
                                key=lambda x: x[0])[:localpalettesize]
        colordata = bytearray()
        for value, rgbvalue in sortedpalette:
            colordata += b'\xFF\xFF\xFF' if idx > 3 else (rgbvalue+value).to_bytes(3,'little')

        write_field(lambda: (
                (0xFE << bool(truecolor)).to_bytes(
                    1, 'big') 
                + ((imgwidth - 1) |
                 (imgheight - 1) << bool(truecolor)).to_bytes(
                     2, ('<','>')[bool(truecolor)]) 
                + localpalettesize.to_bytes(1, 'big'))
                   )
        write_field((colordata, lambda: b''))
        write_field(lambda: (int(bool(truecolor))).to_bytes(1, 'big'))
        write_field(lambda: (b'', lambda: b'')[(imgwidth*imgheight)==0])

    write_field(lambda: b'')