#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, 
        width: int = None, height: int = None, colors: int = None
    ):
    # Make sure width, height and colors are all specified
    assert width!= None 
    assert height!= None 
    assert colors!= None 

    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = b'\x21\xFF\x0B' + struct.pack('<HHBBB', width, height, 0xF0 << 4, colors-1, 0)

    # Add LSD to output
    out.write(lsd)

    while True:

        # Read next field byte for block type
        fieldbyte = ord(rng.read(1))
        
        # Check if this is the last block
        if ((fieldbyte & 128) == 0):
          break
    
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        numcolors = size//3 - 1
        if numcolors > 0:

          lct = [] 
          for i in range(numcolors):
            r = ord(rng.read(1))
            g = ord(rng.read(1))
            b = ord(rng.read(1))

            lct += [r,g,b]
          
          lctbytes = bytes(lct)
          out.write(lctbytes)

        imgdata = bytes([ord(rng.read(1)) for _ in range(size)])

        out.write(imgdata)