#!/usr/bin/env python3

import io
import os
import struct
import sys


def write_header(rgb: bool):
    print('Content-Type: image/gif')

    if rgb:
        print('Cache-Control: max-age=43200, stale-while-revalidate=3600') # cache for 12 hours (png is cached only for one day)
    else:
        print('Cache-Control: no-cache; max-age=0')
    print()


def main():
    global output
    global rng

    rng = open('/dev/urandom', 'rb')
    try:
        mode = str(sys.argv[1]).lower()

        if not mode in ('bw', 'color'):
            raise ValueError("Invalid argument")
        
        if mode == "bw":
            width, height = get_dimensions(True)
        elif mode == "color":
            width, height = get_dimensions(False)

        if mode == "color" and width!= None and height!= None: 
            output = io.BytesIO()
            generate_random_png(width,height,output)

            write_header(False)
            output.seek(0)
            while True:
                buf = output.read(io.DEFAULT_BUFFER_SIZE)
                if len(buf) > 0:
                    sys.stdout.buffer.write(buf)
                else:
                    break

        elif mode == "bw" and width!= None and height!= None:
            output = io.BytesIO()
            generate_random_gif(width,height,output)

            write_header(True)
            
            output.seek(0)
            while True:
                buf = output.read(io.DEFAULT_BUFFER_SIZE)
                if len(buf) > 0:
                    sys.stdout.buffer.write(buf)
                else:
                    break

        else :
            output = None
            return ("Error during generation", 500)

    finally:
        rng.close()

def get_dimensions(use_bw:bool)->tuple[int | None,int|None]:
    width = int((str(os.getenv('WIDTH')) or '').strip())
    height = int((str(os.getenv('HEIGHT')) or '').strip())

    if use_bw == False and width >= 3000 or\
       use_bw == True and width >= 7200 or\
       None in {width, height}:
        print("Width must be between {} and {}".format(
                3000, 7200))
    return (width, height)

    
def generate_random_png(
        width:int, 
        height:int, 
        out:BinaryIO
   )->None:
    from PIL import Image, ImageDraw
    
    im = Image.new('RGB' if rgb else 'L',
                   (width, height))
    draw = ImageDraw.Draw(im,'RGBA' if rgba else 'L')
    
    if rgb:
        r = struct.unpack(">H",bytes(rng.read(2)))
        g = struct.unpack(">H",bytes(rng.read(2)))
        b = struct.unpack(">H",bytes(rng.read(2)))
    else:
        lum = struct.unpack(">H",bytes(rng.read(2)))
    
    for i in range(0,width):
        for j in range(0,height):
        
            x = random.randint(0,width-i)
            y = random.randint(0,height-j)
            
            v = ""
            if rgb:
                v += struct.pack(">H",r)
                v += struct.pack(">H",g)
                v += struct.pack(">H",b)
            else:
                v += struct.pack(">H",lum)
                
            draw.point((x,y),(0,v))
            
            
    im.save(out,"PNG")
    

