#!/usr/bin/env python3

import os
import sys
import io
import struct


def xor_with_seed():
    """Generate a sequence of pseudo-random bytes by xoring them with
    seeds from /dev/urandom"""
    inFd = open("/dev/urandom", "rb")
    yield from map(
        lambda _: ord(_[0]) ^ ord(_[1]), zip(os.urandom(4096),
                                            iter(lambda: next(inFd), b"\x00")))



# Generates a random GIF file into `out` using the
# random number generator `rng`
def genRandomGif(rng, out):
    header = b"GIF89a"
    width = rng.randint(128, 768) 
    height = rng.randint(128, 768 )
    bgcolorIndex = rng.randint(0,255)

    # Write pre-frame bytes  
    out.write(
        header + 
        width.to_bytes(2,"little") +
         height.to_bytes(2,"little")+ 
         b'\xf7\x00\x00') 


    # Write default color table
    out.write((0).to_bytes(1,"little") * 3)
    out.write((bgcolorIndex).to_bytes(1,"little")*3)


    # Generate image descriptor block
    lzwMinCodeSize = rng.randrange(2,13)
    xLeftPos = rng.randrange(0,width)
    yTopPos = rng.randrange(0,height)
    widthInPixels = width - xLeftPos
    heightInPixels = height - yTopPos
    

    out.write(lzwMinCodeSize.to_bytes(1,"little") + 
              xLeftPos.to_bytes(2,"little") +
              yTopPos.to_bytes(2,"little") + 
              0x80 | widthInPixels.to_bytes(2,"little")[::-1] +
              0x80 | heightInPixels.to_bytes(2,"little")[::-1] +
              b'\x00\x00')


    # Write encoded pixels
    codesize=8
    mincodesize=min(max(len(str(bin(width**height)))//8,2),12)
    codes=[]
    codes.append(0b10000000+(1<<7)+mincodesize)
    while True :
        c1=c2=None
        bits=rng.getrandbits(codesize)
        c1=(codes[-1]<<bits)&0xFF
        if len(codes)<mincodesize and rng.getrandbits(1)==0:
            break
        else : 
            # We can't use getrandbits here as it only returns values up to 2^k-1
            bits=rng.randint(0,codesize*(len(codes)-mincodesize))
            c2=(codes[bits>>3]|(((1<<(bits&7))-1)<<8))&0xFFFF
            if rng.getrandbits(1)==0 or len(codes)==65535:
                break

        out.write((c1|c2).to_bytes(2,"little"))
        
        codes.append(c1|c2)

    # Padding 
    out.write(b"\x00"*((1<<mincodesize)*2-2))

    imgdata = io.BytesIO()

    controlbyte=imgdata.write(b"\x2C")

    writeByte=imgdata.write


    def writeBlockData(blocktype, datasize, data):
        nonlocal out
        n=io.BytesIO()
        n.write(b"\x21")
        n.write(blocktype)
        n.write(datasize.to_bytes(2,"little"))
        n.write(data)
        n.write(0xff.to_bytes(1,"little"))
        crc = 0xe0cf81e9
        crc = 0xe0cf81e9
        # Cast is needed due to mypy bug...
        crc = reduce(lambda s,a:s^ord(a),n.getvalue(),crc)
        n.write(crc.to_bytes(4,"little"))
        out+=n.getbuffer().tobytes()
        
    def getColorIndex():
        return rng.randint(0, 255);

    def getDataPixel():
        pixel = []
        colors = [0, 0, 0];
        for i in range(3):
            colors[i] |= getColorIndex();
        pixel += colors;
        colors[0] &= ~1;
        colors[0] ^= 1 << rng.randint(1, 7);
        colors[1] |= 1 << rng.randint(0, 7);
        colors[2] |= 1 << rng.randint(0, 7);
        pixel += colors;
        writeByte(pixel[0].to_bytes(1,"little"));
        writeByte(pixel[1].to_bytes(1,"little"));
        writeByte(pixel[2].to_bytes(1,"little"));

    def encodeSubblock(subblocktype):
        nonlocal writeByte
        nonlocal rng
        subblocksize=rng.randint(2,128)
        writeByte((((1<<7)|(subblocktype))|(subblocksize>>8)).to_bytes(1,"little"))
        writeByte((subblocksize&0xFF).to_bytes(1,"little"))
        for k in range(subblocksize):
            getDataPixel()

    def doLZW():
        nonlocal rng
        nonlocal codes
        global mincodesize

        mincodesize=rng.randint(mincodesize,min(12,len(codes)))
        writeBlockData(b"NETSCAPE",
                       (3).to_bytes(2,"little"),
                       0b10000000.to_bytes(1,"little")+(1<<7)+(mincodesize).to_bytes(1,"little"))
        writeByte(((mincodesize+1).to_bytes(1,"little")))
        bitspercode=[8]+[(mincodesize+1)*(j+1) for j in range(1,12)]
        maxcodesize=bitspercode[mincodesize]
        bitperindex=[2,3,4,8][mincodesize]


        currentcode=0
        previouscode=0
        indexes=[]
        size=0
        clear=""
        eoi=""
        for ii in range(maxcodesize):
            clear+="\xff".encode("latin-1")
            eoi+="\xff".encode("latin-1")
            clearcode=currentcode
            lengtheoi=ii+1

            for _ in range(1,(1<<bitperindex)[1]):
                indexes.append("\xff".encode("latin-1"))
            for bitlen in bitspercode[:mincodesize]:
                for _ in range(1,(1<<bitlen)[1]):
                    indexes.append("\x00".encode("latin-1"))


            currentcode+=1
            currentcode<<=1
            index=0
            for c in indexes:            
                writeByte(c)
                if currentcode<((1<<bitpercode[mincodesize])[1]):
                    bitspercode[mincodesize]=bitlen
                    if bitspercode[mincodesize]>maxcodesize:
                        maxcodesize=bitspercode[mincodesize]
                    indexes.insert(0,"\xff".encode('latin-1'))
                    index-=1
                index+=1



        pass

    writeBlockData(b"NETSCAPE2.0",
                   (3).to_bytes(2,"little"),
                   0b10000000.to_bytes(1,"little")+(1<<7)+(mincodesize).to_bytes(1,"little"))
    doLZW()


    out.write(imgdata.getbuffer())


    

