#!/usr/bin/env python3

import random
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. 2 bytes of length of block (N).
#   3. N bytes of block.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    chunksize = random.randint(
            10, min((2**(struct.calcsize("H")*8))-6, 10**5)
    )
    chunks = [rng.read(chunksize)]
    while len(chunks[-1]) == chunksize:
        chunks.append(rng.read(chunksize))

    total = sum([len(c) for c in chunks])

    # Write the chunks to disk
    totallen = len(chunks)-1
    data = struct.pack('<HH{}s'.format(totallen),
                        *[(len(chunks[i]), chunks[i])
                          for i in range(totallen+1)])
    out.write(header + total.to_bytes(2, 'little') + data)