#!/usr/bin/env python3

import io
import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (or /dev/urandom if none provided).
def generate_random_gif(
        rng: BinaryIO=None, *, size_gb: float=0.125, out: BinaryIO=io.BytesIO()
    ) -> None:

    # Open /dev/urandom or use the user-provided source of bytes.
    if not rng:
        urandom_name = '/dev/urandom'

        try:
            rng = open(urandom_name, 'rb')
        except FileNotFoundError as exc:
            raise RuntimeError('could not find'+ urandom_name) from exc

    # Generate some bytes to use for our header and content.
    width = round((size_gb * 1e9) **.5)
    height = width
    datalen = width * height * 4
    data = rng.read(datalen)

    # Create an empty bytearray and write the header
    # then the pixel data.
    output = bytearray((width*height)*4 + 6)
    output[0] = ord('G')
    output[1] = ord('I')
    output[2] = ord('F')
    output[3] = ord('8')
    output[4] = ord('9')
    output[5] = ord('a')
    output += data

    # Write the output buffer to the specified file object.
    out.write(output)

