#!/usr/bin/env python3

import io
import struct
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. 2 bytes of length of block (N).
#   3. N bytes of block.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Read the header and get the size
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='little', signed=False
    )

    # Build all of the parts up starting with the header itself...
    output = [header]

    # Now add the total size field to our list
    output.append(datalen.to_bytes(2, 'little'))

    # Next we'll read in however much data is necessary for this chunk
    data = rng.read(datalen)
    if len(data)!= datalen:
        print("Data length was not what it should have been.")
        exit()
    
    # Finally append that data onto the list so it gets written later
    output.append(data)

    # Write everything back at once
    out.write(b''.join(output))