#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header', end='', flush=True)
    out.write(header + data[:10])
    print(' done.', flush=True)

    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    num_descriptors = 256
    for i in range(num_descriptors):
        # The label says 10 but there are actually two more
        # bytes after the label indicating the length of
        # the block. We know it's always >10 so just set it to 0x10
        # (https://www.w3.org/Graphics/GIF/spec-gif89a.txt)
        out.write(b'\x2C')
        if i == num_descriptors - 1:
            # Last byte indicates the length of the entire block
            out.write((i+1+2).to_bytes(length=1, byteorder='big'))

            # Write one byte of padding then the number of iterations
            outrem = ((num_descriptors*3)+1).to_bytes(
                    length=2, byteorder='little')
            out.write(outrem[:-1] + (0xC0).to_bytes(length=1,
                                                     byteorder='little'))
        else:
            # All other blocks specify only the size of their contents
            out.write((i+1).to_bytes(length=1, byteorder='big'))

    # Move back to the beginning of the file and add the data
    out.seek(0, 0)
    out.write(data[10:])

