#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (in this case stdin by default)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=None, height=None
    ) -> None:
    if width is not None and height is not None:
        # Use specified dimensions

        out.write(b"GIF89a") 
        wlen = len(width.to_bytes(2,"big"))
        hlen = len(height.to_bytes(2,"big"))
        out.write((wlen+hlen+4).to_bytes(1,"big"))
        out.write(b"\x87\x00\x00")
        out.write(width.to_bytes(2,"big"))
        out.write(height.to_bytes(2,"big"))
        out.write(b"\x00")
        screendescriptor = width.to_bytes(2,"big") + \
                           height.to_bytes(2,"big") + \
                           0b10000100.to_bytes(1,"big") + \
                           0b100.to_bytes(1,"big")
    else:
        raise ValueError("Specify either width or height")


    tablesize = [
        0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 
        0x12, 0x21, 0x31, 0x41, 0x06, 0x13, 0x51, 
    ]

    codesize = [
        0x03, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 
        0x03, 0x01, 0x01, 0x02, 0x04, 0x02, 0x08, 
    ]

    numcodes = { }
    for k in range(len(tablesize)):
        numcodes[2**k] = tablesize[k] * codesize[k] - 1

    pixelmap = [ ]
    while sum([numcodes[2**i]-1 for i in range(12)]) < 255*64:
        l = rng.read()
        pixelmap += [(l >> i) & 1 for i in reversed(range(8))]

    out.write(screendescriptor) 
    out.write((((numcodes[2**9]-len(pixelmap))).to_bytes(1,"big")[0]) | 
               ((numcodes[2**10]-len(pixelmap)) << 8).to_bytes(1,"big")[0])
    out.write(((numcodes[2**11]-len(pixelmap)) << 8).to_bytes(1,"big")[0])
    out.write(b'\x00')

    for n in range(12):
        out.write((numcodes[2**n]).to_bytes(1,"big"))
        
    for c in pixelmap:
        out.write((c ^ 0xff).to_bytes(1,"big")) 

