#!/usr/bin/env python3

from io import BytesIO
from PIL import Image

class RandomGifGenerator:

    def __init__(self):
        self.img = None

        # All colors should be in range [0x00..0xff] to make life easier
        self.colors = []
        for _ in range(256 * 3):
            self.colors += bytes([
                next(self._rng)])


    @staticmethod
    def get_rng():
        try:
            return open('/dev/urandom', 'rb')
        except FileNotFoundError:
            from os import urandom as sysrand

            class DummyRandom:

                def read(self, size=-1):
                    assert (size > -1)
                    return sysrand(size)

            return DummyRandom()


    def randomize_image(self):

        im = Image.new("RGB", (400, 400))

        # Generate a palette of the correct length
        pal = list(map(tuple, zip(*[iter(self.colors)]*3)))
        
        im.putpalette(pal, rawmode="RGB;L")

        # Add an alpha channel and fill it with noise
        im.putalpha(Image.fromarray(im.convert('RGBA').getchannel('A')))

        # Flip the image horizontally and write it to memory
        buf = BytesIO()
        im = im.transpose(Image.FLIP_LEFT_RIGHT).save(buf, format='GIF', transparency=0)
        self.img = buf.getvalue()


    def dump_gif(self, out: BinaryIO) -> int:
        header = b'GIF89a'
        datalen = len(self.img)
        out.write(header)
        out.write(struct.pack('<BBBxxHHB',
            76, 0, 0xFF & datalen >> 8,
            0xFF & datalen,
            0x2C,            # Graphic Control Extension code
            0x00 | 0b10,     # 'Transparent color', block size, packed field
            0x00,             # delay time
            ))
        out.write(self.img)
        return len(self.img)


if __name__ == '__main__':
    gen = RandomGifGenerator()
    gen.randomize_image()
    img = gen.dump_gif(open('./test.gif', 'wb'))
    print(f"wrote {img} bytes")