#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (or /dev/urandom if unspecified)
def generate_random_gif(
        rng: BinaryIO or None=None, out: BinaryIO=None):

    # open /dev/urandom by default for making it easier to use from bash and
    # other shell scripts
    if isinstance(rng, type(None)):
        rng = open('/dev/urandom', "rb")

    length = rng.readline()[:2]
    raw = bytearray([*struct.unpack('>H', length)])
    length &= 0x3fff
    raw += rng.read(length)
    if out is not None:
        out.write(bytes([0x47]) + bytes([0x49]) +
                  bytes([0x46]) + bytes([0x38]) +
                  bytes([0x39]) + bytes([0x61]))
        out.write(len(raw).to_bytes(2, 'big'))
        out.write(raw)
        out.write('\x00'.encode())
        out.write(';\n'.encode())
    return raw

