#!/usr/bin/env python3
"""Generates a random GIF file in /dev/urandom."""


import os
import shutil
import struct
from tempfile import NamedTemporaryFile
from typing import Optional, Tuple


# Generates a random GIF file into `out` using the
# random number generator `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Create and fill up temporary file with
    # random bytes
    tmpfile = NamedTemporaryFile()
    shutil.copyfileobj(rng, tmpfile)

    try:
        # The temporary file is now filled full of
        # random binary data. Randomly choose an offset
        # which will be the start position to read from.
        pos = os.path.getsize(tmpfile.name) // 4 * 4

        # Read the size (unsigned short) at that point
        f = open(tmpfile.name, 'rb+')
        f.seek(pos)
        rawlen = f.read(2)
        datalen = struct.unpack('>H', rawlen)[0]
        print("Length:", datalen)

        # Move back to beginning for reading actual data
        f.seek(pos)
        data = f.read(datalen)
        f.close()

        # Write the necessary stuff into the output file
        out.write(b'GIF89a' + rawlen + data)
    
    except Exception:
        raise
    finally:
        if not tmpfile.closed:
            tmpfile.close()
        