#! /usr/bin/python3
import os
import sys
import time
import argparse

import numpy as np

from PIL import Image


def gen_random_image(size=(512, 512)):
    img = np.zeros((size[0], size[1],
                3), dtype="uint8")
    x, y = np.ogrid[:img.shape[0], :img.shape[1]]
    mask = (x - img.shape[0] // 2) ** 2 + \
           (y - img.shape[1] // 2) ** 2 > (img.shape[0] // 2) ** 2
    img[:, :, 0][mask] = 255
    return Image.fromarray(img)


def save_image_to_file(im:Image, path):
    im.save(path)


def get_argparser():
    parser = argparse.ArgumentParser(description='Generate random gifs')
    parser.add_argument('-s', '--size',
                        nargs=2, type=int, default=[WIDTH, HEIGHT], metavar=('width', 'height'),
                        help='The width and height of each image in pixels')
    parser.add_argument('--output', '-o',
                        dest='output', type=str, required=False, default=None,
                        help='Where to store the generated images.')
    return parser


if __name__ == '__main__':
    args = get_argparser().parse_args()

    WIDTH = args.size[0]
    HEIGHT = args.size[1]

    num_images = 1e6
    #num_images = 10

    if args.output is None:
        base_dir = './gen_img/'
    else:
        base_dir = '/'.join(os.path.realpath(__file__).split('/')[:-1]+['gen_img'])

    if not os.path.isdir(base_dir):
        os.mkdir(base_dir)
    elif input(f"File '{base_dir}' exists! Continue? (y/[n]): ")!= "y":
        exit(1)



    im_paths = [f'{base_dir}/{i}.png'
                for i in range(int(num_images))]

    for i, p in enumerate(im_paths):
        print(p)
        try:
            im = gen_random_image([HEIGHT, WIDTH])

            save_image_to_file(im, p)
            print('\tDone!', flush=True)
        except Exception as e:
            print(f'\tError saving image {i+1}: "'
                  f"{type(e)}({e})".replace("\n","")+'"',flush=True)
            pass