#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    width = int.from_bytes(
            rng.read(1), byteorder='big') & 0x7f
    height = int.from_bytes(
            rng.read(1), byteorder='big') & 0xff

    if width == 0:
        width += 1
    if height == 0:
        height += 1

    globalcolorinfosize = rng.read(1)[0] & 0x07

    gctflag = bool(rng.read(1)[0]>>7)

    localcolorrecordsnumber = \
           (globalcolorinfosize+1)<<bool((rng.read(1)[0]>>7))

    sortedcolor = [None]*(localcolorrecordsnumber+1)

    for i in range(localcolorrecordsnumber+1):

        sortedcolor[i] = [
            int.from_bytes(
                rng.read(1), byteorder='big'),
            int.from_bytes(
                rng.read(1), byteorder='big'),
            int.from_bytes(
                rng.read(1), byteorder='big')]


    size = int.from_bytes(
            rng.read(4), byteorder='big') 
    maxsize = len(sortedcolor)*\
             (globalcolorinfosize+3)+4

    while True:
        blocktype = rng.read(1)[0] & 0xf
        datasize = int.from_bytes(
                rng.read(1),byteorder='big')

        if blocktype==0x2c or blocktype==0x21:
            
            out.write(\
                   struct.pack('>B',blocktype))

            out.write(struct.pack('>HBBBBB',
                        width,height,\
                        globalcolorinfosize,\
                        sortedcolor[0][0],\
                        sortedcolor[0][1],\
                        sortedcolor[0][2]))
            s = ""
            for j in range(1,len(sortedcolor)):

                out.write(\
                       struct.pack('>BBBBBBB',
                           sortedcolor[j][0],\
                           sortedcolor[j][1],\
                           sortedcolor[j][2],\
                           0,0,0))

            out.write(\
                   struct.pack('>B',datasize))

            out.write(
                    rng.read(datasize))
        
        elif blocktype==0x2d:
            break
