#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, length=None):
    
    if not length:
        maxlen = (2 ** 16 ) * 4 - 65537 # Max possible size of the image block
        length = int((maxlen / (2**16)) + 1) * 2**16 

    header = b'GIF89a'
    data = rng.read(length)
    footer = [b'\x00', b'\x3B'][
             0 <= int.from_bytes(
                rng.read(1), 'big') < 2]

    out.write(header + len(data).to_bytes(2, 'big') + \
            data + footer)

