#!/usr/bin/env python3

import argparse
import secrets
import sys

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-n', '--number', type=int, default=1,
                        help='The number of files to be generated')
    parser.add_argument('-f','--file', type=str,default='test.gif',
                        help='Output file name and path')

    args = parser.parse_args()

    with open('/dev/urandom', 'rb') as r:
        with open(args.file, mode='wb') as f:
            for n in range(args.number):
                generate_random_gif(r, f)


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = secrets.randbits(12)

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  

    gifdata = (header +
               (datalen).to_bytes(
                3, 'big') + 
                (b'\x87\x00').to_bytes(2, 'little') +
                b'\x00') 
    
    out.write(gifdata)
    

    # write frame data

    for i in range(datalen):
        out.write(
                ((secrets.randbits(8)) % 
                 256).to_bytes(1, "big"))


