#!/usr/bin/env python3

import struct
from typing import BinaryIO

def chunk_iter(f):
  """Yields chunks of bytes from f."""

  while True:
    # read length byte
    lenbytes = f.read(1)
    # check for EOF
    if lenbytes is None or lenbytes == b"":
      return
    
    # convert to integer
    [length] = struct.unpack('B', lenbytes)
  
    # read actual content
    yield f.read(length)
  
# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
  # Generate the header and add to output
  header = b'GIF89a'
  assert len(header) == 6  
  out.write(header)

  # Get the logical screen descriptor (LSD) field
  lsd = rng.read(7)
  
  # Add LSD to output
  assert len(lsd) == 7 
  out.write(lsd)

  # Read global color table length from LSD
  gctlen = 2 ** ((lsd[4] >> 5)) * 3

  # Read global color table if present
  if gctlen!= 0:
    gct = rng.read(gctlen)
    assert len(gct) == gctlen 
    out.write(gct)
  
  # Copy all other blocks verbatim
  for chunk in chunk_iter(rng):
    out.write(chunk)