#!/usr/bin/env python3

import argparse

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# following format:
#   1. 4 bytes of GIF magic number "GIF89a".
#   2. 2 bytes of length of block (N).
#   3. N bytes of block.
def generate_random_gif(
        args: argparse.Namespace, rng: BinaryIO, out: BinaryIO
    ):
    if not args.no_magic:
        out.write(b'GIF89a')

    for i in range(args.block):
        datalen = int.from_bytes(
                rng.read(2), byteorder='little', signed=False
        )

        while datalen > 0x7fff:
            out.write((0x7fff).to_bytes(2, 'little'))
            out.write(rng.read(0x7fff))

            datalen -= 0x7fff

        if datalen == 0:
            continue

        out.write(datalen.to_bytes(2, 'little'))
        out.write(rng.read(datalen))