#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and all bytes preceding the first descriptor
    print('Writing the header...', end='', flush=True)
    out.write(header + data[:10])
    print('done.')


    # Now we're going to write 256 descriptors at once
    # Using this loop means we avoid making multiple calls to read()
    out.write(struct.pack('<H', 0x2c))
    padding = 0
    for i in range(256):

        # The label says 10 but there are actually two more
        # bytes after the label indicating the length of
        # the block. We know it's always >10 so just set it to 0x10
        # (https://www.w3.org/Graphics/GIF/spec-gif89a.txt)
        out.write(struct.pack('<H', 0x10))
        offset = out.tell()+1
        out.seek(-offset, 1)
        out.write(rng.read(0x10))
        out.seek(offset)

        padding += 14
        # Write the last descriptor!
        if i==255: out.write(b'\x3B')

    # Pad up until the end of the file, which is easy enough because
    # we've already written exactly datalen bytes before ending.
    print('Adding some padding...')
    while True:
        if out.tell() >= datalen+padding: break
        out.write(rng.read(1))

    print('Done!')