#!/usr/bin/env python3

import os
import argparse
import tempfile
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    with tempfile.TemporaryFile() as tempf:
        while True:
            chunklen = rng.read(2)
            if not chunklen or bytearray(chunklen)[-1] == 0: break
            tempf.write(int.from_bytes(
                    chunklen, 'big').to_bytes(
                        4, 'little'))
            tempf.write(rng.read())

        footer = [b'\x00', b'\x3B'][
                0 <= int.from_bytes(
                    rng.read(1), 'big') < 6]
        tempf.seek(0)

        out.write(header)
        for chunk in iter(lambda :tempf.read(512 * 1024), b''):
            out.write(len(chunk).to_bytes(2, 'big'))
            out.write(chunk)
        out.write(footer)



if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Generate a random GIF.')

    parser.add_argument('-o', '--output', default=os.devnull,
                        help='The output file (default is /dev/null)')
    args = parser.parse_args();

    f: BinaryIO = open(args.output, mode='wb+')


    try:
        generate_random_gif(open('/dev/urandom'), f)
    except Exception as e:
        print('Failed to generate a random gif:', str(e))
    finally:
        f.close()