#!/usr/bin/env python3

import argparse
import os
import sys


def get_args() -> dict:
    parser = argparse.ArgumentParser(
        description="generate random gif files.")

    parser.add_argument("-o", "--output",
                        help="[optional] output base filename "
                             "(no extension)", default="")
    parser.add_argument("count", metavar="N", type=int, nargs="+",
                        help="number of images to create")
    args = parser.parse_args()
    return vars(args)


def generate_random_file(args: dict):
    count = min(*args["count"])
    for i in range(count):
        if not args["output"]:
            f = open(f"{i}.gif", 'wb')
        else:
            f = open(f"{args['output']}.{str(i)}.gif", 'wb')

        with open("/dev/urandom", "rb") as r:
            generate_random_gif(r, f)


if __name__ == "__main__":
    arguments = get_args()
    # make sure we're running on linux or whatnot
    try:
        if not hasattr(os, "sysconf"):  # pragma: no cover
            raise NotImplementedError("unsupported platform")

        maxfd = os.sysconf("SC_OPEN_MAX")
    except (AttributeError, ValueError, OSError, NotImplementedError):  # pragma: no cover
        maxfd = 1024

    original_fds = []
    for fd in list(range(maxfd)):
        try:
            flags = fcntl.fcntl(fd, fcntl.F_GETFD)
        except IOError:  # pragma: no cover
            continue
        original_fds.append((fd, flags))

    try:
        generate_random_file(arguments)
    finally:
        for fd, flags in original_fds:  # pragma: no cover
            try:
                fcntl.fcntl(fd, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)
            except IOError:
                pass