#!/usr/bin/python3
"""Generate invalid GIF files."""
import random
from typing import BinaryIO


# This function generates a GIF file that contains an invalid header. It does
# this by reading the entire header from one source, then reading the image
# data from another source. Both header and image data
# are read from different sources to ensure that they're valid.
def generate_random_gif(rng: BinaryIO, out: BinaryIO) -> None:
    
    # Header
    header = b''
    header += b'GIF89a'            # File signature and version
    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
