#!/usr/bin/env python3

import struct
import socket
from ipaddress import ip_network, IPv4Network

import pytest

def test_ip6tables():
    assert "IP6TABLES_EXECUTABLE" in os.environ

def test_ipv4():
    # TODO: see if this is needed at all
    class Ipv4Address(IPv4Network):
        """
        A subclass of IPv4Network that doesn't accept an address-only
        network (e.g., 192.0.2.0).

        The first value is the start of the range; the second value is
        the broadcast address.
        """
        def __new__(cls, *args, **kwds):
            if len(args) > 0 and not args[0][0].isdigit():
                raise ValueError("Expecting IPv4 dotted decimal address as first argument.")
            self = super().__new__(cls, *args, **kwds)
            return self

        @property
        def first(self):
            return str(self.network_address + 1)

        @property
        def last(self):
            return str(self.broadcast_address - 1)

    net = Ipv4Address('192.0.2.0/28')

    print(net.first)

@pytest.mark.parametrize("target", ["::1", "localhost"])
def test_tcp(target):
    sock = socket.socket()
    sock.connect((target, 80))
    sock.sendall(b"GET / HTTP/1.0\r\nHost: localhost\r\n\r\n")
    while True:
        data = sock.recv(4096)
        if not data: break
        print(data.decode(), end="")
    sock.close()
