#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    header = b"GIF89a"
    width = 0xFF & struct.unpack("<B", rng.read(1))[0]
    height = 0xFF & struct.unpack("<B", rng.read(1))[0]
    backgroundColorIndex = 0xFF & struct.unpack(">H", rng.read(2))[0]
    pixelAspectRatio = 0xFF & struct.unpack(">H", rng.read(2))[0]
    
    # Logical screen descriptor
    out.write(struct.pack('<BBHH', width, height, backgroundColorIndex, pixelAspectRatio))
    print(width, height)

    # Graphic control extension
    graphicControlExtensionType = 0xF9
    graphicControlExtensionSize = 4
    packedFields = 0b00000001 if backgroundColorIndex == 0 else 0
    delayTime = 0xFFFF & struct.unpack(">H", rng.read(2))[0]
    transparentColorIndex = 0xFF & struct.unpack(">h", rng.read(2))[0]

    # Extension block
    out.write(
            struct.pack('<BBxxBBHH',
                    graphicControlExtensionType,
                    graphicControlExtensionSize,
                    packedFields,
                    delayTime,
                    transparentColorIndex
                )
             )
    
    # Image descriptor
    imageDescriptorType = 0x2C
    leftOffset = 0xFFFF & struct.unpack(">H", rng.read(2))[0]
    topOffset = 0xFFFF & struct.unpack(">H", rng.read(2))[0]
    width = 0xFF & struct.unpack("B", rng.read(1))[0] + 1
    height = 0xFF & struct.unpack("B", rng.read(1))[0] + 1
    localColorTableFlag = not (backgroundColorIndex != 0 or colorResolutionDepth != 8)
    interlaceFlag = False
    sortFlag = True
    localColorTableSize = 7
    tableBackgroundColorIndex = backgroundColorIndex

    # TODO Figure out how to use this
    # packedFields = (
    #     0 |
    #     (localColorTableFlag << 7) |
    #     (interlaceFlag << 6) |
    #     (sortFlag << 5) |
    #     localColorTableSize
    # )
    out.write(struct.pack('<BBHHBBB',
            imageDescriptorType,
            leftOffset,
            topOffset,
            width,
            height,
            tableBackgroundColorIndex,
            0       # packed fields
        ))

    # Local color table
    for i in range(int((2 ** (localColorTableSize+1)))):
        red = 0xFF & struct.unpack(">H", rng.read(2))[0]
        green = 0xFF & struct.unpack(">H", rng.read(2))[0]
        blue = 0xFF & struct.unpack(">H", rng.read(2))[0]
        
        out.write(struct.pack("<BBBB", red, green, blue, 0))

    # LZW minimum code size
    lzwMinimumCodeSize = 8
    out.write(struct.pack("B", lzwMinimumCodeSize))

    # Read in data
    while True:
        data = rng.read()
        if len(data) <= 0:
            break
        out.write(header + data)
