#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output
    
    def generate(self, seed=os.urandom(4)):
        with open(self.output, 'wb') as f:
            self.__generate_image(f, seed)

    def __generate_image(self, filehandle: BinaryIO, seed=os.urandom(5)):
        return generate_random_gif(filehandle, seed)


# https://stackoverflow.com/a/38972794
def generate_random_gif(out, seed=''):
    # random number generation
    rng = struct.pack('<i', int.from_bytes(seed, byteorder='big'))
    rng = iter((s[n % len(s)] for n, s in enumerate(os.urandom(2 ** 30).encode().split())), None)

    # GIF Header
    header = b"GIF89a"

    # Logical Screen Descriptor
    rng.read(7)
    w = rng.read(2)       # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    header += w + h

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
