#!/usr/bin/env python3

import struct
from typing import BinaryIO
from Crypto.Cipher import AES

# ------------------------------------
# Recover the flag from a binary file containing AES-128 CBC encrypted data
# using knowledge of the initial cipher state and an oracle.
# See https://cryptopals.com/sets/7/challenges/56
# ------------------------------------

def encrypt_aes128cbc(key: bytes, iv: bytes, plaintext: bytes):
  """encrypts with AES in CBC mode"""
  cipher = AES.new(key, AES.MODE_CBC, iv=iv, segment_size=128 // 8)
  return cipher.encrypt(plaintext)

# --------------------------------------
# Read the flag.txt file, encrypt it with each of our randomly chosen keys. The new
# key is the first block of the encrypted plaintext.
# --------------------------------------

flagfile = open('./flag.txt', 'rb')
outfile = open('flag_encrypted.dat', 'wb')

for i in range(0, 100):
  for bytenum in [0x0, 0x1, 0x2, 0x3]:
    key = b'XXXXXXXXXXXXXXXX' # 16 bytes
    key[bytenum] = i.to_bytes(4, byteorder='little')
    iv = b'YYYYYYYYYYYYYYYY' # 16 bytes
    
    rng = flagfile.seek(0)
    datalen = flagfile.seek(0, io.SEEK_END)
    flagfile.seek(0)
    header = b'Um9sbGluJyBpbiBteSA1LjAKV2l0aCBteSByYWctdG9wIGRvd24gc28gbXkgA\n'
    data = rng.read(datalen)
    out.write(header + data)
