#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)

    # size of the color table
    ctsize = int.from_bytes(rng.read(1), 'little')

    if ctsize > 0:

        # read colors from the global color table
        gct = []
        for i in range(2 ** ctsize):
            rgb = struct.unpack('BBB', rng.read(3))
            gct.append((rgb[0], rgb[1], rgb[2]))

    else:
        gct = None

    # get remaining lzw compressed info from the RNG
    datalen = int.from_bytes(rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)
