
def extract_image(inpath, outfile):
    
    import random
    import struct

    # open the rom for reading
    fh = open(inpath, "rb")
    rng = random.Random(42352897)
    rng.seed(fh.read())

    # setup the file header
    header = b"\xff\x2c"

    # get the size of the decompressed image
    imsize = int.from_bytes(rng.read(2), 'big')
    print("Size of uncompressed image is ",imsize)

    # read the start position of the image data
    imgaddr = int.from_bytes(rng.read(4), 'little')

    # jump to the beginning of the image data section and read the width/height
    fh.seek(imgaddr)
    image_width = int.from_bytes(fh.read(2),'big')
    image_height = int.from_bytes(fh.read(2),'big')
    print("Image size is ",image_width," x ",image_height)

    # open a new file for writing
    out = open(outfile,'wb')

    # has a palette?
    palsiz = int.from_bytes(fh.read(1),'big')
    if palsiz != 0:

        # read the size of the palette
        ctsize = int.from_bytes(fh.read(1),'big')
        print ("Size of palette entries is ", ctsize)
        
        # read all of the palettes
        paldata = []
        while True:
            palent = fh.read(ctsize+1)
            if not palent or len(palent) == 0: break
            paldata.append(palent)
        print (len(paldata))

        palents = bytes()
        for i in range(len(paldata)):

            # write out the length byte first
            palents += struct.pack('B', len(paldata[i]))
            palents += paldata[i]

        # write to output stream
        out.write(header + struct.pack('<HHBB', image_width, 
            image_height, ctsize | 0b10000000, 
            0) + palents)

    else:

        # no palette
        out.write(header + struct.pack('<HH', image_width, 
                image_height))

    # get remaining lzw compressed info from the RNG
    datalen = int.from_bytes(rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)
