
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Read in riff header
    assert rng.read(4) == b'RIFF', 'missing RIFF header'
    length = int.from_bytes(rng.read(4), byteorder='little') - 8

    # Read in webp header and skip it
    assert rng.read(12) == b'\x00\x00\x00\x0cWEBPVP8X', 'missing WEBP header'
    width = int.from_bytes(rng.read(4), byteorder='little')
    height = int.from_bytes(rng.read(4), byteorder='little')
    framerate = int.from_bytes(rng.read(4), byteorder='little')
    loop_count = int.from_bytes(rng.read(4), byteorder='little')
    assert loop_count < 65536, 'loop count is too high'
    color_space = int.from_bytes(rng.read(4), byteorder='little')

    # Write image data ...
    while length > 0:
        chunk_id = rng.read(4).decode('ascii').strip()
        chunk_length = int.from_bytes(rng.read(4), byteorder='little')
        if chunk_id == 'ANIM':
            # Read animation chunks until we find one that contains image frames
            while chunk_length != 0:
                subchunk_id = rng.read(4).decode('ascii').strip()
                subchunk_length = int.from_bytes(rng.read(4), byteorder='little')
                if subchunk_id == 'FRAME':
                    break
                else:
                    rng.read(subchunk_length)

            # Now get our raw frame data
            uncompressed_size = int.from_bytes(rng.read(4), byteorder='little')
            compressed_data = rng.read(chunk_length - 12)
            decompressed_size = int.from_bytes(rng.read(4), byteorder='little')
            decoded_data = zlib.decompress(compressed_data)

            # Write the header for this frame
            out.write(b'GIF97a')
            out.write(width.to_bytes(2, byteorder='big'))
            out.write(height.to_bytes(2, byteorder='big'))
            out.write('\x87'.encode())

            # Write the image data itself
            out.write(decoded_data