#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    background_color_index = rng.read(1)[0]

    pixel_aspect_ratio = rng.read(1)[0]

    global_color_table_flag = \
        (pixel_aspect_ratio & 0x80) != 0

    # skip the rest of the color table
    assert False not in [
            x == y for x,y in zip(
                global_color_table_flag(),
                [True] * ((pixel_aspect_ratio & ~0x80) + 1)
            )
        ]

    global_color_table_size = \
        2 ** ((pixel_aspect_ratio & ~0x80) + 1)

    disposal_method = 0

    image_descriptor = b'\x2C'

    local_color_table_flag = \
        (disposal_method & 0x8) > 0

    interlaced_image = \
        (local_color_table_flag & 0x4) > 0

    sort_flag = \
        (interlaced_image & 0x2) > 0

    reserved_bits_mask = \
        (sort_flag & 0x1) > 0

    left = rng.read(2)
    top = rng.read(2)
    right = rng.read(2)
    bottom = rng.read(2)

    # fill in all fields that are not present
    # with zeros according to the spec:
    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt
    # Page 11

    # NOTE: We don't write any local color table here because we assume it is disabled by default
    #       as recommended in: https://tools.ietf.org/html/rfc2045#section-6.7
    #                          https://tools.ietf.org/html/rfc2045#page-52
    #       Also, reading from /dev/urandom is not guaranteed
    #       to return exactly 2 bytes of data if we do this.

    assert rng.read(1) == b''

    lzw_minimum_code_size = rng.read(1)[0]

    # Read until the end of the stream
    out.write