
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Should be a power of two. We're not interested in generating valid GIFs.
    width = 512
    height = 300
    
    # Magic header byte sequence
    out.write(b"GIF89a")

    # Width and height
    write_int(width, out)
    write_int(height, out)

    # Background color (black)
    out.write(b"\x00\x00\x00")

    # Field size (we don't care about transparency, so leave it at zero)
    out.write(b'\x00')

    # Graphic control extension; transparent color index is zero, delay time is 4, no disposal method
    out.write(b"\x21\xf9\x04\x00\x00\xff\x00")

    # Image descriptor block
    out.write(b"\x2c\x00\x00\x00\x00")
    write_int(width, out)
    write_int(height, out)
    out.write(b"\x00")

    for y in range(height):
        for x in range(width):
            # Read one byte from /dev/urandom and check if it's greater than 127
            byte = int.from_bytes(rng.read(1), 'big', signed=False