#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    out.write("GIF89a")

    # Logical screen descriptor
    bits = int(rng.read(1).hex(), 16) & 0x7
    xres = int(rng.read(2).hex(), 16)
    yres = int(rng.read(2).hex(), 16)
    background = int(rng.read(1), 16)
    pixel_aspect = int(rng.read(1).hex(), 16) & 0xF
    out.write(struct.pack('<BBB', bits, xres, yres))
    
    if len(background) == 1:
        out.write(bytes([background[0] | 0x80]))
    else:
        out.write(background)
        
    out.write(b'\x00')

    # Global colour table (if present)
    if bits != 0:
        colours = [rng.read(3) for _ in range((bits + 1) // 2)]
        colours = bytearray().join(colours)
        out.write(colours[:2 ** ((bits + 1) // 2) * 3])

    # Application extension
    out.write(b"\x21\xFF\x0BNETSCAPE2.0\x03\x01")
    loopcount = b'\x00'
    out.write(loopcount)

    # Pixel data
    while True:
        code = ord(rng.read(1).decode())
        if code < 0x80:
            out.write(bytearray([code]))
        elif code <= 0xC7:
            length = code - 0x80 + 1
            old_pixels = []
            for i in range(length):
                color_index = ord(rng.read(1).decode())
                old_pixels.append(color_index)
            out.write(old_pixels)
        elif code == 0xC9:
            break
        else:
