#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# provided RNG. The size of the generated image is determined by
# the first four bytes read from the RNG (which are discarded).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False
    ) & 0xffffff
    data = rng.read(datalen)

    # Write a complete GIF89a header
    out.write(header)
    out.write(struct.pack('<H', 6)) # Logical Screen Descriptor
    out.write(b'\x1f\x00') # Size of global color table
    out.write(b'\x80\x00') # Color descriptor
    out.write(b'\x00') # Background color index
    out.write(b'\x00') # Pixel aspect ratio
    out.write(b'\x2c\x00\x00\x00') # Image Descriptor
    out.write(b'\x00\x00\x00\x00') # Width and height as a 16-bit little endian value
    out.write(b'\x00') # Used local color table?
    out.write(b'\x07') # Local color table size

    # Loop over each pixel and write it to the output GIF
    for i in range(256**3):
        out.write(data[i % len(data)])
