#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
    
        with open(f'./{self.output}', 'wb') as fh:
            generate_random_gif(open('/dev/urandom'), fh)

def generate_random_gif(infile:BinaryIO, outfile:BinaryIO):

    # Generate the GIF header
    width = 100 + (ord(infile.read(1)) % 50)
    height = 100 + (ord(infile.read(1)) % 50)

    header = "GIF89a"                                \
        + chr(width) + chr(width & 255)              \
        + chr(height) + chr(height & 255)            \
        + "\x87\x00\x00\xff\x0bNETSCAPE2.0\x03\x01"   \
        + "\x00;"
    
    # Write the header
    outfile.write(header.encode('ascii'))

    for i in range(300):
        
        # Create a palette color entry
        r = ord(infile.read(1)) % 256
        g = ord(infile.read(1)) % 256
        b = ord(infile.read(1)) % 256
        entry = struct.pack("BBB", r, g, b)

        # Add an image descriptor
        desc = "\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00"

        # Write the image
        outfile.write(desc.encode('ASCII'))
        outfile.write(entry)

        # Write the image pixel data
        pixels = ""
        for y in range(height):
            pixels += infile.read((width+7)/8)
            
        outfile.write(pixels.encode('raw-unicode-escape'))

        # Write the terminator
        outfile.write("\x00".encode('ascii'))

