#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, global color table length,
    # background color index, logical screen descriptor terminator)
    rng.seek(7, 1)

    # get a palette identifier byte
    palette = next(rng)

    if palette < 128:
        
        # has a palette!
        entries = 2**palette

        # read the palette data
        rng.seek(-1, 1)
        palents = rng.read((entries * 3) + 1)

        # write a GIF with a palette to `out`
        out.write(header + struct.pack('<HHB', image_width,
            image_height, palette) + b'\x00' +
            palents[:-1])

        # then add another image block
        # it will use the same palette
        out.write(struct.pack('<BBHBB', 0x2c, 0, 
                image_width, image_height, 
            0) + palents)

    else:

        # no palette
        out.write(header + struct.pack('<HH', image_width, 
                image_height))

    # get remaining lzw compressed info from the RNG
    datalen = int.from_bytes(rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)
