#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=160, height=90) -> None:
    # First 6 bytes are magic header
    out.write(b'GIF87a')

    # Next 4 bytes encode width and height
    out.write(struct.pack('<HH', width, height))

    # Note that we're not actually doing anything meaningful here yet
    # just stream of bytes.

    # Finally 1 byte is color table size
    if (width * height <= 256):
        out.write(struct.pack('B', 0x80 | (width * height)))
    else:
        raise Exception("Too Many Pixels!")

    for i in range(width*height):
        out.write(rng.read(1))
