#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# provided RNG. The size of the generated image is determined by
# the first four bytes read from the RNG (which are discarded).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO) -> None:
    # Read width and height of output image
    wh = struct.unpack("<I", rng.read(4))[0]
    width = (wh >> 16) & 0xFFFF
    height = wh & 0xFFFF

    # Write out the GIF header
    out.write("GIF89a".encode())     # Magic header
    out.write(struct.pack("<HHHBBB", width, height, 0, 0, 0, 0, 0)) # Logical screen size
    out.write(b'\xFF\x0BNETSCAPE2.0\x03\x01')

    # Start first image
    out.write(struct.pack('<BBHH', 0x2C, 0, 0, 0))
    out.write(struct.pack("<HHHHHH", width, height, 0, 0, 0, 0))

    # Write LZW table header; all color codes are unique
    out.write(b'\x00\x00')
    out.write(b'\x00')

    # Write out EOF code
    out.write(b'\x00\x00')

def main() -> None:
    with open("/dev/urandom") as r