#!/usr/bin/env python3

import struct

from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate a random GIF file.
    # 179 bytes is the minimum size of a valid GIF file.

    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt
    out.write(b'GIF89a')

    width = 1 + (rng.read(1)[0] % 64) * 2 + 1
    height = 1 + (rng.read(1)[0] % 64) * 2 + 1
    color_table_size = min(2 ** ((rng.read(1)[0] >> 4) & 0x07), 1 << 4)

    out.write((5, 0))  # Block size (1 entry in global color table).
    out.write(struct.pack('<BBB', width, height, 0xF7))
    out.write(bytes([0xFF for _ in range(color_table_size)]))  # Color table.

    out.write(b'\x2C\x00\x00\x00\x00')  # Image block size.
    out.write(b'\x00\x00\