#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self._outfile = DEFAULT_OUTPUT
        else:
            self._outfile = output

    @property
    def outfile(self) -> str:
        return self._outfile

    def write_to_file(self, file: BinaryIO):
        
        # Beginning of a GIF87a header
        file.write(b'\x47\x49\x46\x38\x37\x61')
        
        # Logical Screen Width and Height (in pixels). 
        # The size of the image is determined by these values
        width = 250
        height = 250
        file.write(struct.pack('<HH', width, height))

        # The background color index. Color 0 is always the background. 
        # This byte is ignored because there are no color map entries
        file.write(struct.pack('B', 0))

        # Indicates the aspect ratio of the pixel grid in the image. 
        # When this field is 0, it indicates that the pixel dimensions 
        # of the image are square
        file.write(b'\x00')

        # Background Color Index
        file.write(b'\x00')

        # No Global Color Map
        file.write(b'\xff')

        for i in range(width * height):
            
            # Image Descriptor
            file.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

            # Local Color Table Flag. 0 is no local color table present
            file.write(b'\x00')

            # Interlace Flag. 0 is non-interlaced
            file.write(b'\x00')

            # Sort Flag. 0 = unsorted
            file.write(b'\x00')

            # Size of Local Color Table. It must be 2^(n+1), 
            # where n is the value in the above Field
            file.write(b'\x00')

            # The number of bits per primary color:  
            # 0=2, 1=4, 2=8, 3=16
            file.write(b'\x02')

            # X location of the upper left corner of the bounding rectangle (=0)
            file.write(struct.pack('<H', 0))

            # Y location of the upper left corner of the bounding rectangle (=0)
            file.write(struct.pack('<H', 0))

            # Width of the bounding rectangle (=250)
            file.write(struct.pack('<H', width))

            # Height of the bounding rectangle (=250)
            file.write(struct.pack('<H', height))

            # LZW Minimum Code Size
            file.write(b'\x02')

            # Write random data to fill up the rest of the frame
            file.write(os.urandom((width*height)))

            # Sub Block Terminator
            file.write(b'\x00')

            # Trailer block
            file.write(b';')

        # Trailer
        file.write(b'!\x00')


def main() -> int:
    
    try:

        output = sys.argv[1]

        with open(output