#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )



# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# are read from different sources to ensure that they're valid.
def generate_random_gif(rng: BinaryIO, out: BinaryIO) -> None:
    
    # Header
    header = b''
    for _ in range(6):
        header += rng.read(1)

    assert header == b'\x47\x49\x46\x38\x39\x61\x00' or \
           header == b'\x47\x49\x46\x38\x37\x61\x00', \
          "Invalid data passed."

    # Logical Screen Descriptor
    lsd = bytearray(7)
    for i in [2, 3]:
        lsd[i] = rng.read(1)[0]
    for i in [4, 6]:
        lsd[i] = rng.read(1)[0] & ~2

    # Global Color Table
    gct = bytearray()
    while True:
        ct_size = (lsd[5] >> 5) + 1
        if ct_size == 2:
            break;

        for i in range(3*2**(ct_size+1)):
            gct.append(rng.read(1)[0])

    # Application Extension Block
    # TODO: This could be improved by reading more bytes than necessary and then discarding them.
    ident = []
    while len(ident) < 12:
        ident.extend([ord(rng.read(1))])
    block = b'\xaae\xc6\x1c\xbf\x77\xbd\x0e\xf7' + struct.pack('<L', 4) + \
              b'\xff\xfe\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00' + \
              struct.pack('<L', 4) + b'\x1f\xe2\x0a\x33\xde\xed\x37\x18'

    # Image Data
    image = bytearray((lsd[2], lsd[3]))
    image.append(lsd[4]);
    color_index = rng.read(1)[0] % (2**(3*(1+(lsd[5]>>7)+1)))
    image.append(color_index)
    image.extend([rng.read(1)[0]]*(image[0]*image[1]))

    # Trailer

    # Output everything now
    out.write(b'GIF89a')
    out.write(bytes(lsd))
    out.write(bytes(gct))
    out.write(block)
    out.write(struct.pack('B', len(image)))
    out.write(bytes(image))