#!/usr/bin/env python3

import struct

from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = bytearray([0x47, 0x49, 0x46])      # GIF87a
    version = b'87a'                            # GIF87a
    width = 256
    height = 256
    size = (width, height)
    background = 0
    pixel_aspect = 0
    global_color_table = False
    resolution = 0b10000000 | (7 - pixel_aspect) << 7 | \
        7 - pixel_aspect
    sort_flag = True
    table_size = 7
    color_table = None

    def write_word(value):
        out.write(struct.pack('!H', value))

    def write_byte(value):
        out.write(struct.pack('!B', value))

    def write_bytes(values):
        for value in values:
            write_byte(value)

    def write_blocks(*blocks):
        for block in blocks:
            write_byte(len(block))
            write_bytes(block)

    def write_image_descriptor():
        out.seek(13 + 16 * 3)
        write_word(0)
        write_word(0)
        write_word(width)
        write_word(height)
        flags = 0b10000000 if not global_color_table else 0
        if color_table is not None:
            flags |= len(color_table[1]) & 0b111
        write_byte(flags)

    def write_logical_screen_descriptor():
        write_word(width)
        write_word(height)
        write_byte(global_color_table)
        write_byte(resolution)
        write_byte(background)
        write_byte(0)
        if color_table is not None:
            write_bytes(color_table[0])
        else:
            write_words(range((table_size + 1) ** 3))

    def write_graphic_control_extension():
        write_word(0b100000000000000)
        write_word(delay)
        write_byte(transparent)

    def write_application_extension():
        write_blocks(b'NETSCAPE2.0')
        write_byte(3)
        write_byte(1)
        write_word(loop)

    def write_comment_extension():
        pass

    delay = rng.randrange(5, 2 ** 16 - 1)
    transparent = rng.randint(0, 2 ** 8 - 1)
    loop = rng.randint(1, 2 ** 16 - 1)

    out.write(header)
