#!/usr/bin/python3

import sys
import os
import struct
import random
from pathlib import Path

if len(sys.argv) < 3:
    print("Usage: gen_bootimage.py [infile] [outfile]")
    exit()

sfu = open(sys.argv[1], "rb")
rng = open(sys.argv[2], "rb")
out = open(sys.argv[3], "wb")

# read and validate the SFU image header
marker, version, numBlocks, flashAddr = struct.unpack("<4sHII", sfu.read(16))
assert marker == b"SFU\x01", "SFU image header invalid"
print("SFU image contains {} blocks".format(numBlocks))

for i in range(numBlocks):
    header = sfu.read(8)         # extract header without block size
    bs = header[7]               # save block size
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
