#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# format specification at https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(out, width=42, height=71) -> None:
    """Generate a random GIF image"""
    rng = open("/dev/urandom", "rb")
    header_signature = b'\x47\x49\x46'
    version =  0x38    # '8'
    background = (rng.read(1)[0] & 0b111) * 16
    flags = 0
    blocksize = 2**16 // 16    # Size of each subblock in bytes
    padding = [bytes([0]*i) for i in range(1, 256)]
    while byte := rng.read(blocksize):
        if blocksize - len(byte) > 0:
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            # Write long blocks without padding
            out.write(byte)
