#!/usr/bin/python3
import os
from configparser import ConfigParser, NoOptionError, NoSectionError
from subprocess import Popen, CalledProcessError
# from db_services import DatabaseService
import logging
import sys
import utils

DESCRIPTION = "Shutdown all containers"
CONFIG_FILE = "/data/config.ini"
COMMON_SECTION = "common"
RUNNING_CONTAINERS_SECTION = "running_containers"
LOGGER = logging.getLogger()

def parse_arguments():
    # Parsing arguments
    if len(sys.argv) != 2:
        print("Wrong number of arguments")
        exit(1)
    
    global CONFIG_FILE
    
    try:
        CONFIG_FILE = str(sys.argv[1])
    except ValueError as e:
        print("Invalid argument type for '{}'".format(e))
        exit(1)
        
def read_configuration():
    # Read configuration file
    global CONFIG_FILE
    
    config = ConfigParser()
    
    try:
        with open(CONFIG_FILE, "r") as cfgfile:
            config.readfp(cfgfile)
            
        return config
    except IOError as e:
        LOGGER.error("Configuration file {} does not exist".format(e))
        exit(1)  
      
def stop_all_containers(docker, appname):
    # Stop all running containers
    
    stopped_container_list = []
    stopped_images_list = []
    
    client = docker.client()
    
    container_list = client.containers(filters={"label": "app=" + appname})
    
    for container in container_list:
        try:
            stopped_container = client.stop(container=container["Id"], timeout=0)
            stopped_container_list.append(stopped_container)
        except Exception as e:
            LOGGER.error("Couldn't shutdown the container.")
            continue
                
        try:
            container_image = client.inspect_image(stopped_container['Image'])
            stopped_images_list.append(container_image)
        except ClientException as e:
            LOGGER.debug("Container image doesn't have a tag to remove.")
        
    LOGGER.info("All services are successfully stopped!")
       
def start_all_containers(docker, appname):
    # Start all containers
    
    client = docker.client()
    
    container_list = client.containers(filters={"label": "app=" + appname})
    
    # Recreate all running containers
    for container in container_list:
        try:
            restarted_container = client.restart(container=container["Id"], timeout=0)
            LOGGER.debug("Restarting service {}. ID {}".format(container["Names"][0], restarted_container["Id"]))
        except Exception as e:
            LOGGER.error("Couldn't start the container.")
            continue

def check_database_connection():
    database_connection = None
    
    try:
        database_connection = utils.connect_db()
    except Exception as e:
        LOGGER.error("Couldn't connect to the PostgreSql container: {}".format(e))
        exit(1)
     
    return database_connection
