#!/usr/bin/env python3
"""This program shows the plot of a given function
in a range specified by user input.
"""
# standard library modules
import argparse
import re

# third-party module(s)
import matplotlib.pyplot as plt
from sympy import *  # noqa: F401,F402,F403

parser = argparse.ArgumentParser(description='Plot a given function')
parser.add_argument('function', metavar='expression', type=str, nargs="+",
                    help='the expression to be plotted')
parser.add_argument('--range' , '-r', type=lambda x: tuple(int(v) for v in x.split(',')), default=(0, 5),
                    help='range of x coordinate (min, max). Default is 0, 5.')
parser.add_argument('--save', type=str, metavar='filename',
                    help='save figure to file')
args = parser.parse_args()

x = Symbol('x')
y = eval(' '.join([re.sub('^f\((.*?)\)$', r'\1', arg) for arg in args.function]))
plt.plot(*[
    (val.subs({x: num}), val.diff().subs({x:num}))
    for num in range(args.range[0], args.range[1])
])
plt.axis('scaled')