#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
    
        with open(f'./{self.output}', 'wb') as fh:
            generate_random_gif(open('/dev/urandom'), fh)


# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    magic = b'GIF89a'
    out.write(magic)
    # Logical screen width and height (in pixels)
    screen_width = 320
    screen_height = 160
    out.write(struct.pack('<HHB', screen_width, screen_height, 0xE))
    # Background color
    out.write(bytes([0xFF]))
    # Pixel aspect ratio
    out.write(bytes([0x7F]))

    # Application extension block
    # Contains an NTS parameter (Network Tablet System)
    out.write(bytes([0x21, 0xff, 0x0b, 0x4e, 0x54, 0x53, 0x21, 0xff, 0x00]))

    # Block terminator
    out.write(b';')

    # Graphic control extension
    # Contains one field: delay time between frames, in hundredths of seconds
    # 0 = infinite loop
    out.write(bytes([0x21, 0xf9, 0x04, 0x00, 0x09]))

    # Image descriptor
    # Top left corner x coordinate for each frame
    out.write(bytes([0x2c, 0x00, 0x00, 0x00, 0x00]))
    # Top left corner y coordinate for each frame
    out.write(bytes([screen_height & 0xff, (screen_height >> 8) & 0xff]))
    # Width and Height
    out.write(bytes([screen_width & 0xff, (screen_width >> 8) & 0xff]))
    # Local color table flag
    out.write(bytes([0]))
    # Size of local color table
    out.write(bytes([(0 << 7)]))
    # Sort flag
    out.write(bytes([0]))
    # Reserved byte
    out.write(bytes([0]))
    # Interlace flag
    out.write(bytes([0]))

    # Indexed Color Table
    indexed_color_table = [
        0xff, 0xff, 0xff,   # White
        0x00, 0x00, 0x00,   # Black
    ] + [i for i in range(255)] * 2
    out.write(bytes(indexed_color_table))

    # image data
    g = 0
    for _ in range(100):
        for _ in range(screen_width):
            g = (g + 1) % 256
            out.write(bytes([g]))

    out.write(bytes([0x3b]))


def main():
    rg = RandomG