#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    """
    Generate a GIF file using /dev/urandom for data and write it to `f`.
    :param rng: The source of entropy
    :param f: The place to write the file
    """
    # Read in a data chunk from urandom
    data = rng.read(1048576)

    # Make headers
    header = b'GIF89a'.encode('ascii') \
             + struct.pack('<H', 1) \
             + b'\xF0' \
             + b'\x00\xFF' \
             + b'\x00\xFF\xFF\xFF' \
             + b'\x00\x00\x00\x00'
    blocksize, padding = 256 * (data[1] & 0x07) + 1, [b'\x00']

    # Write header and length of image to output
    out.write(header + len(data).to_bytes(3, 'big'))
    while byte := rng.read(blocksize):
        if blocksize - len(byte) > 0:
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            # Write long blocks without padding
            out.write(byte)
