#!/usr/bin/env python3

import struct

from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'
    size = struct.pack('<HH', 400, 400)
    color_table = bytes([rng.read(1)[0] for _ in range(768)])

    # Generate a random number of frames (up to 255).
    nb_frames = rng.read(1)[0] % 255 + 1

    # Generate the contents of each frame.
    for _ in range(nb_frames):

        # Generate a random delay time.
        delay = rng.read(1)[0] % 100 if rng.read(1)[0] < 128 else 0
        delay = struct.pack('B', delay)

        # Write the GIF control block.
        data = bytearray()
        data += delay
        data.append(0x2C)             # image separator
        data += size                  # image size
        data.append(0)                # no local colour table
    out.write(header + data)
