#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    blocksize, padding = 256 * (data[1] & 0x07) + 1, [b'\x00']

    # Write header and length of image to output
    out.write(header + len(data).to_bytes(3, 'big'))

    # Write data with alternating padding bytes
    for byte in data:
        if not (len(padding) % blocksize):
            # Write short blocks with zero padding
            out.write(byte + padding[0])
        else:
            # Write long blocks without padding
            out.write(byte)
