
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(rng: Random, out: BinaryIO):
    # https://www.w3schools.com/colors/colors_picker.asp
    rng = xoroshiro128plus.XoroShiro128Plus(rng.getrandbits(64).to_bytes(8, 'little'))

    def next_color():
        r = rng.randint(0x0, 0xFFFFFF)

        if (r & 0xFF) > 0x7F:
            return ((r >> 16) & 0xFF), ((r >>  8) & 0xFF), (r & 0xFF)

        return None

    w = rng.choice([32, 64]) * 256
    h = rng.choice([32, 64]) * 256

    for y in range(h - 1, -1, -1):
        for x in range(w):
            c = next_color()

            while True:
                o = rng.randint(1, 4)

                if not c or o!= 4:
                    break

            out.write(b'\x21')
            out.write(o.to_bytes(1, 'big'))
            if c:
                out.write(b'\x00\x00\x00')
                out.write((c[0] << 16 | c[1] << 8 | c[2]).to_bytes(3, 'big'))
            else:
                out.write(b'\xFF\xFF\xFF')
            out.write(((y // 256) << 8 | (y % 256)).to_bytes(2, 'little'))
            out.write(((x // 256) << 8 | (x % 256)).to_bytes(2, 'little'))

