#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(4), byteorder='big') & 0xFFFFFF
    data = bytes([rng.read(1)[0] for _ in range(datalen)])

    # Write the header
    out.write(header)

    # Write the screen descriptor and color table
    blocklen = (5 + len(data)) % 256
    out.write(struct.pack('>HHBBB',
        0xFFFF, (blocklen + 7), 0b1111, 7, -1))

    # Write the image descriptor
    out.write(struct.pack('<HHHHHH',
        0, 0, 0, 0, datalen, 0))

    # Write all of the data blocks
    while data!= b"":
        blocklen = min(255, len(data))
        out.write((blocklen).to_bytes(
            1, 'little'))

        if type(data[0]) == str:
            out.write(data[:blocklen].encode())
            data = data[blocklen:]
        else:
            out.write(data[:blocklen])
            data = data[blocklen:]

    # Write the final trailer
    out.write(b'\x3B')

