#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# provided RNG. The size of the generated image is determined by
# the first four bytes read from the RNG (which are discarded).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False
    ) & 0xffffff
    data = rng.read(datalen)

    # This padding ensures that we will always have at least one byte in each block
    padlen = 256 - ((datalen + len(header)) % 256)
    padding = [rng.read(padlen)] if (padlen!= 0) else []

    # Write the entire file
    out.write(b''.join([header,
                       struct.pack('<I', datalen),
                       data]
                      + padding))

