#!/usr/bin/env python3

import struct
from sys import argv, exit


if len(argv)!= 6:
    print("Usage:./inject <input file> <output file> <offset> <pattern offset> <hex pattern>")
    exit()

with open(argv[1], 'rb') as f:
    data = bytearray(f.read())

out = open(argv[2], "wb")

off_str = int(argv[3], base=16)
off_pat = int(argv[4], base=16)
pat = bytes.fromhex(''.join([x for x in map(lambda s: s.replace('\\', ''), argv[5].split(','))]))

blocksize = off_pat % 8  # Pattern size needs to be aligned by 8 bytes on ARM (thumb mode)
padding = b'\x00' * max((blocksize - 4), 0)

# Reassemble the binary
for i in range(len(data)):

    if i == off_str and not all([j == '\xFF' for j in pat]):

        byte = data[i] ^ pat[(i-off_pat)//4]

        # Long blocks need a bit of special treatment that allows us to insert arbitrary stream
        if blocksize - len(byte) > 0:
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            # Write long blocks without padding
            out.write(byte)
