#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1<<24
    framesize = (datalen+5)*b'\x00'

    # Write signature and logical screen descriptor
    assert len(header)==6
    out.write(header + struct.pack('<HHHH',
            576,             # Width
            308,             # Height
            0xf7,            # Flags for global color table
            1<<4))           # Color resolution is 10-bit per channel

    # Write color table
    out.write((b'\x00'*7).join([struct.pack('BBB',
            0xff&(r^rng.read(1)[0]),
            0xff&(g^rng.read(1)[0]),
            0xff&(b^rng.read(1)[0]))
            for r in range(2**4)
            for g in range(2**4)
            for b in range(2**4)]))
    out.write(framesize[0:3])     # Write delay of zero
    out.write(framesize[3:])      # Write initial image data
    while True:                   # Repeat forever
        out.write(struct.pack("<B",
                rng.read(1)[0] ^ rng.read(1)[0]))

