
from typing import BinaryIO, Union


def read_primitive(inp: BinaryIO) -> bytes:
    """Read a `Primitive` from `inp`"""

    size = int.from_bytes(inp.read(4), "big")  # type: ignore[call-overload]
    return inp.read(size)


def write_primitive(value: bytes, outp: BinaryIO) -> None:
    """Write a `Primitive` to `outp`"""

    assert len(value) <= (2 ** 31 - 1)
    size = len(value).to_bytes(4, byteorder="big")
    outp.write(size + value)


def read_file_or_ref(inp: BinaryIO) -> FileOrRef:
    """Read a `FileOrRef` from `inp`.

    Returns `String` if the file is external or an inline `Primitive`, otherwise returns
    a `FileID` for local files."""

    tag = inp.read(1)[0]
    if not isinstance(tag, int):
        raise ValueError("Expected 8 bit integer")
    elif tag == 0:
        return String(inp.read(int.from_bytes(inp.read(4), "big")))
    else:
        return Primitive(read_primitive(inp))


def write_file_or_ref(fof: FileOrRef, out: BinaryIO) -> None:
    """Write a `FileOrRef` to `out`"""

    if fof.is_external():
        out.write(b"\x00")
        # TODO: checksums!
        length = len(fof.name()).to_bytes(4, byteorder="big")
        out.write(length + fof.name().encode())
    else:
        out.write(b"\xff")
        write_primitive(fof.data(), out)


class