#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
        ) -> None:
    header = b'GIF89a\x00'

    datalen = int.from_bytes(struct.unpack('>H', rng.read(2))[0], 'big')

    data = rng.read(datalen)
    blocksize, padding = 256 * (data[1] & 0x07) + 1, [b'\x00']

    # Write header and length of image to output
    out.write(header + len(data).to_bytes(3, 'big'))
    while byte := rng.read(blocksize):
        if blocksize - len(byte) > 0:
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            # Write long blocks without padding
            out.write(byte)
