#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = 720 # (pixels)
    height = 480 # (pixels)

    bgcol = ((rng.read(1)[0] & 3) << 6) | \
             ((rng.read(1)[0] & 3) << 4) | \
            (((rng.read(1)[0] >> 2) & 3) << 2) | \
             (rng.read(1)[0] & 3)

    out.write(header)
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))

    flags = 0b11000000 # color table size of 256

    global_ext_size = len('NETSCAPE2.0').to_bytes(1, 'big') + 3
    loop_count = int((rng.read(1)[0]) / 3).to_bytes(1, 'big')
    exts = [
        (b'\x21\xff', b'NETSCAPE2.0'),
        (global_ext_size, b'\x01\x00\x01' + loop_count),
        ]

    for ext in exts:
        block = bytearray([flags] + list(ext[0]))
        datalen = len(ext[1]).to_bytes(1, 'little')

        block += datalen
        block += ext[1].encode()
        block += bytes([(2**(len(block)) % 256)])

        out.write(block)

    data = rng.read(datalen)
    out.write(header + data)
