#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )



def generate_random_gif(rng :BinaryIO,
                        out :BinaryIO) -> None:
    """Generates a random GIF file from /dev/urandom"""
    
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\xf0\x01\x00' + \
             b'\xff\xff\xff\x21\xfe\x2c\x5e\x4b\x4d\x4f\xc0\x9a\x75\x5e' + \
             b'\x00\x3b'

    # Number of frames to write (always 1 in this case)
    num_frames     = int.to_bytes(
        1, length=2, byteorder="little", signed=False
    ) 

    # Width and height of the frame data
    width          = int.to_bytes(
        200, length=2, byteorder="little", signed=False
    ) 
    height         = int.to_bytes(
        200, length=2, byteorder="little", signed=False
    ) 
    
    # Write the fixed part of the GIF header  
    out.write(header + num_frames + width + height)
    
    for i in range(num_frames[0] * num_frames[1]):
        # Use bytes from our RNG as images data
        image_data = rng.read(width[0]*height[0]) 
        out.write(image_data)

    # Add final NULL byte
    out.write(struct.pack('B', 0))

