#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
    
        with open(f'./{self.output}', 'wb') as fh:
            generate_random_gif(open('/dev/urandom'), fh)


# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Signature
    header = b'GIF89a'
    
    # Width and Height
    w = rng.read(2)   # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    header += w + h

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again

    # Global Color Table Flag
    gctf = rng.read(1)
    header += gctf

    # Background color index
    bgci = rng.read(1)
    header += bgci

    # Pixel aspect ratio
    par = rng.read(1)
    header += par

    # Reserved byte
    reserved_byte = bytes([rng.read(1)[0]])
    header += reserved_byte

    # Trailer
    trailer = b';'
    header += trailer

    out.write(header)

    # Use dd command instead of Python implementation so that it's faster 
    os.system('dd if=/dev/urandom bs=$((1<<14)) count=65536 | gzip > image.gz; gunzip <image.gz | head -c $((1<<14)) >>./random.gif; rm image.gz')
