#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1024

    # Write header (Version + Logical Screen Descriptor)
    screenwidth = int(rng.read(1)[0]) % 65535
    screenheight = int(rng.read(1)[0]) % 65535
    out.write(header + bytes([
        7<<5|3,
        ((screenwidth>>8)&0x1f)|((screenheight>>8)&0xe0),
        (screenwidth&0xff),
        (screenheight&0xff)
    ]))

    # Write Graphics Control Extension Block
    out.write(b'\x21\xf9\x04')
    extlen = bytearray([(bytearray(rng.read(1))[0]%0x1f)+1])[0]
    out.write(extlen+bytes([0]))
    out.write(rng.read(extlen-1))

    data = rng.read(datalen)
    out.write(header + data)
