#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):

    inlen = 1 << rng.read(4)[0] >> 2
    jpegmagic = bytes([
        0xff, 0xd8, 0xff,
            0xe0, 0x00, 0x10,
                0x4a, 0x46, 0x49,
                    0x46, 0x00 ])
    blocksize = 512
    padding = (b'\x00' * (-inlen % blocksize)) or (b'\x00' * (blocksize+1))
    
    while True:
        if not inlen: break
        inlen -= len(padding)
        
        byte = rng.read(min(inlen, blocksize))

        if len(byte)!= blocksize and padding == b'\x00'*(blocksize):
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            out.write(byte[:blocksize])

