#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1<<24
    data = rng.read(datalen)

    out.write(header + bytes([0,0])
            + bytearray((
                # Logical Screen Descriptor
                768,0,    # Width
                500,0,    # Height
                0,        # Color Resolution
                0,        # Global Color Table (no)
                0,        # Background Color Index
                0,        # Pixel Aspect Ratio

                0b00010000 | len(data), # Transparent Color Flag and Size of Local Color Table
                248,0,       # LZW Minimum Code Size

                # Image Block Extension - Graphics Control Extension
                0x21,   # Application Extension Introducer
                0xFF,   # Application Extension Label
                4,      # Length of Sub-Block
                0x04,   # Graphic Control Label
                0,0,0,  # Delay Time
                0,      # Transparent Color Index
                0,      # Block Terminator

                0b00010000 | len(data), # Image Data Sub-Block Label & Size of Local Color Table
                247,0,       # Left Position
                34,         # Top Position
                369,0,       # Width
                500,0))     # Height
             )
             # Image Data Sub-Block
    out.write(bytes([len(data)]))
    out.write(data)

