#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
    
        with open(f'./{self.output}', 'wb') as fh:
            generate_random_gif(open('/dev/urandom'), fh)


# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Signature
    header = b'GIF89a'
    
    # Width and Height
    w = rng.read(2)   # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    header += w + h

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add new block size

    # Background color index 
    bgci = rng.read(1)[0]       # background color index is an unsigned char
    header += bytes([bgci])     # add to gif header

    # Pixel Aspect Ratio numerator
    parn = rng.read(1)[0]          # pixel aspect ratio numerator
    header += bytes([parn])        # add to gif header

    # Pixel Aspect Ratio denominator
    pard = rng.read(1)[0]          # pixel aspect ratio denominator
    header += bytes([pard])        # add to gif header

    # Global Color Table flag
    gctflag = rng.read(1)[0]    # global color table flag
    header += bytes([gctflag])  # Add to gif header

    # Image Separator
    gs = b'\x2C'                 # image separator
    header += gs                  # Append to gif header

    # Image Left Position
    ilp = rng.read(2)            # image left position
    header += ilp                # append to gif header 

    # Image Top Position
    itp = rng.read(2)            # image top position
    header += itp                # append to gif header
    
    # Image Width
    iw = rng.read(2)             # image width
    header += iw                 # append to gif header

    # Image Height
    ih = rng.read(2)             # image height
    header += ih                 # append to gif header

    # Local Color Table Flag
    lctflag = rng.read(1)[0]    # local color table flag
    header += bytes([lctflag])  # Append to gif header

    # Interlace Flag
    iff = rng.read(1)[0]           # interlace flag
    header += bytes([iff])        # append to gif header

    # Sort Flag
    sf = rng.read(1)[0]           # sort flag
    header += bytes([sf])         # append to gif header

    # Reserved Field
    rsvfl = rng.read(3)           # reserved field
    header += rsvfl               # append to gif header

    # LZW Minimum Code Size
    lzwmincsz = rng.read(1)[0]    # lzw minimum code size
    header += bytes([lzwmincsz])  # append to gif header

    # Write Header to Output File
    out.write(header)

    # Number of Actual Colors Used
    acu = 2 ** lzwmincsz + 1  # actual colors used

    # Generate code length list
    cl = []                    # code length array
    for i in range(acu): 
        cl.append(rng.read(1)[0])  # read from RNG

    header += bytes(cl)   # write to output file

    # LZW Maximum Code Value
    lzwmaxcval = max(cl)                    # maximum code value
    header += bytes([lzwmaxcval])          # append to gif header

    # End Of File Marker
    eom = b'\x3B'              # end of file marker
    header += eom              # append to gif header

    out.write(header)

    # Read Bytes
    data = bytearray()
    while True:
        data += rng.read(1)[0].to_bytes(1, "little")
        
        # Check for EoF
        try: 
            if data[-1] == 0x3b:
                break
        except IndexError as e:
            print("Index error has occured")

    out.write(data)


if