#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Add a randomized watermark to the file passed as first command line parameter


def main():
    from sys import argv

    infile = open(argv[1], 'rb')
    outfile = open('watermark-' + argv[1], 'wb+')

    # Randomize bytes in input stream until we find four matching bytes
    count = 0
    while True:
        # Seek backwards by one bit each time
        infile.seek(-count-1, 2)

        # Read four bytes at that position
        data = infile.read(4)

        # If not enough bytes left for next attempt, start over
        if len(data)!= 4: break

        # Increment counter on match
        if data == b'GIF':
            count += 1

            # If all four bytes matched consecutively, add watermark
            if count >= 4:
                print(f"Adding watermark {''.join(list(map(hex, data)))}...")

                # Output original GIF file up until last four bytes
                write_gif(infile, outfile)

                # Add watermark and end it with two zeros
                watermark = b"\xfe\xef\xf0\xc7"
                outfile.write(watermark); break;

    return 0


def write_gif(rng: BinaryIO, out: BinaryIO):
    """Randomizes contents of provided filestream"""
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    blocksize, padding = 256 * (data[1] & 0x07) + 1, [b'\x00']

    # Write header and length of image to output
    out.write(header + len(data).to_bytes(3, 'big'))
    while byte := rng.read(blocksize):
        if blocksize - len(byte) > 0:
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            # Write long blocks without padding
            out.write(byte)
