#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        (rng.read(1)[0] & 0x7f).to_bytes(1, byteorder='big'), 
        'big') * 2 # make sure the length is always divisible by two

    # allocate space for the image descriptor and color table;
    # place them at the beginning of the file so that we have one thing 
    # to point to when we need an offset later on
    imdesc = b'\x2c\x00\x00\x00\xff\x00\x4d\x65\x4e\x6b\x5d\x4e\xb5\xd7\xde'
    colortbl = bytes([
        0,  0,  0,
        255,  0,  0,
        0,  255,  0,
        0,  0,  255])
    
    out.write(header + len(imdesc).to_bytes(2,'little', signed=False))
    out.write(colortbl + imdesc)

    data = rng.read(datalen)
    out.write(header + data)
