#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )
            


def save_textfile(out, text):
    
    ns = len(text).to_bytes(2, byteorder="little")   # length of plain text as bytes (little endian)
    bs = len(ns + text)                             # total length of file contents
    hdr = b"GIF89a"                                 # magic number and version string

    out.seek(0)                                     # start at beginning
    out.write(hdr)                                  # put the GIF on disk
    out.write(ns + text)                            # padded to even block size


def generate_random_gif(rng: BinaryIO, out: BinaryIO):
    
    width      = 4567
    height     = 2345
    bitdepth   = b'\x08\x3b'                        # bit depth (LZW)
    palette    = b'\x00\x00\x00\xff\xff\xff\x00\x00\x00\xff\xff\xff' 
    background = b'\xf8\x1f'                        # color index for background colour
    aspectratio = b'\xff\xff\xff\xff'                # pixel ratio
    
    ns = len(width).to_bytes(2, "little")           # length of each field of info as bytes (little endian)
    bs = sum(len(n) for n in [ns] * 3 + [bitdepth, palette, background, aspectratio])
 
    
    header = (b'GIF89a'                 # magic number and version string
              + ns                      # logical screen descriptor
              + width.to_bytes(2, "little")
              + height.to_bytes(2, "little")
              + bitdepth 
              + b'\x00\x00'              # no global colours defined
              + b'\x03\xf0'              # minimum code size
              + palette                  # global colour table
              + background
                  + aspectratio       # graphics control extension (background colour)
                  + b'\x00\x01'       # graphic type (bitmap)
                  + b'\x00\x00\x00\x00'
                  + b'\x00\x00\x00\x00' # delay and loop count (unused)
                  + b'\x00'           # block terminator
                  + b'\xfe'           # application extension
                  + b"\x01NETSCAPE2.0\x03\x01\x00\x00\x00"   # application ID and auth code
                  + b"\x00"                                               # end of app extention
                  + b'\x2c'           # image separator 
                  + ns               # image descriptor
                  + b"\x00\x00"       # left offset
                  + b"\x00\x00"       # top offset
                  + width.to_bytes(2, "little")
                  + height.to_bytes(2, "little"))
             
    
    
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
