#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
    
        with open(f'./{self.output}', 'wb') as fh:
            generate_random_gif(open('/dev/urandom'), fh)


# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(rng: BinaryIO, out: BinaryIO):
    width  = rng.read(1)[0] % 256 #  [0..255] 
    height = rng.read(1)[0] % 256 #  [0..255] 

    # Logical screen descriptor
    out.write(b"GIF89a")
    out.write(struct.pack("<BBHHH",
                          width,     # 2 Bytes -> Width
                          height,    # 2 Bytes -> Height
                          0xF7 | (rng.read(1)[0] & 0x40),  #  1 Byte -> Global Color Table Flag + Sort Flag (0111 1111|0100 0000)
                          7,         # 2 Bytes -> Color Resolution
                          0))        # 1 Byte -> GCT Size

    # Image descriptor
    out.write(b"\x2C\x00,\x00")
    out.write(struct.pack("<BHHHH",
                          0xF7 | ((rng.read(1)[0] >> 7) << 1),  #  1 Byte -> LCT Flag (0111 1111|0000 0000)
                          0,         # 2 Bytes -> Left Position
                          0,         # 2 Bytes -> Top Position
                          width,     # 2 Bytes -> Width
                          height))   # 2 Bytes -> Height
    
    # Local color table - black and white only 
    out.write(b'\xFF\xFF\xFF#\xAA\xAA\xAA\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Pixel data
    for y in range(height):
        for x in range(width):
            byte = rng.read(1)[0]
            out.write((byte).to_bytes(1,'little'))
        padding = 0
        while padding < int(0x3FFF / width): 
            out.write(padding.to_bytes(int(0xFF), 'little'))
            padding += 1




