#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )



# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# are read from different sources to ensure that they're valid.
def generate_random_gif(rng: BinaryIO, out: BinaryIO) -> None:
    
    # Header
    header = b''
    header += b'GIF89a'            # File signature and version
    # Color Table size
    ctsz = rng.read(1)[0] * 3
    header += bytes([ctsz & 255])   # lower byte of color table size
    header += bytes([ctsz >> 8])   # upper byte of color table size
    # Image dimensions
    width  = rng.read(1)[0] + 1       # width must be >= 1
    height = rng.read(1)[0] + 1       # height must be >= 1
    header += bytes([width & 255])   # low byte of width
    header += bytes([(width >> 8) | ((height << 7) + ((width % 256) > 127))]) # high byte of width
                                                                               # OR high byte of height, bit 7 set, low nibble has flags about x-extent
    header += bytes([height & 255])  # low byte of height
    header += bytes([height >> 8])   # upper byte of height

    # Background Color Index
    header += bytes([rng.read(1)[0]])

    # Pixel aspect ratio
    header += bytes([48])           # just use square pixels

    # Global color table
    for i in range(2 ** int((ctsz / 3))): 
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
