#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO, Optional


def read_image(path: str) -> bytes:
    with open(path, "rb") as fp:
        data = fp.read()

    if len(data) > 0x18 and data[0x18] == ord('R'): # RIFF file?
        return data[:0x27 * 4096]

    return b""

def fatfs_walk(sfn: bytes):
    name = sfn[:-1].decode("unicode-escape").lower().strip(".\\").split("/")
    
    for i in range(len(name)):
        yield from (b"/" + f for f in name[i:])

def find_file(root: bytes, path: str) -> Optional[bytes]:
    paths = tuple("/".join(fatfs_walk(p)) for p in map(os.fsdecode, path.split(";")))

    off = root.find(paths)

    if off!= -1:
        return off.to_bytes(4, byteorder="little")
    else:
        return None


