#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), byteorder='big')

    data = []
    for _ in range(datalen):
        data += [struct.pack('B', rng.read()[0])]

    # Write 1st part of GIF header and length field
    out.write(header + len(data).to_bytes(2, 'big'))
    
    # Write data field to output
    out.write(b''.join(data))
    
    # Last two characters should be \x21\xFF (extension block terminator)
    out.write(b'\x21\xff')
