#!/usr/bin/env python3

import struct

from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # 4 bytes of magic
    datalen = int.from_bytes(rng.read(1), 'little') % (2047 - len(header)) + 1
    framesize = 6 * datalen + 5
    data = bytearray()
    for i in range(datalen):
        if i < datalen // 2:
            data += [i]
        else:
            data += [framesize - i - 1]

    # Write the header
    out.write(header)

    # Write the LZW minimum code size value, which is usually 12
    out.write(b'\x0f')

    # Now put a bunch of control blocks until we have enough frames to fill up the entire file
    while True:

        if datalen > 255 or not data:
            break

        num_blocks = min([
            255, 
            max([
                0, 
                int((max(data) / 255)**5), 
                1
                ])
            ]).to_bytes(1,'big') 

        out.write(num_blocks)
        
        # Last block needs to be special cased here
        if num_blocks == b'\xff':
            if data[0] == 0:
                out.write(data[:255])
                data = data[255:]
            elif data[-1]!= 255 and data[-1] == ((len(data)-1)//2):
                out.write(
                    data[:-1].ljust(
                        255, 
                        b'\x00')
                        )
                data = None
            else:
                raise Exception('Unexpected state when generating GIF file.')
        else:
            out.write(data[:255])
            data = data[255:]

    # Finally write an end-of-file marker
    out.write(b';')

