#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
    
        with open(f'./{self.output}', 'wb') as fh:
            generate_random_gif(open('/dev/urandom'), fh)


# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Signature
    header = b'GIF89a'
    
    # Width and Height
    w = rng.read(2)   # width is 16 bit
    h = rng.read(2)   # height is 16 bit
    out.write(struct.unpack('>H', w))       # unpack to big endian short
    out.write(struct.unpack('>H', h))       # unpack to big endian short

    # Flags
    flags = bytearray()
    flagByte = rng.read(1)[0]            # read one byte from RNG
    for i in reversed(range(7)):           # iterate through bits in byte
        flags.append((flagByte >> i) & 1)  # append value of specific bit
        
    globalColorTableFlag = flags[0]      # GCTF
    colorResolution = [ 8, 4, 2 ][flags[1]] # CR 
    sortFlag = flags[2]                 # SF
    interlaceFlag = flags[3]            # IF
    reservedFlags = bytearray(flags[4:]) # rest are reserved
    out.write(globalColorTableFlag)      # Write GCTF
    out.write(bytes([colorResolution]))  # Write CR
    out.write(sortFlag)                  # Write SF
    out.write(interlaceFlag)             # Write IF
    out.write(reservedFlags)             # Write rest of flags

    # Background color index
    bgc = rng.read(1)
    out.write(bgc) 

    # Pixel aspect ratio
    par = rng.read(1)
    out.write(par)   

    # Color Table size
    ctsz = rng.read(1)[0] * 3
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    
