#!/usr/bin/env python3

import struct
from typing import BinaryIO, Tuple


def read_uint64(file: BinaryIO) -> int:
    return _read_number(file, ">Q")


def read_uint32(file: BinaryIO) -> int:
    return _read_number(file, ">I")


def write_uint8(value: int, file: BinaryIO):
    data = struct.pack(">B", value)
    file.write(data)


def _read_number(file: BinaryIO, fmt: str) -> int:

    size = struct.calcsize("=" + fmt)
    bytes = file.read(size)
    if len(bytes)!= size:
        raise EOFError()

    # Unpack the first item in the tuple, which is the number
    (num,) = struct.unpack_from("=" + fmt, bytes)

    return num


def read_utf16le_nullterm(file: BinaryIO) -> str:

    buf = []
    while True:

        char, = bytearray(file.read(2))

        if not char:
            break
        
        buf.append(char)
    
    length = len(buf) // 2
    fmt = f">{length}H"
    numbers = struct.unpack(fmt, b"".join(buf))

    output = [chr(num) for num in numbers]
    return "".join(output)

