#!/usr/bin/python3

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self._output = DEFAULT_OUTPUT
        else:
            self._output = output

        self._width = 1024
        self._height = 768
        self._bitsize = (self._width *
                        self._height) // 8 + ((self._width * self._height) % 8 > 0)


    @property
    def width(self):
        return self._width

    
    @property
    def height(self):
        return self._height
    

    @property
    def bitsize(self):
        return self._bitsize
        
    @property    
    def output(self):
        return self._output
    
    
    def _createHeader(self):
        header = bytearray([
            0x47, 0x49, 0x46, 0x38, # signature
            0x37, 0x0A,             # version
            0x00, 0x00,             # size of the logical screen descriptor
            0x00,                   # global colour table flag
            0xFF, 0xFF, 
            0xFF, 0xFF, 
            int(self._width / 256),   # width of the logical screen descriptor
            int(self._width % 256),
            int(self._height / 256),   # height of the logical screen descriptor
            int(self._height % 256)])

        header[6] |= len(header)-10
        return bytes(header)



    def generate(self, data: bytes):
        
        with open(self._output,'wb') as out:

            out.write(self._createHeader())
            
            chunkData = b''
            chunkSize = -1
            while True:
                chunkSize += 1
                
                if chunkSize == len(data):
                    chunkType = b'\x08\x00'     # image data
                elif chunkSize == len(chunkData)+1:
                    chunkType = b'\x00\x00'     # extension block
                else:
                    chunkType = b'\xFE\x00'     # application extension

                out.write(struct.pack('<H',len(chunkData)))
                out.write(chunkType)
                out.write(chunkData)
             
                if chunkSize >= len(data): 
                    break
            
                chunkData = data[chunkSize:]
        


if