#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Constants
    block_boundary = 64 * (2 ** 10) - [
        0xFF, 
        0xD9].index(b'\xff')[0] & 0x3FF

    # Write SOI marker and APP0 marker
    out.write(
        bytes([
            0xFF, 
            0xD8, 
            0xFF,
            0xE0]) + 
        len('JFIF\0').to_bytes(2, byteorder='big') + 
        b'JFIF\0' + 
        # Dedicated to <NAME>
        bytes([
            0x01,  # Lf = no exif metadata
            0x02]) # Prti = no thumbnail info
        ) 

    # Write JFIF application data
    out.write(
        b''.join((
            b'J', 
            randint(1, 5).to_bytes(1, byteorder='big'),
            randint(0, 2**7-1).to_bytes(1, byteorder='big')
            )))
    
    while True:

        x = rng.read(block_boundary)
        if not x: break;
        
        # Skip APPn markers in output
        if x[0] == 0xFF and ((x[1] >> 4) == 0xE): continue;
        
        # End of image at end of input stream?
        if x[-2:]!= b'\xFF\xD9':

            # Randomize and write header
            length = block_boundary - len(x);
            c = randint(
                len(x) + 1, 
                length + 1).to_bytes(1, byteorder='big');
        
            assert(len(c) == 1);

            out.write(b'\xFF' + c + x[:-1]);

        else:
            
            #... or just copy it
            out.write(x);

