#!/usr/bin/env python3

import gzip
import io
import math
import random
import string
import sys
import time


def main(*args):
    start = time.time()

    width       = int(args[2])   or 512
    height      = int(args[4])   or 512
    colordepths = args[6:]       or [8]

    print('Generating {}x{}@{}'.format(width, height, ', '.join(map(str, colordepths))))

    buff = gen_buffer((width,height), *colordepths)

    end = time.time()
    delta = end - start
    print('Generated in {:.3f} seconds'.format(delta))


def gen_buffer(size, *colordepths):
    '''Generate pixel buffer of given size and color depth.'''
    w, h = size
    buf = bytearray([(w*h*(i+1))%256 for i in range(sum(colordepths))])
    pixels = []
    offset = 0
    for cd in colordepths:
        numbytes = calc_numbytes(cd)
        pixels.append(buf[offset:offset+numbytes])
        offset += numbytes
    return pixels


def calc_numbytes(colordepth):
    if colordepth == 1:
        return 1
    elif colordepth <= 8:
        return 2**(colordepth + 7 // 3)
    elif colordepth <= 16:
        return 2**colordepth
    elif colordepth <= 24:
        return 3 * 2**colordepth
    raise ValueError("Invalid value for colour depth")


if __name__ == '__main__':
    try:
        main(*sys.argv)
    except Exception as e:
        print("{}: {}".format(__file__, str(e)))
        exit(1)