#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # First three bytes are always the same length (7 bytes total)
    # Read exactly seven bytes from /dev/urandom to get this value
    datalen = int.from_bytes(
            rng.read(7), byteorder='big') >> 8
    
    # Read the rest of the image data from /dev/urandom
    data = rng.read()

    # Check that we have enough bytes remaining
    assert len(datalen) == 3 and len(data) >= datalen

    print('Generated GIF',
          f'datalen={datalen} ({len(data)})',
          f'data=({data[-1]})', sep='\n  ')

    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)
