#!/usr/bin/env python3

import os
import sys
import io
import struct


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    print(f"width: {image_width}, height: {image_height}")

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)
    
    # size of the color table
    ctsize = int.from_bytes(rng.read(1), 'little')

    if ctsize > 0:
        
        print("has a color table")
    
        # read in palette values (RGB triples)
        paldata = [struct.pack('BBB', *x) for x in
            [(y % 256, y // 256, y // 256 * 256)
                for y in range(ctsize)]]

        palents = bytes([len(x) for x in paldata])
        for item in paldata:
            palents += item

        # write to output stream
        out.write(header + struct.pack('<HHBB', image_width, 
            image_height, ctsize | 0b10000000, 
            0) + palents)


    else:

        # no palette
        out.write(header + struct.pack('<HH', image_width, 
                image_height))



    # get remaining lzw compressed info from the RNG
    # TODO: process this like we did above with the palette?
    datalen = int.from_bytes(rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)
