#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, numframes: int, out: BinaryIO
    ):
    
    header = b'GIF89a'
    width = 42
    height = 50
    depth = 8
    interlacedFlag = True
    colorTableSize = 7
    backgroundIndex = 0
    aspectRatio = 0 # 1:1 ratio

    # Offsets within the header structure in bytes
    screenWidthOffset = 6 + 3*numframes   # Screen Width
    screenHeightOffset = 8 + 3*numframes  # Screen Height
    colorTableOffset = 10 + 3*numframes   # Color Table Size
    globalColorTableOffset = 11 + 3*numframes  # Global Color Table Flag
    backgroundOffset = 12 + 3*numframes    # Background Color Index
    pixelAspectRatioOffset = 13 + 3*numframes  # Pixel Aspect Ratio
    gctBytesPerBlockOffset = 14 + 3*numframes     # GCT Bytes Per Block

    headerContents = []
    for offset in range(gctBytesPerBlockOffset):

        # Must not read past the end of the header contents
        if offset < len(_headerOffsets):

            # Get the offset in the header structure from _headerOffsets
            offsetInHeaderStruct = _headerOffsets[offset]
            
            if offset == 1 or \
               offset >= 14 and offset <= 17:
                # Header flags are encoded as single bytes
                val = rng.read(1)[0] >> offsetInHeaderStruct % 8
            elif offset == 2 or \
                 offset == 3 or \
                 offset >= 18 and offset <= 23:
                # These fields are encoded as two-byte integers
                val = int.from_bytes(
                        rng.read(2), 
                        byteorder='big') >> offsetInHeaderStruct % 16 
            elif offset == 4 or \
                 offset >= 24 and offset <= 29:
                # This field is encoded as four-byte integers
                val = int.from_bytes(
                        rng.read(4), 
                        byteorder='big') >> offsetInHeaderStruct % 32
            else:
                raise ValueError("Unknown offset defined")
        
            # Append this value to our list of header contents
            headerContents += [(val)]

    # Define an empty color table
    gct = b''

    gctByteLength = pow(2, colorTableSize+1)
    for i in range(pow(2,colorTableSize)):
         r = (rng.read(1)[0] & ~0xE0) | ((i << 5) & 0xE0)
         g = (rng.read(1)[0] & ~0xE0) | (((i>>3)<<5) & 0xE0)
         b = (rng.read(1)[0] & ~0xC0) | (((i>>6)<<7) & 0xC0)
         gct += bytes([r, g, b])
         
    # Add all these values to our header
    header += b''.join((map(lambda x : x.to_bytes(1,'little'),
                          headerContents)))

    
    # Write the header to the output
    out.write(header)


    # Generate and write each frame to the output
    for f in range(numframes):
        


        # TODO