#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    image_width = (1 << rng.read(1)[0]) % 32769
    image_height = (1 << rng.read(1)[0]) % 32769

    global_color_table_flag = bool(rng.read(1))
    local_color_table_flag = bool(rng.read(1))

    color_resolution = rng.read(1)[0] & 0b111
    sort_method = (rng.read(1)[0] >> 4) & 0b111

    bytes_in_palette = 3 * (
        2 ** ((color_resolution + 1) * 3) )

    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]

    print('Width: {}'.format(image_width))
    print('Height: {}'.format(image_height))
    print('Global Color Table Flag: {}'.format(global_color_table_flag))
    print('Local Color Table Flag: {}'.format(local_color_table_flag))
    print('Color Resolution: {}'.format(color_resolution))
    print('Sort Method: {}'.format(sort_method))
    print('Bytes in Palette: {}'.format(bytes_in_palette))
    print('Background Color Index: {}'.format(background_color_index))
    print('Pixel Aspect Ratio: {}'.format(pixel_aspect_ratio))

    if global_color_table_flag == True:
        raise NotImplementedError()

    if local_color_table_flag == False:
        raise NotImplementedError()

    palette = []
    for x in range(256):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]

        palette += [r,g,b]

    out.write(struct.pack('<HHBBB',
             image_width, image_height,
             global_color_table_flag,
             local_color_table_flag,
             0xF7))

            # See https://www.w3.org/Graphics/GIF/spec-gif89a.txt
            # for some explanation of these fields.
            #
            0b000,  # Sort method is not used because
                    # "No disposal specified"

            0xFF,    # Transparency flag; ignored by most
                     # software due to lack of support.

            0b000,   # Size of local color table; set to
                      # full size since we don't have one yet!

            0xFF     # LZW minimum code size; irrelevant
               ))