#!/usr/bin/env python3

import sys
from PIL import Image
from typing import List


def add_padding(image_data: bytes, width: int) -> bytes:
    height = len(image_data)//width//4
    assert len(image_data)%height == 0
    return image_data \
       .ljust((height*width*4), b'\x00')


if __name__ == '__main__':
    try:
        source = sys.argv[-1]
        filename = source[:source.index('.')]

        with open('/dev/urandom', mode='rb') as rng,\
             open('{}.gif'.format(filename), mode='wb+') as f,\
             Image.open(sys.argv[-1]) as img:

            # Extract pixels from image
            rgb = list(*img.convert("RGB").getdata())
            pixels = []
            for r, g, b in zip(*(iter(rgb),)*3):
                pixels += [struct.pack('<BBB', r, g, b)]

            # Convert pixel array into binary stream
            datastream = b"".join([
                b"\x00\x00",
                b"\xff\xfe" if img.is_animated else b"\xff\xff",
                *pixels[:-1],
                b"\x00"
            ])


            # Write GIF file
            f.write(b'GIF89a'
                    +len(datastream).to_bytes(3, 'big')
                    +datastream)
    except:
        raise
        print("Usage: {} image".format(__file__))
        exit()