"""
GIF file format is not well documented so this code was inspired by
https://websnarf.wordpress.com/2007/06/30/creating-animated-gifs-with-python/
and https://gist.github.com/Zireael-N/b5d04f46bf01ecfefeb74e1228df86ef.
"""


from typing import BinaryIO
import struct
import io


class GifWriter:

    def __init__(self):

        self._data = []

    # Writes a single frame to the buffer and returns it as bytes object
    @staticmethod
    def _write_frame(
            imgbuf: io.BytesIO, delay: int
        ) -> bytes:

        bufsize = len(imgbuf.getbuffer())
        data = imgbuf.getvalue()[:bufsize]
        # Add some padding to make sure we don't get EOF
        padlen = (1 << (bufsize - 1).bit_length()) ** 2
        data += bytes([0]) * padlen

        # Write image size
        im_w, im_h = imgbuf.tell(), len(data) // im_w
        packed = struct.pack('<HHB', im_w, im_h, 0x87)

        return b''.join((packed,
                        delay.to_bytes(2, 'big'),
                        0x00,  # No colors in palette
                        0x2c, b'',
                        len(data).to_bytes(2, 'little'),
                        b'',  # LZW minimum code size
                        data))


    def write(self,
              imgbuf: io.BytesIO,
              delay: int):

        self._data.append(GifWriter._write_frame(imgbuf, delay))

    def add_loopback(self, loopcount: int):
        
        self._data[-1] = self._data[-1][:-1] \
                          + (b'\xfe',
                             loopcount.to_bytes(2, 'little'))

    def save(self,
             fp: BinaryIO):

        for d in self._data:
            fp.write(d)