#!/usr/bin/env python3
"""Generates a random GIF file into `out` using the given RNG."""


import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    
    # Create random bytes until we get three that are all distinct
    header = b''
    while len(set([int.from_bytes(rng.read(3), byteorder='big') for _ in range(5)])) < 3: pass
    header += rng.read(3)

    # Read the rest of the GIF's metadata
    datalen = int.from_bytes(header[-3:], byteorder='big')
    data = rng.read(datalen)
    blocksize, padding = 256 * (data[1] & 0x07) + 1, [b'\x00']

    # Write header and length of image to output
    out.write(header[:6] + len(data).to_bytes(3, 'big'))
    while byte := rng.read(blocksize):
        if blocksize - len(byte) > 0:
            # Pad short blocks with zeroes
            out.write(byte + padding[0])
        else:
            # Write long blocks without padding
            out.write(byte)