#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# provided RNG. The size of the generated image is determined by
# the first four bytes read from the RNG (which are discarded).
#
# This function cannot be used as-is to generate randomness for real
# applications because it has no way to provide sufficient entropy to
# the application's cryptographic functions so that they can complete
# their operation properly. See the blog post at
# https://blog.trailofbits.com/2019/05/29/how-to-break-a-random-number-generator/#part-four-fixing-the-rng-for-real
# for an example where this problem was addressed in practice.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Generate the fixed part of the file based on the initial seed;
    # we don't want the user to have any extra information when trying
    # to guess our output.
    header = b'GIF89a'
    initseed = rng.read(9)
    initrand = os.urandom(7) if len(initseed) < 9 else initseed[:7]
    datalen = int.from_bytes(initrand[6:], 'big') + 1
    out.write(header + initseed+ initrand)

    # Now add some more random content until the desired length is reached.
    while out.tell() < datalen:
        out.write(os.urandom((datalen - out.tell())))