#!/usr/bin/env python3

import os
import subprocess as sp
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    datalen = rng.read(1)[0] & 0xFF
    datalen |= (rng.read(1)[0] & 0xFF) << 8
    datalen |= (rng.read(1)[0] & 0xFF) << 16
    datalen += 1

    # Discard metadata for now; we will add it later
    meta = rng.read(4)

    remainder = datalen % 8
    if remainder!= 0:
        datalen -= remainder
        meta = meta[:-remainder]

    data = rng.read(datalen)
    assert len(meta) == 4

    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(meta)
    out.write(data)


