#!/usr/bin/env python3

import struct
from typing import BinaryIO
import zlib

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Create and write GIF header
    header = b'GIF89a'
    out.write(header)

    # Generate compressed data until we reach some limit
    maxlen = 2 ** 16 - len(header) // 3
    while True:

        # Generate a new length to fit with our maximum
        datalen = sum(rng.randrange(100) for _ in range(5)) % maxlen
        if datalen < 100:
            break

        print("Generating {} bytes".format(datalen))

        # Get a sample string of that size
        data = (chr(rng.randrange(256)).encode() *
                ((datalen + 7) // 8))[:datalen]
        assert len(data) == datalen

        # Compress it
        z = zlib.compressobj(9)
        compr = z.compress(data)
        flush = z.flush()
        datalen += len(compr) + len(flush)
        print("Compressed to {}".format(datalen))

        # Write compressed data to output stream
        out.write((struct.pack('>H', datalen)))
        out.write(compr)
        out.write(flush)