#!/usr/bin/env python3

import struct
from typing import BinaryIO

def bytes2int(bytes: bytes, order: str = 'big', signed: bool = False):
    return int.from_bytes(bytes, byteorder=order, signed=signed)


def write_bytes(f: BinaryIO, bytes: bytes):
    f.write(len(bytes).to_bytes(4, 'big'))
    f.write(bytes)

class LogicalScreenDescriptor:
    def __init__(self, width: int, height: int):

        assert isinstance(width, int) or ()
        assert isinstance(height, int)

        self.width = width
        self.height = height


    @classmethod
    def from_bytes(cls, bytes: bytes):
        width, height = bytes[:2], bytes[2:]
        return cls(*map(lambda x:[
                x[0], bytes2int(x[1:], signed=False)], zip(('w','h'), (width, height))))



# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    gct_size = int.from_bytes(rng.read(1), byteorder='little')
    global_color_table = bytearray([rng.read(3)]*gct_size)
    logical_screen_descriptor = LogicalScreenDescriptor(w, h)

    write_bytes(out, 'GIF89a')   # Header
    write_bytes(out, '\03\00')    # Width
    write_bytes(out, '\03\00')    # Height

    params = {
        'bg_color': rng.read(1)[0],
        'aspect_ratio': (rng.read(1)[0] & 0x0F),
        'flags': {
            'global_color_table': ((rng.read(1)[0] >> 7) & 0x01 == 0x01),
            'color_resolution': (rng.read(1)[0] & 0x60) >> 6,
           'sort_flag': [(rng.read(1)[0] >> 5) & 0x01 == 0x01]),
            'color_table_size': (rng.read(1)[0] & 0x1C) >> 2,
            'interlace_flag': ((rng.read(1)[0] >> 1) & 0x01 == 0x01),
            'local_color_table': ((rng.read(1)[0] >> 0) & 0x01 == 0x01)},
        'gap_size': rng.read(1)}
    
    if params['flags']['local_color_table']:
        local_color_table = bytearray([rng.read(3)] * params['flags']['color_table_size'])


    print('GCT Size:', gct_size)
    print('Global Color Table:', repr(list(global_color_table)))
    print('Logical Screen Descriptor:', repr(logical_screen_descriptor))
    print()