#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# MIT License

'''
RandomGifGenerator v1.0
by <NAME> <<EMAIL>>

This program generates a pretty random GIF file. It doesn't use any external
libraries or dependencies, so you can run it on a Raspberry Pi right off the
shelf!

Usage:
  random_gif_generator.py [output_file]

  This will save the generated GIF at './random.gif'.

  You can also specify an alternative filename by providing it after the script name:

 ./random_gif_generator.py cool-gif.gif

Requirements: 
  Python >= 3.5

'''

import sys
import os
import struct
from typing import BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, output=None):

        if not output:
            self.output = DEFAULT_OUTPUT
        elif len(os.path.dirname(output)) == 0:
            self.output = f'./{output}'
        else:
            self.output = output


    def generate_random_gif(self):
        
        with open('/dev/urandom', mode='rb') as rng,\
             open(self.output,    mode='wb') as out:

            return generate_random_gif(
                    rng, out
                )



# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# are read from different sources to ensure that they're valid.
def generate_random_gif(rng: BinaryIO, out: BinaryIO) -> None:
    
    # Header
    header = b''
    header += b'GIF89a'            # File signature and version
    # Color Table size
    ctsz = rng.read(1)[0] * 3
    header += bytes([ctsz])       # Add color table size before adding individual entries
    for _ in range(ctsz):
        header += bytes([rng.read(1)[0]])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    data = rng.read(bs)          # get actual image data
    out.write(header + data)     # write it all together
    