#!/usr/bin/env python3

import argparse
from os import urandom as rand
from struct import pack as pk


# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: bytes, out: BinaryIO
    ) -> None:

    # We're going to use these words for our fields
    hdrword = 'GIF89a'.encode('ascii')
    lenword = 'Content-Length: '.encode('utf-8')
    datword = 'Content-Data: '.encode('utf-8')

    # Format of the output GIF
    yield pk('<4s', hdrword)
    if args.contentlength:
        length_str = str(len(rng)) + '\n'
        yield pk('<{}s{}s{}s'.format(
            len(hdrword),
            len(lenword),
            len(length_str)),
            hdrword,
            lenword,
            length_str.encode('ascii'))

    yield pk('{}s'.format(len(datword)), datword)
    yield rng
    yield pk('B', 0x3b)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--contentlength', action='store_true')
    parser.add_argument('-o', default=False, dest='outfile')
    args = parser.parse_args()

    with open(args.outfile, 'wb') as f:
        for chunk in generate_random_gif(rand(256*1024), f):
            print(chunk)