#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    width = next(struct.unpack('>H', rng.read(2))) % (1 << 10)
    height = next(struct.unpack('>H', rng.read(2))) % (1 << 10)

    globaltable = []
    for i in range(1 << 10):
        globaltable += [next(struct.unpack('>B', rng.read(1)))]

    localtable = list(globaltable)

    out.writelines([
        bytes([width & 0xFF]),
        bytes([(width >> 8) & 0xFF]),
        bytes([height & 0xFF]),
        bytes([(height >> 8) & 0xFF]),
        ])

    flags = 0b00101000 | ((7 // 8) - 1) << 4 # 10 colors, no interlacing, sorted color table
    out.write(flags.to_bytes(1, 'little'))


    out.write((len(localtable) - 1).to_bytes(1, 'little'))
    
    for index, value in enumerate(localtable[:-1]):
        low = index & 0xFF
        high = (index >> 8) & 0xFF

        out.write(low.to_bytes(1, 'little')) 
        out.write(high.to_bytes(1, 'little')) 

        out.write(value.to_bytes(1, 'little')) 

    for rgb in rng.read(3*len(localtable)):
        out.write(rgb)

    pallete = bytes()
    for c in localtable:
        # Convert to greyscale
        r, g, b = (c,)*3
        y = round(.299 * r +.587 * g +.114 * b)
        pallete += bytes([y]*3)

    out.write(pallete)

    screendescriptor = dimmensions | 0b01000000 
    out.write(screendescriptor.to_bytes(1, 'little'))

    loopcontrol = 0b10000100  # No end of animation marker
    out.write(loopcontrol.to_bytes(1, 'little'))

    out.write((0).to_bytes(2, 'little')) # No background color
    out.write(b'\x00'*2)              # Pixel coordinate where drawing starts

    stream = None
    if colored:
        stream = b''
        for x in range(dimmensions[0] // 16):
            for y in range(dimmensions[1] // 16):
                stream += b'\xF9\x04'
                for _i in xrange(16**2):
                    stream += chr(x)
                    stream += chr(y+16)
        
    else:
        stream = b''
        for x in range(dims[0] // 16):
            for y in range(dimmensions[1] // 16):
                stream += b'\xFE\x04'
                for _i in xrange(16**2):
                    stream += chr(x)
                    stream += chr(y+16)


    # Split data into blocks smaller than 255 and write them as is
    while True:
        chunk = stream[:255]
        stream = stream[255:]
        
        if not chunk:
            break
        elif type(chunk) == str:
            chunk = chunk.encode()

        out.write(len(chunk).to_bytes(1, 'little'))
        out.write(chunk)


