#!/usr/bin/env python3

import struct
import sys
from typing import BinaryIO

# Generates a random GIF file into stdout using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO
    ) -> None:

    # We will use this format for all our blocks and headers.
    gifblockformat = '>BBH'
    nextsectionlen = lambda x: ((x+0xFF)&~0xFF)-x
    sectiontype = {
            'HEADER': b'\x21',
            'EXTENSION': b'\x21',
            'IMAGE DATA': b'\x2C',
            'TRAILER': b'\x3B'
    }

    # The first block is always the HEADER
    header = b'GIF89a'
    imagewidth = 512*7//8
    imageheight = 512//8
    aspectratiobits = (imagewidth << 5)|imageheight
    extensionsize = 4
    subblocks = [struct.pack('>BBBBBBBHHHB',
                        *([extensionsize] + list(
                            sectiontype['EXTENSION']))) +
                    bytes([aspectratiobits])]


    transparencyflagbyte = b'\x20\xfe'
    gctbytesperpixel = 6 # NOTE: This must match the size of rgbquantizationtable

    globalcolortablebytes = bytes([transparencyflagbyte]) + \
                                rgbquantizedata[0]*gctbytesperpixel

    subblocks += [globalcolortablebytes]



    # Now we add some frames to demonstrate how that works!
    # We're going to be adding each frame as an image, though if you want
    # You can also add multiple images in one frame... just "bake" them together?
    # It's up to you I guess.
    for currentframe in range(2):
        lzwmincodesize = 1

        subblocks += [struct.pack(
                gifblockformat,
                0xF9,# Application Extension Block
                11, # Number of Bytes after this block
                len(b'NETSCAPE2.0') + 3), # Length of Sub-Block
                b'NETSCAPE2.0',
                3, # Sub-Block Terminator
                int(currentframe/10).to_bytes(length=1, byteorder='little'),# Frame Count
                b'\x00',# Loop Extension Code(0xff)
                ]


        # Now we have to add the LZW encoded image data
        # First LZW Minimum Size code with 8 bits (the default)
        # NOTE: this is before the first lzw min code size so it has to be done here too!
        subblocks += [(lzwmincodesize).to_bytes(length=1, byteorder='big')]
        lzwmaxcodesize = 1<<lzwmincodesize
        numcolors = maxnumcolors
        colormapsize = numcolors+1
        lzwdatacodesize = 8
        clearcode = 1<<(lzwdatacodesize - lzwmincodesize)
        endcode = clearcode + 1
        resetcode = endcode + 1
        subblocks += [clearcode.to_bytes(length=2, byteorder='big')]

        # We'll start off by saying that the previous colors were all black.
        lastrgb = tuple((0,)*3)
        lastdelta = 0
        lastcode = 0
        prevbyte = -1
        codebuf = []
        def sendcodes():
            if not codebuf or len(codebuf)==prevbyte+1: return

            curdata = bytes()
            curnbits = 0
            while codebuf:

                # Make sure we don't overflow the bit limit
                assert curnbits < lzwdatacodesize

                # If there are less than two elements left, just write those
                # Also make sure that they are the same as lasttime
                if len(codebuf)<2 or codebuf[-1]!=lastcode or codebuf[-2]==lastcode:
                    count = lastcode

                    # Write the repeated value now
                    curdata += (count & ~0x80).to_bytes(length=curnbits+1,
                                                        byteorder='big')
                    codebuf = codebuf[:-2]
                    continue


                # Start writing non-repeated values
                # Grab from the buffer until we can get a repeat
                curval = codebuf.pop()
                while True:
                    # Get the second value. We know it exists because
                    # lzwmincodesize was at least 1 and the lastvalue==firstvalue
                    otherval = codebuf.pop()
                    if curval == otherval: break
                    else: codebuf.append(otherval)
                codebuf.append(curval)
                count = (curval<<lzwdatacodesize)+otherval
                writtenbytes = False
                # If the difference between these values is under 128 then we
                # Can pack them into the available space. Otherwise it's an
                # Extended length representation

                # The encoding scheme used is described in RFC1952
                while True:
                    if count >= 0x80:
                        # If the count overflows the available space then
                        # Add that byte to the output
                        curdata += (count&~0x80).to_bytes(length=curnbits+1,
                                                           byteorder='big')
                        count >>= lzwdatacodesize
                        curnbits -= lzwdatacodesize

                        # And then we need more room, set the flag to say such
                        writtenbytes = True
                    elif curnbits+1 <= lzwdatacodesize:
                        # Still have room to fit everything in
                        # Just add this value normally
                        curdata += count.to_bytes(length=1,
                                                  byteorder='big')
                        curnbits += 1

                        # No longer needs more room; exit loop
                        break
                    else:
                        # Again, overflowing the available space means we need
                        # To put this value in extended length mode
                        curdata += (0x80 | (count>>lzwdatacodesize)).to_bytes(
                                length=1, byteorder='big')
                        count &= ~(0x80 << lzwdatacodesize)

                        # Set the flag to say that we have added a byte already
                        writtenbytes = True

                        # Incrememt the curbit counter by what we just wrote
                        curnbits -= lzwdatacodesize
                        curdata += (count & (2**curnbits-1)).to_bytes(
                                    length=1, byteorder='big'
                                )
                        curnbits -= lzwdatacodesize


            # We've finished reading codes. Send the final byte array
            outbuffer[prevbyte:] = curdata

        def sendliteral(val):
            nonlocal lastcode,prevbyte
            val &= 0xFF
            outbuffer[prevbyte]=val
            lastcode=val
            prevbyte+=1

        curpos=0
        outbuffer = bytearray(imagewidth*imageheight)
        while curpos < len(outbuffer):
            r = ord(data[curpos])
            g = ord(data[curpos+1])
            b = ord(data[curpos+2])
            curpos += 3
            rgb = (r,g,b)

            delta = sum(((c-o)**2 for c, o in zip(rgb, lastrgb)), 0)

            # Are we repeating the same thing? Isn't really that important anymore.
            #if delta<=lastdelta: raise Exception("This algorithm won't work!")
            #else: lastdelta = delta
            if delta<lastdelta: pass
            elif delta==(lastdelta+1) and imgnr == 0:
                # We can do literal encoding.
                sendliteral(imgnr)
            else:
                # Either 0-2 or greater than 2 pixels away.
                # We have to switch to LZ encoding.

                # We only allow lzminsize to increase by 1 bit per 128 pixels
                # If we go above it, then we have to switch to lzmaxsize

                if delta > lzwmaxcodesize:
                    lzwmincodesize += 1
                    lzwmaxcodesize = 1<<(lzwmincodesize)
                    datum = resetcode
                    sendliteral(datum)
                else:
                    datum = lastcode

                #print(f'{delta} vs prev {lastdelta}')
                #print(f'datum={datum}')
                #print(codebuf)
                #print(subblocks)
                sendcodes()
                subblocks += [datum.to_bytes(length=2, byteorder='big')]
                lastcode = datum
                lastrgb = rgb
                lastdelta = delta

        print(f'Frame {currentframe}', file=sys.stderr)
        # Add the trailer which should just be an empty block. The total length
        #   should be 0.
        subblocks += [struct.pack(gifblockformat,
                    0x3B,# Trailer block
                    0, # Number of Bytes after this block
                    0)]#(sectionlen))
        # Write the Section
        for sb in subblocks:
            out.write(sb)



def main():

    with open('/dev/urandom','rb') as rng:
        generate_random_gif(rng, out=sys.stdout.buffer)

