#!/usr/bin/env python3

import struct
from math import floor
from os import urandom as get_random_bytes
from typing import BinaryIO

# Generate a WAV format PCM audio stream from an RNG object. The length of that
# stream is controlled by the second argument (in bytes). `sample_rate` and
# `bits_per_sample` control the parameters for the generated audio.
def create_wav_pcm_stream(
        rng: BinaryIO, length: int, sample_rate: int, bits_per_sample: int
    ) -> bytes:
    num_samples = floor((length / bits_per_sample) / sample_rate)

    return b'RIFF' + \
           (num_samples * sample_rate * bits_per_sample // 8 + 44).to_bytes(4, 'little') + \
           b'WAVEfmt'+ \
           ((1 << 12) | (1 << 3)).to_bytes(4, 'little') + \
           (1 << 14).to_bytes(2, 'little') + \
           (((1 << 11) + sample_rate) & 0xffffffff).to_bytes(4, 'little') + \
           (((bits_per_sample >> 3) + (1 << 7)) & 0xffffffff).to_bytes(4, 'little') + \
           ((1 << 0) & 0xffffffff).to_bytes(2, 'little') + \
           b'data' + \
           num_samples.to_bytes(4, 'little') + \
           rng.read()
