#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out`.
# The width and height can be adjusted as well as the color depth (in bits).
def generate_random_gif(
        width=640, height=480, colordepth=8,
        out: BinaryIO
    ):

    # Compute image data size based on color depth
    imdatlen = ((width*height)*colordepth)//8


    # Write magic header
    out.write(b"GIF89a")
    

    # Write screen descriptor
    out.write(struct.pack("<HHHHB", width, height, 0xF7DF, 0, 0))

    # Write global color table
    palette = [0]*((2**colordepth)-1)
    for i in range(len(palette)):
        palette[i] = int(i/(2**(colordepth-1)))
    for p in palette:
        out.write(p.to_bytes(1, "little"))

    # Write extension block to specify loop count
    out.write(b"\x21\xFF\x0B\x4E\x45\x54\x53\x43\x41\x50\x45\x3D\x30")

    # Write image descriptor
    out.write(struct.pack("<HHHBBBBB", 0, 0, width, height, 0, 0, 0, 0))

    # Write local color table
    for p in palette:
        out.write(p.to_bytes(1, "little"))

    # Write graphics control extension info
    out.write(b"\x21\xF9\x04\x08")
    
    # Write image data
    for i in range(imdatlen):
        out.write(int(i/(2**(8-colordepth))).to_bytes(1,"little"))

