#!/usr/bin/env python3

import binascii
from io import BytesIO


def write_size(data: bytes):
    return len(data).to_bytes(2, 'big')


def generate_welcoming_gif():

    # Header
    yield from (b'GIF87a', b'\x01\x00\x01\x00\xf0\x00',
                b'\xff\xff\xff\x00\x00\x00!\xf9\x04\x01\x00\x00\x00\x00,\x00'
               )
    
    # Logical screen descriptor
    yield from (b'#\xb6\xa4U\x0e\x00\x00\x05\x04\xe0\x00\x9dH\x00\x00\x00\x00\x00\x9dH'
              ,b'\x00\x00\x00\x00\x02\x00;')
    
    # Graphic control extension block
    yield from (b'\x21\xf9\x04\x04\x00\x00\x00\x00\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02'
              ,b'\x02D\x01\x00;\x00')
    
    # Image Descriptor
    yield from (b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D'
               b'\x01\x00;\x00\x04')
    
    # Color Table
    yield from ((b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
               b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'),
               *[b'\xff\xff\xff']*27,)
    
    # Image Data Block
    for idx in range(0, 122*122, 7):
        if idx > 0 and not idx % 122:
            continue
        
        color_idx = (~idx)&0xFF

        yield (((color_idx+1).to_bytes(length=1, byteorder='little'))
                 + ((~color_idx&0xFFFFFF)).to_bytes(
                     length=3, byteorder='little'))

        yield ((0).to_bytes(length=1, byteorder='little'))
        yield ((~color_idx).to_bytes(length=3, byteorder='little'))


    yield from (b'\x00\x00\x00\x00',
               b';')

if __name__ == '__main__':
    buffer = BytesIO()
    for i in generate_welcoming_gif():
        buffer.write(i)
    with open('example.gif', mode='wb+') as fd:
        fd.write(buffer.getvalue())