#!/usr/bin/env python3

import sys
import os
import argparse
import struct
from typing import Iterable, BinaryIO

DEFAULT_OUTPUT = 'random.gif'


class RandomGifGenerator():

    def __init__(self, args):

        parser = argparse.ArgumentParser()
        parser.add_argument('--width', type=int, default=50)
        parser.add_argument('--height', type=int, default=50)
        parser.add_argument('output', nargs='?', default=DEFAULT_OUTPUT)
        self._args = parser.parse_args(args[1:])


    @property
    def output(self): return self._args.output


    @property
    def width(self): return self._args.width


    @property
    def height(self): return self._args.height
    

    def generate_random_gif(self):
    	with open(f'./{self.output}', 'wb') as fh:
    	    generate_random_gif(open('/dev/urandom'), fh, int(self.width), int(self.height))



# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# correspond to the information found here:
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, 
        width: int, height: int
    ):

    # Signature
    header = b'GIF89a'
    
    # Width and Height
    w = struct.pack('<H', width)             # Little Endian 16bit
    h = struct.pack('<H', height)            # Little Endian 16bit
    header += w + h

    # Color Table size
    color_table = [bytes([rng.read(1)[0]]) for _ in range(768)]
    ctsz = len(color_table) // 3
    header += bytes([(ctsz)])              # TODO: How to calculate this?
    for ct in color_table:                  # Add each entry of the color table to
        header += ct                       # our final header byte string

    # Image descriptor
    id = struct.pack('B', width % 256)      # Least significant byte first
    id += struct.pack('B', width >> 8)       # Most significant byte next
    id += struct.pack('B', height % 256)     # Least significant byte first
    id += struct.pack('B', height >> 8)      # Most significant byte next
    header += id                           # Append ID
    
    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]                     # block size is an unsigned char
    subblock = rng.read(bs - 1)              # discard sub-block marker
    header += bytes([bs])                    # add block size back again
    data = rng.read(bs)                      # get actual image data

    # Image Separator
    out.write(b'\x2C')  

    out.write(header) 
    out.write(data)