#! /usr/bin/env python3

import os
import subprocess as sp
import struct

from typing import BinaryIO


def main():

    with open('/dev/urandom', 'rb') as r:

        while True:

            print('Generating a new image.')

            # The binary file we write to is deleted automatically when the context exits so no need for an explicit cleanup.
            with open('./img.gif', 'wb') as w:
                generate_random_gif(r, w);

            try:

                cmd = ('convert', './img.gif', '-background',
                       '#{0:x}{1:x}{2:x}'.format(*os.urandom(3)), '+matte', 'PNG32:')
                p = sp.Popen(cmd, stdout=sp.PIPE, stderr=sp.PIPE)
                stdout, _stderr = p.communicate()
                if len(_stderr):
                    raise Exception(_stderr)

                print(stdout[:50])
                print('')

            except FileNotFoundError:
                raise Exception("Couldn't find the `convert` program on your $PATH.")
            except KeyboardInterrupt:
                pass


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    width = (rng.read(1)[0] % 64) + 1
    height = (rng.read(1)[0] % 64) + 1
    
    outfile = BytesIO()
    global palette
    palette = [tuple((rng.read(1)[0] % 255,) * 3) for i in range(768)]

    def output_color_table(palette):
        return [
            struct.pack('<BBB', *c[0], *(0,)) for c in palette
        ]

    color_table_data = output_color_table(palette)
    out.write(struct.pack('<HHB', width, height, len(palette)))
    out.writelines(color_table_data)

    def output_image_descriptor(width, height):
        return struct.pack('<BBBBBBB', 0x2C, 0, 0, x, y, width - 1, height - 1, 0)

    x = y = 0
    out.write(output_image_descriptor(width, height))

    transparancy = True
    data = []
    while len(data) < (len(color_table_data)):
        
        next_byte = rng.read(1)[0]
        
        if ((next_byte & 0b00000001) >> 0 == 1 and not transparancy 
            or next_byte & 0b00000001 == 0 and transparancy):
            
            transparancy = not transparancy
            continue

        if transparancy:
            data += [(0, 0, 0)]
            continue

        data += list(palette[(next_byte << 8 | rng.read(1)[0]) % 768])

    out.write(bytes(data))
