#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out`, whose length is
# between the given lower and upper bounds (inclusive), each value
# being between [0, 256). A fixed header will be used for convenience.
def generate_random_gif(
        high: int, low:int=0, out:BinaryIO
    ):
    # Must use little-endian byte order to match existing examples!
    header = bytes([71, 73, 70, 56, 57, 97])

    if not ((low >= 0) & (high <= 256)):
        raise ValueError("Random values must lie in range [0, 256)")

    datalen = (high - low) // 4
    out.write(header + struct.pack('<L', datalen))
    
    for _ in range(datalen):
        pixelval = struct.unpack('>B',
            rng.read(1))[0] % (high - low + 1)
        out.write(bytes([pixelval]))