import os

from PIL import Image

from.utils import get_rng


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom or provided)
def generate_random_gif(
        rng: BinaryIO | None = None, out: BinaryIO = open('out.gif', 'wb')
    ):

    if rng is None:
        rng = get_rng()

    width = 256 # TODO: Make it configurable?
    height = 256 # TODO: Make it configurable?

    img_data = []
    for i in range(height):
        row = [int.from_bytes(
            rng.read(3), 
            byteorder='little', 
            signed=True
        )] * width
        img_data += row


    im = Image.new("RGB", (width, height))
    im.putdata(img_data)
    
    extension = "gif"
    try:
        from imageio import mimsave as save_gif
    except ImportError:
        try:
            import moviepy.editor as mpy

            def save_gif(f, images, duration=None, loop=0, dispose=None,
                         format="GIF"):
                clip = mpy.ImageSequenceClip(list(images),
                                             fps=(1000 / duration) if duration else None)
                clip.write_gif(f, verbose=False, progress_bar=False)

        except ImportError:
            print("[WARN] Neither imageio nor moviepy are installed."
                  " The animation won't be saved to disk.")
            save_gif = lambda f, x: None
            extension = "png"
    
    filename = f"out.{extension}" if not hasattr(out, 'name') else out.name
    
    save_gif(filename, [im])