#!/usr/local/bin/python3 -u
# -*- coding: utf-8 -*-

import argparse
import collections
import os
import struct
import sys


# Generates a random GIF file into `out` using the
# provided RNG. The size of the generated image is determined by
# the first four bytes read from the RNG (which are discarded).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    def write_chunk(
            chunktype: bytes,
            bufdata: bytes
        ):

        # Write the length field for this chunk...
        if len(bufdata) < 256:
            out.write(struct.pack('BB', 0x0b, len(bufdata)))
        else:
            raise Exception("Chunk too big")
        
        #...and the data itself.
        out.write(chunktype)
        out.write(bufdata)

    header = b'GIF89a'
    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False
    ) & 0xffffff
    
    write_chunk(b'\x21\xf9\x04\x00\x5d\xff\x00', [0])
    write_chunk(b'\x00\x00\x00\x00\xff\xff\xff', [0])

    # We want to make sure that we have an even number of chunks
    # so let's split it in half and then merge them back together at
    # the end.
    nchunks = (datalen+2)//257
    chunks = []
    for _ in range(nchunks):
        chunklen = min(rng.read(1)[0], 257)
        chunktype = rng.read(chunklen)
        chunkdata = rng.read(257-chunklen)
        chunks.append((chunktype, chunkdata))

    for i,(chunktype,chunkdata) in enumerate(chunks[::-1]):
        write_chunk(chunktype, chunkdata)

    # Merge all the chunks together again.
    out.write(header)
    for _,chunkdata in chunks:
        out.write(chunkdata)


