#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given pseudorandom number generator in binary format
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    header = 'GIF89a'.encode('ascii')

    # Logical Screen Descriptor
    width = int.from_bytes(rng.read(2), byteorder='little') % (1<<16-1) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1<<16-1) + 1
    flags = [bool(int.from_bytes(rng.read(1), byteorder='little')) for _ in range(7)]

    globalcolortablesize = int.from_bytes(rng.read(1), byteorder='little')
    backgroundindex = int.from_bytes(rng.read(1), byteorder='little')
    pixelaspectratio = int.from_bytes(rng.read(1), byteorder='little')
    # TODO: The above is not actually part of LSD, it's just reserved space for future extensions

    localcolortablesize = min(max(globalcolortablesize, 1), 256)

    # Global Color Table
    globalcolor = []
    bgcolor = None

    globalcolortable = None if globalcolortablesize == 0 else bytes([
        0xFF * bool(int.from_bytes(rng.read(1), byteorder='little'))
        for i in range(3*globalcolortablesize)])
    globalcolortable += bytes([0] * ((globalcolortablesize - len(globalcolortable)) // 3))
    bgcolor = globalcolor[backgroundindex]

    # Image Descriptor
    left = int.from_bytes(rng.read(2), byteorder='little') % (1<<16-1)
    top = int.from_bytes(rng.read(2), byteorder='little') % (1<<16-1)
    width = int.from_bytes(rng.read(2), byteorder='little') % (1<<16-1) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1<<16-1) + 1
    flags = list((lambda x: tuple(map(bool, reversed(x))))
                ((rng.read(1)[0] >> n) & 1
                 for n in range(8)))
    lctsize = max(min(int.from_bytes(rng.read(1), byteorder='little'), 256), 1)
    lctdata = bytes([
        0xFF * bool(int.from_bytes(rng.read(1), byteorder='little'))
        for i in range(3*lctsize)])
    lctdata += bytes([0] * ((lctsize - len(lctdata))/3))

    palette = [(r << 16) | (g << 8) | b for r, g, b in zip(*(iter(lctdata),)*3)]

    if True:
        transparencyflag = False

        if flags[0]:
            pass

        if flags[1]:
            raise NotImplementedError()

        if flags[2]:
            transparencyflag = True

            transparencymaskindex = int.from_bytes(rng.read(1), byteorder='little')
            transparentcolor = palette[transparencymaskindex]

        interlaceflag = False
        sortflag = False
        localcolortable = None if lctsize == 0 else bytestring([
            0xFF * bool(int.from_bytes(rng.read(1), byteorder='little'))
            for i in range(3*lctsize)])



    # Application Extension Block
    applicationextensionlabel = "NETSCAPE"
    applicationextensionidentifier = "\x0b\x00".encode('ascii')


    subblocksize = int.from_bytes(rng.read(1), byteorder='little')
    blocksdata = bytes([0xff ^ ord(rng.read(1).decode())
                        for i in range(subblocksize)])

    # Comment Extension Block
    commentextensionlabel = '\xfe'
    commentextensionidentifier = '\x01'

