from pathlib import Path
from timeit import default_timer as timer

import click

# Generates a random GIF file into `path` using the
# random number generator `rng` (`/dev/urandom`)
def generate_random_gif(
        path: str, rng: BinaryIO, datalen: int=None
    ) -> None:

    if not datalen or datalen < 0:
        datalen = 1<<26 # ~26MiB
    with open(path, 'wb') as f:
        start = timer()

        header = b'GIF89a'
        f.write(header)

        data = rng.read(datalen)
        f.write(data)

        end = timer() - start
        print('Generating {} took {:.5f} seconds'.format(path, end))


@click.command()
@click.argument("output-file", type=Path)
@click.option("--datalen", "-d", "datalen")
def main(output_file: Path, datalen):
    output_file = output_file / '{:.4}.gif'.format(datalen)

    with open('/dev/urandom', 'rb') as r:
        return generate_random_gif(
            output_file.as_posix(), r,
            datalen=int(datalen) if datalen else None
        )


if __name__ == '__main__':
    main()
