#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header of the GIF file
    out.write('GIF89a'.encode())

    width = (int.from_bytes(rng.read(1), byteorder='big') & 0x7F) + 1
    height = ((int.from_bytes(rng.read(1), byteorder='big') & 0x7F) << 8) | \
             (int.from_bytes(rng.read(1), byteorder='big') & 0xFF)
    bitdepth = (int.from_bytes(rng.read(1), byteorder='big') & 0x07) + 1
    color_flag = bool((int.from_bytes(rng.read(1), byteorder='big') & 0x80))
    have_colormap = not color_flag
    size_of_global_colormap = None if have_colormap else \
                             (((int.from_bytes(rng.read(1), byteorder='big') & 0x3F) << 4) |
                              (int.from_bytes(rng.read(1), byteorder='big') & 0x0F))

    transparent_idx = False if color_flag or not have_colormap else \
                      (bool(int.from_bytes(rng.read(1), byteorder='big') & 0x01))
    # The delay for each frame is set to between one and ten seconds
    delay_time = compare_bit_to_randomness(
            rng, 3)

    # Flags that we don't account for yet
    flags = {
            'global_color_table': color_flag,
           'sort_flag':          None,
            'color_resolution':   None,
            'pixel-aspect-ratio': None,
            }

    # Write global color table flag
    out.write(int(have_colormap).to_bytes(1, byteorder='big'))
    
    if have_colormap:
        # Number of colors in colormap, 2 to 256 inclusive
        out.write(((size_of_global_colormap >> 8) & 0xFF).to_bytes(1, byteorder='big'))
        
        out.write((size_of_global_colormap & 0xFF).to_bytes(1, byteorder='big'))

        # Global colormap
        for i in range(size_of_global_colormap):
            rgb_triplet = rng.read(3)

            # Convert RGB values to local color map
            out.write(rgb_triplet)

    # Image descriptor section contains basic information about the image
    # We don't set any other properties yet though...
    out.write(width.to_bytes(2, byteorder='little', signed=False))
    out.write(height.to_bytes(2, byteorder='little', signed=False))
 
    out.write((bitdepth-1).to_bytes(1,'little',signed=False))
    out.write(int(not transparent_idx).to_bytes(1,'big',signed=False))
    out.write(delay_time.to_bytes(2,'little',signed=False))

    # LZW min code size
    out.write(int(second_value < 12).to_bytes(1, byteorder='big'))

    # Size of LZW compressed data
    out.write(len(compressed_data).to_bytes(2, byteorder='big'))

    # Finally write the LZW-compressed data
    out.write(compressed_data)


