#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
    This is a simple GIF generator script which can be used to test
    the rendering of animated gifs.
    Only the color table size, the block size and the pixel data are randomly
    generated each time the script is run.
    
    Author: <NAME> (<EMAIL>)
"""

# TODO: Use argparse instead of hardcoded defaults

DEFAULT_OUTPUT = 'random.gif'

color_table_size = 256   # Maximum value allowed by specification
block_size       = {
    1 : 255,   # smallest possible size
    2 : 4,      # largest possible size
}                   # Image Data Sub-Blocks of this length or smaller are accepted
                      # All other values will result in invalid files






class RandomGifGenerator():

    def __init__(self, output=None):
    
        if not output:
            self.output = DEFAULT_OUTPUT
        else:
            self.output = output


    def generate_random_gif(self):
    
        try:
            with open('/dev/urandom', mode='rb') as rng,\
                 open(self.output,    mode='wb') as out:

                return generate_random_gif(
                        rng, out
                    )
                
        except Exception as e: 
            print('[!] Error generating GIF:', flush=True)
            raise e
        

# Generates a random GIF file into `out` using the
# random number generator `rng`. The parameters
# are read from different sources to ensure that they're valid.
def generate_random_gif(rng: BinaryIO, out: BinaryIO) -> None:
    
    # Header
    header = b''
    header += b'GIF89a'            # File signature and version
    # Color Table size
    ctsz_list = [rng.read(1)[0]]
    while sum(ctsz_list) < color_table_size: 
        ctsz_list.append(rng.read(1)[0])
    ctsz = len(ctsz_list) // 3
    for i in range(3*ctsz):
        val = ctsz_list[i % len(ctsz_list)]
        if i >= 3*ctsz - len(ctsz_list):
            val &= ~128
        header += bytes([val])

    # Block Size (Data Sub-Block within Data Stream)
    bs = rng.read(1)[0]         # block size is an unsigned char
    subblock = rng.read(bs - 1)  # discard sub-block marker
    header += bytes([bs])        # add block size back again
    pixels = []                  # temporary list holding pixels
    byte_count = 7+sum(ctsz_list) # amount of image data bytes written so far
    while byte_count <= block_size[bs]: # keep adding until we have enough data
        rgb = tuple(rng.read(1)[0] & 0b11111 for _ in range(3))  
        lum = ((rgb[0]//2)+(rgb[1]//2)+(rgb[2]//2))+128
        pixels.extend((lum,)+rgb+(0,)) # R G B Luminance 0
        byte_count += 3              # increment count per RGB triplet
    pixels[-1] |= 0x01 # section disposal method / transparency flag: no disposal
    header += bytes(pixels[:-1]) # write pixels minus termination byte
    out.write(header + bytes(pixels))# write it all together