#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate(out: BinaryIO, rng):
    # GIF header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    print("Logical Screen Descriptor", end=' ', file=out)

    global_width = 640
    global_height = 240
    colors = 1 << 8
    screen_background = 0xff
    pixel_aspect = 0
    out.write(struct.pack('!HBB', global_width, global_height, (colors - 1) | (screen_background << 7)))
    if pixel_aspect != 0:
        print('pixel aspect={}'.format(pixel_aspect), end=' ', file=out)
        out.write(bytes([pixel_aspect]))
    else:
        print()

    print("\tGlobal Color Table: {} colours".format((colors - 1)), file=out)

    def randrange(rng, start, stop):
        return struct.unpack('<L', rng.read(4))[0] % (stop - start + 1) + start

    for _ in range(colors):
        r = round(randrange(rng, 0, 255))
        g = round(randrange(rng, 0, 255))
        b = round(randrange(rng, 0, 255))
        print('\t\t{}: {:02x}{:02x}{:02x}'.format(_ + 1, r, g, b), file=out)
        out.write(struct.pack('BBB', r, g, b))

    # Graphics Control Extension
    print("Application Extension: Netscape 2.0\n\tbodystart=1", file=out)
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
    
    # Image Descriptor
    print("Image Descriptor", end=' ', file=out)
    local_width = global_width
    local_height = global_height
    x = 0
    y = 0
    local_color_table = True
    interlaced = False
    print('local_width={}, local_height={}, left={}, top={}, '
            .format(local_width, local_height, x, y)
          , end="", file=out)
    if local_color_table:
        print("local color table: ", end="")
        code_size = 8
        print(code_size, "bits", end=" ")
        out.write(bytes([(code_size - 1)] + [0] * code_size))
        out.write(struct.pack('!HH', local_width, local_height))
        
        for _ in range(1 << code_size):
            r = round(randrange(rng, 0, 255))
            g = round(randrange(rng, 0, 255))
            b = round(randrange(rng, 0, 255))
            print('\t\t{}: {:02x}{:02x}{:02x}'.format(_ + 1, r, g, b), file=out)
            out.write(struct.pack('BBB', r, g, b))
            
    else:
        print("no local color table", end=" ")
    if not