#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  
    framesize = len(header)+1+2*3+4
    if (framesize%255) != 0:
        padding = (int((framesize/256)) + 1)*255 - framesize
    else:
        padding = 0
        
    out.write(b'\x00\xff') # Extension block
    out.write(struct.pack(">H",framesize))
    out.write(header)
    colortablelength = 7
    out.write(struct.pack("<BHHBB",colortablelength+1,0,0,0))
    for i in range(0,colortablelength+1):
        val= 0xff | i << 8 | i<<16 | i<<24
        out.write(struct.pack(">I",val))
    out.write('\x00') # End of extension block
    
    out.write(b'\x00') # Image separator
    out.write(struct.pack(">H",datalen))
    out.write(struct.pack(">BB",0,0))
    out.write(struct.pack(">H",0xffff))
    out.flush()
    
    # Write data
    while datalen > 0:
        chunklen = min(datalen,255)
        out.write(struct.pack("<B",chunklen))