#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Generate some data for the body
    datalen = int.from_bytes(
            rng.read(2), 'big') + 1
    data = rng.read(datalen * 4)

    # Choose a width and height for our image
    width = int.from_bytes(rng.read(2), 'big') % 65535
    height = int.from_bytes(rng.read(2), 'big') % 65535

    # Next we have to choose a color table
    # Each table is of size 3, so multiply by three
    tablelen = int.from_bytes(rng.read(2), 'big') * 3

    # Read in the whole color table...
    colortable = rng.read(tablelen)

    # ... then split it up into sub-tables of each 256 colors
    colortables = [colortable[:768], colortable[768:] * 2]

    # Now write out the other fields of the header...
    # I'm not sure how many are actually important, but
    # they don't seem to be strictly necessary...
    reserved = rng.read(3)
    backgroundcolorindex = rng.read(1)
    pixelaspectratio = rng.read(1)

    # Write these to the output stream
    out.write(reserved + backgroundcolorindex + pixelaspectratio)

    # Go through each of the frames - the first frame is the header...
    for i in range(int.from_bytes(rng.read(2), 'big')):
        # Skip over the frame sequence number field...
        rng.seek(2, os.SEEK_CUR)

        # Then skip the local color table if one exists...
        flags = rng.read(1)[0]
        tablemask = 1 << 7
        if flags & tablemask != 0:
            tablestart = rng.tell()
            tablesize = int.from_bytes(rng.read(1), 'big')
            rng.seek(tablesize * 3, os.SEEK_CUR)

        # The next two bytes are the x and y coordinates of the top left corner
        rng.seek(2, os.SEEK_CUR)

        # And finally the width and height
        w = int.from_bytes(rng.read(2), 'big')
        h = int.from_bytes(rng.read(2), 'big')

        # Decide which color table we should use...
        tableidx = flags & tablemask == 0

        # We're going to need this later to decide what to do with each byte
        bitdepth = ((flags >> 4) & 7) + 1

        # Go through each row of the image
        for y in range(h):
            # First figure out how many bytes we will have in this row
            numbytes = (w * bitdepth + 7) // 8

            # Then go through each byte in this row
            for x in range(numbytes):
                # Calculate our position within the color table -
                # we can just index into the data here as long as
                # we know how big all the sub-tables are
                idx = colortables[tableidx][data[y * numbytes + x]]

