#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    color_table_length = bool(lsd[4] & 128)

    if color_table_length != 0:
        # If there is a global color table, read it in and write to output
        color_table = rng.read(3 * (2 ** ((lsd[4] & 7) + 1)))
        out.write(color_table)

    while True:
        # Get the image separator; first byte of image data block
        code = rng.read(1)[0]

        # If we get an end of stream marker, return
        if code == 0x3B:
            break
        elif code != 0x2C:
            raise ValueError("Something went wrong")
        
        # Read in the image data block
        size = int.from_bytes(rng.read(3), 'big')
        if size < 0 or size > 65535:
            continue

        imgdata = rng.read(size)
        out.write(imgdata)