#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1 byte signature
    assert rng.readinto(out.write, b'GIF') == 3

    # 3 bytes version
    assert rng.readinto(out.write, b'\x38\x9b\x00') == 3

    # 2 bytes screen width
    width = struct.unpack('>H', rng.read(2))[0] % 65536 + 1
    out.write(struct.pack('<H', width))

    # 2 bytes screen height
    height = struct.unpack('>H', rng.read(2))[0] % 65536 + 1
    out.write(struct.pack('<H', height))

    # Write 7 bits of color depth, set "color table is sorted" flag
    byte = struct.unpack('B', rng.read(1))[0] & 0b1111111
    out.write(struct.pack('B', byte | (1 << 7)))

    # Write 3 bits background color index
    byte = struct.unpack('B', rng.read(1))[0] & 0b11100000
    out.write(struct.pack('B', byte | (byte >> 3)))

    # Zero bits of pixel aspect ratio
    out.write(struct.pack('BB', 0, 0))

    # 1 byte global color table flag and color table size
    out.write(struct.pack('B', (1 if rng.read(1)[0] > 128 else 0) << 7 | 8))

    # Initialize palette to all black for easy visualization in debuggers
    init_palette_bytes = [0] * (width * height)
    while True:
        block_size = rng.read(1)[0]
        if block_size == 0xff:
            # End of gif marker
