#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Magic signatures
    signature = b"GIF87a"
    header_length = len("GIF87a") + 12
    gct_length = 0
    sort_flag = 0
    color_resolution = 7
    background_color_idx = 0
    pixel_aspect_ratio = 0
    num_colors = 2 ** (1 + int.from_bytes(gct_length, "big"))

    # Header
    out.write(signature)
    out.write((header_length).to_bytes(2, "big", signed=False))
    out.write(struct.pack("<BBBbHHHBB", *[
            gct_length, sort_flag, color_resolution,
            background_color_idx,
            pixel_aspect_ratio, num_colors]))
    # Global Color Table

    # Image Descriptor Block
    image_separator = b"\x2c"
    left_pos = 0
    top_pos = 0
    width = 2**16 - 1
    height = 2**16 - 1
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    local_color_table_size = 0
    lct_length = 2**(1 + int.from_bytes(local_color_table_size, "big"))
    num_colors = 2**(1 + int.from_bytes(lct_length, "big"))
    out.write(image_separator)
    out.write((left_pos).to_bytes(2, "big", signed=False))
    out.write((top_pos).to_bytes(2, "big", signed=False))
    out.write((width).to_bytes(2, "big", signed=False))
    out.write((height).to_bytes(2, "big", signed=False))
    out.write(struct.pack("<BBB", *[
            local_color_table_flag,
            interlace_flag, sort_flag]));
    # Local Color Table Block
    if local_color_table_flag == 1:
         for _ in range(num_colors):
             r = rng.read(1)[0] % 256
             g = rng.read(1)[0] % 256
             b = rng.read(1)[0] % 256
             out.write(struct.pack("<BBB", r, g, b))
             out.write(struct.pack("<", rng.read(1)))
    else:
        out.write(struct.pack("<", rng.read(1)))
    # Pixel Data
    pixel_data_block = b""
    while True:
        min_code_size = rng.read(1)[0] % 256
        byte = rng.read(1)[0] % 256
