#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)
    assert len(lsd) == 7, f"Could not read 7 bytes from LSD header"

    # Prepend the width/height of the image
    lsdbuf = struct.pack('<HH', *struct.unpack(">BB", lsd[:2]))
    lsdbuf += lsd[2:]

    # Add the LSD to the output
    out.write(lsdbuf)

    # Process remaining fields in the GIF file
    while True:
        blocksz = rng.read(1)
        if len(blocksz) != 1:
            return
