#!/usr/bin/env python3

import sys
from pathlib import Path
import os
import re
import string
import gzip
import json

def main():
    conf = load_conf()

    with open(sys.argv[1], 'r') as f:
        lines = f.readlines()[-20:-1]
    
    parse_it = False
    for i, line in enumerate(lines):
        if "Node" in line and "Info" in line:
            parse_it = True
        
        if not parse_it: continue

        line = line.strip()

        # exclude non-node lines from processing
        try: int(line)
        except ValueError: continue

        node = line.split(" ")[1]
        IP_addr_list = []
        excluded_IPs = set([])

        for ip in node_info[node].keys():
            IP_addr = ip.split('/')[0]
            prefix = ip.split('/')[1]
            
            if str(prefix) == "24":
                IP_addr_list.append(IP_addr)
                
            else:
                ip = ip.replace(':', '')
                parts = ip.split(".")
                base = '.'.join(parts[:3])
                excluded_IPs.add(base + ".1")

                for j in range (1, 256):
                    if not str(j) in excluded_IPs:
                        IP_addr_list.append(base + "." + str(j))

        print ("%s %s" % (node, (' '.join(IP_addr_list)).strip()))

    return 0

def load_conf():
    with open("/etc/node_exporter/config.json", 'r') as f:
        conf = json.load(f)
         
    hostname = str(os.uname().nodename)

    node_info = {}
    for target in conf['targets']:
        if hostname in target or re.search(hostname, target):
            split = target.split("@")