#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # String signifying GIF version 89a, which has no extensions
    gif_version = b"GIF89a"
    # GIF color table with one entry
    color_table = [0] * (1 << 5)
    color_table[0] = 255<<24 | 255<<16 | 255<<8 | 255
    # GIF image descriptor with left-origin and top-origin
    label_image_descriptor = struct.pack("<BHHBB", 0x2C, 0, 0, 0, 0)
    # GIF graphic control extension for transparency
    label_graphic_control = struct.pack("!BHHBBHB", 0x21, 0xF9, 0x04, 0, 0, 0, 0)
    # GIF application block for Netpbm's ppm reading code
    label_netpbm_block = struct.pack("<BHI", 0xFF, 0xFE, 0x03)

    # Write the header
    print("Writing GIF header")
    out.write(gif_version + "!\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00".encode())

    while True:
        # Pick the value to write across all the LZW codes
        print("Reading random RGB value in range [0, 255]")
        rgb = bytes([next(rng) % 256])*3
        
        # Start an image with this single pixel
        print("Starting new GIF frame")
        out.write(label_image_descriptor)
        # Write out the color table size
        out.write(struct.pack("!B", len(color_table)-1))
        # Dump the color table
        out.write(bytes(color_table))
        # Dump the code for this pixel as an uncompressed string of length one
        out.write(struct.pack("!H", 0x0000 | next(rng) % len(color_table)))

        # Finish the image by writing a transparent graphic control extension
        # indicating that the next frame is independent of this one, then finishing
        # it off with a trailer byte