#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff
    data   = rng.read(datalen)

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\xff\xff\x87\x64')

    palette = b''
    for i in range(1 << 8):
        palette += bytes([i >> 2])
        palette += bytes([i >> 1])
        palette += bytes([i    ])
        
    total_pixels = (1 << 16) - 1
    for row in range(1 << 8):
        buffer = f'{row:03}'
        rowlen = len(buffer) * 3 + 1

        if row == 0:
            color = bytes([i & 0xff for i in range(1 << 8)])
        else:
            color = rng.read(total_pixels // 3)

        pos = 0
        for col in range(rowlen):
            buffer[col] = str((int(buffer[col], 16) ^ color[pos]))
            pos += 1

        for c in buffer:
            out.write(bytes([int(c, 16)]))
            
    out.write(palette)
    while len(data) > 0:
        out.write(data[:255