#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)

    # Frame header
    out.write(header)
    # Image dimensions (plus some padding)
    width = int.from_bytes(rng.read(2), 'big') + 1
    height = int.from_bytes(rng.read(2), 'big') + 1
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))

    # Background color
    out.write(b'\xff\xff\xff')
    # Pixel aspect ratio (unused)
    out.write(b'\x0\x0\x0')

    # Graphic control extension
    gceflags = rng.read(1)[0]
    gcetimer = int.from_bytes(rng.read(2), 'big')
    gcexists = bool(gceflags & 0x1)
    delay = bool(gceflags & 0x4)
    disposal = gceflags >> 2 & 0x7

    if gceexists:
        extsize = 4 + len(rng.read(datalen))
        out.write(struct.pack('<BHBB', 0x21, extsize, 0xf9, gceflags))
        out.write(gcetimer.to_bytes(2, 'big'))

    # Image descriptor
    imgdesc = struct.pack('<BHHHBB', 0x2c, 0, 0, width, height, 0)
    out.write(imgdesc)

    # Local image palette (if any)
    if len(rng.read(1)) > 0:
        numcolors = int.from_bytes(rng.read(1), 'big') + 1
        out.write((numcolors - 1).to_bytes(1, 'big'))
