#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Check if this is an animated gif
    if lsd[0] & 1 == 0:
        return

    # We have an animated gif, so we need to parse each frame
    while True:
        
        # Get the graphic control extension (GCE)
        label = bytes([i for i in rng])
        if label != b'\xF9':
            raise Exception('Missing GCE')
        
        gce_size = rng.read(1)[0]
        gce = rng.read(gce_size)
        out.write(label + gce_size + gce)
        
        # Read the Block Label
        block_label = bytes([i for i in rng])
        if block_label != b'\x21':
            raise Exception('Expected BLACK')
                
        # Get the Image Descriptor (ID) field
        id_field = rng.read(8)
