#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)

    # Frame header.
    gcthdr = {
        1 : (1 << 4),   # 2 colours
        2 : (3 << 4),   # 4 colours
        3 : (7 << 4),   # 8 colours
        4 : (15 << 4),  # 16 colours
        5 : (31 << 4),  # 32 colours
    }[int.from_bytes(rng.read(1), 'big')] | 0b10000000

    print('gcthdr',gcthdr)

    # Number of bits per colour.
    colbits = int.from_bytes(rng.read(1), 'big')
    print('colbits',colbits)

    # Background and aspect ratio.
    bgnd = int.from_bytes(rng.read(1), 'big') % 256
    aspect = int.from_bytes(rng.read(1), 'big') >> 7

    print('bgnd',bgnd,'aspect',aspect)

    # Global table.
    if gcthdr & 0b10000000 != 0:
        # Get the length of the global colour table
        numcols = 1 << ((gcthdr & 0b00001111) + 1)
        print('numcols',numcols)
        for _ in range(numcols):
            cols = [int.from_bytes(rng.read(1), 'big'), ] * 3
            out.write(struct.pack('BBB', *cols))
            # print(cols)

    # Disposal method and user input flag.
    disposal = rng.read(1)[0] % 8
    close = rng.read(1)[0] >> 7

    print('disposal',disposal,'close',close)

    # Image left edge.
    leftoffset = int.from_bytes(
            rng.read(2), byteorder='big') & 0xffff
    print('leftoffset',leftoffset)

    # Image top edge.
    topoffset = int.from_bytes(
            rng.read(2), byteorder='big') & 0xffff
    print('topoffset',topoffset)

    # Width and height.
    width = int.from_bytes(
            rng.read(2), byteorder='big') & 0xffff
    print('width',width)
    height = int.from_bytes(
            rng.read(2), byteorder='big') & 0xffff
    print('height',height)

    # Local palette flags and packed field length.
    lpflags = gcthdr = rng.read(1)[0]
    print('lpflags',lpflags)
    pflength = rng.read(1)[0]
    print('pflength',pflength)

    # LZW code size.
    codesize = rng.read(1)[0]
    print('codesize',codesize)

    # Image data.
    lzwdata = rng.read(width*height)
