#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
import random

VERSION = 1
MAGIC = b'TSPC'

if len(sys.argv) == 2:
    inputfile = sys.argv[-1]
else:
    print("Usage: %s <input file>" % (sys.argv[0]))
    exit(-1)

# Generate a random seed from the input file
seed = 0
with open(inputfile, "rb") as fd:
    for i in range(4):
        seed <<= 8
        seed |= ord(fd.read(1))
random.seed(seed)
print("Seed is %x" % (seed))

rng = random.Random(seed)

out = open('tspc-' + inputfile, 'wb')

out.write(MAGIC)
out.write((VERSION & 0xff).to_bytes(1, 'big'))

# Write out enough bytes of data to fill up the file
while True:
    # Get the length of the next piece of data
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    # Append that data to the output stream and close it
    out.write(len(data).to_bytes(2, 'big'))
    out.write(data)
    out.close()