#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)

    # Frame header
    out.write(header)
    # Image dimensions
    width = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    height = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))

    # write frame data

    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                        rng.read(4), byteorder='big') & 0xff) + 1).to_bytes(1, "big"))

    # image trailer
    out.write(b'\x00')

if __name__ == "__main__":
    with open("/dev/urandom", "rb") as f:
        for _ in range(2**7):
            generate_random_gif(f, sys.stdout)
\end{code}

The PoC code is as follows:

\begin{code}
import gif
with open("myawsomegif.gif","w+") as f:
    for _ in range(2**7):
        f.seek(0)
        gif.generate_random_gif(open('/dev/urandom'), f)
\end{code}

I hope this helps!

Answer: I think you need to use `little endian` instead of big-endian. Big-endian means that the most significant bit goes at the left side while little-endian puts the most significant bit on the right side.

You can check more about it here: https://docs.python.org/3/library/struct.html?highlight=byteorder#byte-order-size-and-alignment

Comment: Thanks for your reply! I tried different combinations but none worked so far. The ImageMagick policy says that all numbers are stored in big endian format and only strings or other types are stored as little endian. This would mean that my interpretation of the documentation on this topic is wrong...

Comment: @x3n0 It seems like the issue is the structure of the GIF files. When you decode them they are not in the same order as the bytes were stored. [This link](https://www.fileformat.info/format/gif/egff.htm) might help you understand how GIFs work better.

It also looks like you are overwriting the last byte written by writing again in the same position. Would something like this work?

```python
    out.write(((int.from_bytes(
                    rng.read(4), byteorder='big') & 0xff) %
                 256)).to_bytes(1, "big"))
