#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):
    # Magic bytes (4B1h)
    out.write(b"GIF89a")

    # Logical screen descriptor
    width = rng.read(2)
    height = rng.read(2)
    packed = bytearray([0x80])
    out.write(bytes([*width, *height, *packed])) # Background color index
    bits = ord(rng.read(1)) % 8 + 1
    out.write(chr(bits).encode('ascii'))                # Pixel aspect ratio

    # Application block
    app_len = 12 if ord(rng.read(1)) > 0 else 0
    app_id = "net{0}".format(ord(rng.read(1)))
    app_auth = "{0}{1}{2}".format(*[rng.read(1) for _ in range(3)])
    app_data = [rng.read(1) for _ in range(app_len - 10)]
    if len(app_data) < 1:
        app_data = [0]
    out.write((app_len).to_bytes(2, 'little'))           # Block size
    out.write("NETSCAPE2.0".encode('ascii'))             # App ID
    out.write(app_auth.encode('ascii'))                  # Auth code
    out.write(struct.pack("H", app_data[0]))              # Loop count  
    out.write(bytes(app_data[1:]))                       # Data subblocks
    
    # Graphic control extension
    bg_color_idx = chr(ord(rng.read(1)) % 256).encode('ascii')
    delay_time = rng.read(2)
    transp = rng.read(1)
    block_size = bytearray([0xF9, 0x04, *delay_time, *transp, *bg_color_idx])
    out.write(bytearray([0x21, 0xf9, 0x04]))             # Extension introducer
    out.write(bytes(block_size))                         # Size and data

    # Image description
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    local_color_table = b"\x00"
    interlace = b""
    packed = bytearray([0x00])
    out.write(bytearray([0x2C, *left, *top, *width, *height, *local_color_table, *interlace, *packed]))

    # Graphic renderer
    pixels = []
    for row in range(struct.unpack("<H", height)[0]):
        for col in range(struct.unpack("<H", width)[0]):
            pixel = rng.read(3)
            pixels.append(pixel)
    out.write(bytes(pixels))

    # Trailer
    out