#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)

    # Frame header
    out.write(header)
    # Image dimensions
    width = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    height = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))

    packedfield = rng.read(1)[0]
    if packedfield == 0:
        localcolor = True
        sortflag = False
        interlaceflag = False
    else:
        localcolor = (packedfield >> 7) % 2 > 0
        sortflag = (packedfield >> 6) % 2 > 0
        interlaceflag = (packedfield >> 5) % 2 > 0
    
    print('localcolor',localcolor,'sortflag',sortflag,
          'interlaceflag',interlaceflag)
        
    background = int.from_bytes(
            rng.read(1), byteorder='big') & 0xff
    pixelaspectratio = int.from_bytes(
            rng.read(1), byteorder='big') & 0xff
    globalcolor = [
            int.from_bytes(rng.read(1), byteorder='big'),
            int.from_bytes(rng.read(1), byteorder='big'),
            int.from_bytes(rng.read(1), byteorder='big')]
    if localcolor:
        ncolors = (int.from_bytes(
                rng.read(1), byteorder='big')) & 0xff
        bgb = [
                int.from_bytes(rng.read(1), byteorder='big'),
                int.from_bytes(rng.read(1), byteorder='big'),
                int.from_bytes(rng.read(1), byteorder='big')]
        fgb = [
                int.from_bytes(rng.read(1), byteorder='big'),
                int.from_bytes(rng.read(1), byteorder='big'),
                int.from_bytes(rng.read(1), byteorder='big')]
    else:
        bgb = globalcolor[:]
        fgb = globalcolor[:]
                
    print('background',background,'pixelaspectratio',
          pixelaspectratio,'globalcolor',globalcolor[0],
          globalcolor[1],globalcolor[2])
    print('bgb',bgb[0],bgb[1],bgb[2],'fgb',fgb[0],fgb[1],fgb[2])
            
    out.write([packedfield])
    out.write([background])
    out.write([pixelaspectratio])
    for color in globalcolor:
        out.write([color])
    if localcolor:
        out.write([ncolors])
        for color in bgb:
            out.write([color])
        for color in fgb:
            out.write([color])
    
    # Graphics control extension
    out.write(b'\x21\xf9')
    blocksize = (int.from_bytes(
            rng.read(1), byteorder='big')) & 0xff
    flags = int.from_bytes(
            rng.read(1), byteorder='big') & 0xff
    delaytime = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xff
    transcolorindex = int.from_bytes(
            rng.read(1), byteorder='big') & 0xff
    out.write([blocksize])
    out.write([flags])
    out.write([delaytime])
    out.write([transcolorindex])
    # Image data
    out.write(b'\x2c')
    out.write([width])
    out.write([height])
    packedfield = rng.read(1)[0]
    if packedfield == 0:
        localcolor = True
        sortflag = False
        interlaceflag = False
    else:
        localcolor = (packedfield >> 7) % 2 > 0
        sortflag = (packedfield >> 6) % 2 > 0
        interlaceflag = (packedfield >> 5) % 2 > 0

    print('localcolor',localcolor,'sortflag',sortflag,
          'interlaceflag',interlaceflag)
      
    if localcolor:
        ncolors = (int.from_bytes(
                rng.read(1), byteorder='big')) & 0xff
        colors = []
        for _ in range(ncolors):
            colors.append([
                int.from_bytes(rng.read(1), byteorder='big'),
                int.from_bytes(rng.read(1), byteorder='big'),
