#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):
    # Start with the header
    out.write(b'GIF87a')  # signature (6 bytes)
    rng.readinto1(out, 2)                     # logical screen width
    rng.readinto1(out, 2)                     # logical screen height
    out.write(bytes([0xF0]))                  # background color index
    out.write(bytes([0]))                     # pixel aspect ratio

    # Write the global palette
    for _ in range(3 * 2 ** rng.read1()):
        rng.readinto1(out, 2)

    # Write the image block
    out.write(b'\x2C\x00\x00\x00')           # image separator, left, top, width, height
    out.write(bytes([0xF0]))                  # local color table flag + interlaced flag
    out.write(bytes([0]))                     # sort flag
    out.write(bytes([0]))                     # local color table size
    rng.readinto1(out, 2)                     # x
    rng.readinto1(out, 2)                     # y
    rng.readinto1(out, 2)                     # width
    rng.readinto1(out, 2)                