#!/usr/bin/env python3

import sys
from math import sin, cos, pi

RADIUS = 1680 / 2
SCALE = 45

if len(sys.argv) != 2:
    raise ValueError("usage: genimg OUT_FILE")

def cartesian(r, t):
    x = r * cos(t)
    y = r * sin(t)
    return (int(x), int(y))

with open(sys.argv[1], 'wb') as out:
    header = b'\xff\xd9' # JPEG EOI marker
    data = bytes()
    
    for i in range(-SCALE, SCALE + 1):
        dtheta = pi / -SCALE
        theta = i * dtheta
        
        coords = list(cartesian(RADIUS, theta))
        coords.extend((0, 0))
        
        for j in range(-SCALE, SCALE + 1):
            dtheta = pi * 2 / SCALE
            theta += dtheta
            
            coords.extend(cartesian(RADIUS, theta))
    
    repl = 1 << 7
    data = bytearray()
    for i in range(0, len(coords), 2):
        p1 = coords[i]
        p2 = coords[i+1]
        
        dx = abs(p1[0]) ^ abs(p2[0])
        dy = abs(p1[1]) ^ abs(p2[1])
        l = max(dx, dy)
        if l > 127:
            l -= 256 * ((l // 128) + 1)
        
        a = angle = round(pi * (atan2(p1[1] - p2[1], p1[0] - p2[0]) % pi) / pi)
        while angle < 0:
            angle += 256
        while angle >= 256:
            angle -= 256
        
        assert a == angle
        
        delta_a = int((angle - 128) ** 3) & 0xff
        delta_l = l & 0x7f
        v = ((delta_a << 7) | delta_l) & 0xffff
        
        data.append(v >> 8)
        data.append(v & 0xff)
        
    out.write(header + data)
    print(f"generated {len(data)}-byte image")
