#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)

    # Frame header
    out.write(header)
    out.write((9<<4 | 0).to_bytes(1, 'big')) # Width and height
    out.write(b'\0\0') # x and y offset
    out.write(b'\0')   # is global palette
    out.write(b'\0')   # color map size
    out.write(b'\0')   # sort method

    # Color table (all white)
    for _ in range(7):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
