#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        out: BinaryIO,
        rng: BinaryIO):
    
    # The first 6 bytes are our magic string "GIF89a"
    out.write("GIF89a".encode('ascii'))

    # The next 4 bytes are the width and height of the image
    w = struct.unpack('<H', rng.read(2))[0] + 1
    h = struct.unpack('<H', rng.read(2))[0] + 1
    out.write((w & 0xFF).to_bytes(1, 'little'))
    out.write((h & 0xFF).to_bytes(1, 'little'))

    # Next byte is the global palette flag
    flags = struct.unpack('B', rng.read(1))[0]
    if (flags >> 7) == 0:
        out.write(b'\x00')
    else:
        out.write(b'\x01')

    # Next 3 bits are reserved; skip them
    # Write the background color index
    bgidx = struct.unpack('B', rng.read(1))[0] % 256
    out.write(bgidx.to_bytes(1, 'little'))

    # Skip the aspect ratio byte for now
    rng.read(1)

    # Write the global color table
    colors = []
    for i in range(2**(flags&7)):
        colors.append(struct.unpack('BBB', rng.read(3)))
    out.write(struct.pack('%sB' % len(colors), *colors))

    # Write the Application Extension header
    appexthead = b"!\xf9\x04"
    appexthead += "NetHack".encode('ascii')
    appexthead += "\x00"*3
    out.write(appexthead)

    # We have no application data at all
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    # Append that data to the output stream and close it
    out.write(len(data).to_bytes(2, 'big'))
    out.write(data)
    out.close()