#!/usr/bin/env python3
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.


import os

from googleapiclient import discovery
from oauth2client.service_account import ServiceAccountCredentials
from airflow import models
from airflow.contrib.hooks.gcs_hook import GoogleCloudStorageHook
from airflow.operators.python_operator import PythonOperator
from datetime import datetime, timedelta
import logging



def upload(**kwargs):
    """Uploads a message to a Cloud Pub/Sub topic."""

    # [START composer_pubsub_tutorial_upload]
    PROJECT = kwargs['dag_run'].conf['project']
    BUCKET = kwargs['dag_run'].conf['bucket']
    PATH = kwargs['dag_run'].conf['path']

    credentials = ServiceAccountCredentials.from_json_keyfile_name(
        '/home/airflow/gcs/data_analytics-1345a86ff48d.json')

    pubsub = discovery.build('pubsub', 'v1', credentials=credentials)
    
    bucket_content = GoogleCloudStorageHook().download(BUCKET,PATH,'tmp/')



    project_id='data-analytics-1345'


    parent = f'projects/{project_id}/locations/us-west1'
    job = {
        'job': {
            'template_id': '5e71b7c4-2187-4c78-bd1f-ee3f2d7fc54b',
            'display_name':'my-etl-job',
            'labels': {},
           'environment': {
            'temp_location': f'gs://{BUCKET}',
            'zone':'us-west1-b',
           'max_workers': 10,
             'autoscaling_algorithm':'THROUGHPUT_BASED'
          },
            # Use the location of the template as the default regional endpoint.
           'region': 'us-west1'
        }
    }
    data_source = {'pubsub_topic':{'topic': f'projects/{PROJECT}/topics/raw_data'}}

    job['job']['placement']={'cluster_selector':{}}
    response = pubsub.projects().locations().jobs().create(parent=parent, body=job).execute()
    print(response)
    # [END composer_pubsub_tutorial_upload]