#!/usr/bin/env python3

import struct
from typing import BinaryIO
import sys


def print_usage():
  with open(__file__) as f:
    lines=f.readlines()

  for line in [line for line in lines if len([c for c in line if c=="="])==3]:
    print(line[:-1])
  exit(-1)



# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add global color table data to output
    out.write(b'\x00'*gctlen)

    # Now we can read the block size of each frame from input
    while True:
      # Read next byte from input
        b=rng.read(1)[0]

        # If it's not an extension introducer 
        # (21h), then just copy it over verbatim
        # but stop if we are getting EOF.
        if b!= 0x21:
          out.write(bytes([b]))
          continue
        
        else:
            break


    # We got an extension block starting now, so 
    # this is an application extension "XMP" block
    out.write(b'\xff\x0bXMP Data XMP\x00')
    
    # Read rest of xmp data from input
    xmpdata = rng.read(16*1024-19+32)

    # Make sure that the entire block is written by adding padding bytes at the end if necessary
    if len(xmpdata)%16!= 0:
        xmpdata += b'\x00'*(16 - len(xmpdata)%16) 

    # Write xmpdata to output
    out.write(xmpdata)

    # Check if there is more data or not (end of stream?)
    try:
        assert(rng.read(1)==b'')
    except AssertionError:
        print('input is too big!')
        raise
    return



if