#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)
    crc = get_crc16(header + data).to_bytes(2, 'big')

    # Write everything to output buffer
    out.write(header)
    out.write(struct.pack('>H', len(data)))  # Length of following blocks
    out.write(data)
    out.write(crc)

