#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Number of frames to write.
    num_frames = int.from_bytes(rng.read(2), byteorder="little", signed=False);

    # Write signature (GIF89a)
    out.write("GIF89a".encode('ascii'));
    
    # Logical screen descriptor
    out.write(struct.pack("<HHBBBHB",
        0x50, 0x01,
        int.from_bytes(rng.read(2), byteorder="little", signed=False),
        int.from_bytes(rng.read(1), byteorder="little", signed=False),
        7,
        0,
        b'\x08' if num_frames > 2 else b'\x00',
        0
    ));
        
    # Image descriptor
    out.write(struct.pack("<BHHBBH", 
        0x2C,
        0x00,
        0x00,
        int.from_bytes(rng.read(2), byteorder="little", signed=False),
        int.from_bytes(rng.read(1), byteorder="little", signed=False),
        0x08,
        0
    ));

    for _ in range(num_frames):
        
        # Frame header
        head = []

        # LZW minimum code size
        head.extend([
            0x21, 0xF9, 0x04, 
            int.from_bytes(rng.read(1), byteorder="little") | 
            0x10
        ]);

        # Graphics control block
        head.append(0x2c);

        # delay time
        head.extend([
            int.from_bytes(rng.read(1), byteorder="little"),
            int.from_bytes(rng.read(1), byteorder="little")
        ])

        # transparent color index
        head.extend(rng.read(1));

        # end of frame
        head.append(0x3b);
    
        out.write(bytearray(head))

        # image data length
        lzw_len = int.from_bytes(rng.read(2), byteorder="little", signed=False);

        # image data
        imgdata = rng.read(lzw_len + 1)[1:]

        out.write(imgdata);
