#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)
    footer = [b'\x00', b'\x3B'][
            0 <= int.from_bytes(
                rng.read(1), 'big') < 6]
    out.write(header + len(data).to_bytes(2, 'big') + \
          data + footer)


# Reads a GIF from `in_` and writes it to
# `out`, which should be /dev/stdout for this program.
def reshuffle(in_: BinaryIO, out: BinaryIO):

    # Read in the header
    header = in_.read(6)
    if (header!= b"GIF89a"):
        raise ValueError("Bad header")
    while True:

        # Figure out what type of block we're looking at
        char = in_.read(1)[0]
        print('char is ', char)
        if char == 0x2C or char == 0x21 or char == 0x3B:

            # Block is an extension
            length = read_block_size(in_)
            print('length is ', length)
            ext_type = in_.read(length)
            out.write((0x21 | char).to_bytes(1,'big'))
            write_block_size(length+1, out)
            out.write(ext_type)
            continue


        elif char == 0x2F:

            # Block is a trailer
            print('trailer found!')
            break

        else:

            # Block is neither an extension nor a trailer...
            assert False


        
        # The input has ended before the end of the image data...
        assert False


    # Write any remaining bits from the input stream
    out.write(in_.read())

# Writes the value `n` as two bytes prefixed by
# the least significant byte first (`lsb`)
def write_block_size(n: int, out: BinaryIO, lsb=True):
    nbytes = ((len(bin(n)) - 2) // 8) + 1
    packed = bin(n).lstrip('-').zfill(nbytes * 8)[:nbytes*8].encode()
    if not lsb:
        packed = packed[::-1]
    
    out.write(struct.pack('<H', int(packed, base=2)))

# Extracts a single sequence of zeroes, non-zero values,
# or an escape character followed by several more 8-bit characters
def read_block_size(in_: BinaryIO):
    result = 0
    while True:
        b = ord(in_.read(1))
        if not (b & 0x80):
            
            return result | b << (result % 7)*8
        else:
            result |= (b & ~0x80) << (result % 7)*8
        


