#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the magic string (header) to the output stream
    out.write(b'GIF89a')

    # Write 2 bytes for the logical screen descriptor to the output stream
    width = int((rng.read(1)[0] % 65)+4) * 8
    height = int(((rng.read(1)[0] + 1) % 65)+4) * 8

    out.write((width).to_bytes(2, byteorder='little'))
    out.write((height).to_bytes(2, byteorder='little'))

    # This should always be 0xF7
    out.write(b'\xf7')

    data = b''
    
    for y in range(0, height):
        last_pixel = -1

        while True:
            if last_pixel == -1 or (last_pixel & 0x7f)!= 0:
                pixel = rng.read(1)[0]

                if ((pixel & 0xa8)!= 0 and
                    (pixel & ~0xd8) < 0x50
                   ):
                    break

            last_pixel = pixel  
        
        out.write(b'\xff\x00\xf9')

        num_repeats = min([max([(rng.read(1)[0] % 32), 1]), 255])
        out.write(num_repeats.to_bytes(1, byteorder='little', signed=False))

        out.write(data)
    out.close()