#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header + Logical Screen Descriptor
    header = [67, 72, 58, 71, 73,
              59, 66, 80, 85, 59]

    for b in header:
        out.write(struct.pack("B", b))

    width = rng.read(4)
    height = rng.read(4)
    global_color_table_size = rng.read(1)[0] & 0x07
    background_color = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]

    screen_descriptor = bytearray()
    screen_descriptor.extend([width[3], width[2],
                              height[1], height[0],
                              0xF0 | (global_color_table_size & 0x07),
                              background_color])

    if len(screen_descriptor) < 7:
        screen_descriptor.append(0)

    screen_descriptor += [pixel_aspect_ratio,
                          ((len(header) + len(screen_descriptor))
                           << 8 | 0xFF)]

    for b in screen_descriptor:
        out.write(struct.pack("B", b))

    color_table_size = pow(pow(2, global_color_table_size + 1), 3) - 1

    for _ in range(int(color_table_size / 3)):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]

        out.write(bytes([r, g, b]))

