#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Read global color table if present
    if gctlen != 0:
      gct = rng.read(gctlen)
      out.write(gct)

    while True:
        blocktype = rng.read(1)[0]
        if blocktype == 0x3b: # stop marker
            break
        elif blocktype != 0x2c: # unknown block type
            continue

        # Read the rest of the extension
        extheader = rng.read(6)
        out.write(extheader)
        
        # Parse extension header
        hasgct   = (extheader[0] & 0x80) != 0
        hasimgdata = (extheader[0] & 0x08) != 0
        size    = int(struct.unpack(">H", extheader[2:])[0]) + 1
        # If there's no image data we're done here
        if not hasimgdata:
          continue

        # Write the local color table if necessary
        if hasgct:
          gctsize     = int(struct.unpack("B", rng.read(1))[0])
          gct         = []
          for i in range(gctsize):
             rgb                  = []
             for j in range(3):
                rgb.append(ord(rng.read(1)))
             gct.append(rgb)
          out.write(bytes([gctsize]))
          for item in gct:
              out.write(bytes([(item[i // 3] >> ((2 - i % 3)*8)) & 255
                               for i in range(9)]))

        # Determine duration
        # TODO: This is actually an unsigned integer but it seems like
        #       the maximum that can be stored in a byte is 255 so just use this?
        # dur = abs(float(rng.read(1))) / float(255)
        dur = ord(rng.read(1))

        # Read LZW min code size
        lzws = rng.read(1)
        out.write(lzws)

        # Read subblocks until end marker or max size reached
        lzwmax = 2**11-1
        lzwhash = [None]*lzwmax
        lzwval = None
        lzwcnt = 1
        lzwmincodesize = int(lzws)
        lzwnextcode = int(1 << lzwmincodesize)
        oldcode = None
        img = []
        idx = 0
        while True:
           while len(img) < size:
               val = ord(rng.read(1))

               if idx > 0:
                   code = val & lzwmask
                   if oldcode < lzwmax and \
                       lzwhash[oldcode] is not None:
                       while code > lzwmax:
                           nextbyte = ord(rng.read(1))
                           if nextbyte >= lzwmax:
                              raise SystemExit('Got bad argument!')
                           else:
                              code = code*lzwmax + nextbyte + 1

               	   if lzwhash[code] is not None:
                      substr = lzwhash[code].copy()
                    
                   else:
                      substr = []
                      if code >= lzwnextcode:
                          print('Bad?')
                          exit(-1)
                      if code < lzwmax and lzwhash[code]:
                         return 'Code error!'
                      while lzwnextcode <= code:
                         lzwhash[code] = None
                         lzwnextcode *= 2

           	       lzwhash[lzwnextcode] = substr
                   	  lzwnextcode += 1

                   	  lzwcnt += 1
                          lzwmincodesize = lzwmin((len(substr)+1).bit_length(),
                                                lzwmincodesize)
                  substring+=lzwval
                  lzwhash[code]=substring
                  lzwhash[oldcode]=""

                  if lzwcnt > lzwmax and lzwnextcode < 1<<lzwmincodesize+1:
                     newcodesize = lzwmincodesize+1
                     lzwmincodesize = lzwmin(newcodesize, lzwmincodesize)
                     for ii in range(lzwmax+1):
                        if lzwhash[ii]!= None:
                           substr = lzwhash[ii].copy()
                           while len(substr) > newcodesize:
                              lzwval = ord(substr[(newcodesize)])
                              if code > lzwmax:
                                 print('Error')
                                 exit(-1)
                              lzwhash[lzwnextcode] = substr[:newcodesize]
                              lzwhash[oldcode] = substr[newcodesize:]
                              lzwhash[lzwnextcode] = substring
                              lzwnextcode += 1
                              lzwcnt += 1
                              lzwmincodesize = lzwmin((len(substring)+1).bit_length(),
                                                    lzwmincodesize)
                              substr=substing
                         
                      lzwmax = 1 << lzwmincodesize-1

           img = img[:idx] + img[-idx:]

         

        