#!/usr/bin/env python3

import sys
from pathlib import Path
from typing import BinaryIO

import click


@click.command()
def main():
    path = Path('out').resolve().absolute()
    if not path.exists():
        path.mkdir()

    outfile = (path / 'random.gif')
    with open(outfile, mode='wb', buffering=0) as f:
        generate_gif(f)


# adapted from https://stackoverflow.com/a/16527147
def generate_gif(out: BinaryIO):
    """Generate a random GIF"""
    from Crypto.Random import get_random_bytes
    rng = get_random_bytes
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
    print(f"generated {len(data)}-byte image")
