#!/usr/bin/env python3

import struct
from typing import Tuple, List


def read_int(f: bytes) -> int:
    return struct.unpack("!I", f[:4])[0]


def find_blocksize(stream: bytes) -> int:

    for i in range(-1024 * 1024, len(stream)):
        block = stream[i:]

        pos = 0
        while True:
            if not (pos + 8 <= len(block)):
                break

            size = read_int(block[pos:])
            pos += size
            if pos >= len(block):
                break

        # print(f"block at {i} has size {pos}")
        if pos == len(block):
            return i
    raise RuntimeError()


# def find_matching_pattern(stream: bytes, pattern: str) -> Tuple[List[str], int]:

#     found = []
#     start = -len(pattern)

#     for s in range(start+1, len(stream)-len(pattern)+1):
#         chunk = stream[s : s + len(pattern)]
#         if pattern!= chunk:
#             continue

#         offset = []
#         for i in range(len(chunk)):
#             # each character should be a space or the original byte value
#             if ord(' ') == chunk[i]:
#                 pass
#             elif chr(ord('a') + i % 26) == chunk[i]:
#                 pass
#             else:
#                 break
#             offset.append((i % 26))

#         if len(offset) and all(((x - y) & 255) < 127 for x,y in zip(range(256), offset)):
#             # print(f"{s}: {offset}")
#             found.append((chr(sum([o << b for o,b in enumerate(reversed(offset))])).encode(), s-start))
#     return sorted([(k, v) for k,v in set(found)], key=lambda e: max(e[1], -e[1]))


def extract_header(stream: bytes) -> Tuple[bytes, bytes]:
    bs = find_blocksize(stream)
    assert stream[:bs].endswith(b"\n")
    hdr = stream[: bs + 1]

    assert hdr[-2:] == b'\r\n'
    assert hdr[:-2].strip().split(b"\r\n")[0][:-9]!= b''
    return hdr[:-2].strip(), stream[bs + 1 :]


def main():
    with open("../test/data/text.html", "rb") as f:
        h, t = extract_header(f.read())
        print(h.decode())
        print("-"*72)
        print(t.decode())


if