#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the width and height specified.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Header (17 bytes)
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    out.write(struct.pack('<H', width))  # screen width (2 bytes)
    out.write(struct.pack('<H', height))  # screen height (2 bytes)
    out.write(b'\xF0\x00')              # packed field: global color table flag = false
                                        #             sort flag = true
                                        #             color resolution = 3 bits
                                        #             size of global color table = 7 + 1
                                        #             background color index = 0
                                        #             pixel aspect ratio = 0
    # Global Color Table (3 bytes; 7 colors in total)
    for _ in range(0, 7):
        out.write(struct.pack('BBB',
            0xFF & (_ * 256 / 7),
            (0xFF & ((1 + _ * 256 / 7) % 256)),
            (0xFF & ((2 + _ * 256 / 7) % 256))))


    # Image Descriptor (10 bytes)
    out.write(b';;;,\xF7\x00\x00\x00\x00\x00\x00\x00\x00,\x00\x00')

    # Local Color Table (3 bytes; no local color table defined here)

    # Block Size (1 byte; block terminator)
    out.write(b'\x00')

