#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Header
    header = (
            b'GIF89a' if rng.peek(
                6)[0:6] == b'GIF87a' else b'GIF89a')

    # Screen descriptor
    width = min(512 * ((int.from_bytes(
                    rng.read(1), 'big')) & 0x07),
                2 ** 11 - 1) >> 4 | 1 << 7
    height = min(512 * (((int.from_bytes(
                     rng.read(1), 'big')) & 0x38) >> 3),
                 2 ** 11 - 1) >> 4 | 1 << 7
    packed = (
            (width & 0xF0) << 4 | (height & 0xF0)) | \
            (int.from_bytes(rng.read(1),
                           'big') & 0x07)
    background = int.from_bytes(rng.read(1),
                                'big')
    aspectratio = int.from_bytes(rng.read(1),
                                 'big')
    out.write(header +
              width.to_bytes(2,
                             byteorder='little') +
              height.to_bytes(2,
                              byteorder='little') +
              struct.pack('=B',
                          packed) +
              background.to_bytes(1,
                                  byteorder='little') +
              aspectratio.to_bytes(1,
                                   byteorder='little'))
    palette = bytes([i for i in range(2**16)])[::len(palette) // 2]
    out.write(palette)

    # Image descriptor
    left = rng.read(2)
    top = rng.read(2)
    width = min(((int.from_bytes(
                      rng.read(2), 'big')) & 0x7FFF) + 1,
                2 ** 12 - 1) >> 4
    height = min(((int.from_bytes(
                       rng.read(2), 'big')) & 0x7FFF) + 1,
                 2 ** 12 - 1) >> 4
    flags = (
            int.from_bytes(rng.read(1),
                           'big') & 0x1C) | 0x80
    packed = flags
    out.write(left +
              top +
              width.to_bytes(2,
                             byteorder='little') +
              height.to_bytes(2,
                              byteorder='little') +
              struct.pack('=B',
                          packed))

    # LZW code size
    lzwmincodesize = max((int.from_bytes(
                            rng.read(1), 'big')),
                        2)
    out.write(struct.pack('=B', lzwmincodesize))

    # Data
    datalen = rng.read(2)
    data = rng.read(datalen)
    footer = [b'\x00', b'\x3B'][
            0 <= int.from_bytes(
                rng.read(1), 'big') < 6]
    out.write(
           header +
           datalen.to_bytes(2, 'big') +
           data +
           footer)
