#!/usr/bin/env python3

import sys
from pathlib import Path
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
) -> None:

    # get header
    gif89a = b'GIF89a' + b'\x00\x00\x00\x00\xff\xff\xff!\xf9\x04\x01\x00\x00\x00\x00,\x00' \
              b'\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'

    # write header
    out.write(gif89a)

    # get random bytes for color table
    colortable = bytearray([
        next(rng),  # red
        next(rng),  # green
        next(rng),  # blue
        next(rng),  # alpha, ignored by decoder
    ] * 256)
    print(colortable[:4])

    # write color table
    out.write(colortable)

    # GIF header block terminator and application extension block
    out.write(b';\x00')
    appext = (
            b"!\xFFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00!@\x00"
            + int.to_bytes(
                70, length=2,
                byteorder='big', signed=False
            )
            + b"\x03\x01\x00\x00\x00"

            + int.to_bytes(
                1, length=2,
                byteorder='big', signed=False
            ))

    out.write(appext)


if __name__ == '__main__':
    if len(sys.argv)!= 3 or not \
            ((Path(sys.argv[2]).exists() is False)):

        print('Usage:', file=sys.stderr)
        print(f'{sys.argv[0]} <seed> <output-file>', file=sys.stderr)
        exit(1)


    with open('/dev/urandom', 'rb') as f:
        seed = int(sys.argv[1]) % (2 ** 32 - 1)
        f.seek(int(sys.argv[1]))
        generate_random_gif(f, open(sys.argv[2], "wb"))