#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    print("Generating a GIF")
    
    # Create a header for our new image
    width = 150 + int.from_bytes(rng.read(2), "little", signed=False) % 64 - 32
    height = (height := 75 + int.from_bytes(rng.read(2), "little", signed=False)) % 64 - 32
    flags = b'\x89\xa'

    out.write(width.to_bytes(2, byteorder="little"))
    out.write(height.to_bytes(2, byteorder="little"))
    out.write(flags)
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)

    # Append that data to the output stream and close it
    out.write(len(data).to_bytes(2, 'big'))
    out.write(data)
    out.close()