#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(4), byteorder='big')

    # Magic word + length of data
    out.write(header+struct.pack('>I', datalen))

    for _ in range(datalen):
        val = rng.read(1)[0]

        if val == 0x2C:
            # Image separator
            out.write(
                (
                    bytes([val])    # 0x2C
                    + rng.read(9)   # width/height info
                    + bytes([0xF7]) # LZW minimum code size
                )
            )

        elif val < 0x2C or val == 0x3B:
            # Other block
            out.write(bytes([val]))
        
        else:
            raise ValueError("Unexpected value")

