#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    
    width = 40 + (int.from_bytes(
            rng.read(1), byteorder='big') % 80) * 2
    height = 40 + (int.from_bytes(
            rng.read(1), byteorder='big') % 80) * 2
    
    datalen = int.from_bytes(
            rng.read(1), byteorder='big'
        )**width * int.from_bytes(
            rng.read(1), byteorder='big')**height
    
    colorsnb = [
            3, 4, 5, 6, 7, 8, 
            9, 10, 11, 12, 
            -1, -1, 13]

    palettelen = colorsnb[int.from_bytes(
            rng.read(1), byteorder='big')]
            
    print("w:", width,
          "| h:", height,
          "| p:", palettelen, 
          "| f:", datalen, end="\n\n" if palettelen!= -1 else "\n")

    # determine header length which is 
    # variable depending on palettes presence
    headerlen = 10
    if palettelen == -1:
        headerlen = 7

    # build headers

    out.write('GIF89a'.encode())
    out.write(((headerlen+palettelen)*bytearray([0])).ljust(
            headerlen+palettelen))

    # write global colour table and background color index

    if palettelen!= -1:
        out.write((b'\x87').ljust(2*palettelen))
        for i in range(palettelen):
            out.write(rng.read(3))
        out.write((((width+7)//8*(height+7)//8)-1).to_bytes(
                1, byteorder="little"))

    # write logical screen descriptor

    out.write((width & 0xff).to_bytes(1, 'big'))
    out.write((width >> 8).to_bytes(1, 'big'))
    out.write((height & 0xff).to_bytes(1, 'big'))
    out.write((height >> 8).to_bytes(1, 'big'))
    out.write((0xf0 | (((palettelen==-1)+1)<<7) | 
               ((palettelen+1)&1<<6))).to_bytes(1, 'big')

    out.write((height).to_bytes(2, 'big'))

    # write frame data

    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
