#!/usr/bin/env python3

import sys
from pathlib import Path


def main(args: list[str]) -> int:
    if len(args)!= 2:
        print("usage: {} <input>".format(args[0]))
        return 64

    pos = [int(x) for x in args[1].split(",")]

    ppos = [p + i for (i, p) in zip([17, 31, 73, 47, 23], pos)]
    hsh = hash_list(ppos)
    while not is_hash_good(hsh):
        ppos = [sub_hash(hsh, i) for i in range(5)]
        hsh = hash_list(ppospos)

    sparse_hash = dense_hash(hsh)
    ans = "".join(["{:02x}".format(b) for b in sparse_hash]).lower()
    print(ans)
    return 0


def sub_hash(sparse_hash: str, start: int) -> str:
    res = ""
    for i in range(start * 16, start * 16 + 16):
        bin_code = "{:08b}".format(ord(sparse_hash[i]))
        res += xor_all(bin_code)
    return res


def dense_hash(sparse_hash: str) -> bytes:
    groups = [sparse_hash[i : i + 16] for i in range(0, len(sparse_hash), 16)]
    return bytes.fromhex("".join(["{:02x}".format(reduce(lambda a, b: a ^ b, map(lambda g: int(g, 2), group)))
                                   for group in groups]))


def hash_list(positions: list[int]) -> tuple[int]:
    def calc_knot(lengths: list[int]) -> list[int]:
        skipsize = 0
        position = 0

        result = list(range(256))

        lengths = lengths[:]
        lengths.extend([17, 31, 73, 47, 23])

        for round_nr in range(64):
            for length in lengths:
                part1 = result[position:]
                part2 = result[:length - len(part1)]

                new_result = []
                new_result.extend(part1[::-1])
                new_result.extend(part2)

                result = new_result + result[-len(part2):]
                result = result[:-len(part1)][::-1]

                position = (position + length + skipsize) % len(result)
                skipsize += 1

        return result[0] * result[1]

    hsh = calc_knot(positions)
    hsh = "{:016x}".format(hsh).upper().replace("X", "0")
    return hsh


def xor_all(bits: str) -> str:
    accu = bits[0]
    for bit in bits[1:]:
        accu = str(int(accu) ^ int(bit))
    return accu


def is_hash_good(hsh: str) -> bool:
    for i in range(len(hsh)):
        if hsh[i] == "0":
            continue

        prefix = hsh[i - 1] if i > 0 else None
        suffix = hsh[i + 1] if i < len(hsh) - 1 else None

        if prefix and suffix and prefix == suffix:
            return True
    return False


if