#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# palette of colors.
def generate(out: BinaryIO, rng: BinaryIO, pallete: list[tuple[int]]) -> None:
    # Set up our global variables
    width = 128
    height = 64

    header_data = b"GIF89a\0\0\0\x7f\xff\xfe\x00"

    # Write out the header (always going to be fixed for this particular example)
    out.write(header_data)

    # Create the color table with size equal to the given pixel count
    colortable = bytes()
    for c in pallete:
        colortable += struct.pack("<BBB", *c)
    out.write(colortable)

    # Write out the picture body
    framecount = 0
    while True:
        delaytime = next(rng) % 5

        localpallete = []
        for _ in range(256):
            index = next(rng) % len(pallete)
            localpallete.append(index)

        subblocks = [b"\xf7"]
        for h in range(height // 8 + ((height % 8) > 0)):
            rowblock = bytearray([])

            for w in range(width):
                if not (w % (width//12)) or not (h % (height//7)):
                    index = next(rng) % len(localpallete)

                    temprowblock = rowblock[:]
                    temprowblock.extend([index])
                    rowblock = temprowblock

                elif (w == width - 1) or (w % (width//8)):
                    index = next(rng) % len(localpallete)
                    
                    temprowblock = rowblock[:]
                    temprowblock.extend([index] * 2)
                    rowblock = temprowblock
                
                else:
                    index = next(rng) % len(localpallete)

                    temprowblock = rowblock[:]
                    temprowblock.extend([index] * 4)
                    rowblock = temprowblock
            
            subblocks.append(bytes(rowblock))
        
        # If we've reached an end-of-image marker then go nuts!
        if all((i == b"\xff") for i in subblocks[-1]):
            break

        subblocks[0] += bytes([delaytime])
        data = b"".join(subblocks)
        out.write(struct.pack(">H", len(data)))
        out.write(data)
    
    out.close()

