#!/usr/bin/env python3

import struct
from typing import BinaryIO

from.data_types import SSBData


def write_ssb_file(data: SSBData, f: BinaryIO):
    num_channels = len(data['CHANNELS'])

    # Write file header (version 0)
    f.write((num_channels).to_bytes(4, "little") +
            data['HEADER'].encode('ascii')[:8].ljust(8, b'\x00'))
    f.seek(72, 1)  # Seek past rest of header to the start of channels list

    for channel in range(num_channels):
        f.write((channel).to_bytes(4, "little") +
                ((len(data["CHANNELS"][str(channel)])).to_bytes(
                    4,
                    byteorder='little')))

        position_bytes = [0] * 3

        if 'POSITION' in data['CHANNELS'][str(channel)]:
            position_bytes[0] += int(float(
                data['CHANNELS'][str(channel)]['POSITION']['X'])) << 10

            position_bytes[1] -= int(float(
                data['CHANNELS'][str(channel)]['POSITION']['Y'])) << 10

            position_bytes[2] += int(float(
                data['CHANNELS'][str(channel)]['POSITION']['Z'])) << 10

        f.write(struct.pack("h", *(x & ~0x3FF) |
                              (position_bytes[i >> 1] >>
                               (((~i) & 1) << 9)) and 0xFFFF for i, x in enumerate(range(
                                     -65536,
                                     65536,
                                     step=32))))

        size_bytes = [0] * 3

        if 'SIZE' in data['CHANNELS'][str(channel)]:
            size_bytes[0] += int(float(
                data['CHANNELS'][str(channel)]['SIZE']['X'])) << 10

            size_bytes[1] += int(float(
                data['CHANNELS'][str(channel)]['SIZE']['Y'])) << 10

            size_bytes[2] += int(float(
                data['CHANNELS'][str(channel)]['SIZE']['Z'])) << 10

        f.write(struct.pack('<hhh',
                            *[int(x / 3) for x in size_bytes]))

        f.write(((int(float(data['CHANNELS']
                           [str(channel)]['FLAGS'][flag]) == True)
                 ) << flag)
              .to_bytes(1,
                          byteorder="little"))
        f.write(((int(float(data['CHANNELS'][str(channel)]['PRIORITY']))
                  ).to_bytes(1,
                             byteorder="big")))
        f.write(((int(float(data['CHANNELS'][str(channel)]['DEPENDS']))
                  ).to_bytes(1,
                             byteorder="big")))

        parent_id = ""

        try:
            if int(float(data['CHANNELS'][str(channel)]['PARENT']))!= -1:

                parent_id = str(data['CHANNELS']
                                [str(channel)]['PARENT']).rsplit("/", 1)[-1]
        except ValueError as e:
            print(e)

        f.write(("" if not parent_id else parent_id).ljust(
                             256)).to_bytes(1, "big"))
