#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)

    # Frame header
    out.write(header)
    # Image dimensions
    width = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    height = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))

    # write frame data

    for i in range(datalen):
        # Each pixel is composed of three bytes
        if i == 0:
            out.write(((int.from_bytes(
                    rng.read(1), byteorder="big") %
                256)).to_bytes(1, "big"))

        elif i <= 4 and i > 1:
            out.write(((int.from_bytes(
                    rng.read(1), byteorder="big") %
                256)).to_bytes(1, "big"))

        else:
            out.write((((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256) << 8) +
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
