#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    out.write(b'GIF89a')

    while True:
        # Read byte containing relevant flags and get bit length
        fieldbyte = ord(rng.read(1)) & 7
        
        # Get size of block
        size = ord(rng.read(1))

        # Check if there's any image data in this block
        hasimgdata = bool(((fieldbyte>>5)&3)>0) 

        # Write fields back to stream
        out.write(struct.pack('=B', fieldbyte | 2))
        out.write(struct.pack('=B', size))

        # Handle case where there's no image data
        if not hasimgdata:
            continue

        imgdata = rng.read(size)
        out.write(imgdata)