#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and add to output
    header = b'GIF89a'
    out.write(header)

    # Get the logical screen descriptor (LSD) field
    lsd = rng.read(7)

    # Add LSD to output
    out.write(lsd)

    # Read global color table length from LSD
    gctlen = 2**(int((lsd[4] >> 5))) * 3

    # Add GCT to output if we have it
    if gctlen > 0:
      gct = rng.read(gctlen)
      out.write(gct)

    # Read block size from LSD
    blocksize = int(lsd[6]) + 1
    
    # While we are not at EOF in the random stream...
    while True:

      # Read a byte as an unsigned integer
      b = ord(rng.read(1))
      
      # If we read a zero terminator then break
      if b == 0x00:
          break
          
      # Otherwise write our read byte to output
      else:
          out.write(bytes([b]))

      # If we've reached the end of this record then skip ahead past it 
      for i in range(blocksize - 1):
          b = ord(rng.read(1))

 