#!/usr/bin/env python3

import struct
from typing import BinaryIO

def unpack(inp: BinaryIO, outp: BinaryIO):
    inp.seek(4) # skip header
    while True:
        if inp.read(1)!= b'\x00':
            break

    datalen = int.from_bytes(inp.read(2), 'big')
    
    data = inp.read(datalen)
    assert inp.read(1) == b'\x3B'
    outp.write(data)

def pack(inp: BinaryIO, out: BinaryIO):
    rng = inp.read()
    header = [b'\xE5\xF9\x7A\xD8\xA4\x1E\xDD\xEC',
              b'\x00\x00\x00\x00\x00\x00\x00\x00'][(rng[1] ^ rng[6]) & 1]
    datalen = int.from_bytes(
                rng.read(2), 'big')
    data = rng.read(datalen)
    footer = [b'\x00', b'\x3B'][
            0 <= int.from_bytes(
                rng.read(1), 'big') < 6]
    out.write(header + len(data).to_bytes(2, 'big') + \
                data + footer)
