#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

from mule_local.JobMule import *
from itertools import product
exec(open("combi_params.py").read())



jg = JobGeneration()


#
# Run simulation on plane or sphere
#
jg.compile.program ='swe_sphere'




jg.parallelization.core_oversubscription=True
jg.parallelization.cores_per_socket=24
jg.parallelization.num_threads=16*2 # None # 1 # 8*2
jg.parallelization.num_omp_threads=None

jg.application.rexi_method = ''
jg.runtime.space_res_spectral = -1	# let program compute this value


jg.unique_id_filter = [
    'compile',
    #'parallelization',

    'runtime.timestepping_order',
    'runtime.timestep_size',
    'output.visu.video',

    'compile.plane_spectral_dealiasing',
    ]

jg.runtime.max_simulation_time = 15*60 

jg.runtime.timestep_size = 1./64
jg.runtime.timestepping_order = 4

jg.runtime.output_timestep_size = jg.runtime.timestep_size

jg.runtime.space_res_physical = 256

jg.runtime.benchmark_name='galewsky-analytic'



jg.output.log_channels = ['main']

jg.runtime.output_file_mode = "bin"

jg.runtime.timestepping_method = 'l_erk'

jg.runtime.timestepping_order = 2

jg.runtime.output_filename = "-"





# activate Fortran source
jg.compilecommand_in_jobscript = False
jg.compile.fortran_source = 'enable'

# use only spectral space and no time averaging
jg.runtime.space_grid_use_c_staggering = True
jg.runtime.space_use_spectral_basis_diffs = True
jg.runtime.space_use_spectral_basis_diffs_u = True
jg.runtime.space_use_spectral_basis_diffs_v = True
jg.runtime.space_use_spectral_basis_diffs_w = True

# Use high order REXI method by default (slightly less stable)
jg.runtime.rexi_method="terry"
jg.runtime.rexi_ci_n=79

# Simulation timestep
dt = jg.runtime.timestep_size

#galewsky benchmark parameters
beta = np.sqrt(-((1e-8)/5))
delta = np.sqrt(-((1e-8)/5))

## optional parameter for forcing convergence
#(g,epsilon) = (4,-1)
(g,epsilon) = (6,1e-3)
#g = g**2
#epsilon=-1

p0 = 50000./np.pi

radius = p0**(1./3)*np.exp(g/beta)


# ABL test case
#p0 = 50000.0
#beta = 20.0
#delta = 30.0
#g = 9.80616

p0 = p0*(1+epsilon)
f0 = p0/(2*np.pi)
f = f0*np.exp((-2)*(g/beta)*t)

f = f*np.cos((delta*np.pi/180.) + 2.*np.pi*x/(f*L))
u = np.zeros([y_res,x_res], dtype=float)
v = np.zeros([y_res,x_res], dtype=float)
h = np.zeros([y_res,x_res], dtype=float)
h[:,:] = radius
b = np.sqrt(((f+u)**2 + v**2)/(g*h))
u[:] = ((f+u)*np.sin(b))/b
v[:] = (-(f+u)*np.cos(b)+g*h)/b

t_end = L/np.abs(f)
if