#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) with the given width and height.
def generate(out: BinaryIO, rng: BinaryIO, width=1280, height=720):
    # Magic header is always 6 bytes long
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHB', width, height, b'\xF0'))

    # Color table
    for i in range(4 * 7):
        out.write(struct.pack('BBBx', rng.read(1)[0] % 0xFF + 0x55,
                             rng.read(1)[0] % 0xFF + 0x6A,
                             rng.read(1)[0] % 0xFF + 0xCE))

    # Image descriptor and graphics control extension
    out.write(struct.pack('<BBHHHBBBB',
                          b'\x2C', b'\0\0',
                          0, 0, width - 1, height - 1,
                          0, 0, b'\0'))

    # Plain color block (no LZW compression). The background is white here so we can see it on dark images.
    out.write(b'\x00\x00\xff\xff')

    # Extension to close logical screen. This actually seems to be redundant but I'm keeping it just because.
    out.write(b'\x3b\x00')


if