#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = bytearray()
    # magic code
    header.extend([0x47, 0x49, 0x46])
    # version (GIF89a)
    header.extend("GIF89a".encode('ascii'))

    width, height = [struct.pack('<H', i) for i in (128, 128)]
    
    color_table_size = b'\xff'  # max size
    background_color_index = b'\xff'
    aspect_ratio = 'F'.encode('ascii')

    flags = '\xe0'     # global color table and sort order
    # TODO: add more pixel formats -- maybe just EGA?
    pixel_format = 'E'.encode('ascii')
    gct_sort = False    # TODO: should be True according to spec
    # LZW min code size
    lzw_min_code_size = '\x02'
    screen_descriptor = b'' + \
                        width + \
                        height + \
                        b'\x00\x00'  # no color resolution, color table sorted
    image_descriptor = b'\x2c\x00\x00\x00'

    templ = lambda x, y : bytes([x] * len(y))
    default_header = templ(b'\x00', range(7))

    content = default_header[:]
    content[5] += 3   # add three reserved fields
    num_loops = 1 
    content.append('\x01')        # packed field
    content.extend(num_loops.to_bytes(2, 'little'))
    content += default_header    
    out.write(header+screen_descriptor+image_descriptor+content)

