#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), 'big')
    data = rng.read(datalen)
    footer = [b'\x00', b'\x3B'][
            0 <= int.from_bytes(
                    rng.read(1), 'big')]

    print(f"header={header}, datalen={datalen}," + \
          f"footer={footer}")

    # Write header and data length to output stream
    out.write(header)
    out.write(struct.pack('>H', datalen))

    # Write data to output stream
    out.write(data)
    
    # Write footer byte ('\x3B') or NULL-byte (zero)
    out.write(footer)