#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  
    out.write(header)
    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") % 254) + 1).to_bytes(
                        length=1, byteorder="big"))
    out.flush()
    
    framelen = []
    while True:
        ctrlbyte = rng.read(1)

        if (ctrlbyte == b'\x2C'):
            break
            
        framelen += [int.from_bytes(ctrlbyte, byteorder="big")]

    # Write frame 
    while len(framelen) > 0 and framelen[0] < 0xff:
        out.write(struct.pack("B", framelen[0]))
        del framelen[0]
        
    while True:
        gifbyte = rng.read(1)
        
        if gifbyte == b'\x2c': # EOF
            return;

        elif gifbyte == b'\x21': # extention 
            pass 

        else:                 # graphic control block
            pass
