#!/usr/bin/env python3

import struct
from typing import BinaryIO
import sys

 
if __name__ == "__main__":
    with open(sys.argv[1], 'rb') as f:
       rng = f.read() 

    out = open('randomized.gif', 'wb+')
    def choose_block():
      """Choose one of the three blocks that can occur"""

      # Choose between extensions, images or terminator
      value = rng.read(1)[0] 

      if value >= 0x21 and value <= 0x3f:
        return "extension"
      elif value == 0x2C:
        return "image"
      else:
        assert value == 0x3B
        return "terminator"


    last_block = ""

    while True:
        blocktype = choose_block()


        if blocktype == "terminator": 
            break

        if blocktype!= last_block: 
          print(" ", end="", flush=True)
          last_block = blocktype
      
        # Read the next byte to decide what kind of extension we should create
        if blocktype == "extension":

          # Read the rest of the extension
          extheader = rng.read(6)
          
          # Write extension header
          out.write(extheader)
       
          # Parse extension header
          hasgct   = (extheader[0] & 0x80)!= 0
          hasimgdata = (extheader[0] & 0x08)!= 0
          size     = int(struct.unpack(">H", extheader[2:])[0]) + 1
          # If there's no image data we're done here
          if not hasimgdata:
            continue

          # Write the local color table if necessary
          numcolors = size//3 - 1
          if numcolors > 0:
             colortable = rng.read(numcolors*3)
             out.write(colortable)

          # Now write image data 
          imgdata = rng.read(size-1)
          out.write(imgdata)

        elif blocktype == "image":
          # Read the rest of this extension
          imgextension = rng.read(6)
          out.write(imgextension)

          codesize = imgextension[0]
          assert imgextension[-1] == 0

          # Parse extension header
          left        = int(struct.unpack(">H", imgextension[1:3])[0])
          top         = int(struct.unpack(">H", imgextension[3:5])[0])
          width       = int(struct.unpack(">H", imgextension[5:7])[0])
          height      = int(struct.unpack(">H", imgextension[7:9])[0])
          flags       = imgextension[9] 
          transparent = imgextension[10]

          reserved = imgextension[11:]
          

          # Create image descriptor
          imgdesc = bytearray()
          imgdesc += bytes([left])
          imgdesc += bytes([top])
          imgdesc += bytes([width])
          imgdesc += bytes([height])
          imgdesc += bytes([(flags << 3) | 0x4])
          imgdesc += bytes([transparent])
          imgdesc += bytes([0]) * 7

          # Write image descriptor
          out.write(bytes(imgdesc))

          if ((flags & 0x1)!= 0):
              palette = []
              for _ in range(2**codesize):
                  r = rng.read(1)[0]
                  g = rng.read(1)[0]
                  b = rng.read(1)[0]

                  palette.append([r, g, b])

              palettesize = codesize**3 * 3
              pallete = bytearray()
              for p in palette:
                pallete += bytes(p)
              

              out.write(pallete)

     