#!/usr/bin/env python3

import struct
from typing import BinaryIO


class RNG:

    def __init__(self):

        self._rng = open('/dev/urandom', mode='rb')

    def randint(self, nbits=None):

        if None is not nbits and 0 >= nbits:

            raise ValueError('argument must be non-negative')

        return int.from_bytes(
                self._rng.read(nbits // 8 or 1), byteorder='big', signed=False
            )

    # Generates a random GIF file (or whatever format you like really)
    # into `out`. Use `randint()` to get your data from the "real"
    # /dev/urandom device. `nbits` is the number of bits that will be
    # read per each pixel (if this information isn't provided in advance).
    def write_random_data(self, out: BinaryIO, nbits=None):

        header = bytes([71 for i in range(6)])
        data_length = len(header) + \
            ((nbits * 4 + 7) // 8 if nbits else 1)
        data_length += 2
        if 0 == (data_length & 1):
            data_length += 1
        end = bytes([0])
        out.write(header +
                  data_length.to_bytes((data_length.bit_length() + 7) //
                                      8, byteorder='big') +
                  end)