#!/usr/bin/env python3
import argparse, os, sys, subprocess, random

# Helper function that prints an error message and exits with status code t
def fail(t: int, msg: str):
  print("ERROR:",msg)
  exit(t)
  
# Run main program when executed as script
if __name__ == "__main__":

  # Parse command line arguments  
  parser = argparse.ArgumentParser()
  
  parser.add_argument("-i", "--input-file", dest="infile", help="Input file name")
  parser.add_argument("--seed", dest="seed", help="Seed for random number generation")
  parser.add_argument(dest='outfile')
  args = parser.parse_args()
  
  infile = None
  outfile = open(args.outfile,'wb+')
  
  try:
    if hasattr(sys.stdin, 'buffer'):
      infile = sys.stdin.buffer
    else:
      infile = sys.stdin
    if args.infile:
      infile = open(args.infile,"rb")
  
    rng = random.Random()
    seed = 0x3C
    if args.seed!= None:
      seed = int(args.seed,0)
    rng.seed(seed+os.getpid())

    if infile:
      while True:
        chunk = infile.read(64*1024)
        if len(chunk)==0: break
        for i in range(0,len(chunk),4096):
          chunk2 = chunk[i:min(len(chunk),i+4096)]
          generate_random_gif(rng, outfile)
          outfile.write(chunk2)
    else:
      generate_random_gif(rng, outfile)
      
  except IOError as e: 
      fail(-1, "I/O Error {}".format(e))
  finally:
    try: 
        if infile: 
          infile.close()
    except: pass
    try:
        if outfile:
            outfile.close()
    except:pass