#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO, 
        width=400, height=300):

    LZW_CODESIZE = 12

    colors_used = [i for i in range(7)]

    def get_byte(): return next(rng_iter)
    

    def get_color() -> tuple[tuple[int], int]:
        r = (get_byte() >> 4) & 0xf; g = get_byte() & 0xf
        b = (get_byte()) & 0xf
        a = (((get_byte() << 4) & 0xf0)) |((get_byte() >> 4) & 0x0f);

        c = (r<<16)|(g<<8)|b ; color_index = len(colors_used)
        
        if not c in colors_used: 
            colors_used.append(c)
        
        return hex(c)[2:], color_index


    global rng_iter 
    rng_iter = iter(rng.read(-1))


    # Build screen descriptor

    screen_descriptor = \
      struct.pack("!HH", width, height)


    # Build global color table

    background_color = '#ffffff'; transparent_color = '#000000'

    global_color_table=[]

    for i in range(len(colors_used)):
        s = rgb_string(hex_string_to_rgb(colors_used[i]))

        global_color_table.append([s])

    global_color_table += [[background_color]] * (2**(LZW_CODESIZE-1)-len(global_color_table))

    global_color_table += [[transparent_color]]

    
    # Build image descriptor

    image_descriptor = struct.pack(
      "!BBHBBBB",
      0x2c,
      0,
      0, 
      width, 
      height, 
      0, 
      0 
    )

    pass

