#!/usr/bin/env python3

import argparse
import math
import os
import sys
import struct
from typing import BinaryIO

def read_rand_file(filename='/dev/urandom', n=4) -> bytes:
    with open(filename, 'rb') as f:
        return f.read(n)

def ensure_bytes(s, encoding='ascii'):
    if isinstance(s, str):
        s = s.encode(encoding)
    elif not isinstance(s, (bytes, bytearray)):
        raise TypeError("Not a string or bytes like object")
    return s


class RngFile:
    def __init__(self, filename='/dev/urandom'):
        self._f = None
        self._filename = filename
        self._position = 0
        self._temp = b''

    @property
    def position(self):
        return self._position

    def next(self, n=None):

        def _next(_n):
            n -= len(self._temp)

            while n > -len(self._temp): 
                new = read_rand_file()
                yield from new
                n -= len(new)

            self._temp += self._temp[:n+1]
            

        assert type(n) is int and n >= 0
        
        self._temp = b''
        ret = []
        try: 
            while True:
                touse = min(n, 4 * 1024**2)
                
                chunk = list(_next(touse))
                if len(chunk) < touse: 
                    break

                ret.append(b''.join(chunk))
                n -= touse
                
            if n > 0 :
                self._temp = b''.join(_next(n + 1))
                            
                
        except GeneratorExit:
            pass
            
        return b''.join(ret)
    



# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0xfff

    print('header',header,'datalen',datalen)
    
    # Write pre-frame bytes  
    out.write(header)
    for i in range(datalen):
        out.write(
                ((int.from_bytes(
                    rng.read(1), byteorder="big") %
                 256)).to_bytes(1, "big"))
