#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO


# Generates a random GIF file into `filepath`, using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        filepath: str, rng: BinaryIO
    ):
    
    with open(filepath, 'wb') as f:

        # Write the magic string (header) to the output stream
        f.write(b'GIF89a')

        # Generate some metadata for the body
        width  = rng.randint(16, 500)
        height = rng.randint(16, 500)
        palette_size = rng.randint(2, 256)
        bgcolor = rng.randint(0x000000, 0xffffff)
        duration = rng.randint(10, 1000) // 100  # milliseconds

        print([width, height, palette_size, bgcolor, duration])

        # Write the metadata...
        # Width
        f.write(struct.pack('B', width))
        # Height
        f.write(struct.pack('B', height))
        # Flags & Background Color Index
        flags = 0b1000 | (rng.randint(1, 255) % (palette_size - 1)) << 4
        bgindex = rng.randint(0, 255) % (palette_size - 1)
        f.write((flags + (bgindex << 4)).to_bytes(1, 'little'))
        # Pixel aspect ratio
        f.write(b'\x00')
        
        # Write the color table entries
        for _ in range(min(256, palette_size)):
            r = rng.randint(0, 255)
            g = rng.randint(0, 255)
            b = rng.randint(0, 255)
            a = 255
            f.write((r + (g << 8) + (b << 16) + (a << 24)).to_bytes(4, 'little'))


        # TODO -- generate some pixel data
        for i in range(height):
            for j in range(width):
                index = rng.randint(0, 255) % (palette_size - 1)
                f.write(index.to_bytes(1, 'little'))

            # Pad rows with two zero bytes if necessary
            if (i+1 < height):
                while ((f.tell() + 2) % 8!= 0):
                    f.write(b'\x00\x00')
                

        # Write the loop control block at the end of the image data
        f.write(
            ((0 <<  7)
             | (duration              <<  0) # Duration
             | (0                      << 10) # Transparent color index
             | (1                     << 11) # Terminator flag
            ).to_bytes(1, 'little'))