#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):


    header = b'GIF89a'
    
    while True:

        # Pick a random width and height between [2..10]
        w = 2 + int.from_bytes(rng.read(1),"little")%8
        h = 2 + int.from_bytes(rng.read(1),"little")%8
        
        print("Width",w,"Height",h)

        # Read the length of the next block
        sublen = int.from_bytes(rng.read(1),"little") & 0xff;
        
        if (sublen == 0) : break
        
        # Read the block content as RGB(A) data
        subdata = bytearray()
        while len(subdata)<sublen:
            subdata += rng.read((sublen - len(subdata)))
            if not subdata[-1:] == b'\x00':
                raise Exception("No null term at end of subblock.")
                
        print("Subdata",subdata)


        # Skip stuff we don't want to see encoded in our output images
        for j in range(4):
        
            # Color Table size 
            ctsize = int.from_bytes(subdata[:1],"little");
            print("Color table size:",ctsize);

            # Ignore color table
            if ctsize > 0: 
                subdata=subdata[3*256+7:]
                    
            # Get graphic control label 
            gclabel = subdata[:1];
            print("Graphic Control Label:",gclabel);
            
            # Set default delay time to zero
            deltime = 0;
                
            # Get delay time 
            dts = subdata[1:3];
            print("Delay Time Specification:",dts);
            deltime = int.from_bytes(dts,"little");  
                
            
            # Get transparent color index 
            tc = subdata[3:4];  
            print("Transparent Color Index:",tc);
            
            # Get num loops 
            numloops = subdata[4:6];   
            print("Number Of Loops:",numloops);
                    
            # Ignore image data
            idata = subdata[6:-1];    
            print("Image Data Length:",len(idata)); 
                    
            subdata=subdata[(6+len(idata)):];                    
            
        # Finally...we have an RGBA image!
        # Blit it onto our canvas
    
        xpos = ypos = 0;
        imgdata = bytearray();
        
        while (True):     

            cmd = subdata[0];
            subdata=subdata[1:];
            
            if cmd==0x2C:                
                
                # Start of Image Descriptor

                lsdat1 = subdata[0:2];  
                lsdat2 = subdata[2:9];   
                lsdat3 = subdata[9:1];                  
                
                lsdlen = int.from_bytes(lsdat1,"little");
                print("LSD LENGTH=",lsdlen)
                 
                lsdwidth = int.from_bytes(lsdat2[0:2],"little");
                print("LSD WIDTH=",lsdwidth);
                
                lsdheight = int.from_bytes(lsdat2[2:4],"little");            
                print("LSD HEIGHT=",lsdheight);
                                 
                lsdleft = int.from_bytes(lsdat2[4:6],"little");
                print("LSD LEFT=",lsdleft);
                
                lsdtop = int.from_bytes(lsdat2[6:8],"little");
                print("LSD TOP=",lsdtop);
            
                lsdislocalcolormap = lsdat2[8:10][0];
                print("LSDI Local Color Map:",lsdislocalcolormap);
                    
                lsdinterlaceflag = bool(lsdat3[0]);
                print("LSD Interlace Flag:",lsdinterlaceflag);
                    
                lsdsortedcolormapflag = bool(lsdat3[1]);
                print("LSD Sorted Colour Map:",lsdsortedcolormapflag);
                    
                lsdbitsperpixel = lsdat3[2];
                print("LSD Bits Per Pixel:",lsdbitsperpixel);
            
                subdata=subdata[(lsdlen+1):];
                
                print("\n\nNew Subimage!\n\n")

            elif cmd==0x3B:
                
                # End of File
                return
                
            else:
                raise NotImplementedError(f"Unknown PNG command {cmd}")
            
            
        
    
            