#!/usr/bin/env python3

import os
import sys
import struct


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, length=1 << 24
    ):
    if not isinstance(length, bytes):
        assert len(length) == 4, f"Length must be four bytes long"
        length = struct.pack("!I", length)


    header = b'GIF89a' + bytes([ord('&')]) + length

    
    # Frame header
    out.write(header)
    # Image dimensions
    width = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    height = (int.from_bytes(
            rng.read(2), byteorder='big')) & 0xffff
    
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))

    # write frame data

    while True:
        try:
            for _ in range(os.sysconf("SC_PAGE_SIZE")):
                out.write(((int.from_bytes(
                        rng.read(1), byteorder='big') %
                          256))).to_bytes(1, 'big')

        except KeyboardInterrupt as e:
            break
