#!/usr/bin/env python3

import struct
from typing import BinaryIO, Tuple

# This is an iterator that yields (size, contents) tuples
def randomBytes() -> Tuple[int, bytes]:
    while True:
        yield from struct.iter_unpack('<H',
                            open('/dev/urandom','rb').read(2))


# Generates a random GIF file into `out` using `randIter`, which
# should be an iteratable over pairs of integers.
def generate_random_gif(
        randIter, out: BinaryIO
    ):

    # Header and logical screen descriptor
    out.write(b'GIF89a' + struct.pack('HHBBB',
                    54+len(next(randIter)[1]),       # Logical Screen Descriptor 
                    768,                               # Width
                    -768 if next(randIter)[0] > 0 else 768,# Height
                    8,                                 # Flags (8bpp)
                    0                                  # Background color index
                ))

    # Global Color Table
    for _ in range((2 ** bitsInColorLookupTable)//3):
        R, G, B = tuple([next(randIter)[0]%256 
                        for i in range(3)])
        out.write(struct.pack('BBB',R,G,B))


    # Image Descriptor
    out.write(struct.pack('BBBBHHBB',
                        0xF9,                            # Extension introducer
                        b'graphic-control'.index(next(randIter)[0]),    # Graphic control label
                        4,                              # Size
                        *next(randIter)[1],             # Packed field
                        0,                              # Delay time
                        0,                              # Transparent color index
                        0                               # Block Terminator
                    ))

    # Application Extension Block
    out.write(struct.pack('BBBBBB',
                        0xFF,                        # Extension introducer
                        b'application'.index(next(randIter)[0]),   # App extension label
                        11,                          # Size
                        *next(randIter)[1][::-1]     # Application identifier and auth code
                    ))
    
    # Loop count subblock
    out.write(b'\x01\x00')                  # Subblock size with loop value set to zero
    out.write(struct.pack('B', next(randIter)[0]))      # Loop counter

    # Image data block
    datalen = next(randIter)[0]
    out.write(b';' + struct.pack('<HLHB',
                                0x2C,                # Image separator
                                datalen,             # Length
                                0,                   # Left position 
                                (-1*datalen)%2**16
                                    ) + 
                next(randIter)[1])                  # Actual image data

    # Footer
    out.write(struct.pack('B',
                        b'standalone'.index(next(randIter)[0])))
