#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):


    frame_data = []

    while True:

        delay = int.from_bytes(
            rng.read(2), byteorder="little", signed=False)

        if delay == 0 or len(frame_data) >= 0xfffffff:
            break

        command = int.from_bytes(
            rng.read(1), byteorder="big", signed=False)
        
        count = int.from_bytes(
            rng.read(1), byteorder="big", signed=False)

        if command!= 0x2c: # Graphics Control Extension

            raise ValueError("Invalid Command")
    
        extra = bytearray()

        while not (extra[-1] == 0x00 and
                   extra[-2] == extra[-3]):
    
            extra += (rng.read(1))

        frame_data += [command,
                       *((delay).to_bytes(2,"little")),
                        *extra ]


        for _ in range(count):
           frame_data += [(int.from_bytes(
                    rng.read(1), byteorder="big" ))]
    

    exts = {
    }

    colors = [b'\xff\xff\xff']

    try:
        palette_size = int.from_bytes(
                rng.read(1), byteorder="big", signed=False)

        if palette_size > 0:

            if palette_size == 7:

                for i in range(3):

                    colors.append((int.from_bytes(
                            rng.read(1),byteorder="big",signed=False)>>5)*4*i+
                                  (int.from_bytes(
                                    rng.read(1),byteorder="big",signed=False)>>6)+
                                  0xf8f8f8)

            elif palette_size < 1 << 8:

                for i in range(palette_size):
                     colors.append(*(rng.read(3)))    
            
            else:
                 raise ValueError("Bad Palette size")

    except IndexError:

        pass
    
    global_color_table = False

    # Encode the color table as an extension
    if len(colors) <= 2**(int.from_bytes(frame_data[1],
                                         byteorder="big")&0x7):

        exts[b'\x21\xf9\x04'] = b'\x01'+ \
                                 (((len(colors)-1)<<5)+
                                  (((int.from_bytes(
                                     frame_data[1],
                                     byteorder="big"))>>4)&0x7)).to_bytes(1,"big")+\
                                b'\0\0\0\0'+ \
                                 b'\x00'.join([struct.pack(">BBB",
                                                    *[int(round(c/(256**k))+
                                                         .5)
                                                      for k in reversed(range(3))])
                                              for c in colors])+ \
                                 b'\0'*(((int.from_bytes(
                                        frame_data[1],byteorder="big")))&0x7)

        global_color_table = True

    # Write the extensions

    for name,value in sorted(exts.items()):
        out.write(name+
                             len(value).to_bytes(length=1,"big")+
                              value)

    if global_color_table:

        frame_data[1] &= ~0x80
        
    out.write(b"".join(frame_data))


