#!/usr/bin/env python3

import sys
import os
os.chdir(sys.path[0])

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Write the header in ascii
    out.write('GIF89a'.encode())

    # Generate the screen descriptor block at offset 6
    width = ord(os.urandom(1)) % 75 + 40
    height = ord(os.urandom(1)) % 45 + 40

    globalcolordepth = ord(os.urandom(1)) & 0x3f | ((ord(os.urandom(1)) & 1) << 6);
    
    colorres = '{:02b}'.format((globalcolordepth >> 7) & 1) + \
                '{:02b}'.format((globalcolordepth >> 6) & 1) + \
                '{:02b}'.format((globalcolordepth >> 5) & 1) + \
                '{:02b}'.format((globalcolordepth >> 4) & 1) + \
                '{:02b}'.format((globalcolordepth   ) & 1) +\
                '{:02b}'.format((globalcolordepth   ) & 1) +\
                '{:02b}'.format((globalcolordepth   ) & 1) +\
                '{:02b}'.format((globalcolordepth   ) & 1) 
                # => XOR with 0b01101000

    sortflag    = '{:02b}'.format((globalcolordepth   ) & 1) 
    gctsize     = '{:02b}{:02b}'.format(((globalcolordepth >> 7) & 1) ^ (((globalcolordepth >> 3) & 1) ^ 1),(globalcolordepth >> 3) & 1)
    bgindex     = ord(os.urandom(1)) % 256
    pixelaspect = ord(os.urandom(1)) % 9 + 1;


    bpc = globalcolordepth // 3
    if globalcolordepth > 8:
        bpc = 8
        extgcblen = 0
    else:
        if bpc < 2:
            bpc = 2
        extgcblen = bpc * 3

    sdblock = bytearray([
        0x2c,                       # Image separator
        (width >> 8) & 0xff,       # Width LSB
        width & 0xff,               # Width MSB
        (height >> 8) & 0xff,      # Height LSB
        height & 0xff,              # Height MSB
        ((pixelaspect - 1) >> 8) & 0xff,
                                # Pixel aspect ratio x
        (pixelaspect - 1) & 0xff,  # Pixel aspect ratio y
        *(bytearray(int(bit, 2)
                     for bit in [
                         sortflag,           # Sort flag
                         0,                  # Global color table size
                         bpc,                # Color resolution LSB
                         bpc,                # Color resolution MSB
                         gctsize[0],         # GCT-size LSB
                         gctsize[1]          # GCT-size MSB
                        ]
                    )),

        pixinfochar = 0b10000000 + \
                       (bgindex & 0xf) + \
                       ({
                           "bb": 0, # Black background
                           "wb": 1, # White background
                           "bbwb": 2, # Black + white background
                         }[(colorres[:2]+colorres[3:])])


        pixinfochar |= {
            0: 0,
            1: 0b10000000,
            2: 0b10000000,
            3: 0b01000000,
            4: 0b01000000,
            5: 0b00100000,
            6: 0b00100000,
            7: 0b00010000,
            8: 0b00010000,
            9: 0b00001000,
            10: 0b00001000,
            11: 0b00000100,
            12: 0b00000100,
            13: 0b00000010,
            14: 0b00000010,
            15: 0b00000001,
            16: 0b00000001,
            17: 0b00000000 
        }[globalcolordepth]

        out.write(pixinfochar.to_bytes(1, 'little')); # Pixel info

        if globalcolordepth <= 8: 
            out.write(extgcblen.to_bytes(2, 'little'))            
            for _ in range(bpc): 
                out.write(os.urandom(1));
        out.write(0x00)
        
    # Create an image descriptor followed by the raw data
    idblock = bytearray([
        *(bytearray(int(bit, 2)
                     for bit in [
                         left,               # Left position LSB
                         top,                # Top position LSB
                         width & 0x3ff,      # Width LSB of the actual picture
                         height & 0x3ff,     # Height LSB of the actual picture
                         1                   # Local color table flag
                     ])),
        1023,                      # LZW min code size
        *data,                     # The actual data
        1                         # Block terminator
    ])

    # Append the blocks together
    out.write(idblock)
