#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Write the GIF header and logical screen descriptor

    out.write((b"GIF89a"+
               struct.pack("H", 40) + 
               struct.pack("HHhh",
                         2**12+768,
                         2**12+768,
                         0, 0)))

    # Write the global color table with an entry 
    # for each pixel value which is randomly generated
    # as a uint16

    palette_size = pow(2, 12)+1
    for _ in range(palette_size):
        r = ord(rng.read(1))
        g = ord(rng.read(1))
        b = ord(rng.read(1))

        out.write(struct.pack("BBB",r,g,b))


    # Write the image descriptor

    out.write(struct.pack("<bbHHHHHBB",
                         0x2c, # Image separator
                         # Left    Top     Width   Widght
                         0,     0,      2**12,  2**12,
                         0,     0,      0,       0,
                         palette_size-1))

    # Write the pixels themselves by reading them from /dev/urandom
    while True:
        try: 
            p = ord(rng.read(1))

            if p == 255: 
                break;

            out.write(struct.pack("B",p))
            
        except EOFError:
            break

