#!/usr/bin/env python3

import sys
from pathlib import Path
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    signature = header \
                + (b'\x00\x00' * 57) \ # screen descriptor block
                + b'\x02'                # color table size 2^2=4 colors per palette entry


    def write_palette():
        for _ in range(1 << 2):
            out.write((rng.read(3)))

    def write_image():

        data_size = rng.read(3)
        data_size = int.from_bytes(data_size, 'big')

        lzw_min, lzw_max = [int(_) for _ in rng.read(2)]
        lzw_min += 2
        lzw_max += 2

        lzw_data = b''
        while len(lzw_data) < data_size:

            if not rng.getrandbits(1):

                # literal
                lzw_data += bytes([rng.getrandbits(8)])
            
            else:
                
                # compressed
                code_size = rng.getrandbits(3) + lzw_min
                code = 0
                for i in reversed(range(code_size)):
                    code |= ((rng.getrandbits(1)) << i)
                length = 1 << code_size
                offset = rng.getrandbits(length - 1)
                lzw_data += bytes([offset])
        
        out.write(lzw_data[:data_size])

    with open("test.gif", "wb") as tmpout: 
        tmpout.write(signature)
        write_palette()
        write_image()
    
    # read from temporary file and copy to output
    with open("test.gif", 'rb') as imgin:
        imgout.write(imgin.read())

    print(f"generated {len(data)}-byte image")

