#!/usr/bin/env python3

import os
import argparse
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (default is /dev/urandom)
def generate_random_gif(
        rng: BinaryIO=None, out:BinaryIO=stdout,
        size=(480, 480), num_frames=10, colors=['red', 'blue'],
        durations=[1*10**2],delays=[0]):
    if not rng:
        rng = open('/dev/urandom','rb')
    
    # http://giflib.sourceforge.net/whatsinagif/index.html
    # Version Signature 	    
    out.write(b"GIF89a")
    # Logical Screen Descriptor
    out.write((size[0]).to_bytes(2,"big"))	# Width 
    out.write((size[1]).to_bytes(2,"big"))	# Height
    out.write(b"\xf7\x00")			# Global Color Table Flag (0=no, 1=yes)
    out.write(b"\x00")			    # Color Resolution
    out.write(b"\x00")			    # Sorting Flag
    out.write(b"\x00")			    # Size of Global Color Table

    out.write((len(colors)*3+1).to_bytes(1,"big"))	# Number of Colors + 1
    colors += [""]
    for c in color:
        out.write(((color.index(c)+1)<<2 | len(rgb)//3-1)\
                .to_bytes(1,"big"))
        rgb = bytes.fromhex(c)[::-1]
        for b in rgb:
            out.write(b)

    # "Netscape" Application Extension Block
    out.write(b"\x21\xff\x0bNETSCAPE2.0")
    out.write(b"\x03\x01")
    out.write((num_frames//256).to_bytes(1,"big"))
    out.write((num_frames%256).to_bytes(1,"big"))

    # Graphics Control Extension block
    out.write(b"\x21\xf9\x04")
    out.write(b"\x08")     # Block Size
    out.write(b"\x00")     # Packed Field - no transparancy, no user input
    out.write(b"\x00\x00") # Delay to next frame in hundredths of seconds
    out.write(b"\x00")     # Transparent color index
    out.write(b"\x00")     # Terminator

    # Plain Text Extension Block
    out.write(b"\x21\x01\x0d")
    text = "<NAME>\x00"
    out.write(text)

    # Graphic Control Label
    out.write(b"\x2C")
    out.write((size[0]//256).to_bytes(1,"big"))   # Left
    out.write((size[0]%256).to_bytes(1,"big"))
    out.write((size[1]//256).to_bytes(1,"big"))   # Top
    out.write((size[1]%256).to_bytes(1,"big"))
    out.write((size[0]//256).to_bytes(1,"big"))   # Right
    out.write((size[0]%256).to_bytes(1,"big"))
    out.write((size[1]//256).to_bytes(1,"big"))   # Bottom
    out.write((size[1]%256).to_bytes(1,"big"))
    out.write(b'\x00')       # No disposal specified
    out.write(b'\x00')       # Use default pallete
    out.write(b'\x00')       # Not interlace

    for _ in range(num_frames):
        out.write(b"\x21\xf9\x04")
        out.write(b"\x08")         # Block Size
        out.write(b"\x00")         # Packed Field - no transparancy, no user input
        duration = durations.pop()
        delay = delays.pop()
        out.write((duration//256).to_bytes(1,"big")) # Delay to next frame in hundredths of seconds
        out.write((duration%256).to_bytes(1,"big"))
        out.write(b"\x00")         # Transparent color index
        out.write(b"\x00")         # Terminator

        out.write(b'\x2C')          # Graphic Control Label
        out.write((size[0]//256).to_bytes(1,"big"))   # Left
        out.write((size[0]%256).to_bytes(1,"big"))
        out.write((size[1]//256).to_bytes(1,"big"))   # Top
        out.write((size[1]%256).to_bytes(1,"big"))
        out.write((size[0]//256).to_bytes(1,"big"))   # Right
        out.write((size[0]%256).to_bytes(1,"big"))
        out.write((size[1]//256).to_bytes(1,"big"))   # Bottom
        out.write((size[1]%256).to_bytes(1,"big"))
        out.write(b'\x00')           # No disposal specified
        out.write(b'\x00')           # Use default pallete
        out.write(b'\x00')           # Not interlace
        
        for x in range(size[0]):
            for y in range(size[1]):
                out.write(
                        (((x+y)%
                          len(colors))+1
                         ).to_bytes(1,"big"))

