import os
import struct
from typing import BinaryIO


# Generates a random GIF file of dimension `size` (`size[0]` x `size[1]`)
# with color depth `depth`, to be written to `out`.
def generate_random_gif(
        size: tuple[int, int],
        depth: bytes,
        out: BinaryIO
    ) -> None:
    
    # Header
    if type(depth)!= bytes or len(depth) > 4:
        raise ValueError("Invalid color depth!")
        
    out.write(b"GIF89a")
    screenWidth, screenHeight   = size[:2]
    globalColorTableSize         = depth[-1]*2**int.from_bytes(depth[:-1], "big")
    backgroundIndex              = 0x00
    aspectRatio                  = 0x00
    flags                        = [ord('S'), ord('R')]
    paletteTransparencyFlag      = False
    sortFlag                     = True
    globalColorTableFlag         = True
    backgroundColor              = [0x00] * 3
    pixelAspectRatio             = 0x00
    screenLeftPos                = 0x00
    screenTopPos                 = 0x00
    screenWidthBytes             = min([screenWidth.bit_length(), 0xff]) + 1
    screenHeightBytes            = min([screenHeight.bit_length(), 0xff]) + 1
    packedField                  = struct.pack("<BBBBBBBBBHBHHB",
                                              screenWidthBytes,
                                              screenHeightBytes,
                                              // bit packed field
                                              *flags,
                                              7 << 4 | globalColorTableSize,
                                              backgroundIndex,
                                              aspectRatio,
                                              b"\x08\x00\xf8\x00"[paletteTransparencyFlag*4:][:3],
                                              sortFlag,
                                              globalColorTableFlag,
                                              backgroundColor[1],
                                              backgroundColor[2],
                                              pixelAspectRatio,
                                              screenLeftPos,
                                              screenTopPos
                                          )
    
    out.write(packedField)
    
    # Global Color Table
    numColors = pow(2,globalColorTableSize+1)-1
    colors = [(i//256, i%256, i*2) for i in range(numColors)]
    if not sortFlag:
        colors = list(map(lambda c : colors[c[0]], enumerate(colors)))
        
    for r,g,b in colors:
        out.write(struct.pack("BBB",r, g, b))
        