#!/usr/bin/env python3
from PIL import ImageFile
ImageFile.LOAD_TRUNCATED_IMAGES = True
## broken image handling!
import os
import struct
import sys
from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import BinaryIO


def parseArgs() -> Namespace:
    parser = ArgumentParser("Generate a random.gif")
    parser.add_argument("-s",
                        "--size",
                        type=str,
                        default="32x32",
                        help="dimensions of gif to create (e.g., \"32x32\")")
    parser.add_argument("--seed",
                        type=str,
                        default="",
                        help="set seed (hex string)")
    args = parser.parse_args()
    return args



def main():
    args = parseArgs()


    if len(sys.argv)>1 and not '-h' in sys.argv and '--help' in sys.argv :

        imsize = [ int( x ) for x in args.size.split('x')]
        outpath = Path( args.output ).resolve()
        seed = args.seed

        if not seed == "" :
            rng = open("/dev/urandom","rb" )
            rng.seek(int(seed[:],16))
        else : 
            rng = None


        with open(outpath,"wb") as f:


            gen_image = ImageFile.new(f"{imsize[0]}x{imsize[1]}", mode="P")

            try : 

                try : 
                    imgdata = list(gen_image.getdata())

                    for i in range(len(imgdata)):
                        imgdata[i]=((int.from_bytes(
                            rng.read(1), byteorder="big") %
                         256)&0xff)

                except AttributeError : 

                    raise Exception("""Error occurred while generating image""") 



                gen_image.putdata(imgdata)

                gen_image.save(f, format="gif")
            finally : 
                try : 
                    f.close()
                except NameError : 
                    pass
                try : 
                    rng.close()
                except NameError : 
                    pass

    else : 
        print("usage: {} <width>x<height>".format(__file__))




if __name__ == "__main__":
    main()