#!/usr/bin/env python3

import math as m
import struct


def xgcd(a, b):
    if a == 0: return (b, 0, 1)
    else:
        g, y, x = xgcd(b % a, a)
        return (g, x - (b // a) * y, y)


def mulinv(a, n):
    g, x, _ = xgcd(a, n)
    if g!= 1: raise Exception('No modular inverse')
    else: return x % n


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Generate a random image size in between 5 and 25 pixels,
    # with a maximum of 4 pixel rows on each side.
    def randimgsize():
        rows = max(
                1, min((int.from_bytes(
                    rng.read(1), 'big'), 4)))
        cols = max(
                5, min((int.from_bytes(
                    rng.read(1), 'big'), 25)) + 2*rows)

        return rows, cols


    imglen = 0
    imgdata = bytearray()
    row = bytes()
    for i in range(randimgsize()[0]):
        lzwcodelen = int.from_bytes(
                rng.read(1), 'big')
        while len(row) + 1 < 2**lzwcodelen:
            row += bytes([int.from_bytes(
                        rng.read(1), 'big')])
        
        prevbyte = None
        pixdata = []
        while len(pixdata)*2**(lzwcodelen-1) < len(row):

            if not prevbyte is None:
                sublen = min(
                            int.from_bytes(
                                rng.read(1), 'big'),
                            2**(lzwcodelen+1)+1)

                if sublen >= (2**(lzwcodelen+1)):
                    pixdata += [(sublen >> 8) & 0xFF,
                                sublen & 0xFF]
                    prevbyte = None

                elif 1 <= sublen <= 2**lzwcodelen:

                    pixdata += [mulinv(
                                    int.from_bytes(
                                        rng.read(1), 'big'),
                                    2**lzwcodelen)]
                    prevbyte = sublen
                    
                elif sublen == 1:
                    pixdata[-1] |= prevbyte << 8
                    
            else:
                sublen = int.from_bytes(
                            rng.read(1), 'big')
                
                if 1 <= sublen <= 2**lzwcodelen:
                    pixdata += [sublen]
                    prevbyte = None
                    
                elif sublen == 0:
                    break
        
        lzwbytes = bytes([(len(pixdata)-1).bit_length(),
                          ((len(pixdata)-1) >> 8)]) + \
                      bytes(pixdata[::-1])
        imgdata += lzwbytes
        imglen += len(lzwbytes)

    header = b'GIF89a'
    out.write(header)
    out.write(((imglen//256)*(m.log2(imglen//256))+7)//8
              .to_bytes(1,'little'))
    
    out.write(bytes([imglen%256]))
    out.write(imgdata[:-1])
    