#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Random GIF generator."""


# Generates an animated GIF with no compression.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = [b"GIF87a",
              b"GIF89a"]
    version = rng.read(len("GIF87a"))
    assert len(version) > 0
    assert version[:len("GIF8")] == bytes("GIF8", encoding='ascii')
    version += b"\x00\x3B"

    frame_size = rng.read(2)
    width = (int.from_bytes(data=frame_size[0:2],
                            byteorder='big')) & 0xffff
    size -= 2
    height = (int.from_bytes(data=frame_size[2:],
                             byteorder='big')) & 0xffff
    size -= 2

    if frame_size < b'\x00\x20':
        raise Exception("Frame size too small!")
    
    flags = rng.read(1)[0] & 0xf0
    size -= 1

    palette_size = None
    table_id = None
    local_color_table = False
    interlace = True
    sort = False
    gct_size = 2 << bool(flags & 0x8)
    has_lct = bool(flags & 0x1c) and not bool(flags & 0x20)
    lct_present = bool(flags & 0x1c) or bool(flags & 0x20)
    lct_size = 2 << bool(flags & 0x10)
    background = -1
    delay = -1
    transparent = -1
    dispose = -1
    loops = -1

    while size > 0:
        chunk_type = rng.read(1)[0]
        size -= 1

        if chunk_type == ord(b'#'):     # plain text extension block
            ext_label = rng.read(1)[0]
            size -= 1

            if ext_label == ord(b'F'):   # graphic control extension
                ext_subblocks = []

                while size > 0:
                    subblock = rng.read(1)

                    if subblock == ord(b';'):
                        break

                    ext_subblocks.append(subblock)
                    size -= len(subblock)

                ext_subblocks = b"".join(ext_subblocks)
                
                try:
                    del ext_subblocks[-1]    # remove last semicolon
                    num_bytes = len(ext_subblocks) // 2
                    ext_subblocks = list(struct.unpack("<%sH" %(num_bytes,),
                                                      ext_subblocks))
                    
                    while len(ext_subblocks)!= 4:
                        ext_subblocks.append(-1)
                    
                    # unpacked as little endian unsigned shorts 
                    flags &= ~0xf0      # clear every bit except f
                    flags |= ext_subblocks[0] & 0xf0
                    table_id = ext_subblocks[0] >> 4
                    interlace = not not (flags & 0x40)
                    sorted = not not (flags & 0x20)
                    gct_size = 2 << bool(flags & 0x8)
                    lct_present = bool(flags & 0x1c) or bool(flags & 0x20)
                    lct_size = 2 << bool(flags & 0x10)                    
                    dispose = ext_subblocks[1]
                    transparency_given = bool(ext_subblocks[2])
                    delay = ext_subblocks[3] * 10
                    loop_count = ext_subblocks[2] & 0xff
                except Exception as e:
                    assert False
        
            elif ext_label == ord(b'C'): # comment extension
                ext_subblocks = []

                while size > 0:
                    subblock = rng.read(1)

                    if subblock == ord(b'\x00'):
                        break

                    ext_subblocks.append(subblock)
                    size -= len(subblock)
            
                pass

            else:                       # unknown extension type
                raise Exception("Unknown extension body.")
        
        elif chunk_type == ord(b'!'):   # image descriptor  
            pass

        elif chunk_type == ord(b','):   # local color table 
            pass

        else:                           # other chunks are ignored
            continue
    
    # write to output 
    out.write(version)
