#!/usr/bin/env python3


from sys import argv as args
from pathlib import Path
from os import urandom as randbytes
import struct
from typing import BinaryIO


def generate_random_gif(
        name: str, size: int
    ) -> None:
    
    if not (Path(".").absolute() / f'{name}.gif').exists():

        with open(f"{name}.gif", 'wb+') as outfile:

            buff = randbytes(size * 4)

            outfile.write("GIF87a".encode())

            # write global color table entries
            gct_entries = [struct.unpack("<H",buff[i*3:(i+1)*3])[0] for i in range(255)]

            print(", ".join([hex(e)[2:] for e in gct_entries]))
            
            num_colors = len(gct_entries)
            color_depth = 8
            colors_sorted = sorted(set(gct_entries))
            print(len(colors_sorted), colors_sorted)
            
            packed_color_map = b""
            for color in colors_sorted:
                entry = [b"\x00"]*(num_colors + 1)

                idx = gct_entries.index(color)
                
                position = idx << (color_depth - 1)
                entry[position // 8] |= 1 << (position % 8)

                alpha = (((idx ^ (idx >> 1)) & 1) * 255)

                entry[-1] += alpha.to_bytes(length=1, byteorder="little")

                packed_color_map += b"".join(entry)
                
                
            palette_size = 2**color_depth * (num_colors + 1) 
            bit_mask = 2**color_depth - 1
            
            print(palette_size, bit_mask)

            # Write global color table
            outfile.write((packed_color_map[:6]).ljust(10).encode())

            # Write image descriptor
            outfile.write(((size//10).to_bytes(length=2, byteorder="little")).ljust(9).encode())
            outfile.write((b"\x00\x00\x00").encode())

            # LZW min code size
            outfile.write(b'\xff')
                
            cell_bits = color_depth * 2
            max_code_bit_count =  12
            clear_code = 1<<(max_code_bit_count - 1)
            end_of_info_code = clear_code + 1
            next_code = clear_code + 2
            max_codes = 4096
            codes = {}  
            last_codes = []
            buffer = ""
            lzw_min_code_size = 6
            current_code_bit_count = 0
            outputting_byte = False
            data_block = b""

            while True:
                    
                code = data_block[0] | (data_block[1] << 8)
                
                try:
                    decoded = codes[code][0]
                    data_block = data_block[4:]
                    
                    if len(decoded) > 1:
                        buffer += decoded

                    if outputting_byte:

                        outfile.write(
                                bytes([(buffer[0])]))

                        current_code_bit_count -= 8
                        
                        if current_code_bit_count == 0:
                            buffer = buffer[:-1]
                            
                            if not any(buffer):
                                break
                            
                            data_block = data_block[1:]

                            outputting_byte = False
                        
                            

                except KeyError:
                    pass

                if len(data_block) < 2:
                    break

                code = data_block[0] | (data_block[1] << 8)

                try:
                    decoded = "".join(last_codes + [codes[code][0]])
                    data_block = data_block[2:]

                    if len(decoded) > 1:
                        buffer += decoded

                    if outputting_byte and len(buffer[:-1]) >= 1:
                        outfile.write(
                                bytes([(buffer[-1])]))

                        current_code_bit_count -= 8
                        
                        if current_code_bit_count == 0:
                            buffer = buffer[:]
                            buffer = buffer[:-1]
                            
                            if not any(buffer):
                                break
                                
                            data_block = data_block[1:]

                            outputting_byte = False
                        
                            

                except KeyError:
                    pass


                if current_code_bit_count <= 24:
                    current_code_bit_count += cell_bits
                    
                    
                    if current_code_bit_count > max_code_bit_count:
                        current_code_bit_count = max_code_bit_count
                    
                
                if len(data_block)!= 2 or \
                   len(codes) == max_codes or \
                   current_code_bit_count == max_code_bit_count:


                    if outputting_byte and len(buffer[:-1]) >= 1:
                        outfile.write(
                                bytes([(buffer[-1])]))

                    wiped_up_codes = dict({v[1]: v[0] for k,v
                                              in codes.items()})
                    for i, c in enumerate(wiped_up_codes):
                        if c is None: continue             
                        codes[(clear_code + i)][0] = wiped_up_codes[c]                       
                        del codes[wiped_up_codes[c]]

                    for lc in last_codes:
                        if lc is None: continue                  
                        codes[(end_of_info_code)][0] = lc
                        del codes[lc]

                    last_code = list(filter(lambda x: x[1],
                                                  [(k,v) for
                                                   k,v in
                                                   codes.items()]))

                    if last_code:
                        last_codes = [last_code[0][1]]
                    else:
                        last_codes = []


                    clear_code <<= 1
                    next_code <<= 1
                    end_of_info_code <<= 1
                    max_code_bit_count += 1

                    outputting_byte = True
                    data_block = data_block[2:]
                    


            

                

                    

        

    
    

