#!/usr/bin/env python3

import argparse
from pathlib import Path
from typing import BinaryIO


def parse_args():
    parser = argparse.ArgumentParser()

    def rand_gen(p):
        if p is None or not p.is_file():
            raise FileNotFoundError("The path to /dev/urandom should be a valid file.")
        return open(p, 'rb', buffering=0)
    
    parser.add_argument('outpath', help="The output path for the generated GIF.", type=Path)
    parser.add_argument('--rand-seed', '-r', 
                        metavar="FILE",
                        default='/dev/urandom',
                        dest='rng',
                        action='store',
                        type=Path,
                        nargs='?',
                        const=None,
                        required=False,
                        help="A file to fetch data from as an RNG (default '/dev/urandom').")

    args = vars(parser.parse_args())

    # process arguments
    if len(sys.argv) <= 1:
        parser.print_usage()
        exit(-1)
    elif len(sys.argv[1:]) == 1 and '--help' in sys.argv:
        pass
    else:
        try:
            args['rng'] = rand_gen(args['rng'])
        except FileNotFoundError as e:
            print(e)
            exit(-2)

        try:
            with open(args['outpath'], "wb+") as f:
                pass     # check writeability of the file
        except Exception as e:
            print(e)
            exit(-4)
    return args
    

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
    print(f"generated {len(data)}-byte image")



if __name__ == "__main__":
    args = parse_args()
    with open(args["outpath"], "wb+") as out:
        msg = input(
            "[!] Do you want to continue? [Y/N] ") \
             .strip().lower() \
             .split()[0].replace('y', 'yes')

        if msg!= 'yes':
            print("[*] Exiting...")
            exit(0)
        
        generate_random_gif(**args)