import argparse
import os
import sys
import time
from io import BytesIO

import numpy as np


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--seed', type=str,
                        help=('The path to a binary file from which '
                              'the seed can be read'))

    args = vars(parser.parse_args())

    if not args['seed']:
        raise RuntimeError("must specify --seed input file")

    with open(args['seed'], 'rb') as f:
        buf = f.read()

    rng = BytesIO(buf)

    while True:
        try:
            with open('/tmp/test.gif', 'wb+') as outf:
                generate_random_gif(rng, outf)

            time.sleep(.5)

        except KeyboardInterrupt:
            break


def generate_random_gif(rng, out):
    header = b'GIF89a'
    datalen = (struct.unpack('<I', rng.read(4)))[0] & 0xfff

    print('header', header, 'datalen', datalen)

    # Write pre-frame bytes
    out.write(header)
    for _ in range(datalen):
        out.write(
            struct.pack("<B", np.random.randint(low=0x00, high=0xFF + 1))
        )


if __name__ == '__main__':
    start = time.time()

    main()

    end = time.time() - start
    print('Complete! Took {} seconds.'.format(end))