#!/usr/bin/env python3

import sys
import os
import shutil
import subprocess
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Open an fd for the /dev/null so we can use it as stderr later on
    null = open('/dev/null', 'w')

    # Run ImageMagick's convert program with parameters from STDIN piped through
    codefile = ['convert'] + [str(x) for x in sys.argv[1:]]
    proc = subprocess.Popen(codefile, stdin=subprocess.PIPE, stdout=out, stderr=null)

    # Pipe 4 bytes at a time to its input port
    while True:
        chunk = rng.read(4)
        if not len(chunk): break

        # Interpret them as little endian integers to write
        proc.stdin.write(struct.pack('<I', int.from_bytes(chunk, byteorder='little')))

    # Close the process's standard input to signal end of transmission
    proc.stdin.close()

    if wait := not proc.wait():
        print('ImageMagick returned %d.' % wait)

    else:
        print('Success.')

    # In case the actual image is really large or something goes wrong...
    out.truncate(proc.stdout.tell())
    out.flush()

