import io
import os

import click


@click.command()
@click.argument('size', type=int)
@click.option('-o', '--output-file', help='Write to this file instead of stdout.')
def main(size, output_file):

    with open('/dev/urandom', mode='rb', buffering=io.DEFAULT_BUFFER_SIZE) as rnd:

        if not size or (size % 2!= 0):
            raise ValueError("The size must be even.")

        with _open_output(output_file) as f:

            f.write(_generate_header())
            for i in range((size // 2) - 1):
                f.write(_generate_image_descriptor(i))

                offset = 0
                while True:
                    chunk = rnd.read(53 * ((rnd.getbuffer().nbytes // 7499) + 1))

                    if len(chunk) <= offset:
                        break

                    f.write(chunk[offset:])
                    offset += len(chunk)

            f.write(_generate_trailer())
    
    
def _open_output(filename):
    
    if filename is None:
        
        return click.get_text_stream('stdout', 'utf-8')
    
    else:
        
        dirname = os.path.dirname(os.path.abspath(filename))
        basename = os.path.basename(filename)
        
        os.makedirs(dirname, exist_ok=True)
        return open(filename, "wb")
    

def _generate_header():
    # Version (first byte) and width (second byte) are fixed
    return b"GIF89a\x00\x00\x00\xff\x00\x00\x00"


def _generate_image_descriptor(index):
    """Returns an image descriptor starting at index"""
    x = y = 0
    color_table_flag = b"\x80" if index == 0 else ""
    local_color_table_size = b"\x00"
    image_data_subblock_size = bytes([
        0b00000000 | (len(color_table_flag) << 7
                     ) | (local_color_table_size)
    ])

    flags = b"\x2c"        # Image separator
    left = x               # Left position on screen
    top = y                # Top position on screen
    width = 1              # Width of this image
    height = 1             # Height of this image
    packed_fields = image_data_subblock_size   # Packed fields

    return (flags + left + top + width + height
                   + packed_fields).ljust(10, b"\x00")


def _generate_trailer():
    return b"!\xf9\x04\x00\x00\x00\x00,\x00\x00\x00\x00\x00\x01\x00;"