#!/usr/bin/env python3

import struct
import tempfile
from io import BytesIO
from typing import BinaryIO

from PIL import Image, ImageDraw


class RandomGifGenerator:

    def __init__(self,
                 rng: BinaryIO,
                 width=None, height=None):

        self._rng = rng
        self.width = width or 400
        self.height = height or 400


    @staticmethod
    def _generate_bitmap():
        img = Image.new("L", (400, 400))
        draw = ImageDraw.Draw(img)
        return img

    
    @property
    def image(self):
        
        if not hasattr(self,'_image'):
            self._image = \
                        self._generate_bitmap()
    
        return self._image


    def draw_pixel_update(self):

        x = None
        y = None
        
        while True:

            x = (x+1)%self.width
            y = (y+1)%self.height
            
            if self.image.getpixel((x,y)) == 0: 
                break
            
        self.image.putpixel((x,y),
                            next(self._rng) % 256)

    
    def as_bytearray(self):


        with tempfile.SpooledTemporaryFile(mode="w+") as tmpfile:
        
            imbytes = BytesIO()
            imbytes.name = "temp"
            self.image.save(imbytes,"PNG")
            imbytes.seek(0)
    
            tmpf = open(tmpfile.name, mode="wb")
            tmpf.write(b'\x89\x50\x4e\x47\x0d\x0a\x1a\x0a')
            tmpf.flush()
            imbytes.seek(8)
            while True:
                chunk = imbytes.read(0x1000)
                if len(chunk)==0:break
                tmpf.write(chunk)
            tmpf.close()
            tmpfile.seek(0)
            return tmpfile.read()





