#!/usr/bin/env python3

import os
import struct
from pathlib import Path


# random number generator `/dev/urandom`
def get_rng() -> BinaryIO:
    path = "/dev/urandom"

    if not os.path.exists(path):
        raise IOError("cannot open %s for reading." % (path,))
    
    return open(path, "rb", buffering=0)


# add a 16-bit integer to end of file
def append(bytes_: bytes, num: int) -> bytes:
    lsb = (num & 255).to_bytes(length=1, byteorder="big")
    msb = ((num >> 8) & 255).to_bytes(length=1, byteorder="big")
    return bytes_ + msb + lsb
    

# write a GIF header and footer to `out`, followed by a single frame of length `datalen`.
# Any generated data is returned in a list.
def generate_random_gif(
        datalen: int, out: BinaryIO
    ) -> [int]:
    # Header
    size = 7 + datalen
    gif_header = b"GIF89a"
    logical_screen = b"\x00\x00\xff\x00\x00\x00\x00"
    global_color_table = b"\x00\x00" * 256 
    application_extension = b""
    extension_label = b"\x21\xf9\x04"
    loopcount = b"\x00"
    netscape_application_block = b"\x0bNETSCAPE2.0\x03\x01"
    netscape_loop_control = b"\x00\x00"
    block_terminator = b"\x00"
    ext_subblocks = netscape_application_block + netscape_loop_control \
                    + loopcount + block_terminator
    subblock_size = len(ext_subblocks)
    app_ext_size = 11 + subblock_size
    application_extension = append(
            application_extension, 
            app_ext_size - 9)
    application_extension += extension_label
    application_extension = append(application_extension, subblock_size)
    application_extension += ext_subblocks
    graphics_control = b'\x21\xf9\x04\x00\x00\x00\x00\x00'
    graphicsdatalen = b''
    for i in range((datalen // 3)):
        graphicsdatalen += graphics_control
        rgb = []

        for j in range(3):
            rgb.append(rng.read(1)[0])
        
        graphicsdatalen += bytes(rgb)


    frame = b""
    frame += b"\x2c"                        # Image separator
    frame += b"\x00"                        # Left offset
    frame += b"\x00"                        # Top offset
    frame += b"\x00"                        # Width
    frame += b"\x00"                        # Height
    frame += b"\x08"                        # Packed field
    frame += b"\x00\x00"                    # Local color table index
    frame += b"\xff\xff\xff"                # Background color
    frame += b"\x00\x00"                    # Pixel aspect ratio
    frame += graphicsdatalen
    frame += b";"                          # Trailer
    frames = [
        frame
    ]
    total_bytes = len(gif_header) + len(logical_screen) + len(global_color_table)\
               + len(application_extension) + sum([len(frame) for frame in frames]) + 1
    
    
    out.write(total_bytes.to_bytes(length=4, byteorder='big', signed=False))
    out.write(gif_header)
    out.write(logical_screen)
    out.write(global_color_table)
    out.write(application_extension)
    for frame in frames:
        out.write(frame)
    # Footer
    out.write(b"\x3b")                  # terminator character

