#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) with
# dimension `width` x `height`.
def write_gif(out: BinaryIO, rng: BinaryIO, width: int, height: int):
    out.write(b"\x47\x49\x46\x38\x39\x61") # "Gif89a" header

    # Logical screen descriptor
    out.write(struct.pack('<HHBB', width, height, 0xf0 | (2 << 4), 0))

    # Global color table
    for i in range(1 << 4):
        out.write(struct.pack("<BBB", *[i for _ in range(3)]))

    # Graphic control extension
    out.write(bytes([0x21, 0xF9]))                          # Extension introducer and label
    out.write(bytes([0x04]))                                # Block size
    out.write(struct.pack("<HBB", 0xffff, 0xff, 0x20 | 1)) # Disposal method and user input flag

    # Image Descriptor
    out.write(b'\x2C')                       # Image separator
    out.write(struct.pack('<HH', 0, 0))      # Origin
    out.write(struct.pack('<HH', width, height))
    out.write(bytes([0]))                    # Packed field: no local colour map

    # Local color table
    for i in range(1 << 4):
        out.write(struct.pack("<BBB", *[i for _ in range(3)]))

    # Image data
    num_data = width * height // 2
    for i in range(num_data):
        out.write(rng.read(1)[0] & 0xf)

    # Trailer
    out.write(b'\x3B') # Trailer

with open("/dev/