#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\xff\xe0\x00'
    padding = [b'\x00'] * blocksize
    out.write(header + len(data).to_bytes(3, 'big'))
    while True:
        datalen = int.from_bytes(
                rng.read(3), byteorder='big')
        if not datalen: break
        data = rng.read(datalen)
        padding[1] = len(data).to_bytes(3, 'little')[-1:]
        out.write(struct.pack('<H', sum([ord(x) for x in padding])) + data)
    # EOF
    padding[1] = b'\x00'*2
    out.write('\xFF\xD9'+b'\x00'*3)

# Generates a random MP4 file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_mp4(
        rng: BinaryIO, out: BinaryIO
    ):
    moov = rng.read()
    out.write(moov[:8]+len(moov).to