#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)

    # size of the color table
    ctsize = int.from_bytes(rng.read(1), 'little')

    if ctsize > 0:

        # read in palette values (RGB triples)
        paldata = [struct.pack('BBB', *x) for x in
            [(y % 256, y // 256, y // 256 * 256)
                for y in range(ctsize)]]

        palents = bytes()
        for i in range(len(paldata)):
            # each entry is 3 bytes long
            palents += paldata[i] + b'\x00\xff'

        # skip image data section to get to trailer
        # (graphic control extension, image descriptor, local color table,
        # body chunk, etc).
        rng.seek((image_width * image_height) + 9)

        d = bytearray()
        d += header
        d += struct.pack('<HH', image_width, image_height)
        d += b'\x87\x00\x00'
        d += struct.pack('B', len(paldata))
        d += palents
        d += b'\x2c\x00\x00\x00\x00'
        d += rng.read(-1)
