#!/usr/bin/env python3

import struct
from typing import List, Dict
from yaml import safe_load as load
from collections import deque

MAC_ADDRESS = 0x5c16d2fffe8e9a5b
IPV4_ADDR   = 0xc0a80104
UDP_PORT    = 7777

#
# NOTE:
#   For the original protocol description please refer to GENIVI's website:
#       https://www.genivi.org/fileadmin/user_upload/files/doc/automotive/vehicle-env/devices/acl_gateway_protocol.html
#

def send(mac:int, ip:int, port:int):
    packet_header = b'\xaa\xbb'
    data          = b''

    # Link-layer header (6 bytes)
    link_layer_header = struct.pack('!Q', mac)

    # Network layer header (8 bytes)
    network_layer_header = struct.pack('!L', ip) + struct.pack('!H', UDP_PORT)

    data += packet_header            \
        + link_layer_header           \
        + network_layer_header        \
        + struct.pack("!BB", 1, 2)    \
        + struct.pack("!B", 3)

    return data
