
# https://www.w3.org/Graphics/GIF/spec-gif89a.txt
import os
import sys
import struct
import argparse
import math
from PIL import Image

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='Convert GIF to LZW img')
    parser.add_argument("infile", help="input file")
    parser.add_argument("outfile", help="output file")
    args = parser.parse_args()
    
    infilename = args.infile
    outfilename = args.outfile

    infile = open(infilename, 'rb')
    outfile = open(outfilename, 'wb')

    makeLZWImg(infile, outfile)

    infile.close()
    outfile.close()

def makeLZWImg(inf, out):

    header = b'\xff\xfe' \
             + struct.pack('<BBHBB',
                 6,      # width
                  6,     # height
                  0x100, # packed field: no global color table, no interlacing
                          # - otherwise it will use the default from the spec
                  0,      # default background color
                  0)      # default pixel aspect ratio
    rng = inf.read_at(0)
    sig, ver, swidth, sheight, flds, bgcol, pxar = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)

    # size of the color table
    ctsize = int.from_bytes(rng.read(1), 'little')

    if ctsize > 0:

        # read in palette values (RGB triples)
        paldata = [struct.pack('BBB', *x) for x in
            [(y % 256, y // 256, y // 256 * 256)
                for y in range(ctsize)]]

        palents = bytes()
        for i in range(len(paldata)):

            # write out the length byte first
            palents += struct.pack('B', len(paldata[i]))
            palents += paldata[i]

    else:
        palents = b''

    # colormap is followed by a local color table
    # TODO: we could do something with this; but not now...
    rng.seek((3 << 8) | ctsize)

    datalen = int.from_bytes(rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)
