#!/usr/bin/env python3

import sys, os

def main():
    if len(sys.argv) != 3:
        raise Exception('Usage: genimage.py <rng> <out>')
    with open(sys.argv[1], 'rb') as rng, \
         open(sys.argv[2], 'wb') as out:
            generate_jfif(rng, out)

def generate_jfif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1024 bytes of binary random data, to be written
    # afterwards as JFIF-formatted image.
    jfif = rng.read(1024)

    # First two bytes are SOI marker (start of image).
    # Last five bytes are EOI marker (end of image).
    jfif = b'\xff\xd8' + jfif + \
           b'\xff\xd9' 

    # Write out data section of JFIF.
    out.write(jfif[2:-5])

