#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    # 6 bytes of fixed data
    while True:
        try:
            assert len(rng.read(len(header))) == len(header)
            break
        except AssertionError as e:
            continue
    width = int.from_bytes(rng.read(2), 'little') + 1
    height = int.from_bytes(rng.read(2), 'little') + 1
    background_color = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    color_table_size = 2 ** (rng.read(1)[0] & 0b111)

    # Write image dimensions and such
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write(struct.pack('B', 0xF7)) # flags
    out.write(background_color.to_bytes(1, 'little'))
    out.write(pixel_aspect_ratio.to_bytes(1, 'little'))

    # Write global color table
    for _ in range(color_table_size):
        c = rng.read(1)[0]
        out.write(c.to_bytes(1, 'little'))
    out.write(b'\x00\x01\xFF') # End of global table

    # Write an Extension Block, which turns out to be required
    # for some implementations? Or something? IDK, but I put it here
    ident_string = b'NETSCAPE2.0\x03\x01'
    while True:
        try:
            assert len(rng.read(len(ident_string))) == len(ident_string)
            break
        except AssertionError as e:
            continue
    out.write(b'\x21')
    out.write(b'\xff')
    out.write(b'\x0b')
    out.write(b'xxxxxxxxx'.encode())
    out.write(ident_string)
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')

    # Finally, write the graphics control extension block
    gceb = bytearray([0x21, 0xf9])
    gceb += (4).to_bytes(1, 'little')
    gceb += b'\x04'
    gceb += (0xff).to_bytes(1, 'little')
    gceb += (0xff).to_bytes(1, 'little')
    gceb += b'\x00'
    gceb += b'\x00'
    out.write(gceb)

    # Now let's output all the pixels
    pass
