#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the image is given by `width` and `height`.
# If no extension data is required (for example if you want to use it in MIME), set `extradata` to None.
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO, width: int, height: int, extradata=None
    ):
    marker = 0xff
    sof = 0xc0
    soi = 0xd8
    eoi = 0xd9
    app0 = 0xe0
    app14 = 0xee
    quantize = 0xdb
    huffman = 0xc4
    dqt = 0xfe
    restart = 0xdd
    dht = 0xc4
    start_scan = 0xda
    scaline = 0x0c
    dri = 0xdd

    # Write header
    out.write(struct.pack('>BBHH', marker, soi, 0, 0))
    out.write(struct.pack('>BBHBHBB', marker, app0, 16, 0xe0, 0x1b, huffman, 0x1b))
    out.write(struct.pack('>BBHB', marker, quantize, 2, 0xf4))
    out.write(struct.pack('>BBH', marker, dqt, 175))
    out.write(struct.pack('>BBHBB', marker, restart, 0, 0, 0))
    out.write(struct.pack('>BBH', marker, dht, 0x100))
    out.write(struct.pack('>BBH', marker, dht, 0x110))
    out.write(struct.pack('>BBH', marker, dht, 0x101))
    out.write(struct.pack('>BBH', marker, dht, 0x111))
    out.write(struct.pack('>BBH', marker, dht, 0x102))
    out.write(struct.pack('>BBH', marker, dht, 0x112))
    out.write(struct.pack('>BBH', marker, dht, 0x103))
    out.write(struct.pack('>BBH', marker, dht, 0x113))
    out.write(struct.pack('>BBH', marker, dht, 0x104))
    out.write(struct.pack('>BBH', marker, dht, 0x114))
    out.write(struct.pack('>BBH', marker, dht, 0x105))
    out.write(struct.pack('>BBH', marker, dht, 0x115))
    out.write(struct.pack('>BBHH', marker, start_scan, 0, 0))
    out.write(struct.pack('>BB', marker, sof, 8+len(extradata)))
    out.write(struct.pack('>HBHLHH', height, sof, width, 3, 0, 0, 0, 0))
    for i in range(height):
        out.write(struct.pack('>'+str(width)+'B', *tuple(rng.read(width))))
    out.flush()

    out.write(struct.pack('>BB', marker, eoi, 0))

    out.flush()

def generate_random_file(rng, name: str, size: int):
    with open(name