#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'little') & 0xffff

    # Image Header
    ihdr = [0] * 10
    ihdr[0] = 0x2c                   # image separator
    ihdr[1] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[2] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[3] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[4] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[5] = (int.from_bytes(rng.read(4), 'little')) & 0xf
    ihdr[6] = (int.from_bytes(rng.read(4), 'little')) & 0xf
    ihdr[7] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[8] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[9] = (int.from_bytes(rng.read(4), 'little')) & 0xff

    ihdrstr = struct.pack('<BHHBBBBBB', *ihdr)
    assert len(ihdrstr) == 10

    # Graphic Control Extension
    gce = [0] * 5
    gce[0] = 0xf9
    gce[1] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    gce[2] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    gce[3] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    gce[4] = ((int.from_bytes(rng.read(4), 'little')) & 0x1f) | ((int.from_bytes(rng.read(4), 'little')) & 0xe0) << 8

    gcestr = struct.pack('<BHBBB', *gce)
    assert len(gcestr) == 6

    # Netscape 2.0 loop extension
    nle = [0] * 3
    nle[0] = 0x21            # extension code
    nle[1] = 0xFF		       # application extension
    nle[2] = 0x0b               # size of block - 1

    nlestr = struct.pack('<BBB', *nle)
    assert len(nlestr) == 3

    # Loop count
    lcount = [0]*2
    lcount[0] = 0x00
    lcount[1] = 0x01

    lcountstr = struct.pack('<BB',*lcount)
    assert len(lcountstr) == 2

    # Digital signature
    dsig = [ 0 ] * 16
    for i in range(0, 16):
        dsig[i] = (int.from_bytes(rng.read(1), 'little')) & 0xff

    dsigstr = struct.pack("!16B",*dsig)
    assert len(dsigstr) == 16

    # Pixel data
    pdata = [0] * datalen
    for i in range(0, datalen):
      pdata[i] = (int.from_bytes(rng.read(1), 'little')) & 0xff

    pdatastr = struct.pack("<{}B".format(datalen),*pdata)
    assert len(pdatastr) == datalen

    # Output File
    out.write(header)
    out.write(struct.pack('<H', datalen))
    out.write(ihdrstr)
    out.write(gcestr)
    out.write(nlestr)
    out.write(lcountstr)
    out