#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Randomly choose between 1 and 255 bytes of data
    size = int.from_bytes(
            rng.read(1),
            byteorder='little',
            signed=False
        ) % (256 - len(header))

    # Generate a random byte stream
    data = rng.read(size)

    # Write it out as "the right thing to do"
    datalen = len(data)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)
