#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, nframes=12) -> None:
    """ Generate a GIF animation with random frames.
    Randomization is done by writing random bytes to the given RNG file object."""
    # First assemble the GIF header.
    out.write(b"GIF89a")  # Header string
    packer = struct.Struct("!H").pack   # Unsigned short (LSB first)
    bwidth = packer(width)[0]           # Width as unsigned byte
    bheight = packer(height)[0]         # Height as unsigned byte
    out.write(b"\x21\xf9\x04\x05\xff\xff\xff\x00")
    # Collect global color table.
    rgbas = [rng.read(4).hex() for _ in range(nframes)]
    # Loop over all frames.
    gif_global_table = True
    for index, rgba in enumerate(rgbas):
        if index != 0 or not gif_global_table:
            out.write(b'\x2c')
        else:
            out.write(b'\x21\xfe\x10\x00'.format(bwidth, bheight))
            out.write(bytes([0x21]))                       # Graphic extension
            out.write(b'NETSCAPE2.0\x03\x01')              # Netscape loop count (3)
            out.write((len(rgbas)-1).to_bytes(2, 'big'))   # NS_Loop_Size
        out.write(b'\x00\x00')                             # Offset x, y
        out.write(bwidth + bheight)                       # Frame dimensions
        out.write(b'\x87')                                # Local color table flag
        out.write('\x00')                