#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    width = 0x1f & ord(rng.read(1))
    height = 0xf & ord(rng.read(1))
    
    out.write("GIF89a\x01\x00")
    out.write(struct.pack("<HHB", width, height, 0xa7))
    out.write("\xff\x0bNETSCAPE2.0\x03\x01") # Loop count of 1
    loop_count_end = out.tell()
    out.write("\x00" * 4)                   # Placeholder for size of loop count
    out.write("\x00")                       # Unused terminator

    def write_frame():
        out.seek(loop_count_end - 4)
        out.truncate()
        
        size = out.tell() + 10
        assert size < 0xffffff
        out.write(struct.pack(">I", size)) # Block length (placeholder)
        out.write("\x2c" + "\x00" * 6)     
        out.write(struct.pack("<HHBB", width, height, 0, 0))
        out.write("\x00")                   # Background color index
        out.write("\xa7")                   # Default graphics control extension
        out.write("\x08")                   # 8-bit pixels
        out.write(ord(rng.read(1)))         # Pixel aspect ratio
        out.write("\x02")                   # Two global colors in graphic control extension
        out.write(rng.read(3))              # Graphic control extension terminator
        out.write("\x00" * ((width+7)/8*height+3)&~3) # Set all pixels to transparent

    if width == 31 and height == 15:
        write_frame()

    out.write("\x3b")

with open("/tmp/test.gif",