#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    background_color_index = 0
    aspect_ratio = 0
    global_palette_flags = 0x80 | 0x07
    palette = [
        0, 0, 0,   # Transparent
        255, 0, 0, # Red
        0, 128, 0, # Green
        0, 0, 255, # Blue
    ]
    graphic_control_extension_block = b'\x21\xf9\x04\x00\xfc\xff\x00'
    application_extension_block = (b'\x21\xff\x0bNETSCAPE2.0\x03\x01' +
                                   bytes([0]) * 16)
    image_descriptor = b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00'
    lzw_minimum_code_size = 2
    trailer = b'\x3b'
    subblocks = []
    while True:
        if rng.read(1)[0] < 128:
            break
    while True:
        n = int.from_bytes(rng.read(1), byteorder='big')
        if n > len(subblocks):
            subblocks += [
                rng.read(n - len(subblocks))
            ]
            break
        subblocks[n] = rng.read(1)
    for i in range(len(subblocks)):
        if not subblocks[i]:
            del subblocks[i]

    head = \
        header + \
        bytes((background_color_index, aspect_ratio)) + \
        bytes((global_palette_flags,)) + \
        palette + \
        application_extension_block + \
        graphic_control_extension_block + \
        image_descriptor + \
        bytes((lzw_minimum_code_size,))
    assert len(head) == 41
    out.write(head)
    for sb in subblocks:
        out.write(struct.pack('!B', len(sb)))
