#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    # 0xFFD8 should be at the beginning of every block
    marker = [
        0xFF, 0xD8, *struct.unpack('<H',
                    rng.read(2)),
        ]
    while True:
        out.write(marker)