#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)

    # size of the color table
    ctsize = int.from_bytes(rng.read(1), 'little')

    if ctsize > 0:
        # skip the first color in the table -- this is an invalid value
        rng.seek(1 * (ctsize + 1), 1)

    # write transparency
    out.write(b'\xff\xf9\x04\x01')
    # read the transparent color index
    tci = struct.unpack('B', rng.read(1))[0]
    out.write(struct.pack('BB', tci, tci))

    # write application extension
    out.write(b'\x21\xf9\x04\x08')
    out.write(b'NETSCAPE2.0\x03\x01')
    out.write(b'\x00\x00\x00\x00')

    # write Netscape loop count (0u == repeat forever)
    out.write(b'\x00\x01')

    # write graphic control extension
    out.write(b'\x21\xf9\x04\x01')
    out.write(b'\x00\x00\x00\x00')

    # write GRPCI
    out.write(struct.pack('B', tci))

    # write Image Descriptor
    out.write(b'\x2c\x00\x00\x00\x00')

    # skip LZW minimum code size field
    rng.seek(1)

    # write interlaced indicator
    out.write(b'\x00')

    # create a lookup table for reading pixels from the input
    lut = [None] * (image_width * image_height)
    for i in range(len(lut)):
        lut[i] = struct.unpack('B', rng.read(1))

    # write the image data to the output stream
    for _ in range((image_height / 8) * image_width):
        # write a pixel from the LUT
        out.write(struct.pack('B', lut[_]))

def main():
    with open('/dev/urandom',