
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    # RGB color palette.
    global_color_table = (
        b'\xFF\x00\x00\xFF\xFF\x00\x00\xFF\x00\xFF\x00\x00\x7F\x7F\x7F' +
        b'\x40\x40\x40\x40\x40\x40\xC0\xC0\xC0'
    )
    # Color table of the GIF itself.
    local_color_table = (
        b'\xFF\x00\x00' +
        b'\x00\x00\xFF' +
        b'\x00\xFF\x00' +
        b'\xFF\xFF\xFF'
    )
    # The first four bytes are the magic signature and version.
    out.write(header)
    # Dimensions is 2-bytes wide each.
    width = rng.read(1)[0] * 256 + rng.read(1)[0]
    height = rng.read(1)[0] * 256 + rng.read(1)[0]
    out.write(width & 0xff)
    out.write((width >> 8) & 0xff)
    out.write(height & 0xff)
    out.write((height >> 8) & 0xff)
    # 8 bits per pixel, global color map enabled.
    out.write(b'\x80\xaa')
    # Global color table begins here.
    # TODO Use length byte?
    # global_color_table = ...
    out.write(global_color_table)
    # End of global color table.
    out.write(b';')
    # Image descriptor block.
    out.write(b',')
    # X position, Y position in pixels.
    out.write(b'\x00\x00\x00\x00')
    # Width and height.
    out.write(width & 0xff)
    out.write((width >> 8) & 0xff)
    out.write(height & 0xff)
    out.write((height >> 8) & 0xff)
    # Local color table enabled.
    out.write(b'\x80')
    # Local color table size, which must be 2^(n+1)-1.
    out.write(b'\x03')
    # Margin between images.
    out.write(b'\x00')
    out.write(b'\x00')
    # Use an interlaced image.
    out.write(b'\x40')
    color_indexes = []
    for _ in range(width*height):
        color_indexes