#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt
    # Global Color Table (offsets 10 through 26)
    global_colorthable = rng.read(768)
    imgwidth, imgheight = (
                rng.read(2),
                rng.read(2))
    # Image Descriptor (offset 4)
    desc = bytes([0x2c]) + rng.read(9)
    repcount = int.from_bytes(rng.read(2), byteorder='big')
    colortable = rng.read(repcount * 3)

    loop_counter = 0

    while True:
        if loop_counter & 0xff == 0 and loop_counter != 0:
            # Need to write an NETSCAPE extension block?
            appextlen = len(b'\x01\xfe\x4e\x53\x43\x41\x50\x50\x45\x32\x2e\x30\x30\x31\x00\x03\x01\x00\x00\x00\x00')
            out.write(struct.pack('<H', appextlen))
            out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')

        out.write(header)
        out.write(imgwidth)
        out.write(imgheight)
        out.write(global_colorthable)
        out.write(desc)
        out.write(data)
        out.write(bytearray([0x3b]*int(datal