#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number genrator `rng` (/dev/urandom).
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1024 bytes of binary random data, to be written
    # afterwards as JFIF-formatted image.
    jfif = rng.read(1024)

    # First two bytes are SOI marker (start of image).
    start = 2
    # Third is JFIF format indicator.
    # Fourth is height in MSBs and fifth is width in MSBs.
    # Sixth is height in LSBs and seventh is width in LSBs.
    # Nine is depth in bits and tenth is a color table flag.
    # If no color table present, next byte indicates number of columns
    # in color table. Else, this byte is zero.

    h, w = struct.unpack('<HH', jfif[5:9])
    d = min(int.from_bytes(jfif[9], 'little'), 8)

    if jfif[10] != 0x0:
        ncol = int.from_bytes(jfif[10], 'little') - 2
    else:
        ncol = None
        
    # We write JFIF and header, but not the random data yet!
    out.write(b'\xff\xd8\xff\xe0')
    out.write(struct.pack('<HBHBBHH', 0x4a46, 0x4946, 0x0001, \
                         