#!/usr/bin/env python3

import sys
import binascii
import os
import time
import math
import struct
from typing import BinaryIO

sys.path.append('/src/python')

import amber

banner = '===================== Amber Gif Gen ====================='
print(banner)
if len(sys.argv) > 1 and not sys.argv[1].endswith('.txt'):
    print("Please supply a .txt file with your seed")
    exit()
elif len(sys.argv) < 2:
    print("No seed supplied, defaulting to plaintext seed 0")
    seedstr = "0"
else:
    with open(sys.argv[1], 'r+b') as f:
        seedstr = f.read().decode()
        while len(seedstr) % 8 != 0:
            seedstr += '\x00'

seed = binascii.unhexlify(seedstr)

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'little') & 0xffff

    # Image Header
    ihdr = [0] * 10
    ihdr[0] = 0x2c                   # image separator
    ihdr[1] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[2] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[3] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[4] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[5] = (int.from_bytes(rng.read(4), 'little')) & 0xf
    ihdr[6] = (int.from_bytes(rng.read(4), 'little')) & 0xf
    ihdr[7] = 0x00                   # no local color table
    ihdr[8] = struct.pack('>B', ((int.from_bytes(rng.read(1), 'little')) & 0xFF))[0]
    ihdr[9] = struct.pack('>B', ((int.from_bytes(rng.read(1), 'little')) & 0xFF))[0]

    out.write(header + bytes(ihdr))

    # LZW Compressed Bytes
    for i in range(datalen-10):
        out.write(struct.pack('>B', ((int.from_bytes(rng.read(1), 'little')) & 0xFF)))

    # Data Block Terminator