#!/usr/bin/env python3

import struct

with open('csr.h', 'w') as out:
    out.write('#ifndef _CSR_H\n#define _CSR_H\n')

for f in ['csr32', 'csr64']:
    with open('/boot/u-boot/{}.csr'.format(f), 'rb') as rng:
        while True:
            try:
                addr, datalen = (struct.unpack('>I', rng.read(4))[0],
                                 struct.unpack('>I', rng.read(4))[0])
            except struct.error:
                break
            header = '#define CSR_{} 0x{:X}\n'.format(f, addr)
            if datalen > 1:
                data = ', '.join(['0x{:08X}'.format(struct.unpack('<I', rng.read(4))[0]) for i in range(datalen // 2)])
                footer = '\n'
            else:
                data = '{:02X}'.format(struct.unpack('B', rng.read(1))[0])
                footer = '\n'
            out.write('{}extern const uchar {}[];\nsize_t sizeof_{};\n'.format(header, f, f, data, footer))

out.write('\n#endif /* _CSR_H */\n')

for f in ['csr32', 'csr64']:
    print('Processing {} ...'.format(f))
    with open('{}.bin'.format(f), 'wb') as out, open('/boot/u-boot/{}.csr'.format(f), 'rb') as rng:
        while True:
            try:
                addr, datalen = (struct.unpack('>I', rng.read(4))[0],
                                 struct.unpack('>I', rng.read(4))[0])
            except struct.error:
                break
            data = rng.read(datalen)
    out.write(header + data)
