#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write Magic Number
    out.write(b'GIF89a')

    width = 120 + rng.read(1)[0] % 400
    height = 120 + rng.read(1)[0] % 400
    gct_size = (rng.read(1)[0]) & 7
    lsda = int.from_bytes(rng.read(3), 'big', signed=True)
    lsdt = bytes([0xF7, 0x21, 0xF9, 0x04, 0x05, 0x00])
    lsdt += int.to_bytes(width, length=2, byteorder='big', signed=False)
    lsdt += int.to_bytes(height, length=2, byteorder='big', signed=False)
    lsdt += b'\x00\x00\xFF' if gct_size else b'\x80'
    lsdt += int.to_bytes(lsda, length=3, byteorder='big', signed=True)

    out.write(b'\x2C\x00\x00\x00\x00\x01\x00')
    out.write(int.to_bytes(width, length=2, byteorder='big', signed=False))
    out.write(int.to_bytes(height, length=2, byteorder='big', signed=False))
    out.write(b'\x00\x00\xFF')

    while True:
        ctrlm = rng.read(1)[0]

        if not ctrlm:
            break

        repcount = ctrlm - 1
        colormap = [None]*repcount
        for i in range(len(colormap)):
            colormap[i] = rng.read(3)

        out.write(ctrlm.to_bytes(length=1, byteorder='big', signed=False))

        # Randomly select color map index
        idx = ctrlm
        for _ in range((width * height * repcount) // 64):
            idx = rng.read(1)[0] >> 1
            out.write(struct.pack('BBB', colormap[idx][0], colormap[idx][1], colormap[