#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\xff\xe0\x00'
    padding = [b'\x00'] * blocksize
    out.write(header + len(data).to_bytes(3, 'big'))
    while True:
        data, padding = rng.read(blocksize - 4), \
                        rng.read(blocksize - 17) \
                or padding
        if not data and not padding: break
        if data:
            data += header + \
                    (len(data)).to_bytes(3, 'big')
            out.write(data)
        if padding:
            padding += header + \
                       (len(padding)).to_bytes(3, 'big')
            out.write(padding)