#!/usr/bin/env python3

import struct
from typing import BinaryIO

def convert(inp: str, out: BinaryIO):

    with open(inp, 'rb') as f:
        convertfile(f, out)

def convertfile(rng: BinaryIO, out: BinaryIO):

    header = b'GIF89a'

    # width and height dimensions
    image_width = int.from_bytes(rng.read(2), 'big')
    image_height = int.from_bytes(rng.read(2), 'big')

    # size of the color table
    ctsize = int.from_bytes(rng.read(1), 'little')

    if ctsize > 0:

        # read in palette values (RGB triples)
        paldata = [struct.pack('BBB', *x) for x in
            [(y % 256, y // 256, y // 256 * 256)
                for y in range(ctsize)]]

        palents = bytes()
        for i in range(len(paldata)):

            # write out the length byte first
            palents += struct.pack('B', len(paldata[i]))
            palents += paldata[i]

        # write to output stream
        out.write(header + struct.pack('<HHBB', image_width, 
            image_height, ctsize | 0b10000000, 
            0b1 << 7 | 0b1 << 5 | 0b1 << 4 | 
            0b111 << 3 | 0b111) + palents)
    
    # get remaining lzw compressed info from the RNG
    datalen = int.from_bytes(rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)
