#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4))

    # skip fields that aren't particularly interesting
    # (the logical screen descriptor - color table size, background
    # color index, pixel aspect ratio, global color table flag,
    # color resolution, sort flag, size of global color table.)
    rng.seek(7)

    # size of the color table
    ctsize = int.from_bytes(rng.read(1), 'little')

    if ctsize > 0:

        # read in palette values (RGB triples)
        paldata = [struct.pack('BBB', *x) for x in
            [(y % 256, y // 256, y // 256 * 256)
             for y in struct.unpack(f'<{ctsize}H', rng.read(2*ctsize))]]

        # write out color table
        out.write(b'\xff\x00' + b''.join(paldata))

    else:

        # no palette provided, create a dummy one
        out.write(b'\xff\x00\x00\x00\x00\x00\x00')

    datalen = int.from_bytes(rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)
