#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # GIF header starts with "GIF89a"
    out.write("GIF89a".encode())

    # LSD width and height are 1 byte each
    width = rng.getrandbits(8) + 1
    height = rng.getrandbits(8) + 1
    out.write(struct.pack("<BB", width, height))

    # The fields after it we don't care about...
    s = b"\xff\0\0\0\0\0\0\0\x00\0\0\0\0\0\0\0\x00\x00\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"

    # Write out the palette
    for _ in range(256):
        out.write("\0\0\0".encode())

    # Image data is stored as RLE
    # We can cheat here and just write every single pixel as non-RLE
    while True:
        for y in range(height):
            for x in range(width):
                if rng.getrandbits(1) == 0:
                    # Background color at this index is used
                    out.write(b"\x00")
                else:
                    # Foreground color at this index is used
                    out.write(b"\x01")
        # End of image marker
        out.write(b";")