
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = rng.read(1)[0]
    height = rng.read(1)[0]
    global_color_table_flag = 0
    color_resolution = 7
    sort_flag = 0
    global_color_table_size = 2 ** (color_resolution + 1) - 1
    background_color_index = global_color_table_size // 2
    pixel_aspect_ratio = 0
    out.write(header)
    out.write(bytes([width]))
    out.write(bytes([height]))
    out.write(bytes(
        [global_color_table_flag << 7 | color_resolution << 4 |
         sort_flag << 3 | global_color_table_size]
    ))
    out.write(b'\x00')
    out.write(bytes([background_color_index]))
    out.write(bytes([pixel_aspect_ratio]))
    # Application extension
    application_extension_label = b'!\xe2\xff\x0bNETSCAPE2.0'
    application_extension_block_length = 3
    loop_count = 1
    block_terminator = b';'
    out.write(application_extension_label)
    out.write(bytes([application_extension_block_length]))
    out.write(b'\x01')
    out.write(bytes([loop_count & 0xFF]))
    out.write(bytes([(loop_count >> 8) & 0xFF]))
    out.write(block_terminator)
    # Image descriptor
    image_descriptor_label = b','
    image_left_position = 0
    image_top_position = 0
    image_width = width
    image_height = height
    local_color_table_flag = 1
    interlace_flag = 0
    sort_flag = 0
    local_color_table_size = 2 ** (color_resolution + 1) - 1
    lct = []
    for _ in range(local_color_table_size):
        red = rng.read(1)[0]
        green = rng.read(1)[0]
        blue = rng.read(1)[0]
        lct.append((red, green, blue))
    out.write(image_descriptor_label)
    out.write(b'\x00')
    out.write(b'\x00')
    out.