#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'little') & 0xffff

    # Image Header
    ihdr = [0] * 10
    ihdr[0] = len('Image Descriptor') + 5
    ihdr[1] = ord('I') # Image Descriptor
    ihdr[2], ihdr[3] = (datalen >> 0) & 0xff, (datalen >> 8) & 0xff
    data_len = datalen - sum([ihdr[i] for i in range(4, 10)]) - 4

    print("Data length:", data_len) # Debug output
    print("Image height:", ihdr[7]) # Debug output
    print("Image width:", ihdr[5]) # Debug output
    if ihdr[6]:
        raise Exception("Non-zero image size: %d" % ihdr[6])

    # Color Table
    ct = [0] * 768
    ct[0] = 3*2**ihdr[8] # Number of colors used
    for i in range(0, 1 << ihdr[8], 1):
        icolor = int.from_bytes(rng.read(3), 'big')
        ct[i+1] = ((icolor >> 16) & 0xff) << 2   \
                 | ((icolor >>  8) & 0xff) << 1   \
                 | ((icolor >>  0) & 0xff)      # Indexed color

    # Image Data
    fdat = bytearray()
    for i in range(0, data_len, 1):
        fdat.append(int.from_bytes(rng.read(1), 'little'))

    # Output
    out.write(header) # Magic bytes
    out.write(struct.pack("<HHHB", ihdr[5], ih