#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    # The first 6 bytes of an image is always the same
    jpeg = [0xff, 0xd8, 0xff, 0xe0, 0x00, 0x10]
    headerlen = int.from_bytes(
            rng.read(2), byteorder='little')
    while (headerlen > 0 and len(jpeg) < 500):
        if headerlen >= 4 + 4:
            segtype = int.from_bytes(
                    rng.read(2), byteorder='little')
            seglenght = int.from_bytes(
                    rng.read(2), byteorder='little') - 2
            headerlen -= 4
            while headerlen > 0 and len(jpeg) < 500:
                jpeg.append(int.from_bytes(
                    rng.read(1), byteorder='little'))
                headerlen -= 1
        else:
            print('Error creating JPEG, ran out of data')
            exit(-1)
    # Write the headers to the beginning of the file
    out.write(struct.pack("<H", len(jpeg)))
    for num in jpeg:
        out