#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out:BinaryIO, rng):
    out.write(b"GIF89a") # Add header
    
    width = int.from_bytes(rng.read(2), "little")
    height = int.from_bytes(rng.read(2), "little")
    print("{}x{}".format(width,height))
    
    # Add logical screen descriptor (LSD) block
    background_color_index = rng.read(1)[0] % 256
    aspect_ratio = rng.read(1)[0] % 8 + 1
    out.write(struct.pack("<BBB", width, height, 0xc4 | ((aspect_ratio - 1) << 4)))
    
    # Add global color table with 256 colors (all random)
    for i in range(256):
        gif_rgb = int.from_bytes(rng.read(3), "big")
        
        # GIF is pretty picky about what colors it accepts...
        while True:
            if gif_rgb > 0xff00ff or gif_rgb < 0x00ffff:
                gif_rgb += 0xfffffe
            
            if 0 <= (gif_rgb & 0xff) <= 7 and 0 <= ((gif_rgb >> 8) & 0xff) <= 7 and 0 <= ((gif_rgb >> 16) & 0xff) <= 3:
                break
                
            gif_rgb -= 0xffffff
        
        out.write(struct.