
# this script is for rendering a simple picture of the number 8 into a TIM

import os
import sys
import struct
from array import array as Array
from random import Random
from PIL import Image

def main():
    if len(sys.argv) != 6:
        print("Usage: %s timfile palfile outfile seed width height" % \
                (os.path.basename(__file__)))
        sys.exit(-1)
        
    # create and open the file streams
    inp = open(sys.argv[1], 'rb')
    palin = open(sys.argv[2], 'rb')
    outhandle = sys.argv[3]
    rngseed = int(sys.argv[4])
    image_width = int(sys.argv[5])
    image_height = int(sys.argv[6])
    
    # create a new rng with the given seed
    rng = Random(rngseed)
    
    # generate the image information
    genimage(rng, inp, palin, outhandle, image_width, image_height)
    
if __name__ == "__main__":
    main()

def genimage(rng, inputstream, palfstream, outfilename, 
        image_width=128, image_height=92):
    
    # open the file for writing
    out = open(outfilename, 'wb')
    
    # read first 5 characters of the header
    header = inputstream.read(5)
    
    # read the palette from the palette file
    if palfstream is not None:
        paldata = []
        while True:
            # read next color code
            colcode = palfstream.read(3)
            if len(colcode) < 3: break
            
            # parse the color value from the code
            r = colcode[0] & 0xff
            g = colcode[1] & 0xff
            b = colcode[2] & 0xff
            
            # build the color tuple
            colors = [r, g, b]
            
            # read the rest of the color attributes
            attribslen = struct.unpack('B', palfstream.read(1))[0]
            attrbytes = palfstream.read(attribslen)
            
            # add all of the color attributes to the list
            colors += Array('H', attrbytes)
            
            # add the built up color data to the dictionary
            paldata.append(struct.pack('<BBBBHBBHBBBBBBHBBBB', *colors))
        
        # compute the total size of the color data table
        ctsize = sum([len(c) for c in paldata])
        
        # create the actual color table entry from the individual blocks
        palents = ""
        for i in range((ctsize + 255) // 256):
            palents += struct.pack('B', len(paldata[i]))
            palents += paldata[i]

        # write to output stream
        out.write(header + struct.pack('<HHBB', image_width, 
            image_height, ctsize | 0b10000000, 
            0b1 << 7 | 0b1 << 5 | 0b1 << 4 | 
            0b111 << 3 | 0b111) + palents)
    
    # get remaining lzw compressed info from the RNG
    datalen = int.from_bytes(rng.read(2), 'big')
    data = rng.read(datalen)
    out.write(header + data)
