#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    # Jim stuff from https://twitter.com/J_im/status/1206559444848223746?s=20
    blocksize = 65536
    for _ in range(int.from_bytes(
                rng.read(4), byteorder='big')):
        data, padding = rng.read(blocksize - 4), \
                        rng.read(blocksize - 17)
        if not (padding and data): break
        out.write(data + padding)
