#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False
    ) & 0xffffff
    data = rng.read(datalen)
    imgdata = b''
    while len(imgdata) < 6 + len(header) + len(data):
        newlen = 2 ** int.from_bytes(rng.read(1), byteorder='little')
        newdata = rng.read(newlen)
        if not newdata:
            raise ValueError('Bad GIF')
        imgdata += newdata
    width = int.from_bytes(imgdata[6+7*5], 'big')
    height = int.from_bytes(imgdata[6+7*4], 'big')
    out.write(b'GIF89a') # header
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\xF0\xFF\x00\x00\xFF\xFF\xFF\xFF\xFF\xFF,\x00\x00\x00')
    out.write(imgdata