#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    while True:
        marker = ord(rng.read(1))
        if not (0xFF <= marker <= 0xFE):
            continue
        size = int.from_bytes(rng.read(2), byteorder='little')
        if (size == 0 or marker in [0xD9, 0xDA]):
            break
        data = rng.read(size - 2)
        # Don't write anything that would mess up decoding
        if len(data) != size-2:
            continue
        out.write(marker.to_bytes(1, 'little') + size.to_bytes(2,