#!/usr/bin/env python3

import struct
from typing import BinaryIO


def decode_vlq(data: bytes, start=0):
    result = 0
    shift = 0

    while True:
        b = ord(data[start]) & 0x7f

        if (ord(data[start]) >> 7) == 1:
            result |= (b << shift)

            return (result, start + 1)

        result |= (b << shift)
        shift += 7
        start += 1


def encode_vlq(value: int):
    result = bytearray()
    buffer = value & 0x7f

    while True:
        value >>= 7

        if value!= 0 or len(result)!= 0:
            buffer |= 0x80

        result.append(buffer)

        if value == 0 and len(result)!= 0:
            break

    return bytes(result)


def get_string_length(s: str):
    return sum([2 for ch in s]) + 1


def write_uint32(out: BinaryIO, val: int):
    out.write(struct.pack('>i', val))


def write_uint16(out: BinaryIO, val: int):
    out.write(struct.pack('>H', val))


def read_uint32(in_: BinaryIO):
    return struct.unpack('>I', in_.read(4))[0]


def read_uint16(in_: BinaryIO):
    return struct.unpack('>H', in_.read(2))[0]


def get_file_size(filename: str) -> int:
    with open(filename, 'rb') as f:
        pos = f.tell()
        size = read_uint32(f)
        f.seek(pos)
        return size


def write_file_size(out: BinaryIO, filename: str):
    header = struct.pack(">I", get_file_size(filename))

    out.write(header + data)
