#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate(
    out: BinaryIO, width: int = 1024, height: int = 768, duration_ms: float = 500
) -> None:

    # Header
    out.write(b"GIF89a")

    # Logical Screen Descriptor
    out.write(struct.pack("<HHHH", width, height, 0xF00, 0x0))

    # Global Color Table
    for i in range(256):
        out.write(bytes([i * (256 // 256), i * (256 % 256), 0]))

    # Image Descriptor
    out.write(
        struct.pack(
            "<HHHHB",
            0,
            0,  # X and Y position of image on screen
            width,
            height,
            0,  # Local color table is present but not used
        )
    )

    # Graphic Control Extension
    out.write(
        bytes(
            [
                0x21,
                0xF9,
                0x04,
                0x01,
                max((duration_ms // 10), 1),  # Duration
                0xFF,
                0x0B,
                0x4E,
                0x45,
                0x54,
                0x53,
                0x43,
                0x41,
                0x50,
                0x45,
                0x32,
                0x2E,
                0x30,
                0x03,
                0x01,
                0x00,
                0x00,
                0x00,
            ]
        )
    )

    # Image data
    palette_index = 0
    image_data_len = (width - 1) // len(palette_index) + 1
    raw_image_data = bytearray()
    for _ in range(height):
        raw_image_data += bytes([(raw_image_data[palette_index] + 1) % 256])
        if ((palette_index & 0xFF) == (image_data_len - 1)):
            palette_index = 1
        else:
            palette_index <<= 1
    out.write(raw_image_data[: width * height])

