#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    header = b'GIF89a'

    # width and height are 16 bits each
    image_width, image_height = \
        struct.unpack('<HH', rng.read(4)) 

    field_size = 0x2c + (image_width*image_height)*3
    out.write(header)
    out.write(struct.pack("<BB",
        0xf7 & field_size >> 8,
        0xff & field_size))
    
    for _ in range(0, 0x9):
        out.write(b'\x00')

    out.write(struct.pack(">HH", 
        image_width,
        image_height))
    flag = 0xe0 | ((0x02&0x01)<<1) | ((0x02&0x02)>>1) 
    out.write(bytes([flag]))

    for _ in range(0, 0x5):
        out.write(b'\x00')

    
    out.write(struct.pack(">H", 
        0x100 | (0x00 & (0x04|0x02))))

    palette = [
        b'\x00\x00\x00',
        b'\xff\x00\x00',
        b'\x00\xff\x00',
        b'\x00\x00\xff']

    out.write(bytes(palette[i%4] for i in range(0, 2**16)))

    for _ in range(0, 0x1f):
        out.write(b'\x00')
    
    global_color_table_flag = False
    local_color_table_flag = True
    color_resolution = 0xc0
    sort_flag = 0
    size_of_global_color_table = 0x1e
    background_color_index = 0x00
    pixel_aspect_ratio = 0x00
    out.write(bytes([
        global_color_table_flag<<7 |
        local_color_table_flag<<6 |
        color_resolution << 3 |
        sort_flag,
        size_of_global_color_table,
        background_color_index,
        pixel_aspect_ratio]))


    lzw_minimum_code_size = 0x12   # chosen at random ;)
    
    out.write(bytes([lzw_minimum_code_size]))

    # encode the image data as lzw
    def write_block(data):

        if len(data) > 255:
            out.write(bytes([
                len(data)&0xff,
                255
            ]))
            out.write(data[:255])

            while len(data)!= 0:

                out.write(bytes([
                    255, 
                    min((len(data)-1), 255)]))
                
                out.write(
                    bytes(
                        data[(min(len(data), 255)):min((
                            len(data)+255), 
                            len(data))]
                    ))

                data=data[min((
                    255+len(data)), 
                    len(data)):]
                
        else:
            out.write(bytes([
                len(data), 
                0
            ]))
            out.write(data)
            

    def compress_lzw():
        
        code_length = 0xb
        dictionary = []

        
        for i in range(lzw_minimum_code_size + 1):
            
            dictionary.append(chr(i).encode('ascii'))


        current_byte = None

        compressed_data = bytearray()

        while current_byte is not None or end_of_file == False:


            if end_of_file == True:
                print("reading from source")
                current_byte = source.read(1)
            else:
                current_byte = input_buff[-1:]

            if current_byte == b'' or current_byte == b'\n': 
                break

            if ord(current_byte) < code_length:
                compressed_data += current_byte
                del input_buff[-1:]
            elif chr(ord(input_buff[-1])+1).encode('ascii') in dict:
                compressed_data += (input_buff[-1:] + 
                                    chr(ord(input_buff[-1])
                                        +1).encode('ascii'))

                del input_buff[-1:]
                
            else:
                compressed_data += (input_buff[-1:] + 
                                chr(ord(input_buff[-1])
                                    +1).encode('ascii'))

                del input_buff[-1:]


    
        return compressed_data

    end_of_file = False

    input_buff = bytearray()
    source = rng

    while True:

        read_buff = source.read(1)

        if read_buff == b'': 
            print("\nEnd of stream reached!")
            exit(-1)
            
        input_buff.extend(

            read_buff
        )


        print(f"Reading {len(read_buff)} bytes.")

        if len(input_buff)==2:
            print(f"{int.from_bytes(input_buff,'little')} <{input_buff}>")



        if len(input_buff) >= 2**(code_length+1):
            print("compressing...")
            output_buff = compress_lzw()

            for o in output_buff:
                try:

                    write_block(o)
                except Exception as e:
                    raise type(e)(str(e)+" "+str(type(output_buff)))+"\n"+str(len(output_buff))+str(type(output_buff))
            print(f"compressed to {len(output_buff)} bytes")
            del input_buff[:]






