#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'little') & 0xffff

    # Image Header
    ihdr = [0] * 10
    ihdr[0] = 0x2c                   # image separator
    ihdr[1] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[2] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[3] = (int.from_bytes(rng.read(3), 'little')) & 0xff
    ihdr[4] = ord(';')               # color table flag
    ihdr[5], _, _ = struct.unpack('<BBH', rng.read(3))    # color resolution
    ihdr[6] = ord(',')               # global palette size
    ihdr[7], ihdr[8] = struct.unpack('<hB', rng.read(3))  # background color index and pixel aspect ratio
    ihdr[9] = int.from_bytes(rng.read(1), 'little')         # LZW minimum code size

    # Write GIF file
    header += bytes(ihdr) + bytes([datalen]) + \
              rng.read((datalen - len(header)))

    # Write final result to output
    out.write(header)

