#!/usr/bin/env python3
from typing import BinaryIO


# Generates a test image with size `datalen`, writes it to
# `out`.
def generate_test_image(
        datalen: int, out: BinaryIO
    ):
    header = b'\xff\xd8\xff\xdb\x00C\x00\x01\x01\x01\x01\x01\x01\x01\x01'
    datalen += len(header)

    pattern = bytes([i % 256 for i in range(datalen)])
    data = bytearray()
    data.extend(pattern)
    padlen = ((datalen // 16) + 1) * 16
    data.extend(bytes([~i & 0xff for i in range(padlen)])) # inverted padding
    datalen += padlen

    assert datalen <= 0x10ffff
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\xff\xe0\x00'
    padding = [b'\x00'] * blocksize
    out.write(header + len(data).to_bytes(3, 'big'))
    while True:
        data, padding = rng.read(blocksize - 4), \
                        rng.read(blocksize - 17)
        if not (padding and data): break
        out.write(data + padding)
