#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

    # Write an image descriptor
    width = 1 << (int.from_bytes(
            rng.read(1), byteorder='big') % 9)
    height = 1 << (int.from_bytes(
            rng.read(1), byteorder='big') % 9)

    flags = [False] * 8
    flags[7] = bool(int.from_bytes(
                rng.read(1), byteorder='big'))
    
    for i in range(4 - ((width & 7) >> 1)):
        flags[i] = False
    print('Flags', flags)

    print('Width:', width)
    out.write((width // 2**height).to_bytes(
            1, byteorder='little'))
    out.write(height.to_bytes(1, byteorder='little'))

    packedflags = bytes([sum([flag * (1<<n)
                            for n, flag
                            in enumerate(reversed(flags))])])
    out.write(packedflags)
    
def write_blocks_into_bytearray(
        data: bytes, rng: BinaryIO, blocksize: int=256
    ) -> list:
    buf = []
    while byte := rng.read(blocksize):
        if not byte and not buf:
            break
        
        buf += [byte[:blocksize]]
        
    if len(buf)!= len(data):
        raise ValueError("len(data) does not match "
                         f"number of blocks generated ({len(buf)})")
            
    return buf

    
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\xff\xe0\x00'
    padding = [b'\x00'] * blocksize
    out.write(header + len(data).to_bytes(3, 'big'))
    while byte := rng.read(blocksize):
        if not byte and not padding:
            break
            
            out.write(byte[:blocksize])

