#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number genrator `rng` (/dev/urandom).
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1024 bytes of binary random data, to be written
    # afterwards as JFIF-formatted image.
    jfif = rng.read(1024)

    # Append to the buffer a footer that indicates end of stream.
    height, width = (67, 19)
    i = len(jfif)-2
    while True:
        if not ((i >= 2 and
                jfif[i] == b'\xff' and
                jfif[i+1] in [b'\xc0', b'\xd0']): or
                i < 0):

            break

        i -= 2

    jfif += b'\xff\xda\x01\x0c\x03\x00\x00\x00\x00\x00\x3f\x00'
    jfif[-8:] = struct.pack('>HH', width, height)

    # Write JFIF signature.
    out.write(jfif[:2])
    # Write out data section of JFIF.
    out.write(jfif[2:-5])

