#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

    # 20 frames in this file!
    for i in range(19):
        newframe = bytes([
            0x2C,   # Start of Image block
            ord('X'), ord('Y'),    # Xpos Ypos
            0xFF, 0x0F, 0,     # LZW mincodesize
            0b11001010,       # Packed field (use palette and interlace mode)
            0, 5,              # Width height
            0b00000000        # No local color table
        ])

        # Write the frame to the output stream with a random delay
        duration = int.from_bytes(
                rng.read(2), byteorder='big')
        newframe += duration.to_bytes(2, 'big')

        # Add LZW compressed version of "I'm a gif" text
        # as per https://www.w3.org/Graphics/GIF/spec-gif89a.txt
        lzw_compressed = bytes()
        lzw_compressed += [47] * 6         # 47 47 47 47 47 47
        lzw_compressed += [49, 33]          # 49 21 33
        lzw_compressed += [46]               # 46
        lzw_compressed += [73, 82, 70]      # 49 4D 47 46
        lzw_compressed += [53, 51, 50, 53]  # 38 39 3A 37
        lzw_compressed += [32, 35, 33, 45]  # 20 23 21 2D
        lzw_compressed += [47, 10, 33, 10]  # 2F 0A 21 0A

        lzwlen = len(lzw_compressed).to_bytes(2, 'big')
        newframe += lzwlen[::-1]           # Reverse bits on little endian
        newframe += lzw_compressed

        # The last chunk is the image descriptor (same all the way down)
        newframe += bytes([
            0, 0,             # Left Top
            5, 0,             # Width Height
            0                 # No local color table
        ])

        out.write(newframe)

