#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    width = height = int.from_bytes(
            rng.read(2), byteorder='big', signed=False)
    
    global colorspace
    gif_colorspace = [b'\x2c', b'\x00']
    if bool(int.from_bytes([rng.read(1)], byteorder='big')):
        gif_colorspace += [
                b'\x76', b'\x00',
                b'\x01', b'\x00'
                ]

        palette = []
        for i in range(768):
            r = rng.read(1)[0] & ~0xf0 | 0xe0
            g = rng.read(1)[0] & ~0xf0 | 0xe0
            b = rng.read(1)[0] & ~0xf0 | 0xe0

            palette.append(r)
            palette.append(g)
            palette.append(b)

    elif bool(int.from_bytes([rng.read(1)], byteorder='big')):
        gif_colorspace += [
                b'\x9f', b'\x00',
                b'\x00', b'\x00'
                ]
    else:
        gif_colorspace += [
                b'\xa2', b'\x00',
                b'\x00', b'\x00'
                ]
        palsize = int.from_bytes(
                rng.read(1), byteorder='big', signed=False) 
        colorspace = rng.read(palsize * 3)

    print(header+datalen.to_bytes(length=3, byteorder='big'))
    print(width.to_bytes(length=2, byteorder='big'), end='')
    print((height>>8).to_bytes(length=1, byteorder='big'), end='')
    print((height&0xFF).to_bytes(length=1, byteorder='big'))
    print(*gif_colorspace)

    def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1024 bytes of binary random data, to be written
    # afterwards as JFIF-formatted image.
    jfif = rng.read(1024)

    # First two bytes are SOI marker (start of image).
    # Last five bytes are EOI marker (end of image).
    jfif = b'\xff\xd8' + jfif + \
           b'\xff\xd9' 

    # Write out data section of JFIF.
    out.write(jfif[2:-5])

