#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    while True:
        block = rng.read(blocksize)

        if len(block) == 0 or len(block)!= blocksize:
            break

        # Every Nth time we write that our buffer is empty but
        # this isn't true, we have to pad a zero and add restart markers
        for i in range((len(block) // (1 << 14))+1):

            byte = block[i*(1<<14):min(((i+1)*(1<<14)), len(block)-int(i*blocksize))]

            if len(byte) < blocksize:
                bytes_missing = blocksize - len(byte)

                padding = [b'\x00'] * bytes_missing
                byte += bytes(padding)
                out.write(byte[:blocksize])

