#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu May 20 17:56:49 2019

@author: marcoaqil
"""


import numpy as np
from matplotlib import pyplot as plt
import warnings
warnings.filterwarnings("ignore")




def akima_spline(X, Y):
    """Returns a Akima spline that goes through the given data points."""

    # Make sure X is sorted!
    order = np.argsort(X)
    Xs = np.array([X[i] for i in order])
    Ys = np.array([Y[i] for i in order])

    # Copy and sort by X
    N = len(Xs)
    t = (np.arange(N - 1) + 1).astype('float')
    h = Xs[1:] - Xs[:-1]
    A = ((Ys[1:] - Ys[:-1]) / h) * (h ** 2)
    B = (((Ys[2:] - Ys[1:-1]) / (h[:-1]))
          + ((Ys[1:-1] - Ys[:-2]) / (h[1:]))) * h
    C = ((Ys[2:] - Ys[0:-2]) / (h[1:])
         + 2 * ((Ys[1:-1] - Ys[:-2]) / (h[:-1]))) * h ** 2
    D = (((Ys[2:] - Ys[0:-2]) / (h[1:] ** 2))
         + ((Ys[1:-1] - Ys[:-2]) / h[:-1])
         + ((Ys[2:] - Ys[1:-1]) / h[1:])) * h ** 3
    S = (-A / 2
            + (B / 2) * t
            - (C / 2) * (t ** 2)) \
        + ((D / 6) * (t ** 3)
           - (C / 2) * (t ** 2)
           + (B / 2) * t)
    
    return S



# Read the file
data= np.loadtxt("test.dat", usecols=(0,1), skiprows=1)

V = akima_spline(data[:,0], data[:,1])

plt.figure()
plt.plot(data[:,0], V, 'r', label='Spline')
plt.scatter(data[:,0], data[:,1], s=7, c='k', marker='+', label="Data Points")
plt.xlabel('Voltage [mV]')
plt.ylabel('Current [pA]')
plt.legend()
