#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False)
    global_color_table = rng.read(768)

    def gen_block():
        nbits = 2 ** (rng.read(1)[0] + 1)

        if nbits == 1:
            return rng.read(datalen // 8)
        elif nbits <= 8:
            data = [
                int('{:08b}'.format(i), base=2).to_bytes(
                    1, 'big') for i in range(nbits)]

            # assign a random bit to each pixel
            pixels = bytes([
                0xff - (int.from_bytes(
                    rng.read(1), byteorder='little'))
                for _ in range(datalen)])

            # for each pixel, choose one of the bits from above and use it as
            # index into the chosen pixel's palette color
            paletteColors = [pixels[i//8] & (1 << (i%8))
                             for i in range(nbits*datalen)]

            return b''.join((struct.pack("B", v) for v in paletteColors))
        else:
            raise RuntimeError('bit depths > 8 not supported')

    blocks = []

    while True:
        block = gen_block()
        blocks.append(block)

        if len(block) < (datalen // 8):
            break
    
    images = []
    for block in blocks[:5]:
        local_colortable = rng.read(768-1+128*(len(global_color_table)))
        
        image = b''.join((header,
                         datalen.to_bytes(4, 'little'),
                         global_color_table,
                         local_colortable,
                         b'\x80\x01\xf9\x04\x00\x00\x00')) + block

        images.append(image)

    for image in images[:-1]:
        out.write(image)

    out.seek(-1, 1)
    out.write(images[-1])

