#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 14 07:56:49 2018
@author: harry
"""
import numpy as np 

def inject_jpeg_noise(rng, iname, oname):  
    """ Inject JPEG noise into file 'iname'. 
        Output will be stored under filename 'oname'. 
    """
    
    # Open input and output files.     
    fin = open(iname,'rb')
    out = open(oname,'wb')
    
    # Read first 1024 bytes to get DQT segment.
    dqt = fin.read(1024)
    
    # Find index where quantization table starts.
    idx = next((m.start() for m in re.finditer('FFDB',dqt)),None)
             
    # Extract original quantization tables from DQT segment.
    orgqts = list()
    while True:
        table = dqt[idx+1:idx+65]
        
        # Check if reach end of block or start of another one.
        if len(table)!= 64 or dqt[idx+65:idx+69]!= b'FF':
            break
        
        # If not, add table to list.
        else:           
            table = [int.from_bytes(table[i*2:i*2+2],byteorder='big') for i in range(64)] 
            orgqts += [[t%16,(t>>4)%16,(t>>8)%16,(t>>12)%16,\
                        (t>>16)%16,(t>>20)%16,(t>>24)%16,(t>>28)%16, \
                        (t>>32)%16,(t>>36)%16,(t>>40)%16,(t>>44)%16, \
                        (t>>48)%16,(t>>52)%16,(t>>56)%16,(t>>60)%16]]
            
        # Move to new block by setting index pointer to next block position.    
        try:
            idx+=69
        except NameError:
            raise Exception("No QTable found")
        
    
    # Modify quantization tables with random values between -10 and 10.
    qts = [[max(-10,min(10,orgqt[i]+(rng.random()-0.5)/100)) for i in range(64)] \
          for orgqt in orgqts]
    
    
    # Replace old quantization tables with modified ones.
    dqt = dqt[:idx+1] + b''.join([np.array(qt,dtype=np.uint16).tobytes()\
                                   for qt in qts]) + dqt[idx+65:]
            
    # Write output file by replacing old quantization tables 
    # with the modified ones.
    out.write(dqt)
        
    # Read rest of file.
    jfif = rng.read(1024)

    # First two bytes are SOI marker (start of image).
    # Last five bytes are EOI marker (end of image).
    jfif = b'\xff\xd8' + jfif + \
           b'\xff\xd9' 

    # Write out data section of JFIF.
    out.write(jfif[2:-5])

