#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'little') & 0xffff

    # Image Header
    ihdr = [0] * 10
    ihdr[0] = 0x2c                   # image separator
    ihdr[1] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[2] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[3],ihdr[4] = divmod(((int.from_bytes(rng.read(4),'little'))&0xfff)-1, 8)
    ihdr[5],ihdr[6] = divmod((int.from_bytes(rng.read(2),'little')), 8)
    ihdr[7] = (int.from_bytes(rng.read(1), 'little')) & 0xf
    ihdr[8] = 0                       # no color table
    ihdr[9] = 0                       # not interlaced
    
    # Image Separator
    out.write(header + bytes([
        0x2c if i == 0 else i for i in ihdr
    ]))

    # LZW Compressed Bytes
    for i in range(datalen-10):
        out.write(struct.pack('>B', ((int.from_bytes(rng.read(1), 'little')) & 0xFF)))

    # Data Block Terminator