#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\xff\xe0\x00'
    padding = 1
    
    datalen = int.from_bytes(
            rng.read(3), byteorder='big', signed=True)

    # Make sure this is at least one full frame of image data. If it's just
    # some random amount of bytes, pad with zeroes to make a complete frame.
    data = rng.read((datalen // blocksize or 1) * blocksize)
    if len(data)!= (datalen // blocksize or 1) * blocksize:
        padding += 1

        out.write(header + len(data).to_bytes(3, 'big'))
    while byte := rng.read(blocksize):
        if not byte and not padding:
            break
            
            out.write(byte[:blocksize])

