#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\xff\xe0\x00'
    padding = [b'\x00'] * blocksize
    out.write(header + len(data).to_bytes(3, 'big'))

    for i in range((len(data)+blocksize-1)//blocksize):
        realpadding = min(blocksize,
                          (i+1)*blocksize - len(data))

        #print('len(data)', len(data))
        #print('len(padding)', len(padding))
        #print('realpadding', realpadding)

        if i >= 1 and realpadding == 0:
            break
        else:
            out.write(data + bytes(padding[:realpadding]))


# The size of the temporary files to be written by this script
TEMPORARY_BLOCKSIZE = 4096*2
# The filename of the output file
OUTPUT_FILEPATH = 'output'

if