#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Make header
    header = b'GIF89a' if len(b'GIF89a') == 6 else b'GIF87a';

    # Write size (1024x512 with color depth of 256) to the file
    data = bytearray((
        0x0F, 0xFF, 0x0F, 0xFF, 0x00, 0x00, # Width x Height
        0xFE,                         # Color Table Size
        0x00, 0x00,                   # Block Terminator
    ))

    for i in range(0, 256 * 3, 3):
        r = ord(rng.read(1)) % 256;
        g = ord(rng.read(1)) % 256;
        b = ord(rng.read(1)) % 256;

        data += bytes([r, g, b]);

    data += b'\x00';

    datalen = len(data);

    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)
