
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        bytearray(rng.read(3)), 'little', signed=False
    )

    # Write Header (6 bytes) and Logical Screen Descriptor (7 bytes)
    out.write(header +
            bytearray([0x21]) +  # Extension Introducer
            bytearray([0xFF]) +  # Application Extension Label
            bytearray([0x0B]))  # Block size: 11 bytes

    for i in range(4):
        out.write(bytearray([random.randrange(256)]))
    out.write(b'\x00\x3B')
    
    # Write Image Descriptor (10 bytes)
    out.write(b'\x2C' +
            bytearray([random.randrange(256)]) * 3 +   # Left offset
            bytearray([random.randrange(256)]) * 3 +   # Top offset
            bytearray([datalen & 0xFF,
                ((datalen >> 8) & 0xFF),
                ((datalen >> 16) & 0xFF)]))            # Width and height 
            
    # Write Local Color Table (3*256 entries, RGB values). We can use the same one throughout
    data = bytearray()
    for _ in range(768):
        data += bytearray([random.randrange(256)]) 
        
    # Write Graphiccontrol Extension (8 bytes)
    out.write(b'\xF9' + bytearray([0x04] + [0x05] + 
            bytearray([(datalen+10)&0xFF, 
                ((datalen+10)>>8) & 0xFF, 
                ((datalen+10)>>16) & 0xFF, 
                0x00])))

    # Write Image Data (n bytes)
    out.write(data)

    out.flush()