#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    while 1:
        byte = rng.read(blocksize)
        if not byte or len(byte)!= blocksize:
            return
        
        # Write each block as-is until we reach SOF0 (start of frame marker)
        for i in range(len(byte)):
            if byte[i] == 0xFF and byte[i+1] == ord('S'):
                if byte[i+2] == ord('O'):
                    if byte[i+3] == ord('F') \
                            and byte[i+4] >= ord('0') \
                            and byte[i+4] <= ord('7'):
                        out.write(byte[ : i+2 ])
                        break
            
            out.write(byte[:blocksize])

