
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        bytearray(rng.read(3)), 'little', signed=False
    ) & 0xffffff
    data = rng.read(datalen)

    # image width (4 bytes) + image height (4 bytes)
    imginfo = bytearray([
        1, 1,  0x7f,   0x7f,    0,     0,     0,     0,
        0xff,  0xff, 0x00,   0x00,    0,     0,     0,     0
    ])

    # color table size (1 byte), pixel aspect ratio (1 byte)
    gifinfo = bytearray([
        0x2c,  0x00,          0x00,        # LZW min code size
        0x21,  0xf9,  0x04,   0x00,        # graphics control extension
        0x6e,  0x00,          0xc8,        # delay in milliseconds
        0xff ,               0x0b,        # application extension
        0x00,  0xe4,  0x00,   0x00,        # sub-block length = 100
        0x52,  0x49,  0x46,   0x46,        # "RIFF" string
        0x00,  0x00,   0x00,   0x0d,        # data block length = 13
        0x49,  0x48,  0x44,   0x52,        # IHDR identifier
        0x00,  0x00,   0x00,   0x01,        # logical screen descriptor
        0x00,  0x00,   0x00,   0x01,        # image width
        0x00,  0x00,   0x00,   0x01,        # image height
        0x08,  0x06,  0x00,   0x00,        # bit depth = 8
        0x00,  0x00,   0x00,   0x00,        # no compression method
        0x00,  0x00,   0x00,   0x00,        # no filter methods
        0xff,  0xfa                   # termination of block
    ])

    # terminator of data section
    giftrailer = bytearray([
        0x3b                                # GIF trailer character
    ])
    numframes = len(data)//imginfo[7]//imginfo[8]
    animation = [header+gifinfo for _ in range(numframes)]
    animation += [(header+gifinfo+imginfo)+data+(giftrailer*2)]


    print(len(animation))
    out.write(b"".join(animation))


