#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False
    ) & 0xffffff
    data = rng.read(datalen)

    width = 64 + (data[7] % 256)
    height = 64 + ((data[7] >> 8) % 256)
    packed = struct.pack('BBBBB', *(width-1)//8+1, 7, 0x80, 0xff, 0xff)
    
    assert len(packed) == 5
    unpacked = struct.unpack('<HBBBB', packed) + (int.from_bytes(data[:7], 'big'),)
    packagain = struct.pack(*unpacked)
    data = data[7:]

    if len(data) < datalen:
        padlen = datalen - (width*height//8 + 4)

        padding = bytes([
            0b00000000 |
            (((padlen // 8)-1)*0b10000000) | 
            ((padlen%8)*0b00000111)
        ])
        print(f'Padding to {padlen} ({padding})')
        
        zeroes = b'\x00'*(width*height//8 + 4)
        padding += zeroes
        padlen -= len(zeroes)

        while padlen > 0:
            thingz = rng.read((padlen % 4) or 4)

            if thingz[-1]!= 0:
                thingz = thingz[:-1]
            
            padding += thingz
            padlen -= len(thingz)

        data += padding

    assert len(data) >= datalen

    # Write out the GIF file!
    out.write(header)
    out.write(struct.pack('<HH', width, height))
    out.write(struct.pack('B', unpacked[0]))
    out.write(struct.pack('B', unpacked[1]))
    out.write(packagain)
    out.write(data)

