#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ):
    header = b'\xff\xe0\x00'
    padding = [b'\x00'] * blocksize

    while True:

        # Insert a new frame after each 100 frames
        if rng.read(1)[0] == 100: break

        out.write(header)
        hlen   = rng.read(2)
        width  = (hlen[0] << 4) | ((hlen[1] & 0xf0) >> 4);
        height = (((hlen[1] & 0x0f) << 8) | hlen[2])
        out.write((width).to_bytes(2, 'little'));
        out.write(((height >> 8) & 0xff).to_bytes(2,'little'));
        out.write((height & 0xff).to_bytes(2,'little'));
        out.write([b'\x0c'])
        out.write(padding[:width*blocksize]);

