#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    while True:
        marker = rng.read(1)[0]

        if (marker < 0xC0): # SOFn block
            width = ((
                rng.read(1)[0] << 4 |
                rng.read(1)[0]) ^
                0x40).to_bytes(2, "little")

            height = ((
                rng.read(1)[0] << 4 |
                rng.read(1)[0]) ^
                0x40).to_bytes(2, "little")

            components = [rng.read(1)[0]] * \
                    rng.read(components[0])

            break

    out.write([
        0xFF & ~0xF0,         # JFIF identifier
        0x4A,                 # J
        (0x57 >> 6) & 0xFF,   # F
        (0x57 >> 4) & 0xFF,   # I
        (0x57 >> 2) & 0xFF,   # F
        (0x57 % 2) * 0xFF,    # J
        len(width),           # length of first field in bytes
        0x00,                 # horizontal size in DPI
        len(height),          # length of second field in bytes
        0x00                  # vertical size in DPI
    ] + width + list(reversed(height)) + components)

    while True:
        for i in range((int(rng.read(1)[0]) & 0x0f)+1):
            r = unhexlify('%02X' % rng.read(1)[0])
            g = unhexlify('%02X' % rng.read(1)[0])
            b = unhexlify('%02X' % rng.read(1)[0])
            out.write([r[1],g[1],b[1]])

        padding = unhexlify('%02X' % rng.read(1)[0])
        out.write(data + padding)
