#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    width  = rng.randint(10, 256)
    height = rng.randint(10, 256)

    background_color    = rng.choice([b'\x00\x00\xff',
                                     b'\x00\xff\x00',
                                     b'\xff\x00\x00'])
    
    global_color_table   = bytes()
    local_color_table    = bytes()
    pixel_data           = bytes()

    for y in range(height):

        if len(global_color_table) < 768:
            indices = [rng.randrange(4) * 256 + x
                       for x in range(len(local_color_table),
                                      min(len(local_color_table)+128, 768))]
            local_color_table += b''.join([(background_color
                                               if i == 0 else
                                               b'\x00' + (b'\xff'*i))
                                              for i in indices])

        line = bytes()

        for x in range(width):

            if global_color_table!= local_color_table and rng.randint(0, 2):
                color = global_color_table[rng.randrange(768)]
            else:
                color = local_color_table[rng.randrange(len(local_color_table))-1]

            line += color
    
        pixel_data += line

    image_descriptor        = struct.pack('<HHHBBBBB',
                                         width,
                                         height,
                                         0xF7 | ((rng.randrange(0, 7) << 5) & 0xFE00), # Local Color Table Flag; 8 bits per primary colour
                                             rng.randint(0, 256),       # Size of Global Color Table
                                             rng.randrange(0, 256),     # Background Colour Index
                                             rng.randint(0, 256),       # Pixel Aspect Ratio
                     0                   )                    # Padding
    application_extension   = b''                                    # Application Extension Block
    graphic_control_extension= struct.pack('<BBBHB',
                                           *(rng.randrange(0, 256)
                                            for _ in range(6)))             # Graphics Control Extension Header
    plain_text_extension    = b''                                    # Plain Text Extension Block
    comment_extension       = b''                                    # Comment Extension Block
    end_of_image            = b';'                                   # Trailer byte
    extensions              = (application_extension
                                  + graphic_control_extension
                                  + plain_text_extension
                                  + comment_extension)

    print('output GIF:')
    print(('  {header}\n'
          ' {image_descriptor}\n' 
          ' {global_color_table}\n'
          ' {local_color_table}\n'
           '{pixel_data}'
          ).format(**locals()))

    out.write((header
               + image_descriptor
               + global_color_table
               + local_color_table
               + pixel_data
               + extensions).replace(b'', b'\xFF'))
    