#!/usr/bin/env python3

def main():
    import sys
    
    if len(sys.argv)!= 4 or any([not a.isnumeric() for a in sys.argv[1:]]) \
      or not (1 <= int(sys.argv[2]) <= 50):
        print('Usage: stich <num_frames> <x_scale> <y_scale>', file=sys.stderr)
        return
        
    num_frames = int(sys.argv[1])
    x_scale = float(sys.argv[2])
    y_scale = float(sys.argv[3])

    rngs = [open(f'/dev/urandom{i}', 'rb') for i in range(num_frames)]
    with open('/dev/urandom', 'wb') as combined:
        while True:
            try:
                next_byte = rngs[0].read(1)[0]
                for r in rngs[1:]:
                    this_byte = r.read(1)[0]
                    next_byte ^= this_byte

                combined.write(next_byte.to_bytes(1, 'little'))
            except IOError: # EOF on one of the files means EOF on all
                break

    with open('stitched-frame', 'wb') as output:
        for frame in generate_frames(rngs):
            transform_frame(frame, x_scale / frame['width'],
                            y_scale / frame['height'],
                            combined, output)


def generate_frames(rngs):
    header = b'GIF89a'
    width = None
    height = None
    gif_width = None
    gif_height = None
    has_color_table = False
    for rng in rngs:
        rng.seek(6) # Seek past signature and screen descriptor
        yield {'size': int.from_bytes(rng.read(2), 'big'),
               'width': int.from_bytes(rng.read(2), 'big'),
               'height': int.from_bytes(rng.read(2), 'big')}

        if gif_width == None:
            gif_width = width = int.from_bytes(rng.read(2), 'big')

        if gif_height == None:
            gif_height = height = int.from_bytes(rng.read(2), 'big')
        
        assert width == gif_width
        assert height == gif_height


def transform_frame(
    frame: dict[str, int], x_ratio: int, y_ratio: int,
            rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)
