#!/usr/bin/env python3
import sys
import os
import struct

from typing import BinaryIO




# Generates a random GIF file into `out`, using the generator
# in `/dev/urandom`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number genrator `rng` (/dev/urandom).
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1024 bytes of binary random data, to be written
    # afterwards as JFIF-formatted image.
    jfif = rng.read(1024)

    # First two bytes are SOI marker (start of image).
    # Last five bytes are EOI marker (end of image).
    jfif = b'\xff\xd8' + jfif + \
           b'\xff\xd9' 

    # Write out data section of JFIF.
    out.write(jfif[2:-5])

