#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the provided parameters.
def make_gif(bg: int, fg: int, width:int=120, height:int=64) -> bytearray:
    out = bytearray()

    header =  b'GIF89a'
    screen = (width & 0xFF).to_bytes(1, 'little') + \
             (height& 0xFF).to_bytes(1, 'little') + \
              b'\x87\x00\x00' # field, bg color, aspect ratio
    loop   = [0] * 8
    fin    = [0] * 3
    table  = [0xFF, 0x0B, 0, 0, 0, 0, 0xFFFFFF, bg,
              0x00, 0x00, fg,
           ]

    palette = bytes([fg]*3+[bg]*3)*256

    size = len(table) + len(palette) + sum(map(len, table)) + sum(map(len, [header,screen]))
    size += 4*(sum(map(len, [loop]+list(fin)+[table])))
    
    out += size.to_bytes(2, 'big')
    out += header
    out += screen
    out += loop # no delay, forever
    out += fin  
    for item in table:
        if type(item) == list or type(item) == tuple:
            out += bytes(item) 
        else:
            out += str(item).encode('ascii') 
    out += palette 

    return out

