#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

# Generates a random JPEG file into `out` using the
# random number genrator `rng` (/dev/urandom).
def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO
    ):
    # 1024 bytes of binary random data, to be written
    # afterwards as JFIF-formatted image.
    jfif = rng.read(1024)

    # We can't just store the length and the data in two
    # variables because we're sampling from the wrong source;
    # in our case, it's the /dev/urandom device that produces
    # all the random data. If we write the length first and then
    # the actual data, Python waits for the entire amount until
    # EOF is reached before it writes anything. This means that
    # some time might pass between writing the length and the
    # data, which causes the read-in length to differ from the
    # one actually written. To avoid this issue, we need both
    # values to be written at once: we use a fixed-length format
    # (int_to_bytes) instead.
    length_prefix = int.from_bytes(jfif[:2], byteorder='big')
    out.write(b'\xFF\xD8\xFF\xE0' +
              length_prefix.to_bytes(2, byteorder='big', signed=False))
    out.write(jfif[2:])

