#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        width: int, height: int, bitdepth: int, numframes: int,
        background_r: int=0x00, background_g:int=0x00, background_b:int=0x00):

    header = b''
    # Header - 6 bytes
    # Signature ("GIF89a" or "GIF87a")
    signature = rng.read(6).decode('ascii').rstrip('\x00')
    if (signature!= 'GIF89a' and signature!= 'GIF87a'):
        raise Exception("Invalid GIF signature.")
    header += signature.encode('ascii')

    version = b'\x01\x00'
    header += bytearray(version)

    screendesc = b''
    # Screen Descriptor - 7 bytes
    # Width in pixels
    widthbytes = [width%256, width//256]
    screendesc += bytearray(widthbytes)


    # Height in pixels
    heightbytes = [height%256, height//256]
    screendesc += bytearray(heightbytes)

    flags = 0x80 # Global Color Table Flag
    screendesc += bytearray([flags])
    
    bgcolorindex = 0
    screendesc += bytearray([bgcolorindex])

    pixelaspectratio = 0
    screendesc += bytearray([pixelaspectratio])

    header += bytearray(screendesc)

    globalcolortable = b''
    for i in range(1<<bitdepth):
        r = rng.read(1)[0] & 0xff
        g = rng.read(1)[0] & 0xff
        b = rng.read(1)[0] & 0xff
        globalcolortable += bytearray([r,g,b])
    header += bytearray(globalcolortable)

    extensionblocksize = 4
    applicationextensionlabel = b'\x21\xFF\x0B'
    subblocksize = len(applicationextensionlabel+b'NETSCAPE'+b'\x03\x01\x00')
    applicationextensionblockdata = \
            applicationextensionlabel + \
                struct.pack('<H',subblocksize)+ \
                    b'NETSCAPE'+struct.pack('<BBB',3,1,numframes)+b'\x00'
    extlen = len(applicationextensionblockdata)
    extensionblock =   bytearray([extlen%256])+bytearray([(extlen>>8)%256])+ \
                        bytearray([extensionblocksize])+bytearray(applicationextensionblockdata)
    header += bytearray(extensionblock)

    label = b'\x2C'
    label_length = 10
    blocksize = width * height * ((bitdepth+7)//8)
    imgdescriptor = b''
    locationx = 0
    locationy = 0
    localcolortable = 0
    localimgdescriptor = bytearray([locationx, locationy,
                                 localcolortable, blocksize % 256,
                                    (blocksize >> 8) % 256 ])
    frame = label + bytearray((label_length-1)*[0]) + localimgdescriptor
    header += frame

    data = rng.read(datalen)
    out.write(header + data)
