#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = bytearray.fromhex("47 49 46")
    version = bytearray([5, 7])
    width  = struct.pack("<H", (2**16 * int.from_bytes(rng.read(1), "big")) % (8*1024))
    height = struct.pack("<H", (2**16 * int.from_bytes(rng.read(1), "big")) % (8*1024))
    flags  = b"\x87\x00"
    bgcolr = (rng.read(1)[0] << 2) | rng.read(1)[0]
    aspect = b"\x00"
    gctnum = 0
    colorspaceflag = False
    backgroundcolorindex = 0
    transparentcolorindex = 0
    nbits = 1
    if rng.read(1) == b'\xff':
        gctsize = (rng.read(1)[0] << 2) | rng.read(1)[0]
        while gctnum < min(gctsize+1, 1<<nbits):
            r = (rng.read(1)[0] << 2) | rng.read(1)[0]
            g = (rng.read(1)[0] << 2) | rng.read(1)[0]
            b = (rng.read(1)[0] << 2) | rng.read(1)[0]
            reserved = rng.read(1)[0]
            gctnum += 1
    lzwmincodesize = 2
    datalen = (width[0]*height[0])/2 - 1
    rawdataoffset = len(header)+len(version)+len(width)+len(height)+len(flags)
    globalcolortablesize = 2**(nbits+1)
    localcolortablesize = 2**(nbits+1)-1
    blocksize = (localcolortablesize*(bool(globalcolortablesize)*3+bool(localcolortablesize)))//8
    data = ((((rawdataoffset+(blocksize*((gctnum and globalcolortablesize)//blocksize))+4)%256)!= 0) or (((rawdataoffset+(blocksize*((gctnum and globalcolortablesize)//blocksize))+4)//256)!= 0))

    # Image Descriptor
    ihdr = [width[1], width[0]] \
         + [height[1], height[0]] \
                       + [flags[1], flags[0]] \
                                     + [bgcolr >> 4 & 0xF] \
                                               + [(bgcolr >> 2 & 0xC)|((bgcolr >> 4 & 0xF)<<2)] \
                                                         + [bgcolr & 0xFC|((bgcolr >> 2&0xF)<<4)] \
                                                                   + [aspect[1], aspect[0]]
           #                      ^--^--^--^--^--^--^--^--^--^--^--^--^--^--^--^
    ihsize = sum(map(lambda x : isinstance(x, int), ihdr))/len(ihdr)# bytes
    out.write(header + bytes(ihdr))

    # LZW Compressed Bytes
    for i in range(datalen-10):
        out.write(struct.pack('>B', ((int.from_bytes(rng.read(1), 'little')) & 0xFF)))

    # Data Block Terminator