#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Zero-length blocks are used to pad out the file
    padding = 0
    while True:
        if not data or (padding % 10 == 0 and padding > 10):
            padding += 1
            continue
        break
    
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

    for i in range(padding):
        out.write(b'\x00')
