#!/usr/bin/env python3

import random
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Generate the basic structure of a GIF file
    data, padding = [], []
    for _ in range(int.from_bytes(
            rng.read(3), byteorder='big')):
        data += list(rng.read(8))
    padding = bytes([rng.getrandbits(8)])*len(data)

    # Add an introductory chunk, then add our desired payload
    out.write(b'GIF89a')
    out.write(struct.pack('>H', sum([len(d) for d in data]) +
                           len(padding)))
    for item in data:
        assert type(item) == int
        out.write(item.to_bytes(length=8, byteorder='big',
                                signed=(item < 0)))
    out.write(padding)


def generate_random_jpeg(
        rng: BinaryIO, out: BinaryIO,
        blocksize: int=256
    ) -> None:

    header = b'\xff\xe0\x00'
    padding = [b'\x00'] * blocksize
    out.write(header + len(data).to_bytes(3, 'big'))
    while True:
        data, padding = rng.read(blocksize - 4), \
                        rng.read(blocksize - 17)
        if not (padding and data): break
        out.write(data + padding)
