import os


def main():

    # Use /dev/urandom as our pseudorandomness source (this may not be
    # available on all systems! If you don't have it then please comment this
    # line and use another source of true randomness instead.
    with open('/dev/urandom', mode='rb') as rng:

        # Open a new "output" file for writing
        with open('test123456789', mode='wb+') as out:

            # Write the GIF to the output
            generate_random_gif(rng, out)


# Generates a random GIF file into `out` using the
# random number generator `rng`
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    from PIL import Image, GifImagePlugin

    # Read in some static values that we will need later
    width = 320
    height = 240
    background_color = (255, 255, 255, 0)
    color_table = [
        (0x00, 0x00, 0x00),
        (0xFF, 0xFF, 0xFF),
        (0x90, 0xC3, 0xFF),
        (0x4D, 0xA0, 0xF0),
        (0x3C, 0x3F, 0xEA),
        (0x2E, 0x0B, 0xFC),
        (0x14, 0x00, 0xFF),
        (0x00, 0x00, 0xF0),
        (0x00, 0x00, 0xD4),
        (0x00, 0x00, 0xB8),
        (0x00, 0x00, 0x9C),
        (0x00, 0x00, 0x80),
        (0x00, 0x00, 0x63),
        (0x00, 0x00, 0x47),
        (0x00, 0x00, 0x2B),
        (0x00, 0x00, 0x00),
        (0x00, 0x03, 0x00),
        (0x00, 0x19, 0x00),
        (0x00, 0x30, 0x00),
        (0x00, 0x46, 0x00),
        (0x00, 0x5C, 0x00),
        (0x00, 0x72, 0x00),
        (0x00, 0x88, 0x00),
        (0x00, 0x9E, 0x00),
        (0x00, 0xB4, 0x00),
        (0x00, 0xBA, 0x00),
        (0x00, 0xCA, 0x00),
        (0x00, 0xD0, 0x00),
        (0x00, 0xDC, 0x00),
        (0x00, 0xE2, 0x00),
        (0x00, 0xEE, 0x00),
        (0x00, 0xFE, 0x00),
    ]
    colors = len(color_table) - 1
    frames = 2000
    delay = 10     # milliseconds between each frame

    # Write the GIF header
    GifImagePlugin.getheader(None).seek(0)
    headdata = GifImagePlugin.getheader(None).read()
    out.write(headdata[:6])    #   Signature for the start of the file
    out.write(b'\0\0\0')       #   Screen width
    out.write(b'\0\0\0')       #   Screen height
    out.write(b'\x87')         #   Flags: global color table, loop forever
    out.write((colors).to_bytes(1, byteorder="little"))    # Color resolution
    out.write((background_color[3]).to_bytes(1, byteorder="little"))
    out.write(b'\xf9\x04\x00')                         # Frame control word
    out.write(delay.to_bytes(2, byteorder="little", signed=False))
    out.write(b'\x00\x00')                             # Transparent color index
    out.write(headdata[-3:])                          # Other parameters

    imgdata = []
    for y in range(height):                        # Loop over rows
        row = bytes([                     # and write them row by row
            color_table[int(rng.read(1)[0] % colors)]
            for x in range(width)])             # Get a color out of RNG
        imgdata.append(row)                       # Put the pixels in a list

    im = Image.new("RGB", (width, height), background_color[:-1])
    im.putdata(imgdata)                            # Create an image
    im.save(out, format="gif")


if __name__ == '__main__':
    main()