#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Get the size of the image data (normally 7 bytes)
    size = rng.read(4)
    data = b''
    
    # Read in the actual data until we get to \x00\x3B
    while True:
        temp = rng.read(1)[0]
        
        if temp == b'\x00':
            nextchar = rng.read(1)[0]
            if nextchar!= b'\x3B':
                raise ValueError('Invalid Gif!')

        else:
            data += temp.to_bytes(length=1, byteorder="big")

    # Write everything back out
    out.write(header + size)
    out.write(data)
    