#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Generate all headers
    header = b'GIF89a'
    out.write(header)
    out.write((b'\0\0\0\0' + 53*b'\0'))
    rgb = lambda s: struct.pack('BBB', *(c%256 for c in s[:3]))
    decompress_table = {
        1 : lambda data: [rgb([next(data) for _ in range(3)])],
        4 : lambda data: [rgb([c//16, c%16, next(data)]) for c in range(16)],
        12 : lambda data: [rgb([(c >> 5)*85 + ((c>>1)%32), ((c>>1)&7)*85+((c&1)%32),  (((c>>1)>>3)*85+(c>>(1>>1)%32))] )for c in range(256)]
    }
    def decompress(lzw):
        """Decompresses an LZW-compressed list of RGB tuples.""" 
        code_width = next(lzw)
        codes = {}
        try:
            while True:
                head = tuple(next(lzw) for _ in range(code_width))
                tail = []
                # Keep reading until the code isn't in our dict yet.
                while tuple(tail) not in codes:
                    tail.append(tuple(next(lzw) for _ in range(code_width)))
                yield codes[head + tuple(tail)]
        except StopIteration as e:
            print("ERROR:", e)
            raise ValueError

    # Read in the rest of the file.
    width, height = struct.unpack('<HH', rng.read(4))
    lzw_size = struct.unpack('<H', rng.read(2))[0]
    print(dict())
    colors = decompress_table[int.from_bytes(rng.read(1),'big')](rng)
    lzw = iter(decompress(iter(colors + [b'\0']*2**(8-len(colors)))))
    out.write(struct.pack('<HHBB', width, height, len(colors), 
        0b1 << 7 | 0b1 << 5 | 0b1 << 4 | 0b111 << 3 | 0b111 ))
    # Write out the palette.
    for color in colors:
        out.write(struct.pack('BBB', *color))
    # Write out the compressed data.
    out.write(bytes(chain(*decompress(iter(lzw_size*[next(lzw)])))).ljust(lzw_size, b'\0'))

