#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import struct
from collections import namedtuple
from enum import IntEnum

class BlockType(IntEnum):
    EXTENSION = 0x21
    IMAGE_DESCRIPTOR = 0x2c

    @classmethod
    def from_byte(cls, b):
        return cls((b >> 5) & 7)


Block = namedtuple('Block', ['type', 'data'])



def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    sizelen = (rng.read(1)[0] & 7) + 1
    size = struct.unpack('<H', b'\x00' + rng.read(sizelen))[0]
    width = rng.read(1)[0] | ((size & 0x6000) << 1)
    height = rng.read(1)[0] | (((size & 0x7000) ^ 0x7000) >> 3)
    global_color_table_flag = bool((size >> 1) & 1)
    local_color_table_flag = bool((size >> 2) & 1)
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0] & 127
    background_color_selector = str(background_color_index).encode()
    blocklen = rng.read(1)[0]
    while True:
        type_ = BlockType.from_byte(blocklen)
        data = rng.read(blocklen - 1)

        if type_ == BlockType.EXTENSION:
            print("Extension!")

            # TODO Extensions are not supported atm!
            #raise NotImplementedError("Extensions are not implemented.")


        elif type_ == BlockType.IMAGE_DESCRIPTOR:
            print("Image descriptor")

            out.write(
                bytes([*header[0], *(width, height)])
            )

            break

        else:
            raise ValueError("Unknown block encountered!")


    out.flush()