import io


class RandomGifGenerator():

    def __init__(self):

        # Bytes to use for generating each frame (the first is always empty!)
        self._frames_data = [
            bytes([0x47, 0x49, 0x46]),     # Header and version
            bytes([0xff, 0xff, 0xff])]    # Color table


    @property
    def frames_count(self) -> int:
        return len(self._frames_data)-1


    def add_frame(self, data: bytes) -> None:
        
        if not isinstance(data, bytes):
            raise TypeError('This method only accepts bytes as input')

        self._frames_data.append(data)

    
    def write_to(self, output: BinaryIO) -> None:

        if not hasattr(output, 'write'):
            raise AttributeError("The given object isn't writable")

        with io.BytesIO() as buffer:

            for i in range(len(self._frames_data)):
                chunk_size = self._make_chunk(buffer, i+1 == len(self._frames_data))
                
                if i!= len(self._frames_data)-1:
                    padding = chunk_size - len(self._frames_data[i])

                    while padding > 0:
                        current_padding = min((2**12), padding);

                        if i < len(self._frames_data)-1:
                            buffer.write(b'\0' * current_padding)
                        
                        padding -= current_padding
            
            output.write(buffer.getvalue())
    

    def _make_chunk(self, buffer: BinaryIO, last_chunk: bool) -> int:
        """ Write a single chunk to the buffer and returns its size """

        length = sum([(len(d)+1) for d in self._frames_data])-1
        buffer.write((length).to_bytes(3, "little", signed=False))
        buffer.write(b'\x00\x21\xf9\x04\x00')
        buffer.write(((~(-length//255))*255).to_bytes(1, "big"))
        buffer.write(b'\x00\x00')

        if last_chunk:
            buffer.write(b';')
                
        else:
            buffer.write(b',')

        for data in self._frames_data:
            buffer.write((len(data)).to_bytes(2, "little", signed=False))
            buffer.write(b'\x00')
            buffer.write(data)
            
        return 13 + length