#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Returns an integer in [min..max]
def randint(
        min: int, max: int, rng: BinaryIO
    ) -> int:
    return rng.randint(min, max)

# Returns true if the next bit from `rng` is set to one, false otherwise.
def prob(
        p: float, rng: BinaryIO
    ) -> bool: 
    return rng.randrange(1 / p) == 0

# Writes `data` into `out`.
def write_all(
        data: bytes, out: BinaryIO
    ):
    out.write(data)

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header length for GIF files is always constant
    headerlen = len('GIF89a')
    write_all(('GIF89a').encode(), out)

    width = randint(65, 250, rng)
    height = randint(65, 250, rng)
    
    globalcolorcount = None
    haslocalcolorcount = False
    while not haslocalcolorcount and not globalcolorcount: 
        localcolorcount = randint(
            0, 2**bpp - 1, rng)

        if localcolorcount!= 0: 
            haslocalcolorcount = True

    globalcolorscalepalette = []
    if globalcolorcount & (globalcolorcount - 1) == 0:
        nbits = bin(globalcolorcount)[::-1].index('1')
        while globalcolorcount > 2 ** nbits:
            nbits += 1

        globalcolorscale = randint(0, 2**(nbits+1)-1, rng)
        
        pixlen = ((height * width) // 8)
        for _ in range(pixlen): 
            pixel = randint(
                0, 2**bpp - 1, rng)
            
            out.write(pixel.to_bytes(1, 'little'))

    # Data block
    data = ''

    # Start of Image Block
    soi = b''

    # Logical Screen Descriptor Block
    lsd = struct.pack(
            '!HHBBB', width, height, gcbpp, bgcol, aspectratio
        )

    # Global Color Table Extension Block
    gceb = struct.pack(
            '!B', gcbpp
    )
    if globalcolorcount <= 2**gcbpp: 
        pass
    elif globalcolorcount < 256: 
        gceb += struct.pack(
                '!B', 7
            ) + globalcolorscale palette
    else: 
        gceb += struct.pack(
                '!B', 15
            ) + globalcolorscale.to_bytes(3, 'big')

    # Plain Text Extension Block
    pteb = struct.pack(
            '!BBBBB%dsB' % len(textutf8), 0x01, xposchar, yposline, textfont,
            textsize, textutf8, 0x00
        )

    # Application Extension Block
    appeb = struct.pack(
            '%dsBBBBBB' % len(appname), appname, 0xFF, applicationversionmajor,
            applicationversionminor, apptype, appauthenticationcode
        )

    # Comment extension block
    ceb = struct.pack('%dsB' % len(commentutf8), commentutf8, 0x00)

    # Trailer which only contains the endmarker
    trailer = b'\x3B'

    # The complete Gif89a structure consists now of its parts.
    gif = soi + lsd + gceb + pteb + appeb + ceb + trailer

    # Finally we can write this stuff to our output stream.
    write_all(gif, out)