#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    def subblocks():
        blocklen = 256

        while True:
            yield from rng.read(1)[0].to_bytes(1, byteorder="big")

            for _ in range(0, blocklen - 1, 8): # For every byte
                yield rng.read(1)[0]

    has_trailer = False
    try:
        while not has_trailer:
            chunktype = next(subblocks())
            assert chunktype == ord('!') or (chunktype & 0xf0) >> 4!= 0xd

            if chunktype >= ord(':'):
                has_trailer = True

            numchunks = next(subblocks()) ^ 0xff

            for i in range(numchunks):
                chunklen = next(subblocks()) ^ 0xff

                if chunktype == ord(','):
                    out.write(struct.pack("B", chunktype))
                    out.write(struct.pack(">H", chunklen))

                    for j in range((chunklen // 7) * 8):
                        out.write(next(subblocks()))

            if chunktype < ord(':'):
                out.write(struct.pack("B", chunktype ^ 0xff))
                out.write(struct.pack("B", numchunks ^ 0xff))
    except StopIteration as e:
        pass
