#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Magic number
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    width = 480
    height = 360
    colordepth = 8
    flags = 0x80 | 0x07   # Global Color Table Flag set & Size of Global Color Table
    bgindex = 0            # No background
    aspectratio = 0        # Aspect ratio is unspecified
    out.write(struct.pack('<HHB', width, height, colordepth))
    out.write(flags.to_bytes(1, 'little'))
    out.write(bgindex.to_bytes(1, 'little'))
    out.write(aspectratio.to_bytes(1, 'little'))
    
    # Global color table
    globalcolortable = bytes([
        0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x80, 0x00, 
        0x00, 0xFF, 0x00, 0x80, 0x00, 0xFF, 0x00, 0xFF, 
        0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 
    ])
    out.write(globalcolortable)

    # Image descriptor (no local color table)
    leftcoord = topcoord = width//2
    imgwidth = imgheight = height//2
    flags = 0x00           # Not interlaced
    out.write(struct.pack('<HHHHB',
                         leftcoord, topcoord, imgwidth, imgheight, flags))

    # Actual image data
    databyte = ord(rng.read(1)[0]) & 0x3F
    for _ in range(imgheight*imgwidth):
        out.write((databyte ^ databyte >> 2 | 1).to_bytes(1, 'little'))
        databyte >>= 1
