"""
This module contains functions for generating example images.
They are used in various tests and benchmarks.
"""
import numpy as np

from.util import write_uint8


def get_noise(shape=(16,)):
    return np.random.randint(-4, 5, shape=shape, dtype=np.int8)


def tone_map_image(img):
    """Map the grayscale values from -4..+4 to 0..255."""
    assert img.dtype == np.int8
    img -= (-4)
    img *= 255 // 8
    return np.clip(img, 0, 255)


def save_as_png(path, image):
    rows, cols = image.shape
    with open(path, "wb") as output:
        # PNG signature and header
        output.write(b"\x89PNG\r\n\x1A\n")

        # IHDR chunk
        write_uint8(output, 13)      # length     - uint32 (4 bytes)
        output.write(b"IHDR")        # type       - fixed string
        write_uint8(output, *image.shape)  # dimensions - width, height
        output.write(b"\x08\x02\x00\x00")    # bit depth, colour type, compression method, filter method
        output.write(b"\x00\x00\x00\x00")    # interlace flag, unused, crc (4 bytes)

        # IDAT chunk
        filtered_row = [
            0,   # No filtering
        ] * cols + list(image.flatten())
        
        write_uint8(output, len(filtered_row))          # length     - uint32 (4 bytes)
        output.write(b"IDAT")                           # type       - fixed string
        compressed = zlib.compress(bytes([filtered_row])) # compressor - deflate compression
        write_uint8(output, *compressed[:-4])           # raw content without checksum
        write_uint8(output, *struct.unpack("!i", compressed[-4:])[0]) # checksum   - 32bit unsigned integer (CRC32)

        # IEND chunk
        write_uint8(output, 0)         # length     - uint32 (4 bytes)
        output.write(b"IEND")           # type       - fixed string
        output.write(b"\xAE\x42\x60\x82")    # CRC32 (4 bytes)

