#!/usr/bin/env python3

import os
import sys
from struct import pack
from typing import BinaryIO


# Generates a random GIF file to stdout or the given filename. If not specified
# the filename will be generated randomly as follows:
#   "random-2015-12-34T12:34:56Z.gif"
def generate_random_gif(filename=None):
    if filename is None:
        filename = f'random-{os.environ["SOURCE_DATE_EPOCH"]}.gif'

    print('generating', filename, file=sys.stderr)

    with open('/dev/urandom', 'rb') as rng:

        # 1. Parse the header (always starts with GIF87A or GIF89A) and find its length
        magic, version, width, height = unpack('<4sHH', readall(rng, 8))
        assert magic == b'GIF87a' or magic == b'GIF89a'
        total_size = len(magic) + 2 + 4 * 2

        # 2. Read the rest of the image
        remaining = 0xffff - total_size
        while True:
            chunktype = rng.read(1)[0]

            if chunktype == 0x3b:
                break
            elif chunktype == 0x2c:
                dlength = unpack('<L', readall(rng, 4))[0]

                total_size += 4
                remaining -= 4

                total_size += dlength
                remaining -= dlength

                # don't care about other chunks for now
                continue
            else:
                raise ValueError("unexpected chunk type")

            if remaining < 0:
                raise RuntimeError("ran out of bytes")


        # 3. Write everything back to a new file
        with open(filename, 'wb') as fid:
            fid.write(magic)
            fid.write(pack('<H', width))
            fid.write(pack('<H', height))

            endpos = fid.tell()
            while True:
                buf = rng.read(remaining)
                if not buf:
                    break
                fid.write(buf)
