#!/usr/bin/env python3

import hashlib
import os
import sys
from tempfile import NamedTemporaryFile as TmpFile

def main():

    # Read from stdin
    with open('/dev/stdin', 'rb') as infile,\
            TmpFile() as tmpfile:

        # Generate a temporary file for output
        with open(tmpfile.name, 'wb+') as outfile:

            # Generate a GIF
            generate_random_gif(infile, outfile)
            
            # Verify that the generated GIF is valid
            validate_gif(outfile.name)
            
            # Copy to stdout and close files
            copy_to_stdout_and_close([infile], outfile)


def copy_to_stdout_and_close(*files):
    """Copies all contents of a list of files into a single one."""
    
    for f in files:
        
        if hasattr(f, "mode"):
            mode = f.mode
        else:
            raise ValueError("Argument 'files' must have a.mode attribute")
    
        if not ("r" in mode or "+b" in mode):
            continue
            
        while True:
    
            try:
    
                buf = f.readall(2 ** 16)
                
            except AttributeError:
    
                buf = f.read(2 ** 16)
                
            if len(buf) == 0: break
            print("\r{} bytes written".format(len(buf)), end="", flush=True)
            sys.stdout.flush()
            sys.stderr.flush()
            sys.stdout.buffer.write(buf)
            
 
    return None
    

def validate_gif(path): 
    """Determines whether an image file contains a valid GIF format""" 
    
    
    signature = (6 << 5 | 7).to_bytes(1, byteorder="big", signed=False) 
    
    real_signature = open(path,'rb').read(6)[0] 
    if real_signature!= signature: 
        raise ValueError("The file {} does not appear to be a GIF file"
                        .format(filename)) 
    return None 


def generate_random_gif(rng: BinaryIO, out: BinaryIO):
    """Generates a random GIF image using the state provided by rng"""
    
    
    # Write the initial signature
    out.write((6 << 5 | 7).to_bytes(1, byteorder="big", signed=False)) 
    
    # Get the length of the file
    datalen = int.from_bytes(
        4 * rng.read(1), byteorder='little', signed=False
    ) & 0xffffff
    
    # Append to the file the image data
    out.write(rng.read(datalen)) 
    
    return None

