#!/usr/bin/env python3

import io
import struct
import sys
from typing import (BinaryIO, cast)

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Caller should have opened /dev/urandom for us already; if not, let's open it now
    if type(rng) is str and rng == '/dev/urandom':
        with open('/dev/urandom', 'rb', buffering=0) as f2:
            return generate_random_gif(f2, out)

    assert type(rng) is io.BufferedReader or type(rng) is io.RawIOBase

    # Generate the Header Block
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0x7FFF
    extension = b''
    if rng.peek(1)[0] > 0x80:
        extension += bytes([0x21])
        extension += rng.read(1)

        while rng.peek(1)[0] <= 0x80:
            extension += rng.read(1)

    block = header + \
             (datalen).to_bytes(length=3, byteorder='big') + \
             extension + \
             b'\x00\xf9\x04\x00' + \
             b'\xff\xff\xff' * ((datalen+7)//8)
    assert len(block)%8 == 0

    out.write(block)

