#!/usr/bin/env python3

import os
import sys
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    def get_color():

        r = ord(rng.read(1)) % 256
        g = ord(rng.read(1)) % 256
        b = ord(rng.read(1)) % 256

        return f'{r:x}{g:x}{b:x}'.encode()


    header = (
        'GIF89a'.encode('ascii') +
        bytes([ord('0'*7+str(i%2)+'0'*1)[::-1] for i in range(3)]) + 
        '\x80\xff\xfe\x00\x00' 
    )
    
    vscreen_size = struct.unpack('<2H',rng.read(4))
    vscreen_ext_block = [vscreen_size[0], vscreen_size[1]]

    global_color_table = {
        0:(0,0,0),
        1:(255,255,255),
        2:(255,0,0),
        3:(0,255,0),
        4:(0,0,255),
        5:(255,255,0),
        6:(255,0,255),
        7:(0,255,255),
    }
    color_table = [global_color_table[i] for i in [0]+list(range(8))]

    with open('/dev/urandom','rb') as urand:

        # Write Header
        out.write(header)

        # Write Logical Screen Descriptor Block
        out.write(struct.pack('<BHHBBBBB',
            2<<7 | 0,
            *(sum([(c[0]*(2**(7-j))) << j 
                     for c,_ in zip(color_table[:7],range(8))]
                   ,0)),
            *vscreen_ext_block, 
            0
        ))

        num_blocks = 3
        if len(vscreen_size)>1 and vscreen_size[1]>2: 
            num_blocks += sum((vscreen_size[1]-2)//256)-1

        for _ in range(num_blocks):

            block_type = ord(rng.read(1)) % 3
            
            if block_type == 0:
                size = int(ord(rng.read(1))*2**int(ord(rng.read(1))))
                out.write(bytes([
                    0x21, 
                    block_type, 
                    size>>8,
                    size&0xff]))
                while size > 0 :
                    chunk = min(255, size)
                    out.write(chunk*get_color())
                    size -= chunk

            elif block_type==1 or block_type==2:

                out.write(bytes([
                    0x21, 
                    block_type, 
                    *[
                        int(ord(rng.read(1))) 
                        for _ in range(3)
                    ]]))

            else: 
                raise ValueError("Invalid Gif Block Type.")
        
        print(out.tell())
        sys.exit(-1)


