#!/usr/bin/env python3

import asyncio
import io
import os
import sys
import time

import aiohttp  # pip install aiohttp


async def get_urandom() -> bytes:
    url = "https://www.random.org/cgi-bin/randbyte?nbytes=1024"

    async with aiohttp.ClientSession() as session:
        async with session.get(url) as resp:
            return await resp.read()



def main():
    if len(sys.argv) > 1 and sys.argv[1] == "--test":

        rng = io.BytesIO(os.urandom(16))
        start = time.monotonic()
        for i in range(10):
            result = generate_random_gif(rng)
            assert result is not None
        end = time.monotonic()
        print(end - start)
    
    else:
        try:
            loop = asyncio.get_event_loop()

            while True:
                result = loop.run_until_complete(
                        get_urandom())
                
                filename = time.strftime("%Y-%m-%d_%H:%M:%S.gif")
                with open(filename, mode="wb") as f:
                    generate_random_gif(
                            io.BytesIO(result), 
                            f)

                print("Wrote", filename)
        except KeyboardInterrupt:
            pass


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big'))
    out.write(data)

