#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the initial buffer from /dev/urandom
    buf = bytes()

    for i in range(1, 7):
        buf += rng.read(i)# Read up to three more byte so we have enough to begin generating the chunk size
    
    # Add all elements together and convert them back into an integer with big endian encoding
    datalen = sum([int.from_bytes(buf[i:(i+1)], byteorder='big', signed=False) << (i*8) for i in range(len(buf))])
    print("Total length of image:", datalen)

    # Write the total length as section 7-9 into the output file
    # Format is LLLLAAAA, where the A's are each individual value of the length in little endian order
    out.write((datalen+4).to_bytes(4, 'little'))

    # After that we need to write the data itself into the output file
    # Note that it's in one read operation because the length includes the size of the entire block
    out.write(rng.read(datalen)[::-1])

    