#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = ord(rng.read(1)) | \
              (ord(rng.read(1)) << 8) 

    # Image Header
    ihdr = bytearray()
    ihdr += struct.pack('<BBHHHHB',
                       0x2C,
                       (ord(rng.read(1))),
                       (ord(rng.read(1))),
                       (((ord(rng.read(1)))      ) +
                        ((ord(rng.read(1)))<< 8)),    # width
                       (((ord(rng.read(1)))      ) +
                        ((ord(rng.read(1)))<< 8)),    # height
                       ((((ord(rng.read(1))))>>7)&0b00001111) +\
                           ((((ord(rng.read(1))))&0b01111111)<<1),   # flags
                       0x00,                         # no local color table
                       ord(rng.read(1)),            # aspect ratio
                      )

    out.write(header + ihdr)

    # LZW Compressed Bytes
    for i in range(datalen - len(ihdr)):
        out.write(struct.pack('>B', ((int.from_bytes(rng.read(1), 'little')) & 0xFF)))

    # Data Block Terminator