#!/usr/bin/env python3

import struct
from typing import BinaryIO
import io

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    nframes = (1 << 16) - 1 # uint16

    framesize = ((1 << 24) * 0.75).to_bytes(
        2, byteorder='little', signed=False
    )


    write_uint16le = lambda v : out.write(v.to_bytes(2, 'little'))
    write_uint8 = lambda v : out.write(struct.pack('<B', v))

    write_fixed_string = lambda s : out.write((s[0:1] + bytes([0x20])*(len(s)-1)).encode('ascii'))

    write_uint16le(nframes)
    for _ in range(nframes):
        write_uint8(0xF7)

        write_fixed_string("NETSCAPE2.0")
        write_uint8(3)
        write_uint16le(0)
        write_uint8(1)

        write_uint16le(10)
        write_uint8(0)
        write_uint16le(1)

        write_uint32le(framesize[0])
        write_uint32le(framesize[1])

    out.flush()