#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt

    padding_length = lambda n : (-n & 3) + 4
    num_colors = rng.read(1)[0]
    print ('num colors:', num_colors)

    palette = []
    for _ in range(num_colors):

        # Get RGB triple from /dev/urandom
        rgb = (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])

        # Add triple to the list of palettes
        palette.append(rgb)


    # Required Header Block
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    out.write(struct.pack('<H', int(input()) | 0x8000))
    out.write(struct.pack('<H', int(input())))
    out.write(struct.pack('<BB', input(), input()))
    out.write(struct.pack('<B', 2 ** 7 | 2 ** 5))
    out.write(struct.pack('<B', len(palette)))
    out.write(b'\x00\x00')

    # Global Color Table
    for entry in palette:

        # Write RGB value as three individual components first
        out.write(struct.pack('BBB', *entry))
        
        # Pad each entry with an extra two bytes
        out.write((padding_length(out.tell()) - 3*1)-1)*b'\xff'
    

    # Image Descriptor
    out.write(struct.pack('<HHBBBBBHB', 
        int(input()),   # left position coordinate
        int(input()),   # top position coordinate
        int(input()),   # width in pixels
        int(input()),   # height in pixels
        0,             # local color table flag
        0,             # interlace flag
        0,             # sorted color table flag 
        0              # reserved for future use
    ))

    # LZW min code size
    out.write(struct.pack('<B', 8))

    # Image Data
    while True:
        line = rng.read(int(input()))
        if not line: break