#!/usr/bin/env python3

import gzip
import os
import re
import secrets
import sys


# The maximum size of the resulting animated image
MAXSIZE = 640 * 2 * 1024 ** 2 # 640 KiB for a 2-bit color image


# Generate a compressed random GIF file
def generate():

    # Get the length of the data stream to compress
    fsize = os.stat('image.png').st_size

    # Check that it is smaller than MAXSIZE
    if (fsize > MAXSIZE): raise ValueError()

    # Generate a random token in hexadecimal representation
    token = hex(secrets.randbits(64))[2:]

    # Write the HTML template
    with open('template.html', mode='rt', encoding='utf-8') as fp:

        html = fp.read()

        # Replace all occurrences of __TOKEN__ with the generated token
        html = re.sub(r'\b__TOKEN__\b', token, html, flags=re.I)

        print('%s bytes available' % str(int((MAXSIZE - fsize) / 8)))
        
        with open('../../index.html', mode='wt', encoding='utf-8') as out:

            # Rewrite 'index.html' replacing '__HTML__' and '__DATASIZE__' 
            out.write(
                html.replace('__HTML__', token).replace('__DATASIZE__', str(int(fsize / 8)))
            )


    # Open the output file
    with gzip.open('animated.' + token + '.dat', mode='wb') as out:

        # Create a fresh random number generator from /dev/urandom 
        rng = open('/dev/urandom', mode='rb')

        # Initialize the cumulative data size counter
        cumulsize = 0

        # While we have not reached the capacity limit...
        while cumulsize < MAXSIZE:
            
            # Compute the next size increment
            incrsize = min(MAXSIZE - cumulsize, int(.5e7))

            # Advance the RNG cursor by the same amount so that it can be reused
            rng.seek(incrsize, io.SEEK_CUR)

            # Generate a new random GIF frame
            generate_random_gif(rng, out)

            # Update the counter
            cumulsize += incrsize
    
    # Add an ending 0xff byte
    out.write(b'\xff')
