"""A simple GIF generator that read from /dev/urandom."""


import os


def main():
    with open('random-gif.gif', 'wb') as f:
        # The size of the first three bytes is encoded in little endian,
        # so we need to generate enough bits for up to 256MiB (1<<28). We use
        # urandom to avoid blocking the thread, and also because it's faster
        # than reading from /dev/random on Linux.
        bits = 28 * 8
        nbytes = -(-bits // 8)
        data = os.urandom(nbytes)

        # Encode the length into the first three bytes
        assert len(data) < 0x1000000
        data = len(data).to_bytes(3, 'little') + data

        # Write the output
        f.write(b'GIF89a')
        f.write(data)



if __name__ == '__main__':
    main()