#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` /dev/urandom
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # start with an empty buffer
    buf = bytearray()

    # fill it up with bytes from urandom
    # until we have enough information to form a valid gif 
    while len(buf) < 10 or \
       not (buf[:6] == b'GIF87a' or buf[:6] == b'GIF89a'):
        
        buf += rng.read(1)
    
    print(buf[:6])
    
    # unpack everything in little-endian order
    w, h = struct.unpack_from('<HH', buf, offset=6)
    print('{} {}'.format(w,h))
    
    out.write(buf)