#!/usr/bin/env python3

import os
import sys
import time
import hashlib
import argparse
import itertools as it
import subprocess as sp
from enum import Enum
from pathlib import Path


def main():
    parser = argparse.ArgumentParser()

    # TODO: Add arguments to specify output and seed files. For now, we use
    # `/dev/urandom`.
    parser.add_argument('outfile', nargs='?', type=argparse.FileType('wb'), default='/dev/stdout')
    parser.add_argument('--seedfile', '-s', help="The file from which to read entropy")
    args = parser.parse_args()

    if'seedfile' in vars(args):
        print("Seeding with: {}".format(vars(args)['seedfile']))

        # Read the input file into memory. We need this for reproducibility, since
        # /dev/urandom is not guaranteed to be seeded (across reboots). Not providing
        # --seedfile will still create randomly-generated gifs that are reproducible
        # across boots.
        start_time = time.monotonic()
        with open('/dev/urandom', "br") as f:
            seed = str(f.read())

        finish_time = time.monotonic() - start_time
        print("Finished reading input file: {:.5} seconds".format(finish_time))

        # Hash the seed to provide some additional seed entropy. This should
        # provide better results than just providing the random number generator
        # directly.
        start_time = time.monotonic()
        seed = hashlib.sha512(bytes(seed)).hexdigest().encode()
        finish_time = time.monotonic() - start_time
        print("Finished hashing seed: {:.5} seconds".format(finish_time))


    process_cmd(['convert', '-depth', '8', '/dev/stdin', '-transparent', '#000', '+dither', '-colors', '256', '-colorspace', 'RGB', ':-'], stdin=sys.stdin)
