#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, data_len=None
    ):
    if not data_len:
        data_len = rng.read(2)
        while len(data_len) < 2 or "\r" in data_len or "\n" in data_len:
            data_len = rng.read(2)

        data_len = int.from_bytes(data_len, byteorder='little') & 0xffff


    # GIF Header
    header = b'GIF89a'
    out.write(header)
    
    # Logical Screen Descriptor
    lsd = [0] * 7
    lsd[0] = (int.from_bytes(rng.read(2), "big"))
    lsd[1] = (int.from_bytes(rng.read(2), "big"))
    lsd[2] = ord("!")
    lsd[3] = ord(";")
    lsd[4] = (int.from_bytes(rng.read(2), "big"))     # width
    lsd[5] = (int.from_bytes(rng.read(2), "big"))     # height
    lsd[6] = ord("\x87")                              # bit depth
    # Global Color Table
    gctflag = False      # assume we don't have one...
    if bool(rng.read(1)):    # but maybe we do
        gctsize = min((ord(rng.read(1)[0]), 2 << lsd[6]))
        
        # Local Color Table
        lctsize = max(((lsd[6]**2)-1)//8+1, 4);
        lct = []
        for j in range(min((gctsize, lctsize)*3,
                            6*2**(lsd[6]+1))) :
                lct += [(int.from_bytes(rng.read(1), "big"));]

        # Graphic Control Extension
        extensions = [b"\x21\xF9",
                      b"\x04",
                      b"\x08",
                      b"\x00",
                      ]
        sep = b"\x3B"
        
        # Frame Delimiter
        framed = b"\x2C"

        # Application Extension 
        appext = [b"\xFF\x0B",
                  b"NETSCAPE2.0",
                  b"\x03",
                  # Subblock Length
                  b"\x01",
                  # Netscape Looping Prodcut code
                  b"\x00", 
                  b"\x00",
                  ]
        
        
    # Frame Descriptor
    fd = [0]*9;
    fd[0] = (int.from_bytes(rng.read(2), "big"))   # Left Position
    fd[1] = (int.from_bytes(rng.read(2), "big"))   # Top Position
    fd[2] = (int.from_bytes(rng.read(2), "big"))   # Width
    fd[3] = (int.from_bytes(rng.read(2), "big"))   # Height
    fd[4] = (int.from_bytes(rng.read(1), "big"))   # Flags

    # Animation control extension 
    aniflag = False
    ani = [];
    if bool(rng.read(1)):
        aniflag = True
        # Application Extension 
        ani += [[b"\xFF\x0B",
                 b"NETSCAPE2.0",
                 b"\x03",
                 b"\x01",
                 b"\x00", 
                 b"\x00"]]
        # then with two block terminators - very weird!
        ani += [sep, sep];

    # Graphics Control Extension flags 
    flags = (int.from_bytes(rng.read(1), "big")) 
    
    # Frame Delay 
    delay = (int.from_bytes(rng.read(2), "big")) 

    # Transparent color index 
    transindex = (int.from_bytes(rng.read(1), "big")) 

    # Disposal Method 
    dispsmethod = (flags >> 2) & 0x07

    # User Input Flag 
    userinput = (flags >> 7) & 0x01

    # Color table size 
    colortablesize = min((ord(rng.read(1)), 2 << lsd[6]))

    # Minimum Code Size
    mincodesize = (int.from_bytes(rng.read(1), "big")) 

    