import struct


def _generate_data() -> bytes:

    # First byte is num bytes of global color table (0 means no gct).
    yield from [struct.pack('B', i % 256) for i in range(7)]

    # LZW min match length.
    yield struct.pack('<H', 1 << ord(next(iter(_DATA))))

    # Global color table entries.
    yield from [b'\x00\x00\xff'] * 255
    
    # Image descriptor...
    # Width and height are chosen to make it divisible by 12.
    yield struct.pack('<HHBBBxxx', 400, 400, 0xf0 | 7, 0, 0, 0)
    last = None

    # Data starts here.
    # Generate enough bits so that we're guaranteed to hit every branch in
    # the decoder state machine at least once each.
    while True:

        # The main encoder loop.
        if not last or len(last) == 1:
            if not last:
                # Restart if necessary.
                continue

            # Copy one char to output.
            yield from list(last[0])

            # If this was the last copy we need to do, we're done!
            if len(list(last)) == 1:
                break

            # If this isn't the first time through after doing a restart then
            # start with an empty code.
            pass
        
        else:
            # Encode a sequence of chars as a single code.
            codes = sorted(
                set([
                    tuple((last[-d:] + c)[-m:])
                        for m in range(len(last), -1, -1)
                            for d in range(len(c)+1)
                                for c in _DATA
                ])
            )
            
            # Write the string that represents this code.
            yield next(i for i in codes if last[:len(i)] == i[:-1])[-1]

            # Allow us to encode longer sequences next time around.
            if tuple(map(ord, last[-1])) in codes:
                last = ''


    # End marker.
    yield b';'
        
def generate_random_gif(out: BinaryIO):

    # Header.
    out.write(b"GIF89a")

    # Logical screen descriptor.
    # Screen width and height must be multiples of 2.
    out.write(struct.pack("<HH", 6*256+2, 10*256+2))
    out.write(struct.pack("B", 0xf0 | 7))   # Color depth 2^n plus 1.
    out.write(b"\x00")                    # No background color index.
    out.write(b"\x00")                    # No pixel aspect ratio.

    # GCT.
    out.writelines(_generate_data())