import struct


def generate_random_gif(
        seed: int, rng: BinaryIO, out: BinaryIO
    ):

    # Seed the RNG with an integer value.
    rng.seek(0)
    rng.write(struct.pack('<I', seed))

    # Generate and write the file header.
    header = b'GIF89a'
    width, height = map(lambda x: (x // 256), [100, 20])
    pixel_map = []
    for i in range(height):
        row = []
        for _ in range(width):
            val = ord(next(rng)) % 2 ** 4
            row.append((val << 4 | val))
        pixel_map += row
    
    out.write(header + bytes([
        7,          # GCTF logical screen descriptor block
        0,          #    global color table size == 2 ** 4
        0, 0,       #   background color index is zero
        0b00010000, #   sort flag set to true, no interlacing
        *map(int.to_bytes, [width, height], 'big'),
        0           # No local color table is used.
    ]) + bytes(pixel_map))