#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the magic identifier
    # (always GIFF version A)
    out.write(b'GIF89a')

    # Generate the logical screen descriptor
    width = rng.randint(1 << 6, 1 << 10) | 1
    height = rng.randint(1 << 6, 1 << 10) | 1
    flags = ord('0')  # no global color table
    background_color = ord('\xf1')   # white
    pixel_aspect_ratio = ord('\x00') # square pixels

    # Write it to the output stream
    out.write(((width & 0xffff).to_bytes(2, 'big') +
               (height & 0xffff).to_bytes(2, 'big') +
               bytes([flags]) +           # colour table size
               bytes([background_color] +
                      [pixel_aspect_ratio]))
              )

    # TODO: Add each frame in turn
    return
