#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    """Generate a random GIF."""
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'little') & 0xffff

    # Image Header
    ihdr = [0] * 10
    ihdr[0] = 0x2c                  # image separator
    ihdr[1] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[2] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[3] = (int.from_bytes(rng.read(4), 'little')) & 0xff
    ihdr[4] = (int.from_bytes(rng.read(4), 'little')) & 0x0f
    ihdr[5] = (int.from_bytes(rng.read(4), 'little')) & 0x0f
    bits = ((int.from_bytes(rng.read(1), 'little')) & 0b1111)[0]
    ihdr[7] = 0 if bits == 0 else 1 << bits   # local color table
    ihdr[8] = (int.from_bytes(rng.read(1), 'little')) & 0xFF
    ihdr[9] = (int.from_bytes(rng.read(1), 'little')) & 0xFF
    out.write(header + bytes(ihdr))

    # LZW Compressed Bytes
    for i in range(datalen - len(header)):
        out.write(struct.pack('>B', ((int.from_bytes(rng.read(1), 'little')) & 0xFF)))

    # Data Block Terminator