#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    
    def _gen_screen() -> str:
        global screen

        return f'''<< /Width {width} /Height {height} /Background {background}
                /BitsPerComponent {bits_per_component} >>'''

    def _gen_palette():
        global palette

        colors = len(palette) // 3
        
        return ''.join([f'<{str(i)}>{rgb}</{i}>' for i, rgb in enumerate(map(' '.join, zip(*[iter(palette)] * 3)))[:colors]])


    width = struct.unpack('<H', rng.read(2))[0] % 65536 + 1
    height = struct.unpack('<H', rng.read(2))[0] % 65536 + 1

    background = tuple(rng.read(3))
    bits_per_component = rng.read(1)[0] & 7
    color_table_flag = bool(rng.read(1)[0] & 1) 
    
    if not color_table_flag:
        palette = []
        colors = max((width * height) // 256 - 1, 256) 

        while len(palette) < colors * 3: 
            palette += rng.read(colors * 3 - len(palette))

    else: 
        raise Exception('Color tables not implemented.')

    print(_gen_palette())
    
    palette = [tuple(rng.read(3)) for i in range(len(palette))]
        
    screen_descriptor = _gen_screen()
    # TODO Image data
    
    out.write('')
