import random
from io import BytesIO

def generate_random_gif(
        rng: BytesIO, size: int
    ) -> bytes:
    """ Generates a GIF file with randomly generated pixel values"""

    # Header fields
    type_length = b'\x47\x49\x46\x38'
    version = b'\x39\x3a'
    flags = (b'\x2c', b'\x01')
    width = struct.pack('<H', size[0])
    height = struct.pack('<H', size[1])
    bitdepth = b'\xff'

    # Global color table and palette resolution
    global_coltablesize = struct.pack('B', 0)
    global_colres = b'\x0e'             # Logarithmic 256 color depth

    # Setup the output stream
    out = []

    def write(data):
      out.append(data)

    # Write the header to the stream
    write(type_length)
    write(version)
    for flag in flags:
      write(flag)
    write(width)
    write(height)
    write(bitdepth)
    write(global_coltablesize)
    write(global_colres)

    # Now we get into the somewhat tedious process of generating a unique paletted image
    coltable = [None] * 256
    remaining_pixels = size[0] * size[1] - len(coltable)
    while True:

        # Generate a new RGB value if needed
        if not coltable[-1]:
            coltable[-1] = tuple([
                random.randint(0, 2 ** 8 - 1), 
                random.randint(0, 2 ** 8 - 1), 
                random.randint(0, 2 ** 8 - 1)
            ])

        # The background color is always black
        bg = struct.pack('BBB', *(int(v / 2) for v in reversed(coltable[-1])))

        # Get the next number of colors to populate
        num_colors = min((num_remaining := random.choice([1, 2])), remaining_pixels // 10)
        remaining_pixels -= num_colors
        
        # Populate some entries in the color table 
        indices = [(i * 10) % len(coltable) for i in range(num_colors)]
        for index in sorted(indices):
          color = coltable[index] or bg

          # We only update the entry in our local copy because there are multiple instances
          # of this method running on different threads. This way the color table stays synced 
          # across all threads which prevents undefined behavior in later steps like merging the streams together
          coltable[index] = color

          # Update the stream
          write(color)
    
    return b"".join(out)