#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Magic header
    out.write("GIF89a".encode())
    
    # Logical screen descriptor
    width = 1024 + (rng.read(1)[0] % 512)
    height = 768 + (rng.read(1)[0] % 512)
    color_table_size = 1 << ((rng.read(1)[0] & 0x07) + 1)
    background_color_index = rng.read(1)[0] % color_table_size
    pixel_aspect_ratio = 0
    out.write(struct.pack("<HHBB", width, height, pixel_aspect_ratio, background_color_index))
    
    # Global color table
    for _ in range(color_table_size):
        out.write(rng.read(3))
    
    # Application extension block
    out.write((0x21).to_bytes(1, "little"))
    out.write((0xFF).to_bytes(1, "little"))
    out.write((0x0B).to_bytes(1, "little"))
    out.write("NETSCAPE2.0".encode())
    out.write((0x03).to_bytes(1, "little"))
    out.write((0x01).to_bytes(1, "little"))
    out.write((0x00).to_bytes(1, "little"))
    out.write((0x00).to_bytes(1, "little"))
    
    # Image descriptor
    out.write((0x2C).to_bytes(1, "little"))
    out.write((0x00).to_bytes(1, "little"))
    out.write((0x00).to_bytes(1, "little"))
    out.write((width - 1).to_bytes(1, "little"))
    out.write((height - 1).to_bytes(1, "little"))
    out.write((0x00).to_bytes(1, "little"))
    
    # Local color table
    for _ in range(color_table_size):
        out.write(rng.read(3))
    
    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write((rng.read(1)[0] % color_table_size).to_bytes(1, "little"))
    
    # Trailer
    out.write((0x3B).to_bytes(1, "little"))
