#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + 2 * b'\x00' # Header
    width = struct.pack('<H', 16)    # Width
    height = struct.pack('<H', 16)   # Height
    color_map = (b'\xff\xff\xff'      # Color map
                + 15 * b'\x00\x00\x00')
    image_descriptor = b'\x2c'       # Image descriptor
    lzw_minimum_code_size = b'\x08'  # LZW minimum code size
    pixel_data = []                  # Pixel data
    for i in range(16*16):
        if i % 16 == 0:
            pixel_data.append(struct.pack('B', rng.read(1)[0]))
        else:
            pixel_data[-1] += struct.pack('B', rng.read(1)[0])
    trailer = b';'                   # Trailer

    out.write(header)
    out.write(width)
    out.write(height)
    out.write(color_map)
    out.write(image_descriptor)
    out.write(lzw_minimum_code_size)
    out.writelines([bytes([i]) for i in pixel_data])
    out.write(trailer)
