#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Magic bytes
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 256
    height = 256
    global_color_table_flag = 0x80
    color_resolution = 7
    sort_flag = 0
    size_of_global_color_table = 7
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBBB', width, height, global_color_table_flag | (size_of_global_color_table << 4), color_resolution | (sort_flag << 1), background_color_index, pixel_aspect_ratio))

    # Global color table
    for i in range(2**(size_of_global_color_table + 1)):
        out.write(struct.pack('<BBB', int(i * 255 / (2**(size_of_global_color_table + 1) - 1)), int(i * 255 / (2**(size_of_global_color_table + 1) - 1)), int(i * 255 / (2**(size_of_global_color_table + 1) - 1))))

    # Application extension
    application_extension_label = b'\xff\xfe'
    block_size = 11
    application_identifier = b'NetHogs'
    authentication_code = b'\x00\x00\x00\x00'
    data_string = b''
    out.write(application_extension_label + struct.pack('<BB', block_size, len(application_identifier)) + application_identifier + authentication_code + struct.pack('<B', len(data_string)) + data_string)

    # Image descriptor
    image_separator = b'\x2c'
    left_position = 0
    top_position = 0
    width = 256
    height = 256
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    size_of_local_color_table = 0
    out.write(image_separator + struct.pack('<HHBBB', left_position, top_position, width, height, local_color_table_flag | (size_of_local_color_table << 4) | (interlace_flag << 1)))

    # Local color table

    # LZW minimum code size
    lzw_minimum_code_size = 8
    out.write(struct.pack('<B', lzw_minimum_code_size))

    # Pixel data
    while True:
        byte = rng.read(1)[0]
        if byte == 0:
            break
        out.write(byte.to_bytes(1, 'little'))

    # Trailer
    trailer = b';'
    out.write(trailer)
