#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write("GIF89a".encode())
    
    # Logical Screen Descriptor
    width = 1 << (rng.read(1)[0] & 7) + 1
    height = 1 << (rng.read(1)[0] & 7) + 1
    color_table_size = 2 ** ((rng.read(1)[0] & 7) + 1)
    background_color_index = rng.read(1)[0] % color_table_size
    pixel_aspect_ratio = rng.read(1)[0] % 4
    out.write(struct.pack("<HHBB", width, height, color_table_size, background_color_index))
    out.write(b"\x00" if pixel_aspect_ratio == 0 else b"\xFF")
    
    # Global Color Table
    for _ in range(color_table_size):
        out.write(rng.read(3))
        
    # Application Extension Block
    out.write(b"\x21\xff\x0BNETSCAPE2.0\x03\x01")
    out.write(struct.pack("<H", 0))
    out.write(b"\x00")
    
    # Image Descriptor
    out.write(b"\x2C")
    out.write(struct.pack("<HHHH", 0, 0, width, height))
    out.write(b"\x00")
    
    # Local Color Table
    local_color_table_size = 2 ** ((rng.read(1)[0] & 7) + 1)
    for _ in range(local_color_table_size):
        out.write(rng.read(3))
    
    # Image Data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack("<B", rng.read(1)[0] % local_color_table_size))
            
    # Trailer
    out.write(b";")
