#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 19 07:54:26 2021

@author: jonas
"""
import numpy as np
from scipy.stats import norm, multivariate_normal

def get_gaussian(mean, cov):
    return lambda x: multivariate_normal.pdf(x, mean=mean, cov=cov)

def get_uniform(a, b):
    return lambda x: (np.abs(x - a) + np.abs(x - b)) / 2

def get_triangular(a, b, c):
    def triangular(x):
        if x < a or x > c:
            return 0
        elif x <= b and x >= a:
            return (x - a) / (b - a)
        else:
            return 1 - (c - x) / (c - b)
    
    return triangular

def get_beta(alpha, beta):
    return lambda x: norm.pdf(x, loc=0, scale=1) * alpha**(x) * beta**(1 - x)

def get_gamma(shape, scale):
    return lambda x: shape * scale**(-shape) * x**(shape - 1) * np.exp(-scale * x)

def get_poisson(lambda_):
    return lambda x: np.exp(-lambda_) * lambda_**x / np.math.factorial(x)

def get_exponential(rate):
    return lambda x: rate * np.exp(-rate * x)

def get_lognormal(mu, sigma):
    return lambda x: np.exp(-(np.log(x) - mu)**2 / (2 * sigma**2)) / (x * sigma * np.sqrt(2 * np.pi))

def get_weibull(k, lam):
    return lambda x: k / lam * (x / lam)**(k - 1) * np.exp(-(x / lam)**k)

def get_rayleigh(sigma):
    return lambda x: (x / sigma)**2 * np.exp(-(x / sigma)**2 / 2) / (2 * np.pi * sigma**2)

def get_pareto(alpha, beta):
    return lambda x: alpha * beta**alpha / x**(alpha + 1)

def get_studentt(df):
    return lambda x: (1 + x**2 / df)**(-(df + 1) / 2) / np.sqrt(np.pi * df / 2)

def get_chi2(df):
    return lambda x: (x / 2)**(df / 2 - 1) * np.exp(-x / 2) / (2 ** (df / 2) * np.math.gamma(df / 2))

def get_f(df1, df2):
    return lambda x: (df1 * x)**(df1 / 2) * (df2 * (1 - x))**(df2 / 2) / (df1 * df2 * (x * (1 - x)))**((df1 + df2) / 2)

def get_wald(lam, phi):
    return lambda x: np.exp(-lam * x) * lam**phi / (x**(phi - 1) * np.math.gamma(phi))

def get_wishart(n, S):
    return lambda X: ((X @ X.T).trace() - n)**(-n / 2) / (2 * np.linalg.det(S)**(n / 2))

def get_inversewishart(n, S):
    return lambda X: np.linalg.det(X)**(-(n + 1) / 2) / (2 * np.linalg.det(S)**(n / 2))

def get_multinomial(n, p):
    return lambda x: np.math.factorial(n) / np.prod([np.math.factorial(xi) for xi in x]) * np.prod([p[i]**xi for i, xi in enumerate(x)])

