#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    header = b'GIF89a'
    width = 160
    height = 120
    color_table_size = 7
    background_color_index = 0
    pixel_aspect_ratio = 0
    flags = 0x80 | (color_table_size << 4)
    screen_descriptor = struct.pack('<HHBB', width, height, flags, background_color_index)
    global_color_table = rng.read((1 << color_table_size) * 3)
    image_separator = b'\x2c'
    left = 0
    top = 0
    width = 160
    height = 120
    local_color_table_flag = 0x80
    interlace_flag = 0
    sort_flag = 0
    reserved_bits = 0
    size_of_local_color_table = 0
    packed_fields = (local_color_table_flag | interlace_flag | sort_flag | reserved_bits | size_of_local_color_table)
    image_descriptor = struct.pack('<HHBBB', left, top, width, height, packed_fields)
    lzw_minimum_code_size = 8
    block_size = 1
    clear_code = 0x100
    end_of_information_code = 0x101
    # The following is a LZW-compressed stream of 160*120 pixels.
    # It encodes a single solid black pixel.
    datalen = 5
    data = rng.read(datalen)
    out.write(header + data)
