#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng):
    # Header
    out.write(b"GIF89a")  # Magic bytes
    width = rng.randint(10, 256)
    height = rng.randint(10, 256)
    out.write(struct.pack("<HH", width, height))
    out.write(b"\xF4\xFF\xFF\xFF")  # Default color table (all white)
    out.write(b"\x00")  # Background color index
    out.write(b"\x00")  # Pixel aspect ratio

    # Logical screen descriptor
    out.write(b"\x21\xff\x0bNETSCAPE2.0\x03\x01")
    out.write(struct.pack(">I", 0))  # Repeat count

    # Application extension
    out.write(b"\x21\xff\x0bNETSCAPE2.0\x03\x01")
    out.write(struct.pack(">I", 0))  # Repeat count

    # Image block
    out.write(b"\x2c\x00\x00\x00\x00")  # Image separator + image header
    out.write(struct.pack("<HH", width, height))
    out.write(b"\x00")  # No local color table
    out.write(b"\x00")  # Reserved
    out.write(b"\x02")  # Two bits per pixel

    # Graphic control extension
    out.write(b"\x21\xf9\x04\x01\x00\x00")
    out.write(struct.pack(">H", 0))  # Delay time in centiseconds
    out.write(b"\x00")  # Transparent color index

    # Table based image data
    out.write(b"\x02")  # LZW minimum code size
    for i in range(width * height):
        if i % 2 == 0:
            out.write(b"\x00")
        else:
            out.write(b"\x01")
