#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 21 09:57:46 2020

@author: jonasg
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
import matplotlib.pyplot as plt

def get_mean(x):
    return x.mean()

def get_variance(x):
    return x.var()

def get_covariance(x):
    return x.cov()

def get_correlation(x):
    return x.corr()
    
def get_entropy(x):
    return np.sum(-np.multiply(x, np.log(x)))

def get_kurtosis(x):
    return x.kurtosis()

def get_skewness(x):
    return x.skew()

def get_median(x):
    return np.median(x)

def get_mode(x):
    return stats.mode(x)[0][0]

def get_percentile(x, percentile):
    return np.percentile(x, percentile)

def get_min(x):
    return x.min()

def get_max(x):
    return x.max()

def get_range(x):
    return (x.max() - x.min())

def get_standard_deviation(x):
    return x.std()

def get_quartiles(x):
    q1 = np.quantile(x, 0.25)
    q2 = np.quantile(x, 0.5)
    q3 = np.quantile(x, 0.75)
    iqr = q3 - q1
    lower = q1 - 1.5 * iqr
    upper = q3 + 1.5 * iqr
    return [lower, q1, q2, q3, upper]

def get_iqr(x):
    q1 = np.quantile(x, 0.25)
    q3 = np.quantile(x, 0.75)
    return q3 - q1

def get_outliers(x):
    q1 = np.quantile(x, 0.25)
    q3 = np.quantile(x, 0.75)
    iqr = q3 - q1
    lower = q1 - 1.5 * iqr
    upper = q3 + 1.5 * iqr
    outliers = []
    for value in x:
        if value < lower or value > upper:
            outliers.append(value)
            
    return outliers

def get_zscore(x):
    mean = np.mean(x)
    std = np.std(x)
    zscores = [(i - mean)/std for i in x]
    return zscores

def get_zscore_outliers(x):
    mean = np.mean(x)
    std = np.std(x)
    zscores = [(i - mean)/std for i in x]
    outliers = []
    for score in zscores:
        if abs(score) > 3:
            outliers.append(score)
        
    return outliers

def get_boxplot(x):
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    ax.set_title('Box Plot')
    ax.set_ylabel('Values')
    ax.set_xlabel('Features')
    ax.boxplot(x)
    return fig

def get_histogram(x):
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    ax.set_title('Histogram')
    ax.set_ylabel('Frequency')
    ax.set_xlabel('Values')
    ax.hist(x, bins=100)
    return fig

def get_density(x):
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    ax.set