#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, duration=1.0):
    # Magic header
    out.write(b'GIF87a')

    # Logical screen descriptor
    out.write(struct.pack('<HHB', width, height, 0xF))

    # Global color table (256 colors)
    for i in range(256):
        out.write(struct.pack('BBB', *[int(i*255/256)]*3))

    # Application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', int(duration*100)))
    out.write(b'\x00')

    # Image descriptor
    out.write(b'\x2C\x00\x00\x00\x00')
    out.write(struct.pack('<HH', width, height))
    out.write(b'\x87\x00')

    # Local color table (256 colors)
    for i in range(256):
        out.write(struct.pack('BBB', *[int(i*255/256)]*3))

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', rng.read(1)[0]))

    # Trailer
    out.write(b';')
