#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=1024, height=768):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HBB', width, height, 0xF))

    # Global color table (black and white)
    out.write(bytes([0] * 768))

    # Graphics control extension
    out.write(b'\x21\xf9\x04\x05\xff\xff\xff\x00')

    # Image descriptor
    out.write(struct.pack('<HHB', 0, 0, 0))

    # Local color table (black and white)
    out.write(bytes([0] * 768))

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([int(rng.read(1)[0])]))

    # Trailer
    out.write(b';')
