#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    header = b'GIF89a'
    width = 160
    height = 120
    color_table_size = 7
    background_color = 0
    pixel_aspect_ratio = 0
    flags = 0x80 | (color_table_size << 4)
    screen_descriptor = struct.pack('!BBB', width, height, flags)
    global_color_table = rng.read((1 << color_table_size) * 3)
    application_extension = b'\x21\xff\x0bNETSCAPE2.0\x03\x01'
    loop_count = struct.pack('<H', 0)
    image_descriptor = b'\x2c\x00\x00\x00\x00'
    datalen = len(global_color_table) + len(application_extension) + len(loop_count)
    data = rng.read(datalen)
    out.write(header + data)
