#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write("GIF89a".encode())
    
    # Logical screen descriptor
    out.write(struct.pack("<HHBB", 1024, 768, 0xF, 0))
    
    # Global color table
    for i in range(1 << 5):
        out.write(struct.pack("<BBB", *[rng.read(1)[0] for _ in range(3)]))
        
    # Application extension
    out.write(b"\x21\xFF\x0BNETSCAPE2.0\x03\x01")
    out.write(struct.pack("<H", 0))
    
    # Image block
    out.write(b"\x2C\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00")
    
    # Local color table
    for i in range(1 << 5):
        out.write(struct.pack("<BBB", *[rng.read(1)[0] for _ in range(3)]))
    
    # Image data
    for y in range(768):
        for x in range(1024):
            out.write(struct.pack("<B", rng.read(1)[0]))
            