#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Apr 17 20:46:59 2020

@author: jorge
"""
import numpy as np
from scipy.spatial import distance_matrix
from scipy.optimize import linear_sum_assignment

def get_distance(x, y):
    """
    Calculates the euclidean distance between two points x and y
    
    Parameters
    ----------
    x : array of shape (n_features)
        First point
        
    y : array of shape (n_features)
        Second point
        
    Returns
    -------
    d : float
        Euclidean distance between x and y
    """
    return np.sqrt(((x - y)**2).sum())

def get_distances(X, Y):
    """
    Calculates the euclidean distances between all points in X and Y
    
    Parameters
    ----------
    X : array of shape (n_samples_x, n_features)
        First set of points
        
    Y : array of shape (n_samples_y, n_features)
        Second set of points
        
    Returns
    -------
    D : array of shape (n_samples_x, n_samples_y)
        Distance matrix between all points in X and Y
    """
    # TODO: Implement this function
    D = distance_matrix(X, Y)
    return D

def hungarian_matching(D):
    """
    Solves the assignment problem using the Hungarian algorithm
    
    Parameters
    ----------
    D : array of shape (n_samples_x, n_samples_y)
        Distance matrix between all points in X and Y
    
    Returns
    -------
    matches : array of shape (n_matches, 2)
        Indices of matched points
    """
    # TODO: Implement this function
    cost_matrix = D.copy()
    cost_matrix[cost_matrix == 0] = np.inf
    row_ind, col_ind = linear_sum_assignment(cost_matrix)
    matches = np.array([row_ind, col_ind]).T
    return matches

def get_inliers(matches, threshold=1e-3):
    """
    Filters outliers from a list of matches
    
    Parameters
    ----------
    matches : array of shape (n_matches, 2)
        Indices of matched points
        
    threshold : float
        Maximum distance for an inlier match
        
    Returns
    -------
    inliers : array of shape (n_inliers, 2)
        Indices of inlier matches
    """
    # TODO: Implement this function
    inliers = []
    for i, j in matches:
        if D[i,j] < threshold:
            inliers.append([i,j])
            
    return np.array(inliers)

def ransac(X, Y, max_iterations=1000, threshold=1e-3, min_inliers=10):
    """
    RANSAC algorithm to find the best transformation between two sets of points
    
    Parameters
    ----------
    X : array of shape (n_samples_x, n_features)
        First set of points
        
    Y : array of shape (n_samples_y, n_features)
        Second set of points
        
    max_iterations : int
        Number of iterations to run RANSAC
        
    threshold : float
        Maximum distance for an inlier match
        
    min_inliers : int
        Minimum number of inliers required for a successful fit
        
    Returns
    -------
    T : array of shape (n_features + 1, n_features + 1)
        Transformation matrix that transforms X into Y
    """
    # TODO: Implement this function
    T = None
    best_inliers = 0
    for _ in range(max_iterations):
        # Sample random subset and indices
        sample_size = np.random.randint(low=min_inliers, high=len(X))
        sample_idx = np.random.choice(range(len(X)), size