#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # 10 bytes of header
    out.write(b'GIF89a')
    # 7-byte logical screen descriptor
    out.write(struct.pack('<HBB', 256, 256, 0))
    # Global color table (24 bits per pixel)
    for i in range(768):
        out.write(bytes([i]))
    # Image block
    out.write(b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\x00')
    # Image descriptor
    out.write(b'\x00\x00\x00\x00\x00')
    # Local color table (24 bits per pixel)
    for i in range(768):
        out.write(bytes([i]))
    # Image data
    while True:
        b = rng.read(1)[0]
        if b == 0:
            break
        out.write(bytes([b]))
    # Trailer
    out.write(b';')
