#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the header
    out.write(b'GIF89a')

    # Write the screen descriptor
    out.write(struct.pack('<H', 1))
    out.write(struct.pack('<H', 1))
    out.write(b'\x00\xFF')
    out.write(b'\x00\xFF')
    out.write(b'\x00')

    # Write the global color table
    for i in range(256):
        out.write(struct.pack('BBB', *[i, i, i]))

    # Write the application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('>I', 0))

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(256):
        out.write(struct.pack('BBB', *[i, i, i]))

    # Write the image data
    while True:
        # Read a byte from /dev/urandom
        b = rng.read(1)

        if len(b) == 0:
            break

        # Write it to the output buffer
        out.write(b)
