#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 256
    height = 256
    color_table_size = 7
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBB', width, height, color_table_size, background_color_index))

    # Global color table
    for i in range(1 << (color_table_size + 1)):
        out.write(struct.pack('<BBB', *[int.from_bytes(rng.read(1), 'big') for _ in range(3)]))

    # Image descriptor
    left = 0
    top = 0
    width = 256
    height = 256
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    reserved_local_color_table_size = 0
    out.write(struct.pack('<HHHBB', left, top, width, height, local_color_table_flag, interlace_flag, sort_flag, reserved_local_color_table_size))

    # Local color table
    for i in range(1 << (reserved_local_color_table_size + 1)):
        out.write(struct.pack('<BBB', *[int.from_bytes(rng.read(1), 'big') for _ in range(3)]))

    # LZW minimum code size
    lzw_minimum_code_size = 8
    out.write(lzw_minimum_code_size.to_bytes(1, 'big'))

    def write_blocks(data: bytes):
        while data:
            block_length = min(len(data), 255)
            out.write((block_length).to_bytes(1, 'big'))
            out.write(data[:block_length])
            data = data[block_length:]

    # Blocks of image data
    write_blocks(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00