#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1

    # Write the header and data length to the output stream
    out.write(header)
    out.write(datalen.to_bytes(3, byteorder='big'))

    # Generate the palette
    palette = [0] * (768 // 4)
    for i in range(len(palette)):
        palette[i] = rng.read(4)

    # Write the palette to the output stream
    for p in palette:
        out.write(p)

    # Generate the image data
    imgdata = []
    while len(imgdata) < datalen - 768:
        imgdata += list(rng.read(2))

    # Write the image data to the output stream
    for d in imgdata:
        out.write(d)

