#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    width = 1 + rng.randint(0x7F)
    height = 1 + rng.randint(0x7F)
    packed = (width - 1).to_bytes(1, 'big') \
        + (height - 1).to_bytes(1, 'big') \
        + b'\x87\x00' \
        + bytes([rng.randint(256)])
    out.write(packed)

    # Global color table
    for i in range(0x100):
        r = rng.randint(256)
        g = rng.randint(256)
        b = rng.randint(256)

        out.write((r, g, b))

    # Image descriptor
    out.write('\x2C\x00\x00\x00\x00\x00\x00\x00\xFF\x00\x00\x00'.encode())

    # Local color table
    for i in range(0x100):
        r = rng.randint(256)
        g = rng.randint(256)
        b = rng.randint(256)

        out.write((r, g, b))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('<B', rng.randint(256)))

