#!/usr/bin/env python3

import struct, sys, os.path
from PIL import Image


def main():
    if len(sys.argv) < 2:
        print("Usage: %s input_file [output_file]" % (os.path.basename(__file__)))
        return

    infilename = sys.argv[1]
    outfilename = None
    if len(sys.argv) > 2:
        outfilename = sys.argv[2]

    with open(infilename, "rb") as infile:
        data = infile.read()

        # Check magic number and version
        assert data[:4] == b"BMP\x00", "Invalid BMP file."
        assert data[10] == 24, "Only 24-bit uncompressed files are supported."

        width = int.from_bytes(data[18:22], byteorder="little")
        height = int.from_bytes(data[22:26], byteorder="little")

        image = []
        for i in range(height):
            row = list(struct.iter_unpack("<I", data[(i * ((width + 3) // 4)) + 54 :]))

            # Remove padding bytes from each row
            while row[-1] == 0:
                del row[-1]

            image += row[:-1]

        img = Image.new("RGB", (width, height), color=None)
        pixels = img.load()

        for y in range(img.size[1]):
            for x in range(img.size[0]):
                index = (y * img.size[0]) + x

                r = image[index] & 0xFF
                g = (image[index] >> 8) & 0xFF
                b = (image[index] >> 16) & 0xFF

                pixels[x, y] = (r, g, b)

        if not outfilename:
            outfilename = "%s.png" % (os.path.splitext(infilename)[0])

        img.save(outfilename)


if