#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
    out: BinaryIO,
    rng: BinaryIO = open("/dev/urandom", "rb")
):

    # Header
    out.write(b"GIF89a")  # Magic Number
    out.write(struct.pack("<HHH", 1024, 768, 0))  # Screen Descriptor
    out.write(b"\x87\x00\x00")  # Global Color Table (256 colors)

    # Image Data
    for i in range(0, 1024 * 768 // 3):
        r = int.from_bytes(rng.read(1), byteorder="little") % 256
        g = int.from_bytes(rng.read(1), byteorder="little") % 256
        b = int.from_bytes(rng.read(1), byteorder="little") % 256

        out.write(struct.pack("BBB", r, g, b))

