#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    print('datalen', datalen)

    # Write the header and screen descriptor
    out.write(header + bytes([0x2c]) +
              struct.pack('<HHHHHBBBBB', 100, 100,
                          0xf7 | (datalen >> 16 & 0x01),
                          datalen >> 8 & 0xff, datalen & 0xff,
                          0x00, 0x00, 0x00, 0x00))

    # Write the palette
    for _ in range(0, 768):
        out.write(struct.pack('<BBBB', *rng.read(4)))

    # Write the image data
    while True:
        chunklen = rng.read(1)[0]

        if chunklen == 0x2c:
            break
        
        out.write(chunklen.to_bytes(1, 'big'))
        out.write(rng.read(chunklen))

