#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(b'\x01\x2c', byteorder='big')
    width = 64 * (int.from_bytes(rng.read(1), byteorder='big') % 5) + 1
    height = 64 * (int.from_bytes(rng.read(1), byteorder='big') % 5) + 1
    globalcolortablesize = int.from_bytes(rng.read(1), byteorder='big') & 7
    localcolortablesize = int.from_bytes(rng.read(1), byteorder='big') & 7

    # Write header
    out.write(header)

    # Write logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height,
            globalcolortablesize << 4 | localcolortablesize,
            0, 0))

    # Write global color table
    for _ in range(globalcolortablesize):
        out.write(rng.read(3))

    # Write image descriptor
    out.write(struct.pack('>BBHHHBB', 0x2C, 0, 0, 0, 0, 0, 0))

    # Write local color table
    for _ in range(localcolortablesize):
        out.write(rng.read(3))

    # Write graphics control extension
    out.write(b'\x21\xF9\x04\x05\x00\x00\x00\x00\x00')

    # Write plain text extension
    out.write(b'\x21\x01\x00\xFF')

    # Write application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0')
    out.write(b'\x03\x01\x00\x00')

    # Write comment extension
    out.write(b'\x21\xFE\x00')

    # Write graphic control label
    out.write(b'\x21\xF9\x04')

    # Write graphic control block
    out.write(struct.pack('>BBBBHB', 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

    # Write plain text label
    out.write(b'\x21\x01\x00')

    # Write plain text block
    out.write(struct.pack('>BB', 0x00, 0x00))

    # Write end of file
    out.write(b';')

            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
