#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_gif(out: BinaryIO, width: int = 1024, height: int = 768):
    # Header (GIF89a)
    out.write(b"GIF89a")

    # Logical screen descriptor
    out.write(struct.pack("<HHBBB", width, height, 0xF0, 0x00, 0x00))

    # Global color table
    for i in range(0, 256 * 3, 3):
        out.write(bytes([i // 3 % 256]))
    out.write(b"\x00\x00\x00")

    # Image descriptor
    out.write(struct.pack("<HHHHB", 0, 0, width, height, 0x00))

    # Local color table
    for i in range(0, 256 * 3, 3):
        out.write(bytes([(i + 1) // 3 % 256]))
    out.write(b"\x00\x00\x00")

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([(y ^ x) & 0xFF]))

