#!/usr/bin/env python3

import struct


def read_uint8(f):
    return int.from_bytes(f.read(1), byteorder='little')


def write_uint8(f, value):
    f.write(value.to_bytes(1, byteorder='little'))


def read_uint16(f):
    return int.from_bytes(f.read(2), byteorder='little', signed=True)


def write_uint16(f, value):
    f.write(value.to_bytes(2, byteorder='little', signed=True))


def read_uint32(f):
    return int.from_bytes(f.read(4), byteorder='little', signed=True)


def write_uint32(f, value):
    f.write(value.to_bytes(4, byteorder='little', signed=True))


def read_int16(f):
    return int.from_bytes(f.read(2), byteorder='little')


def write_int16(f, value):
    f.write(value.to_bytes(2, byteorder='little'))


def read_float(f):
    return struct.unpack('f', f.read(4))[0]


def write_float(f, value):
    f.write(struct.pack('f', value))


def read_string(f):
    length = read_uint16(f)

    if length == 0:
        return ''

    data = f.read(length).decode()

    # Strip trailing nulls and spaces (for some reason they're sometimes present)
    while len(data) > 0 and ((data[-1] == '\x00' or data[-1] =='') and not data[-2].isalnum()):
        data = data[:-1]

    return data


def write_string(f, string):
    encoded = bytes(string + ('\x00' * max(0, 256 - len(string))), encoding='ascii')
    write_uint16(f, len(encoded))
    f.write(encoded)
