#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    header = b'GIF89a' + bytes([0] * 7)

    # Logical screen descriptor
    width = rng.read(1)[0] % (2**64 - 1)
    height = rng.read(1)[0] % (2**64 - 1)
    packed = bytearray()
    packed += struct.pack('>H', width)
    packed += struct.pack('>H', height)
    packed[5] |= rng.read(1)[0] & 0x07
    packed[5] |= ((rng.read(1)[0] & 0x01) << 7)
    packed[6] |= (rng.read(1)[0] & 0x03) << 6
    packed[6] |= ((rng.read(1)[0] & 0x01) << 7) | ((rng.read(1)[0] & 0x01) << 5)
    packed[7] |= (rng.read(1)[0] & 0x0F) << 4
    packed[7] |= ((rng.read(1)[0] & 0x01) << 7) | ((rng.read(1)[0] & 0x01) << 5) | \
                 ((rng.read(1)[0] & 0x01) << 3)
    logical_screen_descriptor = bytes(packed)

    # Global color table
    global_color_table_size = rng.read(1)[0] & 0x07
    global_color_table_flag = bool((rng.read(1)[0] >> 7) & 0x01)
    global_color_table = [None for _ in range(global_color_table_size ** 2)]
    if global_color_table_flag:
        for i in range(len(global_color_table)):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            global_color_table[i] = (r, g, b)

    # Loop control block
    loop_control_flag = bool((rng.read(1)[0] >> 7) & 0x01)
    loop_count = None
    if loop_control_flag:
        loop_count = rng.read(2)

    # Image descriptor
    image_left = rng.read(2)
    image_top = rng.read(2)
    image_width = rng.read(2)
    image_height = rng.read(2)
    packed = bytearray()
    packed += struct.pack('>H', image_left)
    packed += struct.pack('>H', image_top)
    packed += struct.pack('>H', image_width)
    packed += struct.pack('>H', image_height)
    packed[5] |= (rng.read(1)[0] & 0x01) << 7
    packed[5] |= ((rng.read(1)[0] & 0x03) << 6)
    packed[6] |= (rng.read(1)[0] & 0x0F) << 4
    packed[6] |= ((rng.read(1)[0] & 0x01) << 7) | ((rng.read(1)[0] & 0x01) << 5) | \
                 ((rng.read(1)[0] & 0x01) << 3)
    packed[7] |= (rng.read(1)[0] & 0x0F) << 4
    packed[7] |= ((rng.read(1)[0] & 0x01) << 7) | ((rng.read(1)[0] & 0x01) << 5) | \
                 ((rng.read(1)[0] & 0x01) << 3)
    image_descriptor = bytes(packed)

    # Local color table
    local_color_table_size = rng.read(1)[0] & 0x07
    local_color_table_flag = bool((rng.read(1)[0] >> 7) & 0x01)
    local_color_table = [None for _ in range(local_color_table_size ** 2)]
    if local_color_table_flag:
        for i in range(len(local_color_table)):
            r = rng.read(1)[0]
            g = rng.read