#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    width  = int.from_bytes(rng.read(2), byteorder='little') % 10 + 5
    height = int.from_bytes(rng.read(2), byteorder='little') % 10 + 5

    # Logical screen descriptor
    lsd = (width & 0xFF).to_bytes(1, 'big') \
          + ((width >> 8) & 0xFF).to_bytes(1, 'big') \
          + (height & 0xFF).to_bytes(1, 'big') \
          + ((height >> 8) & 0xFF).to_bytes(1, 'big') \
          + bytes([int.from_bytes(rng.read(1), byteorder='little')]) \
          + bytes([int.from_bytes(rng.read(1), byteorder='little')]) \
          + b'\x87\x00'

    # Global color table
    gct = b''
    for i in range(0, 2**(int.from_bytes(rng.read(1), byteorder='little'))):
        r = int.from_bytes(rng.read(1), byteorder='little') % 256
        g = int.from_bytes(rng.read(1), byteorder='little') % 256
        b = int.from_bytes(rng.read(1), byteorder='little') % 256

        gct += bytes([r]) + bytes([g]) + bytes([b])

    # Image descriptor
    idt = (0).to_bytes(1, 'big') \
          + (0).to_bytes(1, 'big') \
          + (width & 0xFF).to_bytes(1, 'big') \
          + ((width >> 8) & 0xFF).to_bytes(1, 'big') \
          + (height & 0xFF).to_bytes(1, 'big') \
          + ((height >> 8) & 0xFF).to_bytes(1, 'big') \
          + b'\x00'

    # Local color table
    lct = b''
    if bool(int.from_bytes(rng.read(1), byteorder='little')):
        for i in range(0, 2**(int.from_bytes(rng.read(1), byteorder='little'))):
            r = int.from_bytes(rng.read(1), byteorder='little') % 256
            g = int.from_bytes(rng.read(1), byteorder='little') % 256
            b = int.from_bytes(rng.read(1), byteorder='little') % 256

            lct += bytes([r]) + bytes([g]) + bytes([b])

    # Graphics control extension
    gce = (0xF9).to_bytes(1, 'big') \
          + (4).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byteorder='little') |
             0x80
           ).to_bytes(1, 'big') \
          + (
             int.from_bytes(rng.read(1), byte