#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write("GIF89a".encode())

    width = 1024
    height = 768
    bitdepth = 8
    colorspace = True
    globalcolortablesize = 256
    backgroundindex = 0
    aspectratio = (width << 8) | height

    out.write(struct.pack("<HHBBB",
        width, height,
        bitdepth,
        0 if colorspace else 1,
        globalcolortablesize))

    out.write(b"\x00" * ((globalcolortablesize + 1) * 3))

    out.write(struct.pack("<HBB",
        backgroundindex,
        0,
        aspectratio))

    # Image descriptor
    out.write(struct.pack("<BHHHHB",
        0x2C,
        x=0, y=0,
        width=width, height=height,
        localcolortablesize=0))

    # LZW minimum code size
    lzwmincodesize = 2
    out.write(bytes([lzwmincodesize]))

    # Pixel data
    for i in range(height):
        row = bytearray()

        while len(row) < width:
            r = ord(rng.read(1))

            if r & 0xF == 0xE:
                n = min((r >> 4) + 1, width - len(row))

                for j in range(n):
                    row.append(ord(rng.read(1)))
            elif r & 0xF == 0xF:
                n = min(((r >> 4) & 0x7) + 1, width - len(row))

                for j in range(n):
                    row.append(0)
            else:
                row.append(r)

        out.write(row)

