#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + bytes([0x12, 0x34]) # Version and size

    width = int.from_bytes(rng.read(1), 'big') % (65536 - 100) + 100
    height = int.from_bytes(rng.read(1), 'big') % (65536 - 100) + 100

    flags = bytearray()
    for _ in range(7):
        flags.append(int.from_bytes(rng.read(1), 'big'))

    background = int.from_bytes(rng.read(1), 'big') & 0xff
    aspect_ratio = int.from_bytes(rng.read(1), 'big') & 0xff

    palette = []
    for i in range(0, 256 * 3, 3):
        r = int.from_bytes(rng.read(1), 'big') & 0xff
        g = int.from_bytes(rng.read(1), 'big') & 0xff
        b = int.from_bytes(rng.read(1), 'big') & 0xff

        palette += [r, g, b]

    image_descriptor = struct.pack('>HHBBB', width, height,
                                   *(flags + [background, aspect_ratio]))

    data = b''
    while len(data) < width * height:
        data += rng.read(width * height // 100)

    out.write(header)
    out.write(image_descriptor)
    out.write(palette)
    out.write(data)

