#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        out: BinaryIO,
        rng: BinaryIO):

    # Header
    header = b'GIF89a'
    width  = int.from_bytes(rng.read(2), 'little') % (1 << 10)
    height = int.from_bytes(rng.read(2), 'little') % (1 << 10)
    color_table_size = int.from_bytes(rng.read(1), 'little') & 7
    global_color_table = bool((int.from_bytes(rng.read(1), 'little') >> 7) & 1)
    background_index   = int.from_bytes(rng.read(1), 'little') & 0x0F
    pixel_aspect       = int.from_bytes(rng.read(1), 'little') & 0x07
    flags              = bytearray([
        0b00000000, # reserved
        global_color_table,
        0b00000000, # sorted table
        0b00000000, # interlace
        0b00000000, # local color table
        0b00000000, # reserved
    ])[::-1]
    flags[5] |= ((width == 1) and 1 or 0)
    flags[5] |= ((height == 1) and 2 or 0)
    out.write(header)
    out.write(struct.pack('<HHBBB', width, height, color_table_size, *flags))
    out.write(bytearray([background_index]))
    out.write(bytearray([pixel_aspect]))

    # Color Table
    if not global_color_table:
        for _ in range(pow(2, color_table_size)):
            r = int.from_bytes(rng.read(1), 'little')
            g = int.from_bytes(rng.read(1), 'little')
            b = int.from_bytes(rng.read(1), 'little')
            out.write(bytearray([r,g,b]))

    # Image Data
    datalen = pow(2, width+height) - 1
    data = rng.read(datalen)
    out.write(header + data)
