#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Write 7 bytes of header
    out.write(header[:6])

    # Write the screen descriptor (10 bytes)
    width = struct.unpack('>H', rng.read(2))[0] % 500
    height = struct.unpack('>H', rng.read(2))[0] % 500
    flags = [True if i == ord(b'\x01') else False for i in rng.read(3)]
    flags[4] = True
    flags[5] = True
    flags[6] = True
    flags[7] = True
    flags[8] = True
    flags[9] = True
    out.write(struct.pack('>HHBBBHB', width, height, *flags))

    # Write the global color table (3 bytes)
    numcolors = rng.read(1)[0] & 0x07
    palette = []
    for _ in range(numcolors):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        palette += [r,g,b]
    out.write(bytes([len(palette)]))
    out.write(bytes(palette))

    # Write the image descriptor (9 bytes)
    left = rng.read(2)
    top = rng.read(2)
    width = rng.read(2)
    height = rng.read(2)
    flags = [True if i == ord(b'\x01') else False for i in rng.read(3)]
    flags[4] = True
    flags[5] = True
    flags[6] = True
    flags[7] = True
    flags[8] = True
    flags[9] = True
    out.write(left+top+width+height+bytes(flags))

    # Write the local color table (3 bytes)
    numcolors = rng.read(1)[0] & 0x07
    palette = []
    for _ in range(numcolors):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        palette += [r,g,b]
    out.write(bytes([len(palette)]))
    out.write(bytes(palette))

    # Write the LZW minimum code size (1 byte)
    mincodesize = rng.read(1)[0] & 0x0F
    out.write(bytes([mincodesize]))

    # Write the image data block
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
