#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(2), byteorder='little', signed=False
    )

    # Write the header and the palette
    out.write(header)
    for _ in range(768):
        out.write(struct.pack('B', rng.read(1)[0]))

    # Write the image descriptor
    out.write(b'\x2C')  # Image separator
    out.write(struct.pack('<HHHBB',
        0, 0, 0, 0, 0
    ))

    # Write the image data
    data = bytes()
    while len(data)!= datalen:
        data += rng.read(datalen - len(data))

    out.write(header + data)
