#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1

    # Write the header and data length to the output file
    out.write(header)
    out.write(datalen.to_bytes(3, 'big'))

    # Generate the image descriptor block
    width = rng.read(1)[0] % 256
    height = rng.read(1)[0] % 256
    flags = (
        0b10000000 | # has local color table
        0b01000000 | # sorted color table
        0b00100000 | # global color table
        0b00010000 | # interlaced
        0b00001000 | # sort flag
        0b00000100 | # reserved bit
        0b00000010   # size of palette in bits - 1
    )
    out.write((width).to_bytes(1, 'little'))
    out.write((height).to_bytes(1, 'little'))
    out.write(flags.to_bytes(1, 'little'))
    out.write(b'\x00\x00') # background index
    out.write(b'\x00\x00') # pixel aspect ratio

    # Generate the local color table
    for _ in range(7):
        out.write(rng.read(1))

    # Write the image data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))

