#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG object, which must have a read() method.
def generate(rng: BinaryIO, out: BinaryIO):
    header = b'GIF89a'

    # Header length is 6 bytes (1+3+3).
    header += struct.pack('>H', 0x20)

    # Logical screen descriptor block size is 7 bytes (3+3+1+1+1).
    header += struct.pack('>HHBBB',
                          0x100, 0x100, 0xf0 | 0b111, 0, 0)

    # Global color table has one entry with value 0.
    header += b'\x00\x00\xff'

    # Image descriptor block size is 10 bytes (4+4+2+1+1+1).
    header += struct.pack('>HHBBBBB',
                          0x100, 0x100, 0b1111, 0, 0, 0)

    # Data sub-block size is 3 bytes (1+2).
    header += b'\x02\x00'

    # Data sub-block contains a single byte of value 0.
    header += b'\x00'

    # Ending block size is 1 byte.
    header += b'\x3b'

    # Write the header to output.
    out.write(header)

    # Generate and write the image data.
    for _ in range(50):
        datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
