#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_random(out: BinaryIO, width: int = 1024, height: int = 768):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0x80, 0, 0))

    # Global color table (black and white)
    out.write(bytes([
        0x00, 0x00, 0x00, 0xff,
        0xff, 0xff, 0xff]))

    # Image descriptor
    out.write(struct.pack('<BHHBB', 0x2c, 0, 0, 0, 0))

    # Local color table (black and white)
    out.write(bytes([
        0x00, 0x00, 0x00, 0xff,
        0xff, 0xff, 0xff]))

    # Image data
    for _ in range(width * height // 256 + 1):
        out.write(bytes([
            0x00] * min(256, width * height - len(out.getbuffer()))))

