#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) & 255
    height = (1 << rng.read(1)[0]) & 255
    flags  = rng.read(1)[0]
    bg     = rng.read(1)[0]
    pxsize = rng.read(1)[0]
    color  = rng.read(1)[0]
    sort   = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags, bg, pxsize))

    # Global color table
    for i in range(color):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('<BBB', r, g, b))

    # Image descriptor
    left    = rng.read(2)
    top     = rng.read(2)
    width   = (1 << rng.read(1)[0]) & 255
    height  = (1 << rng.read(1)[0]) & 255
    flags   = rng.read(1)[0]
    duration= rng.read(2)
    trans   = rng.read(1)[0]
    out.write(left + top + struct.pack('>HHBB', width, height, flags, duration[0]))

    # Local color table
    if flags & 0x80:
        for i in range(color):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(struct.pack('<BBB', r, g, b))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)