#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical screen descriptor
    width  = (1 << 7) | ((os.getpid() & 0x7F) << 2)
    height = (1 << 7) | (((os.getpid() >> 7) & 0x7F) << 2)
    flags  = 0b10000000 | 0b00000001
    color_table_size = 0
    background_color = 0
    pixel_aspect_ratio = 0
    logical_screen_descriptor = struct.pack('>HHBBB',
        width, height, flags, color_table_size,
        background_color, pixel_aspect_ratio)
    out.write(logical_screen_descriptor)

    # Global color table
    global_color_table = bytes([
        0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
        0x00, 0xFF, 0x00, 0x00, 0xFF, 0x00, 0x00, 0xFF])
    out.write(global_color_table)

    # Image descriptor
    left   = 0
    top    = 0
    width  = (1 << 7) | ((os.getpid() & 0x7F) << 2)
    height = (1 << 7) | (((os.getpid() >> 7) & 0x7F) << 2)
    flags  = 0b10000000 | 0b00000001
    image_descriptor = struct.pack('>HHBBBB',
        left, top, width, height, flags, 0)
    out.write(image_descriptor)

    # Local color table
    local_color_table = bytes([
        0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
        0x00, 0xFF, 0x00, 0x00, 0xFF, 0x00, 0x00, 0xFF])
    out.write(local_color_table)

    # Data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)

