#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# the given parameters.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor (7 bytes)
    out.write(struct.pack('>HHBBB', width, height, 0x80, 0, 0))

    # Global color table (1 << 2 * 4 - 1 == 65535 bytes)
    for i in range(65535):
        r = rng.read(1)[0] & 0xf8
        g = rng.read(1)[0] & 0xfc
        b = rng.read(1)[0] & 0xf8

        out.write(struct.pack('<H', ((r << 8) | g) << 8 | b))

    # Image descriptor (10 bytes)
    out.write(struct.pack('>BHHBB', 0x2c, 0, 0, 0, 0))

    # Local color table (1 << 2 * 4 - 1 == 65535 bytes)
    for i in range(65535):
        r = rng.read(1)[0] & 0xf8
        g = rng.read(1)[0] & 0xfc
        b = rng.read(1)[0] & 0xf8

        out.write(struct.pack('<H', ((r << 8) | g) << 8 | b))

    # Image data
    while True:
        chunklen = rng.read(1)[0]
        if not chunklen: break
        out.write(chunklen.to_bytes(1, 'little'))
        out.write(rng.read(chunklen))