#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (defaults to /dev/urandom).
def generate_random_gif(
        rng: BinaryIO=None, out: BinaryIO=sys.stdout.buffer
    ) -> None:

    if not rng:
        rng = open('/dev/urandom', 'rb')

    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical screen descriptor
    width  = struct.pack('>H', randint(1024))
    height = struct.pack('>H', randint(768))
    flags  = struct.pack('B', 0b10000000 | 0b01000000)
    bg     = struct.pack('B', randint(256))
    pxsize = struct.pack('B', 0x08)
    out.write(width + height + flags + bg + pxsize)

    # Global color table
    for i in range(2**pxsize):
        r = randint(256)
        g = randint(256)
        b = randint(256)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = struct.pack('>H', randint(width))
    top    = struct.pack('>H', randint(height))
    width  = struct.pack('>H', randint(width-left))
    height = struct.pack('>H', randint(height-top))
    flags  = struct.pack('B', 0b10000000 | 0b01000000)
    out.write(left + top + width + height + flags)

    # Local color table
    for i in range(2**pxsize):
        r = randint(256)
        g = randint(256)
        b = randint(256)
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    while True:
        blocktype = rng.read(1)[0]

        if blocktype == 0x2C:
            break

        elif blocktype == 0x21:
            pass

        else:
            raise Exception("Unknown block type")

    while True:
        blocklen = rng.read(1)[0]

        if blocklen == 0x00:
            break

        elif blocklen == 0xFF:
            continue

        else:
            out.write(rng.read(blocklen))