#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is determined by
# `width`, `height`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ):

    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical screen descriptor
    lsd = (0x21 << 8) | 0xf9
    lsd += bytes([width & 0xff])
    lsd += bytes([(width >> 8) & 0xff])
    lsd += bytes([height & 0xff])
    lsd += bytes([(height >> 8) & 0xff])
    lsd += b'\x08\x00\x00'
    out.write(lsd)

    # Global color table
    gct = [b'\x00\x00\x00', b'\xff\x00\x00']
    for i in range(len(gct)):
        out.write(gct[i] * 3)

    # Image descriptor
    id = (0x2c << 8) | 0x00
    id += bytes([0x00, 0x00, 0x00, 0x00])
    id += bytes([0x00, 0x00, 0x00, 0x00])
    id += bytes([width & 0xff])
    id += bytes([(width >> 8) & 0xff])
    id += bytes([height & 0xff])
    id += bytes([(height >> 8) & 0xff])
    id += b'\x08\x00\x00'
    out.write(id)

    # Local color table
    lct = []
    for i in range(256):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        lct.append(bytes([r, g, b]))
    for i in range(len(lct)):
        out.write(lct[i] * 3)

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(lct[rng.read(1)[0]])