#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(1024, 65536) // 2
    height = rng.randint(1024, 65536) // 2
    flags  = (
            0x80 |      # Global color table flag
            0x70 |      # Sorting method
            0x00        # Size of global color table
        ).to_bytes(1, 'little', signed=False)
    bgcol  = rng.randint(0, 255).to_bytes(1, 'little', signed=False)
    aspect = (
            0x00 |      # Pixel Aspect Ratio
            0x00        # Pixel Aspect Ratio
        ).to_bytes(1, 'little', signed=False)
    out.write(struct.pack('>HHBBB', width, height, flags[0], bgcol[0], aspect[0]))

    # Color table
    for _ in range(256):
        r = rng.randint(0, 255).to_bytes(1, 'little', signed=False)
        g = rng.randint(0, 255).to_bytes(1, 'little', signed=False)
        b = rng.randint(0, 255).to_bytes(1, 'little', signed=False)
        out.write(r + g + b)

    # Image descriptor
    left   = 0
    top    = 0
    width  = rng.randint(width - 1, width * 2)
    height = rng.randint(height - 1, height * 2)
    flags  = (
            0x80 |      # Local color table flag
            0x00 |      # Interlace flag
            0x00 |      # Sorting method
            0x00        # Reserved
        ).to_bytes(1, 'little', signed=False)
    lctsize = (
            0x00 |      # Size of local color table
            0x00        # Size of local color table
        ).to_bytes(1, 'little', signed=False)
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags[0], lctsize[0]))

    # LZW minimum code size
    mincodesize = rng.randint(2, 12)
    out.write(mincodesize.to_bytes(1, 'little'))

    # Data sub-block
    blocklen = rng.randint(1, 256)
    out.write(blocklen.to_bytes(1, 'little'))
    out.write(rng.read(blocklen))