#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = ((1 << rng.read(1)[0]) & 0x70) | \
             ((1 << rng.read(1)[0]) & 0x08) | \
             0x07
    bgcol  = rng.read(1)[0]
    aspect = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags,
                          bgcol, aspect))

    # Global color table
    for _ in range(256):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = 0
    top    = 0
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = ((1 << rng.read(1)[0]) & 0xC0) | \
             ((1 << rng.read(1)[0]) & 0x20) | \
             0x0C
    lctsize= (1 << rng.read(1)[0]) & 0xFF
    out.write(struct.pack('>HHHHB', left, top, width, height,
                          flags))

    # Local color table
    if lctsize > 0:
        for _ in range(lctsize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(struct.pack('BBB', r, g, b))

    # Image data
    while True:
        blocktype = rng.read(1)[0]

        if blocktype == 0x2C:
            break

        elif blocktype == 0x21:
            pass

        else:
            raise ValueError("Invalid block type")

    while True:
        blocklen = rng.read(1)[0]
        if not blocklen:
            break
        out.write(blocklen)
        out.write(rng.read(blocklen))