#!/usr/bin/env python3

import sys
import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number
    out.write('GIF89a'.encode())

    # Generate a random width between 10 and 256 pixels
    width = (int.from_bytes(rng.read(1), 'big') %
             (256 - 10)) + 10

    # Generate a random height between 10 and 256 pixels
    height = (int.from_bytes(rng.read(1), 'big') %
              (256 - 10)) + 10

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB',
                          width,
                          height,
                          0xf7 | ((rng.read(1)[0] & 0x07) << 4),
                          0xff,
                          0x00))

    # Write the global color table
    for i in range(0, 2**((rng.read(1)[0] & 0x07) + 1)):
        out.write(struct.pack('<BBBB',
                              rng.read(1)[0],
                              rng.read(1)[0],
                              rng.read(1)[0],
                              0x00))

    # Write the image descriptor
    out.write(struct.pack('<HHBBBBB',
                          0,
                          0,
                          width,
                          height,
                          0,
                          0xf7 | ((rng.read(1)[0] & 0x07) << 4)))

    # Write the local color table
    for i in range(0, 2**((rng.read(1)[0] & 0x07) + 1)):
        out.write(struct.pack('<BBBB',
                              rng.read(1)[0],
                              rng.read(1)[0],
                              rng.read(1)[0],
                              0x00))

    # Write the graphics control extension
    out.write(b'\x21\xF9\x04\x05\x00\x00\x00\x00\x00\x00')

    # Write the application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0')
    out.write(b'\x03\x01\x00\x00')
    out.write(b'\x00\x00')

    # Write the plain text extension
    out.write(b'\x21\x00\x0B')
    out.write(('This is a test of the emergency broadcast system.'
               ).ljust(11).encode())

    # Write the comment extension
    out.write(b'\x21\xFE\x0CComment Extension')
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the graphic control label
    out.write(b'\x21\xF9\x04')

    # Write the delay time
    out.write(struct.pack('<H',
                          int.from_bytes(rng.read(2), 'big')))

    # Write the transparent color index
    out.write(struct.pack('<B',
                          rng.read(1)[0]))

    # Write the terminator
    out.write(b'\x3B')

