#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the image is determined by `width`, and
# the color depth (number of colors used in the palette)
# is determined by `depth`.
def generate_random_gif(
        rng: BinaryIO, width: int, depth: int, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, width, 0xf7 & depth, 0x00))

    # Global color table
    for i in range(2**depth):
        r = ord(os.urandom(1))
        g = ord(os.urandom(1))
        b = ord(os.urandom(1))

        out.write(struct.pack('<BBB', r, g, b))

    # Image descriptor
    out.write(struct.pack('>HHHHB', 0, 0, width - 1, width - 1, 0x00))

    # LZW minimum code size
    out.write(b'\x08')

    # Data sub-block
    while True:
        blocklen = ord(os.urandom(1))
        if not blocklen: break
        out.write(os.urandom(blocklen))

    # Trailer
    out.write(b';')


