#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 << (int.from_bytes(
            rng.read(1), byteorder='big')) - 1
    height = 1 << (int.from_bytes(
            rng.read(1), byteorder='big')) - 1
    flags = bytes([0])
    background_color = bytes([0x21])
    pixel_aspect_ratio = bytes([0])
    out.write(struct.pack('>HHBBB',
            width, height, *flags))
    out.write(background_color)
    out.write(pixel_aspect_ratio)

    # Global color table
    global_color_table_size = int.from_bytes(
            rng.read(1), byteorder='big')
    global_color_table_entry_size = int.from_bytes(
            rng.read(1), byteorder='big')
    global_color_table = [
            rng.read(global_color_table_entry_size)
                for _ in range(global_color_table_size)]
    for entry in global_color_table:
        out.write(entry)

    # Image descriptor
    left = 0
    top = 0
    image_width = width
    image_height = height
    flags = bytes([0x2c])
    local_color_table_flag = bytes([0])
    interlace_flag = bytes([0])
    sort_flag = bytes([0])
    reserved = bytes([0] * 3)
    local_color_table_size = int.from_bytes(
            rng.read(1), byteorder='big')
    local_color_table_entry_size = int.from_bytes(
            rng.read(1), byteorder='big')
    local_color_table = [
            rng.read(local_color_table_entry_size)
                for _ in range(local_color_table_size)]
    for entry in local_color_table:
        out.write(entry)
    out.write(struct.pack('>HHBBBBB',
            left, top, image_width, image_height,
            *flags))

    # Image data
    image_data_length = int.from_bytes(
            rng.read(1), byteorder='big')
    image_data = rng.read(image_data_length)
    out.write(image_data)