#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int, height: int
    ):

    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack('<HH', width, height))

    # Global color table (0 colors)
    out.write('\x00\x00\x00'.encode())

    # Image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x18\x00\x00\x00\x00\xFF\xFF\xFF!\xF9\x04\x01\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x18\x00\x00\x00\x00\xFF\xFF\xFF\x21\xFE\x16\xEA\xCF\xB5\x19\x00;')

    # Local color table (0 colors)
    out.write('\x00\x00\x00'.encode())

    # Extension block for application extension containing "NETSCAPE" string
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Application extension block with subblock size 11 bytes
    out.write(b'\x00\x0B\x03\x01\x00\x00\x00\x00\x00\x00')

    # Sub-subblock with loop count of 1
    out.write(b'\x01')

    # Graphics control extension block
    out.write(b'\x21\xF9\x04\x01\x00\x00\x00\x00')

    # Trailer
    out.write(b';')