#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width = 1 << (int.from_bytes(rng.read(1), 'big') & 0x07)
    height = 2 ** ((int.from_bytes(rng.read(1), 'big') >> 3) + 1)
    flags = int.from_bytes(rng.read(1), 'big')
    bgcol = rng.read(1)[0]
    aspect = int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, aspect))

    # Global Color Table
    global_ct_size = int.from_bytes(rng.read(1), 'big')
    if global_ct_size > 0:
        for i in range(global_ct_size):
            out.write(rng.read(1))

    # Loop Control Block
    loop_count = int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>BBH', *rng.read(3)))

    # Image Descriptor
    left = int.from_bytes(rng.read(1), 'big')
    top = int.from_bytes(rng.read(1), 'big')
    width = 1 << (int.from_bytes(rng.read(1), 'big') & 0x07)
    height = 2 ** ((int.from_bytes(rng.read(1), 'big') >> 3) + 1)
    flags = int.from_bytes(rng.read(1), 'big')
    local_ct_size = int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>HHBBBB', left, top, width, height, flags, local_ct_size))

    # Local Color Table
    if local_ct_size > 0:
        for i in range(local_ct_size):
            out.write(rng.read(1))

    # Image Data
    image_data_len = int.from_bytes(rng.read(1), 'big')
    out.write(image_data_len.to_bytes(1, 'big'))
    out.write(rng.read(image_data_len))