#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << 8) | ((int.from_bytes(rng.read(1), 'big')) & 0xff)
    height = (1 << 8) | ((int.from_bytes(rng.read(1), 'big')) & 0xff)
    flags  = (1 << 7) | ((int.from_bytes(rng.read(1), 'big')) & 0x7f)
    bg     =   int.from_bytes(rng.read(1), 'big')
    aspect =   int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    for i in range(256):
        r =   int.from_bytes(rng.read(1), 'big')
        g =   int.from_bytes(rng.read(1), 'big')
        b =   int.from_bytes(rng.read(1), 'big')
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = (1 << 8) | ((int.from_bytes(rng.read(1), 'big')) & 0xff)
    top       = (1 << 8) | ((int.from_bytes(rng.read(1), 'big')) & 0xff)
    width     = (1 << 8) | ((int.from_bytes(rng.read(1), 'big')) & 0xff)
    height    = (1 << 8) | ((int.from_bytes(rng.read(1), 'big')) & 0xff)
    flags     = (1 << 7) | ((int.from_bytes(rng.read(1), 'big')) & 0x7f)
    localsize =         int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags, localsize))

    # Local color table
    for i in range(localsize):
        r =   int.from_bytes(rng.read(1), 'big')
        g =   int.from_bytes(rng.read(1), 'big')
        b =   int.from_bytes(rng.read(1), 'big')
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    datalen = int.from_bytes(rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)