#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xf7, 0x00, 0))

    # Global color table (black only)
    out.write('\xff\x00\x00\x00'.encode() * 256)

    # Image descriptor
    out.write(struct.pack('<BHHBB', 0x2c, 0, 0, 0, 0))

    # Local color table (black only)
    out.write('\xff\x00\x00\x00'.encode() * 256)

    # Image data
    for _ in range(1):
        out.write(b'\x00' * 4 * width * height)

    # Trailer
    out.write(b';')