#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (e.g., /dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int):

    # Write the header
    out.write('GIF89a'.encode())

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xf7, 0x0b))

    # Write the global color table
    out.write(b'\xff\xff\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the graphics control extension block
    out.write(b'\x21\xf9\x04\x05\x00\x00\x00\x00\x00\x2c')

    # Write the local color table
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the image descriptor
    out.write(struct.pack('<BHHHHBB',
            0x2c, 0, 0, width, height, 0, 0))

    # Write the image data
    for i in range((width * height)):
        out.write(rng.read(1))