#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << 6) | (int.from_bytes(rng.read(1), 'big') % 0x3f)
    height = (1 << 6) | (int.from_bytes(rng.read(1), 'big') % 0x3f)
    flags  = bytes([
        0x80 | ((int.from_bytes(rng.read(1), 'big') & 0x7) << 5),
        int.from_bytes(rng.read(1), 'big'),
    ])
    bgcol  = rng.read(1)
    pxsize = bytes([int.from_bytes(rng.read(1), 'big') & 0x7])
    out.write(struct.pack('>HHBBB', width, height, *flags))
    out.write(bgcol)
    out.write(pxsize)

    # Global color table
    numcolors = min((width+height)//2, 256)
    for i in range(numcolors):
        out.write(rng.read(3))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), 'big')
    top    = int.from_bytes(rng.read(2), 'big')
    width  = (1 << 6) | (int.from_bytes(rng.read(1), 'big') % 0x3f)
    height = (1 << 6) | (int.from_bytes(rng.read(1), 'big') % 0x3f)
    flags  = bytes([
        0x80 | ((int.from_bytes(rng.read(1), 'big') & 0x7) << 5),
        int.from_bytes(rng.read(1), 'big'),
    ])
    lctflag = bool(int.from_bytes(rng.read(1), 'big'))
    interlace = bool(int.from_bytes(rng.read(1), 'big'))
    sortflag = bool(int.from_bytes(rng.read(1), 'big'))
    reserved = rng.read(4)
    if not lctflag:
        raise NotImplementedError()
    elif lctflag and len(reserved)!= 0:
        raise NotImplementedError()
    else:
        pass
    out.write(struct.pack('>HHHHB', left, top, width, height, *flags))

    # Local color table
    if lctflag:
        for i in range(numcolors):
            out.write(rng.read(3))

    # Image data
    data = rng.read(width*height)
    out.write(data)