#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    width  = rng.randint(1024, 65536) // 16 * 16
    height = rng.randint(1024, 65536) // 16 * 16
    flags  = (1 << 7) | (1 << 5) | (1 << 4)
    bgcol  = rng.randint(0x00, 0xff)
    aspect = 0x00
    out.write(struct.pack('<HHBBB', width, height, flags, bgcol, aspect))

    # Global color table
    for i in range(0, 256):
        r = rng.randint(0x00, 0xff)
        g = rng.randint(0x00, 0xff)
        b = rng.randint(0x00, 0xff)
        out.write(struct.pack('<BBBB', r, g, b, 0x00))

    # Image descriptor
    left   = 0
    top    = 0
    width  = rng.randint(width - 1024, width)
    height = rng.randint(height - 1024, height)
    flags  = 0b00000000
    trans  = 0x00
    out.write(struct.pack('<HHHHB', left, top, width, height, flags))

    # Local color table
    for i in range(0, 256):
        r = rng.randint(0x00, 0xff)
        g = rng.randint(0x00, 0xff)
        b = rng.randint(0x00, 0xff)
        out.write(struct.pack('<BBBB', r, g, b, 0x00))

    # Image data
    for y in range(0, height):
        for x in range(0, width):
            r = rng.randint(0x00, 0xff)
            g = rng.randint(0x00, 0xff)
            b = rng.randint(0x00, 0xff)
            out.write(struct.pack('<BBBB', r, g, b, 0x00))