#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are given by `width`
# and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table (black and white only)
    out.write(b'\xFF\xFF\xFF\x00\x00\x00')

    # Image descriptor
    out.write(struct.pack('>HHBHBHHHBB',
        0, 0, 0xF7, 0x00, 0, 0, width, height, 0x08))

    # Local color table (black and white only)
    out.write(b'\xFF\xFF\xFF\x00\x00\x00')

    # Image data
    for _ in range((width * height)):
        out.write(rng.read(1))