#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (e.g., /dev/urandom).
# The generated GIF will be of size `width` x `height`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ):

    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xf7, 0x0b))

    # Global color table
    out.write('\x01\xff\x00\x00\x00'.encode())

    # Image descriptor
    out.write(struct.pack('<HHHHB', 0, 0, width - 1, height - 1, 0))

    # Local color table
    out.write('\x01\xff\x00\x00\x00'.encode())

    # Image data
    for _ in range((width * height) // 256):
        out.write(rng.read(256))