#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is determined by
# `width`, `height`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table (black and white only)
    out.write(b'\xFF\xFF\xFF\x00\x00\x00')

    # Image descriptor
    out.write(struct.pack('>BHHHHBB', 0x2C, 0, 0, width, height, 0, 0))

    # Local color table (black and white only)
    out.write(b'\xFF\xFF\xFF\x00\x00\x00')

    # Image data
    for _ in range(height):
        for i in range((width+15)//16):
            out.write(rng.read(16 if ((i+1)*16 <= width) else width-(i*16)))

    # Trailer
    out.write(b';')