#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# the parameters `width`, `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, *,
        width=1024, height=768
    ):

    # Generate the header
    header = b'GIF89a'
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xF0, 0x0, 0))

    # Write the global color table
    for i in range(0, 256):
        out.write(struct.pack('BBB', *rng.read(3)))

    # Write the graphics control extension
    out.write(b'\x21\xF9\x04\x05\x00\x00\x00\x00\x00\x00')

    # Write the image descriptor
    out.write(struct.pack('<BHHHHBB',
            0x2C, 0, 0, width, height, 0, 0))

    # Write the local color table
    for i in range(0, 256):
        out.write(struct.pack('BBB', *rng.read(3)))

    # Write the image data
    while True:

        # Read the next block from /dev/urandom
        chunk = rng.read(1024)

        if len(chunk) == 0: break

        # Write the length of this block to the output stream
        out.write(struct.pack('B', len(chunk)))

        # Write the actual data
        out.write(chunk)