#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are given by `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xf7, 0x00, 0))

    # Global color table (black only)
    out.write(b'\xff\x00\x00\x00')

    # Image descriptor
    out.write(struct.pack('>HHBBHHH', 0, 0, 0xf7, 0x00, 0, 0, 0))

    # Local color table (black only)
    out.write(b'\xff\x00\x00\x00')

    # Image data
    for _ in range((width * height)):
        out.write(b'\x00')