#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 256
    height = (1 << rng.read(1)[0]) % 256
    flags  = rng.read(1)[0] & 7
    bg     = rng.read(1)[0]
    aspect = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    numcolors = (1 << ((flags >> 4) & 7)) * 3
    for i in range(numcolors):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = rng.read(2)
    top    = rng.read(2)
    width  = (1 << rng.read(1)[0]) % 256
    height = (1 << rng.read(1)[0]) % 256
    flags  = rng.read(1)[0] & 7
    local_color_table = bool((flags >> 7) & 1)
    interlace         = bool((flags >> 6) & 1)
    sort              = bool((flags >> 5) & 1)
    reserved          = rng.read(1)[0]
    lctsize           = (1 << ((flags >> 4) & 7)) * 3
    out.write(left+top+width+height+flags.to_bytes(1,'little'))

    if local_color_table:
        for i in range(lctsize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(struct.pack('BBB', r, g, b))

    # LZW min code size
    out.write(rng.read(1))

    # Data sub-block
    while True:
        blocklen = rng.read(1)[0]

        if not blocklen:
            break

        out.write(rng.read(blocklen))

