#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    header = b'GIF89a'

    # Data length
    datalen = 0x1000
    while True:
        if datalen > 0xFFFF: break

        # Generate some data
        data = rng.read(datalen)

        # Check for repeating bytes
        if len(set(data)) == 1: continue

        # Write to output and exit loop
        out.write(header + struct.pack('>H', datalen))
        out.write(data)
        return

