#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (e.g., /dev/urandom).
# The generated GIF is 256x256 pixels with 1-bit color depth.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB',
        0x00FF,       # Width
        0x00FF,       # Height
        0x08,         # Color resolution
        0x01,         # Global color table flag
        0x00))        # Background color index

    # Global color table
    for i in range(256):
        r = ord(rng.read(1)) & 0xF8
        g = ord(rng.read(1)) & 0xFC
        b = ord(rng.read(1)) & 0xF8

        out.write(struct.pack('>BBBB',
            0xFF,   # R
            0xFF,   # G
            0xFF,   # B
            0x00))  # Reserved

    # Image descriptor
    out.write(struct.pack('>HHHHBB',
        0x0000,    # Left position
        0x0000,    # Top position
        0x00FF,    # Width
        0x00FF,    # Height
        0x08,      # Local color table flag
        0x00))     # Interlace flag and sort order

    # LZW minimum code size
    out.write(b'\x02')

    # Pixels
    while True:
        blocksize = ord(rng.read(1))
        if not blocksize: break
        out.write(blocksize.to_bytes(1, 'little'))
        out.write(rng.read(blocksize))