#!/usr/bin/env python3

import sys
import os
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string "GIF89a" to the output stream
    out.write(b'GIF89a')

    # Generate a 16-bit integer from the input stream and write it as big endian
    # to the output stream
    datalen = int.from_bytes(
            rng.read(2), byteorder='big', signed=False)
    out.write(datalen.to_bytes(2, byteorder='big'))

    # Read in the rest of the image data
    data = rng.read(datalen)

    # Write the image data to the output stream
    out.write(data)


if __name__ == '__main__':
    if len(sys.argv)!= 2:
        print('Usage: {} <output>'.format(os.path.basename(__file__)))
        exit(-1)

    with open('/dev/urandom', 'rb') as rng, \
         open(sys.argv[1], 'wb') as out:

        generate_random_gif(rng, out)