#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is determined by
# `width`, `height`.
def generate_random_gif(
        width: int, height: int,
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table (black and white only)
    out.write(b'\xFF\xFF\xFF\x00\x00\x00')

    # Image descriptor
    out.write(struct.pack('>HHBBHHH', 0, 0, 0xF7, 0x00, 0, 0, 0))

    # Local color table (black and white only)
    out.write(b'\xFF\xFF\xFF\x00\x00\x00')

    # Pixels
    for _ in range(width * height):
        out.write(rng.read(1))