#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (e.g., /dev/urandom).
# The size of the generated image is determined by
# `width`, `height`.
def generate_random_gif(
        width: int, height: int, rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xf7, 0x00))

    # Global color table
    for _ in range(256):
        out.write(struct.pack('BBBB', *rng.read(4)))

    # Image descriptor
    out.write(struct.pack('>HHHHB', 0, 0, width, height, 0))

    # LZW minimum code size
    out.write(b'\x1f')

    # Data sub-block
    while True:
        chunklen = rng.read(1)[0]

        if not chunklen: break

        out.write(chunklen.to_bytes(1, 'big'))
        out.write(rng.read(chunklen))