#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The first 1024 bytes of the output are used to store
# the palette (which is generated randomly as well).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Generate a random palette and write it in the beginning
    # of the output stream.
    pal = [struct.pack('BBB', *rng.read(3)) for _ in range(256)]
    out.write(b''.join(pal))

    # Write the rest of the file
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)