#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of each frame is between 1 and max_frame_size bytes long.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, max_frame_size=2048
    ):
    # Write the header
    header = b'GIF89a'
    out.write(header)

    # Generate the global color table
    num_colors = min(int.from_bytes(rng.read(1), 'big'), 256)
    palette = [struct.pack('BBB', *rng.read(3)) for _ in range(num_colors)]
    out.write(b''.join(palette))

    # Generate frames until we reach EOF
    while True:
        try:
            # Get the next frame's size (in bytes)
            framelen = int.from_bytes(
                    rng.read(3), byteorder='big')

            if framelen > max_frame_size:
                continue

            # Read the frame's data
            data = rng.read(framelen)

        except OSError as e:
            break

        # Write the frame to the output stream
        out.write(struct.pack('<H', len(data)))
        out.write(data)