#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the generated image are
# specified by `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):
    # Write the header
    out.write(b'GIF89a')

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB',
            0, 0, 72, 0, 0))

    # Write the global color table (black & white only)
    out.write(b'\x00\x00\x00\xff\x00\x00\x00\xff')

    # Write the graphics control extension
    out.write(struct.pack('<BHB',
            0xf9, 4, 1 << 5 | 1 << 6))

    # Write the image descriptor
    out.write(struct.pack('<BBHHHBB',
            0x2c, 0, 0, width, height, 0, 0))

    # Write the local color table
    out.write(b'\x00\x00\x00\xff\x00\x00\x00\xff')

    # Write the LZW minimum code size
    out.write(b'\x02')

    # Generate the pixel data
    for y in range(height):
        for x in range(width):
            out.write(b'\x0f' if rng.read(1)[0] > 127 else b'\x00')

    # Write the terminator
    out.write(b';')