#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The generated GIF will have a width of 1024 pixels and height of 768 pixels.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>H', 1024))
    out.write(struct.pack('>H', 768))
    out.write(b'\x87\x00')

    # Global color table (black & white only)
    for i in range(0, 256):
        if i % 2 == 0:
            out.write(b'\x00\x00\x00')
        else:
            out.write(b'\xff\xff\xff')

    # Image descriptor
    out.write(b'\x2c')
    out.write(struct.pack('>H', 1024))
    out.write(struct.pack('>H', 768))
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')

    # Local color table (black & white only)
    for i in range(0, 256):
        if i % 2 == 0:
            out.write(b'\x00\x00\x00')
        else:
            out.write(b'\xff\xff\xff')

    # Image data
    for _ in range(0, 768 * 1024 // 8):
        out.write(rng.read(1))