#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are given by `w` and `h`.
def generate_random_gif(
        rng: BinaryIO, w: int, h: int, out: BinaryIO
    ) -> None:

    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', w, h, 0xf7, 0x0b))

    # Global color table (black and white only)
    out.write(b'\xff\xff\xff\x00\x00\x00')

    # Image descriptor
    out.write(struct.pack('<BHHHHBB', 0x2c, 0, 0, w - 1, h - 1, 0, 0))

    # Local color table (black and white only)
    out.write(b'\xff\xff\xff\x00\x00\x00')

    # Image data
    for _ in range((w * h) // 64):
        out.write(b'\x00' * 64)

    # Trailer
    out.write(b';')