#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (e.g., /dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Write the header for an uncompressed image with
    # 256 colors in the global color table.
    out.write(b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\xf0\x00')
    out.write((width * height).to_bytes(2, 'little'))
    out.write(b'\x00\x00\x00\xff\xff\xff\x2c\x00\x00\x00\x00\x01\x00\x01\x00')
    out.write(((width - 1) | ((height - 1) << 4)).to_bytes(1, 'little'))
    out.write(b'\x00')

    # Write the palette entries.
    for i in range(256):
        r = rng.read(1)[0] & ~0xe0
        g = rng.read(1)[0] & ~0xe0
        b = rng.read(1)[0] & ~0xc0
        out.write(struct.pack('BBB', r, g, b))

    # Write the image descriptor.
    out.write((((width - 1) | ((height - 1) << 4)) | 0x80).to_bytes(1, 'little'))
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\