#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (or /dev/urandom if None).
def generate_random_gif(
        rng: BinaryIO=None, out: BinaryIO=sys.stdout.buffer
    ) -> None:

    # If no RNG is provided, use /dev/urandom instead.
    if not rng:
        rng = open('/dev/urandom', 'rb')

    header = b'GIF89a'
    datalen = 0x12345678 & ((1 << 24) - 1)
    data = rng.read(datalen)
    out.write(header + struct.pack('>I', datalen))
    out.write(data)
