#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    height = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    flags  = rng.read(1)[0] & 0x0C | 0x02 | 0x01
    bgcolr = rng.read(1)[0]
    aspect = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags, bgcolr, aspect))

    # Global color table
    gctsize = rng.read(1)[0] & 0x07 | 0x07
    out.write(gctsize.to_bytes(1, 'big'))
    for i in range(gctsize):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    top    = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    width  = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    height = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    flags  = rng.read(1)[0] & 0x0C | 0x02 | 0x01
    lctsize= rng.read(1)[0] & 0x07 | 0x07
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags, lctsize))

    # Local color table
    if lctsize > 0:
        for i in range(lctsize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(struct.pack('BBB', r, g, b))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)