#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 500 + 100
    height = (1 << rng.read(1)[0]) % 500 + 100
    flags  = bytes([
        0x80 | ((rng.read(1)[0] & 0x70) >> 4),
        rng.read(1)[0] & 0x07,
    ])
    bgcolor    = rng.read(1)[0]
    pixelaspect = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags[0], flags[1], bgcolor))

    # Global color table
    ncolors = min((width * height) // 256, 256)
    for i in range(ncolors):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(bytes([r,g,b]))

    # Image descriptor
    left   = 0
    top    = 0
    width  = (1 << rng.read(1)[0]) % 500 + 100
    height = (1 << rng.read(1)[0]) % 500 + 100
    flags  = bytes([
        0x80 | ((rng.read(1)[0] & 0x70) >> 4),
        rng.read(1)[0] & 0x07,
    ])
    duration = rng.read(1)[0]
    transidx = rng.read(1)[0] if (flags[0] & 0x01) else -1
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags[0], flags[1]))

    # LZW minimum code size
    lzwmincodesize = rng.read(1)[0]
    out.write(bytes([lzwmincodesize]))

    # Data sub-block
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)