#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the generated image are given by `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int, height: int):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0x87, 0, 0))

    # Global color table (128 colors)
    for i in range(128):
        r = rng.read(1)[0] & 0xf8 | ((rng.read(1)[0] >> 5) & 0x7)
        g = rng.read(1)[0] & 0xf8 | ((rng.read(1)[0] >> 5) & 0x7)
        b = rng.read(1)[0] & 0xf8 | ((rng.read(1)[0] >> 5) & 0x7)

        out.write(struct.pack('<BBBB', r, g, b, 0xff))

    # Image descriptor
    out.write(struct.pack('>BHHHHBB', 0x2c, 0, 0, width, height, 0, 0))

    # Local color table (no local color table)
    out.write(b'\x00\x00')

    # Image data
    for y in range(height):
        for x in range(width):

            # Get next pixel from /dev/urandom
            c = rng.read(1)[0] & 0x7f

            # Write it to the output stream
            out.write(struct.pack('<B', c))