#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (defaulting to /dev/urandom).
def generate_random_gif(
        rng: BinaryIO=None, out: BinaryIO=sys.stdout.buffer
    ) -> None:

    # If no RNG is provided, use /dev/urandom
    if not rng:
        rng = open('/dev/urandom', 'rb')

    # Generate the header and write it to stdout
    header = b'GIF89a'
    datalen = 1024 * 16
    data = rng.read(datalen)
    out.write(header + len(data).to_bytes(3, byteorder='big'))
    out.write(data)

