#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    width  = struct.pack('<H', 1024)
    height = struct.pack('<H',  768)
    flags  = bytes([
        0b00000001, # Global color table flag
        0b00000000]) # Color resolution
    bg      = bytes([0x00, 0x00, 0x00])
    aspect  = bytes([0x00, 0x00])
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = rng.read(1)[0] & 0x3F
        g = rng.read(1)[0] & 0x3F
        b = rng.read(1)[0] & 0x3F
        out.write(struct.pack('<BBB', r, g, b))

    # Image descriptor
    left    = struct.pack('<h',    0)
    top     = struct.pack('<h',    0)
    width   = struct.pack('<H', 1024)
    height  = struct.pack('<H',  768)
    flags   = bytes([
        0b00000000, # Local color table flag
        0b00000000, # Interlace flag
        0b00000000, # Sort flag
        0b00000100, # Reserved
        0b00000000]) # Size of local color table
    bg      = bytes([0x00, 0x00, 0x00])
    aspect  = bytes([0x00, 0x00])
    out.write(left + top + width + height + flags + bg + aspect)

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)