#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(10, 256)
    height = rng.randint(10, 256)
    flags  = (
            0x80 | # Global color table flag
            7      # Color resolution bits-per-pixel
            )
    bgcol  = 0
    aspect = 0
    out.write(struct.pack('>HHBBB',
            width, height, flags, bgcol, aspect))

    # Global color table
    for i in range(0, 2**flags):
        r = rng.randint(0, 256)
        g = rng.randint(0, 256)
        b = rng.randint(0, 256)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = 0
    top    = 0
    width  = rng.randint(10, 256)
    height = rng.randint(10, 256)
    flags  = 0x00
    out.write(struct.pack('>HHHHB',
            left, top, width, height, flags))

    # Local color table
    if flags & 0x80 == 0x80:
        for i in range(0, 2**flags):
            r = rng.randint(0, 256)
            g = rng.randint(0, 256)
            b = rng.randint(0, 256)
            out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B',
                    rng.randint(0, 256)))