#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width  = (1 << rng.read(1)[0]) % 256
    height = (1 << rng.read(1)[0]) % 256
    flags  = rng.read(1)[0]
    bgcol  = rng.read(1)[0]
    pxsize = rng.read(1)[0] & 7
    colres = ((flags >> 4) & 7) + 1
    sort   = bool((flags >> 3) & 1)
    interlaced = bool((flags >> 2) & 1)
    globalcolmap = bool((flags >> 1) & 1)
    userinput    = bool(flags & 1)
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, pxsize))

    # Global Color Table
    if globalcolmap and colres > 1:
        for i in range(3 * pow(2, colres)):
            out.write(rng.read(1))

    # Loop Control Block
    loopcount = rng.read(1)[0] | (rng.read(1)[0] << 8)
    out.write(struct.pack('BB', loopcount, 0))

    # Image Descriptor
    left   = rng.read(1)[0] | (rng.read(1)[0] << 8)
    top    = rng.read(1)[0] | (rng.read(1)[0] << 8)
    width  = (1 << rng.read(1)[0]) % 256
    height = (1 << rng.read(1)[0]) % 256
    flags  = rng.read(1)[0]
    localcolmap = bool((flags >> 7) & 1)
    interlace   = bool((flags >> 6) & 1)
    sort        = bool((flags >> 5) & 1)
    reserved    = rng.read(1)[0]
    lctsize     = (1 << (flags & 7))
    out.write(struct.pack('>HHBBBB', left, top, width, height, flags, lctsize))

    # Local Color Table
    if localcolmap and lctsize > 0:
        for i in range(3 * lctsize):
            out.write(rng.read(1))

    # Image Data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(pxsize // 8))