#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int):

    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xf7, 0x0b))

    # Global color table
    for i in range(256):
        out.write((i, i, i).to_bytes(1, 'little'))

    # Image descriptor
    out.write(struct.pack('<BHHBBBBBHB',
            0x2c, 0, 0, 0, width, height, 0, 0, 0, 0xff))

    # Image data
    datalen = (width * height // 4) - 1
    data = rng.read(datalen)
    out.write(data)

    # Trailer
    out.write('\x3b'.encode())