#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`, respectively.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00))

    # Global color table (no colors used in this example)
    out.write(b'\x00\x00\x00')

    # Image descriptor
    out.write(struct.pack('>HHBBBBB', 0, 0, 0, 0, 0, 0))

    # Local color table (no colors used in this example)
    out.write(b'\x00\x00\x00')

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)