#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the image is determined by `width`, and
# the color depth by `depth`.
def generate_random_gif(
        rng: BinaryIO, width: int, depth: int, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', 0, 0, width - 1, height - 1, depth))

    # Global color table (black & white)
    if depth == 1:
        out.write(b'\x00\xff\xff\xff\x00\x00\x00')

    # Image descriptor
    out.write(struct.pack('<BBHHHBB',
            0x2c, 0, 0, width - 1, height - 1, depth, 0))

    # Local color table
    if depth > 1:
        for i in range(2 ** depth):
            out.write(struct.pack('BBB', *rng.read(3)))

    # Image data
    while True:
        chunklen = min(rng.read(1)[0], 255)

        if not chunklen: break

        out.write(chunklen.to_bytes(1, 'big'))
        out.write(rng.read(chunklen))