#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The dimensions of the image are determined by `width`, and `height`.
# If either is 0, then it will be randomly generated between 16 and 256 inclusive.
def generate_random_gif(
        rng: BinaryIO, width: int=0, height: int=0, out: BinaryIO
    ):

    if not (width or height):
        # Generate a random dimension between 16 and 256 inclusive
        width = rng.randint(16, 256)
        height = rng.randint(16, 256)

    # Write the header to the output stream
    out.write(b'GIF89a')
    # Write the logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x0B))

    # Write the global color table
    for i in range(0, 256*3, 3):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)

        out.write(struct.pack('BBB', r, g, b))

    # Write the local color table
    out.write(b'\xFF\xFE')

    # Write the graphics control extension
    out.write(b'\x21\xF9\x04\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the image descriptor
    out.write(struct.pack('>HHHHBB', 0, 0, width - 1, height - 1, 0, 0))

    # Write the LZW code size
    out.write(b'\x08')

    # Write the image data
    while True:
        # Get a random length from 2 to 255 inclusive
        lzwlen = rng.randint(2, 255)
        # Read that many bytes from /dev/urandom
        buf = rng.read(lzwlen)
        # If we read less than expected, pad with nulls
        buf += b'\0' * (lzwlen - len(buf))
        # Write them to the output stream
        out.write(buf)
        # Check if this was the last block
        if buf[-1] == 0: break