#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = rng.read(1)[0]
    bgcolr = rng.read(1)[0] | ((1 << rng.read(1)[0]) & 0xFF)
    pxsize = rng.read(1)[0]
    gctlen = rng.read(1)[0] if flags & 0x80 else 0
    out.write(struct.pack('<HHBBB', width, height, flags, bgcolr, pxsize))

    # Global Color Table
    for _ in range(gctlen):
        colr = rng.read(1)[0] | ((1 << rng.read(1)[0]) & 0xFF)
        out.write(struct.pack('B', colr))

    # Image Descriptor
    left   = rng.read(2)[0]
    top    = rng.read(2)[0]
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = rng.read(1)[0]
    lctlen = rng.read(1)[0] if flags & 0x80 else 0
    interv = rng.read(1)[0]
    transd = rng.read(1)[0] if flags & 0x40 else None
    out.write(struct.pack('<HHBBBBBB', left, top, width, height, flags, lctlen, interv))

    # Local Color Table
    for _ in range(lctlen):
        colr = rng.read(1)[0] | ((1 << rng.read(1)[0]) & 0xFF)
        out.write(struct.pack('B', colr))

    # Data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)