#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 640
    height = (1 << rng.read(1)[0]) % 480
    flags  = rng.read(1)[0] & 7
    bgcol  = rng.read(1)[0]
    aspect = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, aspect))

    # Global color table
    ncolors = min((2 ** ((flags >> 3) + 1)), 256)
    for i in range(ncolors):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = rng.read(2)
    top    = rng.read(2)
    width  = (1 << rng.read(1)[0]) % 640
    height = (1 << rng.read(1)[0]) % 480
    flags  = rng.read(1)[0] & 7
    out.write(left + top + bytes([width, height]))
    out.write(bytes([flags]))

    # Local color table
    if flags & 0x80!= 0:
        ncolors = min((2 ** ((flags >> 3) + 1)), 256)
        for i in range(ncolors):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(struct.pack('BBB', r, g, b))

    # LZW minimum code size
    lzwminsize = rng.read(1)[0]
    out.write(bytes([lzwminsize]))

    # Data sub-block
    while True:
        blocklen = rng.read(1)[0]

        if blocklen == 0: break

        out.write(bytes([blocklen]))
        out.write(rng.read(blocklen))