#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is determined by
# `width`, `height`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table (black and white only)
    for i in range(256):
        if ((i & 1) == 0):
            out.write(b'\x00\x00\xff')
        else:
            out.write(b'\xff\x00\x00')

    # Image descriptor
    out.write(struct.pack('>HHHHB', 0, 0, width - 1, height - 1, 0x00))

    # LZW minimum code size
    out.write(b'\x02')

    # Data sub-block
    out.write(b'\x04')

    # Pixels
    while True:

        # Read next pixel from /dev/urandom
        r = ord(rng.read(1)) % 256

        # Write it to output stream
        out.write(bytes([r]))

        # If we've reached the end of this row...
        if (width <= len(data)):

            #... write an EOL marker
            out.write(b'\x00')

            #... and break
            break