#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1024
    height = 768
    color_table_size = 256
    global_color_table = True
    sort_flag = False
    interlace_flag = False
    local_color_table_flag = False
    background_color_index = 0
    pixel_aspect_ratio = 0
    logical_screen_descriptor = (struct.pack('>HHBBB',
        width, height,
        color_table_size,
        0 if global_color_table else 1 << 7 |
            0 if sort_flag else 1 << 6,
            0 if interlace_flag else 1 << 5,
            0 if local_color_table_flag else 1 << 7))
    out.write(logical_screen_descriptor)

    # Global color table
    for i in range(color_table_size):
        r = rng.read(1)[0] & 0x3f
        g = rng.read(1)[0] & 0x3f
        b = rng.read(1)[0] & 0x3f
        out.write((r << 2 | g >> 4).to_bytes(1, 'little'))
        out.write(((g & 0xf) << 4 | b >> 2).to_bytes(1, 'little'))
        out.write((b & 0x3) << 6 | 0x3f).to_bytes(1, 'little')

    # Image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table_flag = False
    interlace_flag = False
    sort_flag = False
    reserved = 0
    local_color_table_size = 256
    image_descriptor = (struct.pack('>HHHHBB',
        left, top,
        width, height,
        0 if local_color_table_flag else 1 << 7 |
            0 if interlace_flag else 1 << 6 |
                0 if sort_flag else 1 << 5,
                reserved, local_color_table_size))
    out.write(image_descriptor)

    # Local color table
    for i in range(local_color_table_size):
        r = rng.read(1)[0] & 0x3f
        g = rng.read(1)[0] & 0x3f
        b = rng.read(1)[0] & 0x3f
        out.write((r << 2 | g >> 4).to_bytes(1, 'little'))
        out.write(((g & 0xf) << 4 | b >> 2).to_bytes(1, 'little'))
        out.write((b & 0x3) << 6 | 0x3f).to_bytes(1, 'little')

    # Data sub-block
    block_size = 2 ** 12 - 1
    while len(data := rng.read(block_size)):
        out.write(len(data).to_bytes(1, 'little'))
        out.write(data)