#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(1024, 65536)
    height = rng.randint(1024, 65536)
    flags  = (
            0x80 | # Global color table flag
            7       # Color resolution
        ) << 4
    bgcol  = rng.randint(0, 255)
    aspect = 0
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, aspect))

    # Global color table
    for i in range(0, 2**flags & ~0x80):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = 0
    top    = 0
    width  = rng.randint(width // 2, width)
    height = rng.randint(height // 2, height)
    flags  = 0x80 | 0x40 | 0x20 # Local color table flag and interlace flag
    lctlen = 2**(flags & 7)
    out.write(struct.pack('>HHHHB', left, top, width, height, flags))

    # Local color table
    for i in range(lctlen):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(struct.pack('BBB', r, g, b))

    # Data sub-block
    blocksize = 2**((flags >> 4) & 7)
    while True:
        if blocksize == 2**((flags >> 4) & 7):
            blocksize += 1

        if blocksize > len(data):
            break

        out.write(data[:blocksize])
        del data[:blocksize]