#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are given by `width`
# and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xf7, 0x00, 0))

    # Global color table (black and white only)
    for i in range(2):
        out.write(struct.pack('BBBB', *([0] * 4)))

    # Image descriptor
    out.write(struct.pack('>BBHHHBB', 0x2c, 0, 0, 0, 0, 0, 0))

    # Local color table (black and white only)
    for i in range(2):
        out.write(struct.pack('BBBB', *([0] * 4)))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)