#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`, respectively.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table (black only)
    out.write(b'\x00\x00\x00\xFF')

    # Image descriptor
    out.write(struct.pack('>HHBHHHBB',
        0, 0, 0xF7, width, height, 0, 0x00))

    # Local color table (black only)
    out.write(b'\x00\x00\x00\xFF')

    # Extension block for graphics control
    out.write(b'\x21\xF9\x04\x05\x00\x0A\x00\x00')

    # Graphics control extension block
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00')

    # Trailer
    out.write(b';')