#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (e.g., /dev/urandom).
# The width and height of the image are given as parameters.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, w: int, h: int):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', w, h, 0xf7, 0x00, 0))

    # Global color table
    for i in range(256):
        r = rng.read(1)[0] & 0xff
        g = rng.read(1)[0] & 0xff
        b = rng.read(1)[0] & 0xff
        out.write(struct.pack('<BBBB', r, g, b, 0))

    # Image descriptor
    out.write(struct.pack('>HHHHB', 0, 0, w - 1, h - 1, 0))

    # Local color table
    for i in range(256):
        r = rng.read(1)[0] & 0xff
        g = rng.read(1)[0] & 0xff
        b = rng.read(1)[0] & 0xff
        out.write(struct.pack('<BBBB', r, g, b, 0))

    # Image data
    for y in range(h):
        for x in range(w):
            out.write(rng.read(1))