#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(1024, 65536)
    height = rng.randint(1024, 65536)
    flags  = (
            0x80 | # Global color table flag
            7      # Color resolution - 7 bits per pixel
        ) << 8
    flags |= (
            0x07 | # Sorting type - not interlaced
            0x00   # GCT size - no global color table
        )
    out.write(struct.pack('<HHBBB', width, height, flags, 0, 0))

    # Global color table
    for _ in range(0, 2**flags & 0xff):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left    = 0
    top     = 0
    width   = rng.randint(width // 2, width * 2)
    height  = rng.randint(height // 2, height * 2)
    flags   = 0x00
    out.write(struct.pack('<HHHHB', left, top, width, height, flags))

    # Local color table
    if flags & 0x80 == 0x80:
        for _ in range(0, 2**((flags >> 8) & 0xff)):
            r = rng.randint(0, 255)
            g = rng.randint(0, 255)
            b = rng.randint(0, 255)
            out.write(struct.pack('BBB', r, g, b))

    # Image data
    datalen = rng.randint(0, 65536)
    data = rng.read(datalen)
    out.write(data)