#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the image is determined by `width`, and
# `height`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xf7, 0x00, 0))

    # Global color table (black only)
    out.write(b'\xff\xff\xff')

    # Image descriptor
    out.write(struct.pack('>BHHHHBB',
            0x2c, 0, 0, width, height, 0, 0xf7))

    # Local color table (black only)
    out.write(b'\xff\xff\xff')

    # Image data
    for _ in range(height):
        out.write(b'\x00' * width)