#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the image is determined by the first 24 bits
# of the generated bytes (i.e., 16777215 pixels max).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header and write it to the output stream.
    header = b'GIF89a'
    out.write(header)

    # Read in the length of the image from the first 3 bytes.
    imglen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write this value as an unsigned integer big-endian
    # into the output stream.
    out.write(imglen.to_bytes(3, 'big'))

    # Now we can start writing the actual image data.
    for _ in range(0, imglen):
        # Each pixel consists of three bytes.
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]

        # Write each color channel as one byte.
        out.write(struct.pack('BBB', r, g, b))