#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Generate the header and write it to the output stream
    header = b'GIF89a'
    out.write(header)

    # Determine how many bytes of data we need for the image
    datalen = 1024 * 64
    if len(sys.argv) > 1:
        try:
            datalen = int(sys.argv[1])
        except ValueError as e:
            print('Invalid argument:', str(e))
            return
    
    # Write the length of the data block to the output stream
    out.write(struct.pack('<H', datalen))

    # Read in the data from the RNG and write it to the output stream
    data = rng.read(datalen)
    out.write(data)

