#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width = 1024
    height = 768
    color_depth = 8
    global_color_table = True
    sort_flag = False
    size = (width * height) // (2 ** color_depth)
    packed = [
            0x80 | ((global_color_table << 7) & 0x80),
            (sort_flag << 5) | (size & 0x1F),
            (height >> 8) & 0xFF,
            height & 0xFF,
            (width >> 8) & 0xFF,
            width & 0xFF,
            (color_depth - 1) & 0x07,
            0]
    out.write(struct.pack('B'*len(packed), *packed))

    # Global Color Table
    if global_color_table:
        for i in range(int(2**(color_depth+1))):
            r = rng.read(1)[0] % 256
            g = rng.read(1)[0] % 256
            b = rng.read(1)[0] % 256
            out.write(struct.pack('<BBB', r, g, b))

    # Image Descriptor
    packed = [
            0x2C,
            0, 0,
            (height >> 8) & 0xFF,
            height & 0xFF,
            (width >> 8) & 0xFF,
            width & 0xFF,
            0,
            (color_depth - 1) & 0x07,
            0]
    out.write(struct.pack('B'*len(packed), *packed))

    # Local Color Table
    if not global_color_table:
        for i in range(int(2**(color_depth+1))):
            r = rng.read(1)[0] % 256
            g = rng.read(1)[0] % 256
            b = rng.read(1)[0] % 256
            out.write(struct.pack('<BBB', r, g, b))

    # Image Data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', rng.read(1)[0]))