#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is 256x256 pixels.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 256
    height = 256
    color_depth = 8
    global_color_table_flag = False
    background_color_index = 0
    pixel_aspect_ratio = 0
    logical_screen_descriptor = (struct.pack('>HHBBB',
        width, height, color_depth,
        global_color_table_flag,
        background_color_index,
        pixel_aspect_ratio))
    out.write(logical_screen_descriptor)

    # Global color table
    global_color_table = bytes([
        0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
        0x00, 0xFF, 0x00, 0x00])
    out.write(global_color_table)

    # Image descriptor
    left = 0
    top = 0
    width = 256
    height = 256
    local_color_table_flag = True
    interlace_flag = False
    sort_flag = False
    reserved = 0
    lct_size = 7
    local_color_table = bytes([
        0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00])
    image_descriptor = (struct.pack('>HHHHBB',
        left, top, width, height,
        local_color_table_flag,
        interlace_flag,
        sort_flag,
        lct_size))
    out.write(image_descriptor)

    # Local color table
    out.write(local_color_table)

    # Pixels
    for _ in range(height):
        line = []

        for _ in range(width // 8):
            byte = rng.read(1)[0]

            for bit in [
                0b10000000 & byte,
                0b01000000 & byte,
                0b00100000 & byte,
                0b00010000 & byte,
                0b00001000 & byte,
                0b00000100 & byte,
                0b00000010 & byte,
                0b00000001 & byte]:
                if bit == 0:
                    line += [0x00] * 3
                else:
                    line += [0xFF] * 3

        out.write(bytes(line))