#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are given by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0))

    # Global color table (black & white)
    for i in range(256):
        if i < 128:
            out.write(struct.pack('B', i * 2))
        else:
            out.write(struct.pack('B', 0))

    # Image descriptor
    out.write(struct.pack('BBHBBBB', 0x2C, 0, 0, 0, 0, 0, 0))

    # Local color table (black & white)
    for i in range(256):
        if i < 128:
            out.write(struct.pack('B', i * 2))
        else:
            out.write(struct.pack('B', 0))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)