#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 256
    height = (1 << rng.read(1)[0]) % 256
    flags  = rng.read(1)[0] & 7
    bg     = rng.read(1)[0]
    pxsize = rng.read(1)[0] & 7
    color  = ((flags >> 4) & 1) | (((flags >> 3) & 1) << 1) | \
             (((flags >> 2) & 1) << 2) | (((flags >> 1) & 1) << 3) | \
             (((flags >> 0) & 1) << 4)
    sort   = (flags >> 7) & 1
    interlaced = (flags >> 6) & 1
    out.write(struct.pack('>HHBBB', width, height,
                         pxsize, color, sort))

    # Global color table
    for i in range(pow(2, pxsize)):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(r.to_bytes(1, 'little'))
        out.write(g.to_bytes(1, 'little'))
        out.write(b.to_bytes(1, 'little'))

    # Image descriptor
    left    = 0
    top     = 0
    width   = (1 << rng.read(1)[0]) % 256
    height  = (1 << rng.read(1)[0]) % 256
    flags   = rng.read(1)[0] & 7
    localct = (flags >> 7) & 1
    interlaced = (flags >> 6) & 1
    sorted   = (flags >> 5) & 1
    reserved = (flags >> 3) & 3
    lctsize  = pow(2, (flags & 7))
    out.write(struct.pack('>HHHHBB', left, top,
                         width, height,
                         localct, interlaced))

    if not localct:
        for i in range(lctsize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(r.to_bytes(1, 'little'))
            out.write(g.to_bytes(1, 'little'))
            out.write(b.to_bytes(1, 'little'))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)