#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 2**int.from_bytes(
            rng.read(1), byteorder='big') % (2**10-1)+1
    height = 2**int.from_bytes(
            rng.read(1), byteorder='big') % (2**10-1)+1
    flags = bytes([int.from_bytes(
            rng.read(1), byteorder='big')])
    background = bytes([int.from_bytes(
            rng.read(1), byteorder='big')])
    aspect = bytes([int.from_bytes(
            rng.read(1), byteorder='big')])
    out.write(struct.pack('>HHBBB', width, height, *flags))
    out.write(background+aspect)

    # Global color table
    global_color_table_size = int.from_bytes(
            rng.read(1), byteorder='big') & 7
    global_color_table = [None] * (global_color_table_size ** 2)
    for i in range(len(global_color_table)):
        r = int.from_bytes(
                rng.read(1), byteorder='big') & 255
        g = int.from_bytes(
                rng.read(1), byteorder='big') & 255
        b = int.from_bytes(
                rng.read(1), byteorder='big') & 255
        global_color_table[i] = bytes([r,g,b])
    for i in range((2**(global_color_table_size+1))*3):
        out.write(global_color_table[i%len(global_color_table)])

    # Image descriptor
    left = int.from_bytes(
            rng.read(2), byteorder='big')
    top = int.from_bytes(
            rng.read(2), byteorder='big')
    width = 2**int.from_bytes(
            rng.read(1), byteorder='big') % (2**10-1)+1
    height = 2**int.from_bytes(
            rng.read(1), byteorder='big') % (2**10-1)+1
    flags = bytes([int.from_bytes(
            rng.read(1), byteorder='big')])
    local_color_table_flag = flags[0] >> 7
    interlace_flag = bool(flags[0] & 64)
    sort_flag = bool(flags[0] & 32)
    reserved = flags[0] & 7
    lct_size = ((flags[0] & 7) << 8) | int.from_bytes(
            rng.read(1), byteorder='big')
    lct = []
    if local_color_table_flag:
        for i in range(lct_size**2):
            r = int.from_bytes(
                    rng.read(1), byteorder='big') & 255
            g = int.from_bytes(
                    rng.read(1), byteorder='big') & 255
            b = int.from_bytes(
                    rng.read(1), byteorder='big') & 255
            lct += [bytes([r,g,b])]
        for i in range(((2**(lct_size+1))*3)-lct_size*3):
            out.write(lct[i%len(lct)])

    # Table based image data
    lzw_min_code_size = int.from_bytes(
            rng.read(1), byteorder='big')
    lzw_max_code_size = min(
            12, max(
                2, lzw_min_code_size+1))
    lzw_clear_code = 1 << lzw_min_code_size
    lzw_end_of_information_code = lzw_clear_code + 1
    lzw_next_code = lzw_end_of_information_code + 1
    lzw_prev_code = None
    lzw_code_length = [0]*(lzw_max_code_size+1)
    lzw_dictionary = {}
    lzw_