#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), 'big') % 65536
    height = int.from_bytes(rng.read(2), 'big') % 65536
    flags  = bytes([int.from_bytes(rng.read(1), 'big') & 0x7])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big') & 0x7])
    color  = bytes([int.from_bytes(rng.read(1), 'big') & 0xf])
    gct    = bytes([int.from_bytes(rng.read(1), 'big') & 0x1])
    out.write(struct.pack('>HHBBB', width, height, *flags))
    out.write(bg + pxsize + color + gct)

    # Global color table
    for _ in range(pow(2, int.from_bytes(color, 'big'))):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')
        out.write(r + g + b)

    # Image descriptor
    left   = int.from_bytes(rng.read(2), 'big') % width
    top    = int.from_bytes(rng.read(2), 'big') % height
    width  = int.from_bytes(rng.read(2), 'big') % (width - left)
    height = int.from_bytes(rng.read(2), 'big') % (height - top)
    flags  = bytes([int.from_bytes(rng.read(1), 'big') & 0xe0 | 0x01])
    lct    = bytes([int.from_bytes(rng.read(1), 'big') & 0x1])
    interl = bytes([int.from_bytes(rng.read(1), 'big') & 0x6])
    out.write(struct.pack('>HHHHBB', left, top, width, height, *flags))
    out.write(lct + interl)

    # Local color table
    if int.from_bytes(lct, 'big'):
        for _ in range(pow(2, int.from_bytes(color, 'big'))):
            r = int.from_bytes(rng.read(1), 'big')
            g = int.from_bytes(rng.read(1), 'big')
            b = int.from_bytes(rng.read(1), 'big')
            out.write(r + g + b)

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([int.from_bytes(rng.read(1), 'big')]))