#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (default to /dev/urandom).
# The size of the generated image is determined by the first 24 bytes
# of the input stream.
def generate_random_gif(
        rng: BinaryIO=None, out: BinaryIO=sys.stdout.buffer
    ) -> None:

    # Read the header and extract the width & height from it
    if not rng:
        rng = open('/dev/urandom', 'rb')
    header = rng.read(6)
    assert len(header) == 6
    assert header[:3] == b'GIF'
    w, h = struct.unpack('<HH', header[3:])

    # Write the header back with the correct dimensions
    out.write(b'GIF89a')
    out.write(struct.pack('<HH', w, h))

    # Copy all remaining bytes verbatim
    while True:
        buf = rng.read(1024 * 1024)
        if not buf: break
        out.write(buf)


