#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (e.g., /dev/urandom).
# The generated file will have dimensions `width` x `height`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ):

    # Write the header to the output stream.
    # We'll fill in the image size later.
    out.write(b'GIF89a')

    # Generate the color table for this frame.
    colors = [0] * 256
    for i in range(len(colors)):
        r = rng.read(1)[0] & 0x7f
        g = rng.read(1)[0] & 0x7f
        b = rng.read(1)[0] & 0x7f

        colors[i] = bytes([r, g, b])

    # Write the palette to the output stream.
    out.write(struct.pack('B', len(colors)))
    for c in colors:
        out.write(c)

    # Write the logical screen descriptor to the output stream.
    out.write(struct.pack('<HHBBB', width, height, 0xf0 | 0, 0, 0))

    # Write the graphics control extension block to the output stream.
    out.write(b'\x21\xf9\x04\x00\x00\x00\xff\x00')

    # Write the application extension block to the output stream.
    out.write(b'\x21\xff\x0bNETSCAPE2.0')
    out.write(b'\x03\x01\x00\x00')
    out.write(b'\x00!')

    # Write the image descriptor to the output stream.
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\