#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 65536
    height = (1 << rng.read(1)[0]) % 65536
    flags  = rng.read(1)[0] & 7
    bg     = rng.read(1)[0]
    aspect = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    for _ in range((2 ** ((flags >> 4) + 1))):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = rng.read(2)
    top    = rng.read(2)
    width  = (1 << rng.read(1)[0]) % 65536
    height = (1 << rng.read(1)[0]) % 65536
    flags  = rng.read(1)[0] & 7
    lct    = bool(flags & 128)
    interlaced = bool(flags & 64)
    sorted = bool(flags & 32)
    reserved = rng.read(1)[0]
    lctsize = 2 ** ((flags & 7) + 1) if lct else 0
    out.write(left + top + struct.pack('>HHBBB', width, height, flags, lctsize))

    # Local color table
    if lct:
        for _ in range(lctsize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))