#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HH', width, height))
    out.write(b'\x87\x00\x00')  # Global color table flag set to true
    out.write(b'\x00')         # Background color index is zero (black)
    out.write(b'\x00')         # Pixel aspect ratio is one (square pixels)

    # Image descriptor
    out.write(b'\x2c')         # Image separator
    out.write(struct.pack('<HHHHHBB',
            0,                      # Left position
            0,                      # Top position
            width,                  # Width in pixels
            height,                 # Height in pixels
            0,                      # No local color table
            1,                      # LZW compression
            0                       # No interlacing
        ))

    # Local color table
    for i in range(0, 256):
        out.write(struct.pack('BBB', *rng.read(3)))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)