#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(1024, 65536) // 16 * 16
    height = rng.randint(768,  4096) // 16 * 16
    flags  = (
            0x80 |    # Global color table flag
            0x70 |    # Sorting method
            0x00      # Size of global color table
        ).to_bytes(1, 'little', signed=False)
    bgcolorindex = 0
    pixelaspectratio = 0
    out.write(struct.pack('<HHBBB',
            width, height, flags[0], bgcolorindex, pixelaspectratio))

    # Global color table
    for i in range(0, 256):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left       = 0
    top        = 0
    flags      = 0x80 | 0x00 | 0x00 | 0x00
    delaytime  = 0
    transindex = 0
    out.write(struct.pack('<HHBBBBBB',
            left, top, width, height, flags, delaytime, transindex))

    # Image data
    datalen = rng.randint(1024, 65536)
    data = rng.read(datalen)
    out.write(data)