#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    height = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    flags  = rng.read(1)[0]
    bgcolr = rng.read(1)[0]
    pxsize = rng.read(1)[0]
    gctlen = rng.read(1)[0] if flags & 0x80 else None
    out.write(struct.pack('>HHBBB', width, height, flags, bgcolr, pxsize))

    # Global color table
    for _ in range(gctlen):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    top    = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    width  = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    height = (1 << rng.read(1)[0]) | ((1 << rng.read(1)[0]) >> 7)
    flags  = rng.read(1)[0]
    lctlen = rng.read(1)[0] if flags & 0x80 else None
    interl = rng.read(1)[0] if flags & 0x40 else None
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags, lctlen))

    # Local color table
    for _ in range(lctlen):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    while True:
        blocktype = rng.read(1)[0]

        if blocktype == 0x2C:
            break

        elif blocktype == 0x21:
            pass

        elif blocktype == 0x3B:
            return

        else:
            raise Exception("Invalid block type")

    while True:
        blockdata = rng.read(1)[0]

        if blockdata == 0x00:
            continue

        elif blockdata == 0xFF:
            break

        else:
            raise Exception("Invalid block data")

    while True:
        blockdata = rng.read(1)[0]

        if blockdata == 0x00:
            continue

        elif blockdata == 0xFF:
            break

        else:
            raise Exception("Invalid block data")

