#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1024
    height = 768
    color_table_size = 7
    global_color_table = True
    background_color = 0x00
    pixel_aspect_ratio = 0x00
    out.write(struct.pack('>HHBBB', width, height,
                          color_table_size,
                          (global_color_table << 7) | background_color,
                          pixel_aspect_ratio))

    # Global color table
    for i in range(pow(2, color_table_size)):
        r = rng.read(1)[0] & 0xf8
        g = rng.read(1)[0] & 0xfc
        b = rng.read(1)[0] & 0xf8
        out.write(struct.pack('<BBB', r, g, b))

    # Image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table = False
    interlace = False
    sort = False
    reserved = 0x00
    lct_size = 0x00
    out.write(struct.pack('>HHHHBB', left, top, width, height,
                          (local_color_table << 7) | (interlace << 6) |
                          (sort << 5) | reserved, lct_size))

    # Local color table
    if not global_color_table and lct_size > 0:
        for i in range(pow(2, lct_size)):
            r = rng.read(1)[0] & 0xf8
            g = rng.read(1)[0] & 0xfc
            b = rng.read(1)[0] & 0xf8
            out.write(struct.pack('<BBB', r, g, b))

    # Table based image data
    num_blocks = pow(width * height // 64, 2)
    for _ in range(num_blocks):
        size = rng.read(1)[0]
        packed = [0] * ((size >> 2) + 1)

        for i in range((size >> 2) + 1):
            packed[i] = rng.read(1)[0]

        out.write(bytes([size]) + bytes(packed))

    # Trailer
    out.write(b'\x3B')