#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int = 1024, height: int = 768):
    # Magic header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBB', width, height, 0xF, 0))

    # Global color table
    for i in range(0, 256):
        out.write(struct.pack('BBB', *[i] * 3))

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\xff\xff\xff\x00')

    # Image descriptor
    out.write(struct.pack('<HHHH', 0, 0, width, height))
    out.write(b'\x00\x00\x00\x00\x00,\x00\x00\x00\x00\x00\x00')

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng.read(1)[0]]))

    # Trailer
    out.write(b';')
