#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<H', 0x200))
    out.write(b'\x01')
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\xFF\xFF\xFF')

    # Write the global color table
    for _ in range(768):
        out.write(bytes([rng.read(1)[0]]))

    # Write the application extension block
    out.write(struct.pack('B', 0x21))
    out.write(struct.pack('B', 0xff))
    out.write(b'NETSCAPE2.0')
    out.write(struct.pack('B', 0x03))
    out.write(struct.pack('>I', 0xffffff))
    out.write(struct.pack('B', 0x00))

