#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given dimensions and color count.
def generate_gif(out: BinaryIO, width: int, height: int, colors: int) -> None:
    # Header
    out.write("GIF89a".encode())                             # Signature
    out.write(struct.pack("<HH", width, height))            # Logical screen size
    out.write(b"\x00\x00")                                  # Background color index
    out.write(b"\x00")                                      # Pixel aspect ratio (unused)
    
    # Global color table
    for i in range(colors):
        r = (i >> 2) & 7
        g = (i >> 1) & 7
        b = (i >> 0) & 7
        
        out.write(bytes([r << 5 | g << 2 | b]))
    
    # Graphic control extension
    out.write(b"\x21\xf9\x04\x08\x00\x00\xff\xff\xff")       # Extension header
    out.write(b"\x00\x00\x00\x2c\x00\x00\x00\x00")           # Delay time (in hundredths of a second)
    
    # Image descriptor
    out.write(b"\x2c\x00\x00\x00\x00")                       # Image separator + image position
    out.write(struct.pack("<HH", width, height))            # Image size
    out.write(b"\x00")                                      # Local color table flag
    out.write(b"\x00")                                      # Interlace flag
    out.write(b"\x00")                                      # Sort flag
    out.write(b"\x00")                                      # Size of local color table
    
    # LZW minimum code size
    out.write(b"\x02")
    
    # Data sub-blocks
    out.write(b"\x00")
    
    # Trailer
    out.write(b";")
