#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<H', 0x200))
    out.write(b'\x01')
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\xFF\xFF\xFF')

    # Write the global color table
    for _ in range(768):
        out.write(bytes([rng.read(1)[0]]))

    # Write the application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xFF\xFF\xFF')

    # Write the local color table
    for _ in range(768):
        out.write(bytes([rng.read(1)[0]]))

    # Write the LZW-compressed data
    while True:
        byte = rng.read(1)
        if not byte:
            break
        out.write(byte)
