#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG. The generated GIF is guaranteed to be valid.
def generate(rng: BinaryIO, out: BinaryIO):
    # Write the header
    out.write(b'GIF89a')

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBB', 1, 1, 0, 0))

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(768):
        out.write(bytes([(i >> ((~i + 1) % 8)) & 0xFF]))

    # Write the image data
    out.write(b'\x00\x00')

    # Write the LZW compressed image data
    lzw_min_code_sizes = [8, 4, 2, 0]
    for lzw_min_code_size in lzw_min_code_sizes:
        out.write(bytes([lzw_min_code_size]))

        # Read the LZW code stream and write it as-is
        while True:
            code = rng.read(1)[0]

            if code == 0x00 or code == 0x01:
                break

            out.write(bytes([code]))

