#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):
    # Magic header
    out.write(b'GIF89a')

    # Screen descriptor
    screen_width = 16 + (rng.read(1)[0] % 256)
    screen_height = 16 + (rng.read(1)[0] % 256)
    out.write(struct.pack('<HHBB', screen_width, screen_height, 0xF7, 0))

    # Global color table
    for i in range(0, 256):
        out.write(struct.pack('<BBB', rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]))
    
    # Image descriptor
    out.write(struct.pack('<HHHBB', 0, 0, screen_width, screen_height, 0))

    # Local color table
    for i in range(0, 256):
        out.write(struct.pack('<BBB', rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]))

    # Pixel data
    for y in range(0, screen_height):
        for x in range(0, screen_width):
            out.write(struct.pack('<B', rng.read(1)[0]))

    # Trailer
    out.write(b'\x3B')
