#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')
    width = rng.read(2)[0] + 1
    height = rng.read(2)[0] + 1
    out.write(struct.pack('<HHB', width, height, 0xF))
    out.write(b'\x00\x00\xFF\xFF\xFF\xFF\xFF\xFF')

    # Image data
    for y in range(height):
        for x in range(width):
            color_index = rng.read(1)[0] & 0x7
            out.write(bytes([color_index]))

    # Trailer
    out.write(b';')
