#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Write the header.
    out.write(b'GIF89a')
    
    # Write the logical screen descriptor.
    out.write(struct.pack('<HBB', width, height, 0xF7))
    
    # Write the global color table.
    for i in range(256):
        r = (i & 0x1F) << 3
        g = ((i >> 5) & 0x1F) << 3
        b = ((i >> 10) & 0x1F) << 3
        
        out.write(bytes([r, g, b]))
    
    # Write the application extension block.
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00')
    
    # Write the image descriptor.
    out.write(b'\x2C\x00\x00\x00\x00' + struct.pack('<HH', width, height) + b'\x00')
    
    # Write the local color table.
    for i in range(256):
        r = (i & 0x1F) << 3
        g = ((i >> 5) & 0x1F) << 3
        b = ((i >> 10) & 0x1F) << 3
        
        out.write(bytes([r, g, b]))
    
    # Write the image data.
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', (y * width + x) % 256))
