#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    width = rng.read(2)[0] + (rng.read(1)[0] << 8) % 0x100
    height = rng.read(2)[0] + (rng.read(1)[0] << 8) % 0x100
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    global_color_table_flag = rng.read(1)[0] & 0b10000000
    color_resolution = rng.read(1)[0] & 0b01110000
    sort_flag = rng.read(1)[0] & 0b00001000
    size_of_global_color_table = rng.read(1)[0] & 0b00000111
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]

    out.write(header)
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(bytes([background_color_index]))
    out.write(bytes([pixel_aspect_ratio]))
    out.write(bytes([global_color_table_flag | color_resolution | sort_flag | size_of_global_color_table]))
    out.write(bytes([background_color_index]))
    out.write(bytes([pixel_aspect_ratio]))

    for i in range(width * height):
        out.write(bytes([i % 256]))

    out.write(b'\x3B')
