#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBB', 0x200, 0x200, 0b10000000))

    # Write the global color table
    out.write(bytes([0] * 768))

    # Write the application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', 0xffff))
    out.write(b'\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    out.write(bytes([0] * 768))

    # Write the image data
    out.write(b'\x00' * 0x10000)

    # Write the trailer
    out.write(b';')
