#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 20 + 5
    color_table_size = rng.read(1)[0] & 7
    background_color_index = rng.read(1)[0] % (1 << color_table_size)
    pixel_aspect_ratio = rng.read(1)[0] % 64
    out.write(struct.pack('<HHBB', width, height, color_table_size, background_color_index))

    # Global color table
    for i in range(1 << color_table_size):
        red   = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue  = rng.read(1)[0] % 256
        out.write(struct.pack('<BBB', red, green, blue))

    # Image descriptor
    left = rng.read(1)[0] % width
    top = rng.read(1)[0] % height
    width = rng.read(1)[0] % (width - left) + 1
    height = rng.read(1)[0] % (height - top) + 1
    local_color_table_flag = rng.read(1)[0] >> 7
    interlace_flag = rng.read(1)[0] >> 6
    sort_flag = rng.read(1)[0] >> 5
    reserved = rng.read(1)[0] >> 4
    color_table_size = rng.read(1)[0] & 7
    background_color_index = rng.read(1)[0] % (1 << color_table_size)
    pixel_data_length = rng.read(1)[0] * 256 + rng.read(1)[0]
    out.write(struct.pack('<HHBBBBH', left, top, width, height, local_color_table_flag, interlace_flag, sort_flag, reserved, color_table_size, background_color_index))

    # Local color table
    if local_color_table_flag == 1:
        for i in range(1 << color_table_size):
            red   = rng.read(1)[0] % 256
            green = rng.read(1)[0] % 256
            blue  = rng.read(1)[0] % 256
            out.write(struct.pack('<BBB', red, green, blue))

    # Pixel data
    for _ in range(pixel_data_length):
        code = rng.read(1)[0]
        out.write(code)

    # Trailer
    out.write(b'\x3b')
