#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    height = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    out.write(struct.pack('<HHBBB', width, height, 0xF0, 0x00, 0x00))

    # Write the global color table
    out.write(b'\xFF\xFF\xFF')

    # Write the image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    out.write(b'\xFF\xFF\xFF')

    # Write the image data
    for _ in range(width * height):
        out.write(bytes([rng.read(1)[0]]))

    # Write the trailer
    out.write(b';')
