#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Jun 14 20:59:36 2020

@author: jorge
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
from matplotlib import pyplot as plt

def f(x):
    return x**2 + 1

def g(x):
    return (x+1)**2 + 1

def h(x):
    return (x-1)**2 + 1

def d_f(x):
    return 2 * x

def d_g(x):
    return 2 * (x+1)

def d_h(x):
    return 2 * (x-1)

def d_d_f(x):
    return 2

def d_d_g(x):
    return 2

def d_d_h(x):
    return 2

def get_data():
    # Generate data
    n = 1000
    X = np.random.uniform(-1, 1, size=n)
    
    Y = f(X) + np.random.randn(n)/np.sqrt(2)
    Z = g(X) + np.random.randn(n)/np.sqrt(2)
    W = h(X) + np.random.randn(n)/np.sqrt(2)
        
    return X, Y, Z, W

def plot_data(X, Y, Z, W):
    fig, ax = plt.subplots()
    ax.scatter(X[Y<Z], Y[Y<Z], c='r', label='Y < Z')
    ax.scatter(X[Y>Z], Y[Y>Z], c='b', label='Y > Z')
    ax.scatter(X[W<Z], W[W<Z], c='g', label='W < Z')
    ax.scatter(X[W>Z], W[W>Z], c='m', label='W > Z')
    ax.legend()

def plot_density(X, Y, Z, W):
    fig, ax = plt.subplots()
    ax.plot(X, norm.pdf(X), 'k--', lw=2, alpha=.7)
    ax.plot(X, norm.pdf(X)+norm.pdf(X)*norm.pdf(X), 'k', lw=2, alpha=.7)
    ax.fill_between(X, norm.pdf(X)+norm.pdf(X)*norm.pdf(X), color='gray', alpha=.3)
    ax.set_ylim([0, 1])
    ax.set_title('Density of $X$ and $X^2 + 1$')

def plot_posterior(X, Y, Z, W):
    fig, ax = plt.subplots()
    for i in range(len(X)):
        if Y[i] < Z[i]:
            ax.plot(X[i], norm.pdf(X[i]), 'r', lw=2, alpha=.7)
        else:
            ax.plot(X[i], norm.pdf(X[i]), 'b', lw=2, alpha=.7)
            
    for i in range(len(X)):
        if W[i] < Z[i]:
            ax.plot(X[i], norm.pdf(X[i]), 'g', lw=2, alpha=.7)
        else:
            ax.plot(X[i], norm.pdf(X[i]), 'm', lw=2, alpha=.7)
            
    ax.set_ylim([0, 1])
    ax.set_title('Posterior density of $X$')

def plot_conditional(X, Y, Z, W):
    fig, ax = plt.subplots()
    for i in range(len(X)):
        if Y[i] < Z[i]:
            ax.plot(X[i], norm.pdf(X[i]), 'r', lw=2, alpha=.7)
        else:
            ax.plot(X[i], norm.pdf(X[i]), 'b', lw=2,