#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=1024, height=768):
    # Magic header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBB', width, height, 0xF, 0))

    # Global color table (black and white)
    out.write(bytes([0] * 768))

    # Image block
    out.write(struct.pack('<HHB', 0, 0, 0x80 | 0x40))
    out.write(struct.pack('<H', 0xFFFF))

    # Pixel data
    for y in range(height):
        for x in range(width):
            pixel = int.from_bytes(rng.read(1), 'little') % 256
            out.write(bytes([pixel]))

    # Trailer
    out.write(b'\x3B')
