#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 10 + rng.read(2)[0] % 50
    height = 10 + rng.read(2)[0] % 50
    flags = rng.read(1)[0] & 7
    background = rng.read(1)[0] % 256
    aspect = rng.read(1)[0] % 4
    out.write(struct.pack('<HHBBB', width, height, flags, background, aspect))

    # Global color table
    if flags & 1:
        for _ in range(2 ** ((flags >> 4) + 1)):
            out.write(rng.read(3))

    # Application extension
    if rng.read(1)[0] & 1:
        out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
        out.write(struct.pack('<H', 0))

    # Image block
    out.write(b'\x2c')

    # Image descriptor
    left = rng.read(2)[0] % width
    top = rng.read(2)[0] % height
    width = rng.read(2)[0] % (width - left)
    height = rng.read(2)[0] % (height - top)
    flags = rng.read(1)[0] & 7
    local_palette = None
    if flags & 1:
        local_palette = []
        for _ in range(2 ** ((flags >> 4) + 1)):
            local_palette.append(rng.read(3))
    interlace = rng.read(1)[0] & 1
    out.write(struct.pack('<HHBBB', left, top, width, height, flags))

    # Local color table
    if local_palette is not None:
        for p in local_palette:
            out.write(p)

    # Raster data
    for y in range(top, top + height):
        for x in range(left, left + width):
            out.write(bytes([rng.read(1)[0]]))

    # Trailer
    out.write(b';')
