#!/usr/bin/env python3

import struct
from typing import BinaryIO

from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad


def encrypt_aes(data: bytes, key: bytes) -> bytes:
    """Encrypts data using AES-256 with PKCS#7 padding"""
    cipher = AES.new(key, AES.MODE_ECB)
    return cipher.encrypt(pad(data, AES.block_size))


def decrypt_aes(data: bytes, key: bytes) -> bytes:
    """Decrypts data using AES-256 with PKCS#7 padding"""
    cipher = AES.new(key, AES.MODE_ECB)
    return unpad(cipher.decrypt(data), AES.block_size)


def read_gif(f: BinaryIO) -> bytes:
    """Reads a GIF file and returns its raw data"""
    f.seek(0)
    header = f.read(6)
    assert header[:6] == b'GIF89a', 'Not a valid GIF file'
    width, height = struct.unpack('<HH', f.read(4))
    flags = ord(f.read(1))
    background = ord(f.read(1))
    aspect_ratio = ord(f.read(1))
    global_color_table_flag = bool(flags & 0x80)
    sort_flag = bool(flags & 0x08)
    color_resolution = (flags & 0x07) + 1
    global_color_table_size = 2 ** (color_resolution + 1)
    background_index = background
    pixel_aspect_ratio = aspect_ratio + 15
    global_color_table = []
    if global_color_table_flag:
        for _ in range(global_color_table_size * 3):
            global_color_table.append(ord(f.read(1)))
    else:
        global_color_table = [0] * 768
    block_size = ord(f.read(1))
    assert block_size == 0x21, 'Invalid block size'
    extension_type = ord(f.read(1))
    assert extension_type == 0xf9, 'Invalid extension type'
    graphic_control_extension_length = ord(f.read(1))
    assert graphic_control_extension_length == 0x04, \
        'Invalid graphic control extension length'
    disposal_method = ord(f.read(1))
    user_input_flag = bool(disposal_method & 0x02)
    transparent_color_flag = bool(disposal_method & 0x01)
    delay_time = struct.unpack('<H', f.read(2))[0]
    transparent_color_index = ord(f.read(1))
    block_terminator = ord(f.read(1))
    assert block_terminator == 0, 'Block terminator not found'
    application_identifier = f.read(8).decode()
    application_authentication_code = f.read(3)
    assert application_identifier == 'NETSCAPE2.0', \
        'Invalid application identifier'
    assert application_authentication_code == b'CE', \
        'Invalid application authentication code'
    data_sub_block_size = ord(f.read(1))
    loop_count = struct.unpack('<H', f.read(2))[0]
    assert data_sub_block_size == 0x03, 'Invalid data sub-block size'
    block_size = ord(f.read(1))
    assert block_size == 0x2c, 'Invalid block size'
    image_descriptor = f.read(9)
    left_position, top_position, width, height = struct.unpack('<HHHH',
                                                               image_descriptor)
    local_color_table_flag = bool(ord(image_descriptor[8]) & 0x80)
    interlace_flag = bool(ord(image_descriptor[8]) & 0x40)
    sort_flag = bool(ord(image
    # Write the local color table
    for i in range(768):
        out.write(bytes([(i >> ((~i + 1) % 8)) & 0xFF]))

    # Write the image data
    out.write(b'\x00\x00')

    # Write the LZW compressed image data
    while True:
        code = rng.read(1)[0]
        if code == 0x00:
            break
        else:
            out.write(bytes([code]))
    # Write the trailer
    out.write(b';')
