#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def main() -> bool:
    if len(sys.argv) != 3:
        print("Usage: {} <input> <output>".format(os.path.basename(__file__)))
        return False

    with open(sys.argv[1], 'rb') as f_in,\
         open(sys.argv[2], 'wb') as f_out:
        return convert(f_in, f_out)


def convert(rng: BinaryIO, out: BinaryIO) -> bool:
    # Header
    out.write(b'GIF89a')
    out.write(b'\x00\x00\x00\x00')
    out.write(b'\x00\x00')
    out.write(b'\xff\xff\xff')
    out.write(b'\x00\x00\x00')

    # Logical Screen Descriptor
    out.write(struct.pack('H', 1024))
    out.write(struct.pack('H', 768))
    out.write(struct.pack('B', 0x00))
    out.write(struct.pack('B', 0x00))
    out.write(struct.pack('B', 0xff))
    out.write(struct.pack('B', 0xff))
    out.write(struct.pack('B', 0xff))
    out.write(struct.pack('B', 0x00))

    # Global Color Table
    for i in range(0, 256):
        r = (i & 0xE0) >> 5
        g = (i & 0x1C) << 3
        b = (i & 0x03) << 6

        if i == 0:
            r = 0
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # LZW minimum code size
    out.write(struct.pack('B', 0x08))

    # Pixel data
    for y in range(0, 768):
        for x in range(0, 1024):
            pixel = struct.unpack('B', rng.read(1))[0]
            out.write(struct.pack('B', pixel))

    # Trailer
    out.write(b';')

    return True


