#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
)-> None:
    width = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1

    # Header
    out.write(b'GIF89a')
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\x87\x00\x00')

    # Global color table
    for i in range(256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Trailer
    out.write(b';')