#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int) -> None:
    # Write the header.
    out.write(b'GIF89a')
    
    # Write the logical screen descriptor.
    out.write(struct.pack('<HBB', width, height, 0xF7))
    
    # Write the global color table (color count = 256).
    for i in range(256):
        r = g = b = i
        
        if i == 0 or i == 1:
            r = g = b = 0
            
        elif i == 2:
            r = g = b = 255
            
        out.write(bytes([r, g, b]))
    
    # Write the graphic control extension.
    out.write(b'\x21\xF9\x04\x01\x00\x00\xFF\x00')
    
    # Write the image descriptor.
    out.write(struct.pack('<HHB', 0, 0, 0x00))
    
    # Write the local color table (color count = 2).
    out.write(b'\x02\x00\x01\x00\x01\x00')
    
    # Write the LZW-compressed data stream.
    out.write(b'\x00\x2C\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;')
