#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    height = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    out.write(struct.pack('<HHBBB', width, height, 0xF0, 0x00, 0x00))

    # Write the global color table
    colors = [rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]]
    for i in range(256):
        out.write(bytes(colors))

    # Write the application extension block
    app_ext_block_size = rng.read(1)[0] & 0xFF | (rng.read(1)[0] & 0xFF) << 8
    out.write(b'\x21\xff')
    out.write(struct.pack('<H', app_ext_block_size))
    for _ in range(app_ext_block_size - 1):
        out.write(bytes([rng.read(1)[0]]))

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')

    # Write the local color table
    lct_flag = rng.read(1)[0] & 0x80 != 0
    if lct_flag:
        lct_size = rng.read(1)[0] & 0x07 + 1
        for i in range(lct_size * 3):
            out.write(bytes([rng.read(1)[0]]))

    # Write the image data
    code_size = rng.read(1)[0] & 0x07 + 1
    while True:
        code = rng.read(1)[0]
        if code == 0:
            break
        out.write(bytes([code]))

    # Write the trailer
    out.write(b'\x3b')
