#!/usr/bin/env python3

import struct
from typing import BinaryIO

with open('/dev/urandom', 'rb') as rng:
    while True:
        # Header
        header = b'GIF89a'
        out.write(header)

        # Logical screen descriptor
        lsdesc = bytes([0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0x00, 0xff])
        lsdesc += rng.read(1)
        out.write(lsdesc)

        # Global color table
        if bool(lsdesc[9] & 0x80):
            for i in range((lsdesc[9] & 0x07) + 1):
                out.write(rng.read(3))

        # Graphic control extension
        gce = bytes([0x21, 0xf9, 0x04])
        gce += bytes([int.from_bytes(rng.read(1), byteorder='little')])
        out.write(gce)

        # Image descriptor
        idesc = bytes([0x2c])
        idesc += generate_image_descriptor(rng, width, height)
        out.write(idesc)

        # Local color table
        if bool(idesc[9] & 0x80):
            for i in range((idesc[9] & 0x07) + 1):
                out.write(rng.read(3))

        # Pixel data
        for y in range(idesc[5]):
            for x in range(idesc[4]):
                out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

        # Trailer
        trailer = bytes([0x3b])
        out.write(trailer)

        # End of stream?
        if bool(rng.read(1)):
            break

# Generates a random image descriptor using the
# random number generator `rng` (/dev/urandom)
def generate_image_descriptor(
        rng: BinaryIO, width: int, height: int
    ) -> bytes:
    left   = int.from_bytes(rng.read(2), byteorder='little') % (width  // 2) + 1
    top    = int.from_bytes(rng.read(2), byteorder='little') % (height // 2) + 1
    width  = int.from_bytes(rng.read(2), byteorder='little') % ((width  - left) // 2) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % ((height - top) // 2) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    lct    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    interlaced = False if not bool(flags[0] & 0x40) else True
    sort    = False if not bool(flags[0] & 0x20) else True
    local_color_table_size = lct[0]
    background_index      = int.from_bytes(rng.read(1), byteorder='little')
    pixel_aspect_ratio    = int.from_bytes(rng.read(1), byteorder='little')
    out.write(struct.pack('<HHHHBB', left, top, width, height, flags[0], lct[0]))
