#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHHBB', width, height, 0xF, 0, 0)
    out.write(packed)

    # Global color table
    for i in range(7):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            if x == 0 and y == 0:
                continue

            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Trailer
    out.write(b';')