#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the header
    out.write(b'GIF89a')

    # Write the screen descriptor
    out.write(struct.pack('<H', 10))
    out.write(struct.pack('<H', 20))
    out.write(b'\xF4\x07')

    # Write the global color table
    for i in range(7):
        out.write(struct.pack('BBB', *[rng.read(1)[0] for _ in range(3)]))

    # Write the application extension block
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00')

    # Write the graphic control extension block
    out.write(b'\x21\xF9\x04\x01\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(7):
        out.write(struct.pack('BBB', *[rng.read(1)[0] for _ in range(3)]))

    # Write the image data
    for y in range(20):
        for x in range(10):
            out.write(struct.pack('B', rng.read(1)[0]))
