#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

if __name__ == '__main__':
    width = int(sys.argv[1])
    height = int(sys.argv[2])
    filename = sys.argv[3]

    with open(filename, 'wb') as f:
        generate_random_image(f, width, height)

def generate_random_image(out: BinaryIO, width: int, height: int):
    rng = os.urandom

    # Header
    out.write(b'P4\n')
    out.write(bytes(str(width), encoding='ascii'))
    out.write(b' ')
    out.write(bytes(str(height), encoding='ascii'))
    out.write(b'\n')

    # Palette
    for i in range(256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            if x == 0 and y == 0:
                continue

            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Trailer
    out.write(b';')