#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Read the width and height of the image
    width, height = struct.unpack('<HH', rng.read(4))

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xf0, 0, 0))

    # Write the global color table
    for _ in range(7):
        out.write(rng.read(3))

    # Write the graphic control extension
    gce = b'\x21\xf9\x04\x05\xff\xff\xff\xff\x00'
    out.write(gce)

    # Write the image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00'
    out.write(idesc)

    # Write the local color table
    for _ in range(7):
        out.write(rng.read(3))

    # Write the LZW data
    lzw_min_code_size = rng.read(1)[0]
    out.write(bytes([lzw_min_code_size]))

    while True:
        code = rng.read(1)[0]
        if (code == 0):
            break
            if (code >= 1 and code <= lzw_min_code_size):
                out.write(bytes([code]))
            elif (code == lzw_min_code_size + 1):
                out.write(b'\x00')
            else:
                raise Exception('Invalid code encountered!')

    # Write the trailer
    out.write(b';')
