#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), byteorder='little') % (width - 1) + 1
    top    = int.from_bytes(rng.read(2), byteorder='little') % (height - 1) + 1
    width  = int.from_bytes(rng.read(2), byteorder='little') % ((width - left) // 2) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % ((height - top) // 2) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    lct    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    interlaced = False if not bool(flags[0] & 0x40) else True
    sort       = False if not bool(flags[0] & 0x20) else True
    out.write(struct.pack('<HHBB', left, top, width, height))
    out.write(struct.pack('<BB', flags[0], lct[0]))

    # Local color table
    for i in range(2 ** (bool(lct[0] & 0x80) * 3)):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Pixel data
    while True:
        block_size = int.from_bytes(rng.read(1), byteorder='little')
        if block_size == 0:
            break
        elif block_size == 1:
            # Extension block
            ext_code = int.from_bytes(rng.read(1), byteorder='little')
            if ext_code == 0xF9:
                # Graphic control extension
                length = int.from_bytes(rng.read(1), byteorder='little')
                assert length == 4
                out.write(b'\x21\xF9')
                out.write(length.to_bytes(1, 'little'))
                out.write(rng.read(length))

                # End of graphic control extension
                out.write(b'\x00')
            elif ext_code == 0xFE:
                # Comment extension
                length = int.from_bytes(rng.read(1), byteorder='little')
                out.write(b'\x21\xFE')
                out.write(length.to_bytes(1, 'little'))
                out.write(rng.read(length))

                # End of comment extension
                out.write(b'\x00')
            elif ext_code == 0xFF:
                # Application extension
                length = int.from_bytes(rng.read(1), byteorder='little')
                out.write(b'\x21\xFF')
                out.write(length.to_bytes(1, 'little'))
                out.write(rng.read(length))

                # End of application extension
                out.write(b'\x00')
            elif ext_code == 0x01:
                # Plain text extension
                length = int.from_bytes(rng.read(1), byteorder='little')
                out.write(b'\x21\x01')
                out.write(length.to_bytes(1, 'little'))
                out.write(rng.read(length))

                # End of plain text extension
                out.write(b'\x00')

        elif block_size > 1:
            out.write(data)

        block_size = int.from_bytes(rng.read(1), byteorder='little')