#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    palette = [i for i in range(256)]
    rng.shuffle(palette)
    for entry in palette:
        out.write(struct.pack('BBBB', *entry, 0x00))

    # Extension block
    out.write(b'\x21\xFF\x0BNETSCAPE2.0')
    # Image data
    for y in range(height):
        for x in range(width):
            if x == 0 and y == 0:
                continue

            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Trailer
    out.write(b';')