#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(256):
        r = rng.read(1)[0] & 0xFF
        g = rng.read(1)[0] & 0xFF
        b = rng.read(1)[0] & 0xFF
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    packed = struct.pack('<HHHHBB', 0, 0, width, height, 0x00, 0x00)
    out.write(packed)

    # Local color table
    for i in range(256):
        r = rng.read(1)[0] & 0xFF
        g = rng.read(1)[0] & 0xFF
        b = rng.read(1)[0] & 0xFF
        out.write(struct.pack('BBB', r, g, b))

    # Pixel data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Trailer
    out.write(b';')