#!/usr/bin/env python3

import os
import sys
import struct
from collections import namedtuple


def parse_header(data):
    header = {}

    magic, version, num_entries = struct.unpack('>4sHH', data[:8])
    if magic!= b'PAK1':
        raise ValueError("Invalid PAK file")
    if version > 2:
        raise ValueError("Unsupported PAK version {}".format(version))

    for i in range(num_entries):
        offset, size = struct.unpack('>II', data[i*8+8:i*8+16])

        filename = ""
        while True:
            char = chr(data[offset])
            offset += 1

            if char == '\0':
                break
            else:
                filename += char

        header[filename] = { 'offset': offset,
                            'size':   size }

    return header


def extract_pak(input_path, output_path):
    with open(input_path, "rb") as f:
        data = bytearray(f.read())

    header = parse_header(data)

    for filename, entry in header.items():
        print("{} -> {}".format(entry['offset'], entry['size']))
        outname = os.path.join(output_path, filename)
        dirname = os.path.dirname(outname)
        if not os.path.exists(dirname):
            os.makedirs(dirname)

        with open(outname, "wb") as f:
            f.write(data[entry['offset']:entry['offset'] + entry['size']])


if