#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_random(out: BinaryIO, width: int = 1024, height: int = 768):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xF0, 0x0, 0x0))

    # Global color table (black and white)
    out.write(b'\x00\x00\x00\xFF\xFF\xFF')

    # Image descriptor
    out.write(struct.pack('<BHHHHBB', 0x2C, 0, 0, width, height, 0, 0))

    # Local color table (black and white)
    out.write(b'\x00\x00\x00\xFF\xFF\xFF')

    # Graphic control extension
    out.write(b'\x21\xF9\x04\x05\x00\x00\x00\x00\x2C\x00')

    # Image data
    for _ in range((width * height) // 2):
        out.write(bytes([
            # Pixels are encoded as bits
            0b00000000 | ((0b00000000 & ord(' ')) << 4),
            0b00000000 | ((0b00000000 & ord(' ')) >> 4),
        ]))

