#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor (LSD)
    lsd = bytearray()
    lsd.extend(struct.pack('<HH', 640, 480))
    lsd.append((0b11 << 5) | (0b11111 << 2))
    lsd.append(0)
    out.write(lsd)

    # Write the global color table
    gct = bytearray()
    for i in range(0x100):
        r = rng.read(1)[0] & 0xf8
        g = rng.read(1)[0] & 0xfc
        b = rng.read(1)[0] & 0xf8

        gct.extend(bytes([(r >> 3), ((g >> 2) & 0x7c), ((b >> 3) & 0xe)]))

    out.write(gct)

    # Write the image block
    img = bytearray()
    img.append(0x2c)
    img.extend(struct.pack('<HHHHB', 0, 0, 640, 480, 0))
    img.append(0)

    out.write(img)

