#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for _ in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    packed = struct.pack('<HHHBB', 0, 0, 0, 0, 0)
    out.write(packed)

    # LZW minimum code size
    out.write(b'\x01')

    # Image data
    while True:
        byte = rng.read(1)

        if len(byte) == 0:
            break

        out.write(byte)


if