#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), byteorder='little') % (width - 1) + 1
    top    = int.from_bytes(rng.read(2), byteorder='little') % (height - 1) + 1
    width  = int.from_bytes(rng.read(2), byteorder='little') % ((width - left) // 2) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % ((height - top) // 2) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    lct    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    interl = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHHHBB', left, top, width, height, flags[0], lct[0], interl[0]))

    # Local color table
    if lct == b'\x80':
        for i in range(3 * pow(2, lct)):
            out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Trailer
    out.write(b';')

