#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<HHHH',
        640, 480,
        7, 0))

    # Write the global color table
    for i in range(256):
        out.write(bytes([i]))

    # Write the image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00,\x00\x00\x00\x00\x00\x01')

    # Write the local color table
    for i in range(256):
        out.write(bytes([i]))

    # Write the graphics control extension
    out.write(b'\x21\xF9\x04\x00\x00\x00\x00\x00\x2C\x00\x00\x00\x00\x00\x01')

    # Write the plain text extension
    out.write(b'\x21\x01\x00\x00\xFF\x00\x0BNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Write the application extension
    out.write(b'\x21\xFF\x0B\x4E\x45\x54\x53\x43\x41\x50\x45\x32\x2E\x30\x03\x01\x00\x00\x00')

    # Write the comment
    out.write(b';Generated by https://github.com/michaelkrukov/random-gif-generator')

    # Write the end of file
    out.write(b'\x3B')

