#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    height = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    out.write(struct.pack('<HHBBB', width, height, 0xF0, 0x00, 0x00))

    # Write the global color table
    colors = [b'\xFF\x00\x00',
              b'\x00\xFF\x00',
              b'\x00\x00\xFF']
    for i in range(len(colors)):
        if i < len(colors)-1:
            out.write(colors[i])
        else:
            out.write(bytes([0x00]))

    # Write the image block
    out.write(b'\x2C')
    left = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    top = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    width = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    height = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    flags = rng.read(1)[0]
    out.write(struct.pack('<HHBBBB',
                         left, top, width, height,
                         0x00, flags))

    # Write the local color table
    num_colors = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    for i in range(num_colors):
        r = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
        g = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
        b = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8

        out.write(struct.pack('BBB',
                             r >> 5, g >> 5, b >> 6))

    # Write the LZW code size byte
    out.write(bytes([rng.read(1)[0]]))

    # Write the compressed data
    while True:
        chunk_size = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
        if not chunk_size: break
        out.write(chunk_size.to_bytes(2, 'big'))
        out.write(rng.read(chunk_size))

