#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    flags = b'\x87\x00' if rng.read(1)[0] & 1 else b'\x07\x00'
    bgcolor = rng.read(1)[0] % 256
    aspect = rng.read(1)[0] % 9 + 1
    out.write(struct.pack('<HHBBB', width, height, flags[0], flags[1], bgcolor, aspect))

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Trailer
    out.write(b';')