#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image descriptor
    packed = struct.pack('<HHHHB', 0, 0, width, height, 0x00)
    out.write(packed)

    # LZW minimum code size
    out.write(struct.pack('<B', 0x08))

    # Data sub-block
    data = bytes([rng.read(1)[0] % 256])
    while len(data)!= 0xFFFF:
        if rng.read(1)[0] & 0x80 == 0:
            count = rng.read(1)[0] & 0x7F
            data += bytes([rng.read(1)[0]] * count)
        else:
            count = rng.read(1)[0]
            data += bytes([data[-count]])
    # Trailer
    out.write(b';')