#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

        if len(header) == 6:
            break

    out.write(header)

    width = rng.read(2)
    height = rng.read(2)
    out.write(width + height)

    palette = [None] * (256 ** 3)

    for i in range(len(palette)):
        r = rng.read(1)
        g = rng.read(1)
        b = rng.read(1)
        palette[i] = r + g + b

    out.write(b"".join(palette))

    while True:
        block_type = rng.read(1)

        if not block_type or block_type == b'\x3B':
            break

        size = struct.unpack("<H", rng.read(2))[0] - 1
        data = rng.read(size)

        out.write(block_type + struct.pack("<H", len(data) + 1) + data)

