#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, *flags))
    out.write(bg)
    out.write(aspect)

    # Global color table
    numcolors = min((width*height)//50, 256)
    for i in range(numcolors):
        r = int.from_bytes(rng.read(1), byteorder='little') & 0x7f
        g = int.from_bytes(rng.read(1), byteorder='little') & 0x7f
        b = int.from_bytes(rng.read(1), byteorder='little') & 0x7f
        out.write(b'\xff\x00\x00')

    # Image descriptor
    left   = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16)
    top    = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16)
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    lctsize= bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHHHB', left, top, width, height, *flags))
    out.write(lctsize)

    # Local color table
    if len(lctsize) > 0 and int.from_bytes(lctsize, byteorder='little')!= 0:
        for i in range(min(2**(int.from_bytes(lctsize, byteorder='little')) - 1, 256)):
            r = int.from_bytes(rng.read(1), byteorder='little') & 0x7f
            g = int.from_bytes(rng.read(1), byteorder='little') & 0x7f
            b = int.from_bytes(rng.read(1), byteorder='little') & 0x7f
            out.write(b'\xff\x00\x00')

    # Image data
    codesize = int.from_bytes(rng.read(1), byteorder='little')
    out.write(codesize)
    for i in range(((width*height)//64)*64):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Trailer
    out.write(b';')

