#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<H', 0x200))
    out.write(struct.pack('B', 0x7d))
    out.write(bytes([rng.read(1)[0]]))
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\x00')
    out.write(bytes([rng.read(1)[0]]))
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\x00\x00')

    # Write the global color table
    out.write(b'\xff\xff\xff\xff\xff\xff')

    # Write the loop control block
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')

    # Write the image descriptor
    out.write(b'\x2c')
    out.write(bytes([rng.read(1)[0]]))
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')

    # Write the local color table
    out.write(b'\xff\xff\xff\xff\xff\xff')

    # Write the graphics control extension
    out.write(b'\x21\xf9\x04')
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')

    # Write the image data
    out.write(b'\x21\xfe\x00')
    out.write(b'\x3b')
    out.write(b'\x00')

