#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases.
    palette = [struct.pack('BBB', *rng.read(3)) for _ in range(0x100)]
    data = []

    while True:

        blocktype = rng.read(1)[0] & 0b00001111

        if blocktype == 0x2c:
            # Image descriptor

            left = rng.read(2)
            top = rng.read(2)
            width = rng.read(2)
            height = rng.read(2)
            flags = rng.read(1)
            lct = (flags[0] >> 7) & 1
            interlace = (flags[0] >> 6) & 1
            sort = (flags[0] >> 5) & 1
            reserved = flags[0] & 0b0001111
            size = None
            bgindex = None
            fgindex = None
            lctsize = None
            lct = None

            if lct:
                lctsize = rng.read(1)[0] + 1
                lct = [palette[i % len(palette)]
                       for i in range(lctsize)]

                if lctsize > 0x100:
                    raise Exception("LCT too large")

            if not interlace:
                size = rng.read(1)[0] + 1

            else:
                size = rng.read(1)[0] + 1
                bgindex = rng.read(1)[0]
                fgindex = rng.read(1)[0]

            data += [left, top, width, height,
                     bytes([flags[0]]),
                     bytes([lctsize]) if lct else b'',
                     bytes([bgindex]) if interlace else b'',
                     bytes([fgindex]) if interlace else b'',
                     b''.join(lct) if lct else b'',
                     rng.read(size -
                             ((len(lct or []) +
                               (interlace and 4 or 0))))]

        elif blocktype == 0x21:
            # Extension block

            exttype = rng.read(1)[0]
            size = rng.read(1)[0] + 1
            data += [bytes([exttype]), bytes([size])] + \
                    list(rng.read(size - 2))]

        elif blocktype == 0x3b:
            # Trailer
            break

        else:
            raise Exception("Invalid block type {}".format(blocktype))

    out.write(header +
              b''.join(data) +
              b'\x3b')

