#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        lzw = bytearray()

        while True:
            if len(lzw) == 0 or rng.read(1)[0] % 2!= 0:
                lzw.append(rng.read(1)[0])

            elif len(lzw) > 1 and rng.read(1)[0] % 16 < len(lzw):
                lzw.append(rng.read(1)[0])

            else:
                break

        lzw += bytes([0x00]) * (8 - ((len(lzw) + 1) & 7))
        lzw[0] |= 0x80
        lzw[-1] |= 0x01
        out.write(bytes(lzw))


if