#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Header
    out.write("GIF89a".encode())  # Magic Number
    width = rng.read(1)[0] % (2**6 + 1)
    height = rng.read(1)[0] % (2**6 + 1)
    flags = b"\x87" if width > 1 else b"\x00"
    out.write((width).to_bytes(1, "little"))
    out.write((height).to_bytes(1, "little"))
    out.write(flags)
    out.write(b"\x00")  # Background Color Index
    out.write(b"\x00")  # Pixel Aspect Ratio

    # Logical Screen Descriptor
    out.write(b"\x2c\x00\x00\x00")  # Image Left Position
    out.write(b"\x00\x00\x00\x00")  # Image Top Position
    out.write(((width * height)).to_bytes(2, "little"))  # Width x Height
    out.write(b"\x00")  # Packed Field
    out.write(b"\xff\xff")  # Global Color Table Size
    out.write(b"\x00\x00\x00\x00")  # Background Color Index
    out.write(b"\x00\x00\x00\x00")  # Pixel Aspect Ratio

    # Global Color Table
    for _ in range(256):
        r = rng.read(1)[0] & 0xf8 | ((rng.read(1)[0]) >> 5)
        g = rng.read(1)[0] & 0xfc | ((rng.read(1)[0]) >> 6)
        b = rng.read(1)[0] & 0xf8 | ((rng.read(1)[0]) >> 5)

        out.write(struct.pack("<BBB", r, g, b))

    # Application Extension Block
    out.write(b"\x21\xff\x0bNETSCAPE2.0")  # Extension Introducer and Label
    out.write(b"\x03\x01\x00\x00")  # Sub-block size and data
    out.write(b"\x00")  # Loop Count

    # Graphics Control Extension Block
    out.write(b"\x21\xf9\x04")  # Extension Introducer and Label
    out.write(b"\x08\x00\x00\x00")  # Sub-block size and data
    out.write(b"\x00")  # Flags
    out.write(b"\x00\x00\x00\x00")  # Delay Time
    out.write(b"\x00\x00")  # Transparent Color Index
    out.write(b"\x00")  # Terminator

    # Image Descriptor
    out.write(b"\x2c\x00\x00\x00")  # Image Left Position
    out.write(b"\x00\x00\x00\x00")  # Image Top Position
    out.write(((width * height)).to_bytes(2, "little"))  # Width x Height
    out.write(b"\x00")  # Packed Field
    out.write(b"\x00\x00")  # Local Color Table Size

    # Image Data
    for y in range(height):
        for x in range(width):
            out.write(b"\x00")

