#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = bytes([
        0x80 | ((width - 1) >> 7),
        (width - 1) & 0xFF,
        0x80 | ((height - 1) >> 7),
        (height - 1) & 0xFF,
        0xF7])
    out.write(packed)

    # Global color table
    palette = [bytes((rng.read(1)[0],)*3) for _ in range(256)]
    out.write(b''.join(palette))

    # Image descriptor
    packed = bytes([
        0x2C,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        (width - 1) & 0xFF,
        (height - 1) & 0xFF,
        0x00,
        0x00])
    out.write(packed)

    # LZW compression
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Trailer
    out.write(b';')