#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = int.from_bytes(rng.read(1), byteorder='little') & 7
    bg     = int.from_bytes(rng.read(1), byteorder='little')
    aspect = int.from_bytes(rng.read(1), byteorder='little')
    out.write(struct.pack('<HHBBB', width, height, flags, bg, aspect))

    # Global color table
    for _ in range(int.from_bytes(rng.read(1), byteorder='little')):
        r = int.from_bytes(rng.read(1), byteorder='little')
        g = int.from_bytes(rng.read(1), byteorder='little')
        b = int.from_bytes(rng.read(1), byteorder='little')
        out.write(struct.pack('BBB', r, g, b))

    # Loop forever until we get to an end-of-file code
    while True:
        data = rng.read(block_size)

        if len(data)!= block_size:
            break

        if block_size == 0:
            continue

        if block_size == 1:
            # Extension block
            ext_type = int.from_bytes(rng.read(1), byteorder='little')

            if ext_type == 0xF9:
                # Graphics control extension
                out.write(ext_type.to_bytes(1, 'little'))
                out.write(b'\x04')
                out.write(rng.read(3))

                # Plain text extension
                out.write((0x01).to_bytes(1, 'little'))
                length = int.from_bytes(rng.read(1), byteorder='little')
                out.write(length.to_bytes(1, 'little'))
                out.write(rng.read(length))

                # End of plain text extension
                out.write(b'\x00')

        elif block_size > 1:
            out.write(data)

        block_size = int.from_bytes(rng.read(1), byteorder='little')