#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header (13 bytes)
    out.write(b'GIF89a')

    # Logical screen descriptor (7 bytes)
    width = 256 + int.from_bytes(rng.read(1), 'big') % 100
    height = 256 + int.from_bytes(rng.read(1), 'big') % 100
    flags = b'\x87\x00' if bool(int.from_bytes(rng.read(1), 'big')) else b'\x07\x00'
    bg_color_index = int.from_bytes(rng.read(1), 'big') & 0xf
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBBB',
        width, height, flags[0], flags[1], bg_color_index, pixel_aspect_ratio))

    # Global color table (3 * 256 = 768 bytes)
    for i in range(0, 256):
        r = int.from_bytes(rng.read(1), 'big') & 0xff
        g = int.from_bytes(rng.read(1), 'big') & 0xff
        b = int.from_bytes(rng.read(1), 'big') & 0xff
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor (10 bytes)
    left = 0
    top = 0
    width = 256 + int.from_bytes(rng.read(1), 'big') % 100
    height = 256 + int.from_bytes(rng.read(1), 'big') % 100
    flags = b'\x87\x00' if bool(int.from_bytes(rng.read(1), 'big')) else b'\x07\x00'
    local_color_table_flag = False
    interlace_flag = False
    sort_flag = False
    reserved = 0
    lct_size = 0
    out.write(struct.pack('<HHHHBBBBB',
        left, top, width, height, flags[0], flags[1],
        local_color_table_flag, interlace_flag, sort_flag, reserved, lct_size))

    # Local color table (3 * 256 = 768 bytes)
    for i in range(0, 256):
        r = int.from_bytes(rng.read(1), 'big') & 0xff
        g = int.from_bytes(rng.read(1), 'big') & 0xff
        b = int.from_bytes(rng.read(1), 'big') & 0xff
        out.write(struct.pack('BBB', r, g, b))

    # Image data block (width * height pixels)
    for y in range(height):
        for x in range(width):
            palette_index = int.from_bytes(rng.read(1), 'big') & 0xf
            out.write(struct.pack('B', palette_index))

