#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB',
        0x200, 0x200, 0xF7, 0x00, 0x00))

    # Write the application extension block
    out.write(b'\x21\xFF\x0BNETSCAPE2.0')
    out.write(b'\x03\x01\x00\x00')
    out.write(b'\xFF\xFF\xFF')

    # Write the global color table
    for i in range(0x200):
        out.write(bytes([rng.read(1)[0]]))

    # Write the first frame
    out.write(b'\x2C')
    out.write(struct.pack('<HHHHB',
        0x200, 0x200, 0x00, 0x00, 0x00))

    # Write the image data
    for i in range(0x4000):
        out.write(bytes([rng.read(1)[0]]))

