#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_random(out: BinaryIO, width: int = 1024, height: int = 768):
    # Header (13 bytes)
    out.write(b'GIF89a')

    # Logical screen descriptor (7 bytes)
    out.write(struct.pack('<HHBBB', width, height, 0x80, 0, 0))

    # Global color table (3 bytes)
    out.write(b'\xFF\xFF\xFF')

    # Image descriptor (10 bytes)
    out.write(struct.pack('<BHHHHBB', 0x2C, 0, 0, width, height, 0, 0))

    # Local color table (3 bytes)
    out.write(b'\xFF\xFF\xFF')

    # Image data block (width * height bytes)
    for i in range(height):
        for j in range(width):
            out.write(bytes([i % 256]))

