#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Graphics control extension
    packed = struct.pack('<BBHBBBB',
        0x21, 0xF9, 0x04, 0x05, 0x00, 0x00, 0x00
    )
    out.write(packed)

    # Application extension
    packed = struct.pack('<BBHBB',
        0x21, 0xFF, 0x0B, 0x4E, 0x45, 0x54
    )
    out.write(packed)
    out.write(b'NETSCAPE2.0')
    out.write(b'\x03\x01\x00')

    # Comment extension
    packed = struct.pack('<BBHB',
        0x21, 0xFE, 0x0C, ord('S'), ord('O')
    )
    out.write(packed)
    out.write(b'Generated by https://github.com/michaelforney/gif-generator')

    # Trailer
    out.write(b';')