#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Write the header
    out.write(b'GIF89a')

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    height = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    out.write(struct.pack('<HHBBB', width, height, 0xF0, 0x00, 0x00))

    # Write the global color table
    for i in range(768):
        out.write(bytes([(i >> ((~i + 1) % 8)) & 0xFF]))

    # Write the application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(768):
        out.write(bytes([(i >> ((~i + 1) % 8)) & 0xFF]))

    # Write the image data
    out.write(b'\x00\x00')

    # Write the LZW compressed image data
    while True:
        code = rng.read(1)[0]
        if code == 0x00:
            break
        elif code <= 0x0f:
            out.write(bytes([code]))
        elif code <= 0x1f:
            out.write(bytes([code - 0x0f]) + rng.read(1))
        elif code <= 0x3f:
            out.write(bytes([code - 0x1f]) + rng.read(2))
        elif code <= 0x7f:
            out.write(bytes([code - 0x3f]) + rng.read(3))
        else:
            raise Exception('Invalid LZW code!')
    # Write the trailer
    out.write(b';')