#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(2)
    height = rng.read(2)
    out.write(width + height)
    out.write(b'\x01')
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\xFF\xFF\xFF')

    # Write the global color table
    for i in range(0x200):
        out.write(bytes([rng.read(1)[0]]))

    # Write the first frame
    out.write(b'\x2C')
    out.write(width + height + b'\x00\x00\x00')

    # Write the image data
    for i in range(int((width[0] << 8) | (width[1])) * int((height[0] << 8) | (height[1]))):
        out.write(bytes([rng.read(1)[0]]))