#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0xF + 16
    height = (rng.read(1)[0] & 0xF) * 256 + rng.read(1)[0] & 0xFF + 16
    out.write(struct.pack('<H', width))
    out.write(bytes([height]))
    out.write(b'\x08')
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\xFF\xFF\xFF')

    # Write the global color table
    for i in range(0x200):
        out.write(bytes([rng.read(1)[0]]))

    # Write the first frame
    out.write(b'\x2C')
    out.write(struct.pack('<HHHHB',
        width, height, 0x00, 0x00, 0x00))

    # Write the image data
    for i in range((width // 8) * (height // 8)):
        out.write(bytes([rng.read(1)[0]]))