#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    height = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    flags = rng.read(1)[0]
    bgColorIndex = rng.read(1)[0]
    aspectRatio = rng.read(1)[0]
    out.write(struct.pack('<H', width))
    out.write(bytes([height]))
    out.write(bytes([flags]))
    out.write(bytes([bgColorIndex]))
    out.write(bytes([aspectRatio]))

    # Write the global color table
    for i in range(0x200):
        out.write(bytes([rng.read(1)[0]]))

    # Write the first frame
    lctSize = rng.read(1)[0]
    lctFlags = rng.read(1)[0]
    xPos = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    yPos = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    w = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    h = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    lctSize = min(lctSize, 0xF)
    lctFlags = min(lctFlags, 0xF)
    out.write(b'\x2C')
    out.write(struct.pack('<HHHHB',
        w, h, xPos, yPos, lctFlags))

    # Write the image data
    for i in range(w * h):
        out.write(bytes([rng.read(1)[0]]))