#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for _ in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Graphics control extension
    packed = struct.pack('<BBHBBBBB',
        0x21, 0xF9, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00
    )
    out.write(packed)

    # Application extension
    packed = struct.pack('<BBBBB',
        0x21, 0xFF, 0x0B, 0x4E, 0x45, 0x54, 0x53, 0x43,
        0x41, 0x50, 0x45, 0x32, 0x2E, 0x30, 0x03, 0x01,
        0x00, 0x00, 0x00
    )
    out.write(packed)

    # Comment extension
    comment = bytes([rng.read(1)[0] % 256]) * (rng.read(1)[0] % 100)
    packed = struct.pack('<BB%dsB' % len(comment),
        0x21, 0xFE, len(comment), comment, 0x00
    )
    out.write(packed)

    # Plain text extension
    plain_text = bytes([rng.read(1)[0] % 256]) * (rng.read(1)[0] % 100)
    packed = struct.pack('<BB%dsB' % len(plain_text),
        0x21, 0x01, len(plain_text), plain_text, 0x00
    )
    out.write(packed)

    # Trailer
    out.write(b'\x3B')