#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    height = rng.read(1)[0] & 0x7F | (rng.read(1)[0] & 0x7F) << 8
    out.write(struct.pack('<HHBBB', width, height, 0xF0, 0x00, 0x00))

    # Write the global color table
    for i in range(768):
        out.write(bytes([(i >> ((~i + 1) % 8)) & 0xFF]))

    # Write the application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(768):
        out.write(bytes([(i >> ((~i + 1) % 8)) & 0xFF]))

    # Write the image data
    out.write(b'\x00\x00')

    # Write the LZW compressed image data
    while True:
        code = rng.read(1)[0]
        if code == 0x00:
            break

        elif code >= 0x01 and code <= 0x0B:
            out.write(bytes([code]))

            for i in range(code):
                out.write(bytes([rng.read(1)[0]]))

        elif code >= 0x0E and code <= 0x1A:
            out.write(bytes([code]))

            for i in range(2 ** (code - 0x0E)):
                out.write(bytes([rng.read(1)[0]]))

        elif code == 0x1B:
            out.write(bytes([code]))

            for i in range(5):
                out.write(bytes([rng.read(1)[0]]))

        elif code == 0x1C or code == 0x1D:
            out.write(bytes([code]))

            for i in range(9):
                out.write(bytes([rng.read(1)[0]]))

        elif code == 0x1E or code == 0x1F:
            out.write(bytes([code]))

            for i in range(11):
                out.write(bytes([rng.read(1)[0]]))

        elif code >= 0x20 and code <= 0x3F:
            out.write(bytes([code]))

            for i in range(1 + (code & 1), 4 + 2 ** ((code & ~1) >> 1)):
                out.write(bytes([rng.read(1)[0]]))

        elif code >= 0x40 and code <= 0xFE:
            out.write(bytes([code]))

            for i in range(3):
                out.write(bytes([rng.read(1)[0]]))

        elif code == 0xFF:
            out.write(bytes([code]))

            for i in range(19):
                out.write(bytes([rng.read(1)[0]]))

    # Write the trailer
    out.write(b';')
