#!/usr/bin/env python3

import sys, os, re, json, argparse, subprocess, time
from collections import defaultdict
from itertools import chain
from multiprocessing import Pool

def parse_args():
    parser = argparse.ArgumentParser(description='')
    parser.add_argument('--input', type=str)
    parser.add_argument('--output', type=str)
    return parser.parse_args()

def main():
    args = parse_args()
    
    with open(args.input, 'r') as f:
        data = json.load(f)
        
    # get all the unique words in the dataset
    word2id = {}
    for i, (text, label) in enumerate(data):
        text = text.lower().split()
        for w in text:
            if w not in word2id:
                word2id[w] = len(word2id) + 1
                
    # create a new dataset where each word is replaced by its id
    new_data = []
    for i, (text, label) in enumerate(data):
        text = [word2id[w] for w in text.lower().split()]
        new_data.append((text, label))
            
    # save the new dataset and the mapping from ids to words
    with open(args.output, 'w') as f:
        json.dump(new_data, f)
    with open(os.path.join(os.path.dirname(args.output), 'word2id.json'), 'w') as f:
        json.dump(word2id, f)
