#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(out: BinaryIO, rng: callable):
    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack("<HHBB", 1024, 768, 0xF, 0))

    # Global color table
    for i in range(1 << 5):
        out.write(bytes([i * 2, i * 2 + 1, i * 2]))

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00')

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\x00')

    # Local color table
    for i in range(1 << 5):
        out.write(bytes([i * 2, i * 2 + 1, i * 2]))

    # Raster data
    for y in range(768):
        for x in range(1024):
            out.write(bytes([rng() % 256]))

