#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 16 + rng.read(2)[0] % 15
    height = 16 + rng.read(2)[0] % 15
    out.write(struct.pack('<HHB', width, height, 0xF))

    # Global color table
    for i in range(4):
        out.write(bytes([i * 0xFF // 3]))

    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Image block
    out.write(b'\x2c\x00\x00\x00\x00')
    out.write(struct.pack('<HHBB', width, height, 0, 0))
    out.write(b'\x07\x00')

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([y * x % 4]))
