#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Write the header.
    out.write(b'GIF89a')  # Magic number.
    out.write(struct.pack('<H', width))  # Width.
    out.write(struct.pack('<H', height))  # Height.
    out.write(b'\x00\x00')  # Background color index.
    out.write(b'\x00')  # Aspect ratio (unused).
    
    # Write the global color table.
    for i in range(256):
        r = g = b = i
        out.write(bytes([r, g, b]))
        
    # Write the image descriptor.
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\xff\xff\xff\xff')
    
    # Write the local color table.
    for i in range(256):
        r = g = b = i
        out.write(bytes([r, g, b]))
    
    # Write the pixel data.
    for y in range(height):
        for x in range(width):
            out.write(bytes([y % 256]))
            
    # Write the trailer.
    out.write(b';')
