#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    width = 16 + (next(rng) & 0xff) % 256
    height = 16 + (next(rng) & 0xff) % 256
    background_color_index = next(rng) & 0xff
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBB', width, height, background_color_index, pixel_aspect_ratio))
    
    # Global color table
    for i in range(7):
        red = next(rng) & 0xff
        green = next(rng) & 0xff
        blue = next(rng) & 0xff
        
        out.write(struct.pack('<BBB', red, green, blue))
    
    # Image descriptor
    left = 0
    top = 0
    width = 16 + (next(rng) & 0xff) % 256
    height = 16 + (next(rng) & 0xff) % 256
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    reserved_local_color_table_size = 0
    out.write(struct.pack('<HHHBBBB', left, top, width, height, local_color_table_flag, interlace_flag, sort_flag, reserved_local_color_table_size))
    
    # Local color table
    for i in range(7):
        red = next(rng) & 0xff
        green = next(rng) & 0xff
        blue = next(rng) & 0xff
        
        out.write(struct.pack('<BBB', red, green, blue))
    
    # LZW minimum code size
    lzw_minimum_code_size = 8
    out.write(struct.pack('<B', lzw_minimum_code_size))
    
    # Raster data
    while True:
        if next(rng) & 0xff == 0:
            break
    
    # Trailer
    out.write(b';')
