#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f
        out.write(struct.pack('BBB', r, g, b))

    # Application extension
    ext_type = struct.pack('B', 0xFF)
    app_id   = 'FUZZ'.encode()
    block_size = struct.pack('B', len(app_id))
    out.write(ext_type + block_size + app_id)

    # Image descriptor
    desc = struct.pack('B', 0x2C)
    left = struct.pack('>H', 0)
    top  = struct.pack('>H', 0)
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H', 768)
    flags  = struct.pack('B', 0x00)
    out.write(desc + left + top + width + height + flags)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    return True


