#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main(argv):
    if len(argv) < 2:
        print("Usage: {} <file>".format(os.path.basename(__file__)))
        return 1
    
    with open(argv[1], "rb") as f:
        data = f.read()
        
    # Read the header
    magic, version, entry_count = struct.unpack("<4sLL", data[:16])
    assert magic == b"MHDR"
    assert version == 0x00010000
    
    # Read the entries
    offset = 16
    for i in range(entry_count):
        name_len, name_offset, file_size, file_offset = struct.unpack("<LQQL", data[offset : offset + 28])
        name = data[name_offset : name_offset + name_len].decode().rstrip("\x00")
        print("{}: size={} offset={}".format(name, file_size, file_offset))
        offset += 28
    
    return 0
