#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a' + rng.read(7)
    width, height = struct.unpack('<HH', rng.read(4))
    assert len(header) == 10
    assert width > 0 and height > 0
    
    # Write the header to the output
    out.write(header)

    # Write the global color table
    out.write(b'\xFF\xFF\xFF')

    # Write the graphic control extension
    out.write(struct.pack('<BBBH', 0x21, 0xf9, 4, 0))

    # Write the image descriptor
    out.write(struct.pack('<BBHHBB', 0x2c, 0, 0, width, height, 0))

    # Write the local color table
    out.write(b'\xFF\xFF\xFF')

    # Write the image data
    for _ in range(width * height):
        out.write(struct.pack('<BB', 0, 0))

    # Write the trailer
    out.write(b';')
