#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (os.urandom(1)[0] % 64) + 128
    height = (os.urandom(1)[0] % 64) + 128
    flags  = bytes([
        0x80 | ((os.urandom(1)[0] & 7) << 5),
        (os.urandom(1)[0] & 0x01) << 7,
        (os.urandom(1)[0] & 0x01) << 6])
    bgcol  = bytes([os.urandom(1)[0], os.urandom(1)[0], os.urandom(1)[0]])
    pxsize = bytes([os.urandom(1)[0]])
    out.write(struct.pack('>HHBBB', width, height, flags[0], flags[1], flags[2]))
    out.write(bgcol)
    out.write(pxsize)

    # Global color table
    gctlen = (os.urandom(1)[0] % 256) + 1
    for i in range(gctlen):
        r = os.urandom(1)[0]
        g = os.urandom(1)[0]
        b = os.urandom(1)[0]
        out.write(bytes([r, g, b]))

    # Application extension
    appext = b'\x21\xFF\x0BNETSCAPE2.0\x03\x01'
    out.write(appext)
    out.write(b'\x00')

    # Graphic control extension
    gcxtra = b'\x21\xF9\x04\x01\x00\x00\x00'
    delay = struct.pack('<H', (os.urandom(1)[0] % 65536))
    trans = bytes([os.urandom(1)[0]])
    out.write(gcxtra)
    out.write(delay)
    out.write(trans)

    # Plain text extension
    txtext = b'\x21\x01\x04\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x0        0x80 | ((os.urandom(1)[0] & 7) << 5),
        (os.urandom(1)[0] & 0x01) << 7,
        (os.urandom(1)[0] & 0x01) << 6])
    lctlen = (os.urandom(1)[0] % 256) + 1
    interl = bytes([os.urandom(1)[0]])
    out.write(struct.pack('>HHBBBBBB', left, top, width, height, flags[0], flags[1], flags[2], lctlen))
    if lctlen > 0:
        for i in range(lctlen):
            r = os.urandom(1)[0]
            g = os.urandom(1)[0]
            b = os.urandom(1)[0]
            out.write(bytes([r, g, b]))
    out.write(interl)

    # Image data
    datalen = (os.urandom(1)[0] % 1024) + 1
    out.write(rng.read(datalen))

    # Trailer
    out.write(b';')

