#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Magic bytes
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 2 ** (1 + rng.read(1)[0] % 7)
    height = 2 ** (1 + rng.read(1)[0] % 7)
    color_table_size = 2 ** (1 + rng.read(1)[0] % 4)
    background_color_index = rng.read(1)[0] % color_table_size
    pixel_aspect_ratio = rng.read(1)[0] % 16
    out.write(struct.pack('<HHBB', width, height, color_table_size, background_color_index))

    # Global color table
    for _ in range(color_table_size):
        out.write(rng.read(3))

    # Image descriptor
    left = rng.read(2)[:1]
    top = rng.read(2)[:1]
    width = rng.read(2)[:1]
    height = rng.read(2)[:1]
    local_color_table_flag = bool(rng.read(1)[0] & 0x80)
    interlace_flag = bool(rng.read(1)[0] & 0x40)
    sort_flag = bool(rng.read(1)[0] & 0x20)
    reserved = bool(rng.read(1)[0] & 0x10)
    size_of_local_color_table = bool(rng.read(1)[0] & 0x08)
    out.write(left + top + width + height + bytes([local_color_table_flag | interlace_flag << 1]))

    if local_color_table_flag:
        # Local color table
        for _ in range(2 ** size_of_local_color_table):
            out.write(rng.read(3))

    # LZW minimum code size
    out.write(bytes([rng.read(1)[0] % 9]))

    # Image data
    while True:
        block_size = rng.read(1)[0]
        if block_size == 0:
            break
        out.write(rng.read(block_size))
