#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical screen descriptor
    width = 2**10 * (struct.unpack('B', rng.read(1))[0] % 7 + 1)
    height = 2**10 * (struct.unpack('B', rng.read(1))[0] % 5 + 1)
    packed = struct.pack('<HHBBB', width, height, 0xf0 | 0x07, 0, 0)
    out.write(packed)

    # Global color table
    for i in range(4):
        rgb = [struct.unpack('B', rng.read(1))[0] for _ in range(3)]
        out.write(bytes([rgb[i] for i in reversed(range(3))]))

    # Image descriptor
    packed = struct.pack('<HHBB', 0, 0, 0xf0 | 0x07, 0)
    out.write(packed)

    # Local color table
    for i in range(4):
        rgb = [struct.unpack('B', rng.read(1))[0] for _ in range(3)]
        out.write(bytes([rgb[i] for i in reversed(range(3))]))

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', struct.unpack('B', rng.read(1))[0] & 0x0f))

    # Trailer
    out.write(b'\x3b')

