#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (os.urandom(1)[0] % 64) + 128
    height = (os.urandom(1)[0] % 64) + 128
    flags  = bytes([
        0x80 | ((os.urandom(1)[0] & 7) << 5),
        (os.urandom(1)[0] & 0x01) << 7,
        (os.urandom(1)[0] & 0x01) << 6])
    bgcol  = bytes([os.urandom(1)[0], os.urandom(1)[0], os.urandom(1)[0]])
    pxsize = bytes([os.urandom(1)[0]])
    out.write(struct.pack('>HHBBB', width, height, flags[0], flags[1], flags[2]))
    out.write(bgcol)
    out.write(pxsize)

    # Global color table
    gctlen = (os.urandom(1)[0] % 256) + 1
    for i in range(gctlen):
        r = os.urandom(1)[0]
        g = os.urandom(1)[0]
        b = os.urandom(1)[0]
        out.write(bytes([r, g, b]))

    # Image descriptor
    left   = os.urandom(1)[0] % width
    top    = os.urandom(1)[0] % height
    width  = (os.urandom(1)[0] % min(width - left, 64)) + 1
    height = (os.urandom(1)[0] % min(height - top, 64)) + 1
    flags  = bytes([
        0x80 | ((os.urandom(1)[0] & 7) << 5),
        (os.urandom(1)[0] & 0x01) << 7,
        (os.urandom(1)[0] & 0x01) << 6])
    lctlen = (os.urandom(1)[0] % 256) + 1
    out.write(struct.pack('>HHBBB', left, top, width, height, flags[0]))
    out.write(flags[1:])
    out.write(bytes([lctlen]))

    # Local color table
    for i in range(lctlen):
        r = os.urandom(1)[0]
        g = os.urandom(1)[0]
        b = os.urandom(1)[0]
        out.write(bytes([r, g, b]))

    # Raster data
    for y in range(height):
        for x in range(width):
            idx = os.urandom(1)[0] % (lctlen + 1)
            out.write(bytes([idx]))

    # Trailer
    out.write(b'\x3b')

