#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng):
    # 10 bytes of header
    out.write(b'GIF89a')
    width = rng.randint(256, 4096)
    height = rng.randint(256, 4096)
    out.write(struct.pack('<HHB', width, height, 0x80))
    out.write(b'\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF')
    
    # 7 bytes of global color table
    for i in range(3 * 256):
        out.write(bytes([rng.getrandbits(8)]))
        
    # 1 byte of image descriptor
    out.write(b'\x2C')
    
    # 9 bytes of image descriptor
    out.write(struct.pack('<HHBB', 0, 0, width, height))
    out.write(b'\x00')
    
    # 1 byte of local color table flag
    out.write(b'\x80')
    
    # 1 byte of code size
    out.write(b'\x00')
    
    # 1 byte of block terminator
    out.write(b'\x00')
    
    # 1 byte of trailer
    out.write(b';')
