#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # 0x47494638 is "GIF8" in ASCII
    out.write(b'GIF8')

    # Read 2 bytes of random data and write them as width and height
    width = int.from_bytes(rng.read(2), 'little', signed=False)
    height = int.from_bytes(rng.read(2), 'little', signed=False)
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))

    # Write a single byte with value 0xF7 (color table size -1)
    out.write(b'\xf7')

    # Write 5 bytes of random data to be interpreted as background color index
    out.write(rng.read(5))

    # Write a single byte with value 0x00 (no pixel aspect ratio)
    out.write(b'\x00')

    # Write 256*3 bytes of random data to be interpreted as color table
    out.write(rng.read(256 * 3))

    # Write a single byte with value 0x2C (image descriptor)
    out.write(b'\x2c')

    # Write 2 bytes of random data to be interpreted as image left position
    out.write(rng.read(2))

    # Write 2 bytes of random data to be interpreted as image top position
    out.write(rng.read(2))

    # Write 2 bytes of random data to be interpreted as image width
    out.write(rng.read(2))

    # Write 2 bytes of random data to be interpreted as image height
    out.write(rng.read(2))

    # Write a single byte with value 0x00 (no local color table)
    out.write(b'\x00')

    # Write a single byte with value 0x00 (no interlacing)
    out.write(b'\x00')

    # Write 256*3 bytes of random data to be interpreted as image data
    out.write(rng.read(width * height * 3))

    # Write a single byte with value 0x3B (trailer)
    out.write(b'\x3b')
