#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f
        out.write(struct.pack('BBB', r, g, b))

    # Application extension block
    appext_size = struct.pack('B', 11)
    appext_id   = struct.pack('B', 0xff)
    appext_data = struct.pack('!I', 0xdeadbeef)
    out.write(appext_size + appext_id + appext_data)

    # Image descriptor
    idesc_left   = struct.pack('>H', 0)
    idesc_top    = struct.pack('>H', 0)
    idesc_width  = struct.pack('>H', 1024)
    idesc_height = struct.pack('>H',  768)
    idesc_flags  = struct.pack('B',   0x80 | 0x08)
    out.write(idesc_left + idesc_top + idesc_width + idesc_height + idesc_flags)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f
        out.write(struct.pack('BBB', r, g, b))

    # Graphic control extension
    gce_block_size = struct.pack('B', 4)
    gce_flags      = struct.pack('B', 0xf0)
    gce_delay      = struct.pack('<H', 0xffff)
    gce_transp     = struct.pack('B', 0xff)
    out.write(gce_block_size + gce_flags + gce_delay + gce_transp)

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    # Trailer
    out.write(b'\x3b')

