#!/usr/bin/env python3
import sys
import struct
import os
from random import Random

if len(sys.argv) != 2:
    print("Usage: {} <output_file>".format(sys.argv[0]), file=sys.stderr)
    exit(-1)

out = open(sys.argv[1], 'wb')

# Header
out.write(b'GIF89a')

# Logical screen descriptor
width  = (os.urandom(1)[0] % 1024) + 1
height = (os.urandom(1)[0] % 1024) + 1
flags  = bytes([
    0x80 | ((os.urandom(1)[0] & 7) << 5),
    (os.urandom(1)[0] & 0x01) << 7,
    (os.urandom(1)[0] & 0x01) << 6])
bgcolor = os.urandom(1)[0]
aspect  = os.urandom(1)[0]
out.write(struct.pack('<HHB3BB', width, height, flags[0], flags[1], flags[2], bgcolor, aspect))

# Global color table
for i in range((flags[0] >> 7) + 1):
    r = os.urandom(1)[0]
    g = os.urandom(1)[0]
    b = os.urandom(1)[0]
    out.write(bytes([r, g, b]))

# Application extension
out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')

# Loop count
out.write(struct.pack('<H', 0))

# Data sub-blocks
while True:
    # Graphic control extension
    flags = bytes([
        (os.urandom(1)[0] & 0x07) << 2,
        (os.urandom(1)[0] & 0x01) << 1,
        (os.urandom(1)[0] & 0x01) << 0])
    delay = (os.urandom(1)[0] % 100) * 10
    trans = os.urandom(1)[0]
    out.write(struct.pack('>BBB', 0xf9, flags[0], delay))
    if flags[0] & 0x01:
        out.write(bytes([trans]))

    # Color table
    for i in range((flags[0] >> 2) + 1):
        r = os.urandom(1)[0]
        g = os.urandom(1)[0]
        b = os.urandom(1)[0]
        out.write(bytes([r, g, b]))

    # Image descriptor
    left   = os.urandom(1)[0] % width
    top    = os.urandom(1)[0] % height
    width  = (os.urandom(1)[0] % min(width - left, 64)) + 1
    height = (os.urandom(1)[0] % min(height - top, 64)) + 1
    flags  = bytes([
        0x80 | ((os.urandom(1)[0] & 7) << 5),
        (os.urandom(1)[0] & 0x01) << 7,
        (os.urandom(1)[0] & 0x01) << 6])
    lctlen = (os.urandom(1)[0] % 256) + 1
    interl = bytes([os.urandom(1)[0]])
    out.write(struct.pack('>HHBBBBBB', left, top, width, height, flags[0], flags[1], flags[2], lctlen))
    if lctlen > 0:
        for i in range(lctlen):
            r = os.urandom(1)[0]
            g = os.urandom(1)[0]
            b = os.urandom(1)[0]
            out.write(bytes([r, g, b]))
    out.write(interl)

    # Image data
    datalen = (os.urandom(1)[0] % 1024) + 1
    out.write(rng.read(datalen))

    # Trailer
    out.write(b';')

