#!/usr/bin/env python3

import os
import random
import struct
import sys
from typing import BinaryIO


def generate_random_gif(rng: BinaryIO, out: BinaryIO):
    # Header
    header = b'GIF89a' + b'\x01\x00' * 7 + b'\xff\xff\xff'
    out.write(header)

    # Logical Screen Descriptor
    lsd = struct.pack('<HHBBB', 256, 256, 0xf0, 0, 0)
    out.write(lsd)

    # Image Descriptor
    idesc = struct.pack('<HHHBB', 0, 0, 256, 256, 0)
    out.write(idesc)

    # Local Color Table
    ctab = bytes([0xff for _ in range(768)])
    out.write(ctab)

    # Data Sub-Blocks
    data = bytearray()
    while True:
        nbytes = random.randint(1, 254)
        data += rng.read(nbytes)
        out.write(data)

    # Trailer
    trailer = bytes([0x3b])
    out.write(trailer)


if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1] == '-':
        rng = sys.stdin.buffer
    else:
        try:
            rng = open('/dev/urandom', 'rb')
        except FileNotFoundError:
            rng = open(os.path.join('/', 'dev', 'urandom'), 'rb')
    with open('out.gif', 'wb') as f:
        generate_random_gif(rng, f)