#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Read the length of the data to be written
    datalen = int.from_bytes(rng.read(10), 'big', signed=False)

    # Write the header
    out.write(b'GIF89a\x01\x00\x01\x00\x00\xff\x00,\x00\x00\x00')

    while datalen > 0:
        # Read the length of the next chunk
        chunklen = int.from_bytes(rng.read(2), 'big', signed=False)
        chunk = rng.read(chunklen)

        if len(chunk)!= chunklen:
            raise RuntimeError('Unexpected end of input')

        out.write(chunk)
        datalen -= chunklen

