#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Application extension
    app_ext_block_size = struct.pack('B', 0x0b)
    app_ext_app_id     = struct.pack('!I', 0x4e455447)
    app_ext_auth_code  = struct.pack('!I', 0x20434452)
    app_ext_data       = struct.pack('!I', 0x00000000)
    out.write(app_ext_block_size + app_ext_app_id + app_ext_auth_code + app_ext_data)

    # Image block
    image_separator      = struct.pack('B', 0x2c)
    image_left           = struct.pack('>H', 0)
    image_top            = struct.pack('>H', 0)
    image_width          = struct.pack('>H', 1024)
    image_height         = struct.pack('>H', 768)
    image_flags          = struct.pack('B', 0x80)
    local_color_table    = struct.pack('B', 0x00)
    interlaced           = struct.pack('B', 0x00)
    sort_flag            = struct.pack('B', 0x00)
    size_of_global_ct    = struct.pack('B', 0x00)
    out.write(image_separator + image_left + image_top + image_width + image_height + image_flags + local_color_table + interlaced + sort_flag + size_of_global_ct)

    # LZW minimum code size
    lzw_minimum_code_size = struct.pack('B', 0x08)
    out.write(lzw_minimum_code_size)

    # Random data
    while True:
        byte = ord(rng.read(1))
        if byte != 0x00:
            out.write(struct.pack('B', byte))
        else:
            break

    # Trailer
    out.write(b'\x3b')

