#!/usr/bin/env python3

import os
import sys
import random
import struct

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: {} <output_file>".format(sys.argv[0]))
        exit(-1)

    output_path = sys.argv[1]

    with open('/dev/urandom', 'rb') as f:
        rng = f.read()

    out = open(output_path, 'wb')

    # Magic number
    magic = b'GIF89a'
    out.write(magic)

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHHBB', width, height, 0, 0, 0)
        out.write(packed)

    # Image descriptor
    left   = 0
    top    = 0
    width  = 1024
    height = 768
    packed = struct.pack('>HHHHB', left, top, width, height, 0)
    out.write(packed)

    # Local color table
    for i in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

