#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = 1024
    height = 768
    background_color = 0xff0000
    pixel_aspect_ratio = 0
    global_color_table_flag = 0
    color_resolution = 7
    sort_flag = 0
    size_of_global_color_table = 7
    background_color_index = 0
    pixel_data_subblocks = []

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(struct.pack('B', background_color))
    out.write(struct.pack('B', pixel_aspect_ratio))
    out.write(struct.pack('B', (
            global_color_table_flag << 7 |
            color_resolution << 4 |
            sort_flag << 3 |
            size_of_global_color_table
        )))
    out.write(struct.pack('B', background_color_index))

    # Write the global color table
    for i in range(2**size_of_global_color_table):
        out.write(struct.pack('BBB', *[
                int.from_bytes(rng.read(1), 'big')
                for _ in range(3)]))

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(2**size_of_global_color_table):
        out.write(struct.pack('BBB', *[
                int.from_bytes(rng.read(1), 'big')
                for _ in range(3)]))

    # Write the LZW minimum code size
    out.write(struct.pack('B', 8))

    # Write the pixel data subblocks
    while True:
        block_length = int.from_bytes(rng.read(1), 'big')
        if block_length == 0:
            break
        pixel_data_subblocks.append(rng.read(block_length))
    out.write(b''.join([
            struct.pack('B', len(pixel_data_subblock)),
            pixel_data_subblock
            for pixel_data_subblock in pixel_data_subblocks]))

    # Write the trailer
    out.write(b';')
