#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for i in range(3 * 256):
        out.write(struct.pack('B', rng.read(1)[0]))

    # Graphics control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\xff\x00')

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Local color table
    for i in range(3 * 256):
        out.write(struct.pack('B', rng.read(1)[0]))

    # Raster data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', rng.read(1)[0]))
