#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            continue

        out.write(struct.pack('BBB', r, g, b))

    # Graphic control extension
    out.write(struct.pack('BB', 0x21, 0xf9))
    out.write(struct.pack('B',  0x04))
    out.write(struct.pack('<H', 0xffff))
    out.write(struct.pack('BB', 0x00, 0x00))

    # Image descriptor
    out.write(struct.pack('BB', 0x2c, 0x00))
    out.write(struct.pack('>HH', 0, 0))
    out.write(struct.pack('>HH', 1024, 768))
    out.write(struct.pack('BB', 0x00, 0x00))

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            continue

        out.write(struct.pack('BBB', r, g, b))

    # LZW minimum code size
    out.write(struct.pack('B', 0x08))

    # Pixel data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', 0x00))

    # Trailer
    out.write(struct.pack('B', 0x3b))

