#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# with the given width, height, color depth and background.
def generate_gif(out: BinaryIO, rng: BinaryIO, width: int = 1024,
                 height: int = 768, depth: int = 8, bg: bytes = b'\x00'):
    # Header (13 bytes).
    out.write(b'GIF89a')

    # Logical screen descriptor (7 bytes).
    out.write(struct.pack('<HHBBB', width, height, depth, 0, 0, bg))

    # Global color table (3 * 256 bytes).
    for i in range(256):
        out.write(bg if i == 0 else rng.read(depth // 8))

    # Image descriptor (10 bytes).
    out.write(struct.pack('<BHHBB', 0x2c, 0, 0, 0, 0, 0))

    # Local color table (3 * 256 bytes).
    for i in range(256):
        out.write(bg if i == 0 else rng.read(depth // 8))

    # Image data block (width * height bytes).
    for _ in range(height):
        out.write(rng.read(width * ((depth + 7) // 8)))

