#!/usr/bin/env python3

import os
import sys
import struct
from pathlib import Path


def main():
    if len(sys.argv) < 2:
        print("Usage: {} <file>".format(Path(sys.argv[0]).name))
        return

    with open(sys.argv[1], "rb") as f:
        data = bytearray(f.read())

        # Check for a valid header
        magic, version, num_entries = struct.unpack("<III", data[:12])
        assert magic == 0x4D5A6978 and version in (0x00000002, 0x00000003), \
            "Invalid PE file"

        # Find the export directory entry
        base = None
        for i in range(num_entries):
            offset = 12 + i * 8
            name_rva, rva = struct.unpack("<II", data[offset : offset+8])

            if name_rva == 0 and rva!= 0:
                base = rva - 0x10
                break

        if not base:
            raise RuntimeError("Export directory not found")

        # Read the names of all exported functions
        names = []
        for i in range(data[base] // 4):
            addr = base + 4 + i*4
            func_addr = struct.unpack("<I", data[addr : addr+4])[0]
            names.append((func_addr, data[addr+4 :].split(b"\x00")[0]))

        # Print them out
        for addr, name in sorted(names):
            print("{:#010x} {}".format(addr, name.decode()))


if