#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for i in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Graphics control extension
    packed = struct.pack('>BBHBBBB', 0x21, 0xF9, 4, 0xFF, 0, 0)
    out.write(packed)

    # Plain text extension
    packed = struct.pack('>BBHBB', 0x21, 0x01, 11, ord('A'), ord('N'))
    out.write(packed)
    out.write(b'Animation by <NAME>')

    # Application extension
    packed = struct.pack('>BBHBB', 0x21, 0xFF, 11, ord('X'), ord('F'))
    out.write(packed)
    out.write(b'https://github.com/dbrgn/random-gif')

    # Graphic control label
    packed = struct.pack('>BB', 0x2C, 0)

    # Frame header
    left   = 0
    top    = 0
    width  = 1024
    height = 768
    packed = struct.pack('>HHHHB', left, top, width, height, 0)
    out.write(packed)

    # Local color table
    for i in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

