#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_gif(out: BinaryIO, width: int = 1024, height: int = 768):
    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    out.write(struct.pack('!HHBBB', width, height, 0xF0, 0x00, 0x00))

    # Global Color Table (black)
    for i in range(0, 256):
        out.write(struct.pack('B', 0x00))

    # Image Descriptor
    out.write(struct.pack('!BBHHHBB', 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

    # Local Color Table (black)
    for i in range(0, 256):
        out.write(struct.pack('B', 0x00))

    # Graphic Control Extension
    out.write(struct.pack('!BBHB', 0x21, 0xF9, 0x04, 0x00))

    # Application Extension
    out.write(struct.pack('!BBHBB', 0xFF, 0x0B, 0x4E, 0x45, 0x54))
    out.write(b'NETSCAPE2.0\x03\x01\x00\x00')

    # Graphics Control Block
    out.write(struct.pack('!BBBBHH', 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

    # Image Data
    for y in range(height):
        for x in range(width):
            if ((y & 0xFF) + (x >> 8)) % 2 == 0:
                out.write(struct.pack('B', 0x00))
            else:
                out.write(struct.pack('B', 0xFF))

