#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):
	# Header
	out.write(b"GIF89a")

	# Logical screen descriptor
	width = 256 + (rng.read(1)[0] % 128)
	height = 256 + (rng.read(1)[0] % 128)
	flags = b"\x87\x00" # Global color table, sorted
	bgcolor = bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]])
	aspect = bytes([48, 0])
	out.write(struct.pack("<HHBBB", width, height, flags[0], flags[1], bgcolor[0], bgcolor[1]))
	out.write(aspect)

	# Global color table
	for i in range(0, 256 * 3 // 4):
		out.write(bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]]))
	
	# Image descriptor
	left = 0
	top = 0
	width = 256 + (rng.read(1)[0] % 128)
	height = 256 + (rng.read(1)[0] % 128)
	flags = b"\x80\x00" # Local color table, not interlaced
	out.write(struct.pack("<HHHHB", left, top, width, height, flags[0]))

	# Local color table
	for i in range(0, 256 * 3 // 4):
		out.write(bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]]))

	# LZW minimum code size
	out.write(b"\x08")

	# Frame data
	# TODO: Implement this
