#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 768, delay: int = 50
) -> None:

    # Header
    out.write(b"GIF89a")
    out.write(struct.pack("<HH", width, height))
    out.write(b"\x87\x00\x00")

    # Logical screen descriptor
    out.write(b"\x2c\x00\x00")
    out.write(struct.pack("<HH", width, height))
    out.write(b"\x00\x00")
    out.write(b"\x00")

    # Global color table
    for i in range(256):
        out.write(bytes([i * 255 // (256 - 1), 0, 0]))

    # Image descriptor
    out.write(b"\x2c\x00\x00")
    out.write(struct.pack("<HH", width, height))
    out.write(b"\x00\x00")
    out.write(b"\x00")
    out.write(b"\x00\x00\x00\x00")
    out.write(struct.pack("<B", 0))

    # Local color table
    for i in range(256):
        out.write(bytes([i * 255 // (256 - 1), 0, 0]))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([(y + x) % 256]))

    # Trailer
    out.write(b";")


if