#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (os.urandom(1)[0] % 64) + 128
    height = (os.urandom(1)[0] % 64) + 128
    flags  = bytes([
        0x80 | ((os.urandom(1)[0] & 7) << 5),
        (os.urandom(1)[0] & 0x01) << 7,
        (os.urandom(1)[0] & 0x01) << 6])
    bgcol  = bytes([os.urandom(1)[0], os.urandom(1)[0], os.urandom(1)[0]])
    pxsize = bytes([os.urandom(1)[0]])
    out.write(struct.pack('>HHBBB', width, height, flags[0], flags[1], flags[2]))
    out.write(bgcol)
    out.write(pxsize)

    # Global color table
    lctlen = os.urandom(1)[0] & 7
    left   = bytes([os.urandom(1)[0]])
    top    = bytes([os.urandom(1)[0]])
    width  = bytes([os.urandom(1)[0]])
    height = bytes([os.urandom(1)[0]])
    flags  = bytes([
        0x80 | ((os.urandom(1)[0] & 7) << 5),
        (os.urandom(1)[0] & 0x01) << 7,
        (os.urandom(1)[0] & 0x01) << 6])
    interl = bytes([os.urandom(1)[0]])
    out.write(struct.pack('>HHBBBBBB', left, top, width, height, flags[0], flags[1], flags[2], lctlen))
    if lctlen > 0:
        for i in range(lctlen):
            r = os.urandom(1)[0]
            g = os.urandom(1)[0]
            b = os.urandom(1)[0]
            out.write(bytes([r, g, b]))
    out.write(interl)

    # Image data
    datalen = (os.urandom(1)[0] % 1024) + 1
    out.write(rng.read(datalen))

    # Trailer
    out.write(b';')

