#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b"GIF89a")

    # Logical screen descriptor
    width = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1
    packed = (
        ((width - 1) & 0x7f) |
        0x80 |
        (((height - 1) & 0x7f) << 8)
    )
    out.write(struct.pack("<H", packed))
    out.write(b"\x00\x00")

    # Global color table
    for i in range(256):
        out.write(rng.read(3))

    # Frame control block
    out.write(b"\x2c")
    out.write(struct.pack("<HHB", 0, 0, 0))
    out.write(b"\x00")

    # Image data
    lzw_mincodesize = rng.read(1)[0] % 256 + 1
    out.write(lzw_mincodesize.to_bytes(1, "little"))

    # LZW code size table
    for i in range(lzw_mincodesize):
        out.write((i).to_bytes(1, "little"))

    # LZW encoded image data
    while True:
        b = rng.read(1)[0]

        if b == 0xff:
            break

        out.write(b.to_bytes(1, "little"))

    out.write(b"\x00")


if