#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):
    # Read the header (10 bytes) and determine the image size
    header = rng.read(10)

    if len(header)!= 10 or header[0]!= b'G':
        raise ValueError("Invalid GIF header")

    width, height = struct.unpack('<HH', header[6:8])

    print(f"Generating {width}x{height}")

    # Write the header to the output
    out.write(b'GIF89a')
    out.write(struct.pack('<HH', width, height))

    # Copy all chunks until we reach the image data
    while True:
        chunk = rng.read(12)

        if not chunk:
            break

        chunktype = chunk[:4].decode()
        chunklen = int.from_bytes(chunk[4:8], byteorder='little')

        if chunktype == 'IDAT':
            break

        elif chunktype in ('PLTE', 'gAMA'):
            pass

        else:
            continue

        if chunklen > 100 * 1024 * 1024:
            raise RuntimeError('Chunk too large')

        chunkdata = rng.read(chunklen)

        if len(chunkdata)!= chunklen:
            raise RuntimeError('Unexpected end of input')

        out.write(chunk)
        datalen -= chunklen

