#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# returns its size in bytes.
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write('GIF89a'.encode())

    width = 128 + int.from_bytes(rng.read(1), 'little') % 64
    height = 128 + int.from_bytes(rng.read(1), 'little') % 64
    flags = b'\x00' * 7
    bg_color = b'\x00\x00\xff'
    aspect_ratio = b'\x00' * 4
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg_color[0],
                          bg_color[1], bg_color[2]))
    out.write(aspect_ratio)

    # Logical screen descriptor
    out.write(b'\x00\x3B')

    # Global color table (palette)
    palette = [
        b'\x00\x00\x00', b'\x00\x00\xff', b'\x00\xff\x00', b'\x00\xff\xff',
        b'\xff\x00\x00', b'\xff\x00\xff', b'\xff\xff\x00', b'\xff\xff\xff'
    ]
    for i in range(len(palette)):
        out.write(palette[i])

    # Image descriptor
    left = 0
    top = 0
    flags = b'\x80'  # has local color table
    lct_size = len(palette).to_bytes(1, byteorder='little')
    out.write(struct.pack('<HHBB', left, top, flags[0], lct_size))

    # Local color table
    for i in range(len(palette)):
        out.write(palette[i])

    # Image data
    image_data = []
    for y in range(height):
        line = []

        for x in range(width):
            line += [int.from_bytes(rng.read(1), 'little')]

            if len(line) == 255 or x == width - 1:
                while len(line) < 255:
                    line += [line[-1]]

                image_data += line
                line = []

        while len(image_data) % 256!= 0:
            image_data += [0]

    for i in range(len(image_data)):
        out.write(image_data[i].to_bytes(1, byteorder='little'))

    return out.tell()


if