#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1

    # Write the header and length of data to output
    out.write(header)
    out.write(datalen.to_bytes(3, byteorder='big'))

    # Generate the palette
    for i in range(768):
        out.write(rng.read(1))

    # Write the image descriptor
    out.write((0x2C).to_bytes(1, byteorder='little'))
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
    out.write(((datalen - 1) % 256).to_bytes(1, byteorder='little'))
    out.write((((datalen - 1) // 256) & 0xFF).to_bytes(1, byteorder='little'))
    out.write(b'\x00')

    # Write the actual data
    while True:
        chunklen = min(int.from_bytes(
                rng.read(1), byteorder='little'), 255)

        if not chunklen: break

        out.write(chunklen.to_bytes(1, byteorder='little'))
        out.write(rng.read(chunklen))

