#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

if len(sys.argv)!= 4:
    print("Usage: {} <input_file> <output_file> <seed>".format(os.path.basename(__file__)))
    exit()

with open(sys.argv[1], 'rb') as f:
    with open(sys.argv[2], 'wb+') as out:
        generate_gif(f, out, int(sys.argv[3]))

def generate_gif(rng: BinaryIO, out: BinaryIO, seed: int):
    # Header
    header = bytearray([
        71, 73, 70, 56,   # GIF89a
        0x21,             # Extension introducer
        0xFF,             # Application extension label
        0x0B,             # Size of application block
        0x4E, 0x45, 0x54, 0x53, 0x43, 0x41, 0x50, 0x45, 0x32, 0x2E, 0x30,  # NETSCAPE2.0
        0x03,             # Sub-block size
        0x01,             # Loop sub-block identifier
        0x00, 0x00       # No loops (infinite)
    ])
    out.write(header)

    # Logical screen descriptor
    width = rng.read(1)[0] & 0xff | ((rng.read(1)[0] & 0xff) << 8)
    height = rng.read(1)[0] & 0xff | ((rng.read(1)[0] & 0xff) << 8)
    left = 0
    top = 0
    packed = struct.pack('>HHHHB', left, top, width, height, 0)
    out.write(packed)

    # Local color table
    for i in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

