#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
def generate_gif(out: BinaryIO, rng: BinaryIO):
    header = b'GIF89a\x01\x00\x01\x00\x80\xff\x00\xc0\xc0\xc0\x00\x00\x00!\xf9\x04\x01\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'

    # Write the header first
    out.write(header)

    # Generate the image size and color table
    width = rng.randint(1, 256) * 8
    height = rng.randint(1, 256) * 8
    colors = [struct.pack('<BBB', rng.randint(0, 255), rng.randint(0, 255), rng.randint(0, 255)) for _ in range(256)]

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, (width + 7) // 8, (height + 7) // 8, 7))

    # Write the global color table
    out.write(b''.join(colors))

    # Write the local color table
    out.write(b'\x00' * 3 * 256)

    # Write the graphics control extension block
    out.write(b'\x21\xf9\x04\x01\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00;')

    # Write the plain text extension block
    out.write(b'\x21\x01\x00\x00\x00\x00\xff\x00\x0bNETSCAPE2.0\x03\x01\x00\x00\x00!')

    # Write the application extension block
    out.write(b'\x21\xff\x0bXMP Data XMP\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\    out.write(header)

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(256, datalen)
        chunk = rng.read(chunklen)

        if len(chunk)!= chunklen:
            raise RuntimeError('Unexpected end of input')

        out.write(chunk)
        datalen -= chunklen

