#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = int.from_bytes(rng.read(2), byteorder='little')
    h = int.from_bytes(rng.read(2), byteorder='little')

    # packed field (global color table flag, color resolution, sort order)
    flags = rng.read(1)[0]
    # global color table size (bits per pixel)
    gctsize = ((flags >> 7) & 1) << (((flags >> 4) & 7) + 1)

    # background color index
    bgindex = rng.read(1)[0]
    # pixel aspect ratio
    pxaspect = rng.read(1)[0]

    # write header
    out.write(header)
    out.write(struct.pack('<HHBBB', w, h, flags, bgindex, pxaspect))

    # write palette if needed
    if gctsize > 0:
        for i in range(gctsize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(b'\x00\xff\xff\xff')

    # write image descriptor
    out.write(b'\x2c')
    out.write(struct.pack('<HHBB', 0, 0, w, h))

    # write local color table
    for i in range(gctsize):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(b'\x00\xff\xff\xff')

    # write LZW minimum code size
    lzwmincodesize = rng.read(1)[0]
    out.write(lzwmincodesize.to_bytes(1, 'big'))

    # write compressed data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)

