#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    header = b'GIF89a'

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), 'big') % (1 << 10) + 1
    height = int.from_bytes(rng.read(2), 'big') % (1 << 10) + 1
    flags  = bytes([int.from_bytes(rng.read(1), 'big')]) * 3
    bg     = bytes([int.from_bytes(rng.read(1), 'big')]) * 3
    aspect = int.from_bytes(rng.read(1), 'big') & 7

    lsd = struct.pack('>HHBBB', width, height, flags[0], flags[1], flags[2],
                      bg[0], bg[1], bg[2])
    lsd += bytes([aspect])

    # Global color table
    gctsize = ((width + 7) // 8) * ((height + 7) // 8)
    gct = bytearray()
    for _ in range(gctsize):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')

        gct += bytes([r, g, b])

    # Image descriptor
    imgdesc = struct.pack('>BBH', ord(','), ord(';'), 0)

    # Graphics control extension
    gce = struct.pack('>BHB', 0x21, 0xf9, 4)
    gce += bytes([int.from_bytes(rng.read(1), 'big') | 0x80])
    gce += bytes([int.from_bytes(rng.read(1), 'big')]) * 3
    gce += bytes([int.from_bytes(rng.read(1), 'big')])

    # Plain text extension
    pte = struct.pack('>BBH', 0x21, 0x01, 12)
    pte += b'\x0b\x54\x68\x69\x73\x20\x69\x73\x20\x74\x68\x65\x20\x74\x65\x73\x74'

    # Application extension
    appext = struct.pack('>BBH', 0x21, 0xff, 11)
    appext += b'NETSCAPE2.0'
    appext += bytes([3, 1])
    appext += bytes([int.from_bytes(rng.read(1), 'big')]) * 6

    # Comment extension
    cmt = struct.pack('>BBH', 0x21, 0xfe, 3)
    cmt += b'abc'

    # Trailer
    trailer = b';'

    out.write(header)
    out.write(lsd)
    out.write(gct)
    out.write(imgdesc)
    out.write(gce)
    out.write(pte)
    out.write(appext)
    out.write(cmt)
    out.write(trailer)
