#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:

        chunksize = min(datalen, 256)
        out.write(rng.read(chunksize))
        datalen -= chunksize


if __name__ == '__main__':

    if len(sys.argv)!= 2 or not sys.argv[1].endswith('.gif'):
        print('Usage: {} <output>'.format(sys.argv[0]))
        exit()

    with open('/dev/urandom', 'rb') as rng,\
         open(sys.argv[1], 'wb') as out:
        generate_random_gif(rng, out)
