#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(1), 'big') % 256 + \
              (int.from_bytes(rng.read(1), 'big') % 256) * 0x100

    # Write header and length of data to output
    out.write(header)
    out.write(struct.pack('>H', datalen))

    # Write color table
    for i in range(7):
        out.write(b'\x00\x00\x00')

    # Write lzw minimum code size byte
    out.write(b'\x08')

    # Generate LZW compressed data
    compressor = LzwCompressor()
    while True:
        chunklen = min(datalen - len(compressor.data), 255)

        if chunklen == 0:
            break
        
        chunk = rng.read(chunklen)
        compressor.add_data(chunk)
    
    compressor.flush()
    out.write(compressor.data)


class