#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given PRNG and with the given width, height, and color depth.
def generate_gif(rng: BinaryIO,
                 width: int,
                 height: int,
                 colordepth: int,
                 out: BinaryIO):
    header = b'GIF89a' \
             + bytes([width & 0xFF]) \
             + bytes([(width >> 8) & 0xFF]) \
             + bytes([height & 0xFF]) \
             + bytes([(height >> 8) & 0xFF]) \
             + b'\x87\x00\x00' \
             + bytes([colordepth]) \
             + b'\x00\x00'

    datalen = (1 << colordepth) * width * height // 8

    data = rng.read(datalen)

    out.write(header + data)
