#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Generate a random width between 1024 and 512 pixels
    width = rng.randint(1024, 512)
    height = rng.randint(width // 2, width * 2)

    # Write the header and dimensions
    out.write(header + (width - 1).to_bytes(2, 'little'))
    out.write((height - 1).to_bytes(2, 'little'))

    # Write the palette
    for _ in range(768):
        out.write(rng.read(1))

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\xff\xff')

    # Write the LZW minimum code size
    out.write(b'\xf7')

    # Write the actual data
    while True:
        # Read a block from /dev/urandom
        block = rng.read(128)

        if not block: break

        # Write the block's length as a short integer
        out.write(len(block).to_bytes(2, 'little'))

        # Write the block
        out.write(block)