#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # The first 6 bytes are always "GIF89a".
    out.write(b'GIF89a')

    # Next comes the logical screen descriptor. This is where we specify
    # the width, height, color depth (bits per pixel), etc.
    # We'll just use some random values for now.
    w = int.from_bytes(rng.read(2), byteorder='little') % 1000
    h = int.from_bytes(rng.read(2), byteorder='little') % 1000
    flags = 0x87
    bgcolorindex = 0
    aspectratio = 0
    out.write(struct.pack('<HHBBB', w, h, flags, bgcolorindex, aspectratio))

    # Now comes the global color table. It's optional but it's nice to have one.
    # We'll just make an empty one with no colors in it.
    numcolors = 2 ** ((flags & 0x07) + 1)
    palette = [0] * (numcolors * 3)
    out.write(bytes(palette))

    # Finally comes the image descriptor. This tells us that this is going to be
    # a single frame animation with no local color table or interlacing.
    left = top = 0
    width = w
    height = h
    flags = 0x00
    bgcolorindex = 0
    aspectratio = 0
    out.write(struct.pack('<HHHHBB', left, top, width, height, flags, bgcolorindex, aspectratio))

    # And finally the actual image data!
    datalen = int.from_bytes(rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)