#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << 6) | ((os.getpid() & 0x7f) % 0x20)
    height = (1 << 6) | (((os.getpid() >> 7) & 0x7f) % 0x20)
    flags  = 0
    bg     = 0xff
    aspect = 0x40
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    for i in range(0, 256):
        r = (i * 256 // 256) % 256
        g = (i * 256 // 256) % 256
        b = (i * 256 // 256) % 256
        out.write(struct.pack('BBBB', r, g, b, 0))

    # Image descriptor
    left   = 0
    top    = 0
    width  = 1 << 6
    height = 1 << 6
    flags  = 0b00000010
    out.write(struct.pack('>HHHHB', left, top, width, height, flags))

    # Local color table
    for i in range(0, 256):
        r = (i * 256 // 256) % 256
        g = (i * 256 // 256) % 256
        b = (i * 256 // 256) % 256
        out.write(struct.pack('BBBB', r, g, b, 0))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)

