#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int):
    
    # Write the header to the output stream
    out.write('GIF89a'.encode())

    # Generate the logical screen descriptor
    lsd = [
            0x21, 0xFF, 0x0B,       # Application Extension Block
            0x4E, 0x45, 0x54, 0x53, # NETSCAPE2.0
            0x43, 0x41, 0x50, 0x45,
            0x32, 0x2E, 0x30,

            7,                      # Size of application block
            0,                      # No sub-block identifier
            0, 0,                   # No authentication code

            ord('!'),               # Graphic Control Extension Block
            0x0A,                   # Size of block in bytes
            0xF9,                   # Flags (no transparancy)
            0x00, 0x00,             # Delay time in milliseconds
            0x00,                   # Transparent color index
            0x00,                   # Terminator for graphic control extension

            ord(','),               # Image Descriptor
            0x00, 0x00, 0x00, 0x00, # Left position
            0x00, 0x00, 0x00, 0x00, # Top position
            width % 256,           # Width
            width // 256,           # Height
            0x00                    # LZW minimum size
        ]

    # Write the logical screen descriptor to the output stream
    out.write(struct.pack('<{}B'.format(len(lsd)), *lsd))

    # Generate the global color table
    gct = []
    for i in range(256):
        r = rng.read(1)[0] & 0b11111
        g = rng.read(1)[0] & 0b11111
        b = rng.read(1)[0] & 0b11111

        gct += [r, g, b]

    # Write the global color table to the output stream
    out.write(struct.pack('<{}B'.format(len(gct)), *gct))

    # Generate the image data
    imgdata = []
    for y in range(height):
        for x in range(width):
            imgdata += [rng.read(1)[0]]

    # Write the image data to the output stream
    out.write(imgdata)