#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int):
    # Write the header
    out.write(b'GIF89a')

    # Write the logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00))

    # Write the global color table (black & white)
    out.write(b'\xFF\xFF\xFF\x00\x00\x00')

    # Write the graphics control extension
    out.write(b'\x21\xF9\x04\x05\x00\x00\x00\x00\x00\x00\x00')

    # Write the image descriptor
    out.write(struct.pack('>HHHHBB', 0, 0, width - 1, height - 1,
                           0x00, 0x00))

    # Write the local color table (black & white)
    out.write(b'\xFF\xFF\xFF\x00\x00\x00')

    # Write the image data
    for _ in range((width * height)):
        out.write(b'\x00')

    # Write the trailer
    out.write(b';')