#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Generate the size of the data section
    datalen = 0x10000 | (int.from_bytes(
            rng.read(2), byteorder='big') & 0xf)

    # Write the header and length of the data
    out.write(header + datalen.to_bytes(2, 'big'))

    # Write the color table
    for _ in range(768):
        out.write(rng.read(1))

    # Write the image descriptor
    out.write((
        0b10000000 |    # Image is interlaced
        0b00000000 |    # No local colour table
        0b00000000 |    # Sort flag
        0b00000000     # Reserved bits
    ).to_bytes(1, 'big'))
    out.write(struct.pack('>HH', 0, 0))   # Left position, top position
    out.write((
        0b00000000 |    # Width is not specified
        0b00000000      # Height is not specified
    ).to_bytes(1, 'big'))
    out.write((
        0b00000000 |    # Local colour table exists
        0b00000000      # Interlace flag is set
    ).to_bytes(1, 'big'))
    out.write(b'\x00\x00')                  # LZW minimum code size
    # Write the palette
    for i in range(256 * 3):
        out.write(rng.read(1))
    
    # Write the data itself
    while datalen > 0:
        chunklen = min(datalen, 0xffff)
        out.write(chunklen.to_bytes(2, 'little'))

        # TODO: Use an RNG to make this more interesting
        out.write(b'\xff' * chunklen)
        
        datalen -= chunklen