#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = 1024
    height = 768
    color_table_size = 256
    global_color_table = True
    background_index = 0
    aspect_ratio = 0x04
    flags = (
        0b1 << 7 | # has color table
        0b1 << 5 | # sorted color table
        0b1 << 4   # interlaced
    )
    frame_delay = 100
    transparent_index = None
    num_of_loops = 0

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write((flags & 0xff).to_bytes(1, 'little'))
    out.write(((flags >> 8) & 0xff).to_bytes(1, 'little'))
    out.write(color_table_size.to_bytes(1, 'little'))
    out.write(background_index.to_bytes(1, 'little'))
    out.write(aspect_ratio.to_bytes(1, 'little'))

    # Write the global color table if it exists
    if global_color_table:
        for i in range(color_table_size):
            r = rng.read(1)[0] % 256
            g = rng.read(1)[0] % 256
            b = rng.read(1)[0] % 256

            out.write(r.to_bytes(1, 'little'))
            out.write(g.to_bytes(1, 'little'))
            out.write(b.to_bytes(1, 'little'))

    # Write the image descriptor
    left = 0
    top = 0
    width -= 1
    height -= 1
    local_color_table = False
    interlace = ((flags >> 4) & 0x1 == 1)
    lct_size = 2 ** (((flags >> 5) & 0x1) + 1)
    reserved = 0
    sort_flag = ((flags >> 7) & 0x1 == 1)
    local_color_table = False
    lct_size = 2 ** (((flags >> 5) & 0x1) + 1)
    lct_entries = lct_size * lct_size
    local_color_table = False
    interlace = ((flags >> 4) & 0x1 == 1)
    lct_size = 2 ** (((flags >> 5) & 0x1) + 1)
    lct_entries = lct_size * lct_size
    out.write(left.to_bytes(2, 'little'))
    out.write(top.to_bytes(2, 'little'))
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write((flags & 0xff).to_bytes(1, 'little'))
    out.write(((flags >> 8) & 0xff).to_bytes(1, 'little'))
    out.write(lct_size.to_bytes(1, 'little'))
    out.write(reserved.to_bytes(1, 'little'))

    # Write the local color table if it exists
    if local_color_table:
        for i in range(lct_entries):
            r = rng.read(1)[0] % 256
            g = rng.read(1)[0] % 256
            b = rng.read(1)[0] % 256

            out.write(r.to_bytes(1, 'little'))
            out.write(g.to_bytes(1, 'little'))
            out.write(b.to_bytes(1, 'little'))

    # Write the graphics control extension
    out.write(b'\x21\xf9\x04')
    out.write(transparent_index.to_bytes(1, 'little'))
    out.write(frame_delay.to_bytes(2, 'little'))
    out.write(transparent