#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table (black only)
    out.write(b'\x00\x00\x00\xFF')

    # Image descriptor
    out.write(struct.pack('>HHHHBB', 0, 0, width - 1, height - 1, 0x00, 0x00))

    # Local color table (black only)
    out.write(b'\x00\x00\x00\xFF')

    # Extension block for graphic control
    out.write(b'\x21\xF9\x04\x05\x00\x00\x00\x00\x00\x2C')

    # Trailer
    out.write(b';')

    # Data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    out.write(data)