#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical screen descriptor
    width = 1024
    height = 768
    color_table_size = 256
    global_color_table = True
    background_color = 0
    pixel_aspect_ratio = 0
    logical_screen_descriptor = (struct.pack('>HHBBB', width, height,
            color_table_size, global_color_table, background_color,
            pixel_aspect_ratio))
    out.write(logical_screen_descriptor)

    # Global color table
    for i in range(color_table_size):
        red   = rng.read(1)[0]
        green = rng.read(1)[0]
        blue  = rng.read(1)[0]
        alpha = 0xff

        color_table_entry = bytes([red, green, blue, alpha])
        out.write(color_table_entry)

    # Image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table = False
    interlace = False
    sort = False
    reserved = 0
    lzw_minimum_code_size = 2
    image_descriptor = (struct.pack('>HHHHBBBBB', left, top, width, height,
            local_color_table, interlace, sort, reserved, lzw_minimum_code_size))
    out.write(image_descriptor)

    # LZW minimum code size
    min_code_size = 2
    out.write(min_code_size.to_bytes(1, 'big'))

    # Data sub-block
    block_size = 128
    while block_size > 0:
        if block_size < 255:
            block_size += 1
        else:
            block_size -= 1
        block = rng.read(block_size)
        out.write(len(block).to_bytes(1, 'big'))
        out.write(block)

    # Trailer
    trailer = b'\x3B'
    out.write(trailer)


