#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Generate the header
    header = b'GIF89a'
    # Get the size of the data section (max 256k)
    datalen = min(int.from_bytes(
            rng.read(3), byteorder='big'), 0xFFFF)
    # Read the data section from /dev/urandom
    data = rng.read(datalen)

    # Write the header and length of the data block
    out.write(header + len(data).to_bytes(3, 'big'))
    # Write the data block itself
    out.write(data)