#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(256, datalen)
        chunk = rng.read(chunklen)

        if len(chunk)!= chunklen:
            raise RuntimeError('Unexpected end of input')

        out.write(chunk)
        datalen -= chunklen

    # Pad with zeroes until we reach an even multiple of 256
    padding = (256 - (datalen % 256)) % 256
    out.write(b'\x00'*padding)