#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The first 6 bytes are fixed for all valid GIF files
    out.write(b'GIF89a')

    # Read in the size of the image from /dev/urandom
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the size of the image to the output
    out.write(datalen.to_bytes(3, 'big'))

    # Generate the actual image
    img = []
    while True:

        # Get a random color (0-255)
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        # Add it to our list
        img += [r,g,b]

        if len(img) >= datalen: break

    # Write the image to the output
    out.write(bytearray(img))