#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is determined by
# the first 24 bytes of `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the data section
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output stream
    out.write(header)

    # Copy the data from the input stream to the output stream
    while True:

        # Determine how much data we can copy at once
        chunksize = min(1024*1024, datalen - out.tell())

        if not chunksize: break

        # Read some data from the input stream
        buf = rng.read(chunksize)

        # If there was no more data in the input stream,
        # fill it with zeros until we reach the desired
        # data length
        if len(buf) < chunksize:

            paddinglen = chunksize - len(buf)
            padding = b'\x00'*paddinglen
            buf += padding

        assert len(buf) == chunksize

        # Write the data to the output stream
        out.write(buf)


