#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << 6) | ((int.from_bytes(rng.read(2), 'big') & 0x3f) + 1)
    height = (1 << 6) | ((int.from_bytes(rng.read(2), 'big') & 0x3f) + 1)
    flags  = 0
    bg     = int.from_bytes(rng.read(1), 'big')
    aspect = 0
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    numcolors = min((width * height) // 512, 256)
    for i in range(numcolors):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = 0
    top    = 0
    width  |= 1 << 7
    height |= 1 << 7
    flags   = 0
    out.write(struct.pack('>HHBB', left, top, width, height, flags))

    # Local color table
    if numcolors > 0:
        for i in range(numcolors):
            r = int.from_bytes(rng.read(1), 'big')
            g = int.from_bytes(rng.read(1), 'big')
            b = int.from_bytes(rng.read(1), 'big')
            out.write(struct.pack('BBB', r, g, b))

    # Image data
    datalen = int.from_bytes(rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)