#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b"GIF89a")

    # Logical screen descriptor
    width  = struct.pack("<H", 1024)
    height = struct.pack("<H",  768)
    packed = width + height + bytes([
        0xF7 | ((int(width[0]) >> 7) & 1),
        0x00,
        0x00,
        0x00,
    ])
    out.write(packed)

    # Global color table
    for i in range(0, 256):
        rgb = [
            0xFF,
            0x00,
            0x00,
        ]

        if i == 0:
            rgb[0] = 0x00
            rgb[1] = 0x00
            rgb[2] = 0x00

        elif i < 128:
            rgb[0] = 0x00
            rgb[1] = 0x00
            rgb[2] = 0xFF

        else:
            rgb[0] = 0x00
            rgb[1] = 0xFF
            rgb[2] = 0x00

        out.write(struct.pack("BBB", *rgb))

    # Image descriptor
    out.write(bytes([
        0x2C,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
    ]))

    # Local color table
    out.write(bytes([
        0x00,
        0x00,
        0x00,
    ]))

    # Image data
    imagedata = []
    for _ in range(0, 1024*768//8):
        imagedata += [
            ord(rng.read(1)),
        ]

    out.write(bytes(imagedata))