#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), 'big') % 65536
    height = int.from_bytes(rng.read(2), 'big') % 65536
    flags  = bytes([int.from_bytes(rng.read(1), 'big') & ~0x40])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(struct.pack('>HHBBB', width, height, *flags))
    out.write(bg + pxsize + color)

    # Global color table
    for _ in range(2**color[0]):
        r = bytes([int.from_bytes(rng.read(1), 'big')])
        g = bytes([int.from_bytes(rng.read(1), 'big')])
        b = bytes([int.from_bytes(rng.read(1), 'big')])
        out.write(r+g+b)

    # Image descriptor
    left   = bytes([int.from_bytes(rng.read(1), 'big')])
    top    = bytes([int.from_bytes(rng.read(1), 'big')])
    width  = bytes([int.from_bytes(rng.read(1), 'big')])
    height = bytes([int.from_bytes(rng.read(1), 'big')])
    flags  = bytes([int.from_bytes(rng.read(1), 'big') | 0x80])
    trans  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(left + top + width + height + flags + trans)

    # Image data
    datalen = int.from_bytes(rng.read(3), 'big') % 65536
    data = rng.read(datalen)
    out.write(data)

    # Trailer
    out.write(b';')

