#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Generate the screen descriptor
    width  = rng.read(1)[0] % (2**7 - 1) + 1
    height = rng.read(1)[0] % (2**7 - 1) + 1
    flags  = rng.read(1)[0] & ~0b11110000 | 0b00000000
    bg     = rng.read(1)[0]
    aspect = rng.read(1)[0] & ~0b11110000 | 0b00000000

    # Write the header and length of the data
    out.write(header)
    # Write the screen descriptor
    out.write(struct.pack('>BBBHHB', width, height, flags, bg, aspect))

    # Generate the global color table
    for _ in range(int.from_bytes(rng.read(1), byteorder='big')):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(r.to_bytes(1, 'big') +
                 g.to_bytes(1, 'big') +
                 b.to_bytes(1, 'big'))

    # Write the local color table
    out.write(b'\x00\x00\x00')

    # Write the image descriptor
    left   = rng.read(2)[0] % (2**16 - 1)
    top    = rng.read(2)[0] % (2**16 - 1)
    width  = rng.read(2)[0] % (2**16 - 1) + 1
    height = rng.read(2)[0] % (2**16 - 1) + 1
    flags  = rng.read(1)[0] & ~0b11110000 | 0b00000000
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags))

    # Write the LZW minimum code size
    out.write(b'\x08')

    # Write the compressed data
    while True:
        chunktype = rng.read(1)[0]

        if chunktype == 0x00:
            break

        elif chunktype == 0x01:
            pass

        else:
            raise Exception("Unknown chunk type")

    # Write the end marker
    out.write(b'\x00')