#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1 << (int.from_bytes(rng.read(1), 'little'))
    height = 1 << (int.from_bytes(rng.read(1), 'little'))
    packed = [0x70 | int.from_bytes(rng.read(1), 'little')]
    packed += list(rng.read(3))
    packed[4] |= 0x80
    out.write(struct.pack('<HHBBB', width, height, *packed))

    # Global color table
    numcolors = min((width+height)//256, 256)
    for i in range(numcolors):
        rgb = []
        for j in range(3):
            rgb.append(int.from_bytes(rng.read(1), 'little'))

        out.write(struct.pack('BBB', *rgb))

    # Image descriptor
    left   = int.from_bytes(rng.read(1), 'little')
    top    = int.from_bytes(rng.read(1), 'little')
    width  = 1 << (int.from_bytes(rng.read(1), 'little'))
    height = 1 << (int.from_bytes(rng.read(1), 'little'))
    packed = [0x70 | int.from_bytes(rng.read(1), 'little')]
    packed += list(rng.read(3))
    packed[4] |= 0x80
    out.write(struct.pack('<HHBBB', left, top, width, height, *packed))

    # Local color table
    numcolors = min(((width//8)*(height//8)), 256)
    if numcolors > 0:
        for i in range(numcolors):
            rgb = []
            for j in range(3):
                rgb.append(int.from_bytes(rng.read(1), 'little'))

            out.write(struct.pack('BBB', *rgb))

    # Data sub-block
    while True:
        blocksize = int.from_bytes(rng.read(1), 'little')
        if not blocksize: break
        out.write(rng.read(blocksize))