#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The generated image is of size (width x height) pixels and has
# 256 different colors.
def generate_random_gif(
        width: int, height: int, rng: BinaryIO, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xf7, 0x00, 0))

    # Global color table
    for i in range(256):
        r = rng.read(1)[0] & 0xf7
        g = rng.read(1)[0] & 0xf7
        b = rng.read(1)[0] & 0xf7

        out.write(struct.pack('BBBB', r, g, b, 0))

    # Image descriptor
    out.write(struct.pack('>HHHHB', 0, 0, width - 1, height - 1, 0))

    # LZW minimum code size
    out.write(b'\x08')

    # Data sub-block
    lzw_mincodesize = 8

    def write_subblocks():
        nonlocal lzw_mincodesize

        while True:
            chunklen = rng.read(1)[0]

            if not chunklen:
                break

            if chunklen == 255:
                continue

            if chunklen > 254:
                chunklen += 1

                if lzw_mincodesize < 12:
                    lzw_mincodesize += 1

                chunklen -= 256

            out.write(chunklen.to_bytes(1, 'little'))
            out.write(rng.read(chunklen))

    write_subblocks()