#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Generate the header
    header = b'GIF89a'
    # Read 10 bytes from /dev/urandom to get the size of the data block
    datalen = int.from_bytes(
            rng.read(10), byteorder='big')
    # Read the rest of the data block
    data = rng.read(datalen - 10)

    # Write the header and length of the data block
    out.write(header + len(data).to_bytes(3, 'big'))
    # Write the data block itself
    out.write(data)