#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    def write_header() -> None:
        header = b'GIF89a'
        datalen = int.from_bytes(
                rng.read(3), byteorder='big')
        out.write(header + datalen.to_bytes(3, 'big'))

    def write_data() -> None:
        datalen = int.from_bytes(
                rng.read(3), byteorder='big')
        data = rng.read(datalen)
        out.write(data)

    write_header()
    while True:
        if rng.read(1)[0] & 0b1 == 0: break
        write_data()