#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (e.g., /dev/urandom).
# The GIF is 1024 bytes long and contains one frame with
# width 64 and height 64.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB',
        1024, 768, 0x80 | 0x07, 0, 0))

    # Global color table
    for _ in range(0, 256):
        r = ord(rng.read(1)) & 0xf0
        g = ord(rng.read(1)) & 0xf0
        b = ord(rng.read(1)) & 0xf0

        out.write(struct.pack('>BBBB',
            0xff, r >> 4, g >> 4, b >> 4))

    # Image descriptor
    out.write(struct.pack('>HHHHB',
        0, 0, 64, 64, 0x80 | 0x07))

    # Local color table
    for _ in range(0, 256):
        r = ord(rng.read(1)) & 0xf0
        g = ord(rng.read(1)) & 0xf0
        b = ord(rng.read(1)) & 0xf0

        out.write(struct.pack('>BBBB',
            0xff, r >> 4, g >> 4, b >> 4))

    # Image data
    for y in range(0, 64):
        for x in range(0, 64):
            out.write(struct.pack('>B',
                ord(rng.read(1))))