#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1024
    height = 768
    color_depth = 8
    global_color_table = True
    background_color = 0x00
    aspect_ratio = 0x00
    out.write(struct.pack('>HHBBB', width, height, color_depth,
                          global_color_table, background_color))

    # Global color table (black and white)
    for i in range(2 ** color_depth):
        if i == 0 or i == 255:
            r = g = b = 0
        else:
            r = g = b = 0xff

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left = top = 0
    width = 1024
    height = 768
    local_color_table = False
    interlace = False
    sort = False
    reserved = 0x00
    lct_size = 0x00
    out.write(struct.pack('>HHHHBBBBB', left, top, width, height,
                          local_color_table, interlace, sort, reserved, lct_size))

    # Local color table (black and white)
    for i in range(2 ** color_depth):
        if i == 0 or i == 255:
            r = g = b = 0
        else:
            r = g = b = 0xff

        out.write(struct.pack('BBB', r, g, b))

    # Data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)