#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    width = 16 + (rng.read(1)[0] % 256)
    height = 16 + (rng.read(1)[0] % 256)
    flags = rng.read(1)[0] & 7
    background = rng.read(1)[0] & 255
    aspect_ratio = rng.read(1)[0] & 255
    out.write(struct.pack('<HHBB', width, height, flags, background))
    out.write(aspect_ratio.to_bytes(1, 'little'))
    
    # Global color table
    for i in range(3 * (2 ** ((flags & 7) + 1))):
        out.write(rng.read(1))
        
    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write((0).to_bytes(2, 'little'))
    out.write(b'\x00')
    
    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00')
    delay = (rng.read(1)[0] % 256) | (rng.read(1)[0] << 8)
    out.write(delay.to_bytes(2, 'little'))
    out.write(b'\x00')
    
    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    flags = rng.read(1)[0] & 7
    out.write(flags.to_bytes(1, 'little'))
    
    # Local color table
    if flags != 0:
        for i in range(3 * (2 ** ((flags & 7) + 1))):
            out.write(rng.read(1))
            
    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))
                
    # Trailer
    out.write(b'\x3b')
