#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 160 + (rng.read(1)[0] % 256)
    height = 144 + (rng.read(1)[0] % 256)
    out.write(struct.pack('<HHBB', width, height, 0xF7, 0))

    # Global color table
    for i in range(3 * 2 ** ((rng.read(1)[0] % 8) + 1)):
        out.write(rng.read(1))

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\xff\xff\xff\xff')

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00')

    # Local color table
    for i in range(3 * 2 ** ((rng.read(1)[0] % 8) + 1)):
        out.write(rng.read(1))

    # LZW-compressed image data
    while True:
        code = rng.read(1)[0]
        if code == 0xFF:
            break
        elif code < 0x80:
            out.write(code.to_bytes(1, 'little'))
        else:
            out.write((code | 0x100).to_bytes(1, 'little'))

    # Trailer
    out.write(b'\x3b')
