#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Write the screen descriptor
    out.write(struct.pack('<H', 16))
    out.write(struct.pack('<H', 16))
    out.write(b'\x00\x00')

    # Write the global color table
    for i in range(256):
        out.write(struct.pack('BBB', i, i, i))

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\xff\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(256):
        out.write(struct.pack('BBB', i, i, i))

    # Write the image data
    for _ in range(16 * 16):
        out.write(struct.pack('B', rng.read(1)[0] % 256))

    # Write the trailer
    out.write(b';')
