#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Write the screen descriptor.
    buf = bytearray()
    buf += b'\x01\x00'  # Width
    buf += b'\x01\x00'  # Height
    buf += b'\x80'      # Background color index
    buf += b'\x00'      # Pixel aspect ratio
    out.write(buf)

    # Write the global color table.
    for i in range(256):
        out.write(struct.pack('BBB', i, i, i))

    # Write the graphic control extension.
    buf = bytearray()
    buf += b'\x21\xf9\x04\x01\x00\x00\xff\x00'
    out.write(buf)

    # Write the image descriptor.
    buf = bytearray()
    buf += b'\x2c\x00\x00\x00\x00\x00\x01\x00'
    buf += b'\x01\x00'  # Width
    buf += b'\x01\x00'  # Height
    buf += b'\x00'      # Local color table flag
    buf += b'\x00'      # Interlace flag
    buf += b'\x00'      # Sort flag
    buf += b'\x00'      # Size of local color table
        out.write(buf)

