#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    # global color table flag
    gctf = rng.read(1)[0] >> 7

    # color table size
    cts = rng.read(1)[0] & 7

    # color table
    if gctf == 1:
        colors = [None] * (2 ** (cts + 1))
        for i in range(len(colors)):
            colors[i] = rng.read(3)

    # image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00'

    # local color table flag
    lctf = rng.read(1)[0] >> 7

    # interlaced flag
    ilaced = rng.read(1)[0] >> 6

    # sorted flag
    sorted = rng.read(1)[0] >> 5

    # reserved
    reserved = rng.read(1)[0] >> 4

    # local color table size
    lcts = rng.read(1)[0] & 7

    # x offset
    xoff = rng.read(2)

    # y offset
    yoff = rng.read(2)

    # width
    width = rng.read(2)

    # height
    height = rng.read(2)

    # packed field of LCT bits per pixel
    lctr = rng.read(1)[0] & 7
    lctg = rng.read(1)[0] & 7
    lctb = rng.read(1)[0] & 7
    lctpacked = bytes([lctr | (lctg << 3) | (lctb << 6)])

    # local color table
    if lctf == 1:
        lcolors = [None] * (2 ** (lcts + 1))
        for i in range(len(lcolors)):
            lcolors[i] = rng.read(3)

    # image data
    data = rng.read()

    out.write(header + data)
