#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate(out: BinaryIO, rng: BinaryIO):
    # Write header
    out.write(b'GIF89a')
    
    # Write screen descriptor
    width = 1024
    height = 768
    background_color = 0xFF0000
    pixel_aspect_ratio = 0
    global_color_table_flag = 0
    color_resolution = 8
    sort_flag = 0
    size_of_global_color_table = 7
    out.write(struct.pack('<HBB', width, height, pixel_aspect_ratio))
    out.write(bytes([background_color, global_color_table_flag, color_resolution, sort_flag, size_of_global_color_table]))
    
    # Write global color table
    for i in range(2**(size_of_global_color_table+1)):
        out.write(struct.pack('<B', (i >> 5) & 0xF8 | (i >> 1) & 0x07 | (i << 2) & 0xE0))
        
    # Write application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', 0))
    
    # Write image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    size_of_local_color_table = 0
    out.write(struct.pack('<HHBBB', left, top, width, height, local_color_table_flag, interlace_flag, sort_flag, size_of_local_color_table))
    
    # Write LZW minimum code size
    out.write(b'\x08')
    
    # Write data sub-blocks
    while True:
        # Read next byte from RNG
        b = rng.read(1)[0]
        
        # If it is zero, write end of data sub-block
        if b == 0:
            out.write(b'\x00')
            
            # If we have just written an end of data sub-block, then
            # we are done writing data sub-blocks and can exit the loop
            break
            
        # Otherwise, write the byte as a single data sub-block
        else:
            out.write(struct.pack('<B', b))

if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rng:
