#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = 160
    height = 120
    background = 0xff
    aspect_ratio = 0
    global_color_table = True
    color_resolution = 7
    sort_flag = False
    color_table_size = 7 + 1
    pixel_aspect_ratio = 0
    image_descriptor = bytes([
        0x2c, # Image separator
        0x00, 0x00, # Image position X
        0x00, 0x00, # Image position Y
        0x00, 0x80, # Image width
        0x00, 0x80, # Image height
        0b00000000 | (background << 4), # Local color table flag, Color table size
        0b00000000 | (sort_flag << 1) | (global_color_table << 0), # Sort flag, Global color table flag
        0b00000000 | (pixel_aspect_ratio << 5) | (color_resolution << 2) | (aspect_ratio << 0) # Pixel aspect ratio
    ])
    local_color_table = [
        b'\xff\xff\xff',
        b'\x00\x00\x00',
        b'\x00\x00\xff',
        b'\x00\xff\x00',
        b'\x00\xff\xff',
        b'\xff\x00\x00',
        b'\xff\x00\xff',
        b'\xff\xff\x00',
        b'\xff\xff\xff',
        b'\xff\xaa\xaa',
        b'\xaa\xff\xaa',
        b'\xaa\xaa\xff',
        b'\xaa\x55\xaa',
        b'\xaa\xaa\x55',
        b'\x55\xaa\xaa',
        b'\x55\x55\xaa',
        b'\xaa\x55\x55',
        b'\x55\xaa\x55',
        b'\x55\x55\x55',
        b'\x55\xff\xff',
        b'\xff\x55\xff',
        b'\xff\xff\x55',
        b'\xff\xff\xff',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x00',
        b'\x00\x00\x0