#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    background = rng.read(1)[0] & 0xff
    aspect_ratio = rng.read(1)[0] & 0xff

    # 1 byte for each color in the palette
    palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    local_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    local_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    graphic_control_extension_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    graphic_control_extension_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    application_extension_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    application_extension_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    comment_extension_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    comment_extension_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    plaintext_extension_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    plaintext_extension_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    unknown_extension_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    unknown_extension_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    extension_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    extension_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    trailer_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    trailer_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    terminator_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    terminator_pixels = [rng.read(1)[0] for _ in range((w[0]*h[0]))]

    # 1 byte for each color in the palette
    eof_palette = [rng.read(1)[0] for _ in range(2**(r+g+b))]

    # 1 byte for each pixel in the image
    out.write(header + data)
