#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# the given width and height.
def generate_gif(out: BinaryIO, rng: BinaryIO, width: int, height: int):
    # Write the GIF header.
    out.write(b'GIF89a')
    
    # Write the screen descriptor.
    out.write(bytes([width & 0xff, (width >> 8) & 0xff]))
    out.write(bytes([height & 0xff, (height >> 8) & 0xff]))
    out.write(b'\x87\x00\x00')
    
    # Write the global color table.
    for i in range(256):
        out.write(bytes([i, i, i]))
        
    # Write the graphic control extension.
    out.write(b'\x21\xf9\x04\x01\x00\x00\xff\xff\xff\x00')
    
    # Write the image descriptor.
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # Write the local color table.
    for i in range(256):
        out.write(bytes([i, i, i]))
        
    # Write the pixel data.
    for y in range(height):
        for x in range(width):
            out.write(bytes([int.from_bytes(rng.read(1), 'big'), 0]))
            
    # Write the trailer.
    out.write(b';')
