#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Get the width and height of the image from the RNG.
    width = int.from_bytes(rng.read(2), byteorder='big', signed=False) % 100 + 1
    height = int.from_bytes(rng.read(2), byteorder='big', signed=False) % 100 + 1

    # Write the width and height to the output stream.
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))

    # Write the global color table to the output stream.
    for _ in range(768):
        out.write(rng.read(1))

    # Write the background color index to the output stream.
    out.write(rng.read(1))

    # Write the pixel aspect ratio to the output stream.
    out.write(rng.read(1))

    # Write the application extension header to the output stream.
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')

    # Write the loop count to the output stream.
    out.write(b'\x00\x00\x00\x00')

    # Write the trailer to the output stream.
    out.write(b'\x3b')
