#!/usr/bin/env python3

import struct
from typing import BinaryIO

with open('data/palette.dat', 'rb') as f:
    palette = [struct.unpack('BBB', f.read(3)) for _ in range(256)]

with open('data/sprites.dat', 'rb') as f, \
     open('out/sprites.dat', 'wb') as out:
    rng = iter(f.read())

    header = b'\x89PNG\r\n'

    # width and height
    w = h = 16
    wh = bytes((w, h))

    # pixel depth
    pd = 4
    pdh = bytes((pd,))

    # color type
    ct = 3
    cth = bytes((ct,))

    # compression method
    cm = 0
    cmh = bytes((cm,))

    # filter method
    fm = 0
    fmh = bytes((fm,))

    # interlace method
    im = 0
    imh = bytes((im,))

    # IHDR chunk
    ihdr = b'IHDR' + wh + pdh + cth + cmh + fmh + imh

    # PLTE chunk
    plte = b'PLTE' + bytes(sum(palette,()))

    # IDAT chunk

    # first pixel
    r, g, b = palette[bg]
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
