#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Write the magic header
    out.write(b'GIF89a')

    # Generate a random color table size (1-256 colors)
    colortablelen = 1 + rng.read(1)[0] % 256
    out.write(colortablelen.to_bytes(1, 'big'))

    # Generate a random background color index
    bgcoloridx = rng.read(1)[0] % colortablelen
    out.write(bgcoloridx.to_bytes(1, 'big'))

    # Generate a random pixel aspect ratio
    par = rng.read(1)[0] % 256
    out.write(par.to_bytes(1, 'big'))

    # Generate a random image width and height
    w = 1 + rng.read(2).hex()
    h = 1 + rng.read(2).hex()
    out.write((w+h).encode())

    # Generate a random delay time
    delaytime = rng.read(2).hex()
    out.write(delaytime.encode())

    # Generate a random transparent color flag
    transflag = bool(rng.read(1)[0])
    out.write(transflag.to_bytes(1, 'big'))

    # Generate a random block size
    blocksize = rng.read(1)[0] % 256
    out.write(blocksize.to_bytes(1, 'big'))

    # Generate a random image data length
    datalen = rng.read(3).hex()
    out.write(datalen.to_bytes(3, 'big'))

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)