#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Write header
    out.write(b"GIF89a")
    
    # Write screen descriptor
    out.write(struct.pack("!HHBB", 256, 256, 0xF7, 0))
    
    # Write global color table (1 byte per entry)
    for i in range(0x100):
        out.write(bytes([i]))
        
    # Write image block
    out.write(b"\x2C\x00\x00\x00\x00\x00\xFF\xFF\x00")
    
    # Write local color table
    for i in range(0x100):
        out.write(bytes([i]))
        
    # Write image data
    while True:
        # Read one byte from /dev/urandom
        b = int.from_bytes(rng.read(1), "big")
        
        # If it's zero, we're done
        if b == 0:
            break
            
        # Otherwise, write it as a run of that many zeros
        out.write(bytes([b]))
