#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, duration=10):
    # Write the header
    out.write(b'GIF89a')
    
    # Write the screen descriptor
    out.write(struct.pack('<HHB', width, height, 0xF7))
    
    # Write the global color table
    for i in range(256):
        out.write(bytes([i]))
        
    # Write the application extension block
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', duration * 10))
    out.write(b'\x00')
    
    # Write the image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # Write the local color table
    for i in range(256):
        out.write(bytes([i]))
        
    # Write the pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([int.from_bytes(rng.read(1), 'big') % 256]))
            
    # Write the trailer
    out.write(b';')
