#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 640
    height = (1 << rng.read(1)[0]) % 480
    flags  = bytes([
        0x80 | ((rng.read(1)[0] & 0x70) >> 4),
        rng.read(1)[0] & 0x0f,
    ])
    bgcol  = rng.read(1)[0]
    pxsize = rng.read(1)[0] & 0x07
    out.write(struct.pack('<HBBB', width, height, flags[0], flags[1]))
    out.write(bytes([bgcol, pxsize]))

    # Global color table
    if flags[0] & 0x80:
        ncolors = 2 ** (pxsize + 1)
        for _ in range(ncolors):
            out.write(rng.read(3))

    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', 0))

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00')
    out.write(struct.pack('<H', 0))

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')
    out.write(struct.pack('<HH', width, height))
    out.write(flags)

    # Local color table
    if flags[0] & 0x80:
        ncolors = 2 ** (pxsize + 1)
        for _ in range(ncolors):
            out.write(rng.read(3))

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng.read(1)[0]]))

    # Trailer
    out.write(b'\x3b')

