#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = int.from_bytes(rng.read(2), byteorder='big')
    h = int.from_bytes(rng.read(2), byteorder='big')

    # global color table flag
    ct = rng.read(1)[0] >> 7

    # color resolution
    cr = rng.read(1)[0] & 7

    # sort flag
    sf = rng.read(1)[0] >> 7

    # size of global color table
    sct = rng.read(1)[0] & 7

    # background color index
    bi = rng.read(1)[0]

    # pixel aspect ratio
    par = rng.read(1)[0]

    # global color table
    if ct == 1:
        for i in range(sct * 3):
            out.write(rng.read(1))

    # image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\xff\x00'
    out.write(idesc)

    # local color table flag
    lctf = rng.read(1)[0] >> 7

    # interlace flag
    ifl = rng.read(1)[0] >> 6

    # sorting flag
    sl = rng.read(1)[0] >> 5

    # size of local color table
    slct = rng.read(1)[0] & 7

    # local color table
    if lctf == 1:
        for i in range(slct * 3):
            out.write(rng.read(1))

    # min code size
    mcs = rng.read(1)[0]

    # LZW data
    lzwlen = int.from_bytes(
            rng.read(3), byteorder='big')
    lzwdata = rng.read(lzwlen)
    out.write(lzwdata)

    # trailer
    trailer = b';'
    out.write(trailer)

# Generates a random PNG file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_png(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'\x89PNG\r\n\x1a\n'

    # IHDR chunk
    ihdr = b'IHDR'
    ihdr += len(ihdr).to_bytes(4, 'big')
    ihdr += int.to_bytes(int.from_bytes(
                rng.read(2), byteorder='big'), 4, 'big')
    ihdr += int.to_bytes(int.from_bytes(
                rng.read(2), byteorder='big'), 4, 'big')
    ihdr += b'\x08\x06\x00\x00\x00'
    out.write(ihdr)

    # IDAT chunk
    idat = b'IDAT'
    idat += len(idat).to_bytes(4, 'big')
    idat += rng.read(int.from_bytes(
                    rng.read(4), byteorder='big'))
    out.write(idat)

    # IEND chunk
    iend = b'IEND'
    iend += len(iend).to_bytes(4, 'big')
    out.write(iend)

# Generates a random BMP file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_bmp(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'BM'

    # file size
    fsize = int.from_bytes(rng.read(4), byteorder='little')
    header += fsize.to_bytes(4, 'little')

    # reserved
    header += rng.read(4)

    # offset
    header += rng.read(4)

    # DIB header
       g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + data)
