#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun 15 20:46:57 2021

@author: alessandro
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
import matplotlib.pyplot as plt

def get_data(n=100):
    """
    Genera un dataset di n osservazioni da una distribuzione bivariata
    con covarianza 0.5 e media (0,0)
    
    Parameters
    ----------
    n : int, optional
        Numero di osservazioni. The default is 100.

    Returns
    -------
    x : array
        Array delle osservazioni.

    """
    mu = [0,0] #media
    sigma = [[0.5, 0], [0, 0.5]] #covariance matrix
    x = multivariate_normal.rvs(mean=mu, cov=sigma, size=n)
    return x

def get_pdf(x, y, xi, yi):
    """
    Calcola la densità di probabilità del punto (xi,yi) in una distribuzione
    bivariata con media (0,0) e covarianza 0.5
    
    Parameters
    ----------
    x : array
        Array delle osservazioni.
        
    y : array
        Array delle osservazioni.
        
    xi : float
        Valore della coordinata x del punto.
        
    yi : float
        Valore della coordinata y del punto.

    Returns
    -------
    pdf : float
        Densità di probabilità del punto (xi,yi).

    """
    mu = [0,0] #media
    sigma = [[0.5, 0], [0, 0.5]] #covariance matrix
    pdf = multivariate_normal.pdf([[xi],[yi]], mean=mu, cov=sigma)
    return pdf

def get_cdf(x, y, xi, yi):
    """
    Calcola la densità di probabilità del punto (xi,yi) in una distribuzione
    bivariata con media (0,0) e covarianza 0.5
    
    Parameters
    ----------
    x : array
        Array delle osservazioni.
        
    y : array
        Array delle osservazioni.
        
    xi : float
        Valore della coordinata x del punto.
        
    yi : float
        Valore della coordinata y del punto.

    Returns
    -------
    cdf : float
        Cumulativa di probabilità del punto (xi,yi).

    """
    mu = [0,0] #media
    sigma = [[0.5, 0], [0, 0.5]] #covariance matrix
    cdf = multivariate_normal.cdf([[xi],[yi]], mean=mu, cov=sigma)
    return cdf

def get_marginal_pdf(x, y, xi):
    """
    Calcola la densità di probabilità del punto (xi,y) in una distribuzione
    bivariata con media (0,0) e covarianza 0.5
    
    Parameters
    ----------
    x : array
        Array delle osservazioni.
        
    y : array
        Array delle osservazioni.
        
    xi : float
        Valore della coordinata x del punto.
        
    yi : float
        Valore della coordinata y del punto.

    Returns
    -------
    marginal_pdf : float
        Densità di probabilità del punto (xi,y).

    """
    mu = [0,0] #media
    sigma = [[0.5, 0], [0, 0.5]] #covariance matrix
    marginal_pdf = norm.pdf(xi, loc=mu[0], scale=np.sqrt(sigma[0][0]))
    return marginal_pdf

def get_marginal_cdf(x, y, xi):
    """
    Calcola la cumulativa di probabilità del punto (xi,y) in una distrib