#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Write the header.
    out.write(b'GIF89a')
    
    # Write the logical screen descriptor.
    out.write(struct.pack('<HBB', width, height, 0xF7))
    
    # Write the global color table.
    for i in range(256):
        r = (i >> 5) & 0x1F
        g = (i >> 2) & 0x1F
        b = (i << 3) & 0x1F
        
        out.write(bytes([r, g, b]))
    
    # Write the graphic control extension.
    out.write(b'\x21\xF9\x04\x08\x00\x00\xFF\xFF\xFF\x00')
    
    # Write the image descriptor.
    out.write(b'\x2C\x00\x00\x00\x00' + bytes([width, height]) + b'\x00')
    
    # Write the local color table.
    for i in range(256):
        out.write(bytes([i, i, i]))
    
    # Write the pixel data.
    for y in range(height):
        for x in range(width):
            out.write(bytes([y % 256, x % 256]))
