#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Write the GIF header
    out.write(b'GIF89a')

    # Read 2 bytes from the RNG as width and height
    w, h = struct.unpack('<HH', rng.read(4))

    # Write the width and height fields
    out.write(struct.pack('<HH', w, h))

    # Write the global color table
    out.write(rng.read(768))

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x05\xff\xff\xff\xff\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    out.write(rng.read(768))

    # Write the LZW-compressed image data
    datalen = (w * h + 128) // 8
    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)