#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # GIF header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    width = 16 + (rng.read(1)[0] % 256)
    height = 16 + (rng.read(1)[0] % 256)
    color_table_size = 7 + (rng.read(1)[0] % 8)
    background_color_index = rng.read(1)[0] % 2**color_table_size
    pixel_aspect_ratio = rng.read(1)[0] % 4
    out.write(struct.pack('<HHBB', width, height, color_table_size, background_color_index))
    
    # Global color table
    for i in range(2 ** color_table_size):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))
        
    # Image descriptor
    left = rng.read(1)[0] % width
    top = rng.read(1)[0] % height
    width = rng.read(1)[0] % (width - left)
    height = rng.read(1)[0] % (height - top)
    local_color_table_flag = bool(rng.read(1)[0] & 1)
    interlace_flag = bool(rng.read(1)[0] & 2)
    sort_flag = bool(rng.read(1)[0] & 4)
    reserved_bits = rng.read(1)[0] & 8
    if local_color_table_flag:
        local_color_table_size = 7 + (rng.read(1)[0] % 8)
    else:
        local_color_table_size = 0
    out.write(struct.pack('<HHBBBB', left, top, width, height, local_color_table_flag, interlace_flag, sort_flag, reserved_bits))
    
    # Local color table
    for i in range(2 ** local_color_table_size):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))
    
    # Image data
    lzw_minimum_code_size = rng.read(1)[0] % 12
    out.write(lzw_minimum_code_size.to_bytes(1, 'big'))
    
    # LZW-compressed image data
    while True:
        code = rng.read(1)[0] % 256
        if code == 0xFF:
            break
        out.write(code.to_bytes(1, 'big'))
