
#!/usr/bin/env python3

import sys, os, struct
from PIL import Image

if len(sys.argv) != 2:
    print("Usage: {} <input_file>".format(os.path.basename(__file__)))
    exit(-1)

inp = open(sys.argv[1], "rb")
out = open(sys.argv[1] + ".bmp", "wb")

rng = inp.read()

# Write a BMP header to the output file.
out.write(struct.pack("<IHHI", 0x4D42, 54, 40, 0))

# Read the width and height of the image from the input stream.
width, height = struct.unpack("<HH", rng[:4])

# Write the width and height of the image to the output stream.
out.write(struct.pack("<II", width, height))

# Skip past the pixel format field (1 byte).
rng = rng[5:]

# Write the number of planes to the output stream.
out.write(struct.pack("<H", 1))

# Write the bits per pixel to the output stream.
out.write(struct.pack("<H", 8))

# Write the compression method to the output stream.
out.write(struct.pack("<I", 0))

# Write the size of the raw bitmap data to the output stream.
datalen = struct.pack("<I", width * height)
out.write(datalen)

# Write the horizontal resolution to the output stream.
out.write(struct.pack("<I", 72))

# Write the vertical resolution to the output stream.
out.write(struct.pack("<I", 72))

# Write the number of colors in the palette to the output stream.
out.write(struct.pack("<I", 0))

# Write the number of important colors to the output stream.
out.write(struct.pack("<I", 0))

# Skip past the color table fields (1024 bytes).
rng = rng[1024:]

# Write the raw bitmap data to the output stream.
while True:
    # Write the start of the image data section to the output stream.
    # Write the image data length to the output stream.
    out.write(datalen)

    # Copy the image data from the input stream to the output stream.
    while True:
        buf = rng.read(4096)

        if len(buf) == 0: break;

        out.write(buf)

