#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        out: BinaryIO,
        rng: BinaryIO):
    
    # Write the magic header
    out.write('GIF89a'.encode())

    # Generate some random bytes for the width and height
    # and write them to the output stream
    width = int.from_bytes(
            rng.read(2), byteorder='big')
    height = int.from_bytes(
            rng.read(2), byteorder='big')
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))

    # Generate some random bytes for the length of the image data
    # and write them to the output stream
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(datalen.to_bytes(3, 'big'))

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)