#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    header = b'GIF89a'
    width = 2048
    height = 1536
    color_table_size = 7
    background_color_index = 0
    pixel_aspect_ratio = 0
    global_color_table_flag = 1
    sort_flag = 0
    color_resolution = 7
    gct_size = (1 << color_table_size) - 1
    fields = [
        0xff & width,
        0xff & (width >> 8),
        0xff & height,
        0xff & (height >> 8),
        color_table_size,
        background_color_index,
        pixel_aspect_ratio,
        global_color_table_flag | (sort_flag << 7) | (color_resolution << 4),
    ]
    header += bytes([sum(fields[i * 2:(i + 1) * 2]) for i in range(len(fields) // 2)])

    # Color table
    colors = []
    for _ in range(gct_size + 1):
        colors.append(struct.unpack('BBB', rng.read(3)))

    # Image descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00'
    fields = [
        0xff & width,
        0xff & (width >> 8),
        0xff & height,
        0xff & (height >> 8),
        local_color_table_flag = 0,
        interlace_flag = 0,
        sort_flag = 0,
        reserved = 0,
        size_of_local_color_table = 0,
    ]
    image_descriptor += bytes([sum(fields[i * 2:(i + 1) * 2]) for i in range(len(fields) // 2)])

    # LZW-compressed data
    data = rng.read()
    out.write(header + data)
