#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0]

    # global color table flag
    gctf = rng.read(1)[0] >> 7

    # color table size
    ctsize = rng.read(1)[0] & 7

    # color table
    if gctf == 1:
        gct = rng.read(3 * (2 ** (ctsize + 1)))
    else:
        gct = None

    # image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'

    # local color table flag
    lctf = rng.read(1)[0] >> 7

    # interlaced flag
    ilace = rng.read(1)[0] >> 6

    # sort flag
    sort = rng.read(1)[0] >> 5

    # reserved
    res = rng.read(1)[0] >> 4

    # local color table size
    lctsize = rng.read(1)[0] & 7

    # min code length
    mcl = rng.read(1)[0]

    # LZW dictionary size
    dsz = 2 ** (lctsize + 1)

    # LZW dictionary
    dict = [bytes([i]) for i in range(dsz)]

    # LZW clear code
    clr = dsz

    # LZW end code
    eod = clr + 1

    # LZW next code
    nxt = eod + 1

    # LZW code size
    cs = 9

    # LZW current code
    cur = -1

    # LZW stack
    stk = []

    # LZW data length
    datalen = (w[0] << 8) | w[1]

    # LZW data
    data = rng.read(datalen)
    out.write(header + data)
