#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 29 15:07:46 2019

@author: kazuki.onodera
"""

import numpy as np
import pandas as pd
from glob import glob
import os, gc
from tqdm import tqdm

import sys
sys.path.append(f'/home/{os.environ.get("USER")}/PythonLibrary')
import lgbextension as ex
import lightgbm as lgb
from multiprocessing import cpu_count, Pool

import utils
utils.start(__file__)
#==============================================================================

SUBMIT_FILE_PATH = '../output/0529-1.csv.gz'

COMMENT = 'top100 features * 3 + f009 ~ f019'

EXE_SUBMIT = True

DROP = ['f001_hostgal_specz',]

SEED = np.random.randint(9999)
print('SEED:', SEED)

NFOLD = 5

LOOP = 1

param = {
         'objective': 'binary',
         'metric': 'auc',
         
         'learning_rate': 0.01,
         'max_depth': 6,
         'num_leaves': 63,
         'max_bin': 255,
         
         'min_child_weight': 10,
         'min_data_in_leaf': 150,
         'reg_lambda': 0.5,  # L2 regularization term on weights.
         'reg_alpha': 0.5,  # L1 regularization term on weights.
         
         'colsample_bytree': 0.9,
         'subsample': 0.9,
#         'nthread': 32,
         'nthread': cpu_count(),
         'bagging_freq': 1,
         'verbose':-1,
         }


usecols = [f'f{i:03d}' for i in range(100)]
usecols += ['object_id']

# =============================================================================
# load
# =============================================================================
train = pd.read_feather('../data/train_f002_f012.f')[usecols]
test = pd.read_feather('../data/test_f002_f012.f')[usecols]

y_train = utils.load_target().target

X_train = train.drop(['object_id'], axis=1).values
X_test = test.drop(['object_id'], axis=1).values

# =============================================================================
# cv
# =============================================================================
dtrain = lgb.Dataset(X_train, y_train, categorical_feature=list(range(100)))
gc.collect()

ret = lgb.cv(param, dtrain, 9999, nfold=NFOLD, 
             early_stopping_rounds=100, verbose_eval=50, seed=SEED)

result = f"CV auc-mean: {ret['auc-mean'][-1]} + {ret['auc-stdv'][-1]}"
print(result)

utils.send_line(result)

# =============================================================================
# model
# =============================================================================
dtrain = lgb.Dataset(X_train, y_train, categorical_feature=list(range(100)))
model = lgb.train(param, dtrain, len(ret['auc-mean']))

imp = ex.getImp(model).sort_values(['gain', 'feature'], ascending=[False, True])
imp.to_csv(f'LOG/imp_{__file__}.csv', index=False)

# =============================================================================
# predict
# =============================================================================
preds = []