#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b"GIF89a"
    width = 160
    height = 120
    color_table = bytes([
        0xff, 0xff, 0xff, # white
        0x00, 0x00, 0x00, # black
    ])
    image_data = bytearray()
    for y in range(height):
        for x in range(width):
            image_data.append(int.from_bytes(rng.read(1), "big"))
    data = (
        b"\x07\x00\x01\x00" +
        struct.pack(">H", width) +
        struct.pack(">H", height) +
        b"\x80\x00\x00\xff\x0bNETSCAPE2.0\x03\x01" +
        struct.pack(">I", 0) +
        b";"
    )
    out.write(header + data)
