#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 15 20:46:47 2020

@author: jonasg
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
from scipy.optimize import minimize
from scipy.special import logsumexp

def logp(x, mu, sigma):
    return norm.logpdf(x, loc=mu, scale=sigma)

def logp_mvnorm(x, mu, Sigma):
    return multivariate_normal.logpdf(x, mean=mu, cov=Sigma)

def logp_mvnorm_diagonal(x, mu, Sigma):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.diag(Sigma))

def logp_mvnorm_fullcovariance(x, mu, Sigma):
    return multivariate_normal.logpdf(x, mean=mu, cov=Sigma)

def logp_mvnorm_cholesky(x, mu, L):
    return multivariate_normal.logpdf(x, mean=mu, cov=L @ L.T)

def logp_mvnorm_cholesky_lower(x, mu, L):
    return multivariate_normal.logpdf(x, mean=mu, cov=L.dot(L.T))

def logp_mvnorm_cholesky_upper(x, mu, U):
    return multivariate_normal.logpdf(x, mean=mu, cov=(U.T).dot(U))

def logp_mvnorm_cholesky_lower_inverse(x, mu, L):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.inv((L.T).dot(L)))

def logp_mvnorm_cholesky_upper_inverse(x, mu, U):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.inv((U.T).dot(U)))

def logp_mvnorm_cholesky_lower_determinant(x, mu, L):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.det((L.T).dot(L)))

def logp_mvnorm_cholesky_upper_determinant(x, mu, U):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.det((U.T).dot(U)))

def logp_mvnorm_cholesky_lower_inverse_determinant(x, mu, L):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.slogdet((L.T).dot(L))[1])

def logp_mvnorm_cholesky_upper_inverse_determinant(x, mu, U):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.slogdet((U.T).dot(U))[1])

def logp_mvnorm_cholesky_lower_inverse_determinant_sqrt(x, mu, L):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.slogdet((L.T).dot(L))[1]**(-0.5))

def logp_mvnorm_cholesky_upper_inverse_determinant_sqrt(x, mu, U):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.slogdet((U.T).dot(U))[1]**(-0.5))

def logp_mvnorm_cholesky_lower_inverse_determinant_sqrt_reciprocal(x, mu, L):
    return multivariate_normal.logpdf(x, mean=mu, cov=np.linalg.slogdet((L.T).dot(L))[1]**(-0.5)*(-1))

def logp_mvnorm_