#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Magic header
    out.write(b'GIF89a')
    
    # Screen descriptor
    out.write(struct.pack('<HHBB', 1024, 768, 0xF, 0))
    
    # Global color table
    for i in range(256):
        out.write(struct.pack('BBB', i, i, i))
        
    # Graphic control extension
    out.write(struct.pack('<BBHB', 0x21, 0xf9, 4, 0))
    
    # Image descriptor
    out.write(struct.pack('<HHHHBB', 0, 0, 1024, 768, 0, 0))
    
    # Local color table
    out.write(b'\x00\x00\x00\xff\xff\xff')
    
    # Pixel data
    while True:
        byte = rng.read(1)
        if not byte:
            break
            
        out.write(byte * 1024)
