#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    out.write(struct.pack('<HHBB', 200, 150, 0xF7, 0))

    # Global Color Table
    for i in range(4):
        out.write(struct.pack('<BBB', *[rng.read(1)[0] for _ in range(3)]))

    # Graphics Control Extension
    out.write(struct.pack('<BBHB', 0x21, 0xf9, 4, 0))

    # Image Data
    out.write(struct.pack('<BBHHBB', 0x2c, 0, 0, 200, 150, 0))

    # Local Color Table (same as global color table)
    for i in range(4):
        out.write(struct.pack('<BBB', *[rng.read(1)[0] for _ in range(3)]))

    # LZW Minimum Code Size
    out.write(struct.pack('<B', 8))

    # Pixel data
    for y in range(150):
        for x in range(200):
            out.write(struct.pack('<B', rng.read(1)[0]))

