#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0]

    # write header
    out.write(header + w + h + packed + bytes([bg]) + bytes([ar]))

    # global color table
    for _ in range(2 ** (packed[0] + 1)):
        out.write(rng.read(3))

    # image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\xff\x00'
    out.write(idesc)

    # local color table
    lct = b''
    for i in range(2 ** (packed[0] + 1)):
        lct += rng.read(3)
    out.write(lct)

    # LZW minimum code size
    out.write(bytes([rng.read(1)[0]]))

    # image data
    while True:
        # read a byte from the RNG
        b = rng.read(1)[0]

        if b == 0:
            break

        # write it to the output
        out.write(bytes([b]))
