#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate a random width and height
    width = int.from_bytes(rng.read(2), 'big', signed=False)
    height = int.from_bytes(rng.read(2), 'big', signed=False)

    # Write the width and height to the output stream
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))

    # Generate a random background color index
    bgcolor = int.from_bytes(rng.read(1), 'big', signed=False)

    # Write the background color index to the output stream
    out.write(bgcolor.to_bytes(1, 'big'))

    # Generate a random pixel aspect ratio
    par = int.from_bytes(rng.read(1), 'big', signed=False)

    # Write the pixel aspect ratio to the output stream
    out.write(par.to_bytes(1, 'big'))

    # Generate a random global color table flag
    gctflag = bool(int.from_bytes(rng.read(1), 'big', signed=False))

    # Write the global color table flag to the output stream
    if gctflag:
        out.write(b'\x01')
    else:
        out.write(b'\x00')

    # Generate a random global color table size
    gctsize = int.from_bytes(rng.read(1), 'big', signed=False)

    # Write the global color table size to the output stream
    out.write(gctsize.to_bytes(1, 'big'))

    # Generate a random global color table
    gct = []
    for i in range(2 ** (gctsize + 1)):
        rgb = [int.from_bytes(rng.read(1), 'big', signed=False) for _ in range(3)]
        gct.append(rgb)

    # Write the global color table to the output stream
    for rgb in gct:
        out.write(struct.pack('BBB', *rgb))

    # Generate a random block size
    blocksize = int.from_bytes(rng.read(1), 'big', signed=False)

    # Write the block size to the output stream
    out.write(blocksize.to_bytes(1, 'big'))

    # Generate a random graphic control extension
    gce = b''
    gce += b'\x21\xf9'
    gce += b'\x04'
    gce += rng.read(1)
    gce += rng.read(2)
    gce += rng.read(1)

    # Write the graphic control extension to the output stream
    out.write(gce)

    # Generate a random application extension
    appext = b''
    appext += b'\x21\xff'
    appext += b'\x0bNETSCAPE2.0'
    appext += b'\x03\x01'
    appext += rng.read(1)

    # Write the application extension to the output stream
    out.write(appext)

    # Generate a random image descriptor
    idesc = b''
    idesc += b'\x2c'
    idesc += rng.read(2)
    idesc += rng.read(2)
    idesc += rng.read(1)
    idesc += rng.read(1)

    # Write the image descriptor to the output stream
    out.write(idesc)

    # Generate a random local color table flag
    lctflag = bool(int.from_bytes(rng.read(1), 'big', signed=False))

    # Write the local color table flag to the output stream
    if lctflag:
        out.write(b'\x01')
    else:
        out.write(b'\x00')

    # Generate a random interlaced flag
    interlace = bool(int.from_bytes(rng.read(1), 'big', signed=False))

    # Write the interlaced flag to the output stream
    if interlace:
    out.write(datalen.to_bytes(3, 'big'))

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)