#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b"GIF89a")

    # Logical screen descriptor
    width = 1 + (rng.read(1)[0] % 256)
    height = 1 + (rng.read(1)[0] % 256)
    packed = bytes([
        0x87 | ((width >> 4) & 0x07),
        (width & 0x0F) << 4 | ((height >> 4) & 0x0F),
        (height & 0x0F) << 4 | 0x07])
    out.write(packed)

    # Global color table
    for _ in range(0, 2**((packed[1] >> 4) + 1)):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(bytes([r, g, b]))

    # Image descriptor
    out.write(bytes([
        0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xFF, 0xFF, 0xFF, 0x00]))

    # Local color table
    for _ in range(0, 2**((packed[2] >> 4) + 1)):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(bytes([r, g, b]))

    # Image data
    for i in range(0, width * height):
        out.write(bytes([rng.read(1)[0]]))


if __name__ == "__main__":
    if len(sys.argv)!= 2:
        print("Usage: {} <output>".format(sys.argv[0]), file=sys.stderr)
        exit(-1)

    with open("/dev/urandom", "rb") as rng:
        with open(sys.argv[1], "wb") as f:
            generate_random_gif(rng, f)