#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header (6 bytes)
    out.write(b'GIF89a')

    # Write the logical screen descriptor (7 bytes)
    width = 1 + int(rng.read(1)[0] % 255)
    height = 1 + int(rng.read(1)[0] % 255)
    packed = struct.pack('>HHBBB', width, height, 0xF0 | rng.read(1)[0], 0, 0)
    out.write(packed)

    # Write the global color table (3 * 256 = 768 bytes)
    for _ in range(256):
        r = rng.read(1)[0] & 0x0F
        g = rng.read(1)[0] & 0x0F
        b = rng.read(1)[0] & 0x0F

        packed = struct.pack('<BBB', r << 4 | r, g << 4 | g, b << 4 | b)
        out.write(packed)

    # Write the image descriptor (10 bytes)
    packed = struct.pack('>HHBHB', 0, 0, 0xF0 | rng.read(1)[0], 0, 0)
    out.write(packed)

    # Write the local color table (3 * 256 = 768 bytes)
    for _ in range(256):
        r = rng.read(1)[0] & 0x0F
        g = rng.read(1)[0] & 0x0F
        b = rng.read(1)[0] & 0x0F

        packed = struct.pack('<BBB', r << 4 | r, g << 4 | g, b << 4 | b)
        out.write(packed)

    # Write the graphics control extension block (8 bytes)
    packed = struct.pack('>BBHBBBBB', 0x21, 0xF9, 4, 0xFF, 0, 0, 0, 0)
    out.write(packed)

    # Write the plain text extension block (19 bytes)
    packed = struct.pack('>BBHBBBBBBBBBBBB',
        0x21, 0x01, 12, ord('A'), ord('S'), ord('C'), ord('I'), ord('I'),
        ord('F'), ord('F'), ord(' '), ord(' '), ord(' ')
    )
    out.write(packed)

    # Write the application extension block (11 bytes)
    packed = struct.pack('>BBHBBBBBB',
        0x21, 0xFF, 11, ord('A'), ord('S'), ord('C'), ord('I'), ord('I'),
        ord('F'), ord('F'), ord(' ')
    )
    out.write(packed)

    # Write the comment extension block (23 bytes)
    packed = struct.pack('>BBHBBBBBBBBBBBBBBBB',
        0x21, 0xFE, 21, ord('A'), ord('S'), ord('C'), ord('I'), ord('I'),
        ord('F'), ord('F'), ord(' '), ord(' '), ord(' '), ord(' '), ord(' '),
        ord(' '), ord(' '), ord(' '), ord(' '), ord(' '), ord(' ')
    )
    out.write(packed)

    # Write the graphic data sub-block (1 byte)
    out.write(b'\x01')

