#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
	width = 1024
	height = 768

	# Header
	out.write(b'GIF') # Signature
	out.write(struct.pack('<HH', width, height)) # Screen descriptor
	out.write(b'\x87\x00') # Global color table with 256 colors and no sort flag set

	# Image block
	out.write(b'\x2C') # Image separator
	out.write(struct.pack('<HHHHB', 0, 0, width, height, 0)) # Left-top corner position + size of image
	out.write(b'\x00') # No local color table
	out.write(b'\x00') # LZW minimum code size

	# Pixel data
	for y in range(height):
		for x in range(width // 8):
			byte = rng.read(1)[0]

			for i in reversed(range(8)):
				out.write((bytes([byte & 1]) if byte & 1 else b'\x00'))

				byte >>= 1

		if len(out.getbuffer()) % 2 == 1:
			out.write(b'\x00')

# Main function
if