#!/usr/bin/env python3

import struct
from typing import BinaryIO

with open('rng.bin', 'rb') as in_file:
    with open('rng-fixed.bin', 'wb') as out_file:
        fix_rng(in_file, out_file)

def fix_rng(inp: BinaryIO, out: BinaryIO):
    header = inp.read(8)

    rng = inp.read()

    datalen = int.from_bytes(
            rng[0x14:0x16], byteorder='little', signed=False
        ) - 0x16
    if len(rng)!= datalen + 0x16:
        raise ValueError("Invalid RNG file")

    # Fix the seed
    rng = bytes([
        0x5a, 0xa9, 0xd7, 0x9c, 0x3b, 0x6e, 0x9d, 0x3f,
        0x66, 0xdc, 0xb9, 0x73, 0xe6, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x99, 0x33,
        0x66, 0xcc, 0x99, 0x33, 0x66, 0xcc, 0x9            rng.read(2), byteorder='big', signed=False
        )
    data = rng.read(datalen)
    out.write(header + data)
