#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write('GIF89a'.encode())

    width = 1024
    height = 768
    bitdepth = 8
    colorres = 7
    background = 0x00
    aspectratio = 0x40
    flags = 0b10000000 | (bitdepth - 1) << 5 | (colorres - 1) << 3 | (aspectratio - 1)
    palette = bytes([background] * ((1 << bitdepth) ** 3))

    # Logical screen descriptor
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write(flags.to_bytes(1, 'little'))
    out.write(palette[:3].ljust(3, b'\x00'))

    # Global color table
    out.write(palette)

    # Image descriptor
    out.write(b'\x2c')
    out.write((0).to_bytes(2, 'little'))
    out.write((0).to_bytes(2, 'little'))
    out.write(((width // 8) * height).to_bytes(2, 'little'))
    out.write((0).to_bytes(1, 'little'))
    out.write((0).to_bytes(1, 'little'))
    out.write((0).to_bytes(1, 'little'))
    out.write((0).to_bytes(1, 'little'))

    # LZW minimum code size
    out.write(b'\x08')

    # Pixel data
    for y in range(height):
        byte = 0
        for x in range(width // 8):
            if x % 2 == 0:
                byte |= int(next(rng)) << 7

            else:
                byte |= int(next(rng)) << 6

                out.write(byte.to_bytes(1, 'little'))

        if width % 8!= 0:
            out.write(byte.to_bytes(1, 'little'))


if