#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio - fixed to 4:3
    ar = b'\x00'

    # global palette
    palette = [bytes([i]) for i in range(256)]
    palette[bg] += b'\xff\xff\xff'

    # image descriptor
    imgdesc = b'\x2c' + w + h + packed + b'\x00'

    # graphics control extension
    gce = b'\x21\xf9\x04' + bytes([(rng.read(1)[0] & 0x3f) | 0x80]) + \
            b'\x00\x00' + bytes([rng.read(1)[0]]) + b'\x00'

    # application extension
    appext = b'\x21\xff\x0bNETSCAPE2.0' + b'\x03\x01' + \
                bytes([rng.read(1)[0], rng.read(1)[0]]) + b'\x00'

    # comment extension
    coment = b'\x21\xfe' + bytes([rng.read(1)[0]]) + \
                    b'\x00' + rng.read(rng.read(1)[0])

    # plain text extension
    ptext = b'\x21\x01' + bytes([rng.read(1)[0]]) + \
                        b'\x00' + rng.read(rng.read(1)[0])

    # graphic control label
    gclabel = b'\x21\xf9\x04'

    # plain text label
    plabel = b'\x21\x01'

    # block size
    bs = b'\x00'

    # data subblock
    ds = b'\x00' * rng.read(1)[0]

    # trailer
    trailer = b'\x3b'

    out.write(header + w + h + packed + bg + ar + imgdesc + gce + appext +
              coment + ptext + gclabel + bs + ds + trailer)

