#!/usr/bin/env python3

import os
from random import SystemRandom

# The path of this script file.
script_path = os.path.dirname(__file__)

# The path of the directory where the images are located.
image_dir = os.path.join(script_path, 'images')

# The path of the output file.
output_path = os.path.join(script_path,'seedgen.bin')

with open(output_path, mode='wb') as out:
    # Open a pseudo-random number generator based on system entropy sources.
    rng = SystemRandom()

    # Read an image filename from the list of files in the image directory.
    with open(os.path.join(image_dir, rng.choice(os.listdir(image_dir))),
              mode='rb') as img:
        # Write the image header (magic and size).
        out.write(img.read(8))

        # Read the image data length from the image file.
        datalen = int.from_bytes(
            img.read(2), byteorder='big', signed=False) + \
                 int.from_bytes(
                     bytes([rng.randint(0x00, 0xff)]),
                     byteorder='big', signed=False) * 0x10000

        # Write the image data length to the output stream.
        out.write(datalen.to_bytes(length=2, byteorder='big'))

        # Copy the image data from the input stream to the output stream.
        while True:
            buf = img.read(4096)

            if len(buf) == 0: break;

            out.write(buf)

    # Read another image filename from the list of files in the image directory.
    with open(os.path.join(image_dir, rng.choice(os.listdir(image_dir))),
              mode='rb') as img:
        # Write the image header (magic and size).
        out.write(img.read(8))

        # Read the image data length from the image file.
        datalen = int.from_bytes(
            img.read(2), byteorder='big', signed=False) + \
                 int.from_bytes(
                     bytes([rng.randint(0x00, 0xff)]),
                     byteorder='big', signed=False) * 0x10000

        # Write the image data length to the output stream.
        out.write(datalen.to_bytes(length=2, byteorder='big'))

        # Copy the image data from the input stream to the output stream.
        while True:
            buf = img.read(4096)

            if len(buf) == 0: break;

            out.write(buf)

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(32)).to_bytes(length=4, byteorder='big'))

    # Write the seed value to the output stream.
    out.write(((rng.getrandbits(32) << 32) |
               (rng.getrandbits(32))).to_bytes(
                   length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(64)).to_bytes(length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(64)).to_bytes(length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(64)).to_bytes(length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(64)).to_bytes(length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(64)).to_bytes(length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(64)).to_bytes(length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(64)).to_bytes(length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.write((rng.getrandbits(64)).to_bytes(length=8, byteorder='big'))

    # Write the seed value to the output stream.
    out.            rng.read(2), byteorder='big'))

    # Write the image data length to the output stream.
    out.write(datalen)

    # Copy the image data from the input stream to the output stream.
    while True:
        buf = rng.read(4096)

        if len(buf) == 0: break;

        out.write(buf)

