#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a' # 6 bytes
    width = int.from_bytes(
            rng.read(2), byteorder='little', signed=False)
    height = int.from_bytes(
            rng.read(2), byteorder='little', signed=False)
    flags = int.from_bytes(
            rng.read(1), byteorder='little', signed=False)
    background_color_index = int.from_bytes(
            rng.read(1), byteorder='little', signed=False)
    pixel_aspect_ratio = int.from_bytes(
            rng.read(1), byteorder='little', signed=False)
    global_color_table_flag = (flags >> 7) & 0x1
    color_resolution = ((flags >> 4) & 0x7) + 1
    sort_flag = (flags >> 3) & 0x1
    size_of_global_color_table = (((flags << 1) & 0xe) |
                                 ((flags >> 5) & 0x1)) + 1
    local_color_table_flag = flags & 0x1
    interlace_flag = (flags >> 6) & 0x1
    sorted_color_table_flag = (flags >> 2) & 0x1
    reserved_bits = flags & 0x3
    frame_count = int.from_bytes(
            rng.read(1), byteorder='little', signed=False)
    for i in range(frame_count):
        left = int.from_bytes(
                rng.read(2), byteorder='little', signed=False)
        top = int.from_bytes(
                rng.read(2), byteorder='little', signed=False)
        width = int.from_bytes(
                rng.read(2), byteorder='little', signed=False)
        height = int.from_bytes(
                rng.read(2), byteorder='little', signed=False)
        flags = int.from_bytes(
                rng.read(1), byteorder='little', signed=False)
        delay_time = int.from_bytes(
                rng.read(2), byteorder='little', signed=False)
        transparent_color_index = int.from_bytes(
                rng.read(1), byteorder='little', signed=False)
        blocksize = int.from_bytes(
                rng.read(1), byteorder='little', signed=False)
        if blocksize!= 0:
            raise ValueError('Invalid blocksize')
        lzwmincodesize = int.from_bytes(
                rng.read(1), byteorder='little', signed=False)
        data = rng.read()
        while len(data)!= 0 and data[-1] == 0:
            data = data[:-1]
        out.write(struct.pack('<HHHHBBHBH',
                              left, top, width, height, flags,
                              delay_time, transparent_color_index,
                              blocksize, lzwmincodesize))
        out.write(data)


# Generates an invalid GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_invalid_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a' # 6 bytes
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big') + data)
