#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        )

    # Write header and screen descriptor
    out.write(header + bytes([0x1f]) + b'\x00\x00')
    out.write(struct.pack('>HHHBBBBBHB', 640, 480,
        0xf7, 0xff, 0x0b, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00))

    # Write global color table (black and white only)
    for i in range(2):
        out.write(b'\x00\x00\x00\xff\xff\xff')

    # Write image data
    while datalen > 0:
        chunklen = min(datalen, 255)

        if chunklen == 255:
            out.write(chunklen.to_bytes(1, 'big'))
        else:
            out.write((chunklen | 0x80).to_bytes(1, 'big'))

        out.write(rng.read(chunklen))
        datalen -= chunklen

