#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate the width and height of the image
    width  = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB',
        0x78 + (width >> 4),
        width & 0xf,
        0x78 + (height >> 4),
        height & 0xf,
        0x87))

    # Write the global color table
    for i in range(0, 256):
        out.write(struct.pack('BBB',
            rng.read(1)[0],
            rng.read(1)[0],
            rng.read(1)[0]))

    # Write the loop control block
    out.write(b'\x01\x00\x00')

    # Write the image descriptor
    out.write(struct.pack('<HHHHBB',
        0,
        0,
        width - 1,
        height - 1,
        0,
        0))

    # Get the length of the image data from the RNG
    datalen = rng.read(1)[0] % 256 + 1

    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)