#!/usr/bin/env python3

import os
import sys
import struct
from collections import namedtuple


def read_file(filename):
    with open(filename, 'rb') as f:
        return f.read()


def write_file(filename, data):
    with open(filename, 'wb') as f:
        f.write(data)


def get_header():
    header = b''

    # Magic number
    header += bytes([0x41, 0x56, 0x49])

    # Version
    header += bytes([0x02, 0x00, 0x00, 0x00])

    # File size (will be overwritten later)
    header += bytes([0x00, 0x00, 0x00, 0x00])

    # Header length
    header += bytes([0x08 * 4 + 1])

    # Number of frames in file
    header += bytes([0x00, 0x00, 0x00, 0x00])

    # Unknown value
    header += bytes([0x00, 0x00, 0x00, 0x00])

    # Frames per second
    header += bytes([0x00, 0x00, 0x00, 0x00])

    # Width and height
    header += bytes([0x00, 0x00, 0x00, 0x00])
    header += bytes([0x00, 0x00, 0x00, 0x00])

    # Unknown values
    for i in range(7):
        header += bytes([0x00, 0x00, 0x00, 0x00])

    return header


def get_frame_size(width, height):
    frame_size = width * height * 3 // 2
    if frame_size % 4!= 0:
        frame_size += 4 - (frame_size % 4)
    return frame_size


def convert_frame(frame):
    result = bytearray()

    for i in range(len(frame)):
        if i % 2 == 0:
            result.append((frame[i] & 0xF0) | ((frame[i+1] >> 4) & 0x0F))

        else:
            result.append(((frame[i-1] << 4) & 0xF0) | (frame[i] & 0x0F))

    return bytes(result)


def main():
    if len(sys.argv) < 3 or not os.path.isfile(sys.argv[1]):
        print('Usage:', sys.argv[0], '<input> <output>')
        exit(-1)

    input_filename = sys.argv[1]
    output_filename = sys.argv[2]

    input_data = read_file(input_filename)
    input_frames = []

    while True:
        if len(input_data) >= 4:
            frame_size = struct.unpack('<I', input_data[:4])[0]

            if len(input_data) >= 4 + frame_size:
                input_frames.append(convert_frame(
                    input_data[4:4+frame_size]))

                input_data = input_data[4+frame_size:]

            else:
                break

        else:
            break

    header = get_header()
    header = header[:4] + \
             struct.pack('<I', len(header) +
                         sum([get_frame_size(*struct.unpack('<II', header[-8:-4]))
                              for _ in input_frames])) + \
             header[8:]

    write_file(output_filename, header +
               b''.join(input_frames))


if