#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + bytes([0x21]) + b'NETSCAPE2.0\x03\x01\x00\x00'

    # Write the header to the output file
    out.write(header)

    # Generate the color table
    colors = [struct.pack('BBB', *rng.read(3)) for _ in range(768)]

    # Write it to the output file
    for c in colors:
        out.write(c)

    # Write the image block
    out.write(b'\x2C')

    # Write the width and height of the image
    out.write(bytes([rng.read(1)[0], 0]))

    # Write the flags byte (no local colour map, interlaced, no transparency)
    out.write(b'\x40')

    # Write the background color index
    out.write(bytes([rng.read(1)[0]]))

    # Write the pixel aspect ratio
    out.write(b'\x00')

    # Write the graphics control extension
    out.write(b'\x21\xF9\x04')

    # Write the delay time and transparent color index
    out.write(bytes([rng.read(1)[0], rng.read(1)[0]]))

    # Write the end of graphic control extension code
    out.write(b'\x00')

    # Write the image data
    out.write(b'\x21\xFF\x0B')

    # Write the LZW minimum code size
    out.write(b'\x08')

    # Write the image data
    while True:
        chunk = rng.read(1)

        if not len(chunk):
            break

        out.write(chunk)

