#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate the logical screen descriptor
    width = 2**10 + int.from_bytes(rng.read(1), 'little') % (2**10 - 16)
    height = 2**10 + int.from_bytes(rng.read(1), 'little') % (2**10 - 16)
    packed = ((width & 0x7F) << 8) | (height & 0xFF)
    out.write(struct.pack('<H', packed))
    out.write(b'\x87\x00')

    # Write the global color table
    for _ in range(4):
        out.write(int.to_bytes(int.from_bytes(rng.read(1), 'little'), length=1, byteorder='big'))

    # Write the image descriptor
    out.write(b'\x2C')
    left = 0
    top = 0
    width -= 15
    height -= 15
    packed = (((left & 0xF8) >> 3) | ((top & 0xFE) << 3)) \
            | ((width & 0x07) << 12) \
            | ((height & 0x07) << 15)
    out.write(struct.pack('<L', packed))
    out.write(b'\x00')

    # Write the local color table
    for _ in range(4):
        out.write(int.to_bytes(int.from_bytes(rng.read(1), 'little'), length=1, byteorder='big'))

    # Write the LZW minimum code size
    out.write(b'\x08')

    # Write the image data
    while True:

        # Get the next bit from the RNG and write it to the stream
        b = int.from_bytes(rng.read(1), 'little') & 0x01
        out.write(int.to_bytes(b, length=1, byteorder='big'))

        if not b:

            # If we just wrote a zero, get another bit and repeat until we have
            # a one followed by a zero
            while True:
                b = int.from_bytes(rng.read(1), 'little') & 0x01
                out.write(int.to_bytes(b, length=1, byteorder='big'))

                if b == 0:
                    break

