#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# returns its size in bytes.
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng.randint(0x7f, 0xff)
    height = 1 + rng.randint(0x7f, 0xff)
    flags = (width & 0x80) | ((height & 0x80) >> 4)
    out.write(struct.pack('<2BHB', width, height, flags, 0))

    # Global color table
    for i in range(0, 256 * 3 // 4):
        out.write(bytes([rng.randint(0, 255)]))

    # Image descriptor
    left = rng.randint(0, 0xfff)
    top = rng.randint(0, 0xfff)
    width = 1 + rng.randint(0x7f, 0xff)
    height = 1 + rng.randint(0x7f, 0xff)
    flags = 0x80 | rng.randint(0, 7)
    out.write(struct.pack('<HHHBB', left, top, width, height, flags))

    # Local color table
    for i in range(0, 256 * 3 // 4):
        out.write(bytes([rng.randint(0, 255)]))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng.randint(0, 255)]))

    return 13 + 7 + 256 * 3 // 4 + 13 + width * height

