#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng.randint(0x7f)
    height = 1 + rng.randint(0x7f)
    flags = (width & 0x80) | ((height & 0x80) >> 7)
    bgColorIndex = rng.randint(256)
    pixelAspectRatio = 0
    out.write(struct.pack('<HHBBB',
        width, height,
        flags, bgColorIndex, pixelAspectRatio))

    # Global color table
    for i in range(256):
        r = rng.randint(256)
        g = rng.randint(256)
        b = rng.randint(256)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    leftPos = 0
    topPos = 0
    width = 1 + rng.randint(0x7f)
    height = 1 + rng.randint(0x7f)
    flags = (leftPos & 0x80) | ((topPos & 0x80) >> 7) \
            | ((width & 0x80) >> 14) | ((height & 0x80) >> 15)
    localColorTableFlag = False
    interlaceFlag = False
    sortFlag = False
    reserved = 0
    lctSize = 0
    out.write(struct.pack('<HHBBBBB',
        leftPos, topPos,
        width, height,
        flags, localColorTableFlag,
        interlaceFlag, sortFlag,
        lctSize, reserved))

    # Local color table
    if localColorTableFlag:
        for i in range(lctSize):
            r = rng.randint(256)
            g = rng.randint(256)
            b = rng.randint(256)
            out.write(struct.pack('BBB', r, g, b))

    # LZW minimum code size
    minCodeSize = 2 + rng.randint(3)
    out.write(bytes([minCodeSize]))

    # Data blocks
    while True:
        blockSize = rng.randint(128)
        if blockSize == 0: break

        blockType = rng.randint(2)
        if blockType == 0:
            data = bytes([rng.randint(256)]) * blockSize
        elif blockType == 1:
            data = bytes([(blockSize << 8) | rng.randint(256)])
        else: assert False

        out.write(data)

    # Trailer
    out.write(b';')

