#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    header = b'GIF89a\x01\x00\x01\x00\x80\xff\x00\xc0\xff\x00\x00\x00!\xf9\x04\t\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (2**16-1) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (2**16-1) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    pxsize = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    color  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    gct    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], pxsize[0], color[0]))

    if gct == b'\x00':

        # Global Color Table
        for i in range(pow(2,color[0]) * 3):
            out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    else:

        # Local Color Table
        for i in range(pow(2,gct[0]) * 3):
            out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))


    # Image Descriptor
    left   = int.from_bytes(rng.read(2), byteorder='little') % (width - 1)
    top    = int.from_bytes(rng.read(2), byteorder='little') % (height - 1)
    width  = int.from_bytes(rng.read(2), byteorder='little') % (width - left) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (height - top) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    lct    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    interl = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHHHBB', left, top, width, height, flags[0], lct[0], interl[0]))

    if lct == b'\x00':

        # Local Color Table
        for i in range(pow(2,color[0]) * 3):
            out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Image Data
    data = []
    for _ in range(height):
        for _ in range((width+7)//8):
            data += [int.from_bytes(rng.read(1), byteorder='little')]
    out.write(bytes(data))

    # Trailer
    trailer = b'\x3b'
    out.write(trailer)



if