#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = bytes([rng.read(1)[0] for _ in range(datalen)])

    out.write(header + struct.pack('>H', len(data)))
    out.write(b'\x87\x00\x00') # Logical screen descriptor
    out.write(b'\x00\x00')     # Global color table (no colors)
    out.write(b'\xF7\x00')     # Graphics control extension
    out.write(b'\x04')         # 4 bytes of block size
    out.write(b'\x00\x00')     # No transparancy
    out.write(b'\x00\x00')     # Delay time
    out.write(b'\x00')         # Transparent color index
    out.write(b'\x2C')         # Image descriptor
    out.write(b'\x00\x00')     # Left position
    out.write(b'\x00\x00')     # Top position
    out.write(b'\x01\x00\x01\x00')   # Width and height
    out.write(b'\x00')         # Local color table flag
    out.write(b'\x00')         # Interlace flag
    out.write(b'\x00')         # Sort flag
    out.write(b'\x00')         # Reserved
    out.write(b'\x00')         # Size of local color table
    out.write(b'\x00')         # Background color index
    out.write(b'\x00')         # Pixel aspect ratio
    out.write(data)           # Data

