#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
    f: BinaryIO,
    width: int = 1024, height: int = 768,
    color_depth: int = 8, bg_color: int = 0xFFFFFF, fg_color: int = 0x000000):

    # Header
    f.write(b'GIF89a')
    f.write(struct.pack('<HH', width, height))
    f.write(bytes([
        (0x80 | 0x70),
        0x0B,
        0, # No global palette
        0xFF, 0xFF, 0xFF, # White background
        0x00, 0x00, 0x00])) # Black foreground

    # Logical screen descriptor
    f.write(bytes([
        color_depth - 1,
        0, 0, # X and Y size of the image in pixels
        0xF7, # Packed field: Global color table flag set, sort flag clear,
                 # Size of global color table is 256 bytes
        0])) # Background color index
    for _ in range(3 * ((2 << color_depth) - 1)):
        f.write(struct.pack('B', bg_color & 0xFF))
        bg_color >>= 8
    f.write(bytes([
        0, 0])) # Pixel aspect ratio

    # Image descriptor
    f.write(bytes([
        0x2C, # Image separator
        0, 0, # Left position
        0, 0, # Top position
        width % 256, width // 256, # Width
        height % 256, height // 256, # Height
        0, # Local color table flag clear
        0, # Interlace flag clear
        0, # Sort flag clear
        0, # Reserved
        0, 0])) # LZW minimum code size
    for _ in range((2 << color_depth) - 1):
        f.write(struct.pack('B', fg_color & 0xFF))
        fg_color >>= 8

    # Image data
    lzw_min_code_size = 2
    while True:
        if lzw_min_code_size == 12:
            break

        # Write a single-byte block containing all possible codes with this min code size
        num_codes = 1 << lzw_min_code_size
        for i in range(num_codes):
            f.write(bytes([i]))

        # Write an empty byte to indicate end of data
        f.write(b'\x00')

        # Increase min code size by one
        lzw_min_code_size += 1

