#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# returns its size in bytes.
def generate_random(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b"GIF89a")

    width = 256 + int.from_bytes(rng.read(1), byteorder="little", signed=False) % (
        256 - 100
    )
    height = 256 + int.from_bytes(rng.read(1), byteorder="little", signed=False) % (
        256 - 100
    )

    # Logical screen descriptor
    out.write(struct.pack("<HHBBB", width, height, 0xF7, 0x0B, 0))

    # Global color table
    for i in range(0, 256 * 3, 3):
        out.write(rng.read(1))

    # Image descriptor
    out.write(struct.pack("<HHHHBB", 0, 0, width, height, 0, 0))

    # Local color table
    for i in range(0, 256 * 3, 3):
        out.write(rng.read(1))

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))

    return out.tell()


if