#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    imgdata = bytearray()
    while len(imgdata) < 250 * 1024:

        # Generate a new byte from the RNG.
        b = int.from_bytes(rng.read(1), 'big', signed=False)

        if b == 0x00:
            continue
        elif b in [0x2C, 0x21]:

            # If we see an image separator or extension block,
            # write it to the output stream as is.
            out.write(struct.pack('B', b))

            # Read the rest of the block.
            while True:
                b = int.from_bytes(rng.read(1), 'big', signed=False)

                if b == 0x00:
                    continue
                else:

                    # We've reached the end of the block;
                    # write the last byte back to the input buffer.
                    rng.seek(-1, os.SEEK_CUR)
                    break

        else:

            # Otherwise, add this byte to our image data.
            imgdata += bytes([b])

    # Write the image descriptor header to the output stream.
    out.write(struct.pack('<HHHHBB',
        0, 0, 0, len(imgdata), 0, 0
    ))

    # Write the image data to the output stream.
    for i in range(len(imgdata)):
        out.write(struct.pack('B', imgdata[i]))


if