#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main(argv):
    if len(argv)!= 2:
        print('Usage:', argv[0], '<input file>')
        return

    with open(argv[1], 'rb') as f:
        data = f.read()

        # Check magic number
        assert data[:4] == b'Yaz0', "Invalid Yaz0 header"

        # Get decompressed size from the header
        decomp_size, = struct.unpack('<I', data[8:12])

        # Decompress to a new buffer
        outdata = bytearray(decomp_size)
        inpos = 12
        outpos = 0
        while True:
            ctrlbyte = data[inpos]

            for i in range(7, -1, -1):
                bit = (ctrlbyte >> i) & 1

                if bit == 0:
                    # Copy one byte verbatim
                    outdata[outpos] = data[inpos + 1]
                    outpos += 1

                    inpos += 2
                else:
                    # Copy two bytes from backreference
                    offset, length = struct.unpack('<HB', data[inpos + 1 : inpos + 4])
                    offset &= ~0x8000
                    offset |= ((ctrlbyte << 8) | data[inpos + 4]) & 0x7FFF
                    length += 2

                    for j in range(length):
                        outdata[outpos] = outdata[outpos - offset]
                        outpos += 1

                    inpos += 5

            if inpos >= len(data):
                break

        # Write output
        with open(os.path.splitext(argv[1])[0], 'wb') as of:
            of.write(bytes(outdata))


if