#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a' + bytes([
        0x12,  # width
        0x34,  # height
        0xf7,  # flags (global color table present)
        0x0b   # background color index
    ])

    palette = [struct.pack('BBB', *rng.read(3)) for _ in range(0xff)]

    data = []
    for i in range(0x1234):
        data += [palette[i % len(palette)], b'\x00']

    out.write(header)
    out.write(bytes(data))

