#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
    out: BinaryIO,
    rng: BinaryIO = open("/dev/urandom", "rb"),
):
    # Header
    header = b"GIF89a\x01\x00\x01\x00\x80\xff\x00\xc0\xff\x00\x00\x00!\xf9\x04\t\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;"

    # Logical screen descriptor (7 bytes)
    lsd = bytearray()
    lsd.append((rng.read(1)[0] & 0b1111111) | 0b10000000)  # Screen width
    lsd.append(((rng.read(1)[0] & 0b111111) << 3) | ((lsd[0] >> 5) & 0b111))  # Screen height
    lsd.append(
        0b10000000
        | (((rng.read(1)[0] & 0b111) << 6) | ((lsd[0] & 0b11111) << 2) | ((lsd[1] >> 6) & 0b11)))  # Packed field
    lsd.append((((rng.read(1)[0] & 0b11) << 5) | ((lsd[1] & 0b111111) >> 3)))  # Background color index
    lsd.append(rng.read(1)[0])  # Pixel aspect ratio
    lsd.append(0)  # Garbage padding

    # Global color table (768 bytes)
    gct = bytearray([rng.read(1)[0] for _ in range(768)])

    # Image descriptor (10 bytes)
    id = bytearray()
    id.append(rng.read(1)[0] & 0b1111111)  # Left position of image
    id.append(rng.read(1)[0] & 0b1111111)  # Top position of image
    id.append(((rng.read(1)[0] & 0b111111) << 3) | ((id[0] >> 5) & 0b111))  # Image width
    id.append(
        0b10000000
        | (((rng.read(1)[0] & 0b111) << 6) | ((id[0] & 0b11111) << 2) | ((id[1] >> 6) & 0b11)))  # Packed field
    id.append((((rng.read(1)[0] & 0b11) << 5) | ((id[1] & 0b111111) >> 3)))  # Local color table flag and interlace flag
    id.append(rng.read(1)[0])  # Sorting flag
    id.append(rng.read(1)[0])  # Reserved
    id.append(rng.read(1)[0])  # LZW minimum code size

    # Write everything to `out`
    out.write(header + lsd + gct + id)


if __name__ == "__main__":
    if len(sys.argv)!= 2 or not sys.argv[1].endswith(".gif"):
        print("Usage: {} <output-file>".format(sys.argv[0]))
        exit(-1)

    with open(sys.argv[1], "wb") as f:
        generate_random_gif(f)