#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):
    
    # Write the magic header (GIF89a) to the output stream
    out.write('GIF89a'.encode())

    # Generate 16-bit integers for width and height of the image
    # and write them to the output stream
    w = int.from_bytes(rng.read(2), byteorder='big')
    h = int.from_bytes(rng.read(2), byteorder='big')
    out.write(w.to_bytes(2, 'big'))
    out.write(h.to_bytes(2, 'big'))

    # Generate a single byte for the color table size and write it
    # to the output stream
    ctsize = int.from_bytes(rng.read(1), byteorder='big')
    out.write(ctsize.to_bytes(1, 'big'))

    # Generate an integer for the global color table flag and write it
    gctflag = int.from_bytes(rng.read(1), byteorder='big')
    out.write(gctflag.to_bytes(1, 'big'))

    # Generate an integer for the background color index and write it
    bgindex = int.from_bytes(rng.read(1), byteorder='big')
    out.write(bgindex.to_bytes(1, 'big'))

    # Generate an integer for the pixel aspect ratio and write it
    pxaspect = int.from_bytes(rng.read(1), byteorder='big')
    out.write(pxaspect.to_bytes(1, 'big'))

    # Generate three bytes for the length of the local color table
    # and write them to the output stream
    lctlen = int.from_bytes(rng.read(3), byteorder='big')
    out.write(lctlen.to_bytes(3, 'big'))

    # Generate some random bytes for the local color table
    # and write them to the output stream
    lct = rng.read(lctlen)
    out.write(lct)

    # Generate one byte for the loop count and write it to the output stream
    loopcount = int.from_bytes(rng.read(1), byteorder='big')
    out.write(loopcount.to_bytes(1, 'big'))

    # Generate three bytes for the length of the image data
    # and write them to the output stream
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(datalen.to_bytes(3, 'big'))

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)