#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom) and
# writes it to disk.
def generate_random(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor (10 bytes)
    width = 2**rng.read(1)[0] % 50 + 10
    height = 2**rng.read(1)[0] % 50 + 10
    flags = b'\x87' if rng.read(1)[0] & 1 else b'\x00'
    bgColorIndex = rng.read(1)[0] % 256
    pixelAspectRatio = b'\x00'

    out.write(struct.pack('<HHBBB',
        width, height,
        flags[0], bgColorIndex, pixelAspectRatio[0]))

    # Global color table (3 * 256 bytes)
    for i in range(256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        out.write(struct.pack('<BBBB',
            r, g, b,
            0))

    # Image descriptor (10 bytes)
    leftPos = 0
    topPos = 0
    flags = b'\x80\x00'
    delayTime = 0
    transparentColorIndex = 0
    endOfImage = b''

    out.write(struct.pack('<HHBBHB',
        leftPos, topPos,
        flags[0], delayTime,
        transparentColorIndex,
        0))

    # Image data block
    size = int((width / 8) * height)
    lzwSize = size // 255 + 1
    clearCode = 2 ** 8 - 1
    eoiCode = 2 ** 8 - 1

    out.write(struct.pack('<B', lzwSize))
    out.write(b'\xFF')
    out.write(clearCode.to_bytes(1, 'big'))
    out.write(eoiCode.to_bytes(1, 'big'))

    for _ in range(size):
        out.write(rng.read(1))

    out.write(endOfImage)

