#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    header = b"GIF89a"

    width = 1024
    height = 768
    color_table_size = 256
    global_color_table = True
    background_index = 0
    aspect_ratio = 0x00

    # Write header
    out.write(header)

    # Logical screen descriptor
    out.write(struct.pack("<HHBBB",
                          width,
                          height,
                          (global_color_table << 7 |
                           color_table_size),
                          background_index,
                          aspect_ratio))

    # Global color table
    for i in range(0, color_table_size * 3, 3):
        r = int.from_bytes(rng.read(1), byteorder="little") % 256
        g = int.from_bytes(rng.read(1), byteorder="little") % 256
        b = int.from_bytes(rng.read(1), byteorder="little") % 256

        out.write(struct.pack("BBB",
                              r,
                              g,
                              b))

    # Image descriptor
    image_descriptor = struct.pack("<BBHBBHHB",
                                   0x2C,
                                   (global_color_table << 7 |
                                    0),
                                   width,
                                   height,
                                   0,
                                   0,
                                   0,
                                   0)

    out.write(image_descriptor)

    # Pixel data
    for y in range(height):
        for x in range(width):
            index = int.from_bytes(rng.read(1), byteorder="little")

            if index >= color_table_size:
                raise Exception("Invalid pixel value")

            out.write(struct.pack("B",
                                  index))


if