#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1

    # Write header and screen descriptor
    out.write(header)
    out.write(struct.pack('>HHBBB',
            0x200, 0x200, 0xf7, 0xff, 0))

    # Write global color table (black & white only)
    for _ in range(0, 256):
        out.write(b'\x00\x00\x00')

    # Write image data
    while datalen > 0:
        chunklen = min(datalen, 255)

        out.write(chunklen.to_bytes(
                1, byteorder='big'))
        out.write(rng.read(chunklen))

        datalen -= chunklen

