#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    datalen = struct.pack('>H', int.from_bytes(
            rng.read(2), byteorder='big'))

    # Write the image data length to the output stream.
    out.write(datalen)

    # Generate an unsigned byte with the color table size.
    colortablesize = struct.pack('B', int.from_bytes(
            rng.read(1), byteorder='big'))

    # Write the color table size to the output stream.
    out.write(colortablesize)

    # Generate an unsigned byte with the background color index.
    bgcolorindex = struct.pack('B', int.from_bytes(
            rng.read(1), byteorder='big'))

    # Write the background color index to the output stream.
    out.write(bgcolorindex)

    # Generate an unsigned byte with the pixel aspect ratio.
    pixaspectratio = struct.pack('B', int.from_bytes(
            rng.read(1), byteorder='big'))

    # Write the pixel aspect ratio to the output stream.
    out.write(pixaspectratio)

    # Generate a 16-bit integer with the width of the image in pixels.
    imgwidth = struct.pack('>H', int.from_bytes(
            rng.read(2), byteorder='big'))

    # Write the image width to the output stream.
    out.write(imgwidth)

    # Generate a 16-bit integer with the height of the image in pixels.
    imgheight = struct.pack('>H', int.from_bytes(
            rng.read(2), byteorder='big'))

    # Write the image height to the output stream.
    out.write(imgheight)

    # Generate an unsigned byte with the flags for the local color table.
    lctflags = struct.pack('B', int.from_bytes(
            rng.read(1), byteorder='big'))

    # Write the local color table flags to the output stream.
    out.write(lctflags)

    # Generate an unsigned byte with the global color table size.
    gctsize = struct.pack('B', int.from_bytes(
            rng.read(1), byteorder='big'))

    # Write the global color table size to the output stream.
    out.write(gctsize)

    # Read bytes from the RNG until we've filled the image data section.
    while True:
        buf = rng.read()
        if len(buf) == 0: break;

        out.write(buf)

