#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')
    width  = rng.read(2)
    height = rng.read(2)
    out.write(width + height)

    # Global color table
    numcolors = (int.from_bytes(width, 'big') *
                 int.from_bytes(height, 'big')) // 64
    colors = [struct.pack('BBB', *[rng.read(1)[0] for _ in range(3)])
              for _ in range(numcolors)]
    out.write(b'\x87' + struct.pack('B', numcolors))
    out.write(b''.join(colors))

    # Image descriptor
    out.write(b'\x2C\x00\x00\x00' + width + height)
    out.write(b'\x00')

    # Local color table
    out.write(b'\xFF\x00')

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = bytes([rng.read(1)[0] for _ in range(datalen)])
    out.write(data)