#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(100,256)
    height = rng.randint(100,256)
    flags  = (
            0x80 | # Global color table flag
            7      # Color resolution - 7 bits per pixel
        ) << 4
    bg     = rng.randint(0,256)
    aspect = 0
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    for _ in range(0, 2**flags & 0x07):
        r = rng.randint(0,256)
        g = rng.randint(0,256)
        b = rng.randint(0,256)
        a = 0xff
        out.write(struct.pack('<BBBB', r,g,b,a))

    # Image descriptor
    left   = 0
    top    = 0
    flags  = 0x80 | 0x40 # Local color table flag and interlace flag
    delay  = 0
    trans  = 0
    width  = rng.randint(width//2, width*2)
    height = rng.randint(height//2, height*2)
    out.write(struct.pack('>HHBBHHH', left, top, width, height, flags, delay, trans))

    # Local color table
    for _ in range(0, 2**(flags&0x07)):
        r = rng.randint(0,256)
        g = rng.randint(0,256)
        b = rng.randint(0,256)
        a = 0xff
        out.write(struct.pack('<BBBB', r,g,b,a))

    # Image data
    bytes_per_pixel = 1 if flags & 0x07 == 1 else 2
    rowlen          = ((width * bytes_per_pixel) + 1) // 2
    for y in range(top, top+height):
        out.write(b'\x00')
        for x in range(left, left+width):
            if bytes_per_pixel == 1:
                out.write(rng.read(1))
            elif bytes_per_pixel == 2:
                out.write(rng.read(2))

        if rowlen % 2!= 0:
            out.write(b'\x00')