#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1024
    height = 768
    color_table_size = 256
    global_color_table = True
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write(struct.pack('>HHBBB', width, height,
                         color_table_size, global_color_table << 7 |
                         background_color_index, pixel_aspect_ratio))

    # Global color table
    for i in range(0, (1 << color_table_size)):
        r = rng.read(1)[0] & ~0x80
        g = rng.read(1)[0] & ~0x80
        b = rng.read(1)[0] & ~0x80

        out.write(struct.pack('<BBB', r, g, b))

    # Image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table = False
    interlace = False
    sort = False
    reserved = 0
    lct_size = 256
    out.write(struct.pack('>HHHHBB', left, top, width, height,
                         local_color_table << 7 | interlace << 6 |
                         sort << 5 | reserved, lct_size))

    # Local color table
    if not local_color_table:
        return

    for i in range(0, (1 << lct_size)):
        r = rng.read(1)[0] & ~0x80
        g = rng.read(1)[0] & ~0x80
        b = rng.read(1)[0] & ~0x80

        out.write(struct.pack('<BBB', r, g, b))

    # Table based image data
    lzw_min_code_size = 8
    out.write(lzw_min_code_size.to_bytes(1, 'little'))

    # LZW code stream
    while True:
        code = rng.read(1)[0]
        if code == 0xFF:
            break

        out.write(code.to_bytes(1, 'little'))