#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 256
    height = (1 << rng.read(1)[0]) % 256
    flags  = rng.read(1)[0] & 7
    bg     = rng.read(1)[0]
    aspect = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    global_ct_size = ((flags >> 4) & 7) + 1
    for _ in range(global_ct_size):
        out.write(rng.read(3))

    # Image descriptor
    left   = rng.read(2)
    top    = rng.read(2)
    width  = (1 << rng.read(1)[0]) % 256
    height = (1 << rng.read(1)[0]) % 256
    flags  = rng.read(1)[0] & 7
    local_ct_size = ((flags >> 4) & 7) + 1
    lct = []
    for _ in range(local_ct_size):
        lct += [rng.read(3)]
    out.write(left + top + width.to_bytes(1, 'big') +
             height.to_bytes(1, 'big'))
    out.write((flags | 0x80).to_bytes(1, 'big'))
    if len(lct) > 0:
        for c in lct:
            out.write(c)

    # Image data
    image_data_len = width * height // 8
    out.write(image_data_len.to_bytes(1, 'big'))
    for i in range(image_data_len):
        out.write(rng.read(1))