#!/usr/bin/env python3

import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical screen descriptor
    width = 2**rng.randint(4,10)
    height = 2**rng.randint(4,10)
    packed = (width & 0x7fff).to_bytes(2,'little') \
           + ((height >> 8) & 0xff).to_bytes(1,'little') \
           + (height & 0xff).to_bytes(1,'little') \
           + bytes([0]) \
           + bytes([0]) \
           + bytes([0])
    out.write(packed)

    # Global color table
    ncolors = min((width+height)//64, 256)
    for i in range(ncolors):
        r = rng.randint(0,255)
        g = rng.randint(0,255)
        b = rng.randint(0,255)
        a = 0
        packed = bytes([r]) \
               + bytes([g]) \
               + bytes([b]) \
               + bytes([a])
        out.write(packed)

    # Image descriptor
    packed = [0] * 9
    packed[0] = 0x2c
    packed[1] = 0
    packed[2] = 0
    packed[3] = 0
    packed[4] = 0
    packed[5] = 0
    packed[6] = width % 256
    packed[7] = width // 256
    packed[8] = height % 256
    out.write(bytes(packed))

    # Local color table
    if ncolors > 0:
        for i in range(ncolors):
            r = rng.randint(0,255)
            g = rng.randint(0,255)
            b = rng.randint(0,255)
            a = 0
            packed = bytes([r]) \
                   + bytes([g]) \
                   + bytes([b]) \
                   + bytes([a])
            out.write(packed)

    # Image data
    datalen = width*height//2
    data = rng.read(datalen)
    out.write(data)