#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (int.from_bytes(rng.read(2), 'big') % 65536) // 2
    height = (int.from_bytes(rng.read(2), 'big') % 65536) // 2
    flags  = int.from_bytes(rng.read(1), 'big') & 7
    bgcol  = int.from_bytes(rng.read(1), 'big')
    aspect = int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, aspect))

    # Global color table
    global_colors = [int.from_bytes(rng.read(1), 'big') for i in range(256)]
    out.write(bytes(global_colors))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), 'big')
    top    = int.from_bytes(rng.read(2), 'big')
    width  = ((int.from_bytes(rng.read(2), 'big')) % 65536) // 2
    height = ((int.from_bytes(rng.read(2), 'big')) % 65536) // 2
    flags  = int.from_bytes(rng.read(1), 'big') & 7
    lctsize= int.from_bytes(rng.read(1), 'big')
    if lctsize!= 0:
        raise Exception("LCT size not zero")
    out.write(struct.pack('>HHHHB', left, top, width, height, flags))

    # Local color table
    local_colors = [int.from_bytes(rng.read(1), 'big') for i in range(256)]
    out.write(bytes(local_colors))

    # Data
    datalen = int.from_bytes(rng.read(3), 'big')
    data = bytes([rng.read(1)[0] for _ in range(datalen)])
    out.write(data)