#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic bytes and version
    out.write(b'GIF89a')

    # Generate a random width between 10 and 256 pixels
    width = ord(rng.read(1)) % 247 + 10

    # Generate a random height between 10 and 256 pixels
    height = ord(rng.read(1)) % 247 + 10

    # Write the screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xf0 | 0x07, 0, 0))

    # Write the global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0xff
        g = ord(rng.read(1)) & 0xff
        b = ord(rng.read(1)) & 0xff
        out.write(struct.pack('BBB', r, g, b))

    # Write the image descriptor
    out.write(struct.pack('<HHBHB', 0, 0, 0xf0 | 0x07, 0, 0))

    # Write the local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0xff
        g = ord(rng.read(1)) & 0xff
        b = ord(rng.read(1)) & 0xff
        out.write(struct.pack('BBB', r, g, b))

    # Write the LZW code size
    out.write(b'\x08')

    # Write the image data
    for y in range(height):

        # Write the first byte of each line with its length
        out.write((y % 256).to_bytes(1, 'little'))

        # Write the rest of the line
        for x in range(width):

            # Get a random pixel value from the palette
            p = ord(rng.read(1)) & 0xff

            # Write it to the output stream
            out.write(p.to_bytes(1, 'little'))