#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = rng.read(1)[0]
    bgcol  = rng.read(1)[0]
    pxsize = rng.read(1)[0]
    colorres = ((flags >> 7) & 0x01) | (((flags >> 5) & 0x01) << 1)
    gctflag = (flags >> 4) & 0x01
    sortflag = (flags >> 3) & 0x01
    interlace = (flags >> 2) & 0x01
    colortable = [bgcol]
    for i in range(0, 2**colorres):
        colortable += [rng.read(1)[0]]
    
    # Global color table
    if gctflag == 1:
        for c in colortable:
            out.write(struct.pack('BBB', c, c, c))

    # Image descriptor
    left   = 0
    top    = 0
    width -= 1
    height -= 1
    lctflag = 0
    lctsize = 0
    lct = []
    if sortflag == 1:
        lctsize = 2**(lctsize + 1)
        for i in range(0, lctsize):
            lct += [rng.read(1)[0]]
    else:
        lctsize = 2**lctsize
        for i in range(0, lctsize):
            lct += [rng.read(1)[0]]
    interlacedata = bytes()
    for y in range(0, height+1):
        line = bytes()
        for x in range(0, width+1):
            line += bytes([lct[rng.read(1)[0]]])
        interlacedata += line
    out.write(struct.pack('<HHBBBBB', left, top, width, height,
                          8, pxsize, lctflag))
    if lctflag == 1:
        for c in lct:
            out.write(struct.pack('BBB', c, c, c))
    out.write(interlacedata)