#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.read(2)
    height = rng.read(2)
    flags  = rng.read(1)[0]
    bgcol  = rng.read(1)[0]
    pxsize = rng.read(1)[0]
    gctlen = rng.read(1)[0]

    out.write(width + height + bytes([flags]) +
              bytes([bgcol]) + bytes([pxsize]) +
              bytes([gctlen]))

    # Global color table (if any)
    if gctlen > 0:
        for _ in range(gctlen):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]

            out.write(bytes([r]) + bytes([g]) +
                      bytes([b]))

        # Trailer
        out.write(b'\x3B')

    # Image descriptor
    left   = rng.read(2)
    top    = rng.read(2)
    width  = rng.read(2)
    height = rng.read(2)
    flags  = rng.read(1)[0]

    out.write(left + top + width + height +
              bytes([flags]))

    # Local color table (if any)
    lctlen = rng.read(1)[0]

    if lctlen > 0:
        for _ in range(lctlen):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]

            out.write(bytes([r]) + bytes([g]) +
                      bytes([b]))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    out.write(data)

    # Trailer
    out.write(b'\x3B')