#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 65536
    height = (1 << rng.read(1)[0]) % 65536
    flags  = bytes([rng.read(1)[0] | 0x80])
    background_color = bytes([rng.read(1)[0]])
    pixel_aspect_ratio = bytes([rng.read(1)[0]])
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))
    out.write(flags)
    out.write(background_color)
    out.write(pixel_aspect_ratio)

    # Global color table
    global_color_table_size = rng.read(1)[0] & 7
    if global_color_table_size!= 0:
        global_color_table = [
                rng.read(1)[0] for _ in range(
                    3 * pow(2, global_color_table_size))
            ]
        out.write(global_color_table)

    # Image descriptor
    left   = (1 << rng.read(1)[0]) % width
    top    = (1 << rng.read(1)[0]) % height
    width  = (1 << rng.read(1)[0]) % (width - left)
    height = (1 << rng.read(1)[0]) % (height - top)
    flags  = bytes([rng.read(1)[0] | 0x80])
    local_color_table_size = rng.read(1)[0] & 7
    interlace = bool((rng.read(1)[0] >> 7) & 1)
    sort      = bool((rng.read(1)[0] >> 6) & 1)
    reserved  = bytes([rng.read(1)[0], rng.read(1)[0]])
    lct       = None
    if local_color_table_size!= 0:
        lct = [
                rng.read(1)[0] for _ in range(
                    3 * pow(2, local_color_table_size))
            ]
    out.write(left.to_bytes(2, 'big'))
    out.write(top.to_bytes(2, 'big'))
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))
    out.write(flags)
    out.write(local_color_table_size.to_bytes(1, 'big'))
    out.write(interlace.to_bytes(1, 'big'))
    out.write(sort.to_bytes(1, 'big'))
    out.write(reserved)
    if lct is not None:
        out.write(lct)

    # Image data
    image_data = []
    while True:
        blocktype = rng.read(1)[0]

        if blocktype == 0x2C:
            # Image descriptor
            left   = (1 << rng.read(1)[0]) % width
            top    = (1 << rng.read(1)[0]) % height
            width  = (1 << rng.read(1)[0]) % (width - left)
            height = (1 << rng.read(1)[0]) % (height - top)
            flags  = bytes([rng.read(1)[0] | 0x80])
            local_color_table_size = rng.read(1)[0] & 7
            interlace = bool((rng.read(1)[0] >> 7) & 1)
            sort      = bool((rng.read(1)[0] >> 6) & 1)
            reserved  = bytes([rng.read(1)[0], rng.read(1)[0]])

            # Local color table
            lct = None
            if local_color_table_size!= 0:
                lct = [
                        rng.read(1)[0] for _ in range(
                            3 * pow(2, local_color_table_size))
                    ]

            # LZW min code size
            lzwmincodesize = rng.read(1)[0]

           