#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    datalen = struct.pack('>H', int.from_bytes(
            rng.read(2), byteorder='big'))

    # Write the image data length to the output stream.
    out.write(datalen)

    # Generate an array of bytes containing the color table for this frame.
    colortable = [
        0x00, 0x00, 0x00,     # Black
        0xff, 0x00, 0x00,     # Red
        0x00, 0xff, 0x00,     # Green
        0xff, 0xff, 0x00      # Yellow
    ] * 256

    # Write the color table to the output stream.
    out.write(bytearray(colortable))

    # Copy the image data from the input stream to the output stream.
    while True:
        buf = rng.read(4096)

        if len(buf) == 0: break;

        out.write(buf)