#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << 6) | ((os.getpid() % 256) & 0x3f)
    height = (1 << 6) | (((os.getpid() >> 8) % 256) & 0x3f)
    flags  = 0
    bg     = 0
    aspect = 0
    out.write(struct.pack('<HHBBB', width, height, flags, bg, aspect))

    # Global color table
    palette = [
        0x00, 0x00, 0x00,
        0xff, 0x00, 0x00,
        0x00, 0xff, 0x00,
        0xff, 0xff, 0x00,
        0x00, 0x00, 0xff,
        0xff, 0x00, 0xff,
        0x00, 0xff, 0xff,
        0xff, 0xff, 0xff]
    for i in range(len(palette)):
        out.write(struct.pack('B', palette[i]))

    # Image descriptor
    left   = 0
    top    = 0
    width  = 1 << 6
    height = 1 << 6
    flags  = 0b11000000
    out.write(struct.pack('<HHHHB', left, top, width, height, flags))

    # Local color table
    for i in range(len(palette)):
        out.write(struct.pack('B', palette[i]))

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)

