#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) % 65536
    height = (1 << rng.read(1)[0]) % 65536
    flags  = bytes([rng.read(1)[0] | 0x80])
    bg     = bytes([rng.read(1)[0]])
    pxsize = bytes([rng.read(1)[0]])
    color  = bytes([rng.read(1)[0]])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Global color table
    for _ in range(int.from_bytes(color, 'big')):
        r = bytes([rng.read(1)[0]])
        g = bytes([rng.read(1)[0]])
        b = bytes([rng.read(1)[0]])
        out.write(r+g+b)

    # Image descriptor
    left   = bytes([rng.read(1)[0]])
    top    = bytes([rng.read(1)[0]])
    width  = bytes([rng.read(1)[0]])
    height = bytes([rng.read(1)[0]])
    flags  = bytes([rng.read(1)[0]])
    out.write(left)
    out.write(top)
    out.write(width)
    out.write(height)
    out.write(flags)

    # Local color table
    if flags[0] & 0x80 == 0x80:
        for _ in range(int.from_bytes(color, 'big')):
            r = bytes([rng.read(1)[0]])
            g = bytes([rng.read(1)[0]])
            b = bytes([rng.read(1)[0]])
            out.write(r+g+b)

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)

