#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1024
    height = 768
    color_depth = 8
    global_color_table = False
    sort_flag = True
    size = (width * height) // (2 ** color_depth) - 1
    background_index = 0
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBBBHB',
            width, height, color_depth,
            global_color_table, sort_flag,
            size, background_index, pixel_aspect_ratio
        ))

    # Global color table
    for i in range(size):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table = False
    interlace_flag = False
    sort_flag = True
    reserved = 0
    lct_size = 0
    out.write(struct.pack('<HHHHBB',
            left, top, width, height,
            local_color_table, interlace_flag,
            sort_flag, reserved
        ))

    # Local color table
    if not local_color_table:
        for i in range(lct_size):
            r = rng.read(1)[0] % 256
            g = rng.read(1)[0] % 256
            b = rng.read(1)[0] % 256
            out.write(struct.pack('BBB', r, g, b))

    # LZW minimum code size
    min_code_size = 2
    out.write(min_code_size.to_bytes(1, 'little'))

    # Data sub-block
    data = rng.read((width * height) // (2 ** color_depth))
    while len(data) > 255:
        out.write(len(data).to_bytes(1, 'little'))
        out.write(data[:255])
        data = data[255:]
    out.write(len(data).to_bytes(1, 'little'))
    out.write(data)