#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(1024, 65535)
    height = rng.randint(768,  65535)
    flags  = (width & 0x8000) >> 15 | \
             ((height & 0x8000) >> 14) << 1 | \
             0b10000000
    bgcol  = rng.randint(0, 255)
    aspect = 0b10000000
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, aspect))

    # Global color table
    ncolors = min((rng.randint(2, 256)), 256)
    for i in range(ncolors):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        a = 0xff
        out.write(struct.pack('<BBBB', r, g, b, a))

    # Image descriptor
    left      = 0
    top       = 0
    width     = width
    height    = height
    flags     = 0b10000000
    lctsize   = -1
    interlace = False
    sortflag  = True
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags, lctsize))

    # Local color table
    if lctsize!= -1:
        for i in range(lctsize):
            r = rng.randint(0, 255)
            g = rng.randint(0, 255)
            b = rng.randint(0, 255)
            a = 0xff
            out.write(struct.pack('<BBBB', r, g, b, a))

    # Table based image data
    lzwmincodesize = 2
    out.write(struct.pack('B', lzwmincodesize))

    # LZW code size
    lzwmaxcodesize = max(rng.randint(lzwmincodesize+1, 12), lzwmincodesize+1)
    out.write(struct.pack('B', lzwmaxcodesize))

    # Pixels
    while not interlace and len(data) < width * height:

        # Block size
        blocklen = rng.randint(1, 256)
        out.write(struct.pack('B', blocklen))

        # Data
        data = bytes([rng.randint(0, 255)])
        while len(data) < blocklen:

            # RLE?
            if rng.randint(0, 1):
                count = rng.randint(2, 128)

                # Repeat last pixel
                if rng.randint(0, 1):
                    data += bytes([data[-1]]*count)

                # Random pixels
                else:
                    for _ in range(count):
                        data += bytes([rng.randint(0, 255)])

            # Raw
            else:
                data += bytes([rng.randint(0, 255)])

        # Write data
        out.write(data)

