#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are given by `width`, `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table (black and white only)
    out.write(b'\x02\x00\xFF\x00\x00\x00\x00\xFF')

    # Image descriptor
    out.write(struct.pack('>HHBBHHH', 0, 0, 0xF7, 0x00, 0, 0, width * height))

    # Local color table
    out.write(b'\x01\x00\xFF\x00')

    # Image data
    for i in range(width * height):
        if i % 64 == 0:
            out.flush()

        out.write(b'\x00' if bool(int.from_bytes(
                rng.read(1), byteorder='big')) else b'\xFF')