#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.read(2)
    height = rng.read(2)
    flags  = rng.read(1)
    bg     = rng.read(1)
    aspect = rng.read(1)
    out.write(width + height + flags + bg + aspect)

    # Global color table (palette)
    palette = [rng.read(3) for _ in range(int.from_bytes(flags, 'big'))]
    for p in palette:
        out.write(p)

    # Image descriptor
    left   = rng.read(2)
    top    = rng.read(2)
    width  = rng.read(2)
    height = rng.read(2)
    flags  = rng.read(1)
    lct    = rng.read(1)
    interl = rng.read(1)
    out.write(left + top + width + height + flags + lct + interl)

    # Local color table (palette)
    if lct[0]:
        for p in palette:
            out.write(p)

    # Data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = bytes([rng.read(1)[0] for _ in range(datalen)])
    out.write(data)