#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width  = rng.randint(1024, 65536) // 16 * 16
    height = rng.randint(1024, 65536) // 16 * 16
    flags  = (rng.randint(0, 7) << 5) | rng.randint(0, 7)
    bgcol  = rng.randint(0x00, 0xFF)
    aspect = 0
    out.write(struct.pack('<HHBBB', width, height, flags, bgcol, aspect))

    # Global Color Table
    for _ in range(0, 2**flags):
        r = rng.randint(0x00, 0xFF)
        g = rng.randint(0x00, 0xFF)
        b = rng.randint(0x00, 0xFF)
        a = rng.randint(0x00, 0xFF)
        out.write(struct.pack('BBBB', r, g, b, a))

    # Image Descriptor
    left   = 0
    top    = 0
    width  = rng.randint(width//2, width*2)
    height = rng.randint(height//2, height*2)
    flags  = 0x80 | rng.randint(0, 7)
    out.write(struct.pack('<HHHHB', left, top, width, height, flags))

    # Local Color Table
    if flags & 0x80:
        for _ in range(0, 2**flags):
            r = rng.randint(0x00, 0xFF)
            g = rng.randint(0x00, 0xFF)
            b = rng.randint(0x00, 0xFF)
            a = rng.randint(0x00, 0xFF)
            out.write(struct.pack('BBBB', r, g, b, a))

    # Image Data
    for y in range(top, top+height):
        for x in range(left, left+width):
            i = ((y - top) * width + (x - left)) % len(data)
            out.write(data[i])

