#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), 'big') % (1 << 12)
    height = int.from_bytes(rng.read(2), 'big') % (1 << 12)
    flags  = int.from_bytes(rng.read(1), 'big') & 0x7
    bg     = int.from_bytes(rng.read(1), 'big')
    aspect = int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    for _ in range((flags >> 1) + 1):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), 'big') % (1 << 12)
    top    = int.from_bytes(rng.read(2), 'big') % (1 << 12)
    width  = int.from_bytes(rng.read(2), 'big') % (1 << 12)
    height = int.from_bytes(rng.read(2), 'big') % (1 << 12)
    flags  = int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>HHHHB', left, top, width, height, flags))

    # Local color table
    if flags & 0x80:
        for _ in range((flags >> 4) + 1):
            r = int.from_bytes(rng.read(1), 'big')
            g = int.from_bytes(rng.read(1), 'big')
            b = int.from_bytes(rng.read(1), 'big')
            out.write(struct.pack('BBB', r, g, b))

    # LZW minimum code size
    lzwmincodesize = int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('B', lzwmincodesize))

    # Data sub-block
    while True:
        blocklen = int.from_bytes(rng.read(1), 'big')

        if not blocklen: break

        out.write(rng.read(blocklen))