#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (stdin by default).
def generate_random_gif(
        rng: BinaryIO=sys.stdin.buffer, out: BinaryIO=sys.stdout.buffer
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little', signed=False)
    height = int.from_bytes(rng.read(2), byteorder='little', signed=False)
    flags  = int.from_bytes(rng.read(1), byteorder='little', signed=False)
    bgcolr = rng.read(1)[0]
    aspect = int.from_bytes(rng.read(1), byteorder='little', signed=False)
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write(flags.to_bytes(1, 'little'))
    out.write(bgcolr.to_bytes(1, 'little'))
    out.write(aspect.to_bytes(1, 'little'))

    # Global color table
    globalctsize = int.from_bytes(rng.read(1), byteorder='little', signed=False)
    if globalctsize > 0:
        for i in range(globalctsize):
            colr = rng.read(1)[0]
            out.write(colr.to_bytes(1, 'little'))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), byteorder='little', signed=False)
    top    = int.from_bytes(rng.read(2), byteorder='little', signed=False)
    width  = int.from_bytes(rng.read(2), byteorder='little', signed=False)
    height = int.from_bytes(rng.read(2), byteorder='little', signed=False)
    flags  = int.from_bytes(rng.read(1), byteorder='little', signed=False)
    lctsize = int.from_bytes(rng.read(1), byteorder='little', signed=False)
    interlaced = bool(flags & 0x40)
    sortflag = bool(flags & 0x20)
    reserved = rng.read(1)[0]
    lctfirst = rng.read(1)[0]
    lctlen   = rng.read(1)[0]
    out.write(left.to_bytes(2, 'little'))
    out.write(top.to_bytes(2, 'little'))
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(flags.to_bytes(1, 'little'))
    out.write(lctsize.to_bytes(1, 'little'))

    # Local color table
    localctsize = int.from_bytes(rng.read(1), byteorder='little', signed=False)
    if localctsize > 0:
        for i in range(localctsize):
            colr = rng.read(1)[0]
            out.write(colr.to_bytes(1, 'little'))

    # Image data
    codesize = int.from_bytes(rng.read(1), byteorder='little', signed=False)
    while True:
        blocktype = rng.read(1)[0]

        if blocktype == 0x2C:
            # Image descriptor
            left   = int.from_bytes(rng.read(2), byteorder='little', signed=False)
            top    = int.from_bytes(rng.read(2), byteorder='little', signed=False)
            width  = int.from_bytes(rng.read(2), byteorder='little', signed=False)
            height = int.from_bytes(rng.read(2), byteorder='little', signed=False)
            flags  = int.from_bytes(rng.read(1), byteorder='little', signed=False)
            lctsize = int.from_bytes(rng.read(1), byteorder='little', signed=False)

            out.write(blocktype.to_bytes(1, 'little'))
            out.write(left.to_bytes(2, 'little'))
            out.write(top.to_bytes(2, '