#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width  = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        ) % (1 << 16 - 1)
    height = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        ) % (1 << 16 - 1)
    flags  = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) & 0x7
    bgcol  = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) % (1 <<  8 - 1)
    pxsize = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) & 0x7
    gctlen = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) & 0x7
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, pxsize))

    # Global Color Table
    for _ in range(gctlen):
        r = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            ) % (1 <<  8 - 1)
        g = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            ) % (1 <<  8 - 1)
        b = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            ) % (1 <<  8 - 1)
        out.write(struct.pack('BBB', r, g, b))

    # Image Descriptor
    left    = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        ) % (1 << 16 - 1)
    top     = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        ) % (1 << 16 - 1)
    width   = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        ) % (1 << 16 - 1)
    height  = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        ) % (1 << 16 - 1)
    flags   = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) & 0xF
    lctlen  = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) & 0x7
    interlv = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) & 0x7
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags, lctlen))

    # Local Color Table
    for _ in range(lctlen):
        r = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            ) % (1 <<  8 - 1)
        g = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            ) % (1 <<  8 - 1)
        b = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            ) % (1 <<  8 - 1)
        out.write(struct.pack('BBB', r, g, b))

    # Data
    datalen = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        ) % (1 << 16 - 1)
    data = rng.read(datalen)
    out.write(data)