#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (or /dev/urandom if None).
def generate_random_gif(
        rng: BinaryIO=None, out: BinaryIO=sys.stdout.buffer
    ) -> None:
    # If no RNG is provided, use /dev/urandom instead
    if not rng:
        rng = open('/dev/urandom', 'rb')

    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = bytes([rng.read(1)[0] for _ in range(datalen)])

    out.write(header + struct.pack('>H', len(data)))
    out.write(data)