#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
# The size of the image is determined by `width`, `height`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write((width & 0xff).to_bytes(1, 'little'))
    out.write((width >> 8).to_bytes(1, 'little'))
    out.write((height & 0xff).to_bytes(1, 'little'))
    out.write((height >> 8).to_bytes(1, 'little'))
    out.write(b'\x87\x00')

    # Global color table
    for i in range(256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(r.to_bytes(1, 'little'))
        out.write(g.to_bytes(1, 'little'))
        out.write(b.to_bytes(1, 'little'))

    # Image descriptor
    out.write(b'\x2c')
    out.write((width & 0xff).to_bytes(1, 'little'))
    out.write((width >> 8).to_bytes(1, 'little'))
    out.write((height & 0xff).to_bytes(1, 'little'))
    out.write((height >> 8).to_bytes(1, 'little'))
    out.write(b'\x00')

    # Local color table
    out.write(b'\x00')

    # LZW minimum code size
    out.write(b'\xf7')

    # Data
    while True:
        chunksize = rng.read(1)[0]

        if not chunksize: break

        out.write(chunksize.to_bytes(1, 'little'))
        out.write(rng.read(chunksize))
