#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (defaults to /dev/urandom).
def generate_random_gif(
        rng: BinaryIO=None, out: BinaryIO=sys.stdout.buffer
    ) -> None:

    if not rng:
        from os import urandom as rng

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(1024, 65535)
    height = rng.randint(768, 65535)
    flags  = bytes([0x80 | 0x70])
    bg     = bytes([rng.randint(0, 255)]) * 3
    aspect = bytes([0x00])
    out.write((width).to_bytes(2, 'big'))
    out.write((height).to_bytes(2, 'big'))
    out.write(flags)
    out.write(bg)
    out.write(aspect)

    # Global color table
    ncolors = min(2**(int.from_bytes(flags[0], 'big')), 256) - 1
    for i in range(ncolors):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(bytes([r,g,b]))

    # Image descriptor
    left   = 0
    top    = 0
    width  = rng.randint(1024, 65535)
    height = rng.randint(768, 65535)
    flags  = bytes([0x00])
    out.write(left.to_bytes(2, 'big'))
    out.write(top.to_bytes(2, 'big'))
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))
    out.write(flags)

    # Local color table
    ncolors = min(2**(int.from_bytes(flags[0], 'big')), 256) - 1
    for i in range(ncolors):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(bytes([r,g,b]))

    # Image data
    datalen = width*height
    data = rng.read(datalen)
    out.write(data)