#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = ((1 << rng.read(1)[0]) & 0x70) | \
             ((1 << rng.read(1)[0]) & 0x08) | \
             0x07
    bgcolor = rng.read(1)[0]
    aspectratio = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags, bgcolor, aspectratio))

    # Global color table
    numcolors = 2 ** (((flags >> 4) & 0x07) + 1)
    for i in range(numcolors):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left   = 0
    top    = 0
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = ((1 << rng.read(1)[0]) & 0xC0) | \
             0x0C
    bgcolorindex = rng.read(1)[0]
    pixelaspectratio = rng.read(1)[0]
    out.write(struct.pack('>HHBBBBBB', left, top, width, height, flags, bgcolorindex, pixelaspectratio))

    # Local color table
    numcolors = 2 ** (((flags >> 6) & 0x03) + 1)
    for i in range(numcolors):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    lzwmincodesize = rng.read(1)[0]
    out.write(lzwmincodesize.to_bytes(1, 'little'))

    # LZW code stream
    while True:
        chunktype = rng.read(1)[0]

        if chunktype == 0x00:
            break;
        elif chunktype == 0x01:
            pass
        elif chunktype == 0xFE:
            pass
        else:
            raise Exception("Invalid chunk type")