#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the generated image is determined by
# `width`, `height`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0))

    # Global color table (black and white only)
    out.write(b'\x02\xFF\x00\xFF')

    # Image descriptor
    out.write(struct.pack('>HHHHB', 0, 0, width, height, 0))

    # Local color table (black and white only)
    out.write(b'\x02\xFF\x00\xFF')

    # Image data
    for _ in range(width * height):
        out.write(rng.read(1))