#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = (
            0x80 |    # Global color table flag
            0x7      # Color resolution - 7 bits per pixel
        ) << 5 | (
            0x0 |     # Sorting method - none
            0x0      # Size of global color table - 2^n entries
        )
    out.write(struct.pack('>HHBBB', width, height, packed, 0, 0))

    # Global color table
    for _ in range(2**packed & 0xFF):
        r = next(rng) % 256
        g = next(rng) % 256
        b = next(rng) % 256
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    out.write(struct.pack('>BHHBBBBB',
        0x2C,   # Image separator
        0,       # Left position
        0,       # Top position
        width,   # Width
        height,  # Height
        0x00,    # Local color table flag
        0x00,    # Interlace flag
        0x00,    # Sorting flag
        0x00    # Reserved
    ))

    # Image data
    datalen = width * height // 2
    data = bytes([next(rng) % 256 for _ in range(datalen)])
    out.write(data)