#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 2**int.from_bytes(rng.read(1), 'big')
    height = 2**int.from_bytes(rng.read(1), 'big')
    flags = (
            1 << 7 |    # Global color table flag
            1 << 4      # Color resolution bit
        )
    bgcolorindex = 0
    pixelaspectratio = 0
    out.write(struct.pack('<HHBBB', width, height, flags, bgcolorindex, pixelaspectratio))

    # Global color table
    globalcolortablesize = 2**(int.from_bytes(rng.read(1), 'big')+1)
    globalcolortable = [rng.read(3) for _ in range(globalcolortablesize)]
    for col in globalcolortable:
        out.write(col)

    # Image descriptor
    left = 0
    top = 0
    width = 2**int.from_bytes(rng.read(1), 'big')
    height = 2**int.from_bytes(rng.read(1), 'big')
    flags = (
            1 << 5 |     # Local color table flag
            1 << 6       # Interlace flag
        )
    localcolortablesize = 2**(int.from_bytes(rng.read(1), 'big')+1)
    localcolortable = [rng.read(3) for _ in range(localcolortablesize)]
    for col in localcolortable:
        out.write(col)
    imagedata = bytes([rng.read(1)[0] for _ in range((width*height)//2)])
    lzwmincodesize = 2**(int.from_bytes(rng.read(1), 'big'))
    out.write(struct.pack('<HHBB', left, top, width, height))
    out.write(flags.to_bytes(1, 'big'))
    if flags & (1 << 5):
        out.write(struct.pack('B', localcolortablesize-1))
    out.write(imagedata)
    out.write(lzwmincodesize.to_bytes(1, 'big'))