#!/usr/bin/env python3

import argparse
import os
import struct
from typing import BinaryIO


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('rng', type=argparse.FileType('rb'))
    parser.add_argument('-o', '--output', default='out.gif')

    args = parser.parse_args()

    with open(args.output, 'wb') as f:
        generate_random_gif(
                args.rng, f)


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical Screen Descriptor
    width = 1024
    height = 768
    color_table_size = 0
    global_color_table = False
    background_color = 0
    pixel_aspect_ratio = 0
    logical_screen_descriptor = struct.pack('<HHBBB',
            width, height, color_table_size,
            global_color_table, background_color,
            pixel_aspect_ratio)
    out.write(logical_screen_descriptor)

    # Global Color Table
    for i in range(256):
        r = rng.read(1)[0] & 0x3f | (i << 2)
        g = rng.read(1)[0] & 0x3f | (i << 2)
        b = rng.read(1)[0] & 0x3f | (i << 2)
        a = 0xff
        palette_entry = bytes([r, g, b])
        out.write(palette_entry)

    # Image Descriptor
    image_left = 0
    image_top = 0
    image_width = width
    image_height = height
    local_color_table = True
    interlace = False
    sort = False
    reserved = 0
    lct_size = 0
    image_descriptor = struct.pack('<BBHHB',
            image_left, image_top, image_width,
            image_height, local_color_table)
    out.write(image_descriptor)

    # Local Color Table
    for i in range(256):
        r = rng.read(1)[0] & 0x3f | (i << 2)
        g = rng.read(1)[0] & 0x3f | (i << 2)
        b = rng.read(1)[0] & 0x3f | (i << 2)
        a = 0xff
        palette_entry = bytes([r, g, b])
        out.write(palette_entry)

    # Image Data
    for y in range(height):
        line = []

        for x in range(width // 8):
            byte = 0

            for bit in range(8):
                if rng.read(1)[0] > 127:
                    byte |= 1 << bit

            line += [byte]

        out.write(bytes(line))

    # Trailer
    trailer = b'\x3b'
    out.write(trailer)


if __name__ == '__main__':
    main()