#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width is set to 1024 pixels by default but can be changed with the
# optional argument `width`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width=1024
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Calculate the size of the image data in bytes (including color table)
    datalen = ((width * height) // 8) + 3

    # Write the size of the image data to the output stream
    out.write((datalen).to_bytes(3, 'big'))

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen - 3)
    out.write(data)