#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (defaults to /dev/urandom).
# The size of the generated image is determined by `width`, and
# the color depth by `colordepth`.
def generate_random_gif(
        width: int, height: int, colordepth: int=256,
        rng: BinaryIO=None, out: BinaryIO=sys.stdout.buffer
    ) -> None:

    if not 1 <= colordepth <= 256:
        raise ValueError('Invalid color depth.')

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write((width & 0xFF).to_bytes(1, 'little'))
    out.write((height & 0xFF).to_bytes(1, 'little'))
    out.write((0x80 | ((colordepth - 1) >> 4)).to_bytes(1, 'little'))
    out.write(((colordepth - 1) & 0xF).to_bytes(1, 'little'))
    out.write(b'\0\0')

    # Global color table
    for i in range(pow(2, colordepth)):
        out.write(i.to_bytes(1, 'little'))

    # Image descriptor
    out.write(b'\x2C')
    out.write((0 << 7) | (0 << 6) | (0 << 5) | (0 << 4) |
              (0 << 3) | (0 << 2) | (0 << 1) | (0 << 0))
    out.write((0).to_bytes(2, 'little'))
    out.write((0).to_bytes(2, 'little'))
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write((0).to_bytes(1, 'little'))
    out.write((0).to_bytes(1, 'little'))
    out.write(b'\0')

    # Local color table
    for i in range(pow(2, colordepth)):
        out.write(i.to_bytes(1, 'little'))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))