#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(0x1, 0xFFFF)
    height = rng.randint(0x1, 0xFFFF)
    flags  = (
            0x80 | # Global color table flag
            0x70   # Color resolution and sort order
        )
    bgcol  = rng.randint(0x0, 0xFF)
    aspect = 0x40 # Pixels per metre
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, aspect))

    # Global color table
    for _ in range(0x10):
        r = rng.randint(0x0, 0xFF)
        g = rng.randint(0x0, 0xFF)
        b = rng.randint(0x0, 0xFF)
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = 0x0
    top       = 0x0
    width     = width
    height    = height
    flags     = 0x80 # Local color table flag
    lctsize   = 0x0
    interlaced= False
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags, lctsize))

    # Local color table
    for _ in range(0x10):
        r = rng.randint(0x0, 0xFF)
        g = rng.randint(0x0, 0xFF)
        b = rng.randint(0x0, 0xFF)
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    datalen = rng.randint(0x1, 0xFFFFF)
    data = rng.read(datalen)
    out.write(data)