#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = rng.read(1)[0]
    bgcolorindex = rng.read(1)[0]
    aspectratio  = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags, bgcolorindex, aspectratio))

    # Global color table
    globalcolortablesize = rng.read(1)[0]
    if globalcolortablesize > 0:
        for i in range(globalcolortablesize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(struct.pack('BBB', r, g, b))

    # Loop control block
    loopcount = rng.read(1)[0]
    out.write(struct.pack('B', loopcount))

    # Image descriptor
    left   = rng.read(2)[0]
    top    = rng.read(2)[0]
    width  = (1 << rng.read(1)[0]) & 0xFF
    height = (1 << rng.read(1)[0]) & 0xFF
    flags  = rng.read(1)[0]
    localcolortablesize = rng.read(1)[0]
    interlacedflag     = rng.read(1)[0]
    sortflag           = rng.read(1)[0]
    reserved           = rng.read(1)[0]
    out.write(struct.pack('>HHBBBBBB', left, top, width, height, flags, localcolortablesize, interlacedflag, sortflag, reserved))

    # Local color table
    if localcolortablesize > 0:
        for i in range(localcolortablesize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write(struct.pack('BBB', r, g, b))

    # Image data
    imagedata = rng.read()
    out.write(imagedata)