#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 2**int.from_bytes(rng.read(1), 'big')
    height = 2**int.from_bytes(rng.read(1), 'big')
    flags = (
            0x80 | # Global color table flag
            0x70 | # Color resolution
            0x07   # Sorting method
        )
    background = 0
    pixelaspectratio = 0
    out.write(struct.pack('<HHBBB', width, height, flags, background, pixelaspectratio))

    # Global color table
    globalcolortablesize = 2**(int.from_bytes(rng.read(1), 'big')+1)
    palette = [
            tuple(rng.read(3))
                for i in range(globalcolortablesize)
        ]
    out.write(b''.join((
            bytes([i]) + bytes(rgb)
                for rgb, i in zip(palette, range(len(palette)))
        )))

    # Image descriptor
    left = 0
    top = 0
    flags = 0x80 | 0x40 # Local color table flag and interlace flag
    localcolortablesize = 2**(int.from_bytes(rng.read(1), 'big')+1)
    out.write(struct.pack('<HHBBBBB', left, top, width, height, flags, localcolortablesize))

    # Local color table
    palette = [
            tuple(rng.read(3))
                for i in range(localcolortablesize)
        ]
    out.write(b''.join((
            bytes([i]) + bytes(rgb)
                for rgb, i in zip(palette, range(len(palette)))
        )))

    # LZW minimum code size
    lzwmincodesize = 2**(int.from_bytes(rng.read(1), 'big'))
    out.write(lzwmincodesize.to_bytes(1, 'big'))

    # Data sub-block
    datalen = int.from_bytes(rng.read(3), 'big')
    data = bytes([rng.read(1)[0] for _ in range(datalen)])
    out.write(data)