#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(100, 256)
    height = rng.randint(100, 256)
    flags  = (
            0x80 |    # Global color table flag set
            0x70 |    # Sort flag set to size of global color table
            0x07      # Color resolution is 7 bits per pixel
        ).to_bytes(1, 'little', signed=False)
    bgcol  = bytes([rng.randint(0, 255)]) * 3
    aspect = (4 << 4).to_bytes(1, 'little', signed=False)
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(flags)
    out.write(bgcol)
    out.write(aspect)

    # Global color table
    for i in range(2**((flags[0] & 0x07) + 1)):
        col = bytes([rng.randint(0, 255)]) * 3
        out.write(col)

    # Image descriptor
    left   = 0
    top    = 0
    width  = rng.randint(100, 256)
    height = rng.randint(100, 256)
    flags  = (
            0x80 |     # Local color table flag set
            0x00       # No sort flag
        ).to_bytes(1, 'little', signed=False)
    lctsize = (2**((flags[0] & 0x07) + 1)).to_bytes(1, 'little', signed=False)
    out.write(left.to_bytes(2, 'little'))
    out.write(top.to_bytes(2, 'little'))
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(flags)
    out.write(lctsize)

    # Local color table
    for i in range(2**((flags[0] & 0x07) + 1)):
        col = bytes([rng.randint(0, 255)]) * 3
        out.write(col)

    # Table based image data
    lzwmincodesize = 2 ** ((flags[0] & 0x07) + 1)
    while True:
        codesize = rng.randint(lzwmincodesize, 12)

        if codesize == 12 and len(data) < 256:
            break

        if len(data) > 0:
            out.write(data[-1].to_bytes(1, 'little', signed=False))
        else:
            out.write(b'\x00')

        data += [rng.randint(0, 255)] * (codesize - 1)