#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the generated image are given by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ):

    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    out.write(struct.pack('>HBBHHHH', 0x100, 0, 0, width, height, 0, 0))

    # Global Color Table (black and white only)
    out.write(b'\x00\x00\x00\xff\xff\xff')

    # Image Descriptor
    out.write(struct.pack('>BHHHHB', 0x2c, 0, 0, width, height, 0))

    # Local Color Table (black and white only)
    out.write(b'\x00\x00\x00\xff\xff\xff')

    # Image Data
    for _ in range(height):
        out.write(b'\x00' * width)

    # Trailer
    out.write(b';')