#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    width = 2**int.from_bytes(rng.read(1), 'big')
    height = 2**int.from_bytes(rng.read(1), 'big')
    flags = (
        0x80 | # Global color table flag
        7      # Color resolution
    ) << 4
    bgcoloridx = 0
    aspectratio = 0
    out.write(struct.pack('<HHBBB', width, height, flags, bgcoloridx, aspectratio))

    # Global color table
    palette = [
        (rng.read(1)[0], rng.read(1)[0], rng.read(1)[0])
        for _ in range(256)]
    out.write(bytes([
        r<<4|g<<2|b
        for r, g, b in palette]))

    # Image descriptor
    left = 0
    top = 0
    flags = 0x80 # Local color table flag
    imgwidth = width
    imgheight = height
    localpalette = []
    out.write(struct.pack('<HHBBHHB', left, top, flags, imgwidth, imgheight, len(localpalette)))

    # Local color table
    if len(localpalette):
        out.write(bytes([
            r<<4|g<<2|b
            for r, g, b in localpalette]))

    # Image data
    datalen = int.from_bytes(rng.read(3), byteorder='big')
    data = bytes([rng.read(1)[0] for _ in range(datalen)])
    out.write(data)