#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width is determined by `width`, and the height
# is determined by `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Calculate the size of the image data (in bytes)
    datalen = 4 + ((width * height) // 256) + \
              (((width * height) % 256) > 0)

    # Write it as a three-byte big endian integer
    out.write(datalen.to_bytes(3, 'big'))

    # Write the screen descriptor
    out.write((width & 0xff).to_bytes(1, 'little'))
    out.write(((width >> 8) & 0xff).to_bytes(1, 'little'))
    out.write((height & 0xff).to_bytes(1, 'little'))
    out.write(((height >> 8) & 0xff).to_bytes(1, 'little'))
    out.write(b'\x87\x00')

    # Write the global color table
    out.write(b'\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    out.write(b'\x00\x00\x00\x00\x00\x00')

    # Write the LZW minimum code size
    out.write(b'\x08')

    # Write the image data
    for i in range(0, datalen):

        # If we're at an even index, then we need to write
        # two pixels from our random bytes
        if i % 2 == 0:

            # Get the next two bytes from /dev/urandom
            b1 = ord(rng.read(1))
            b2 = ord(rng.read(1))

            # Write those two bytes to the output stream
            out.write(struct.pack('>H',
                    (b1 << 8) | b2))