#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
    ) -> None:
    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table (black only)
    out.write(b'\x00\x00\x00\xFF')

    # Image descriptor
    out.write(struct.pack('<BHHBB', 0x2C, 0, 0, 0, 0))

    # Local color table (black only)
    out.write(b'\x00\x00\x00\xFF')

    # Image data
    for i in range(16):
        out.write(b'\x00\x00\x00\x00')

    # Trailer
    out.write(b';')