#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = (int.from_bytes(rng.read(2), 'big') % 64) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 64) + 1
    flags  = (int.from_bytes(rng.read(1), 'big') & ~7) | 5
    bgcol  = int.from_bytes(rng.read(1), 'big')
    aspect = int.from_bytes(rng.read(1), 'big')
    out.write(struct.pack('>HHBBB', width, height, flags, bgcol, aspect))

    # Global color table
    global_ctsize = ((int.from_bytes(rng.read(1), 'big') & 7) + 1) * 3
    if global_ctsize > 0:
        out.write(bytes([global_ctsize]))
        out.write(bytes([rng.read(1)[0] for _ in range(global_ctsize)]))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), 'big')
    top    = int.from_bytes(rng.read(2), 'big')
    width  = (int.from_bytes(rng.read(2), 'big') % 64) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 64) + 1
    flags  = (int.from_bytes(rng.read(1), 'big') & ~7) | 5
    lctsize= ((int.from_bytes(rng.read(1), 'big') & 7) + 1) * 3
    interlaced = bool((int.from_bytes(rng.read(1), 'big') >> 7) & 1)
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags, lctsize))

    # Local color table
    local_ctsize = ((int.from_bytes(rng.read(1), 'big') & 7) + 1) * 3
    if local_ctsize > 0:
        out.write(bytes([local_ctsize]))
        out.write(bytes([rng.read(1)[0] for _ in range(local_ctsize)]))

    # Data sub-block
    datalen = int.from_bytes(rng.read(3), 'big')
    out.write(struct.pack('>B', datalen))
    out.write(bytes([rng.read(1)[0] for _ in range(datalen)]))