#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = rng.randint(1024, 512*1024) // 64
    height = rng.randint(768,  384*1024) // 64
    flags  = 0x80 | rng.randint(0, 0x0C)
    bg     = rng.randint(0, 255)
    aspect = 0
    out.write(struct.pack('>HHBBB', width, height, flags, bg, aspect))

    # Global color table
    ncolors = min((width+height)//2, 256)
    for i in range(ncolors):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        out.write(struct.pack('BBBB', r, g, b, 0))

    # Image descriptor
    left      = 0
    top       = 0
    width     = width * 64
    height    = height * 64
    flags     = 0x80 | rng.randint(0, 0x0C)
    delaytime = rng.randint(0, 0xFF)
    transindex= 0
    out.write(struct.pack('>HHHHBB', left, top, width, height, flags,
                          delaytime, transindex))

    # Image data
    datalen = rng.randint(1024, 512*1024)
    data = rng.read(datalen)
    out.write(data)

