#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 19 09:26:57 2021

@author: lukas
"""
import numpy as np
from scipy.optimize import minimize
import matplotlib.pyplot as plt
import pandas as pd

def objective(x, *args):
    """Objective function for the optimization problem"""
    
    # extract data from args
    x_data = args[0]
    y_data = args[1]
    sigma = args[2]
    n = len(y_data)
    
    # calculate model prediction
    y_model = np.zeros(n)
    for i in range(n):
        y_model[i] = x[0]*np.exp(-(x_data[i]/x[1])**2/(2*sigma**2)) + x[2]
        
    # calculate residuals
    res = y_data - y_model
    
    return np.sum((res/sigma)**2)

def fit_gaussian(x_data, y_data, sigma=None):
    """Fit a Gaussian to the given data points (x_data, y_data)"""
    
    if sigma is None:
        sigma = np.std(y_data)
    
    # initial guesses for parameters
    p0 = [np.max(y_data), 1, np.min(y_data)]
    
    # optimize parameters
    res = minimize(objective, p0, args=(x_data, y_data, sigma))
    
    # extract optimized parameters and covariance matrix
    popt = res.x
    pcov = res.hess_inv
    
    # calculate standard errors of the parameters
    perr = np.sqrt(np.diag(pcov))
    
    return popt, perr
