#!/usr/bin/env python3

import sys, os
from argparse import ArgumentParser
from collections import defaultdict
from itertools import chain

def parse_args():
    parser = ArgumentParser(description="")
    parser.add_argument("--input", type=str)
    parser.add_argument("--output", type=str)
    return parser.parse_args()

def main():
    args = parse_args()
    
    with open(args.input) as f:
        lines = [line for line in f]
        
    # Parse the input file into a list of lists of tuples (i, j, k) where i is the index of the first element, j is the second and k is the third
    parsed = []
    for line in lines:
        if len(line.strip()) == 0:
            continue
            
        parts = line.split(", ")
        parsed.append([tuple(map(int, part.split())) for part in parts])
    
    # Find all possible triangles
    triangles = set()
    for triangle in chain(*parsed):
        triangles.add((triangle[0], triangle[1]))
        triangles.add((triangle[1], triangle[2]))
        triangles.add((triangle[2], triangle[0]))
    
    # Count how many times each triangle occurs
    counts = defaultdict(lambda: 0)
    for triangle in triangles:
        counts[triangle] += 1
    
    # Write to output file
    with open(args.output, "w") as f:
        for triangle, count in sorted(counts.items(), key=lambda x: x[1], reverse=True):
            f.write("%d,%d,%d\t%d\n" % (triangle[0], triangle[1], triangle[2], count))
