#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the screen descriptor
    out.write(struct.pack('<H', rng.read(2)))
    out.write(struct.pack('BBB', *[rng.read(1)[0] for _ in range(3)]))

    # Write the global color table
    out.write(b'\x00')
    out.write(bytes([rng.read(1)[0]]))
    out.write(bytes([rng.read(1)[0]]))

    # Write the image descriptor
    out.write(struct.pack('<HHBB', *[rng.read(2) for _ in range(4)]))
    out.write(b'\x00\x00')

    # Write the local color table
    out.write(b'\x00')
    out.write(bytes([rng.read(1)[0]]))
    out.write(bytes([rng.read(1)[0]]))

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x01\xff\x00')
    out.write(struct.pack('<H', rng.read(2)))

    # Write the application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Write the loop count
    out.write(struct.pack('<H', rng.read(2)))

    # Write the data sub-blocks
    while True:
        block_size = rng.read(1)[0]
        if not block_size:
            break
        out.write(bytes([block_size]))
        out.write(rng.read(block_size))
