#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0xff)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(256):
        r = g = b = struct.pack('B', i)
        out.write(r + g + b)

    # Graphics control extension
    ext_code = struct.pack('B', 0x21)
    ext_len  = struct.pack('B', 0xF9)
    block    = struct.pack('B', 0x04)
    delay    = struct.pack('<H', 0xffff)
    transp   = struct.pack('B', 0xff)
    term     = struct.pack('B', 0x00)
    out.write(ext_code + ext_len + block + delay + transp + term)

    # Image descriptor
    desc_code = struct.pack('B', 0x2C)
    x         = struct.pack('>H', 0)
    y         = struct.pack('>H', 0)
    w         = struct.pack('>H', 1024)
    h         = struct.pack('>H', 768)
    flags     = struct.pack('B', 0x80)
    lct       = struct.pack('B', 0x00)
    interl    = struct.pack('B', 0x00)
    out.write(desc_code + x + y + w + h + flags + lct + interl)

    # Local color table
    for i in range(256):
        r = g = b = struct.pack('B', i)
        out.write(r + g + b)

    # Raster data
    for _ in range(768 * 1024 // 8):
        byte = rng.read(1)[0]
        out.write(struct.pack('B', byte))

    return True

