#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')
    
    # Logical Screen Descriptor
    width = 16 + (rng.read(1)[0] % 256)
    height = 16 + (rng.read(1)[0] % 256)
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBB', width, height, background_color_index, pixel_aspect_ratio))
    
    # Global Color Table
    color_table_size = 7 + (rng.read(1)[0] % 8)
    color_resolution = 1 + (rng.read(1)[0] % 4)
    sort_flag = False
    global_color_table = []
    for i in range(2 ** color_table_size):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        global_color_table += [red, green, blue]
    out.write(bytes([color_table_size | (sort_flag << 7), color_resolution]))
    out.write(bytes(global_color_table))
    
    # Image Data
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\xff\x00'
    out.write(image_descriptor)
    
    # Local Color Table
    local_color_table_size = 7 + (rng.read(1)[0] % 8)
    local_color_table = []
    for i in range(2 ** local_color_table_size):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        local_color_table += [red, green, blue]
    out.write(bytes([local_color_table_size | (sort_flag << 7)]))
    out.write(bytes(local_color_table))
    
    # LZW Minimum Code Size
    lzw_minimum_code_size = 1 + (rng.read(1)[0] % 8)
    out.write(bytes([lzw_minimum_code_size]))
    
    # Pixel data
    pixels = []
    for y in range(height):
        for x in range(width):
            pixels += [rng.read(1)[0] % 256]
    out.write(bytes(pixels))
    
    # Trailer
    out.write(b';')
