#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
) -> bool:
    # Header
    out.write(b'GIF89a')
    out.write(struct.pack('<H', 1024))
    out.write(struct.pack('<H', 768))
    out.write(b'\x00\x00')

    # Global color table
    for i in range(0, 768):
        r = ord(rng.read(1)) & 0xff
        g = ord(rng.read(1)) & 0xff
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    return True


