#!/usr/bin/env python3

import os
import sys
from pathlib import Path

# Add the parent directory to the PATH environment variable
sys.path.append(str(Path(__file__).resolve().parent.parent))

from utils.config_loader import try_to_find_and_load_dotenv, load_environment_variables
from utils.logger import logger
from utils.utils import get_current_time

try:
    from dotenv import load_dotenv
except ImportError as e:
    print("You need to install the 'python-dotenv' package in order to use this script.")
    exit()


def main():
    # Load environment variables from .env file (if it exists)
    try_to_find_and_load_dotenv()
    load_environment_variables()

    # Get the current time
    now = get_current_time()

    # Create a new directory for the output files
    if not os.path.exists('output'):
        os.mkdir('output')

    # Create a new directory for the output files of this run
    if not os.path.exists(f'output/{now}'):
        os.mkdir(f'output/{now}')

    # Run the program
    logger.info("Starting the program...")
    from src.main import main
    main()
