#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main(argv: list[str]) -> int:
    if len(argv) != 2:
        print("Usage: %s <file>" % argv[0], file=sys.stderr)
        return 1

    with open(argv[1], "rb") as fp:
        data = fp.read()

    # Read the header
    magic, version, num_entries = struct.unpack("<4sLL", data[:16])
    assert magic == b"DNAM"
    assert version == 1
    
    # Read the entries
    offset = 16
    for i in range(num_entries):
        name, size = struct.unpack("<16sL", data[offset : offset + 20])
        offset += 20
        
        # Skip empty names and names that are not valid UTF-8
        name = name.decode('utf-8', 'ignore')
        if name == "" or name == "\xff\xfe":
            continue
            
        # Write the entry to a file
        filename = "%s.dna" % name
        with open(filename, "wb") as outfp:
            outfp.write(data[offset : offset + size])
