#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header
    out.write(header)

    # Write the screen descriptor
    out.write(b'\x01\x00\x01\x00\x80\xff\x00\xff')

    # Write the global color table (256 colors)
    for i in range(256):
        out.write(struct.pack('BBB', *[i] * 3))

    # Write the application extension
    out.write(b'\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00')

    # Write the local color table (256 colors)
    for i in range(256):
        out.write(struct.pack('BBB', *[i] * 3))

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x01\x00\x00')

    # Write the plain text extension
    out.write(b'\x21\x01\x04\x01\x00\x01\x00\x00\x00\x00')

    # Write the comment extension
    out.write(b'\x21\xfe\x0b\x47\x49\x46\x38\x39\x61\x00\x00')

    # Write the image data block
    out.write(data)

    # Write the trailer
    out.write(b'\x3b')
