#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(out: BinaryIO, rng: callable):
    # Magic header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    width = 1 + rng() % 256
    height = 1 + rng() % 256
    out.write(struct.pack('<HHB', width, height, 0))
    
    # Global color table
    for i in range(7):
        out.write(bytes([i]))
        
    # Graphic control extension
    out.write(b'\x21\xf9\x04\x00\xff\xff\xff\x00')
    
    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # Local color table
    for i in range(7):
        out.write(bytes([i]))
    
    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([y * width + x]))
            
    # Trailer
    out.write(b'\x3b')
