#!/usr/bin/env python3

import struct
from typing import BinaryIO

def read_gif(rng: BinaryIO) -> bytes:
    header = rng.read(6)
    if header != b'GIF87a' and header != b'GIF89a':
        raise ValueError('Not a GIF file')
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    return rng.read(datalen)

def write_gif(
        rng: BinaryIO,
        out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and length of the data block
    out.write(header + len(data).to_bytes(3, 'big'))
    # Write the data block itself
    out.write(data)
