#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        datalen = int.from_bytes(rng.read(2), 'little') & 0xffff
        if datalen == 0:
            break
        data = rng.read(datalen)
        assert len(data) == datalen
        out.write(struct.pack('<H', datalen))
        out.write(data)

    # Write the trailer
    out.write(b'\x3B')
