#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1
    
    # Write the header
    out.write(header)

    # Write the width and height
    out.write(b'\x02\x00')
    out.write(b'\x02\x00')

    # Write the color table
    out.write(b'\xff\xff\xff\x00\x00\x00\x00\x00\x00\x00')

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x05\x00\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x02\x00')

    # Write the image data
    for i in range(datalen):
        out.write(struct.pack('B', rng.read(1)[0]))

    # Write the trailer
    out.write(b';')
