#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = 1024
    height = 768
    background_color = 0xFF
    pixel_aspect_ratio = 0
    global_color_table_flag = 0
    color_resolution = 8
    sort_flag = 0
    size_of_global_color_table = 7
    bg_color_index = 0
    pixel_data_subblocks = []
    trailer = b'\x3B'

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<HBB', width, height, pixel_aspect_ratio))
    out.write(struct.pack('BB', (background_color << 4) | global_color_table_flag,
                          (color_resolution << 4) | sort_flag))
    out.write(struct.pack('B', size_of_global_color_table))
    out.write(struct.pack('B', bg_color_index))

    # Write the global color table
    for i in range(2 ** (size_of_global_color_table + 1)):
        out.write(struct.pack('BBB', rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]))

    # Write the image descriptor
    out.write(b'\x2C')
    out.write(struct.pack('<HH', 0, 0))
    out.write(struct.pack('<HH', width, height))
    out.write(struct.pack('BB', local_color_table_flag, interlaced_flag))
    out.write(struct.pack('B', size_of_local_color_table))
    out.write(struct.pack('B', bg_color_index))

    # Write the local color table
    for i in range(2 ** (size_of_local_color_table + 1)):
        out.write(struct.pack('BBB', rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]))

    # Write the LZW-compressed pixel data subblocks
    while True:
        length = rng.read(1)[0]
        if length == 0:
            break
        else:
