#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for _ in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(b'\x00\x00\x00')

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\x00')

    # Plain text extension
    out.write(b'\x21\x01\x04\x01\x00\x00\x00')

    # Comment extension
    out.write(b'\x21\xfe\x0c\x4a\x6f\x68\x6e\x20\x3c\x68\x74\x74\x70\x3e\x03\xe9')

    # Image block
    left   = 0
    top    = 0
    width  = 1024
    height = 768
    packed = struct.pack('>HHHHB', left, top, width, height, 0)
    out.write(packed)

    # Local color table
    for _ in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))