#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    width = 1024
    height = 768
    background_color = 0xff
    pixel_aspect_ratio = 0
    global_color_table = True
    color_resolution = 8
    sort_flag = False
    color_table_size = 7
    block_terminator = b';'

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(struct.pack('B', background_color))
    out.write(struct.pack('B', pixel_aspect_ratio))

    # Write the global color table
    flags = (global_color_table << 7) | \
            (color_resolution << 4) | \
            (sort_flag << 3) | \
            color_table_size
    out.write(struct.pack('B', flags))

    for i in range(2 ** (color_table_size + 1)):
        out.write(struct.pack('BBB', *[rng.read(1)[0] for _ in range(3)]))

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(struct.pack('B', background_color))
    out.write(struct.pack('B', pixel_aspect_ratio))

    # Write the local color table
    flags = (global_color_table << 7) | \
            (color_resolution << 4) | \
            (sort_flag << 3) | \
            color_table_size
    out.write(struct.pack('B', flags))

    for i in range(2 ** (color_table_size + 1)):
        out.write(struct.pack('BBB', *[rng.read(1)[0] for _ in range(3)]))

    # Write the LZW minimum code size
    out.write(struct.pack('B', color_table_size))

    # Write the image data
    while True:
        # Read a byte from the RNG and write it as an index to the color table
        out.write(struct.pack('B', rng.read(1)[0]))

        if rng.read(1)[0]:
            break

    # Write the block terminator
    out.write(block_terminator)
