#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H', 0)
    top       = struct.pack('>H', 0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x00 | 0x08 | 0x04)
    bg        = struct.pack('B',   0x00)
    trans     = struct.pack('B',   0x00)
    interlace = struct.pack('B',   0x00)
    sort      = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + bg + trans + interlace + sort)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Graphic control extension
    ext_type = struct.pack('B', 0xf9)
    size     = struct.pack('B', 0x04)
    flags    = struct.pack('B', 0x00 | 0x01)
    delay    = struct.pack('>H', 0xffff)
    trans    = struct.pack('B', 0x00)
    out.write(ext_type + size + flags + delay + trans)

    # Plain text extension
    ext_type = struct.pack('B', 0xfe)
    size     = struct.pack('B', 0x0c)
    flags    = struct.pack('B', 0x00 | 0x01)
    bg       = struct.pack('B', 0x00)
    fg       = struct.pack('B', 0xff)
    x        = struct.pack('>H', 0x0000)
    y        = struct.pack('>H', 0x0000)
    width    = struct.pack('>H', 1024)
    height   = struct.pack('>H',  768)
    cellw    = struct.pack('B', 0x00)
    cellh    = struct.pack('B', 0x00)
    data     = struct.pack('B', 0x00) * 12
    out.write(ext_type + size + flags + bg + fg + x + y + width + height + cellw + cellh + data)

    # Application extension
    ext_type = struct.pack('B', 0xff)
    size     = struct.pack('B', 0x0b)
    id       = struct.pack('11s', b'NETSCAPE2.0')
    block    = struct.pack('>I', 0x030104)
    loop     = struct.pack('>H', 0xffff)
    out.write(ext_type + size + id + block + loop)

    # Image data
    out.write(struct.pack('B', 0x2c))
    for _ in range(0, 1024*768):
        out.write(struct.pack('B', 0x00))

    # Trailer
    out.write(struct.pack('B', 0x3b))

