#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO, List


def read_uint16(f: BinaryIO) -> int:
    return struct.unpack('>H', f.read(2))[0]


def write_uint16(f: BinaryIO, value: int):
    f.write(struct.pack('>H', value))


def read_uint8(f: BinaryIO) -> int:
    return struct.unpack('B', f.read(1))[0]


def write_uint8(f: BinaryIO, value: int):
    f.write(struct.pack('B', value))


def read_int8(f: BinaryIO) -> int:
    return struct.unpack('b', f.read(1))[0]


def write_int8(f: BinaryIO, value: int):
    f.write(struct.pack('b', value))


def read_string(f: BinaryIO, length: int = 0) -> str:
    if not length:
        length = read_uint16(f)

    string = b''
    while len(string) < length:
        byte = f.read(1)

        # End of file reached?
        if not byte:
            raise EOFError()

        if byte == b'\x00':
            break

        string += byte

    return string.decode('utf-8')


def write_string(f: BinaryIO, string: str):
    encoded = bytes(string, 'utf-8') + b'\x00'
    write_uint16(f, len(encoded))
    f.write(encoded[:len(encoded)])


class