#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<H', 0x200))   # Width
    out.write(struct.pack('<H', 0x100))   # Height
    out.write(b'\xf7')                    # Flags (global color table)
    out.write(b'\x00')                    # Background color index
    out.write(b'\x00')                    # Pixel aspect ratio

    # Write the global color table
    for i in range(0x100):
        out.write(bytes([i]))            # Red
        out.write(bytes([i]))            # Green
        out.write(bytes([i]))            # Blue

    # Write the image descriptor
    out.write(b'\x2c')                    # Image separator
    out.write(struct.pack('<H', 0x200))   # Left position
    out.write(struct.pack('<H', 0x100))   # Top position
    out.write(struct.pack('<H', 0x200))   # Width
    out.write(struct.pack('<H', 0x100))   # Height
    out.write(b'\x00')                    # Flags (no local color table)
    out.write(b'\x00')                    # Interlace flag
    out.write(b'\x00')                    # Sort flag
    out.write(b'\x00')                    # Reserved byte

    # Write the local color table
    for i in range(0x100):
        out.write(bytes([i]))            # Red
        out.write(bytes([i]))            # Green
        out.write(bytes([i]))            # Blue

    # Write the graphics control extension
    out.write(b'\x21')                    # Extension introducer
    out.write(b'\xf9')                    # Graphic control label
    out.write(b'\x04')                    # Block size
    out.write(b'\x05')                    # Packed fields
    out.write(b'\xff')                    # Delay time
    out.write(b'\x00')                    # Transparent color index
    out.write(b'\x00')                    # Terminator

    # Write the plain text extension
    out.write(b'\x21')                    # Extension introducer
    out.write(b'\x01')                    # Plain text label
    out.write(b'\x0f')                    # Block size
    out.write(b'Lorem ipsum dolor sit amet, consectetur adipiscing elit.')
    out.write(b'\x00')                    # Terminator

    # Write the application extension
    out.write(b'\x21')                    # Extension introducer
    out.write(b'\xff')                    # Application label
    out.write(b'\x0b')                    # Block size
    out.write(b'NETSCAPE2.0')              # Application identifier
    out.write(b'\x03')                    # Sub-block size
    out.write(b'\x01')                    # Loop sub-block ID
    out.write(b'\x00\x00')                # Loop count
    out.write(b'\x00')                    # Terminator

    # Write the comment extension
    out.write(b'\x21')                    # Extension introducer
    out.write(b'\xfe')                    # Comment label
    out.write(b'\x0d')                    # Block size
    out.write(b'Created by <NAME>')
    out.write(b'\x00')                    # Terminator

    # Write the graphic data block
    for y in range(0x100):
        for x in range(0x200):
            out.write(bytes([y ^ x]))     # XOR with pixel value

