#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def generate_gif(out: BinaryIO, rng: BinaryIO) -> bool:
    """Generate a GIF image."""

    # Header
    signature = bytes([
        0x47, 0x49, 0x46, 0x38, 0x39, 0x61, 0x01, 0x00, 0x01, 0x00, 0x80, 0x00, 0x00])
    version   = bytes([0x00, 0x00, 0x00, 0x00])
    out.write(signature + version)

    # Logical screen descriptor
    left      = struct.pack('>H',  0x00)
    top       = struct.pack('>H',  0x00)
    width     = struct.pack('>H',  1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x00 | 0x08 | 0x04)
    bg        = struct.pack('B',   0x00)
    trans     = struct.pack('B',   0x00)
    interlace = struct.pack('B',   0x00)
    sort      = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + bg + trans + interlace + sort)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    return True


