#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Read 10 bytes from /dev/urandom to get the first three bytes
    # which will be used as the magic number for this gif file
    header = b'GIF' + rng.read(7)

    # Read another 4 bytes from /dev/urandom to get the length of the
    # data block that follows (the rest of the header is already known)
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header and length of the data block
    out.write(header + len(data).to_bytes(3, 'big'))
    # Write the data block itself
    out.write(data)
