#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
) -> None:
    # Read 10 bytes to determine image dimensions.
    header = rng.read(10)

    width, height = struct.unpack("<HH", header[:4])
    print("Generating %dx%d gif..." % (width, height))

    # Write magic string and screen descriptor.
    out.write(b"GIF89a")
    out.write(struct.pack("<HHBBB", width, height, 0xF7, 0x0B, 0))

    # Write color table with one entry.
    out.write(bytes([
        0xFF, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00
    ]))

    # Write graphic control extension block.
    out.write(bytes([
        0x21, 0xF9, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x2C, 0x00
    ]))

    # Write image data sub-block.
    out.write(bytes([
        0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x