#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Generate some random numbers to use in our header and data blocks
    width = int.from_bytes(rng.read(2), byteorder='little') % 100 + 50
    height = int.from_bytes(rng.read(2), byteorder='little') % 100 + 50

    # Construct the header bytes
    header = b'GIF89a' \
             + width.to_bytes(2, byteorder='little', signed=False) \
             + height.to_bytes(2, byteorder='little', signed=False) \
             + b'\x87\x00\x00'

    # Construct the data block bytes (all zeroes for now)
    data = b'\x00' * ((width * height) // 8)
    # Write the header
    out.write(header)
    # Write the data block itself
    out.write(data)

