#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for i in range(256):
        r = rng.read(1)[0] & 0xFF
        g = rng.read(1)[0] & 0xFF
        b = rng.read(1)[0] & 0xFF
        out.write(struct.pack('BBB', r, g, b))

    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0')
    out.write(b'\x03\x01\x00\x00')
    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))