#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1024 * 512 # 512KB

    # Write the header and logical screen descriptor
    out.write(header + bytes([
        0x7D, 0x00, 0x00, 0x00, # Logical Screen Descriptor
        0x00, 0x00,             # Width
        0x00, 0x00,             # Height
        0xFB,                   # Color Table Size
        0x00                    # Background Color Index
    ]))

    # Generate the color table
    for i in range(0, 2 ** 8):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        out.write(bytes([r, g, b]))

    # Write the image block
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the data
    while True:
        chunklen = rng.read(1)[0] & 0xFF
        if not chunklen:
            break
        
        out.write(chunklen.to_bytes(1, 'big'))
        out.write(rng.read(chunklen))
    
    # Write the end of file marker
    out.write(b';')


if