#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_gif(out: BinaryIO, rng: BinaryIO):

    # Header (6 bytes)
    header = b'GIF89a' + \
             struct.pack('<HH', 1024, 768) + \
             b'\x87\x00\x00'

    # Logical screen descriptor (7 bytes)
    lsd = struct.pack('<HHBBBHB',
                      1024, 768,
                      0xf7, 0, 0,
                      True, 0)

    # Global color table (768 bytes)
    gct = bytearray()
    for i in range(0, 768, 3):
        r = next(rng) % 256
        g = next(rng) % 256
        b = next(rng) % 256

        gct += struct.pack('BBB', r, g, b)

    # Image descriptor (10 bytes)
    id = struct.pack('<HHHHB',
                    0, 0,
                    1024, 768,
                    False)

    # Local color table (no local color table)
    lct = b''

    # Data block (image data)
    data = bytearray()
    for y in range(768):
        for x in range(1024):
            data += struct.pack('B',
                                ((next(rng) >> 1) & 0xff))

    # Write everything to output
    out.write(header)
    out.write(lsd)
    out.write(gct)
    out.write(id)
    out.write(lct)

    # Write the data block itself
    out.write(data)
