#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('>HHBBB', 1024, 768, 0x80, 0, 0))

    # Write the global color table (black and white only)
    for i in range(256):
        if i % 2 == 0:
            out.write(b'\x00\x00\x00')
        else:
            out.write(b'\xff\xff\xff')

    # Write the image descriptor
    out.write(struct.pack('>HHHHB', 0, 0, 1024, 768, 0))

    # Write the local color table (black and white only)
    for i in range(256):
        if i % 2 == 0:
            out.write(b'\x00\x00\x00')
        else:
            out.write(b'\xff\xff\xff')

    # Write the graphics control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\x00\x00\x00\x00')

    # Write the plain text extension
    out.write(b'\x21\x01\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0')
    out.write(b'\x03\x01\x00\x00')

    # Write the comment extension
    out.write(b'\x21\xfe\x0cComment Block')

    # Write the graphic control label
    out.write(b'\x21\xf9\x04')

    # Write the plain text label
    out.write(b'\x21\x01\x00')

    # Write the application label
    out.write(b'\x21\xff\x0b')

    # Write the comment label
    out.write(b'\x21\xfe\x0c')

    # Write the end of file marker
    out.write(b';')

    # Write the actual data
    out.write(data)

