#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H', 0)
    top       = struct.pack('>H', 0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x00 | 0x08 | 0x04)
    bg        = struct.pack('B',   0x00)
    trans     = struct.pack('B',   0x00)
    interlace = struct.pack('B',   0x00)
    sort      = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + bg + trans + interlace + sort)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):

            p = ord(rng.read(1)) & 0x07

            if (p == 0):
                out.write(struct.pack('BBB', 0x00, 0x00, 0x00))
            elif (p == 1):
                out.write(struct.pack('BBB', 0x00, 0x00, 0xaa))
            elif (p == 2):
                out.write(struct.pack('BBB', 0x00, 0xaa, 0x00))
            elif (p == 3):
                out.write(struct.pack('BBB', 0x00, 0xaa, 0xaa))
            elif (p == 4):
                out.write(struct.pack('BBB', 0xaa, 0x00, 0x00))
            elif (p == 5):
                out.write(struct.pack('BBB', 0xaa, 0x00, 0xaa))
            elif (p == 6):
                out.write(struct.pack('BBB', 0xaa, 0xaa, 0x00))
            else:
                out.write(struct.pack('BBB', 0xaa, 0xaa, 0xaa))

    # Trailer
    out.write(b';')


if