#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#=======================================================================
#
# gif_gen.py
# ---------
# Simple Python script to generate a random GIF file. Used as an
# example of how to use the rng_test.vhd testbench with Vivado.
#
# Note that this is not a "real" RNG! It's just meant to be used as
# a simple way to get some data into memory so we can check it later.
#
# 
# Author: <NAME>
# Copyright (c) 2014, Secworks Sweden AB
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or 
# without modification, are permitted provided that the following 
# conditions are met: 
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer. 
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#=======================================================================
 
#-------------------------------------------------------------------
# Python module imports.
#-------------------------------------------------------------------
import sys
import os
import time
import struct


#-------------------------------------------------------------------
# gen_gif()
#
# Generate a random GIF image.
#-------------------------------------------------------------------
def gen_gif(filename, seed=None):
    if seed!= None:
        rng = RNG(seed)
    else:
        rng = RNG()

    try:
        out = open(filename, 'wb')
    except IOError:
        print("Error opening %s." % filename)
        return False


    # Header
    out.write(struct.pack('<ccccBBBBBHHH',
                         ord('G'), ord('I'), ord('F'), ord('8'),
                         0x0a, 0x00, 0x00, 0x00,
                         0x00, 0x00, 0xf1, 0x00))

    # Logical screen descriptor
    left   = struct.pack('H',    0x00)
    top    = struct.pack('H',    0x00)
    width  = struct.pack('H',    1024)
    height = struct.pack('H',    768)
    flags  = struct.pack('B',   0x87)
    bg     = struct.pack('B',   0xff)
    trans  = struct.pack('B',   0x00)
    interlace = struct.pack('B',   0x00)
    sort      = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + bg + trans + interlace + sort)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    return True


