#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given PRNG object.
def generate_gif(rng: BinaryIO, out: BinaryIO):
    # Header
    header = b"GIF89a" + bytes([
        1 << 7 |      # Global Color Table Flag
        7,           # Color Resolution
        0,           # Sorting Method (not sorted)
        0, 0         # Size of Global Color Table
    ])

    out.write(header)

    # Logical Screen Descriptor
    lsd = bytes([
        0, 0,        # Screen Width
        0, 0,        # Screen Height
        0b10000000,  # Packed Fields: GCT Present
        0b00000000   # Background Color Index
    ])

    out.write(lsd)

    # Global Color Table
    gct = [
        0x00, 0x00, 0x00, 0xff,
        0xff, 0xff, 0xff, 0xff
    ]

    for i in range(4 * len(gct)):
        out.write(bytes([gct[i]]))

    # Image Descriptor
    id = bytes([
        0, 0, 0,     # Left Position
        0, 0, 0,     # Top Position
        0, 0, 0,     # Image Width
        0, 0, 0,     # Image Height
        0b10000000,  # Packed Fields: Local Color Table
        0          # LZW Minimum Code Size
    ])

    out.write(id)

    # Data
    datalen = 1024 * 1024
    while datalen > 0:
        chunklen = min(256, datalen)
        chunk = rng.read(chunklen)
        out.write(chunk)
        datalen -= chunklen

