#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_gif(out: BinaryIO, rng):
    # Header
    out.write("GIF89a".encode())

    # Logical screen descriptor
    width = 1280
    height = 720
    packed = b""
    packed += struct.pack("<HHB", width, height, 0xF0 | 0x04)
    packed += bytes([0]) * 3
    packed += struct.pack("<BBB", 0, 0, 0)
    out.write(packed)

    # Global color table
    for i in range(0, 256):
        r = next(rng) & 0xFF
        g = next(rng) & 0xFF
        b = next(rng) & 0xFF
        out.write(struct.pack("<BBB", r, g, b))

    # Image descriptor
    packed = b""
    packed += struct.pack("<HHBB", 0, 0, 0b10000000 | 0b00000000, 0)
    packed += struct.pack("<HHH", 0, 0, 0)
    out.write(packed)

    # LZW minimum code size
    out.write(bytes([0b11]))

    # Data sub-block
    data = bytearray()
    while len(data) < 1024:
        if len(data) == 0:
            data.append(next(rng) & 0xFF)
        else:
            data.append((data[-1] + 1) & 0xFF)

        if len(data) % 128 == 0 and len(data)!= 0:
            out.write(data)
