#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random(out: BinaryIO, rng: BinaryIO):
    # Header (13 bytes)
    out.write(b'GIF89a')

    width = 256 + int.from_bytes(rng.read(1), 'big') % 40
    height = 256 + int.from_bytes(rng.read(1), 'big') % 40

    flags = b'\x87\x00' if bool(int.from_bytes(rng.read(1), 'big')) else b'\x00\x00'

    # Logical screen descriptor (7 bytes)
    out.write(struct.pack('>HHBBB', width, height, *flags))

    color_table_size = min((width*height)//64, 256) // 8
    global_color_table = [
        int.from_bytes(rng.read(1), 'big') for _ in range(color_table_size)
    ]

    # Global color table (3*(2**(n+1))) bytes
    out.write(b''.join([struct.pack('B', c) for c in global_color_table]))

    # Application extension block (11 bytes)
    out.write(b'\x21\xFF\x0BNETSCAPE2.0')
    out.write(b'\x03\x01')
    out.write(b'\x00\x00')

    # Graphics control extension block (8 bytes)
    out.write(b'\x21\xF9\x04')
    out.write(b'\x08\x00\x00')
    out.write(b'\x00\x00')

    # Image descriptor (10 bytes)
    out.write(struct.pack('>HBHBB', 0, 0, width, height, 0))

    # Plain text data sub-block (1 byte)
    out.write(b'\x01')

    # Plain text data sub-block (256 bytes)
    out.write(b'\x00' * 256)

    # Trailer (1 byte)
    out.write(b';')

