#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1024 * 512 # 512KB

    width = int.from_bytes(rng.read(1), byteorder='little') % 64 + 1
    height = int.from_bytes(rng.read(1), byteorder='little') % 64 + 1

    palette = [b'\x00\x00\x00'] * 256
    for i in range(len(palette)):
        palette[i] += bytes([
            rng.read(1)[0],
            rng.read(1)[0],
            rng.read(1)[0],
            0xFF])
    
    data = []
    for _ in range(datalen):
        data.append(rng.read(1))

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB',
        0, # Screen Width
        0, # Screen Height
        7, # Color Depth
        0, # Global Color Table Flag
        0   # Background Color Index
    ))

    # Write the global color table
    for p in palette:
        out.write(p)

    # Write the image descriptor
    out.write(struct.pack('<HHHHBB',
        0, # Left Position
        0, # Top Position
        width - 1, # Image Width
        height - 1, # Image Height
        0, # Local Color Table Flag
        0   # Interlace Flag
    ))

    # Write the local color table
    for p in palette:
        out.write(p)

    # Write the image data
    for d in data:
        out.write(d)


if