#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBBBBB',
        1024, 768, 8, 0, 0, 0))

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    return True


