#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_gif(out: BinaryIO, width: int = 1024, height: int = 768):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HH', width, height))
    out.write(b'\x87\x00')

    # Global color table
    for i in range(0, 256 * 3, 3):
        out.write(bytes([i // 3]))

    # Image descriptor
    out.write(b'\x2c')
    out.write(struct.pack('<HHHHHBB', 0, 0, width - 1, height - 1, 0, 0))

    # Local color table
    out.write(b'\xff')

    # LZW minimum code size
    out.write(b'\x08')

    # Image data
    out.write(b'\x00')

