#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for _ in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image descriptor
    left   = 0
    top    = 0
    width  = 1024
    height = 768
    packed = struct.pack('>HHHHB', left, top, width, height, 0)
    out.write(packed)

    # LZW minimum code size
    out.write(b'\x08')

    # Pixel data
    # TODO: Use an actual compressor instead of this dumb thing
    pixel_data = []
    for i in range(width * height):
        if not i % 1024:
            print('\r{:.2f}%'.format((i / (width * height)) * 100), end='')

        index = rng.read(1)[0] % 256
                out.write(struct.pack('B', index))