#!/usr/bin/env python3

import os
import sys
import struct
from PIL import Image


def generate_random_gif(rng, out):
    # Header
    signature = struct.pack('3s3s<HBHHBB',
                            bytes([0x47]),
                            bytes([0x49]),
                            0x00,
                            0x00,
                            0x00,
                            0x00,
                            0x00,
                            0x00)
    version   = struct.pack('<H', 0x000a)
    screen    = struct.pack('<HHHHBB', 1024, 768, 0xf000, 0xf000, 0x00, 0x00)
    global_ct = struct.pack('B', 0xff)
    out.write(signature + version + screen + global_ct)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H', 0)
    top       = struct.pack('>H', 0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x00 | 0x08 | 0x04)
    bg        = struct.pack('B',   0x00)
    trans     = struct.pack('B',   0x00)
    interlace = struct.pack('B',   0x00)
    sort      = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + bg + trans + interlace + sort)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))


if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rng:
        with open('./flag.gif', 'wb+') as out:
            generate_random_gif(rng, out)

