#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = 1024 + int.from_bytes(rng.read(1), byteorder='little') % 512
    height = 768 + int.from_bytes(rng.read(1), byteorder='little') % 512

    # Generate the palette and image data
    palette = [struct.pack('BBB', *rng.read(3)) for _ in range(256)]
    data = bytes()
    for y in range(height):
        for x in range(width // 8):
            byte = 0x00
            for i in reversed(range(8)):
                if (int.from_bytes(rng.read(1), byteorder='little') & 0b1)!= 0:
                    byte |= 0b1 << i
            data += bytes([byte])
    
    # Write the header
    out.write(struct.pack('<HHB', width, height, len(palette)))
    # Write each color to the palette
    for c in palette:
        out.write(c)
    # Write the terminator
    out.write(header)
    # Write the data block itself
    out.write(data)

