#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for i in range(0x100):
        datalen = int.from_bytes(
            rng.read(2), 'big')

        if (i == 0):
            # The first frame must have a width and height of at least 16px
            # This ensures that we can test the decoder with a small image
            w = max(int.from_bytes(
                rng.read(2), 'big'), 16)
            h = max(int.from_bytes(
                rng.read(2), 'big'), 16)

            out.write(header + bytes([w & 0xff]) +
                      bytes([(w >> 8) & 0xff]) +
                      bytes([h & 0xff]) +
                      bytes([(h >> 8) & 0xff]))

        data = rng.read(datalen)
        out.write(data)

