#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Generate the palette
    pal = [struct.pack('BBB', *rng.read(3)) for _ in range(256)]

    # Write the palette to the output
    out.write(b''.join(pal))

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(256, datalen)

        # Write the chunk to the output
        out.write(
                struct.pack(
                    f'<{chunklen}B',
                    *[rng.read(1)[0] & 0x7f | (0x80 if i < 255 else 0x00)
                      for i in range(chunklen)]))

        datalen -= chunklen