#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Write the header
    out.write(b'GIF89a')

    # Write the logical screen descriptor
    out.write(struct.pack('<H', 0x200))
    out.write(b'\x01')
    out.write(bytes([rng.read(1)[0]]))
    out.write(b'\xFF\xFF\xFF')

    # Write the global color table
    for i in range(768):
        out.write(bytes([i % 256]))

    # Write the application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<I', 0))

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')

    # Write the local color table
    for i in range(768):
        out.write(bytes([(i + 1) % 256]))

    # Write the image data
    while True:
        lzw_min_code_size = rng.read(1)[0] & 0x0F
        if lzw_min_code_size == 0:
            break

        out.write(bytes([lzw_min_code_size]))

        # Read the LZW code stream and write it as-is
        while True:
            code = rng.read(1)[0]
            if code == 0:
                break
            elif (code >= lzw_min_code_size):
                out.write(bytes([code - lzw_min_code_size]))
            else:
                out.write(bytes([code]))

    # Write the trailer
    out.write(b';')