#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for i in range(0x100):
        datalen = int.from_bytes(
            rng.read(2), 'big')
        data = rng.read(datalen)

        if (i == 0):
            # The first frame must have an application extension
            appext = bytes([
                0x21, 0xFF, 0x0B,
                len('NETSCAPE2.0'),
                ord('N'), ord('E'), ord('T'), ord('S'), ord('C'), ord('A'), ord('P'), ord('E'), ord('2'), ord('.'), ord('0')])

            out.write(header + appext + data)
        else:
            out.write(data)