#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Generate a 16-bit integer for the width and height
    w = rng.read(2)
    h = rng.read(2)
    # Convert to integers
    width = int.from_bytes(w, byteorder='little', signed=False)
    height = int.from_bytes(h, byteorder='little', signed=False)
    print('Width:', width, 'Height:', height)

    # The color table is 768 bytes long (256 * 3)
    colortable = [0] * 768
    for i in range(len(colortable)):
        colortable[i] = rng.read(1)[0]

    # Write the header
    out.write(b'GIF89a')
    # Write the logical screen descriptor
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write((1 << 7).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write(b'\x00\x00')
    # Write the global color table
    out.write(b''.join([c.to_bytes(1, 'little') for c in colortable]))

    # Write the image descriptor
    out.write(b'\x2C')
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write(((1 << 7) | (0x07)).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write(b'\x00\x00')
    # Write the local color table
    out.write(b''.join([c.to_bytes(1, 'little') for c in colortable]))

    # Write the graphics control extension
    out.write(b'\x21\xF9\x04')
    out.write((0x0B).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1, 'little'))
    out.write((0x00).to_bytes(1