#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def main():
    if len(sys.argv)!= 2:
        print('Usage: {} <output>'.format(os.path.basename(__file__)),
              file=sys.stderr)
        return -1

    with open('/dev/urandom', 'rb') as rng,\
         open(sys.argv[1], 'wb') as out:

        generate_random_gif(rng, out)

    return 0


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        datalen = int.from_bytes(
            rng.read(2), 'big')
        data = rng.read(datalen)
        out.write(struct.pack('<H', datalen))
        out.write(data)
