#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = (rng.read(1)[0] % 400) + 1
    height = (rng.read(1)[0] % 400) + 1
    out.write(struct.pack('<HHBBB', width, height, 0x80 | ((rng.read(1)[0] & 0x07) << 5), 0x00, 0x00))

    # Write the global color table
    for i in range(768):
        out.write(bytes([(i >> 2) & 0xFF]))

    # Write the application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<I', 0))

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')

    # Write the local color table
    for i in range(768):
        out.write(bytes([(i >> 2) & 0xFF]))

    # Write the image data
    while True:
        lzw_min_code_size = rng.read(1)[0] & 0x0F
        if lzw_min_code_size == 0:
            break

        out.write(bytes([lzw_min_code_size]))

        # Read the LZW code stream and write it as-is
        while True:
            code = rng.read(1)[0]
            if code == 0:
                break
            elif code <= lzw_min_code_size:
                out.write(bytes([code]))
            else:
                out.write(bytes([code - lzw_min_code_size]))

    # Write the trailer
    out.write(b';')
