#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the image is determined by `width`, and
# the color depth (bits per pixel) by `bpp`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, bpp: int, out: BinaryIO
    ) -> None:
    if not 1 <= bpp <= 8: raise ValueError('Invalid bit depth!')
    if not 0 < width <= 2**16: raise ValueError('Invalid width!')
    if not 0 < height <= 2**16: raise ValueError('Invalid height!')

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, bpp, 0, 0))

    # Global color table
    for i in range(2**(bpp)):
        out.write(struct.pack('<BBBB', *rng.read(4)))

    # Image descriptor
    out.write(struct.pack('<BBHBBHHB',
            0x2C, 0, 0, 0, 0, width, height, 0))

    # Local color table
    for i in range(2**(bpp)):
        out.write(struct.pack('<BBBB', *rng.read(4)))

    # Image data
    while True:
        chunklen = rng.read(1)[0]

        if chunklen == 0x00: break

        out.write(chunklen.to_bytes(1, 'little'))
        out.write(rng.read(chunklen))