#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    width = 640
    height = 480
    color_table_size = 7
    background_color = 0x0F
    pixel_aspect_ratio = 0x00
    global_color_table = [
        0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF,
        0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0xFF,
        0xFF, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00,
        0x00, 0xFF, 0xFF, 0x00, 0xFF, 0xFF, 0x00,
        0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0xFF,
        0x00, 0xFF, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
        0xFF, 0x00, 0xFF, 0x00, 0x00, 0xFF, 0x00,
        0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
    ]

    def write_block(
            blocktype: bytes, content: bytes
        ) -> None:
        out.write(struct.pack('B', len(content)))
        out.write(blocktype)
        out.write(content)

    def write_logical_screen():
        logical_screen = struct.pack('<HHBBBH',
                width,
                height,
                0xF7, # Global Color Table Flag set to true
                color_table_size,
                background_color,
                pixel_aspect_ratio
            )

        write_block(b'\x2C', logical_screen)

    def write_global_color_table() -> None:
        table = bytearray(global_color_table[:color_table_size*3])
        write_block(b'\x21\xF9\x04', table)

    def write_image_descriptor() -> None:
        image_descriptor = struct.pack('<HHBBBBB',
            0,
            0,
            width & 0xFF,
            height & 0xFF,
            0,
            0,
            0xF7 # Local Color Table Flag set to true
        )

        write_block(b'\x2C', image_descriptor)

    def write_local_color_table() -> None:
        local_color_table = bytearray([
            0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF,
            0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0xFF,
            0xFF, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00,
            0x00, 0xFF, 0xFF, 0x00, 0xFF, 0xFF, 0x00,
            0xFF, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0xFF,
            0x00, 0xFF, 0xFF, 0x00, 0xFF, 0x00, 0xFF,
            0xFF, 0x00, 0xFF, 0x00, 0x00, 0xFF, 0x00,
            0xFF, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00,
        ])

        write_block(b'\x2C', local_color_table)

    def write_graphics_control_extension() -> None:
        graphics_control_extension = struct.pack