#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = 1024 * (int.from_bytes(
            rng.read(3), byteorder='big')) + 16
    data = bytes([i % 256 for i in range(datalen)])

    # Write the header
    out.write(header)
    # Write the logical screen descriptor
    out.write(struct.pack('>HHBBB',
            1024, 768, 0x80 | 0x40 | 0x20 | 0x10, 0, 0))
    # Write the global color table
    out.write(b'\x00\x00\x00\xff\x00\x00\x00\xff')
    # Write the image descriptor
    out.write(struct.pack('>HHHHBB',
            0, 0, 1024, 768, 0, 0))
    # Write the local color table
    out.write(b'\x00\x00\x00\xff\x00\x00\x00\xff')
    # Write the image data
    out.write(data)