#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(datalen, 256)

        # Generate the chunk
        chunk = [
                *struct.pack('>HH', 0x2c00, chunklen),
                0xf7, 0xff, 0xff] + \
                list(rng.read(chunklen))

        # Write it to the output
        out.write(bytes(chunk))

        # Update the remaining length
        datalen -= chunklen

