#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of each frame is between 1 and 256 bytes.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    frames = []
    while True:
        try:
            length = rng.read(1)[0] + 1
        except IndexError:
            break

        if length > 256:
            continue

        frames.append((length, rng.read(length)))

    for (length, data) in frames:
        # Write the header
        out.write(header)
        # Write the data block itself
        out.write(struct.pack('B', length))
        out.write(data)