#!/usr/bin/env python3

import sys
import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(256, datalen)

        # Generate a random color for each pixel
        colors = [
                (rng.read(1)[0] & 0x7f,
                 rng.read(1)[0] & 0x7f,
                 rng.read(1)[0] & 0x7f)
                for _ in range(chunklen)]
        
        # Convert the list of tuples to a packed string
        pixels = struct.pack('B'*chunklen*3, *sum(colors, ()))

        # Write the chunk to the output
        out.write(pixels)

        datalen -= chunklen

