#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'
    datalen = 256 * (1 + rng.randint(0, 4))
    data = bytes([rng.randint(0, 255)] * datalen)

    # Write the header
    out.write(header)
    # Write the logical screen descriptor
    out.write(struct.pack('>HHBBB',
            256, 256, 7, 0, 0))
    # Write the global color table
    for i in range(0, 256):
        out.write(struct.pack('BBB',
                rng.randint(0, 255),
                rng.randint(0, 255),
                rng.randint(0, 255)))
    # Write the image descriptor
    out.write(struct.pack('>HHHHB',
            0, 0, 256, 256, 0))
    # Write the local color table
    for i in range(0, 256):
        out.write(struct.pack('BBB',
                rng.randint(0, 255),
                rng.randint(0, 255),
                rng.randint(0, 255)))
    # Write the image data
    out.write(data)