#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = int.from_bytes(
        rng.read(2), byteorder='big', signed=False) % (2**16 - 1)
    height = int.from_bytes(
        rng.read(2), byteorder='big', signed=False) % (2**16 - 1)
    flags = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0] & 7
    background = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0]
    aspect_ratio = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0]
    out.write(struct.pack('>HHBBB',
                          width, height, flags,
                          background, aspect_ratio))

    # Global color table
    global_color_table_size = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0] & 7
    if global_color_table_size!= 0:
        global_color_table = [bytes([int.from_bytes(
            rng.read(1), byteorder='big', signed=False)])[0]
                              for i in range(global_color_table_size ** 2)]

        out.write(bytes(global_color_table))

    # Loop control block
    loop_control_flags = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0]
    loop_count = None
    if loop_control_flags == 1:
        loop_count = int.from_bytes(
            rng.read(2), byteorder='big', signed=False)
    out.write(loop_control_flags.to_bytes(
        1, byteorder='big', signed=False))
    if loop_count is not None:
        out.write(loop_count.to_bytes(
            2, byteorder='big', signed=False))

    # Image descriptor
    image_left = int.from_bytes(
        rng.read(2), byteorder='big', signed=False) % (2**16 - 1)
    image_top = int.from_bytes(
        rng.read(2), byteorder='big', signed=False) % (2**16 - 1)
    image_width = int.from_bytes(
        rng.read(2), byteorder='big', signed=False) % (2**16 - 1)
    image_height = int.from_bytes(
        rng.read(2), byteorder='big', signed=False) % (2**16 - 1)
    local_color_table_flag = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0] & 1
    interlace_flag = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0] & 1
    sort_flag = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0] & 1
    reserved = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0]
    lct_size = bytes([int.from_bytes(
        rng.read(1), byteorder='big', signed=False)])[0] & 7
    out.write(struct.pack('>HHHHBB',
                          image_left, image_top,
                          image_width, image_height,
                          local_color_table_flag,
                          interlace_flag | sort_flag << 1 | reserved << 4 | lct_size << 5))

    # Local color table
    if lct_size!= 0:
        local_color_table = [bytes([int.from_bytes(
            rng.read(1), byteorder='big', signed=False)])[0]
                             for i in range(lct_size ** 2)]
        out.write(bytes(local_color_table))

    # Image data
    while True