#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # 10 bytes of header
    out.write(b'GIF89a')
    
    # 7 bytes of screen descriptor
    out.write(b'\x00\x00\x00\x00\x00\x00\x00')
    
    # 1 byte of global color table flag
    out.write(b'\x80')
    
    # 3 bytes of background color index
    out.write(bytes([next(rng) & 0xff]))
    
    # 3 bytes of pixel aspect ratio
    out.write(b'\x00\x00\x00')
    
    # 1 byte of global color table size
    out.write(b'\x00')
    
    # 256 * 3 bytes of global color table
    for i in range(256):
        out.write(bytes([next(rng) & 0xff] * 3))
        
    # 1 byte of image separator
    out.write(b',')
    
    # 9 bytes of image descriptor
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # 1 byte of local color table flag
    out.write(b'\x80')
    
    # 3 bytes of local color table size
    out.write(b'\x00\x00\x00')
    
    # 256 * 3 bytes of local color table
    for i in range(256):
        out.write(bytes([next(rng) & 0xff] * 3))
        
    # 1 byte of image data block terminator
    out.write(b';')
