#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # The first 6 bytes are the magic header
    out.write(b'GIF89a')

    # The next 7 bytes contain the logical screen size
    width = rng.read(2)
    height = rng.read(2)
    packed = b'\x00\x00\xff\xff' + width + height
    out.write(struct.pack('<HBBHH', *struct.unpack('>HBBHH', packed)))

    # The next byte is the background color index
    out.write(rng.read(1))

    # The next byte is the pixel aspect ratio
    out.write(rng.read(1))

    # The next block contains global color table data
    # We don't need to write it because we aren't using it
    rng.read(768)

    # The next block is the application extension
    # It has two sub-blocks: one for the application name and one for the comment
    # We don't need to write them because they're not used by any browsers
    rng.read(14)
    rng.read(15)

    # The next block is the image descriptor
    # We only care about the left position and top position
    # The rest of the fields can be ignored
    out.write(b'\x2c\x00\x00\x00\x00')

    # The next block contains image data
    # We don't need to write it because it's not used by any browsers
    rng.read(1)

    # The last block is the trailer
    out.write(b';')
