#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480) -> None:
    # Write the header.
    out.write(b'GIF89a')  # Magic bytes.
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\xF7\x00\x00')  # Global color table flag + background color.
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00')  # Loop count.

    # Write the image data.
    for y in range(height):
        out.write(b'\x2C\x00\x00\x00\x00')  # Image descriptor.
        out.write(struct.pack('<H', width))
        out.write(struct.pack('<H', height - y - 1))
        out.write(b'\x00')  # Local color table flag.
        out.write(b'\x08')  # Minimum code size.
        for x in range((width * 3 + 7) // 8):
            out.write(rng.read(1))
