#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Generate a random GIF file.
    #
    # The format is described in the GIF89a specification at
    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt
    #
    # This function generates a 1x1 pixel image with a single frame.
    #
    # Parameters
    # ----------
    # rng : io.BytesIO
    #     A binary stream containing random bytes.
    # out : io.BytesIO
    #     A binary stream to write the generated GIF file to.

    # Write the header.
    out.write(b'GIF89a')

    # Write the Logical Screen Descriptor.
    out.write(struct.pack('<HBB', 1, 1, 0))

    # Write the Global Color Table.
    out.write(b'\xFF\xFF\xFF')

    # Write the Graphic Control Extension.
    out.write(b'\x21\xF9\x04\x05\x00\x01\x00')

    # Write the Image Descriptor.
    out.write(b'\x2C\x00\x00\x00\x00\x01\x00')

    # Write the Local Color Table.
    out.write(b'\x00')

    # Write the LZW-compressed image data.
    out.write(b'\x02\x02\x4c\x01\x00')

    # Write the trailer.
    out.write(b';')

