#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a random color table.
    colors = [struct.unpack('<BBB', rng.read(3)) for _ in range(256)]

    # Generate a random background color index.
    bg_color_index = int.from_bytes(rng.read(1), 'little') % 256

    # Generate a random pixel aspect ratio.
    pixel_aspect_ratio = int.from_bytes(rng.read(1), 'little') % 8 + 1

    # Generate a random image width and height.
    img_width = int.from_bytes(rng.read(2), 'little') % 400 + 1
    img_height = int.from_bytes(rng.read(2), 'little') % 400 + 1

    # Write the logical screen descriptor to the output stream.
    out.write(struct.pack('<HHBB', img_width, img_height, pixel_aspect_ratio, 0))

    # Write the global color table to the output stream.
    out.write(struct.pack('<BB', 7, 0x80 | bg_color_index))
    for c in colors:
        out.write(struct.pack('<BBB', *c))

    # Write the application extension block to the output stream.
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', 0))

    # Write the graphic control extension to the output stream.
    out.write(b'\x21\xf9\x04\x01\x00\x00\x00')

    # Write the image descriptor to the output stream.
    out.write(b'\x2c\x00\x00\x00\x00')

    # Write the local color table to the output stream.
    out.write(struct.pack('<BB', 7, 0x80 | bg_color_index))
    for c in colors:
        out.write(struct.pack('<BBB', *c))

    # Write the image data to the output stream.
    out.write(b'\x00\x00')
    for y in range(img_height):
        for x in range(img_width):
            out.write(struct.pack('<B', int.from_bytes(rng.read(1), 'little') % 256))

    # Write the trailer to the output stream.
    out.write(b'\x3b')
