#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # 8-bit background color index
    bg = rng.read(1)

    # 8-bit aspect ratio
    ar = rng.read(1)

    # 8-bit global color table flag (0x80)
    gctf = rng.read(1)

    # 7-bit global color table size (0x00)
    gcts = rng.read(1)

    # 4-bit resolution
    res = rng.read(1)

    # 3-bit sort flag (0x00)
    sf = rng.read(1)

    # 3-bit color table size (0x00)
    cts = rng.read(1)

    # Write the header
    out.write(header)

    # Write the width and height
    out.write(w)
    out.write(h)

    # Write the background color index
    out.write(bg)

    # Write the aspect ratio
    out.write(ar)

    # Write the global color table flag
    out.write(gctf)

    # Write the global color table size
    out.write(gcts)

    # Write the resolution
    out.write(res)

    # Write the sort flag
    out.write(sf)

    # Write the color table size
    out.write(cts)

    # Write the image descriptor
    idd = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\xff\x00'
    out.write(idd)

    # Write the local color table
    lct = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x