#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(pow(2, int.from_bytes(color, 'big'))):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')
        out.write(bytes([r,g,b]))

    # Generate and write the graphics control extension
    block = bytes([0x21, 0xf9, 0x04, 0x05, 0x00, 0xff, 0xff, 0xff])
    out.write(block)

    # Generate and write the image descriptor
    left   = (int.from_bytes(rng.read(1), 'big') % width)
    top    = (int.from_bytes(rng.read(1), 'big') % height)
    width  = (int.from_bytes(rng.read(1), 'big') % width) + 1
    height = (int.from_bytes(rng.read(1), 'big') % height) + 1
    flags  = bytes([0x00])
    out.write(bytes([0x2c]))
    out.write((left).to_bytes(2,'big'))
    out.write((top).to_bytes(2,'big'))
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)

    # Generate and write the local color table
    for i in range(pow(2, int.from_bytes(pxsize, 'big'))):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')
        out.write(bytes([r,g,b]))

    # Generate and write the image data
    for y in range(height):
        # Encode the row of pixels
        encoded = []
        for x in range(width):
            pixel = int.from_bytes(rng.read(1), 'big')
            if len(encoded) == 0 or pixel != encoded[-1][-1]:
                encoded.append(''.join(chr(pixel)))
            else:
                encoded[-1] += chr(pixel)
        # Write the encoded characters
        for e in encoded:
            out.write(bytes([len(e)]))
            for c in e:
                out.write(bytes([ord(c)]))