#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# given RNG.
def generate(rng: BinaryIO, out: BinaryIO):
    header = b'GIF89a'

    # width and height
    w = rng.read(2)
    h = rng.read(2)
    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    # image descriptor
    out.write(header + w + h + packed +
             bytes([bg]) + bytes([ar]))

    # LZW minimum code size
    mincodesize = rng.read(1)[0]
    out.write(bytes([mincodesize]))

    # image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)