#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # First, we need to write a GIF header.
    out.write(bytes([71, 73, 70]))

    # Next, we need to write the logical screen descriptor.
    out.write(struct.pack('<HHBBB', 256, 256, 8, 4, 0))

    # Then, we need to write the global color table.
    for i in range(0, 256):
        out.write(struct.pack('BBB', i, i, i))

    # We also need to write the graphic control extension.
    out.write(bytes([0x21, 0xf9, 0x04, 0x05, 0, 0, 0, 0, 0]))

    # Finally, we need to write the image descriptor.
    out.write(struct.pack('<HHHBB', 0, 0, 256, 256, 0))

    # The next byte is the LZW minimum code size.
    lzwmin = rng.read(1)[0]
    out.write(lzwmin)

    # Now we need to calculate how many bytes of data we'll be writing.
    datalen = (2 ** (lzwmin + 1)) * 3 - 1
    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)