#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Write the screen width and height to the output stream.
    out.write(struct.pack('<HH', 128, 128))

    # Write the background color index to the output stream.
    out.write(b'\x00')

    # Write the pixel aspect ratio to the output stream.
    out.write(b'\x00')

    # Write the global color table flag to the output stream.
    out.write(b'\x80')

    # Write the color resolution to the output stream.
    out.write(b'\x07')

    # Write the sort flag to the output stream.
    out.write(b'\x00')

    # Write the size of the global color table to the output stream.
    out.write(b'\x07')

    # Write the global color table to the output stream.
    for i in range(3 * 2**(7 - 1)):
        out.write(struct.pack('BBB', i, i, i))

    # Write the graphics control extension block header to the output stream.
    out.write(b'\x21\xf9\x04\x05\xff\xff\xff\xff\x00')

    # Write the delay time to the output stream.
    out.write(b'\x00\x00')

    # Write the transparent color index to the output stream.
    out.write(b'\x00')

    # Write the terminator to the output stream.
    out.write(b'\x00')

    # Write the image descriptor to the output stream.
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table flag to the output stream.
    out.write(b'\x80')

    # Write the image width to the output stream.
    out.write(b'\x00\x80')

    # Write the image height to the output stream.
    out.write(b'\x00\x80')

    # Write the local color table flag to the output stream.
    out.write(b'\x07')

    # Write the local color table to the output stream.
    for i in range(3 * 2**(7 - 1)):
        out.write(struct.pack('BBB', i, i, i))

    # Write the LZW minimum code size to the output stream.
    out.write(b'\x08')

    # Read the image data length from the input stream.
    datalen = rng.read(4)

    # Write the image data length to the output stream.
    out.write(datalen)

    # Copy the image data from the input stream to the output stream.
    while True:
        buf = rng.read(4096)
        if not buf: break
        out.write(buf)
