#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Write the header
    out.write(b'GIF89a')

    # Write the screen descriptor
    out.write(struct.pack('<HHBB', 1024, 768, 0xF, 0))

    # Write the global color table
    for i in range(1 << 4):
        out.write(bytes([i * 5]))

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x01\xff\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(1 << 4):
        out.write(bytes([i * 5]))

    # Write the pixel data
    for y in range(768):
        for x in range(1024):
            out.write(bytes([rng.read(1)[0] & 0xF]))

    # Write the trailer
    out.write(b';')
