#!/usr/bin/env python3

import os
import sys
from pathlib import Path
import argparse
import subprocess
import shutil
import json
import re
import time
import datetime
import logging
import tempfile
import numpy as np
import pandas as pd
from collections import defaultdict, Counter
from Bio.SeqIO.FastaIO import SimpleFastaParser
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio.Blast.Applications import NcbiblastnCommandline
from Bio.Blast import NCBIXML
from Bio.SearchIO import read
from Bio.SeqFeature import FeatureLocation
from Bio.Data.CodonTable import TranslationError
from Bio.SeqUtils import GC
from Bio.SeqUtils.ProtParam import ProteinAnalysis
from Bio.Align import MultipleSeqAlignment
from Bio.Align.Applications import MuscleCommandline
from Bio.Phylo.PAML import codeml
from Bio.Phylo.PAML import Model
from Bio.Phylo.PAML import Control
from Bio.Phylo.PAML import Tree
from Bio.Phylo.PAML import Phylip
from Bio.Phylo.PAML import CodonFreqs
from Bio.Phylo.PAML import Utilities
from Bio.Phylo.PAML._paml import _Paml
from Bio.Phylo.PAML._paml import _PamlError
from Bio.Phylo.PAML._paml import _PamlInternalError
from Bio.Phylo.PAML._paml import _PamlNotEnoughDataError
from Bio.Phylo.PAML._paml import _PamlTooManyStatesError
from Bio.Phylo.PAML._paml import _PamlNegativeSiteRatesError
from Bio.Phylo.PAML._paml import _PamlRateHeterogeneityError
from Bio.Phylo.PAML._paml import _PamlModelSelectionError
from Bio.Phylo.PAML._paml import _PamlMaximumIterationsReachedError
from Bio.Phylo.PAML._paml import _PamlConvergenceError
from Bio.Phylo.PAML._paml import _PamlLikelihoodError
from Bio.Phylo.PAML._paml import _PamlParameterOutOfBoundsError
from Bio.Phylo.PAML._paml import _PamlInsufficientNSitesError
from Bio.Phylo.PAML._paml import _PamlNoVariationError
from Bio.Phylo.PAML._paml import _PamlUnknownError
from Bio.Phylo.PAML._paml import _PamlMemoryError
from Bio.Phylo.PAML._paml import _PamlFileFormatError
from Bio.Phylo.PAML._paml import _PamlMiscellaneousError
from Bio.Phylo.PAML._paml import _PamlSaturatedError
from Bio.Phylo.PAML._paml import _PamlEigenvalueError
from Bio.Phylo.PAML._paml import _PamlMissingInformationError
from Bio.Phylo.PAML._paml import _PamlBadDirectoryPathError
from Bio.Phylo.PAML._paml import _PamlBadFileNameError
from Bio.Phylo.PAML._paml import _PamlBadSequenceError
from Bio.Phylo.PAML._paml import _PamlBadTreeError
from Bio.Phylo.PAML._paml import _PamlBadFrequenciesError
from Bio.Phylo.PAML._paml import _PamlBadBaseFrequenciesError
from Bio.Phylo.PAML._paml import _PamlBadCodonFrequenciesError
from Bio.Phylo.PAML._paml import _PamlBadCodonFreqsAndDataError
from Bio.Phylo.PAML._paml import _PamlBadOmegaError
from Bio.Phylo.PAML._paml import _PamlBadKappaOrOmegaError
from Bio.Phylo.PAML._paml import _PamlBadAlphaError
from Bio.Phylo.PAML._paml import _PamlBadGammaError
from Bio.Phylo.PAML._paml import _PamlBadPInvarError
