#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    out.write(struct.pack('<HBB', width, height, 0xF7))
    
    # Global color table (16 colors)
    for i in range(256 // 4):
        r = g = b = i * 4
        out.write(bytes([r, g, b]))
        
    # Graphics control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\xff\x00')
    
    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # Local color table (16 colors)
    for i in range(256 // 4):
        r = g = b = i * 4
        out.write(bytes([r, g, b]))
        
    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([y % 256, x % 256]))
