#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # get some of our data into the file before writing the rest.
    out.write(struct.pack('<6sHHB', b'GIF89a', 0xffff, 0xffff, 0))

    while True:
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)