#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate some random bytes for the length of the image data
    # and write them to the output stream
    datalen = int.from_bytes(
        rng.read(2), 'little', signed=False
    )
    out.write(struct.pack('<H', datalen))

    # Generate some random bytes for the image data
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)