#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate some random bytes for the length of the image data
    # and write them to the output stream
    datalen = int.from_bytes(
            rng.read(3), byteorder='big', signed=False) + 1
    out.write(struct.pack('<H', datalen))

    # Generate some random bytes for the width and height
    # and write them to the output stream
    width = int.from_bytes(
            rng.read(2), byteorder='big', signed=False) + 1
    height = int.from_bytes(
            rng.read(2), byteorder='big', signed=False) + 1
    out.write(struct.pack('<HH', width, height))

    # Write the global color table
    out.write(b'\x00\xff\xff\xff')

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x05\x00\x00\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    out.write(b'\x00\xff\xff\xff')

    # Write the image data
    out.write(rng.read(datalen))

    # Write the trailer
    out.write(b'\x3b')
