#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(2**(pxsize[0] & 0x07)):
        out.write(bytes([int.from_bytes(rng.read(1), 'big')]))
        out.write(bytes([int.from_bytes(rng.read(1), 'big')]))
        out.write(bytes([int.from_bytes(rng.read(1), 'big')]))

    # Generate and write the image descriptor
    xpos = int.from_bytes(rng.read(2), 'big')
    ypos = int.from_bytes(rng.read(2), 'big')
    w    = int.from_bytes(rng.read(2), 'big')
    h    = int.from_bytes(rng.read(2), 'big')
    flags = bytes([0x80 | ((w >> 12) & 0x0F)])
    out.write(b'\x2C')
    out.write((xpos).to_bytes(2,'big'))
    out.write((ypos).to_bytes(2,'big'))
    out.write((w).to_bytes(2,'big'))
    out.write((h).to_bytes(2,'big'))
    out.write(flags)

    # Generate and write the local color table
    for i in range(2**(pxsize[0] & 0x07)):
        out.write(bytes([int.from_bytes(rng.read(1), 'big')]))
        out.write(bytes([int.from_bytes(rng.read(1), 'big')]))
        out.write(bytes([int.from_bytes(rng.read(1), 'big')]))

    # Generate and write the pixel data
    for _ in range(w*h):
        out.write(bytes([int.from_bytes(rng.read(1), 'big')]))

    # Generate and write the trailer
    out.write(b';')
