#!/usr/bin/env python3

import os
import sys
import struct

if __name__ == '__main__':
    try:
        infile = open(sys.argv[1], 'rb')
        outfile = open(sys.argv[2], 'wb')
    except IndexError:
        print("Usage: {} <input file> <output file>".format(sys.argv[0]))
        exit(1)
    except FileNotFoundError as e:
        print("File not found:", e)
        exit(1)

    generate_random_gif(infile, outfile)

    infile.close()
    outfile.close()

# Generates a GIF image by copying the image data from an input stream to
# an output stream. The image data is generated randomly using the given
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    datalen = struct.pack('>H', int.from_bytes(
            rng.read(2), byteorder='big'))

    # Write the image data length to the output stream.
    out.write(datalen)

    # Copy the image data from the input stream to the output stream.
    while True:
        buf = rng.read(4096)
        if not buf: break
        out.write(buf)
