#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given dimensions.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    out.write(struct.pack('<HBB', width, height, 0xF7))
    
    # Global color table
    for i in range(256):
        r = (i >> 5) & 0x1f
        g = (i >> 2) & 0x1f
        b = (i << 3) & 0x1f
        
        out.write(bytes([r * 8, g * 8, b * 8]))
    
    # Image block
    out.write(b'\x2C\x00\x00\x00\x00')
    out.write(struct.pack('B', width - 1))
    out.write(struct.pack('B', height - 1))
    out.write(b'\x00')
    
    # Local color table
    for i in range(256):
        r = (i >> 5) & 0x1f
        g = (i >> 2) & 0x1f
        b = (i << 3) & 0x1f
        
        out.write(bytes([r * 8, g * 8, b * 8]))
    
    # Image data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', ((y + x) % 256)))
