#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate some random bytes for the length of the image data
    # and write them to the output stream
    datalen = int.from_bytes(
            rng.read(3), byteorder='big', signed=False) + 1
    out.write(struct.pack('>H', datalen))

    # Generate some random bytes for the width and height of the image
    # and write them to the output stream
    width = int.from_bytes(
            rng.read(2), byteorder='big', signed=False) + 1
    height = int.from_bytes(
            rng.read(2), byteorder='big', signed=False) + 1
    out.write(struct.pack('>HH', width, height))

    # Generate some random bytes for the color table size and write
    # them to the output stream
    colortable_size = int.from_bytes(
            rng.read(1), byteorder='big', signed=False) + 1
    out.write(struct.pack('B', colortable_size))

    # Generate some random bytes for the background color index and
    # write them to the output stream
    backcolorindex = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', backcolorindex))

    # Generate some random bytes for the aspect ratio and write
    # them to the output stream
    aspectratio = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', aspectratio))

    # Generate some random bytes for the flags and write
    # them to the output stream
    flags = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', flags))

    # Generate some random bytes for the transparent color index and
    # write them to the output stream
    transpcolorindex = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', transpcolorindex))

    # Generate some random bytes for the delay time and
    # write them to the output stream
    delaytime = int.from_bytes(
            rng.read(2), byteorder='big', signed=False)
    out.write(struct.pack('>H', delaytime))

    # Generate some random bytes for the transparent color index and
    # write them to the output stream
    transpcolordisposal = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', transpcolordisposal))

    # Generate some random bytes for the block terminator and
    # write them to the output stream
    blockterminator = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', blockterminator))

    # Generate some random bytes for the extension introducer and
    # write them to the output stream
    extensionintroducer = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', extensionintroducer))

    # Generate some random bytes for the graphic control label and
    # write them to the output stream
    graphiccontrollabel = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', graphiccontrollabel))

    # Generate some random bytes for the block size and
    # write them to the output stream
    blocksizegraphiccontrol = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', blocksizegraphiccontrol))

    # Generate some random bytes for the flags and
    # write them to the output stream
    flagsgraphiccontrol = int.from_bytes(
            rng.read(1), byteorder='big', signed=False)
    out.write(struct.pack('B', flagsgraphiccontrol))

    # Generate some random bytes for the delay time and
    # write them to the output stream
    delaytimegraphiccontrol = int.from_bytes(
            r