#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    # global color table flag
    gctf = rng.read(1)[0] >> 7

    # color table size
    cts = 2 ** ((rng.read(1)[0] & 0x7) + 1)

    # color table
    colors = [None] * cts
    for i in range(cts):
        colors[i] = rng.read(3)

    # image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00'

    # local color table flag
    lctf = rng.read(1)[0] >> 7

    # interlaced flag
    ifl = rng.read(1)[0] >> 6

    # sorted flag
    sfl = rng.read(1)[0] >> 5

    # reserved
    res = rng.read(1)[0] >> 4

    # local color table size
    lcts = 2 ** ((rng.read(1)[0] & 0x7) + 1)

    # image data
    img = []
    for y in range(h):
        row = []
        for x in range(w):
            idx = rng.read(1)[0] % len(colors)
            row.append(idx)
        img.append(row)

    # write the header
    out.write(header)
    out.write(w)
    out.write(h)
    out.write(packed)
    out.write(bytes([bg]))
    out.write(bytes([ar]))
    out.write(bytes([gctf]))
    out.write(bytes([cts - 1]))
    for color in colors:
        out.write(color)
    out.write(idesc)
    out.write(bytes([lctf]))
    out.write(bytes([ifl]))
    out.write(bytes([sfl]))
    out.write(bytes([res]))
    out.write(bytes([lcts - 1]))
    for row in img:
        out.write(struct.pack('B', len(row)))
        out.write(bytes(row))
    out.write(b';')

