#!/usr/bin/env python3

import random

def generate_random_gif(rng, out):
    """Generate a random GIF image and write it to `out`.

    The GIF is generated by reading three bytes from `rng` and using them as
    the length of the data section.  This means that the data section will be
    between 0 and 2^24 bytes long.

    Args:
        rng (RandomIOBase): A source of randomness.
        out (BinaryIO): Where to write the GIF.
    """
    # an unbounded data section.  We'll just write the header first so we can
    # get some of our data into the file before writing the rest.
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)