#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    ncolors = int.from_bytes(rng.read(1), 'big')
    if ncolors == 0:
        raise Exception('No colors!')
    else:
        out.write(rng.read(ncolors * 3))

    # Generate and write the application extension block
    appname = b'\x41\x50\x50\x4C\x4F\x43\x4B\x2E\x43\x6F\x6D'
    appdata = rng.read(12)
    out.write(b'\x21')
    out.write(b'\xFF')
    out.write(appname)
    out.write(appdata)

    # Generate and write the image data
    while True:
        # Read the next byte of input
        b = rng.read(1)
        if len(b) == 0:
            break
        b = b[0]

        # Encode the byte as LZW
        encoded = []
        prev = ''
        while b != 0:
            # Get the next bit
            bit = b & 1
            b >>= 1

            # Add this bit to the current code
            curr = prev + str(bit)

            # If we have an existing code with this prefix,
            # then continue building it
            if curr in encoded:
                prev = curr
                continue

            # Otherwise, add the new code to the list
            encoded.append(curr)
            # Update the previous character
            prev = ''.join(encoded[-1])

        # Write the encoded characters
        for e in encoded:
            out.write(bytes([len(e)]))
            for c in e:
                out.write(bytes([ord(c)]))