#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int = 1024, height: int = 768):
    # Write GIF header
    out.write(b'GIF89a')

    # Write screen descriptor
    out.write(struct.pack('<HHBB', width, height, 0xF & rng.read(1)[0], 0))

    # Write global color table
    for i in range(0xFF + 1):
        out.write(struct.pack('<BBB', rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]))

    # Write image descriptor
    out.write(struct.pack('<HHHBB', 0, 0, width, height, 0))

    # Write local color table
    for i in range(0xFF + 1):
        out.write(struct.pack('<BBB', rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]))

    # Write LZW-compressed data
    while True:
        code = rng.read(1)[0]
        if not code:
            break
        out.write(code)
