#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    p = rng.read(1)[0] & 7

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0]

    # write the header
    out.write(header + w + h + bytes([(r << 4) | g]) + bytes([(b << 4) | p]) + bytes([bg]) + bytes([ar]))

    # write the global color table
    for _ in range(2 ** (p + 1)):
        out.write(rng.read(3))

    # write an image descriptor
    id = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\xff\x00'
    out.write(id)

    # write the local color table
    for _ in range(2 ** (r + 1)):
        out.write(rng.read(3))

    # write the LZW compressed data
    while True:
        code = rng.read(1)[0]
        if code == 0:
            break
        else:
            out.write(bytes([code]))
