#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb 27 16:49:50 2021

@author: jonas
"""
import numpy as np
from scipy.stats import norm, gamma, multivariate_normal
import matplotlib.pyplot as plt
import seaborn as sns
sns.set()

def get_data(n=100):
    """Generates data for the example."""
    
    # Generate some random data
    x = np.random.randint(-10, 10, size=(n,))
    y = np.random.randint(-10, 10, size=(n,))
    
    # Define a function that maps x and y to z
    def f(x,y):
        return (x**2 + y**2) / 10
        
    # Apply the function to generate z
    z = f(x,y)
    
    # Create a dictionary with all three variables
    data = {'x': x, 'y': y, 'z': z}
    
    return data

def plot_data(data):
    """Plots the data in a scatterplot."""
    
    # Extract the variables from the dictionary
    x = data['x']
    y = data['y']
    z = data['z']
    
    # Plot the data using a scatterplot
    fig, ax = plt.subplots(figsize=(8,8))
    ax.scatter(x, y, c=z, marker='o', edgecolors='none')
    
    # Set the limits of the axes
    ax.set_xlim([-10, 10])
    ax.set_ylim([-10, 10])
    
    # Add a colorbar
    cb = plt.colorbar(ax.collections[0], shrink=0.8)
    cb.set_label('$z$')
    
    return fig, ax

def plot_prior(mu, cov, ax=None):
    """Plots the prior distribution."""
    
    if ax is None:
        fig, ax = plt.subplots(figsize=(8,8))
    else:
        fig = None
    
    # Get the mean and covariance matrix of the prior
    mu_prior = [mu[0] - 2 * np.sqrt(cov[0][0]), mu[1] - 2 * np.sqrt(cov[1][1])]
    cov_prior = [[cov[0][0], 0], [0, cov[1][1]]]
    
    # Draw an ellipse representing the prior
    ell = mpl.patches.Ellipse(xy=mu_prior, width=4 * np.sqrt(cov_prior[0][0]), height=4 * np.sqrt(cov_prior[1][1]), angle=0, facecolor='gray', alpha=0.5)
    ax.add_artist(ell)
    
    return fig, ax

def plot_posterior(mu, cov, ax=None):
    """Plots the posterior distribution."""
    
    if ax is None:
        fig, ax = plt.subplots(figsize=(8,8))
    else:
        fig = None
    
    # Draw an ellipse representing the posterior
    ell = mpl.patches.Ellipse(xy=mu, width=4 * np.sqrt(cov[0][0]), height=4 * np.sqrt(cov[1][1]), angle=0, facecolor='red', alpha=0.5)
    ax.add_artist(ell)
    
    return fig, ax

def plot_predictive(mu, cov, ax=None):
    """Plots the predictive distribution."""
    
    if ax is None:
        fig, ax = plt.subplots(figsize=(8,8))
    else:
        fig = None
    
    # Get the mean and covariance matrix of the prior
    mu_pred = [mu[0] + 2 * np.sqrt(cov[0][0]), mu[1] + 2 * np.sqrt(cov[1][1])]
    cov_pred = [[cov[0][0], 0], [0