#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(0, int.from_bytes(rng.read(1), 'big')):
        red   = int.from_bytes(rng.read(1), 'big')
        green = int.from_bytes(rng.read(1), 'big')
        blue  = int.from_bytes(rng.read(1), 'big')
        out.write(bytes([red,green,blue]))

    # Generate and write the image descriptor
    left   = (int.from_bytes(rng.read(1), 'big') % width)
    top    = (int.from_bytes(rng.read(1), 'big') % height)
    width  = (int.from_bytes(rng.read(1), 'big') % (width - left)) + 1
    height = (int.from_bytes(rng.read(1), 'big') % (height - top)) + 1
    flags  = bytes([0x80])
    lct    = bytes([0x00])
    out.write(b'\x2c')
    out.write((left).to_bytes(2,'big'))
    out.write((top).to_bytes(2,'big'))
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(lct)

    # Generate and write the local color table
    for i in range(0, int.from_bytes(rng.read(1), 'big')):
        red   = int.from_bytes(rng.read(1), 'big')
        green = int.from_bytes(rng.read(1), 'big')
        blue  = int.from_bytes(rng.read(1), 'big')
        out.write(bytes([red,green,blue]))

    # Generate and write the image data
    for y in range(0, height):
        for x in range(0, width):
            index = int.from_bytes(rng.read(1), 'big')
            out.write(bytes([index]))
