#!/usr/bin/env python3
import sys
import struct

def main() -> None:
    """
    This program reads GIF images from standard input and writes them to
    standard output. It is intended for use in conjunction with the
    `gifgen` program.
    """

    # Read the entire input file into memory.
    data = sys.stdin.buffer.read()

    # Create a new random access object that can read from the input data.
    rng = io.BytesIO(data)

    # Create a new binary output stream that will write to standard output.
    out = sys.stdout.buffer

    # Process each frame in the GIF image.
    while True:
        try: process_frame(rng, out)
        except EOFError: break

def process_frame(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    datalen = struct.pack('>H', int.from_bytes(
            rng.read(2), byteorder='big'))

    # Write the image data length to the output stream.
    out.write(datalen)

    # Copy the image data from the input stream to the output stream.
    while True:
        buf = rng.read(4096)
        if not buf: break
        out.write(buf)
