#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    # global color table flag
    gctf = rng.read(1)[0] >> 7

    # color table size
    cts = rng.read(1)[0] & 7

    # color table
    if gctf == 1:
        for i in range(2 ** (cts + 1)):
            out.write(struct.pack('BBB', *[rng.read(1)[0] for _ in range(3)]))

    # image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00'

    # local color table flag
    lctf = rng.read(1)[0] >> 7

    # interlaced flag
    ilcf = rng.read(1)[0] >> 6

    # sort flag
    srtf = rng.read(1)[0] >> 5

    # reserved
    resv = rng.read(1)[0] >> 4

    # local color table size
    lcts = rng.read(1)[0] & 7

    # minimum code length
    mcl = rng.read(1)[0]

    # LZW data
    while True:
        try:
            out.write(rng.read(1))
        except EOFError as e:
            break
