#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def gen_gif(out: BinaryIO, width: int = 1024, height: int = 768) -> None:
    # Header
    out.write("GIF89a".encode())

    # Logical screen descriptor
    out.write(struct.pack("<HHBBB", width, height, 0xF0, 0x00, 0x00))

    # Global color table (black and white only).
    for i in range(256):
        if i < 128:
            out.write(b"\xFF\xFF\xFF")
        else:
            out.write(b"\x00\x00\x00")

    # Image descriptor
    out.write(struct.pack("<BBHHHBB", 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

    # Local color table (black and white only).
    for i in range(256):
        if i < 128:
            out.write(b"\xFF\xFF\xFF")
        else:
            out.write(b"\x00\x00\x00")

    # Image data
    for y in range(height):
        for x in range(width):
            if ((y // 16 + x // 16) % 2 == 0):
                out.write(b"\x00")
            else:
                out.write(b"\xFF")

