#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    datalen = struct.pack('>H', int.from_bytes(
            rng.read(2), byteorder='big'))

    # Write the image data size to the output stream.
    out.write(datalen)

    # Write the screen descriptor block to the output stream.
    out.write(struct.pack('<HHHHB',
        75, 75, 0, 0, 0))

    while True:
        buf = rng.read(4096)
        if not buf: break
        out.write(buf)
