#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def main():
    if len(sys.argv) < 2:
        print('Usage:', sys.argv[0], '<input file>', file=sys.stderr)
        return -1

    with open(sys.argv[1], 'rb') as f:
        data = f.read()

        # Check magic number
        magic_number, = struct.unpack('<I', data[:4])
        if magic_number!= 0x56789ABC:
            raise ValueError('Invalid magic number!')

        # Get size of the image and offset to the pixel data
        width, height, offset = struct.unpack('<III', data[4:])

        # Read the pixels from the input file
        with open(os.path.splitext(sys.argv[1])[0] + '.png', 'wb') as outf:
            outf.write(data[offset:])

            # Write PNG header (https://en.wikipedia.org/wiki/Portable_Network_Graphics#File_format)
            outf.write(b'\x89PNG\r\n\x1a\n')
            outf.write(struct.pack('<I', 0))    # Placeholder for chunk length
            outf.write(b'IHDR')                  # Image Header Chunk
            outf.write(struct.pack('<IIII', width, height, 8, 2))   # Width, Height, Bit depth, Color type (RGB)
            outf.write(b'\x00\x00\x00\x00')      # Compression method, Filter method, Interlace method
            outf.seek(-4, os.SEEK_CUR)          # Go back to write the correct value for the chunk length
            outf.write(struct.pack('<I', outf.tell()))     # Write the actual chunk length

            # Write IDAT chunk
            outf.write(b'IDAT')                  # Image Data Chunk
            outf.write(struct.pack('<I', 0))    # Placeholder for chunk length
            compressor = zlib.compressobj()
            compressed = compressor.compress(data[offset:])
            compressed += compressor.flush()
            outf.write(compressed)
            outf.seek(-4, os.SEEK_CUR)          # Go back to write the correct value for the chunk length
            outf.write(struct.pack('<I', outf.tell()))     # Write the actual chunk length

            # Write IEND chunk
            outf.write(b'IEND')                  # Image End Chunk
            outf.write(struct.pack('<I', 0))    # Placeholder for chunk length
            outf.seek(-4, os.SEEK_CUR)          # Go back to write the correct value for the chunk length
            outf.write(struct.pack('<I', outf.tell()))     # Write the actual chunk length


