#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(2), 'big') + 1
    data = rng.read(datalen)
    while len(data)!= datalen:
        data += rng.read(datalen - len(data))

    # Write the header and length of the image data to the output stream
    out.write(header)
    out.write(struct.pack('>H', len(data)))
    # Write the image data itself
    out.write(data)

