#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b"GIF89a")

    # Logical screen descriptor
    width = 2**16 * int.from_bytes(rng.read(1), "big") + \
            int.from_bytes(rng.read(1), "big")
    height = 2**16 * int.from_bytes(rng.read(1), "big") + \
             int.from_bytes(rng.read(1), "big")
    flags = b"\x07\x00" if (width >= 256 and height >= 256) else b"\x00\x00"
    bg_color = bytes([int.from_bytes(rng.read(1), "big")])
    pixel_aspect_ratio = b"\xff"
    out.write(struct.pack("<HHBBB", width, height, flags[0], flags[1],
                          bg_color[0]))
    out.write(pixel_aspect_ratio)

    # Global color table
    for _ in range(2**(flags[1] + 1)):
        out.write(bytes([int.from_bytes(rng.read(1), "big")]))

    # Image descriptor
    left = 2**16 * int.from_bytes(rng.read(1), "big") + \
           int.from_bytes(rng.read(1), "big")
    top = 2**16 * int.from_bytes(rng.read(1), "big") + \
          int.from_bytes(rng.read(1), "big")
    width = 2**16 * int.from_bytes(rng.read(1), "big") + \
            int.from_bytes(rng.read(1), "big")
    height = 2**16 * int.from_bytes(rng.read(1), "big") + \
             int.from_bytes(rng.read(1), "big")
    flags = b"\x0c\x00"
    local_color_table_flag = b"\x01"
    interlace_flag = b"\x00"
    sort_flag = b"\x00"
    reserved = b"\x00\x00"
    lct_size = 2**(flags[1] + 1)
    out.write(struct.pack("<HHHHBBBBB", left, top, width, height, flags[0],
                          flags[1], local_color_table_flag[0],
                          interlace_flag[0], sort_flag[0], reserved[0],
                          reserved[1]))

    # Local color table
    for _ in range(lct_size):
        out.write(bytes([int.from_bytes(rng.read(1), "big")]))

    # Table based image data
    while True:
        block_size = int.from_bytes(rng.read(1), "big")

        if block_size == 0: break

        out.write(block_size.to_bytes(1, byteorder="big"))
        out.write(rng.read(block_size))

