#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate some random bytes for the length of the image data
    # and write them to the output stream
    datalen = int.from_bytes(
            rng.read(3), byteorder='big', signed=False)
    out.write(datalen.to_bytes(3, 'big'))

    # Write the palette color table (256 colors)
    for i in range(0, 768):
        out.write(struct.pack('B', rng.read(1)[0]))

    # Write the global color table flag
    out.write(b'\x87\x00')

    # Write the background color index
    out.write(rng.read(1))

    # Write the pixel aspect ratio
    out.write(b'\x00')

    # Write the image descriptor
    out.write((0).to_bytes(4, 'little'))
    out.write((0).to_bytes(4, 'little'))
    out.write(((datalen >> 8) & 0xff).to_bytes(1, 'little'))
    out.write((datalen & 0xff).to_bytes(1, 'little'))
    out.write((0).to_bytes(1, 'little'))
    out.write((0).to_bytes(1, 'little'))
    out.write((0).to_bytes(1, 'little'))

    # Write the image data
    data = b''
    while len(data)!= datalen:
        data += rng.read(min(datalen - len(data), 256))

    out.write(data)