#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(pow(2, int.from_bytes(color, 'big'))):
        r = int.from_bytes(rng.read(1),'big')
        g = int.from_bytes(rng.read(1),'big')
        b = int.from_bytes(rng.read(1),'big')
        out.write(r.to_bytes(1,'big'))
        out.write(g.to_bytes(1,'big'))
        out.write(b.to_bytes(1,'big'))

    # Generate and write the image descriptor
    left   = bytes([0x00])
    top    = bytes([0x00])
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    out.write(left)
    out.write(top)
    out.write(width.to_bytes(2,'big'))
    out.write(height.to_bytes(2,'big'))
    out.write(flags)

    # Generate and write the local color table
    if pow(2, int.from_bytes(pxsize, 'big')) == 2**24:
        for i in range(256):
            r = int.from_bytes(rng.read(1),'big')
            g = int.from_bytes(rng.read(1),'big')
            b = int.from_bytes(rng.read(1),'big')
            out.write(r.to_bytes(1,'big'))
            out.write(g.to_bytes(1,'big'))
            out.write(b.to_bytes(1,'big'))

    # Generate and write the graphics control extension
    label      = bytes([0x21])
    blocksize  = bytes([0xF9])
    size       = bytes([0x04])
    packed     = bytes([0x0B])
    delaytime  = bytes([(int.from_bytes(rng.read(1), 'big') % 256)])
    transindex = bytes([0x00])
    terminator = bytes([0x00])
    out.write(label)
    out.write(blocksize)
    out.write(size)
    out.write(packed)
    out.write(delaytime)
    out.write(transindex)
    out.write(terminator)

    # Generate and write the plain text extension
    label        = bytes([0x21])
    blocksize    = bytes([0x0C])
    size         = bytes([0x0F])
    authcode     = bytes([0x8B])
    title        = bytes([ord('T'), ord('i'), ord('t'), ord('l'), ord('e')])
    author       = bytes([ord('A'), ord('u'), ord('t'), ord('h'), ord('o'), ord('r')])
    copyright    = bytes([ord('C'), ord('o'), ord('p'), ord('y'), ord('r'), ord('i'), ord('g'), ord('h'), ord('t')])
    comment      = bytes([ord('C'), ord('o'), ord('m'), ord('m'), ord('e'), ord('n'), ord('t')])
    terminator   = bytes([0x00])
    out.write(label)
    out.write(blocksize)
    out.write(size)
    out.write(authcode)
    out.write(title)
    out.write(author)
    out.write(copyright)
    out.write(comment)
    out.write(terminator)

    # Generate and write the application extension
    label        = bytes([0x21])
    blocksize    = bytes([0xFF])
    size         = bytes([0x0B])
    appid        = bytes([ord('S'), ord('M'), ord('A'), ord('R'), ord('T'), ord(' '), ord('I'), ord('M'), ord('A'), ord('G'), ord('E')])
    authcode     = bytes([0x8B])
    version      = bytes([0x03])
    terminator   = bytes([0x00])
    out.write(label)
    out.write(blocksize)
    out.write(size)
    out.write(appid)
    out.write(authcode)
    out.write(version)
    out.write(terminator)

    # Generate and write the image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([int.from_bytes(rng.read(1), 'big')] * int.from_bytes(pxsize, 'big')))


if