#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a' # 6 bytes
    width = rng.randint(1024, 512*1024) # 2 bytes
    height = rng.randint(1024, 512*1024) # 2 bytes
    color_table_size = rng.randint(0, 256) # 1 byte
    bg_color_index = rng.randint(0, 256) # 1 byte
    aspect_ratio = rng.randint(0, 256) # 1 byte

    out.write(header)
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write((color_table_size).to_bytes(1, 'little'))
    out.write((bg_color_index).to_bytes(1, 'little'))
    out.write((aspect_ratio).to_bytes(1, 'little'))

    # Write palette (if any)
    if color_table_size > 0:
        for i in range(0, color_table_size * 3, 3):
            r = rng.randint(0, 256)
            g = rng.randint(0, 256)
            b = rng.randint(0, 256)

            out.write(r.to_bytes(1, 'little'))
            out.write(g.to_bytes(1, 'little'))
            out.write(b.to_bytes(1, 'little'))

    # Write image data
    datalen = width * height
    while datalen % 8!= 0:
        data = rng.read(datalen)
    out.write(header + datalen.to_bytes(3, 'big') + data)
