#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 7

    # global palette
    colors = [None for _ in range(256)]
    for i in range(256):
        red   = rng.read(1)[0]
        green = rng.read(1)[0]
        blue  = rng.read(1)[0]

        colors[i] = bytes([red, green, blue])

    # image descriptor
    id = bytes([
        0x2C, # image separator
        0x00, 0x00, 0x00, # left offset
        0x00, 0x00, 0x00, # top offset
        *w, *h, # width and height
        packed, # packed field of color resolution (bits per pixel)
        bg, # background color index
        0x00, # reserved byte
    ])

    data = []
    for y in range(int.from_bytes(h, 'little')):
        row = []
        for x in range(int.from_bytes(w, 'little')):
            row += colors[rng.read(1)[0]]
        data += row

    # write everything to output
    out.write(header)
    out.write(*w)
    out.write(*h)
    out.write(packed)
    out.write(bg.to_bytes(1, 'little'))
    out.write(ar.to_bytes(1, 'little'))
    out.write(data)