#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 768, delay: int = 50, loops: int = 0
) -> None:

    # Header
    out.write("GIF89a".encode())
    out.write(struct.pack("<HH", width, height))
    out.write("\x87\x00\x00".encode())

    # Logical screen descriptor
    out.write(",".encode())
    out.write(struct.pack("<BBBHB", 0xF7, 0x00, 0x00, 0x00, 0x00))

    # Global color table
    for _ in range(0, 256):
        out.write(struct.pack("<BBBB", *([0] * 3)))

    # Image descriptor
    out.write(";".encode())
    out.write(struct.pack("<BBHBBHHB", 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))

    # Local color table
    for _ in range(0, 256):
        out.write(struct.pack("<BBBB", *([0] * 3)))

    # Graphics control extension
    out.write("!".encode())
    out.write(struct.pack("<BBHB", 0xF9, 0x04, 0x00, 0x00))
    out.write(struct.pack("<B", (delay & 0xFF00) >> 8))
    out.write(struct.pack("<B", delay & 0xFF))
    out.write(struct.pack("<B", ((loops & 0b1111) << 2) | 0b11))
    out.write("\x00".encode())

    # Image data
    for y in range(height - 1, -1, -1):
        for x in range(width):
            out.write(struct.pack("<B", (((y + x) % 256) ^ 0xAA) & 0xFF))


if