#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    # image descriptor
    out.write(header + w + h + packed + bg.to_bytes(1, 'little'))

    # graphics control extension
    out.write(b'\x21\xF9\x04')
    out.write((ar).to_bytes(1, 'little'))
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')

    # application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0')
    out.write(b'\x03\x01')
    out.write(b'\x00\x00')
    out.write(b'\x00')
    out.write(b'\x00\x00')

    # image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)