#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given width and height, with each pixel having 1 bit of depth.
def generate_random_gif(width: int, height: int, out: BinaryIO):

    # Header
    header = b'GIF89a' + \
             struct.pack('<HH', width, height) + \
             b'\x87\x00\x00'

    # Logical screen descriptor
    lsd = struct.pack('BBB', 255, 255, 1)

    # Global color table (black & white only)
    gct = bytes([
        0b11111100,
        0b11111100,
        0b11111100,
        0b11111100,
        0b11111100,
        0b11111100])

    # Image descriptor
    id = struct.pack('<HHBBHHH',
                     0, 0,
                     width, height,
                     1, 0, 0)

    # Local color table (empty)
    lct = b''

    # Trailer
    trailer = b';'

    data = header + lsd + gct + id + lct + trailer

    out.write(data)
    out.write(trailer)
