#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(rng: BinaryIO, outf: BinaryIO):
    # Header
    header = b'GIF89a' + bytes([0x1D])  # 257 frames

    # Logical Screen Descriptor
    lsd = bytearray()
    lsd += struct.pack('<HHHBB', 640, 480, 0xF000 | 0x0010, 0b10000000, 0)
    lsd[1] |= 0b10000000  # Global Color Table Flag set to true
    lsd[1] |= 0b00000001  # Sort flag set to false (not interlaced)
    lsd[1] |= 0b00000010  # Size of global color table - 256 entries
    lsd[1] |= 0b00000100  # Background color index is 0
    lsd[1] |= 0b00001000  # Pixel aspect ratio - square pixels
    lsd += bytes([0xFF, 0xFF, 0xFF])  # Garbage for the rest of the LSD

    # Global Color Table
    gct = bytearray()
    for i in range(256 * 3):
        gct.append(rng.read(1)[0])

    # Image Descriptor
    id = bytearray()
    id += struct.pack('<HHHHB', 0, 0, 640, 480, 0b10000000)
    id += bytes([0xFF, 0xFF, 0xFF])  # Garbage for the rest of the ID

    # Image Data
    imagedata = bytearray()
    for _ in range(257):
        for y in range(480):
            for x in range(640 // 8):
                byte = rng.read(1)[0]

                for bit in range(8):
                    if not ((byte >> bit) & 1):
                        continue

                    pixel = bytearray()
                    pixel += struct.pack('<BBB', 0, 0, 0)
                    pixel[-1] = 0xFF

                    imagedata += pixel

            padding = bytearray()
            padding += struct.pack('<BBB', 0, 0, 0)
            padding[-1] = 0x00

            imagedata += padding

        # Trailer
        trailer = bytearray()
        trailer += struct.pack('<BBBB', 0x3B, 0, 0, 0)

        imagedata += trailer

    outf.write(header)
    outf.write(lsd)
    outf.write(gct)
    outf.write(id)
    outf.write(imagedata)



if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)