#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the width of the image in pixels.
    width = int.from_bytes(rng.read(2), byteorder='little', signed=False)

    # Generate a 16-bit integer with the height of the image in pixels.
    height = int.from_bytes(rng.read(2), byteorder='little', signed=False)

    # Write the width and height as unsigned bytes to the output stream.
    out.write(width.to_bytes(2, byteorder='big'))
    out.write(height.to_bytes(2, byteorder='big'))

    # Write an empty byte to indicate that there is no color table for this
    # image.
    out.write(b'\x00')

    # Write the pixel aspect ratio value to the output stream.
    out.write(b'\x40\x00')

    # Write the background color index to the output stream.
    out.write(b'\xff')

    # Write the pixel aspect ratio value to the output stream.
    out.write(b'\x00')

    # Write the global color table flag set to true.
    out.write(b'\x01')

    # Write the length of the global color table to the output stream.
    out.write(b'\x07')

    # Write the RGB values of each entry in the global color table to the
    # output stream.
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\xff\xff\xff')

    # Write the size of the local color table to the output stream.
    out.write(b'\x00')

    # Write the LZW minimum code size to the output stream.
    out.write(b'\x03')

    # Write the first two bytes of the logical screen descriptor to the output
    # stream.
    out.write(width.to_bytes(2, byteorder='big'))
    out.write(height.to_bytes(2, byteorder='big'))

    # Write the packed field representing the global color table flag, the size
    # of the global color table, the background color index, the pixel aspect
    # ratio, and the sort order of the colors within the global color table.
    out.write(b'\x01\x07\xff\x00\x40\x00')

    # Write the initial code size used by the encoder to encode the data from
    # the frame.
    out.write(b'\x03')

    # Write the initial code size used by the encoder to encode the data from
    # the frame.
    out.write(b'\x03')

    # Write the first three bytes of the image descriptor to the output stream.
    out.write(b'\x2c\x00\x00')

    # Write the width and height of the image in pixels to the output stream.
    out.write(width.to_bytes(2, byteorder='big'))
    out.write(height.to_bytes(2, byteorder='big'))

    # Write the packed field representing the local color table flag, the size
    # of the local color table, the interlace flag, the sort order of the
    # colors within the local color table, the reserved flags, and the local
    # color table size.
    out.write(b'\x00\x00\x00\x00\x00\x00')

    # Write the LZW minimum code size to the output stream.
    out.write(b'\x03')

    # Write the first byte of the graphic control extension to the output
    # stream.
    out.write(b'\xf9')

    # Write the size of the graphic control extension block to the output
    # stream.
    out.write(b'\x04')

    # Write the packed field representing the disposal method, user input
    # flag, transparency flag, and delay time in hundredths of seconds.
    out.write(b'\x00\x00\x00\x00')

    # Write the last byte of the graphic control extension to the output
    # stream.
    out.write(b'\x00')

    # Write the first byte of the plain text extension to the output stream.
    out.write(b'\xfe')

    # Write the size of the plain text extension block to the output stream.
    out.write(b'\x01')

    # Write the one byte of the plain text extension to the output stream.
    out.write(b