#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    datalen = struct.pack('>H', int.from_bytes(rng.read(2), byteorder='big'))

    # Write the image header information to the output stream.
    out.write(datalen)
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00')

    # Write the palette color table to the output stream.
    for _ in range(768):
        out.write(rng.read(1))

    # Write the image data to the output stream.
    while True:

        # Read one byte from the input stream.
        c = rng.read(1)

        if len(c) == 0:
            break
        
        # If we read an escape code, write it to the output stream.
        elif ord(c[0]) >= 0x21 and ord(c[0]) <= 0x2C:

            # Write the escape code to the output stream.
            out.write(c)

            # Read another byte from the input stream.
            d = rng.read(1)
            
            # If that byte is not zero, write it to the output stream.
            if len(d) > 0 and ord(d[0])!= 0:
                out.write(d)

        else:

            # Otherwise, just write the byte to the output stream.
            out.write(c)


if