#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = rng.read(2)
    h = rng.read(2)

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    # global color table size
    ctsize = rng.read(1)[0] & 0x7f

    # sort color table by decreasing importance
    ctsize |= 0x80

    # write header to output file
    data = bytearray()
    data += header + w + h + packed + bg + ar + ctsize
    out.write(data)

    # write palette entries to output file
    for i in range(ctsize):
        data[i*3+0] = rng.read(1)[0]
        data[i*3+1] = rng.read(1)[0]
        data[i*3+2] = rng.read(1)[0]
    out.write(data)

    # write image descriptor to output file
    data[0] = 0x2c
    data[1] = w[0]
    data[2] = w[1]
    data[3] = h[0]
    data[4] = h[1]
    data[5] = packed[0]
    data[6] = bg
    data[7] = ar

    out.write(data)