#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# given RNG object, which should have a read() method
def generate(rng: BinaryIO, out: BinaryIO):
    header = b'GIF89a'  # magic number
    w = (rng.read(2))[::-1]  # width in big endian
    h = (rng.read(2))[::-1]  # height in big endian
    packed = rng.read(1)[0] & 7  # packed field
    bg = rng.read(1)[0]  # background color index
    ar = rng.read(1)[0]  # aspect ratio

    out.write(header + w + h + packed +
             bytes([bg]) + bytes([ar]))

    # LZW minimum code size
    mincodesize = rng.read(1)[0]
    out.write(bytes([mincodesize]))

    # image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)