#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    imglen = int.from_bytes(rng.read(2), byteorder='little', signed=False)

    # Generate an unsigned byte for each color in the palette.
    colors = [int.from_bytes(rng.read(1), byteorder='big', signed=False)
              for _ in range(768)]

    # Write the global color table to the output stream.
    out.write(struct.pack('B'*768, *colors))

    # Write the logical screen descriptor to the output stream.
    out.write(struct.pack('<HHBBBHBH',
                          imglen,
                          0,
                          0,
                          768,
                          0,
                          0,
                          0))

    # Copy the image data from the input stream to the output stream.
    while True:
        buf = rng.read(4096)
        if not buf: break
        out.write(buf)
