#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the magic string and version number to the output stream.
    out.write(b'GIF89a')

    # Generate a 16-bit integer with the length of the image data.
    width = int.from_bytes(rng.read(2), byteorder='big', signed=False)
    height = int.from_bytes(rng.read(2), byteorder='big', signed=False)
    size = width * height
    print('Image dimensions:', width, 'x', height, '=', size, 'pixels.')

    # Generate an unsigned byte for each pixel in the image.
    pixels = [int.from_bytes(rng.read(1), byteorder='big', signed=False)
              for _ in range(size)]

    # Write the screen descriptor block to the output stream.
    out.write(struct.pack('<HHBBB',
                          width, height,
                          0xf7 | ((width >> 11) & 0x07),
                          0xff & (height >> 8),
                          0xff & height))

    # Write the global color table to the output stream.
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table to the output stream.
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the graphics control extension to the output stream.
    out.write(struct.pack('<BBHB',
                          0x21, 0xf9, 4, 0x01, 0x00))

    # Write the application extension to the output stream.
    out.write(struct.pack('<BBHBB',
                          0x21, 0xff, 11, b'NETSCAPE', 3, 1, 0))

    # Write the comment extension to the output stream.
    out.write(struct.pack('<BBH',
                          0x21, 0xfe, 0))

    # Write the plain text subblock to the output stream.
    out.write(b'Created by Python!')

    # Write the trailer to the output stream.
    out.write(b';')


if