#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(pow(2, int.from_bytes(color, 'big'))):
        r = int.from_bytes(rng.read(1),'big')
        g = int.from_bytes(rng.read(1),'big')
        b = int.from_bytes(rng.read(1),'big')
        out.write(r.to_bytes(1,'big'))
        out.write(g.to_bytes(1,'big'))
        out.write(b.to_bytes(1,'big'))

    # Write the image descriptor
    left   = bytes([0x00])
    top    = bytes([0x00])
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    out.write(left)
    out.write(top)
    out.write(width.to_bytes(2,'big'))
    out.write(height.to_bytes(2,'big'))
    out.write(flags)

    # Write the local color table
    if pow(2, int.from_bytes(color, 'big')) > 0:
        for i in range(pow(2, int.from_bytes(color, 'big'))):
            r = int.from_bytes(rng.read(1),'big')
            g = int.from_bytes(rng.read(1),'big')
            b = int.from_bytes(rng.read(1),'big')
            out.write(r.to_bytes(1,'big'))
            out.write(g.to_bytes(1,'big'))
            out.write(b.to_bytes(1,'big'))

    # Write the graphics control extension block
    label      = bytes([0x21])
    size       = bytes([0xf9])
    flags      = bytes([0x40])
    delay_time = bytes([int.from_bytes(rng.read(1), 'big'),
                        int.from_bytes(rng.read(1), 'big')])
    trans_idx  = bytes([int.from_bytes(rng.read(1), 'big')])
    terminator = bytes([0x00])
    out.write(label)
    out.write(size)
    out.write(flags)
    out.write(delay_time)
    out.write(trans_idx)
    out.write(terminator)

    # Write the plain text extension block
    label        = bytes([0x21])
    size         = bytes([0x01])
    sub_label    = bytes([0xfe])
    auth_code    = bytes([0x00, 0x0B, 0x4E, 0x45, 0x54, 0x53,
                          0x43, 0x41, 0x50, 0x45, 0x32, 0x2E,
                          0x30, 0x03, 0x01, 0x00, 0x00, 0x00])
    terminator   = bytes([0x00])
    out.write(label)
    out.write(size)
    out.write(sub_label)
    out.write(auth_code)
    out.write(terminator)

    # Write the application extension block
    label          = bytes([0x21])
    size           = bytes([0xff])
    app_label      = bytes([0xFF, 0x0B, 0x4A, 0x46, 0x49,
                            0x46, 0x0D, 0x0A, 0x1A, 0x0A])
    netscape_label = bytes([0x01])
    junk           = bytes([0x00, 0x3B])
    terminator     = bytes([0x00])
    out.write(label)
    out.write(size)
    out.write(app_label)
    out.write(netscape_label)
    out.write(junk)
    out.write(terminator)

    # Write the image data
    lzw_min_code_size = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(lzw_min_code_size)
    clear_code = bytes([int.from_bytes(rng.read(1), 'big')])
    end_of_info= bytes([int.from_bytes(rng.read(1), 'big')])
    end_of_data= bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(clear_code)
    out.write(end_of_info)
    out.write(end_of_data)

    # Write the trailer
    trailer = bytes([0x3B])
    out.write(trailer)


if