#!/usr/bin/env python3

import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # header
    out.write(b'GIF89a')  # signature
    width = int.from_bytes(
            rng.read(2), byteorder='little')
    height = int.from_bytes(
            rng.read(2), byteorder='little')
    flags = bytes([int.from_bytes(
            rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHB', width, height, *flags))
    mincodesize = int.from_bytes(
            rng.read(1), byteorder='little')
    out.write(bytes([mincodesize]))

    # image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)