#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Header
    header = b'GIF89a\x01\x00\x01\x00\x80\xff\x00\xc0\xff\x00\x00\x00!\xf9\x04\t\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'

    # Logical screen descriptor
    lsd = bytearray([
        0b00000000 | 7,   # Width (7)
        0b00000000 | 5,   # Height (5)
        0b00000000 | 0b00000000,   # Flags
        0b00000000 | 0b11111111,   # Background color index
        0b00000000 | 0b11111111,   # Pixel aspect ratio
    ])

    # Global color table
    gct = bytes()
    for i in range(0, 256):
        r = rng.read(1)[0] & 0b11111111
        g = rng.read(1)[0] & 0b11111111
        b = rng.read(1)[0] & 0b11111111

        gct += struct.pack('<BBB', r, g, b)

    # Image descriptor
    id = bytearray([
        0b00000000 | 0,   # Left position
        0b00000000 | 0,   # Top position
        0b00000000 | 7,   # Width (7)
        0b00000000 | 5,   # Height (5)
        0b00000000 | 0b00000000,   # Flags
        0b00000000 | 0b11111111,   # Local color table size
    ])

    # Local color table
    lct = bytes()
    for i in range(0, 256):
        r = rng.read(1)[0] & 0b11111111
        g = rng.read(1)[0] & 0b11111111
        b = rng.read(1)[0] & 0b11111111

        lct += struct.pack('<BBB', r, g, b)

    # Graphic control extension
    gce = bytearray([
        0b00000000 | 0b00000000,   # Block size
        0b00000000 | 0b00000000,   # Flags
        0b00000000 | 0b11111111,   # Delay time
        0b00000000 | 0b11111111,   # Transparent color index
        0b00000000 | 0b00000000,   # Terminator
    ])

    # Plain text extension
    pte = bytearray([
        0b00000000 | 0b00000000,   # Block size
        0b00000000 | 0b00000000,   # Text grid left
        0b00000000 | 0b00000000,   # Text grid top
        0b00000000 | 0b00000000,   # Text grid width
        0b00000000 | 0b00000000,   # Text grid height
        0b00000000 | 0b00000000,   # Cell width
        0b00000000 | 0b00000000,   # Cell height
        0b00000000 |         with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)