#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The generated GIF will have a width of 256 pixels and height of 100 pixels.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write((
        0x00, 0x00, 0x00, 0x07,    # Width (256px)
        0x00, 0x00, 0x00, 0x64,    # Height (100px)
        0x80 | 0x03,               # Flags: Global color table, 8-bit pixel size
        0x00                       # Background color index
    ).__bytes__())

    # Color table
    for i in range(0x10):
        out.write((
            0xFF, 0x00, 0x00,         # Red
            0x00, 0xFF, 0x00,         # Green
            0x00, 0x00, 0xFF          # Blue
        ).__bytes__())

    # Image descriptor
    out.write((
        0x00, 0x00, 0x00, 0x00,     # Left position
        0x00, 0x00, 0x00, 0x00,     # Top position
        0x00, 0x00, 0x00, 0x64,     # Width (256px)
        0x00, 0x00, 0x00, 0x64      # Height (100px)
    ).__bytes__())

    # LZW minimum code size
    out.write(b'\x02')

    # Pixels
    while True:

        # Read a block from /dev/urandom
        block = rng.read(0xFFFF - 0x10)

        if len(block) == 0: break

        # Write the length of this block as a short integer
        out.write(struct.pack('>H', len(block)))

        # Write the block itself
        out.write(block)