#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # get some of our data into the file before writing the rest.
    header = b'GIF89a'
    width = 256
    height = 256
    color_table_size = 7
    background_color_index = 0
    pixel_aspect_ratio = 0
    global_color_table = [
            0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff] * (
                2 ** color_table_size)
    local_color_table = []
    image_descriptor = bytes([
        0x2c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00])
    image_data = bytes()
    for y in range(height):
        for x in range(width):
            image_data += bytes([
                global_color_table[
                    rng.randint(
                        0, len(global_color_table)-1)]])
    trailer = bytes([0x3b])

    out.write(header)
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write(((color_table_size << 4) |
               (background_color_index)).to_bytes(1, 'little'))
    out.write(pixel_aspect_ratio.to_bytes(1, 'little'))
    if len(local_color_table) > 0:
        out.write(struct.pack('<H', len(local_color_table)))
        out.write(bytes(local_color_table))
    else:
        out.write(bytes([0]))
    out.write(image_descriptor)
    out.write(image_data)
    out.write(trailer)


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)