#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    width = (int.from_bytes(rng.read(2), 'big') % 65536) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 65536) + 1
    flags = bytes([0])
    bgcolorindex = bytes([0])
    pixelaspectratio = bytes([0])
    out.write(struct.pack('<HHBBB', width, height, *flags, *bgcolorindex, *pixelaspectratio))

    # Global color table
    globalcolortablesize = bytes([7 << 4 | 1])
    out.write(globalcolortablesize)
    for i in range(7):
        r = int.from_bytes(rng.read(1), 'big') & 0xff
        g = int.from_bytes(rng.read(1), 'big') & 0xff
        b = int.from_bytes(rng.read(1), 'big') & 0xff
        out.write(struct.pack('<BBB', r, g, b))

    # Image descriptor
    left = bytes([0])
    top = bytes([0])
    width = (int.from_bytes(rng.read(2), 'big') % 65536) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 65536) + 1
    flags = bytes([0b00000100])
    bgcolorindex = bytes([0])
    pixelaspectratio = bytes([0])
    out.write(left + top + width + height + flags + bgcolorindex + pixelaspectratio)

    # Local color table
    localcolortablesize = bytes([7 << 4 | 1])
    out.write(localcolortablesize)
    for i in range(7):
        r = int.from_bytes(rng.read(1), 'big') & 0xff
        g = int.from_bytes(rng.read(1), 'big') & 0xff
        b = int.from_bytes(rng.read(1), 'big') & 0xff
        out.write(struct.pack('<BBB', r, g, b))

    # Extension block
    label = bytes([0xf9])
    exttype = bytes([0])
    bytecount = bytes([4])
    loopcount = int.from_bytes(rng.read(2), 'big') % 65536
    out.write(label + exttype + bytecount + struct.pack('<H', loopcount))

    # Graphics control extension
    label = bytes([0xf9])
    exttype = bytes([0])
    bytecount = bytes([4])
    flags = bytes([0b00000000])
    delaytime = bytes([0])
    transparentidx = bytes([0])
    terminator = bytes([0])
    out.write(label + exttype + bytecount + flags + delaytime + transparentidx + terminator)

    # Plain text extension
    label = bytes([0xfe])
    exttype = bytes([1])
    bytecount = bytes([12])
    gridleft = bytes([0])
    gridtop = bytes([0])
    gridwidth = bytes([1])
    gridheight = bytes([1])
    charcellwidth = bytes([8])
    charcellheight = bytes([8])
    fgcoloridx = bytes([0])
    bgcoloridx = bytes([0])
    terminator = bytes([0])
    out.write(label + exttype + bytecount + gridleft + gridtop + gridwidth + gridheight
            + charcellwidth + charcellheight + fgcoloridx + bgcoloridx + terminator)

    # Application extension
    label = bytes([0xff])
    appid = bytes([0])
    appauthcode = bytes([0] * 11)
    bytecount = bytes([19])
    appname = "NETSCAPE2.0".encode()
    applength = len(appname)
    out.write(label + appid + appauthcode + bytecount + appname + struct.pack('<B', applength))
    out.write(b"\x03\x01")
    out.write(b"Loop")

   