#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The generated image is always square with side length
# equal to `size`.
def generate_random_gif(
        rng: BinaryIO, size: int, out: BinaryIO
    ):
    header = b'GIF89a'

    # 16-bit width and height
    w = size.to_bytes(2, 'big')
    h = size.to_bytes(2, 'big')

    # packed field of color resolution (bits per pixel)
    r = rng.read(1)[0] & 7
    g = rng.read(1)[0] & 7
    b = rng.read(1)[0] & 7
    packed = bytes([r | (g << 3) | (b << 6)])

    # background color index
    bg = rng.read(1)[0]

    # aspect ratio
    ar = rng.read(1)[0] & 0xf

    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(header + w + h + packed + bg.to_bytes(1, 'little'))
    out.write(ar.to_bytes(1, 'little'))
    out.write(data)