#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of each frame are given by
# `width`, `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0x87, 0, 0))

    # Global color table (empty)
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Loop control block
    out.write(b'\x21\xFF\x0B\x4E\x45\x54\x53\x43\x41\x50\x45\x32\x2E\x30\x03\x01\x00\x00\x00')

    # Image descriptor
    out.write(struct.pack('>BBHHHBB', 0x2C, 0, 0, width, height, 0, 0, 0))

    # Local color table (empty)
    out.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Image data
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)