#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # get some of our data into the file before writing the rest.
    header = b'GIF89a'
    width = int.from_bytes(
            rng.read(2), byteorder='little') % (640 - 50) + 50
    height = int.from_bytes(
            rng.read(2), byteorder='little') % (480 - 50) + 50
    globalcolorcount = int.from_bytes(
            rng.read(1), byteorder='little') % 256
    localcolorcount = int.from_bytes(
            rng.read(1), byteorder='little') % 256
    backgroundindex = int.from_bytes(
            rng.read(1), byteorder='little') % globalcolorcount
    aspectratio = int.from_bytes(
            rng.read(1), byteorder='little') % 4
    datalen = ((width * height) // 8) + \
            1 + len(b'\x2c\x00\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00') + \
            1 + len(b'\x00\x00