#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate some random bytes for the length of the image data
    # and write them to the output stream
    datalen = int.from_bytes(
            rng.read(3), byteorder='big', signed=False)
    out.write(datalen.to_bytes(3, 'big'))

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)

    # Pad the rest with zeroes until we reach 256 bytes
    paddinglen = (256 - len(data)) % 256
    if paddinglen!= 0:
        out.write(b'\x00' * paddinglen)