#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Generate the header
    header = b'GIF89a'
    # Read 16 bits from /dev/urandom as an unsigned integer (uint16)
    width = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        )
    height = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        )
    # The color table is fixed at 256 colors in our implementation
    colortablesize = 768
    # We don't need any global color table for this example
    backgroundcolorindex = 0
    pixelaspectratio = 0
    # Pack all these values together into one binary string
    header += struct.pack('>HHBBB',
                          width, height, colortablesize,
                          backgroundcolorindex, pixelaspectratio
                         )
    # Read the rest of the bytes from /dev/urandom as the image data
    datalen = int.from_bytes(
            rng.read(2), byteorder='big', signed=False
        )
    data = rng.read(datalen)

    # Write the header and length of the data to the output
    out.write(header + len(data).to_bytes(2, 'big'))
    # Write the data itself
    out.write(data)

