#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int, height: int
):
    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    out.write((width & 0xff).to_bytes(1, 'little'))
    out.write((height & 0xff).to_bytes(1, 'little'))
    out.write(b'\x87\x00')      # Global Color Table Flag (1 bit)
                                # Sort Flag (1 bit)
                                # Size of Global Color Table (3 bits)
                                # Background Color Index (4 bits)
                                # Pixel Aspect Ratio (3 bits)
    out.write(b'\x00')         # GCT Size (3 bytes)

    # Image Descriptor
    out.write(b'\x2c')         # Image Separator
    out.write((0 & 0xff).to_bytes(1, 'little'))
    out.write((0 & 0xff).to_bytes(1, 'little'))
    out.write((width & 0xff).to_bytes(1, 'little'))
    out.write((height & 0xff).to_bytes(1, 'little'))
    out.write(b'\x00')         # Flags (1 byte)
                                # Local Color Table Flag (1 bit)
                                # Interlace Flag (1 bit)
                                # Sort Flag (1 bit)
                                # Reserved (2 bits)
                                # Size of Local Color Table (3 bits)
    out.write(b'\x00')         # LCT Size (3 bytes)

    # Image Data
    datalen = width * height // 64    # Number of blocks in each row
    for _ in range(datalen):
        blocklen = min(rng.read(1)[0], 255)   # Block size is at most 255
        out.write(blocklen.to_bytes(1, 'little'))

        if blocklen > 0:
            out.write(rng.read(blocklen))

    # Trailer
    out.write(b';')