#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The generated GIF has 10 frames and each frame is
# 256x256 pixels with 4 bits per pixel (i.e., 16 colors).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 256
    height = 256
    color_depth = 4
    global_color_table_flag = True
    background_color_index = 0
    aspect_ratio = 0
    out.write(struct.pack('<HHBBB',
                         width, height,
                         color_depth,
                         global_color_table_flag << 7 | background_color_index,
                         aspect_ratio))

    # Global color table
    for i in range(1 << color_depth):
        r = rng.read(1)[0] & ~0xf
        g = rng.read(1)[0] & ~0xf
        b = rng.read(1)[0] & ~0xc

        if i == 0:
            r |= 0xe0
            g |= 0xe0
            b |= 0xc0

        out.write(r.to_bytes(1, 'little'))
        out.write(g.to_bytes(1, 'little'))
        out.write(b.to_bytes(1, 'little'))

    # Image descriptor
    left = 0
    top = 0
    width = 256
    height = 256
    local_color_table_flag = False
    interlace_flag = False
    sort_flag = False
    reserved = 0
    lct_size = 0
    out.write(struct.pack('<BBHHB',
                          left, top,
                          width, height,
                          0x80 | local_color_table_flag << 7 | interlace_flag << 6 | sort_flag << 5 | reserved))

    # Local color table
    for i in range(lct_size):
        r = rng.read(1)[0] & ~0xf
        g = rng.read(1)[0] & ~0xf
        b = rng.read(1)[0] & ~0xc

        if i == 0:
            r |= 0xe0
            g |= 0xe0
            b |= 0xc0

        out.write(r.to_bytes(1, 'little'))
        out.write(g.to_bytes(1, 'little'))
        out.write(b.to_bytes(1, 'little'))

    # Image data
    for _ in range(10):
        for y in range(height):
            for x in range(width // 8):
                out.write((rng.read(1)[0] >> ((~x & 7) * 4)).to_bytes(1, 'little'))