#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = (int.from_bytes(rng.read(2), 'big') % 640) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 480) + 1
    flags = bytes([int.from_bytes(rng.read(1), 'big') & 7])
    background_color_index = bytes([int.from_bytes(rng.read(1), 'big')])
    pixel_aspect_ratio = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(width)
    out.write(height)
    out.write(flags)
    out.write(background_color_index)
    out.write(pixel_aspect_ratio)

    # Global color table
    global_color_table_size = bytes([int.from_bytes(rng.read(1), 'big')])
    global_color_table_sorted = bytes([int.from_bytes(rng.read(1), 'big')])
    global_color_table_entry_size = bytes([int.from_bytes(rng.read(1), 'big')])
    for _ in range(global_color_table_size[0]):
        r = bytes([int.from_bytes(rng.read(1), 'big')])
        g = bytes([int.from_bytes(rng.read(1), 'big')])
        b = bytes([int.from_bytes(rng.read(1), 'big')])
        out.write(r)
        out.write(g)
        out.write(b)

    # Image descriptor
    left = bytes([int.from_bytes(rng.read(2), 'big')])
    top = bytes([int.from_bytes(rng.read(2), 'big')])
    width = bytes([int.from_bytes(rng.read(2), 'big')])
    height = bytes([int.from_bytes(rng.read(2), 'big')])
    flags = bytes([int.from_bytes(rng.read(1), 'big') | 0xe0])
    local_color_table_flag = bytes([(int.from_bytes(rng.read(1), 'big')) & 1])
    interlace_flag = bytes([(int.from_bytes(rng.read(1), 'big')) & 1])
    sort_flag = bytes([(int.from_bytes(rng.read(1), 'big')) & 1])
    reserved = bytes([int.from_bytes(rng.read(1), 'big')])
    local_color_table_size = bytes([int.from_bytes(rng.read(1), 'big')])
    local_color_table_entry_size = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(left)
    out.write(top)
    out.write(width)
    out.write(height)
    out.write(flags)
    out.write(local_color_table_flag)
    out.write(interlace_flag)
    out.write(sort_flag)
    out.write(reserved)
    out.write(local_color_table_size)
    out.write(local_color_table_entry_size)

    # Local color table
    if local_color_table_flag == b'\x01':
        for _ in range(local_color_table_size):
            r = bytes([int.from_bytes(rng.read(1), 'big')])
            g = bytes([int.from_bytes(rng.read(1), 'big')])
            b = bytes([int.from_bytes(rng.read(1), 'big')])
            out.write(r)
            out.write(g)
            out.write(b)

    # LZW minimum code size
    lzw_minimum_code_size = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(lzw_minimum_code_size)

    # Data sub-block
    while True:
        block_type = bytes([int.from_bytes(rng.read(1), 'big')])

        if block_type == b'\x2c':
            pass
        elif