#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1024
    height = 768
    color_depth = 8
    global_color_table = True
    sort_flag = False
    size = (width * height).bit_length() // 8
    packed = ((size - 1) & 0x07) | \
             (((global_color_table << 3) |
               (sort_flag << 2)) << 5) | \
             (color_depth << 8)
    out.write(struct.pack('>H', packed))
    out.write(struct.pack('>HHBBB',
                          width, height,
                          0xF7, 0, 0))

    # Global color table
    for i in range(2**color_depth):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(struct.pack('BBB',
                              r, g, b))

    # Image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table = False
    interlace = False
    sort_flag = False
    reserved = 0
    lct_size = 0
    packed = ((left & 0xFF) << 2) | \
             ((top & 0xFF) << 10) | \
             ((width - 1) & 0x0FFF) | \
             ((height - 1) & 0x0FFF) << 12 | \
             ((local_color_table << 14) |
              (interlace << 15) |
              (sort_flag << 16) |
              (reserved << 17) |
              (lct_size << 18))
    out.write(struct.pack('>BHHBBBBB',
                          0x2C,
                          packed,
                          0, 0, 0, 0))

    # Local color table
    if not local_color_table:
        return

    for i in range(2**lct_size):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(struct.pack('BBB',
                              r, g, b))

    # Image data
    data = rng.read((width*height)//8)
    out.write(data)

