#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are given by `width`
# and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int, height: int
):
    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Global color table
    for _ in range(256):
        out.write(rng.read(3))

    # Image descriptor
    out.write(struct.pack('<BHHBB', 0x2C, 0, 0, 0, 0b10000000 | (0 << 4)))

    # Local color table
    out.write(rng.read(3))

    # Image data
    while True:
        chunksize = rng.read(1)[0]
        if not chunksize: break

        out.write(chunksize.to_bytes(1, 'little'))
        out.write(rng.read(chunksize))