#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are determined by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write((width & 0xFF).to_bytes(1, 'little'))
    out.write((height & 0xFF).to_bytes(1, 'little'))
    out.write(b'\x87\x00')    # Global color table flag set to true
    out.write(b'\x00')        # Background color index is zero (black)
    out.write(b'\x00')        # Pixel aspect ratio is one (square pixels)

    # Image descriptor
    out.write(b'\x2C')       # Image separator
    out.write((0 & 0xFF).to_bytes(1, 'little'))   # Left position
    out.write((0 & 0xFF).to_bytes(1, 'little'))   # Top position
    out.write((width & 0xFF).to_bytes(1, 'little')) # Width
    out.write((height & 0xFF).to_bytes(1, 'little'))# Height
    out.write(b'\x00')        # No local color table
    out.write(b'\x00')        # Interlace flag not set
    out.write(b'\x00')        # Sort flag not set
    out.write(b'\x00')        # Reserved for future use

    # Local color table
    out.write(b'\x00\x00\x00\x00\x00\x00')
    out.write(b'\x00\x00\x00\x00\x00\x00')
    out.write(b'\x00\x00\x00\x00\x00\x00')
    out.write(b'\x00\x00\x00\x00\x00\x00')
    out.write(b'\x00\x00\x00\x00\x00\x00')
    out.write(b'\x00\x00\x00\x00\x00\x00')

    # LZW minimum code size
    out.write(b'\x02')

    # Data sub-block
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    data = rng.read(datalen)
    out.write(data)