#!/usr/bin/env python3

import os
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.randint(0, 640)
    height = rng.randint(0, 480)
    flags = 0b10000000 | rng.randint(0, 7)
    bgcol = bytes([rng.randint(0, 255)]) * 3
    aspect = rng.randint(0, 128)
    out.write(struct.pack('<HHBBB', width, height, flags, bgcol[0], bgcol[1], bgcol[2]))

    # Global color table
    gctsize = rng.randint(0, 256)
    out.write(struct.pack('B', gctsize))
    for _ in range(gctsize):
        col = bytes([rng.randint(0, 255)]) * 3
        out.write(col)

    # Image descriptor
    left = rng.randint(0, 640 - width)
    top = rng.randint(0, 480 - height)
    flags = 0b10000000 | rng.randint(0, 7)
    lctsize = rng.randint(0, 256)
    out.write(struct.pack('<HHBB', left, top, width, height))
    out.write(flags.to_bytes(1, byteorder='little'))
    if flags & 0b10000000!= 0:
        out.write(lctsize.to_bytes(1, byteorder='little'))
        for _ in range(lctsize):
            col = bytes([rng.randint(0, 255)]) * 3
            out.write(col)

    # Image data
    lzwmincodesize = rng.randint(0, 256)
    out.write(lzwmincodesize.to_bytes(1, byteorder='little'))
    imagedata = []
    while True:
        blocktype = rng.randint(0, 2)
        if blocktype == 0:
            break

        elif blocktype == 1:
            numcodes = rng.randint(0, 256)
            for i in range(numcodes):
                codelen = rng.randint(0, 256)
                imagedata.append((i, codelen))

        else:
            raise Exception("Invalid LZW block type")

    out.write(imagedata)

    # Trailer
    out.write(b';')