#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width and height of the image are specified by
# `width` and `height`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO,
        width: int, height: int
):
    # Header
    out.write('GIF89a'.encode())

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xf7, 0x0b))

    # Global color table (black & white only)
    out.write('\xff\xff\xff\x00\x00\x00'.encode())

    # Image descriptor
    out.write(struct.pack('<BHHHHBB',
            0x2c, 0, 0, width, height, 0, 0xf7))

    # Local color table (black & white only)
    out.write('\xff\xff\xff\x00\x00\x00'.encode())

    # Image data
    for _ in range(height):
        out.write('\x00' * width)

    # Trailer
    out.write(';\n'.encode())