#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # get some of our data into the file before writing the rest.
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    while True:
        chunklen = min(datalen, 256)

        if not chunklen:
            break

        out.write(chunklen.to_bytes(1, 'little'))
        out.write(rng.read(chunklen))
        datalen -= chunklen

    # Finally, pad out the last block with zeroes.
    padding = (4 - ((out.tell() % 4) or 4)) % 4
    out.write(b'\x00' * padding)


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)