#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    color_table_size = 256
    global_color_table = True
    background_index = 0
    pixel_aspect_ratio = 0
    out.write(struct.pack('<HHBBB',
            width, height, color_table_size,
            0 if global_color_table else 2**global_color_table,
            background_index, pixel_aspect_ratio))

    # Global color table (if any)
    for i in range(color_table_size):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(r.to_bytes(1, 'little'))
        out.write(g.to_bytes(1, 'little'))
        out.write(b.to_bytes(1, 'little'))

    # Image descriptor
    left   = 0
    top    = 0
    width  = 1024
    height = 768
    local_color_table = False
    interlace = False
    sort     = False
    reserved = 0
    lct_size = 2**local_color_table
    out.write(struct.pack('<HHHHBB',
            left, top, width, height,
            lct_size,
            0 if interlace else 2**interlace |
                0 if sort      else 2**sort |
                reserved))

    # Local color table (if any)
    for i in range(lct_size):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(r.to_bytes(1, 'little'))
        out.write(g.to_bytes(1, 'little'))
        out.write(b.to_bytes(1, 'little'))

    # LZW minimum code size
    mincodesize = 2
    out.write((mincodesize).to_bytes(1, 'little'))

    # Data sub-block
    blocklen = 2**mincodesize
    while len(data := rng.read(blocklen)):
        out.write(((~blocklen & 0xFF)).to_bytes(1, 'little'))
        out.write(data)
        blocklen += 1
    out.write((0x00).to_bytes(1, 'little'))

