#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The first byte is always fixed to \x47
    out.write(b'\x47')

    # Next comes the size of the image in bytes (including this field).
    # We use the maximum possible value here.
    out.write(b'\xff\xff')

    # Then we write the actual image data.
    # This is done by reading from /dev/urandom until we have enough bytes.
    while True:
        chunk = rng.read(65535 - len(out.getvalue()))

        if not chunk:
            break

        out.write(chunk)

    # Finally we add the terminator for the image.
    out.write(b';')


if __name__ == '__main__':
    with open('/dev/urandom', 'rb') as rng:
        try:
            filename = sys.argv[1]
        except IndexError:
            print('Usage:', sys.argv[0], '<output-file>')
            exit()

        with open(filename, 'wb') as f:
            generate_random_gif(rng, f)