#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # get some of our data into the file before writing the rest.
    header = b'GIF89a'
    width = 256
    height = 256
    color_table_size = 7
    global_color_table = [
            0x00, 0x00, 0x00,
            0xff, 0x00, 0x00,
            0x00, 0xff, 0x00,
            0xff, 0xff, 0x00,
            0x00, 0x00, 0xff,
            0xff, 0x00, 0xff,
            0x00, 0xff, 0xff]
    background_color = 0x00
    pixel_aspect_ratio = 0x00
    flags = (width & 0x07) | ((height & 0x07) << 4)
    flags |= (((color_table_size - 1) & 0x07) << 8)
    flags |= (background_color & 0xf8) << 12
    flags |= (pixel_aspect_ratio & 0x07) << 15
    out.write(header + bytes([flags]))
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write(((color_table_size * 3)).to_bytes(1, 'little'))
    out.write(b'\x00\x00')
    for i in range(0, len(global_color_table)):
        out.write(bytes([global_color_table[i]]))

    # Now we can write the actual data to the file.
    for y in range(0, height):
        for x in range(0, width):
            out.write(bytes([rng.read(1)]))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)