#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width = (int.from_bytes(rng.read(2), 'big') % 65536) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 65536) + 1
    flags = bytes([0])
    background = bytes([0]) * 3
    aspect = bytes([48])
    screendesc = struct.pack('>HHBBB', width, height, flags[0], background[0],
                            background[1], background[2], aspect[0])
    out.write(screendesc)

    # Global Color Table
    globalcolors = [bytes([i]) for i in range(768)]
    out.write(b''.join(globalcolors))

    # Image Descriptor
    left = bytes([0]) * 2
    top = bytes([0]) * 2
    width = (int.from_bytes(rng.read(2), 'big') % 65536) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 65536) + 1
    flags = bytes([0])
    localcolors = bytes([0]) * 3
    imagedesc = struct.pack('>HHBBBBB', left[0], left[1], top[0], top[1],
                           width, height, flags[0], localcolors[0])
    out.write(imagedesc)

    # Local Color Table
    localcolors = [bytes([i]) for i in range(768)]
    out.write(b''.join(localcolors))

    # Data
    datalen = int.from_bytes(rng.read(2), 'big') + 1
    data = rng.read(datalen)
    out.write(data)

    # Trailer
    out.write(b';')