#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom or /dev/urandom).
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The first 6 bytes are always "GIF89a"
    out.write(b'GIF89a')

    # Next comes the logical screen descriptor
    width = int.from_bytes(rng.read(2), byteorder='little') % 1000
    height = int.from_bytes(rng.read(2), byteorder='little') % 1000
    flags = int.from_bytes(rng.read(1), byteorder='little') & 7
    bgcolor = int.from_bytes(rng.read(1), byteorder='little')
    aspect = int.from_bytes(rng.read(1), byteorder='little')
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(flags.to_bytes(1, 'little'))
    out.write(bgcolor.to_bytes(1, 'little'))
    out.write(aspect.to_bytes(1, 'little'))

    # Then follows the global color table
    colors = [int.from_bytes(rng.read(1), byteorder='little') for _ in range(256)]
    out.write((len(colors)*3).to_bytes(1, 'little'))
    out.write(b'\x00\x00\x00'.join([c.to_bytes(1, 'little') for c in colors]))

    # Finally we have the image descriptor and the image data
    left = int.from_bytes(rng.read(2), byteorder='little')
    top = int.from_bytes(rng.read(2), byteorder='little')
    width = int.from_bytes(rng.read(2), byteorder='little') % 1000
    height = int.from_bytes(rng.read(2), byteorder='little') % 1000
    flags = int.from_bytes(rng.read(1), byteorder='little') & 7
    bgcolor = int.from_bytes(rng.read(1), byteorder='little')
    aspect = int.from_bytes(rng.read(1), byteorder='little')
    lctsize = int.from_bytes(rng.read(1), byteorder='little')
    if lctsize!= 0:
        raise ValueError('LCT size must be zero!')
    out.write(left.to_bytes(2, 'little'))
    out.write(top.to_bytes(2, 'little'))
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(flags.to_bytes(1, 'little'))
    out.write(bgcolor.to_bytes(1, 'little'))
    out.write(aspect.to_bytes(1, 'little'))
    out.write(lctsize.to_bytes(1, 'little'))
    data = rng.read(width * height)
    out.write(data)


