#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):
    # Header
    out.write("GIF89a".encode())
    
    # Logical screen descriptor
    width = 256
    height = 256
    global_color_table_flag = 0x80
    color_resolution = 7
    sort_flag = 0
    size_of_global_color_table = 1 << (color_resolution + 1)
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write(struct.pack("<HHBBB", width, height, global_color_table_flag | size_of_global_color_table, background_color_index, pixel_aspect_ratio))
    
    # Global color table
    for i in range(size_of_global_color_table):
        red = int.from_bytes(rng.read(1), "big") % 256
        green = int.from_bytes(rng.read(1), "big") % 256
        blue = int.from_bytes(rng.read(1), "big") % 256
        out.write(struct.pack("<BBB", red, green, blue))
        
    # Application extension
    block_type = 0xFF
    application_identifier = b"RANDOM"
    authentication_code = b"\x00\x00\x00\x00"
    data = b""
    while len(data) < 255:
        data += rng.read(1)
    block_size = len(application_identifier) + len(authentication_code) + len(data)
    out.write(struct.pack("<BB{}s{}s{}s".format(len(application_identifier), len(authentication_code), len(data)), block_type, block_size, application_identifier, authentication_code, data))
    
    # Image descriptor
    image_separator = 0x2C
    left = 0
    top = 0
    width = 256
    height = 256
    local_color_table_flag = 0x00
    interlace_flag = 0
    sort_flag = 0
    size_of_local_color_table = 0
    out.write(struct.pack("<BBHHHBB", image_separator, left, top, width, height, local_color_table_flag | size_of_local_color_table, interlace_flag, sort_flag))
    
    # Local color table
    # None
    
    # LZW minimum code size
    lzw_minimum_code_size = 8
    out.write(struct.pack("<B", lzw_minimum_code_size))
    
    # Pixel data
    for y in range(height):
        for x in range(width):
            index = int.from_bytes(rng.read(1), "big") % size_of_global_color_table
            out.write(struct.pack("<B", index))
            
    # Trailer
    trailer = 0x3B
    out.write(struct.pack("<B", trailer))
