#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(10), 'big')
    
    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<HBB', 256, 256, 0))

    # Write the global color table (all black)
    for i in range(7):
        out.write(b'\x00\x00\x00')

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x01\xff\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table (all black)
    for i in range(7):
        out.write(b'\x00\x00\x00')

    # Write the image data
    while datalen > 0:
        chunklen = min(datalen, 255)
        out.write(struct.pack('B', chunklen))
        out.write(rng.read(chunklen))
        datalen -= chunklen

    # Write the trailer
    out.write(b';')
