#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(256):
        red   = struct.pack('B', i)
        green = struct.pack('B', i)
        blue  = struct.pack('B', i)
        out.write(red + green + blue)

    # Application extension
    ext_label = b'\x21\xFF\x0B'
    app_id    = b'NETSCAPE2.0'
    auth_code = b'\x03\x01'
    loop      = struct.pack('<H', 0)
    out.write(ext_label + app_id + auth_code + loop)

    # Image block
    image_separator = b'\x2C'
    left            = struct.pack('>H', 0)
    top             = struct.pack('>H', 0)
    width           = struct.pack('>H', 1024)
    height          = struct.pack('>H',  768)
    local_flags     = struct.pack('B',   0x80)
    interlace       = struct.pack('B',   0x00)
    sort            = struct.pack('B',   0x00)
    size            = struct.pack('B',   0x00)
    out.write(image_separator + left + top + width + height + local_flags + interlace + sort + size)

    # Local color table
    for i in range(256):
        red   = struct.pack('B', i)
        green = struct.pack('B', i)
        blue  = struct.pack('B', i)
        out.write(red + green + blue)

    # Image data
    for y in range(768):
        for x in range(1024):
            pixel = struct.unpack('B', rng.read(1))[0]
            out.write(struct.pack('B', pixel))

