
import os
import sys
import random
import struct

def generate_gif(outfile, seed=None):
    """
    Generate a GIF file with the given name using the given seed.
    If no seed is provided, one will be randomly generated.
    Returns True on success, False otherwise.
    """

    if not outfile:
        print("No output filename specified.")
        return False

    if not seed:
        seed = random.randint(0, 0xffffffff)

    rng = random.Random()
    rng.seed(seed)

    try:
        out = open(outfile, 'wb')
    except IOError as e:
        print("Failed to create '%s': %s" % (outfile, str(e)))
        return False

    # Magic number
    out.write(struct.pack('<3s', "GIF"))

    # Version
    out.write(struct.pack('<3s', "89a"))

    # Logical screen descriptor
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x00 | 0x08 | 0x04)
    bg        = struct.pack('B',   0x00)
    trans     = struct.pack('B',   0x00)
    aspect    = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + trans + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H', 0)
    top       = struct.pack('>H', 0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x00 | 0x08 | 0x04)
    bg        = struct.pack('B',   0x00)
    trans     = struct.pack('B',   0x00)
    interlace = struct.pack('B',   0x00)
    sort      = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + bg + trans + interlace + sort)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    return True


