#!/usr/bin/env python3
import sys
from collections import defaultdict, deque

lzw_min_code_size = 8

def main():
    if len(sys.argv) != 2:
        print('Usage: {} <file>'.format(sys.argv[0]), file=sys.stderr)
        return 1

    with open(sys.argv[1], 'rb') as inp, \
         open(sys.argv[1] + '.lzw', 'wb') as out:
        compress(inp, out)

def compress(inp, out):
    # Initialize the dictionary
    dict = {i: bytes([i]) for i in range(lzw_min_code_size)}
    next_code = lzw_min_code_size + 1

    # Read the first byte
    b = inp.read(1)[0]

    # Encode the rest of the data
    while True:
        # Find a match
        match = None
        for k, v in dict.items():
            if v[-1:] == b and len(v) > 1:
                match = (k, v)
                break

        # If there is no match, output the current byte and add it to the
        # dictionary
        if not match:
            out.write(bytes([next_code]))
            dict[next_code] = dict[b][:-1] + b
            next_code += 1

            # Read the next byte
            try:
                b = inp.read(1)[0]
            except IndexError:
                break

        # Otherwise, output the matched code and update the dictionary
        else:
            code, prefix = match
            elif code <= lzw_min_code_size:
                out.write(bytes([code]))
            else:
                out.write(bytes([code - lzw_min_code_size]))

    # Write the trailer
    out.write(b';')