#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')
    width = int.from_bytes(rng.read(2), 'big', signed=False) + 1
    height = int.from_bytes(rng.read(2), 'big', signed=False) + 1
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))
    out.write(b'\x00\x00\xff\xff')

    # Global color table
    for i in range(768):
        out.write(rng.read(1))

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Local color table
    for i in range(256):
        out.write(rng.read(1))

    # LZW-compressed image data
    while True:
        code = int.from_bytes(rng.read(1), 'big', signed=False)
        if code == 0:
            break
        out.write(code.to_bytes(1, 'big'))
