#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Graphic control extension
    ext_code = struct.pack('B', 0xf9)
    ext_len  = struct.pack('B', 0x04)
    flags    = struct.pack('B', 0x00)
    delay    = struct.pack('<H', 0xffff)
    trans    = struct.pack('B', 0xff)
    out.write(ext_code + ext_len + flags + delay + trans)

    # Image descriptor
    desc_code = struct.pack('B', 0x2c)
    x         = struct.pack('>H', 0x0000)
    y         = struct.pack('>H', 0x0000)
    w         = struct.pack('>H', 1024)
    h         = struct.pack('>H',  768)
    flags     = struct.pack('B', 0x80)
    lct_size  = struct.pack('B', 0x00)
    out.write(desc_code + x + y + w + h + flags + lct_size)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for i in range(0, 1024 * 768):
        out.write(struct.pack('B', 0xff))

    return True

