#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Image descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\xff\x00'
    image_descriptor += struct.pack('<HH', width, height)
    image_descriptor += bytes([flags[0]])
    image_descriptor += bytes([bg[0]])
    image_descriptor += bytes([aspect[0]])
    out.write(image_descriptor)

    # Local color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Extension block
    extension_block = b'\x21\xf9\x04\x05\x00\x00\x00\x00\x00'
    out.write(extension_block)

    # Graphic control extension
    graphic_control_extension = b'\x21\xf9\x04\x01\x00\x00\x00\x00'
    delay = int.from_bytes(rng.read(2), byteorder='little')
    graphic_control_extension += struct.pack('<H', delay)
    graphic_control_extension += bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(graphic_control_extension)

    # Application extension
    application_extension = b'\x21\xff\x0b\x4e\x45\x54\x53\x43\x41\x50\x45\x32\x2e\x30\x03\x01'
    out.write(application_extension)

    # Comment extension
    comment_extension = b'\x21\xfe\x0c\x43\x6f\x6d\x6d\x65\x6e\x74\x20\x54\x65\x73\x74'
    out.write(comment_extension)

    # Trailer
    trailer = b'\x3b'
    out.write(trailer)

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_animated_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Loop count
    loop_count = b'\x21\xff\x0bNETSCAPE2.0\x03\x01'
    out.write(loop_count)

    # Image descriptor
    image_descriptor = b'\x2c\x00\x00\x00\x00\x00\xff\xff\xff\xff\x00'
    image_descriptor += struct.pack('<HH', width, height)
    image_descriptor += bytes([flags[0]])
    image_descriptor += bytes([bg[0]])
    image
        # Block size
        block_size = int.from_bytes(rng.read(1), byteorder='little')

    # Image data
    for y in range(height):
        for x in range(width):
            index = int.from_bytes(rng.read(1), byteorder='little')
            out.write(bytes([index]))

