#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Application extension
    ext_type   = bytes([0xFF])
    app_id     = bytes([0xF9])
    block_size = bytes([0x04])
    app_data   = rng.read(3)
    out.write(ext_type + app_id + block_size + app_data)

    # Image descriptor
    image_separator = bytes([0x2C])
    left            = bytes([0x00, 0x00])
    top             = bytes([0x00, 0x00])
    width           = bytes([width & 0xFF, width >> 8])
    height          = bytes([height & 0xFF, height >> 8])
    flags           = bytes([0x00])
    lct_flag        = bytes([0x00])
    interlace       = bytes([0x00])
    sort            = bytes([0x00])
    size            = bytes([0x00])
    out.write(image_separator + left + top + width + height + flags + lct_flag + interlace + sort + size)

    # Local color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Data sub-blocks
    while True:
        size = int.from_bytes(rng.read(1), byteorder='little')
        if size == 0:
            break
        data = rng.read(size)
        out.write(bytes([size]) + data)

    # Trailer
    trailer = bytes([0x3B])
    out.write(trailer)
