#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for _ in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Graphic control extension
    packed = struct.pack('BBB', 0xF9, 0x04, 0)
    out.write(packed)

    # Image descriptor
    packed = struct.pack('>BBHBB', 0x2C, 0, 0, width, height)
    out.write(packed)

    # Local color table
    for _ in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # LZW image data
    lzw_minimum_code_size = 8
    out.write(struct.pack('B', lzw_minimum_code_size))

    # TODO: Implement LZW compression
    # https://www.w3.org/Graphics/GIF/spec-gif89a.txt
    pass
