#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Application extension block
    appext_size = struct.pack('B', 11)
    appext_id   = struct.pack('B', 0xff)
    appext_data = b'\x0b\x4e\x45\x54\x53\x43\x41\x50\x45\x32\x2e\x30'
    out.write(appext_size + appext_id + appext_data)

    # Graphic control extension block
    gcext_size = struct.pack('B', 4)
    gcext_id   = struct.pack('B', 0xf9)
    gcext_flag = struct.pack('B', 0x04)
    delay      = struct.pack('<H', 10)
    gcext_term = struct.pack('B', 0x00)
    out.write(gcext_size + gcext_id + gcext_flag + delay + gcext_term)

    # Image descriptor
    idesc_separator = struct.pack('B', 0x2c)
    idesc_left      = struct.pack('>H', 0)
    idesc_top       = struct.pack('>H', 0)
    idesc_width     = struct.pack('>H', 1024)
    idesc_height    = struct.pack('>H',  768)
    lct_flag        = struct.pack('B',   0x80)
    interlace       = struct.pack('B',   0x00)
    sort           = struct.pack('B',   0x00)
    idesc_lct_size  = struct.pack('B',   0x00)
    out.write(idesc_separator + idesc_left + idesc_top + idesc_width + idesc_height + lct_flag + interlace + sort + idesc_lct_size)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for i in range(0, 1024 * 768):
        index = struct.pack('B', ord(rng.read(1)))
        out.write(index)

    return True

