#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = (rng.read(1)[0] % 50) + 1
    height = (rng.read(1)[0] % 50) + 1
    packed = rng.read(1)[0]
    global_color_table_flag = bool((packed & 0b10000000) >> 7)
    color_resolution = ((packed & 0b01110000) >> 4) + 1
    sort_flag = bool((packed & 0b00001000) >> 3)
    size_of_global_color_table = (packed & 0b00000111) + 1
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]

    out.write(struct.pack('<HHBBB', width, height, packed, background_color_index, pixel_aspect_ratio))

    if global_color_table_flag:
        for _ in range(size_of_global_color_table * size_of_global_color_table * color_resolution // 8):
            out.write(bytes([rng.read(1)[0]]))

    # Write the application extension block
    out.write(b'\x21\xf9\x04\xff\xff\xff\xff')

    # Write the graphic control extension block
    out.write(b'\x21\xf9\x04\x01\x00\x00\x00')

    # Write the image descriptor
    left = rng.read(1)[0]
    top = rng.read(1)[0]
    width = rng.read(1)[0]
    height = rng.read(1)[0]
    packed = rng.read(1)[0]
    local_color_table_flag = bool((packed & 0b10000000) >> 7)
    interlace_flag = bool((packed & 0b01000000) >> 6)
    sort_flag = bool((packed & 0b00100000) >> 5)
    size_of_local_color_table = (packed & 0b00011111) + 1

    out.write(struct.pack('<HHBBB', left, top, width, height, packed))

    if local_color_table_flag:
        for _ in range(size_of_local_color_table * size_of_local_color_table * color_resolution // 8):
            out.write(bytes([rng.read(1)[0]]))

    # Write the LZW minimum code size byte
    out.write(bytes([rng.read(1)[0]]))

    # Write the data sub-blocks
    while True:
        length = rng.read(1)[0]
        if length == 0:
            break
        out.write(bytes([length]))
        for _ in range(length):
            out.write(bytes([rng.read(1)[0]]))
