#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for _ in range(0, 256):
        out.write(struct.pack('BBB', *[rng.read(1)[0] for _ in range(0, 3)]))

    # Graphics control extension
    gce = b'\x21\xf9\x04\x08\x00\x00\xff\xff\xff\xff\x00'
    out.write(gce)

    # Image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
    out.write(idesc)

    # Local color table
    lct = b''
    for _ in range(0, 256):
        lct += struct.pack('BBB', *[rng.read(1)[0] for _ in range(0, 3)])
    out.write(lct)

    # Image data
    image_data = b''
    for y in range(0, height):
        for x in range(0, width):
            image_data += struct.pack('B', rng.read(1)[0])
    out.write(image_data)
