#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG.
def generate_gif(out: BinaryIO, rng: BinaryIO):
    # Magic number
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    out.write(struct.pack('<HHBB', 100, 100, 0xF7, 0))
    
    # Global color table
    for i in range(0xFF + 1):
        out.write(struct.pack('BBB', i, i, i))
        
    # Image block
    out.write(b'\x2C\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x00;')
    
    # Pixel data
    datalen = 100 * 100
    while datalen > 0:
        chunklen = min(256, datalen)
        out.write(rng.read(chunklen))
        datalen -= chunklen
