#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# given width and height.
def generate_gif(out: BinaryIO, width: int, height: int):
    # Write the header.
    out.write(b"GIF89a")
    out.write(struct.pack("<HHB", width, height, 0xF7))

    # Write the color table.
    for i in range(256):
        r = (i >> 5) & 0x1F
        g = (i >> 2) & 0x1F
        b = i & 0x1F
        out.write(bytes([r << 3 | r >> 2, g << 3 | g >> 2, b << 3 | b >> 2]))

    # Write the image descriptor.
    out.write(b"\x2C\x00\x00\x00\x00\x00")
    out.write(struct.pack("BB", width - 1, height - 1))
    out.write(b"\x00")

    # Write the local color table.
    for i in range(256):
        r = (i >> 5) & 0x1F
        g = (i >> 2) & 0x1F
        b = i & 0x1F
        out.write(bytes([r << 3 | r >> 2, g << 3 | g >> 2, b << 3 | b >> 2]))

    # Write the pixel data.
    for y in range(height):
        for x in range(width):
            out.write(bytes([y * width + x]))

    # Write the trailer.
    out.write(b";")


if __name__ == "__main__":
    if len(sys.argv) != 4:
        print("Usage: %s <output> <width> <height>" % sys.argv[0])
        exit(-1)

    with open(sys.argv[1], "wb") as f: