#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG. The RNG should be a binary stream that returns
# pseudo-random bytes.
def generate(rng: BinaryIO, out: BinaryIO):
    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(256, datalen)
        out.write(rng.read(chunklen))
        datalen -= chunklen
