#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(0, 256):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))

    # Image descriptor
    out.write(b'\x2C\x00\x00\x00\x00')

    # Local color table
    for i in range(0, 256):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))

    # Image data
    for y in range(0, height):
        for x in range(0, width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))