#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng.read(2)[0]
    height = 1 + rng.read(2)[0]
    global_color_table_flag = rng.read(1)[0] & 0x80
    color_resolution = rng.read(1)[0] & 0x70 >> 4
    sort_flag = rng.read(1)[0] & 0x08 >> 3
    size_of_global_color_table = rng.read(1)[0] & 0x07
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBBB', width, height, global_color_table_flag | color_resolution | sort_flag | size_of_global_color_table, background_color_index, pixel_aspect_ratio))

    # Global color table
    if global_color_table_flag != 0:
        for i in range(2 ** (size_of_global_color_table + 1)):
            red = rng.read(1)[0]
            green = rng.read(1)[0]
            blue = rng.read(1)[0]
            out.write(struct.pack('BBB', red, green, blue))

    # Application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
    loop_count = rng.read(2)[0]
    out.write(struct.pack('<H', loop_count))
    out.write(b'\x00')

    # Image descriptor
    left = rng.read(2)[0]
    top = rng.read(2)[0]
    width = rng.read(2)[0]
    height = rng.read(2)[0]
    local_color_table_flag = rng.read(1)[0] & 0x80
    interlace_flag = rng.read(1)[0] & 0x40
    sort_flag = rng.read(1)[0] & 0x08 >> 3
    size_of_local_color_table = rng.read(1)[0] & 0x07
    out.write(struct.pack('<HHHHBB', left, top, width, height, local_color_table_flag | interlace_flag | sort_flag | size_of_local_color_table, 0))

    # Local color table
    if local_color_table_flag != 0:
        for i in range(2 ** (size_of_local_color_table + 1)):
            red = rng.read(1)[0]
            green = rng.read(1)[0]
            blue = rng.read(1)[0]
            out.write(struct.pack('BBB', red, green, blue))

    # Image data
    lzw_minimum_code_size = rng.read(1)[0]
    out.write(lzw_minimum_code_size.to_bytes(1, 'big'))
    while True:
        code = rng.read(1)[0]
        if code == 0x00:
            break
        else:
            out.write(code.to_bytes(1, 'big'))
