#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0xff)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(256):
        out.write(struct.pack('BBB', *[i] * 3))

    # Graphic control extension
    gce = b'\x21\xf9\x04\x00\x00\xff\xff\xff\x00'
    out.write(gce)

    # Image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
    out.write(idesc)

    # Local color table
    lct = b''
    for i in range(256):
        lct += struct.pack('BBB', *[i] * 3)
    out.write(lct)

    # Pixel data
    pixels = b''
    for y in range(768):
        for x in range(1024):
            pixels += struct.pack('B', rng.read(1)[0])
    out.write(pixels)

    # Trailer
    out.write(b'\x3b')

