#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(256):
        red   = struct.pack('B', i)
        green = struct.pack('B', i)
        blue  = struct.pack('B', i)
        out.write(red + green + blue)

    # Application extension
    block_size = struct.pack('B', 0x0b)
    app_id     = b'FUZZY'
    auth_code  = b'\x00\x00\x00'
    data       = b''
    while len(data) < 255 - 11:
        data += rng.read(1)
    if not data:
        return False
    out.write(block_size + app_id + auth_code + data)

    # Image descriptor
    image_separator = struct.pack('B', 0x2c)
    left            = struct.pack('>H', 0)
    top             = struct.pack('>H', 0)
    width           = struct.pack('>H', 1024)
    height          = struct.pack('>H',  768)
    flags           = struct.pack('B',   0x80)
    local_color_table_size = struct.pack('B', 0)
    interlaced              = struct.pack('B', 0)
    sort                    = struct.pack('B', 0)
    size_of_local_ct        = struct.pack('B', 0)
    out.write(image_separator + left + top + width + height + flags + local_color_table_size + interlaced + sort + size_of_local_ct)

    # LZW minimum code size
    lzw_minimum_code_size = struct.pack('B', 8)
    out.write(lzw_minimum_code_size)

    # Data sub-blocks
    while True:
        length = rng.read(1)
        if not length:
            break
        length = ord(length)
        if length == 0:
            continue
        data = rng.read(length)
        if not data:
            break
        out.write(struct.pack('B', length))
        out.write(data)

    # Trailer
    trailer = struct.pack('B', 0x3b)
    out.write(trailer)

    return True

