#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        red   = struct.pack('B', i)
        green = struct.pack('B', i)
        blue  = struct.pack('B', i)
        out.write(red + green + blue)

    # Graphic control extension
    gce = struct.pack('BBB', 0x21, 0xf9, 0x04)
    delay = struct.pack('<H', 10)
    trans = struct.pack('B', 0xff)
    blocksize = struct.pack('B', 0x00)
    out.write(gce + delay + trans + blocksize)

    # Image descriptor
    idesc = struct.pack('BBHH', 0x2c, 0x00, 0x00, 0x00, 0x00)
    lsd   = struct.pack('BBB', 0x80 | 0x00 | 0x00)
    out.write(idesc + lsd)

    # Local color table
    for i in range(0, 256):
        red   = struct.pack('B', i)
        green = struct.pack('B', i)
        blue  = struct.pack('B', i)
        out.write(red + green + blue)

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    return True


