#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Read the size of the image.
    size = int.from_bytes(rng.read(10), byteorder='little')
    
    # Write the header.
    out.write(b'GIF89a')
    out.write(struct.pack('<H', size))
    out.write(struct.pack('<H', size))
    out.write(b'\x00\x00')
    
    # Write the color table.
    for i in range(256):
        out.write(struct.pack('BBB', i, i, i))
        
    # Write the image data.
    for y in range(size):
        for x in range(size):
            out.write(struct.pack('B', rng.read(1)[0]))
            
    # Write the trailer.
    out.write(b';')
