#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    colors = [
        0x000000, 0xFFFFFF, 0x0000FF, 0xFF0000, 0x00FF00, 0xFF00FF,
        0x00FFFF, 0xFFFF00, 0x7F7F7F, 0xC0C0C0, 0x404040, 0x808080,
        0xFFA500, 0xFFD700, 0xFA8072, 0xE9967A, 0xF0FFF0, 0xF5DEB3,
        0xDEB887, 0xD2B48C, 0xBC8F8F, 0xF4A460, 0xDAA520, 0xBDB76B,
        0xADFF2F, 0xFFDAB9, 0xFFE4E1, 0xFFE4B5, 0xFFDEAD, 0xD8BFD8,
        0xDA70D6, 0xEEE8AA, 0xCE93D8, 0x808000, 0xFF00FF, 0x00FF00,
        0x00FFFF, 0xFF0000, 0x0000FF, 0xFFFF00, 0x000000, 0xFFFFFF,
    ]
    for c in colors:
        out.write(struct.pack('<BBBB', *((c >> i*8) & 0xff for i in range(3))))

    # Application extension
    ext = b'\x21\xFF\x0BNETSCAPE2.0\x03\x01'
    out.write(ext)

    # Image descriptor
    idesc = b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00'
    out.write(idesc)

    # Local color table
    lct = b''
    for _ in range(0, 3):
        lct += struct.pack('<BBBB', *((colors[i] >> j*8) & 0xff for j in range(3)))
    out.write(lct)

    # Image data
    image = b''
    for y in range(0, height):
        for x in range(0, width):
            image += struct.pack('<BB', *((colors[i] >> j*8) & 0xff for j in range(3)))
    out.write(image)

    # Trailer
    trailer = b';'
    out.write(trailer)
