#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Application extension block
    appext_block_size = struct.pack('B', 0x0b)
    appext_id         = struct.pack('B', 0xfe)
    appext_auth       = 'FUZZY'.encode()
    appext_data       = rng.read(11)
    out.write(appext_block_size + appext_id + appext_auth + appext_data)

    # Image descriptor
    image_desc_start = struct.pack('B', 0x2c)
    left             = struct.pack('>H', 0)
    top              = struct.pack('>H', 0)
    width            = struct.pack('>H', 1024)
    height           = struct.pack('>H', 768)
    flags            = struct.pack('B', 0x80)
    out.write(image_desc_start + left + top + width + height + flags)

    # Local color table
    local_color_table = rng.read(3 * 256)
    out.write(local_color_table)

    # LZW minimum code size
    lzw_min_code_size = struct.pack('B', 0x08)
    out.write(lzw_min_code_size)

    # LZW compressed data
    lzw_compressed_data = rng.read(1024 * 768 // 8)
    out.write(lzw_compressed_data)

    return True

