#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(out: BinaryIO, rng: callable):
    # Write header
    out.write(b'GIF89a')
    width = 1 + int.from_bytes(rng(2), 'big', signed=False) % 640
    height = 1 + int.from_bytes(rng(2), 'big', signed=False) % 480
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\xF7\xFF\xFF')
    
    # Write frames
    for i in range(1 + int.from_bytes(rng(1), 'big', signed=False)):
        # Write frame header
        out.write(b'\x2C')
        
        # Write frame dimensions
        out.write(struct.pack('<H', width))
        out.write(struct.pack('<H', height))
        
        # Write frame flags
        out.write(b'\x08\x00')
        
        # Write color table
        out.write(b'\xFF\xFF\xFF')
        out.write(b'\x00\x00\x00')
        out.write(b'\xFF\xFF\xFF')
        out.write(b'\x00\x00\x00')
        
        # Write image data
        for y in range(height):
            for x in range(width):
                out.write(b'\x00')
            
        # Write terminator
        out.write(b';')
