#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') + 1
    
    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(b'\x00\x00\x00\x00\x00\x00\xff\x00\x00\xff')

    # Write the global color table (256 colors)
    for i in range(256):
        out.write(struct.pack('BBB', *[i] * 3))

    # Write the application extension block
    appext = b'\x21\xFF\x0BNETSCAPE2.0\x03\x01'
    out.write(appext)

    # Write the loop count
    out.write(b'\x00\x00')

    # Write the trailer
    out.write(b'\x3B')

    # Write the image data
    while datalen > 0:
        chunklen = min(datalen, 255)
        out.write(struct.pack('B', chunklen))
        
        for _ in range(chunklen):
            out.write(struct.pack('B', rng.read(1)[0]))
            
        datalen -= chunklen

