#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xFF)
    aspect = struct.pack('B',   0xFF)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        out.write(struct.pack('BBB', *[i]*3))

    # Graphic control extension
    gce = struct.pack('BB', 0xF9, 0x04)
    delay = struct.pack('<H', 0xFFFF)
    trans = struct.pack('B', 0xFF)
    blocksize = struct.pack('B', 0x00)
    out.write(gce + delay + trans + blocksize)

    # Image descriptor
    idesc = struct.pack('BBBHH', 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00)
    out.write(idesc)

    # Local color table
    lct = struct.pack('BBB', 0x00, 0x00, 0x00)
    out.write(lct)

    # Image data
    while True:
        try:
            out.write(rng.read(1))
        except Exception as e:
            print("Error writing to output stream", file=sys.stderr)
            return False

