#!/usr/bin/env python3

import struct
from typing import BinaryIO

def gif_to_gif(inp:BinaryIO, out:BinaryIO):
    rng = iter(inp.read())

    # Header
    header = b'GIF89a'
    out.write(header)

    # Logical screen descriptor
    width                 = int.from_bytes(rng.read(2), byteorder='little')
    height                = int.from_bytes(rng.read(2), byteorder='little')
    global_color_table_size= int.from_bytes(rng.read(1), byteorder='little')
    flags                 = [int.from_bytes(rng.read(1), byteorder='little'),]
    background_color      = int.from_bytes(rng.read(1), byteorder='little')
    pixel_aspect_ratio    = int.from_bytes(rng.read(1), byteorder='little')
    out.write(struct.pack('<HHBHBB', width, height, global_color_table_size, flags[0], background_color, pixel_aspect_ratio))

    # Global color table
    for i in range((global_color_table_size ** 3)):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Graphic control extension
    label = b'\xF9\x04'
    flags = [int.from_bytes(rng.read(1), byteorder='little'),]
    delay = int.from_bytes(rng.read(2), byteorder='little')
    transparent_color     = int.from_bytes(rng.read(1), byteorder='little')
    out.write(b'\x21')
    out.write(label)
    out.write(struct.pack('<BBHB', flags[0], delay, transparent_color, 0))

    # Application extension
    label = b'\xFF\x0BNETSCAPE2.0'
    out.write(b'\x21')
    out.write(label)
    out.write(b'\x03\x01')
    out.write(b'\x00')

    # Plain text extension
    label = b'\x01\x00'
    grid_left             = int.from_bytes(rng.read(2), byteorder='little')
    grid_top              = int.from_bytes(rng.read(2), byteorder='little')
    grid_width            = int.from_bytes(rng.read(2), byteorder='little')
    grid_height           = int.from_bytes(rng.read(2), byteorder='little')
    cell_width            = int.from_bytes(rng.read(1), byteorder='little')
    cell_height           = int.from_bytes(rng.read(1), byteorder='little')
    text_foreground       = int.from_bytes(rng.read(1), byteorder='little')
    text_background       = int.from_bytes(rng.read(1), byteorder='little')
    out.write(b'\x21')
    out.write(label)
    out.write(struct.pack('<HHHHBB', grid_left, grid_top, grid_width, grid_height, cell_width, cell_height))
    out.write(struct.pack('<BB', text_foreground, text_background))

    # Comment extension
    label = b'\xFE\x00'
    out.write(b'\x21')
    out.write(label)

    # Graphic control extension
    label = b'\xF9\x04'
    flags = [int.from_bytes(rng.read(1), byteorder='little'),]
    delay = int.from_bytes(rng.read(2), byteorder='little')
    transparent_color     = int.from_bytes(rng.read(1), byteorder='little')
    out.write(b'\x21')
    out.write(label)
    out.write(struct.pack('<BBHB', flags[    pixel_aspect_ratio    = int.from_bytes(rng.read(1), byteorder='little')
    out.write(struct.pack('<HHHHBB', left, top, width, height, flags[0], lct[0]))

    # Local color table
    for i in range((local_color_table_size ** 3)):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Block size
    block_size = int.from_bytes(rng.read(1), byteorder='little')
    while block_size!= 0:

        # Extension
        label = bytes([int.from_bytes(rng.read(1), byteorder='little')])
        length = int.from_bytes(rng.read(1), byteorder='little')
        data = rng.read(length)
        out.write(b'\x21')
        out.write(label)
        out.write(data)

        # Block size
        block_size = int.from_bytes(rng.read(1), byteorder='little')

    # Image data
    for y in range(height):
        for x in range(width):
            index = int.from_bytes(rng.read(1), byteorder='little')
            out.write(bytes([index]))

