#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(2)[0] + 1
    height = rng.read(2)[0] + 1
    out.write(struct.pack('<HHBB', width, height, 0xF7, 0))

    # Global color table
    for i in range(256):
        out.write(bytes([i, i, i]))

    # Image descriptor
    out.write(struct.pack('<HHHBB', 0, 0, width, height, 0))

    # Local color table
    for i in range(256):
        out.write(bytes([i, i, i]))

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))