#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    out.write(struct.pack('<H', 0x200))
    out.write(b'\x01')
    out.write(b'\x00')
    out.write(b'\xff\x0b')

    # Write the global color table
    for i in range(7):
        out.write(b'\xff\xff\xff')

    # Write the application extension block
    out.write(b'\x21\xf9\x04\xff\xff\xff\xff\xff\xff\xff\xff')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00')

    # Write the local color table
    for i in range(7):
        out.write(b'\xff\xff\xff')

    # Write the image data
    while True:
        lzw_min_code_size = rng.read(1)[0] & 0x7f
        out.write(bytes([lzw_min_code_size]))

        # Read the LZW code stream and write it as-is
        while True:
            code = rng.read(1)[0]
            if code == 0:
                break
            else:
                out.write(bytes([code]))

    # Write the trailer
    out.write(b';')
