#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H', 768)
    packed = b'\x00\x00\xff\x00\x00\xff\x00\x00'
    out.write(width + height + packed)

    # Global color table
    for i in range(0, 256):
        out.write(struct.pack('BBB', i, i, i))

    # Graphics control extension
    gce = b'\x21\xf9\x04\x04\x00\x00\x00\x00'
    out.write(gce)

    # Image descriptor
    idesc = b'\x2c\x00\x00\x00\x00\x00\x00\x00'
    out.write(idesc)

    # Local color table
    lct = b'\x00\x00\x00\x00\x00\x00\x00\x00' * 256
    out.write(lct)

    # Pixel data
    while True:
        chunk_size = rng.read(1)[0] & 0xFF
        if not chunk_size:
            break
        out.write(rng.read(chunk_size))

    # Trailer
    out.write(b';')

