#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the size of the output file
    size = int.from_bytes(rng.read(10), 'big')

    # Write the header
    out.write('GIF89a'.encode())
    out.write(struct.pack('<HH', 400, 400))
    out.write(b'\xF7\x00\x00\xFF\xFF\xFF')

    # Write the graphic control extension
    out.write(b'\x21\xF9\x04\x01\x00\x00\x00\x00')

    # Write the application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the image data length
    datalen = size - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 - 13 - 10 -    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(256, datalen)
        out.write(rng.read(chunklen))
        datalen -= chunklen
