#!/usr/bin/env python3

import os, sys, re
from datetime import date

def main():
    if len(sys.argv)!= 2:
        print("Usage: {} <path>".format(os.path.basename(__file__)))
        return

    path = sys.argv[1]
    with open(path, 'r') as f:
        lines = [line for line in f.readlines() if not line.startswith('//')]

    # Find the first comment block and extract the version number from it.
    match = re.search(r'\/\*\s*Version:\s+(\d+\.\d+\.\d+)\s*\*\/', lines[0])
    if not match:
        raise Exception("Could not find version number")
    version_number = match.group(1)

    # Find the second comment block and extract the release date from it.
    match = re.search(r'\/\*\s*Release Date:\s+(.*)\s*\*\/', lines[1])
    if not match:
        raise Exception("Could not find release date")
    release_date = match.group(1).strip()

    today = date.today().strftime('%B %d, %Y')
    if release_date == today:
        print("{} (unreleased)".format(version_number))
    else:
        print("{} ({})".format(version_number, release_date))


if