#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H', 0)
    top       = struct.pack('>H', 0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x00 | 0x08 | 0x10)
    frame_cnt = struct.pack('B',   0x00)
    delay     = struct.pack('>H', 0x00)
    trans     = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + frame_cnt + delay + trans)

    # LZW minimum code size
    min_code_size = struct.pack('B', 0xf9)
    out.write(min_code_size)

    # Pixel data
    lzw_table = [None] * 256
    lzw_table[0] = ''
    lzw_table[1] =''

    last_str = None
    while True:
        if not last_str or len(last_str) == 0:
            new_str = chr(ord(rng.read(1)) % 256)

            if new_str == '!':
                continue

            last_str = new_str
            continue

        new_char = chr(ord(rng.read(1)) % 256)

        if new_char!= '!':
            last_str += new_char
        else:
            out.write(lzw_table[len(last_str)].encode())

            if len(last_str) >= 2**12:
                print("Error: Code length exceeded maximum")
                return False

            if len(lzw_table) < 2**12:
                lzw_table.append(last_str)

            last_str = None

    out.write(b'\x00\x3b')

    return True


if