#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = rng.read(10)
    (width, height,) = struct.unpack("<HH", header[6:8])
    datalen = width * height // 4 + 1

    while datalen > 0:

        # Write up to 256 bytes at a time
        chunklen = min(256, datalen)
        out.write(rng.read(chunklen))
        datalen -= chunklen
