#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    global_color_table_size = int.from_bytes(rng.read(1), byteorder='little')
    if global_color_table_size == 0:
        local_color_table_size = int.from_bytes(rng.read(1), byteorder='little')**3
    else:
        local_color_table_size = global_color_table_size**3
    # Local color table
    for i in range((local_color_table_size ** 3)):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Block size
    block_size = int.from_bytes(rng.read(1), byteorder='little')
    while block_size!= 0:

        # Extension
        label = bytes([int.from_bytes(rng.read(1), byteorder='little')])
        length = int.from_bytes(rng.read(1), byteorder='little')
        data = rng.read(length)
        out.write(b'\x21')
        out.write(label)
        out.write(data)

        # Block size
        block_size = int.from_bytes(rng.read(1), byteorder='little')

    # Image data
    for y in range(height):
        for x in range(width):
            index = int.from_bytes(rng.read(1), byteorder='little')
            out.write(bytes([index]))

