#!/usr/bin/env python3

import os
import sys
import struct
from PIL import Image


def main():
    if len(sys.argv)!= 2:
        print("Usage: {} <input_file>".format(os.path.basename(__file__)))
        return

    with open(sys.argv[1], "rb") as f:
        data = bytearray(f.read())

        # Check magic number
        assert data[:4] == b"BMP\x00", "Invalid file format!"

        # Read header
        (
            _,
            width,
            height,
            planes,
            bits_per_pixel,
            compression,
            image_size,
            hres,
            vres,
            colors,
            important_colors,
        ) = struct.unpack("<LHHLLLLHHL", data[10:54])

        # Check parameters
        assert planes == 1 and bits_per_pixel in [8, 24], "Unsupported format!"
        assert compression == 0, "Compression not supported!"

        # Skip to pixel data
        offset = 54 + ((width * bits_per_pixel // 8 + 3) & ~3)
        data = data[offset:]

        # Convert to RGB or RGBA
        pixels = []
        for i in range(height):
            row = []

            if bits_per_pixel == 8:
                row += [(data[i::height])]
            else:
                for j in range(0, width * 3, 3):
                    r, g, b = data[(i * width * 3 + j)::(height * width * 3)]

                    row += [[r, g, b]]

            pixels += [row]

        img = Image.new("RGB" if bits_per_pixel == 24 else "RGBA", (width, height))
        img.putdata([tuple(p) for p in pixels])
        img.save(sys.stdout.buffer, format="png")


if