#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the given parameters.
def generate_gif(
    out: BinaryIO, width: int = 1024, height: int = 576, color_depth: int = 8
) -> None:

    # Header
    out.write("GIF89a".encode())
    out.write(struct.pack("<HH", width, height))
    out.write("\x87\x00\x00".encode())

    # Logical screen descriptor
    out.write(",".encode())
    out.write(struct.pack("<BBBHB", color_depth, 0, 0, 0, 0))

    # Global color table (black and white only)
    for i in range(color_depth):
        if i == 0 or i == 2 ** color_depth - 1:
            out.write(b"\xff")
        else:
            out.write(b"\x00")

    # Image descriptor
    out.write(";".encode())
    out.write(struct.pack("<HHHHB", 0, 0, width, height, 0))

    # Graphic control extension
    out.write("!".encode())
    out.write(struct.pack("<BBHBB", 0xf9, 4, 0, 0, 0))

    # Pixel data
    for y in range(height):
        for x in range(width):
            if ((y * width + x) % 2) == 0:
                out.write(b"\x00")
            else:
                out.write(b"\xff")


if