#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(out: BinaryIO, rng: bytes):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + rng[0] % 256
    height = 1 + rng[1] % 256
    flags = b'\x87\x00' if rng[2] & 1 else b'\x00\x00'
    background = rng[3] % 256
    aspect_ratio = 0
    out.write(struct.pack('<HHBBB', width, height, flags, background, aspect_ratio))

    # Global color table
    num_colors = min((width * height), 256)
    for i in range(num_colors):
        r = rng[(i*4)+0] % 256
        g = rng[(i*4)+1] % 256
        b = rng[(i*4)+2] % 256
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left = 0
    top = 0
    flags = b'\x80\x00'
    delay = 0
    transparent_color = -1
    out.write(struct.pack('<HHBHB', left, top, width, height, flags, delay, transparent_color))

    # Image data
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng[(y*width+x)]]))


if