#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
	width  = int.from_bytes(rng.read(2), 'big') % (1 << 10) + 1
	height = int.from_bytes(rng.read(2), 'big') % (1 << 10) + 1

	out.write(header) # Header
	out.write(struct.pack('<HH', width, height)) # Logical screen descriptor
	out.write(b'\x87\x00\x00') # Global color table with 256 entries and no sort flag set
	out.write(b'\x00') # Background color index
	out.write(b'\x00') # Pixel aspect ratio

    for y in range(height):
		for x in range(width):
			out.write(b'\xff') # Set pixel to white
