#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom) and returns it as bytes.
def generate_gif(out: BinaryIO, rng):
    # Header
    out.write(b"GIF89a")

    width = 120 + int.from_bytes(rng(1), "little") % 50
    height = 60 + int.from_bytes(rng(1), "little") % 40

    # Logical Screen Descriptor
    out.write(struct.pack("<HHBBB", width, height, 0xF7, 0x00, 0x00))

    # Global Color Table
    for _ in range(0, 256 * 3 // 4):
        out.write(rng(1))

    # Image Descriptor
    out.write(struct.pack("<BHHBB", 0x2C, 0, 0, 0, 0))

    # Local Color Table
    for _ in range(0, 256 * 3 // 4):
        out.write(rng(1))

    # Image Data
    for y in range(height):
        for x in range(width):
            out.write(rng(1))

    return out.getvalue()


if