#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 25 + 10
    packed = bytes([width & 0xff,
                    (height >> 8) | ((width << 3) & 0xf8),
                    0x87])
    out.write(packed)

    # Global color table
    for _ in range(0, 2**((packed[1] & 0x07)+1)):
        rgb = [rng.read(1)[0],
               rng.read(1)[0],
               rng.read(1)[0]]

        out.write(bytes(rgb))

    # Image descriptor
    packed = b'\x2c\x00\x00\x00'
    packed += bytes([(width >> 8) | 0x80,
                     width & 0xff,
                     0x00,
                     (height >> 8) | 0x80,
                     height & 0xff])
    out.write(packed)

    # LZW minimum code size
    out.write(b'\x02')

    # Data sub-block
    while True:
        block_size = rng.read(1)[0]
        if block_size == 0x00:
            break
        
        out.write(bytes([block_size]))
        out.write(rng.read(block_size))


if