#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header.
    header = rng.read(6)

    # Write the header to the output.
    out.write(header)

    # Read the rest of the header and write it to the output.
    while True:

        byte = rng.read(1)
        if len(byte) == 0 or byte[0] == b'\x2C':
            break;
        else:
            out.write(byte)

    # Read the image descriptor.
    imgdesc = rng.read(9)
    out.write(imgdesc)

    # Read the color table.
    for i in range(0, (2 ** int(imgdesc[8]))):
        rgb = [int(rng.read(1)[0]) for j in range(0, 3)]
        out.write(struct.pack('BBB', *rgb))

    # Read the LZW minimum code size.
    lzwmincodesize = int(rng.read(1)[0])
    out.write(bytes([lzwmincodesize]))

    # Read the compressed data.
    while True:

        blocktype = int(rng.read(1)[0])
        if blocktype == 0:

            # End of stream marker.
            return

        elif blocktype == 1:

            # Image separator.
            continue

        elif blocktype == 2:

            # Local color table.
            raise NotImplementedError("Local color tables are not supported.")

        elif blocktype >= 3 and blocktype <= 4:

            # Image data.
            length = int(rng.read(1)[0]) + 1
            data = rng.read(length)
            out.write(data)


if