#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

def main(argv):
    if len(argv)!= 2:
        print("Usage: {} <file>".format(argv[0]), file=sys.stderr)
        return 1

    with open(argv[1], "rb") as f:
        data = read_data(f)

        for i, entry in enumerate(data["entries"]):
            print("Entry {}".format(i))

            for j, (name, value) in enumerate(entry.items()):
                print("\t{}: {}".format(name, value))


def read_data(f: BinaryIO) -> dict:
    magic, version, num_entries = struct.unpack("<III", f.read(4*3))

    assert magic == 0x5448494E # THIN
    assert version == 1

    entries = []
    for _ in range(num_entries):
        name_len, = struct.unpack("<I", f.read(4))
        name = f.read(name_len).decode()

        type_, size = struct.unpack("<II", f.read(4*2))

        if type_ == 0:
            value = None
        elif type_ == 1:
            value = struct.unpack("<Q", f.read(8))[0]
        else:
            raise ValueError("Unknown type {}".format(type_))

        entries.append({
            "name": name,
            "type": type_,
            "size": size,
            "value": value,
        })

    return {
        "magic": magic,
        "version": version,
        "num_entries": num_entries,
        "entries": entries,
    }


if