#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    depth  = struct.pack('B',    8)
    flags  = struct.pack('B',     0)
    bgcol  = struct.pack('BBBB', *([0] * 4))
    aspect = struct.pack('B',     0)

    out.write(width + height + depth + flags + bgcol + aspect)

    # Global color table
    out.write(bytes([0] * 3 * 256))

    # Graphics control extension
    out.write(b'\x21\xF9\x04')
    out.write(b'\x01\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')
    out.write(b'\x00\x00')

    # Plain text extension
    out.write(b'\x21\x01\x00\x00')

    # Application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0')
    out.write(b'\x03\x01')
    out.write(b'\x00\x00\x00')
    out.write(b'\x00\x00\x00')
    out.write(b'\x00')

    # Comment extension
    out.write(b'\x21\xFE\x00')

    # Trailer
    out.write(b';')

    # Frames
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    return True


