#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read in the header.
    header = bytearray()
    for i in range(6):
        header += rng.read(1)

    # Write it to the output.
    out.write(header)

    # Read in the logical screen descriptor and write it back out.
    lsd = bytearray()
    for i in range(7):
        lsd += rng.read(1)
    out.write(lsd)

    # Determine how many frames we need to generate.
    width = (struct.unpack('>H', lsd[4:6])[0] + 7) // 8
    height = struct.unpack('>H', lsd[6:8])[0]
    num_frames = int((width * height / 256))

    # Generate each frame.
    for _ in range(num_frames):

        # Write the image descriptor.
        idd = bytearray()
        for i in range(9):
            idd += rng.read(1)
        out.write(idd)

        # Write the local color table.
        lct = bytearray()
        for i in range(3 * 256):
            lct += rng.read(1)
        out.write(lct)

        # Write the image data.
        imgdata = bytearray()
        for i in range(height):

            # Each row contains two bytes per pixel.
            for j in range(width):
                imgdata += rng.read(2)

        out.write(imgdata)


if