#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    header = b'GIF89a' + bytes([0x12, 0x34]) + \
             bytes([0x56, 0x78, 0x9A, 0xBC, 0xDE, 0xF0])
    out.write(header)

    # Logical Screen Descriptor
    lsd = [
        0b10000000 | 0b00000000,      # Size of global color table
        0x00,                       # Background Color Index
        0x00,                       # Pixel aspect ratio
        0x00, 0x00,                 # Width and height in pixels
        0x00,                       # Global color table size
        0x00                        # Sorting flag
    ]
    out.write(bytes(lsd))

    # Image descriptor
    id = [
        0x00,                       # Left position
        0x00,                       # Top position
        0x00, 0x00,                 # Width and Height in pixels
        0b00000000,                 # Local color table flag
        0b00000000,                 # Interlace flag
        0b00000000,                 # Sorted color table flag
        0x00                        # LZW minimum code size
    ]
    out.write(bytes(id))

    # Image data
    for i in range(0, 0xFFFF):

        # Write a single byte to the image data section
        out.write(bytes([rng.read(1)[0]]))

        if ((i % 0xFFF) == 0):
            print('{:d} / {:d}'.format(i, 0xFFFF), end='\r')

            # Reposition to the start of the image data section
            out.seek(-len(id), os.SEEK_CUR)

            # Update the image descriptor with new dimensions
            id[2] = (i >> 8) & 0xFF
            id[3] = (i     ) & 0xFF
            out.write(bytes(id))

            # Reposition back to the end of the image data section
            out.seek(0, os.SEEK_END)

    return True



if