#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] & 0x7F | ((rng.read(1)[0] & 0x7F) << 8)
    height = rng.read(1)[0] & 0x7F | ((rng.read(1)[0] & 0x7F) << 8)

    packed = bytearray()
    packed += b'\x80' if rng.read(1)[0] > 127 else b'\x00'
    packed += b'\x00' if rng.read(1)[0] > 127 else b'\xC0'
    packed += b'\x00' if rng.read(1)[0] > 127 else b'\xE0'
    packed[0] |= rng.read(1)[0] & 0x07
    packed[1] |= rng.read(1)[0] & 0x3F
    packed[2] |= rng.read(1)[0] & 0x1F

    out.write(packed)
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))

    # Global color table
    for _ in range(0, 2 ** int((packed[0] >> 4) + 1)):
        red   = rng.read(1)[0]
        green = rng.read(1)[0]
        blue  = rng.read(1)[0]

        out.write(red.to_bytes(1, 'little'))
        out.write(green.to_bytes(1, 'little'))
        out.write(blue.to_bytes(1, 'little'))

    # Image descriptor
    packed = bytearray()
    packed += b'\x2C'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x00'
    packed += b'\x00\x0            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))