#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = b'\x00\x00'
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = rng.read(1)[0] & 0xf8
        g = rng.read(1)[0] & 0xfc
        b = rng.read(1)[0] & 0xf8

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H',      0)
    top       = struct.pack('>H',      0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',       0x80)
    trans_idx = struct.pack('B',        0x00)
    out.write(left + top + width + height + flags + trans_idx)

    # LZW minimum code size
    min_code_size = rng.read(1)[0] & 0x0f
    out.write(min_code_size.to_bytes(1, 'big'))

    # Pixel data
    lzw_table = [None] * 4096
    lzw_table[0] = b'\x00'
    lzw_table[1] = b'\x01'
    lzw_table[2] = b'\x02'
    lzw_table[3] = b'\x03'
    lzw_table[4] = b'\x04'
    lzw_table[5] = b'\x05'
    lzw_table[6] = b'\x06'
    lzw_table[7] = b'\x07'
    lzw_table[8] = b'\x08'
    lzw_table[9] = b'\t'
    lzw_table[10] = b'\n'
    lzw_table[11] = b'\x0b'
    lzw_table[12] = b'\x0c'
    lzw_table[13] = b'\r'
    lzw_table[14] = b'\x0e'
    lzw_table[15] = b'\x0f'
    lzw_table[16] = b'\x10'
    lzw_table[17] = b'\x11'
    lzw_table[18] = b'\x12'
    lzw_table[19] = b'\x13'
    lzw_table[20] = b'\x14'
    lzw_table[21] = b'\x15'
    lzw_table[22] = b'\x16'
    lzw_table[23] = b'\x17'
    lzw_table[24] = b'\x18'
    lzw_table[25] = b'\x19'
    lzw_table[26] = b'\x1a'
    lzw_table[27] = b'\x1b'
    lzw_table[28] = b'\x1c'
    lzw_table[29] = b'\x1d'
    lzw_table[30] = b'\x1e'
    lzw_table[31] = b'\x1f'
    lzw_table[32] = b' '
    lzw_table[33] = b'!'
    lzw_table[34] = b'"'
    lzw_table[35] = b'#'
    lzw_table[36] = b'$'
    lzw_table[37] = b'%'
    lzw_table[38] = b'&'
    lzw_table[39] = b"'"
    lzw_table[40] = b'('
    lzw_table[41] = b')'
    lzw_table[42] = b'*'
    lzw_table[43] = b'+'
    lzw_table[44] = b','
    lzw_table[45] = b'-'
    lzw_table[46] = b'.'
    lzw_table[47] = b'/'
    lzw_table[48] = b'0'
    lzw_table[49] = b'1