#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big') & 0x7fff
    print('datalen:', datalen)

    # Write the header to the output
    out.write(header)

    # Generate the palette
    palette = [struct.pack('<BBB', *rng.read(3)) for _ in range(256)]
    out.write(b''.join(palette))

    # Write the logical screen descriptor
    width = (datalen + 1) % 256
    height = ((datalen + 1) // 256) % 256
    flags = 0x80 | 0x40 | 0x20 | 0x10 | 0x08 | 0x04 | 0x01
    gctsize = 0xff
    bgindex = 0
    aspect = 0
    lsd = struct.pack('<HHBBBHBB',
            0xffff, 0xffff, width, height, flags, gctsize, bgindex, aspect)
    out.write(lsd)

    # Write the global color table
    out.write(b'\x00\x00\x00')

    # Write the image descriptor
    left = 0
    top = 0
    width = 1
    height = 1
    flags = 0x80 | 0x40 | 0x20 | 0x10 | 0x08 | 0x04 | 0x01
    lctsize = 0xff
    interlace = 0
    sort = 0
    reserved = 0
    idesc = struct.pack('<HHHHBCC',
            left, top, width, height, flags, lctsize, interlace, sort, reserved)
    out.write(idesc)

    # Write the local color table
    out.write(b'\x00\x00\x00')

    # Write the image data
    out.write(rng.read(datalen))


if