#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Loop forever until we get to an end of image marker
    while True:
        # Block size
        block_size = int.from_bytes(rng.read(1), byteorder='little')

        if block_size == 0x3B: break

        # Subblocks
        subblock_count = int.from_bytes(rng.read(1), byteorder='little')
        for _ in range(subblock_count):
            subblock_size = int.from_bytes(rng.read(1), byteorder='little')
            subblock      = rng.read(subblock_size)

            # Graphics control extension
            if subblock[:4] == b'\x21\xF9\x04':
                out.write(subblock)

                # Plain text extension
                if subblock[-1:]!= b'\x00':
                    out.write(b'\x00')

                continue

            # Application extension
            elif subblock[:11] == b'\xFF\x0BNETSCAPE':
        # Extension
        label = bytes([int.from_bytes(rng.read(1), byteorder='little')])
        length = int.from_bytes(rng.read(1), byteorder='little')
        data = rng.read(length)
        out.write(b'\x21')
        out.write(label)
        out.write(data)

        # Block size
        block_size = int.from_bytes(rng.read(1), byteorder='little')

    # Image data
    for y in range(height):
        for x in range(width):
            index = int.from_bytes(rng.read(1), byteorder='little')
            out.write(bytes([index]))

