#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for _ in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image descriptor
    packed = struct.pack('>HHBB', 0, 0, 0, 0)
    out.write(packed)

    # LZW minimum code size
    out.write(b'\x08')

    # Image data
    while True:
        index = rng.read(1)[0] % 256

        if index == 0xFF:
            out.write(index.to_bytes(1, 'little'))
        else:
            out.write(struct.pack('B', index))