#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0xF + 1
    height = (rng.read(1)[0] & 0xF) << 4 | rng.read(1)[0] & 0xF + 1
    out.write(struct.pack('<HHBBB', width * 10, height * 10, 0x80, 0x00, 0x00))

    # Write the global color table
    for i in range(768):
        out.write(bytes([(i >> j & 1) * 0xFF for j in reversed(range(8))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(8, 16))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(16, 24))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(24, 32))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(32, 40))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(40, 48))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(48, 56))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(56, 64))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(64, 72))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(72, 80))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(80, 88))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(88, 96))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(96, 104))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(104, 112))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(112, 120))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(120, 128))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(128, 136))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(136, 144))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(144, 152))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(152, 160))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(160, 168))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(168, 176))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(176, 184))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(184, 192))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(192, 200))] +
                        [(i >> j & 1) * 0xFF for j in reversed(range(200, 208))] +
                        [(i >> j & 1) * 0xFF for j in reversed(