#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

            if (((y % 256) == 0) or
                ((x % 256) == 0)):

                out.write(struct.pack('B', 0x2c))

                left      = struct.pack('>H',      0)
                top       = struct.pack('>H',      0)
                width     = struct.pack('>H', 1024 - 1)
                height    = struct.pack('>H',  768 - 1)
                flags     = struct.pack('B',         0)
                trans_idx = struct.pack('B',         0)

                out.write(left + top + width + height +
                          flags + trans_idx)


    # Trailer
    out.write(b';')

    return True


