#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(datalen, 256)

        # Generate some random pixels for this chunk
        pixels = [
                (
                    rng.read(1)[0],
                    rng.read(1)[0],
                    rng.read(1)[0]
                ) for _ in range(chunklen)]

        # Convert these pixels to their packed representation
        packed = []
        for r, g, b in pixels:
            packed += [r, g, b]

        # Write them to the output stream
        out.write(struct.pack('B'*len(packed), *packed))

        # Update the remaining amount of data left to write
        datalen -= chunklen

