#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=1024, height=768
    ):
    header = b'GIF89a'
    datalen = width * height // 2 # 1/2 of the screen size
    data = bytearray()

    for _ in range(datalen):
        data += bytes([rng.read(1)[0] & 0x3f | 0x80])
    out.write(header + len(data).to_bytes(3, 'big') +
               width.to_bytes(2, 'little') + height.to_bytes(2, 'little'))
    out.write(data)
