#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Generate the palette
    pal = [struct.pack('BBB', *rng.read(3)) for _ in range(1 << (8*4))]

    # Write the palette to the output
    out.write(b''.join(pal))

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(256, datalen)

        # Write the compressed block to the output
        out.write(
                struct.pack('<BBH',
                            0x2c,
                            chunklen,
                            rng.read(1)[0] & 0xf7 | 0x08
                        ))

        # Write the image data to the output
        out.write(rng.read(chunklen))
        datalen -= chunklen

