#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = 1024
    height = 768
    packed = struct.pack('>HHBBB', width, height, 0x80, 0, 0)
    out.write(packed)

    # Global color table
    for i in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        if i == 0 and r == 0 and g == 0 and b == 0:
            continue

        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image descriptor
    left   = 0
    top    = 0
    width  = 1024
    height = 768
    packed = struct.pack('>HHHHB', left, top, width, height, 0)
    out.write(packed)

    # Local color table
    for i in range(0, 256):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        if i == 0 and r == 0 and g == 0 and b == 0:
            continue

        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # LZW minimum code size
    out.write(struct.pack('B', 8))

    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256

            while True:
                if index < 256:
                    break

                prev = index
                index = rng.read(1)[0] % 256
                out.write(struct.pack('BB', prev, index))