#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:

        # Determine how many bytes we can write this iteration
        chunklen = min(datalen, 256)

        # Generate some random bytes for the current chunk
        chunkdata = rng.read(chunklen)

        # Write them to the output
        out.write(chunkdata)

        # Update our remaining data length counter
        datalen -= len(chunkdata)

