#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Generate the palette
    pal = [struct.pack('BBB', *rng.read(3)) for _ in range(1 << 7)]

    # Write the palette to the output
    out.write(b''.join(pal))

    # Copy the image data in chunks of 256 bytes at a time
    while datalen > 0:
        chunklen = min(256, datalen)

        # Write the next chunk to the output
        out.write(
                (
                    # LZW minimum code size
                    1 << rng.read(1)[0] |

                    # Image descriptor block
                    b'\x2c\x00\x00\x00\x00\x00,\xff\x0b\x4e' +

                    # Image data
                    rng.read(chunklen - 1)
                ))

        datalen -= chunklen
