#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480):
    # Write header
    out.write(b'GIF89a')

    # Write logical screen descriptor
    out.write(struct.pack('<HBB', width, height, 0xF7))

    # Write global color table (1-based index)
    for i in range(256):
        out.write(bytes([i]))

    # Write graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\xff\xff\xff\x00')

    # Write image descriptor
    out.write(struct.pack('<HHHHB', 0, 0, width, height, 0))

    # Write local color table (1-based index)
    for i in range(256):
        out.write(bytes([i]))

    # Write LZW minimum code size byte
    out.write(b'\x00')

    # Write pixel data
    while True:
        # Read a byte from /dev/urandom
        b = rng.read(1)
        if not b:
            break
        out.write(b)
