#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(pow(2, int.from_bytes(color, 'big'))):
        r = int.from_bytes(rng.read(1),'big')
        g = int.from_bytes(rng.read(1),'big')
        b = int.from_bytes(rng.read(1),'big')
        out.write(r.to_bytes(1,'big'))
        out.write(g.to_bytes(1,'big'))
        out.write(b.to_bytes(1,'big'))

    # Write the image descriptor
    left   = (int.from_bytes(rng.read(1), 'big') % width)
    top    = (int.from_bytes(rng.read(1), 'big') % height)
    width  = (int.from_bytes(rng.read(1), 'big') % width) + 1
    height = (int.from_bytes(rng.read(1), 'big') % height) + 1
    flags  = bytes([0x80 | int.from_bytes(rng.read(1), 'big')])
    out.write(left.to_bytes(2,'big'))
    out.write(top.to_bytes(2,'big'))
    out.write(width.to_bytes(2,'big'))
    out.write(height.to_bytes(2,'big'))
    out.write(flags)

    # Write the local color table
    for i in range(pow(2, int.from_bytes(color, 'big'))):
        r = int.from_bytes(rng.read(1),'big')
        g = int.from_bytes(rng.read(1),'big')
        b = int.from_bytes(rng.read(1),'big')
        out.write(r.to_bytes(1,'big'))
        out.write(g.to_bytes(1,'big'))
        out.write(b.to_bytes(1,'big'))

    # Write the image data
    while True:
        # Read the next byte from the RNG
        code = int.from_bytes(rng.read(1), 'big')

        # Check if this is the end of the stream
        if code == 0: break

        # Encode the string of characters
        encoded = []
        prev = ''
        for _ in range(code):
            # Read the next character from the RNG
            char = chr(int.from_bytes(rng.read(1), 'big'))

            # If this is not the same as the previous one, add it to the list
            if len(prev)==0 or char!= prev[-1]:
                encoded += [prev]

                # Add the current character as the first entry
                encoded += [[char]]
            else:
                # Append it to the last entry
                encoded[-1].append(char)

            # Update the previous character
            prev = ''.join(encoded[-1])

        # Write the encoded characters
        for e in encoded:
            out.write(bytes([len(e)+1]))
            for c in e+[chr(0)]:
                out.write(bytes([ord(c)]))