#!/usr/bin/env python3

import os
import sys
from pathlib import Path

# Add the parent directory to the PATH so that we can import the shared
# libraries.
sys.path.append(str(Path(__file__).resolve().parent.parent))

import shared as lib

def main():
    # We need a valid context to use the API.
    ctx = lib.get_context()
    
    # Create an instance of the API.
    api = lib.get_api(ctx)

    # Get the list of all available devices.
    devices = api.device_list(ctx, None)

    # Print out the device information.
    for dev in devices:
        print("Device:")
        print("\tName:", dev.name())
        print("\tDescription:", dev.description())
        print("\tID:", dev.id())
        print("\tSerial Number:", dev.serial_number())
        print("\tFirmware Version:", dev.firmware_version())
        print("\tHardware Version:", dev.hardware_version())
        print("\tProtocol Version:", dev.protocol_version())
        print("\tVendor ID:", hex(dev.vendor_id()))
        print("\tProduct ID:", hex(dev.product_id()))
        print("\tUSB Bus Number:", dev.usb_bus_number())
        print("\tUSB Address:", dev.usb_address())
        print("\tInterface Number:", dev.interface_number())
        print("\tLocation ID:", dev.location_id())
        print("\tMount Point:", dev.mount_point())
        print("\tIs Mounted:", dev.is_mounted())
        print("\tIs Formatted:", dev.is_formatted())
        print("\tCan Open:", dev.can_open())
        print("\tCan Close:", dev.can_close())
        print("\tCan Remove:", dev.can_remove())
        print("\tCan Lock:", dev.can_lock())
        print("\tCan Unlock:", dev.can_unlock())
        print("\tCan Format:", dev.can_format())
        print("\tCan Reset:", dev.can_reset())
        print("\tCan Set Password:", dev.can_set_password())
        print("\tCan Change Password:", dev.can_change_password())
        print("\tHas Password:", dev.has_password())
        print("\tHas Passphrase Support:", dev.has_passphrase_support())
        print("\tIs PASSPHRASE Required For Decryption:", dev.is_passphrase_required_for_decryption())
        print("\tIs Initialized:", dev.is_initialized())
        print("\tNeeds Key Derivation:", dev.needs_key_derivation())
        print("\tIs Key Derivation Done:", dev.is_key_derivation_done())
        print("\tMaximum File Size:", dev.maximum_file_size())
        print("\tFree Space:", dev.free_space())
        print("\tTotal Space:", dev.total_space())
        print("\tBlock Size:", dev.block_size())
        print("\tOEM Block Size:", dev.oem_block_size())
        print("\tHeader Version:", dev.header_version())
        print("\tEncrypted Header Size:", dev.encrypted_header_size())
        print("\tMass Storage Mode:", dev.mass_storage_mode())
        print("\tFile System Type:", dev.filesystem_type())
        print("\tFilesystem Label:", dev.filesystem_label())
        print("\tPartition Type GUID:", dev.partition_type_guid())
        print("\tUnique Partition GUID:", dev.unique_partition_guid())
        print("\tCurrent LBA:", dev.current_lba())
        print("\tBackup LBA:", dev.backup_lba())
        print("\tFirst Usable LBA:", dev.first_usable_lba())
        print("\tLast Usable LBA:", dev.last_usable_lba())
        print("\tDisk GUID:", dev.disk_guid())
        print("\tLogical Sector Size:", dev.logical_sector_size())
        print("\