#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header (6 bytes)
    out.write(b'GIF89a')
    
    # Logical screen descriptor (7 bytes)
    out.write(struct.pack('<HBB', 1024, 512, 0))
    
    # Global color table (768 bytes)
    for _ in range(768 // 3):
        out.write(struct.pack('<BBB', *[rng.read(1)[0] for _ in range(3)]))
    
    # Graphic control extension (4 bytes)
    out.write(struct.pack('<BBHH', 0xF9, 4, 0, 0))
    
    # Image descriptor (10 bytes)
    out.write(struct.pack('<BBHHBB', 0x2C, 0, 0, 1024, 512, 0))
    
    # Local color table (768 bytes)
    for _ in range(768 // 3):
        out.write(struct.pack('<BBB', *[rng.read(1)[0] for _ in range(3)]))
    
    # LZW-compressed image data (variable length)
    while True:
        byte = rng.read(1)[0]
        
        if byte == 0:
            break
        
        out.write(byte.to_bytes(1, 'little'))
