#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    # Write header and data length to output
    out.write(header + datalen.to_bytes(1, 'big'))

    # Write 256 color palette entries (0-255)
    for i in range(256):
        r = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        g = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        b = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        out.write((r << 16 | g << 8 | b).to_bytes(3, 'big'))

    # Write image descriptor block
    out.write(b'\x2c\x00\x00\x00\x00')

    # Write width and height
    w = int.from_bytes(
        rng.read(2), byteorder='big', signed=False
    )

    h = int.from_bytes(
        rng.read(2), byteorder='big', signed=False
    )

    out.write(w.to_bytes(2, 'big'))
    out.write(h.to_bytes(2, 'big'))

    # Write flags
    flags = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(flags.to_bytes(1, 'big'))

    # Write background color index
    bgcolor = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(bgcolor.to_bytes(1, 'big'))

    # Write aspect ratio
    aspectratio = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(aspectratio.to_bytes(1, 'big'))

    # Write pixel data
    for y in range(h):
        for x in range(w):
            # Read next pixel from RNG
            pix = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            )

            # Write pixel to output
            out.write(pix.to_bytes(1, 'big'))

    # Write trailer block
    out.write(b'\x3b')
