#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Read 2 bytes from the RNG as the width and height of the image
    width, height = struct.unpack('<HH', rng.read(4))

    # Write the width and height to the output stream
    out.write(struct.pack('<HH', width, height))

    # Read 1 byte from the RNG as the color depth
    colordepth = struct.unpack('B', rng.read(1))[0]

    # Write the color depth to the output stream
    out.write(struct.pack('B', colordepth))

    # Read 1 byte from the RNG as the background color index
    bgcoloridx = struct.unpack('B', rng.read(1))[0]

    # Write the background color index to the output stream
    out.write(struct.pack('B', bgcoloridx))

    # Read 1 byte from the RNG as the pixel aspect ratio
    par = struct.unpack('B', rng.read(1))[0]

    # Write the pixel aspect ratio to the output stream
    out.write(struct.pack('B', par))

    # Read 1 byte from the RNG as the flags
    flags = struct.unpack('B', rng.read(1))[0]

    # Write the flags to the output stream
    out.write(struct.pack('B', flags))

    # Calculate the size of the image data section
    datalen = (width * height * (colordepth + 7) // 8)

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)