#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate some random bytes for the length of the image data
    # and write them to the output stream
    datalen = int.from_bytes(
            rng.read(3), byteorder='big', signed=False) + 1
    out.write(struct.pack('<H', datalen))

    # Generate some random bytes for the width and height of the image
    # and write them to the output stream
    width = int.from_bytes(
            rng.read(2), byteorder='big', signed=False) + 1
    height = int.from_bytes(
            rng.read(2), byteorder='big', signed=False) + 1
    out.write(struct.pack('<HH', width, height))

    # Generate some random bytes for the color table flags and background
    # color index, and write them to the output stream
    colortableflags = rng.read(1)[0] & 0x7F
    bgcolorindex = rng.read(1)[0]
    out.write(struct.pack('BB', colortableflags, bgcolorindex))

    # Generate some random bytes for the pixel aspect ratio and write
    # them to the output stream
    par = rng.read(1)[0]
    out.write(struct.pack('B', par))

    # Generate some random bytes for the color table and write them
    # to the output stream
    colortable = rng.read(768)
    out.write(colortable)

    # Generate some random bytes for the image data and write them
    # to the output stream
    imagedata = rng.read(datalen - 1)
    out.write(imagedata)

    # Generate some random bytes for the trailer and write them
    # to the output stream
    out.write(b'\x3B')
