#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'
    out.write(header)

    # Next, we need to figure out how many bytes of random data we should write
    # to the file.  This is done by reading a single byte from /dev/urandom and
    # using that as the number of bytes to write.
    num_bytes = int.from_bytes(rng.read(1), 'big') + 1

    # Now we can write the rest of the file.  We do this by writing one byte at
    # a time, each with a value chosen randomly from /dev/urandom.
    for _ in range(num_bytes):
        out.write(rng.read(1))

