#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    # Write header and data length to output
    out.write(header + datalen.to_bytes(1, 'big'))

    # Write 256 color palette entries (0-255)
    for i in range(256):
        r = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        g = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        b = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        out.write(r.to_bytes(1, 'big') +
                  g.to_bytes(1, 'big') +
                  b.to_bytes(1, 'big'))

    # Write image descriptor block
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\xff\xff\xff\xff')

    # Write image data
    for _ in range(datalen):
        out.write(rng.read(1))
