#!/usr/bin/env python3
import sys
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
rng = open('/dev/urandom', 'rb')

def read_int(f: BinaryIO, nbytes: int):
    return int.from_bytes(f.read(nbytes), byteorder='little', signed=False)

def write_int(f: BinaryIO, x: int, nbytes: int):
    f.write(x.to_bytes(nbytes, byteorder='little'))

def main():
    if len(sys.argv) != 2:
        print('Usage: {} <output-file>'.format(sys.argv[0]))
        exit(-1)

    # generate a random key pair
    p = rng.read(32)
    q = rng.read(32)
    while True:
        g = rng.read(32)
        if pow(g, (p * q - 1) // 2, p * q) == 1: break
    
    # write the public key to file
    with open(sys.argv[1], 'wb') as f:
        write_int(f, p, 32)
        write_int(f, q, 32)
        write_int(f, g, 32)
