#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(2)[0] + 1
    height = rng.read(2)[0] + 1
    global_color_table_flag = rng.read(1)[0] & 1
    color_resolution = (rng.read(1)[0] >> 4) & 7
    sort_flag = rng.read(1)[0] & 1
    size_of_global_color_table = 2 ** ((rng.read(1)[0] >> 5) & 7)
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBBB', width, height, global_color_table_flag << 7 | color_resolution << 4 | sort_flag << 3 | size_of_global_color_table, background_color_index, pixel_aspect_ratio))

    # Global color table
    if global_color_table_flag == 1:
        for i in range(size_of_global_color_table * 3):
            out.write(rng.read(1))
    
    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', 0))

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00')
    delay_time = rng.read(2)[0]
    transparent_color_flag = rng.read(1)[0] & 1
    transparent_color_index = rng.read(1)[0]
    out.write(struct.pack('<HH', delay_time, transparent_color_flag << 7 | transparent_color_index))

    # Image descriptor
    out.write(b'\x2c')
    left = rng.read(2)[0]
    top = rng.read(2)[0]
    width = rng.read(2)[0] + 1
    height = rng.read(2)[0] + 1
    local_color_table_flag = rng.read(1)[0] & 1
    interlace_flag = rng.read(1)[0] & 1
    sort_flag = rng.read(1)[0] & 1
    size_of_local_color_table = 2 ** ((rng.read(1)[0] >> 5) & 7)
    out.write(struct.pack('<HHBBB', left, top, width, height, local_color_table_flag << 7 | interlace_flag << 6 | sort_flag << 5 | size_of_local_color_table))

    # Local color table
    if local_color_table_flag == 1:
        for i in range(size_of_local_color_table * 3):
            out.write(rng.read(1))
    
    # Raster data
    for y in range(height):
        for x in range(width):
            out.write(rng.read(1))

    # Trailer
    out.write(b'\x3b')
