#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(pow(2, int.from_bytes(color, 'big'))):
        r = int.from_bytes(rng.read(1),'big')
        g = int.from_bytes(rng.read(1),'big')
        b = int.from_bytes(rng.read(1),'big')
        out.write(r.to_bytes(1,'big'))
        out.write(g.to_bytes(1,'big'))
        out.write(b.to_bytes(1,'big'))

    # Write the image descriptor
    left   = bytes([0x00])
    top    = bytes([0x00])
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    out.write(left)
    out.write(top)
    out.write(width.to_bytes(2,'big'))
    out.write(height.to_bytes(2,'big'))
    out.write(flags)

    # Write the local color table
    for i in range(pow(2, int.from_bytes(color, 'big'))):
        r = int.from_bytes(rng.read(1),'big')
        g = int.from_bytes(rng.read(1),'big')
        b = int.from_bytes(rng.read(1),'big')
        out.write(r.to_bytes(1,'big'))
        out.write(g.to_bytes(1,'big'))
        out.write(b.to_bytes(1,'big'))

    # Write the image data
    while True:
        # Read the code size
        codesize = int.from_bytes(rng.read(1), 'big')
        if codesize == 0: break
        
        # Read the encoded characters
        encoded = []
        while True:
            # Read the next character
            c = int.from_bytes(rng.read(1), 'big')
            
            # If it is not an end-of-stream marker, add it to the list
            if c != 0:
                encoded.append(chr(c))
                
            # Otherwise, we are done
            else:
                break

        # Write the encoded characters
        for e in encoded:
            out.write(bytes([len(e)+1]))
            for c in e+[chr(0)]:
                out.write(bytes([ord(c)]))