#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(out: BinaryIO, rng: BinaryIO):
    # Magic header
    out.write(b'GIF89a')
    
    # Screen descriptor
    out.write(struct.pack('<HHBB', 200, 150, 0xF7, 0))
    
    # Global color table (black and white)
    out.write(b'\xFF\xFF\xFF\xFF\xFF\xFF')
    
    # Graphic control extension
    out.write(b'\x21\xF9\x04\x08\x00\x00\x00')
    
    # Image descriptor
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00\x00')
    
    # Local color table (black and white)
    out.write(b'\xFF\xFF\xFF\xFF\xFF\xFF')
    
    # Pixel data
    for _ in range(200 * 150):
        out.write(bytes([next(rng) & 0xFF]))
    
    # Trailer
    out.write(b';')
