#!/usr/bin/env python3

import os
from random import randint


def generate_random_gif(rng, out):
    """Generate a GIF image of random size and color."""

    # The first three bytes are the magic number "GIF".
    out.write(b'GIF')

    # Next is the screen width and height. We pick a random value between 1
    # and 256 (inclusive).
    width = randint(1, 256)
    height = randint(1, 256)
    out.write((width & 0xFF).to_bytes(1, 'big'))
    out.write((height & 0xFF).to_bytes(1, 'big'))

    # Then comes the global color table flag, which is always 0.
    out.write(b'\x87')

    # Then the background color index, which is also always 0.
    out.write(b'\x00')

    # Then the pixel aspect ratio, which is always 0.
    out.write(b'\x00')

    # Now we have to write the image descriptor. First, the magic number
    # "GIF" again.
    out.write(b'GIF')

    # Then the left position of the image on the canvas.
    out.write(b'\x00\x00')

    # Then the top position of the image on the canvas.
    out.write(b'\x00\x00')

    # Then the width of the image.
    out.write((width & 0xFF).to_bytes(1, 'big'))

    # Then the height of the image.
    out.write((height & 0xFF).to_bytes(1, 'big'))

    # Then the local color table flag, which is always 0.
    out.write(b'\x87')

    # Then the code size, which is always 7.
    out.write(b'\x07')

    # Now we have to write the LZW-compressed image data. First, we need to
    # calculate how many bytes it will take up.
    datalen = ((width * height) // 8) + 1

    # Then we write the header, which consists of the magic number "GIF",
    # followed by the length of the data block.
    header = b'GIF' + int.to_bytes(datalen,
                                    rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)