#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given `seed`. The seed is used as an index into a
# table of 16-byte blocks that are then XOR'd together
# to generate the actual image data.
def generate(out: BinaryIO, seed: int):
    # Header for a GIF file
    header = b'\x47\x49\x46\x38\x39\x61'
    
    # Length of the data section in bytes
    datalen = 0
    
    # Data to write to the output
    data = bytearray()
    
    # Index into the block table
    i = seed % len(blocks)
    
    # Loop through all blocks until we have enough data
    while datalen < 5000:
        # Add the current block to the data
        data += blocks[i]
        
        # Increment the length counter
        datalen += 16
        
        # Get the next block index
        i = (i + 1) % len(blocks)

    # Write the header and length of the data to the output
    out.write(header + datalen.to_bytes(2, 'big'))
    out.write(data)

