#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int) -> None:
    # Write the header.
    out.write(b'GIF89a')

    # Write the logical screen descriptor.
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(b'\x00\x00')  # Background color index.
    out.write(b'\x00')      # Pixel aspect ratio (unused).

    # Write the global color table.
    for i in range(16):
        r = ((i & 0b00000001) << 7) | ((i & 0b00000010) << 5) | ((i & 0b00000100) << 3) | ((i & 0b00001000) << 1)
        g = ((i & 0b00010000) >> 4) | ((i & 0b00100000) >> 2) | ((i & 0b01000000) >> 0)
        b = ((i & 0b10000000) >> 7) | ((i & 0b00000001) << 1) | ((i & 0b00000010) << 3)
        out.write((r << 4) + r)
        out.write((g << 4) + g)
        out.write((b << 4) + b)

    # Write the application extension block.
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(b'\x00\x00\x00')

    # Write the image descriptor.
    out.write(b'\x2c\x00\x00\x00\x00')
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(b'\x00')                   # Local color table flag.
    out.write(b'\x00')                   # Interlace flag.
    out.write(b'\x00')                   # Sort flag.
    out.write(b'\x00')                   # Size of local color table.

    # Write the first frame.
    for y in range(height):
        for x in range(width):
            out.write(b'\x00')           # Transparent pixel.

    # Write the trailer.
    out.write(b'\x3b')
