#!/usr/bin/env python3
import sys
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    header = b'GIF89a'
    # 1 byte for width (0-255), 1 byte for height (0-255)
    width, height = rng.read(2)
    # 1 byte for color table size (0-7)
    color_table_size = rng.read(1)[0] & 0b111
    # 1 byte for background color index (0-255)
    background_color_index = rng.read(1)[0]
    # 1 byte for pixel aspect ratio (0-255)
    pixel_aspect_ratio = rng.read(1)[0]
    # 1 byte for flags (0-255)
    flags = rng.read(1)[0]
    # 1 byte for transparent color index (-1 if no transparency)
    transparent_color_index = -1
    if flags & 0b10000000:
        transparent_color_index = rng.read(1)[0]
    # 1 byte for block terminator (always 0x3B)
    assert rng.read(1)[0] == 0x3B
    # 1 byte for image separator (always 0x2C)
    assert rng.read(1)[0] == 0x2C
    # 1 byte for left position of image (0-65535)
    left = rng.read(2)
    # 1 byte for top position of image (0-65535)
    top = rng.read(2)
    # 1 byte for width of image (0-65535)
    width = rng.read(2)
    # 1 byte for height of image (0-65535)
    height = rng.read(2)
    # 1 byte for flags (0-255)
    flags = rng.read(1)[0]
    # 1 byte for local color table size (0-7)
    local_color_table_size = rng.read(1)[0] & 0b111
    # 1 byte for interlaced flag (0 or 1)
    interlaced = bool(flags & 0b10000000)
    # 1 byte for sort flag (0 or 1)
    sorted = bool(flags & 0b01000000)
    # 1 byte for reserved flag (must be zero)
    assert not (flags & 0b00100000)
    # 1 byte for global color table size (0-7)
    global_color_table_size = flags & 0b00011111
    # 1 byte for background color index (0-255)
    background_color_index = rng.read(1)[0]
    # 1 byte for pixel aspect ratio (0-255)
    pixel_aspect_ratio = rng.read(1)[0]
    # 1 byte for block terminator (always 0x3B)
    assert rng.read(1)[0] == 0x3B
    # 1 byte for graphic control extension label (always 0xF9)
    assert rng.read(1)[0] == 0xF9
    # 1 byte for graphic control extension length (always 4)
    assert rng.read(1)[0] == 4
    # 1 byte for disposal method (0-3)
    disposal_method = rng.read(1)[0] & 0b111
    # 1 byte for user input flag (0 or 1)
    user_input = bool(rng.read(1)[0])
    # 1 byte for transparent color flag (0 or 1)
    transparent_color = bool(rng.read(1)[0])
    # 2 bytes for delay time in hundredths of a second (0-65535)
    delay_time = rng.read(2)
    # 1 byte for transparent color index (-1 if no transparency)
    transparent_color_index = -1
    if transparent_color:
        transparent_color_index = rng.read(1)[0]
    # 1 byte