#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    # Write header and data length to output
    out.write(header + datalen.to_bytes(1, 'big'))

    # Write 256 color palette entries (0-255)
    for i in range(256):
        r = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        g = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        b = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        out.write(struct.pack('BBB', r, g, b))

    # Write image descriptor block
    out.write(b'\x2c\x00\x00\x00\x00')

    # Write image width and height
    width = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    height = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(width.to_bytes(1, 'big'))
    out.write(height.to_bytes(1, 'big'))

    # Write flags
    flags = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(flags.to_bytes(1, 'big'))

    # Write local color table flag
    if flags & 0b10000000 == 0:
        # Write background color index
        bgcolor = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        out.write(bgcolor.to_bytes(1, 'big'))

        # Write pixel aspect ratio
        par = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        out.write(par.to_bytes(1, 'big'))

    # Write LZW minimum code size
    lzwmincodesize = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(lzwmincodesize.to_bytes(1, 'big'))

    # Write LZW compressed data
    while True:
        # Read next byte from input
        c = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        # If it is zero, we are done
        if c == 0:
            break

        # Otherwise write it to output
        out.write(c.to_bytes(1, 'big'))
