#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def generate_random_gif(rng: int, outfile: BinaryIO):
    # GIF header
    outfile.write(b'GIF89a')
    
    # Screen descriptor
    outfile.write(struct.pack('<HBB', 200, 150, 0))
    
    # Global color table
    for i in range(7):
        r = (i * 4 + 1) % 256
        g = (i * 4 + 2) % 256
        b = (i * 4 + 3) % 256
        
        outfile.write(struct.pack('BBB', r, g, b))
    
    # Graphic control extension
    outfile.write(struct.pack('BB', 0xF9, 0x04))
    outfile.write(struct.pack('<H', rng & 0xFFFF))
    outfile.write(b'\x00\x00')
    
    # Image descriptor
    outfile.write(struct.pack('<HHBB', 0, 0, 200, 150))
    outfile.write(b'\x00')
    
    # Local color table
    for i in range(7):
        r = (i * 4 + 1) % 256
        g = (i * 4 + 2) % 256
        b = (i * 4 + 3) % 256
        
        outfile.write(struct.pack('BBB', r, g, b))
    
    # Image data
    for y in range(150):
        for x in range(200):
            r = (y * 4 + x * 4 + 1) % 256
            g = (y * 4 + x * 4 + 2) % 256
            b = (y * 4 + x * 4 + 3) % 256
            
            outfile.write(struct.pack('BBB', r, g, b))

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: {} <output directory>".format(os.path.basename(__file__)))
        exit(1)
    
    output_dir = sys.argv[1]
    
    if not os.path.isdir(output_dir):
        print("{} is not a directory.".format(output_dir))
        exit(1)
    
    for i in range(256):
        with open(os.path.join(output_dir, "{:02X}.gif".format(i)), 'wb') as f:
            generate_random_gif(rng, f)