#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Get the width and height of the image from the RNG
    width = int.from_bytes(rng.read(2), 'little', signed=False) % 100 + 1
    height = int.from_bytes(rng.read(2), 'little', signed=False) % 100 + 1
    out.write(struct.pack('<HHB', width, height, 0x87))
    
    # Write the global color table (black and white only)
    out.write(b'\xff\xff\xff\xff\xff\xff')

    # Write the graphic control extension
    out.write(b'\x21\xf9\x04\x08\x00\x00\xff\xff\xff\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00')

    # Write the local color table (black and white only)
    out.write(b'\xff\xff\xff\xff\xff\xff')

    # Write the pixel data
    for y in range(height):
        for x in range(width):
            out.write(b'\xff')

    # Write the trailer
    out.write(b';')
