#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write((width).to_bytes(2,'big'))
    out.write((height).to_bytes(2,'big'))
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(2**(pxsize[0] + 1)):
        out.write(rng.read(3))

    # Generate and write the graphics control extension
    gce = b'\xF9\x04\x08\x00\x00\xFF'
    out.write(gce)

    # Generate and write the image descriptor
    idesc = b'\x2C\x00\x00\x00\x00\x00\xFF\xFF\x00'
    out.write(idesc)

    # Generate and write the local color table
    lct = b''
    for i in range(2**(pxsize[0] + 1)):
        lct += rng.read(3)
    out.write(lct)

    # Generate and write the image data
    imgdata = b''
    for y in range(height):
        for x in range(width):
            imgdata += rng.read(3)
    out.write(imgdata)

    # Generate and write the trailer
    out.write(b';')
