#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # get some of our data into the file before writing the rest.
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


def main() -> int:
    if len(sys.argv)!= 2 or not os.path.exists(sys.argv[1]):
        print("Usage: {} <output>".format(os.path.basename(__file__)),
              file=sys.stderr)
        return -1

    with open(sys.argv[1], mode='wb') as f:
        with open('/dev/urandom', mode='rb') as rng:
            generate_random_gif(rng, f)


if __name__ == '__main__':
    exit(main())