#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    # Write header and data length to output
    out.write(header + datalen.to_bytes(1, 'big'))

    # Write 256 color palette entries (0-255)
    for i in range(256):
        r = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        g = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        b = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        out.write((r << 16 | g << 8 | b).to_bytes(3, 'big'))

    # Write image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')

    # Write width and height of image
    w = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    h = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(w.to_bytes(2, 'big') + h.to_bytes(2, 'big'))

    # Write flags
    flags = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(flags.to_bytes(1, 'big'))

    # Write background color index
    bgidx = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(bgidx.to_bytes(1, 'big'))

    # Write pixel aspect ratio
    par = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    out.write(par.to_bytes(1, 'big'))

    # Write image data
    while True:
        code = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        if code == 0xFF:
            break

        elif code == 0xFE:
            # Read block size
            size = int.from_bytes(
                rng.read(1), byteorder='big', signed=False
            )

            # Read block data
            out.write(rng.read(size))

        else:
            # Read subblocks until we hit an empty one
            while True:
                # Read subblock size
                size = int.from_bytes(
                    rng.read(1), byteorder='big', signed=False
                )

                # If this is an empty subblock, stop reading
                if size == 0:
                    break

                # Otherwise read the subblock data
                out.write(rng.read(size))

    # Write trailer
    out.write(b'\x3B')
