#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    out.write(struct.pack('<HHBB', 100, 100, 0xF7, 0))

    # Global color table (256 colors)
    for i in range(256):
        out.write(struct.pack('B', i))
        out.write(struct.pack('B', i))
        out.write(struct.pack('B', i))

    # Image block
    out.write(struct.pack('<HBB', 0, 0, 0))
    out.write(b'\x00\xFF')

    # Image data
    while True:
        data = rng.read(1024)
        if not data:
            break
        out.write(data)

