#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 27 10:45:26 2020

@author: jonasg
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
import matplotlib.pyplot as plt

def get_sample(mu, sigma):
    return np.random.multivariate_normal(mean=mu, cov=sigma)

def get_samples(mu, sigma, n_samples):
    return [get_sample(mu, sigma) for _ in range(n_samples)]

def plot_samples(samples, title='', color='b'):
    plt.scatter(*zip(*samples), marker='o', c=color)
    plt.title(title)
    
def plot_density(x, y, xlabel='', ylabel='', title=''):
    plt.plot(x, y, 'k')
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_contour(x, y, z, xlabel='', ylabel='', title=''):
    plt.contourf(x, y, z)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_histogram(data, bins=None, xlabel='', ylabel='', title=''):
    plt.hist(data, bins=bins)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)

def plot_pdf(x, pdf, xlabel='', ylabel='', title=''):
    plt.plot(x, pdf, 'r')
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_cdf(x, cdf, xlabel='', ylabel='', title=''):
    plt.plot(x, cdf, 'r')
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_qq(data, qq, xlabel='', ylabel='', title=''):
    plt.plot([min(data), max(data)], [min(data), max(data)], 'k--')
    plt.plot(data, qq, 'ro')
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_pvalue(data, pvalue, xlabel='', ylabel='', title=''):
    plt.bar(range(len(data)), pvalue, align='center')
    plt.xticks(range(len(data)), data)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_correlation(corr, xlabel='', ylabel='', title=''):
    plt.imshow(corr, interpolation='nearest', cmap=plt.cm.Blues)
    plt.colorbar()
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_covariance(cov, xlabel='', ylabel='', title=''):
    plt.imshow(cov, interpolation='nearest', cmap=plt.cm.Blues)
    plt.colorbar()
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_matrix(mat, xlabel='', ylabel='', title=''):
    plt.imshow(mat, interpolation='nearest', cmap=plt.cm.Blues)
    plt.colorbar()
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    
def plot_confusion_matrix(cm, classes,
                          normalize=False,
                          title='Confusion matrix',
                          cmap=plt.cm.Blues):
    """
    This function prints and plots the confusion matrix.
    Normalization can be applied by setting `normalize