#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=1024, height=768):
    # Header
    out.write(b'GIF89a')
    
    # Logical screen descriptor
    out.write(struct.pack('<HHB', width, height, 0))
    out.write(b'\x00\x00\xff\xff\xff\xff')
    
    # Graphic control extension
    out.write(b'\x21\xf9\x04\x05\x00\x00\xff\xff\xff\xff')
    
    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00' + struct.pack('<HHBB', width, height, 0, 0))
    
    # Local color table
    for i in range(256):
        out.write(bytes([i]))
        out.write(bytes([i]))
        out.write(bytes([i]))
    
    # Image data
    while True:
        try:
            b = rng.read(width * height)
        except KeyboardInterrupt:
            break
        
        if len(b) != (width * height):
            raise Exception("Couldn't read enough bytes from RNG")
            
        out.write(b)
