#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width=640, height=480, duration=10):
    # Magic header
    out.write(b"GIF89a")

    # Logical screen descriptor
    out.write(struct.pack("<HHBB", width, height, 0xF7, 0))

    # Global color table (256 colors)
    for i in range(256):
        out.write(bytes([i] * 3))

    # Graphic control extension
    out.write(b"\x21\xf9\x04\x01\x00\x00\xff\x00")

    # Image descriptor
    out.write(struct.pack("<HHHHB", 0, 0, width, height, 0))

    # Local color table (256 colors)
    for i in range(256):
        out.write(bytes([i] * 3))

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng.read(1)[0]]))

    # Trailer
    out.write(b";")
