#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int = 1024, height: int = 768) -> None:
    # GIF header
    out.write(b'GIF89a')
    # Logical screen descriptor
    out.write(struct.pack('<HHBB', width, height, 0xF0 | (width >> 7), 0))
    # Global color table
    for i in range(0xFF):
        out.write(bytes([i] * 3))
    # Graphic control extension
    out.write(b'\x21\xF9\x04\x05\x00\x00\xFF\x00')
    # Image descriptor
    out.write(b'\x2C\x00\x00\x00\x00' + bytes([height >> 8, height & 0xFF]))
    # Local color table
    for i in range(0xFF):
        out.write(bytes([i] * 3))
    # Pixel data
    while True:
        b = rng.read(1)[0]
        if b == 0:
            break
        out.write(bytes([b]) * 2 ** b)

