#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
    rng: BinaryIO,
    outfile: BinaryIO,
):
    # GIF header
    outfile.write(b'GIF89a')

    # Screen descriptor
    outfile.write(struct.pack('<HBB', 128, 128, 0))

    # Global color table
    for _ in range(768):
        outfile.write(os.urandom(3))

    # Graphic control extension
    outfile.write(b'\x21\xf9\x04\x05\xff\x00')

    # Image descriptor
    outfile.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Local color table
    for _ in range(768):
        outfile.write(os.urandom(3))

    # LZW data stream
    while True:
        code = ord(rng.read(1)) % 256
        if code == 0:
            break
        else:
            outfile.write(code.to_bytes(1, byteorder='big'))


def main() -> int:
    if len(sys.argv) != 2:
        print("Usage: {} <outfile>".format(sys.argv[0]))
        return 1

    with open(sys.argv[1], mode='wb') as f:
        with open('/dev/urandom', mode='rb') as rng:
            generate_random_gif(rng, f)

    return 0


if __name__ == '__main__':
    exit(main())