#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def main(argv):
    if len(argv)!= 2:
        print("Usage: {} <file>".format(os.path.basename(__file__)), file=sys.stderr)
        return -1

    with open(argv[1], "rb") as f:
        magic = f.read(4).decode()

        if magic == "Yaz0":
            decompress_yaz0(f, argv[1])
        elif magic in ("Yaz", "Yaz1"):
            decompress_yaz1(f, argv[1])
        else:
            print("{} is not a Yaz archive".format(argv[1]), file=sys.stderr)
            return -1

    return 0


def decompress_yaz0(f: BinaryIO, filename: str):
    yaz0_header = f.read(8)
    assert yaz0_header[:4] == b"Yaz0"
    uncompressed_size = struct.unpack("<I", yaz0_header[-4:])[0]

    # Read the compressed data from the end of the file to avoid having to seek back and forth
    f.seek(-len(yaz0_header), os.SEEK_END)
    compressed_data = f.read()

    # Decompress the data using yaz0dec
    try:
        p = subprocess.Popen(["yaz0dec"], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
        output, _ = p.communicate(input=compressed_data)
    except FileNotFoundError:
        print("Could not find yaz0dec executable.", file=sys.stderr)
        raise SystemExit(-1)

    # Write the decompressed data to the original file name
    with open(filename[:-4], "wb") as out:
        out.write(output)


def decompress_yaz1(f: BinaryIO, filename: str):
    yaz1_header = f.read(8)
    assert yaz1_header[:4] in (b"Yaz", b"Yaz1")
    uncompressed_size = struct.unpack("<I", yaz1_header[-4:])[0]

    # Read the compressed data from the end of the file to avoid having to seek back and forth
    f.seek(-len(yaz1_header), os.SEEK_END)
    compressed_data = f.read()

    # Decompress the data using yazdec
    try:
        p = subprocess.Popen(["yazdec"], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
        output, _ = p.communicate(input=compressed_data)
    except FileNotFoundError:
        print("Could not find yazdec executable.", file=sys.stderr)
        raise SystemExit(-1)

    # Write the decompressed data to the original file name
    with open(filename[:-4], "wb") as out:
        out.write(output)


if