#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Get a random width and height between 100 and 500
    width = int.from_bytes(rng.read(2), 'big') % (500 - 100 + 1) + 100
    height = int.from_bytes(rng.read(2), 'big') % (500 - 100 + 1) + 100

    # Write the logical screen descriptor
    out.write(struct.pack('>HHBBB', width, height, 0xF7, 0x00, 0x00))

    # Write the global color table
    for _ in range(0, 2**int((width * height / 64))):

        # Generate a random RGB value
        r = rng.read(1)[0] & 0xFF
        g = rng.read(1)[0] & 0xFF
        b = rng.read(1)[0] & 0xFF

        # Write it to the output stream
        out.write(r.to_bytes(1, byteorder='big'))
        out.write(g.to_bytes(1, byteorder='big'))
        out.write(b.to_bytes(1, byteorder='big'))

    # Write the image descriptor
    out.write(struct.pack('>HHHHB', 0, 0, width, height, 0x00))

    # Write the local color table
    for i in range(0, 2**(width * height)):

        # Generate a random RGB value
        r = rng.read(1)[0] & 0xFF
        g = rng.read(1)[0] & 0xFF
        b = rng.read(1)[0] & 0xFF

        # Write it to the output stream
        out.write(r.to_bytes(1, byteorder='big'))
        out.write(g.to_bytes(1, byteorder='big'))
        out.write(b.to_bytes(1, byteorder='big'))

    # Write the graphics control extension
    out.write(b'\x21\xF9\x04')
    out.write(b'\x08')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')

    # Write the application extension
    out.write(b'\x21\xFF\x0B')
    out.write(b'NETSCAPE2.0')
    out.write(b'\x03')
    out.write(b'\x01')
    out.write(b'\x00\x00')
    out.write(b'\x00')

    # Write the plain text data block
    out.write(b'\x21\x01\x00')
    out.write(b'\x03')
    out.write(b'ABC')

    # Write the end of file marker
    out.write(b';')

