#!/usr/bin/env python3

import os
import sys
import struct
from PIL import Image


def main():
    if len(sys.argv)!= 2:
        print("Usage: {} <image>".format(os.path.basename(__file__)))
        return

    with open(sys.argv[1], "rb") as f:
        data = bytearray(f.read())

        # Check for magic number and file size
        assert data[:4] == b"BM\x00"
        assert len(data) >= 58 + (struct.unpack("<L", data[26:])[0]) * 4

        width, height = struct.unpack("<LL", data[18:26])
        pixels = [
            ((r >> 3) << 11) | ((g >> 2) << 5) | (b >> 3)
            for r, g, b in zip(*[iter(data)]*3)[58:]
        ]

    img = Image.new('RGB', (width, height))
    img.putdata(pixels)
    img.save(sys.stdout.buffer, format='png')


if