#!/usr/bin/env python3

import struct
from typing import BinaryIO

def generate_image(out: BinaryIO):
    """Generate a PNG file with a random size and RGB color."""
    
    # Create an instance of a cryptographically secure pseudo-random number generator
    from secrets import SystemRandom
    rng = SystemRandom()

    # Choose a random width between 10 and 256 pixels (inclusive)
    width = rng.randint(10, 256)

    # Choose a random height between 10 and 256 pixels (inclusive)
    height = rng.randint(10, 256)

    # Write the header information to the output stream
    # This is based on the example in RFC 2083
    out.write(b'\x89PNG\r\n')
    out.write(struct.pack('>I', 13))
    out.write(b'IHDR')
    out.write(width.to_bytes(4, 'big'))
    out.write(height.to_bytes(4, 'big'))
    out.write(b'\x08\x02\x00\x00')
    out.write(b'IDAT')

    # Calculate how many bytes will be needed to store the pixel data
    datalen = width * height * 3

    # Write the length of the pixel data as a big endian unsigned integer
    out.write(datalen.to_bytes(3, 'big'))

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)