#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + (rng.read(1)[0] % 256)
    height = 1 + (rng.read(1)[0] % 256)
    flags = b'\x87\x00' if width > height else b'\x00\x00'
    background_color = rng.read(1)
    pixel_aspect_ratio = b'\x00'
    out.write(struct.pack('>HHBBB', width, height, *flags))
    out.write(background_color)
    out.write(pixel_aspect_ratio)

    # Global color table
    global_color_table_size = 2 ** ((rng.read(1)[0] & 0x07) + 1)
    global_color_table_sort = bool((rng.read(1)[0] >> 3) & 0x01)
    global_color_table_bpp = 2 ** (((rng.read(1)[0] >> 4) & 0x07) + 1)
    global_color_table_entries = global_color_table_size ** global_color_table_bpp
    global_color_table_packed = True if global_color_table_bpp < 8 else False
    global_color_table_interlace = bool((rng.read(1)[0] >> 7) & 0x01)
    global_color_table_sorted = bool((rng.read(1)[0] >> 6) & 0x01)
    global_color_table_reserved = rng.read(1)
    global_color_table_user_input = rng.read(1)
    for i in range(global_color_table_entries):
        r = rng.read(1)
        g = rng.read(1)
        b = rng.read(1)

        if not global_color_table_packed and \
                global_color_table_bpp == 32:
            a = rng.read(1)

            out.write(r)
            out.write(g)
            out.write(b)
            out.write(a)
        elif not global_color_table_packed and \
                global_color_table_bpp == 16:
            a = b'\x00'

            out.write(r)
            out.write(g)
            out.write(b)
            out.write(a)
        else:
            out.write(r)
            out.write(g)
            out.write(b)


    # Image descriptor
    left = 0
    top = 0
    image_width = 1 + (rng.read(1)[0] % 256)
    image_height = 1 + (rng.read(1)[0] % 256)
    flags = b'\x87\x00' if image_width > image_height else b'\x00\x00'
    local_color_table_flag = bool((rng.read(1)[0] >> 7) & 0x01)
    interlace_flag = bool((rng.read(1)[0] >> 6) & 0x01)
    sort_flag = bool((rng.read(1)[0] >> 5) & 0x01)
    reserved = rng.read(1)
    lct_size = 2 ** ((rng.read(1)[0] & 0x07) + 1)
    lct_packed = True if lct_size < 8 else False
    lct_interlace = bool((rng.read(1)[0] >> 7) & 0x01)
    lct_sorted = bool((rng.read(1)[0] >> 6) & 0x01)
    lct_reserved = rng.read(1)
    lct_user_input = rng.read(1)
    for i in range(lct_size**lct_bpp):
        r = rng.read(1)
        g = rng.read(1)
        b = rng.read(1)

        if not lct_packed and \
                lct_bpp == 32:
            a = rng.read(1)

            out.write(r)
            out.write(g)
            out.write(b)
            out.write(a)
        elif not lct_packed and \
                lct_bpp == 16:
            a = b'\x00