#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # 10 bytes of global color table (RGB)
    colortable = [struct.unpack('BBB', rng.read(3)) for _ in range(10)]

    # 1 byte of flags
    flags = ord(rng.read(1))

    # 1 byte of background index
    bgindex = ord(rng.read(1))

    # 1 byte of pixel aspect ratio
    pxaspectratio = ord(rng.read(1))

    # 4 bytes of width and height
    w, h = struct.unpack('<HH', rng.read(4))

    # Write header
    out.write(header)

    # Write logical screen descriptor
    out.write(w.to_bytes(2, 'little') + h.to_bytes(2, 'little'))
    out.write((flags & 7).to_bytes(1, 'little'))
    out.write(((flags >> 3) & 7).to_bytes(1, 'little'))
    out.write(bgindex.to_bytes(1, 'little'))
    out.write(pxaspectratio.to_bytes(1, 'little'))

    # Write global color table
    for rgb in colortable:
        out.write(b'\x00' + bytes(rgb))

    # Write image descriptor
    out.write(b'\x2c\x00\x00\x00')
    out.write(w.to_bytes(2, 'little') + h.to_bytes(2, 'little'))
    out.write((flags & 7).to_bytes(1, 'little'))
    out.write(((flags >> 3) & 7).to_bytes(1, 'little'))
    out.write(bgindex.to_bytes(1, 'little'))
    out.write(pxaspectratio.to_bytes(1, 'little'))

    # Write local color table
    for i in range(1 << ((flags >> 3) & 7)):
        out.write(b'\x00' + bytes([i] * 3))

    # Write data
    datalen = 65536 - out.tell()
    if datalen > 0:
        data = [ord(rng.read(1)) for _ in range(datalen)]
        out.write(header + datalen.to_bytes(2, 'big'))
    out.write(data)

