#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(width)
    out.write(height)
    out.write(flags)
    out.write(bg)
    out.write(pxsize)
    out.write(color)

    # Generate and write the global color table
    for i in range(0, int.from_bytes(color, 'big')):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')
        out.write(r)
        out.write(g)
        out.write(b)

    # Generate and write the image descriptor
    left   = bytes([0x00])
    top    = bytes([0x00])
    width  = bytes([(int.from_bytes(rng.read(1), 'big')) & 0xff])
    height = bytes([(int.from_bytes(rng.read(1), 'big')) & 0xff])
    flags  = bytes([0x0c])
    trans  = bytes([0x00])
    out.write(left)
    out.write(top)
    out.write(width)
    out.write(height)
    out.write(flags)
    out.write(trans)

    # Generate and write the local color table
    if int.from_bytes(flags, 'big') & 0x80 == 0x80:
        for i in range(0, int.from_bytes(color, 'big')):
            r = int.from_bytes(rng.read(1), 'big')
            g = int.from_bytes(rng.read(1), 'big')
            b = int.from_bytes(rng.read(1), 'big')
            out.write(r)
            out.write(g)
            out.write(b)

    # Generate and write the graphics control extension
    label      = bytes([0x21])
    blocksize  = bytes([0xf9])
    size       = bytes([0x4])
    flags      = bytes([0x04])
    delaytime  = bytes([0x00])
    transindex = bytes([0x00])
    out.write(label)
    out.write(blocksize)
    out.write(size)
    out.write(flags)
    out.write(delaytime)
    out.write(transindex)

    # Generate and write the plain text extension
    label        = bytes([0x21])
    blocksize    = bytes([0x0b])
    name         = bytes([0x54])
    size         = bytes([0x07])
    authorname   = bytes([0x4d])
    authoremail  = bytes([0x6f])
    description  = bytes([0x6e])
    title        = bytes([0x65])
    version      = bytes([0x73])
    url          = bytes([0x61])
    copyright    = bytes([0x6f])
    licencetype  = bytes([0x72])
    licenseurl   = bytes([0x6f])
    out.write(label)
    out.write(blocksize)
    out.write(name)
    out.write(size)
    out.write(authorname)
    out.write(authoremail)
    out.write(description)
    out.write(title)
    out.write(version)
    out.write(url)
    out.write(copyright)
    out.write(licencetype)
    out.write(licenseurl)

    # Generate and write the application extension
    label      = bytes([0x21])
    blocksize  = bytes([0xff])
    appid      = bytes([0x51])
    appauth    = bytes([0x75])
    appver     = bytes([0x69])
    applang    = bytes([0x63])
    applanglen = bytes([0x00])
    data       = bytes([0x47])
    datalen    = bytes([0x49])
    out.write(label)
    out.write(blocksize)
    out.write(appid)
    out.write(appauth)
    out.write(appver)
    out.write(applang)
    out.write(applanglen)
    out.write(data)
    out.write(datalen)

    # Generate and write the comment extension
    label      = bytes([0x21])
    blocksize  = bytes([0xfe])
    size       = bytes([0x00])
    out.