#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    
    # Write the magic header
    out.write(b'GIF89a')

    # Generate some random values
    width  = rng.read(2)
    height = rng.read(2)
    bpp    = rng.read(1)[0] & 7
    flags  = rng.read(1)[0]
    
    # Pack these together as little-endian unsigned shorts
    width  = struct.unpack('<H', width)[0]
    height = struct.unpack('<H', height)[0]

    # Write the screen descriptor block
    out.write(struct.pack('<HHBBB', width, height, bpp, 0x80 | (flags & 0x4), 0))

    # Calculate the length of the palette
    pallen = ((bpp + 1) * 3) if (flags & 0x8)!= 0 else 0

    # Generate some random bytes for the global color table
    # and write them to the output stream
    gct = rng.read((pallen * 256) - 1)
    out.write(gct)

    # Calculate the size of each frame in bytes
    datalen = (((width * bpp) + 7) // 8) * height

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)