#!/usr/bin/env python3
import sys
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Generate and write the logical screen descriptor
    width  = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    height = (int.from_bytes(rng.read(1), 'big') % 256) + 1
    flags  = bytes([0x87])
    bg     = bytes([int.from_bytes(rng.read(1), 'big')])
    pxsize = bytes([int.from_bytes(rng.read(1), 'big')])
    color  = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(width+height+flags+bg+pxsize+color)

    # Generate and write the global color table
    for i in range(pow(2, int.from_bytes(pxsize, 'big'))):
        r = bytes([(i >>  0) & 0xFF])
        g = bytes([(i >>  8) & 0xFF])
        b = bytes([(i >> 16) & 0xFF])

        out.write(r+g+b)

    # Generate and write the image descriptor
    left      = bytes([0x00])
    top       = bytes([0x00])
    width     = bytes([width])
    height    = bytes([height])
    flags     = bytes([0x00])
    delay     = bytes([0xFB])
    trans_idx = bytes([0x00])
    out.write(left+top+width+height+flags+delay+trans_idx)

    # Generate and write the local color table
    if int.from_bytes(pxsize, 'big') == 8:
        for i in range(256):
            r = bytes([(i >>  0) & 0xFF])
            g = bytes([(i >>  8) & 0xFF])
            b = bytes([(i >> 16) & 0xFF])

            out.write(r+g+b)

    # Generate and write the pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([int.from_bytes(rng.read(1), 'big')]))


if