#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The generated file has a size of at most `maxsize`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, maxsize=5000
    ):

    # Generate a random color table with 256 colors
    colortable = [None] * (256*3)
    for i in range(len(colortable)):
        colortable[i] = rng.read(1)[0]

    # Write the GIF header and color table to the output stream
    out.write(b'GIF89a')
    out.write(struct.pack('<HHB', 768, 0, 255))
    for c in colortable:
        out.write(c)

    # Write the image descriptor to the output stream
    out.write(struct.pack('<HHHHBBBBBHB',
        0, 0, 768, 768, 0, 0, 0, 0, 0, 0, 0
    ))

    # Write the graphics control extension block to the output stream
    out.write(b'\x21\xf9\x04\x00\x00\x00\xff\x00\x00\x00\x00')

    # Write the application extension block to the output stream
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')
    out.write(b'\x00\x00\x00!\xf9\x04\x00\x00\x00\xff\x00\x00\x00\x00,\x00')

    # Write the plain text extension block to the output stream
    out.write(b'\x21\x00\x0bComment Block\x00')

    # Write the logical screen descriptor to the output stream
    out.write(struct.pack('<HHHHBBBBB',
        768, 768, 0, 0, 0, 0, 0, 0, 255
    ))

    # Write the global color table to the output stream
    out.write(b'\x00' * len(colortable))

    # Write the image data to the output stream
    datalen = min(int.from_bytes(rng.read(2), 'big'), maxsize - out.tell()) + 1
    data = rng.read(datalen)
    out.write(struct.pack('<H', datalen))
    out.write(data)

    # Write the GIF trailer to the output stream
    out.write(b'\x3b')
