#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The width is specified by `width`, while the height
# is calculated from it.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int
    ) -> None:

    # Write the header
    out.write(b'GIF89a')

    # Calculate the size of the image data (in bytes)
    datalen = 4 * width**2

    # Write the image data size in three bytes
    out.write((datalen & 0xFF0000).to_bytes(1, 'big'))
    out.write((datalen & 0x00FF00).to_bytes(1, 'big'))
    out.write((datalen & 0x0000FF).to_bytes(1, 'big'))

    # Generate some random bytes for the image data itself
    # and write them to the output stream
    data = rng.read(datalen)
    out.write(data)