#!/usr/bin/env python3
import sys
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1024
    height = 768
    color_depth = 8
    global_color_table_flag = True
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write((color_depth << 4 | (global_color_table_flag & 1)).to_bytes(
            1, 'little'))
    out.write(background_color_index.to_bytes(1, 'little'))
    out.write(pixel_aspect_ratio.to_bytes(1, 'little'))

    # Global Color Table
    for i in range(pow(2, color_depth)):
        out.write(i.to_bytes(1, 'little'))
        out.write(i.to_bytes(1, 'little'))
        out.write(i.to_bytes(1, 'little'))

    # Image Descriptor
    image_left = 0
    image_top = 0
    image_width = width
    image_height = height
    local_color_table_flag = False
    interlace_flag = False
    sort_flag = False
    reserved = 0
    lct_size = pow(2, color_depth) - 1
    out.write(image_left.to_bytes(2, 'little'))
    out.write(image_top.to_bytes(2, 'little'))
    out.write(image_width.to_bytes(2, 'little'))
    out.write(image_height.to_bytes(2, 'little'))
    out.write(((local_color_table_flag & 1) << 7
               | (interlace_flag & 1) << 6
               | (sort_flag & 1) << 5
               | (reserved & 3)).to_bytes(1, 'little'))
    out.write(lct_size.to_bytes(1, 'little'))

    # Local Color Table
    if not local_color_table_flag:
        for i in range(pow(2, color_depth)):
            out.write(i.to_bytes(1, 'little'))
            out.write(i.to_bytes(1, 'little'))
            out.write(i.to_bytes(1, 'little'))

    # LZW minimum code size
    mincodesize = 2
    out.write(mincodesize.to_bytes(1, 'little'))

    # Data sub-block
    while True:
        blocklen = rng.read(1)[0]

        if blocklen == 0x00:
            break

        out.write(blocklen.to_bytes(1, 'little'))
        out.write(rng.read(blocklen))

    # Trailer
    out.write(b'\x3B')