#!/usr/bin/env python3
import sys
from typing import BinaryIO


# random number generator `rng` (stdin or /dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # read the first 6 bytes of the file
    header = rng.read(6)

    if len(header) < 6:
        raise EOFError('file too short to be a GIF image')

    # check that it is indeed a valid GIF header
    if not all((
            header[:6] == b'GIF87a',
            header[6:] in [b'', b'00'],
    )):
        raise ValueError('not a valid GIF header')

    # write the same header back
    out.write(header)

    # read the rest of the file and write it back
    while True:
        chunk = rng.read(1024 * 1024)

        if not chunk:
            break

        out.write(chunk)