#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = (int.from_bytes(rng.read(2), 'big') % 640) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 480) + 1
    flags = bytes([
        0b10000000 | # Global color table flag
        0b00000000 | # Sorting flag
        0b00000000 | # Size of global color table
        7            # Background color index
    ])
    bg_color_index = bytes([7])
    pixel_aspect_ratio = bytes([0])
    out.write(width)
    out.write(height)
    out.write(flags)
    out.write(bg_color_index)
    out.write(pixel_aspect_ratio)

    # Local color table
    for i in range(7):
        r = int.from_bytes(rng.read(1), 'big') & 0xff
        g = int.from_bytes(rng.read(1), 'big') & 0xff
        b = int.from_bytes(rng.read(1), 'big') & 0xff
        out.write(r)
        out.write(g)
        out.write(b)

    # Image descriptor
    left = bytes([0])
    top = bytes([0])
    width = bytes([(int.from_bytes(rng.read(2), 'big') % 640) + 1])
    height = bytes([(int.from_bytes(rng.read(2), 'big') % 480) + 1])
    flags = bytes([
        0b10000000 | # Local color table flag
        0b00000000   # Interlace flag
    ])
    lct_size = bytes([7])
    out.write(left)
    out.write(top)
    out.write(width)
    out.write(height)
    out.write(flags)
    out.write(lct_size)

    # Table based image data
    for y in range(height[0]):
        for x in range(width[0]):
            palette_index = bytes([
                int.from_bytes(rng.read(1), 'big') & 0b00001111
            ])
            out.write(palette_index)

    # Trailer
    out.write(b';')

