#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'
    datalen = int.from_bytes(
        rng.read(1), byteorder='big', signed=False
    )

    # Write header and data length to output
    out.write(header + datalen.to_bytes(1, 'big'))

    # Write 256 color palette entries (0-255)
    for i in range(256):
        r = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        g = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        b = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        )

        out.write(r.to_bytes(1, 'big') +
                  g.to_bytes(1, 'big') +
                  b.to_bytes(1, 'big'))

    # Write image descriptor block
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\xff\xff\xff\xff')

    # Write LZW minimum code size
    out.write(b'\x80')

    # Write subblocks until we reach the desired data length
    while len(out.getbuffer()) < datalen:
        lzwminsize = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) % 128

        if lzwminsize == 0:
            lzwminsize += 1

        out.write(lzwminsize.to_bytes(1, 'big'))

        # Generate bytes for each subblock
        subblocklen = int.from_bytes(
            rng.read(1), byteorder='big', signed=False
        ) % 256

        if subblocklen == 0:
            subblocklen += 1

        out.write(subblocklen.to_bytes(1, 'big'))

        for i in range(subblocklen):
            out.write(
                int.from_bytes(
                    rng.read(1), byteorder='big', signed=False
                ).to_bytes(1, 'big')
            )

    # Write trailer
    out.write(b'\x3b')