#!/usr/bin/env python3
import sys
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1024
    height = 768
    color_depth = 8
    global_color_table_flag = False
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write(((color_depth << 7) | (global_color_table_flag << 5)
              | (background_color_index)).to_bytes(1, 'little'))
    out.write(pixel_aspect_ratio.to_bytes(1, 'little'))

    # Global color table
    for i in range(pow(2, color_depth)):
        out.write(i.to_bytes(1, 'little'))

    # Image descriptor
    image_left_position = 0
    image_top_position = 0
    image_width = width
    image_height = height
    local_color_table_flag = True
    interlace_flag = False
    sort_flag = False
    reserved_bits = 0
    lct_size = pow(2, color_depth) - 1
    out.write((image_left_position).to_bytes(2, 'little'))
    out.write((image_top_position).to_bytes(2, 'little'))
    out.write((image_width).to_bytes(2, 'little'))
    out.write((image_height).to_bytes(2, 'little'))
    out.write(((local_color_table_flag << 7) | (interlace_flag << 6)
              | (sort_flag << 5) | (reserved_bits << 3) | lct_size).to_bytes(1, 'little'))

    # Local color table
    for i in range(lct_size):
        out.write(i.to_bytes(1, 'little'))

    # LZW minimum code size
    min_code_size = 2
    out.write(min_code_size.to_bytes(1, 'little'))

    # Data sub-block
    block_size = 255
    while len(data := rng.read(block_size)) > 0:
        if len(data) < block_size:
            block_size = len(data)

        out.write(len(data).to_bytes(1, 'little'))
        out.write(data)