#!/usr/bin/env python3
import sys
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # header
    out.write(b'GIF89a')

    # logical screen descriptor
    width = 1024
    height = 768
    color_depth = 8
    global_color_table_flag = False
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    packed = (
            0x80 if global_color_table_flag else 0 |
            ((color_depth - 1) << 4) |
            (background_color_index & 0xf))
    out.write(packed.to_bytes(1, 'little'))
    out.write((pixel_aspect_ratio).to_bytes(1, 'little'))

    # local color table
    for i in range(0, pow(2, color_depth)):
        out.write(((i * 51) % 256).to_bytes(1, 'little'))
        out.write(((i * 101) % 256).to_bytes(1, 'little'))
        out.write(((i * 151) % 256).to_bytes(1, 'little'))

    # image descriptor
    left = 0
    top = 0
    width = 1024
    height = 768
    local_color_table_flag = True
    interlace_flag = False
    sort_flag = False
    reserved = 0
    lctsize = 2 ** color_depth
    out.write(left.to_bytes(2, 'little'))
    out.write(top.to_bytes(2, 'little'))
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    packed = (
            0x80 if local_color_table_flag else 0 |
            (interlace_flag << 6) |
            (sort_flag << 5) |
            (reserved << 3) |
            (lctsize - 1))
    out.write(packed.to_bytes(1, 'little'))

    # graphics control extension
    block_size = 0x04
    packed = 0x00
    delaytime = 0xff
    transparent_idx = 0x00
    terminator = 0x00
    out.write(block_size.to_bytes(1, 'little'))
    out.write(packed.to_bytes(1, 'little'))
    out.write(delaytime.to_bytes(2, 'little'))
    out.write(transparent_idx.to_bytes(1, 'little'))
    out.write(terminator.to_bytes(1, 'little'))

    # plain text extension
    block_size = 0x0c
    label_length = len('Comment').to_bytes(1, 'little')
    label = bytes('Comment', encoding='ascii')
    text_length = len('Made with Python!').to_bytes(1, 'little')
    text = bytes('Made with Python!', encoding='ascii')
    terminator = 0x00
    out.write(block_size.to_bytes(1, 'little'))
    out.write(label_length)
    out.write(label)
    out.write(text_length)
    out.write(text)
    out.write(terminator.to_bytes(1, 'little'))

    # application extension
    block_size = 0x0b
    app_id_length = len('NETSCAPE2.0').to_bytes(1, 'little')
    app_id = bytes('NETSCAPE2.0', encoding='ascii')
    sub_block_size = 0x03
    loop_count = 0x01
    terminator = 0x00
    out.write(block_size.to_bytes(1, 'little'))
    out.write(app_id_length)
    out.write(app_id)
    out.write(sub_block_size.to_bytes(1, 'little'))
    out.write(loop_count.to_bytes(2, 'little'))
    out.write(terminator.to_bytes(1, 'little'))

    # image data
    for y in