#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = (int.from_bytes(rng.read(2), byteorder='big')) % 65536
    height = (int.from_bytes(rng.read(2), byteorder='big')) % 65536
    flags = bytes([0])
    bg_color = bytes([0, 0, 0])
    pixel_aspect_ratio = bytes([0])
    out.write(width.to_bytes(2, byteorder='big'))
    out.write(height.to_bytes(2, byteorder='big'))
    out.write(flags)
    out.write(bg_color)
    out.write(pixel_aspect_ratio)

    # Global color table
    global_color_table_size = bytes([7 << 4 | 1])
    out.write(global_color_table_size)
    for i in range(7):
        r = int.from_bytes(rng.read(1), byteorder='big') & 0x0f
        g = int.from_bytes(rng.read(1), byteorder='big') & 0x0f
        b = int.from_bytes(rng.read(1), byteorder='big') & 0x0f
        out.write((r << 4 | g).to_bytes(1, byteorder='big'))
        out.write(b.to_bytes(1, byteorder='big'))

    # Image descriptor
    left = bytes([0])
    top = bytes([0])
    image_width = width.to_bytes(2, byteorder='big')
    image_height = height.to_bytes(2, byteorder='big')
    flags = bytes([0x80])
    local_color_table_flag = bytes([0])
    interlace_flag = bytes([0])
    sort_flag = bytes([0])
    reserved = bytes([0, 0])
    lct_size = bytes([7 << 4 | 1])
    out.write(left)
    out.write(top)
    out.write(image_width)
    out.write(image_height)
    out.write(flags)
    out.write(local_color_table_flag)
    out.write(interlace_flag)
    out.write(sort_flag)
    out.write(reserved)
    out.write(lct_size)

    # Local color table
    for i in range(7):
        r = int.from_bytes(rng.read(1), byteorder='big') & 0x0f
        g = int.from_bytes(rng.read(1), byteorder='big') & 0x0f
        b = int.from_bytes(rng.read(1), byteorder='big') & 0x0f
        out.write((r << 4 | g).to_bytes(1, byteorder='big'))
        out.write(b.to_bytes(1, byteorder='big'))

    # LZW minimum code size
    mincodesize = bytes([8])
    out.write(mincodesize)

    # Data sub-block
    block_size = bytes([0xff])
    while True:
        block_size = rng.read(1)
        if not len(block_size):
            break

        out.write(block_size)
        block_size = ord(block_size[0])
        out.write(rng.read(block_size))

    # Trailer
    out.write(b';')

