#!/usr/bin/env python3
import sys
from typing import BinaryIO

# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # GIF Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width  = (1 << 6) | ((256 >>  0) & 0x3f)
    height = (1 << 6) | ((240 >>  6) & 0x3f)
    flags  =      (240 &   0xc0)
    bg     =       (240 &   0xf0)
    aspect =        (240 &   0x0f)
    out.write((width).to_bytes(1, 'big'))
    out.write((height).to_bytes(1, 'big'))
    out.write(flags.to_bytes(1, 'big'))
    out.write(bg.to_bytes(1, 'big'))
    out.write(aspect.to_bytes(1, 'big'))

    # Global Color Table
    for i in range(768 // 3):
        r = next(rng) % 256
        g = next(rng) % 256
        b = next(rng) % 256
        out.write(r.to_bytes(1, 'big'))
        out.write(g.to_bytes(1, 'big'))
        out.write(b.to_bytes(1, 'big'))

    # Image Descriptor
    left   =        0
    top    =        0
    width  = (1 << 6) | ((256 >>  0) & 0x3f)
    height = (1 << 6) | ((240 >>  6) & 0x3f)
    flags  =      (240 &   0xc0)
    out.write(left.to_bytes(2, 'big'))
    out.write(top.to_bytes(2, 'big'))
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))
    out.write(flags.to_bytes(1, 'big'))

    # Local Color Table
    for i in range(768 // 3):
        r = next(rng) % 256
        g = next(rng) % 256
        b = next(rng) % 256
        out.write(r.to_bytes(1, 'big'))
        out.write(g.to_bytes(1, 'big'))
        out.write(b.to_bytes(1, 'big'))

    # LZW Minimum Code Size
    mincodesize = 2
    out.write(mincodesize.to_bytes(1, 'big'))

    # Data Stream
    lzwdata = []
    while True:
        # Read a new code from input stream
        c = next(rng)

        if len(lzwdata) == 0xff:
            # Write current buffer to output stream and reset it
            out.write(len(lzwdata).to_bytes(1, 'big'))
            out.write(bytes(lzwdata))

            lzwdata = [c]
        else:
            # Add the new code to the buffer
            lzwdata += [c]

        # Check if we have enough codes to decode a new symbol
        if len(lzwdata) >= 2**mincodesize:
            # Decode a new symbol
            nbits = 1 + mincodesize
            while True:
                try:
                    sym = lzwdata[nbits - 1]
                    break
                except IndexError:
                    pass

                nbits += 1

            # Remove decoded symbols from the buffer
            del lzwdata[:nbits]

            # Write the decoded symbol
            out.write(((sym << 1) | 1).to_bytes(1, 'big'))