#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = (int.from_bytes(rng.read(2), 'big') % 640) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 480) + 1
    flags = bytes([
        0b10000000 | # Global color table flag
        ((int.from_bytes(rng.read(1), 'big') & 7) << 5) | # Size of global color table
        (((int.from_bytes(rng.read(1), 'big') >> 3) & 1) << 7) # Sort flag
    ])
    bg_color = int.from_bytes(rng.read(1), 'big')
    aspect_ratio = bytes([int.from_bytes(rng.read(1), 'big')])
    out.write(width)
    out.write(height)
    out.write(flags)
    out.write(bg_color)
    out.write(aspect_ratio)

    # Global color table
    for i in range(pow(2, flags[1] & 7)):
        r = int.from_bytes(rng.read(1), 'big')
        g = int.from_bytes(rng.read(1), 'big')
        b = int.from_bytes(rng.read(1), 'big')
        out.write(r)
        out.write(g)
        out.write(b)

    # Image descriptor
    left = int.from_bytes(rng.read(2), 'big')
    top = int.from_bytes(rng.read(2), 'big')
    width = (int.from_bytes(rng.read(2), 'big') % 640) + 1
    height = (int.from_bytes(rng.read(2), 'big') % 480) + 1
    flags = bytes([
        0b10000000 | # Local color table flag
        ((int.from_bytes(rng.read(1), 'big') & 7) << 5) | # Size of local color table
        (((int.from_bytes(rng.read(1), 'big') >> 3) & 1) << 7) # Interlace flag
    ])
    delay = int.from_bytes(rng.read(2), 'big')
    transparent_index = int.from_bytes(rng.read(1), 'big')
    out.write(left)
    out.write(top)
    out.write(width)
    out.write(height)
    out.write(flags)
    out.write(delay)
    out.write(transparent_index)

    # Local color table
    if flags[0]:
        for i in range(pow(2, flags[1] & 7)):
            r = int.from_bytes(rng.read(1), 'big')
            g = int.from_bytes(rng.read(1), 'big')
            b = int.from_bytes(rng.read(1), 'big')
            out.write(r)
            out.write(g)
            out.write(b)

    # LZW minimum code size
    lzwmincodesize = pow(2, int.from_bytes(rng.read(1), 'big'))
    out.write(lzwmincodesize)

    # Data sub-block
    while True:
        blocklen = int.from_bytes(rng.read(1), 'big')

        if not blocklen: break

        out.write(blocklen)
        out.write(rng.read(blocklen))

    # Trailer
    out.write(b'\x3b')