#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = int.from_bytes(rng.read(2), byteorder='big') % (2**10 - 1) + 1
    height = int.from_bytes(rng.read(2), byteorder='big') % (2**10 - 1) + 1
    flags = bytes([int.from_bytes(rng.read(1), byteorder='big')])
    background = bytes([int.from_bytes(rng.read(1), byteorder='big')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='big')])
    out.write(struct.pack('>HHBBB', width, height, flags[0], background[0], aspect[0]))

    # Global color table
    colors = [int.from_bytes(rng.read(1), byteorder='big') for _ in range(768)]
    out.write(bytes(colors))

    # Image descriptor
    left = int.from_bytes(rng.read(2), byteorder='big') % (width - 1)
    top = int.from_bytes(rng.read(2), byteorder='big') % (height - 1)
    flags = bytes([int.from_bytes(rng.read(1), byteorder='big')])
    delay = int.from_bytes(rng.read(2), byteorder='big') % (2**10 - 1) + 1
    trans = bytes([int.from_bytes(rng.read(1), byteorder='big')])
    out.write(struct.pack('>HHBBBBB', left, top, flags[0], delay, trans[0], 0))

    # Local color table
    if flags[0] & 0b1000 == 0:
        colors = [int.from_bytes(rng.read(1), byteorder='big') for _ in range(768)]
        out.write(bytes(colors))

    # Image data
    size = int.from_bytes(rng.read(1), byteorder='big') % ((width * height) // 4) + 1
    lzwmincodesize = int.from_bytes(rng.read(1), byteorder='big') % (2**3 - 1) + 1
    out.write(bytes([lzwmincodesize]))
    out.write(rng.read(size))

    # Trailer
    out.write(b';')

