#!/usr/bin/env python3
import sys
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 256
    height = 100
    color_depth = 8
    global_color_table_flag = False
    background_color_index = 0
    pixel_aspect_ratio = 0
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write(((color_depth << 7) | (global_color_table_flag << 6)
              | (background_color_index)).to_bytes(1, 'little'))
    out.write(pixel_aspect_ratio.to_bytes(1, 'little'))

    # Global color table
    for i in range(pow(2, color_depth)):
        out.write(i.to_bytes(1, 'little'))
        out.write(i.to_bytes(1, 'little'))
        out.write(i.to_bytes(1, 'little'))

    # Image descriptor
    left = 0
    top = 0
    width = 256
    height = 100
    local_color_table_flag = True
    interlace_flag = False
    sort_flag = False
    reserved = 0
    lct_size = pow(2, color_depth) - 1
    out.write((left).to_bytes(2, 'little'))
    out.write((top).to_bytes(2, 'little'))
    out.write((width).to_bytes(2, 'little'))
    out.write((height).to_bytes(2, 'little'))
    out.write((((local_color_table_flag << 7)
               | (interlace_flag << 6)) | (sort_flag << 5)
               | (reserved << 3) | lct_size).to_bytes(1, 'little'))

    # Local color table
    for i in range(lct_size):
        out.write(i.to_bytes(1, 'little'))
        out.write(i.to_bytes(1, 'little'))
        out.write(i.to_bytes(1, 'little'))

    # Data sub-block
    block_size = 256 * 100 // 8
    while block_size > 0:
        if block_size >= 255:
            out.write(255.to_bytes(1, 'little'))
            out.write(rng.read(255))
            block_size -= 255
        else:
            out.write(block_size.to_bytes(1, 'little'))
            out.write(rng.read(block_size))
            break
