#!/usr/bin/env python3
import sys
from typing import BinaryIO


# random number generator `rng` (stdin or /dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # read the first 6 bytes of the file to get the version and size
    header = rng.read(6)

    if len(header) < 6:
        raise ValueError('File too small.')

    # check that this is a valid GIF
    if not header[:6] == b'GIF87a':
        print("Invalid GIF.")
        return -1

    # extract the length from the header
    datalen = int.from_bytes(header[3:], byteorder='big')

    # read the rest of the file
    data = rng.read()

    # write it all back to stdout
    out.write(header + data)