#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The generated image is of size (width x height) pixels with
# each pixel having color depth `depth`.
def generate_random_gif(
        rng: BinaryIO, width: int, height: int, depth: int, out: BinaryIO
    ) -> None:

    # Header and logical screen descriptor
    header = b'GIF89a'
    out.write(header)
    out.write(struct.pack('<HHBBB', width, height, depth, 0, 0))

    # Global color table
    global_color_table_size = pow(2, depth) - 1
    for i in range(global_color_table_size):
        r = ord(rng.read(1))
        g = ord(rng.read(1))
        b = ord(rng.read(1))
        out.write(struct.pack('BBB', r, g, b))
    
    # Image descriptor
    out.write(struct.pack('<BHBHHBB', 0x2c, 0, 0, width, height, 0, 0))

    # Local color table
    local_color_table_size = 0
    if global_color_table_size < 256:
        local_color_table_size = global_color_table_size
    else:
        local_color_table_size = 256
    for i in range(local_color_table_size):
        r = ord(rng.read(1))
        g = ord(rng.read(1))
        b = ord(rng.read(1))
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    num_pixels = width * height
    for i in range(num_pixels):
        out.write(struct.pack('B', ord(rng.read(1))))

    # Trailer
    out.write(b';')