#!/usr/bin/env python3

import sys, os
from pathlib import Path
import subprocess
import time
import shutil
import json

# ------------------------------------------------------------------------------
def run_cmd(cmd):
    print("Running:", cmd)
    return subprocess.run([cmd], shell=True).returncode

# ------------------------------------------------------------------------------
def get_version():
    version = None
    with open('package.json') as fp:
        data = json.load(fp)
        version = data['version']
    if not version:
        raise Exception("Couldn't find version in package.json")
    return version

# ------------------------------------------------------------------------------
def build_electron():
    # Build the electron app
    if run_cmd('npm run dist'):
        raise Exception("Failed to build electron app")

# ------------------------------------------------------------------------------
def copy_files():
    # Copy files into the release directory
    release_dir = 'release'
    if os.path.exists(release_dir):
        shutil.rmtree(release_dir)
    os.mkdir(release_dir)
    
    for file in ['README.md', 'LICENSE', 'package.json', 'main.js', 'renderer.js']:
        shutil.copyfile(file, os.path.join(release_dir, file))
        
    shutil.copytree('assets', os.path.join(release_dir, 'assets'))
    shutil.copytree('node_modules', os.path.join(release_dir, 'node_modules'))
    shutil.copytree('dist', os.path.join(release_dir, 'dist'))

# ------------------------------------------------------------------------------
def create_dmg():
    # Create a dmg from the release directory
    release_dir = 'release'
    dmg_name = "Songbook-{}.dmg".format(get_version())
    dmg_path = os.path.join('build', dmg_name)
    if os.path.exists(dmg_path):
        os.remove(dmg_path)

    if run_cmd('hdiutil create {} -srcfolder {}'.format(dmg_path, release_dir)):
        raise Exception("Failed to create dmg")

# ------------------------------------------------------------------------------
def create_debian():
    # Create a debian package from the release directory
    release_dir = 'release'
    deb_name = "songbook_{}_amd64.deb".format(get_version().replace('.', '_'))
    deb_path = os.path.join('build', deb_name)
    if os.path.exists(deb_path):
        os.remove(deb_path)

    if run_cmd('fpm -s dir -t deb --deb-compression xz -C {} -v {} -n songbook -a amd64 -m "Nathaniel Foster <nate@natefoster.net>" .'.format(release_dir, get_version())):
        raise Exception("Failed to create debian package")

# ------------------------------------------------------------------------------