#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical Screen Descriptor
    width = 1 << rng.read(1)[0] & 0x7f
    height = 1 << rng.read(1)[0] & 0x7f
    color_table_size = 2 ** rng.read(1)[0] & 0x07
    background_color_index = rng.read(1)[0] & 0xff
    pixel_aspect_ratio = rng.read(1)[0] & 0xff
    out.write(struct.pack('<HBBB', width, height, color_table_size, background_color_index))
    out.write(pixel_aspect_ratio.to_bytes(1, 'little'))

    # Global Color Table
    for _ in range(0, 3 * (2**(color_table_size+1))):
        out.write(rng.read(1))

    # Application Extension Block
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
    out.write(b'\x00')
    out.write(b'\x00')
    out.write(b'\x00')

    # Image Descriptor
    out.write(b'\x2C')
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(b'\x00')
    out.write(b'\x00')

    # Local Color Table
    out.write(b'\x00')

    # Image Data
    for y in range(0, height):
        for x in range(0, width):
            out.write(rng.read(1))

    return True

