#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG and with the given dimensions.
def generate_gif(rng: BinaryIO, out: BinaryIO, width: int, height: int) -> None:
    # Header
    out.write(b'GIF')
    out.write(b'89a')
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    global_color_table_size = 7 + rng.read(1)[0] % 8
    global_color_table_sort_flag = rng.read(1)[0] & 1
    global_color_table_colors = [None] * (2 ** global_color_table_size)
    for i in range(len(global_color_table_colors)):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        global_color_table_colors[i] = (red, green, blue)
    out.write(struct.pack('B', global_color_table_size | (global_color_table_sort_flag << 7)))
    for rgb in global_color_table_colors:
        out.write(struct.pack('BBB', *rgb))

    # Image block
    image_left = rng.read(1)[0] % 256
    image_top = rng.read(1)[0] % 256
    image_width = rng.read(1)[0] % 256 + 1
    image_height = rng.read(1)[0] % 256 + 1
    local_color_table_flag = rng.read(1)[0] >> 7
    interlace_flag = rng.read(1)[0] & 1
    sort_flag = rng.read(1)[0] & 1
    local_color_table_size = 7 + rng.read(1)[0] % 8
    out.write(struct.pack('<HHBB', image_left, image_top, image_width, image_height))
    out.write(struct.pack('BB', local_color_table_flag | (interlace_flag << 1) | (sort_flag << 2), local_color_table_size))
    if local_color_table_flag == 1:
        local_color_table_colors = [None] * (2 ** local_color_table_size)
        for i in range(len(local_color_table_colors)):
            red = rng.read(1)[0] % 256
            green = rng.read(1)[0] % 256
            blue = rng.read(1)[0] % 256
            local_color_table_colors[i] = (red, green, blue)
        for rgb in local_color_table_colors:
            out.write(struct.pack('BBB', *rgb))

    # LZW minimum code size
    lzw_minimum_code_size = rng.read(1)[0] % 128
    out.write(struct.pack('B', lzw_minimum_code_size))

    # Pixel data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))