#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')
    
    width = 10 + rng.read(2)[0] % 50
    height = 10 + rng.read(2)[0] % 50
    out.write(struct.pack('<HH', width, height))

    # Global color table
    out.write(b'\x00\xff\xff\xff')

    # Logical screen descriptor
    out.write(b'\x00\x00\xff\x0bNETSCAPE2.0\x03\x01')

    # Application extension block
    out.write(b'\x21\xf9\x04\x01\x00\x00\x00')

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00'+struct.pack('<HH', width, height)+b'\x00')

    # Local color table
    out.write(b'\x00\xff\xff\xff')

    # Raster data
    for y in range(height):
        for x in range(width):
            out.write(b'\x00')

    return True

