#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 16 + rng.read(2)[0] % 40
    height = 16 + rng.read(2)[0] % 40
    background = rng.read(1)[0] & 7
    color_table_size = 1 << ((rng.read(1)[0] >> 4) + 1)
    global_color_table = bool(rng.read(1)[0] & 0x80)
    sort_flag = bool(rng.read(1)[0] & 0x08)
    color_resolution = (rng.read(1)[0] >> 4) + 1
    pixel_aspect_ratio = rng.read(1)[0] & 0x0f
    out.write(struct.pack('<HHBB', width, height, background, color_table_size))
    if global_color_table:
        for i in range(color_table_size * 3):
            out.write(bytes([rng.read(1)[0]]))
    else:
        out.write(b'\xff\xff\xff')

    # Global color table
    out.write(b'\x00')

    # Image descriptor
    left = rng.read(2)[0] % width
    top = rng.read(2)[0] % height
    width = 16 + rng.read(2)[0] % (width - left)
    height = 16 + rng.read(2)[0] % (height - top)
    local_color_table = bool(rng.read(1)[0] & 0x80)
    interlace = bool(rng.read(1)[0] & 0x40)
    sort_flag = bool(rng.read(1)[0] & 0x08)
    reserved = rng.read(1)[0] & 0x07
    out.write(struct.pack('<HHBBB', left, top, width, height, local_color_table))

    # Local color table
    if local_color_table:
        for i in range(color_table_size * 3):
            out.write(bytes([rng.read(1)[0]]))
    else:
        out.write(b'\xff\xff\xff')

    # LZW minimum code size
    out.write(bytes([rng.read(1)[0] & 0x07]))

    # Pixel data
    while True:
        code = rng.read(1)[0] & 0xff
        if code == 0x00:
            break
        elif code == 0x01:
            out.write(b'\x00')
        elif code <= 0x80:
            out.write(bytes([code]))
        else:
            raise Exception("Invalid code")
