#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HBBB', width, height, flags, bg))
    out.write(aspect)

    # Global color table
    colors = [
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng.read(3), byteorder='big'),
        0xff000000 | int.from_bytes(rng