#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    width = rng.read(1)[0] % 40 + 1
    height = rng.read(1)[0] % 40 + 1
    flags = bytes([
        0x80 | rng.read(1)[0], # Global color table flag
        0x07 & rng.read(1)[0]]) # Color resolution and sort order
    bg = bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]])
    aspect = bytes([rng.read(1)[0], rng.read(1)[0]])
    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags)
    out.write(bg)
    out.write(aspect)

    # Global color table
    for _ in range(0xFF):
        out.write(bytes([
            rng.read(1)[0],
            rng.read(1)[0],
            rng.read(1)[0]]))

    # Application extension block
    ext_block_size = bytes([0x0B])
    app_id = b'NETSCAPE2.0'
    loop_count = bytes([0x00, 0x01, 0x00])
    out.write(ext_block_size)
    out.write(app_id)
    out.write(loop_count)

    # Image descriptor
    image_descriptor = bytes([
        0x2C, # Image separator
        0x00, 0x00, # Image position X
        0x00, 0x00, # Image position Y
        0x00, 0x00, # Image width
        0x00, 0x00]) # Image height
    out.write(image_descriptor)

    # Local color table
    local_color_table_flag = bytes([0x80])
    local_color_table_size = bytes([0x07])
    out.write(local_color_table_flag)
    out.write(local_color_table_size)
    for _ in range(0xFF):
        out.write(bytes([
            rng.read(1)[0],
            rng.read(1)[0],
            rng.read(1)[0]]))

    # LZW minimum code size
    lzw_minimum_code_size = bytes([0x08])
    out.write(lzw_minimum_code_size)

    # Image data
    while True:
        try:
            out.write(bytes([rng.read(1)[0]]))
        except IndexError:
            break
