#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main(argv: list[str]) -> int:
    if len(argv) != 2:
        print("Usage: {} <file>".format(os.path.basename(__file__)), file=sys.stderr)
        return 1
    
    with open(argv[1], "rb") as f:
        data = f.read()
        
    # Parse the header
    magic, version, num_entries = struct.unpack("<4sLL", data[:16])
    assert magic == b"FONT"
    assert version == 0x00010000
    
    # Parse the entries
    entry_size = 8 + 5 * 4
    for i in range(num_entries):
        offset, width, height, baseline, left, right = struct.unpack("<QHHHHH", data[16 + i*entry_size : 16 + (i+1)*entry_size])
        print("{}: offset={:#x}, size={}x{}".format(i, offset, width, height))
    
    return 0
