#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xFF)
    aspect = struct.pack('B',   0xFF)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        red   = struct.pack('B', i)
        green = struct.pack('B', i)
        blue  = struct.pack('B', i)
        out.write(red + green + blue)

    # Graphic control extension
    ext_header = struct.pack('BB', 0xF9, 0x04)
    delay      = struct.pack('<H', 0xFFFF)
    transp     = struct.pack('B',  0xFF)
    term       = struct.pack('B',  0x00)
    out.write(ext_header + delay + transp + term)

    # Image descriptor
    desc_header = struct.pack('BB', 0x2C, 0x00)
    xpos        = struct.pack('>H', 0x0000)
    ypos        = struct.pack('>H', 0x0000)
    width       = struct.pack('>H', 1024)
    height      = struct.pack('>H',  768)
    packed      = struct.pack('B',   0x80 | 0x00)
    lct         = struct.pack('B',   0x00)
    interlace   = struct.pack('B',   0x00)
    out.write(desc_header + xpos + ypos + width + height + packed + lct + interlace)

    # Local color table
    for i in range(0, 256):
        red   = struct.pack('B', i)
        green = struct.pack('B', i)
        blue  = struct.pack('B', i)
        out.write(red + green + blue)

    # Pixel data
    for _ in range(0, 1024 * 768):
        pixel = struct.pack('B', rng.read(1)[0])
        out.write(pixel)

    return True

