#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:
    # Header
    out.write(b"GIF89a")
    
    # Logical Screen Descriptor
    width = rng.read(2)[0] + 1
    height = rng.read(2)[0] + 1
    flags = rng.read(1)[0] & 7
    background = rng.read(1)[0]
    aspect = rng.read(1)[0]
    out.write(struct.pack("!HHBB", width, height, flags, background))
    out.write(aspect)
    
    # Global Color Table
    if flags & 1 == 1:
        colors = 2 ** ((flags >> 4) + 1)
        
        for _ in range(colors):
            out.write(rng.read(3))
            
        out.write(b"\x00")
    
    # Application Extension
    out.write(b"\x21\xff\x0bNETSCAPE2.0\x03\x01")
    out.write(struct.pack("!H", 0))
    out.write(b"\x00")
    
    # Graphic Control Extension
    out.write(b"\x21\xf9\x04\x01\x00")
    out.write(rng.read(2))
    out.write(b"\x00")
    
    # Image Descriptor
    out.write(b"\x2c\x00\x00\x00\x00")
    out.write(struct.pack("!HH", width, height))
    out.write(b"\x00")
    
    # Local Color Table
    if flags & 1 == 1:
        out.write(b"\x00")
    
    # Image Data
    out.write(rng.read(width * height))
    
    return True

