#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    pixel  = struct.pack('BBBB', *[ord(rng.read(1)) for _ in range(0, 4)])
    out.write(width + height + flags + bg + pixel)

    # Graphic control extension
    gce = b'\x21\xf9\x04\x05\xff\xff\xff\x00'
    out.write(gce)

    # Image data
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('B', ord(rng.read(1))))

    # Trailer
    out.write(b';')

    return True


