#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    width = rng.read(1)[0] % 40 + 1
    height = rng.read(1)[0] % 40 + 1
    flags = bytes([
        0x80 | rng.read(1)[0], # Global color table flag
        0x07 & rng.read(1)[0]]) # Color resolution and sort order
    bg = bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]])
    aspect = bytes([rng.read(1)[0], rng.read(1)[0]])
    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags)
    out.write(bg)
    out.write(aspect)

    # Logical screen descriptor
    packed = bytes([
        0x00, 0x00, # X position of the window
        0x00, 0x00, # Y position of the window
        width - 1,   # Width of the window
        height - 1]) # Height of the window
    out.write(packed)

    # Global color table
    for i in range(2 ** ((flags[0] >> 4) + 1)):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        packed = bytes([r, g, b])
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))