#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(0xFF):
        out.write(struct.pack('<BBBB', i, i, i, 0))

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x05\xff\x00')

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')

    # Local color table
    for i in range(0xFF):
        out.write(struct.pack('<BBBB', i, i, i, 0))

    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng.read(1)[0]]))

    # Trailer
    out.write(b';')
