#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for _ in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(struct.pack('<BBB', r, g, b))

    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(b'\x00\x00\x00')

    # Graphic control extension
    out.write(b'\x21\xf9\x04\x01\x00\x00\x00')

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')

    # Local color table
    for _ in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        out.write(struct.pack('<BBB', r, g, b))

    # LZW-compressed image data
    for _ in range(width * height):
        out.write(b'\x00')

    # Trailer
    out.write(b'\x3b')
