#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H', 0)
    top       = struct.pack('>H', 0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x80)
    local_map = struct.pack('B',   0x00)
    interlace = struct.pack('B',   0x00)
    sort      = struct.pack('B',   0x00)
    size      = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + local_map + interlace + sort + size)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Graphic control extension
    ext_code = struct.pack('B', 0xf9)
    block_size = struct.pack('B', 0x04)
    flags = struct.pack('B', 0x00)
    delay = struct.pack('>H', 0xffff)
    trans = struct.pack('B', 0x00)
    out.write(ext_code + block_size + flags + delay + trans)

    # Plain text extension
    ext_code = struct.pack('B', 0xfe)
    block_size = struct.pack('B', 0x0c)
    grid_left = struct.pack('>H', 0x0000)
    grid_top = struct.pack('>H', 0x0000)
    grid_width = struct.pack('>H', 0x0000)
    grid_height = struct.pack('>H', 0x0000)
    cell_width = struct.pack('B', 0x00)
    cell_height = struct.pack('B', 0x00)
    fg = struct.pack('B', 0xff)
    bg = struct.pack('B', 0x00)
    out.write(ext_code + block_size + grid_left + grid_top + grid_width + grid_height + cell_width + cell_height + fg + bg)

    # Application extension
    ext_code = struct.pack('B', 0xff)
    block_size = struct.pack('B', 0x0b)
    app_id = struct.pack('>I', 0x4e455447)
    auth_code = struct.pack('>I', 0x20212020)
    data = struct.pack('>I', 0x00000000)
    out.write(ext_code + block_size + app_id + auth_code + data)

    # Trailer
    out.write(b'\x3b')

    return True

