#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):
    # Header
    signature = b'GIF89a'
    version   = b'a' * 3
    width     = int.from_bytes(rng.read(2), byteorder='little') % 65536 + 1
    height    = int.from_bytes(rng.read(2), byteorder='little') % 65536 + 1
    fields    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg_color  = int.from_bytes(rng.read(1), byteorder='little')
    aspect    = int.from_bytes(rng.read(1), byteorder='little')
    out.write(signature + version + struct.pack('<H', width) + struct.pack('<H', height))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), byteorder='little') % (width - 1) + 1
    top    = int.from_bytes(rng.read(2), byteorder='little') % (height - 1) + 1
    width  = int.from_bytes(rng.read(2), byteorder='little') % ((width - left) // 2) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % ((height - top) // 2) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    lct    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    interlaced = False if not bool(flags[0] & 0x40) else True
    sort    = False if not bool(flags[0] & 0x20) else True
    local_color_table_size = lct[0]
    background_index      = int.from_bytes(rng.read(1), byteorder='little')
    pixel_aspect_ratio    = int.from_bytes(rng.read(1), byteorder='little')
    out.write(struct.pack('<HHHHBB', left, top, width, height, flags[0], lct[0]))
