#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the given width and height.
def generate_gif(out: BinaryIO, width: int, height: int) -> None:
    # Magic number.
    out.write(b"GIF89a")

    # Logical screen descriptor.
    out.write(struct.pack("<HHBB", width, height, 0xF7, 0))

    # Global color table.
    for i in range(256):
        r = (i >> 5) & 0x1F
        g = (i >> 2) & 0x1F
        b = (i >> 0) & 0x1F
        out.write(bytes([r << 3 | r >> 2, g << 3 | g >> 2, b << 3 | b >> 2]))

    # Graphic control extension.
    out.write(b"\x21\xF9\x04\x08\x00\x00\xFF\xFF\xFF")

    # Image descriptor.
    out.write(struct.pack("<HHHHB", 0, 0, width, height, 0))

    # Local color table.
    for i in range(256):
        r = (i >> 5) & 0x1F
        g = (i >> 2) & 0x1F
        b = (i >> 0) & 0x1F
        out.write(bytes([r << 3 | r >> 2, g << 3 | g >> 2, b << 3 | b >> 2]))

    # LZW-compressed image data.
    out.write(b"\x00\x2C\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00;")

    # Trailer.
    out.write(b";")
