#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f
        out.write(struct.pack('BBB', r, g, b))

    # Graphic control extension
    block_size       = struct.pack('B', 0x04)
    block_id         = struct.pack('B', 0xf9)
    flags            = struct.pack('B', 0x00)
    delay_time       = struct.pack('<H', 0xffff)
    transparent_idx  = struct.pack('B', 0xff)
    terminator       = struct.pack('B', 0x00)
    out.write(block_size + block_id + flags + delay_time + transparent_idx + terminator)

    # Image descriptor
    image_separator  = struct.pack('B', 0x2c)
    left             = struct.pack('>H', 0x0000)
    top              = struct.pack('>H', 0x0000)
    width            = struct.pack('>H', 1024)
    height           = struct.pack('>H',  768)
    flags            = struct.pack('B',   0x80)
    out.write(image_separator + left + top + width + height + flags)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f
        out.write(struct.pack('BBB', r, g, b))

    # LZW minimum code size
    lzw_minimum_code_size = struct.pack('B', 0x08)
    out.write(lzw_minimum_code_size)

    # Pixel data
    while True:
        pixel = ord(rng.read(1)) % 256
        if pixel == 0x00:
            break
        else:
            out.write(struct.pack('B', pixel))

    return True

