#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    width = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1
    color_table_size = rng.read(1)[0] % 8 + 1
    background_color_index = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] % 8 + 1

    # Header
    out.write(b'GIF')
    out.write(b'89a')
    out.write(struct.pack('<HHHBB', width, height, 0xF7 & ~(1 << (color_table_size - 1)), 0, 0))

    # Color table
    for i in range(2 ** color_table_size):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))