#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical Screen Descriptor
    width = 1024
    height = 768
    background = 0xFF
    aspect = 0
    out.write(struct.pack('<HHBB', width, height, background, aspect))

    # Global Color Table
    for i in range(0, 256):
        color = (i << 16) | (i << 8) | i
        out.write(struct.pack('<I', color))

    # Application Extension Block
    extension_block_size = 11
    application_id = 'A' * 8
    authentication_code = 'B' * 3
    data = 'C' * 10
    block = struct.pack('<BB8s3s10s', extension_block_size, 0xff, application_id, authentication_code, data)
    out.write(struct.pack('<H', len(block)))
    out.write(block)

    # Image Descriptor
    image_left = 0
    image_top = 0
    image_width = width
    image_height = height
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    reserved = 0
    lzw_minimum_code_size = 8
    image_descriptor = struct.pack('<HHBBBBB', image_left, image_top, image_width, image_height, local_color_table_flag, interlace_flag, sort_flag, reserved, lzw_minimum_code_size)
    out.write(image_descriptor)

    # Local Color Table
    for i in range(0, 256):
        color = (i << 16) | (i << 8) | i
        out.write(struct.pack('<I', color))

    # LZW-compressed data
    for _ in range(0, 1000000):
        out.write(struct.pack('<B', rng.read(1)[0]))
