#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 25 + 10
    background = rng.read(1)[0] & 0xF
    color_table_size = 7
    flags = 0
    out.write(struct.pack('<HBB', width, height, flags))

    # Global color table
    for i in range(3 * (1 << color_table_size)):
        out.write(bytes([rng.read(1)[0]]))

    # Application extension
    out.write(b'\x21\xFF\x0BNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<I', 0))

    # Image block
    out.write(b'\x2C\x00\x00\x00\x00')
    out.write(struct.pack('<HH', width, height))
    out.write(b'\x00')

    # Local color table
    for i in range(3 * (1 << color_table_size)):
        out.write(bytes([rng.read(1)[0]]))

    # LZW minimum code size
    out.write(b'\x08')

    # Pixel data
    while True:
        # Block header
        if rng.read(1)[0] == 0:
            break
        else:
            out.write(b'\x21')
        
        # Block size
        out.write(bytes([rng.read(1)[0]]))

        # Block data
        for _ in range(rng.read(1)[0]):
            out.write(bytes([rng.read(1)[0]]))
