#!/usr/bin/env python3
import random
import sys

if len(sys.argv) != 2:
    print('Usage: {} <output>'.format(sys.argv[0]), file=sys.stderr)
    exit(1)

with open(sys.argv[1], 'wb') as out:
    rng = random.Random()

    # Write the GIF header
    out.write(b'GIF89a')
    # Write the global color table
    for i in range(768):
        out.write(bytes([(i >> ((~i + 1) % 8)) & 0xFF]))

    # Write the application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Write the image descriptor
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Write the local color table
    for i in range(768):
        out.write(bytes([(i >> ((~i + 1) % 8)) & 0xFF]))

    # Write the image data
    out.write(b'\x00\x00')

    # Write the LZW compressed image data
    lzw_min_code_sizes = [8, 4, 2, 0]
    for lzw_min_code_size in lzw_min_code_sizes:
        out.write(bytes([lzw_min_code_size]))

        # Read the LZW code stream and write it as-is
        while True:
            code = rng.read(1)[0]
            if code == 0:
                break
            else:
                out.write(bytes([code]))

    # Write the trailer
    out.write(b';')
