#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1
    background_color = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] % 256
    out.write(struct.pack('<HHBB', width, height, background_color, pixel_aspect_ratio))

    # Global color table
    global_color_table_size = 7 + rng.read(1)[0] % 8
    global_color_table_sort_flag = rng.read(1)[0] & 1
    global_color_table_colors = [None] * (2 ** global_color_table_size)
    for i in range(len(global_color_table_colors)):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        global_color_table_colors[i] = (red, green, blue)
    out.write(struct.pack('B', global_color_table_size | (global_color_table_sort_flag << 7)))
    for color in global_color_table_colors:
        if color is None:
            continue
        out.write(bytes(color))

    # Pixel data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))