#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H', 0)
    top       = struct.pack('>H', 0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',   0x00 | 0x08 | 0x04)
    bg        = struct.pack('B',   0x00)
    trans     = struct.pack('B',   0x00)
    interlace = struct.pack('B',   0x00)
    out.write(left + top + width + height + flags + bg + trans + interlace)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            r += 1
            g += 1
            b += 1

        out.write(struct.pack('BBB', r, g, b))

    # Graphic control extension
    ext_code = struct.pack('B', 0xf9)
    length   = struct.pack('B', 0x04)
    flags    = struct.pack('B', 0x00 | 0x01)
    delay    = struct.pack('<H', 0xffff)
    trans    = struct.pack('B', 0x00)
    out.write(ext_code + length + flags + delay + trans)

    # Plain text extension
    ext_code = struct.pack('B', 0xfe)
    length   = struct.pack('B', 0x0c)
    block    = struct.pack('B', 0x01)
    grid     = struct.pack('B', 0x00)
    cellw    = struct.pack('B', 0x08)
    cellh    = struct.pack('B', 0x08)
    fg       = struct.pack('B', 0xff)
    bg       = struct.pack('B', 0x00)
    out.write(ext_code + length + block + grid + cellw + cellh + fg + bg)

    # Text data
    for y in range(0, 768 // 8):
        for x in range(0, 1024 // 8):
            byte = 0x00
            for bit in range(0, 8):
                byte <<= 1
                byte |= ord(rng.read(1)) & 0x01
            out.write(struct.pack('B', byte))

    return True


