#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG object, which must implement `.read()` and return bytes.
def generate_gif(rng: BinaryIO, out: BinaryIO) -> None:
    width = 800
    height = 400

    # Header
    out.write(b'GIF89a')
    out.write(struct.pack('<H', width))
    out.write(struct.pack('<H', height))
    out.write(b'\xF7\xFF\xFF')

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))