#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the image descriptor
    out.write(struct.pack('<H', rng.read(2)))
    out.write(struct.pack('B', rng.read(1)[0]))
    out.write(struct.pack('B', rng.read(1)[0]))
    out.write(struct.pack('B', rng.read(1)[0]))
    out.write(struct.pack('B', rng.read(1)[0]))
    out.write(struct.pack('B', rng.read(1)[0]))
    out.write(struct.pack('B', rng.read(1)[0]))
    out.write(struct.pack('B', rng.read(1)[0]))
    out.write(struct.pack('B', rng.read(1)[0]))
    out.write(struct.pack('B', rng.read(1)[0]))

    # Write the LZW data
    while True:
        byte = rng.read(1)
        if not byte:
            break
        out.write(byte)

    # Write the trailer
    out.write(b';')
