#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width, height = 120, 60
    color_table_size = 7
    global_color_table_flag = 0x80
    sort_flag = 0
    color_resolution = 7
    background_color_index = 0
    pixel_aspect_ratio = 0
    lsd = struct.pack('<HHBBB', width, height, color_table_size | global_color_table_flag, sort_flag, color_resolution, background_color_index, pixel_aspect_ratio)
    out.write(lsd)

    # Global color table
    for i in range(2**color_table_size):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))

    # Image block
    image_separator = b'\x2c\x00\x00\x00'
    out.write(image_separator)

    # Image descriptor
    left, top, width, height = 0, 0, 120, 60
    local_color_table_flag = 0
    interlace_flag = 0
    sort_flag = 0
    reserved = 0
    lct_size = 0
    id = struct.pack('<HHBBBB', left, top, width, height, local_color_table_flag | lct_size, interlace_flag, sort_flag, reserved)
    out.write(id)

    # Local color table
    for i in range(2**lct_size):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack('BBB', red, green, blue))

    # Raster data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % (2 ** (color_table_size + 1))
            out.write(struct.pack('B', index))

    # Trailer
    trailer = b'\x3b'
    out.write(trailer)
