#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    width = rng.read(1)[0] % 40 + 1
    height = rng.read(1)[0] % 40 + 1
    flags = bytes([
        0x80 | rng.read(1)[0], # Global color table flag
        0x07 & rng.read(1)[0]]) # Color resolution and sort order
    bg = bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]])
    aspect = bytes([rng.read(1)[0], rng.read(1)[0]])
    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags)
    out.write(bg)
    out.write(aspect)

    # Logical screen descriptor
    out.write(b'\x00\x21\xf9\x04\x01\x00\x00\x00\x00\x00')

    # Global color table
    for i in range(2 << ((flags[0] >> 4) & 0x07)):
        out.write(bytes([
            rng.read(1)[0],
            rng.read(1)[0],
            rng.read(1)[0]]))

    # Image block
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00\x00')
    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags)
    out.write(bg)

    # Local color table
    for i in range(2 << ((flags[0] >> 4) & 0x07)):
        out.write(bytes([
            rng.read(1)[0],
            rng.read(1)[0],
            rng.read(1)[0]]))

    # LZW minimum code size
    out.write(bytes([0x08]))

    # Image data
    while True:
        if rng.read(1)[0] == 0:
            break
        else:
            out.write(bytes([rng.read(1)[0]]))
