#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 2 ** rng.read(1)[0] + 1
    height = 2 ** rng.read(1)[0] + 1
    background = rng.read(1)[0] % 256
    aspect_ratio = rng.read(1)[0] % 4
    global_color_table_flag = rng.read(1)[0] & 0x80 != 0
    color_resolution = (rng.read(1)[0] & 0x70) >> 4
    sort_flag = rng.read(1)[0] & 0x08 != 0
    size_of_global_color_table = 2 ** ((rng.read(1)[0] & 0x07) + 1)
    out.write(struct.pack('<HHBBB', width, height, background, aspect_ratio,
                          global_color_table_flag | (color_resolution << 7) |
                          (sort_flag << 3) | size_of_global_color_table))

    # Global color table
    if global_color_table_flag:
        for _ in range(size_of_global_color_table * 3):
            out.write(bytes([rng.read(1)[0]]))

    # Application extension
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01')
    out.write(struct.pack('<H', 0))

    # Image descriptor
    out.write(b'\x2c\x00\x00\x00\x00')

    # Local color table
    local_color_table_flag = rng.read(1)[0] & 0x80 != 0
    interlace_flag = rng.read(1)[0] & 0x40 != 0
    sort_flag = rng.read(1)[0] & 0x08 != 0
    size_of_local_color_table = 2 ** ((rng.read(1)[0] & 0x07) + 1)
    out.write(struct.pack('<BBB', local_color_table_flag | (interlace_flag << 7) |
                          (sort_flag << 3) | size_of_local_color_table))

    # Local color table
    if local_color_table_flag:
        for _ in range(size_of_local_color_table * 3):
            out.write(bytes([rng.read(1)[0]]))

    # LZW minimum code size
    out.write(bytes([rng.read(1)[0]]))

    # Data sub-blocks
    while True:
        block_size = rng.read(1)[0]
        if block_size == 0:
            break
        else:
            out.write(bytes([block_size]))
            for _ in range(block_size):
                out.write(bytes([rng.read(1)[0]]))
