#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        out.write(struct.pack('BBB', *[i] * 3))

    # Graphic control extension
    gc_ext = struct.pack('BBB', 0x21, 0xf9, 0x04)
    gc_ext += struct.pack('B', 0x00)
    gc_ext += struct.pack('<H', 0xffff)
    gc_ext += struct.pack('B', 0x00)
    out.write(gc_ext)

    # Image descriptor
    idesc = struct.pack('BBB', 0x2c, 0x00, 0x00)
    idesc += struct.pack('B', 0x00)
    idesc += struct.pack('<HH', 1024, 768)
    idesc += struct.pack('BB', 0x00, 0x00)
    out.write(idesc)

    # Local color table
    lct = b'\x00\x00\x00' * 256
    out.write(lct)

    # Pixel data
    pixels = b''
    while True:
        try:
            pixels += rng.read(1024*768)
        except Exception as e:
            print("Failed to read from RNG", file=sys.stderr)
            return False

    out.write(pixels)

    # Trailer
    out.write(b';')

    return True

