#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.


def main(argv: list[str]) -> int:
    if len(argv)!= 2:
        print("Usage:", argv[0], "<output>")

    with open("/dev/urandom", "rb") as rng, open(argv[1], "wb+") as out:
        # Read the header.
        data = rng.read(10)

        # Parse the header.
        width, height = struct.unpack("<HH", data[:4])
        color_table_size = (data[4] & 0b1111) + 1
        has_color_table = bool(data[9] & 1 << 7)

        # Write the header.
        out.write(struct.pack("<6sH3B", b"GIF89a", width, height,
                              color_table_size, 0, 0))

        # Generate the image.
        for y in range(height):
            for x in range(width):
                # Get the next byte and write it to the stream.
                out.write(bytes([next(rng)]))

            # If we're at the end of a line, add an extra byte so that
            # the decoder doesn't get confused.
            out.write(b"\x00")

    return 0


if