#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO):
    # Header
    header = b'GIF89a' + bytes([0x1D])

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % 65536
    height = int.from_bytes(rng.read(2), byteorder='little') % 65536
    flags  = [0b00000000] * 7
    flags[4] |= 0b00000001 # Global color table flag
    flags   = bytes(flags)
    bg      = rng.read(1)[0] & 0xF
    aspect  = rng.read(1)[0] & 0xFF
    lsd     = struct.pack('<HHBBB', width, height, flags[0], flags[1],
                          flags[2], flags[3], bg, aspect)

    # Color table
    colors = []
    for i in range(0, 2**bg):
        r = rng.read(1)[0] & 0xFF
        g = rng.read(1)[0] & 0xFF
        b = rng.read(1)[0] & 0xFF
        colors += [(r,g,b)]

    # Image descriptor
    img_lsd    = rng.read(9)
    img_width  = int.from_bytes(img_lsd[:2], byteorder='little') % 65536
    img_height = int.from_bytes(img_lsd[2:4], byteorder='little') % 65536
    img_flags  = [0b00000000] * 9
    img_flags[8] |= 0b00000001 # Local color table flag
    img_flags   = bytes(img_flags)
    img_lct     = rng.read((1 << bg) * 3)
    id         = struct.pack('<HHBBBBB', img_width, img_height,
                            img_flags[0], img_flags[1], img_flags[2],
                            img_flags[3], img_flags[4], img_flags[5],
                            img_flags[6])

    # Image data
    img_data = b''
    while len(img_data) < img_width*img_height:
        img_data += rng.read(1)

    # Trailer
    trailer = b'\x3B'

    # Write to output
    out.write(header)
    out.write(lsd)
    for c in colors:
        out.write(struct.pack('BBB', c[0], c[1], c[2]))
    out.write(id)
    out.write(img_data)
    out.write(trailer)


if