#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(rng.read(4), 'big') - len(header)

    # Write the header to the output
    out.write(header)

    # Generate the pixel data for each frame in turn
    while True:

        # Determine how many pixels there are in this frame
        width = (int.from_bytes(rng.read(1), 'big') + 1) & 255
        height = (int.from_bytes(rng.read(1), 'big') + 1) & 255
        num_pixels = width * height

        # If we've reached the end of the image then stop generating frames
        if not num_pixels:
            break

        # Write the block size and the LZW minimum code size
        out.write(struct.pack('>H', 7 + num_pixels))
        out.write(b'\x08')

        # Write the palette
        for i in range(0, 256):

            # Get three random bytes
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]

            # Write them as RGB triples
            out.write(struct.pack('<BBB', r, g, b))

        # Write the image descriptor
        out.write(struct.pack('>HHHHB',
            0, 0, width, height, 0
        ))

        # Write the actual pixel data
        for _ in range(num_pixels):
            out.write(rng.read(1))


if