#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 25 + 10
    color_table_size = rng.read(1)[0] & 7
    background_color = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0] & 7

    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height,
                         color_table_size << 4 | pixel_aspect_ratio,
                         0, background_color))

    # Global color table
    for _ in range(1 << color_table_size):
        out.write(struct.pack('BBBB', *rng.read(4)))

    # Image descriptor
    left = rng.read(1)[0] % width
    top = rng.read(1)[0] % height
    width = min((width - left), rng.read(1)[0] % 10 + 1)
    height = min((height - top), rng.read(1)[0] % 10 + 1)
    local_color_table_flag = bool(rng.read(1)[0] & 1)
    interlace_flag = bool(rng.read(1)[0] & 64)
    sort_flag = bool(rng.read(1)[0] & 128)
    reserved = rng.read(1)[0]
    lzw_minimum_code_size = rng.read(1)[0] & 0x0F

    out.write(struct.pack('<HHBBBBB', left, top, width, height,
                         local_color_table_flag << 7 | interlace_flag << 6 |
                         sort_flag << 5 | reserved, lzw_minimum_code_size))

    if not local_color_table_flag:
        for _ in range(1 << color_table_size):
            out.write(struct.pack('BBBB', *rng.read(4)))

    # LZW code stream
    while True:
        byte = rng.read(1)[0]

        if byte == 0xFF:
            break

        out.write(bytes([byte]))


if