#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image descriptor
    packed = struct.pack('<HHBB', 0, 0, 0xF7, 0x00)
    out.write(packed)

    # Local color table
    for i in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    while True:
        chunk_size = rng.read(1)[0]

        if not chunk_size:
            break

        out.write(chunk_size.to_bytes(1, 'little'))
        out.write(rng.read(chunk_size))


if