#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    header = b'GIF89a'

    # Header
    out.write(header)

    # Logical screen descriptor
    width  = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 16) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    bg     = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    aspect = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    out.write(struct.pack('<HHBBB', width, height, flags[0], bg[0], aspect[0]))

    # Global color table
    for i in range(768):
        out.write(bytes([int.from_bytes(rng.read(1), byteorder='little')]))

    # Image descriptor
    left   = int.from_bytes(rng.read(2), byteorder='little') % (width - 1) + 1
    top    = int.from_bytes(rng.read(2), byteorder='little') % (height - 1) + 1
    width  = int.from_bytes(rng.read(2), byteorder='little') % ((width - left) // 2) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % ((height - top) // 2) + 1
    flags  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    lct    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    interlaced = False if not bool(flags[0] & 0x40) else True
    sort    = False if not bool(flags[0] & 0x20) else True
    local_color_table_size = int((lct[0] >> 5) + 1)
    global_color_table_flag = False if not bool(lct[0] & 0b10000000) else True
    size_of_local_color_table = None
    if local_color_table_size == 1:
        size_of_local_color_table = 2 ** 12
    elif local_color_table_size == 2:
        size_of_local_color_table = 2 ** 13
    elif local_color_table_size == 3:
        size_of_local_color_table = 2 ** 14
    elif local_color_table_size == 4:
        size_of_local_color_table = 2 ** 15
    elif local_color_table_size == 5:
        size_of_local_color_table = 2 ** 16
    elif local_color_table_size == 6:
        size_of_local_color_table = 2 ** 17
    elif local_color_table_size == 7:
        size_of_local_color_table = 2 ** 18
    else:
        raise ValueError('Invalid value of LCT Size!')
    if global_color_table_flag and size_of_local_color_table is not None:
        raise ValueError('Global Color Table Flag set but Local Color Table Size is also set!')
    local_color_table_sort_flag = False if not bool(lct[0] & 0b01000000) else True
    reserved                  = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    local_color_table_size    = bytes([int.from_bytes(rng.read(1), byteorder='little')])
    local_color_table        = []
    if local_color_table_size!= 0:
        for _ in range(size_of_local_color_table):
            local_color_table.append(bytes([int.from_bytes(rng.read(1), byteorder='little')] * 3))
    disposal_method          = int.from_bytes(rng.read(1), byteorder='little')
    user_input               = int.from_bytes(rng.read(1), byteorder='little')
    background_index      = int.from_bytes(rng.read(1), byteorder='little')
    pixel_aspect_ratio    = int.from_bytes(rng.read(1), byteorder='little')
    out.write(struct.pack('<HHHHBB', left, top, width, height, flags[0], lct[0]))
