#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

if __name__ == '__main__':
    width: int = 800
    height: int = 480
    filename: str = 'test_image.ppm'

    try:
        with open(filename, mode='wb') as f:
            write_ppm(f, width=width, height=height)
    except Exception as e:
        print("Error:", e)
        exit(-1)

    print("Done")

def write_ppm(out: BinaryIO, width: int, height: int) -> None:
    """Write a PPM file to the given output stream"""
    # Magic number
    out.write(b"P6\n")

    # Comment line
    out.write(str.encode("# Created by Python script\n"))

    # Dimensions
    out.write(str.encode("%d %d\n" % (width, height)))

    # Maximum color value
    out.write(b"255\n")

    # Pixels
    rng = os.urandom()
    for _ in range(width * height):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256

            if not ((index == 0) and (y < height // 2)):
                out.write(struct.pack('B', index))