#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)
    out.flush()

    while datalen > 0:

        # Determine how much we can write in this chunk
        chunklen = min(256, datalen)
        out.write(rng.read(chunklen))
        datalen -= chunklen
