#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1
    background_color = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] % 256
    out.write(struct.pack('<HHBB', width, height, background_color, pixel_aspect_ratio))

    # Global color table
    global_color_table_size = 7 + rng.read(1)[0] % 8
    global_color_table_sort_flag = rng.read(1)[0] & 1
    global_color_table_colors = [None] * (2 ** global_color_table_size)
    for i in range(len(global_color_table_colors)):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        global_color_table_colors[i] = (red, green, blue)
    out.write(struct.pack('B', global_color_table_size | (global_color_table_sort_flag << 7)))
    for rgb in global_color_table_colors:
        out.write(struct.pack('BBB', *rgb))

    # Image block
    image_left = rng.read(1)[0] % 256
    image_top = rng.read(1)[0] % 256
    image_width = rng.read(1)[0] % 256 + 1
    image_height = rng.read(1)[0] % 256 + 1
    local_color_table_flag = rng.read(1)[0] & 1
    interlace_flag = rng.read(1)[0] & 1
    sort_flag = rng.read(1)[0] & 1
    reserved_flags = rng.read(1)[0] & 3
    lct_size = 7 + rng.read(1)[0] % 8
    out.write(struct.pack('<HHBBBBBB', image_left, image_top, image_width, image_height, \
        local_color_table_flag, interlace_flag, sort_flag, reserved_flags, lct_size))
    if not local_color_table_flag:
        for rgb in global_color_table_colors:
            out.write(struct.pack('BBB', *rgb))

    # Pixel data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))