#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def read_uint16(f: BinaryIO) -> int:
    return struct.unpack('>H', f.read(2))[0]

def write_uint16(f: BinaryIO, value: int):
    f.write(struct.pack('>H', value))


def main():
    if len(sys.argv)!= 4 or not (os.path.isfile(sys.argv[1]) and os.path.isdir(sys.argv[2])):
        print("Usage: {} <input-file> <output-dir>".format(sys.argv[0]), file=sys.stderr)
        exit(-1)

    with open(sys.argv[1], 'rb') as input_file:
        # Read header
        magic = input_file.read(8).decode()

        if magic!= "WAVEfmt ":
            raise ValueError("Invalid WAV file")

        chunk_size = read_uint32(input_file)
        audio_format = read_uint16(input_file)
        num_channels = read_uint16(input_file)
        sample_rate = read_uint32(input_file)
        byte_rate = read_uint32(input_file)
        block_align = read_uint16(input_file)
        bits_per_sample = read_uint16(input_file)

        if chunk_size > 16:
            extra_chunk_size = chunk_size - 16

            while extra_chunk_size > 0:
                sub_chunk_id = input_file.read(4).decode()

                if sub_chunk_id == "data":
                    break

                sub_chunk_size = read_uint32(input_file)
                extra_chunk_size -= sub_chunk_size + 8

                input_file.seek(sub_chunk_size, 1)

        data_chunk_id = input_file.read(4).decode()

        if data_chunk_id!= "data":
            raise ValueError("Invalid WAV file")

        data_chunk_size = read_uint32(input_file)


        # Write output files
        for i in range(num_channels):
            channel_name = chr((i % 26) + ord('a'))

            with open("{}/{}.wav".format(sys.argv[2], channel_name), 'wb') as output_file:
                output_file.write("RIFF".encode())
                write_uint32(output_file, 36)
                output_file.write("WAVE".encode())
                output_file.write("fmt ".encode())
                write_uint32(output_file, 16)
                write_uint16(output_file, audio_format)
                write_uint16(output_file, 1)
                write_uint32(output_file, sample_rate)
                write_uint32(output_file, byte_rate)
                write_uint16(output_file, block_align)
                write_uint16(output_file, bits_per_sample)
                output_file.write("data".encode())
                write_uint32(output_file, data_chunk_size // num_channels)

                for j in range(data_chunk_size // num_channels):
                    output_file.write(bytes([next(input_file) for _ in range(num_channels)]))



if