#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(0xFF):
        r = rng.read(1)[0] & 0xE0 | 0x0F
        g = rng.read(1)[0] & 0xE0 | 0x0F
        b = rng.read(1)[0] & 0xC0 | 0x1F
        out.write(struct.pack('BBBB', r, g, b, 0))

    # Image descriptor
    packed = struct.pack('<HHHHB', 0, 0, width, height, 0)
    out.write(packed)

    # LZW minimum code size
    out.write(b'\x02')

    # Frame data
    lzw_mincodesize = int.from_bytes(out.read(1), 'little')
    lzw_table = [i for i in range(lzw_mincodesize)]
    lzw_prev = None
    while True:
        if len(lzw_table) == 511:
            lzw_mincodesize += 1

        # Generate new byte from RNG and append to frame data
        lzw_next = rng.read(1)[0]
        if lzw_next not in lzw_table:
            raise Exception("LZW table overflow")
        out.write(bytes([lzw_next]))

        # If we have reached the end of the table, reset it
        if len(lzw_table) == 2 ** lzw_mincodesize - 1:
            lzw_table = []
        
        # Add new bytes to table as needed
        if lzw_prev is not None:
            lzw_table.append(lzw_prev)
            lzw_table.append((lzw_prev << 8) | lzw_next)

        lzw_prev = lzw_next


if