#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1
    background_color = rng.read(1)[0] % 256
    pixel_aspect_ratio = rng.read(1)[0] % 256
    out.write(struct.pack('<HHBB', width, height, background_color, pixel_aspect_ratio))

    # Global color table
    global_color_table_size = 7 + rng.read(1)[0] % 8
    global_color_table_sort_flag = rng.read(1)[0] & 1
    global_color_table_global_flag = rng.read(1)[0] & 1
    if not global_color_table_global_flag:
        raise Exception("Global color table flag must be set")
    out.write(struct.pack('BBB', global_color_table_size, global_color_table_sort_flag, global_color_table_global_flag))
    for i in range(global_color_table_size ** 3):
        rgb = [rng.read(1)[0] % 256 for _ in range(3)]
        out.write(struct.pack('BBB', *rgb))

    # Image descriptor
    image_left_position = rng.read(1)[0] % 256
    image_top_position = rng.read(1)[0] % 256
    image_width = rng.read(1)[0] % 256 + 1
    image_height = rng.read(1)[0] % 256 + 1
    local_color_table_flag = rng.read(1)[0] & 1
    interlace_flag = rng.read(1)[0] & 1
    sort_flag = rng.read(1)[0] & 1
    reserved_flags = rng.read(1)[0] & 7
    lct_size = 7 + rng.read(1)[0] % 8
    out.write(struct.pack('<HHBBBBBB',
                         image_left_position, image_top_position,
                         image_width, image_height,
                         local_color_table_flag, interlace_flag, sort_flag, reserved_flags, lct_size))

    # Local color table
    if local_color_table_flag:
        for i in range(lct_size ** 3):
            rgb = [rng.read(1)[0] % 256 for _ in range(3)]
            out.write(struct.pack('BBB', *rgb))

    # LZW minimum code size
    lzw_minimum_code_size = rng.read(1)[0] % 128
    out.write(struct.pack('B', lzw_minimum_code_size))

    # Pixel data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))