#!/usr/bin/env python3

import os, sys, argparse, re


def main():
    parser = argparse.ArgumentParser(description="""
        This script is used to generate a list of all the files in a directory and its subdirectories that match a given pattern.

        The output file can be used as input for `xargs` or other command line tools such as `grep`.""")

    parser.add_argument("directory", help="The root directory from which to search")
    parser.add_argument("-p", "--pattern", default=".*", help="A regular expression pattern (default: %(default)s)")
    parser.add_argument("--include-dirs", action='store_true', help="Include directories in the output")
    args = parser.parse_args()

    if not os.path.isdir(args.directory):
        print("Error: {} is not a valid directory".format(args.directory), file=sys.stderr)
        return 1

    regex = re.compile(args.pattern)

    def matches(filename):
        if filename == ".":
            # Skip current directory
            return False
        elif filename == "..":
            # Skip parent directory
            return False
        else:
            return bool(regex.match(filename))

    with open(os.path.join(args.directory, "files"), 'w') as f:
        for dirpath, _, filenames in os.walk(args.directory):
            for filename in sorted(filenames):
                filepath = os.path.join(dirpath, filename)

                if matches(filepath):
                    f.write(filepath + "\n")

            if args.include_dirs:
                for dirname in sorted(filter(matches, os.listdir(dirpath))):
                    f.write(os.path.join(dirpath, dirname) + "/")


if