#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):

    # Header
    header = b'GIF89a' + bytes([0x12]) + bytes([0x34]) + \
             bytes([0x56]) + bytes([0x78]) + bytes([0x9A])

    # Logical screen descriptor
    lsd = bytearray()
    lsd += struct.pack('>H', 0xF000 | ((rng.read(1)[0] & 0x07) << 8))
    lsd += struct.pack('>H', 0x0000 | (rng.read(1)[0]))
    lsd += struct.pack('>H', 0x0000 | (rng.read(1)[0] & 0x0F))
    lsd[3] |= 0xE0
    lsd += struct.pack('>B', 0x00)
    lsd += struct.pack('>B', 0x00)

    # Global color table
    gct = bytearray()
    for i in range(0, 2**lsd[3]+1):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]

        gct += bytes([r])
        gct += bytes([g])
        gct += bytes([b])

    # Image descriptor
    id = bytearray()
    id += struct.pack('>B', 0x2C)
    id += struct.pack('>H', 0x0000 | (rng.read(1)[0] & 0x0F))
    id += struct.pack('>H', 0x0000 | (rng.read(1)[0]))
    id += struct.pack('>H', 0x0000 | (rng.read(1)[0] & 0x0F))
    id += struct.pack('>H', 0x0000 | (rng.read(1)[0]))
    id += struct.pack('>B', 0x00)
    id += struct.pack('>B', 0x00)
    id += struct.pack('>B', 0x00)

    # Local color table
    lct = bytearray()
    if len(gct) > 0:
        lct += bytes([0xFF]) * 3
        lct += bytes([0x00]) * 3
        lct += bytes([0x00]) * 3

    # Image data
    imgdata = bytearray()
    imgdata += bytes([0x00]) * 1024

    # Trailer
    trailer = b';'

    # Write everything to output
    out.write(header)
    out.write(lsd)
    out.write(gct)
    out.write(id)
    out.write(lct)
    out.write(imgdata)
    out.write(trailer)


if