#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    width = rng.read(1)[0] % 40 + 1
    height = rng.read(1)[0] % 40 + 1
    flags = bytes([
        0x80 | rng.read(1)[0], # Global color table flag
        0x07 & rng.read(1)[0]]) # Color resolution and sort order
    bg = bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]])
    aspect = bytes([rng.read(1)[0], rng.read(1)[0]])
    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags)
    out.write(bg)
    out.write(aspect)

    # Global color table
    for _ in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image descriptor
    left = 0
    top = 0
    width -= 1
    height -= 1
    flags = bytes([
        0x80 | rng.read(1)[0], # Local color table flag
        0x07 & rng.read(1)[0]]) # Color resolution and sort order
    lctsize = 0
    if flags[0]:
        lctsize = 1 << ((flags[1] >> 3) + 1)
        for i in range(lctsize):
            r = rng.read(1)[0] % 256
            g = rng.read(1)[0] % 256
            b = rng.read(1)[0] % 256
            packed = struct.pack('BBB', r, g, b)
            out.write(packed)
    else:
        out.write(b'\x00\x00\x00')
    out.write(left.to_bytes(2, 'little'))
    out.write(top.to_bytes(2, 'little'))
    out.write(width.to_bytes(2, 'little'))
    out.write(height.to_bytes(2, 'little'))
    out.write(flags)
    out.write((lctsize - 1).to_bytes(1, 'little'))

    # Pixel data
    while True:
        control = rng.read(1)[0]
        if not control or control == 0xFF: break

        copy = min(control, 0xF9)
        out.write(struct.pack('<B', copy))

        if copy >= 0xE0: continue

        if copy > 0x0F:
            offset = rng.read(2)[0] * 256 + rng.read(1)[0]
            length = rng.read(1)[0] + 1

            out.seek(-offset, os.SEEK_END)
            buf = out.read(length)
            out.seek(-offset, os.SEEK_END)
            out.write(buf)
        elif copy == 0x00:
            out.write(b'\x00')
        elif copy == 0xF9:
            out.write(b'\x00\x00')


if