#!/usr/bin/env python3
import sys
import struct
from random import Random


if __name__ == '__main__':
    width = int(sys.argv[1])
    height = int(sys.argv[2])
    seed = int(sys.argv[3])

    rng = Random()
    rng.seed(seed)

    with open('out.gif', 'wb') as out:

        # Header
        signature = b'GIF89a'
        version = b'\x00\x00'
        screen_width = width
        screen_height = height
        global_color_table_flag = 1
        color_resolution = 7
        sort_flag = 1
        global_color_table_size = 7
        background_color_index = 0
        pixel_aspect_ratio = 0
        out.write(signature)
        out.write(version)
        out.write(struct.pack('<HHBBB', screen_width, screen_height, global_color_table_flag, color_resolution, sort_flag, global_color_table_size))
        # Image block
    image_left = rng.read(1)[0] % 256
    image_top = rng.read(1)[0] % 256
    image_width = rng.read(1)[0] % 256 + 1
    image_height = rng.read(1)[0] % 256 + 1
    local_color_table_flag = rng.read(1)[0] >> 7
    interlace_flag = rng.read(1)[0] & 1
    sort_flag = rng.read(1)[0] & 1
    local_color_table_size = 7 + rng.read(1)[0] % 8
    out.write(struct.pack('<HHBB', image_left, image_top, image_width, image_height))
    out.write(struct.pack('BB', local_color_table_flag | (interlace_flag << 1) | (sort_flag << 2), local_color_table_size))
    if local_color_table_flag == 1:
        local_color_table_colors = [None] * (2 ** local_color_table_size)
        for i in range(len(local_color_table_colors)):
            red = rng.read(1)[0] % 256
            green = rng.read(1)[0] % 256
            blue = rng.read(1)[0] % 256
            local_color_table_colors[i] = (red, green, blue)
        for rgb in local_color_table_colors:
            out.write(struct.pack('BBB', *rgb))

    # LZW minimum code size
    lzw_minimum_code_size = rng.read(1)[0] % 128
    out.write(struct.pack('B', lzw_minimum_code_size))

    # Pixel data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))