#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 256 + int.from_bytes(rng.read(1), 'big') % 256
    height = 256 + int.from_bytes(rng.read(1), 'big') % 256
    packed = b'\x07\x00\x00' if width < height else b'\x00\x07\x00'
    packed += bytes([width & 0xff])
    packed += bytes([(width >> 8) & 0xff])
    packed += bytes([height & 0xff])
    packed += bytes([(height >> 8) & 0xff])
    packed += b'\x87\x00'
    out.write(packed)

    # Global color table
    for i in range(0, 256):
        r = int.from_bytes(rng.read(1), 'big') % 256
        g = int.from_bytes(rng.read(1), 'big') % 256
        b = int.from_bytes(rng.read(1), 'big') % 256
        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    packed = b'\x2c'
    packed += bytes([0, 0, 0, 0])
    packed += bytes([0, 0])
    packed += b'\x00'
    out.write(packed)

    # Local color table
    for i in range(0, 256):
        r = int.from_bytes(rng.read(1), 'big') % 256
        g = int.from_bytes(rng.read(1), 'big') % 256
        b = int.from_bytes(rng.read(1), 'big') % 256
        out.write(struct.pack('BBB', r, g, b))

    # Image data
    while True:
        block_size = int.from_bytes(rng.read(1), 'big')

        if block_size == 0:
            break
        
        out.write(block_size.to_bytes(1, byteorder='big'))
        out.write(rng.read(block_size))


if