#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    palette = [struct.pack('BBB', *rng.read(3)) for _ in range(256)]
    out.write(bytes(palette))

    # Image descriptor
    packed = struct.pack('<HHBBH', 0, 0, 0xF7, 0x00, 0x00)
    out.write(packed)

    # LZW minimum code size
    out.write(b'\x01')

    # Pixel data
    while True:
        byte = rng.read(1)[0]

        if byte == 0xFF:
            break

        elif byte >= 0xE0 and byte <= 0xEF:
            continue

        else:
            out.write(byte.to_bytes(1, 'big'))


if