#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    # Read the header and determine the length of the image data
    header = bytearray()
    for i in range(10):
        header += struct.pack('B', int.from_bytes(rng.read(1), 'big'))
    width, height = struct.unpack('<HH', header[6:8])
    datalen = (width * height) // 8

    # Write the header to the output
    out.write(header)

    # Write the rest of the data to the output
    while datalen > 0:
        chunklen = min(256, datalen)
        out.write(rng.read(chunklen))
        datalen -= chunklen
