#!/usr/bin/env python3

import os
import sys
import struct
from collections import namedtuple


# https://github.com/python-pillow/Pillow/blob/master/src/PIL/_binary.py
def _unpack_bytes(fmt, data):
    size = struct.calcsize("=" + fmt)
    return struct.unpack("!" + fmt, data[:size])[0], data[size:]


def _unpack_string(data):
    length, data = _unpack_bytes(">I", data)

    if len(data) < length:
        raise ValueError("not enough bytes to unpack string")

    value, data = data[:length].decode(), data[length:]
    return value, data


def _unpack_list(data):
    count, data = _unpack_bytes(">I", data)

    values = []
    for i in range(count):
        value, data = _unpack_value(data)

        values.append(value)

    return values, data


def _unpack_dict(data):
    keys, data = _unpack_list(data)
    values, data = _unpack_list(data)

    return dict(zip(keys, values)), data


def _unpack_value(data):
    type_, data = _unpack_bytes("B", data)

    if type_ == 1:
        # null
        return None, data

    elif type_ == 2:
        # boolean
        value, data = _unpack_bytes("?", data)
        return bool(value), data

    elif type_ == 3:
        # integer
        value, data = _unpack_bytes(">q", data)
        return int(value), data

    elif type_ == 4:
        # float
        value, data = _unpack_bytes(">d", data)
        return float(value), data

    elif type_ == 5:
        # string
        return _unpack_string(data)

    elif type_ == 6:
        # list
        return _unpack_list(data)

    elif type_ == 7:
        # dictionary
        return _unpack_dict(data)

    else:
        raise ValueError("unknown value type %s" % (type_))


def load(filename):
    with open(filename, "rb") as f:
        data = f.read()

    result, data = _unpack_value(data)

    assert not data

    return result


if