#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        width: int, height: int,
        num_frames: int,
        out: BinaryIO):

    # Write the header
    out.write(b'GIF89a')

    # Write the logical screen descriptor
    out.write(struct.pack('<HHBBB', width, height, 0xF7, 0xFF, 0))

    # Write the global color table (all black)
    for i in range(256):
        out.write(b'\x00\x00')

    # Write the LZW compressed image data
    lzw_min_code_sizes = [8, 4, 2, 0]
    for lzw_min_code_size in lzw_min_code_sizes:
        out.write(bytes([lzw_min_code_size]))

        # Read the LZW code stream and write it as-is
        while True:
            code = rng.read(1)[0]
            if code == 0:
                break
            else:
                out.write(bytes([code]))

    # Write the trailer
    out.write(b';')
