#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    width = rng.read(1)[0] % 40 + 1
    height = rng.read(1)[0] % 40 + 1
    flags = bytes([
        0x80 | rng.read(1)[0], # Global color table flag
        0x07 & rng.read(1)[0]]) # Color resolution and sort order
    bg = bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]])
    aspect = bytes([rng.read(1)[0], rng.read(1)[0]])
    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags)
    out.write(bg)
    out.write(aspect)

    # Global color table
    for _ in range(0xFF):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]
        out.write((r << 2).to_bytes(1, 'little'))
        out.write((g << 2).to_bytes(1, 'little'))
        out.write((b << 2).to_bytes(1, 'little'))

    # Image descriptor
    left = 0
    top = 0
    width -= 1
    height -= 1
    flags = bytes([
        0x80 | rng.read(1)[0], # Local color table flag
        0x07 & rng.read(1)[0]]) # Color resolution and sort order
    lct = [None]*0xFF if (flags[0] >> 7) == 1 else None
    interlace = bool(rng.read(1)[0])
    sorted = bool(rng.read(1)[0])
    reserved = rng.read(1)[0]
    lctSize = 2**(flags[0] & 0x07)
    lctIndex = 0
    out.write(left.to_bytes(1, 'little'))
    out.write(top.to_bytes(1, 'little'))
    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags)
    if lct is not None:
        for i in range(lctSize):
            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]
            out.write((r << 2).to_bytes(1, 'little'))
            out.write((g << 2).to_bytes(1, 'little'))
            out.write((b << 2).to_bytes(1, 'little'))

    # LZW minimum code size
    minCodeSize = rng.read(1)[0]
    out.write(minCodeSize.to_bytes(1, 'little'))

    # Data sub-block
    dataSubBlock = bytearray()
    while len(dataSubBlock) < ((width+1)*(height+1)):
        dataSubBlock += rng.read(1)
    out.write(dataSubBlock)

    # Trailer
    out.write(b';')


if