#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = 1 + int.from_bytes(rng.read(1), 'big') % 256
    height = 1 + int.from_bytes(rng.read(1), 'big') % 256
    flags = b'\x07\x00' if width > height else b'\x00\x07'
    bg = rng.read(1)[0]
    aspect = rng.read(1)[0]
    out.write(struct.pack('>HHBBB', width, height, flags[0], flags[1], bg, aspect))

    # Global color table
    for _ in range(int.from_bytes(rng.read(1), 'big')):
        out.write(rng.read(3))

    # Image descriptor
    left = 0
    top = 0
    width = 1 + int.from_bytes(rng.read(1), 'big') % 256
    height = 1 + int.from_bytes(rng.read(1), 'big') % 256
    flags = b'\x04'
    out.write(struct.pack('>HHHHB', left, top, width, height, flags[0]))

    # Local color table
    for _ in range(int.from_bytes(rng.read(1), 'big')):
        out.write(rng.read(3))

    # LZW minimum code size
    lzwminsize = rng.read(1)[0]
    out.write(lzwminsize.to_bytes(1, byteorder='big'))

    # Image data
    while True:
        chunktype = rng.read(1)[0]

        if chunktype == 0x2C:
            break

        elif chunktype!= 0x21:
            continue

        subchunktype = rng.read(1)[0]

        if subchunktype == 0xF9:
            length = int.from_bytes(rng.read(1), 'big')
            end = rng.tell() + length

            while rng.tell() < end:
                out.write(rng.read(1))

        elif subchunktype == 0xFE:
            length = int.from_bytes(rng.read(1), 'big')
            out.write(rng.read(length))

        elif subchunktype == 0xFF:
            length = int.from_bytes(rng.read(1), 'big')
            out.write(rng.read(length))


if