#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> bool:

    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width  = struct.pack('>H', 1024)
    height = struct.pack('>H',  768)
    flags  = struct.pack('B',   0x80 | 0x40 | 0x20)
    bg     = struct.pack('B',   0xff)
    aspect = struct.pack('B',   0x00)
    out.write(width + height + flags + bg + aspect)

    # Global color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            continue

        out.write(struct.pack('BBB', r, g, b))

    # Image descriptor
    left      = struct.pack('>H',      0)
    top       = struct.pack('>H',      0)
    width     = struct.pack('>H', 1024)
    height    = struct.pack('>H',  768)
    flags     = struct.pack('B',       0x80 | 0x40)
    local_map = struct.pack('B',         0x00)
    out.write(left + top + width + height + flags + local_map)

    # Local color table
    for i in range(0, 256):
        r = ord(rng.read(1)) & 0x3f
        g = ord(rng.read(1)) & 0x3f
        b = ord(rng.read(1)) & 0x3f

        if ((r == 0) and (g == 0) and (b == 0)):
            continue

        out.write(struct.pack('BBB', r, g, b))

    # Pixel data
    for y in range(0, 768):
        for x in range(0, 1024):

            # Get next byte from RNG
            c = ord(rng.read(1)) & 0xff

            # If it's zero, use the previous one instead
            if (c == 0):
                c = prev_byte

            # Write to output stream
            out.write(bytes([c]))

            # Update previous byte
            prev_byte = c

    return True


if