#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    width = rng.read(1)[0] % 40 + 1
    height = rng.read(1)[0] % 40 + 1
    flags = bytes([
        0x80 | rng.read(1)[0], # Global color table flag
        0x07 & rng.read(1)[0]]) # Color resolution and sort order
    bg = bytes([rng.read(1)[0], rng.read(1)[0], rng.read(1)[0]])
    aspect = bytes([rng.read(1)[0], rng.read(1)[0]])
    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags)
    out.write(bg)
    out.write(aspect)

    # Global color table
    for i in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            if index == 0:
                continue

            out.write(struct.pack('BB', 0x21, 0xF9))
            out.write(struct.pack('BBBB',
                0x04, # Graphics control label
                0x08, # Length of block
                0x00, # Packed field
                0x00)) # Delay time

            out.write(struct.pack('BB', 0x2C, 0x00))
            out.write(struct.pack('HHHBB',
                0x0000, # Left position
                0x0000, # Top position
                x * 2 - 1, # Width
                y * 2 - 1, # Height
                0x00)) # Flags

            out.write(struct.pack('B', index))