#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Write the rest of the data to the output
    while datalen > 0:
        chunklen = min(datalen, 256)

        if chunklen == 256:
            out.write(b'\x00\xff\xff')
        else:
            out.write((chunklen - 1).to_bytes(length=1, byteorder='big'))

        out.write(rng.read(chunklen))
        datalen -= chunklen

