#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (os.urandom)
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ):
    # Header
    out.write(b'GIF89a')

    # Logical screen descriptor
    width = rng.read(1)[0] % 40 + 10
    height = rng.read(1)[0] % 40 + 10
    packed = struct.pack('<HHBBB', width, height, 0xF7, 0x00, 0x00)
    out.write(packed)

    # Global color table
    for i in range(0xFF):
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256
        packed = struct.pack('BBB', r, g, b)
        out.write(packed)

    # Image data
    for y in range(height):
        for x in range(width):
            index = rng.read(1)[0] % 256
            out.write(struct.pack('B', index))

    # Graphics control extension
    packed = struct.pack('<BBHB', 0x21, 0xF9, 4, 0b11111111)
    out.write(packed)

    # Application extension
    packed = struct.pack('<BBH', 0x21, 0xFF, 11)
    out.write(packed)
    out.write(b'NETSCAPE2.0')
    out.write(b'\x03\x01')
    out.write(b'\x00')

    # Comment extension
    comment_length = rng.read(1)[0] % 100
    packed = struct.pack('<BBH', 0x21, 0xFE, comment_length)
    out.write(packed)
    for _ in range(comment_length):
        c = rng.read(1)[0] % 256
        if not chr(c).isprintable():
            c = ord('.')
        out.write(bytes([c]))

    # Trailer
    out.write(b';')