#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom)
def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):

    header = b'GIF89a'
    # 0x100 is arbitrary but should be enough to cover most cases
    for _ in range(0x100):
        header += bytes([rng.read(1)[0]])

    # Write the header
    out.write(header)

    # Write the logical screen descriptor
    width = rng.read(1)[0] & 0x3F | (rng.read(1)[0] & 0xC0) << 2
    height = rng.read(1)[0] & 0x3F | (rng.read(1)[0] & 0xC0) << 2
    packed = rng.read(1)[0]
    background_color_index = rng.read(1)[0]
    pixel_aspect_ratio = rng.read(1)[0]
    out.write(struct.pack('<HHBBB', width, height, packed,
                          background_color_index, pixel_aspect_ratio))

    # Write the global color table
    for i in range(768):
        out.write(bytes([rng.read(1)[0]]))

    # Write the application extension block
    out.write(b'\x21\xff\x0bNETSCAPE2.0\x03\x01\x00\x00\x00')

    # Write the image descriptor
    left = rng.read(1)[0] | ((rng.read(1)[0]) << 8)
    top = rng.read(1)[0] | ((rng.read(1)[0]) << 8)
    width = rng.read(1)[0] | ((rng.read(1)[0]) << 8)
    height = rng.read(1)[0] | ((rng.read(1)[0]) << 8)
    packed = rng.read(1)[0]
    local_color_table_flag = bool((packed >> 7) & 1)
    interlace_flag = bool((packed >> 6) & 1)
    sort_flag = bool((packed >> 5) & 1)
    reserved = rng.read(1)[0]
    local_color_table_size = (packed & 0x07) + 1
    out.write(struct.pack('<HHHHBB', left, top, width, height, packed,
                          local_color_table_size))

    # Write the local color table
    for i in range(local_color_table_size ** 2):
        out.write(bytes([rng.read(1)[0]]))

    # Write the image data
    lzw_min_code_sizes = [8, 4, 2, 0]
    lzw_min_code_size = rng.read(1)[0]
    out.write(bytes([lzw_min_code_size]))

    while True:
        code = rng.read(1)[0]
        if code == 0:
            break
        else:
            out.write(bytes([code]))

    # Write the trailer
    out.write(b';')
