#!/usr/bin/env python3

import sys
import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# random number generator `rng` (/dev/urandom).
# The size of the output is determined by the first 10 bytes read from `rng`.
def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Read the header and determine the length of the image data
    header = b'GIF89a'
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')

    # Write the header to the output
    out.write(header)

    # Write the rest of the data to the output
    while datalen > 0:
        chunklen = min(datalen, 256)

        # Generate some random color values for this chunk
        r = rng.read(1)[0] % 256
        g = rng.read(1)[0] % 256
        b = rng.read(1)[0] % 256

        # Write the chunk's header
        out.write(struct.pack('>B', chunklen))
        out.write(b'\x00\x00')
        out.write(struct.pack('>BBB', r, g, b))

        # Write the actual pixel data
        out.write(b'\x00'*chunklen)

        datalen -= chunklen

