#!/usr/bin/env python3

import os
import sys
import struct

def generate_random_gif(rng, out):
    """Generate a random GIF image."""

    # The first three bytes of any valid GIF image are "GIF89a".
    out.write(b'GIF89a')

    # Next comes the logical screen descriptor.  We don't care about most of
    # these values, so just use some dummy values.
    out.write(struct.pack("<H", 640))   # Screen width
    out.write(struct.pack("<H", 480))   # Screen height

    # Color table size.  We only need one color (black).
    out.write(b'\x00')

    # Background color index.
    out.write(b'\x00')

    # Pixel aspect ratio.
    out.write(b'\x00')

    # Global color table flag.
    out.write(b'\x00')

    # Image descriptor.
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00\x00\x00')

    # Image width.
    out.write(struct.pack("<H", 640))

    # Image height.
    out.write(struct.pack("<H", 480))

    # Local color table flag.
    out.write(b'\x00')

    # Interlace flag.
    out.write(b'\x00')

    # Sort flag.
    out.write(b'\x00')

    # Size of local color table.
    out.write(b'\x00')

    # Now we have to figure out how many bytes of data we want to put
    # into the file.  This is done by reading three bytes from /dev/urandom and
    # interpreting them as a little-endian unsigned integer.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)