#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(rng: BinaryIO, out: BinaryIO):
    """Generate a random GIF image."""
    # First we need to write the header.
    out.write(struct.pack('<6s', b'GIF87a'))  # Magic number
    width = rng.read(2)[::-1]                 # Width
    height = rng.read(2)[::-1]                # Height
    out.write(width + height)
    out.write(b'\x00\x00')                    # Background color index
    out.write(b'\x00')                        # Pixel aspect ratio
    # We don't care about the global color table, so just skip it.
    out.write(b'\x00')

    # Next we need to write the image descriptor.
    out.write(struct.pack('<4B', 0x2c, *[int.from_bytes(rng.read(1), byteorder='big') for _ in range(0, 3)]))
    out.write(b'\x00')                        # Local color table flag
    out.write(b'\x00')                        # Interlace flag
    out.write(b'\x00')                        # Sort flag
    out.write(b'\x00')                        # Size of local color table

    # Now we need to write the LZW minimum code size.
    out.write(rng.read(1))

    # Finally we need to write the length of the compressed data.
    datalen = int.from_bytes(rng.read(2), byteorder='big')
    out.write(datalen.to_bytes(2, byteorder='big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)