#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:
    # Write GIF header.
    out.write(b'GIF89a')

    # Get width and height of image.
    width = int.from_bytes(
                rng.read(2), byteorder='big')
    height = int.from_bytes(
                 rng.read(2), byteorder='big')
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))

    # Write color table size (always 7).
    out.write(b'\x07')

    # Write background color index.
    out.write(rng.read(1))

    # Write pixel aspect ratio (always 0).
    out.write(b'\x00')

    # Write global color table.
    for _ in range(0, 768):
        out.write(rng.read(3))

    # Write image descriptor.
    out.write(b'\x2c\x00\x00\x00\x00\x00')

    # Write local color table size (always 0).
    out.write(b'\x00')

    # Write image position.
    out.write(b'\x00\x00\x00\x00')

    # Write image size.
    out.write(width.to_bytes(2, 'big'))
    out.write(height.to_bytes(2, 'big'))

    # Write flags (always 0).
    out.write(b'\x00')

    # Write number of bytes in image data block.
    datalen = int.from_bytes(
                rng.read(3), byteorder='big')
    out.write(datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)