#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Nov 26 15:47:09 2020

@author: jonas
"""
import numpy as np
from scipy.special import gammaln, psi
from scipy.stats import norm
from scipy.optimize import minimize
from scipy.integrate import quad
from scipy.interpolate import interp1d
import matplotlib.pyplot as plt
import seaborn as sns
sns.set_style("whitegrid")

def log_normal(x, mu, sigma):
    return -np.log(sigma) - .5 * np.log(2 * np.pi) - .5 * (x - mu)**2 / sigma**2

def log_normal_pdf(x, mu, sigma):
    return np.exp(log_normal(x, mu, sigma))

def log_normal_cdf(x, mu, sigma):
    return .5 + .5 * erf((x - mu) / (sigma * sqrt(2)))

def log_normal_ppf(q, mu, sigma):
    return mu + sigma * sqrt(2) * erfinv(2 * q - 1)

def log_normal_sf(x, mu, sigma):
    return .5 * (1 + erf((x - mu) / (sigma * sqrt(2))))

def log_normal_isf(q, mu, sigma):
    return mu + sigma * sqrt(2) * erfinv(2 * q - 1)

def log_normal_moment(k, mu, sigma):
    return exp(mu + sigma ** 2 / 2 + k * sigma ** 2 / 2)

def log_normal_mean(mu, sigma):
    return exp(mu + sigma ** 2 / 2)

def log_normal_variance(mu, sigma):
    return (exp(sigma ** 2) - 1) * exp(2 * mu + sigma ** 2)

def log_normal_skewness(mu, sigma):
    return (exp(sigma ** 2) - 1) * sqrt(exp(sigma ** 2))

def log_normal_entropy(mu, sigma):
    return .5 + .5 * log(2 * pi * e * sigma ** 2)

def log_normal_median(mu, sigma):
    return exp(mu - sigma ** 2)

def log_normal_mode(mu, sigma):
    return exp(mu - sigma ** 2)

def log_normal_lower_bound(mu, sigma):
    return exp(mu - 3 * sigma ** 2)

def log_normal_upper_bound(mu, sigma):
    return exp(mu + 3 * sigma ** 2)

def log_normal_fit(data, xmin=None, xmax=None, start={'mu':0., 'sigma':1.}):
    if xmin is None or xmax is None:
        xmin = min(data)
        xmax = max(data)
    
    def nll(params):
        mu, sigma = params
        
        ll = 0
        for d in data:
            if d < xmin or d > xmax: continue
            
            ll += log_normal(d, mu, sigma)
        return -ll
    
    res = minimize(nll, start, method='L-BFGS-B')
    return res['x'][0], res['x'][1]

def log_normal_fit_mle(data, xmin=None, xmax=None, start={'mu':0., 'sigma':1.}):
    if xmin is None or xmax is None:
        xmin = min(data)
        xmax = max(data)
    
    def nll(params):
        mu, sigma = params
        
        ll = 0
        for d in data:
            if d < xmin or d > xmax: continue
            
            ll += log_normal(d, mu, sigma)
        return -ll
    
    res = minimize