#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main() -> None:
    if len(sys.argv) != 2:
        print('Usage: {} <file>'.format(os.path.basename(__file__)), file=sys.stderr)
        exit(1)

    with open(sys.argv[1], 'rb') as f:
        data = f.read()

    # Parse the header
    magic, version, num_entries = struct.unpack('<4sLL', data[:16])
    assert magic == b'MTHD' and version == 0x00010000 and num_entries > 0

    # Parse the entries
    offset = 16
    for i in range(num_entries):
        entry_type, entry_size = struct.unpack('<LL', data[offset:offset+8])
        offset += 8

        if entry_type == 0x00000001:
            # MIDI channel message
            status, channel, param1, param2 = struct.unpack('<BBBH', data[offset:offset+6])
            offset += 6
            print('Channel {}: {:02X} {:02X} {:02X}'.format(channel, status, param1, param2))
        elif entry_type == 0x00000002:
            # MIDI system common message
            status, param1, param2 = struct.unpack('<BBH', data[offset:offset+5])
            offset += 5
            print('System common: {:02X} {:02X} {:02X}'.format(status, param1, param2))
        elif entry_type == 0x00000003:
            # MIDI system real-time message
            status = data[offset]
            offset += 1
            print('System real-time: {:02X}'.format(status))
        else:
            raise Exception('Unknown entry type: {}'.format(entry_type))
