#!/usr/bin/env python3

import os
import sys
from random import randint

def generate_random_gif(rng, out):
    # First, let's write the GIF header.
    out.write(b'GIF89a')

    # Next, we need a screen descriptor.  This is 7 bytes long and contains
    # the logical screen width, height, color resolution, background color
    # index, aspect ratio, and sort of flags.
    out.write(struct.pack('<HHBBB', 256, 256, 8, 0, 0))

    # The next byte is the "global color table" flag.  If this is set, then
    # there will be a global color table immediately following the screen
    # descriptor.  We don't have one, so we'll just set it to zero.
    out.write(b'\x00')

    # Now we need to write the image descriptor.  This is also 7 bytes long.
    out.write(b'\x2c\x00\x00\x00\x00\x00\x00')

    # The next two bytes are the image left and top position.  Since this is
    # a full-screen image, we'll put it at (0, 0).
    out.write(b'\x00\x00')

    # The next two bytes are the image width and height.  For now, we'll make
    # it the same size as the logical screen.
    out.write(struct.pack('<HH', 256, 256))

    # The next byte is the "local color table" flag and the number of colors
    # in the local color table minus 1.  Since we don't have a local color
    # table, we'll set both of these to zero.
    out.write(b'\x00')

    # The next byte is the code size.  For now, we'll use 8 bits per pixel.
    out.write(b'\x08')

    # Finally, we need to calculate how many bytes of data we're going to
    # write.  This is equal to the product of the image width and height by
    # the number of bits per pixel divided by eight.
    datalen = 256 * 256 // 8

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)