#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def generate_random_gif(rng: BinaryIO, out: BinaryIO) -> None:
    """Generate a random GIF image."""

    # First we need to write the GIF header.
    out.write(b'GIF89a')  # Magic number
    out.write(struct.pack('<H', 256))  # Width
    out.write(struct.pack('<H', 256))  # Height
    out.write(b'\xF7\x04\x04\x00\x00\xFF\xFF\xFF')  # Background color
    out.write(b'\x00')  # Aspect ratio

    # Then we need to write the graphic control extension.
    out.write(b'\x21\xF9\x04\x01\x00\x00\x00')  # Graphic control extension
    out.write(b'\x00')  # Block terminator

    # Next we need to write the image descriptor.
    out.write(b'\x2C\x00\x00\x00\x00\x00\x00\x00\x00')  # Image descriptor

    # We now need to write the local color table.
    for _ in range(0, 256):
        out.write(rng.read(3))

    # Finally we need to write the image data.
    out.write(b'\x00')  # LZW minimum code size

    # The first byte of the image data is the length of the following data.
    datalen = int.from_bytes(rng.read(1), 'big')
    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))

    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)