#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main(argv: list[str]) -> int:
    if len(argv) != 2:
        print("Usage: {} <file>".format(os.path.basename(__file__)), file=sys.stderr)
        return 1
    
    with open(argv[1], "rb") as f:
        data = f.read()
        
    # Read the header
    magic, version, num_entries = struct.unpack("<4sLL", data[:12])
    assert magic == b"NARC"
    assert version in (0x00010000, 0x00020000)
    
    # Skip past the header and read each entry
    offset = 12
    for i in range(num_entries):
        name_len, unk1, unk2, size = struct.unpack("<LHHLL", data[offset:offset+16])
        name = data[offset + 16 : offset + 16 + name_len].decode().rstrip("\x00")
        offset += 16 + name_len
        
        # Read the actual data
        data = data[offset:offset+size]
        offset += size
        
        # Write it to a file
        filename = "{}_{}.dat".format(i, name)
        with open(filename, "wb") as f:
            f.write(data)
            
        print("Wrote {} bytes to {}".format(len(data), filename))
