#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO, List, Tuple

def read_file(filename: str) -> bytes:
    with open(filename, 'rb') as f:
        return f.read()

def write_file(filename: str, data: bytes):
    with open(filename, 'wb') as f:
        f.write(data)

def read_int16(f: BinaryIO) -> int:
    return struct.unpack('<h', f.read(2))[0]

def read_uint8(f: BinaryIO) -> int:
    return struct.unpack('B', f.read(1))[0]

def read_uint16(f: BinaryIO) -> int:
    return struct.unpack('H', f.read(2))[0]

def read_uint32(f: BinaryIO) -> int:
    return struct.unpack('I', f.read(4))[0]

def read_string(f: BinaryIO) -> str:
    length = read_uint8(f)
    return f.read(length).decode('ascii').rstrip('\x00')
