#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # variable-sized data. So we have to write the header first, then seek back
    # to the beginning of the file so we can write the data.
    out.write(b'GIF89a')  # Magic number
    out.write(struct.pack('<H', 0x21f9))  # Logical screen width (640)
    out.write(struct.pack('<H', 0x12c))   # Logical screen height (480)
    out.write(b'\x00\x00')                # Background color index
    out.write(b'\x00')                    # Pixel aspect ratio
    out.seek(0)

    # Write the image descriptor block
    out.write(b'\x2c\x00\x00\x00\x00\x00')
    out.write(b'\xff\xff\xff')            # Local color table size
    out.write(b'\x00')                     # Image left position
    out.write(b'\x00')                     # Image top position
    out.write(b'\x00\xff')                 # Image width
    out.write(b'\x00\xff')                 # Image height
    out.write(b'\x87')                     # LCT flag + interlace flag

    # Write the local color table
    for i in range(256):
        out.write(struct.pack('BBB', *[i] * 3))

    # Write the image data
    while True:
        byte = rng.read(1)
        if not byte:
            break
        out.write(byte)

