#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.

    # Write the magic number.
    out.write(b"GIF89a")

    # Write the screen descriptor.
    width = rng.read(1)[0] % 256
    height = rng.read(1)[0] % 256
    background = rng.read(1)[0] % 256
    aspect = rng.read(1)[0] % 256
    out.write(struct.pack("<HHBB", width, height, background, aspect))

    # Write the global color table.
    for _ in range(7):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack("BBB", red, green, blue))

    # Write the image descriptor.
    left = rng.read(1)[0] % 256
    top = rng.read(1)[0] % 256
    width = rng.read(1)[0] % 256
    height = rng.read(1)[0] % 256
    local_color_table = rng.read(1)[0] % 256
    interlaced = rng.read(1)[0] % 256
    out.write(struct.pack("<HHHBB", left, top, width, height, local_color_table, interlaced))

    # Write the local color table.
    for _ in range(local_color_table):
        red = rng.read(1)[0] % 256
        green = rng.read(1)[0] % 256
        blue = rng.read(1)[0] % 256
        out.write(struct.pack("BBB", red, green, blue))

    # Write the LZW code stream.
    while True:
        code_size = rng.read(1)[0] % 256
        if code_size == 0xFF:
            break
        else:
            out.write(code_size.to_bytes(1, "little"))

            # Write the rest of the code stream.
            while True:
                code = rng.read(1)[0] % 256
                if code == 0xFF:
                    break
                elif code == 0xFE:
                    pass
                else:
                    out.write(code.to_bytes(1, "little"))

