#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
) -> None:
    # GIF header
    out.write(struct.pack("!6s", b"GIF89a"))
    width = 2 ** (rng.read(1)[0] % 5 + 3)
    height = 2 ** (rng.read(1)[0] % 4 + 2)
    out.write(struct.pack("<HHB", width, height, 0xF7))

    # Global color table
    out.write(struct.pack("BB", 0xFF, 0xFF))
    for i in range(0, 7):
        out.write(rng.read(1))

    # Image descriptor
    out.write(struct.pack("!B", 0x2C))
    out.write(struct.pack("<HHB", 0, 0, 0))
    out.write(struct.pack("<HH", width, height))
    out.write(struct.pack("B", 0))

    # Local color table
    out.write(struct.pack("BB", 0xFF, 0xFF))
    for i in range(0, 7):
        out.write(rng.read(1))

    # Data sub-blocks
    datalen = rng.read(1)[0] % 255 + 1
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)