#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # a variable number of frames. We'll just use the same frame over and over
    # again to keep things simple.

    # Write the GIF header.
    out.write(b'GIF89a')  # Magic bytes.
    out.write(struct.pack('<HHB', 1024, 768, 0))  # Screen size (width x height).
    out.write(b'\x00\x00\xFF\xFF\xFF\xFF')  # Background color.
    out.write(b'\x00')  # Pixel aspect ratio.

    # Generate a random frame.
    for y in range(0, 768):
        for x in range(0, 1024):
            out.write(struct.pack('BBB', *[int(v*255) for v in rng.read(3)]))

    # Write the GIF trailer.
    out.write(b';')

if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)