#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main():
    if len(sys.argv) < 2:
        print("Usage: {} <file>".format(os.path.basename(__file__)))
        return 1
    
    with open(sys.argv[1], "rb") as f:
        data = f.read()
        
    # Check for the magic number
    if not data[:4] == b"RIFF":
        raise Exception("Not a WAV file!")
    
    # Skip over the rest of the RIFF header
    data = data[8:]
    
    # Check for the format chunk
    if not data[:4] == b"fmt ":
        raise Exception("Invalid WAV file (no fmt chunk)!")
    
    # Read the size of the format chunk
    size, = struct.unpack("<L", data[4:8])
    
    # Skip over the format chunk
    data = data[size + 8:]
    
    # Check for the data chunk
    if not data[:4] == b"data":
        raise Exception("Invalid WAV file (no data chunk)!")
    
    # Read the size of the data chunk
    size, = struct.unpack("<L", data[4:8])
    
    # Skip over the data chunk
    data = data[size + 8:]
    
    # Write the output to stdout
    sys.stdout.buffer.write(data)
    