#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'

    # Next comes the length of the data section.  It's stored as a 3-byte big-endian
    # integer, so we need to read three bytes from /dev/urandom and interpret those
    # into the file.  This is done by reading three bytes from /dev/urandom and
    # interpreting them as a little-endian unsigned integer.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we write the actual data section.  We're going to use the same trick as
    # before to get the length of this section.  However, since we know the size
    # of the header already, we can subtract that from the total length of the
    # file to get the length of the data section.
    datalen -= len(header) + 3

    # Now we can write the data section.  For each byte, we'll read one byte from
    # /dev/urandom and write it to the output file.
    for i in range(datalen):
        out.write(rng.read(1))

