#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(out: BinaryIO, size: int) -> None:
    """Generate a random GIF file.

    Args:
        out (BinaryIO): Output stream.
        size (int): Size of the output in bytes.
    """
    # The first 6 bytes are the magic number "GIF87a".
    out.write("GIF87a".encode())

    # The next two bytes are the width and height of the image.
    out.write(struct.pack("<H", size))
    out.write(struct.pack("<H", size))

    # The next byte is the number of colors in the palette.
    out.write(b"\x01")

    # The next three bytes are the background color index.
    out.write(b"\xff\xff\xff")

    # The next byte is the pixel aspect ratio.
    out.write(b"\x00")

    # The next block is the global color table.
    out.write(b"\x00" * 256 * 3)

    # The next block is the graphic control extension.
    out.write(b"\x21\xf9\x04\x01\x00\x00\x00\x00")

    # The next block is the image descriptor.
    out.write(b"\x2c\x00\x00\x00\x00\x00\x00\x00\x00")

    # The next block is the local color table.
    out.write(b"\x00" * 256 * 3)

    # The last block is the image data.
    out.write(os.urandom(size - 102))

