#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'

    # Next comes the length of the data section.  It's stored as a 3-byte big-endian
    # integer.
    out.write(struct.pack('>I', len(header)))

    # Now we can write the rest of the file.
    out.write(header)

    # Now we need to generate some random data for the image itself.  We'll use
    # a fixed seed so this is deterministic.
    rng.seek(0)
    rng.readinto(out)

