#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # come back to it later.
    out.write(b"GIF89a")
    out.write(struct.pack("<H", 0))
    out.write(struct.pack("BBB", 7, 0, 0))

    # Write the global color table.
    for _ in range(256):
        out.write(struct.pack("BBB", *rng.read(3)))

    # Write the image descriptor.
    out.write(struct.pack("!HHBB", 0, 0, 1, 0))

    # Write the local color table.
    for _ in range(256):
        out.write(struct.pack("BBB", *rng.read(3)))

    # Write the LZW code stream.
    while True:
        # Read a byte from the RNG.
        b = rng.read(1)
        if not b:
            break
        b = ord(b)

        # Write the byte as a single-byte clear code.
        out.write(struct.pack("B", b & 0xFF))

