#!/usr/bin/env python3

import os
import sys

# The GIF header consists of the following bytes:
#   - "GIF" (ASCII characters).
#   - A version number (usually 87h or 89h).
#   - Logical screen width (two bytes, little endian).
#   - Logical screen height (two bytes, little endian).
#   - Packed field:
#     - Bit 0: Global color table flag (set if there's a global color table).
#     - Bits 1-4: Color resolution (number of bits per primary color).
#     - Bit 5: Sort flag (not used).
#     - Bits 6-7: Size of global color table (2**n colors).
#   - Background color index (one byte).
#   - Pixel aspect ratio (one byte).
header = b'GIF89a\x01\x01\x01\x00\x00\xff\x00\x00'

def generate_random_gif(rng, out):
    """Generate a random GIF image."""

    # We first need to determine how many bytes of data we will be writing
    # into the file.  This is done by reading three bytes from /dev/urandom and
    # interpreting them as a little-endian unsigned integer.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)