#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    out.write(b"GIF89a")

    # Write the width and height of the image.
    out.write(struct.pack("!H", 256))
    out.write(struct.pack("!H", 256))

    # Write the background color index (black).
    out.write(b"\x00")

    # Write the aspect ratio (unused).
    out.write(b"\x00")

    # Write the global color table flag (no global color table).
    out.write(b"\x00")

    # Write the number of colors in the global color table (zero).
    out.write(b"\x00")

    # Write the color resolution (7 bits per pixel).
    out.write(b"\x07")

    # Write the sort flag (not sorted).
    out.write(b"\x00")

    # Write the size of the global color table (zero).
    out.write(b"\x00")

    # Write the background color index (black).
    out.write(b"\x00")

    # Write the pixel aspect ratio (unused).
    out.write(b"\x00")

    # Write the global color table (empty).
    out.write(b"\x00\x00\x00")

    # Write the application extension block.
    out.write(b"\x21\xf9\x04\xff\xff\xff\x00")

    # Write the Netscape loop extension block.
    out.write(b"\x21\xff\x0bNETSCAPE2.0\x03\x01")

    # Write the data subblock length.
    out.write(b"\x00")

    # Write the data subblock terminator.
    out.write(b";")

    # Write the graphic control extension block.
    out.write(b"\x21\xf9\x04\x01\x00\x00\x00")

    # Write the image descriptor block.
    out.write(b"\x2c\x00\x00\x00\x00\x00\x00\x00\x00")

    # Write the local color table flag (no local color table).
    out.write(b"\x00")

    # Write the number of colors in the local color table (zero).
    out.write(b"\x00")

    # Write the color resolution (7 bits per pixel).
    out.write(b"\x07")

    # Write the sort flag (not sorted).
    out.write(b"\x00")

    # Write the size of the local color table (zero).
    out.write(b"\x00")

    # Write the LZW minimum code size (8 bits per code).
    out.write(b"\x08")

    # Read the length of the data section from the RNG.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)