#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO, width: int = 1024, height: int = 768) -> None:
    # GIF header
    out.write(b'GIF89a')
    # Logical screen descriptor
    out.write(struct.pack('<HHBB', width, height, 0xF0 | (width >> 7), 0))
    # Global color table (16 colors)
    for i in range(1 << 4):
        out.write(bytes([i] * 3))
    # Image block
    out.write(b'\x2C\x00\x00\x00\x00')
    # Local color table (16 colors)
    for i in range(1 << 4):
        out.write(bytes([i] * 3))
    # Image descriptor
    out.write(struct.pack('<HHB', 0, 0, 0x80 | (height >> 7)))
    # LZW minimum code size
    out.write(b'\x08')
    # Pixel data
    while True:
        b = rng.read(1)[0]
        if b == 0:
            break
        out.write(bytes([b]))

