#!/usr/bin/env python3

import os
import sys
import struct

def generate_random_gif(rng, out):
    """Generate a GIF image with random contents."""

    # First, we'll write the GIF header.
    out.write(b'GIF89a')

    # Next, we need to figure out how many bytes of random data we should write
    # into the file.  This is done by reading three bytes from /dev/urandom and
    # interpreting them as a little-endian unsigned integer.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)