#!/usr/bin/env python3

import os
import sys
import struct
from random import Random

def generate_random_gif(rng, out):
    # GIF header
    out.write(b'GIF89a')
    
    # Screen descriptor
    width = rng.randint(10, 256)
    height = rng.randint(10, 256)
    out.write(struct.pack('<HHB', width, height, 0xF))
    
    # Global color table
    for i in range(0x100):
        out.write(struct.pack('BBB', rng.randint(0, 255), rng.randint(0, 255), rng.randint(0, 255)))
        
    # Graphic control extension
    out.write(b'\x21\xf9\x04\x00\xff\x00\x00')
    
    # Image descriptor
    out.write(struct.pack('<HHHBB', 0, 0, width, height, 0))
    
    # Local color table
    for i in range(0x100):
        out.write(struct.pack('BBB', rng.randint(0, 255), rng.randint(0, 255), rng.randint(0, 255)))
        
    # LZW minimum code size
    out.write(b'\x00')
    
    # Pixel data
    for y in range(height):
        for x in range(width):
            out.write(struct.pack('B', rng.randint(0, 0xFF)))
            
if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1] == '-f':
        with open('test.gif', mode='wb') as f:
            generate_random_gif(Random(), f)
    else:
        seed = int(os.urandom(4).hex(), 16)
        print("Seed: %d" % seed)
        rng = Random(seed)
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)