#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Oct 20 15:47:09 2020

@author: jonasg
"""
import numpy as np
from scipy.stats import norm, multivariate_normal
import matplotlib.pyplot as plt

def generate_data(n=100):
    """Generates a dataset with two classes of points."""
    
    # Generate class 1
    x = np.random.multivariate_normal([0, 0], [[1, 0], [0, 1]], size=int(n/2))
    y = np.ones((len(x), 1))
    
    # Generate class 2
    x = np.vstack((x, np.random.multivariate_normal([2, 2], [[1, 0], [0, 1]], size=int(n/2))))
    y = np.vstack((y, np.zeros((len(x)-len(y), 1))))
    
    return x, y

def plot_dataset(X, Y):
    """Plots the data set X and labels Y."""
    
    plt.figure()
    plt.scatter(X[:, 0], X[:, 1], c=Y)
    plt.show()

def gaussian_naive_bayes(X, Y, prior=None):
    """Compute Gaussian Naive Bayes parameters for each class in X and Y.
    
    Parameters
    ----------
    X : array-like, shape (n_samples, n_features)
        Training vector, where `n_samples` is the number of samples and
        `n_features` is the number of features.
        
    Y : array-like, shape (n_samples, )
        Target values.
        
    prior : float or None, default=None
        If not None, use this value as the prior probability for the positive
        class. Otherwise, compute it from the data.
        
    Returns
    -------
    mu : array-like, shape (n_classes, n_features)
        Mean vectors for each class in the model.
        
    sigma : array-like, shape (n_classes, n_features)
        Covariance matrices for each class in the model.
        
    p : array-like, shape (n_classes, )
        Class priors.
    """
    
    if len(np.unique(Y)) != 2:
        raise ValueError('Gaussian Naive Bayes requires binary classification.')
    
    # Compute mean and covariance for each class
    mu = []
    sigma = []
    for i in range(2):
        x = X[Y == i]
        mu.append(np.mean(x, axis=0))
        sigma.append(np.cov(x.T))
    mu = np.array(mu).reshape(-1, 2)
    sigma = np.array(sigma).reshape(-1, 2, 2)
    
    # Compute class prior
    if prior is None:
        p = np.sum(Y)/len(Y)
    else:
        p = prior
    
    return mu, sigma, p

def predict_gaussian_naive_bayes(X, mu, sigma, p):
    """Predict using Gaussian Naive Bayes.
    
    Parameters
    ----------
    X : array-like, shape (n_samples, n_features)
        Testing vector, where `n_samples` is the number of samples and
        `n_features` is the number of features.
        
    mu : array-like, shape (n_classes, n_features)
        Mean vectors for each class in the model.
        
    sigma : array-like, shape (n_classes, n_features)
        Covariance matrices for each class in the model.
        
    p : array-like, shape (n_classes, )
        Class priors.
        
    Returns
    -------
    y : array-like, shape (n_samples, )
        Predicted target values.
    """
    
    # Compute likelihoods
    lh = np.empty((len(X), 2))
   