#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def main(argv):
    if len(argv) != 2:
        print("Usage: {} <file>".format(os.path.basename(__file__)))
        return 1
    
    with open(argv[1], "rb") as f:
        data = f.read()
        
    # Skip the header
    offset = 0x800
    while True:
        magic, size = struct.unpack_from("<4sI", data, offset)
        if magic == b"DATA":
            break
        offset += size + 8
    offset += 8
    
    # Read the file table
    files = []
    while True:
        name, size = struct.unpack_from("<64sQ", data, offset)
        if not name:
            break
        files.append((name, size))
        offset += 72
    
    # Extract all files
    for name, size in files:
        path = name.decode().rstrip("\x00").replace("/", os.sep)
        print("Extracting {}...".format(path))
        with open(path, "wb") as f:
            f.write(data[offset : offset+size])
        offset += size
    
    return 0
