#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # variable-sized data. We'll just write the header first, then write some
    # random data to fill up the rest of the file.

    # First we need to figure out how big our image will be.
    width = 1 + (rng.read(1)[0] % 640)
    height = 1 + (rng.read(1)[0] % 480)

    # Now we can write the header.
    out.write(struct.pack('<6sHHB', b'GIF87a', width, height, 0xF))

    # Then we write the color table.
    for _ in range(256):
        out.write(struct.pack('BBB', *[rng.read(1)[0] for _ in range(3)]))

    # Finally we write the image data.
    out.write(rng.read())


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)