#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # the actual image data.  This means we need to write our own header first.
    # We'll just use a random width and height for now.
    width = int.from_bytes(rng.read(2), byteorder='little') % (1 << 8) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (1 << 8) + 1

    # Write the header
    out.write(b'GIF89a')
    out.write(struct.pack('<HH', width, height))
    out.write(b'\x87\x00\x00')

    # Now write the palette
    out.write(b'\xff\xff\xff\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')

    # Finally, write the pixel data
    for _ in range(height):
        for _ in range(width):
            out.write(b'\x00')


if