#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'
    out.write(header)

    # Now we need to find the size of the data section by writing random bytes
    # until we get back to where the header started.
    while True:

        # Generate some random bytes.
        data = rng.read(1024 * 1024)

        # Write them to the output file.
        out.write(data)

        # If we've returned to the start of the header, stop.
        if out.tell() == len(header):
            break

    # Finally, pad the rest of the file with zeroes.
    padding = (out.tell() + 15) & ~15 - out.tell()
    out.write(bytes([0] * padding))


if