#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Sep  9 23:45:46 2020

@author: <NAME>
"""
import struct


def generate_random_gif(rng, out):
    """Generate a random GIF image."""
    header = b'GIF89a'

    # The first three bytes of the file are always fixed.
    out.write(header)

    # Next comes a single field that tells us how many color table entries there
    # will be (always 256).
    out.write(b'\x00\x00\x00')

    # Then follows the global color table.  Each entry consists of three bytes
    # representing the red, green, and blue components respectively.
    for i in range(0, 256):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]

        out.write(struct.pack('<BBB', r, g, b))

    # Finally comes an image descriptor which specifies the size of the image
    # and its position relative to the screen.  It also has a flag indicating if
    # it's interlaced or not.
    width = rng.read(1)[0] % 16 + 1
    height = rng.read(1)[0] % 16 + 1
    flags = rng.read(1)[0] & 0b00001111 | 0b01000000
    xpos = rng.read(1)[0] % 256
    ypos = rng.read(1)[0] % 256

    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags.to_bytes(1, 'little'))
    out.write(xpos.to_bytes(1, 'little'))
    out.write(ypos.to_bytes(1, 'little'))

    # After the image descriptor comes the LZW minimum code size.  We'll just use
    # one byte here.
    out.write(b'\x01')

    # Next up is the actual image data.  First comes the length of the data,
    # encoded as a big endian unsigned short.  Note that this doesn't include the
    # initial two bytes used to encode the length itself!  Also note that since
    # we're generating random data, the length might actually be zero.  In that
    # case, we don't want to write anything at all because that would make the
    # GIF invalid.
    datalen = rng.read(1)[0] << 8 | rng.read(1)[0]
    if datalen > 0:
        out.write(datalen.to_bytes(2, 'big'))

        # And finally the actual data.
        for _ in range(0, datalen):
            out.write(rng.read(1))

    # Now comes another image descriptor specifying the size of the next frame.
    width = rng.read(1)[0] % 16 + 1
    height = rng.read(1)[0] % 16 + 1
    flags = rng.read(1)[0] & 0b00001111 | 0b01000000
    xpos = rng.read(1)[0] % 256
    ypos = rng.read(1)[0] % 256

    out.write(width.to_bytes(1, 'little'))
    out.write(height.to_bytes(1, 'little'))
    out.write(flags.to_bytes(1, 'little'))
    out.write(xpos.to_bytes(1, 'little'))
    out.write(ypos.to_bytes(1, 'little'))

    # Again, the LZW minimum code size.
    out.write(b'\x01')

    # And now comes the length of the data.  Since this time we know what the
    # length should be, we can simply write it directly    # into the file.  This is done by reading three bytes from /dev/urandom and
    # interpreting them as a little-endian unsigned integer.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)