#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded number of blocks.  This means we have to write the header
    # first before writing any other data.  We'll just use some random values
    # for now.

    # Header
    out.write(b'GIF89a')
    out.write(struct.pack('<HH', *rng.read(4)))    # Logical screen descriptor
    out.write(struct.pack('B', *rng.read(1)))      # Background color index
    out.write(struct.pack('B', *rng.read(1)))      # Pixel aspect ratio
    out.write(struct.pack('BBB', *rng.read(3)))    # Global color table
    out.write(struct.pack('BBBBBB', *rng.read(6)))  # Application extension

    # Graphics control block (for animation)
    out.write(b'\x21\xF9\x04')   # Extension introducer + graphics control label
    out.write(b'\x08')           # Block size
    out.write(struct.pack('<H', *rng.read(2)))     # Delay time
    out.write(b'\x00')           # Transparent color index
    out.write(b'\x00')           # Terminator

    # Image descriptor
    out.write(b'\x2C')       # Image separator
    out.write(struct.pack('<HHHHHH', *rng.read(10)))        # Left, top, width, height
    out.write(struct.pack('B', *rng.read(1)))              # Local color table flag
    out.write(struct.pack('B', *rng.read(1)))              # Interlace flag
    out.write(struct.pack('B', *rng.read(1)))              # Sort flag
    out.write(struct.pack('B', *rng.read(1)))              # Reserved
    out.write(struct.pack('BBB', *rng.read(3)))            # LZW minimum code size

    # Plain text extension
    out.write(b'\x21\x01\x0F')   # Extension introducer + plain text label
    out.write(b'\x00')           # Block size
    out.write(b'\x00')           # Text grid left position
    out.write(b'\x00')           # Text grid top position
    out.write(b'\x00')           # Text grid width
    out.write(b'\x00')           # Text grid height
    out.write(b'\x00')           # Cell width
    out.write(b'\x00')           # Cell height
    out.write(b'\x00')           # Foreground palette entry
    out.write(b'\x00')           # Background palette entry
    out.write(b'\x00')           # Text data length byte 1
    out.write(b'\x00')           # Text data length byte 2
    out.write(b'\x00')           # Text data length byte 3
    out.write(b'\x00')           # Text data length byte 4
    out.write(b'\x00')           # Character cell value

    # Graphic control extension
    out.write(b'\x21\xF9\x04')   # Extension introducer + graphics control label
    out.write(b'\x08')           # Block size
    out.write(struct.pack('<H', *rng.read(2)))     # Delay time
    out.write(b'\x00')           # Transparent color index
    out.write(b'\x00')           # Terminator

    # Image descriptor
    out.write(b'\x2C')       # Image separator
    out.write(struct.pack('<HHHHHH', *rng.read(10)))        # Left, top, width, height
    out.write(struct.pack('B', *rng.read(1)))              # Local color table flag
    out.write(struct.pack('B', *rng.read(1)))              # Interlace flag
    out.write(struct.pack('B', *rng.read(1)))              # Sort flag
    out.write(struct.pack('B', *rng.read(1)))              # Reserved
    out.write(struct.pack('BBB', *rng.read(3)))            # LZW minimum code size

    # Plain text extension
    out.write(b'\x21\x01\x0F')   # Extension introducer + plain text label
    out.write(b'\x00')           # Block size
   