#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'
    out.write(header)
    
    # Now we need to figure out how many bytes of data there are.  This is
    # done by writing a random number of zeroes until we get back to the start
    # of the header again (i.e., when we see the magic number).
    while True:

        byte = rng.read(1)[0]
        
        if byte == ord('\x47'):
            break

        assert byte == 0
        
    # Read one more byte for good measure.
    _ = rng.read(1)

    # Now we know how big our data section should be.  Write it all at once!
    data = rng.read((out.tell() - len(header)) // 2)
    out.write(data)
    

if