#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Sep 25 19:47:26 2018

@author: <NAME> <<EMAIL>>
"""
import struct
from PIL import Image


def generate_random_gif(rng, out):
    """Generate a random GIF image."""

    # We need at least one frame and palette entry.
    num_frames = max(1, int((rng.getrandbits(16) / (2**16 - 1))))
    num_colors = max(1, int((rng.getrandbits(8) / (2**8 - 1))))
    
    print("Generating {} frames with {} colors".format(num_frames, num_colors))

    # Write header
    out.write(b'GIF89a')
    # Logical screen descriptor
    out.write(struct.pack('<HHBBB', 100, 100, 7, 0, 0))
    # Global color table
    out.write(bytes([num_colors * 3]))
    for i in range(0, num_colors*3, 3):
        out.write(bytes([(rng.getrandbits(8) % 256),
                         (rng.getrandbits(8) % 256),
                         (rng.getrandbits(8) % 256)]))
    # Loop control block
    out.write(b'\x01\x00\x00\x00\xff\x00\x00\x00')
    # Frames
    for i in range(0, num_frames):
        # Image descriptor
        out.write(struct.pack('<BHHHHBB',
                              ord(','),
                              0,
                              0,
                              rng.getrandbits(16) % 100 + 100,
                              rng.getrandbits(16) % 100 + 100,
                              0,
                              0,
                              0xf9))
        # LZW minimum code size
        out.write(b'\x02')

        img = Image.new('RGB', (rng.getrandbits(16) % 100 + 100,
                                rng.getrandbits(16) % 100 + 100))
        imgdata = list()
        for y in range(img.size[1]):
            row = []
            for x in range(img.size[0]):
                row += [rng.getrandbits(8) % len(img.getpalette())]
            imgdata += [row]
        
        img.putdata(imgdata)
        img.save(out, format="gif")

    # Trailer
    out.write(b';')


def generate_random_file(rng, out, datalen=None):
    if not datalen:
        datalen = rng.getrandbits(32)
    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)