#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO,
        out: BinaryIO):
    """Generate a random GIF image."""

    header = b'GIF89a'  # Version and size of the file.
    width = rng.read(2)  # Width of the image (2 bytes).
    height = rng.read(2)  # Height of the image (2 bytes).
    color_table_size = rng.read(1)[0] & 7  # Color table size (1 byte).
    background_color = rng.read(1)[0]  # Background color index (1 byte).
    aspect_ratio = rng.read(1)[0]  # Aspect ratio (1 byte).
    flags = rng.read(1)[0]  # Flags (1 byte).

    # Write the header to the output file.
    out.write(header + width + height +
              color_table_size.to_bytes(1, 'big') +
              background_color.to_bytes(1, 'big') +
              aspect_ratio.to_bytes(1, 'big') +
              flags.to_bytes(1, 'big'))

    # Generate the palette.
    palette = []
    for i in range(0, pow(2, color_table_size)):
        r = rng.read(1)[0]
        g = rng.read(1)[0]
        b = rng.read(1)[0]

        palette += [r, g, b]

    # Write the palette to the output file.
    for p in palette:
        out.write(p.to_bytes(1, 'big'))

    # Get the length of the image data from the RNG.
    if flags & 0x80:
        datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)