#!/usr/bin/env python3

import os
import sys
import struct


def main():
    if len(sys.argv)!= 2:
        print("Usage: %s <file>" % (os.path.basename(__file__)))
        return

    with open(sys.argv[1], "rb") as f:
        data = bytearray(f.read())

        # Check for the magic number
        if not data[:4] == b"BMP\x00":
            raise Exception("Invalid BMP file.")

        # Read the header
        offset, dib_size, width, height, planes, depth, compression, size, \
                hres, vres, ncolors, impcolors = struct.unpack("<LLLHHHIIIIIII", data[10:])

        # Skip to the bitmap data
        data = data[offset:]

        # Convert the image into a list of pixels
        pixels = []
        for i in range(height):
            row = [data[(i * width + j) * depth:(i * width + j) * depth + depth]
                   for j in range(width)]

            # If this is an indexed color image, convert it to RGB
            if depth <= 8 and ncolors > 0:
                palette = data[dib_size + 4 : dib_size + 4 + 256*4]

                def rgb(color):
                    r, g, b = struct.unpack("<BBB", palette[color*4:color*4+3])

                    return (r / 255., g / 255., b / 255.)

                row = [[rgb(pixel[0])] for pixel in row]

            pixels += row

        # Print the pixels
        for y in reversed(range(len(pixels))):
            line = ""

            for x in range(len(pixels[y])):
                line += "%.2f %.2f %.2f " % tuple(pixels[y][x][0])

            print(line)


if