#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO

def generate_random_gif(rng: BinaryIO, outf: BinaryIO):
    # Header
    header = b'GIF89a' + bytes([0x21]) + b'\xf9\x04'

    # Logical Screen Descriptor
    width  = struct.pack('<H', int(os.environ['WIDTH']))
    height = struct.pack('<H', int(os.environ['HEIGHT']))
    depth  = struct.pack('<B',  8)
    color  = struct.pack('<B',  7)
    bg     = struct.pack('<B',  0)
    flags  = struct.pack('<B',  0)
    lsd    = width + height + depth + color + bg + flags

    # Global Color Table (palette)
    palette = [struct.pack('<BBB', *rng.read(3)) for _ in range(int(os.environ['PALETTE']))]
    gct = b''.join(palette)

    # Image Descriptor
    left   = struct.pack('<H', 0)
    top    = struct.pack('<H', 0)
    width  = struct.pack('<H', int(os.environ['WIDTH']))
    height = struct.pack('<H', int(os.environ['HEIGHT']))
    flags  = struct.pack('<B', 0b10000000 | 0b00000001)
    idd    = left + top + width + height + flags

    # Image Data
    data = bytearray()
    while len(data) < int(os.environ['SIZE']):
        data += rng.read(1)

    # Trailer
    trailer = b''

    outf.write(header + lsd + gct + idd + data + trailer)

if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)