#!/usr/bin/env python3

import os
import sys
import struct
from PIL import Image


def generate_random_gif(rng, out):

    # First 6 bytes are GIF signature and version number.
    out.write(b'GIF89a')

    # Next comes a screen descriptor block (7 bytes).
    width = rng.randint(256, 4096)
    height = rng.randint(256, 4096)
    flags = b'\x80\x00' if rng.choice([True, False]) else b'\x00\x00'
    bgcol = rng.randint(0, 255)
    aspect = rng.randint(0, 255)
    out.write(struct.pack('<HHBBB', width, height, flags[0], flags[1],
                          bgcol, aspect))

    # Then follows an optional global color table (3 bytes + 256*3 bytes), but
    # we don't care about that here. Just skip it.
    out.write(b'\x00' * 3)
    out.write(b'\xFF' * 768)

    # Finally there's the image descriptor block (10 bytes).
    left = rng.randint(0, width - 1)
    top = rng.randint(0, height - 1)
    width = rng.randint(1, min(width-left, 256))
    height = rng.randint(1, min(height-top, 256))
    flags = b'\x80\x00' if rng.choice([True, False]) else b'\x00\x00'
    lctsize = 0
    interlace = 0
    sort = 0
    reserved = b'\x00\x00\x00'
    out.write(struct.pack('<HHHHBBBBB', left, top, width, height, flags[0],
                          flags[1], lctsize, interlace, sort) + reserved)

    # The next byte is the local color table size, which should be zero.
    assert ord(out.read(1)) == 0

    # We now need to create a palette of random colors.
    pal = []
    for i in range(0, 256):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        pal += [r,g,b]

    # Write the palette to the output file.
    for p in pal:
        out.write(bytes([p]))

    # Now we can write the actual data to the file.
    imgdata = []
    for y in range(0, height):
        row = []
        for x in range(0, width):
            row += [rng.randint(0, len(pal)-1)]

        while len(row)!= width:
            row += [0]

        imgdata += row

    im = Image.new("P", (width, height))
    im.putpalette(pal)
    im.putdata(imgdata)
    im.save(out, format="gif")


def generate_random_png(rng, out):

    # PNG signature and IHDR chunk header.
    out.write(b'\x89PNG\r\n\x1A\n')

    # Width and height of the image.
    width = rng.randint(256, 4096)
    height = rng.randint(256, 4096)
    bitdepth = 8
    colortype = 2
    compression = 0
    filtertype = 0
    interlaced = 0
    out.write(struct.pack('<IIBBBBB', width, height, bitdepth, colortype,
                          compression, filtertype, interlaced))

    # CRC for IHDR chunk.
    crc = zlib.crc32(out.getvalue()) & 0xFFFFFFFF
    out.write(struct.pack('<I', crc))

    # IDAT chunk header.
    out.write(b'IDAT')

    # Create a palette of random colors.
    pal = []
    for i in range(0, 256):
        r = rng.randint(0, 255)
        g = rng.randint(0, 255)
        b = rng.randint(0, 255)
        pal += [r,g,b]

    # Write the palette to the output file.
    for p in pal:
        out.write(bytes([p]))

    # Now we can write the actual data to the file.
    imgdata
    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)