#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
):

    # header
    out.write(b'GIF89a')  # signature
    out.write(struct.pack('<HH', 1024, 768))  # screen size (width, height)
    out.write(b'\x87\x00\x00')  # color table size = 256

    # global color table
    for i in range(256):
        r = int(rng.read(1)[0]) % 256
        g = int(rng.read(1)[0]) % 256
        b = int(rng.read(1)[0]) % 256

        out.write(bytes([r, g, b]))

    # image descriptor
    out.write(b'\x2c')  # image separator
    out.write(struct.pack('<HHHHB', 0, 0, 1024, 768, 0))  # position and size of the image
    out.write(b'\x00')  # no local color table

    # LZW minimum code size
    out.write(b'\x08')

    # data sub-block
    while True:
        block_size = rng.read(1)[0] & 0xff
        if not block_size:
            break

        out.write(bytes([block_size]))
        out.write(rng.read(block_size))

    # terminator
    out.write(b';')


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)