#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'
    assert len(header) == 13

    # Write the header to disk.
    out.write(header)

    # Figure out how many bytes of data there are by reading from the random number generator until we get a zero byte.
    num_bytes = 0
    while True:
        byte = rng.read(1)[0]
        if byte!= 0:
            num_bytes += 1
        else:
            break

    # Now write the actual data.
    for _ in range(num_bytes):
        out.write(rng.read(1))

    # Finally, pad with zeros up to the next multiple of four.
    padding = (4 - ((out.tell() + 1) % 4)) % 4
    out.write(b'\x00' * padding)


if