#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'
    out.write(header)

    # Next, we need to figure out how many bytes of random data we should write
    # into the file.  This is done by reading three bytes from /dev/urandom and
    # treating them as a little endian unsigned integer.
    num_bytes = int.from_bytes(rng.read(3), byteorder='little')

    # Now, we can actually write some random data!
    for _ in range(num_bytes):
        out.write(struct.pack('B', rng.read(1)[0]))

    # Finally, we need to pad the output with zeros until the end of the block
    # (which is 256 bytes).
    padding_needed = -out.tell() % 256
    if padding_needed!= 0:
        out.write(b'\x00' * padding_needed)


if