#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'
    out.write(header)

    # Next, we need to figure out how many bytes of random data we should write
    # into the file.  This is done by reading three bytes from /dev/urandom and
    # interpreting them as a little-endian unsigned integer.
    num_bytes = int.from_bytes(rng.read(3), 'little')

    # Now we can actually write some data!
    for _ in range(num_bytes):
        byte = rng.read(1)[0]

        if byte == 0xFF:
            # If this is a special byte (0xFF), we need to escape it with another
             # 0xFF byte before writing it.
            out.write(b'\xFF\xFF')
        else:
            out.write(struct.pack('B', byte))

    # Finally, we need to pad the rest of the file until the end of the last byte
    # is aligned on a four-byte boundary.
    padding = -out.tell() % 4
    out.write(b'\x00' * padding)


if