#!/usr/bin/env python3

import os
import sys
import struct
from PIL import Image


def main():
    if len(sys.argv)!= 2:
        print("Usage: %s <input_file>" % (os.path.basename(__file__)))
        return

    input_file = sys.argv[1]
    with open(input_file, "rb") as f:
        data = bytearray(f.read())

        # Check magic number and version
        assert data[:4] == b"BMP\x00", "Invalid file format!"
        assert data[10] in [0, 1], "Unsupported BMP version!"

        width = int.from_bytes(data[18:22], byteorder="little")
        height = abs(int.from_bytes(data[22:26], byteorder="little"))
        image_size = int.from_bytes(data[28:32], byteorder="little")
        offset = int.from_bytes(data[10:], byteorder="little")

        # Read pixel data
        pixels = []
        for i in range(offset, offset + image_size):
            r = data[i * 3]
            g = data[(i * 3) + 1]
            b = data[(i * 3) + 2]

            # Convert to grayscale
            luma = round((r + g + b) / 3)
            pixels.append(luma)

        # Create output image
        img = Image.new('L', (width, height))
        img.putdata(pixels)
        img.save("%s.png" % (os.path.splitext(input_file)[0]))


if