#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def main():
    if len(sys.argv)!= 2:
        print("Usage: {} <file>".format(os.path.basename(__file__)), file=sys.stderr)
        return -1

    with open(sys.argv[1], "rb") as f:
        data = read_data(f, 0x4000)

        # Find the first valid entry in the table
        for i in range(len(data)):
            if data[i] == 0xFF and data[i + 1] == 0xFF:
                break

        # Read all entries from the table
        while True:
            if (data[i] & 0xC0)!= 0xC0 or data[i + 1] == 0xFF:
                break

            offset = ((data[i] & 0x3F) << 8) | data[i + 1]
            size   = data[i + 2]

            print("{}: {:#06X} ({})".format(i // 3, offset, size))

            i += 3


def read_data(f: BinaryIO, length: int):
    """Read exactly `length` bytes from a binary stream."""
    result = bytearray()

    while len(result) < length:
        chunk = f.read(length - len(result))
        if not chunk:
            raise EOFError("Unexpected end of file.")

        result.extend(chunk)

    return result


if