#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.

    # Header magic number (4 bytes)
    out.write(b'GIF89a')

    # Screen descriptor block (7 bytes)
    width = rng.read(1)[0] % 256 + 1
    height = rng.read(1)[0] % 256 + 1
    flags = b'\x80\x00' if rng.read(1)[0] & 1 else b'\x00\x00'
    bg_color = rng.read(1)[0] % 256
    aspect_ratio = rng.read(1)[0] % 256
    out.write(struct.pack('<HHBBB', width, height, flags[0], flags[1],
                          bg_color, aspect_ratio))

    # Global color table (3 * 256 bytes)
    for _ in range(3):
        for i in range(256):
            out.write(bytes([rng.read(1)[0]]))

    # Image descriptor block (10 bytes)
    left = rng.read(1)[0] % width
    top = rng.read(1)[0] % height
    width = rng.read(1)[0] % (width - left) + 1
    height = rng.read(1)[0] % (height - top) + 1
    flags = b'\x80\x00' if rng.read(1)[0] & 1 else b'\x00\x00'
    local_color_table = bool((flags[0] >> 7) & 1)
    interlace = bool((flags[0] >> 6) & 1)
    sort = bool((flags[0] >> 5) & 1)
    reserved = rng.read(1)[0]
    lct_size = 2 << ((flags[0] >> 3) & 3)
    lct_sort = bool((flags[0] >> 2) & 1)
    lct_int = bool((flags[0] >> 1) & 1)
    lct_reserved = bool(flags[0] & 1)
    out.write(struct.pack('<HHHHBBBBB', left, top, width, height, flags[0],
                          flags[1], local_color_table, interlace, sort,
                          reserved))

    # Local color table (lct_size * 3 bytes)
    if local_color_table:
        for _ in range(lct_size):
            for i in range(3):
                out.write(bytes([rng.read(1)[0]]))

    # Image data (width * height bytes)
    for y in range(height):
        for x in range(width):
            out.write(bytes([rng.read(1)[0]]))


if