#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'
    out.write(header)

    # Next, we need to figure out how many bytes of random data we should write
    # into our file.  This is stored at offset 6 of the header (the length of
    # the logical screen descriptor).
    lsd_len = int.from_bytes(header[6:8], byteorder='little')
    width = int.from_bytes(header[8:10], byteorder='little')
    height = int.from_bytes(header[10:12], byteorder='little')
    print("Logical Screen Descriptor Length:", lsd_len)
    print("Width:", width)
    print("Height:", height)
    num_pixels = width * height
    num_bytes = num_pixels // 8 + ((num_pixels % 8)!= 0)
    print("Number of Bytes:", num_bytes)

    for _ in range(num_bytes):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)