#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'
    out.write(header)

    # Now we need to pick a random number of frames for our animation.
    num_frames = rng.read(1)[0] % (256 - 1) + 1

    # Write the rest of the file.
    out.write(struct.pack('B', num_frames))

    for _ in range(num_frames):

        # Pick a random delay time between 1 and 10 seconds.
        delay = rng.read(1)[0] % (10 * 100) + 1

        # Pick a random width between 1 and 255 pixels.
        width = rng.read(1)[0] % (255 - 1) + 1

        # Pick a random height between 1 and 255 pixels.
        height = rng.read(1)[0] % (255 - 1) + 1

        # Write this frame's metadata.
        out.write(
            struct.pack('<HHBBB',
                        0,
                        delay * 10,
                        width,
                        height,
                        0x80 | 0x40 | 0x20
                    )
        )

        # Generate a random palette entry.
        r = rng.read(1)[0] & 0x3f
        g = rng.read(1)[0] & 0x3f
        b = rng.read(1)[0] & 0x3f

        # Write this frame's palette entry.
        out.write(struct.pack('BBB', r, g, b))

        # Generate a random image.
        for y in range(height):

            # Write each row of the image.
            for x in range(width):

                # Write one pixel at a time.
                out.write(struct.pack('B', rng.read(1)[0]))


if