#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # Header

    header = b'GIF89a' + bytes([0x12, 0x34]) + \
             bytes([0x56, 0x78]) + bytes([0x9A, 0xBC]) + \
             bytes([0xDE, 0xF0])

    out.write(header)

    # Logical screen descriptor

    width = int.from_bytes(rng.read(2), byteorder='little') % (2**12-1) + 1
    height = int.from_bytes(rng.read(2), byteorder='little') % (2**12-1) + 1

    flags = 0b10000000 | ((width >> 10) & 0b00000111) << 7 | \
            ((height >> 10) & 0b00000111) << 4

    gct_size = int.from_bytes(rng.read(1), byteorder='little') & 0b00000111

    bg_color = int.from_bytes(rng.read(1), byteorder='little')

    aspect_ratio = int.from_bytes(rng.read(1), byteorder='little')

    lsd = [
        0x21, 0xFF, 0x0B,     # Application extension block
        0x00,                 # Size of the application data subblock
        0x4E, 0x45, 0x54, 0x53, 0x43, 0x41, 0x50, 0x45, 0x32, 0x2E, 0x30, 0x03, 0x01, 0x00, 0x00, 0x00   # NETSCAPE2.0
    ]

    lsd += [
        0x2C,             # Image descriptor
        0x00,             # Left position
        0x00,             # Top position
        0x00,             # Width
        0x00,             # Height
        0b10000000,       # Flags
        0x00              # Background color index
    ]

    lsd[2] |= len(lsd)-1

    for i in range(len(lsd)):
        out.write(struct.pack('<B', lsd[i]))

    # Global color table

    if gct_size > 0:

        palette = []

        for _ in range(gct_size):

            r = rng.read(1)[0]
            g = rng.read(1)[0]
            b = rng.read(1)[0]

            palette += [r, g, b]

        for i in range(len(palette)):
            out.write(struct.pack('<BBB', *palette[i*3 : (i+1)*3]))

    # Graphics control extension

    out.write(struct.pack('<BBBBBHB', 0x21, 0xF9, 0x04,
                         0x00, 0x00, 0x00, 0x00, 0x00))

    # Image data

    for y in range(height):

        for x in range(width):

            out.write(struct.pack('<B', rng.read(1)[0]))
if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)