#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
    rng: BinaryIO,
    out: BinaryIO,
):

    # Header
    header = b'GIF89a\x00\x00\x00\x00\xff\xff\xff!\xf9\x04\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'
    out.write(header)

    # Logical screen descriptor
    width = 640
    height = 480
    color_table_size = 7
    global_color_table = True
    background_index = 0
    pixel_aspect_ratio = 0
    logical_screen_descriptor = struct.pack('<HHBBBH', width, height, color_table_size,
                                            int(global_color_table), background_index, pixel_aspect_ratio)
    out.write(logical_screen_descriptor)

    # Global color table
    for i in range(0, (1 << color_table_size)):
        rgb = [int(rng.read(1)[0]) & 0b11111] * 3 + [0xFF]
        out.write(bytes(rgb))

    # Image descriptor
    left = 0
    top = 0
    image_width = width
    image_height = height
    local_color_table = False
    interlace = False
    sort = False
    reserved = 0
    lct_size = 0
    image_descriptor = struct.pack('<HHHHBBBBB', left, top, image_width,
                                   image_height, lct_size, int(local_color_table), interlace, sort, reserved)
    out.write(image_descriptor)

    # Local color table
    if not global_color_table:
        for i in range(0, (1 << color_table_size)):
            rgb = [int(rng.read(1)[0]) & 0b11111] * 3 + [0xFF]
            out.write(bytes(rgb))

    # Pixel data
    for _ in range(0, width * height):
        out.write(rng.read(1))


if __name__ == '__main__':
    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)