#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO, List


def main(argv: List[str]) -> int:
    if len(argv) < 2 or argv[1] in ("-h", "--help"):
        print("Usage: {} FILE [FILE...]".format(os.path.basename(__file__)))
        return 0

    for filename in argv[1:]:
        with open(filename, "rb") as f:
            data = f.read()

        # Check magic number
        if not (data[:4] == b"\x7fELF" and
                data[5] == 2):
            continue

        offset_e_phoff = struct.unpack("<I", data[68:72])[0]
        e_phnum = struct.unpack("<H", data[78:80])[0]

        phdrs = []
        for i in range(e_phnum):
            off = offset_e_phoff + 40 * i

            p_type = struct.unpack("<I", data[off+0:off+4])[0]
            p_offset = struct.unpack("<I", data[off+4:off+8])[0]
            p_vaddr = struct.unpack("<I", data[off+8:off+12])[0]
            p_paddr = struct.unpack("<I", data[off+12:off+16])[0]
            p_filesz = struct.unpack("<I", data[off+16:off+20])[0]
            p_memsz = struct.unpack("<I", data[off+20:off+24])[0]
            p_flags = struct.unpack("<I", data[off+24:off+28])[0]
            p_align = struct.unpack("<I", data[off+28:off+32])[0]

            phdrs.append((p_type, p_offset, p_vaddr, p_paddr,
                          p_filesz, p_memsz, p_flags, p_align))

        for type_, _, vaddr, _, filesz, memsz, flags, _ in phdrs:
            if type_!= 1:
                continue

            if flags & 4:
                # RWE
                continue

            if vaddr % 0x1000!= 0:
                continue

            if filesz > memsz:
                continue

            print("{}: {:#x} - {:#x}".format(filename, vaddr,
                                             vaddr + memsz - 1))

    return 0


if