#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section. We'll just write the header first with some
    # dummy values for now.

    # Header
    out.write(b'GIF89a')  # Magic number
    out.write(struct.pack('<HH', 1024, 768))  # Screen size (width x height)
    out.write(b'\x87\x00')  # Flags: global color table present + sort flag
    out.write(b'\x00')      # Background color index
    out.write(b'\x00')      # Pixel aspect ratio

    # Global color table
    colors = [
        b'\xff\x00\x00',   # Red
        b'\x00\xff\x00',   # Green
        b'\x00\x00\xff',   # Blue
        b'\xff\xff\xff',   # White
        b'\x00\x00\x00',   # Black
    ] * 256
    out.write(bytes(colors[:256]))

    # Image descriptor
    out.write(b'\x2c')     # Image separator
    out.write(struct.pack('<HHHHHBB',
        0,                  # Left position
        0,                  # Top position
        1024,               # Width
        768,                # Height
        0,                  # Local color table
        0                   # LZW minimum code size
    ))

    # Image data
    while True:

        # Get random byte
        c = rng.read(1)[0]

        if not c:
            break

        # Write compressed block
        out.write(bytes([
            0x21,             # Extension introducer
            0xf9,             # Graphic control label
            0x04,             # Block size
            0x01 |            # Packed fields
                0x02 |         # Disposal method: leave image in place
                0x08,          # User input flag: no user input expected
            0x00,             # Delay time
            0x00,             # Transparent color index
            0x00              # Block terminator
        ]))
        out.write(bytes([c]))    # Actual pixel value
        out.write(b'\x00')       # Block terminator

    # Trailer
    out.write(b';')        # GIF trailer


if