#!/usr/bin/env python3

import os
import sys
from PIL import Image


def generate_random_gif(rng, out):
    """Generate a random GIF image."""
    frames = 256
    width = height = 80
    palette = [i for i in range(768)]
    im = Image.new("P", (width, height), color=0)
    im.putpalette(palette)

    for frame in range(frames):
        print(f'Generating frame {frame+1}/{frames}...', end='\r')

        # Generate some random pixels and add them to our image.
        for y in range(height):
            for x in range(width):
                pixel = rng.read(1)[0] % len(palette)
                im.putpixel((x,y), pixel)

        # Save this frame of animation.
        im.save(out, format="GIF", append_images=[im], save_all=True, duration=40, loop=0)

    print()


if __name__ == '__main__':
    if not os.path.exists('test.gif'):
        with open('test.gif', 'w'): pass

    # Open the output file so that we can get its size.
    out = open('test.gif', 'ab')
    datalen = os.stat('test.gif').st_size

    # Open /dev/urandom to read from.
    with open('/dev/urandom', mode='rb') as rng:
        # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))

    with open('/dev/urandom', mode='rb') as rng:
        with open('test.gif', mode='wb') as f:
            generate_random_gif(rng, f)