#!/usr/bin/env python3

import os
import sys
import struct
from typing import BinaryIO


def generate_random_gif(
        rng: BinaryIO, out: BinaryIO
    ) -> None:

    # The GIF format is a bit weird in that it has a fixed-size header and then
    # an unbounded data section.  We'll just write the header first so we can
    # figure out how long the data section will be later on.
    header = b'\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\xff\xff\xff\x21\xf9\x04\x01\x0a\x00\x01\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x44\x01\x00\x3b'

    # Next comes the length of the data section.  It's stored as a 3-byte big-endian
    # integer, so we need to read three bytes from /dev/urandom and interpret those
    # into the file.  This is done by reading three bytes from /dev/urandom and
    # interpreting them as a little-endian unsigned integer.
    datalen = int.from_bytes(
            rng.read(3), byteorder='big')
    out.write(header + datalen.to_bytes(3, 'big'))

    # Now we can write the actual data to the file.
    for _ in range(0, datalen):
        out.write(rng.read(1))


if __name__ == '__main__':
    if len(sys.argv)!= 2 or not os.path.exists(sys.argv[1]):
        print("Usage: {} <output>".format(os.path.basename(__file__)),
                file=sys.stderr)
        exit(-1)

    with open(sys.argv[1], mode='wb') as f:
        with open('/dev/urandom', mode='rb') as rng:
            generate_random_gif(rng, f)